// AnnoDoc.java
// Copyright (c) 1999 by Xerox Corporation
// bchang@parc.xerox.com

// $Header: /FluidDocs/annotation/AnnoDoc.java 32    9/20/99 8:38a Bchang $

// entry point
// "NS11" - see FluidUI.java

package annotation;

import fluid.*;
import java.awt.Font;
import java.awt.Dimension;
import java.awt.Color;

public class AnnoDoc extends FluidUI {

  public AnnoDoc() {
    super(new Page());
  }
  
  // public methods accessible to javascript

  // this does not seem to work in general without doing a load of document
  // see loadDocument(doc, tech) below.
  public void setTechnique(String tech) {
    if (tech.equalsIgnoreCase("FI")) {
      technique = "movethensquash";
      usefreezebutton = true;
    } else if (tech.equalsIgnoreCase("FO")) {
      technique = "overlay";
      usefreezebutton = true;
    } else if (tech.equalsIgnoreCase("FM")) {
      technique = "margincallout";
      usefreezebutton = false;
    } else if (tech.equalsIgnoreCase("PG")) {
      technique = "tooltip";
      usefreezebutton = false;
    } else if (tech.equalsIgnoreCase("NG")) {
      technique = "footnote";
      usefreezebutton = false;
    } else if (tech.equalsIgnoreCase("HT")) {
      technique = "nogloss";
    } else {
      System.out.println("unknown technique in AnnoDoc.setTechnique()");
    }
  }

  // set the technique _before_ you load the document
  public void loadDocument(String doc) {
    page.loadnew(doc);
  }

  public void loadDocument(String doc, String tech) {
    setTechnique(tech);
    loadDocument(doc);
  }

  public void whatitis(String w) {
    technique = w;
    loadDocument(textfile);
  }

  public void reset() {
    page.reload();
  }

  public void back() {
    ((Page)page).back();
  }

  public void rep() {
    ((Page)page).repaint();
  }

	public void triggerLog(String s) {
		fluidLog.triggerLog(s);
	}
  
  
  // convenience methods to test params

  public static boolean clickToOpen() {
    return trigger.equals(TRIGGER_CLICK);
  }

  public static boolean wideLayout() {
    return annolayout.equals(ANNOLAYOUT_WIDE);
  }
  public static boolean pyramidLayout() {
    return annolayout.equals(ANNOLAYOUT_PYRAMID);
  }
  public static boolean hypersentenceLayout() {
    return wideLayout() || pyramidLayout();
  }

  
  // parameters

  public static String technique = "squash";
  public static String directory = "";
  public static String textfile = "foobar.txt";

  public static final String TRIGGER_CLICK = "click";
  public static final String TRIGGER_MOUSEOVER = "mouseover";
  public static String trigger = TRIGGER_MOUSEOVER;

  public static final String ANNOLAYOUT_WIDE = "wide";
  public static final String ANNOLAYOUT_PYRAMID = "pyramid";
  public static final String ANNOLAYOUT_GLOSS = "gloss";
  public static String annolayout = ANNOLAYOUT_GLOSS;

  public static int pyramidstep = 10;

  public static int annoindent = 10;

  public static int linkreturnpause = 1000;

  public static boolean singleAnnoPath = false;

  public static boolean breakWordsIfNecessary = true;
  public static boolean usefreezebutton = false;
  public static int freezebuttonwidth = 12;
  public static int freezebuttonheight = 12;
  public static int freezebuttongutter = 5;
  public static int freezebuttony = 5;
  public static int freezebuttonovershoot = 20;
//  public static boolean freezebuttonverticalovershoot = true;

  public static String textfontname = "Serif";
  public static int textfontsize = 22;
  public static int textlinespacing = 28;
  public static int text_fontstyle = Font.PLAIN;
  public static int title_fontstyle = Font.BOLD;
  public static String annofontname = "SansSerif";
  public static int annofontsize = 16;
  public static int annolinespacing = 16;
  public static int anno_fontstyle = Font.BOLD;
  public static int annooffset = 3;
  public static int nestedannooffset = 0;
  public static boolean hasNesting = false;
  public static Color annocolor = Color.red;
  public static Color annocolor1 = Color.red;
  public static Color annocolor2 = Color.red;
  public static Color annocolor3 = Color.red;
  public static Color annocolor4 = Color.red;
  // for backward compatibility -- it used to be that frozencolor
  // was always computed rather than specified in the params.
  // so if you want it to be computed to be annocolor.darker().darker(), leave it null
  public static Color frozencolor = null;
  public static int annowidth = 300;
  public static Color grayoutcolor = new Color(200, 200, 200);
  public static Color nogrowcolor = Color.blue;
  public static Color lasttraversedcolor = Color.cyan.darker();
  public static Color traversedcolor = Color.green.darker();
  public static int calloutfontsize = 12;
  public static int calloutlinespacing = 12;

  public static String fileformat = "xml";

  public static String annoalignment = "left";

  public static int minfontsize = 10;
  public static int minlinespacing = 7;

  public static int topmargin = 70;
  public static int leftmargin = 100;
  public static int bottommargin = 70;
  public static int rightmargin = 100;

  public static int tooltiptopmargin = 5;
  public static int tooltipleftmargin = 7;
  public static int tooltiprightmargin = 5;
  public static int tooltipbottommargin = 7;
  public static Color tooltipcolor = new Color(255, 255, 231);
  public static Color tooltipborder = Color.black;
  public static int tooltipxoffset = 10;
  public static int tooltipyoffset = 20;

  public static String logdir = "";
  public static long logmousedelta = 200;
  public static boolean logdata = false;

  public static int minmargin = 10; 
  
  

  protected void initializeFromParams() {
    // this next fixes (for now) the problem that textfile is set in Page in the constructor
    // before these parameters are read in. Should change when Page is created.
    ((Page)page).filename = textfile;
  }

  
  // entry point for application
  
	public static void main(String[] args) {
    inApplet = false;
		try {
      String paramfilename = "";
      if (args.length > 0) {
        paramfilename = args[0];
      }
      mainframe(new AnnoDoc(), paramfilename);
		}	catch (Exception e) {
      e.printStackTrace();
      try {
        // wait for user to hit return
        System.in.read();
      } catch (Exception ioe) { 
      }
		}
	}

}

/* $Log: /FluidDocs/annotation/AnnoDoc.java $
 * 
 * 32    9/20/99 8:38a Bchang
 * moved animduration param to FluidUI
 * 
 * 31    7/23/99 9:07a Bchang
 * minmargin (used in block move)
 * commented out freezebuttonverticalovershoot
 * 
 * 30    7/20/99 9:49a Bchang
 * whatitis now loads the document -- required if you're going to switch
 * to or from margincallout 
 * 
 * 29    7/19/99 12:57p Bchang
 * now automatically sets params -- all you have to do to add a param is
 * to declare a public variable.
 * All primitive data types + strings are handled.
 * In addition:
 *   fontstyle is handled: if you have such a param, you must name it with
 * suffix _fontstyle
 *   Color is handled
 * 
 * 28    7/16/99 3:28p Bchang
 * freezebuttonovershoot param
 * 
 * 27    7/15/99 1:09p Mackinla
 * parameter logdata to turn off logging
 * 
 * 26    7/15/99 11:59a Mackinla
 * 
 * 25    7/15/99 11:42a Bchang
 * because we are hardwiring annocolor to be black in the tooltip and
 * footnote cases, i removed the hardwiring of annocolor for the fluid
 * cases when setTechnique() is used
 *
 * 24    7/15/99 11:20a Bchang
 * in setTechnique, a technique of "interline" translates to
 * "movethensquash"
 *
 * 23    7/14/99 12:42p Bchang
 * setTechnique and loadDocument. setTechnique is used by the fluid
 * studies experiment.
 * 
 * 22    7/14/99 11:46a Bchang
 * added placeholders for setTechnique() and setDocument()
 * 
 * 21    7/14/99 11:39a Mackinla
 * fluidLog - html access for filename and dir
 * 
 * 20    7/14/99 8:40a Bchang
 * added params: linkreturnpause, frozencolor, freezebutton and
 * freezebutton size/placement
 *
 * 19    7/13/99 5:43p Mackinla
 * Logging - basic functionality
 * 
 * 18    6/30/99 1:10p Bchang
 * Add tooltip params.
 * 
 * 17    6/28/99 10:50a Bchang
 * Added "grouped" as an option for the fileformat param.
 * 
 * 16    6/24/99 9:45a Bchang
 * added params: annolayout (choices are: wide, pyramid, or gloss --
 * default is gloss), pyramidstep (amount to step each line in the pyramid
 * layout), annoindent (amount to indent annotations in the wide or
 * pyramid layouts), singleannopath (close previous annotation when
 * opening a new one -- works with trigger=click), breakwordsifnecessary
 * (determines whether line layout algorithm should break a word if it
 * will extend past the max line length -- default true), and
 * annocolor{1,2,3,4} which are the colors assigned to nested annotations
 * in the wide and pyramid layouts (order is annocolor, annocolor1,
 * annocolor2, ...).
 * 
 * 15    6/11/99 12:47p Bchang
 * Added the param "trigger". Defaults to "mouseover". If set to "click",
 * you have to left-click on an anchor to open the annotation and freeze
 * (same as right-click); dwell does nothing.
 * 
 * 14    5/20/99 12:13p Bchang
 * added "fileformat" param. values are "xml" and "outline". default is
 * xml.
 * 
 * 13    5/18/99 4:42p Bchang
 * added annoalignment param
 * 
 * 12    5/18/99 4:12p Bchang
 * (Pollez) added lasttraversedcolor and traversedcolor. Still need to add
 * support for setting these from the applet's PARAMs.
 * 
 * 11    5/12/99 11:26a Bchang
 * Moved hexToColor() code out of this class and into Fluid.Util.
 * 
 * 10    1/27/99 5:55p Bchang
 * add parameters: calloutfontsize, calloutlinespacing, annowidth
 * 
 * 9     1/27/99 2:38p Bchang
 * added hasnesting parameter -- currently, must set this parameter to
 * true in the applet tag in order to get correct nesting behavior
 * (shrinking)
 * 
 * 8     1/26/99 11:59p Bchang
 * add nestedannooffset parameter.
 * change default technique to squash (interline compression).
 * 
 * 7     1/22/99 2:59p Bchang
 * add support for titlefontstyle, the style for <title> paragraphs
 * 
 * 6     1/21/99 4:37p Bchang
 * removed a println
 * 
 * 5     1/15/99 4:25p Bchang
 * Added a hack to update the starting filename in Page. Problem was that
 * applet parameters were being read and set _after_ Page was created, and
 * the filename is set in Page's constructor. Probable fix: read in
 * parameters before creating Page.
 * 
 * 4     1/13/99 5:30p Bchang
 */
