// Annotation.java
// Copyright (c) 1999 by Xerox Corporation
// bchang@parc.xerox.com

// $Header: /FluidDocs/annotation/Annotation.java 37    8/31/99 4:19p Bchang $

package annotation;

import fluid.*;
import java.awt.*;
import java.util.*;

public class Annotation extends TextBlock implements Cloneable, AnimationListener {

  // the first component is the freeze button
  protected int getLineCount() {
    return usefreezebutton ? getComponentCount() - 1 : getComponentCount(); }
  protected TextLine getLine(int i) {
    return usefreezebutton ? (TextLine)getComponent(i + 1) : (TextLine)getComponent(i); }
  protected void removeLines() {
    if (usefreezebutton) {
      FluidComponent save = getComponent(0);
      removeAllComponents();
      add(save);
    } else {
      removeAllComponents();
    }
  }

  // parent of an annotation is a TextLine, whose parent is a TextBlock

  protected int topLevelTextWidth() {
    return ((TextBlock)(getParent().getPrimary())).topLevelTextWidth();
  }

  protected void reconstructTextLines() {
    int w;
    if (footnote()) {
      w = ((Page)getPage()).getFootnoteWidth();
    } else if (AnnoDoc.hypersentenceLayout()) {
      int indent = nestingLevel() * AnnoDoc.annoindent;
      w = topLevelTextWidth() - indent;
    } else {
      // AnnoDoc.annowidth is a default width (pixels) for annotations
      w = AnnoDoc.annowidth;
    }
    reconstructTextLines(w);
  }

  protected Color annoColor = Color.red;
  protected Color frozenColor = Color.red.darker().darker();
  protected Color nogrowColor = Color.blue;
  protected Color lastTraversedColor = Color.cyan.darker();
  protected Color traversedColor = Color.green.darker();
  protected Color frozenTraversedColor = traversedColor.darker().darker();

  protected boolean usefreezebutton;

  int drawc = -1;

  int fullSizeFontSize;
  int fullSizeLineSpacing;

  // used to be private, but reuse stuff needs access
  Negotiation negotiation;

  protected Animation currentAnimation = null;

  protected Point endpoint;

  protected Dimension fullSize;

  public boolean nogrow = false;

  int animduration;

  protected boolean frozen = false;  // frozen either by user or temporarily by system on link traversal
  public void setfrozen() { frozen = true; ((Page)getPage()).openAnnoAtLevel(this, nestingLevel()); };
  public void clearfrozen() { frozen = false; ((Page)getPage()).closeAnnoAtLevel(nestingLevel()); };

  protected boolean traversed = false;  // affects the color of the anno and perhaps its underline cue
  protected boolean lastTraversed = false;
    // the gloss for the most-recently-traversed link will be frozen with a unique color
    // until the user explicitly thaws it
  protected boolean saveFrozen = false;  // previous state of frozen if lastTraversed is currently true  
  
  protected int minFontSize;
  protected int minLineSpacing;

  // how many pixels the underline should be under its anchor text
  int lineoffset = 1;
  
  protected int nestedoffset;
  
  public final static int ALIGN_LEFT = 0;
  public final static int ALIGN_CENTER = 1;
  public final static int ALIGN_RIGHT = 2;
  protected int alignment;

  protected void setParams() {
    fontName = AnnoDoc.annofontname;
    fontStyle = AnnoDoc.anno_fontstyle;
    if (margincallout()) {
      initFontSizeAndLineSpacing(AnnoDoc.calloutfontsize, AnnoDoc.calloutlinespacing);
    } else {
      initFontSizeAndLineSpacing(AnnoDoc.annofontsize, AnnoDoc.annolinespacing);
    }

    animduration = AnnoDoc.animduration;

    if (AnnoDoc.annoalignment.equalsIgnoreCase("center")) {
      alignment = Annotation.ALIGN_CENTER;
    } else if (AnnoDoc.annoalignment.equals("right")) {
      alignment = Annotation.ALIGN_RIGHT;
    } else {
      alignment = Annotation.ALIGN_LEFT;
    }

    if (tooltip() || footnote()) {
      annoColor = Color.black;
      lastTraversedColor = Color.black;
    } else {
      annoColor = AnnoDoc.annocolor;
  	  lastTraversedColor = AnnoDoc.lasttraversedcolor;
    }

    traversedColor = AnnoDoc.traversedcolor;
    if (AnnoDoc.frozencolor == null) {
      // for backward compatibility -- it used to be that frozencolor
      // was always computed rather than specified in the params
      frozenColor = annoColor.darker().darker();
    } else {
      frozenColor = AnnoDoc.frozencolor;
    }
    nogrowColor = AnnoDoc.nogrowcolor;
//	  frozenTraversedColor = traversedColor.darker().darker();
	  frozenTraversedColor = frozenColor;

    offset = AnnoDoc.annooffset;
    nestedoffset = AnnoDoc.nestedannooffset;

    minFontSize = AnnoDoc.minfontsize;
    minLineSpacing = AnnoDoc.minlinespacing;

    if (margincallout() || footnote() || tooltip()) {
      usefreezebutton = false;
    } else {
      usefreezebutton = AnnoDoc.usefreezebutton;
    }

    setGraphicalProtocol("fluid.TextGChar");
  }

  protected void initFontSizeAndLineSpacing(int fs, int ls) {
    fontSize = fs;
    fullSizeFontSize = fontSize;
    lineSpacing = ls;
    fullSizeLineSpacing = lineSpacing;
  }

  public Annotation() {
    this(null);
  }

  protected int xOffset = 0; // the relative X location specified by the gloss attribute "x"
  protected FreezeButton freezeButton;

  public Annotation(GlossAttributes ga) {
    setDwellable(true);
    setParams();

    if (ga != null) {
     if (ga.color != null) {
        annoColor = ga.color;
      }
      if (!ga.fontname.equals("")) {
        fontName = ga.fontname;
      }
      if (ga.fontstyle != -1) {
        fontStyle = ga.fontstyle;
      }

      int fs = fontSize;
      int ls = lineSpacing;
      if (ga.fontsize != -1) {
        fs = ga.fontsize;
      }
      if (ga.linespacing != -1) {
        ls = ga.linespacing;
      } else if (ga.fontsize != -1) {
        // if gloss specifies fontsize but not linespacing, use fontsize as the linespacing
        ls = ga.fontsize;
      }
      initFontSizeAndLineSpacing(fs, ls);

      xOffset = ga.x;
    }

    smallState();
    color = Color.black;

    // now add the freeze button
    if (usefreezebutton) {
      freezeButton = new FreezeButton();
      add(freezeButton);
    }
  }

	public String getID () {
		FluidComponent parent = getParent();
		if (parent != null) {
			Vector components = parent.getComponents();
			if (components.contains(this)) {
				return "[" + components.indexOf(this) + "]";
			}
		}
		return "[]";
	}

  protected int startchar;
  public void initializePositions(int start, int len) {
    startchar = start;
    anchorLength = len;
    setInitialPosition();
  }

  protected Vector sharedAnchor = null;
  public void setSharedAnchor(Vector v) { sharedAnchor = v; }
  public Vector getSharedAnchor() { return sharedAnchor; }
  public boolean sharingAnchor() { return sharedAnchor != null; }

  protected void setTraversed(boolean b) {
    if (sharingAnchor()) {
      for (int i = 0; i < sharedAnchor.size(); i++) {
        ((Annotation)sharedAnchor.elementAt(i)).traversed = b;
      }
    } else {
      traversed = b;
    }
  }

  public Annotation newCopyAnchor() {
    Annotation anno = new Annotation();
    anno.setAnchor(anchorParagraphNumber, anchorStart, anchorLength);
    anno.paragraphs = paragraphs;
    anno.setHref(getHref());
    return anno;
  }

  protected int anchorParagraphNumber;
  protected int anchorStart;
  protected int anchorLength;
  protected void setAnchor(int paranum, int start, int length) {
    anchorParagraphNumber = paranum;
    anchorStart = start;
    anchorLength = length;
  }
  protected String anchorText;
  public void setAnchorText(String str) { anchorText = str; }
  public String getAnchorText() { return anchorText; }

  protected Point getCharLocation(int i) {
    return ((TextLine)getParent()).getCharLocation(i);
  }

  protected int underlineY;

  public boolean childIsNested() { return true; }

  public int nestingLevel() { return ((TextBlock)(getParent().getPrimary())).nestingLevel() + 1; }

  protected int defaultOffsetFromAnchor() {
    return underlineWidth() / 2;
  }

  protected int firstLineIndent() {
    if (AnnoDoc.hypersentenceLayout()) {
      return (-expandedXDefault) + defaultOffsetFromAnchor();
    } else {
      return 0;
    }
  }

  protected int getTopLevelAbsoluteX() {
    return ((TextBlock)(getParent().getPrimary())).getTopLevelAbsoluteX();
  }

  protected int xOffsetFromTopLevelText() {
//    return getX() + ((TextBlock)getParent().getPrimary()).xOffsetFromTopLevelText();
    return absoluteX() - getTopLevelAbsoluteX();
  }

  // only for pyramid layout!
  protected int getTrueAbsoluteXOfAnchor() {
    TextLine parentLine =(TextLine)getParent();
    TextBlock parentAnno = (TextBlock)(parentLine.getPrimary());

    int stepout = parentLine.getLineNumber() * AnnoDoc.pyramidstep;
    stepout = Math.min(stepout, -parentAnno.getExpandedXDefault());

    int charoffset = getCharLocation(startchar).x - parentLine.absoluteX();

    if (nestingLevel() == 1) {
      return charoffset + parentLine.absoluteX();
    } else {
      return parentAnno.getTrueAbsoluteXOfAnchor()
             + parentAnno.defaultOffsetFromAnchor()
             - stepout
             + charoffset;
    }
  }

  protected int getExpandedXDefault() { return expandedXDefault; }

  // setInitialPosition sets the position and size of the underline
  // it does not handle position or size of the expanded annotation
  // (see setLinePositions for that) -- except it does set expandedXDefault in some cases
  protected void setInitialPosition() {
    int absx = getCharLocation(startchar).x;
    int x = absx - getParent().absoluteX();
    underlineY = ((TextLine)parent).getAscent() + lineoffset;
//    setLocation(new Point(x, ((TextLine)parent).getAscent() + lineoffset));
    setLocation(new Point(x, 0));

    endpoint = new Point(getCharLocation(startchar + anchorLength).x - getParent().absoluteX(),
                            getParent().getHeight());
    smallBounds = new Rectangle(0, 0, endpoint.x - location.x, ((TextLine)parent).getAscent() + lineoffset);
    setSize(new Dimension(smallBounds.width, smallBounds.height));
    boolean nested = ((TextBlock)(getParent().getPrimary())).childIsNested();
    int off = nested ? nestedoffset : offset;

    if (footnote()) {
      int absExpY = ((Page)getPage()).getFootnoteY();
      expandedY = absExpY - absoluteY();
    } else {
      expandedY = ((TextLine)parent).getAscent() + lineoffset + off;
      if (usefreezebutton) {
        freezeButton.setY(expandedY);
      }
    }

    if (AnnoDoc.pyramidLayout()) {
      // expandedXDefault is set in setInitialPosition() for hypersentences
      // because TextBlock.reconstructTextLines() needs to reference it
      // (via firstLineIndent()).
      // setting it in setLinePositions() is too late.
      int indent = AnnoDoc.annoindent * nestingLevel();
      // getTrueAbsoluteXOfAnchor is only valid for pyramid layout
      expandedXDefault = -(getTrueAbsoluteXOfAnchor() - getTopLevelAbsoluteX() - indent);
    } else if (AnnoDoc.wideLayout()) {
      int indent = AnnoDoc.annoindent * nestingLevel();
      expandedXDefault = -(x - indent);
    } else if (footnote()) {
      expandedXDefault = -x;
    } else {
      // normally, expandedXDefault (nee' moveBack) is set in setLinePositions() because
      // it uses fullSize. For now, ensure that it is zero so that Textblock().reconstructTextLines()
      // works correctly for non-hypersentence layout.
      expandedXDefault = 0;
    }

    if (AnnoDoc.hypersentenceLayout()) {
      switch (nestingLevel()) {
        case 1: annoColor = AnnoDoc.annocolor; frozenColor = AnnoDoc.annocolor; break;
        case 2: annoColor = AnnoDoc.annocolor1; frozenColor = AnnoDoc.annocolor1; break;
        case 3: annoColor = AnnoDoc.annocolor2; frozenColor = AnnoDoc.annocolor2; break;
        case 4: annoColor = AnnoDoc.annocolor3; frozenColor = AnnoDoc.annocolor3; break;
        default: annoColor = AnnoDoc.annocolor4; frozenColor = AnnoDoc.annocolor4;
      }
    }
  }

  public int expandedY;
  public int absoluteExpandedY() { return expandedY + absoluteY(); }

  protected Rectangle smallBounds;


  protected int expandedXDefault = 0;
  protected int expandedX = 0;
  public int getExpandedX() { return expandedX; }

  protected void setLinePositions() {

    for (int i = 0; i < getLineCount(); i++) {
      getLine(i).setOffset(expandedY);
      getLine(i).setLineSpacing(0); // sets Y
      getLine(i).setFontSize(fontSize);
      getLine(i).setColor(Color.white);
    }

    fullSize = calculateFullSize();

    // calculateCalloutSize does stuff that should only be done in the margin
    // callout technique -- this means that we are no longer able to switch to
    // margin callout without reloading the page
    if (margincallout()) {
      calculateCalloutSize();
    }

    if (usefreezebutton) {
      freezeButton.setHeight(fullSize.height);
    }
    
    /* disable ALIGNMENT and XOFFSET for now. Instead, we will use hypersentence layout. 
    
    // this is a crappy reuse of expandedXDefault -- change soon please!
    if (alignment == ALIGN_RIGHT) {
      expandedXDefault = underlineWidth();
    } else if (alignment == ALIGN_CENTER) {
      expandedXDefault = underlineWidth() / 2;
    } else { // ALIGN_LEFT
      expandedXDefault = 0;
    }
    
    // xOffset for now overrides expandedXDefault, so it doesn't behave well at right edge of page
    // need to figure out what we want to do in that case
    if (xOffset != 0) {
      expandedXDefault += xOffset;
    }
    
    */

    if (!AnnoDoc.hypersentenceLayout()) {
      // preserve the old expandedXDefault behavior when alignment & xoffset aren't doing their thing
      // (by the way, expandedXDefault used to be called moveBack)
      if (expandedXDefault == 0) {
//    int widthToEndOfLine = getParent().getParent().absoluteRight() - absoluteX();
        int widthToEndOfLine = ((Page)getPage()).textblock.absoluteRight() - absoluteX();
        expandedXDefault = Math.min(0, widthToEndOfLine - fullSize.width);
        if (usefreezebutton && (expandedXDefault == 0)) {
          int m = Math.min(getX(), freezeButton.offset());
          m = Math.max(m, freezeButton.getOvershoot());
          expandedXDefault = -m;
        }
      }
    }
    
    expandedX = expandedXDefault;    
    
    for (int i = 0; i < getLineCount(); i++) {
      if (!margincallout()) {
        getLine(i).setX(0);
      }
      getLine(i).setFontSize(3);
    }
  } 


  protected void setCalloutLinePositions(int x, int y) {
    for (int i = 0; i < getLineCount(); i++) {
      getLine(i).setOffset(y);
      getLine(i).setLineSpacing(0); // sets Y
      getLine(i).setFontSize(fontSize);
      getLine(i).setColor(Color.white);
    }
    fullSize = calculateFullSize();
    expandedXDefault = 0;
    expandedX = 0;
    for (int i = 0; i < getLineCount(); i++) {
      getLine(i).setX(x);
      getLine(i).setFontSize(3);
    }
  }

  protected String href = "";
  public void setHref(String hr) { href = hr; }
  public String getHref() { return href; }

  public void structureLoaded() {
    // if the page this link points to has been visited at some point, set traversed
    if (((Page)getPage()).visited(href)) {
      setTraversed(true);
    }

    // if this link has been "last traversed" in the back history, make it
    // frozen now, whether or not it was frozen before (eg, override savefrozen)
    if (!lastTraversed && frozen && !saveFrozen) {
      saveFrozen = true;
      if (AnnoDoc.usefreezebutton) {
        freezeButton.setOn();
      }
    }
  }

  protected boolean stopAddingLines() { return false; }

  private int state;
  private static int small = 0;
  private static int big = 1;
  protected boolean isSmall() { 
    return animationDone() && (state == small);
  }
  protected boolean isBig() {
    return state == big;
  }
  protected void smallState() { state = small; }
  protected void bigState() { state = big; }

  public Color subAnnoColor(Color color) { return determineColor(); } // TODO PTZ check this!
  
  protected boolean completelyGrown;
  public boolean isCompletelyGrown() { return completelyGrown; }
  protected Animation growAnimation;
  public void animationDone(Animation anim) {
    if ((anim == growAnimation) && (anim.completed())){
      completelyGrown = true;
      repaint();
    }
  }

  public void drawSelf(Graphics g) {
		/*
		if (FluidUI.fluidLog.logDataP()) {
			String details = getLogID() +
											 " LRTB: " + getLocationStr() +
											 " " + getLogText();
			FluidUI.fluidLog.addLog("ANNO", details, false);
		}
		*/

    if (((TextBlock)(getParent().getPrimary())).isCompletelyGrown()) {
      drawAnchorUnderline(g);

      /* draw bounding boxes for visual debugging */
      /*
      if (isBig()) {
        g.setColor(Color.darkGray);
        Rectangle s = new Rectangle(smallBounds);
        g.drawRect(s.x, s.y, s.width, s.height);
        g.setColor(Color.gray);
        Rectangle b = bigBoundsWithNested();
        g.drawRect(b.x, b.y, b.width, b.height);
      }
      */
    }

    if (tooltip() && isBig()) {
      Rectangle b = tooltipBounds();
      g.setColor(AnnoDoc.tooltipcolor);
      g.fillRect(b.x, b.y, b.width, b.height);
      g.setColor(AnnoDoc.tooltipborder);
      g.drawRect(b.x, b.y, b.width, b.height);
    }
  }

  protected Rectangle tooltipBounds() {
    return new Rectangle(expandedX, expandedY,
        fullSize.width + AnnoDoc.tooltipleftmargin + AnnoDoc.tooltiprightmargin,
        fullSize.height + AnnoDoc.tooltiptopmargin + AnnoDoc.tooltipbottommargin);
  }

  public Color determineColor() {
    return determineColor(false);
  }

  public Color determineColor(TextLine tl) {
    return determineColor(true);
  }

  public Color determineColor(boolean isTextLine) {
	  if (frozen) {
      if (!overlappedByIsEmpty()) {
        return color;
      } else if (traversingLink) {
        return lastTraversedColor;
      } else if (lastTraversed) {
		    return lastTraversedColor;
  		} else if (traversed) {
		    return frozenTraversedColor;
		  } else {
  		  return frozenColor;
	  	}
    } else if (traversingLink) {
      return lastTraversedColor;
	  } else if (traversed && !isTextLine) {
		  return traversedColor;
	  } else { // normal state
		  return annoColor;
	  }
  }

  protected void drawAnchorUnderline(Graphics g) {

    Color col;

    // right way to do this is just to do a setColor() when traversing
    // or lastTraversed. hold off on doing this until we decide for certain
    // what the right color behavior is.
    if (traversed) {
      if (!overlappedByIsEmpty()) {  // grayout; anno is open
        col = color;
      } else if (!((TextLine)getParent()).getColor().equals(Color.black)) {
        // grayout; anno is closed, so much check parent textline
        col = ((TextLine)getParent()).getColor();
      } else {
        col = traversedColor;
      }
    } else {
      col = ((TextBlock)getParent().getPrimary()).subAnnoColor(color);
    }
    
    // BEGIN MOD
    g.setColor(AspectJDemoConfig.ANNOTATION_UNDERLINE_COLOR);
    // END MOD
    
    //g.setColor(new Color(0,150,0));
    /*
    if (isSmall()) {
      // my own shameless hack
      if (determineColor().equals(annoColor)) { // no additional information; don't drag eye here
	      g.setColor( ((TextBlock)getParent().getPrimary()).subAnnoColor(color) );
		  } else {
        g.setColor(determineColor());
  		}
    } else {
      if (nogrow) {
        g.setColor(nogrowColor);
      } else if (!frozen) {
        g.setColor(determineColor());
      } else {
        // shameless hack to allow the frozen color to be grayed out
        if (color.equals(Color.black)) {
          g.setColor(determineColor());
        } else {
          g.setColor(color);
        }
      }
    }
    */


    g.drawLine(0, underlineY, localEndpointX(), underlineY);

    /*
    if (isBig()) {
    g.setColor(Color.green);
    g.drawRect(smallBounds.x, smallBounds.y, smallBounds.width, smallBounds.height);
    g.setColor(Color.blue);
    g.drawRect(bigBounds().x, bigBounds().y, bigBounds().width, bigBounds().height);
    }
    */

  }

  private boolean animationDone() {
    return ((currentAnimation == null) || currentAnimation.isDone());
  }

  public void drawComponents(Graphics g) {
    if (drawc < 0) {
      super.drawComponents(g);
    } else {
      for (int i = 0; i < drawc; i++) {
        getLine(i).draw(g);
      }
      // draw ellipses
      if ((drawc > 0) && (drawc < getLineCount())) {
        TextLine line = getLine(drawc - 1);
        if (line.getColor().equals(Color.white)) return;
        int w = line.getWidth();
        g.setFont(line.getFont());
        g.setColor(line.getColor());
        g.drawString("...", (w + line.absoluteX()) - absoluteX(),
                     (line.absoluteY() - absoluteY()) + line.getAscent());
      }
    }
  }

  protected Rectangle absoluteSmallBounds() {
    Rectangle b =  new Rectangle(smallBounds);
    b.translate(absoluteX(), absoluteY());
    return b;
  }

  public Rectangle absoluteBigBounds() {
    if (margincallout()) {
      return calloutAbsoluteOrphanBounds;
    }
    Rectangle b =  new Rectangle(bigBounds());
    b.translate(absoluteX(), absoluteY());
    return b;
  }

  public Rectangle bigBounds() {
    int x, y, w, h;
    if (footnote() || tooltip()) {
      return smallBounds;
    }
    if (margincallout()) {
      if (isBig()) {
        if (calloutAbsoluteOrphanBounds == null) {
          System.out.println("calloutAbsoluteOrphanBounds is null in BigBounds()");
          return smallBounds;
        }
        Rectangle b = new Rectangle(calloutAbsoluteOrphanBounds);
        b.translate(-absoluteX(), -absoluteY());
        return b;
      } else {
        return smallBounds;
      }
    }
    y = underlineY;
    h = fullSize.height + offset;
    if (AnnoDoc.pyramidLayout()) {
      // subtracting getX() changes it from parent's coords into this coordinate system
      x = annostartx - ((getLineCount() - 1) * AnnoDoc.pyramidstep) - getX();
      w = Math.max(fullSize.width, underlineWidth() - x);

    } else { // default (and also for wideLayout -- though it's not perfect)
      x = Math.min(0, expandedX);
      // x is 0 or negative; so underlineWidth() - x is >= underlineWidth()
      w = Math.max(fullSize.width, underlineWidth() - x);
      // in case expandedX is positive -- can happen in reuse
      w = Math.max(w, fullSize.width + expandedX);
      if (usefreezebutton) {
        w += freezeButton.offset();
        /* works for vertical overshoot -- BUT others (like reusespace) use
           bigbounds, and then end up getting a one-line too-high height
        if (AnnoDoc.freezebuttonverticalovershoot) {
          y = 0;
          h += underlineY;
        }
        */
      }
    }
    return new Rectangle(x, y, w, h);
  }

  protected int localEndpointX() { return endpoint.x - getX(); }
  protected int absoluteX2() { return endpoint.x + parent.absoluteX(); }

  int underlineWidth() { return absoluteX2() - absoluteX(); }

  // used in pyramidlayout
  // set in calculateFullSize()
  // this value is in terms of the parent coordinate system
  protected int annostartx;

  public Dimension calculateFullSize() {
    int w = 0;
    if (AnnoDoc.pyramidLayout()) {
      int anchorstart = getCharLocation(startchar).x - getParent().absoluteX();
      annostartx = anchorstart + defaultOffsetFromAnchor();
      int firstLineWidth = topLevelTextWidth() - annostartx;
      w = ((getLineCount() - 1) * AnnoDoc.pyramidstep) + firstLineWidth;
    } else { // default (also approximately correct for wideLayout)
      for (int i = 0; i < getLineCount(); i++) {
        w = Math.max(w, getLine(i).getWidth());
      }
    }
    int h = getLineCount() * lineSpacing;
    return new Dimension(w, h);
  }

  public void negotiationSelectPresentation(Negotiation ngt) {
    ngt.addAnimation(changeSize(ngt.offeredSize));
    if (ngt.offeredSize.equals(ngt.desiredSize)) {
      drawc = -1;
      growLineSpacing = fullSizeLineSpacing;
      growFontSize = fullSizeFontSize;
    } else {
      // only handles vertical amount right now
      if (AnnoDoc.technique.equals("move")) {
        drawc = -1;
        double f = (double)ngt.offeredSize.height / ngt.desiredSize.height;
        growLineSpacing = Math.max((int)Math.round(f * fullSizeLineSpacing), 0);
        growFontSize = Math.max((int)Math.round(f * fullSizeFontSize), minFontSize);
        if (growLineSpacing < minLineSpacing) {
          drawc = (int)((double)ngt.offeredSize.height / minLineSpacing);
          growLineSpacing = (int)((double)ngt.offeredSize.height / drawc);
        }
        if (drawc == 0) {
          nogrow = true;
        }
      } else if (AnnoDoc.technique.equals("move2")) { 
        growLineSpacing = fullSizeLineSpacing;
        growFontSize = fullSizeFontSize;
        drawc = ngt.offeredSize.height / growLineSpacing;
        if (drawc == 0) {
          nogrow = true;
        }
      } else if (AnnoDoc.technique.equals("moveonly")) {
        nogrow = true;
        // would use ngt.usedSize if it existed
        ngt.offeredSize.height = 0;
        drawc = 0;
      } else if (AnnoDoc.technique.equals("movenolimit")) {
        // should never happen, because movenolimit allows infinite growth
        System.out.println("error - movenolimit did not give desired size");
      } else if (AnnoDoc.technique.equals("margincallout")) {
        
      } else { // for compress-move and compress-overlay and compress-move-overlay
        growLineSpacing = fullSizeLineSpacing;
        growFontSize = fullSizeFontSize;
        drawc = -1;
      }
    }
  }

  public void moveExpandedX(int n) {
    expandedX += n;
  }

  public void moveExpandedXToDefault() {
    expandedX = expandedXDefault;
  }

  public AnimationGroup changeExpandedXToDefault() {
    moveExpandedXToDefault();
    return changeToExpandedX();
  }

  public AnimationGroup changeExpandedX(int n) {
    moveExpandedX(n);
    return changeToExpandedX();
  }

  protected AnimationGroup changeToExpandedX() {
    AnimationGroup ag = new AnimationGroup();
    int newx = expandedX;
    if (usefreezebutton) {
      newx += freezeButton.offset();
      ag.add(freezeButton.changeX(expandedX));
    }
    for (int i = 0; i < getLineCount(); i++) {
      ag.add(getLine(i).changeX(newx));
    }
    return ag;
  }

  public AnimationGroup changeExpandedColorToFrozen() {
    return changeExpandedColor(frozenColor);
  }

  public AnimationGroup changeExpandedColorToAnno() {
    return changeExpandedColor(annoColor);
  }
  
  /*
  public AnimationGroup changeExpandedColorToFaded() {
    int rgb = frozenColor.getRGB();
    int r = (rgb & 0xff0000) >> 16;
    int g = (rgb & 0xff00) >> 8;
    int b = (rgb & 0xff);
    float[] hsb = new float[3];
    Color.RGBtoHSB(r, g, b, hsb);
    Color faded = Color.getHSBColor(hsb[0], hsb[1] / 100, (float)1.0);
    return changeExpandedColor(faded);
  }
  */
  public AnimationGroup changeExpandedColor(Color col) {
    AnimationGroup ag = new AnimationGroup();
    for (int i = 0; i < getLineCount(); i++) {
      ag.add(getLine(i).changeColor(col));
    }
    if (usefreezebutton) {
      ag.add(freezeButton.changeColor(col));
    }
    if (margincallout()) {
      if (cline != null) { // sanity check
        ag.add(cline.changeColor(col));
      }
    }
    return ag;
  }

  protected Vector overlappedBy = new Vector();
  public void addToOverlappedBy(Annotation anno) { overlappedBy.addElement(anno); }
  public void removeFromOverlappedBy(Annotation anno) { overlappedBy.removeElement(anno); }
  public boolean overlappedByIsEmpty() { return overlappedBy.isEmpty(); }

  // the annos that this is overlapping
  protected Vector annosIOverlap = new Vector();
  public void addToAnnosIOverlap(Annotation anno) { annosIOverlap.addElement(anno); }
  public void removeFromAnnosIOverlap(Annotation anno) { annosIOverlap.removeElement(anno); }
  public boolean annosIOverlapIsEmpty() { return annosIOverlap.isEmpty(); }

  public void removeFromExpandedAnnos() {
    ((Page)getPage()).removeFromExpandedAnnos(this);
  }
  public void addToExpandedAnnos() {
    ((Page)getPage()).addToExpandedAnnos(this);
  }

  // for nested annotations, we only support move policy for now
  boolean moveonly() { return true; }
  
  // and we implement our own move policy, that only moves downward
  public boolean MovePolicy(Negotiation ngt, boolean execute) {
    return MoveDownwardPolicy(ngt, execute);
  }
  
  public boolean MoveDownwardPolicy(Negotiation ngt, boolean execute) {
    // we're not checking yet whether there's enough space, so always return true
    if (!execute) return true;
    int dy = ngt.offeredSize.height;
    for (int i = 0; i < getLineCount(); i++) {
      int newy;
      if (i <= ((TextLine)ngt.secondary.getParent()).getLineNumber()) {
        // do nothing
      } else {
        newy = getLine(i).getY() + dy;
        ngt.addAnimation(getLine(i).changeY(newy));
      }
    }
    return true;
  }
  
  Vector nestedAnnotations = new Vector();
  Hashtable nestedAnnoSet = new Hashtable();
  protected void addNestedAnno(FluidComponent anno) {
    nestedAnnotations.addElement(anno);
    nestedAnnoSet.put(anno, anno);
  }
  protected void removeNestedAnno(FluidComponent anno) {
    nestedAnnotations.removeElement(anno);
    nestedAnnoSet.remove(anno);
  }

  
 
  public void negotiationSetGuidelines(Negotiation ngt) {
    // for nested annotations:
    // for now, only support move policy;
    
    // we cascade the move into the textblock parent

    TextGChar gchar = new TextGChar();
    gchar.setSize(ngt.desiredSize);
    ngt.subngt = new Negotiation(this, getParent().getPrimary(), gchar);


    //ngt.animation = ngt2.animation;
    ngt.animation.add(ngt.subngt.animation);

    // if the main technique is overlay, do the move downwards only
    ngt.offeredSize = ngt.desiredSize;
    // openedAnnos.addElement(ngt);

    // as we dive deeper and deeper into nested annotations,
    // annotations in the chain will accumulate multiple copies of
    // their directly nested annos in nestedAnnotations.
    // typically, if anno A has nested anno B, and B has C, and C has D,
    // by the time we open up D, A's nestedAnnotations will contain
    // three elements, all B: {B, B, B}.
    // this is used as each level is reverted.
    addNestedAnno(ngt.secondary);

  }
  
  
  public void negotiationRevert(Negotiation ngt, double frac) {
    ngt.offeredSize = new Dimension(-ngt.offeredSize.width, (int)(-ngt.offeredSize.height * frac));
    MovePolicy(ngt);
    
    ngt.subngt.revert();
    ngt.animation.add(ngt.subngt.animation);

    removeNestedAnno(ngt.secondary);
  }

  int growLineSpacing, growFontSize;

  public boolean reconstructOnGrow = false;

  protected void grow() {
    grow(0, 0);
  }

  protected void grow(int mx, int my) {
    if (nogloss()) return;

		if (FluidUI.fluidLog.logDataP()) {
			String details = getLogID() + " " + mx + " " + my + " " + getLogText();
			FluidUI.fluidLog.addLog("GROW", details, true);
		}

    if (margincallout()) {
      growmargincallout();
      return;
    }

    if (tooltip()) {
      growtooltip(mx, my);
      return;
    }

    if (footnote()) {
      growfootnote();
      return;
    }

    if (frozen) return;

    // this has to follow all shrink() animations! that's why it's here.
    if (reconstructOnGrow) {
      reconstructTextLines();
      setLinePositions();
    }

    setTopDrawn(true); // ensures that, in overlay, this appears above all other expanded annos
    bigState();
    TextGChar gchar = new TextGChar();
    gchar.setSize(fullSize);
    negotiation = new Negotiation(this, getParent().getPrimary(), gchar);
    if (!nogrow) {
      int newx = 0;

      for (int i = 0; i < getLineCount(); i++) {
        negotiation.animation.add(getLine(i).changeLineSpacing(growLineSpacing));
        negotiation.animation.add(getLine(i).changeFontSize(growFontSize));
        negotiation.animation.add(getLine(i).changeColor(annoColor));
        if ((expandedX != 0) || usefreezebutton){
          if (AnnoDoc.wideLayout()) {
            newx = (i == 0) ? expandedX + firstLineIndent() : expandedX;
          } else if (AnnoDoc.pyramidLayout()) {
            newx = defaultOffsetFromAnchor() - (AnnoDoc.pyramidstep * i);
            newx = Math.max(newx, expandedX); // we're dealing w/ negative numbers
          } else {
            newx = expandedX;
          }
          if (usefreezebutton) {
            newx += freezeButton.offset();
          }
        }
        if (newx != 0) {
          negotiation.animation.add(getLine(i).changeX(newx));
        }
      }
      if (usefreezebutton) {
        // assumes we are using standard layout, not wide or pyramid
        negotiation.animation.add(freezeButton.changeX(expandedX));
        negotiation.animation.add(freezeButton.changeColor(annoColor));
        negotiation.animation.add(freezeButton.changeDrawnSize(freezeButton.bigSize));
      }
    }
    negotiation.animation.setDuration(animduration);
    growAnimation = negotiation.animation;
    negotiation.animation.addAnimationListener(this);
    startAnimation(negotiation.animation);
    currentAnimation = negotiation.animation;
    addToExpandedAnnos();
  }

  protected void shrink() {
    if (nogloss()) return;

		if (FluidUI.fluidLog.logDataP()) {
			String details = getLogID() + " " + getLogText();
			FluidUI.fluidLog.addLog("SHRINK", details, true);
		}

    if (margincallout()) {
      shrinkmargincallout();
      return;
    }

    if (tooltip()) {
      shrinktooltip();
      return;
    }

    if (footnote()) {
      shrinkfootnote();
      return;
    }

    nogrow = false;
    if (frozen) return;

    completelyGrown = false; // for nested annotations, remove the underline

    removeFromExpandedAnnos();
    setTopDrawn(false);
    // close up the text lines
    boolean dorevert = negotiation.revert();
    if (!dorevert) {
      return;
    }
    // now shrink the annotation
    for (int i = 0; i < getLineCount(); i++) {
      negotiation.animation.add(getLine(i).changeLineSpacing(0));
      negotiation.animation.add(getLine(i).changeFontSize(3));
      negotiation.animation.add(getLine(i).changeColor(Color.white));
      if (getLine(i).getX() != 0) {
        negotiation.animation.add(getLine(i).changeX(0));
      }
    }
    if (usefreezebutton) {
      negotiation.animation.add(freezeButton.changeX(freezeButton.getButtonX()));
      negotiation.animation.add(freezeButton.changeColor(Color.white));
      negotiation.animation.add(freezeButton.changeDrawnSize(freezeButton.smallSize));
    }
    for (int i = 0; i < overlappedBy.size(); i++) {
      Annotation anno = (Annotation)overlappedBy.elementAt(i);
      anno.removeFromAnnosIOverlap(this);
    }
    overlappedBy.removeAllElements();
    negotiation.animation.setDuration(animduration);
    ((Page)getPage()).revertingAnimation = negotiation.animation;

    if (AnnoDoc.hasNesting) {
      negotiation.animation.addAnimationListener(new AnimationListener() {
        public void animationDone(Animation anim) {
          nextRevertQueue();
        }
      });
    }

    startAnimation(negotiation.animation);
    currentAnimation = negotiation.animation;
    smallState();

  }

  // called ONLY from the animationDoneListener
  public void nextRevertQueue() {
    Vector q = ((Page)getPage()).revertQueue;
    synchronized(q) {
      if (!q.isEmpty()) { // need to worry about this in case it gets removed elsewhere, in leave() for example
        q.removeElementAt(0); // remove this shrinking guy
      }
      if (!q.isEmpty()) {
        Annotation anno = (Annotation)q.firstElement();
        anno.leave();
      } else {
        // q is empty, so check if there's something to expand
        ((Page)getPage()).revertDone();
      }
    }
  }

  public boolean governs(int x) {
    return (absoluteX() <= x) && (x <= absoluteX2());
  }

  public boolean inBounds(Point p) {
    Rectangle s = new Rectangle(smallBounds);
    s.translate(absoluteX(), absoluteY());
    if (isSmall()) {
      // "small" inbounds check happens only when shrinking animation is done.
      // so if we are in the process of shrinking we check the "big bounds".
      // this ensures that enter/leave check doesn't regard the cursor to be
      // leaving the annotation during the shrinking. Thus the dwellCheck will
      // not be turned on, and dwells while we are still in the now-shrunken
      // annotation will not trigger a grow.
      return s.contains(p);
    } else {
      Rectangle b =  new Rectangle(bigBounds());
      if (nestedAnnotations.isEmpty()) {
        b.translate(absoluteX(), absoluteY());
        return s.contains(p) || b.contains(p);
      } else {
        Rectangle bn = bigBoundsWithNested();
        bn.translate(absoluteX(), absoluteY());
        return s.contains(p) || bn.contains(p);
      }
    }
  }

  // this still has a bug making nested annos much wider (on the right side)
  // than they should be.
  // also bug: it doesn't expand the bigboundwithnested if the nested anno
  // goes further to the left
  public Rectangle bigBoundsWithNested() {
    int dy = 0;
    Rectangle b = new Rectangle(bigBounds());
    int x0 = b.x;
    int x1 = b.x + b.width;

    for (Enumeration enum = nestedAnnoSet.elements(); enum.hasMoreElements(); ) {
      Annotation anno = (Annotation)enum.nextElement();
      Rectangle r = anno.bigBoundsWithNested();
      x0 = Math.min(x0, r.x);
      x1 = Math.max(x1, r.x + r.width);
      dy += r.height;
    }
    return new Rectangle(x0, b.y, x1 - x0, b.height + dy);

  }

  public String toString() {
    if (false) {
      StringBuffer sb = new StringBuffer();
      for (int i = 0; i < getLineCount(); i++) {
        sb.append(getLine(i).getText() + "\n");
      }
      return "Annotation[" + sb + "]";
    } else {
      return "anno(" + getLine(0).getText() + ") [" + nestedAnnotations.size() + "]";
    }
  }

  protected boolean traversingLink = false;
  protected Color pretraversingColor = null;

  protected void setColorToFrozen() {
    for (int i = 0; i < getLineCount(); i++) {
      getLine(i).setColor(frozenColor);
    }
    if (usefreezebutton) {
      freezeButton.setColor(frozenColor);
    }
    if (margincallout()) {
      if (cline != null) {
        cline.setColor(frozenColor);
      }
    }
  }

  protected void setColorToAnno() {
    for (int i = 0; i < getLineCount(); i++) {
      getLine(i).setColor(annoColor);
    }
    if (usefreezebutton) {
      freezeButton.setColor(annoColor);
    }
    if (margincallout()) {
      if (cline != null) {
        cline.setColor(annoColor);
      }
    }
  }

  protected void setColorToTraversing() {
    if (getLineCount() > 0) {
      pretraversingColor = getLine(0).getColor();
    }
    if (pretraversingColor == Color.white) {
      pretraversingColor = null;
      return;
    }
    for (int i = 0; i < getLineCount(); i++) {
      getLine(i).setColor(lastTraversedColor);
    }
    if (usefreezebutton) {
      freezeButton.setColor(lastTraversedColor);
    }
    if (margincallout()) {
      if (cline != null) {
        cline.setColor(lastTraversedColor);
      }
    }
  }

  protected void setColorBackFromTraversing() {
    if (pretraversingColor != null) {
      for (int i = 0; i < getLineCount(); i++) {
        getLine(i).setColor(pretraversingColor);
      }
      if (usefreezebutton) {
        freezeButton.setColor(pretraversingColor);
      }
      if (margincallout()) {
        if (cline != null) {
          cline.setColor(pretraversingColor);
        }
      }
    }
    pretraversingColor = null;
  }


  public int hackx1, hackx2, hacky1, hacky2;
  public Annotation hackother;
  public boolean hackstop = false;

  public void hackMouseDown(Event evt, int x, int y) {
    if (!hackstop) {
      hackother.hackstop = true;
      mouseDown(evt, x, y);
    }
    hackother.hackstop = false;
    hackstop = false;
  }

  public void hackMouseUp(Event evt, int x, int y) {
    if (!hackstop) {
      hackother.hackstop = true;
      mouseUp(evt, x, y);
    }
    hackother.hackstop = false;
    hackstop = false;
  }

  public void hackMouseUpNotice() {
    if (!hackstop) {
      hackother.hackstop = true;
      mouseUpNotice();
    }
    hackother.hackstop = false;
    hackstop = false;
  }

  public void hackRightMouseDown(Event evt, int x, int y) {
    if (!hackstop) {
      hackother.hackstop = true;
      rightMouseDown(evt, x, y);
    }
    hackother.hackstop = false;
    hackstop = false;
  }


  public void mouseDown(Event evt, int x, int y) {
    disallowDwellCheck();

    if ((hackother != null) && ((Page)getPage()).isOrphan(this)) {
      if (((x < hackx1) || (x > hackx2) || (y < hacky1) || (y > hacky2)) &&
          (!absoluteSmallBounds().contains(x, y))) {
        hackother.hackMouseDown(evt, x, y);
        return;
      }
    }

    if (!href.equals("")) {
      FluidUI.setLastMouseDownComponent(this);
      if ((currentAnimation == null) ||
          ((currentAnimation != null) && currentAnimation.isDone())) {
        if (isBig()) {
          setTraversing();
          repaint();
        }
      } else { // there is a current animation
        currentAnimation.addAnimationListener(new AnimationListener() {
          public void animationDone(Animation anim) {
            if (isBig()) {
              setTraversing();
              repaint();
            }
          }
        });
      }
    }
  }

  public void setTraversing() {
    lastTraversed = true;
    traversingLink = true;
    setColorToTraversing();
  }

  protected boolean followedAgain = false;

  public void mouseUp(Event evt, int x, int y) {  // follow link
    disallowDwellCheck();

    if ((hackother != null) && ((Page)getPage()).isOrphan(this)) {
      if (((x < hackx1) || (x > hackx2) || (y < hacky1) || (y > hacky2)) &&
          (!absoluteSmallBounds().contains(x, y))) {
        hackother.hackMouseUp(evt, x, y);
        return;
      }
    }

    if (AnnoDoc.clickToOpen()) {
      rightMouseDown(evt, x, y);
      return;
    }

    if (!href.equals("")) {
      if (FluidUI.lastMouseDownComponentEquals(this)) {
        followedAgain = true; // coordinate with returnFromLink()

      //when a link is taken:
      //    freeze this anno (and all of its ancestors) and change its color to lastTraversedColor
      //    change the color of the previous lastAnnoTraversed, if it is currently expanded
      //            and is not this same anno, to traversedColor or frozenTraversedColor
      //do not animate; wait for next draw (at link return)
        FluidUI.clearLastMouseDownComponent();
        setLastTraversed();

        // check for isBig before making it frozen
        // thus if we click on a link without dwelling long enough to expand it,
        // it doesn't get set to frozen state
        if (!noFreeze() && isBig()) {
  	      //if this anno is nested, freeze all ancestors also

    	    setfrozen(); // freeze self
          TextBlock parent = (TextBlock)(getParent().getPrimary());
 	        while (parent.childIsNested()) {
  	        Annotation anno = (Annotation)parent;
  	        anno.setfrozen();
       		  parent = (TextBlock)(parent.getParent().getPrimary());
          }
        }
        ((Page)getPage()).jump(href);
      }
    }
  }

  // this means that mouseup occurred over some other component
  public void mouseUpNotice() {
    if ((currentAnimation != null) && currentAnimation.isDone()) {
      traversingLink = false;
      if (isBig()) {
        if (frozen) {
          setColorToFrozen();
        } else {
          setColorToAnno();
        }
        repaint();
      }
    } else {
      currentAnimation.addAnimationListener(new AnimationListener() {
        public void animationDone(Animation anim) {
          traversingLink = false;
          if (isBig()) {
            if (frozen) {
              setColorToFrozen();
            } else {
              setColorToAnno();
            }
            repaint();
          }
        }
      });
    }
  }

  public void clearLastTraversed() {
    lastTraversed = false;
    if (isBig()) {
      setColorBackFromTraversing();
    }
  }

  public void setLastTraversed() {
 		Annotation lastAnnoTraversed = ((Page)getPage()).getLastAnnoTraversed();
   	if (lastAnnoTraversed != null) {
  	 	lastAnnoTraversed.clearLastTraversed();
  	}
    ((Page)getPage()).setLastAnnoTraversed(this);
    lastTraversed = true;
    setTraversed(true);
  }

  public void returnFromLink() {
    cancelReturnFromLink = false;
    followedAgain = false;
    TimerListener listener = new TimerListener() {
      public void timerExpired(int id, Object obj) {
        traversingLink = false;
        if (isBig() && !followedAgain) { // should synchronize to avoid race
          if (saveFrozen) {
            freeze(); // changes color to frozen
          } else {
            if (cancelReturnFromLink) { // should synchronize to avoid race
              unfreezeWithoutShrinking();
            } else {
              unfreeze();
//      	    ((Page)getPage()).unsetExpandedAnno();
            }
          }
        }
      }
    };
    new Timer(listener, AnnoDoc.linkreturnpause, 0, null);
  }

  public Annotation unfreezeNested() {
    clearfrozen();
    Vector q = ((Page)getPage()).revertQueue;
    synchronized (q) {
      q.insertElementAt(this, 0);
    }
    Annotation returnAnno = this;
    Vector visited = new Vector();
    for (int i = 0; i < nestedAnnotations.size(); i++) {
      Annotation anno = (Annotation)nestedAnnotations.elementAt(i);
      // because nestedAnnotations can have multiple copies of a given anno
      if (!visited.contains(anno)) {
        returnAnno = anno.unfreezeNested();
        visited.addElement(anno);
      }
    }
    return returnAnno;
  }

  protected boolean noFreeze() {
    return tooltip() || footnote() || nogloss();
  }


  public void rightMouseDown(Event evt, int x, int y) {  // freeze/thaw toggle
    if ((hackother != null) && ((Page)getPage()).isOrphan(this)) {
      if (((x < hackx1) || (x > hackx2) || (y < hacky1) || (y > hacky2)) &&
          (!absoluteSmallBounds().contains(x, y))) {
        hackother.hackRightMouseDown(evt, x, y);
        return;
      }
    }

    toggleFreeze(true);
  }

  public void toggleFreeze(boolean shrinkOnUnfreeze) {
    if (noFreeze()) {
      return;
    }

    disallowDwellCheck();

    if (frozen) {
      if (shrinkOnUnfreeze) {
        unfreeze();
      } else {
        unfreezeWithoutShrinking();
      }
      if (usefreezebutton) {
        freezeButton.setOff();
      }
//	    ((Page)getPage()).unsetExpandedAnno();
    } else {
      // put this into freeze()! or something.
      if (AnnoDoc.singleAnnoPath && (((Page)getPage()).isAnnoOpenAt(nestingLevel()))) {
        ((Page)getPage()).addRevertListener(new AnimationListener() {
          public void animationDone(Animation anim) {
            growFreeze();
          }
        });
        ((Page)getPage()).getOpenAnnoAt(nestingLevel()).unfreeze();
      } else {
        growFreeze();
      }
      if (usefreezebutton) {
        freezeButton.setOn();
      }
    }
  }

  public void growFreeze() {
    if (margincallout()) {
      if (isSmall()) {
        growmargincallout(frozenColor);
      } else {
        setColorToFrozen();
      }
      setfrozen();
    }
    if (isSmall()) {
//      ((Page)getPage()).setExpandedAnno(this);
      grow();
    }
    freeze();
  }

  protected void freeze() {
    if (noFreeze()) {
      return;
    }

    setfrozen(); // freeze self
    saveFrozen = true;

		if (FluidUI.fluidLog.logDataP()) {
			String details = getLogID() + " " + getLogText();
			FluidUI.fluidLog.addLog("FREEZE", details, true);
		}

    if (!AnnoDoc.clickToOpen()) {
      AnimationGroup animation = changeExpandedColorToFrozen();
      //if this anno is nested, freeze all ancestors also
      TextBlock parent = (TextBlock)(getParent().getPrimary());
      while (parent.childIsNested()) {
        Annotation anno = (Annotation)parent;
        anno.setfrozen();
        animation.add(anno.changeExpandedColorToFrozen());
        parent = (TextBlock)(parent.getParent().getPrimary());
      }
      animation.setDuration(animduration);
      startAnimation(animation);
      currentAnimation = animation;
    }
  }

  /*
  public void mouseMove(Event evt, int x, int y) {
    if (tooltip() && isBig()) {
      leave();
      allowDwellCheck();
    }
  }
  */

  // be careful about what you put in here. mouseLeave() gets called when you do a jump!
  // probably should check whether we are in the same page as is being displayed
  // but for now everything seems to work, so let's not mess with it.
  public void mouseLeave(Event evt, int x, int y,
                         FluidComponent originatingComp, FluidComponent enteredComp) {
    if (AnnoDoc.clickToOpen()) return;

    // be sure to check for this before putting this anno on the revert queue!
    if (frozen) return;

    if (AnnoDoc.hasNesting) {
      Vector q = ((Page)getPage()).revertQueue;

      // first thing on q is always the active shrinking anno
      synchronized(q) {
        if (!isBig()) {
          return;
        }
        q.addElement(this);
        if (q.size() == 1) {
          // if there's no one else slated to shrink yet, then begin shrinking
          // this only happens if we're leaving a non-nested anno, or if we're leaving a
          // the most deeply nested anno
          leave();
        } else {
          // we've left an anno that is in some chain of nested annos (which have already been left)
          Annotation anno = (Annotation)q.elementAt(0);
          // this is safe from a race condition on isDone(). that's because nextRevertQueue()
          //   is guaranteed to be called AFTER the current animation sets its done flag --
          //   that's an invariant in Animation
          if ((anno.currentAnimation == null) || (anno.currentAnimation.isDone())) {
            q.removeElementAt(0);
            ((Annotation)q.elementAt(0)).leave();
          }
        }
      }
    } else {
      leave();
    }
  }

  public void unfreeze() {
    if (frozen) {
			if (FluidUI.fluidLog.logDataP()) {
				String details = getLogID() + " " + getLogText();
				FluidUI.fluidLog.addLog("THAW", details, true);
			}
      saveFrozen = false;
      if (AnnoDoc.hasNesting) {
        unfreezeNested().shrink();
      } else {
        clearfrozen();
        shrink();
  	  }
    }
  }

  // should add LOG THAW here!
  // not clear how this works with nested annos - untested
  public void unfreezeWithoutShrinking() {
    if (frozen) {
      clearfrozen();
      saveFrozen = false;
      AnimationGroup animation = changeExpandedColorToAnno();
      animation.setDuration(animduration);
      startAnimation(animation);
      currentAnimation = animation;
    }
  }

  public void leave() {
    if (frozen) return;
    if (isBig()) {
      shrink();

    }
  }


  protected boolean cancelReturnFromLink = false;

  public void mouseDwell(int x, int y) {
    if (AnnoDoc.clickToOpen()) return;
    if (frozen) {
      cancelReturnFromLink = true;
      return;
    }

    if (isBig()) return;

//    Annotation exp = ((Page)getPage()).getExpandedAnno();
//    if (exp != this) {
//      ((Page)getPage()).setExpandedAnno(this);
      if (animationDone()) { // need to do this check because if shrink doesn't finish,
                             // the grow will widen the lines an amount that is added
                             // to the amount left over by the unfinished shrink
        grow(x, y);
      }
//    }
  }

  public void growtooltip(int x, int y) {
    if (frozen) return;

    // this has to follow all shrink() animations! that's why it's here.
    if (reconstructOnGrow) {
      reconstructTextLines();
      setLinePositions();
    }

    setTopDrawn(true); // ensures that, in overlay, this appears above all other expanded annos
    bigState();
    if (!nogrow) {
      expandedY = (y - absoluteY()) + AnnoDoc.tooltipyoffset;
      expandedX = (x - absoluteX()) + AnnoDoc.tooltipxoffset;
      Rectangle ttb = tooltipBounds();
      int dx = ((Page)getPage()).getWidth() - (absoluteX() + ttb.x + ttb.width);
      if (dx < 0) {
        expandedX += dx - 10; // the -10 is so that it won't go all the way to the edge
      }
      int dy = ((Page)getPage()).getHeight() - (absoluteY() + ttb.y + ttb.height);
      if (dy < 0) {
        expandedY = getParent().absoluteY() - ttb.height - absoluteY();
        if (dx >= 0) {
          expandedX -= AnnoDoc.tooltipxoffset;
        }
      }


      for (int i = 0; i < getLineCount(); i++) {
        getLine(i).setOffset(expandedY + AnnoDoc.tooltiptopmargin);
        getLine(i).setLineSpacing(fullSizeLineSpacing);
        getLine(i).setFontSize(fullSizeFontSize);
        getLine(i).setColor(annoColor);
        getLine(i).setX(expandedX + AnnoDoc.tooltipleftmargin);
      }
    }
    addToExpandedAnnos();
    repaint();
  }

  public void shrinktooltip() {
    nogrow = false;
    if (frozen) return;

    completelyGrown = false; // for nested annotations, remove the underline

    removeFromExpandedAnnos();
    setTopDrawn(false);
    // close up the text lines
    // now shrink the annotation
    for (int i = 0; i < getLineCount(); i++) {
      getLine(i).setLineSpacing(0);
      getLine(i).setFontSize(3);
      getLine(i).setColor(Color.white);
      if (getLine(i).getX() != 0) {
        getLine(i).setX(0);
      }
    }
    smallState();
    repaint();
  }

  public void growfootnote() {
    if (frozen) return;

    // this has to follow all shrink() animations! that's why it's here.
    if (reconstructOnGrow) {
      reconstructTextLines();
      setLinePositions();
    }

    setTopDrawn(true); // ensures that, in overlay, this appears above all other expanded annos
    bigState();
    if (!nogrow) {
      for (int i = 0; i < getLineCount(); i++) {
        getLine(i).setLineSpacing(fullSizeLineSpacing);
        getLine(i).setFontSize(fullSizeFontSize);
        getLine(i).setColor(annoColor);
        if (expandedX != 0) {
          getLine(i).setX(expandedX);
        }
      }
    }
    addToExpandedAnnos();
    repaint();
  }

  public void shrinkfootnote() {
    nogrow = false;
    if (frozen) return;

    completelyGrown = false; // for nested annotations, remove the underline

    removeFromExpandedAnnos();
    setTopDrawn(false);
    // close up the text lines
    // now shrink the annotation
    for (int i = 0; i < getLineCount(); i++) {
      getLine(i).setLineSpacing(0);
      getLine(i).setFontSize(3);
      getLine(i).setColor(Color.white);
      if (getLine(i).getX() != 0) {
        getLine(i).setX(0);
      }
    }
    smallState();
    repaint();
  }

  CalloutLine cline = null;
  int platformMargin = 5;
  int calloutX, calloutY;
  int calloutTextX, calloutTextY;

  public Rectangle calloutAbsoluteOrphanBounds;
  public Rectangle getAbsoluteOrphanBounds() {
    return absoluteBigBounds();
  }

  protected final int CALLOUT_NEAREST = 0;
  protected final int CALLOUT_LEFT = 1;
  protected final int CALLOUT_RIGHT = 2;

  protected int otherside() {
    return otherside(actualcallout);
  }
  protected int otherside(int side) {
    if (side == CALLOUT_LEFT) {
      return CALLOUT_RIGHT;
    } else {
      return CALLOUT_LEFT;
    }
  }

  protected int preferredcallout;
  protected int actualcallout;

  protected void calculateCalloutSize() {
    calculateCalloutSize(CALLOUT_NEAREST);
  }

  protected void calculateCalloutSize(int whichmargin) {
    FluidComponent prim = getParent().getPrimary();
    int left = ((TextBlock)prim).getLeftCalloutX() - getX();
    int right = ((TextBlock)prim).getRightCalloutX() - getX();
    int x1, x2, x3, x4, w;
    // absoluteX() and absoluteY() is here because we are attaching the
    // calloutLine to the Page, rather than this Annotation.
    // we do this because Annotation, via TextBlock, iterates through
    // its components with getLine(), and casts the components to TextLines.
    // ie, it doesn't expect a non-TextLine component. That should be fixed.
    // 7/15/99: okay, it's fixed and used in the case of freezebuttons
    // but i haven't moved calloutlines over to this yet.

    if (whichmargin == CALLOUT_NEAREST) {
      if ((Math.abs(left) + getWidth()) < Math.abs(right)) {
        whichmargin = CALLOUT_LEFT;
        preferredcallout = CALLOUT_LEFT;
      } else {
        whichmargin = CALLOUT_RIGHT;
        preferredcallout = CALLOUT_RIGHT;
      }
    }

    if (whichmargin == CALLOUT_LEFT) {
      actualcallout = CALLOUT_LEFT;
      x1 = localEndpointX() + absoluteX();
      x2 = left + absoluteX();
      w = ((TextBlock)prim).getLeftCalloutWidth();
      x3 = left - platformMargin;
      x4 = x3 - w;
    } else {
      actualcallout = CALLOUT_RIGHT;
      x1 = 0 + absoluteX();
      x2 = right + absoluteX();
      x3 = right + platformMargin;
      x4 = x3;
      w = ((TextBlock)prim).getRightCalloutWidth();
    }

    setFontSize(AnnoDoc.calloutfontsize);
    setLineSpacing(AnnoDoc.calloutlinespacing);
    fullSizeFontSize = fontSize;
    fullSizeLineSpacing = lineSpacing;

// these seem to be handled during negotiationSelectPresentation in growmargincallout
    growLineSpacing = fullSizeLineSpacing;
    growFontSize = fullSizeFontSize;

    reconstructTextLines(w);
// 7/20/99: I don't think we need this anymore -- callouts no longer play with other techniques
    // 7/22/99: but we reuse uses reconstructongrow if it reflows the new anno
//    reconstructOnGrow = true;

    calloutY = underlineY + 1;
    setCalloutLinePositions(x3, calloutY); // this does a calculateFullSize()
    calloutX = x3;
    int h = fullSize.height;

    int calloutYAbsolute = calloutY + absoluteY();

    // make sure callout stays on screen
    int newY = calloutY - (h / 2);
    int newYAbsolute = newY + absoluteY();
    int tinymargin = 10;
    if (newYAbsolute < tinymargin) {
      newY = tinymargin - absoluteY();
      // if the callout is too tall for the page, you are out of luck right now
    } else if (newYAbsolute + h + tinymargin > getPage().getHeight()) {
      newY -= (newYAbsolute + h + tinymargin) - getPage().getHeight();
    } else {
      // normal case; don't change newY
    }

    newYAbsolute = newY + absoluteY();

    // the "+ 1" on newYAbsolute seems to adjust the vertical platform better
    cline = new CalloutLine(x1, x2, calloutYAbsolute, newYAbsolute + 1, h, annoColor);

    calloutTextX = x4;
    calloutTextY = newY;

    // move lines to the margin, so they grow up from the right place
    for (int i = 0; i < getLineCount(); i++) {
      getLine(i).setX(calloutX);
      getLine(i).setOffset(calloutY);
    }

    int orphanX = x4 + absoluteX();
    int orphanY = newY + absoluteY();
    calloutAbsoluteOrphanBounds = new Rectangle(orphanX, orphanY, fullSize.width, fullSize.height);
  }

  public void growmargincallout() {
    growmargincallout(annoColor);
  }
  public void growmargincallout(Color col) {
    setTopDrawn(true);
    bigState();

    // workaround the "orphaned platform" bug
    cline.resetPlatform();

    Vector ea = ((Page)getPage()).getExpandedAnnos();
    if (ea.size() == 1) {
      // if there's exactly one anno open, show on the other side
      Annotation anno = (Annotation)ea.elementAt(0);
      if (actualcallout == anno.actualcallout) {
        calculateCalloutSize(otherside());
      }
    } else {
      // in all other cases, show on preferred side
      if (actualcallout != preferredcallout) {
        calculateCalloutSize(otherside());
      }
    }

    TextGChar gchar = new TextGChar();
    gchar.setSize(fullSize);
    negotiation = new Negotiation(this, getParent().getPrimary(), gchar);

    getPage().add(cline);

    AnimationGroup ag = negotiation.animation;
    for (int i = 0; i < getLineCount(); i++) {
      ag.add(getLine(i).changeOffset(calloutTextY));
      ag.add(getLine(i).changeLineSpacing(growLineSpacing));
      ag.add(getLine(i).changeFontSize(growFontSize));
      ag.add(getLine(i).changeColor(col));
      ag.add(getLine(i).changeX(calloutTextX));
    }

    cline.setColor(col);
    cline.grow(ag);

    ((Page)getPage()).addOrphanAnno(this);
    addToExpandedAnnos();
  }

  public void shrinkmargincallout() {
    ((Page)getPage()).removeOrphanAnno(this);
    removeFromExpandedAnnos();
    setTopDrawn(false);

    boolean dorevert = negotiation.revert();

    AnimationGroup ag = negotiation.animation;
    for (int i = 0; i < getLineCount(); i++) {
      ag.add(getLine(i).changeOffset(calloutY));
      ag.add(getLine(i).changeLineSpacing(0));
      ag.add(getLine(i).changeFontSize(3));
      ag.add(getLine(i).changeColor(Color.white));
      ag.add(getLine(i).changeX(calloutX));
    }

    cline.shrink(ag);
    smallState();
    /* // this was here to make sure that callout played well with other annos
       // but now we no longer allow callout to mix w/ other annos; you have
       // to reload the page.
    setFontSize(AnnoDoc.calloutfontsize);
    setLineSpacing(AnnoDoc.calloutlinespacing);
    fullSizeLineSpacing = lineSpacing;
    */
  }
}

/* $Log: /FluidDocs/annotation/Annotation.java $
 * 
 * 37    8/31/99 4:19p Bchang
 * added set/get/anchorText().
 * 
 * 36    8/24/99 10:54a Bchang
 * added "followedAgain" and removed saveFrozen = frozen in
 * setLastTraversed() -- this is to fix the situation in which you open a
 * gloss, follow it, go back, follow it again before the auto-timer runs
 * out, and go back (bug was that it would then be frozen).
 * 
 * 35    7/26/99 11:02a Bchang
 * implement following behavior: if you follow a link, and then eventually
 * browse to that page (by following a circular set of links), if the
 * gloss is open it will "really" freeze (whether or not you originally
 * froze it when you followed the link). Also, if you unfreeze it, it will
 * also really unfreeze. This means that when you Back up to the original
 * time you saw the page, it will reflect what you most recently saw on
 * the page.
 * 
 * 34    7/26/99 9:35a Bchang
 * call resetPlatform() on calloutline in growmargincallout(). this is a
 * workaround for the "orphaned callout platform bug" if you
 * mousein/mouseout of the anchor very quickly (but not too quickly)
 * 
 * 33    7/23/99 1:42p Bchang
 * hack (hackx, hacky, hackother) to get the right anno to receive events
 * in the "reuse" condition -- when more than one anno opens up in the
 * same interline space
 * 
 * 32    7/23/99 9:08a Bchang
 * shared anchors (anchors that span more than one line) update their
 * traversed state together
 * absoluteOrphanBounds -> calloutAbsoluteOrphanBounds
 * 
 * 31    7/22/99 8:22p Bchang
 * 
 * 30    7/22/99 8:05p Bchang
 * fix changeToExpandedX to also move the freeze button.
 * margincallout: if there's only one callout on the screen, the new
 * callout will open in opposite margin
 * 
 * 29    7/22/99 10:05a Bchang
 * fixed: anchor underlines grayout again
 * 
 * 28    7/20/99 5:37p Bchang
 * 1. tooltip glosses stay up during dwell over the anchor (rather than
 * disappear on any mousemove)
 * 2. margincallouts start at the right place for their first animation,
 * rather than starting at the anchor
 * 3. footnote & tooltip anchor underlines seem to color correctly on
 * return from traversal
 * 
 * 27    7/20/99 9:44a Bchang
 * margin callout now grays out any overlapped callouts.
 * margin callout now does not play well with other annos -- to switch
 * between them, you have to reload the corpus
 *
 * 26    7/19/99 12:58p Bchang
 * AnnoDoc.what is removed in favor of AnnoDoc.technique
 * 
 * 25    7/16/99 3:24p Bchang
 * usefreezebutton set locally so that even if AnnoDoc.usefreezebutton is
 * true, margincallout, footnote, and tooltip never use it.
 * margin callout makes sure that callout stays on screen
 * 
 * 24    7/16/99 10:40a Bchang
 * use structureLoaded() to set traversed boolean for global visited links
 * margin callouts use Page's orphanAnnos so that they can be clicked on
 * when frozen
 * traversedColor is once again used by footnote & tooltip (negating
 * previous change which made it black, which was a bug)
 * 
 * 23    7/15/99 6:34p Bchang
 * hardwire lastTraversed and traversedColor to black for footnote and
 * tooltip
 * fixed grayout of underlines when overlapped by another anno
 * margincallout and footnote have bigbounds = smallbounds
 * margincallout lines and text color correctly when frozen/unfrozen
 * 
 * 22    7/15/99 11:59a Mackinla
 * 
 * 21    7/15/99 11:42a Bchang
 * hardwire annocolor for tooltip and footnote to be black
 *
 * 20    7/15/99 11:18a Bchang
 * more fixes to the previous bug (see previous message)
 * fixed color of anchor underlines -- only three possibilities --
 * traversedColor, black, or being-traversed color (lastTraversedColor)
 *
 * 19    7/15/99 10:19a Bchang
 * fixed bug: clicking on a link without allowing the gloss to expand now
 * works -- used to try to freeze the gloss (wrong) and then on return try
 * to shrink it (wrong)
 *
 * 18    7/14/99 8:37a Bchang
 * added freeze button
 * changed color behavior for traversal
 * added feedback color on mousedown traversal; traversal happens on mouseup
 * return from traversal: animated shrink or return to frozen color after delay
 *
 * 17    7/13/99 5:43p Mackinla
 * Logging - basic functionality
 * 
 * 16    6/30/99 1:11p Bchang
 * support tooltip and footnote techniques
 *
 * 15    6/24/99 9:49a Bchang
 * support for wide and pyramid layouts
 * partial fix for the inaccurate bounding box around glosses (fixed the
 * heights of the bounding boxes, but the widths are still buggy)
 * changed the variable moveBack to expandedXDefault
 * support "singleAnnoPath" (only one anno open at a time)
 * 
 * 14    6/11/99 12:46p Bchang
 * add support for "click to open" behavior -- left button clicks &
 * freezes; dwell does nothing. This is specified by the param
 * trigger="click".
 * 
 * 13    5/18/99 4:47p Bchang
 * added support for alignment of the annotation (left, center, right with
 * respect to the anchor's width)
 * moveBack -- which handles annotations at the right edge of the page --
 * is being ignored if alignment is not LEFT; it's also ignored if xOffset
 * is not zero. This should be fixed.
 *
 * 12    5/18/99 4:13p Bchang (pollez)
 * (Pollez) Support for freezing nested annotations. Support for
 * lasttraversedcolor (the link that we just visited) and traversedcolor
 * (a link that's been visited).
 * 
 * 11    5/12/99 11:29a Bchang
 * Added Annotation(GlossAttributes) to allow specifying fontname,
 * fontstyle, fontsize, linespacing, color, and xoffset. This is in
 * support of Rich Gold's hypersentences.
 *
 * 10    1/27/99 6:58p Bchang
 * 
 * 9     1/27/99 5:56p Bchang
 * configure from parameters: calloutfontsize, calloutlinespacing,
 * annowidth
 * 
 * 8     1/27/99 2:39p Bchang
 * if AnnoDoc.hasNesting is true, this will use Page.revertQueue to queue
 * up shrink requests and handle them serially. This allows nested
 * annotations to shrink properly when you wildly swing the mouse around.
 * 
 * 7     1/27/99 3:03a Bchang
 * support for nested annotations.
 * They do not close perfectly: move out of a deeply nested annotation
 * slowly and carefully.
 * Need to fix: moving out of a deeply nested annotation quickly,
 * triggering leave events on many of the parent annotations.
 * 
 * 6     1/21/99 4:39p Bchang
 * added margin callout technique. (does not yet use negotiation though)
 * changed mouse button mappings: left = follow link; right = freeze.
 * 
 * 5     1/13/99 5:31p Bchang
 * Added $Log at bottom of file
 */
