// ControlAnnotation.java
// Copyright (c) 1999 by Xerox Corporation
// bchang@parc.xerox.com

// $Header: /FluidDocs/annotation/ControlAnnotation.java 5     7/19/99 12:59p Bchang $

package annotation;

import fluid.*;
import java.awt.*;
import java.net.*;

public class ControlAnnotation extends ImageAnnotation {

  public ControlAnnotation() {
    super();
    imageSize = new Dimension(90, 30);
    setColor(Color.orange);
  }
  
  protected Dimension controlSize;
  
  boolean fill0 = false;
  boolean fill1 = false;
  boolean fill2 = false;
  
  public void drawSelf(Graphics g) {
    drawAnchorUnderline(g);
    int w = (int)((float)drawnSize.width / 3.0);
    g.setColor(Color.red.darker());
//    g.drawRect(0, expandedY, w, drawnSize.height);
    g.fillRect(0, expandedY, w, drawnSize.height);
    if (fill0) {
      g.setColor(Color.red.brighter());
      g.fillRect(0, expandedY, w, drawnSize.height);
    }
    g.setColor(Color.yellow.darker());
//    g.drawRect(w, expandedY, w, drawnSize.height);
    g.fillRect(w, expandedY, w, drawnSize.height);
    if (fill1) {
      g.setColor(Color.yellow.brighter());
      g.fillRect(w, expandedY, w, drawnSize.height);
    }
    g.setColor(Color.green.darker());
//    g.drawRect(2 * w, expandedY, w, drawnSize.height);
    g.fillRect(2 * w, expandedY, w, drawnSize.height);
    if (fill2) {
      g.setColor(Color.green.brighter());
      g.fillRect(2 * w, expandedY, w, drawnSize.height);
    }
  }

  public Dimension calculateFullSize() { return imageSize; }
  
  protected void chooseTechnique(int i) {
    String tech, str;
    switch (i) {
      case 0: tech = "squash"; str = "[interline compression]"; break;
      case 1: tech = "moveonly"; str = "[block movement]"; break;
      case 2: tech = "overlay"; str = "[overlay]"; break;
      default: tech = "squash"; str = "[interline compression]";
    }
    AnnoDoc.technique = tech;
    ((TextLine)getParent()).setText(str);
    anchorLength = str.length(); // hack alert! (anchorLength is set during parse so it's hard to change)
    setInitialPosition();
  }
  
  public void mouseDown(Event evt, int x, int y) {
    if (isBig()) {
      int w = (int)((float)drawnSize.width / 3.0);
      if (x - absoluteX() < w) {
        fill0 = true;
        fill1 = false;
        fill2 = false;
        chooseTechnique(0);
      } else if (x - absoluteX() < 2 * w) {
        fill0 = false;
        fill1 = true;
        fill2 = false;
        chooseTechnique(1);
      } else {
        fill0 = false;
        fill1 = false;
        fill2 = true;
        chooseTechnique(2);
      }
      repaint();
    }
  }
  
  public void mouseUp(Event evt, int x, int y) {
    if (isBig()) {
      fill0 = false;
      fill1 = false;
      fill2 = false;
      repaint();
    }
  }
  
}

/* $Log: /FluidDocs/annotation/ControlAnnotation.java $
 * 
 * 5     7/19/99 12:59p Bchang
 * AnnoDoc.what replaced by AnnoDoc.technique
 * 
 * 4     1/27/99 6:46p Bchang
 * red-green-blue switch
 * 
 * 3     1/27/99 2:35p Bchang
 * made the contol smaller
 * changed the string displayed to a longer name with brackets
 * 
 * 2     1/22/99 3:33p Bchang
 * as a demo: clicking the control changes the text of the text line it is
 * hanging off of to the name of the technique
 * 
 * 1     1/21/99 4:40p Bchang
 * Initial version. Simple control that toggles fluid technique used for
 * entire page.
 */
