// FreezeButton.java
// Copyright (c) 1999 by Xerox Corporation
// bchang@parc.xerox.com

// $Header: /FluidDocs/annotation/FreezeButton.java 7     7/26/99 11:00a Bchang $

package annotation;

import fluid.*;

import java.awt.*;

public class FreezeButton extends FluidComponent {
  protected int buttonWidth;
  protected int buttonHeight;
  protected int buttonX;
  protected int buttonY;
  protected int gutter;
  protected int overshoot;
  public int getOvershoot() { return overshoot; }
  public int offset() { return overshoot + buttonWidth + gutter; }
  public int getButtonX() { return buttonX; }
  public Dimension smallSize;
  public Dimension bigSize;

  protected void initializeSizeAndPosition() {
    buttonWidth = AnnoDoc.freezebuttonwidth;
    buttonHeight = AnnoDoc.freezebuttonheight;
    gutter = AnnoDoc.freezebuttongutter;
    buttonX = -(buttonWidth + gutter);
    buttonY = AnnoDoc.freezebuttony;
    overshoot = AnnoDoc.freezebuttonovershoot;

    smallSize = new Dimension(1, 1);
    bigSize = new Dimension(buttonWidth, buttonHeight);

    setDrawnSize(smallSize);
    setSize(new Dimension(offset(), 0)); // must call setHeight() later
  }

  public FreezeButton() {
    initializeSizeAndPosition();
    setColor(Color.white);
  }

  protected Dimension drawnSize;
  public void setDrawnSize(Dimension s) { drawnSize = s; }
  public Dimension getDrawnSize() { return drawnSize; }

  protected Animation drawnSizeAnimation = null;
  public AnimDimension changeDrawnSize(final Dimension tosize) {
    return new AnimDimension() {
      protected void setStart() { start = getDrawnSize(); }
      protected void setEnd() { end = tosize; }
      protected void stepTarget(double fraction) {
        setDrawnSize(currentValue(fraction));
      }
      protected Animation getCurrentAnimation() { return drawnSizeAnimation; }
      protected void setCurrentAnimation(Animation anim) { drawnSizeAnimation = anim; }
    };
  }

  protected Color offColor = AnnoDoc.annocolor;
  protected Color onColor = AnnoDoc.frozencolor;
  protected Color overColor = AnnoDoc.frozencolor;

  protected boolean fill = false;
  protected boolean over = false;

  public void drawSelf(Graphics g) {
    /*
    if (drawnSize.equals(bigSize)) {
      g.setColor(Color.orange);
      g.drawRect(0, 0, getWidth(), getHeight());
    }
    */

    g.setColor(getColor());
    int x = overshoot;
    int y = buttonY;
    int h = drawnSize.height;
    int w = drawnSize.width;
    int hHalf = h / 2;
    int wHalf = w / 2;
    int[] xPoly = {x, x + wHalf, x + w, x + wHalf};
    int[] yPoly = {y + hHalf, y, y + hHalf, y + h};
    if (fill) {
      g.fillPolygon(xPoly, yPoly, 4);
    } else {
      g.drawPolygon(xPoly, yPoly, 4);
    }

    Color c = getColor();
    if (!c.equals(Color.white)) {
      String p = "XY: " + x + " " + y;
      String sz = "SIZE: " + w + " " + h;
      String rgb = "RGB: " + c.getRed() + " " + c.getGreen() + " " + c.getBlue();
      String tx = "TX: " + ((Annotation)getParent()).getLogText();
      String details = p + " " + sz + " " + rgb + " " + tx;

  	  FluidUI.fluidLog.addLog("FREEZEBUTTON", details, false);
    }
  }

  protected boolean inClickRegion(int x, int y) {
    Rectangle rect = new Rectangle(overshoot, buttonY,
                                   bigSize.width, bigSize.height);
    return rect.contains(x - absoluteX(), y - absoluteY());
  }

  public void mouseDown(Event evt, int x, int y) {
    // need this here to shadow Anno mousedown, which would try to take the link
  }

  public void mouseUp(Event evt, int x, int y) {
    if (inClickRegion(x, y)) {
      toggleFreeze();
    }
  }

  public void rightMouseDown(Event evt, int x, int y) {
    if (inClickRegion(x, y)) {
      toggleFreeze();
    }
  }

  public void toggleFreeze() {
    ((Annotation)getParent()).toggleFreeze(true);
    // as a side effect, toggleFreeze will call setOn() or setOff()
    repaint();
  }

  public void setOn() {
    fill = true;
    setColor(onColor);
  }

  public void setOff() {
    fill = false;
    setColor(offColor);
  }
}

/* $Log: /FluidDocs/annotation/FreezeButton.java $
 * 
 * 7     7/26/99 11:00a Bchang
 * removed repaint() in setOn() and setOff() -- unnecessary
 * 
 * 6     7/23/99 1:41p Bchang
 * add logging
 * 
 * 5     7/23/99 9:04a Bchang
 * draw bounding box for debugging (commented out)
 * 
 * 4     7/20/99 9:43a Bchang
 * added buttonX to control where the freezebutton is at its small size
 * (affects how the animation looks)
 * 
 * 3     7/16/99 3:28p Bchang
 * added overshoot margin to the left of the button
 *
 * 2     7/14/99 11:56a Bchang
 * clicking a frozen freezebutton will now cause the anno to shrink too,
 * just like right-click
 *
 * 1     7/14/99 8:40a Bchang
 *
 */
