// GetTextLines.java
// Copyright (c) 1999 by Xerox Corporation
// bchang@parc.xerox.com

// a utility program for writing out information matching text lines to their
// ids. this is to supplement the information gathered by the logging facility.
// See main() for arguments.

// $Header: /FluidDocs/annotation/GetTextLines.java 1     8/31/99 4:21p Bchang $

package annotation;

import fluid.*;
import java.awt.*;
import java.awt.event.*;
import java.net.URL;
import java.io.*;
import java.util.*;

public abstract class GetTextLines extends AnnoDoc {

  FluidCanvas annodoc;
  protected static String hypertexts;
  protected static String format;
  protected static String tech = "FI";

  public GetTextLines() {}

  // entry point
	public static void main(String[] args) {
    inApplet = false;

		try {
      String paramfilename = "";
      if (args.length >= 3) {
        paramfilename = args[0];
        hypertexts = args[1];
        format = args[2];
        if (args.length >= 4) {
          if (args[3].equalsIgnoreCase("default") ||
              args[3].equalsIgnoreCase("interline") ||
              args[3].equalsIgnoreCase("overlay") ||
              args[3].equalsIgnoreCase("popup") ||
              args[3].equalsIgnoreCase("tooltip")) {
            tech = "FI";
          } else if (args[3].equalsIgnoreCase("margin")) {
            tech = "FM";
          } else if (args[3].equalsIgnoreCase("footnote")) {
            tech = "NG";
          } else {
            System.out.println("fourth argument has to be 'default', 'margin', or 'footnote'");
            System.exit(0);
          }
        }
        GetTextLines gtl;
        if (format.equalsIgnoreCase("jock")) {
          gtl = new GetTextLinesForJock();
          gtl.create(paramfilename);
        } else if (format.equalsIgnoreCase("polle")) {
          gtl = new GetTextLinesForPolle();
          gtl.create(paramfilename);
        } else {
          System.out.println("third argument has to be 'jock' or 'polle'");
          System.exit(0);
        }
      } else {
        System.out.println("Not enough arguments. GetTextLines takes 4 args:");
        System.out.println(" param file (full pathname)");
        System.out.println(" file containing the hypertext filenames, one to a line");
        System.out.println(" 'jock' or 'polle' format");
        System.out.println(" (optional 4th arg for width of glosses): 'default' 'margin' or 'footnote'");
        try {
          // wait for user to hit return
          System.in.read();
        } catch (Exception ioe) {}
      }
		}	catch (Exception e) {
      e.printStackTrace();
      try {
        // wait for user to hit return
        System.in.read();
      } catch (Exception ioe) {}
		}
  }

  protected void create(String paramfilename) {
    page = new Page();

    Frame frame = new Frame() {
      public boolean handleEvent(Event evt)	{
        switch (evt.id) {
        case Event.WINDOW_DESTROY:
          dispose();
          System.exit(0);
          return true;
        default:
          return super.handleEvent(evt);
        }
      }
    };

    readAppParams(paramfilename);

		FluidUI.fluidLog.setParams();
    frame.setTitle(windowtitle);
    initializeFromParams();

    annodoc = new FluidCanvas(page);
    annodoc.setDefaultSize(defaultWidth, defaultHeight);
    frame.add(annodoc);

		initializePage((URL)null, annodoc);
    loadDocument("blank.txt", tech);

    frame.pack();
    frame.setVisible(true);

    writeTextLines();
    System.exit(0);
  }

  protected void writeTextLines() {
    File infile = new File(hypertexts);
    File outfile = new File("textline." + format + "." + technique);
    try {
      BufferedReader in = new BufferedReader(new FileReader(infile));
      BufferedWriter out = new BufferedWriter(new FileWriter(outfile));
      String filename;
      while ((filename = in.readLine()) != null) {
        writeTextLines(filename, out);
      }
      in.close();
      out.close();
    } catch (IOException e) { System.out.println(e); }
  }

  protected void writeTextLines(String filename, BufferedWriter out) {
    loadDocument(filename, tech);
    try {
      writeFilename(filename, out);
      TextBlock tb = (TextBlock)page.getComponents().elementAt(0);
      Vector lines = tb.getComponents();
      Vector annos = new Vector();
      for (int i = 0; i < lines.size(); i++) {
        TextLine line = (TextLine)lines.elementAt(i);
        if (i == 0) {
          writeFirstLine(line, out);
        }
        writeBodyLine(line, out);
        Vector a = line.getComponents();
        for (int j = 0; j < a.size(); j++) {
          annos.addElement(a.elementAt(j));
        }
      }
      try {
        Class freezebutton = Class.forName("annotation.FreezeButton");
        for (int i = 0; i < annos.size(); i++) {
          Annotation anno = (Annotation)annos.elementAt(i);
          Vector t = anno.getComponents();
          for (int j = 0; j < t.size(); j++) {
            if (t.elementAt(j).getClass() == freezebutton) continue;
            TextLine line = (TextLine)t.elementAt(j);
            writeAnnoLine(anno, line, out);
          }
          writeEndAnno(anno, out);
        }
      } catch (ClassNotFoundException e) { System.out.println(e); }
      writeEndFile(out);
    } catch (IOException e) { System.out.println(e); }
  }

  protected void writeFilename(String filename, BufferedWriter out) throws IOException {}
  protected void writeFirstLine(TextLine line, BufferedWriter out) throws IOException {}
  protected void writeBodyLine(TextLine line, BufferedWriter out) throws IOException {}
  protected void writeAnnoLine(Annotation anno, TextLine line, BufferedWriter out) throws IOException {}
  protected void writeEndAnno(Annotation anno, BufferedWriter out) throws IOException {}
  protected void writeEndFile(BufferedWriter out) throws IOException {}
}

class GetTextLinesForJock extends GetTextLines {
  protected void writeFilename(String filename, BufferedWriter out) throws IOException {
    out.write("FILE\t" + filename + "\n");
  }

  protected void writeBodyLine(TextLine line, BufferedWriter out) throws IOException {
    out.write(line.getLogID() + "\t" + line.getText() + "\n");
  }

  protected void writeAnnoLine(Annotation anno, TextLine line, BufferedWriter out) throws IOException {
    writeBodyLine(line, out);
  }
}

class GetTextLinesForPolle extends GetTextLines {
  protected void writeFilename(String filename, BufferedWriter out) throws IOException {
    out.write("FILE " + filename + " ");
  }

  protected void writeFirstLine(TextLine line, BufferedWriter out) throws IOException {
    out.write(line.getText() + "\n");
  }

  protected void writeAnnoLine(Annotation anno, TextLine line, BufferedWriter out) throws IOException {
    if (line.getLineNumber() == 0) {
      String anchor = "foo";
      try {
        anchor = anno.getAnchorText();
      } catch (Exception e) { System.out.println(e); }
      out.write("^" + anchor + "^ $" + anno.getLogID() + line.getText());
    } else {
      out.write(line.getText());
    }
  }

  protected void writeEndAnno(Annotation anno, BufferedWriter out) throws IOException {
    out.write("%" + anno.getHref() + "%\n");
  }

  protected void writeEndFile(BufferedWriter out) throws IOException {
    out.write("\n");
  }
}

/* $Log: /FluidDocs/annotation/GetTextLines.java $
 * 
 * 1     8/31/99 4:21p Bchang
 */
