// GlossExperiment.java
// Copyright (c) 1999 by Xerox Corporation
// bchang@parc.xerox.com

// $Header: /FluidDocs/annotation/GlossExperiment.java 6     9/28/99 1:31p Bchang $

package annotation;

import fluid.*;
import java.awt.*;
import java.awt.event.*;
import java.net.URL;
import java.io.*;
import java.util.*;

public class GlossExperiment extends AnnoDoc {

  FluidCanvas annodoc;

  public static Color controlpanelcolor = new Color(221, 221, 221);
  public static String controlfontname = "Serif";
  public static int control_fontstyle = Font.PLAIN;
  public static int controlfontsize = 12;
  public static String controlbuttonfontname = "SanSerif";
  public static int controlbuttonfontsize = 8;

  public static String questionfile;
  public static String baselinetext;
  public static String[] questionsteps;
  public static String[] filesteps;

  public static String[] finalconditions;

  public static String readquestion;
  public static String waitquestion;
  public static String discussquestion;

  protected final String QUESTION = "question";
  protected final String WAIT = "wait";
  protected final String DISCUSS = "discuss";

  protected static String subject;
  protected static String[] conditions;

  protected int currentStep = 0;
  protected Vector steps = new Vector();

  protected GlossExptControls controls;


  public GlossExperiment() {}

  // entry point
	public static void main(String[] args) {
    inApplet = false;

		try {
      String paramfilename = "";
      if (args.length >= 3) {
        paramfilename = args[0];
        subject = args[1];
        conditions = parseConditions(args[2]);
        GlossExperiment ge = new GlossExperiment();
        ge.create(paramfilename);
      } else {
        System.out.println("Not enough arguments. GlossExperiment takes 3 args:");
        System.out.println(" param file (full pathname)");
        System.out.println(" subject identifier (e.g. A)");
        System.out.println(" condition order (e.g. \"HT FM FI FO PG NG\")");
        try {
          // wait for user to hit return
          System.in.read();
        } catch (Exception ioe) {}
      }
		}	catch (Exception e) {
      e.printStackTrace();
      try {
        // wait for user to hit return
        System.in.read();
      } catch (Exception ioe) {}
		}
  }

  protected static String[] parseConditions(String conditionorder) {
    StringTokenizer elementTokens = new StringTokenizer(conditionorder);
    int numElements = elementTokens.countTokens();
    String[] cond = new String[numElements];
    for (int i = 0; i < numElements; i++) {
      cond[i] = elementTokens.nextToken();
    }
    return cond;
  }

  protected void create(String paramfilename) {
    page = new Page();

    Frame frame = new Frame() {
      public boolean handleEvent(Event evt)	{
        switch (evt.id) {
        case Event.WINDOW_DESTROY:
          // To avoid subjects from accidentally closing the window,
          // we do nothing when the window is asked to be destroyed.
          // To end the application, close the MS-DOS window that runs java.
          // dispose();
          // System.exit(0);
          return true;
        default:
          return super.handleEvent(evt);
        }
      }
    };

    readAppParams(paramfilename);
		FluidUI.fluidLog.setParams();
    frame.setTitle(windowtitle);
    initializeFromParams();

    initializeSteps();

    GridBagLayout gridbag = new GridBagLayout();
    GridBagConstraints constraints = new GridBagConstraints();

    frame.setLayout(gridbag);
    controls = new GlossExptControls(this);
    controls.setBackground(controlpanelcolor);
    constraints.gridx = 0;
    constraints.gridy = 0;
    constraints.gridwidth = 4;
    constraints.gridheight = 1;
    constraints.weightx = 1;
    constraints.weighty = 1;
    constraints.fill = GridBagConstraints.BOTH;
    gridbag.setConstraints(controls, constraints);
    frame.add(controls);

    // don't know how to control sliver's height
    Panel sliver = new Panel();
    sliver.setBackground(Color.black);
    constraints.gridx = 0;
    constraints.gridy = 1;
    constraints.gridwidth = 4;
    constraints.gridheight = 1;
    constraints.weightx = 1;
    constraints.weighty = 0;
    constraints.fill = GridBagConstraints.HORIZONTAL;
    gridbag.setConstraints(sliver, constraints);
    frame.add(sliver);

    Panel left = new Panel();
    left.setBackground(Color.black);
    constraints.gridx = 0;
    constraints.gridy = 2;
    constraints.gridwidth = 1;
    constraints.gridheight = 1;
    constraints.weightx = 0.5;
    constraints.weighty = 0;
    constraints.fill = GridBagConstraints.BOTH;
    gridbag.setConstraints(left, constraints);
    frame.add(left);

    annodoc = new FluidCanvas(page);
    annodoc.setDefaultSize(defaultWidth, defaultHeight);
    constraints.gridx = 1;
    constraints.gridy = 2;
    constraints.gridwidth = 1;
    constraints.gridheight = 1;
    constraints.weightx = 0;
    constraints.weighty = 0;
    constraints.fill = GridBagConstraints.NONE;
    gridbag.setConstraints(annodoc, constraints);
    frame.add(annodoc);

    Panel right = new Panel();
    right.setBackground(Color.black);
    constraints.gridx = 2;
    constraints.gridy = 2;
    constraints.gridwidth = 1;
    constraints.gridheight = 1;
    constraints.weightx = 0.5;
    constraints.weighty = 0;
    constraints.fill = GridBagConstraints.BOTH;
    gridbag.setConstraints(right, constraints);
    frame.add(right);

		initializePage((URL)null, annodoc);
    goToStep(0);

    frame.pack();
    frame.setVisible(true);
	}

  public class Step {
    public Step() {}
    public Step(String q, String f, String t) {
      question = q;
      file = f;
      technique = t;
    }
    public String question;
    public String file;
    public String technique;
    public boolean isFinal = false;
 }

  protected void initializeSteps() {
    BufferedReader qfile;
    String qfilename = AnnoDoc.directory + questionfile;
    try {
      qfile = new BufferedReader(new FileReader(new File(qfilename)));
    } catch (FileNotFoundException e) {
      System.out.println("could not find question file: " + qfilename);
      System.out.println(e);
      return;
    }
    String question = "";
    // first screen
    question = subject + "0. " + waitquestion;
    steps.addElement(new Step(question, "blank.txt", "HT"));
    // second screen
    question = subject + "1. " + readquestion;
    steps.addElement(new Step(question, baselinetext, "HT")); 
    for (int i = 0; i < conditions.length; i++) {
      String technique = conditions[i];
      for (int j = 0; j < questionsteps.length; j++) {
        String questiontype = questionsteps[j];
        String file = filesteps[j];
        if (questiontype.equalsIgnoreCase(WAIT)) {
          question = waitquestion;
        } else if (questiontype.equalsIgnoreCase(DISCUSS)) {
          question = discussquestion;
        } else if (questiontype.equalsIgnoreCase(QUESTION)) {
          try {
            question = qfile.readLine();
          } catch (IOException e) {
            System.out.println("io exception while reading question file: " + qfilename);
            System.out.println(e);
            return;
          }
        }
        int stepnum = (i * questionsteps.length) + j + 2;
        question = subject + stepnum + ". " + question;
        steps.addElement(new Step(question, file, technique));
      }
    }
    Step finalstep = new Step();
    finalstep.isFinal = true;
    steps.addElement(finalstep);
  }

  protected void goToStep(int n) {
    currentStep = n;
    Step step = (Step)steps.elementAt(currentStep);
    if (!step.isFinal) {
      controls.setQuestion(step.question);
      loadDocument(step.file, step.technique);
      if (FluidUI.fluidLog.logDataP()) {
        FluidUI.fluidLog.addLog("QUESTION", step.question, true);
        FluidUI.fluidLog.addLog("CONDITION", techniqueName(step.technique), true);
      }
    } else {
      if (FluidUI.fluidLog.logDataP()) {
        FluidUI.fluidLog.addLog("QUESTION", "-- final screen --", true);
        FluidUI.fluidLog.addLog("CONDITION", techniqueName(conditions[0]), true);
      }
      String lastfile = ((Step)steps.elementAt(currentStep - 1)).file;
      controls.switchToTechniquePanel(lastfile);
      loadDocument(lastfile, finalconditions[0]);
    }
  }

  public void proceed() {
    fluidLog.writeLog(subject + currentStep + ".log");
    if (currentStep < (steps.size() - 1)) {
      goToStep(currentStep + 1);
    }
  }

  public void back() {
    Step step = (Step)steps.elementAt(currentStep);
    if (step.isFinal) {
      controls.switchToQuestion();
    }
    if (currentStep > 0) {
      goToStep(currentStep - 1);
    }
  }

  public void refresh() {
    reset();
  }

  public String techniqueName(String tech) {
    if (tech.equalsIgnoreCase("HT")) {
      return "Hypertext";
    } else if (tech.equalsIgnoreCase("NG")) {
      return "Footnote";
    } else if (tech.equalsIgnoreCase("PG")) {
      return "Popup";
    } else if (tech.equalsIgnoreCase("FI")) {
      return "Interline";
    } else if (tech.equalsIgnoreCase("FM")) {
      return "Margin";
    } else if (tech.equalsIgnoreCase("FO")) {
      return "Overlay";
    } else {
      return "unknown";
    }
  }
}

/* $Log: /FluidDocs/annotation/GlossExperiment.java $
 * 
 * 6     9/28/99 1:31p Bchang
 * To avoid subjects from accidentally closing the window, we do nothing
 * when the window is asked to be destroyed. To end the application, close
 * the MS-DOS window that runs java.
 * 
 * 5     7/22/99 11:52a Bchang
 * param finalconditions is the order that the conditions appear on the
 * final screen
 * 
 * 4     7/22/99 11:12a Bchang
 * added logging of question & condition
 * moved techniqueName from glossExptControls to here
 *
 * 3     7/20/99 10:43a Bchang
 * added controlbutton params
 *
 * 2     7/19/99 4:29p Mackinla
 * fixed parameter setting
 *
 * 1     7/19/99 1:24p Bchang
 */
