// GlossExptControls.java
// Copyright (c) 1999 by Xerox Corporation
// bchang@parc.xerox.com

// $Header: /FluidDocs/annotation/GlossExptControls.java 4     7/22/99 11:52a Bchang $

package annotation;

import fluid.*;
import java.awt.*;
import java.awt.event.*;

public class GlossExptControls extends Panel {
  protected GlossExperiment experiment;

  protected GridBagLayout gridbag = new GridBagLayout();
  protected GridBagConstraints constraints = new GridBagConstraints();

//  protected TextArea question;
  protected Label question;
  protected Panel techniquePanel;

  protected Insets insets = new Insets(2, 2, 2, 2);
  protected Insets buttoninsets = new Insets(0, 0, 0, 0);

  public GlossExptControls(GlossExperiment expt) {
    experiment = expt;

    setFont(new Font(GlossExperiment.controlfontname,
                     GlossExperiment.control_fontstyle,
                     GlossExperiment.controlfontsize));

    setLayout(gridbag);

    SmallButton back = new SmallButton("B") {
      public boolean action(Event e, Object arg) {
        back();
        return true;
      }
    };
    constraints.gridx = 0;
    constraints.gridy = 0;
    constraints.gridwidth = 1;
    constraints.gridheight = 1;
    constraints.weightx = 0;
    constraints.weighty = 0;
    constraints.anchor = GridBagConstraints.NORTHWEST;
    constraints.fill = GridBagConstraints.NONE;
    constraints.insets = buttoninsets;
    gridbag.setConstraints(back, constraints);
    add(back);

    SmallButton refresh = new SmallButton("R") {
      public boolean action(Event e, Object arg) {
        refresh();
        return true;
      }
    };
    constraints.gridx = 1;
    constraints.gridy = 0;
    constraints.gridwidth = 1;
    constraints.gridheight = 1;
    constraints.weightx = 0;
    constraints.weighty = 0;
    constraints.anchor = GridBagConstraints.NORTHWEST;
    constraints.fill = GridBagConstraints.NONE;
    buttoninsets.right = 5;
    constraints.insets = buttoninsets;
    gridbag.setConstraints(refresh, constraints);
    add(refresh);

    addQuestion();

    /*
    Panel filler = new Panel();
    filler.setBackground(GlossExperiment.controlpanelcolor);
    constraints.gridx = 2;
    constraints.gridy = 1;
    constraints.gridwidth = 1;
    constraints.gridheight = 1;
    constraints.weightx = 1;
    constraints.weighty = 0;
    constraints.fill = GridBagConstraints.BOTH;
    constraints.insets = insets;
    gridbag.setConstraints(filler, constraints);
    add(filler);
    */

    Label proceedlabel = new Label("Proceed:");
    constraints.gridx = 3;
    constraints.gridy = 0;
    constraints.gridwidth = 1;
    constraints.gridheight = 1;
    constraints.weightx = 0;
    constraints.weighty = 0;
    insets.right = 0;
    constraints.anchor = GridBagConstraints.NORTHWEST;
    constraints.fill = GridBagConstraints.NONE;
    constraints.insets = insets;
    gridbag.setConstraints(proceedlabel, constraints);
    add(proceedlabel);

    Checkbox proceed = new Checkbox("", new CheckboxGroup(), true) {
      public boolean action(Event e, Object arg) {
        proceed();
        return true;
      }
    };
    constraints.gridx = 4;
    constraints.gridy = 0;
    constraints.gridwidth = 1;
    constraints.gridheight = 1;
    constraints.weightx = 0;
    constraints.weighty = 0;
    insets.left = 0;
    insets.right = 2;
    constraints.anchor = GridBagConstraints.NORTHWEST;
    constraints.fill = GridBagConstraints.NONE;
    constraints.insets = insets;
    gridbag.setConstraints(proceed, constraints);
    add(proceed);
	}

  class SmallButton extends Button {
    public SmallButton(String label) {
      super(label);
      setFont(new Font(GlossExperiment.controlbuttonfontname,
                       Font.PLAIN,
                       GlossExperiment.controlbuttonfontsize));
    }

  }

  protected void addQuestion() {
  /*
    question = new TextArea("", 2, 40, TextArea.SCROLLBARS_NONE);
    question.setEditable(false);
    */
    question = new Label("");
    setConstraintsForUpperPanel();
    gridbag.setConstraints(question, constraints);
    add(question);
  }

  protected void setConstraintsForUpperPanel() {
    constraints.gridx = 2;
    constraints.gridy = 0;
    constraints.gridwidth = 1;
    constraints.gridheight = 1;
    constraints.weightx = 1;
    constraints.weighty = 0;
    constraints.anchor = GridBagConstraints.NORTHWEST;
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.insets = insets;
  }

  public void switchToTechniquePanel(String file) {
    remove(question);
    techniquePanel = new Panel();
    setConstraintsForUpperPanel();
    gridbag.setConstraints(techniquePanel, constraints);
    add(techniquePanel);
    CheckboxGroup group = new CheckboxGroup();
    boolean selected = true;
    for (int i = 0; i < experiment.finalconditions.length; i++) {
      String tech = experiment.finalconditions[i];
      addTechniqueRadio(tech, i, selected, group, file);
      selected = false;
    }
    validate();
  }

  protected void addTechniqueRadio(final String tech, int x, boolean selected,
                                   CheckboxGroup group, final String file) {
    Checkbox radio = new Checkbox(experiment.techniqueName(tech), group, selected) {
      public boolean action(Event e, Object arg) {
        loadDocument(file, tech);
        if (FluidUI.fluidLog.logDataP()) {
          FluidUI.fluidLog.addLog("CONDITION", experiment.techniqueName(tech), true);
        }
        return true;
      }
    };
    /*
    constraints.gridx = x;
    constraints.gridy = 0;
    constraints.gridwidth = 1;
    constraints.gridheight = 1;
    constraints.weightx = 0;
    constraints.weighty = 1;
    insets.left = 0;
    insets.right = 5;
    constraints.insets = insets;
    constraints.anchor = GridBagConstraints.NORTHWEST;
    constraints.fill = GridBagConstraints.NONE;
    gridbag.setConstraints(radio, constraints);
    */
    techniquePanel.add(radio);
  }

  public void switchToQuestion() {
    remove(techniquePanel);
    addQuestion();
    validate();
  }

  public void setQuestion(String q) {
    question.setText(q);
  }

  protected void loadDocument(String file, String tech) {
    experiment.loadDocument(file, tech);
  }

  protected void proceed() {
    experiment.proceed();
  }

  protected void back() {
    experiment.back();
  }

  protected void refresh() {
    experiment.refresh();
  }

}

/* $Log: /FluidDocs/annotation/GlossExptControls.java $
 * 
 * 4     7/22/99 11:52a Bchang
 * use param finalconditions as the order to show the conditions in the
 * final screen
 * 
 * 3     7/22/99 11:13a Bchang
 * moved techniqueName to GlossExperiment
 * log technique changes
 *
 * 2     7/20/99 10:43a Bchang
 * changed layout to use a single line for the question -- needed for
 * 800x600 screen
 *
 * 1     7/19/99 1:24p Bchang
 */
