// ImageAnnotation.java
// Copyright (c) 1999 by Xerox Corporation
// bchang@parc.xerox.com

// $Header: /FluidDocs/annotation/ImageAnnotation.java 8     7/23/99 9:03a Bchang $

package annotation;

import fluid.*;
import java.awt.*;
import java.net.*;

public class ImageAnnotation extends Annotation {
  
  public ImageAnnotation() {
    super();
  }

  public ImageAnnotation(URL docbase, String src, Dimension sz, FluidUI canvas) {
    super();
    setImageSource(src);
    localCanvas = canvas;
    imageSize = sz;
    loadImage(docbase);
  }
  
  protected URL docbase;
  protected FluidUI localCanvas;
  
  // reason I'm using drawnSize rather than just size:
  // I want to draw it at size (0, 0) to start, but
  // that would interfere with the unexpanded hot area
  // (because hit testing uses size)
  protected Dimension drawnSize = new Dimension(0, 0);
  public void setDrawnSize(Dimension s) { drawnSize = s; }
  public Dimension getDrawnSize() { return drawnSize; }
  
  protected String imageSource = "";
  public void setImageSource(String src) { imageSource = src; }
  public String getImageSource() { return imageSource; }
  
  protected Image image = null;
  protected Dimension imageSize = new Dimension(0, 0);
  
  protected void loadImage(URL docbase) {
    Toolkit toolkit = Toolkit.getDefaultToolkit();
    if (AnnoDoc.inApplet) {
      URL url;
      try {
        url = new URL(docbase, getImageSource());
      } catch (MalformedURLException e) {
        System.out.println("Could not load image '" + getImageSource() + "' because of badly formed URL.");
        return;
      }
      image = toolkit.getImage(url);
    } else {
      image = toolkit.getImage(AnnoDoc.directory + getImageSource());
    }
    try {
      MediaTracker tracker = new MediaTracker(localCanvas);
      tracker.addImage(image, 0);
      tracker.waitForID(0);
    } catch (InterruptedException e) {}
    int w = image.getWidth(localCanvas);
    int h = image.getHeight(localCanvas);
    if ((w == -1) || (h == -1)) {
      // couldn't get width or height for some reason
      image = null;
      System.out.println("Could not get size of image.");
    } else {
      if (imageSize.equals(new Dimension(0, 0))) {
        imageSize = new Dimension(w, h);
      } else if (imageSize.width == 0) {
        imageSize.width = Math.round(((float)imageSize.height / (float)h) * (float)w);
      } else if (imageSize.height == 0) {
        imageSize.height = Math.round(((float)imageSize.width / (float)w) * (float)h);
      } else {
        // keep specified imageSize
      }
    }
  }

  public void drawSelf(Graphics g) {
    //drawAnchorUnderline(g);
    super.drawSelf(g);
//    g.drawRect(0, expandedY, drawnSize.width, drawnSize.height);
    if (image != null) {
      g.drawImage(image, expandedX, expandedY, drawnSize.width, drawnSize.height, localCanvas);
    }
  }

  public Dimension calculateFullSize() { return imageSize; }
  
  protected Animation drawnSizeAnimation = null;
  public AnimDimension changeDrawnSize(final Dimension tosize) {
    return new AnimDimension() {
      protected void setStart() { start = getDrawnSize(); }
      protected void setEnd() { end = tosize; }
      protected void stepTarget(double fraction) {
        setDrawnSize(currentValue(fraction));
      }
      protected Animation getCurrentAnimation() { return drawnSizeAnimation; }
      protected void setCurrentAnimation(Animation anim) { drawnSizeAnimation = anim; }
    };
  }

  
  // very unfortunate name clash with changeExpandedX in Annotation
  protected Animation eXAnimation = null;
  public AnimInt changeEX(final int tox) {
    return new AnimInt() {
      protected void setStart() { start = expandedX; }
      protected void setEnd() { end = tox; }
      protected void stepTarget(double fraction) {
        expandedX = currentValue(fraction);
      }
      protected Animation getCurrentAnimation() { return eXAnimation; }
      protected void setCurrentAnimation(Animation anim) { eXAnimation = anim; }
    };
  }  
  
  protected Animation expandedYAnimation = null;
  public AnimInt changeExpandedY(final int toy) {
    return new AnimInt() {
      protected void setStart() { start = expandedY; }
      protected void setEnd() { end = toy; }
      protected void stepTarget(double fraction) {
        expandedY = currentValue(fraction);
      }
      protected Animation getCurrentAnimation() { return expandedYAnimation; }
      protected void setCurrentAnimation(Animation anim) { expandedYAnimation = anim; }
    };
  }

  

  public void negotiationSelectPresentation(Negotiation ngt) {
    if (ngt.offeredSize.equals(ngt.desiredSize)) {
      ngt.addAnimation(changeDrawnSize(ngt.offeredSize));
      drawc = -1;
      /*
      growLineSpacing = fullSizeLineSpacing;
      growFontSize = fullSizeFontSize;
      */
    } else {
      double wp = (double)ngt.offeredSize.width / (double)imageSize.width;
      double hp = (double)ngt.offeredSize.height / (double)imageSize.height;
      double m = Math.min(wp, hp);
      ngt.addAnimation(changeDrawnSize(new Dimension((int)Math.round(m * imageSize.width),
                                                     (int)Math.round(m * imageSize.height))));
      // only handles vertical amount right now

    }
  }

  protected void grow() {
    if (margincallout()) {
      growmargincallout();
      return;
    }
    
    // this is set to the margin x by margin callout, so we need to reset it
    expandedX = 0;

    if (frozen) return;

    // we always are top drawn; 
    if (!isTopDrawn()) {
      setTopDrawn(true); // ensures that, in overlay, this appears above all other expanded annos
    }
    bigState();

    ImageGChar gchar = new ImageGChar();
    gchar.setSize(fullSize);
    negotiation = new Negotiation(this, getParent().getPrimary(), gchar);

    AnimationGroup ag = new AnimationGroup();
    if (!nogrow) {
//      negotiation.animation.add(changeSize(fullSize));
//      if (expandedX != 0) {
//        negotiation.animation.add(changeX(expandedX));
//      }
    }

    negotiation.animation.setDuration(animduration);
    startAnimation(negotiation.animation);
    currentAnimation = negotiation.animation;
    
    addToExpandedAnnos();
  }

  protected void shrink() {
    /*
    Animation rev = ((Page)getPage()).revertingAnimation;
    boolean yo = false;
    if (rev != null) {
      yo = rev.addAnimationListenerIfNotDone(new AnimationListener() {
        public void animationDone(Animation anim) {
          reallyShrink();
        }
      });
    }
    if (!yo || (rev == null)) {
      reallyShrink();
    }
    */
    reallyShrink();
  }
    
  protected void reallyShrink() {
    if (margincallout()) {
      shrinkmargincallout();
      return;
    }
    
    nogrow = false;
    if (frozen) return;
    removeFromExpandedAnnos();
// on overlay, it looks better if we are top drawn, so unlike textual annotations,
// we always stay topdrawn
//    setTopDrawn(false);
    // close up the text lines
    negotiation.revert();
    // now shrink the annotation
    negotiation.animation.add(changeDrawnSize(new Dimension(0, 0)));
//      negotiation.animation.add(changeX(0));
    for (int i = 0; i < overlappedBy.size(); i++) {
      Annotation anno = (Annotation)overlappedBy.elementAt(i);
      anno.removeFromAnnosIOverlap(this);
    }
    overlappedBy.removeAllElements();
    negotiation.animation.setDuration(animduration);
    
    ((Page)getPage()).revertingAnimation = negotiation.animation;

    if (AnnoDoc.hasNesting) {
      negotiation.animation.addAnimationListener(new AnimationListener() {
        public void animationDone(Animation anim) {
          nextRevertQueue();
        }
      });
    }
    
    startAnimation(negotiation.animation);
    currentAnimation = negotiation.animation;
    smallState();
  }

  public void growmargincallout() {
    bigState();

    /*
    TextGChar gchar = new TextGChar();
    gchar.setSize(fullSize);
    negotiation = new Negotiation(this, getParent().getPrimary(), gchar);
    */
    
    FluidComponent prim = getParent().getPrimary();
    int left = ((TextBlock)prim).getLeftCalloutX() - getX();
    int right = ((TextBlock)prim).getRightCalloutX() - getX();
    int x1, x2, x3, x4, w;
    // absoluteX() and absoluteY() is here because we are attaching the
    // calloutLine to the Page, rather than this Annotation.
    // we do this because Annotation, via TextBlock, iterates through
    // its components with getLine(), and casts the components to TextLines.
    // ie, it doesn't expect a non-TextLine component. That should be fixed.

    if ((Math.abs(left) + getWidth()) < Math.abs(right)) {
      x1 = localEndpointX() + absoluteX();
      x2 = left + absoluteX();
      w = ((TextBlock)prim).getLeftCalloutWidth();
      x3 = left - platformMargin;
      x4 = x3 - w;
    } else {
      x1 = 0 + absoluteX();
      x2 = right + absoluteX();
      x3 = right + platformMargin;
      x4 = x3;
      w = ((TextBlock)prim).getRightCalloutWidth();
    }

    /*
    growLineSpacing = fullSizeLineSpacing;
    growFontSize = fullSizeFontSize;
    reconstructTextLines(w);
    */

    calloutY = underlineY + 1;
    //setCalloutLinePositions(x3, calloutY);

    calloutX = x3;
    expandedX = calloutX;
    expandedY = calloutY;    

    double wp = (double)w / (double)imageSize.width;
    int h = (int)Math.round(wp * imageSize.height);
    Dimension s = new Dimension(w, h);

    //int h = fullSize.height;
    
    int calloutYAbsolute = calloutY + absoluteY();

    // make sure callout stays on screen
    int newY = calloutY - (h / 2);
    int newYAbsolute = newY + absoluteY();
    int tinymargin = 10;
    System.out.println(getPage().getHeight());
    System.out.println(calloutYAbsolute);
    if (newYAbsolute < tinymargin) {
      newY = tinymargin - absoluteY();
    } else if (newYAbsolute + h + tinymargin > getPage().getHeight()) {
      newY -= (newYAbsolute + h + tinymargin) - getPage().getHeight();
    } else {
      // normal case; don't change newY
    }
    newYAbsolute = newY + absoluteY();

    if (cline == null) {
      cline = new CalloutLine(x1, x2, calloutYAbsolute, newYAbsolute, h, annoColor);
      getPage().add(cline);
    }

    AnimationGroup ag = new AnimationGroup();
    ag.add(changeExpandedY(newY));
    ag.add(changeEX(x4));
    ag.add(changeDrawnSize(s));

    cline.grow(ag);

    int orphanX = x4 + absoluteX();
    int orphanY = newY + absoluteY();
    calloutAbsoluteOrphanBounds = new Rectangle(orphanX, orphanY, fullSize.width, fullSize.height);
    ((Page)getPage()).addOrphanAnno(this);
  }

  public void shrinkmargincallout() {
    ((Page)getPage()).removeOrphanAnno(this);
    AnimationGroup ag = new AnimationGroup();
    ag.add(changeExpandedY(calloutY));
    ag.add(changeEX(calloutX));
    ag.add(changeDrawnSize(new Dimension(0, 0)));
    
    cline.shrink(ag);
    smallState();
  }

}

/* $Log: /FluidDocs/annotation/ImageAnnotation.java $
 * 
 * 8     7/23/99 9:03a Bchang
 * absoluteOrphanBounds -> calloutAbsoluteOrphanBounds
 * 
 * 7     7/19/99 12:59p Bchang
 * now works as an app as well as applet
 * 
 * 6     7/16/99 3:25p Bchang
 * copied the code from Annotation that ensures that margin callouts stay
 * on screen (not fully tested for images, but should work)
 * 
 * 5     5/18/99 4:14p Bchang
 * Bug fix: image annotations now call nextRevertQueue to properly handle
 * shrinking nested annotations.
 * 
 * 4     1/27/99 2:37p Bchang
 * no functional change; added some commented-out code as a possibility
 * for handing nesting
 * 
 * 3     1/27/99 12:53a Bchang
 * drawSelf() calls super.drawSelf() rather than directly calling
 * drawUnderline
 * 
 * 2     1/26/99 11:58p Bchang
 * added support for width & height attributes for image glosses.
 * also made images work as nested glosses: had to cache docbase and
 * canvas in the xmlhandler, because nested glosses don't have a path
 * up the component hierarchy during parsing
 * 
 * 1     1/21/99 4:41p Bchang
 * Initial version. Image that can be used as a gloss, using all available
 * fluid techniques.
 */
