// Page.java
// Copyright (c) 1999 by Xerox Corporation
// bchang@parc.xerox.com

// $Header: /FluidDocs/annotation/Page.java 25    7/26/99 11:04a Bchang $

package annotation;

import fluid.*;

import com.microstar.xml.SAXDriver;
import org.xml.sax.*;

import java.awt.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class Page extends FluidPage {
  Insets margins;

  String filename;

  TextBlock textblock;

  URL loaded_docbase;

  Animation revertingAnimation = null;
  Vector revertQueue = new Vector();

  public URL getDocBase() { return loaded_docbase; }

  protected int footnoteX0, footnoteX1, footnoteY;

  protected void initFootnoteDivider() {
    footnoteX0 = margins.left;
    footnoteX1 = getWidth() - margins.right;
    footnoteY = getHeight() - margins.bottom;
  }

  public int getFootnoteY() {
    return footnoteY;
  }

  public int getFootnoteWidth() {
    return footnoteX1 - footnoteX0;
  }

  public Page() {
    super();
    setLocation(new Point(0, 0));
    filename = AnnoDoc.textfile;
  }

  public void reload() {
    // do we need to removeAllTopDrawnComponents()?
    removeAllComponents();
    removeAllOrphanAnnos();
    removeAllExpandedAnnos();
    load(loaded_docbase);
    repaint();
  }

  public void loadnew(String fname) {
    clearCorpus();
    loadPage(fname, false);
  }

  protected Stack files = new Stack();
  protected Stack traversals = new Stack();

  protected Annotation lastAnnoTraversed = null;
  public void setLastAnnoTraversed(Annotation anno) { lastAnnoTraversed = anno;};
  public Annotation getLastAnnoTraversed() { return lastAnnoTraversed;};

  protected void clearCorpus() {
    files.removeAllElements();
    traversals.removeAllElements();

    history.clear();
    removeAllExpandedAnnos();
    removeAllOrphanAnnos();
    removeAllRevertListeners();
  }

  protected Hashtable history = new Hashtable();

  public boolean visited(String fname) {
    return history.containsKey(fname);
  }

  public void jump(String fname) {
    files.push(filename);
    traversals.push(lastAnnoTraversed);

    lastAnnoTraversed.clearLastTraversed();
    
    loadPage(fname);

		if (FluidUI.fluidLog.logDataP()) {
			FluidUI.fluidLog.addLog("FOLLOW", fname, true);
 		}
  }

  protected void savePageInHistory() {
    // this will put the current state of the page into the history
    history.put(filename, new PageData((Vector)getComponents().clone(),
                                       (Vector)getTopDrawnComponents().clone(),
                                       (Vector)orphanAnnos.clone(),
                                       (Vector)expandedAnnos.clone()));
  }

  protected void loadPage(String fname) {
    loadPage(fname, true);
  }
  protected void loadPage(String fname, boolean savehistory) {
    if (savehistory) {
      savePageInHistory();
    }

    filename = fname;
    if (history.containsKey(filename)) {
      loadCached();
    } else {
      reload();
    }
    notifyStructureLoaded();
  }

  public class PageData {
    public PageData(Vector comp, Vector topcomp, Vector orphs, Vector exp) {
      components = comp;
      topComponents = topcomp;
      orphans = orphs;
      expanded = exp;
    }
    Vector components;
    Vector topComponents;
    Vector orphans;
    Vector expanded;
  }

  public void loadCached() {
    // filename is already set
    PageData pagedata = (PageData)history.get(filename);
    setComponents(pagedata.components);
    setTopDrawnComponents(pagedata.topComponents);
    setOrphanAnnos(pagedata.orphans);
    setExpandedAnnos(pagedata.expanded);
    repaint();
  }

  public void back() {
    if (files.isEmpty()) {
      return;
    }

    lastAnnoTraversed = (Annotation)traversals.pop();
    if (lastAnnoTraversed != null) {
      if (lastAnnoTraversed.isBig()) {
        lastAnnoTraversed.setTraversing();
      }
    }

    loadPage((String)files.pop());

    if (lastAnnoTraversed != null) {
      lastAnnoTraversed.returnFromLink();
    }

		if (FluidUI.fluidLog.logDataP()) {
			FluidUI.fluidLog.addLog("BACK", filename, true);
		}
  }

  public void load(URL docbase) {
    if (!files.isEmpty()) {
      add(new BackButton(margins));
    }
    margins = new Insets(AnnoDoc.topmargin, AnnoDoc.leftmargin, AnnoDoc.bottommargin,
                         AnnoDoc.rightmargin);
    initFootnoteDivider();
    loaded_docbase = docbase;
    Point textOffset = new Point(margins.left, margins.top);
    Dimension textSize = new Dimension(getWidth() - margins.left - margins.right,
                                       getHeight() - margins.top - margins.bottom);
    textblock = new TextBlock(textOffset, textSize);
    add(textblock);
    InputStreamReader isr = null;
    if (FluidUI.inApplet) {
      try {
        URL url = new URL(docbase, filename);
        isr = new InputStreamReader(url.openStream());
      } catch (MalformedURLException e) {
        // e.printStackTrace();
        System.out.println("Could not jump to file '" + filename + "' because it was a badly formed URL.");
        back();
        return;
      } catch (IOException e) {
        // e.printStackTrace();
        System.out.println("Could not jump to file '" + filename + "' because of an I/O Exception.");
        back();
        return;
      }
    } else {
      try {
        File infile = new File(AnnoDoc.directory + filename);
        isr = new FileReader(infile);
      } catch (FileNotFoundException e ) {
        // e.printStackTrace();
//        System.out.println("Could not jump to file '" + filename + "' because it was not found.");
        back();
        return;
      }
    }
    if (AnnoDoc.fileformat.equalsIgnoreCase("outline")) {
      OutlineParser op = new OutlineParser(textblock, loaded_docbase);
      op.parse(isr);
    } else if (AnnoDoc.fileformat.equalsIgnoreCase("grouped")) {
      GroupedGlossParser gp = new GroupedGlossParser(textblock, loaded_docbase);
      gp.parse(isr);
    } else { // FORMAT_XML
      Parser parser = new SAXDriver();
      FluidLinksXMLHandler handler = new FluidLinksXMLHandler(textblock, loaded_docbase);
      parser.setDocumentHandler(handler);
      parser.setErrorHandler(handler);
      try {
        if (isr != null) {
          parser.parse(new InputSource(isr));
        }
      } catch (SAXException e) {
        e.printStackTrace();
        System.out.println("Could not jump to file '" + filename + "' because it had bad XML formatting.");
        back();
        return;
      } catch (IOException e) {
        // e.printStackTrace();
        System.out.println("Could not jump to file '" + filename + "' because of an I/O Exception during parsing.");
        back();
        return;
      }
    } 
    textblock.reconstruct();
		if (FluidUI.fluidLog.logDataP()) {
			FluidUI.fluidLog.addLog("NEWPAGE", filename, true);
		}
  }

  public void load() {
    load(null);
  }

  public void drawSelf(Graphics g) {
    /*
    g.setColor(Color.orange);
    g.drawRect(0, getBottom() - AnnoDoc.minmargin, size.width, AnnoDoc.minmargin - 1);
    g.drawRect(0, 0, size.width, AnnoDoc.minmargin - 1);
    */

    if (AnnoDoc.technique.equals("footnote")) {
      if (textblock.getComponentCount() != 0) {
        g.setColor(Color.black);
        g.drawLine(footnoteX0, footnoteY, footnoteX1, footnoteY);
      }
    }

    /*
    g.setColor(Color.blue);
    for (int i = 0; i < orphanAnnos.size(); i++) {
      Annotation anno = (Annotation)orphanAnnos.elementAt(i);
      Rectangle rect = anno.getAbsoluteOrphanBounds();
      g.drawRect(rect.x, rect.y , rect.width, rect.height);
    }
    */

  }

  Vector revertListeners = new Vector();
  public void addRevertListener(AnimationListener al) { revertListeners.addElement(al); }
  protected void revertDone() {
    for (int i = 0; i < revertListeners.size(); i++) {
      ((AnimationListener) revertListeners.elementAt(i)).animationDone(null);
    }
    removeAllRevertListeners();
  }
  protected void removeAllRevertListeners() {
    revertListeners.removeAllElements();
  }

  // openAnnos is used only when AnnoDoc.singleAnnoPath = true
  Annotation openAnnos[] = new Annotation[100]; // this limits the number of nested annotation to 100 deep
  public void openAnnoAtLevel(Annotation anno, int n) {
    openAnnos[n] = anno;
  }
  public void closeAnnoAtLevel(int n) {
    openAnnos[n] = null;
  }
  public boolean isAnnoOpenAt(int n) {
    return openAnnos[n] != null;
  }
  public Annotation getOpenAnnoAt(int n) {
    return openAnnos[n];
  }

  /* archaic?
  Annotation expandedAnno = null;
  void setExpandedAnno(Annotation anno) { expandedAnno = anno; }
  void unsetExpandedAnno() { expandedAnno = null; }
  Annotation getExpandedAnno() { return expandedAnno; }
  */

  FluidComponent activeComponent = null;
  void setActiveComponent(FluidComponent comp) { activeComponent = comp; }
  void unsetActiveComponent() { activeComponent = null; }
  FluidComponent getActiveComponent() { return activeComponent; }

  boolean stillInside(Annotation exp, int x, int y) {
    if (exp == null) {
      return true;
    }
    return exp.bigBounds().contains(x, y);
  }

  protected Vector expandedAnnos = new Vector();
  public Vector getExpandedAnnos() { return expandedAnnos; }
  public void setExpandedAnnos(Vector v) { expandedAnnos = v; }  
  public void removeFromExpandedAnnos(Annotation anno) {
    expandedAnnos.removeElement(anno);
  }
  protected void removeAllExpandedAnnos() {
    expandedAnnos.removeAllElements();
  }
  public void addToExpandedAnnos(Annotation anno) {
    expandedAnnos.addElement(anno);
  }
  public Vector expandedAnnosIntersecting(Rectangle rect) {
    Vector v = new Vector();
    for (int i = 0; i < expandedAnnos.size(); i++) {
      Annotation anno = (Annotation)expandedAnnos.elementAt(i);
      if (anno.absoluteBigBounds().intersects(rect)) {
        v.addElement(anno);
      }
    }
    return v;
  }

  // orphanAnnos are to handle frozen margin callouts, but can also handle
  // other annos whose bboxes don't show up in their anchor bboxes
  protected Vector orphanAnnos = new Vector();
  protected void setOrphanAnnos(Vector v) { orphanAnnos = v; }
  public void removeOrphanAnno(Annotation anno) {
    orphanAnnos.removeElement(anno);
  }
  protected void removeAllOrphanAnnos() {
    orphanAnnos.removeAllElements();
  }
  public void addOrphanAnno(Annotation anno) {
    orphanAnnos.insertElementAt(anno, 0);
  }

  public boolean isOrphan(Annotation anno) {
    return orphanAnnos.contains(anno);
  }

  public FluidComponent getComponentAt(Point p) {
    for (int i = 0; i < orphanAnnos.size(); i++) {
      Annotation anno = (Annotation)orphanAnnos.elementAt(i);
      if (anno.getAbsoluteOrphanBounds().contains(p.x, p.y)) {
        return anno.getComponentAt(p);
      }
    }
    return super.getComponentAt(p);
  }

  public void mouseDown(Event evt, int x, int y) {
    if (evt.clickCount >= 2) {
      reload();
    } else if (evt.controlDown()) {
      back();
    }
  }

}


/* $Log: /FluidDocs/annotation/Page.java $
 * 
 * 25    7/26/99 11:04a Bchang
 * fixed history mechanism -- now it works correctly even if you visit a
 * page you've already seen in the "back" history list
 * 
 * 24    7/23/99 1:46p Bchang
 * replaced orphan event handling with better scheme -- just override
 * getComponentAt()
 * 
 * 23    7/23/99 9:09a Bchang
 * allow orphans that are clicked on to forward the event to their
 * components
 * orphans are now added to the front of the orphan list
 * 
 * 22    7/22/99 8:07p Bchang
 * add expanded annos to the things we save and restore on load/back/etc
 * 
 * 21    7/20/99 11:19a Bchang
 * bug fix: remove all vectors of annos on clearStack() (which is called
 * when a new corpus is loaded with loadnew()).
 * 
 * 20    7/19/99 1:00p Bchang
 * uses AnnoDoc.directory to find files in the app version
 * 
 * 19    7/16/99 3:27p Bchang
 * commented out some debugging code
 * 
 * 18    7/16/99 10:38a Bchang
 * add orphanAnnos to support clicking on frozen margin callouts
 *
 * 17    7/15/99 6:36p Bchang
 * added history mechanism which keeps the state of all pages seen
 * footnote technique will not draw footnote line if page is blank
 *
 * 16    7/15/99 11:16a Bchang
 * created a stack for lastAnnoTraversed -- now it works for successive
 * 'back's.
 *
 * 15    7/14/99 12:41p Bchang
 * added loadnew() to load in a new file (blowing away any "back" history)
 *
 * 14    7/14/99 8:33a Bchang
 * Added call to lastAnnoTraversed.returnFromLink() on back().
 * added logging of BACK
 * commented out expandedAnno -- no longer used
 *
 * 13    7/13/99 5:43p Mackinla
 * Logging - basic functionality
 *
 * 12    6/30/99 1:13p Bchang
 * add drawSelf for footnote technique.
 * add methods for calculating footnote placement.
 * 
 * 11    6/28/99 10:49a Bchang
 * added GroupedGlossParser option
 * 
 * 10    6/24/99 9:50a Bchang
 * support for singleAnnoPath (only one anno open at a time) -- added
 * revertListeners and openAnnos
 * 
 * 9     5/20/99 12:15p Bchang
 * checks if fileformat is Outline; if so, call outlineparser instead of
 * xml parser.
 * 
 * 8     5/18/99 4:15p Bchang
 * (Pollez) lastAnnoTraversed points to the last gloss that was followed
 * 
 * 7     1/27/99 2:37p Bchang
 * added revertQueue for nested annotation handling
 * 
 * 6     1/27/99 3:02a Bchang
 * added revertingAnimation instance var for nested annotations
 * 
 * 5     1/27/99 12:01a Bchang
 * add back button.
 * pass docbase in to xml handler, in order to make it available to nested
 * glosses during parsing (like image glosses).
 * 
 * 4     1/21/99 11:58a Bchang
 * Added access to docbase via getDocBase().
 * Improved error messages when reading in bad files;
 * now clicking on a non-existent link will be a no-op.
 * 
 * 3     1/13/99 5:31p Bchang
 * Added $Log at bottom of file
 */
