// Picture.java
// Copyright (c) 1999 by Xerox Corporation
// bchang@parc.xerox.com

// $Header: /FluidDocs/annotation/Picture.java 4     7/19/99 1:01p Bchang $

package annotation;

import fluid.*;
import java.awt.*;
import java.net.*;

public class Picture extends FluidComponent {
  public Picture(String src, String cap, String fname, int fstyle, int fsize) {
    source = src;
    caption = cap;
    setSize(new Dimension(0, 0));
    font = new Font(fname, fstyle, fsize);
    FontMetrics fontMetrics = Toolkit.getDefaultToolkit().getFontMetrics(font);
    ascent = fontMetrics.getAscent();
    setColor(Color.black);
  }

  protected int ascent;
  protected Font font;
  protected String source = "";
  protected String caption = "";
  
  protected Image image = null;
  protected Dimension imageSize = new Dimension(0, 0);
  
  // call after it's been added to the component hierarchy
  protected void loadImage() {
    Toolkit toolkit = Toolkit.getDefaultToolkit();
    if (AnnoDoc.inApplet) {
      URL docbase = ((Page)getPage()).getDocBase();
      URL url;
      try {
        url = new URL(docbase, source);
      } catch (MalformedURLException e) {
        System.out.println("Could not load image '" + source + "' because of badly formed URL.");
        return;
      }
      image = toolkit.getImage(url);
    } else {
      image = toolkit.getImage(AnnoDoc.directory + source);
    }
    FluidUI canvas = getCanvas();
    try {
      MediaTracker tracker = new MediaTracker(canvas);
      tracker.addImage(image, 0);
      tracker.waitForID(0);
    } catch (InterruptedException e) {}
    int w = image.getWidth(canvas);
    int h = image.getHeight(canvas);
    if ((w == -1) || (h == -1)) {
      // couldn't get width or height for some reason
      image = null;
      System.out.println("Could not get size of image.");
    } else {
      imageSize = new Dimension(w, h);
    }
    setSize(imageSize);
  }

  public void drawSelf(Graphics g) {
    drawCaption(g);
    drawImage(g);
  }


  protected void drawCaption(Graphics g) {
    g.setColor(color);
    g.setFont(font);
    g.drawString(caption, 0, ascent);
  }
  
  int imageX = 0;
  int imageY = 30;
  
  protected void drawImage(Graphics g) {
    if (image != null) {
      g.drawImage(image, imageX, imageY, imageSize.width, imageSize.height, getCanvas());
    }
  }

}

/* $Log: /FluidDocs/annotation/Picture.java $
 * 
 * 4     7/19/99 1:01p Bchang
 * now works in apps as well as applets
 * 
 * 3     1/25/99 11:06a Bchang
 * added $Log.
 * added ascent so that the caption is drawn at the right y location.
 *
 */
