// TextBlock.java
// Copyright (c) 1999 by Xerox Corporation
// bchang@parc.xerox.com

// $Header: /FluidDocs/annotation/TextBlock.java 29    8/31/99 4:20p Bchang $

package annotation;

import com.microstar.xml.SAXDriver;
import org.xml.sax.*;

import fluid.*;
import java.awt.*;
import java.io.*;
import java.net.*;
import java.util.Vector;

public class TextBlock extends FluidComponent {

  public TextBlock() {}

  protected Vector glosses = new Vector();

  void setParams() {  
    setFontName(AnnoDoc.textfontname);
    setFontSize(AnnoDoc.textfontsize);
    setFontStyle(AnnoDoc.text_fontstyle);
    setTitleFontStyle(AnnoDoc.title_fontstyle);
    setLineSpacing(AnnoDoc.textlinespacing);
    setGrayoutColor(AnnoDoc.grayoutcolor);
    setOffset(0);
  }

  public TextBlock(Point loc, Dimension sz) {
    setParams();
    setLocation(loc);
    setBaseY(loc.y);
    setSize(sz);
    setGraphicalProtocol("fluid.TextGChar");
  }

  protected int baseY;
  public int getBaseY() { return baseY; }
  public void setBaseY(int y) { baseY = y; }

  /*
  public void drawSelf(Graphics g) {
		if (FluidUI.fluidLog.logDataP()) {
			String details = getLogID() + " " + getLogText();
			FluidUI.fluidLog.addLog("TEXTBLOCK", details, false);
		}
    g.setColor(Color.gray);
    g.drawRect(0, 0, size.width, size.height);
  }
  */


  // added for nested annotations: drawing anchorunderline checks this
  public boolean isCompletelyGrown() { return true; }
  public Color subAnnoColor(Color color) { return color; }
  
  public boolean childIsNested() { return false; }

  public Color determineColor(TextLine tl) { return tl.getColor(); }
  
  protected Color grayoutColor = new Color(200, 200, 200);
  public void setGrayoutColor(Color col) { grayoutColor = col; }

  protected int getLineCount() { return getComponentCount(); }
  protected TextLine getLine(int i) { return (TextLine)getComponent(i); }
  protected void removeLines() { removeAllComponents(); }

  // beginning of TextBlock.Paragraph inner class
  protected static class Paragraph {
    public StringBuffer text = new StringBuffer();
    public Vector lines = new Vector();
    public int getLineCount() { return lines.size(); }
    public Line getLine(int i) { return (Line)lines.elementAt(i); }
    public void append(String str) { text.append(expandTabs(str)); }
    protected String expandTabs(String str) {
      // currently expands each tab to a single space
      return str.replace('\t', ' ');
    }
    public String toString() { return text.toString(); }
    public int length() { return text.length(); }
    public String getText(int start, int length) {
      char[] chars = new char[length];
      text.getChars(start, start + length - 1, chars, 0);
      return new String(chars);
    }
    
    public boolean isTitle = false;
    public boolean isTitle() {
      return isTitle;
    }

    // beginning of TextBlock.Paragraph.Line inner class
    protected static class Line {
      public int start;
      public TextLine textLine;
      public Line(int s, TextLine tl) {
        start = s;
        textLine = tl;
      }
      public boolean contains(int n) {
        return (start <= n) &&
               (start + textLine.getTextLength() > n);
      }
    }
    // end of Line inner class

    public void addLine(int start, TextLine tl) {
      lines.addElement(new Line(start, tl));
    }
    public int getLineNumberContaining(int n) {
      for (int i = 0; i < getLineCount(); i++) {
        if (getLine(i).contains(n)) {
          return i;
        }
      }
      return -1;
    }
    public Paragraph.Line getLineContaining(int n) {
      for (int i = 0; i < getLineCount(); i++) {
        Line line = getLine(i);
        if (line.contains(n)) {
          return line;
        }
      }
      return null;
    }
    public Vector getLinesContaining(int a, int b) {
      Vector results = new Vector();
      boolean collect = false;
      for (int i = 0; i < getLineCount(); i++) {
        Line line = getLine(i);
        if (line.contains(a)) {
          collect = true;
        }
        results.addElement(line);
        if (line.contains(b)) {
          return results;
        }
      }
      // should not get here, though
      return results;
    }
  }
	// end of Paragraph inner class

  public int getLineNumberContainingStart(Annotation anno) {
    return getParagraph(anno.anchorParagraphNumber).getLineNumberContaining(anno.anchorStart);
  }

  protected Vector paragraphs = new Vector();
  public int getParagraphCount() { return paragraphs.size(); }
  public Paragraph newParagraph() {
    Paragraph p = new Paragraph();
    paragraphs.addElement(p);
    return p;
  }
  public Paragraph getParagraph(int i) {
    return (Paragraph)paragraphs.elementAt(i);
  }
  public Paragraph currentParagraph() {
    return getParagraph(paragraphs.size() - 1);
  }
  public int currentParagraphNumber() {
    return paragraphs.size() - 1;
  }
  public String getText(int para, int start, int length) {
    return getParagraph(para).getText(start, length);
  }
	public String getLogText() {
		Paragraph firstPara = getParagraph(0);
		if (firstPara.length() > 0) {
			return firstPara.getLine(0).textLine.getLogText();
		}
		return "|?pic?|";
	}
		
  public void appendText(String str) {
    currentParagraph().append(str);
  }


  protected int lineSpacing;
  void setLineSpacing(int ls) { lineSpacing = ls; }
  public int getLineSpacing() { return lineSpacing; }

  protected int offset;
  public void setOffset(int o) { offset = o; }
  public int getOffset() { return offset; }

  // font
  protected String fontName;
  public void setFontName(String fn) {
    fontName = fn;
    invalidateFont();
  }
  public String getFontName() { return fontName; }

  protected int fontStyle;
  public void setFontStyle(int fs) { 
    fontStyle = fs;
    invalidateFont();
  }
  public int getFontStyle() { return fontStyle; }

  protected int titleFontStyle;
  public void setTitleFontStyle(int fs) { 
    titleFontStyle = fs;
  }
  public int getTitleFontStyle() { return titleFontStyle; }
  
  protected int fontSize;
  public void setFontSize(int fsize) {
    fontSize = fsize;
    invalidateFont();
  }
  public int getFontSize() { return fontSize; }

  protected Font font;
  public void setFont(String name, int style, int size) {
    setFontName(name);
    setFontStyle(style);
    setFontSize(size);
    font = new Font(name, style, size);
  }
  protected void setFont() {
    setFont(fontName, fontStyle, fontSize);
  }
  public Font getFont() {
    if (font == null) {
      setFont();
    }
    return font;
  }

  protected FontMetrics fontMetrics;
  protected void setFontMetrics() {
      fontMetrics = Toolkit.getDefaultToolkit().getFontMetrics(getFont());
  }
  protected FontMetrics getFontMetrics() {
    if (fontMetrics == null) {
      setFontMetrics();
    }
    return fontMetrics;
  }

  protected int stringWidth(char[] chars, int start, int length) {
    return getFontMetrics().charsWidth(chars, start, length);
  }

  protected void invalidateFont() {
    font = null;
    fontMetrics = null;
  }

  //

  // creating TextLines and Annotations

  // *** parse document ***
  public void startDocument() {}
  public void endDocument() {}

  // *** create picture page ***
  protected boolean isPicture = false;
  protected String pictureSource, pictureCaption;
  public void startPicture(String source, String caption) {
    isPicture = true;
    pictureSource = source;
    pictureCaption = caption;
  }
  public void endPicture() {}

  // *** parse title ***
  public void startTitle() {
    Paragraph p = newParagraph();
    p.isTitle = true;
  }
  public void endTitle() {
    endParagraph();
  }


  // BEGIN MOD
  public void startKeyword() {
  /*
    Annotation anno = new Annotation(ga);
    anno.setAnchor(currentAnchorParagraphNumber, currentAnchorStart, currentAnchorLength);

    // a weird hack here to get one more character and then throw away the trailing null
    // this has something to do with how Paragraph implements getText(); no time to debug now
    String anchor = getText(currentAnchorParagraphNumber, currentAnchorStart, currentAnchorLength + 1);
    anno.setAnchorText(anchor.substring(0, currentAnchorLength));

    anno.startParagraph();
    anno.setHref("");
    addGloss(anno);
    return anno;
    */
    Paragraph p = newParagraph();
    // p.isTitle = true;
  }
  
  public void endKeyword() {
    endParagraph();
  }
  
  //END MOD


 // *** parse paragraph ***
  public void startParagraph() {
    newParagraph();
  }
  public void endParagraph() {}

  // *** parse anchor ***
  private boolean startingAnchor = false;
  private int currentAnchorParagraphNumber;
  private int currentAnchorStart;
  protected void markAnchorStart() {
    currentAnchorParagraphNumber = currentParagraphNumber();
    currentAnchorStart = currentParagraph().length();
    startingAnchor = true;
  }
  public void startAnchor() {
    markAnchorStart();
  }
  private void adjustAnchor(boolean adjacentToPrevious) {
    if (startingAnchor) {
      if (!adjacentToPrevious) currentAnchorStart++;
      startingAnchor = false;
    }
  }
  private int currentAnchorLength;
  protected void markAnchorEnd() {
    currentAnchorLength = currentParagraph().length() - currentAnchorStart;
  }
  public void endAnchor() {
    markAnchorEnd();
  }

  // *** parse gloss ***
  public void addGloss(TextBlock tb) {
    glosses.addElement(tb);
  }
  // Annotation defines setAnchor()
  protected void setAnchor(StringBuffer para, int start, int length) {}

  public Annotation startGloss(URL docbase, String href, GlossAttributes ga, FluidUI canvas) {
    boolean isImage = ga.type.equalsIgnoreCase("image");
    boolean isControl = ga.type.equalsIgnoreCase("control");
    boolean isComputed = ga.type.equalsIgnoreCase("computed");
    Annotation anno;
    if (isImage) {
      anno = new ImageAnnotation(docbase, ga.source, new Dimension(ga.width, ga.height), canvas);
    } else if (isControl) {
      anno = new ControlAnnotation();
    } else {
      anno = new Annotation(ga);
    }
    anno.setAnchor(currentAnchorParagraphNumber, currentAnchorStart, currentAnchorLength);

    // a weird hack here to get one more character and then throw away the trailing null
    // this has something to do with how Paragraph implements getText(); no time to debug now
    String anchor = getText(currentAnchorParagraphNumber, currentAnchorStart, currentAnchorLength + 1);
    anno.setAnchorText(anchor.substring(0, currentAnchorLength));

    anno.startParagraph();
    anno.setHref(href);
    addGloss(anno);
    if (isComputed) {
	    computeGloss(anno, href, ga.ctag);
	    // any chars inside the gloss field will be tacked on the end for now
	  }
    return anno;
  }

  protected void computeGloss(Annotation anno, String href, String ctag) {
	InputStreamReader isr = null;
    if (FluidUI.inApplet) {
      try {
        URL url = new URL(((Page)getPage()).getDocBase(), href);
        isr = new InputStreamReader(url.openStream());
      } catch (MalformedURLException e) {
        // e.printStackTrace();
        System.out.println("Could not fetch gloss from file '" + href + "' because it was a badly formed URL.");
        return;
      } catch (IOException e) {
        // e.printStackTrace();
        System.out.println("Could not fetch gloss from file '" + href + "' because of an I/O Exception.");
        return;
      }
    } else {
      try {
        File infile = new File(AnnoDoc.directory + href);
        isr = new FileReader(infile);
      } catch (FileNotFoundException e ) {
        // e.printStackTrace();
        System.out.println("Could not fetch gloss from file '" + href + "' because it was not found.");
        return;
      }
    }
    Parser parser = new SAXDriver();
    XMLGetComputedGloss handler = new XMLGetComputedGloss(anno, ctag);
    parser.setDocumentHandler(handler);
    parser.setErrorHandler(handler);
    try {
      if (isr != null) {
        parser.parse(new InputSource(isr));
      }
	} catch (SAXException e) {
      System.out.println("Could not fetch gloss from file '" + href + "' because it had bad XML formatting.");
      e.printStackTrace();
      return;
    } catch (IOException e) {
      // e.printStackTrace(); 
      System.out.println("Could not fetch gloss from file '" + href + "' because of an I/O Exception during parsing.");
      return;
    } 
  }    
	
  public void endGloss() {
    endParagraph();
  }


  public void reconstruct() {
    if (isPicture) {
      constructPicture();
    } else {
      reconstructTextLines();
      reconstructGlosses();
    }
  }

  protected void constructPicture() {
    Picture pic = new Picture(pictureSource, pictureCaption, fontName, titleFontStyle, fontSize);
    add(pic);
    pic.loadImage();
  }

  protected void reconstructGlosses() {
    for (int i = 0; i < glosses.size(); i++) {
      Annotation anno = (Annotation)glosses.elementAt(i);
      reconstructGloss(anno);
    }
  }

  protected void reconstructGloss(Annotation anno) {
    Paragraph para = getParagraph(anno.anchorParagraphNumber);
    int j = getLineNumberContainingStart(anno);
    // check j == -1 because there may be annotations for lines
    // that are past the end of the page (won't be displayed)
    if (j == -1) return;
    int start = anno.anchorStart - para.getLine(j).start;
    int charsleft = anno.anchorLength;
    // check j < para.getLineCount because the last line may have
    // an anchor that continues to the next (unprinted) line
    Vector all = new Vector();
    while ((charsleft > 0) && (j < para.getLineCount())) {
      all.addElement(anno);
      Paragraph.Line line = (Paragraph.Line)para.getLine(j);
      line.textLine.add(anno);
      int len = Math.min(charsleft, line.textLine.getTextLength() - start);
      charsleft -= len;
      int k;
      for (k = start + len - 1; k >= start; k--) {
        if (!Util.isWhitespace(line.textLine.getText().charAt(k))) {
          break;
        }
      }
      anno.initializePositions(start, k - start + 1);
      anno.reconstruct();
      anno.setLinePositions();
      start = 0;
      anno = (Annotation)anno.newCopyAnchor();
      j++;
    }
    if (all.size() > 1) {
      for (int i = 0; i < all.size(); i++) {
        ((Annotation)all.elementAt(i)).setSharedAnchor(all);
      }
    }
  }

  protected void reconstructTextLines() {
    reconstructTextLines(getWidth());
  }
  
  public int nestingLevel() { return 0; }
  protected int topLevelTextWidth() { return getWidth(); }
  protected int firstLineIndent() { return 0; }
  
  protected void reconstructTextLines(int maxwidth) {
    removeLines();
    nextLineNumber = 0;
    for (int p = 0; p < getParagraphCount(); p++) {
      Vector lines = divideIntoLines(getParagraph(p).toString(), firstLineIndent(),
                                     maxwidth, getFontMetrics());
      // if this is an empty paragraph, give it a single blank line
      if (lines.size() == 0) {
        lines.addElement(new LineItem(" ", 0));
      }
      for (int i = 0; i < lines.size(); i++) {
        LineItem line = (LineItem)lines.elementAt(i);
        line.paragraphNumber = p;
        addLine(line, p);
        if (stopAddingLines()) return;
      }
    }
  }

  protected boolean stopAddingLines() {
    int maxLineNumber = getHeight() / lineSpacing;
    return nextLineNumber > maxLineNumber;
  }

  protected static class LineItem {
    public LineItem(String t, int s) {
      text = t;
      start = s;
    }
    public String text;
    public int start;
    public int paragraphNumber;
  }

  protected int xOffsetFromTopLevelText() { return 0; }
  protected int getTopLevelAbsoluteX() { return absoluteX(); }
  protected int getTrueAbsoluteXOfAnchor() {
    System.out.println("Error: should never call getTrueAbsoluteXOfAnchor() on a non-Annotation TextBlock");
    return 0;
  }
  protected int defaultOffsetFromAnchor() {
    System.out.println("Error: should never call defaultOffsetFromAnchor() on a non-Annotation TextBlock");
    return 0;
  }

  protected Vector divideIntoLines(String str, int firstLineIndent,
                                   int maxwidth, FontMetrics metrics) {
    char[] chars = str.toCharArray();
    Vector lines = new Vector();
    for (int i = 0; i < str.length(); ) {
      int m, k;
      int max;
      if (AnnoDoc.wideLayout()) {
        max = (lines.size() == 0) ? maxwidth - firstLineIndent : maxwidth;
      } else if (AnnoDoc.pyramidLayout()) {
        max = (lines.size() * AnnoDoc.pyramidstep) + (maxwidth - firstLineIndent);
        max = Math.min(maxwidth, max);
      } else {
        max = maxwidth;
      }

      for (m = i; m < str.length(); m++) {
        if (metrics.charsWidth(chars, i, m - i + 1) > max) {
          break;
        }
      }
      if (m == str.length()) {
        lines.addElement(new LineItem(str.substring(i), i));
        return lines;
      }
      for (k = m; k > i; k--) {
        if (str.charAt(k - 1) == ' ') break;
      }
      if (k == i) {
        // no spaces were found under the max line width
        if (AnnoDoc.breakWordsIfNecessary) {
          // use the maximum line width
          k = m;
        } else {
          // keep looking for next space
          for (k = m; k < str.length(); k++) {
            if (str.charAt(k - 1) == ' ') break;
          }
        }
      }
      lines.addElement(new LineItem(str.substring(i, k), i));
      i = k;
    }
    return lines;
  }


  /*
  // right now, does not handle annotations with anchors spanning multiple lines
  public void addAnnotations() {

      AnnotationInfo a = (AnnotationInfo)pendingAnnotations.elementAt(i);
      for (int j = a.startLine; j <= a.endLine; j++) {
        if (j < getLineCount()) {
          TextLine tl = getLine(j);
          int start = (j == a.startLine) ? a.startChar : 0;
          int end = (j == a.endLine) ? a.endChar : tl.getText().length();
          if (start != end) { // ie, annotation really starts at beginning of next line
            Annotation anno = new Annotation(a.text, start, end - start, tl);
            tl.add(anno);
          }
        }
      }
    }
  }
  */
  
  public void addToText(String s) {
    addToText(s, false);
  }

  public void addToText(String s, boolean adjacentToPrevious) {
    if (!adjacentToPrevious) appendText(" ");
    adjustAnchor(adjacentToPrevious);
    appendText(s);
  }
  
  
  protected boolean isTitleParagraph(int n) {
    return getParagraph(n).isTitle();
  }

  public boolean isFirstLine(TextLine line) {
    return getLine(0) == line;
  }

  protected int getExpandedXDefault() { return 0; }


  protected int nextLineNumber = 0;

  protected TextLine addLine(LineItem line, int paranum) {
      int style = fontStyle;
      TextLine tline = null;
      if (isTitleParagraph(paranum)) {
          style = titleFontStyle;
          tline = new TextLine(line.text, fontName, style, fontSize,
                               nextLineNumber, lineSpacing, offset);
          tline.setColor(AspectJDemoConfig.TITLE_COLOR);
      }
      else {
          tline = new TextLine(line.text, fontName, style, fontSize,
                                        nextLineNumber, lineSpacing, offset);
      }

    add(tline);
    tline.setBase(); // must call this right here, to set the canonical position
    getParagraph(paranum).addLine(line.start, tline);
    nextLineNumber++;
    return tline;
  }



  // *** negotiation ***

  public void negotiationRevert(Negotiation ngt, double frac) {
    // System.out.println("reverting: " + ngt.policy);
    if (ngt.policy.equals("overlay")) {
      OverlayPolicy(ngt, Color.black, true);
    } else if (ngt.policy.equals("tooltip")) {
      // actually this should never be called because tooltips do not use negotiation
      TooltipPolicy(ngt);
    } else if (ngt.policy.equals("move")) {
      if (openedAnnos.size() == 1) { // ie, this is the last anno
        RevertToCanonical(ngt);
      } else {
        if (ngt.reuser != null) { // ie, someone else has reused this space
          Negotiation rngt = ((Annotation)ngt.reuser).negotiation;
          rngt.policy = "move"; // OR SQUASH?!!!!!
          // two cases: the reuser has opened up extra space or used the same amount
          // or the reuser is smaller
          if (rngt.squashheight >= 0) {
            // case 1: reuser has opened up extra space or used same amount
            // do nothing, just let it shrink
            rngt.offeredSize.height = ngt.offeredSize.height + rngt.squashheight;
          } else {
            // case 2: reuser is smaller than orig anno
            rngt.offeredSize.height = ngt.offeredSize.height + rngt.squashheight;
            int h = -(ngt.offeredSize.height - rngt.offeredSize.height);
            ngt.offeredSize = new Dimension(-ngt.offeredSize.width, h);
            MovePolicy(ngt);
          }
          ((Annotation)ngt.secondary).moveExpandedXToDefault();
          ngt.addAnimation(((Annotation)rngt.secondary).changeExpandedXToDefault());
          ((Page)getPage()).removeOrphanAnno((Annotation)ngt.secondary);
          ((Page)getPage()).removeOrphanAnno((Annotation)rngt.secondary);
          ((Annotation)rngt.secondary).hackother = null;
          ((Annotation)ngt.secondary).hackother = null;
        } else {
          ngt.offeredSize = new Dimension(-ngt.offeredSize.width, (int)(-ngt.offeredSize.height * frac));
          MovePolicy(ngt);
        }
      }
    } else if (ngt.policy.equals("squash")) {

      if (ngt.reuser != null) { // ie, someone else has reused this space
        Negotiation rngt = ((Annotation)ngt.reuser).negotiation;
        rngt.policy = "squash";
        // two cases: the reuser has opened up extra space or used the same amount
        // or the reuser is smaller
        if (rngt.squashheight >= 0) {
          // case 1: reuser has opened up extra space or used same amount
          // do nothing, just let it shrink
          rngt.offeredSize.height = ngt.offeredSize.height + rngt.squashheight;
        } else {
          // case 2: reuser is smaller than orig anno
          rngt.offeredSize.height = ngt.offeredSize.height + rngt.squashheight;
          int h = -(ngt.offeredSize.height - rngt.offeredSize.height);
          ngt.offeredSize = new Dimension(-ngt.offeredSize.width, h);
          SquashPolicy(ngt);
        }
        ((Annotation)ngt.secondary).moveExpandedXToDefault();
         ngt.addAnimation(((Annotation)rngt.secondary).changeExpandedXToDefault());
         ((Page)getPage()).removeOrphanAnno((Annotation)ngt.secondary);
         ((Page)getPage()).removeOrphanAnno((Annotation)rngt.secondary);
      } else { // normal unsquash case
        if (openedAnnos.size() == 1) { // ie, this is the last anno
          RevertToCanonical(ngt);
        } else {
          ngt.offeredSize = new Dimension(-ngt.offeredSize.width, (int)(-ngt.offeredSize.height * frac));
          SquashPolicy(ngt);
        }
      }

    } else if (ngt.policy.equals("interline")) {
      if (openedAnnos.size() == 1) { // ie, this is the last anno
        RevertToCanonical(ngt);
      } else {
        //RevertInterlinePolicy(ngt);
        int remaining = RevertNewMovePolicy(ngt, (int)(-ngt.offeredSize.height * frac));
        if (remaining > 0) {
          System.out.println("revert: ah ha");
        }
      }
    } else if (ngt.policy.equals("combine")) {
      RevertOverlayPolicy(ngt, ((TextLine)ngt.secondary.getParent()).getLineNumber(), ngt.ngray);
      ngt.offeredSize = new Dimension(-ngt.offeredSize.width, (int)(-ngt.squashheight * frac));
      SquashPolicy(ngt);
    } else if (ngt.policy.equals("reuse")) {
      if (openedAnnos.size() == 1) { // ie, this is the last anno
        RevertToCanonical(ngt);
      } else {
        Annotation anno = (Annotation)ngt.reuseanno;
        ngt.addAnimation(anno.changeExpandedXToDefault());
        ngt.addAnimation(((Annotation)ngt.secondary).changeExpandedXToDefault());
        if (ngt.squashheight > 0) {
          ngt.offeredSize = new Dimension(-ngt.offeredSize.width, (int)(-ngt.squashheight * frac));
          if (ngt.subpolicy.equals("squash")) {
            SquashPolicy(ngt);
          } else {
            MovePolicy(ngt);
          }
        }
       ((Page)getPage()).removeOrphanAnno(anno);
       ((Page)getPage()).removeOrphanAnno((Annotation)ngt.secondary);
        anno.hackother = null;
        ((Annotation)ngt.secondary).hackother = null;

        anno.negotiation.reuser = null;
     }

    } else if (ngt.policy.equals("multireuse")) {
      if (openedAnnos.size() == 1) { // ie, this is the last anno
        RevertToCanonical(ngt);
      } else {
        handleOverlappingAnnos(ngt, Color.black, true); // color is ignored here when reverting
        if (ngt.squashheight > 0) {
          ngt.offeredSize = new Dimension(-ngt.offeredSize.width, (int)(-ngt.squashheight * frac));
          if (ngt.subpolicy.equals("squash")) {
            SquashPolicy(ngt);
          } else {
            MovePolicy(ngt);
          }
        }
       ((Page)getPage()).removeOrphanAnno((Annotation)ngt.secondary);
      }
    } else if (ngt.policy.equals("margincallout")) {
      MarginCalloutPolicy(ngt, true);
    } else {
      System.out.println("unknown policy to revert: " + ngt.policy);
    }
    openedAnnos.removeElement(ngt);
  }

  protected Vector openedAnnos = new Vector();

  // actually this already sets the offered size! should move out to a separate method
  public void negotiationSetGuidelines(Negotiation ngt) {
    if (moveonly()) {
      int t = CalculateMovePolicy(ngt);
      if (t > 0) {
        t = t - (t % ((Annotation)ngt.secondary).getLineSpacing());
        ngt.offeredSize = new Dimension(ngt.desiredSize.width, t);
      } else {
        ngt.abort = true;
        return;
      }
    } else {
      ngt.offeredSize = ngt.desiredSize;
    }
    openedAnnos.addElement(ngt);
  }

  boolean overlayonly() {
    return (AnnoDoc.technique.equals("overlay"));
  }

  boolean moveonly() {
    return (AnnoDoc.technique.equals("move") || AnnoDoc.technique.equals("move2") || AnnoDoc.technique.equals("moveonly")
            || AnnoDoc.technique.equals("movenolimit"));
  }

  boolean combine() {
    return (AnnoDoc.technique.equals("combine"));
  }

  boolean squashonly() {
    return (AnnoDoc.technique.equals("squash"));
  }

  boolean squashthenmove() {
    return (AnnoDoc.technique.equals("stm"));
  }

  boolean interline() {
    return (AnnoDoc.technique.equals("xxinterline"));
  }

  boolean reusespace() {
    return (AnnoDoc.technique.equals("reuse"));
  }

  boolean margincallout() {
    return (AnnoDoc.technique.equals("margincallout"));
  }

  boolean tooltip() {
    return (AnnoDoc.technique.equals("tooltip"));
  }

  boolean footnote() {
    return (AnnoDoc.technique.equals("footnote"));
  }

  boolean nogloss() {
    return (AnnoDoc.technique.equals("nogloss"));
  }

  public void negotiationSelectSpaceStrategy(Negotiation ngt) {
    if (moveonly()) {
      MovePolicy(ngt);
      ngt.policy = "move";
    } else if (overlayonly()) {
      OverlayPolicy(ngt);
      ngt.policy = "overlay";
    } else if (tooltip()) {
    //actually, this should never get called because tooltips do not use negotiation
      TooltipPolicy(ngt);
      ngt.policy = "tooltip";
    } else if (squashonly()) {
      SquashPolicy(ngt);
      ngt.policy = "squash";
    } else if (squashthenmove()) {
      if (!SquashPolicy(ngt, false)) {
        if (!MovePolicy(ngt, false)) {
          OverlayPolicy(ngt);
          ngt.policy = "overlay";
        } else {
          MovePolicy(ngt);
          ngt.policy = "move";
        }
      } else {
        SquashPolicy(ngt);
        ngt.policy = "squash";
      }
    } else if (interline()) {
      //InterlinePolicy(ngt);
      int remaining = NewMovePolicy(ngt, ngt.offeredSize.height);
      if (remaining > 0) {
        System.out.println("ah ha");
      }
      ngt.policy = "interline";
    } else if (combine()) {
      int currentLineSpacing = getLine(1).absoluteY() - getLine(0).absoluteY();
      int delta = currentLineSpacing - minLineSpacing;
      int available = delta * (getLineCount() - 1);
      if (available > ngt.desiredSize.height) {
        SquashPolicy(ngt);
        ngt.policy = "squash";
      } else {
        int extra = ngt.desiredSize.height - available;
        ngt.offeredSize.height = available;
        ngt.squashheight = available;
        SquashPolicy(ngt);
        ngt.policy = "squash";
        ngt.offeredSize.height = extra;
        ngt.ngray = CountOverlayPolicy(ngt);
        // make sure policy is set after SquashPolicy() [because SquashPolicy() sets policy too]
        ngt.policy = "combine";
      }

    } else if (reusespace()) {
      // else just squash
      SquashPolicy(ngt);
      ngt.policy = "squash";
    } else if (margincallout()) {
      MarginCalloutPolicy(ngt, false);
      ngt.policy = "margincallout";

    } else { // default is movethensquash
      // check if we can just open up into the same space
      boolean reuse = false;
      String policy = "";
      Annotation newanno = (Annotation)ngt.secondary;
      Vector others = new Vector();
      // there's at least one opened anno -- this one: it's already included
      for (int i = 0; i < openedAnnos.size(); i++) {
        Annotation anno = (Annotation)((Negotiation)openedAnnos.elementAt(i)).secondary;
        if ((anno != ngt.secondary) && (anno.absoluteY() == newanno.absoluteY())) {
          others.addElement(anno);
        }
      }
      if (others.size() == 1) {
        ReuseSpace(ngt, (Annotation)others.elementAt(0));
        reuse = true;
        policy = "reuse";
      } else if (others.size() > 1) {
        handleOverlappingAnnos(ngt, grayoutColor, false);
        int max = 0;
        for (int j = 0; j < others.size(); j++) {
          Annotation other = (Annotation)others.elementAt(j);
          max = Math.max(other.getHeight(), max);
        }
        ngt.squashheight = ngt.offeredSize.height - max;

        ((Page)getPage()).addOrphanAnno((Annotation)ngt.secondary);
        ((Annotation)ngt.secondary).hackother = null;

        reuse = true;
        policy = "multireuse";
      }

      int desired;
      if (reuse) {
        if (ngt.squashheight <= 0) {
          // new anno is smaller than existing space, so do no animation, just return
          ngt.policy = policy;
          return;
        } else {
          desired = ngt.squashheight;
        }
      } else {
        desired = ngt.offeredSize.height;
      }

      if (!MovePolicy(ngt, desired, false)) {
        SquashPolicy(ngt, desired, true);
        if (reuse) {
          ngt.policy = policy;
          ngt.subpolicy = "squash";
        } else {
          ngt.policy = "squash";
        }
      } else {
        MovePolicy(ngt, desired, true);
        if (reuse) {
          ngt.policy = policy;
          ngt.subpolicy = "move";
        } else {
          ngt.policy = "move";
        }
      }
    }
  }

  /* a little bit of archived code: move then squash then overlay
      if (!MovePolicy(ngt, false)) {
        if (!SquashPolicy(ngt, false)) {
          OverlayPolicy(ngt);
          ngt.policy = "overlay";
        } else {
          SquashPolicy(ngt);
          ngt.policy = "squash";
        }
      } else {
        MovePolicy(ngt);
        ngt.policy = "move";
      }
  */

  protected void ReuseSpace(Negotiation ngt, Annotation anno) {
    Annotation newanno = (Annotation)ngt.secondary;

    Rectangle abb = anno.absoluteBigBounds();
    Rectangle nabb = newanno.absoluteBigBounds();

    /* I've disabled this for now
    // reflow the new anno if there's not enough width
    int maxwidth = getWidth() - abb.width;
    if (nabb.width > maxwidth) {
      newanno.reconstructTextLines(maxwidth);
      newanno.setLinePositions();
      newanno.reconstructOnGrow = true;
    }
    nabb = newanno.absoluteBigBounds();
    */

    // handle whatever sliding horizontally is necessary
    if (anno.absoluteX() < newanno.absoluteX()) {
      int rightedge = abb.x + abb.width;
      // mb is the amount the two overlap initially (before we slide one over)
      int mb = nabb.x - rightedge;
      if (mb > 0) {
        // do nothing
      } else {
        int moveorig = mb;
        int movenew = 0;
        if ((abb.x + mb) < 0) { // remember, mb is negative
          moveorig = -abb.x;
          movenew = -mb + moveorig;
          newanno.moveExpandedX(movenew);
        }
        ngt.addAnimation(anno.changeExpandedX(moveorig));

        anno.hackx1 = abb.x + moveorig;
        anno.hackx2 = abb.x + abb.width + moveorig;
        anno.hacky1 = abb.y;
        anno.hacky2 = abb.y + abb.height;
        anno.hackother = newanno;
        newanno.hackx1 = nabb.x + movenew;
        newanno.hackx2 = nabb.x + nabb.width + movenew;
        newanno.hacky1 = nabb.y;
        newanno.hacky2 = nabb.y + nabb.height;
        newanno.hackother = anno;

        // should check these are actually in the margins
        ((Page)getPage()).addOrphanAnno(anno);
        ((Page)getPage()).addOrphanAnno(newanno);
      }
    } else {
      int leftedge = abb.x;
      int mf = nabb.x + nabb.width - leftedge;

      if (mf < 0) {
        // do nothing
      } else {
        int moveorig = mf;
        int movenew = 0;
        int farright = getParent().getWidth() - 20;
        if ((mf + abb.x + abb.width) > farright) {
          moveorig = farright - (abb.x + abb.width);
          movenew = -(mf - moveorig);
          newanno.moveExpandedX(movenew);
        }
        // this adds moveorig to expandedX
        ngt.addAnimation(anno.changeExpandedX(moveorig));

        anno.hackx1 = abb.x + moveorig;
        anno.hackx2 = abb.x + abb.width + moveorig;
        anno.hacky1 = abb.y;
        anno.hacky2 = abb.y + abb.height;
        anno.hackother = newanno;
        newanno.hackx1 = nabb.x + movenew;
        newanno.hackx2 = nabb.x + nabb.width + movenew;
        newanno.hacky1 = nabb.y;
        newanno.hacky2 = nabb.y + nabb.height;
        newanno.hackother = anno;

        // should check these are actually in the margins
        ((Page)getPage()).addOrphanAnno(anno);
        ((Page)getPage()).addOrphanAnno(newanno);

      }
    }


    // note to self: should introduce the concept of ngt.usedSize
    // squashheight can be negative -- which means that the already open anno is larger than the new anno
    ngt.squashheight = nabb.height - anno.negotiation.offeredSize.height;

    ngt.reuseanno = anno;
    anno.negotiation.reuser = ngt.secondary; // also known as newanno
  }


  // animates all lines to their canonical y locations
  public void RevertToCanonical(Negotiation ngt) {
    if (getY() != getBaseY()) {
      ngt.addAnimation(changeY(getBaseY()));
    }
    for (int i = 0; i < getLineCount(); i++) {
      TextLine line = getLine(i);
      ngt.addAnimation(line.changeY(line.getBaseY()));
    }
  }

  public void MarginCalloutPolicy(Negotiation ngt, boolean reverting) {
    handleOverlappingAnnos(ngt, grayoutColor, reverting);
  }

  protected int calloutXOffset = 10;

  public int getLeftCalloutX() {
    return -calloutXOffset;
  }

  public int getRightCalloutX() {
    return getWidth() + calloutXOffset;
  }

  public int getLeftCalloutWidth() {
    return getX() - (2 * calloutXOffset);
  }

  public int getRightCalloutWidth() {
    return getParent().getWidth() - getRight() - (2 * calloutXOffset);
  }

  public boolean TooltipPolicy(Negotiation ngt) {
    // always return true because tooltips don't require anything of the primary
    return true;
  }

  public boolean OverlayPolicy(Negotiation ngt) {
    return OverlayPolicy(ngt, grayoutColor, false);
  }
  public boolean OverlayPolicy(Negotiation ngt, Color col, boolean reverting) {
    // right now do an Anno cast, but should this be elevated to FluidComp?
    int y1 = ((Annotation)ngt.secondary).absoluteExpandedY();
    int y2 = y1 + ngt.offeredSize.height;
    for (int i = 0; i < getLineCount(); i++) {
      boolean dochange = true;
      TextLine line = getLine(i);
      int y = line.absoluteY();
      if ((y1 <= y + line.getAscent()) && (y <= y2)) {
        if (reverting) {
          line.removeOverlay(ngt);
          // if this line still has overlays, don't fade back to black
          if (!line.overlaysIsEmpty()) {
            dochange = false;
          };
        } else {
          line.addOverlay(ngt);
        }
        if (dochange) {
          ngt.addAnimation(line.changeColor(col));
          for (int j = 0; j < line.getComponentCount(); j++) {
            ngt.addAnimation(line.getComponent(j).changeColor(col));
          }
        }
      }
    }
    handleOverlappingAnnos(ngt, col, reverting);
    return true;
  }

  protected void handleOverlappingAnnos(Negotiation ngt, Color col, boolean reverting) {
    Rectangle rect = ((Annotation)ngt.secondary).absoluteBigBounds();
    Vector annosIOverlap =((Page)getPage()).expandedAnnosIntersecting(rect);
    for (int i = 0; i < annosIOverlap.size(); i++) {
      Annotation anno = (Annotation)annosIOverlap.elementAt(i);
      if (reverting) {
        ((Annotation)ngt.secondary).removeFromAnnosIOverlap(anno);
        anno.removeFromOverlappedBy((Annotation)ngt.secondary);
        if (anno.overlappedByIsEmpty()) {
          ngt.addAnimation(anno.changeExpandedColorToFrozen());
        }
      } else {
        ((Annotation)ngt.secondary).addToAnnosIOverlap(anno);
        anno.addToOverlappedBy((Annotation)ngt.secondary);
        ngt.addAnimation(anno.changeExpandedColor(col));
      }
    }
  }

  public int CountOverlayPolicy(Negotiation ngt) {
    Color col = new Color(200, 200, 200);
    int y1 = ((Annotation)ngt.secondary).absoluteExpandedY();
    int y2 = y1 + ngt.offeredSize.height;
    int n = 0;
    for (int i = 0; i < getLineCount(); i++) {
      TextLine line = getLine(i);
      int y = line.absoluteY();
      if ((y1 <= y + line.getAscent()) && (y <= y2)) {
        n++;
        ngt.addAnimation(line.changeColor(col));
        for (int j = 0; j < line.getComponentCount(); j++) {
          ngt.addAnimation(line.getComponent(j).changeColor(col));
        }
      }
    }
    return n;
  }

  public void RevertOverlayPolicy(Negotiation ngt, int linenum, int n) {
    for (int i = linenum + 1; i <= linenum + n; i++) {
      boolean dochange = true;
      TextLine line = getLine(i);
      line.removeOverlay(ngt);
      // if this line still has overlays, don't fade back to black
      if (!line.overlaysIsEmpty()) {
        dochange = false;
      }
      if (dochange) {
        ngt.addAnimation(line.changeColor(Color.black));
        for (int j = 0; j < line.getComponentCount(); j++) {
          ngt.addAnimation(line.getComponent(j).changeColor(Color.black));
        }
      }
    }
    Rectangle rect = ((Annotation)ngt.secondary).absoluteBigBounds();
    Vector annosIOverlap = ((Page)getPage()).expandedAnnosIntersecting(rect);
    for (int i = 0; i < annosIOverlap.size(); i++) {
      Annotation anno = (Annotation)annosIOverlap.elementAt(i);
      ((Annotation)ngt.secondary).removeFromAnnosIOverlap(anno);
      anno.removeFromOverlappedBy((Annotation)ngt.secondary);
      if (anno.overlappedByIsEmpty()) {
        ngt.addAnimation(anno.changeExpandedColorToFrozen());
      }
    }
  }

  protected int minLineSpacing = 20;

  

  public boolean SquashPolicy(Negotiation ngt) {
    return SquashPolicy(ngt, true);
  }
  public boolean SquashPolicy(Negotiation ngt, boolean execute) {
    return SquashPolicy(ngt, ngt.offeredSize.height, true);
  }
  public boolean SquashPolicy(Negotiation ngt, int desired, boolean execute) {

    // d is the lineheight of the annotation
    double d = ((double)desired) / (getLineCount() - 1);
    int y1 = 0;
    for (int i = 0; i < getLineCount(); i++) {
      int newy;
      if (i <= ((TextLine)ngt.secondary.getParent()).getLineNumber()) {
        newy = getLine(i).getY() - (int)Math.round(d * i);
      } else {
        newy = getLine(i).getY() + (int)Math.round(d * (getLineCount() - i));
      }
      if (execute) {
        ngt.addAnimation(getLine(i).changeY(newy));
      } else {
        // this tests the linespacing between lines, using lines 0 and 1
        // if the linespacing is less than the minimum, return false
        if (i == 0) {
          y1 = newy;
        }
        if (i == 1) {
          if ((newy - y1) < minLineSpacing) {
            return false;
          } else {
           return true;
          }
        }
      }
    }
    return true;
  }


  public boolean OriginalSquashPolicy(Negotiation ngt, boolean execute) {
    double d = ((double)ngt.offeredSize.height) / (getLineCount() - 2);
    int y1 = 0;
    for (int i = 0; i < getLineCount(); i++) {
      int newy;
      if (getLine(i).absoluteY() <= ngt.secondary.absoluteY()) {
        newy = getLine(i).getY() - (int)Math.round(d * i);
      } else {
        newy = getLine(i).getY() + (int)Math.round(d * (getLineCount() - i));
      }
      if (execute) {
        ngt.addAnimation(getLine(i).changeY(newy));
      } else {
        if (i == 0) {
          y1 = newy;
        }
        if (i == 1) {
          if ((newy - y1) < 16) {
            return false;
          } else {
            return true;
          }
        }
      }
    }
    return true;
  }
  
  
  protected int remainderLineNum = -1;
  
  public void InterlinePolicy(Negotiation ngt, int desired) {
    // we subtract 2 from getLineCount() because we do not move the positions of
    // the first or last lines.
    // probably should change it so we don't take any out from already open annotations also.
    int amount = desired / (getLineCount() - 2);
    int remainder = desired % (getLineCount() - 2);
    int thislinenum = ((TextLine)ngt.secondary.getParent()).getLineNumber();

    // note boundaries: since we're not going to move the position of first or last lines
    for (int i = 1; i < (getLineCount() - 1); i++) {
      TextLine line = getLine(i);
      int newy;
      if (i <= thislinenum) {
        newy = line.getY() - (amount * i);
      } else {
        newy = line.getY() + (amount * (getLineCount() - i));
      }
      ngt.addAnimation(line.changeY(newy));
    }
  }

  // always move down as far as possible, then move up
  // -- makes sense if the page's text doesn't go all the way to the bottom
  // returns amount remaining
  public int NewMovePolicy(Negotiation ngt, int desired) {
    if (desired == 0) {
      return 0;
    }
    if (desired < 0) {
      return RevertNewMovePolicy(ngt, desired);
    }

    TextLine lastline = getLine(getLineCount() - 1);
    int spaceBelow = Math.max(0, getParent().getBottom() - (lastline.absoluteY() + lastline.getAscent())
                                - AnnoDoc.minmargin);
    int spaceAbove = Math.max(0, absoluteY() - (getParent().getTop() + AnnoDoc.minmargin));

    int thislinenum = ((TextLine)ngt.secondary.getParent()).getLineNumber();

    // case 1: enough room below
    if (spaceBelow >= desired) {
      for (int i = thislinenum + 1; i < getLineCount(); i++) {
        TextLine line = getLine(i);
        ngt.addAnimation(line.changeY(line.getY() + desired));
      }
      return 0;

    // case 2: enough room above and below
    } else if (spaceBelow + spaceAbove >= desired) {
      // move the whole block up
      ngt.addAnimation(changeY(getY() - (desired - spaceBelow)));
      // now move the lines below the anno down
      if (spaceBelow > 0) {
        for (int i = thislinenum + 1; i < getLineCount(); i++) {
          TextLine line = getLine(i);
          ngt.addAnimation(line.changeY(line.getY() + desired));
        }
      }
      return 0;

    // case 3: not enough room above & below, just move as much as possible
    } else {
      // first, move into any available margin space
      int remaining = desired;
      int marginSpace = spaceBelow + spaceAbove;
      if (marginSpace > 0) {
        if (spaceBelow > 0) {
          for (int i = thislinenum + 1; i < getLineCount(); i++) {
            TextLine line = getLine(i);
            ngt.addAnimation(line.changeY(line.getY() + marginSpace));
          }
        }
        if (spaceAbove > 0) { // because we know that all of spaceBelow and spaceAbove have to be used
          ngt.addAnimation(changeY(getY() - spaceAbove));
        }
        remaining -= marginSpace;
      }
      return remaining;
    }
  }

  // returns amount remaining
  public int RevertNewMovePolicy(Negotiation ngt, int negdesired) {
    if (negdesired >= 0) {
      return RevertNewMovePolicy(ngt, negdesired);
    }

    int desired = -negdesired;

    int usedSpaceAbove = Math.max(0, getBaseY() - getY());
    TextLine lastline = getLine(getLineCount() - 1);
    int usedSpaceBelow = Math.max(0, lastline.absoluteY() - lastline.getAbsoluteBaseY());

    int thislinenum = ((TextLine)ngt.secondary.getParent()).getLineNumber();

    // case 1: enough room above
    if (usedSpaceAbove > desired) {
      System.out.println("1");
      ngt.addAnimation(changeY(getY() + desired));
      for (int i = thislinenum + 1; i < getLineCount(); i++) {
        TextLine line = getLine(i);
        ngt.addAnimation(line.changeY(line.getY() - desired));
      }
      return 0;

    // case 2: enough room above and below
    } else if (usedSpaceAbove + usedSpaceBelow >= desired) {
      System.out.println("2");
      if (usedSpaceAbove > 0) {
        ngt.addAnimation(changeY(getY() + usedSpaceAbove));
      }
      for (int i = thislinenum + 1; i < getLineCount(); i++) {
        TextLine line = getLine(i);
        ngt.addAnimation(line.changeY(line.getY() - desired));
      }
      return 0;

    // case 3: not enough room above & below, just move as much as possible
    } else {
      System.out.println("3");
      if (usedSpaceAbove > 0) {
        ngt.addAnimation(changeY(getY() + usedSpaceAbove));
      }
      if (usedSpaceBelow > 0) {
        for (int i = thislinenum + 1; i < getLineCount(); i++) {
          TextLine line = getLine(i);
          ngt.addAnimation(line.changeY(line.getY() - usedSpaceBelow));
        }
      }
      return desired - usedSpaceAbove - usedSpaceBelow;
    }
  }




  public boolean MovePolicy(Negotiation ngt) {
    return MovePolicy(ngt, true);
  }
  public boolean MovePolicy(Negotiation ngt, boolean execute) {
    return MovePolicy(ngt, ngt.offeredSize.height, true);
  }

  public boolean MovePolicy(Negotiation ngt, int desired, boolean execute) {
    if (desired < 0) {
      return RevertMovePolicy(ngt, -desired, execute);
    }
    int dyAbove = 0, dyBelow = 0;

    TextLine lastline = getLine(getLineCount() - 1);
    int spaceBelow = Math.max(0, getParent().getBottom() - (lastline.absoluteY() + lastline.getAscent())
                                - AnnoDoc.minmargin);
    int spaceAbove = Math.max(0, absoluteY() - (getParent().getTop() + AnnoDoc.minmargin));

    int thislinenum = ((TextLine)ngt.secondary.getParent()).getLineNumber();

    if (spaceBelow > desired) {
      dyBelow = desired;
    } else if (spaceBelow + spaceAbove > desired) {
      dyBelow = spaceBelow;
      dyAbove = desired - spaceBelow;
    } else {
      // whether execute is true or not, don't move and return false
      return false;
    }

    if (!execute) return true;

    for (int i = 0; i < getLineCount(); i++) {
      int newy;
      if (i <= thislinenum) {
        newy = getLine(i).getY() - dyAbove;
      } else {
        newy = getLine(i).getY() + dyBelow;
      }
      ngt.addAnimation(getLine(i).changeY(newy));
    }

    // since TextBlocks do not really track the size of their components,
    // it can happen that components end up outside the bounds of the textblock.
    // this is to ensure that the textblock at least is as tall as its last line.
    int lastlineBottom = lastline.getBottom() + dyBelow;
    int diff = lastlineBottom - getHeight();
    if (diff > 0) {
      setHeight(getHeight() + diff);
    }
    return true;
  }

  public boolean RevertMovePolicy(Negotiation ngt, int desired, boolean execute) {
    int dyAbove = 0, dyBelow = 0;

    int usedSpaceAbove = Math.max(0, getBaseY() - getY());
    TextLine lastline = getLine(getLineCount() - 1);
    int usedSpaceBelow = Math.max(0, lastline.absoluteY() - lastline.getAbsoluteBaseY());

    int thislinenum = ((TextLine)ngt.secondary.getParent()).getLineNumber();

    // case 1: enough room above
    if (usedSpaceAbove > desired) {
      if (!execute) return true;
      ngt.addAnimation(changeY(getY() + desired));
      for (int i = thislinenum + 1; i < getLineCount(); i++) {
        TextLine line = getLine(i);
        ngt.addAnimation(line.changeY(line.getY() - desired));
      }
      return true;

    // case 2: enough room above and below
    } else if (usedSpaceAbove + usedSpaceBelow >= desired) {
      if (!execute) return true;
      if (usedSpaceAbove > 0) {
        ngt.addAnimation(changeY(getY() + usedSpaceAbove));
      }
      for (int i = thislinenum + 1; i < getLineCount(); i++) {
        TextLine line = getLine(i);
        ngt.addAnimation(line.changeY(line.getY() - desired));
      }
      return true;

    // case 3: not enough room above & below, just move as much as possible
    } else {
      if (!execute) return false;
      if (usedSpaceAbove > 0) {
        ngt.addAnimation(changeY(getY() + usedSpaceAbove));
      }
      if (usedSpaceBelow > 0) {
        for (int i = thislinenum + 1; i < getLineCount(); i++) {
          TextLine line = getLine(i);
          ngt.addAnimation(line.changeY(line.getY() - usedSpaceBelow));
        }
      }
      return false;
    }
  }

  public boolean ProportionalMovePolicy(Negotiation ngt, boolean execute) {
    int dyAbove, dyBelow;
    int dy = ngt.offeredSize.height;

    // move above and below proportional to where we are on the screen
    double f;
    if (dy < 0) { // we are reverting
      f = ngt.verticalFraction;
    } else {
      f = ((double)(ngt.secondary.absoluteY() - absoluteY())) / getHeight();
      ngt.verticalFraction = f;
    }

    dyAbove = (int)Math.round(f * dy);
    dyBelow = (int)Math.round((1 - f) * dy);

    /* old policy was to move up and down equally
      dyAbove = dy / 2;
      dyBelow = dy / 2;
    */

    int newy = getY() - dyAbove;
    if (execute) {
      ngt.addAnimation(changeY(newy));
      size.height += dy;
    } else {
      if (newy + absoluteY() < AnnoDoc.minmargin) {
        return false;
      }
    }

    for (int i = 0; i < getLineCount(); i++) {
      if (i > ((TextLine)ngt.secondary.getParent()).getLineNumber()) {
        newy = getLine(i).getY() + dy;
        if (execute) {
          ngt.addAnimation(getLine(i).changeY(newy));
        } else {
          if (newy + absoluteY() > getParent().getBottom() - AnnoDoc.minmargin) {
            return false;
          }
        }
      }
    }

    return true;
  }


  public boolean MovePolicyWithoutAdjustingBounds(Negotiation ngt, boolean execute) {
    int dyAbove, dyBelow;
    int dy = ngt.offeredSize.height;

    // move above and below proportional to where we are on the screen
    double f;
    if (dy < 0) { // we are reverting
      f = ngt.verticalFraction;
    } else {
      f = ((double)(ngt.secondary.absoluteY() - absoluteY())) / getHeight();
      ngt.verticalFraction = f;
    }

    dyAbove = (int)Math.round(f * dy);
    dyBelow = (int)Math.round((1 - f) * dy);

    /* old policy was to move up and down equally
      dyAbove = dy / 2;
      dyBelow = dy / 2;
    */

    for (int i = 0; i < getLineCount(); i++) {
      int newy;
      if (i <= ((TextLine)ngt.secondary.getParent()).getLineNumber()) {
        newy = getLine(i).getY() - dyAbove;
      } else {
        newy = getLine(i).getY() + dyBelow;
      }
      if (execute) {
        ngt.addAnimation(getLine(i).changeY(newy));
      } else {
        if ((newy + absoluteY() < AnnoDoc.minmargin) ||
            (newy + absoluteY() > getParent().getBottom() - AnnoDoc.minmargin)) {
          return false;
        }
      }
    }

    return true;
  }

  public int CalculateMovePolicy(Negotiation ngt) {

    int r = ngt.desiredSize.height / 2;
    if (!AnnoDoc.technique.equals("movenolimit")) {
      int top = getLine(0).absoluteY() - AnnoDoc.minmargin;
      int bot = getParent().getBottom() - (getLine(getLineCount() - 1).absoluteY() -  + getHeight()) - AnnoDoc.minmargin;
      r = Math.min(r, top);
      r = Math.min(r, bot);
    }
    return r * 2;
  }

}

/* $Log: /FluidDocs/annotation/TextBlock.java $
 * 
 * 29    8/31/99 4:20p Bchang
 * setAnchorText() for annos during parsing -- added for GetTextLines
 * 
 * 28    7/23/99 1:41p Bchang
 * hack (hackx, hacky, hackother) to get the right anno to receive events
 * in the "reuse" condition -- when more than one anno opens up in the
 * same interline space
 * 
 * 27    7/23/99 9:06a Bchang
 * shared anchors (anchors that span more than one line)
 * able to mouse on "reused" annos that have moved into the margin
 * 
 * 26    7/22/99 8:10p Bchang
 * reuse policy now works for movethensquash. it allows 2 annos to be
 * shown simultaneously on the same line; they will move aside if
 * necessary. a 3rd anno can be expanded, in which case any of the other
 * are two that are overlapped are grayed out. however, the behavior only
 * works correctly if that 3rd anno is unfrozen first, before the original
 * 2. you may see some layout problems if one of the original 2 is
 * unfrozen before the 3rd anno. however, all will be restored when
 * they're all unfrozen. four or more open per line should work if they
 * are closed in the same order that they are open.
 * 
 * 25    7/22/99 4:52p Bchang
 * use AnnoDoc.minmargin for move technique
 * change move technique to use space below first, then space above
 * make reuse w/ grayout work for 2 annotations on a single line under
 * movethensquash
 * 
 * 24    7/22/99 9:14a Bchang
 * for squash & move conditions: if reverting back to
 * no-annotations-opened, then animate directly to saved canonical Y
 * positions for all text lines and for the text block
 * 
 * 23    7/20/99 9:45a Bchang
 * margin callout now grays out any overlapped callouts
 * 
 * 22    7/19/99 1:02p Bchang
 * AnnoDoc.what replaced by AnnoDoc.technique
 * file prepended with AnnoDoc.technique
 * 
 * 21    7/15/99 11:19a Bchang
 * made default (unrecognized) technique "move then squash" 
 * used to be move then squash then grayout
 * squash will keep on squashing...
 * 
 * 20    7/15/99 10:20a Bchang
 * expand tabs to a single space
 *
 * 18    7/13/99 5:44p Mackinla
 * Logging - basic functionality
 *
 * 17    6/30/99 1:15p Bchang
 * add tooltip() and footnote() for checking if that technique is active.
 * also added some tooltip() code in negotiationrevert and
 * negotiationselectstrategy, but this code does nothing because tooltip
 * and footnote do not use negotiation.
 *
 * 16    6/24/99 9:53a Bchang
 * support for wide and pyramid layouts
 * option for not breaking words when dividing text into lines
 * new "movenolimit" policy which permits moving past the page top and
 * bottom
 * movepolicy now chooses top and bottom move amount based on where on the
 * page the anno is opening -- used to be just 50% up and 50% down.
 *
 * 15    5/20/99 12:16p Bchang
 * added addToText(String) which defaults the value of adjacentToPrevious
 * 
 * 14    5/18/99 4:16p Bchang
 * (Pollez) add determineColor() -- TextLine calls this to decide what
 * color to display in.
 * 
 * 13    5/12/99 11:33a Bchang
 * startGloss() now get a GlossAttributes object rather than the
 * individual attributes when it is called from FluidLinksXMLHandler.
 * 
 * 12    1/26/99 11:58p Bchang
 * added support for width & height attributes for image glosses.
 * also made images work as nested glosses: had to cache docbase and
 * canvas in the xmlhandler, because nested glosses don't have a path
 * up the component hierarchy during parsing
 * 
 * 11    1/22/99 3:03p Bchang
 * picture captions use titleFontStyle
 * 
 * 10    1/22/99 3:01p Bchang
 * add support for Picture pages, which have the form:
 * <picture source="image.gif" caption="my dog"/>
 * add support for separate font style for <title> paragraphs
 * 
 * 9     1/21/99 4:42p Bchang
 * Support for callout measurements.
 * Support for image and control glosses.
 * 
 * 8     1/21/99 3:16p Pollez
 * added computed glosses. changes to startGloss, add computeGloss.
 * 
 * 7     1/19/99 5:00p Bchang
 * Added a comment in reconstructTextLines() explaining that
 * empty paragraphs are given a single blank line.
 * 
 * 6     1/19/99 3:51p Bchang
 * (Added comment for version 5.)
 * 
 * 5     1/19/99 3:45p Bchang
 * Changed it so that empty paragraphs <para></para> are created
 * as one line with a single space. This allows you to put in blank lines.
 * 
 * 4     1/14/99 4:42p Mackinla
 * added comment to show where Paragraph inner class ends
 * 
 * 3     1/13/99 5:31p Bchang
 * Added $Log at bottom of file
 */
