// TextLine.java
// Copyright (c) 1999 by Xerox Corporation
// bchang@parc.xerox.com

// $Header: /FluidDocs/annotation/TextLine.java 11    7/22/99 4:49p Bchang $

package annotation;

import fluid.*;
import java.awt.*;
import java.util.*;

public class TextLine extends FluidComponent {

    /*
  protected Vector segments = new Vector();
  protected void initialSegment(String str) {
    TextSegment seg = new TextSegment(str);
    segments.addElement(seg);
  }
  */

  // constructor
  public TextLine(String str, String fname, int fstyle, int fsize,
                  int linenum, int lspace, int off) {
    setText(str);
    setFont(fname, fstyle, fsize);
    setLocation(0, linenum, lspace, off);
  }

  // location
  protected void setLocation(int x, int linenum, int lspace, int off) {
    setX(x);
    setLineNumber(linenum);
    setOffset(off);
    setLineSpacing(lspace); // sets y; need to set linenum & offset first
    setBaseY(getY());
  }

  protected int baseY;
  public void setBaseY(int y) { baseY = y; }
  public int getBaseY() { return baseY; }

  protected int absoluteBaseY;
  public void setBase() {
    setBaseY(getY());
    absoluteBaseY = absoluteY();
  }
  public int getAbsoluteBaseY() { return absoluteBaseY; }

  protected int lineNumber;
  public void setLineNumber(int ln) { lineNumber = ln; }
  public int getLineNumber() { return lineNumber; }

	// getLogID
	public String getID() { return Integer.toString(getLineNumber()); }

  protected int lineSpacing;
  void setLineSpacing(int ls) {
    lineSpacing = ls;
    setY((getLineNumber() * lineSpacing) + getOffset());
  }
  public int getLineSpacing() { return lineSpacing; }

  protected int offset;
  public void setOffset(int o) { offset = o; }
  public int getOffset() { return offset; }

	// useColor
	protected boolean useColor = false;
	public void setUseColor(boolean b) { useColor = b; }
	public boolean getUseColor() { return useColor; }
  // font
  protected String fontName;
  public void setFontName(String fn) {
    fontName = fn;
    invalidateFont();
  }
  public String getFontName() { return fontName; }

  protected int fontStyle;
  public void setFontStyle(int fs) { 
    fontStyle = fs;
    invalidateFont();
  }
  public int getFontStyle() { return fontStyle; }

  protected int fontSize;
  public void setFontSize(int fsize) {
    fontSize = fsize;
    invalidateFont();
  }
  public int getFontSize() { return fontSize; }

  protected Font font;
  public void setFont(String name, int style, int size) {
    setFontName(name);
    setFontStyle(style);
    setFontSize(size);
    font = new Font(name, style, size);
  }
  protected void setFont() {
    setFont(fontName, fontStyle, fontSize);
  }
  public Font getFont() {
    if (font == null) {
      setFont();
    }
    return font;
  }

  protected FontMetrics fontMetrics;
  protected void setFontMetrics() {
      fontMetrics = Toolkit.getDefaultToolkit().getFontMetrics(getFont());
  }
  protected FontMetrics getFontMetrics() {
    if (fontMetrics == null) {
      setFontMetrics();
    }
    return fontMetrics;
  }

  protected void invalidateFont() {
    font = null;
    fontMetrics = null;
    ascent = -1;
    invalidateSize();
  }
  
  protected void invalidateSize() {
    width = -1;
    height = -1;
  }

  // size
  protected int width = -1;
  public void setWidth(int w) {
    // raise error
  }
  public int getWidth() {
    if (width == -1) {
      width = getFontMetrics().stringWidth(getText());
    }
    return width;
  }

  protected int height = -1;
  public void setHeight(int h) {
    // raise error
  }
  public int getHeight() {
    if (height == -1) {
      FontMetrics fm = getFontMetrics();
      height = fm.getAscent() + fm.getDescent();
    }
    return height;
  }

  protected int ascent = -1;
  public int getAscent() {
    if (ascent == -1) {
      ascent = getFontMetrics().getAscent();
    }
    return ascent;
  }

  public void setSize(Dimension s) {
    // raise error
  }
  public Dimension getSize() {
    return new Dimension(getWidth(), getHeight());
  }

  public boolean inBounds(Point p) {
    if (super.inBounds(p)) return true;
    for (int i = 0; i < getComponentCount(); i++) {
      Annotation anno = (Annotation)getComponent(i);
      if (!anno.isSmall()) {
        if (anno.inBounds(p)) {
          return true;
        }
      }
    }
    return false;
  }

  // content
  protected String text;
  public void setText (String t) { text = t; invalidateSize(); }
  public String getText() { return text; }
	public String getLogText() {
		if (text == null) return "";
		if (text.length() > 15) {
			return "|" + text.substring(0, 15) + "|";
		}
		return "|" + text + "|";
	}
  public int getTextLength() { return text.length(); }

  public FluidComponent getPrimary() { return getParent(); }

  // animation
  protected Animation lineSpacingAnimation = null;
  public AnimInt changeLineSpacing(final int tols) {
    return new AnimInt() {
      protected void setStart() { start = getLineSpacing(); }
      protected void setEnd() { end = tols; }
      protected void stepTarget(double fraction) {
        setLineSpacing(currentValue(fraction));
      }
      protected Animation getCurrentAnimation() { return lineSpacingAnimation; }
      protected void setCurrentAnimation(Animation anim) { lineSpacingAnimation = anim; }
    };
  }

  protected Animation fontSizeAnimation = null;
  public AnimInt changeFontSize(final int tofs) {
    return new AnimInt() {
      protected void setStart() { start = getFontSize(); }
      protected void setEnd() { end = tofs; }
      protected void stepTarget(double fraction) {
        setFontSize(currentValue(fraction));
      }
      protected Animation getCurrentAnimation() { return fontSizeAnimation; }
      protected void setCurrentAnimation(Animation anim) { fontSizeAnimation = anim; }
    };
  }

  protected Animation offsetAnimation = null;
  public AnimInt changeOffset(final int toos) {
    return new AnimInt() {
      protected void setStart() { start = getOffset(); }
      protected void setEnd() { end = toos; }
      protected void stepTarget(double fraction) {
        setOffset(currentValue(fraction));
      }
      protected Animation getCurrentAnimation() { return offsetAnimation; }
      protected void setCurrentAnimation(Animation anim) { offsetAnimation = anim; }
    };
  }

  // 
  public Point getCharLocation(int i) {
    return new Point(absoluteX() + getFontMetrics().bytesWidth(text.getBytes(), 0, i),
                     absoluteY());
  }

  // drawing
  public void drawSelf(Graphics g) {
    if (getColor().equals(Color.white)) return;

    determineColor(this);

  	g.setColor(getColor());
    /*
		if (getUseColor()) {
		} else {
			g.setColor(determineColor(this));
		}
    */
    g.setFont(getFont());
    g.drawString(text, 0, getAscent());
		if (FluidUI.fluidLog.logDataP()) {
			Color c = getUseColor()?getColor():determineColor(this);
			Font f = getFont();
			String details = "ID: " + getLogID() + 
											 " LRTB: " + getLocationStr() +
											 " F: " + f.getName() + " " + f.getStyle() + " " + f.getSize() +
											 " RGB: " + c.getRed() + " " + c.getGreen() + " " + c.getBlue() +
											 " Tx: " + getLogText();
			FluidUI.fluidLog.addLog("TEXTLINE", details, false);
		}
  }
  
  protected Color determineColor(TextLine tl) {return ((TextBlock)getParent()).determineColor(tl);};

  public String toString() { return "TextLine[" + getText() + "]"; }

  protected Vector overlays = new Vector();
  public void addOverlay(Negotiation ngt) { overlays.addElement(ngt); }
  public void removeOverlay(Negotiation ngt) { overlays.removeElement(ngt); }
  public boolean overlaysIsEmpty() { return overlays.isEmpty(); }

  // events

  /*
  public void mouseDwell(int x, int y) {
    for (int i = 0; i < getComponentCount(); i++) {
      if (((Annotation)getComponent(i)).governs(x)) {
        getComponent(i).mouseDwell(x, y);
        return;
      }
    }
    super.mouseDwell(x, y);
  }


  public boolean isDwellable(int x, int y) {
    for (int i = 0; i < getComponentCount(); i++) {
      if (((Annotation)getComponent(i)).governs(x)) {
        return true;
      }
    }
    return super.isDwellable(x, y);
  }
  */


  public void mouseDown(Event evt, int x, int y) {
    for (int i = 0; i < getComponentCount(); i++) {
      if (((Annotation)getComponent(i)).governs(x)) {
        getComponent(i).mouseDown(evt, x, y);
        return;
      }
    }
    super.mouseDown(evt, x, y);
  }

  public void mouseUp(Event evt, int x, int y) {
    for (int i = 0; i < getComponentCount(); i++) {
      if (((Annotation)getComponent(i)).governs(x)) {
        getComponent(i).mouseUp(evt, x, y);
        return;
      }
    }
    super.mouseUp(evt, x, y);
  }

  public void rightMouseDown(Event evt, int x, int y) {
    for (int i = 0; i < getComponentCount(); i++) {
      if (((Annotation)getComponent(i)).governs(x)) {
        getComponent(i).rightMouseDown(evt, x, y);
        return;
      }
    }
    super.rightMouseDown(evt, x, y);
  }
  
  public boolean isFirstLine() { return ((TextBlock)getParent()).isFirstLine(this); }

}

/* $Log: /FluidDocs/annotation/TextLine.java $
 * 
 * 11    7/22/99 4:49p Bchang
 * add absoluteBaseY
 * 
 * 10    7/20/99 11:22a Mackinla
 * add FontStyle to logging
 * 
 * 9     7/14/99 8:39a Bchang
 * forward mouseup to child annos
 * futz with color (no longer uses determinecolor)
 *
 * 8     7/13/99 5:44p Mackinla
 * Logging - basic functionality
 *
 * 7     6/24/99 9:53a Bchang
 * added isFirstLine() to support pyramid and wide layouts
 * 
 * 6     5/18/99 4:17p Bchang
 * (Pollez) useColor indicates whether to use its native color
 * (usecolor=true) or call determineColor() on its parent to determine the
 * color (usecolor=false).
 * 
 * 5     1/21/99 12:36p Bchang
 * ensure that setText() calls invalidateSize().
 * support for rightMouseDown() added.
 * 
 * 4     1/13/99 6:18p Bchang
 * Moved changeX() to FluidComponent.
 * Added setStart(), setEnd() to changeLineSpacing() and changeFontSize(),
 * to allow for chained animations.
 * 
 * 3     1/13/99 5:31p Bchang
 * Added $Log at bottom of file
 */
