// XMLGetComputedGloss.java
// Copyright (c) 1999 by Xerox Corporation
// zellweger@parc.xerox.com

// $Header: /FluidDocs/annotation/XMLGetComputedGloss.java 2     7/19/99 1:02p Bchang $

package annotation;

import org.xml.sax.*;
import java.io.IOException;
import java.util.Stack;

public class XMLGetComputedGloss extends HandlerBase {
  // gets the contents of the first instance of the specified XML tag 
  // from the file.
  // for efficiency, should stop parsing at that point -- not yet implemented

  private boolean debug = false;

  protected TextBlock textBlock;
  
  protected String desiredTag;
  protected boolean inTag;

  public XMLGetComputedGloss(TextBlock tb, String tag) {
    textBlock = tb;
	  desiredTag = tag;
  }
  
  public void startElement(String name, AttributeList attributes) {
    if (debug) System.out.println("<" + name + ">");
    if (name.equals(desiredTag)) {
	  inTag = true;
    }
  }

  public void endElement(String name) {
    if (debug) System.out.println("</" + name + ">");
    if (name.equals(desiredTag)) {
      inTag = false;
	  // now need to signal that we can quit parsing the file -- we've done enough!
    } 
  }

  public void characters(char ch[], int start, int length) {
    if (debug) System.out.println("[chars]");
    String str = new String(ch, start, length);
	  if (inTag) {
      textBlock.addToText(str, false);
	  }
  }
	
  // exceptions
  public void warning(SAXParseException exception) {
    System.out.println("warning");
    System.out.println("Warning: " + exception.getMessage() +
		                   " (" + exception.getSystemId() + ':' +
		                   exception.getLineNumber() + ',' +
		                   exception.getColumnNumber() + ')');
  }

  public void error(SAXParseException exception) {
    System.out.println("recoverable error");
    System.out.println("Recoverable Error: " + exception.getMessage() +
		                   " (" + exception.getSystemId() + ':' +
		                   exception.getLineNumber() + ',' +
		                   exception.getColumnNumber() + ')');
  }

  public void fatalError(SAXParseException exception) {
    System.out.println("fatal error");
    System.out.println("Fatal Error: " + exception.getMessage() +
                       " (" + exception.getSystemId() + ':' +
		                   exception.getLineNumber() + ',' +
		                   exception.getColumnNumber() + ')');
  }

}

/* $Log: /FluidDocs/annotation/XMLGetComputedGloss.java $
 * 
 * 2     7/19/99 1:02p Bchang
 * no changes, just some tabs were different
 * 
 * 1     1/21/99 3:10p Pollez
 * new file. special handler just searches for a given input tag & gathers
 * its contents.  still reads the rest of the file; should optimize to
 * return early.
 *
*/