// AnimationGroup.java
// Copyright (c) 1999 by Xerox Corporation
// bchang@parc.xerox.com

// $Header: /FluidDocs/fluid/AnimationGroup.java 9     7/26/99 9:36a Bchang $

package fluid;

import java.util.Vector;

public class AnimationGroup extends Animation {
  private Vector group = new Vector();
  
  protected Animation currentAnimation;

  protected void setCurrentAnimation(Animation anim) { currentAnimation = anim; }
  protected Animation getCurrentAnimation() { return currentAnimation; }

  public void add(Animation a) {
    group.addElement(a);
  }
  
  protected void stepTarget(double fraction) {
    for (int i = 0; i < group.size(); i++) {
      ((Animation)(group.elementAt(i))).stepTarget(fraction);
    }
  }
  
  private void startNoDelay(Animator animator) {
    start(currentTime(), animator);
  }  
  
  // AnimGroups don't set a current animation, so they don't have a
  // previous animation to check & kill (see Animation.start()).
  protected void start(long stime, Animator animator) {
    controllingAnimator = animator;
    startTime = stime;
    synchronized (animator) {
      for (int i = 0; i < group.size(); i++) {
        Animation anim = (Animation)(group.elementAt(i));
        anim.start(stime, duration, animator);
      }
      Animation prevanim = getCurrentAnimation();
      controllingAnimator = animator;
      if ((prevanim != null) && !prevanim.isDone()) {
        prevanim.stop();
      }
      setCurrentAnimation(this);
      animator.add(this);
      setStarted();
    }
  }

  // in order to trigger the done condition (alerting animationListeners)
  // we have to register the animationGroup with animator as an animation
  // but we do no work in the step() method, just check for end
  boolean step(long currentTime) {
    stepFraction = (((double)(currentTime - startTime)) / duration);
    stepFraction = Math.min(stepFraction, 1.0);
    if (stepFraction == 1.0) {
      setDone();
      follow();
    }
    return (stepFraction == 1.0);
	}

  protected void start(long stime, int dur, Animator animator) {
    setDuration(dur);
    start(stime, animator); 
  }

  public void stop() {
    if (!isDone()) {
      if (hasStarted()) {
        // is this synchronized necessary? think about it.
        synchronized (controllingAnimator) {
          for (int i = 0; i < group.size(); i++) {
            Animation anim = (Animation)(group.elementAt(i));
            anim.stop();
          }
        }
      } else {
// I believe there is a race condition here in which not all
// the animations get stopped.
//        System.out.println("groggle");
        for (int i = 0; i < group.size(); i++) {
          Animation anim = (Animation)(group.elementAt(i));
          anim.stop();
        }
      }
      super.stop();
    }
  }

}

/* $Log: /FluidDocs/fluid/AnimationGroup.java $
 * 
 * 9     7/26/99 9:36a Bchang
 * add a comment about a potential race condition
 * 
 * 8     7/23/99 9:10a Bchang
 * removed some commented out code (groupDone())
 *
 * 7     1/27/99 2:40p Bchang
 * animationGroups now keep track of their currentAnimation.
 *
 * 6     1/26/99 6:43p Bchang
 * AnimationGroup now uses the same "done" mechanism as Animation --
 * no need to iterate through group members to determine if it's done.
 * Also: in stop(), check if animation has started before asking for
 * controllingAnimator.
 * 
 * 5     1/26/99 5:40p Bchang
 * moved follow() out of setDone() and into step() -- this ensures
 * that follow animations are not triggered if this animation is
 * interrupted.
 * 
 * 4     1/25/99 4:14p Bchang
 * support for triggering events when an animation completes:
 * call addAnimationListener(yourObj) and implement the AnimationListener
 * interface.
 * also added "isCompleted()": flag indicates whether animation ran to
 * completion (if not, use getStepFraction() to find out how far the
 * animation got). (isDone() still means "animation is no longer
 * running".)
 * 
 * 3     1/13/99 5:49p Bchang
 * Added $Log at bottom of file.
 * Added ability to set a delay before starting the animation: setDelay().
 * Enabled chained animation: setFollower().
 */
