// Animator.java
// Copyright (c) 1999 by Xerox Corporation
// bchang@parc.xerox.com

// $Header: /FluidDocs/fluid/Animator.java 4     1/26/99 5:41p Bchang $

package fluid;

import java.util.Date;
import java.util.Vector;

// Animator runs as its own thread. To start an Animation object,
// add it to the Animator's list of things to animate using add().

public class Animator extends Thread {
  
  private FluidUI canvas;
  private Vector active = new Vector();
  private Vector finished = new Vector();

  Animator(FluidUI c) {
    canvas = c;
    setPriority(MIN_PRIORITY);
    setDaemon(true);
    start();
  }

  void startAnimation(Animation animation) {
    animation.start(this);
  }
  
  void startAnimation(Animation anim1, Animation anim2) {
    // need to synchronize these with the same startTime
    anim1.start(this);
    anim2.start(this);
  }
  
  void startAnimation(Vector animvec) {
    // need to synchronize these with the same startTime
    for (int i = 0; i < animvec.size(); i++) {
      ((Animation)animvec.elementAt(i)).start(this);
    }
  }

  // do not add AnimationGroups; only add individual Animations
  synchronized void add(Animation animation) {
    active.addElement(animation);
    notify();
  }

  synchronized void remove(Animation animation) {
    active.removeElement(animation);
  }

  private boolean nothingToAnimate() {
    return active.size() == 0;
  }

  private void repaint() {
    canvas.repaint();
  }

  // called when thread is started
  public void run() {
    while (true) {
      step();
      yield();
    } 
  }

  private synchronized void step() {
    if (nothingToAnimate()) {
      try {
        wait();
      } catch (InterruptedException e) {}
    } else {
      for (int i = 0; i < active.size(); i++) {
        Animation anim = (Animation)active.elementAt(i);
        long currenttime = new Date().getTime();
        boolean done = anim.step(currenttime);
        if (done) {
          finished.addElement(anim);
        }
      }
      repaint();
      for (int i = 0; i < finished.size(); i++) {
        Animation anim = (Animation)finished.elementAt(i);
        remove(anim);
      }
      finished.removeAllElements();
    }
  }

}

/* $Log: /FluidDocs/fluid/Animator.java $
 * 
 * 4     1/26/99 5:41p Bchang
 * added preliminary support for starting multiple animations at once,
 * with startAnimation(anim1, anim2) and startAnimation(animvec).
 * Need to do more in order to actually ensure that the startTimes of
 * the animations are synchronized.
 * 
 * 3     1/13/99 5:48p Bchang
 * Added $Log at bottom of file
 */
