// FluidLog.java
// Copyright (c) 1999 by Xerox Corporation
// mackinla@parc.xerox.com

// $Header: /FluidDocs/fluid/FluidLog.java 15    7/23/99 1:43p Bchang $

package fluid;

import java.util.Vector;
import java.util.Enumeration;
import java.util.Date;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.io.*;
import annotation.*;

public class FluidLog
{
	// constructors
	public FluidLog () {
    setParams();
		addAction("MOUSEMOVE",mousemoveAct);
		addAction("TEXTLINE",textlineAct);
		addAction("NEWPAGE",newpageAct);
		addAction("TEXTBLOCK",textblockAct);
		addAction("ANNO",annoAct);
		addAction("FOLLOW",followAct);
		addAction("BACK",backAct);
		addAction("GROW",growAct);
		addAction("SHRINK",shrinkAct);
		addAction("FREEZE",freezeAct);
		addAction("THAW",thawAct);
		addAction("SWAP-BUFFER",swapbufferAct);
		addAction("QUESTION",questionAct);
		addAction("CONDITION",conditionAct);
		addAction("CALLOUTLINE",calloutlineAct);
		addAction("FREEZEBUTTON",freezebuttonAct);
	}

  public void setParams() {
    //setLogData(AnnoDoc.logdata);
    //setLogDir(AnnoDoc.logdir);
    //setMouseDelta(AnnoDoc.logmousedelta);
  }

	// internal vars
	protected Vector log = new Vector();
	protected Hashtable eventHash = new Hashtable();
	protected long startTime = System.currentTimeMillis();
	protected long lastWriteTime = startTime;
	protected long lastTime = startTime;
	protected int lineCnt = 0;
	protected int writeCnt = 0;
	protected static boolean trigger = false;
	protected static String triggerStr = "";
	protected String logDir = "";
	protected boolean logData = false;
	protected long mouseDelta = 200;
	// 0 = ignore, 1 = write, 2 = write and show, 3 = show only
	protected int defaultAct = 1;
	protected int mousemoveAct = defaultAct;
	protected int swapbufferAct = defaultAct;
	protected int textlineAct = defaultAct;
	protected int newpageAct = defaultAct;
	protected int textblockAct = defaultAct;
	protected int annoAct = defaultAct;
	protected int followAct = defaultAct;
	protected int growAct = defaultAct;
	protected int shrinkAct = defaultAct;
	protected int backAct = defaultAct;
	protected int freezeAct = defaultAct;
	protected int thawAct = defaultAct;
	protected int questionAct = defaultAct;
	protected int conditionAct = defaultAct;
	protected int calloutlineAct = defaultAct;
	protected int freezebuttonAct = defaultAct;

	// external vars
	// logDataP - turns logging off
	public boolean logDataP() { return logData; }
	// triggerLog
	public void triggerLog(String s) {
		trigger = true;
		triggerStr = s;
	}
	// logData
	public void setLogData(boolean b) { logData = b; }
	// logDir
	public void setLogDir(String s) { logDir = s; }
	// mouseDelta
	public void setMouseDelta(long l) { mouseDelta = l; }
	public long getMouseDelta() { return mouseDelta; }
	// action
	public void addAction(String id, int action) { eventHash.put(id, new Integer(action)); }
	public int findAction(String id) {
		Object value = eventHash.get(id);
		return (value==null)? -1:((Integer) value).intValue();
	}
	
	// methods
	
	// clearLog
	public void clearLog() {
		log = new Vector();
	}

	// addLog
	public void addLog(String id, String details, boolean showTime) {
		// hack to write log
		if (trigger) {
			trigger = false;
			writeLog(triggerStr);
		}
		// add msg
		int action = findAction(id);
		String msg = formatMsg(id, details, showTime);
		if (action == -1) {
			System.out.println("UNRECOGNIZED LOG EVENT |" + id + "|");
			action = defaultAct;
		}
		// show
		if ((action == 2) || (action == 3)) {
			System.out.println(msg);
		}
		// log
		if ((action == 1) || (action == 2)) {
			log.addElement(msg);
			lineCnt = lineCnt + 1;
		}
	}
	
	// formatMsg
	public String formatMsg(String id, String details, boolean showTime) {
		String fullMsg = "(" + id;
		if (details == null) {
			details = "";
		}
		fullMsg = fullMsg + " (" + details  + ")";
		if (showTime) {
			fullMsg = fullMsg + " Time: " + curTime();
		}
		fullMsg = fullMsg	+ ")";
		//System.out.println(fullMsg);
		return fullMsg;
	}
	
	public String curTime() {
		long curTime = System.currentTimeMillis();
		long startDelta = curTime - startTime;
		long lastDelta = curTime - lastTime;
		long writeDelta = curTime - lastWriteTime;
		lastTime = curTime;
		return startDelta + " " + lastDelta + " " + writeDelta + " " + curTime;		
	}
	
	public void writeLog(String logName) {
    if (!logDataP()) return;
		BufferedWriter outStream = null;
		File outFile = null;
		Enumeration enum = log.elements();
		String fullName;
		StringTokenizer tokens = new StringTokenizer(logName, ".");
		String name = (tokens.hasMoreTokens())?tokens.nextToken():null;
		String extension = (tokens.hasMoreTokens())?tokens.nextToken():null;
		// make sure logName is not empty and has an extension
		if ((name == null) || (extension == null)) {
			if (name == null) {
				System.out.println("writeLog has empty file name - using fluidLog");
				name = "fluidLog";
			}
			if (extension == null) {
				System.out.println("writeLog file name " + logName + " has missing extension - using txt");
				extension = "txt";
			}
			logName = name + "." + extension;
		}
		// write file
		fullName = logDir + logName;
		try { // catch IOExceptions
			outFile = new File(fullName);
			while (outFile.exists()) {
				System.out.println("writeLog file name exists " + logName + " adding +");
				name = name + "+";
				logName = name + "." + extension;
				fullName = logDir + logName;
				outFile = new File(fullName);
			}
			try { // make sure to close outStream
				outStream = new BufferedWriter(new FileWriter(outFile));
				Date modDate = new Date(outFile.lastModified());
				String metaDetails = logName + 
														 " DIR: " + logDir + 
														 " DATE: " + modDate.toString() +
														 " STARTTIME: " + startTime +
														 " LASTWRITETIME: " + lastWriteTime +
														 " CURTIME: " + System.currentTimeMillis();
				String metaMsg = formatMsg("META-DATA", metaDetails, true);
				System.out.println(metaMsg);
				outStream.write(metaMsg);
				outStream.newLine();
				while (enum.hasMoreElements()) {
					outStream.write((String) enum.nextElement());
					outStream.newLine();
				}
				// clear log
				clearLog();
				lastWriteTime = System.currentTimeMillis();
			}
			finally {
				if (outStream != null) try { outStream.close(); } catch (IOException e) { ; }
			}
		}
		catch (IOException e) { System.err.println(e.getMessage()); }
	}
	
}

/* $Log: /FluidDocs/fluid/FluidLog.java $
 * 
 * 15    7/23/99 1:43p Bchang
 * add calloutline, freezebutton logging
 * 
 * 14    7/22/99 11:17a Bchang
 * 
 * 13    7/22/99 11:12a Bchang
 * added QUESTION and CONDITION
 * 
 * 12    7/19/99 4:27p Mackinla
 * fixed parameter setting
 * 
 * 11    7/19/99 1:17p Bchang
 * added setParams()
 * also disabled writing of log if logging is supposed to be turned off
 */

