// FluidPage.java
// Copyright (c) 1999 by Xerox Corporation
// bchang@parc.xerox.com

// $Header: /FluidDocs/fluid/FluidPage.java 8     7/19/99 1:19p Bchang $

package fluid;

import java.net.URL;
import java.awt.*;
import java.util.Vector;

public class FluidPage extends FluidComponent {

  public FluidPage() {
    componentAtMouse = this;
  }

  protected Animator animator;
  public Animator getAnimator() { return animator; }
  void setAnimator(Animator anim) { animator = anim; }

  protected FluidUI canvas;
  public FluidUI getCanvas() { return canvas; }
  void setCanvas(FluidUI c) { canvas = c; }

  public FluidPage getPage() { return this; }

  public Graphics getGraphics() { return canvas.getOffscreenGraphics(); }

  public void removeAllComponents() {
    removeAllTopDrawnComponents();
    super.removeAllComponents();
  }

  // this should be a Set, not a Vector, so that components never get added more than once
  protected Vector topDrawnComponents = new Vector();
  public Vector getTopDrawnComponents() { return topDrawnComponents; }
  public void setTopDrawnComponents(Vector tdc) { topDrawnComponents = tdc; }
  public void addTopDrawnComponent(FluidComponent comp) { topDrawnComponents.addElement(comp); }
  public void removeTopDrawnComponent(FluidComponent comp) { topDrawnComponents.removeElement(comp); }
  public void removeAllTopDrawnComponents() { topDrawnComponents.removeAllElements(); }

  public void draw(Graphics g) {
    super.draw(g);
    for (int i = 0; i < topDrawnComponents.size(); i++) {
      FluidComponent comp = (FluidComponent)topDrawnComponents.elementAt(i);
      int x = comp.getParent().absoluteX();
      int y = comp.getParent().absoluteY();
      g.translate(x, y);
      try {
        comp.doDraw(g);
      } catch (Exception e) {
        // in case doDraw or drawSelf etc throws an Exception,
        // we still do the "un"translate.
        // why would this happen? it happens when an animation finishes or is interrupted
        // and an object no longer exists or something, and you get a null pointer exception
        System.out.println("Caught an exception during FluidPage topdraw.");
        e.printStackTrace();
      }
      g.translate(-x, -y);
    }
  }


  public int absoluteX() { return 0; }
  public int absoluteY() { return 0; }

  public void repaint() { canvas.repaint(); }

  protected FluidComponent componentAtMouse;
  public void setComponentAtMouse(FluidComponent comp) { componentAtMouse = comp; }
  public FluidComponent getComponentAtMouse() { return componentAtMouse; }

  public void mouseDown(Event evt, int x, int y) {}
  public void middleMouseDown(Event evt, int x, int y) {}
  public void rightMouseDown(Event evt, int x, int y) {}
  public void mouseUp(Event evt, int x, int y) {}
  public void middleMouseUp(Event evt, int x, int y) {}
  public void rightMouseUp(Event evt, int x, int y) {}
  public void mouseMove(Event evt, int x, int y) {}
  public void mouseDwell(int x, int y) {}

  public boolean isDwellable(int x, int y) { return getDwellable(); }

  public void load(URL url) {}
  public void load() {}
  public void reload() {}
  public void loadnew(String doc) {}
}

/* $Log: /FluidDocs/fluid/FluidPage.java $
 * 
 * 8     7/19/99 1:19p Bchang
 * changed to import java.awt.*
 * 
 * 7     7/14/99 12:42p Bchang
 * added loadnew() abstract method
 * 
 * 6     3/31/99 10:58a Bchang
 * put in a comment in doDraw explaining how g.translate() is always
 * called, even when an exception occurs
 * 
 * 5     1/26/99 10:42a Bchang
 * Wrapped the doDraw() method with a try/catch block in order to
 * protect against the "shifted world" bug: if the draw fails, the
 * graphics
 * context didn't get a chance to un-translate. With the catch, the
 * un-translate should always occur.
 * 
 * 4     1/21/99 4:45p Bchang
 * Support for middle and right mouse buttons.
 * 
 * 3     1/13/99 5:48p Bchang
 * Added $Log at bottom of file
 */
