// FluidUI.java
// Copyright (c) 1999 by Xerox Corporation
// bchang@parc.xerox.com

// $Header: /FluidDocs/fluid/FluidUI.java 12    9/20/99 8:37a Bchang $

// NS11: code commented out and labelled "NS11" is because of
// Netscape 4.05(PC)'s apparent incomplete implementation of JDK1.1
// Thus I've replaced those calls with deprecated 1.0 methods

package fluid;

import java.applet.Applet;
import java.net.URL;
import java.awt.event.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import com.oroinc.text.regex.*;

public class FluidUI extends Applet {
	public static boolean inApplet = true;
  protected FluidPage page;

  // params (defaultWidth & defaultHeight are gleaned from the applet width and height)
  public static int dwelltime = 200;
  protected static int defaultWidth = 760;
  protected static int defaultHeight = 820;

  public static int animduration = 300;

	public static FluidLog fluidLog = new FluidLog(); // a singleton

  public FluidUI() {}
  public FluidUI(FluidPage p) {
    page = p;
  }

  public void reset() {
    page.reload();
  }
  
  public void readAppParams(String paramfile) {
    if (paramfile == "") return;

    StringBuffer inputstring = new StringBuffer();

    try {
      BufferedReader file = new BufferedReader(new FileReader(new File(paramfile)));
      try {
        for (String line = file.readLine(); line != null; line = file.readLine()) {
          inputstring.append(line);
        }
      } catch (IOException e) {
        System.out.println("io exception while reading param file: " + paramfile);
        System.out.println(e);
        System.exit(1);
      }
    } catch (FileNotFoundException e) {
      System.out.println("Could not find param file: " + paramfile);
      System.out.println("Make sure you've specified the full pathname.");
      System.exit(1);
    }
    
    PatternCompiler compiler = new Perl5Compiler();
    PatternMatcher matcher = new Perl5Matcher();

    Pattern sizepat, altsizepat, parampat;
    PatternMatcherInput input;
    MatchResult result; 
    
    String sizePattern = "<applet\\s+.*width=\"?(\\d*)\"?\\s+.*height=\"?(\\d*)\"?";
    String altSizePattern = "<applet\\s+.*height=\"?(\\d*)\"?\\s+.*width=\"?(\\d*)\"?";
    String paramPattern = "<param\\s+name=\"?(\\S*)\"?\\s+value=\"?([^\">]*)\"?\\s*>";
    
    try {
      sizepat = compiler.compile(sizePattern, Perl5Compiler.CASE_INSENSITIVE_MASK);
      altsizepat = compiler.compile(altSizePattern, Perl5Compiler.CASE_INSENSITIVE_MASK);
      parampat = compiler.compile(paramPattern, Perl5Compiler.CASE_INSENSITIVE_MASK);
    } catch(MalformedPatternException e) {
      System.out.println("Bad pattern.");
      System.out.println(e.getMessage());
      return;
    }
    
    input = new PatternMatcherInput(inputstring.toString());
    if (matcher.contains(input, sizepat)) {
      result = matcher.getMatch();
      defaultWidth = Integer.parseInt(result.group(1));
      defaultHeight = Integer.parseInt(result.group(2));
    } else if (matcher.contains(input, altsizepat)) {
      result = matcher.getMatch();
      defaultWidth = Integer.parseInt(result.group(2));
      defaultHeight = Integer.parseInt(result.group(1));
    }
    
    Dictionary params = new Hashtable();
    while (matcher.contains(input, parampat)) {
      result = matcher.getMatch();
      String name = result.group(1);
      String value = result.group(2);
      params.put(name, value);
    }
    
    Util.setAppParams(this, params);
  }

  public static String windowtitle = "";
  
	public static void mainframe(FluidUI fluid, String paramfile) {
    Frame frame = new Frame() {
      public boolean handleEvent(Event evt)	{
        switch (evt.id) {
        case Event.WINDOW_DESTROY:
          dispose();
          System.exit(0);
          return true;
        default:
          return super.handleEvent(evt);
        }			 
      }
    };
    /* NS11
    WindowListener listener = new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        System.exit(0);
      }
    };
    frame.addWindowListener(listener);
    */
    
    fluid.readAppParams(paramfile);
    frame.setTitle(windowtitle);
    fluid.initializeFromParams();
    fluid.setSize(new Dimension(defaultWidth, defaultHeight));
		frame.setSize(fluid.getSize());

		frame.add(fluid, "Center");
		fluid.inApplet = false;
		fluid.initializePage();
    frame.setVisible(true);
	}

  protected void readFluidUIParams() {
    String param;
    
    param = getParameter("width");
    if (param != null) {
      defaultWidth = Integer.parseInt(param);
    }
    param = getParameter("height");
    if (param != null) {
      defaultHeight = Integer.parseInt(param);
    }

    /* actually, this should be set in Util.setAppletParams()
    param = getParameter("dwelltime");
    if (param != null) {
      dwelltime = Integer.parseInt(param);
    } 
    */
  }
  
  protected void initializeFromParams() {}

	public void init() {
    try {
      readFluidUIParams();
      setSize(new Dimension(defaultWidth, defaultHeight));
      Util.setAppletParams(this);
      initializeFromParams();
      initializePage(getDocumentBase());
    } catch (Exception e) {
      e.printStackTrace();
    }
	}

  protected void initializePage() { initializePage((URL)null); }
  protected void initializePage(URL docbase) { initializePage(docbase, this); }
  protected void initializePage(URL docbase, FluidUI comp) {
    page.setSize(comp.getSize());
    page.setCanvas(comp);
    page.load(docbase);
    Animator animator = new Animator(comp);
    page.setAnimator(animator);
    setVisible(true);
  }

  // the "canvas" behavior
  
	protected Graphics offscreenGraphics;
	protected Dimension screenSize;
	protected Image offscreenImage;

  public Graphics getOffscreenGraphics() { return offscreenGraphics; }

	public void createOffscreenBuffer() {
		screenSize = getSize();
		offscreenImage = createImage(screenSize.width, screenSize.height);
		offscreenGraphics = offscreenImage.getGraphics();
	}

  public void update(Graphics g) {
    // for double-buffering:
	  // override the default update to not clear the screen before painting
		paint(g);
  }

	public synchronized void paint(Graphics g) {
		if (offscreenGraphics == null) {
			createOffscreenBuffer();
		}
    offscreenGraphics.setColor(Color.white);
    offscreenGraphics.fillRect(0, 0, screenSize.width, screenSize.height);
		page.draw(offscreenGraphics);
		if (FluidUI.fluidLog.logDataP()) {
			FluidUI.fluidLog.addLog("SWAP-BUFFER", null, true);
		}
		g.drawImage(offscreenImage, 0, 0, null);

    /* draw vertical lines for visual debugging */
    /*
    for (int x = 100; x < screenSize.width; x += 100) {
      g.drawLine(x, 0, x, screenSize.height);
    }
    g.setColor(Color.yellow);
    g.drawLine(120, 0, 120, screenSize.height);
    g.drawLine(140, 0, 140, screenSize.height);
    g.drawLine(160, 0, 160, screenSize.height);
    g.drawLine(180, 0, 180, screenSize.height);
    */
	}

  // events
	public boolean mouseDown(Event evt, int x, int y) {
    FluidComponent comp = page.getComponentAt(new Point(x, y));
    if ((evt.modifiers ^ evt.ALT_MASK) == 0) {
		  comp.middleMouseDown(evt, x, y);
    } else if ((evt.modifiers ^ evt.META_MASK) == 0) {
		  comp.rightMouseDown(evt, x, y);
    } else {
		  comp.mouseDown(evt, x, y);
    }
    return true;
	}

  // if components want to track the mouse up that follows a
  // mouse down on itself, they need to set this variable.
  // could make this more general by allowing any component
  // to register a listener on events
  protected static FluidComponent lastMouseDownComponent = null;
  public static void setLastMouseDownComponent(FluidComponent comp) {
    lastMouseDownComponent = comp;
  }
  public static FluidComponent getLastMouseDownComponent() {
    return lastMouseDownComponent;
  }
  public static void clearLastMouseDownComponent() {
    lastMouseDownComponent = null;
  }
  public static boolean lastMouseDownComponentEquals(FluidComponent comp) {
    return lastMouseDownComponent == comp;
  }

	public boolean mouseUp(Event evt, int x, int y) {
    FluidComponent comp = page.getComponentAt(new Point(x, y));
    if ((evt.modifiers ^ evt.ALT_MASK) == 0) {
		  comp.middleMouseUp(evt, x, y);
    } else if ((evt.modifiers ^ evt.META_MASK) == 0) {
		  comp.rightMouseUp(evt, x, y);
    } else {
		  comp.mouseUp(evt, x, y);
      if (lastMouseDownComponent != null) {
        lastMouseDownComponent.mouseUpNotice();
        clearLastMouseDownComponent();
      }
    }
    return true;
	}

	protected long nextMouseTime = 0;
	
  public boolean mouseMove(Event evt, int x, int y) {
    FluidComponent comp = page.getComponentAt(new Point(x, y));
		if (FluidUI.fluidLog.logDataP()) {
			long curTime = System.currentTimeMillis();
			if (curTime > nextMouseTime) {
				nextMouseTime = curTime + 200;
				FluidUI.fluidLog.addLog("MOUSEMOVE", x + " " + y, true);
			}
		}
    comp.delegateMouseMotion(evt, x, y);
    return true;
  }

}

/* $Log: /FluidDocs/fluid/FluidUI.java $
 * 
 * 12    9/20/99 8:37a Bchang
 * moved animduration param from AnnoDoc to here
 * 
 * 11    7/19/99 1:21p Bchang
 * apps can parse applet param files to get their params
 * 
 * 10    7/15/99 11:59a Mackinla
 * 
 * 9     7/14/99 8:45a Bchang
 * lastMouseDownComponent is registered by a component which wants to know
 * if the mouse up will occur on some other component (in which case it
 * gets a mouseUpNotice rather than mouseUp)
 *
 * 8     7/13/99 5:44p Mackinla
 * Logging - basic functionality
 * 
 * 7     7/08/99 2:40p Mackinla
 * Logging v1
 *
 * 6     6/24/99 9:23a Bchang
 * no real change; added a commented out section for visual debugging
 * 
 * 5     1/26/99 10:40a Bchang
 * Removed extraneous "screenGraphics".
 * 
 * 4     1/21/99 4:46p Bchang
 * Support for middle and right mouse buttons.
 * 
 * 3     1/13/99 5:48p Bchang
 * Added $Log at bottom of file
 */
