// Negotiation.java
// Copyright (c) 1999 by Xerox Corporation
// bchang@parc.xerox.com

// $Header: /FluidDocs/fluid/Negotiation.java 7     7/22/99 4:53p Bchang $

package fluid;

import java.awt.*;

public class Negotiation {

  public FluidComponent primary, secondary;

  public Dimension desiredSize, offeredSize;

  public boolean overlap = false;
  public String policy = "";
  public String subpolicy = "";

  public int ngray;
  public int squashheight = 0;
  public FluidComponent reuseanno = null;
  public FluidComponent reuser = null;

  public double verticalFraction;
  
  public AnimationGroup animation = new AnimationGroup();
  
  public Negotiation subngt = null;

  public boolean abort = false;

  public Negotiation(FluidComponent sec, FluidComponent prim, GChar gchar) {
    primary = prim;
    secondary = sec;

    desiredSize = new Dimension(gchar.getSize());

    establishGraphicalProtocol();

    primary.negotiationSetGuidelines(this);
    if (!abort) {
      secondary.negotiationSelectPresentation(this);
      primary.negotiationSelectSpaceStrategy(this);
    }
  }

  // graphical protocol
  protected String graphicalProtocol;
  public void setGraphicalProtocol(String prot) { graphicalProtocol = prot; }
  public String getGraphicalProtocol() { return graphicalProtocol; }

  public void establishGraphicalProtocol() {
    String pprot = primary.getGraphicalProtocol();
    String sprot = secondary.getGraphicalProtocol();
    setGraphicalProtocol(compatibleProtocol(pprot, sprot));
  }

  protected String compatibleProtocol(String classname1, String classname2) {
    try {
      Class c = nearestSharedAncestor(Class.forName(classname1), Class.forName(classname2));
      return c.getName();
    } catch (ClassNotFoundException e) {
      e.printStackTrace();
      return "";
    }
  }

  private Class nearestSharedAncestor(Class class1, Class class2) {
    if (class1 == null) ; // error!
    if (class1.isAssignableFrom(class2)) return class1;
    return nearestSharedAncestor(class1.getSuperclass(), class2);
  }

  // animation
  public void addAnimation(Animation anim) {
    animation.add(anim);
  }

  protected boolean reverted = false;

  public boolean revert() {
    if (reverted) return false;
    animation.stop(); // in case it's still animating
    double frac = animation.getStepFraction();
    animation = new AnimationGroup();
    primary.negotiationRevert(this, frac);
    reverted = true;
    return true;
  }
}

/* $Log: /FluidDocs/fluid/Negotiation.java $
 * 
 * 7     7/22/99 4:53p Bchang
 * add "subpolicy" variable (used by "reuse")
 * 
 * 6     7/14/99 8:46a Bchang
 * only allow one revert of a negotiation -- return false if a revert has
 * already been performed. this helps squash some bugs when negotiations
 * are reverted more than once (usually due to event race conditions)
 * 
 * 5     6/24/99 9:29a Bchang
 * added verticalFraction for move policy
 * 
 * 4     1/27/99 10:48a Bchang
 * added subngt.
 * 
 * 3     1/13/99 5:48p Bchang
 * Added $Log at bottom of file
 */
