// Util.java
// Copyright (c) 1999 by Xerox Corporation
// bchang@parc.xerox.com

// $Header: /FluidDocs/fluid/Util.java 5     7/19/99 1:23p Bchang $

package fluid;

import java.awt.*;
import java.util.*;
import java.applet.Applet;
import java.lang.reflect.*;

public class Util {

  public static int randomint(int low, int high) {
    return (int)(low + Math.round(Math.random() * (high - low)));
  }

  public static boolean isWhitespace(char c) {
    return (c == ' ') || (c == '\t') || (c == '\n');
  }

  public static Color hexToColor(String hex) {
    // as a convenience, this allows the six-char hex string to be optionally prefixed by #
    String hex2 = (hex.charAt(0) == '#') ? hex.substring(1): hex;
    return new Color(hexToInt(hex2.substring(0, 2)),
                     hexToInt(hex2.substring(2, 4)),
                     hexToInt(hex2.substring(4, 6)));
  }

  // only 2-char hex strings
  protected static int hexToInt(String hex) {
    char[] chars = hex.toCharArray();
    return (16 * hexCharToInt(chars[0])) + hexCharToInt(chars[1]);
  }

  protected static int hexCharToInt(char hexchar) {
    switch (hexchar) {
    case '0': return 0;
    case '1': return 1;
    case '2': return 2;
    case '3': return 3;
    case '4': return 4;
    case '5': return 5;
    case '6': return 6;
    case '7': return 7;
    case '8': return 8;
    case '9': return 9;
    case 'a': return 10;
    case 'b': return 11;
    case 'c': return 12;
    case 'd': return 13;
    case 'e': return 14;
    case 'f': return 15;
    }
    return 0;
  }

  /**
   * [based on JavaWorld Tip #57, by Yvon Sauvageau]
   * Initializes an Applet's public non-final fields
   * The initial values will be read from the HTML PARAM tags.
   * The PARAM tags must be in lower case.
   *
   * @param applet the applet to initialize.
   */
  public static void setAppletParams(Applet applet) {
    Class metaclass = applet.getClass();
    Field[] fields = metaclass.getFields();
    String param = null;

    for (int i = 0; i < fields.length; i++) {
      param = applet.getParameter(fields[i].getName().toLowerCase());
      if ((param != null) && !Modifier.isFinal(fields[i].getModifiers())) {
        setField(applet, fields[i], param);
      }
    }
  }
  
  public static void setAppParams(Applet applet, Dictionary dict) {
    Class metaclass = applet.getClass();
    Field[] fields = metaclass.getFields();
    String param = null;

    for (int i = 0; i < fields.length; i++) {
      param = (String)dict.get(fields[i].getName().toLowerCase());
      if ((param != null) && !Modifier.isFinal(fields[i].getModifiers())) {
        setField(applet, fields[i], param);
      }
    }
  }
  
  public static void setField(Applet applet, Field field, String param) {
    try {
      Class fieldType = field.getType();
            
      if (field.getName().endsWith("_fontstyle")) {
        if (param.equalsIgnoreCase("italic")) {
          field.setInt(applet, Font.ITALIC);
        } else if (param.equalsIgnoreCase("bold")) {
          field.setInt(applet, Font.BOLD);
        } else if (param.equalsIgnoreCase("bolditalic") ||
                   param.equalsIgnoreCase("italicbold")) {
          field.setInt(applet, Font.ITALIC | Font.BOLD);
        } else {
          field.setInt(applet, Font.PLAIN);
        }
        return;
      }

      if (fieldType.equals(boolean.class)) {
        field.setBoolean(applet, Boolean.valueOf(param).booleanValue());
      } else if (fieldType.equals(byte.class)) {
        field.setByte(applet, Byte.valueOf(param).byteValue());
      } else if (fieldType.equals(char.class)) {
        field.setChar(applet, param.charAt(0));
      } else if (fieldType.equals(double.class)) {
        field.setDouble(applet, Double.valueOf(param).doubleValue());
      } else if (fieldType.equals(float.class)) {
        field.setFloat(applet, Float.valueOf(param).floatValue());
      } else if (fieldType.equals(int.class)) {
        field.setInt(applet, Integer.valueOf(param).intValue());
      } else if (fieldType.equals(long.class)) {
        field.setLong(applet, Long.valueOf(param).longValue());
      } else if (fieldType.equals(short.class)) {
        field.setShort(applet, Short.valueOf(param).shortValue());
      } else if (fieldType.equals(String.class)) {
        field.set(applet, param);
      } else if (fieldType.equals(Color.class)) {
        field.set(applet, hexToColor(param));

      } else if (fieldType.isArray()) {
        Class componentType = fieldType.getComponentType();
        if (componentType.isPrimitive() ||
            componentType.equals(String.class)) {
          // 1d array
          StringTokenizer elementTokens = new StringTokenizer(param);
          int numElements = elementTokens.countTokens();
          Object array = field.get(applet);
          if (array == null) {
            field.set(applet, Array.newInstance(componentType, numElements));
            array = field.get(applet);
          }
          fillOneDimensionalArray(array, elementTokens);
        } else if (componentType.isArray() &&
                   (componentType.getComponentType().isPrimitive() || 
                    componentType.getComponentType().equals(String.class))) {
          // 2d array
          StringTokenizer subarrayTokens = new StringTokenizer(param, "|");
          int numSubarrays = subarrayTokens.countTokens();                        
          Object array = field.get(applet); 
          if (array == null) {
            field.set(applet, Array.newInstance(componentType, numSubarrays));
            array = field.get(applet);
          }
          
          for (int j = 0; j < Array.getLength(array) && j < numSubarrays; j++) {
            StringTokenizer elementTokens = new StringTokenizer(subarrayTokens.nextToken());
            int numElements = elementTokens.countTokens();                        
            
            Object subArray = ((Object[])array)[j];
            if (subArray == null) {
              subArray = Array.newInstance(componentType.getComponentType(), numElements);
            }
            ((Object[])array)[j] = subArray; 
            fillOneDimensionalArray(subArray, elementTokens); 
          }
        }
      }
    } catch (Exception e) {
      System.err.println(e + " while initializing " + field);
    }
  }

  /**
   * Fills a one-dimensional array with the contents of a tokenizer.
   * The tokens are converted to the array's content type.
   *
   * @param array array to be filled.
   * @param elementTokens tokenizer containing the tokens which the array will be filled with.
   *
   * @exception java.lang.IllegalAccessException .
   */
  private static void fillOneDimensionalArray(Object array, 
                                              StringTokenizer elementTokens) 
    throws IllegalAccessException {
    if (array != null && elementTokens != null && array.getClass().isArray()) { // Double check.
      Class componentType = array.getClass().getComponentType();
      int numElements = elementTokens.countTokens();                        

      // Note that we make sure the index won't go out of bounds. The array may have
      // been allocated unsuficient space to hold all parsed elements.  
      for (int j = 0; j < Array.getLength(array) && j < numElements; j++) {
        if (componentType.equals(boolean.class))
          Array.setBoolean(array, j, Boolean.valueOf(elementTokens.nextToken().trim()).booleanValue());
        else if (componentType.equals(byte.class))
          Array.setByte(array, j, Byte.valueOf(elementTokens.nextToken().trim()).byteValue());
        else if (componentType.equals(char.class))
          Array.setChar(array, j, elementTokens.nextToken().charAt(0));
        else if (componentType.equals(double.class))
          Array.setDouble(array, j, Double.valueOf(elementTokens.nextToken().trim()).doubleValue());
        else if (componentType.equals(float.class))
          Array.setFloat(array, j, Float.valueOf(elementTokens.nextToken().trim()).floatValue());
        else if (componentType.equals(int.class))
          Array.setInt(array, j, Integer.valueOf(elementTokens.nextToken().trim()).intValue());
        else if (componentType.equals(long.class))
          Array.setLong(array, j, Long.valueOf(elementTokens.nextToken().trim()).longValue());
        else if (componentType.equals(short.class))
          Array.setShort(array, j, Short.valueOf(elementTokens.nextToken().trim()).shortValue());
        else if (componentType.equals(String.class))
          Array.set(array, j, elementTokens.nextToken());
      }
    }
  }

}

/* $Log: /FluidDocs/fluid/Util.java $
 * 
 * 5     7/19/99 1:23p Bchang
 * added support for automatically reading in params -- based on JavaWorld
 * tips #57 & 59.
 * 
 * 4     5/12/99 11:34a Bchang
 * Added hexToColor() and its auxiliary methods. hexToColor() now allows
 * an optional # in front of the 6-char hex string.
 * 
 * 3     1/13/99 5:48p Bchang
 * Added $Log at bottom of file
 */
