/**********************************************************************
Copyright (c) 2002 IBM Corporation and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the Common Public License v1.0
which accompanies this distribution, and is available at
http://www.eclipse.org/legal/cpl-v10.html
Contributors:
Adrian Colyer, Andy Clement, Tracy Gardner - initial version
...
**********************************************************************/
package org.eclipse.ajdt.internal.ui;

import org.eclipse.ajdt.ui.AspectJPlugin;
import org.eclipse.jface.preference.IPreferenceStore;

/**
 * Holds the preferences for AspectJ as set via the Workbench->preferences
 * pages.
 */
public class AspectJPreferences {
		

	/**
	 * Identifier for compiler options preference
	 */   
    public static final String COMPILER_OPTIONS= "aspectj.compiler.options.flags";

	/**
	 * Identifier for outline view mode selection
	 */
	public static final String ASPECTJ_OUTLINE = "org.eclipse.ajdt.ui.ajoutline";
	
	public static final String AUTOBUILD_SUPPRESSED = "org.eclipse.ajdt.ui.preferences.autobuildSuppressed";
	
	/**
	 * Helper get method used by AspectJPreference page
	 */
	static public String getCompilerOptions() {
		IPreferenceStore store = AspectJPlugin.getDefault().getPreferenceStore();
		return store.getString(COMPILER_OPTIONS);
	}
	
	/**
	 * Helper set method used by AspectJPreference page
	 */
	static public void setCompilerOptions(String value) {
		IPreferenceStore store = AspectJPlugin.getDefault().getPreferenceStore();
		store.setValue(COMPILER_OPTIONS, value);
	}	

	/**
	 * Helper get method used by AspectJPreference page
	 */
	static public boolean isAspectJOutlineEnabled() {
		IPreferenceStore store = AspectJPlugin.getDefault().getPreferenceStore();
		return store.getBoolean(ASPECTJ_OUTLINE);
	}
	
	static public boolean isAutobuildSuppressed() {
		IPreferenceStore store = AspectJPlugin.getDefault().getPreferenceStore();
		return store.getBoolean(AUTOBUILD_SUPPRESSED);
	}
	
	

}
