/**********************************************************************
Copyright (c) 2002 IBM Corporation and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the Common Public License v1.0
which accompanies this distribution, and is available at
http://www.eclipse.org/legal/cpl-v10.html
Contributors:
Adrian Colyer, Andy Clement - initial version

**********************************************************************/
package org.eclipse.ajdt.internal.ui;

import java.io.File;

import org.eclipse.ajdt.internal.core.AJDTEventTrace;
import org.eclipse.ajdt.internal.ui.ajde.BuildOptionsAdapter;
import org.eclipse.ajdt.ui.AspectJPlugin;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.dialogs.PropertyPage;

/**
 * The properties page for the AspectJ compiler options that can be set.
 * These options can be set on a per-project basis and so because of that
 * are held as persistent properties against the project resource.
 * 
 * ASCFIXME: Well, if I'd thought of it earlier, I would have put all knowledge of
 * state persistent in the BuildOptionsAdapter and used set/get methods to access it.
 * The get methods already exist, I would have to add the set methods.
 */
public class AspectJProjectPropertiesPage
	extends PropertyPage
	implements SelectionListener {

	// Start of compiler options for ajc /////////////////////////////////////////
	// These are the widgets for all the options.

	// Use javac to create the .class files - no longer supported in 1.1
//	private Button javacModeBtn;

	// Where to place intermediate files when using javac mode
	// or pre-process mode
//	private StringFieldEditor workingDirEditor; - no longer supported in 1.1
	
	//ASCFIXME - Do the javadoc you lazy git
	private StringFieldEditor inputJarsEditor,outputJarEditor,aspectJarsEditor,sourceRootsEditor;

	// Use pre-process mode to create .java files - don't build
	// .class files
//	private Button preprocessModeBtn; - no longer supported in 1.1

	// Character encoding for source files
	// private StringFieldEditor characterEncodingEditor;

	// Supporting assertions as defined in JLS-1.4
//	private Button source14modeBtn; - no longer supported in 1.1

	// Report the use of some features from pre-1.0 versions 
	// of AspectJ be warnings to ease porting of old code.
//	private Button portingModeBtn; - no longer supported in 1.1

	// Non standard compiler options that should be passed to ajc
	private StringFieldEditor nonStandardOptionsEditor;

	// Set of radio buttons to select the compilation strictness choice:
	// 1) Do a normal compile
	// 2) Be extra lenient in interpreting the Java spec
	// 3) Be extra strict in interpreting the Java spec
//	private Button compileMode_normalBtn;  - no longer supported in 1.1
//	private Button compileMode_lenientBtn;
//	private Button compileMode_strictBtn;
	// End of compiler options for ajc ///////////////////////////////////////////

	// Relevant project for which the properties are being set
	private IProject thisProject;

	/**
	 * Build the page of properties that can be set on a per project basis for the
	 * AspectJ compiler.
	 * 
	 */
	protected Control createContents(Composite parent) {

		// Grab the resource (must be a project) for which this property page
		// is being created
		thisProject = (IProject) getElement();

		BuildOptionsAdapter.ensurePropertiesInitialized(thisProject);

		Composite pageComposite = createComposite(parent, 2);

		Label optionsLabel =
			createLabel(
				pageComposite,
				AspectJPlugin.getResourceString("compilerPropsPage.description"));

	   /*
	   	ITextViewer itw2 = new TextViewer(pageComposite, SWT.READ_ONLY);
		
		itw2.setDocument(
			new Document(
			  "The 'use javac' option, the 'pre-process' option and \n"+
			  "the related 'working dir' are not currently supported\n"+
			  "by AspectJ1.1 - altering their values below will not \n"+
			  "affect the compilation.  This page will be updated in\n"+
			  "a future AJDT release to reflect the new compiler    \n"+
			  "options available.  If you need to use the new \n"+
			  "AspectJ1.1 compiler options right now, pass them in  \n"+
			  "the 'Non standard compiler options' text field below.\n"));
			  */

				
				
		inputJarsEditor =
		  new StringFieldEditor("",
		    AspectJPlugin.getResourceString("compilerPropsPage.inputJars"),
		    pageComposite);
		    
		aspectJarsEditor =
		  new StringFieldEditor("",
		    AspectJPlugin.getResourceString("compilerPropsPage.aspectJars"),
		    pageComposite);

		outputJarEditor =
		  new StringFieldEditor("",
		    AspectJPlugin.getResourceString("compilerPropsPage.outputJar"),
		    pageComposite);

		sourceRootsEditor =
		  new StringFieldEditor("",
		    AspectJPlugin.getResourceString("compilerPropsPage.sourceRoots"),
		    pageComposite);
	
	Label spacerLabel008 =
			createLabel(
				pageComposite,"");

//		javacModeBtn =
//			buildButton(
//				pageComposite,
//				AspectJPlugin.getResourceString("compilerPropsPage.useJavacMode"));
//
//		preprocessModeBtn =
//			buildButton(
//				pageComposite,
//				AspectJPlugin.getResourceString("compilerPropsPage.preprocessMode"));
//
//		workingDirEditor =
//			new StringFieldEditor(
//				"",
//				AspectJPlugin.getResourceString("compilerPropsPage.workingDirPath"),
//				StringFieldEditor.UNLIMITED,
//				pageComposite);


		
		/* Removed, doesn't seem to be useful when invoking Compiler from Eclipse environment
		characterEncodingEditor =
			new StringFieldEditor(
				"",
				AspectJPlugin.getResourceString("compilerPropsPage.characterEncoding"),
				StringFieldEditor.UNLIMITED,
				pageComposite);
				*/

//		source14modeBtn =
//			buildButton(
//				pageComposite,
//				AspectJPlugin.getResourceString("compilerPropsPage.source14"));
//
//		portingModeBtn =
//			buildButton(
//				pageComposite,
//				AspectJPlugin.getResourceString("compilerPropsPage.portingMode"));
//
//Label spacerLabel001 =
//			createLabel(
//				pageComposite,"");
//				
//		Label strictnessLabel =
//			createLabel(
//				pageComposite,
//				AspectJPlugin.getResourceString("compilerPropsPage.compilerStrictness"));
//
//		compileMode_normalBtn =
//			createRadioButton(
//				pageComposite,
//				AspectJPlugin.getResourceString("compilerPropsPage.compilerStrictnessNormal"));
//		compileMode_lenientBtn =
//			createRadioButton(
//				pageComposite,
//				AspectJPlugin.getResourceString("compilerPropsPage.compilerStrictnessLenient"));
//		compileMode_strictBtn =
//			createRadioButton(
//				pageComposite,
//				AspectJPlugin.getResourceString("compilerPropsPage.compilerStrictnessStrict"));
//				
//				Label spacerLabel002 =
//			createLabel(
//				pageComposite,"");

		nonStandardOptionsEditor =
			new StringFieldEditor(
				"",
				AspectJPlugin.getResourceString("compilerPropsPage.nonStandardOptions"),
				StringFieldEditor.UNLIMITED,
				pageComposite);

		updatePageContents();

		return pageComposite;
	}

	/**
	 * Helper method to build a labelled check box button.
	 */
	private Button buildButton(Composite container, String label) {
		Button btn = new Button(container, SWT.CHECK);
		btn.setText(label);
		

		GridData data = new GridData();
		data.horizontalSpan = 2;
		data.horizontalAlignment = GridData.FILL;
		btn.setLayoutData(data);

		return btn;
	}

	/**
	 * Helper method to build a labelled radio button.
	 */
	private Button createRadioButton(Composite parent, String label) {
		Button button = new Button(parent, SWT.RADIO | SWT.LEFT);
		button.setText(label);

		button.addSelectionListener(this);
		GridData data = new GridData();
		data.horizontalSpan = 2;
		button.setLayoutData(data);

		return button;
	}

	/**
	 * Helper method that creates a label instance.
	 *
	 */
	private Label createLabel(Composite parent, String text) {
		Label label = new Label(parent, SWT.LEFT);
		label.setText(text);

		GridData data = new GridData();
		data.horizontalSpan = 2;
		data.horizontalAlignment = GridData.FILL;
		label.setLayoutData(data);

		return label;
	}

	/**
	* Creates composite control and sets the default layout data.
	*
	* @param parent  the parent of the new composite
	* @param numColumns  the number of columns for the new composite
	* @return the newly-created coposite
	*/
	private Composite createComposite(Composite parent, int numColumns) {
		Composite composite = new Composite(parent, SWT.NULL);

		//GridLayout
		GridLayout layout = new GridLayout();
		layout.numColumns = numColumns;
		composite.setLayout(layout);

		//GridData
		GridData data = new GridData();
		data.verticalAlignment = GridData.FILL;
		data.horizontalAlignment = GridData.FILL;
		composite.setLayoutData(data);

		return composite;
	}

	/**
	 * Preserves a boolean value for a named field on the project resource.
	 * Actually converts the boolean to a string and delegates to the 
	 * other implementation of preserveSetting() that takes a string as the
	 * value to store.
	 */
	private void preserveSetting(QualifiedName key, boolean flag)
		throws CoreException {
		preserveSetting(key, new Boolean(flag).toString());
	}

	/**
	 * Preserve a key/value pair as a persistent property against the 
	 * project resource.
	 */
	private void preserveSetting(QualifiedName key, String value)
		throws CoreException {
		thisProject.setPersistentProperty(key, value);
	}

	/** 
	 * Retrieve a persistent property value and return it.  If the key is
	 * not found, this will return null *but* that should not occur because
	 * ensurePropertiesInitialized() makes sure all the keys have valid 
	 * values when the properties page first appears for a project.
	 */
	private String retrieveSettingString(QualifiedName key) {
		try {

			String value = thisProject.getPersistentProperty(key);
			return value;
		} catch (CoreException ce) {
			AspectJPlugin.getDefault().getErrorHandler().handleError(
				AspectJPlugin.getResourceString("projectProperties.exceptionDuringRetrieve"),
				ce);
		}
		return "";
	}

	/**
	 * Retrieve a persistent property value, convert it to a boolean and return it.
	 * If the key is not found, this will return 'false' *but* that should not 
	 * occur because ensurePropertiesInitialized() makes sure all the keys have 
	 * valid values when the properties page first appears for a project.
	 */
	private boolean retrieveSettingBoolean(QualifiedName key) {
		try {
			String value = thisProject.getPersistentProperty(key);
			if (value == null)
				return false;
			boolean valueB = new Boolean(value).booleanValue();
			return valueB;
		} catch (CoreException ce) {
			AspectJPlugin.getDefault().getErrorHandler().handleError(
				AspectJPlugin.getResourceString("projectProperties.exceptionDuringRetrieve"),
				ce);
		}
		return false;
	}
	
	private String findInvalidJars(String setOfJars) {

		if (setOfJars.length()==0) return null;
		String inputCopy = setOfJars;
		
		StringBuffer invalidEntries = new StringBuffer();
		
		// For relative paths (they don't start with a File.separator
		// or a drive letter on windows) - we prepend the projectBaseDirectory
		String projectBaseDirectory = 
		  AspectJPlugin.getDefault().getCurrentProject().
		  getLocation().toOSString();
		
		while (inputCopy.indexOf(java.io.File.pathSeparator)!=-1) {
		  int idx = inputCopy.indexOf(java.io.File.pathSeparator);
		  String path = inputCopy.substring(0,idx);
		  
		  java.io.File f = new java.io.File(path);
		  if (!f.isAbsolute())
		  	f = new File(projectBaseDirectory+java.io.File.separator+path);
		  	if (!f.exists()) invalidEntries.append(f+"\n");
		  inputCopy = inputCopy.substring(idx+1);	
		}
		
		// Process the final element
		if (inputCopy.length()!=0) {
		  java.io.File f = new java.io.File(inputCopy);
		  if (!f.isAbsolute())
		  	f = new File(projectBaseDirectory+java.io.File.separator+inputCopy);
		  	if (!f.exists()) invalidEntries.append(f+"\n");
	
		}
		
		if (invalidEntries.length()==0) return null;
		
		return invalidEntries.toString();
		
		
	}
	
	private String findInvalidDirs(String setOfDirs) {

		if (setOfDirs.length()==0) return null;
		String inputCopy = setOfDirs;
		
		StringBuffer invalidEntries = new StringBuffer();
		
		// For relative paths (they don't start with a File.separator
		// or a drive letter on windows) - we prepend the projectBaseDirectory
		String projectBaseDirectory = 
		  AspectJPlugin.getDefault().getCurrentProject().
		  getLocation().toOSString();
		
		while (inputCopy.indexOf(java.io.File.pathSeparator)!=-1) {
		  int idx = inputCopy.indexOf(java.io.File.pathSeparator);
		  String path = inputCopy.substring(0,idx);
		  
		  java.io.File f = new java.io.File(path);
		  if (!f.isAbsolute())
		  	f = new File(projectBaseDirectory+java.io.File.separator+path);
		  if (!f.isDirectory()) invalidEntries.append(f+"\n");
		  inputCopy = inputCopy.substring(idx+1);	
		}
		
		// Process the final element
		if (inputCopy.length()!=0) {
		  java.io.File f = new java.io.File(inputCopy);
		  if (!f.isAbsolute())
		  	f = new File(projectBaseDirectory+java.io.File.separator+inputCopy);
		  	if (!f.isDirectory()) invalidEntries.append(f+"\n");
	
		}
		
		if (invalidEntries.length()==0) return null;
		return invalidEntries.toString();
	}


	/**
	 * When OK is clicked on the property page, this method stores the current
	 * values of all the buttons/fields on the page.  The state is stored as a set
	 * of persistent properties against the project resource.
	 * This method is also called if the user clicks 'Apply' on the property page.
	 */
	
	public boolean performOk() {
	
	    // Verify the input sets of jars are valid (both classes and aspects)
	
	    String invalidClassesJars = findInvalidJars(inputJarsEditor.getStringValue());
	    //ASCFIXME - Need to NLSify this string...
		if (invalidClassesJars!=null) {
		  AspectJPlugin.getDefault().getErrorHandler().handleWarning(
		    "The following files specified on the list of input "+
		    "(classes) jars do not exist:\n"+invalidClassesJars);
		    return false;
		}
	    
	    String invalidAspectJars = findInvalidJars(aspectJarsEditor.getStringValue());
	    //ASCFIXME - Need to NLSify this string...
		if (invalidAspectJars!=null) {
		  AspectJPlugin.getDefault().getErrorHandler().handleWarning(
		    "The following files specified on the list of input "+
		    "(aspect) jars do not exist:\n"+invalidAspectJars);
		    return false;
		}
		
	    String invalidSourceDirs = findInvalidDirs(sourceRootsEditor.getStringValue());
	    //ASCFIXME - Need to NLSify this string...
		if (invalidSourceDirs!=null) {
		  AspectJPlugin.getDefault().getErrorHandler().handleWarning(
		    "The following locations specified on the list of source "+
		    "directories are not valid:\n"+invalidSourceDirs);
		    return false;
		}

	

	
		AJDTEventTrace.projectPropertiesChanged(thisProject);
		try {
//			preserveSetting(BuildOptionsAdapter.USE_JAVAC, javacModeBtn.getSelection());
//			preserveSetting(
//				BuildOptionsAdapter.PREPROCESS,
//				preprocessModeBtn.getSelection());
//			preserveSetting(BuildOptionsAdapter.SOURCE14, source14modeBtn.getSelection());
//			preserveSetting(BuildOptionsAdapter.WORKING_DIR,workingDirEditor.getStringValue());
			preserveSetting(BuildOptionsAdapter.INPUTJARS,inputJarsEditor.getStringValue());
			preserveSetting(BuildOptionsAdapter.OUTPUTJAR,outputJarEditor.getStringValue());
			preserveSetting(BuildOptionsAdapter.ASPECTJARS,aspectJarsEditor.getStringValue());
			preserveSetting(BuildOptionsAdapter.SOURCEROOTS,sourceRootsEditor.getStringValue());

			preserveSetting(
				BuildOptionsAdapter.CHAR_ENC,"");
				//characterEncodingEditor.getStringValue());
//			preserveSetting(
//				BuildOptionsAdapter.COMPILATION_STRICTNESS,
//				calculateStrictnessFromButtons());
//			preserveSetting(
//				BuildOptionsAdapter.PORTING_MODE,
//				portingModeBtn.getSelection());
			preserveSetting(
				BuildOptionsAdapter.NON_STANDARD_OPTS,
				nonStandardOptionsEditor.getStringValue());

		} catch (CoreException ce) {
			AspectJPlugin.getDefault().getErrorHandler().handleError(
				AspectJPlugin.getResourceString("projectProperties.exceptionDuringStore"),
				ce);
			return false;
		}
		return true;
	}

	/**
	 * All compiler properties for the project are reset to their default
	 * values.  This is called when 'Defaults' is clicked on the property
	 * page.  The implementation actually deletes all the property values by
	 * setting them to null, then calls "ensurePropertiesInitialized" to
	 * get them reset to the defaults - this means the default setting logic
	 * is all in one place - BuildOptionsAdapter.ensurePropertiesInitialized
	 */
	public void performDefaults() {
		AJDTEventTrace.projectPropertiesDefaulted(thisProject);
		try {
//			thisProject.setPersistentProperty(BuildOptionsAdapter.USE_JAVAC, null);
//			thisProject.setPersistentProperty(BuildOptionsAdapter.PREPROCESS, null);
//			thisProject.setPersistentProperty(BuildOptionsAdapter.SOURCE14, null);
//			thisProject.setPersistentProperty(BuildOptionsAdapter.WORKING_DIR, null);
			thisProject.setPersistentProperty(BuildOptionsAdapter.OUTPUTJAR,null);
			thisProject.setPersistentProperty(BuildOptionsAdapter.ASPECTJARS,null);
			thisProject.setPersistentProperty(BuildOptionsAdapter.SOURCEROOTS,null);
			thisProject.setPersistentProperty(BuildOptionsAdapter.INPUTJARS,null);

			thisProject.setPersistentProperty(BuildOptionsAdapter.CHAR_ENC, null);
//			thisProject.setPersistentProperty(
//				BuildOptionsAdapter.COMPILATION_STRICTNESS,
//				null);
//			thisProject.setPersistentProperty(BuildOptionsAdapter.PORTING_MODE, null);
//			thisProject.setPersistentProperty(BuildOptionsAdapter.NON_STANDARD_OPTS, null);
			BuildOptionsAdapter.ensurePropertiesInitialized(thisProject);

			// Keep the widgets up to date!
			updatePageContents();

		} catch (CoreException ce) {
			AspectJPlugin.getDefault().getErrorHandler().handleError(
				AspectJPlugin.getResourceString(
					"projectProperties.exceptionDefaultingProperties"),
				ce);
		}
	}

	/**
	 * Handle selection of an item in the menu.
	 */
	public void widgetDefaultSelected(SelectionEvent se) {
		widgetSelected(se);

	}

	/**
	 * Handle selection of an item in the menu.
	 */
	public void widgetSelected(SelectionEvent se) {
		Object source = se.getSource();
		if (source instanceof Button) {
			Button btn = (Button) source;
			// Keep the buttons in the radio group consistent
//			if (btn.equals(compileMode_normalBtn)) {
//				compileMode_normalBtn.setSelection(true);
//				compileMode_lenientBtn.setSelection(false);
//				compileMode_strictBtn.setSelection(false);
//			} else if (btn.equals(compileMode_lenientBtn)) {
//				compileMode_normalBtn.setSelection(false);
//				compileMode_lenientBtn.setSelection(true);
//				compileMode_strictBtn.setSelection(false);
//			} else if (btn.equals(compileMode_strictBtn)) {
//				compileMode_normalBtn.setSelection(false);
//				compileMode_lenientBtn.setSelection(false);
//				compileMode_strictBtn.setSelection(true);
//			}
		}
	}

	/**
	 * Convert the choice shown on the radio button group into a string
	 * for storing as a persistent property.
	 */
	public String calculateStrictnessFromButtons() {
//		if (compileMode_normalBtn.getSelection())
//			return BuildOptionsAdapter.COMPILATION_STRICTNESS_NORMAL;
//		if (compileMode_lenientBtn.getSelection())
//			return BuildOptionsAdapter.COMPILATION_STRICTNESS_LENIENT;
//		if (compileMode_strictBtn.getSelection())
//			return BuildOptionsAdapter.COMPILATION_STRICTNESS_STRICT;
		return null;
	}

	/**
	 * Ensure the widgets state reflects the persistent property values.
	 */
	public void updatePageContents() {
//		javacModeBtn.setSelection(
//			retrieveSettingBoolean(BuildOptionsAdapter.USE_JAVAC));
//
//		preprocessModeBtn.setSelection(
//			retrieveSettingBoolean(BuildOptionsAdapter.PREPROCESS));
//
//		workingDirEditor.setStringValue(
//			retrieveSettingString(BuildOptionsAdapter.WORKING_DIR));
			
		inputJarsEditor.setStringValue(
		    retrieveSettingString(BuildOptionsAdapter.INPUTJARS));
		outputJarEditor.setStringValue(
		    retrieveSettingString(BuildOptionsAdapter.OUTPUTJAR));
		aspectJarsEditor.setStringValue(
		    retrieveSettingString(BuildOptionsAdapter.ASPECTJARS));
		sourceRootsEditor.setStringValue(
		    retrieveSettingString(BuildOptionsAdapter.SOURCEROOTS));
		//characterEncodingEditor.setStringValue(
		//	retrieveSettingString(BuildOptionsAdapter.CHAR_ENC));

//		source14modeBtn.setSelection(
//			retrieveSettingBoolean(BuildOptionsAdapter.SOURCE14));
//
//		portingModeBtn.setSelection(
//			retrieveSettingBoolean(BuildOptionsAdapter.PORTING_MODE));
//
//		// Check the strictness and set the three radio buttons appropriately.
//		String strictness =
//			retrieveSettingString(BuildOptionsAdapter.COMPILATION_STRICTNESS);
//		boolean b1 = false;
//		boolean b2 = false;
//		boolean b3 = false;
//		if (strictness.equals(BuildOptionsAdapter.COMPILATION_STRICTNESS_NORMAL))
//			b1 = true;
//
//		else if (strictness.equals(BuildOptionsAdapter.COMPILATION_STRICTNESS_LENIENT))
//			b2 = true;
//
//		else if (strictness.equals(BuildOptionsAdapter.COMPILATION_STRICTNESS_STRICT))
//			b3 = true;
//
//		compileMode_normalBtn.setSelection(b1);
//		compileMode_lenientBtn.setSelection(b2);
//		compileMode_strictBtn.setSelection(b3);

		nonStandardOptionsEditor.setStringValue(
			retrieveSettingString(BuildOptionsAdapter.NON_STANDARD_OPTS));
	}
}