/**********************************************************************
Copyright (c) 2002 IBM Corporation and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the Common Public License v1.0
which accompanies this distribution, and is available at
http://www.eclipse.org/legal/cpl-v10.html
Contributors:
Adrian Colyer, Andy Clement, Tracy Gardner - initial version
...
**********************************************************************/
package org.eclipse.ajdt.internal.ui.ajde;

import java.io.File;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.eclipse.ajdt.ui.AspectJPlugin;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.jdt.core.JavaCore;

public class BuildOptionsAdapter
	implements org.aspectj.ajde.BuildOptionsAdapter {

//	public final static QualifiedName USE_JAVAC =
//		new QualifiedName(AspectJPlugin.PLUGIN_ID, "BuildOptions.useJavac");
//	public final static QualifiedName PREPROCESS =
//		new QualifiedName(AspectJPlugin.PLUGIN_ID, "BuildOptions.preProcess");
//	public final static QualifiedName SOURCE14 =
//		new QualifiedName(AspectJPlugin.PLUGIN_ID, "BuildOptions.source14");
//	public final static QualifiedName PORTING_MODE =
//		new QualifiedName(AspectJPlugin.PLUGIN_ID, "BuildOptions.portingMode");
//	public final static QualifiedName WORKING_DIR =
//		new QualifiedName(AspectJPlugin.PLUGIN_ID, "BuildOptions.workingDirectory");
		
	public final static QualifiedName INPUTJARS =
	    new QualifiedName(AspectJPlugin.PLUGIN_ID, "BuildOptions.inputJars");    
	public final static QualifiedName OUTPUTJAR = 
	    new QualifiedName(AspectJPlugin.PLUGIN_ID, "BuildOptions.outputJar");	    
	public final static QualifiedName SOURCEROOTS =
	    new QualifiedName(AspectJPlugin.PLUGIN_ID, "BuildOptions.sourceRoots");
	public final static QualifiedName ASPECTJARS =
	    new QualifiedName(AspectJPlugin.PLUGIN_ID, "BuildOptions.aspectJars");
	
		// Temporarily removing this option, will return nothing to the caller.
	public final static QualifiedName CHAR_ENC =
	
		new QualifiedName(AspectJPlugin.PLUGIN_ID, "BuildOptions.characterEncoding");
	public final static QualifiedName NON_STANDARD_OPTS =
		new QualifiedName(AspectJPlugin.PLUGIN_ID, "BuildOptions.nonStandardOptions");
	public final static QualifiedName COMPILATION_STRICTNESS =
		new QualifiedName(
			AspectJPlugin.PLUGIN_ID,
			"BuildOptions.compilationStrictness");

	// Enumeration of values for COMPILATION_STRICTNESS key
//	public final static String COMPILATION_STRICTNESS_NORMAL = "NORMAL";
//	public final static String COMPILATION_STRICTNESS_LENIENT = "LENIENT";
//	public final static String COMPILATION_STRICTNESS_STRICT = "STRICT";

	// Default values for those options that are easy to default...
//	public final static boolean USE_JAVAC_DEFAULT = false;
//	public final static boolean PREPROCESS_DEFAULT = false;
//	public final static boolean SOURCE14_DEFAULT = false;
//	public final static boolean PORTING_MODE_DEFAULT = false;
//	public final static String COMPILATION_STRICTNESS_DEFAULT =
//		COMPILATION_STRICTNESS_NORMAL;
	public final static String NON_STANDARD_OPTS_DEFAULT = "-Xlint";
	public final static String CHAR_ENC_DEFAULT =
		System.getProperty("file.encoding");
	//	java.util.Locale.getDefault().getDisplayName();

	// Return formatted version of the current build options set
	public String toString() {
		StringBuffer formattedOptions = new StringBuffer();
		formattedOptions.append("Current Compiler options set:");
//		formattedOptions.append(
//			"[Use javac for compilation=" + getUseJavacMode() + "]");
//		formattedOptions.append("[Just preprocess=" + getPreprocessMode() + "]");
//		formattedOptions.append(
//			"[Allow 1.4 assertions=" + getSourceOnePointFourMode() + "]");
//		formattedOptions.append("[Porting mode=" + getPortingMode() + "]");
//		formattedOptions.append("[WorkingDirectory='" + getWorkingOutputPath() + "']");
		formattedOptions.append(
			"[NonStandard options='" + getNonStandardOptions() + "']");
//		if (getLenientSpecMode())
//			formattedOptions.append("[Compilation Strictness=LENIENT]");
//		else if (this.getStrictSpecMode())
//			formattedOptions.append("[Compilation Strictness=STRICT]");
//		else
//			formattedOptions.append("[Compilation Strictness=NORMAL]");

		return formattedOptions.toString();
	}

	private boolean retrieveSettingBoolean(QualifiedName key) {

		IProject thisProject = AspectJPlugin.getDefault().getCurrentProject();

		try {
			String value = thisProject.getPersistentProperty(key);

			boolean valueB = new Boolean(value).booleanValue();
			return valueB;
		} catch (CoreException ce) {
			AspectJPlugin.getDefault().getErrorHandler().handleError(
				AspectJPlugin.getResourceString("buildOptionsAdapter.exceptionDuringRetrieve"),
				ce);
		}
		return false;
	}

	private String retrieveSettingString(QualifiedName key) {

		IProject thisProject = AspectJPlugin.getDefault().getCurrentProject();
		try {
			String value = thisProject.getPersistentProperty(key);
			return value;
		} catch (CoreException ce) {
			AspectJPlugin.getDefault().getErrorHandler().handleError(
				AspectJPlugin.getResourceString("buildOptionsAdapter.exceptionDuringRetrieve"),
				ce);
		}
		return "";
	}

	/**
	 * @see BuildOptionsAdapter#getCharacterEncoding()
	 */
	public String getCharacterEncoding() {
		ensurePropertiesInitialized();
		
		return "";
		/*
		 * Currently commented out since it has no meaning running in Eclipse 
		String encoding = retrieveSettingString(CHAR_ENC);
		if (AspectJPlugin.DEBUG_BUILDER) {
			System.out.println(
				"BuildOptionsAdapter.getCharacterEncoding called, returning :" + encoding);
		}
		return encoding;
		*/
	}

	/**
	 * @see BuildOptionsAdapter#getLenientSpecMode()
	 */
	public boolean getLenientSpecMode() {
		return false;
	}
//		ensurePropertiesInitialized();
//
//		String compilationStrictness = retrieveSettingString(COMPILATION_STRICTNESS);
//		boolean lenientMode =
//			compilationStrictness.equals(COMPILATION_STRICTNESS_LENIENT);
//
//		if (AspectJPlugin.DEBUG_BUILDER) {
//			System.out.println(
//				"BuildOptionsAdapter.getLenientSpecMode called, returning :"
//					+ new Boolean(lenientMode));
//		}
//
//		return lenientMode;
//	}

	/**
	 * @see BuildOptionsAdapter#getNonStandardOptions()
	 */
	public String getNonStandardOptions() {
		ensurePropertiesInitialized();

		String nonStandardOptions = retrieveSettingString(NON_STANDARD_OPTS);
		if (AspectJPlugin.DEBUG_BUILDER) {
			System.out.println(
				"BuildOptionsAdapter.getNonStandardOptions called, returning :"
					+ nonStandardOptions);
		}
		getInJars(); //ASCFIXME - In for testing purposes!
		return nonStandardOptions;
	}

	/**
	 * @see BuildOptionsAdapter#getPortingMode()
	 */
	public boolean getPortingMode() {
		return false;
	}
//		ensurePropertiesInitialized();
//
//		boolean portingMode = retrieveSettingBoolean(PORTING_MODE);
//		if (AspectJPlugin.DEBUG_BUILDER) {
//			System.out.println(
//				"BuildOptionsAdapter.getPortingMode called, returning :"
//					+ new Boolean(portingMode));
//		}
//		return portingMode;
//	}

	/**
	 * @see BuildOptionsAdapter#getPreprocessMode()
	 */
	public boolean getPreprocessMode() {
		return false;
	}
//		ensurePropertiesInitialized();
//
//		boolean preprocessMode = retrieveSettingBoolean(PREPROCESS);
//
//		if (AspectJPlugin.DEBUG_BUILDER) {
//			System.out.println(
//				"BuildOptionsAdapter.getPreprocessMode called, returning :"
//					+ new Boolean(preprocessMode));
//		}
//		return preprocessMode;
//
//	}

	/**
	 * @see BuildOptionsAdapter#getSourceOnePointFourMode()
	 */
	public boolean getSourceOnePointFourMode() {
		return false;
	}
//		ensurePropertiesInitialized();
//
//		boolean source14 = retrieveSettingBoolean(SOURCE14);
//		if (AspectJPlugin.DEBUG_BUILDER) {
//			System.out.println(
//				"BuildOptionsAdapter.getSourceOnePointFourMode called, returning :"
//					+ new Boolean(source14));
//		}
//		return source14;
//
//	}

	/**
	 * @see BuildOptionsAdapter#getStrictSpecMode()
	 */
	public boolean getStrictSpecMode() {
		return false;
	}
//		ensurePropertiesInitialized();
//
//		String compilationStrictness = retrieveSettingString(COMPILATION_STRICTNESS);
//		boolean strictMode =
//			compilationStrictness.equals(COMPILATION_STRICTNESS_STRICT);
//
//		if (AspectJPlugin.DEBUG_BUILDER) {
//			System.out.println(
//				"BuildOptionsAdapter.getStrictSpecMode called, returning :"
//					+ new Boolean(strictMode));
//		}
//
//		return strictMode;
//
//	}

	/**
	 * @see BuildOptionsAdapter#getUseJavacMode()
	 */
	public boolean getUseJavacMode() {
		return false;
	}
//		ensurePropertiesInitialized();
//
//		boolean useJavacMode = retrieveSettingBoolean(USE_JAVAC);
//
//		if (AspectJPlugin.DEBUG_BUILDER) {
//			System.out.println(
//				"BuildOptionsAdapter.getUseJavacMode called, returning :"
//					+ new Boolean(useJavacMode));
//		}
//
//		return useJavacMode;
//	}

	/**
	 * @see BuildOptionsAdapter#getWorkingOutputPath()
	 */
	public String getWorkingOutputPath() {
		return "";
	}
//		ensurePropertiesInitialized();
//
//		String workingDir = retrieveSettingString(WORKING_DIR);
//
//		if (AspectJPlugin.DEBUG_BUILDER) {
//			System.out.println(
//				"BuildOptionsAdapter.getWorkingOutputPath called, returning :" + workingDir);
//		}
//
//		return workingDir;
//	}
	
	
	
	/*
	 * The next few options are those configurable on the Java Compiler
	 * options panel.  Rather than duplicate the options panel,
	 * we query the settings and pass them to aspectj.
	 */
	  
	/**
	 * JDK Compliance level to be used by the compiler, either 
	 * VERSION_13 or VERSION_14.
	 * From -1.3 / -1.4
	 */
	public String getComplianceLevel() {
		String jdtComplianceLevel =
		  JavaCore.getOption(JavaCore.COMPILER_COMPLIANCE);
		String ajcComplianceLevel = null;
		  
		if (jdtComplianceLevel.equals(JavaCore.VERSION_1_4)) {
			ajcComplianceLevel = org.aspectj.ajde.BuildOptionsAdapter.VERSION_14;
		} else {
			// All other options map to 1.3
			ajcComplianceLevel = org.aspectj.ajde.BuildOptionsAdapter.VERSION_13;
		}

  	    if (AspectJPlugin.DEBUG_BUILDER) {
		  System.out.println(
		    "BuildOptionsAdapter.getComplianceLevel called.  Mapped JDT(" +
		     jdtComplianceLevel+") to AJC("+ajcComplianceLevel+")");
		}

		return ajcComplianceLevel;
	}
	
	/**
	 * Debug level. DEBUG_ALL == {SOURCE, LINES, VARS}.
	 * Empty list is equivalent to -g:none, returning
	 * non uses eclipse compiler default settings
	 * From -g:xxx
	 * 
	 * From JavaCore:
	 * 
	 * 	COMPILER / Generating Local Variable Debug Attribute
	 *          - option field:     COMPILER_LOCAL_VARIABLE_ATTR
 	 *     		- option id:		"org.eclipse.jdt.core.compiler.debug.localVariable"
	 *     		- possible values:	{ "generate", "do not generate" }
	 *     		- default:			"generate"
	 *
	 *	COMPILER / Generating Line Number Debug Attribute 
     *          - option field:     COMPILER_LINE_NUMBER_ATTR
	 *     		- option id:		"org.eclipse.jdt.core.compiler.debug.lineNumber"
	 *     		- possible values:	{ "generate", "do not generate" }
	 *     		- default:			"generate"
	 *		
	 *	COMPILER / Generating Source Debug Attribute 
	 *		    - option field:     COMPILER_SOURCE_FILE_ATTR
	 *     		- option id:		"org.eclipse.jdt.core.compiler.debug.sourceFile"
	 *     		- possible values:	{ "generate", "do not generate" }
	 *     		- default:			"generate"
	 * 
	 */
	public Set getDebugLevel() {
		Set ajcDebugLevel = new HashSet();
		
	    if (!JavaCore.getOption(JavaCore.COMPILER_LOCAL_VARIABLE_ATTR).equals(JavaCore.GENERATE))
	      ajcDebugLevel.add(org.aspectj.ajde.BuildOptionsAdapter.DEBUG_VARS);
	    
	    if (!JavaCore.getOption(JavaCore.COMPILER_LINE_NUMBER_ATTR).equals(JavaCore.GENERATE))
	      ajcDebugLevel.add(org.aspectj.ajde.BuildOptionsAdapter.DEBUG_LINES);
	    
	    if (!JavaCore.getOption(JavaCore.COMPILER_SOURCE_FILE_ATTR).equals(JavaCore.GENERATE))
	      ajcDebugLevel.add(org.aspectj.ajde.BuildOptionsAdapter.DEBUG_SOURCE);
	       
	    if (AspectJPlugin.DEBUG_BUILDER) {
	    	if (ajcDebugLevel.size()!=0) {
		  System.out.println(
		    "BuildOptionsAdapter.getDebugLevel called.  Returning set:");
		 
		    System.out.println("-- START --");
		    Iterator i = ajcDebugLevel.iterator();
		    while (i.hasNext()) {
		      String option = (String)i.next();
		      System.out.println("Debug generated for "+option);
		    }
		    System.out.println("--  END  --");
	    	} else {
	    	  System.out.println(
		    "BuildOptionsAdapter.getDebugLevel called.  Returning an empty set");
	    	}
	    		
		}
		
		return ajcDebugLevel;
	}
	
    /**
	 * No errors generated for unresolved imports
	 * From -noImportError
	 * 
	 * From JavaCore:
	 * - option field:     COMPILER_PB_INVALID_IMPORT
	 * - option id:		   "org.eclipse.jdt.core.compiler.problem.invalidImport"
	 * - possible values:  { "error", "warning", "ignore" }
	 * - default:		   "error"
	 */
	public boolean getNoImportError() {
		boolean ajcNoImportError = false;
		String jdtInvalidImportOption = 
		  JavaCore.getOption(JavaCore.COMPILER_PB_INVALID_IMPORT);
		
		if (jdtInvalidImportOption.equals(JavaCore.ERROR)) {
			// If the user asks for unresolved imports to be shown as ERRORs then
			// AJC should switch OFF this option.
			ajcNoImportError=false;
		} else if (jdtInvalidImportOption.equals(JavaCore.WARNING)) {
			ajcNoImportError=true;
		} else if (jdtInvalidImportOption.equals(JavaCore.IGNORE)) {
			ajcNoImportError=true;
		}
		
		
 	    if (AspectJPlugin.DEBUG_BUILDER) {
		  System.out.println(
		    "BuildOptionsAdapter.getNoImportError called.  Mapped JDT(" +
		     jdtInvalidImportOption+") to AJC("+ajcNoImportError+")");
		}

		return ajcNoImportError;		
	}
	
	/**
	 * Preserve all unused local variables (for debug)
	 * From -preserveAllLocals
	 * 
	 * From JavaCore:
	 * - option field: COMPILER_CODEGEN_UNUSED_LOCAL
	 * - option id:		"org.eclipse.jdt.core.compiler.codegen.unusedLocal"
	 * - possible values:	{ "preserve", "optimize out" }
	 * - default:			"preserve"
	 * 
	 */	
	public boolean getPreserveAllLocals() {
		boolean ajcPreserveAllLocals = false;
		String jdtUnusedLocalOption = 
		  JavaCore.getOption(JavaCore.COMPILER_CODEGEN_UNUSED_LOCAL);
		
		if (jdtUnusedLocalOption.equals(JavaCore.PRESERVE)) {
			ajcPreserveAllLocals=true;
		} else if (jdtUnusedLocalOption.equals(JavaCore.OPTIMIZE_OUT)) {
			ajcPreserveAllLocals=false;
		}		
		
 	    if (AspectJPlugin.DEBUG_BUILDER) {
		  System.out.println(
		    "BuildOptionsAdapter.getPreserveAllLocals called.  Mapped JDT(" +
		     jdtUnusedLocalOption+") to AJC("+ajcPreserveAllLocals+")");
		}

		return ajcPreserveAllLocals;		

	}
	
	/**
	 * Source compatibility level, either VERSION_13 or VERSION_14.
	 * From -source (eclipse option)
	 * 
	 * From JavaCore:
	 * - option field:      COMPILER_SOURCE
	 * - option id:			"org.eclipse.jdt.core.compiler.source"
	 * - possible values:	{ "1.3", "1.4" }
	 * - default:			"1.3"
	 */
	public String getSourceCompatibilityLevel() {
		String jdtSourceCompatLevel =
		  JavaCore.getOption(JavaCore.COMPILER_SOURCE);
		String ajcSourceCompatLevel = null;
		  
		if (jdtSourceCompatLevel.equals(JavaCore.VERSION_1_4)) {
			ajcSourceCompatLevel = org.aspectj.ajde.BuildOptionsAdapter.VERSION_14;
		} if (jdtSourceCompatLevel.equals(JavaCore.VERSION_1_3)) {
			ajcSourceCompatLevel = org.aspectj.ajde.BuildOptionsAdapter.VERSION_13;
		}

  	    if (AspectJPlugin.DEBUG_BUILDER) {
		  System.out.println(
		    "BuildOptionsAdapter.getSourceCompatibilityLevel called.  Mapped JDT(" +
		     jdtSourceCompatLevel+") to AJC("+ajcSourceCompatLevel+")");
		}

		return ajcSourceCompatLevel;
	}
	
	
    /**
	 * Optional warnings, empty List is equivalent to -warn:none,
	 * returning null uses eclipse compiler default settings
	 * From -warn:xxx,yyy
	 * 
	 * The value passed to ajc is a consolidation of several fields from the Java preferences page.
	 * These are shown below, a value of either 'error' or 'warning' on the JDT panel is mapped
	 * to an ajc 'warning'.
	 * 
	 * From JavaCore:
	 * 
	 * COMPILER / Reporting Attempt to Override Package-Default Method
	 *          - option field:     COMPILER_PB_OVERRIDING_PACKAGE_DEFAULT_METHOD
	 *     		- option id:		"org.eclipse.jdt.core.compiler.problem.overridingPackageDefaultMethod"
	 *     		- possible values:	{ "error", "warning", "ignore" }
	 *     		- default:			"warning"
	 *
	 * COMPILER / Reporting Method With Constructor Name
	 *          - option field:     COMPILER_PB_METHOD_WITH_CONSTRUCTOR_NAME
	 *     		- option id:		"org.eclipse.jdt.core.compiler.problem.methodWithConstructorName"
	 *     		- possible values:	{ "error", "warning", "ignore" }
	 *     		- default:			"warning"
	 * 
	 * COMPILER / Reporting Deprecation
	 *          - option field:     COMPILER_PB_DEPRECATION
	 *     		- option id:		"org.eclipse.jdt.core.compiler.problem.deprecation"
	 *     		- possible values:	{ "error", "warning", "ignore" }
	 *     		- default:			"warning"
	 * 
	 * COMPILER / Reporting Hidden Catch Block
	 * 			- option field:     COMPILER_PB_HIDDEN_CATCH_BLOCK
	 *     		- option id:		"org.eclipse.jdt.core.compiler.problem.hiddenCatchBlock"
	 *     		- possible values:	{ "error", "warning", "ignore" }
	 *     		- default:			"warning"
	 * 
	 * COMPILER / Reporting Unused Local
	 *          - option field:     COMPILER_PB_UNUSED_LOCAL
	 *     		- option id:		"org.eclipse.jdt.core.compiler.problem.unusedLocal"
	 *     		- possible values:	{ "error", "warning", "ignore" }
	 *     		- default:			"ignore"
	 * 
	 * COMPILER / Reporting Unused Parameter
	 *          - option field:     COMPILER_PB_UNUSED_PARAMETER
	 *     		- option id:		"org.eclipse.jdt.core.compiler.problem.unusedParameter"
	 *     		- possible values:	{ "error", "warning", "ignore" }
	 *     		- default:			"ignore"
	 *
	 * COMPILER / Reporting Unused Import
	 *          - option field:     COMPILER_PB_UNUSED_IMPORT
	 *     		- option id:		"org.eclipse.jdt.core.compiler.problem.unusedImport"
	 *     		- possible values:	{ "error", "warning", "ignore" }
	 *     		- default:			"ignore"
     *
     * COMPILER / Reporting Synthetic Access Emulation
     *          - option field:     COMPILER_PB_SYNTHETIC_ACCESS_EMULATION
	 *     		- option id:		"org.eclipse.jdt.core.compiler.problem.syntheticAccessEmulation"
	 *     		- possible values:	{ "error", "warning", "ignore" }
	 *     		- default:			"ignore"
	 *
	 * COMPILER / Reporting Non-Externalized String Literal
	 *          - option field:     COMPILER_PB_NON_NLS_STRING_LITERAL
	 *     		- option id:		"org.eclipse.jdt.core.compiler.problem.nonExternalizedStringLiteral"
	 *     		- possible values:	{ "error", "warning", "ignore" }
	 *     		- default:			"ignore"
	 * 
	 * COMPILER / Reporting Usage of 'assert' Identifier
	 *     - option field:      COMPILER_PB_ASSERT_IDENTIFIER
	 *     - option id:			"org.eclipse.jdt.core.compiler.problem.assertIdentifier"
	 *     - possible values:	{ "error", "warning", "ignore" }
	 *     - default:			"ignore"
	 */
	public Set getWarnings() {
		Set ajcWarnings = new HashSet();
		
		if (!JavaCore.getOption(JavaCore.COMPILER_PB_OVERRIDING_PACKAGE_DEFAULT_METHOD).
		    equals(JavaCore.IGNORE)) {
		    	ajcWarnings.add(org.aspectj.ajde.BuildOptionsAdapter.WARN_PACKAGE_DEFAULT_METHOD);
	    }
	    
	    if (!JavaCore.getOption(JavaCore.COMPILER_PB_METHOD_WITH_CONSTRUCTOR_NAME).
	        equals(JavaCore.IGNORE)) {
	        	ajcWarnings.add(org.aspectj.ajde.BuildOptionsAdapter.WARN_CONSTRUCTOR_NAME);
	    }
	    
	    if (!JavaCore.getOption(JavaCore.COMPILER_PB_DEPRECATION).
	        equals(JavaCore.IGNORE)) {
	        	ajcWarnings.add(org.aspectj.ajde.BuildOptionsAdapter.WARN_DEPRECATION);
	    }
	    
	    if (!JavaCore.getOption(JavaCore.COMPILER_PB_HIDDEN_CATCH_BLOCK).
	        equals(JavaCore.IGNORE)) {
	        	ajcWarnings.add(org.aspectj.ajde.BuildOptionsAdapter.WARN_MASKED_CATCH_BLOCKS);
	    }
	    
	    if (!JavaCore.getOption(JavaCore.COMPILER_PB_UNUSED_LOCAL).
	        equals(JavaCore.IGNORE)) {
	        	ajcWarnings.add(org.aspectj.ajde.BuildOptionsAdapter.WARN_UNUSED_LOCALS);
	    }

	    if (!JavaCore.getOption(JavaCore.COMPILER_PB_UNUSED_PARAMETER).
	        equals(JavaCore.IGNORE)) {
	        	ajcWarnings.add(org.aspectj.ajde.BuildOptionsAdapter.WARN_UNUSED_ARGUMENTS);
	    }
	    
	    if (!JavaCore.getOption(JavaCore.COMPILER_PB_UNUSED_IMPORT).
	        equals(JavaCore.IGNORE)) {
	        	ajcWarnings.add(org.aspectj.ajde.BuildOptionsAdapter.WARN_UNUSED_IMPORTS);
	    }

	    if (!JavaCore.getOption(JavaCore.COMPILER_PB_SYNTHETIC_ACCESS_EMULATION).
	        equals(JavaCore.IGNORE)) {
	        	ajcWarnings.add(org.aspectj.ajde.BuildOptionsAdapter.WARN_SYNTHETIC_ACCESS);
	    }

	    if (!JavaCore.getOption(JavaCore.COMPILER_PB_NON_NLS_STRING_LITERAL).
	        equals(JavaCore.IGNORE)) {
	        	ajcWarnings.add(org.aspectj.ajde.BuildOptionsAdapter.WARN_NLS);
	    }

	    if (!JavaCore.getOption(JavaCore.COMPILER_PB_ASSERT_IDENTIFIER).
	        equals(JavaCore.IGNORE)) {
	        	ajcWarnings.add(org.aspectj.ajde.BuildOptionsAdapter.WARN_ASSERT_IDENITIFIER);
	    }
	    
	    if (AspectJPlugin.DEBUG_BUILDER) {
		  System.out.println(
		    "BuildOptionsAdapter.getWarnings called.  Returning set:");
		    System.out.println("-- START --");
		    Iterator i = ajcWarnings.iterator();
		    while (i.hasNext()) {
		      String option = (String)i.next();
		      System.out.println("Warnings for "+option);
		    }
		    System.out.println("--  END  --");
		}

		return ajcWarnings;			
	}
	
	
	
	
	
	
	
	
	

	/**
	* Static form of preserveSetting that takes a project name, a key and a boolean,
	* it then preserves the boolean value for the named key on the supplied project resource.
	* Actually converts the boolean to a string and delegates to the 
	* other implementation of preserveSetting() that takes a string as the
	* value to store.
	*/
	private static void preserveSetting(
		IProject project,
		QualifiedName key,
		boolean flag)
		throws CoreException {
		preserveSetting(project, key, new Boolean(flag).toString());
	}

	/**
	 * Static form of preserveSetting that takes a project name, a key and a value,
	 * it then preserves the key/value pair as a persistent property against the 
	 * supplied project resource.
	 */
	private static void preserveSetting(
		IProject project,
		QualifiedName key,
		String value)
		throws CoreException {
		project.setPersistentProperty(key, value);
	}

	/** 
	 * Ensure the properties are initialized before they are used.
	 */
	private void ensurePropertiesInitialized() { 
		ensurePropertiesInitialized(AspectJPlugin.getDefault().getCurrentProject());
	}

	public static void ensurePropertiesInitialized(IProject project) {
		try {
//			if (project.getPersistentProperty(BuildOptionsAdapter.USE_JAVAC) == null)
//				preserveSetting(
//					project,
//					BuildOptionsAdapter.USE_JAVAC,
//					BuildOptionsAdapter.USE_JAVAC_DEFAULT);
//			if (project.getPersistentProperty(BuildOptionsAdapter.PREPROCESS) == null)
//				preserveSetting(
//					project,
//					BuildOptionsAdapter.PREPROCESS,
//					BuildOptionsAdapter.PREPROCESS_DEFAULT);
//			if (project.getPersistentProperty(BuildOptionsAdapter.SOURCE14) == null)
//				preserveSetting(
//					project,
//					BuildOptionsAdapter.SOURCE14,
//					BuildOptionsAdapter.SOURCE14_DEFAULT);
//			if (project.getPersistentProperty(BuildOptionsAdapter.WORKING_DIR) == null)
//				// ASCFIXME - Is this the right default?
//				preserveSetting(
//					project,
//					BuildOptionsAdapter.WORKING_DIR,
//					project.getLocation().toOSString());
					
			if (project.getPersistentProperty(BuildOptionsAdapter.INPUTJARS) == null)
			  preserveSetting(project,BuildOptionsAdapter.INPUTJARS,"");
			if (project.getPersistentProperty(BuildOptionsAdapter.OUTPUTJAR) == null)
			  preserveSetting(project,BuildOptionsAdapter.OUTPUTJAR,"");
			if (project.getPersistentProperty(BuildOptionsAdapter.ASPECTJARS) == null)
			  preserveSetting(project,BuildOptionsAdapter.ASPECTJARS,"");
			if (project.getPersistentProperty(BuildOptionsAdapter.SOURCEROOTS) == null)
			  preserveSetting(project,BuildOptionsAdapter.SOURCEROOTS,"");
			  
			if (project.getPersistentProperty(BuildOptionsAdapter.CHAR_ENC) == null)
				// Is this right? It comes out as "English" when perhaps it ought to be "ISO8859-1"
				preserveSetting(
					project,
					BuildOptionsAdapter.CHAR_ENC,
					BuildOptionsAdapter.CHAR_ENC_DEFAULT);
//			if (project.getPersistentProperty(BuildOptionsAdapter.COMPILATION_STRICTNESS)
//				== null)
//				preserveSetting(
//					project,
//					BuildOptionsAdapter.COMPILATION_STRICTNESS,
//					BuildOptionsAdapter.COMPILATION_STRICTNESS_DEFAULT);
//			if (project.getPersistentProperty(BuildOptionsAdapter.PORTING_MODE) == null)
//				preserveSetting(
//					project,
//					BuildOptionsAdapter.PORTING_MODE,
//					BuildOptionsAdapter.PORTING_MODE_DEFAULT);
//			if (project.getPersistentProperty(BuildOptionsAdapter.NON_STANDARD_OPTS)
//				== null)
//				preserveSetting(
//					project,
//					BuildOptionsAdapter.NON_STANDARD_OPTS,
//					BuildOptionsAdapter.NON_STANDARD_OPTS_DEFAULT);

		} catch (CoreException ce) {
			AspectJPlugin.getDefault().getErrorHandler().handleError(
				AspectJPlugin.getResourceString(
					"buildOptionsAdapter.exceptionInitializingProperties"),
				ce);
		}
	}

	/**
	 * Method getOutJar.
	 * @return String
	 */
	public String getOutJar() {
		ensurePropertiesInitialized();
		String outputJar = retrieveSettingString(BuildOptionsAdapter.OUTPUTJAR);
		if (AspectJPlugin.DEBUG_BUILDER) {
			System.out.println(
				"BuildOptionsAdapter.getOutJar called, returning :"
					+ outputJar);
		}

		return outputJar;
	}

	/**
	 * Method getInJars.
	 * @return Set
	 */
	public Set getInJars() {
		ensurePropertiesInitialized();
		String inputJars= retrieveSettingString(BuildOptionsAdapter.INPUTJARS);

		if (AspectJPlugin.DEBUG_BUILDER) {
			System.out.println(
				"BuildOptionsAdapter.getInJars called, returning :"
					+ inputJars);
		}
		if (inputJars.length()==0) return null; 
		
		return mapStringToSet(inputJars,false);
	}
	
	/**
	 * Utility method for converting a semicolon separated list of
	 * files stored in a string into a Set of java.io.File objects.
	 * 
	 */
	private Set mapStringToSet(String input, boolean validateFiles) {
		if (input.length()==0) return null;
		String inputCopy = input;
		
		StringBuffer invalidEntries = new StringBuffer();
		
		// For relative paths (they don't start with a File.separator
		// or a drive letter on windows) - we prepend the projectBaseDirectory
		String projectBaseDirectory = 
		  AspectJPlugin.getDefault().getCurrentProject().
		  getLocation().toOSString();
		  
		if (AspectJPlugin.DEBUG_BUILDER)
			System.out.println("Converting ]"+input+"[");
		
		Set fileSet = new HashSet();
		while (inputCopy.indexOf(java.io.File.pathSeparator)!=-1) {  //ASCFIXME - Bit too platform specific!
		  int idx = inputCopy.indexOf(java.io.File.pathSeparator);
		  String path = inputCopy.substring(0,idx);
		  
		  java.io.File f = new java.io.File(path);
		  if (!f.isAbsolute())
		  	f = new File(projectBaseDirectory+java.io.File.separator+path);
		  	if (validateFiles && !f.exists()) {
		  		invalidEntries.append(f+"\n");
		  		if (AspectJPlugin.DEBUG_BUILDER)System.out.println("Skipping file ]"+f.toString()+"[");  
		  	} else {
		  	  fileSet.add(f);
		  	  if (AspectJPlugin.DEBUG_BUILDER) System.out.println("Adding file ]"+f.toString()+"[");  
		  	}
		  inputCopy = inputCopy.substring(idx+1);	
		  
		}
		// Process the final element
		if (inputCopy.length()!=0) {
		  java.io.File f = new java.io.File(inputCopy);
		  if (!f.isAbsolute())
		  	f = new File(projectBaseDirectory+java.io.File.separator+inputCopy);
		  	if (validateFiles && !f.exists()) {
		  		invalidEntries.append(f+"\n");
		  		if (AspectJPlugin.DEBUG_BUILDER) System.out.println("Skipping file ]"+f.toString()+"[");
		  	} else {
		  fileSet.add(f);
		  if (AspectJPlugin.DEBUG_BUILDER) System.out.println("Adding file ]"+f.toString()+"[");  
	}
	
		}
		
		//ASCFIXME - Need to NLSify this string...
		if (validateFiles && invalidEntries.length()!=0) {
		  AspectJPlugin.getDefault().getErrorHandler().handleWarning(
		    "The following jar files do not exist and are being ignored:\n"+invalidEntries.toString());
		}
		return fileSet;
	}
	

	/**
	 * Method getSourceRoots.
	 * @return Set
	 */
	public Set getSourceRoots() {
		ensurePropertiesInitialized();
		String sourceRoots= retrieveSettingString(BuildOptionsAdapter.SOURCEROOTS);

		if (AspectJPlugin.DEBUG_BUILDER) {
			System.out.println(
				"BuildOptionsAdapter.getSourceRoots called, returning :"
					+ sourceRoots);
		}
		
		return mapStringToSet(sourceRoots,false);

	}

	/**
	 * Method getAspectPath.
	 * @return Set
	 */
	public Set getAspectPath() {
		ensurePropertiesInitialized();
		String aspectJars = retrieveSettingString(BuildOptionsAdapter.ASPECTJARS);

		if (AspectJPlugin.DEBUG_BUILDER) {
			System.out.println(
				"BuildOptionsAdapter.getAspectPath called, returning :"
					+ aspectJars);
		}
		
		return mapStringToSet(aspectJars,false);
	}

}