/**********************************************************************
Copyright (c) 2002 IBM Corporation and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the Common Public License v1.0
which accompanies this distribution, and is available at
http://www.eclipse.org/legal/cpl-v10.html
Contributors:
Adrian Colyer, Andy Clement, Tracy Gardner - initial version
...
**********************************************************************/
package org.eclipse.ajdt.internal.ui.editor;

import org.eclipse.jdt.ui.text.JavaTextTools;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.text.rules.RuleBasedScanner;
import org.eclipse.jface.util.PropertyChangeEvent;

/**
 * Identical to the Java text tools except that we use our own scanner.
 */
public class AspectJTextTools extends JavaTextTools {

	private AspectJCodeScanner aspectjCodeScanner;

	/**
	 * Constructor for AspectJTextTools
	 */
	public AspectJTextTools(IPreferenceStore ips) {
		super(ips);

		aspectjCodeScanner = new AspectJCodeScanner(getColorManager(), ips);
	} 
	
   /**
	* Returns a scanner which is configured to scan AspectJ source code.
	*
	* @return an AspectJsource code scanner
	*/
	public RuleBasedScanner getCodeScanner() {
		return aspectjCodeScanner;
	}
	/**
	 * Disposes all the individual tools of this tools collection.
	 */
	public void dispose() {

		aspectjCodeScanner = null;

		super.dispose();
	}

	public boolean affectsBehavior(PropertyChangeEvent event) {
		return  aspectjCodeScanner.affectsBehavior(event) ||
		  super.affectsBehavior(event);
	}
	
	/**
	 * Adapts the behavior of the contained components to the change
	 * encoded in the given event.
	 * 
	 * @param event the event to whch to adapt
	 */
	protected void adaptToPreferenceChange(PropertyChangeEvent event) {
		if (aspectjCodeScanner.affectsBehavior(event))
			aspectjCodeScanner.adaptToPreferenceChange(event);
			
			super.adaptToPreferenceChange(event);
	}

}