/*
 *                         Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License Version
 * 1.0 (the "License"). You may not use this file except in compliance with 
 * the License. A copy of the License is available at http://www.sun.com/
 *
 * The Original Code is the Ant module
 * The Initial Developer of the Original Code is Jayme C. Edwards.
 * Portions created by Jayme C. Edwards are Copyright (c) 2000.
 * All Rights Reserved.
 *
 * Contributor(s): Jesse Glick.
 */

package org.apache.tools.ant.module;

import java.awt.Image;
import java.beans.*;

import org.openide.util.NbBundle;

import org.apache.tools.ant.Project;

public class AntSettingsBeanInfo extends SimpleBeanInfo {

    public PropertyDescriptor[] getPropertyDescriptors () {
        try {
            PropertyDescriptor verbosity = new PropertyDescriptor ("verbosity", AntSettings.class);
            verbosity.setDisplayName (NbBundle.getMessage (AntSettingsBeanInfo.class, "PROP_verbosity"));
            verbosity.setShortDescription (NbBundle.getMessage (AntSettingsBeanInfo.class, "HINT_verbosity"));
            verbosity.setPropertyEditorClass (VerbosityEditor.class);
            PropertyDescriptor properties = new PropertyDescriptor ("properties", AntSettings.class);
            properties.setDisplayName (NbBundle.getMessage (AntSettingsBeanInfo.class, "PROP_properties"));
            properties.setShortDescription (NbBundle.getMessage (AntSettingsBeanInfo.class, "HINT_properties"));
            PropertyDescriptor saveAll = new PropertyDescriptor ("saveAll", AntSettings.class);
            saveAll.setDisplayName (NbBundle.getMessage (AntSettingsBeanInfo.class, "PROP_saveAll"));
            saveAll.setShortDescription (NbBundle.getMessage (AntSettingsBeanInfo.class, "HINT_saveAll"));
            PropertyDescriptor customDefs = new PropertyDescriptor ("customDefs", AntSettings.class);
            // [PENDING] later make it visible as an Expert property when there is a property editor for it...
            customDefs.setHidden (true);
            return new PropertyDescriptor[] { verbosity, properties, saveAll, customDefs };
        } catch (IntrospectionException ie) {
            if (Boolean.getBoolean ("netbeans.debug.exceptions"))
                ie.printStackTrace ();
            return null;
        }
    }

    private static Image icon;
    public Image getIcon (int type) {
        if (type == BeanInfo.ICON_COLOR_16x16 || type == BeanInfo.ICON_MONO_16x16) {
            if (icon == null)
                icon = loadImage ("/org/apache/tools/ant/module/resources/AntIcon.gif");
            return icon;
        } else {
            return null;
        }
    }

    public static class VerbosityEditor extends PropertyEditorSupport {

        private static final String[] tags = {
            NbBundle.getMessage (AntSettingsBeanInfo.class, "LBL_verbosity_err"),
            NbBundle.getMessage (AntSettingsBeanInfo.class, "LBL_verbosity_warn"),
            NbBundle.getMessage (AntSettingsBeanInfo.class, "LBL_verbosity_info"),
            NbBundle.getMessage (AntSettingsBeanInfo.class, "LBL_verbosity_verbose"),
            NbBundle.getMessage (AntSettingsBeanInfo.class, "LBL_verbosity_debug"),
	};
        
        private static final int[] levels = {
            Project.MSG_ERR,
            Project.MSG_WARN,
            Project.MSG_INFO,
            Project.MSG_VERBOSE,
            Project.MSG_DEBUG,
        };

        public String[] getTags () {
            return tags;
        }

        public String getAsText () {
            int l = ((Integer) getValue ()).intValue ();
            for (int i = 0; i < levels.length; i++) {
                if (l == levels[i]) {
                    return tags[i];
                }
            }
            return "???"; // NOI18N
        }

        public void setAsText (String text) throws IllegalArgumentException {
            for (int i = 0; i < tags.length; i++) {
                if (tags[i].equals (text)) {
                    setValue (new Integer (levels[i]));
                    return;
                }
            }
            throw new IllegalArgumentException ();
        }

    }

}
