/*
 *                         Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License Version
 * 1.0 (the "License"). You may not use this file except in compliance with 
 * the License. A copy of the License is available at http://www.sun.com/
 *
 * The Original Code is the Ant module
 * The Initial Developer of the Original Code is Jayme C. Edwards.
 * Portions created by Jayme C. Edwards are Copyright (c) 2000.
 * All Rights Reserved.
 *
 * Contributor(s): Jesse Glick.
 */

package org.apache.tools.ant.module.loader;

import java.awt.Image;
import java.beans.*;
import java.io.ObjectStreamException;

import org.openide.TopManager;
import org.openide.compiler.*;
import org.openide.compiler.Compiler; // ovverride java.lang.Compiler
import org.openide.cookies.CompilerCookie;
import org.openide.loaders.CompilerSupport;
import org.openide.loaders.DataObject;
import org.openide.loaders.MultiDataObject;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;

public class AntCompilerSupport extends CompilerSupport {
    
    protected AntCompilerSupport (MultiDataObject.Entry entry, Class cookie) {
        super (entry, cookie);
    }
    
    protected CompilerType defaultCompilerType () {
        //return CompilerType.find (AntCompilerType.class);
        return NoCompiler.NO_COMPILER;
    }
    
    public static class Compile extends AntCompilerSupport implements CompilerCookie.Compile {
        
        public Compile (MultiDataObject.Entry entry) {
            super (entry, CompilerCookie.Compile.class);
        }
        
    }
    
    public static class Build extends AntCompilerSupport implements CompilerCookie.Build {
        
        public Build (MultiDataObject.Entry entry) {
            super (entry, CompilerCookie.Build.class);
        }
        
    }
    
    public static class Clean extends AntCompilerSupport implements CompilerCookie.Clean {
        
        public Clean (MultiDataObject.Entry entry) {
            super (entry, CompilerCookie.Clean.class);
        }
        
    }

    // Copied from org.netbeans.beaninfo.editors.CompilerTypeEditor:
    public static final class NoCompiler extends CompilerType {
        
        public static final CompilerType NO_COMPILER = new NoCompiler ();

        private static final long serialVersionUID = -7936401242112258011L;
        
        private NoCompiler () {
        }

        protected String displayName () {
            return NbBundle.getBundle (AntCompilerSupport.class).getString ("LAB_NoCompiler");
        }

        public HelpCtx getHelpCtx () {
            return new HelpCtx ("org.apache.tools.ant.module.adv-comp-exec"); // NOI18N
        }

        public void prepareJob (CompilerJob ign1, Class ign2, DataObject dob) {
            // do nothing
        }

        private Object readResolve () throws ObjectStreamException {
            return NO_COMPILER;
        }

    }

    public static final class NoCompilerBeanInfo extends SimpleBeanInfo {

        public PropertyDescriptor[] getPropertyDescriptors () {
            return new PropertyDescriptor[] { };
        }

        private Image icon;
        public Image getIcon (int type) {
            if (type == BeanInfo.ICON_COLOR_16x16) {
                if (icon == null) icon = loadImage ("/org/apache/tools/ant/module/resources/NoCompilerIcon.gif"); // NOI18N
                return icon;
            } else {
                return null;
            }
        }

    }

}
