/*
 *                         Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License Version
 * 1.0 (the "License"). You may not use this file except in compliance with 
 * the License. A copy of the License is available at http://www.sun.com/
 *
 * The Original Code is the Ant module
 * The Initial Developer of the Original Code is Jayme C. Edwards.
 * Portions created by Jayme C. Edwards are Copyright (c) 2000.
 * All Rights Reserved.
 *
 * Contributor(s): Jesse Glick.
 */

package org.apache.tools.ant.module.loader;

import java.awt.*;
import java.beans.*;
import java.io.File;

import org.openide.TopManager;
import org.openide.explorer.ExplorerManager;
import org.openide.explorer.ExplorerPanel;
import org.openide.explorer.propertysheet.ExPropertyEditor;
import org.openide.explorer.view.BeanTreeView;
import org.openide.loaders.DataFilter;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObject;
import org.openide.nodes.Node;

import org.apache.tools.ant.module.api.AntProjectCookie;
import org.apache.tools.ant.module.xml.AntProjectSupport;

/** A property editor for properties of type AntProjectCookie.
 */
public class AntProjectCookieEditor implements PropertyEditor, PropertyChangeListener, DataFilter {

    private PropertyChangeSupport supp = new PropertyChangeSupport (this);
    private AntProjectCookie value = null;
    
    public Object getValue () {
        return value;
    }
    
    public void setValue (Object o) {
        value = (AntProjectCookie) o;
        supp.firePropertyChange (null, null, null);
    }
    
    public void addPropertyChangeListener (PropertyChangeListener l) {
        supp.addPropertyChangeListener (l);
    }
    
    public void removePropertyChangeListener (PropertyChangeListener l) {
        supp.removePropertyChangeListener (l);
    }
    
    public String getAsText () {
        AntProjectCookie cookie = (AntProjectCookie) getValue ();
        if (cookie == null) {
            return ""; // NOI18N
        } else {
            return cookie.toString ();
        }
    }
    
    public void setAsText (String text) {
        if (text.equals ("")) { // NOI18N
            setValue (null);
        } else {
            setValue (new AntProjectSupport (new File (text)));
        }
    }
    
    public boolean supportsCustomEditor () {
        return true;
    }
    
    public Component getCustomEditor () {
        ExplorerPanel panel = new ExplorerPanel ();
        panel.setLayout (new BorderLayout ());
        panel.add (new BeanTreeView (), BorderLayout.CENTER);
        panel.getExplorerManager ().addPropertyChangeListener (this);
        Node root = TopManager.getDefault ().getPlaces ().nodes ().repository (this);
        panel.getExplorerManager ().setRootContext (root);
        // Does not work:
        //supp.firePropertyChange (ExPropertyEditor.PROP_VALUE_VALID, null, Boolean.FALSE);
        return panel;
    }
    
    public void propertyChange (PropertyChangeEvent ev) {
        if (ExplorerManager.PROP_SELECTED_NODES.equals (ev.getPropertyName ())) {
            Node[] nodes = (Node[]) ev.getNewValue ();
            if (nodes.length == 1 && nodes[0].getCookie (AntProjectCookie.class) != null) {
                setValue (nodes[0].getCookie (AntProjectCookie.class));
                supp.firePropertyChange (ExPropertyEditor.PROP_VALUE_VALID, null, Boolean.TRUE);
            } else {
                setValue (null);
                supp.firePropertyChange (ExPropertyEditor.PROP_VALUE_VALID, null, Boolean.FALSE);
            }
        }
    }
    
    public boolean acceptDataObject (DataObject obj) {
        return (obj instanceof DataFolder) || obj.getCookie (AntProjectCookie.class) != null;
    }
    
    public String getJavaInitializationString () {
        return null;
    }
    
    public String[] getTags () {
        return null;
    }
    
    public boolean isPaintable () {
        return false;
    }
    
    public void paintValue (Graphics g, Rectangle r) {
    }
    
}
