/*
 *                         Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License Version
 * 1.0 (the "License"). You may not use this file except in compliance with 
 * the License. A copy of the License is available at http://www.sun.com/
 *
 * The Original Code is the Ant module
 * The Initial Developer of the Original Code is Jayme C. Edwards.
 * Portions created by Jayme C. Edwards are Copyright (c) 2000.
 * All Rights Reserved.
 *
 * Contributor(s): Jesse Glick.
 */
 
package org.apache.tools.ant.module.loader;

import java.io.*;
import java.util.*;
import javax.swing.event.ChangeListener;

import org.openide.WizardDescriptor;
import org.openide.cookies.OpenCookie;
import org.openide.filesystems.FileObject;
import org.openide.loaders.*;

/** Template wizard iterator for Ant projects.
 * Only differs from regular iterators in one respect:
 * rather than running the newly created object's default
 * action (BuildProjectAction, probably), it merely opens
 * it as text.
 * See bug #8016.
 */
public class AntTemplateIterator implements TemplateWizard.Iterator {

    private TemplateWizard wiz;
    
    private static final long serialVersionUID = -3632382761348346822L;
    
    public void initialize (TemplateWizard wiz) {
        this.wiz = wiz;
        //System.err.println("Current DO: " + wiz.getTemplate ());
    }
    
    public void uninitialize (TemplateWizard wiz) {
        this.wiz = null; // possible after #8028 fixed
    }
    
    public Set instantiate (TemplateWizard wiz) throws IOException {
        String name = wiz.getTargetName ();
        DataFolder targetFolder = wiz.getTargetFolder ();
        DataObject template = wiz.getTemplate ();
        DataObject result;
        if (name == null) {
            result = template.createFromTemplate (targetFolder);
        } else {
            result = template.createFromTemplate (targetFolder, name);
        }
        OpenCookie open = (OpenCookie) result.getCookie (OpenCookie.class);
        if (open != null) {
            open.open ();
        }
        return Collections.singleton (result);
    }
    
    public String name () {
        return ""; // NOI18N
    }
    
    public WizardDescriptor.Panel current () {
        //Thread.dumpStack ();
        // Default name for all Ant scripts is build.xml:
        wiz.setTargetName ("build"); // NOI18N
        return wiz.targetChooser ();
    }
    
    public boolean hasNext () {
        return false;
    }
    
    public boolean hasPrevious () {
        return false;
    }
    
    public void nextPanel () {
        throw new NoSuchElementException ();
    }
    
    public void previousPanel () {
        throw new NoSuchElementException ();
    }
    
    public void addChangeListener (ChangeListener l) {
    }
    
    public void removeChangeListener (ChangeListener l) {
    }
    
}
