/*
 *                         Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License Version
 * 1.0 (the "License"). You may not use this file except in compliance with 
 * the License. A copy of the License is available at http://www.sun.com/
 *
 * The Original Code is the Ant module
 * The Initial Developer of the Original Code is Jayme C. Edwards.
 * Portions created by Jayme C. Edwards are Copyright (c) 2000.
 * All Rights Reserved.
 *
 * Contributor(s): Jesse Glick.
 */
 
package org.apache.tools.ant.module.nodes;

import org.w3c.dom.Element;

import org.openide.nodes.Children;
import org.openide.nodes.Sheet;
import org.openide.util.HelpCtx;

import org.apache.tools.ant.module.AntSettings;
import org.apache.tools.ant.module.api.ElementCookie;
import org.apache.tools.ant.module.api.IntrospectedInfo;
import org.apache.tools.ant.module.xml.ElementSupport;

public class AntTaskNode extends DataTypeNode {
    
    public AntTaskNode (Element el) {
        super (el, computeChildren (el));
    }
    
    private static String findTaskClazz (Element el) {
        String taskname = el.getNodeName ();
        String clazz = (String) IntrospectedInfo.getDefaults ().getTaskdefs ().get (taskname);
        if (clazz == null) {
            clazz = (String) AntSettings.getDefault ().getCustomDefs ().getTaskdefs ().get (taskname);
        }
        return clazz;
    }
    
    private static Children computeChildren (Element el) {
        String clazz = findTaskClazz (el);
        if (clazz != null && isEmpty (clazz)) {
            return Children.LEAF;
        } else {
            return new ElementChildren (el);
        }
    }
    
    protected ElementCookie createElementCookie () {
        String clazz = findTaskClazz (el);
        if (clazz != null) {
            return new ElementSupport.Instance (el, clazz);
        } else {
            return new ElementSupport (el);
        }
    }
    
    protected void initDisplay () {
        super.initDisplay ();
        setIconBase ("/org/apache/tools/ant/module/resources/TaskIcon");
    }
    
    public HelpCtx getHelpCtx () {
        String name = el.getNodeName ();
        if (IntrospectedInfo.getDefaults ().getTaskdefs ().containsKey (name)) {
            // Standard task.
            return new HelpCtx ("org.apache.tools.ant.module.tasks." + name);
        } else {
            // Custom.
            return new HelpCtx ("org.apache.tools.ant.module.custom-tasks");
        }
    }
    
    protected void addProperties (Sheet.Set props) {
        super.addProperties (props);
        // I18N if uncommented:
        //props.put (new AntProperty (el, "description")); // NOI18N
    }
    
}
