/*
 *                         Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License Version
 * 1.0 (the "License"). You may not use this file except in compliance with 
 * the License. A copy of the License is available at http://www.sun.com/
 *
 * The Original Code is the Ant module
 * The Initial Developer of the Original Code is Jayme C. Edwards.
 * Portions created by Jayme C. Edwards are Copyright (c) 2000.
 * All Rights Reserved.
 *
 * Contributor(s): Jesse Glick.
 */
 
package org.apache.tools.ant.module.nodes;

import java.io.IOException;
import java.util.*;

import org.w3c.dom.*;
import org.w3c.dom.events.*;

import org.openide.*;
import org.openide.nodes.*;
import org.openide.util.NbBundle;

import org.apache.tools.ant.module.AntModule;
import org.apache.tools.ant.module.xml.ElementSupport;

// key type: Element
public class ElementChildren extends Children.Keys implements org.w3c.dom.events.EventListener {

    protected Element el;
    
    public ElementChildren (Element el) {
        this.el = el;
    }
    
    protected ElementChildren () {
    }
    
    protected Element getElement () {
        return el;
    }

    protected void addNotify () {
        AntModule.err.log ("ElementChildren.addNotify");
        refreshKeys ();
        Element el = getElement ();
        if (el == null) return;
        if (el instanceof EventTarget) {
            // #10756: DOMNodeRemoved is fired *before* the event occurs.
            // DOMSubtreeModified is best because it is always fired later.
            ((EventTarget) el).addEventListener ("DOMSubtreeModified", this, false); // NOI18N
            //((EventTarget) el).addEventListener ("DOMAttrModified", this, false); // NOI18N
            //((EventTarget) el).addEventListener ("DOMNodeInserted", this, false); // NOI18N
            //((EventTarget) el).addEventListener ("DOMNodeRemoved", this, false); // NOI18N
            //((EventTarget) el).addEventListener ("DOMCharacterDataModified", this, false); // NOI18N
        } else {
            AntModule.err.log (ErrorManager.WARNING, "Element was not an EventTarget: " + el); // NOI18N
        }
    }

    protected void removeNotify () {
        AntModule.err.log ("ElementChildren.removeNotify");
        Element el = getElement ();
        if (el == null) return;
        if (el instanceof EventTarget) {
            ((EventTarget) el).removeEventListener ("DOMSubtreeModified", this, false); // NOI18N
            //((EventTarget) el).removeEventListener ("DOMAttrModified", this, false); // NOI18N
            //((EventTarget) el).removeEventListener ("DOMNodeInserted", this, false); // NOI18N
            //((EventTarget) el).removeEventListener ("DOMNodeRemoved", this, false); // NOI18N
            //((EventTarget) el).removeEventListener ("DOMCharacterDataModified", this, false); // NOI18N
        } else {
            AntModule.err.log (ErrorManager.WARNING, "Element was not an EventTarget: " + el); // NOI18N
        }
        setKeys (Collections.EMPTY_SET);
    }
    
    protected void refreshKeys () {
        AntModule.err.log ("ElementChildren.refreshKeys: node=" + getNode ().getDisplayName ());
        Element el = getElement ();
        if (el == null) {
            setKeys (Collections.EMPTY_SET);
            return;
        }
        NodeList l1 = el.getChildNodes ();
        int len = l1.getLength ();
        List l2 = new ArrayList (len);
        for (int i = 0; i < len; i++) {
            org.w3c.dom.Node node = l1.item (i);
            if (node instanceof Element) {
                AntModule.err.log ("\tnode=" + node.getNodeName ());
                l2.add (node);
            //} else {
            //    AntModule.err.log ("\tmisc node=" + node);
            }
        }
        setKeys (l2);
    }

    protected org.openide.nodes.Node[] createNodes (Object key) {
        Element el = (Element) key;
        ElementSupport.Instance instance = (ElementSupport.Instance) getNode ().getCookie (ElementSupport.Instance.class);
        String error;
        if (instance != null) {
            String type = el.getNodeName ();
            try {
                java.util.Map subels = instance.getSubelements ();
                if (subels != null) {
                    String clazz = (String) subels.get (type);
                    if (clazz != null) {
                        //AntModule.err.log ("ElementChildren.createNodes: me=" + getNode ().getDisplayName () + " type=" + type + " clazz=" + clazz);
                        return new org.openide.nodes.Node[] { new DataTypeNode (el, clazz) };
                    } else {
                        error = NbBundle.getMessage (ElementChildren.class, "ERR_no_subelements", instance.instanceName (), type);
                    }
                } else {
                    error = NbBundle.getMessage (ElementChildren.class, "ERR_not_analyzed", instance.instanceName ());
                }
            } catch (ClassNotFoundException cnfe) {
                error = cnfe.toString ();
            } catch (IOException ioe) {
                error = ioe.toString ();
            }
        } else {
            error = NbBundle.getMessage (ElementChildren.class, "ERR_parent_also");
        }
        return new org.openide.nodes.Node[] { new ElementNode (el, error) };
    }

    public void handleEvent (Event evt) {
        AntModule.err.log ("ElementChildren.handleEvent: " + evt.getType ());
        refreshKeys ();
    }
    
}
