/*
 *                         Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License Version
 * 1.0 (the "License"). You may not use this file except in compliance with 
 * the License. A copy of the License is available at http://www.sun.com/
 *
 * The Original Code is the Ant module
 * The Initial Developer of the Original Code is Jayme C. Edwards.
 * Portions created by Jayme C. Edwards are Copyright (c) 2000.
 * All Rights Reserved.
 *
 * Contributor(s): Jesse Glick.
 */

package org.apache.tools.ant.module.run;

import java.util.*;

import org.openide.compiler.*;
import org.openide.compiler.Compiler;
import org.openide.filesystems.FileObject;
import org.openide.util.Utilities;

import org.apache.tools.ant.module.api.AntProjectCookie;

public class AntCompiler extends Compiler {

    private AntProjectCookie pcookie;
    private String target;
    private Properties properties;

    /** target may be null for default target */
    public AntCompiler (AntProjectCookie pcookie, String target, Properties properties) {
        this.pcookie = pcookie;
        this.target = target;
        this.properties = properties;
    }

    public boolean equals (Object o) {
        if (! (o instanceof AntCompiler)) return false;
        AntCompiler other = (AntCompiler) o;
        return pcookie.equals (other.pcookie) &&
            Utilities.compareObjects (target, other.target) &&
            properties.equals (other.properties);
    }

    public int hashCode () {
        return 4882 ^
            pcookie.hashCode () ^
            ((target == null) ? 0 : target.hashCode ()) ^
            properties.hashCode ();
    }

    public String toString () {
        // For debugging e.g. #10585:
        return "AntCompiler[project=" + pcookie + ",target=" + target + ",props=" + properties.keySet () + "]"; // NOI18N
    }

    public AntProjectCookie getProjectCookie () {
        return pcookie;
    }

    public String getTarget () {
        return target;
    }
    
    public Properties getProperties () {
        return properties;
    }

    public boolean isUpToDate () {
        // Only Ant knows for sure:
        return false;
    }

    public Class compilerGroupClass () {
        return AntCompilerGroup.class;
    }
    
    // Group by properties:
    public Object compilerGroupKey () {
        List l = new ArrayList (2);
        l.add (super.compilerGroupKey ());
        l.add (properties);
        return l;
    }

}
