/*
 *                         Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License Version
 * 1.0 (the "License"). You may not use this file except in compliance with 
 * the License. A copy of the License is available at http://www.sun.com/
 *
 * The Original Code is the Ant module
 * The Initial Developer of the Original Code is Jayme C. Edwards.
 * Portions created by Jayme C. Edwards are Copyright (c) 2000.
 * All Rights Reserved.
 *
 * Contributor(s): Jesse Glick.
 */

package org.apache.tools.ant.module.run;

import java.io.IOException;
import java.util.*;

import org.openide.TopManager;
import org.openide.compiler.*;
import org.openide.compiler.Compiler;
import org.openide.execution.NbClassPath;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;

import org.apache.tools.ant.module.api.AntProjectCookie;
import org.apache.tools.ant.module.api.AntTargetCookie;

public class AntCompilerGroup extends CompilerGroup {

    private Map torun = new HashMap (); // Map<AntProjectCookie,Set<AntTargetCookie>>
    private Properties properties;

    public void add (Compiler c) throws IllegalArgumentException {
        if (! (c instanceof AntCompiler)) throw new IllegalArgumentException ();
        AntCompiler comp = (AntCompiler) c;
        AntProjectCookie proj = comp.getProjectCookie ();
        Set targets = (Set) torun.get (proj);
        if (targets == null) {
            targets = new HashSet ();
            torun.put (proj, targets);
        }
        targets.add (comp.getTarget ());
        if (properties == null) properties = comp.getProperties ();
    }
    
    public boolean start () {
        Iterator scripts = torun.entrySet ().iterator ();
        while (scripts.hasNext ()) {
            Map.Entry entry = (Map.Entry) scripts.next ();
            AntProjectCookie script = (AntProjectCookie) entry.getKey ();
            if (script.getFileObject () != null) {
                fireProgressEvent (new ProgressEvent (this, script.getFileObject (), ProgressEvent.TASK_UNKNOWN));
            }
            Set targets = (Set) entry.getValue ();
            // [PENDING] this could be smarter (preferably with a later version of Ant):
            // try to execute all the targets in this script at once, calculating their
            // dependencies all at once. Unfortunately there is not a good API in Ant to
            // do this. However there is a trick possible; see for example:
            // nbbuild/antsrc/org/netbeans/nbbuild/NbMerge.java.
            Iterator targetsit = targets.iterator ();
            while (targetsit.hasNext ()) {
                String target = (String) targetsit.next ();
                try {
                    TargetExecutor te = new TargetExecutor (script, (target == null) ? null : new String[] { target });
                    te.addProperties (properties);
                    if (te.execute ().result () != 0) {
                        // More annoying than useful:
                        //fireErrorEvent (new ErrorEvent (this, script, 0, 0, null, null));
                        return false;
                    }
                } catch (IOException ioe) {
                    TopManager.getDefault ().notifyException (ioe);
                    return false;
                }
            }
        }
        return true;
    }

}
