/*
 *                         Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License Version
 * 1.0 (the "License"). You may not use this file except in compliance with 
 * the License. A copy of the License is available at http://www.sun.com/
 *
 * The Original Code is the Ant module
 * The Initial Developer of the Original Code is Jayme C. Edwards.
 * Portions created by Jayme C. Edwards are Copyright (c) 2000.
 * All Rights Reserved.
 *
 * Contributor(s): Jesse Glick.
 */

package org.apache.tools.ant.module.run;

import java.awt.Image;
import java.beans.*;

import org.openide.compiler.CompilerType;
import org.openide.util.NbBundle;

public class AntCompilerTypeBeanInfo extends SimpleBeanInfo {

    public BeanInfo[] getAdditionalBeanInfo () {
        try {
            return new BeanInfo[] {
                       Introspector.getBeanInfo (CompilerType.class)
                   };
        } catch (IntrospectionException ie) {
            if (Boolean.getBoolean ("netbeans.debug.exceptions"))
                ie.printStackTrace ();
            return null;
        }
    }

    public BeanDescriptor getBeanDescriptor () {
        BeanDescriptor desc = new BeanDescriptor (AntCompilerType.class);
        desc.setDisplayName (NbBundle.getMessage (AntCompilerTypeBeanInfo.class, "LBL_CompilerType"));
        desc.setShortDescription (NbBundle.getMessage (AntCompilerTypeBeanInfo.class, "HINT_CompilerType"));
        desc.setValue ("version", "1.1"); // NOI18N
        return desc;
    }

    public PropertyDescriptor[] getPropertyDescriptors () {
        try {
            PropertyDescriptor compileTarget = new PropertyDescriptor ("compileTarget", AntCompilerType.class);
            compileTarget.setDisplayName (NbBundle.getMessage (AntCompilerTypeBeanInfo.class, "PROP_compileTarget"));
            compileTarget.setShortDescription (NbBundle.getMessage (AntCompilerTypeBeanInfo.class, "HINT_compileTarget"));
            PropertyDescriptor buildTarget = new PropertyDescriptor ("buildTarget", AntCompilerType.class);
            buildTarget.setDisplayName (NbBundle.getMessage (AntCompilerTypeBeanInfo.class, "PROP_buildTarget"));
            buildTarget.setShortDescription (NbBundle.getMessage (AntCompilerTypeBeanInfo.class, "HINT_buildTarget"));
            PropertyDescriptor cleanTarget = new PropertyDescriptor ("cleanTarget", AntCompilerType.class);
            cleanTarget.setDisplayName (NbBundle.getMessage (AntCompilerTypeBeanInfo.class, "PROP_cleanTarget"));
            cleanTarget.setShortDescription (NbBundle.getMessage (AntCompilerTypeBeanInfo.class, "HINT_cleanTarget"));
            PropertyDescriptor properties = new PropertyDescriptor ("properties", AntCompilerType.class);
            properties.setDisplayName (NbBundle.getMessage (AntCompilerTypeBeanInfo.class, "PROP_compiler_properties"));
            properties.setShortDescription (NbBundle.getMessage (AntCompilerTypeBeanInfo.class, "HINT_compiler_properties"));
            return new PropertyDescriptor[] { compileTarget, buildTarget, cleanTarget, properties };
        } catch (IntrospectionException ie) {
            if (Boolean.getBoolean ("netbeans.debug.exceptions"))
                ie.printStackTrace ();
            return null;
        }
    }

    private static Image icon;
    public Image getIcon (int type) {
        if (type == BeanInfo.ICON_COLOR_16x16 || type == BeanInfo.ICON_MONO_16x16) {
            if (icon == null)
                icon = loadImage ("/org/apache/tools/ant/module/resources/AntIcon.gif");
            return icon;
        } else {
            return null;
        }
    }
    
}
