/*
 *                         Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License Version
 * 1.0 (the "License"). You may not use this file except in compliance with 
 * the License. A copy of the License is available at http://www.sun.com/
 *
 * The Original Code is the Ant module
 * The Initial Developer of the Original Code is Jayme C. Edwards.
 * Portions created by Jayme C. Edwards are Copyright (c) 2000.
 * All Rights Reserved.
 *
 * Contributor(s): Jesse Glick.
 */

package org.apache.tools.ant.module.run;

import java.io.IOException;
import java.util.*;

import org.openide.execution.*;
import org.openide.loaders.DataObject;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.windows.InputOutput;

import org.w3c.dom.*;

import org.apache.tools.ant.module.AntModule;
import org.apache.tools.ant.module.api.AntProjectCookie;
import org.apache.tools.ant.module.api.AntTargetCookie;
import org.apache.tools.ant.module.xml.AntTargetSupport;

public class AntExecutor extends Executor {

    private String target = ""; // NOI18N
    private Properties properties = new Properties ();

    private static final long serialVersionUID = 4921782425419448313L;
    
    public String getTarget () {
        return target;
    }

    public synchronized void setTarget (String nue) {
        String old = target;
        target = nue;
        firePropertyChange ("target", old, nue); // NOI18N
    }
    
    public Properties getProperties () {
        return properties;
    }
    
    public synchronized void setProperties (Properties nue) {
        Properties old = properties;
        properties = nue;
        firePropertyChange ("properties", old, nue); // NOI18N
    }
    
    public ExecutorTask execute (ExecInfo info) throws IOException {
        throw new IOException ("unimplemented"); // NOI18N
    }
    
    public ExecutorTask execute (DataObject obj) throws IOException {
        AntProjectCookie cookie = (AntProjectCookie) obj.getCookie (AntProjectCookie.class);
        if (cookie == null) throw new IOException ("no AntProjectCookie"); // NOI18N
        if (target == null || target.equals ("")) { // NOI18N
            TargetExecutor texec = new TargetExecutor (cookie, null);
            texec.addProperties (properties);
            return texec.execute ();
        } else {
            List targets = new ArrayList (); // List<String>
            StringTokenizer tok = new StringTokenizer (target, ", "); // NOI18N
            while (tok.hasMoreTokens ()) {
                targets.add (tok.nextToken ());
            }
            TargetExecutor texec = new TargetExecutor (cookie, (String[]) targets.toArray (new String[targets.size ()]));
            texec.addProperties (properties);
            return texec.execute ();
        }
    }
    
    public HelpCtx getHelpCtx () {
        return new HelpCtx ("org.apache.tools.ant.module.adv-comp-exec");
    }

}
