/*
 *                         Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License Version
 * 1.0 (the "License"). You may not use this file except in compliance with 
 * the License. A copy of the License is available at http://www.sun.com/
 *
 * The Original Code is the Ant module
 * The Initial Developer of the Original Code is Jayme C. Edwards.
 * Portions created by Jayme C. Edwards are Copyright (c) 2000.
 * All Rights Reserved.
 *
 * Contributor(s): Jesse Glick.
 */

package org.apache.tools.ant.module.run;

import java.io.File;
import java.io.IOException;
import java.util.Map;

import org.openide.compiler.*;
import org.openide.compiler.Compiler;
import org.openide.cookies.CompilerCookie;
import org.openide.execution.NbClassPath;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObject;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;

import org.apache.tools.ant.module.AntModule;
import org.apache.tools.ant.module.api.AntProjectCookie;
import org.apache.tools.ant.module.xml.AntProjectSupport;

public class IndirectAntCompilerType extends AntCompilerType {

    private AntProjectCookie script = null;

    private static final long serialVersionUID = -7398778010799711047L;
    
    public AntProjectCookie getScript () {
        return script;
    }

    public synchronized void setScript (AntProjectCookie nue) {
        AntProjectCookie old = script;
        script = nue;
        firePropertyChange ("script", old, nue); // NOI18N
    }

    public HelpCtx getHelpCtx () {
        return new HelpCtx ("org.apache.tools.ant.module.adv-comp-exec");
    }

    protected void prepareJob0 (CompilerJob job, Class type, DataObject obj) throws IllegalArgumentException {
        if (obj.getCookie (AntProjectCookie.class) != null) { // #10680
            IllegalArgumentException iae = new IllegalArgumentException ("Is a build file: " + obj.getPrimaryFile ()); // NOI18N
            AntModule.err.annotate (iae, NbBundle.getMessage
                                    (IndirectAntCompilerType.class, "EXC_indirect_compiler_on_script", // NOI18N
                                     getName (), obj.getNodeDelegate ().getDisplayName ()));
            throw iae;
        }
        AntProjectCookie cookie = script;
    FOUND:
        if (cookie == null) {
            // Search for build file up the source tree.
            DataFolder folder = DataFolder.findFolder (obj.getPrimaryFile ().getParent ());
            while (true) {
                DataObject[] objs = folder.getChildren ();
                for (int i = 0; i < objs.length; i++) {
                    if ("build".equals (objs[i].getName ())) { // NOI18N
                        AntProjectCookie test = (AntProjectCookie) objs[i].getCookie (AntProjectCookie.class);
                        if (test != null) {
                            cookie = test;
                            break FOUND;
                        }
                    }
                }
                FileObject parent = folder.getPrimaryFile ().getParent ();
                if (parent == null) {
                    break;
                } else {
                    folder = DataFolder.findFolder (parent);
                }
            }
            // Not in filesystems, search up the raw file tree:
            File f = NbClassPath.toFile (folder.getPrimaryFile ());
            if (f == null) {
                IllegalArgumentException iae = new IllegalArgumentException ("Not local: " + obj.getPrimaryFile ()); // NOI18N
                AntModule.err.annotate (iae, NbBundle.getMessage (IndirectAntCompilerType.class, "EXC_must_be_local", obj.getNodeDelegate ().getDisplayName ()));
                throw iae;
            } else {
                while (true) {
                    f = f.getParentFile ();
                    if (f == null) {
                        IllegalArgumentException iae = new IllegalArgumentException ("No build.xml found for: " + obj.getPrimaryFile ()); // NOI18N
                        AntModule.err.annotate (iae, NbBundle.getMessage (IndirectAntCompilerType.class, "EXC_build_xml_not_found", obj.getNodeDelegate ().getDisplayName ()));
                        throw iae;
                    }
                    File[] kids = f.listFiles ();
                    if (kids == null) {
                        IllegalArgumentException iae = new IllegalArgumentException ("unreadable: " + f); // NOI18N
                        AntModule.err.annotate (iae, NbBundle.getMessage (IndirectAntCompilerType.class, "EXC_dir_not_readable", f.getAbsolutePath ()));
                        throw iae;
                    }
                    for (int i = 0; i < kids.length; i++) {
                        if (kids[i].isFile () && kids[i].getName ().equals ("build.xml")) { // NOI18N
                            cookie = new AntProjectSupport (kids[i]);
                            break FOUND;
                        }
                    }
                }
            }
        }
        reallyPrepareJob (job, type, cookie);
    }

}
