/*
 *                         Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License Version
 * 1.0 (the "License"). You may not use this file except in compliance with 
 * the License. A copy of the License is available at http://www.sun.com/
 *
 * The Original Code is the Ant module
 * The Initial Developer of the Original Code is Jayme C. Edwards.
 * Portions created by Jayme C. Edwards are Copyright (c) 2000.
 * All Rights Reserved.
 *
 * Contributor(s): Jesse Glick.
 */

package org.apache.tools.ant.module.run;

import java.awt.Image;
import java.beans.*;

import org.openide.util.NbBundle;

/** Description of the compiler type.
 *
 * @author jglick
 */
public class IndirectAntCompilerTypeBeanInfo extends SimpleBeanInfo {

    // Inherit properties and so on from ExternalCompilerType.
    public BeanInfo[] getAdditionalBeanInfo () {
        try {
            return new BeanInfo[] {
                       Introspector.getBeanInfo (AntCompilerType.class)
                   };
        } catch (IntrospectionException ie) {
            if (Boolean.getBoolean ("netbeans.debug.exceptions"))
                ie.printStackTrace ();
            return null;
        }
    }

    public BeanDescriptor getBeanDescriptor () {
        // Here is where to include a customizer class if you have one:
        BeanDescriptor desc = new BeanDescriptor (IndirectAntCompilerType.class);
        // Display name also serves as default for instances:
        desc.setDisplayName (NbBundle.getMessage (IndirectAntCompilerTypeBeanInfo.class, "LBL_IndirectCompilerType"));
        desc.setShortDescription (NbBundle.getMessage (IndirectAntCompilerTypeBeanInfo.class, "HINT_IndirectCompilerType"));
        desc.setValue ("version", "1.1"); // NOI18N
        return desc;
    }

    public PropertyDescriptor[] getPropertyDescriptors () {
        try {
            PropertyDescriptor script = new PropertyDescriptor ("script", IndirectAntCompilerType.class);
            script.setDisplayName (NbBundle.getMessage (IndirectAntCompilerTypeBeanInfo.class, "PROP_script"));
            script.setShortDescription (NbBundle.getMessage (IndirectAntCompilerTypeBeanInfo.class, "HINT_script"));
            //script.setPropertyEditorClass (AntProjectCookieEditor.class);
            return new PropertyDescriptor[] { script };
        } catch (IntrospectionException ie) {
            if (Boolean.getBoolean ("netbeans.debug.exceptions"))
                ie.printStackTrace ();
            return null;
        }
    }

    private static Image icon;
    public Image getIcon (int type) {
        if (type == BeanInfo.ICON_COLOR_16x16 || type == BeanInfo.ICON_MONO_16x16) {
            if (icon == null)
                // [PENDING] a different icon would be preferable...
                icon = loadImage ("/org/apache/tools/ant/module/resources/AntIcon.gif");
            return icon;
        } else {
            return null;
        }
    }

}
