/*
 *                         Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License Version
 * 1.0 (the "License"). You may not use this file except in compliance with 
 * the License. A copy of the License is available at http://www.sun.com/
 *
 * The Original Code is the Ant module
 * The Initial Developer of the Original Code is Jayme C. Edwards.
 * Portions created by Jayme C. Edwards are Copyright (c) 2000.
 * All Rights Reserved.
 *
 * Contributor(s): Jesse Glick
 */
 
package org.apache.tools.ant.module.run;

import java.io.*;

import org.openide.TopManager;
import org.openide.util.NbBundle;

import org.apache.tools.ant.*;

/** NetBeans-sensitive build logger.
 */
public class NetBeansLogger implements BuildLogger {
    
    // [PENDING] more coherent selection of messages based on verbosity
    
    private int level = Project.MSG_INFO;
    private PrintStream out, err;

    public void setEmacsMode (boolean ignore) {
        // Gack! Why is this part of an *interface*?!
    }
    
    public void setMessageOutputLevel (int l) {
        level = l;
    }
    
    public void setErrorPrintStream (PrintStream ps) {
        err = ps;
    }
    
    public void setOutputPrintStream (PrintStream ps) {
        out = ps;
    }
    
    public void messageLogged (BuildEvent ev) {
        if (ev.getPriority () <= level) {
            if (ev.getPriority () <= Project.MSG_WARN) {
                /* No good! Overlinks e.g. compile messages. How to solve??
                Task t = ev.getTask ();
                if (t != null) {
                    // This hyperlinks it:
                    err.print (t.getLocation ());
                }
                 */
                err.println (ev.getMessage ());
            } else {
                out.println (ev.getMessage ());
            }
        }
    }
    
    public void buildStarted (BuildEvent ev) {
        TopManager.getDefault ().setStatusText (NbBundle.getMessage (NetBeansLogger.class, "MSG_running_ant"));
        // no messages printed for now
    }
    
    public void buildFinished (BuildEvent ev) {
        Throwable t = ev.getException ();
        if (t == null) {
            // [PENDING] should check for target member (and TargetExecutor should set it...)
            err.println (NbBundle.getMessage (NetBeansLogger.class, "MSG_finished_target_printed"));
            TopManager.getDefault ().setStatusText (NbBundle.getMessage (NetBeansLogger.class, "MSG_finished_target_status"));
        } else {
            if ((t instanceof BuildException) && level < Project.MSG_VERBOSE) {
                // Stack trace probably not required.
                err.println (t);
            } else {
                t.printStackTrace (err);
            }
            TopManager.getDefault ().setStatusText (NbBundle.getMessage (NetBeansLogger.class, "MSG_target_failed_status"));
        }
    }
    
    public void targetStarted (BuildEvent ev) {
        out.println (NbBundle.getMessage (NetBeansLogger.class, "MSG_target_started_printed", ev.getTarget ().getName ()));
    }
    
    public void targetFinished (BuildEvent ev) {
        // ignore for now
    }
    
    public void taskStarted (BuildEvent ev) {
        // ignore for now
    }
    
    public void taskFinished (BuildEvent ev) {
        // ignore for now
    }
    
}
