/*
 *                         Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License Version
 * 1.0 (the "License"). You may not use this file except in compliance with 
 * the License. A copy of the License is available at http://www.sun.com/
 *
 * The Original Code is the Ant module
 * The Initial Developer of the Original Code is Jayme C. Edwards.
 * Portions created by Jayme C. Edwards are Copyright (c) 2000.
 * All Rights Reserved.
 *
 * Contributor(s): Jesse Glick.
 */
 
package org.apache.tools.ant.module.xml;

import javax.swing.event.*;

import org.w3c.dom.*;

import org.openide.util.*;

import org.apache.tools.ant.module.api.AntProjectCookie;
import org.apache.tools.ant.module.api.AntTargetCookie;

public class AntTargetSupport implements AntTargetCookie, ChangeListener {
    
    private AntProjectCookie project;
    private Element target;
    private String targetName;
    
    public AntTargetSupport (AntProjectCookie project) {
        this (project, (String) null);
    }
    
    public AntTargetSupport (AntProjectCookie project, String target) {
        this (project, findTarget (project, target));
        if (target != null) {
            targetName = target;
        }
    }
    
    private static Element findTarget (AntProjectCookie project, String target) {
        Element pel = project.getProjectElement ();
        if (pel == null) {
            return null;
        }
        if (target == null) {
            target = pel.getAttribute ("default"); // NOI18N
            if (target.length () == 0) {
                return null;
            }
        }
        NodeList children = pel.getElementsByTagName ("target"); // NOI18N
        for (int i = 0; i < children.getLength (); i++) {
            Element el = (Element) children.item (i);
            if (target.equals (el.getAttribute ("name"))) { // NOI18N
                return el;
            }
        }
        return null;
    }
    
    public AntTargetSupport (AntProjectCookie project, Element target) {
        this.project = project;
        this.target = target;
        if (target != null) {
            targetName = target.getAttribute ("name"); // NOI18N
            if (targetName.length () == 0) {
                targetName = null;
            }
        }
        project.addChangeListener (WeakListener.change (this, project));
    }

    public AntProjectCookie getProjectCookie () {
        return project;
    }
    
    public Element getTargetElement () {
        return target;
    }
    
    public boolean equals (Object o) {
        if (! (o instanceof AntTargetSupport)) return false;
        AntTargetSupport other = (AntTargetSupport) o;
        return project.equals (other.project) &&
            Utilities.compareObjects (targetName, other.targetName);
    }
    
    public int hashCode () {
        return 7718 ^ project.hashCode () ^ (targetName == null ? 0 : targetName.hashCode ());
    }
    
    public void stateChanged (ChangeEvent ev) {
        // Project was reparsed; try to find the new target element.
        // But first, maybe it was a direct DOM modification and this element is still valid.
        if (target != null && target.getParentNode () == project.getProjectElement ()) {
            return;
        }
        target = findTarget (project, targetName);
    }
    
}
