/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.apisupport;

import java.io.*;
import java.util.Enumeration;
import java.util.jar.*;

import org.openide.TopManager;
import org.openide.execution.*;
import org.openide.filesystems.FileObject;
import org.openide.loaders.*;
import org.openide.modules.ModuleDescription;
import org.openide.util.*;
import org.openide.windows.InputOutput;

import org.netbeans.core.ModuleInstaller;

public class InstallModuleExecutor extends Executor {

    private static final long serialVersionUID =3672055366671872774L;
    public HelpCtx getHelpCtx () {
        return new HelpCtx ("org.netbeans.modules.apisupport.modules");
    }

    // This method should not actually be called anymore, hopefully:
    public ExecutorTask execute (final ExecInfo info) throws IOException {
        String name = info.getClassName ();
        String packageName, baseName;
        int idx = name.lastIndexOf ((int) '.');
        if (idx == -1) {
            packageName = "";
            baseName = name;
        } else {
            packageName = name.substring (0, idx);
            baseName = name.substring (idx + 1);
        }
        //System.err.println("packageName=" + packageName + " baseName=" + baseName);
        Enumeration folders = TopManager.getDefault ().getRepository ().findAll (packageName, null, null);
        if (! folders.hasMoreElements ()) throw new IOException ("No such folder: " + packageName);
        while (folders.hasMoreElements ()) {
            FileObject folder = (FileObject) folders.nextElement ();
            //System.err.println("folder: " + folder.getPackageNameExt ('/', '.'));
            FileObject[] children = folder.getChildren ();
            for (int i = 0; i < children.length; i++) {
                FileObject fo = children[i];
                //System.err.println("child: " + fo.getPackageNameExt ('/', '.'));
                if (fo.getName ().equals (baseName)) {
                    //System.err.println("right basename");
                    try {
                        DataObject dob = DataObject.find (fo);
                        //System.err.println("data object: " + dob);
                        ManifestProvider p = (ManifestProvider) dob.getCookie (ManifestProvider.class);
                        //System.err.println("manifest provider: " + p);
                        if (p != null) return execute (dob);
                    } catch (DataObjectNotFoundException donfe) {
                        donfe.printStackTrace ();
                    }
                }
            }
        }
        throw new IOException ("No such manifest provider: " + name);
    }
    
    // Now this one ought to be invoked directly:
    public ExecutorTask execute (DataObject obj) throws IOException {
        ManifestProvider p = (ManifestProvider) obj.getCookie (ManifestProvider.class);
        if (p == null) throw new IOException ("Only applicable to module JARs");
        final File f = p.getModuleJar ();
        if (f == null) {
            throw new IOException ("Deploying test modules without JARs is no longer supported!");
        }
        Attributes a = p.getManifest ().getMainAttributes ();
        String moduleName = a.getValue (ModuleDescription.TAG_NAME);
        if (moduleName == null) moduleName = a.getValue (ModuleDescription.TAG_MAGIC);
        return TopManager.getDefault ().getExecutionEngine ().execute ("Installing " + moduleName, new Runnable () {
            public void run () {
                try {
                    ModuleInstaller.deployTestModule (f);
                } catch (IOException ioe) {
                    ioe.printStackTrace ();
                }
            }
        }, null);
    }

}
