/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.apisupport;

import java.awt.*;
import java.awt.event.*;
import javax.help.*;
import javax.swing.*;

import org.openide.TopManager;
import org.openide.cookies.ExecCookie;
import org.openide.loaders.XMLDataObject;

public class JavaHelpProcessor implements XMLDataObject.Processor, ExecCookie, Runnable, WindowListener {
    
    private XMLDataObject obj;
    private HelpSet hs;
    private String title;

    public void attachTo (XMLDataObject obj) {
        this.obj = obj;
    }
    
    public void start () {
        try {
            hs = new HelpSet (TopManager.getDefault ().currentClassLoader (), obj.getPrimaryFile ().getURL ());
        } catch (Exception e) {
            TopManager.getDefault ().notifyException (e);
            return;
        }
        title = hs.getTitle ();
        if (title == null) title = obj.getName ();
        TopManager.getDefault ().getExecutionEngine ().execute (title, this, null);
    }
    
    public void run () {
        try {
            JHelp jh = new JHelp (hs);
            JDialog dlg = new JDialog (TopManager.getDefault ().getWindowManager ().getMainWindow (), title, false);
            dlg.getContentPane ().add (new JHelp (hs), BorderLayout.CENTER);
            dlg.setSize (920, 660); // match min size of IDE's viewer
            dlg.pack ();
            dlg.addWindowListener (this);
            dlg.show ();
            synchronized (this) {
                wait ();
            }
        } catch (Exception e) {
            e.printStackTrace ();
        }
    }
    
    
    public synchronized void windowClosing (WindowEvent ev) {
        notify ();
    }
    
    public void windowDeactivated (WindowEvent ev) {}
    public void windowDeiconified (WindowEvent ev) {}
    public void windowOpened (WindowEvent ev) {}
    public void windowIconified (WindowEvent ev) {}
    public void windowClosed (WindowEvent ev) {}
    public void windowActivated (WindowEvent ev) {}
    
}
