/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.apisupport;

import javax.swing.SwingUtilities;
import javax.swing.event.*;

import org.openide.actions.OpenAction;
import org.openide.loaders.*;
import org.openide.nodes.*;
import org.openide.util.*;
import org.openide.util.actions.SystemAction;

public class ManifestDataNode extends DataNode implements Runnable, ChangeListener {

    private boolean haveCreatedSheet = false;

    public ManifestDataNode(ManifestDataObject obj) {
        this (obj, new ManifestProvider.CategoryChildren ((ManifestProvider) obj.getCookie (ManifestProvider.class)));
    }

    public ManifestDataNode(ManifestDataObject obj,Children ch) {
        super (obj, ch);
        RequestProcessor.postRequest (this);
        ManifestProvider provider = (ManifestProvider) getCookie (ManifestProvider.class);
        provider.addChangeListener (WeakListener.change (this, provider));
    }

    // Do not run synch in constructor, can deadlock:
    public void run () {
        if (SwingUtilities.isEventDispatchThread ()) {
            RequestProcessor.postRequest (this);
            return;
        }
        ManifestProvider.Util.updateName (this);
        updateIcon ();
        if (haveCreatedSheet) {
            firePropertyChange (null, null, null);
        }
    }

    public void stateChanged (ChangeEvent ev) {
        run ();
    }

    private void updateIcon () {
        if (ManifestProvider.Util.checkForException ((ManifestProvider) getCookie (ManifestProvider.class)) == null) {
            //System.err.println("icon: ok");
            setIconBase ("/org/netbeans/modules/apisupport/resources/ManifestDataIcon");
        } else {
            //System.err.println("icon: bad");
            setIconBase ("/org/netbeans/modules/apisupport/resources/ManifestDataIconError");
        }
    }

    protected Sheet createSheet () {
        Sheet sheet = super.createSheet ();
        Sheet.Set set = sheet.get (ExecSupport.PROP_EXECUTION);
        if (set == null) {
            set = new Sheet.Set ();
            set.setName (ExecSupport.PROP_EXECUTION);
            set.setDisplayName ("Execution");
        }
        ((ExecSupport) getCookie (ExecSupport.class)).addProperties (set);
        set.remove (ExecSupport.PROP_FILE_PARAMS);
        set.remove (ExecSupport.PROP_DEBUGGER_TYPE);
        sheet.put (set);
        ManifestProvider provider = (ManifestProvider) getCookie (ManifestProvider.class);
        ManifestProvider.Util.addToSheet (sheet, provider);
        haveCreatedSheet = true;
        return sheet;
    }

    public SystemAction getDefaultAction () {
        return SystemAction.get (OpenAction.class);
    }

}
