/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.apisupport;

import java.io.IOException;

import org.openide.cookies.*;
import org.openide.filesystems.FileLock;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObject;
import org.openide.text.DataEditorSupport;
import org.openide.windows.CloneableOpenSupport;

class ManifestEditorSupport extends DataEditorSupport implements EditorCookie, OpenCookie, LineCookie, CloseCookie, PrintCookie {

    public ManifestEditorSupport (ManifestDataObject obj) {
        super (obj, new ManifestEnv (obj));
        setMIMEType ("text/plain");
    }

    protected boolean notifyModified () {
        if (! super.notifyModified ()) {
            return false;
        }
        ManifestDataObject obj = (ManifestDataObject) getDataObject ();
        if (obj.getCookie (SaveCookie.class) == null) {
            obj.addSaveCookie (new Save ());
            obj.setModified (true);
        }
        return true;
    }

    protected void notifyUnmodified () {
        super.notifyUnmodified ();
        ManifestDataObject obj = (ManifestDataObject) getDataObject ();
        SaveCookie save = (SaveCookie) obj.getCookie (SaveCookie.class);
        if (save != null) {
            obj.removeSaveCookie (save);
            obj.setModified (false);
        }
    }

    private class Save implements SaveCookie {
        public void save () throws IOException {
            saveDocument ();
            getDataObject ().setModified (false);
        }
    }

    private static class ManifestEnv extends DataEditorSupport.Env {

        private static final long serialVersionUID = -760189334183961017L;
        
        public ManifestEnv (ManifestDataObject obj) {
            super (obj);
        }

        protected FileObject getFile () {
            return getDataObject ().getPrimaryFile ();
        }

        protected FileLock takeLock () throws IOException {
            return ((ManifestDataObject) getDataObject ()).getPrimaryEntry ().takeLock ();
        }

        public CloneableOpenSupport findCloneableOpenSupport () {
            return (ManifestEditorSupport) getDataObject ().getCookie (ManifestEditorSupport.class);
        }

    }

}
