/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.apisupport;

import java.io.*;

import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.*;
import org.openide.modules.ModuleDescription;

import org.netbeans.modules.jarpackager.*;

public class ModuleDataLoader extends JarDataLoader {

    private static final long serialVersionUID =-5017581069154105157L;

    public ModuleDataLoader () {
        this (ModuleDataObject.class);
    }

    public ModuleDataLoader (Class recognizedObject) {
        super (recognizedObject);
    }
    
    protected String defaultDisplayName () {
        return "OpenIDE Modules";
    }

    // Use same actions as JAR packager, so no initialize() needed.

    protected FileObject findPrimaryFile (FileObject fo) {
        if (fo.getNameExt ().endsWith ("-test.jar")) { // NOI18N
            // myjar54506-test.jar -> myjar.jarContent
            String jcext = getExtension ();
            String name = fo.getName ();
            name = name.substring (0, name.length () - "-test".length ()); // NOI18N
            FileObject folder = fo.getParent ();
            while (name.length () > 0) {
                FileObject jc = folder.getFileObject (name, jcext);
                if (jc != null) return jc;
                name = name.substring (0, name.length () - 1);
            }
            // No luck.
            return null;
        } else if (fo.hasExt (getExtension ())) {
            if (! isModuleContent (fo))
                return null;
            else
                return fo;
        } else if (fo.hasExt (getArchiveExt ())) {
            FileObject content = FileUtil.findBrother (fo, getExtension ());
            if (content == null || ! isModuleContent (content))
                return null;
            else
                return content;
        } else {
            return null;
        }
    }

    private boolean isModuleContent (FileObject fo) {
        if (fo.getSize () == 0) {
            // This is now fine: new jarpackager uses null templates & it is OK.
            return false;
        }
        try {
            InputStream is = fo.getInputStream ();
            try {
                JarContent content = new JarContent ();
                ObjectInputStream ois = new ObjectInputStream (is);
                try {
                    content.readContent (ois);
                    return content.getManifest ().getMainAttributes ().
                           getValue (ModuleDescription.TAG_MAGIC) != null;
                } finally {
                    ois.close ();
                }
            } finally {
                is.close ();
            }
        } catch (Exception e) {
            if (Boolean.getBoolean ("netbeans.debug.exceptions")) {
                e.printStackTrace ();
                System.err.println ("While checking: " + fo.getPackageNameExt ('/', '.'));
            }
            return false;
        }
    }

    protected MultiDataObject createMultiObject (FileObject primaryFile) throws IOException {
        return new ModuleDataObject (primaryFile, this);
    }
    
    protected MultiDataObject.Entry createSecondaryEntry (MultiDataObject obj, FileObject fo) {
        if (fo.getNameExt ().endsWith ("-test.jar")) { // NOI18N
            return new FileEntry.Numb (obj, fo);
        } else {
            return super.createSecondaryEntry (obj, fo);
        }
    }

}
