/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.apisupport;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.jar.*;
import javax.swing.SwingUtilities;
import javax.swing.event.*;

import org.openide.loaders.*;
import org.openide.modules.ModuleDescription;
import org.openide.nodes.*;
import org.openide.util.*;

import org.netbeans.modules.jarpackager.*;

public class ModuleDataNode extends JarDataObject.JarNode implements Runnable, ChangeListener {

    private boolean haveCreatedSheet = false;

    public ModuleDataNode(ModuleDataObject obj) {
        this (obj, new ManifestProvider.CategoryChildren ((ManifestProvider) obj.getCookie (ManifestProvider.class)));
    }

    public ModuleDataNode(ModuleDataObject obj, Children ch) {
        super (obj, ch);
        RequestProcessor.postRequest (this);
        ManifestProvider provider = (ManifestProvider) getCookie (ManifestProvider.class);
        provider.addChangeListener (WeakListener.change (this, provider));
    }

    // Do not run synch in constructor, can deadlock:
    public void run () {
        if (SwingUtilities.isEventDispatchThread ()) {
            RequestProcessor.postRequest (this);
            return;
        }
        ManifestProvider.Util.updateName (this);
        updateIcon ();
        if (haveCreatedSheet) {
            firePropertyChange (null, null, null);
        }
    }

    public void stateChanged (ChangeEvent ev) {
        run ();
    }

    private void updateIcon () {
        if (ManifestProvider.Util.checkForException ((ManifestProvider) getCookie (ManifestProvider.class)) == null)
            setIconBase ("/org/netbeans/modules/apisupport/resources/ModuleDataIcon");
        else
            setIconBase ("/org/netbeans/modules/apisupport/resources/ModuleDataIconError");
    }

    protected Sheet createSheet () {
        Sheet sheet = super.createSheet ();
        ManifestProvider provider = (ManifestProvider) getCookie (ManifestProvider.class);
        ManifestProvider.Util.addToSheet (sheet, provider);
        haveCreatedSheet = true;
        return sheet;
    }

}
