/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.apisupport;

import java.io.*;
import java.net.URL;
import java.util.*;
import java.util.jar.Manifest;
import javax.swing.SwingUtilities;
import javax.swing.event.*;

import org.openide.*;
import org.openide.cookies.ExecCookie;
import org.openide.filesystems.*;
import org.openide.loaders.*;
import org.openide.nodes.*;
import org.openide.util.HelpCtx;

import org.netbeans.modules.jarpackager.*;

public class ModuleDataObject extends JarDataObject {

    private static boolean warnedToAddToJar = false;

    private static final long serialVersionUID =4724305481753911436L;
    public ModuleDataObject(FileObject pf, MultiFileLoader loader) throws DataObjectExistsException {
        super (pf, loader);
        CookieSet cookies = getCookieSet ();
        cookies.add (new ManifestProviderSupport ());
        Node.Cookie formerExecCookie = cookies.getCookie (ExecCookie.class);
        if (formerExecCookie != null) cookies.remove (formerExecCookie);
        cookies.add (new ManifestProvider.ModuleExecSupport (getPrimaryEntry ()));
        // Necessary because JarCreator just writes out new content, does not
        // use setJarContent. Need to actually listen to the file itself.
        findContentFile ().addFileChangeListener (new FileChangeAdapter () {
                    public void fileChanged (FileEvent ev) {
                        ((ManifestProviderSupport) getCookie (ManifestProviderSupport.class)).fireStateChange ();
                    }
                });
    }

    public HelpCtx getHelpCtx () {
        return new HelpCtx ("org.netbeans.modules.apisupport.modules");
    }

    protected org.openide.nodes.Node createNodeDelegate () {
        return new ModuleDataNode (this);
    }

    private class ManifestProviderSupport implements ManifestProvider {

        private Set listeners = new HashSet (); // Set<ChangeListener>

        private JarContent jc () throws IOException {
            JarContent jarC = getJarContent ();
            if (jarC != null)
                return jarC;
            else
                throw new IOException ("No JAR content found for " + getPrimaryFile ().getPackageNameExt ('/', '.'));
        }

        public synchronized Manifest getManifest () throws IOException {
            return jc ().getManifest ();
        }

        public synchronized void setManifest (Manifest m) throws IOException {
            JarContent content = jc ();
            content.setManifest (m);
            setModified (true);
            fireStateChange ();
        }

        public synchronized void addFiles (Set files) throws IOException {
            if (! warnedToAddToJar) {
                warnedToAddToJar = true;
                SwingUtilities.invokeLater (new Runnable () {
                        public void run () {
                            TopManager.getDefault ().notify (new NotifyDescriptor.Message
                                ("Do not forget to ensure that the JAR file contains all files referenced by the manifest."));
                        }
                    });
            }
        }

        public synchronized void removeFiles (Set files) throws IOException {
        }

        public synchronized Set getFiles () throws IOException {
            List e = jc ().getContentList ().getAllFiles (jc ().getFilter());
            return new HashSet (e);
        }

        public synchronized void addChangeListener (ChangeListener list) {
            listeners.add (list);
        }

        public synchronized void removeChangeListener (ChangeListener list) {
            listeners.remove (list);
        }

        synchronized void fireStateChange () {
            SwingUtilities.invokeLater (new Runnable () {
                                            public void run () {
                                                ChangeEvent ev = new ChangeEvent (this);
                                                Set _listeners = new HashSet (listeners);
                                                Iterator it = _listeners.iterator ();
                                                while (it.hasNext ())
                                                    ((ChangeListener) it.next ()).stateChanged (ev);
                                            }
                                        });
        }

        public boolean isValid () {
            return true;
        }

        public Exception getParseException () {
            return null;
        }

        public File getManifestAsFile () {
            return null;
        }

        public File getModuleJar () {
            JarContent jc = getJarContent ();
            if (jc == null) return null;
            URL u = jc.getTargetResource ();
            if (u == null) return null;
            if (! u.getProtocol ().equals ("file")) return null;
            return new File (u.getFile ());
        }

    }

}
