/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.apisupport;

import java.awt.*;
import java.awt.event.*;
import java.io.IOException;
import javax.swing.*;

import org.openide.TopManager;
import org.openide.execution.ThreadExecutor;
import org.openide.util.HelpCtx;
import org.openide.util.SharedClassObject;
import org.openide.util.actions.Presenter;

public class PresenterTester extends ThreadExecutor {

    private static final long serialVersionUID =-7652741600072204927L;
    public HelpCtx getHelpCtx () {
        return new HelpCtx ("org.netbeans.modules.apisupport.testers.Presenter");
    }
    
    private Object instantiate  (Class clazz) throws Throwable {
        if (SharedClassObject.class.isAssignableFrom (clazz)) {
            return SharedClassObject.findObject (clazz, true);
        } else {
            return clazz.newInstance ();
        }
    }

    protected void checkClass (Class clazz) throws IOException {
        try {
            Object action = instantiate (clazz);
            if (! (action instanceof Presenter.Menu) && ! (action instanceof Presenter.Toolbar))
                throw new IOException ("Must implement menu or toolbar presenter (or both)!");
        } catch (ThreadDeath td) {
            throw td;
        } catch (Throwable t) {
            IOException ioe = new IOException ();
            TopManager.getDefault ().getErrorManager ().annotate (ioe, t);
            throw ioe;
        }
    }

    protected void executeClass (Class clazz, String[] args) {
        try {
            Object action = instantiate (clazz);
            System.err.println ("Will show " + clazz.getName () + " ...");
            JMenuItem menuitem = null;
            if (action instanceof Presenter.Menu)
                menuitem = ((Presenter.Menu) action).getMenuPresenter ();
            Component toolbar = null;
            if (action instanceof Presenter.Toolbar)
                toolbar = ((Presenter.Toolbar) action).getToolbarPresenter ();
            JFrame frame = new JFrame ("Testing " + clazz.getName ());
            if (menuitem != null) {
                JMenu menu = new JMenu ("Menu...");
                menu.add (menuitem);
                JMenuBar menubar = new JMenuBar ();
                menubar.add (menu);
                frame.setJMenuBar (menubar);
            }
            if (toolbar != null) {
                frame.getContentPane ().add (toolbar, BorderLayout.CENTER);
            }
            final Object pause = new Object ();
            frame.addWindowListener (new WindowAdapter () {
                                         public void windowClosing (WindowEvent ev) {
                                             synchronized (pause) {
                                                 pause.notify ();
                                             }
                                         }
                                     });
            frame.pack ();
            frame.show ();
            synchronized (pause) {
                pause.wait ();
            }
            System.err.println ("Presenter closed.");
        } catch (ThreadDeath td) {
            throw td;
        } catch (Throwable t) {
            t.printStackTrace ();
        }
    }

}
