/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.apisupport;

import java.io.IOException;
import java.lang.reflect.*;

import org.openide.*;
import org.openide.execution.ThreadExecutor;
import org.openide.nodes.*;
import org.openide.util.HelpCtx;
import org.openide.util.UserCancelException;
import org.openide.util.actions.*;
import org.openide.windows.InputOutput;

public class SystemActionTester extends ThreadExecutor {

    private static final long serialVersionUID =-1835218579791857931L;
    public HelpCtx getHelpCtx () {
        return new HelpCtx ("org.netbeans.modules.apisupport.testers.SystemAction");
    }

    protected void checkClass (Class clazz) throws IOException {
        try {
            SystemAction action = (SystemAction) clazz.newInstance ();
        } catch (ThreadDeath td) {
            throw td;
        } catch (Throwable t) {
            IOException ioe = new IOException ();
            TopManager.getDefault ().getErrorManager ().annotate (ioe, t);
            throw ioe;
        }
    }

    protected void executeClass (Class clazz, String[] args) {
        try {
            SystemAction action = SystemAction.get (clazz);
            System.err.println ("Will run " + clazz.getName () + " ...");
            if (action instanceof NodeAction) {
                NodeAction na = (NodeAction) action;
                Class search;
                Method m1 = null;
                for (search = clazz; ! search.equals (CallableSystemAction.class); search = search.getSuperclass ()) {
                    try {
                        m1 = search.getDeclaredMethod ("enable", new Class[] { Node[].class });
                        break;
                    } catch (NoSuchMethodException nsme) {
                    }
                }
                m1.setAccessible (true);
                try {
                    Method m2 = null;
                    for (search = clazz; ! search.equals (CallableSystemAction.class); search = search.getSuperclass ()) {
                        try {
                            m2 = search.getDeclaredMethod ("performAction", new Class[] { Node[].class });
                            break;
                        } catch (NoSuchMethodException nsme) {
                        }
                    }
                    m2.setAccessible (true);
                    try {
                        final NodeAction na_ = na;
                        final Method m1_ = m1;
                        Node[] roots = TopManager.getDefault ().getPlaces ().nodes ().roots ();
                        Node[] roots2 = new Node[roots.length + 3];
                        Places.Nodes pn = TopManager.getDefault ().getPlaces ().nodes ();
                        roots2[0] = pn.repository ().cloneNode ();
                        roots2[1] = pn.projectDesktop ().cloneNode ();
                        for (int i = 0; i < roots.length; i++) {
                            roots2[i + 2] = roots[i].cloneNode ();
                        }
                        roots2[roots.length + 2] = pn.environment ().cloneNode ();
                        Children.Array kids = new Children.Array ();
                        kids.add (roots2);
                        Node master = new AbstractNode (kids);
                        master.setName ("master");
                        master.setDisplayName ("Explorer");
                        try {
                            Node[] selection = TopManager.getDefault ().getNodeOperation ().select
                                ("Select some nodes for " + clazz.getName () + " ...", "Typical explorer nodes",
                                 master,
                                 new NodeAcceptor () {
                                         public boolean acceptNodes (Node[] nodes) {
                                             try {
                                                 return ((Boolean) m1_.invoke (na_, new Object[] { nodes })).booleanValue ();
                                             } catch (Exception e) {
                                                 e.printStackTrace ();
                                                 return false;
                                             }
                                         }
                                     });
                            m2.invoke (na, new Object[] { selection });
                        } catch (UserCancelException uce) {
                            System.err.println ("Cancelled.");
                            return;
                        }
                    } finally {
                        m2.setAccessible (false);
                    }
                } finally {
                    m1.setAccessible (false);
                }
            } else if (action instanceof CallableSystemAction) {
                ((CallableSystemAction) action).performAction ();
            } else {
                action.actionPerformed (null);
            }
            System.err.println ("Action finished.");
        } catch (ThreadDeath td) {
            throw td;
        } catch (Throwable t) {
            t.printStackTrace ();
        }
    }

}
