/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.apisupport.beanbrowser;

import java.beans.*;
import java.util.*;

import org.openide.*;
import org.openide.nodes.*;
import org.openide.util.HelpCtx;

/** Node to browse the desktop, other roots, and the TopManager. */
public class MainNode extends AbstractNode {

    public MainNode () {
        super (new MainChildren ());
        setName (getClass ().getName ()); // something to placate node handles
        setDisplayName ("Bean Browser");
        setIconBase ("/org/netbeans/modules/apisupport/resources/BeanBrowserIcon");
    }

    public HelpCtx getHelpCtx () {
        return new HelpCtx ("org.netbeans.modules.apisupport.beanbrowser");
    }

    // Key class: Node
    private static class MainChildren extends Children.Keys implements PropertyChangeListener {

        protected void addNotify () {
            refreshKeys ();
            TopManager.getDefault ().addPropertyChangeListener (this);
        }

        private void refreshKeys () {
            List l = new LinkedList ();
            Places.Nodes pn = TopManager.getDefault ().getPlaces ().nodes ();
            l.add (pn.projectDesktop ());
            l.addAll (Arrays.asList (pn.roots ()));
            try {
                Node n = new RefinedBeanNode (TopManager.getDefault ());
                n.setDisplayName ("TopManager");
                l.add (n);
            } catch (IntrospectionException e) {
                e.printStackTrace ();
            }
            setKeys (l);
        }

        protected void removeNotify () {
            TopManager.getDefault ().removePropertyChangeListener (this);
            setKeys (Collections.EMPTY_SET);
        }

        public void propertyChange (PropertyChangeEvent ev) {
            String prop = ev.getPropertyName ();
            if (prop == null || prop.equals (TopManager.PROP_PLACES)) {
                refreshKeys ();
            }
        }

        protected Node[] createNodes (Object key) {
            Node n = (Node) key;
            return new Node[] { Wrapper.make (n) };
        }

    }

}
