/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.apisupport.lite;

import java.io.File;
import java.io.IOException;

import org.openide.TopManager;
import org.openide.execution.*;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObject;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;

import org.netbeans.core.ModuleInstaller;

import org.netbeans.modules.jarpackager.JarDataObject;

/** Lightweight version of InstallModuleExecutor.
 * @author Jesse Glick
 */
public class LiteInstaller extends Executor {

   private static final long serialVersionUID = -5782560602441021826L;

   public HelpCtx getHelpCtx () {
        return new HelpCtx ("org.netbeans.modules.apisupport.lite"); // NOI18N
    }

    public ExecutorTask execute (final ExecInfo info) throws IOException {
        throw new IOException ("should not happen");
    }
    
    public ExecutorTask execute (DataObject obj) throws IOException {
        if (! (obj instanceof JarDataObject)) {
            IOException ioe = new IOException ("! instanceof JarDataObject"); // NOI18N
            TopManager.getDefault ().getErrorManager ().annotate (ioe, NbBundle.getMessage (LiteInstaller.class, "EXC_not_instanceof_JarDataObject"));
            throw ioe;
        }
        FileObject fo = ((JarDataObject) obj).findArchiveFile ();
        if (fo == null) {
            IOException ioe = new IOException ("archiveFile == null"); // NOI18N
            TopManager.getDefault ().getErrorManager ().annotate (ioe, NbBundle.getMessage (LiteInstaller.class, "EXC_no_archive"));
            throw ioe;
        }
        final File f = NbClassPath.toFile (fo);
        if (f == null) {
            IOException ioe = new IOException ("JAR not on local disk"); // NOI18N
            TopManager.getDefault ().getErrorManager ().annotate (ioe, NbBundle.getMessage (LiteInstaller.class, "EXC_not_local_jar"));
            throw ioe;
        }
        return TopManager.getDefault ().getExecutionEngine ().execute (NbBundle.getMessage (LiteInstaller.class, "MSG_installing_jar", f.getName ()), new Runnable () {
            public void run () {
                try {
                    ModuleInstaller.deployTestModule (f);
                } catch (IOException ioe) {
                    ioe.printStackTrace ();
                }
            }
        }, null);
    }

}
