/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.apisupport.lite;

import java.io.*;
import java.util.StringTokenizer;

import org.openide.*;
import org.openide.cookies.*;
import org.openide.filesystems.*;
import org.openide.loaders.*;
import org.openide.modules.*;
import org.openide.nodes.*;
import org.openide.src.*;

/** Data of panel for new module wizard
*
* @author Ales Kemr
*/
public class LoaderData extends WizardData {

    /** Holds value of property loaderName. */
    private String loaderName;

    /** Holds value of property extensions. */
    private Object[] extensions;

    SharedData shared;

    /** Holds value of property loaderCode. */
    private String loaderCode;

    /** Creates new WizardData */
    public LoaderData() {
        shared = SharedData.getShared();
    }

    /** Getter for property loaderName.
     * @return Value of property loaderName.
    */
    public String getLoaderName() {
        return loaderName;
    }

    /** Setter for property loaderName.
     * @param loaderName New value of property loaderName.
    */
    public void setLoaderName(String loaderName) {
        this.loaderName = loaderName;
    }

    /** Getter for property extensions.
     * @return Value of property extensions.
    */
    public Object[] getExtensions() {
        return extensions;
    }

    /** Setter for property extensions.
     * @param extensions New value of property extensions.
    */
    public void setExtensions(Object[] extensions) {
        this.extensions = extensions;
    }

    public void run() {
        try {
            createDO(getLoaderCode());
        } catch (IOException e) {
        }
    }

    void createDO(String name) throws IOException{
        FileObject fo = TopManager.getDefault ().getRepository ().getDefaultFileSystem ().findResource ("apisupport-lite-data/datasystems/Simple_loader.group");
        if (fo != null) {
            try {
                DataObject tryTemplate = DataObject.find (fo);
                tryTemplate.createFromTemplate(shared.getMainFolder(),name);
                shared.manageBundle();
                shared.replaceBundleEntry("LBL_loaderName", loaderName);
            } catch (IOException e) {
                TopManager.getDefault ().notifyException (e);
            }
            FileObject loader = shared.getMainFolder().getPrimaryFile().getFileObject(name + "DataLoader", "java");
            shared.setDataLoaderClass(loader.getPackageName('/'));
            modifyExtensions(
                loader,
                getExtensions()
            );
        }
    }

    void modifyExtensions(FileObject fo, Object[] extensions) {
        ClassElement clsElem = ClassElement.forName (fo.getPackageName('.'));
        if (clsElem != null) {
            MethodElement me = clsElem.getMethod(Identifier.create("initialize"),new Type[] {});

            StringBuffer body = new StringBuffer(me.getBody());
            int pos = body.toString().indexOf("extensions.addExtension");

            String mez = "";
            for(int i=pos; body.toString().charAt(i - 1) == ' '; i--)
                mez = mez + " ";

            int pos2 = body.toString().indexOf("setExtensions",pos);

            String replacement = "";
            for (int i=0;i<extensions.length;i++)
                replacement = replacement + "extensions.addExtension(\""
                    + (String)extensions[i] + "\");\n" + mez;

            body = body.replace(pos,pos2,replacement);
            try{
                me.setBody(body.toString());
                SaveCookie save = (SaveCookie) DataObject.find(fo).getCookie (SaveCookie.class);
                if (save != null) {
                    save.save ();
                }
            } catch (Exception e) {}
        }
    }

    /** Getter for property loaderCode.
     * @return Value of property loaderCode.
    */
    public String getLoaderCode() {
        return loaderCode;
    }

    /** Setter for property loaderCode.
     * @param loaderCode New value of property loaderCode.
    */
    public void setLoaderCode(String loaderCode) {
        this.loaderCode = loaderCode;
    }

}
