/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.apisupport.lite;

import java.util.Vector;
import javax.swing.event.ChangeListener;

import org.openide.WizardDescriptor;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;

/** Panel for new module wizard
*
* @author Ales Kemr
*/
public class LoaderPanel extends javax.swing.JPanel {

    private Wizard.Validator validator;

    public LoaderPanel(Wizard.Validator validator) {
        putClientProperty("WizardPanel_contentSelectedIndex", new Integer(3));
        setName(bundle.getString("LAB_Loader"));

        initComponents();
        setPreferredSize(new java.awt.Dimension(500, 400));

        extensionsLabel.setDisplayedMnemonic(bundle.getString("LAB_Extensions_Mnemonic").charAt(0));

        this.validator = validator;

        checkValid();
    }

    private void initComponents() {//GEN-BEGIN:initComponents
        descriptionTextArea = new javax.swing.JTextArea();
        extensionsLabel = new javax.swing.JLabel();
        extensionsTextField = new javax.swing.JTextField();
        jPanel1 = new javax.swing.JPanel();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        descriptionTextArea.setWrapStyleWord(true);
        descriptionTextArea.setLineWrap(true);
        descriptionTextArea.setEditable(false);
        descriptionTextArea.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/apisupport/lite/Bundle").getString("TXT_Loader"));
        descriptionTextArea.setBackground(extensionsLabel.getBackground());
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.weightx = 1.0;
        add(descriptionTextArea, gridBagConstraints1);
        
        extensionsLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/apisupport/lite/Bundle").getString("LAB_Extensions"));
        extensionsLabel.setLabelFor(extensionsTextField);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.insets = new java.awt.Insets(11, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(extensionsLabel, gridBagConstraints1);
        
        extensionsTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                extensionsTextFieldActionPerformed(evt);
            }
        });
        
        extensionsTextField.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                extensionsTextFieldFocusLost(evt);
            }
        });
        
        extensionsTextField.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                extensionsTextFieldKeyPressed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(11, 11, 0, 0);
        gridBagConstraints1.weightx = 1.0;
        add(extensionsTextField, gridBagConstraints1);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(jPanel1, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void extensionsTextFieldKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_extensionsTextFieldKeyPressed
        checkValid();
    }//GEN-LAST:event_extensionsTextFieldKeyPressed

    private void extensionsTextFieldFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_extensionsTextFieldFocusLost
        checkValid();
    }//GEN-LAST:event_extensionsTextFieldFocusLost

    private void extensionsTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_extensionsTextFieldActionPerformed
        checkValid();
    }//GEN-LAST:event_extensionsTextFieldActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextArea descriptionTextArea;
    private javax.swing.JLabel extensionsLabel;
    private javax.swing.JTextField extensionsTextField;
    private javax.swing.JPanel jPanel1;
    // End of variables declaration//GEN-END:variables

    private static final java.util.ResourceBundle bundle = NbBundle.getBundle( LoaderPanel.class );

    /** Getter for property extensions.
     * @return Value of property extensions.
    */
    public Object[] getExtensions() {
        String exts = extensionsTextField.getText();
        String one;
        int pos1,pos2;
        Vector v = new Vector();
        
        while ( exts.indexOf(' ') >= 0 || exts.indexOf(',') >= 0 ) {
            if (( exts.indexOf(' ') >= 0 ) &&
                    ( exts.indexOf(',') == -1 || exts.indexOf(',') > exts.indexOf(' ') ))
                pos1 = exts.indexOf(' ');                
            else
                pos1 = exts.indexOf(',');
            if (pos1 > 0) {
                v.add(exts.substring(0,pos1));
                exts = exts.substring(pos1);
            }
            while ( exts.indexOf(' ') == 0 || exts.indexOf(',') == 0 )            
                exts = exts.substring(1);
        }
        if ( exts.length() >= 0 )
            v.add(exts);

        return v.toArray();
    }
    
    private void checkValid() {
        if ( extensionsTextField.getText() == null || extensionsTextField.getText().length() == 0 )
            validator.setValid(false);
        else
            validator.setValid(true);
    }
}
