/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.apisupport.lite;

import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;
import javax.swing.event.ChangeListener;

import org.openide.WizardDescriptor;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.modules.ModuleDescription;

import org.netbeans.core.UpdateSupport;

/** Panel for new module wizard
*
* @author Ales Kemr
*/
public class ManifestPanel extends javax.swing.JPanel {

    private Wizard.Validator validator;

    public ManifestPanel(Wizard.Validator validator) {
        putClientProperty("WizardPanel_contentSelectedIndex", new Integer(5));
        setName(bundle.getString("LAB_Manifest"));
        initComponents();
        setPreferredSize(new java.awt.Dimension(500, 400));

        versionLabel.setDisplayedMnemonic(bundle.getString("LBL_Version_Mnemonic").charAt(0));
        descriptionLabel.setDisplayedMnemonic(bundle.getString("LBL_ModuleDescription_Mnemonic").charAt(0));
        groupLabel.setDisplayedMnemonic(bundle.getString("LBL_Category_Mnemonic").charAt(0));

        this.validator = validator;
        
        fillCategories();
    }

    private void initComponents() {//GEN-BEGIN:initComponents
        versionLabel = new javax.swing.JLabel();
        versionTextField = new javax.swing.JTextField();
        descriptionLabel = new javax.swing.JLabel();
        descriptionScrollPane = new javax.swing.JScrollPane();
        descriptionTextArea = new javax.swing.JTextArea();
        groupLabel = new javax.swing.JLabel();
        groupComboBox = new javax.swing.JComboBox();
        commentTextArea = new javax.swing.JTextArea();
        jPanel1 = new javax.swing.JPanel();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        versionLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/apisupport/lite/Bundle").getString("LBL_Version"));
        versionLabel.setLabelFor(versionTextField);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.insets = new java.awt.Insets(12, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(versionLabel, gridBagConstraints1);
        
        versionTextField.setText("0.1");
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(12, 11, 0, 0);
        gridBagConstraints1.weightx = 1.0;
        add(versionTextField, gridBagConstraints1);
        
        descriptionLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/apisupport/lite/Bundle").getString("LBL_ModuleDescription"));
        descriptionLabel.setLabelFor(descriptionTextArea);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.insets = new java.awt.Insets(11, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        add(descriptionLabel, gridBagConstraints1);
        
        descriptionScrollPane.setViewportView(descriptionTextArea);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(11, 11, 0, 0);
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(descriptionScrollPane, gridBagConstraints1);
        
        groupLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/apisupport/lite/Bundle").getString("LBL_Category"));
        groupLabel.setLabelFor(groupComboBox);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.insets = new java.awt.Insets(11, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(groupLabel, gridBagConstraints1);
        
        groupComboBox.setBackground(versionLabel.getBackground());
        groupComboBox.setEditable(true);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(11, 11, 0, 0);
        gridBagConstraints1.weightx = 1.0;
        add(groupComboBox, gridBagConstraints1);
        
        commentTextArea.setWrapStyleWord(true);
        commentTextArea.setLineWrap(true);
        commentTextArea.setEditable(false);
        commentTextArea.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/apisupport/lite/Bundle").getString("TXT_Manifest"));
        commentTextArea.setBackground(new java.awt.Color(204, 204, 204));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.weightx = 1.0;
        add(commentTextArea, gridBagConstraints1);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(jPanel1, gridBagConstraints1);
        
    }//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel versionLabel;
    private javax.swing.JTextField versionTextField;
    private javax.swing.JLabel descriptionLabel;
    private javax.swing.JScrollPane descriptionScrollPane;
    private javax.swing.JTextArea descriptionTextArea;
    private javax.swing.JLabel groupLabel;
    private javax.swing.JComboBox groupComboBox;
    private javax.swing.JTextArea commentTextArea;
    private javax.swing.JPanel jPanel1;
    // End of variables declaration//GEN-END:variables

    private static final java.util.ResourceBundle bundle = NbBundle.getBundle( ManifestPanel.class );
    
    /** Getter for property version.
     * @return Value of property version.
    */
    public String getVersion() {
        return versionTextField.getText();        
    }
    
    /** Getter for property moduleDesc.
     * @return Value of property moduleDesc.
    */
    public String getModuleDesc() {
        return descriptionTextArea.getText();
    }
    
    /** Getter for property category.
     * @return Value of property category.
    */
    public String getCategory() {
        return (String)groupComboBox.getSelectedItem();
    }

    private void fillCategories() {
        ModuleDescription[] modules = UpdateSupport.getModuleDescriptions();
        String cat;
        Set set = new HashSet();
        for (int i=0; i<modules.length; i++) {
            cat = modules[i].getDisplayCategory();
            if (cat != null && cat.length() > 0 && !set.contains(cat))
                set.add(cat);
        }
        groupComboBox.addItem("");
        Iterator it = set.iterator();
        while (it.hasNext())
            groupComboBox.addItem((String)it.next());        
    }
}
