/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.apisupport.lite;

import java.io.*;
import java.util.List;
import java.util.StringTokenizer;
import javax.swing.KeyStroke;
import java.lang.reflect.*;

import org.w3c.dom.*;

import org.openide.*;
import org.openide.cookies.*;
import org.openide.filesystems.*;
import org.openide.loaders.*;
import org.openide.modules.*;
import org.openide.xml.*;

/** Data of panel for new module wizard
*
* @author Ales Kemr
*/
public class MenuData extends WizardData {

    /** Holds value of property menuName. */
    private String menuName;

    /** Holds value of property menuCode. */
    private String menuCode;

    private String actionClass;

    SharedData shared;

    /** Holds value of property otherMenuClass. */
    private String otherMenuClass;

    /** Holds value of property otherToolbarClass. */
    private String otherToolbarClass;

    /** Holds value of property beforeOtherMenu. */
    private boolean beforeOtherMenu;

    /** Holds value of property beforeOtherToolbar. */
    private boolean beforeOtherToolbar;

    /** Holds value of property separatorBefore. */
    private boolean separatorBefore;

    /** Holds value of property separatorAfter. */
    private boolean separatorAfter;

    /** Holds value of property shortcut. */
    private String shortcut;

    /** Holds value of property menuHierarchy. */
    private List menuHierarchy;

    /** Holds value of property toolHierarchy. */
    private List toolHierarchy;

    /** Holds value of property toolbar. */
    private boolean toolbar;

    /** Holds value of property beforeMenuClass. */
    private String beforeMenuClass;

    /** Holds value of property afterMenuClass. */
    private String afterMenuClass;

    /** Holds value of property iconFile. */
    private File iconFile;

    /** Creates new WizardData */
    public MenuData() {
        shared = SharedData.getShared();
    }


    public void run() {
        try {
            createAction(getMenuCode());
        } catch (IOException e) {            
        }
        xml();
        modifyIcon();
    }

    void createAction(String name) throws IOException{
        FileObject fo = TopManager.getDefault ().getRepository ().getDefaultFileSystem ().findResource ("apisupport-lite-data/actions/Callable_action.group");
        if (fo != null) {
            try {
                DataObject tryTemplate = DataObject.find (fo);
                tryTemplate.createFromTemplate(shared.getMainFolder(),name);
                shared.manageBundle();
                shared.replaceBundleEntry("LBL_Action", menuName);
            } catch (IOException e) {
                TopManager.getDefault ().notifyException (e);
            }
            FileObject action = shared.getMainFolder().getPrimaryFile().getFileObject(name + "Action", "java");
            actionClass = action.getPackageName('-');
        }
    }

    void modifyIcon() {
        if (iconFile == null)
            return;
        FileLock lock = null;
        FileObject fo = shared.getMainFolder().getPrimaryFile().getFileObject(SharedData.CLASS_PREFIX + "ActionIcon","gif");
        if (fo == null)
            return;
        try {
            lock=fo.lock();
            copyStreams(
                    new FileInputStream(iconFile),
                    fo.getOutputStream(lock)
                );
        } catch (java.io.IOException e) {
        } finally {
            lock.releaseLock();
        }
    }

    private void copyStreams( InputStream src, OutputStream dest ) throws java.io.IOException {

        BufferedInputStream bsrc = new BufferedInputStream( src );
        BufferedOutputStream bdest = new BufferedOutputStream( dest );

        int c;

        try {
            while( ( c = bsrc.read() ) != -1 )
                bdest.write( c );
        }
        finally {
            bsrc.close();
            bdest.close();
        }
    }

    void xml() {

        FileLock lock = null;
        FileObject fo = null;
        try {
            fo = shared.getMainFolder().getPrimaryFile().createData("mf-layer","xml");
            shared.setLayerClass(fo.getPackageName('/'));
            lock=fo.lock();
        } catch (IOException e) {
            e.printStackTrace();
        }

        Document document = XMLUtil.createDocument("filesystem", null, "-//NetBeans//DTD Filesystem 1.0//EN",
                "http://www.netbeans.org/dtds/filesystem-1_0.dtd");
        
        Element root = document.getDocumentElement();
        
        Element folder = null;

        Element file = null;

        Element attr = null;

        List list = null;

        Node last = null;

        if ( menuCode != null ) {

            last = root;
            list = getMenuHierarchy();
            for (int i=0; i<list.size(); i++) {
                folder = document.createElement("folder");
                folder.setAttribute("name",(String)list.get(i));
                last.appendChild( folder );
                last = folder;
            }

            file = document.createElement("file");

            folder.appendChild( file );

            file.setAttribute("name", actionClass + ".instance");

            attr = document.createElement("attr");
            if ( beforeOtherMenu ) {
                if ( separatorBefore || separatorAfter) {
                    if ( separatorBefore ) {
                        attr = document.createElement("attr");
                        attr.setAttribute("name", makeAttr(beforeMenuClass,mySeparator(true)));
                        attr.setAttribute("boolvalue", "true");
                        folder.appendChild( attr );
                        attr = document.createElement("attr");
                        attr.setAttribute("name", makeAttr(mySeparator(true), actionClass));
                        attr.setAttribute("boolvalue", "true");
                        folder.appendChild( attr );
                        file = document.createElement("file");
                        folder.appendChild( file );
                        file.setAttribute("name", mySeparator(true) + ".instance");
                    } else {
                        attr = document.createElement("attr");
                        attr.setAttribute("name", makeAttr(mySeparator(false),otherMenuClass));
                        attr.setAttribute("boolvalue", "true");
                        folder.appendChild( attr );
                        attr = document.createElement("attr");
                        attr.setAttribute("name", makeAttr(actionClass,mySeparator(true)));
                        attr.setAttribute("boolvalue", "true");
                        folder.appendChild( attr );
                        file = document.createElement("file");
                        folder.appendChild( file );
                        file.setAttribute("name", mySeparator(false) + ".instance");
                    }
                } else {
                    if ( beforeMenuClass != null ) {                        
                        attr.setAttribute("name", makeAttr(beforeMenuClass,actionClass));
                        attr.setAttribute("boolvalue", "true");
                        folder.appendChild( attr );
                        attr = document.createElement("attr");
                    }
                    attr.setAttribute("name", makeAttr(actionClass,otherMenuClass));
                    attr.setAttribute("boolvalue", "true");
                    folder.appendChild( attr );
                }                
            } else {
                if ( separatorBefore || separatorAfter) {                
                    if ( separatorBefore ) {
                        attr = document.createElement("attr");
                        attr.setAttribute("name", makeAttr(otherMenuClass,mySeparator(true)));
                        attr.setAttribute("boolvalue", "true");
                        folder.appendChild( attr );
                        attr = document.createElement("attr");
                        attr.setAttribute("name", makeAttr(actionClass,mySeparator(true)));
                        attr.setAttribute("boolvalue", "true");
                        folder.appendChild( attr );
                        file = document.createElement("file");
                        folder.appendChild( file );
                        file.setAttribute("name", mySeparator(true) + ".instance");
                    } else {
                        attr = document.createElement("attr");
                        attr.setAttribute("name", makeAttr(mySeparator(false),afterMenuClass));
                        attr.setAttribute("boolvalue", "true");
                        folder.appendChild( attr );
                        attr = document.createElement("attr");
                        attr.setAttribute("name", makeAttr(mySeparator(true), actionClass));
                        attr.setAttribute("boolvalue", "true");
                        folder.appendChild( attr );
                        file = document.createElement("file");
                        folder.appendChild( file );
                        file.setAttribute("name", mySeparator(false) + ".instance");
                    }
                } else {
                    attr.setAttribute("name", makeAttr(otherMenuClass,actionClass));
                    attr.setAttribute("boolvalue", "true");
                    folder.appendChild( attr );
                    if ( afterMenuClass != null ) {
                        attr = document.createElement("attr");
                        attr.setAttribute("name", makeAttr(actionClass,afterMenuClass));
                        attr.setAttribute("boolvalue", "true");
                        folder.appendChild( attr );
                    }
                }
            }

        }

        if ( isToolbar() ) {

            last = root;
            list = getToolHierarchy();
            for (int i=0; i<list.size(); i++) {
                folder = document.createElement("folder");
                folder.setAttribute("name",(String)list.get(i));
                last.appendChild( folder );
                last = folder;
            }

            file = document.createElement("file");

            folder.appendChild( file );

            file.setAttribute("name", actionClass + ".instance");
            attr = document.createElement("attr");
            if ( beforeOtherToolbar ) {
                attr.setAttribute("name", makeAttr(actionClass,otherToolbarClass));
                attr.setAttribute("boolvalue", "true");
                folder.appendChild( attr );
            } else {
                attr.setAttribute("name", makeAttr(otherToolbarClass,actionClass));
                attr.setAttribute("boolvalue", "true");
                folder.appendChild( attr );
            }
        }

        if ( getShortcut() != null && getShortcut().length() > 0 ) {

            folder = document.createElement("folder");
            folder.setAttribute("name","Shortcuts");
            root.appendChild( folder );

            file = document.createElement("file");

            folder.appendChild( file );

            file.setAttribute("name", getShortCutXML() + "[" + actionClass + "].instance");
        }

        document.getDocumentElement().normalize();

        ClassLoader cl = org.openide.TopManager.getDefault().systemClassLoader();
        try {
            Class serka = 
            Class.forName("org.apache.xml.serialize.XMLSerializer", true, cl);   //NOI18N

            Class forka =
            Class.forName("org.apache.xml.serialize.OutputFormat", true, cl);    //NOI18N

            Object serin = serka.newInstance();                
            Object forin = forka.newInstance();

            Method setmet = forka.getMethod("setIndenting", new Class[] { Boolean.TYPE } ); //NOI18N                
            setmet.invoke(forin, new Object[] { new Boolean(true)});

            setmet = serka.getMethod("setOutputFormat", new Class[] {forka}); //NOI18N                
            setmet.invoke(serin, new Object[] { forin });                

            setmet = serka.getMethod("setOutputByteStream", new Class[] { OutputStream.class }); //NOI18N                
            setmet.invoke(serin, new Object[] { fo.getOutputStream(lock) });                

            setmet = serka.getMethod("serialize", new Class[] { Document.class }); //NOI18N                
            setmet.invoke(serin, new Object[] { document });                
            
        } catch (Exception e){
            e.printStackTrace();
        } finally {
            lock.releaseLock();
        }

    }

    String mySeparator( boolean before ) {
        if ( before )
            return (menuCode + "-sep1[javax-swing-JSeparator]");
        else
            return (menuCode + "-sep2[javax-swing-JSeparator]");
    }

    String makeAttr(String first, String second) {
        return (first + ".instance" + "/" + second + ".instance");
    }

    /** Getter for property menuName.
     * @return Value of property menuName.
     */
    public String getMenuName() {
        return menuName;
    }

    /** Setter for property menuName.
     * @param menuName New value of property menuName.
     */
    public void setMenuName(String menuName) {
        this.menuName = menuName;
    }

    /** Getter for property menuCode.
     * @return Value of property menuCode.
     */
    public String getMenuCode() {
        return menuCode;
    }

    /** Setter for property menuCode.
     * @param menuCode New value of property menuCode.
     */
    public void setMenuCode(String menuCode) {
        this.menuCode = menuCode;
    }

    /** Getter for property otherMenuClass.
     * @return Value of property otherMenuClass.
     */
    public String getOtherMenuClass() {
        return otherMenuClass;
    }

    /** Setter for property otherMenuClass.
     * @param otherMenuClass New value of property otherMenuClass.
     */
    public void setOtherMenuClass(String otherMenuClass) {
        this.otherMenuClass = otherMenuClass;
    }

    /** Getter for property otherToolbarClass.
     * @return Value of property otherToolbarClass.
     */
    public String getOtherToolbarClass() {
        return otherToolbarClass;
    }

    /** Setter for property otherToolbarClass.
     * @param otherToolbarClass New value of property otherToolbarClass.
     */
    public void setOtherToolbarClass(String otherToolbarClass) {
        this.otherToolbarClass = otherToolbarClass;
    }

    /** Getter for property beforeOtherMenu.
     * @return Value of property beforeOtherMenu.
     */
    public boolean isBeforeOtherMenu() {
        return beforeOtherMenu;
    }

    /** Setter for property beforeOtherMenu.
     * @param beforeOtherMenu New value of property beforeOtherMenu.
     */
    public void setBeforeOtherMenu(boolean beforeOtherMenu) {
        this.beforeOtherMenu = beforeOtherMenu;
    }

    /** Getter for property beforeOtherToolbar.
     * @return Value of property beforeOtherToolbar.
     */
    public boolean isBeforeOtherToolbar() {
        return beforeOtherToolbar;
    }

    /** Setter for property beforeOtherToolbar.
     * @param beforeOtherToolbar New value of property beforeOtherToolbar.
     */
    public void setBeforeOtherToolbar(boolean beforeOtherToolbar) {
        this.beforeOtherToolbar = beforeOtherToolbar;
    }

    /** Getter for property separatorBefore.
     * @return Value of property separatorBefore.
     */
    public boolean isSeparatorBefore() {
        return separatorBefore;
    }

    /** Setter for property separatorBefore.
     * @param separatorBefore New value of property separatorBefore.
     */
    public void setSeparatorBefore(boolean separatorBefore) {
        this.separatorBefore = separatorBefore;
    }

    /** Getter for property separatorAfter.
     * @return Value of property separatorAfter.
     */
    public boolean isSeparatorAfter() {
        return separatorAfter;
    }

    /** Setter for property separatorAfter.
     * @param separatorAfter New value of property separatorAfter.
     */
    public void setSeparatorAfter(boolean separatorAfter) {
        this.separatorAfter = separatorAfter;
    }

    /** Getter for property shortcut.
     * @return Value of property shortcut.
     */
    public String getShortcut() {
        return shortcut;
    }

    private String getShortCutXML() {
        String ret="";
        while (shortcut.startsWith("Ctrl+") ||
                shortcut.startsWith("Alt+") ||
                shortcut.startsWith("Shift+")) {
            ret = ret + shortcut.charAt(0);
            shortcut = shortcut.substring(shortcut.indexOf('+')+1);
        }
        return ret + "-" + shortcut.toUpperCase();
    }

    /** Setter for property shortcut.
     * @param shortcut New value of property shortcut.
     */
    public void setShortcut(String shortcut) {
        this.shortcut = shortcut;
    }

    /** Getter for property menuHierarchy.
     * @return Value of property menuHierarchy.
     */
    public List getMenuHierarchy() {
        return menuHierarchy;
    }

    /** Setter for property menuHierarchy.
     * @param menuHierarchy New value of property menuHierarchy.
     */
    public void setMenuHierarchy(List menuHierarchy) {
        this.menuHierarchy = menuHierarchy;
    }

    /** Getter for property toolHierarchy.
     * @return Value of property toolHierarchy.
     */
    public List getToolHierarchy() {
        return toolHierarchy;
    }

    /** Setter for property toolHierarchy.
     * @param toolHierarchy New value of property toolHierarchy.
     */
    public void setToolHierarchy(List toolHierarchy) {
        this.toolHierarchy = toolHierarchy;
    }

    /** Getter for property toolbar.
     * @return Value of property toolbar.
     */
    public boolean isToolbar() {
        return toolbar;
    }

    /** Setter for property toolbar.
     * @param toolbar New value of property toolbar.
     */
    public void setToolbar(boolean toolbar) {
        this.toolbar = toolbar;
    }

    /** Getter for property beforeMenuClass.
     * @return Value of property beforeMenuClass.
     */
    public String getBeforeMenuClass() {
        return beforeMenuClass;
    }

    /** Setter for property beforeMenuClass.
     * @param beforeMenuClass New value of property beforeMenuClass.
     */
    public void setBeforeMenuClass(String beforeMenuClass) {
        this.beforeMenuClass = beforeMenuClass;
    }

    /** Getter for property afterMenuClass.
     * @return Value of property afterMenuClass.
     */
    public String getAfterMenuClass() {
        return afterMenuClass;
    }

    /** Setter for property afterMenuClass.
     * @param afterMenuClass New value of property afterMenuClass.
     */
    public void setAfterMenuClass(String afterMenuClass) {
        this.afterMenuClass = afterMenuClass;
    }

    /** Getter for property iconFile.
     * @return Value of property iconFile.
     */
    public File getIconFile() {
        return iconFile;
    }

    /** Setter for property iconFile.
     * @param iconFile New value of property iconFile.
     */
    public void setIconFile(File iconFile) {
        this.iconFile = iconFile;
    }

}
