/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.apisupport.lite;

import java.awt.event.KeyEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JFileChooser;
import javax.swing.event.ChangeListener;
import javax.swing.filechooser.FileFilter;

import org.openide.TopManager;
import org.openide.WizardDescriptor;
import org.openide.cookies.*;
import org.openide.explorer.ExplorerManager;
import org.openide.explorer.ExplorerPanel;
import org.openide.explorer.view.BeanTreeView;
import org.openide.loaders.*;
import org.openide.nodes.Node;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;

/** Panel for new module wizard
*
* @author Ales Kemr
*/
public class MenuPanel extends javax.swing.JPanel {

    private Wizard.Validator validator;

    ExplorerPanel epm = new ExplorerPanel();
    ExplorerManager emm = epm.getExplorerManager();

    ExplorerPanel ept = new ExplorerPanel();
    ExplorerManager emt = ept.getExplorerManager();

    /** Holds value of property otherMenuClass. */
    private String otherMenuClass;

    /** Holds value of property otherToolbarClass. */
    private String otherToolbarClass;

    /** Holds value of property menuHierarchy. */
    private List menuHierarchy = new ArrayList();

    /** Holds value of property toolHierarchy. */
    private List toolHierarchy = new ArrayList();

    /** Holds value of property beforeMenuClass. */
    private String beforeMenuClass;

    /** Holds value of property afterMenuClass. */
    private String afterMenuClass;

    private File iconFile;

    private int currentKeyCode;
    private int currentModifiers;

    private boolean menuValid = false;
    private boolean toolValid = false;
    
    private BeanTreeView btvt;

    private PropertyChangeListener menuListener;
    private PropertyChangeListener toolListener;
    
    public MenuPanel(Wizard.Validator validator) {
        putClientProperty("WizardPanel_contentSelectedIndex", new Integer(4));
        setName(bundle.getString("LAB_Menu"));

        initComponents();
        setPreferredSize(new java.awt.Dimension(500, 400));

        nameLabel.setDisplayedMnemonic(bundle.getString("LBL_ItemName_Mnemonic").charAt(0));
        menuItemLabel.setDisplayedMnemonic(bundle.getString("LBL_MenuPlace_Mnemonic").charAt(0));
        toolbarItemLabel.setDisplayedMnemonic(bundle.getString("LBL_ToolPlace_Mnemonic").charAt(0));
        iconLabel.setDisplayedMnemonic(bundle.getString("LBL_Icon_Mnemonic").charAt(0));
        shortcutLabel.setDisplayedMnemonic(bundle.getString("LBL_Shortcut_Mnemonic").charAt(0));

        this.validator = validator;

        BeanTreeView btvm;
        btvm = new BeanTreeView ();
        btvm.setPopupAllowed( false );
        btvm.setDefaultActionAllowed( true );
        epm.setLayout (new java.awt.BorderLayout());
        epm.add ( java.awt.BorderLayout.CENTER, btvm );

        Node nm = TopManager.getDefault().getPlaces().folders ().menus().getNodeDelegate();

        emm.setRootContext(nm);
        emm.setExploredContext(nm);

        menuListener = new PropertyChangeListener() {
                public void propertyChange( PropertyChangeEvent evt ) {
                    if ( evt.getPropertyName() == ExplorerManager.PROP_SELECTED_NODES ) {
                        menuSelectionChange();
                    }
                }
            };        

        menuItemScrollPane.setViewportView(epm);
        menuItemLabel.setLabelFor(btvm);

        btvt = new BeanTreeView ();
        btvt.setPopupAllowed( false );
        btvt.setDefaultActionAllowed( true );
        ept.setLayout (new java.awt.BorderLayout());
        ept.add ( java.awt.BorderLayout.CENTER, btvt );

        Node nt = TopManager.getDefault().getPlaces().folders ().toolbars().getNodeDelegate();

        emt.setRootContext(nt);
        emt.setExploredContext(nt);

        toolListener = new PropertyChangeListener() {
                public void propertyChange( PropertyChangeEvent evt ) {
                    if ( evt.getPropertyName() == ExplorerManager.PROP_SELECTED_NODES ) {
                        toolbarSelectionChange();
                    }
                }
            };        

        toolbarItemScrollPane.setViewportView(ept);
        toolbarItemLabel.setLabelFor(btvt);

        shortcutTextField.addKeyListener (new java.awt.event.KeyAdapter () {
                                              public void keyPressed (java.awt.event.KeyEvent evt) {
                                                  shortcutFieldKeyPressed (evt);
                                              }
                                              public void keyReleased (java.awt.event.KeyEvent evt) {
                                                  shortcutFieldKeyReleased (evt);
                                              }
                                              public void keyTyped (java.awt.event.KeyEvent evt) {
                                                  shortcutFieldKeyTyped (evt);
                                              }
                                          }
                                         );
        checkValid();
        btvt.setEnabled(false);
        beforeToolbarItemCheckBox.setEnabled(false);
    }

    private void menuSelectionChange() {

        Node[] no = emm.getSelectedNodes();
        if (no != null && no.length > 0) {
            menuValid = true;
            otherMenuClass = getInstanceName(no[0]);
            Node nn = no[0].getParentNode();
            int i=0;
            menuHierarchy.clear();
            while ( nn != null && i < 10) {
                menuHierarchy.add(0,nn.getName());
                nn = nn.getParentNode();
                i++;
            }
            if ( i < 2 )
                menuValid = false;
            else {
                Node[] children = no[0].getParentNode().getChildren().getNodes();
                Node bn = null;
                Node an = null;
                for (int j=0;j<children.length;j++) {
                    if (children[j] == no[0]) {
                        if (j > 0)
                            bn = children[j-1];
                        if (j < children.length - 1)
                            an = children[j+1];
                        break;
                    }
                }
                if (bn != null)
                    beforeMenuClass = getInstanceName(bn);
                else
                    beforeMenuClass = null;
                if (an != null)
                    afterMenuClass = getInstanceName(an);
                else
                    afterMenuClass = null;
            }
        } else
            menuValid = false;
        checkValid();
    }

    private void toolbarSelectionChange() {

        Node[] no = emt.getSelectedNodes();
        if (no != null && no.length > 0) {
            toolValid = true;
            otherToolbarClass = getInstanceName(no[0]);
            Node nn = no[0].getParentNode();
            int i=0;
            toolHierarchy.clear();
            while ( nn != null && i++ < 10) {
                toolHierarchy.add(0,nn.getName());
                nn = nn.getParentNode();
            }
        } else
            toolValid = false;
        checkValid();
    }

    void addListeners() {
        emm.addPropertyChangeListener( menuListener );
        emt.addPropertyChangeListener( toolListener );
    }
    
    void removeListeners() {
        emm.removePropertyChangeListener( menuListener );
        emt.removePropertyChangeListener( toolListener );
    }
    
    private String getInstanceName(Node n) {
        String ret = null;
        InstanceCookie ic = (InstanceCookie) n.getCookie (InstanceCookie.class);
        if (ic != null) {
            ret = ic.instanceName().replace('.','-');
            if ( ret.endsWith("JSeparator") )
                ret = "[" + n.getName() + "]" + ret;
        }
        return ret;
    }

    private void shortcutFieldKeyReleased (java.awt.event.KeyEvent evt) {
        evt.consume();
        if ((currentKeyCode == KeyEvent.VK_ALT) || (currentKeyCode == KeyEvent.VK_ALT_GRAPH) || (currentKeyCode == KeyEvent.VK_CONTROL) || (currentKeyCode == KeyEvent.VK_SHIFT)) {
            // Not finished entering key
            shortcutTextField.setText(""); // NOI18N
        }
    }

    private void shortcutFieldKeyTyped (java.awt.event.KeyEvent evt) {
        evt.consume();
    }

    private void shortcutFieldKeyPressed (java.awt.event.KeyEvent evt) {
        evt.consume();
        currentKeyCode = evt.getKeyCode();
        currentModifiers = evt.getModifiers();
        shortcutTextField.setText(getKeyText (currentKeyCode, currentModifiers));
    }

    private String getKeyText (int keyCode, int modifiers) {
        String modifText = java.awt.event.KeyEvent.getKeyModifiersText(modifiers);
        if ("".equals (modifText)) return java.awt.event.KeyEvent.getKeyText(keyCode); // NOI18N
        else {
            if ((keyCode == KeyEvent.VK_ALT) || (keyCode == KeyEvent.VK_ALT_GRAPH) || (keyCode == KeyEvent.VK_CONTROL) || (keyCode == KeyEvent.VK_SHIFT)) {
                return modifText + "+"; // in this case the keyCode text is also among the modifiers // NOI18N
            } else {
                return modifText + "+" + java.awt.event.KeyEvent.getKeyText(keyCode); // NOI18N
            }
        }
    }


    private void initComponents() {//GEN-BEGIN:initComponents
        createToolbarButtonGroup = new javax.swing.ButtonGroup();
        descriptionTextArea = new javax.swing.JTextArea();
        nameLabel = new javax.swing.JLabel();
        nameTextField = new javax.swing.JTextField();
        menuItemLabel = new javax.swing.JLabel();
        menuItemScrollPane = new javax.swing.JScrollPane();
        menuItemPanel = new javax.swing.JPanel();
        beforeMenuItemCheckBox = new javax.swing.JCheckBox();
        separatorBeforeMenuItemCheckBox = new javax.swing.JCheckBox();
        separatorAfterMenuItemCheckBox = new javax.swing.JCheckBox();
        createToolbarLabel = new javax.swing.JLabel();
        createToolbarPanel = new javax.swing.JPanel();
        yesCreateToolbarRadioButton = new javax.swing.JRadioButton();
        noCreateToolbarRadioButton = new javax.swing.JRadioButton();
        toolbarItemLabel = new javax.swing.JLabel();
        toolbarItemScrollPane = new javax.swing.JScrollPane();
        toolbarItemPanel = new javax.swing.JPanel();
        beforeToolbarItemCheckBox = new javax.swing.JCheckBox();
        iconLabel = new javax.swing.JLabel();
        iconTextField = new javax.swing.JTextField();
        iconButton = new javax.swing.JButton();
        shortcutLabel = new javax.swing.JLabel();
        shortcutTextField = new javax.swing.JTextField();
        
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        setPreferredSize(new java.awt.Dimension(500, 400));
        descriptionTextArea.setWrapStyleWord(true);
        descriptionTextArea.setLineWrap(true);
        descriptionTextArea.setEditable(false);
        descriptionTextArea.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/apisupport/lite/Bundle").getString("DESC_Menu"));
        descriptionTextArea.setBackground(menuItemLabel.getBackground());
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.gridwidth = 3;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.weightx = 1.0;
        add(descriptionTextArea, gridBagConstraints1);
        
        nameLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/apisupport/lite/Bundle").getString("LBL_ItemName"));
        nameLabel.setLabelFor(nameTextField);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.insets = new java.awt.Insets(11, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(nameLabel, gridBagConstraints1);
        
        nameTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                nameTextFieldActionPerformed(evt);
            }
        });
        
        nameTextField.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                nameTextFieldFocusLost(evt);
            }
        });
        
        nameTextField.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                nameTextFieldKeyPressed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(11, 11, 0, 0);
        gridBagConstraints1.weightx = 1.0;
        add(nameTextField, gridBagConstraints1);
        
        menuItemLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/apisupport/lite/Bundle").getString("LBL_MenuPlace"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.insets = new java.awt.Insets(11, 12, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        add(menuItemLabel, gridBagConstraints1);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(11, 11, 0, 0);
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(menuItemScrollPane, gridBagConstraints1);
        
        menuItemPanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints2;
        
        beforeMenuItemCheckBox.setMnemonic(bundle.getString("CBX_BeforeMenu_Mnemonic").charAt(0));
        beforeMenuItemCheckBox.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/apisupport/lite/Bundle").getString("CBX_BeforeMenu"));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        menuItemPanel.add(beforeMenuItemCheckBox, gridBagConstraints2);
        
        separatorBeforeMenuItemCheckBox.setMnemonic(bundle.getString("CBX_SepBefore_Mnemonic").charAt(0));
        separatorBeforeMenuItemCheckBox.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/apisupport/lite/Bundle").getString("CBX_SepBefore"));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 0;
        gridBagConstraints2.gridy = 1;
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        menuItemPanel.add(separatorBeforeMenuItemCheckBox, gridBagConstraints2);
        
        separatorAfterMenuItemCheckBox.setMnemonic(bundle.getString("CBX_SepAfter_Mnemonic").charAt(0));
        separatorAfterMenuItemCheckBox.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/apisupport/lite/Bundle").getString("CBX_SepAfter"));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 0;
        gridBagConstraints2.gridy = 2;
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        menuItemPanel.add(separatorAfterMenuItemCheckBox, gridBagConstraints2);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 2;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(11, 11, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTH;
        add(menuItemPanel, gridBagConstraints1);
        
        createToolbarLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/apisupport/lite/Bundle").getString("CBX_Tool"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.insets = new java.awt.Insets(11, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(createToolbarLabel, gridBagConstraints1);
        
        yesCreateToolbarRadioButton.setMnemonic(bundle.getString("LBL_Tool_Yes_Mnemonic").charAt(0));
        yesCreateToolbarRadioButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/apisupport/lite/Bundle").getString("LBL_Tool_Yes"));
        createToolbarButtonGroup.add(yesCreateToolbarRadioButton);
        yesCreateToolbarRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                yesCreateToolbarRadioButtonActionPerformed(evt);
            }
        });
        
        createToolbarPanel.add(yesCreateToolbarRadioButton);
        
        noCreateToolbarRadioButton.setMnemonic(bundle.getString("LBL_Tool_No_Mnemonic").charAt(0));
        noCreateToolbarRadioButton.setSelected(true);
        noCreateToolbarRadioButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/apisupport/lite/Bundle").getString("LBL_Tool_No"));
        createToolbarButtonGroup.add(noCreateToolbarRadioButton);
        noCreateToolbarRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                noCreateToolbarRadioButtonActionPerformed(evt);
            }
        });
        
        createToolbarPanel.add(noCreateToolbarRadioButton);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.insets = new java.awt.Insets(11, 11, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints1.weightx = 1.0;
        add(createToolbarPanel, gridBagConstraints1);
        
        toolbarItemLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/apisupport/lite/Bundle").getString("LBL_ToolPlace"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.insets = new java.awt.Insets(11, 12, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        add(toolbarItemLabel, gridBagConstraints1);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(11, 11, 0, 0);
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(toolbarItemScrollPane, gridBagConstraints1);
        
        toolbarItemPanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints3;
        
        beforeToolbarItemCheckBox.setMnemonic(bundle.getString("CBX_BeforeToolbar_Mnemonic").charAt(0));
        beforeToolbarItemCheckBox.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/apisupport/lite/Bundle").getString("CBX_BeforeToolbar"));
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.anchor = java.awt.GridBagConstraints.WEST;
        toolbarItemPanel.add(beforeToolbarItemCheckBox, gridBagConstraints3);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 2;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(11, 11, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTH;
        add(toolbarItemPanel, gridBagConstraints1);
        
        iconLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/apisupport/lite/Bundle").getString("LBL_Icon"));
        iconLabel.setLabelFor(iconTextField);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 5;
        gridBagConstraints1.insets = new java.awt.Insets(11, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(iconLabel, gridBagConstraints1);
        
        iconTextField.setEditable(false);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 5;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(11, 11, 0, 0);
        gridBagConstraints1.weightx = 1.0;
        add(iconTextField, gridBagConstraints1);
        
        iconButton.setText("...");
        iconButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                iconButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 2;
        gridBagConstraints1.gridy = 5;
        gridBagConstraints1.insets = new java.awt.Insets(11, 11, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(iconButton, gridBagConstraints1);
        
        shortcutLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/apisupport/lite/Bundle").getString("LBL_Shortcut"));
        shortcutLabel.setLabelFor(shortcutTextField);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 6;
        gridBagConstraints1.insets = new java.awt.Insets(11, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(shortcutLabel, gridBagConstraints1);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 6;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(11, 11, 0, 0);
        gridBagConstraints1.weightx = 1.0;
        add(shortcutTextField, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void nameTextFieldKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_nameTextFieldKeyPressed
        checkValid();
    }//GEN-LAST:event_nameTextFieldKeyPressed

    private void noCreateToolbarRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_noCreateToolbarRadioButtonActionPerformed
        checkValid();
        btvt.setEnabled(false);
        beforeToolbarItemCheckBox.setEnabled(false);
    }//GEN-LAST:event_noCreateToolbarRadioButtonActionPerformed

    private void yesCreateToolbarRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_yesCreateToolbarRadioButtonActionPerformed
        checkValid();
        btvt.setEnabled(true);
        beforeToolbarItemCheckBox.setEnabled(true);
    }//GEN-LAST:event_yesCreateToolbarRadioButtonActionPerformed

    private void nameTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_nameTextFieldActionPerformed
        checkValid();
    }//GEN-LAST:event_nameTextFieldActionPerformed

    private void nameTextFieldFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_nameTextFieldFocusLost
        checkValid();
    }//GEN-LAST:event_nameTextFieldFocusLost

    private void iconButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_iconButtonActionPerformed
        JFileChooser chooser = new JFileChooser();
        chooser.setFileFilter(new FileFilter() {
                                  public boolean accept(File f) {
                                      return (f.isDirectory() || f.getName().endsWith(".gif")); // NOI18N
                                  }
                                  public String getDescription() {
                                      return bundle.getString("TXT_GIF");
                                  }
                              });

        if (iconFile != null) {
            chooser.setCurrentDirectory(iconFile);
        }

        if (chooser.showOpenDialog(TopManager.getDefault ().getWindowManager ().getMainWindow ())
                == JFileChooser.APPROVE_OPTION) {
            File f = chooser.getSelectedFile();
            if ((f != null) && (f.isFile())) {
                try {
                    iconFile = f;
                    iconTextField.setText(iconFile.getAbsolutePath());
                } finally {
                }
            }
        }
    }//GEN-LAST:event_iconButtonActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup createToolbarButtonGroup;
    private javax.swing.JTextArea descriptionTextArea;
    private javax.swing.JLabel nameLabel;
    private javax.swing.JTextField nameTextField;
    private javax.swing.JLabel menuItemLabel;
    private javax.swing.JScrollPane menuItemScrollPane;
    private javax.swing.JPanel menuItemPanel;
    private javax.swing.JCheckBox beforeMenuItemCheckBox;
    private javax.swing.JCheckBox separatorBeforeMenuItemCheckBox;
    private javax.swing.JCheckBox separatorAfterMenuItemCheckBox;
    private javax.swing.JLabel createToolbarLabel;
    private javax.swing.JPanel createToolbarPanel;
    private javax.swing.JRadioButton yesCreateToolbarRadioButton;
    private javax.swing.JRadioButton noCreateToolbarRadioButton;
    private javax.swing.JLabel toolbarItemLabel;
    private javax.swing.JScrollPane toolbarItemScrollPane;
    private javax.swing.JPanel toolbarItemPanel;
    private javax.swing.JCheckBox beforeToolbarItemCheckBox;
    private javax.swing.JLabel iconLabel;
    private javax.swing.JTextField iconTextField;
    private javax.swing.JButton iconButton;
    private javax.swing.JLabel shortcutLabel;
    private javax.swing.JTextField shortcutTextField;
    // End of variables declaration//GEN-END:variables

    private static final java.util.ResourceBundle bundle = NbBundle.getBundle( MenuPanel.class );

    /** Getter for property menuName.
     * @return Value of property menuName.
     */
    public String getMenuName() {
        return nameTextField.getText();
    }

    /** Getter for property otherMenuClass.
     * @return Value of property otherMenuClass.
     */
    public String getOtherMenuClass() {
        return otherMenuClass;
    }

    /** Getter for property otherToolbarClass.
     * @return Value of property otherToolbarClass.
     */
    public String getOtherToolbarClass() {
        return otherToolbarClass;
    }

    /** Getter for property beforeOtherMenu.
     * @return Value of property beforeOtherMenu.
     */
    public boolean isBeforeOtherMenu() {
        return beforeMenuItemCheckBox.isSelected();
    }

    /** Getter for property beforeOtherToolbar.
     * @return Value of property beforeOtherToolbar.
     */
    public boolean isBeforeOtherToolbar() {
        return beforeToolbarItemCheckBox.isSelected();
    }

    /** Getter for property separatorBefore.
     * @return Value of property separatorBefore.
     */
    public boolean isSeparatorBefore() {
        return separatorBeforeMenuItemCheckBox.isSelected();
    }

    /** Getter for property separatorAfter.
     * @return Value of property separatorAfter.
     */
    public boolean isSeparatorAfter() {
        return separatorAfterMenuItemCheckBox.isSelected();
    }

    /** Getter for property shortcut.
     * @return Value of property shortcut.
     */
    public String getShortcut() {
        return shortcutTextField.getText();
    }

    /** Getter for property menuHierarchy.
     * @return Value of property menuHierarchy.
     */
    public List getMenuHierarchy() {
        return menuHierarchy;
    }

    /** Getter for property toolHierarchy.
     * @return Value of property toolHierarchy.
     */
    public List getToolHierarchy() {
        return toolHierarchy;
    }

    /** Getter for property toolbar.
     * @return Value of property toolbar.
     */
    public boolean isToolbar() {
//SPUR
        return yesCreateToolbarRadioButton.isSelected();
    }

    /** Getter for property beforeMenuClass.
     * @return Value of property beforeMenuClass.
     */
    public String getBeforeMenuClass() {
        return beforeMenuClass;
    }

    /** Getter for property afterMenuClass.
     * @return Value of property afterMenuClass.
     */
    public String getAfterMenuClass() {
        return afterMenuClass;
    }

    /** Getter for property iconFile.
     * @return Value of property iconFile.
     */
    public File getIconFile() {
        return iconFile;
    }

    private void checkValid() {
        if (( nameTextField.getText() == null || nameTextField.getText().length() == 0 || !menuValid ) ||
                isToolbar() && !toolValid )                
            validator.setValid(false);
        else
            validator.setValid(true);
    }

}
