/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.apisupport.lite;

import java.io.*;
import java.util.StringTokenizer;

import org.openide.filesystems.*;
import org.openide.filesystems.FileSystem; // override java.io.FileSystem
import org.openide.loaders.*;
import org.openide.modules.*;

/** Data of panel for new module wizard
*
* @author Ales Kemr
*/
public class ModuleData extends WizardData {

    /** Holds value of property moduleName. */
    private String moduleName;

    /** Holds value of property packageName. */
    private String packageName;

    /** Holds value of property targetFS. */
    private FileSystem targetFS;

    /** Holds value of property dataLoader. */
    private String dataLoader;

    SharedData shared;

    /** Creates new WizardData */
    public ModuleData() {
        shared = SharedData.getShared();
    }

    /** Getter for property moduleName.
     * @return Value of property moduleName.
    */
    public String getModuleName() {
        return moduleName;
    }

    /** Setter for property moduleName.
     * @param moduleName New value of property moduleName.
    */
    public void setModuleName(String moduleName) {
        this.moduleName = moduleName;
        SharedData.getShared().setModuleName(moduleName);
    }

    /** Getter for property packageName.
     * @return Value of property packageName.
    */
    public String getPackageName() {
        return packageName;
    }

    /** Setter for property packageName.
     * @param packageName New value of property packageName.
    */
    public void setPackageName(String packageName) {
        this.packageName = packageName;
    }

    /** Getter for property targetFS.
     * @return Value of property targetFS.
    */
    public FileSystem getTargetFS() {
        return targetFS;
    }

    /** Setter for property targetFS.
     * @param targetFS New value of property targetFS.
    */
    public void setTargetFS(FileSystem targetFS) {
        this.targetFS = targetFS;

    }

    public void run() {
        shared.setTargetFS(targetFS);
        shared.setModuleName(moduleName);
        shared.setPackageName(packageName);
        try {
            createPackage();
        } catch (IOException e) {
        }
    }

    private void createPackage() throws IOException{
        StringTokenizer stok = new StringTokenizer(getPackageName(),".");
        FileObject fo = getTargetFS().getRoot();
        while (stok.hasMoreElements()) {
            String newFolder = stok.nextToken();
            if (fo.getFileObject(newFolder) == null)
                fo = fo.createFolder(newFolder);
            else
                fo = fo.getFileObject(newFolder);
        }
        shared.setMainFolder( (DataFolder)DataObject.find(fo) );
    }

}
