/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.apisupport.lite;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.event.ChangeListener;

import org.openide.TopManager;
import org.openide.WizardDescriptor;
import org.openide.cookies.*;
import org.openide.explorer.ExplorerManager;
import org.openide.explorer.ExplorerPanel;
import org.openide.explorer.view.BeanTreeView;
import org.openide.filesystems.*;
import org.openide.loaders.*;
import org.openide.nodes.Node;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;

/** Panel for new module wizard
*
* @author Ales Kemr
*/
public class ModulePanel extends javax.swing.JPanel{

    private Wizard.Validator validator;

    ExplorerPanel ep = new ExplorerPanel();
    ExplorerManager em = ep.getExplorerManager();

    private char separator;

    private FileSystem targetFS;
    
    private PropertyChangeListener listener;

    public ModulePanel(Wizard.Validator validator) {
        GridBagConstraints constraint = new GridBagConstraints();

        putClientProperty("WizardPanel_contentSelectedIndex", new Integer(1));
        setName(bundle.getString("LAB_Module"));

        initComponents();
        setPreferredSize(new java.awt.Dimension(500, 400));

        nameLabel.setDisplayedMnemonic(bundle.getString("LBL_Module_Mnemonic").charAt(0));
        packageListLabel.setDisplayedMnemonic(bundle.getString("LBL_SelectPackage_Mnemonic").charAt(0));
        packageLabel.setDisplayedMnemonic(bundle.getString("LBL_Package_Mnemonic").charAt(0));

        this.validator = validator;

        if (System.getProperty ("file.separator") != null)
            separator = System.getProperty ("file.separator").charAt(0);

        BeanTreeView btv;
        btv = new BeanTreeView ();
        btv.setPopupAllowed( false );
        btv.setDefaultActionAllowed( true );
        ep.setLayout (new java.awt.BorderLayout());
        ep.add ( java.awt.BorderLayout.CENTER, btv );

        Node n1 = TopManager.getDefault().getPlaces().nodes().repository();

        em.setRootContext(n1);
        em.setExploredContext(n1);

        
        listener = new PropertyChangeListener() {
                public void propertyChange( PropertyChangeEvent evt ) {
                    if ( evt.getPropertyName() == ExplorerManager.PROP_SELECTED_NODES ) {
                        selectionChange();
                    }
                }
            };        
        
        constraint.gridx = 1;
        constraint.gridy = 2;
        constraint.fill = GridBagConstraints.BOTH;
        constraint.insets = new Insets(11, 11, 0, 0);
        constraint.weightx = 1.0;
        constraint.weighty = 1.0;
        add(ep, constraint);
        packageListLabel.setLabelFor(btv);

        checkValid();
    }

    void addListeners() {
        em.addPropertyChangeListener( listener );        
    }
    
    void removeListeners() {
        em.removePropertyChangeListener( listener );
    }
    
    private void selectionChange() {
        Node[] no = em.getSelectedNodes();
        if (no != null && no.length > 0) {
            DataFolder df = (DataFolder) no[0].getCookie (DataFolder.class);
            if (df != null) {
                org.openide.filesystems.FileObject fo = df.getPrimaryFile();
                String pack = fo.getPackageName('.');
                if ( pack != null && pack.length() > 0 ) {
                    if ( packageTextField.getText().length() > 0 )
                        pack = pack + "." + packageTextField.getText();                    
                    packageTextField.setText(pack);                
                }
                String fsName = "";                
                try {
                    targetFS = fo.getFileSystem();
                    if (targetFS.isDefault()) {
                        targetFS = null;
                        directoryTextField.setText("");
                    }
                    else {
                        fsName = getTargetFS().getDisplayName();
                        directoryTextField.setText(fsName + separator + fo.getPackageName(separator));
                    }
                } catch (org.openide.filesystems.FileStateInvalidException e) {
                }                
            } else {
                DataObject dob = (DataObject) no[0].getCookie (DataObject.class);
                if (dob == null) {
                    targetFS = null;
                    directoryTextField.setText("");
                }
            }
        }
        checkValid();
    }

    private void initComponents() {//GEN-BEGIN:initComponents
        descriptionTextArea = new javax.swing.JTextArea();
        nameLabel = new javax.swing.JLabel();
        nameTextField = new javax.swing.JTextField();
        packageListLabel = new javax.swing.JLabel();
        packageLabel = new javax.swing.JLabel();
        packageTextField = new javax.swing.JTextField();
        directoryLabel = new javax.swing.JLabel();
        directoryTextField = new javax.swing.JTextField();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        descriptionTextArea.setWrapStyleWord(true);
        descriptionTextArea.setLineWrap(true);
        descriptionTextArea.setEditable(false);
        descriptionTextArea.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/apisupport/lite/Bundle").getString("DESC_Module"));
        descriptionTextArea.setBackground(nameLabel.getBackground());
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints1.weightx = 1.0;
        add(descriptionTextArea, gridBagConstraints1);
        
        nameLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/apisupport/lite/Bundle").getString("LBL_Module"));
        nameLabel.setLabelFor(nameTextField);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.insets = new java.awt.Insets(11, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(nameLabel, gridBagConstraints1);
        
        nameTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                nameTextFieldActionPerformed(evt);
            }
        });
        
        nameTextField.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                nameTextFieldKeyPressed(evt);
            }
            public void keyReleased(java.awt.event.KeyEvent evt) {
                nameTextFieldKeyReleased(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(11, 11, 0, 0);
        gridBagConstraints1.weightx = 1.0;
        add(nameTextField, gridBagConstraints1);
        
        packageListLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/apisupport/lite/Bundle").getString("LBL_SelectPackage"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.insets = new java.awt.Insets(11, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        add(packageListLabel, gridBagConstraints1);
        
        packageLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/apisupport/lite/Bundle").getString("LBL_Package"));
        packageLabel.setLabelFor(packageTextField);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.insets = new java.awt.Insets(11, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(packageLabel, gridBagConstraints1);
        
        packageTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                packageTextFieldActionPerformed(evt);
            }
        });
        
        packageTextField.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                packageTextFieldFocusLost(evt);
            }
        });
        
        packageTextField.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                packageTextFieldKeyPressed(evt);
            }
            public void keyReleased(java.awt.event.KeyEvent evt) {
                packageTextFieldKeyReleased(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(11, 11, 0, 0);
        gridBagConstraints1.weightx = 1.0;
        add(packageTextField, gridBagConstraints1);
        
        directoryLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/apisupport/lite/Bundle").getString("LBL_Directory"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.insets = new java.awt.Insets(11, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(directoryLabel, gridBagConstraints1);
        
        directoryTextField.setEditable(false);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(11, 11, 0, 0);
        gridBagConstraints1.weightx = 1.0;
        add(directoryTextField, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void packageTextFieldKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_packageTextFieldKeyReleased
        checkValid();
    }//GEN-LAST:event_packageTextFieldKeyReleased

    private void nameTextFieldKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_nameTextFieldKeyReleased
        checkValid();
    }//GEN-LAST:event_nameTextFieldKeyReleased

    private void packageTextFieldKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_packageTextFieldKeyPressed
        checkValid();
    }//GEN-LAST:event_packageTextFieldKeyPressed

    private void nameTextFieldKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_nameTextFieldKeyPressed
        checkValid();
    }//GEN-LAST:event_nameTextFieldKeyPressed

    private void packageTextFieldFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_packageTextFieldFocusLost
        checkValid();
    }//GEN-LAST:event_packageTextFieldFocusLost

    private void packageTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_packageTextFieldActionPerformed
        checkValid();
    }//GEN-LAST:event_packageTextFieldActionPerformed

    private void nameTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_nameTextFieldActionPerformed
        checkValid();
    }//GEN-LAST:event_nameTextFieldActionPerformed

    private void checkValid() {
        if (nameTextField.getText() != null && nameTextField.getText().length() > 0 &&
                packageTextField.getText() != null && packageTextField.getText().length() > 0 &&
                targetFS != null)
            validator.setValid(true);
        else
            validator.setValid(false);
    }

    /** Getter for property targetFS.
     * @return Value of property targetFS.
     */
    public FileSystem getTargetFS() {
        return targetFS;
    }

    /** Getter for property moduleName.
     * @return Value of property moduleName.
     */
    public String getModuleName() {
        return nameTextField.getText();
    }

    /** Getter for property packageName.
     * @return Value of property packageName.
     */
    public String getPackageName() {
        return packageTextField.getText();
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextArea descriptionTextArea;
    private javax.swing.JLabel nameLabel;
    private javax.swing.JTextField nameTextField;
    private javax.swing.JLabel packageListLabel;
    private javax.swing.JLabel packageLabel;
    private javax.swing.JTextField packageTextField;
    private javax.swing.JLabel directoryLabel;
    private javax.swing.JTextField directoryTextField;
    // End of variables declaration//GEN-END:variables

    private static final java.util.ResourceBundle bundle = NbBundle.getBundle( ModulePanel.class );

}
