/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.apisupport.lite;

import java.io.*;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.jar.*;

import org.openide.*;
import org.openide.cookies.*;
import org.openide.filesystems.*;
import org.openide.filesystems.FileSystem; // override java.io.FileSystem
import org.openide.loaders.*;
import org.openide.modules.*;
import org.openide.util.NbBundle;

// Should not depend on main apisupport at all: --jglick
//import org.netbeans.modules.apisupport.ManifestProvider;
// Use JAR Packager directly instead:
import org.netbeans.modules.jarpackager.JarContent;
import org.netbeans.modules.jarpackager.JarDataObject;

/** Data of panel for new module wizard
*
* @author Ales Kemr, Jesse Glick
*/
public class SharedData extends WizardData {

    final static String CLASS_PREFIX = "My";

    /** Holds value of property moduleName. */
    private String moduleName;

    /** Holds value of property packageName. */
    private String packageName;

    /** Holds value of property targetFS. */
    private FileSystem targetFS;

    /** Holds value of property mainFolder. */
    private DataFolder mainFolder;

    /** Holds value of property dataLoaderClass. */
    private String dataLoaderClass;

    private static SharedData sharedData;

    /** Holds value of property layer. */
    private String layerClass;

    /** Holds value of property properties. */
    private Properties properties = new Properties();

    /** Holds value of property version. */
    private String version;
    
    /** Holds value of property moduleDesc. */
    private String moduleDesc;
    
    /** Holds value of property category. */
    private String category;
    
    private static final java.util.ResourceBundle bundle = NbBundle.getBundle( SharedData.class );
    
    //private Manifest manifest = null;

    /** Creates new WizardData */
    private SharedData() {
    }

    public static SharedData getShared() {
        if (sharedData == null)
            sharedData = new SharedData();
        return sharedData;
    }

    /** Getter for property moduleName.
     * @return Value of property moduleName.
    */
    public String getModuleName() {
        return moduleName;
    }

    /** Setter for property moduleName.
     * @param moduleName New value of property moduleName.
    */
    public void setModuleName(String moduleName) {
        this.moduleName = moduleName;
    }

    /** Getter for property packageName.
     * @return Value of property packageName.
    */
    public String getPackageName() {
        return packageName;
    }

    /** Setter for property packageName.
     * @param packageName New value of property packageName.
    */
    public void setPackageName(String packageName) {
        this.packageName = packageName;
    }

    /** Getter for property targetFS.
     * @return Value of property targetFS.
    */
    public FileSystem getTargetFS() {
        return targetFS;
    }

    /** Setter for property targetFS.
     * @param targetFS New value of property targetFS.
    */
    public void setTargetFS(FileSystem targetFS) {
        this.targetFS = targetFS;
    }

    public void run() {
        try {
            createPackage();
        } catch (IOException e) {
        }
    }

    private void createPackage() throws IOException{
        StringTokenizer stok = new StringTokenizer(getPackageName(),".");
        FileObject fo = getTargetFS().getRoot();
        while (stok.hasMoreElements()) {
            String newFolder = stok.nextToken();
            if (fo.getFileObject(newFolder) == null)
                fo = fo.createFolder(newFolder);
            else
                fo = fo.getFileObject(newFolder);
        }
    }
    
    private DataFolder getTopFolder() {
        DataFolder daf = mainFolder;
        while ( daf.getFolder() != null)
            daf = daf.getFolder();
        return daf;
    }

    void manifest () {
        FileObject fo = TopManager.getDefault ().getRepository ().getDefaultFileSystem ().findResource ("apisupport-lite-data/modules/Module.jarContent");
        if (fo != null) {
            try {
                DataObject tryTemplate = DataObject.find (fo);
                DataObject manifestDO = tryTemplate.createFromTemplate (getTopFolder(), bundle.getString("TXT_JarName"));
                if (manifestDO instanceof JarDataObject) {
                    JarDataObject jarFile = (JarDataObject) manifestDO;
                    JarContent jc = jarFile.getJarContent ();
                    Manifest manifest = jc.getManifest ();

                    manifest.getMainAttributes ().put (ModuleDescription.TAG_MAGIC, packageName); // NOI18N
                    manifest.getMainAttributes ().put (ModuleDescription.TAG_NAME, moduleName);
                    
                    if ( version == null || version.length()==0 )
                        setVersion("0.1");
                    manifest.getMainAttributes ().put (ModuleDescription.TAG_SPEC_VERSION, version); // NOI18N
                    
                    if ( moduleDesc != null && moduleDesc.length()>0 )
                        manifest.getMainAttributes ().put (ModuleDescription.TAG_LONG_DESCRIPTION, moduleDesc); // NOI18N
                    
                    if ( category != null && category.length()>0 )
                        manifest.getMainAttributes ().put (ModuleDescription.TAG_CATEGORY, category); // NOI18N

                    if ( layerClass != null )
                        manifest.getMainAttributes ().put (ModuleDescription.TAG_LAYER, layerClass + ".xml"); // NOI18N

                    if ( dataLoaderClass != null ) {
                        Attributes atr = new Attributes();
                        atr.put(ModuleDescription.TAG_SECTION_CLASS,ModuleDescription.SECTION_LOADER);
                        manifest.getEntries().put(new String(dataLoaderClass + ".class"), atr);
                    }

                    jc.setManifest (manifest);
                    jarFile.writeJarContent (jc);
                    // No need to save after a call to setJarContent, it really writes data

                    // Since it will not be a ModuleDataObject, let's explicitly set its executor this way:
                    jarFile.getPrimaryFile ().setAttribute ("NetBeansAttrExecutor", new ServiceType.Handle (new LiteInstaller ()));
                } else {
                    TopManager.getDefault ().getErrorManager ().log (ErrorManager.INFORMATIONAL, "JAR file was of the wrong class: " + manifestDO); // NOI18N
                }
            } catch (IOException e) {
                TopManager.getDefault ().notifyException (e);
            }
        } else {
            TopManager.getDefault ().getErrorManager ().log (ErrorManager.INFORMATIONAL, "No module JAR found"); // NOI18N
        }
    }

    /* Removed, depended on main apisupport --jglick
    void manifest() {
        FileObject fo = TopManager.getDefault ().getRepository ().getDefaultFileSystem ().findResource ("apisupport-lite-data/modules/Manifest.mf");
        DataObject manifestDO;
        if (fo != null) {
            try {
                DataObject tryTemplate = DataObject.find (fo);
                manifestDO = tryTemplate.createFromTemplate(mainFolder);
                updateManifest( manifestDO );
                if ( manifest != null )
                    jar();
            } catch (IOException e) {
                TopManager.getDefault ().notifyException (e);
            }
        }
    }

    private void updateManifest( DataObject manifestDO ) throws IOException{

        ManifestProvider p = (ManifestProvider) manifestDO.getCookie (ManifestProvider.class);
        if (p == null) {
            throw new IOException ();
        }
        // Use a copy of the template's manifest:
        manifest = new Manifest (p.getManifest ());

        manifest.getMainAttributes ().put (ModuleDescription.TAG_MAGIC, packageName); // NOI18N
        manifest.getMainAttributes ().put (ModuleDescription.TAG_NAME, moduleName);
        manifest.getMainAttributes ().put (ModuleDescription.TAG_SPEC_VERSION, "0.1"); // NOI18N

        if ( layerClass != null )
            manifest.getMainAttributes ().put (ModuleDescription.TAG_LAYER, layerClass + ".xml"); // NOI18N

        if ( dataLoaderClass != null ) {
            Attributes atr = new Attributes();
            atr.put(ModuleDescription.TAG_SECTION_CLASS,ModuleDescription.SECTION_LOADER);
            manifest.getEntries().put(new String(dataLoaderClass + ".class"), atr);
        }

        p.setManifest (manifest);
        SaveCookie save = (SaveCookie) manifestDO.getCookie (SaveCookie.class);
        if (save != null) {
            save.save ();
        }
    }

    private void jar() {
        FileObject fo = TopManager.getDefault ().getRepository ().getDefaultFileSystem ().findResource ("apisupport-lite-data/modules/Module.jarContent");
        DataObject jarCont;
        if (fo != null) {
            try {
                DataObject tryTemplate = DataObject.find (fo);
                jarCont = tryTemplate.createFromTemplate(mainFolder);
                ManifestProvider p = (ManifestProvider) jarCont.getCookie (ManifestProvider.class);
                if ( p != null )
                    p.setManifest( manifest );

            } catch (IOException e) {
                TopManager.getDefault ().notifyException (e);
            }
        }
    }
    */

    void manageBundle() {
        FileObject fo = mainFolder.getPrimaryFile().getFileObject("Bundle","properties");
        FileLock lock = null;
        try {
            InputStream is = fo.getInputStream();
            properties.load(is);
            is.close();
            lock = fo.lock();
            fo.delete(lock);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if ( lock != null )
                lock.releaseLock();
        }
    }

    void replaceBundleEntry(String key, String value) {
        properties.setProperty(key, value);
    }

    void generateBundle() {
        FileLock lock = null;
        try {
            FileObject fo = mainFolder.getPrimaryFile().createData("Bundle","properties");
            lock=fo.lock();
            properties.store(fo.getOutputStream(lock),"# Resource bundle for package " + packageName);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            lock.releaseLock();
        }

    }

    /** Getter for property mainFolder.
     * @return Value of property mainFolder.
    */
    public DataFolder getMainFolder() {
        return mainFolder;
    }

    /** Setter for property mainFolder.
     * @param mainFolder New value of property mainFolder.
    */
    public void setMainFolder(DataFolder mainFolder) {
        this.mainFolder = mainFolder;
    }

    /** Getter for property dataLoaderClass.
     * @return Value of property dataLoaderClass.
    */
    public String getDataLoaderClass() {
        return dataLoaderClass;
    }

    /** Setter for property dataLoaderClass.
     * @param dataLoaderClass New value of property dataLoaderClass.
    */
    public void setDataLoaderClass(String dataLoaderClass) {
        this.dataLoaderClass = dataLoaderClass;
    }

    /** Getter for property layer.
     * @return Value of property layer.
    */
    public String getLayerClass() {
        return layerClass;
    }

    /** Setter for property layer.
     * @param layer New value of property layer.
    */
    public void setLayerClass(String layerClass) {
        this.layerClass = layerClass;
    }

    /** Getter for property version.
     * @return Value of property version.
    */
    public String getVersion() {
        return version;
    }
    
    /** Setter for property version.
     * @param version New value of property version.
    */
    public void setVersion(String version) {
        this.version = version;
    }
    
    /** Getter for property moduleDesc.
     * @return Value of property moduleDesc.
    */
    public String getModuleDesc() {
        return moduleDesc;
    }
    
    /** Setter for property moduleDesc.
     * @param moduleDesc New value of property moduleDesc.
    */
    public void setModuleDesc(String moduleDesc) {
        this.moduleDesc = moduleDesc;
    }
    
    /** Getter for property category.
     * @return Value of property category.
    */
    public String getCategory() {
        return category;
    }
    
    /** Setter for property category.
     * @param category New value of property category.
    */
    public void setCategory(String category) {
        this.category = category;
    }
    
}
