/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.apisupport.wizard;

import java.io.*;
import java.util.*;

import org.openide.*;
import org.openide.loaders.*;

/**
 * @author Jesse Glick
 */
public final class APIIterator extends PolymorphicIterator {

    // XXX replace this static var with a hidden bean property in APISettings:
    /** DataObject with ManifestProvider cookie which is the location of the
     * module manifest last in use by the wizard.
     */
    private static DataObject MANIFEST = null;
    public static DataObject getManifestDataObject () {
        return MANIFEST;
    }
    public static void setManifestDataObject (DataObject obj) {
        System.err.println("setManifestDataObject: " + obj);//XXX
        MANIFEST = obj;
    }
    
    private List panels = null; // List<APIPanel>
    private Map panelData = null; // Map<APIPanel,APIWizardData>
    private DataFolder preferredFolder = null;
    
    private static final long serialVersionUID = 7341217155773713139L;

    public APIIterator () {
    }
    
    public void initialize (TemplateWizard wiz) {
        super.initialize (wiz);
        try {
            preferredFolder = wiz.getTargetFolder ();
        } catch (IOException ioe) {
            TopManager.getDefault ().notifyException (ioe);
        }
        panels = new ArrayList ();
        panelData = new HashMap ();
    }
    
    public void uninitialize (TemplateWizard wiz) {
        super.uninitialize (wiz);
        panels = null;
        panelData = null;
    }

    public Set instantiate (TemplateWizard ignore) throws IOException {
        WizardDescriptor.Panel curr;
        while (! ((curr = super.current ()) instanceof APIPanel) && hasPrevious ()) {
            // Go back to the last APIPanel.
            previousPanel ();
        }
        System.err.println("Finishing panel: " + curr);//XXX
        System.err.println("panels=" + panels);//XXX
        System.err.println("panelData=" + panelData);//XXX
        if (curr instanceof APIPanel) {
            APIWizardData data = (APIWizardData) panelData.get (curr);
            return process (data);
        } else {
            // We are in some initial panel sequence that is not APIPanel's.
            // Can do nothing.
            return Collections.EMPTY_SET;
        }
    }
    
    public WizardDescriptor.Panel current () {
        WizardDescriptor.Panel panel = super.current ();
        if (panel instanceof APIPanel) {
            APIPanel apipanel = (APIPanel) panel;
            if (! panels.contains (apipanel)) {
                panels.add (apipanel);
            }
            int idx = panels.indexOf (apipanel);
            APIWizardData data;
            if (idx == 0) {
                data = new APIWizardData ();
                // Hint about user's preferred folder to store things in:
                data.getProperties ().put (APIWizardData.PROP_MAIN_FOLDER, preferredFolder);
            } else {
                try {
                    data = (APIWizardData) ((APIWizardData) panelData.get (panels.get (idx - 1))).clone ();
                } catch (CloneNotSupportedException cnse) {
                    TopManager.getDefault ().notifyException (cnse);
                    data = new APIWizardData ();
                }
            }
            panelData.put (apipanel, data);
            apipanel.data = data;
        }
        return panel;
    }
    
    private static Set process (APIWizardData data) throws IOException {
        System.err.println("process: data=" + data); // XXX
        Set result = new HashSet (); // Set<DataObject>
        Iterator it = data.instantiations.iterator ();
        while (it.hasNext ()) {
            APIWizardData.Instantiate inst = (APIWizardData.Instantiate) it.next ();
            DataObject nue = inst.getTemplate ().createFromTemplate (inst.getFolder (), inst.getName ());
            inst.result = nue;
            System.err.println("instantiated: inst=" + inst);
            result.add (nue);
            // XXX
            if (! inst.getTokens ().isEmpty ())
                throw new IOException ("Token substitution not yet implemented");
        }
        it = data.tasks.iterator ();
        while (it.hasNext ()) {
            APIWizardData.ExtraTask task = (APIWizardData.ExtraTask) it.next ();
            task.run ();
            result.addAll (task.objs);
        }
        System.err.println("process: result=" + result); // XXX
        return result;
    }
    
}
