/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.apisupport.wizard;

import java.awt.Component;
import java.util.*;
import javax.swing.event.*;

import org.openide.*;
import org.openide.loaders.*;
import org.openide.util.*;

/**
 * @author Jesse Glick
 */
public abstract class APIPanel implements WizardDescriptor.Panel {
    
    private final Set listeners = new HashSet (); // Set<ChangeListener>
    // Accessed from APIIterator:
    APIWizardData data = null;

    protected APIPanel () {
    }
    
    public final synchronized void addChangeListener (ChangeListener l) {
        listeners.add (l);
    }
    
    public final synchronized void removeChangeListener (ChangeListener l) {
        listeners.remove (l);
    }
    
    protected final synchronized void fireChangeEvent () {
        // [PENDING] fire asynch & clone listeners?
        ChangeEvent ev = new ChangeEvent (this);
        Iterator it = listeners.iterator ();
        while (it.hasNext ()) {
            ChangeListener l = (ChangeListener) it.next ();
            l.stateChanged (ev);
        }
    }
    
    public abstract Component getComponent ();
    
    public HelpCtx getHelp () {
        return HelpCtx.DEFAULT_HELP;
    }
    
    public abstract boolean isValid ();
    
    public final void readSettings (Object ignore) {
        if (data == null) throw new IllegalStateException ();
        readSettings (data);
    }
    
    protected abstract void readSettings (APIWizardData data);
    
    public final void storeSettings (Object ignore) {
        if (data == null) throw new IllegalStateException ();
        storeSettings (data);
    }
    
    protected abstract void storeSettings (APIWizardData data);
    
}
