/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

// XXX stolen from org.netbeans.core.ui rev 1.3 with modifications (marked)
// When moved into APIs, should be used from there
// Do not put anything non-generic in here!

package org.netbeans.modules.apisupport.wizard;

import java.util.NoSuchElementException;
import java.awt.Component;

import org.openide.TopManager;
import org.openide.loaders.FolderInstance;
import org.openide.WizardDescriptor;
import org.openide.cookies.InstanceCookie;
import org.openide.loaders.DataFolder;
import org.openide.util.HelpCtx;
import org.openide.filesystems.FileLock;

import javax.swing.event.ChangeListener;

/**
 * This class allows you to create the steps of the wizard
 * by providing a folder with .instance files. The wizard
 * will ask dataobjects in the folder for InstanceCookie.
 * If this will be an instance of JPanel or
 * org.openide.WizardDescriptor.Iterator, it will be
 * used as a step of the wizard.
 * @author  dstrupl
 */
public class WizardFolder extends org.openide.loaders.FolderInstance {

    // --> REMOVED: <--
    /**
     * Currently active iterator.
     *
    private WizardDescriptor.Iterator iterator;
     */
    
    /** Creates new WizardFolder */
    public WizardFolder(DataFolder theFolder) {
        super(theFolder);
        // --> REMOVED: <-- run();
    }
    
    /**
     * Returns the same cookie if the instance class is
     * java.awt.Component, WizardDescriptor.Iterator or
     * WizardDescriptor.Panel.
     * @param cookie the instance cookie to test
     * @return the cookie to use or <code>null</code> if this cookie should not
     *   be used
     * @exception IOException if an I/O error occurred calling a cookie method
     * @exception ClassNotFoundException if a class is not found in a call to a cookie method
     */
    protected InstanceCookie acceptCookie(InstanceCookie cookie) throws java.io.IOException, ClassNotFoundException {
        if (java.awt.Component.class.isAssignableFrom(cookie.instanceClass())) {
            return cookie;
        }
        if (org.openide.WizardDescriptor.Iterator.class.isAssignableFrom(cookie.instanceClass())) {
            return cookie;
        }
        if (org.openide.WizardDescriptor.Panel.class.isAssignableFrom(cookie.instanceClass())) {
            return cookie;
        }
        return null;
    }
    
    /** Notifies subclasses that the set of cookies for this folder
     * has changed.
     * A new object representing the folder should
     * be created (or the old one updated).
     * Called both upon initialization of the class, and change of its cookies.
     *
     * @param cookies updated array of instance cookies for the folder
     * @return object to represent these cookies
     *
     * @exception IOException an I/O error occured
     * @exception ClassNotFoundException a class has not been found
     */
    protected Object createInstance(InstanceCookie[] cookies) throws java.io.IOException, ClassNotFoundException {
        WizardDescriptor.Iterator []arr = new WizardDescriptor.Iterator[cookies.length];
        for (int i = 0; i < cookies.length; i++) {
            if (org.openide.WizardDescriptor.Iterator.class.isAssignableFrom(cookies[i].instanceClass())) {
                arr[i] = (WizardDescriptor.Iterator) cookies[i].instanceCreate();
                continue;
            }
            if (org.openide.WizardDescriptor.Panel.class.isAssignableFrom(cookies[i].instanceClass())) {
                WizardDescriptor.Panel p = (WizardDescriptor.Panel) cookies[i].instanceCreate();
                arr[i] = new PanelWrapper(p, cookies[i].instanceName());
                continue;
            }
            if (java.awt.Component.class.isAssignableFrom(cookies[i].instanceClass())) {
                java.awt.Component c = (java.awt.Component) cookies[i].instanceCreate();
//                c.setName(cookies[i].instanceName());
                arr[i] = new PanelWrapper(c, cookies[i].instanceName());
                continue;
            }
        }
        // --> CHANGED: <--
        /*
        if (arr.length > 0) {
            iterator = new CompoundIterator(arr);
        }
        return this; // where is this used anyway? (dstrupl)
         */
        if (arr.length > 0) {
            return new CompoundIterator (arr);
        } else {
            throw new java.io.IOException ("Cannot make an empty wizard");
        }
    }

    // --> REMOVED: <--
    /** @return currently active iterator /
    public WizardDescriptor.Iterator getIterator() {
        return iterator;
    }
     */

    /**
     * This iterator delegates to the iterators given in constructor.
     */ 
    private static class CompoundIterator implements org.openide.WizardDescriptor.Iterator {
        
        /** Iterators we to which we delegate. */
        private WizardDescriptor.Iterator [] array;
        
        /** 
         * Currently active itaratori index. The index is for array variable,
         * so the values are between 0 ... array.length-1
         */
        private int active = 0;
        
        /** Registered change listener. */
        private ChangeListener listener = null;
        
        /** It assumes that all elements of the array are non-null. */
        public CompoundIterator(WizardDescriptor.Iterator []arr) {
            array = arr;
        }
        
        /** Add a listener to changes of the current panel.
         * The listener is notified when the possibility to move forward/backward changes.
         * @param l the listener to add
         */
        public void addChangeListener(ChangeListener l) {
            listener = l;
            // add the listener also to the delegated iterators
            for (int i = 0; i < array.length; i++) {
                array[i].addChangeListener(l);
            }
        }
        
        /** Get the current panel.
         * @return the panel
         */
        public WizardDescriptor.Panel current() {
            return array[active].current();
            // [PENDING] ??? wrap the panel to disable finishing 
        }
        
        /** Test whether there is a next panel.
         * @return <code>true</code> if so
         */
        public boolean hasNext() {
            if (active != array.length - 1) {
                // it is not the last one
                return true;
            }
            return array[active].hasNext();
        }
        
        /** Test whether there is a previous panel.
         * @return <code>true</code> if so
         */
        public boolean hasPrevious() {
            if (active != 0) {
                // not the first iterator
                return true;
            }
            return array[active].hasPrevious();
        }
        
        /** Get the name of the current panel.
         * @return the name
         */
        public String name() {
            return array[active].name();
        }
        
        /** Move to the next panel.
         * I.e. increment its index, need not actually change any GUI itself.
         * @exception NoSuchElementException if the panel does not exist
         */
        public void nextPanel() {
            if (array[active].hasNext()) {
                array[active].nextPanel();
                return;
            }
            if (active < array.length-1) {
                active++;
                return;
            }
            throw new NoSuchElementException();
        }
        
        /** Move to the previous panel.
         * I.e. decrement its index, need not actually change any GUI itself.
         * @exception NoSuchElementException if the panel does not exist
         */
        public void previousPanel() {
            if (array[active].hasPrevious()) {
                array[active].previousPanel();
                return;
            }
            if (active > 0) {
                active--;
                return;
            }
            throw new NoSuchElementException();
        }
        
        /** Remove a listener to changes of the current panel.
         * @param l the listener to remove
         */
        public void removeChangeListener(ChangeListener l) {
            l = null;
            // remove the listener also from the delegated iterators
            for (int i = 0; i < array.length; i++) {
                array[i].removeChangeListener(l);
            }
        }
    }

    /** Simple iterator that iterates over just one panel */
    private static class PanelWrapper implements org.openide.WizardDescriptor.Iterator {
        
        private WizardDescriptor.Panel panel;
        
        private String name;
        
        public PanelWrapper(WizardDescriptor.Panel p, String name) {
            panel = p;
            this.name = name;
        }
        
        public PanelWrapper(java.awt.Component c, String name) {
            panel = new ComponentPanel(c);
            this.name = name;
        }
        
        /** Add a listener to changes of the current panel.
         * The listener is notified when the possibility to move forward/backward changes.
         * @param l the listener to add
         */
        public void addChangeListener(ChangeListener l) {
//            panel.addChangeListener(l);
        }
        
        /** Get the current panel.
         * @return the panel
         */
        public WizardDescriptor.Panel current() {
            return panel;
        }
        
        /** Test whether there is a next panel.
         * @return <code>true</code> if so
         */
        public boolean hasNext() {
            return false;
        }
        
        /** Test whether there is a previous panel.
         * @return <code>true</code> if so
         */
        public boolean hasPrevious() {
            return false;
        }
        
        /** Get the name of the current panel.
         * @return the name
         */
        public String name() {
            return name;
        }
        
        /** Move to the next panel.
         * I.e. increment its index, need not actually change any GUI itself.
         * @exception NoSuchElementException if the panel does not exist
         */
        public void nextPanel() {
            throw new NoSuchElementException();
        }
        
        /** Move to the previous panel.
         * I.e. decrement its index, need not actually change any GUI itself.
         * @exception NoSuchElementException if the panel does not exist
         */
        public void previousPanel() {
            throw new NoSuchElementException();
        }
        
        /** Remove a listener to changes of the current panel.
         * @param l the listener to remove
         */
        public void removeChangeListener(ChangeListener l) {
//            panel.removeChangeListener(l);
        }
    }

    /**
     * ComponentPanel instance wraps a java.awt.Component to
     * get a WizardDescriptor.Panel
     */
    private static class ComponentPanel implements WizardDescriptor.Panel {
        
        /** Remembered component*/
        private Component component;
        
        /** Just remembers the component.*/
        public ComponentPanel(Component c) {
            component = c;
        }
        
        /**
         * Get the component displayed in this panel.
         * @return the component
         */
        public java.awt.Component getComponent () {
            return component;
        }

        /** Help for this panel.
         * When the panel is active, this is used as the help for the wizard dialog.
         * @return the help or <code>null</code> if no help is supplied
         */
        public HelpCtx getHelp () {
            return null;
        }

        /** Provides the wizard panel with the current data--either
         * the default data or already-modified settings, if the user used the previous and/or next buttons.
         * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
         * @param settings the object representing wizard panel state, as originally supplied to {@link WizardDescriptor#WizardDescriptor(WizardDescriptor.Iterator,Object)}
         */
        public void readSettings (Object settings) {
        }

        /** Provides the wizard panel with the opportunity to update the
         * settings with its current customized state.
         * Rather than updating its settings with every change in the GUI, it should collect them,
         * and then only save them when requested to by this method.
         * Also, the original settings passed to {@link #readSettings} should not be modified (mutated);
         * rather, the (copy) passed in here should be mutated according to the collected changes.
         * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
         * @param settings the object representing a settings of the wizard
         */
        public void storeSettings (Object settings) {
        }

        /** Test whether the panel is finished and it is safe to proceed to the next one.
         * If the panel is valid, the "Next" (or "Finish") button will be enabled.
         * @return <code>true</code> if the user has entered satisfactory information
         */
        public boolean isValid () {
            return true;
        }

        /** Add a listener to changes of the panel's validity.
         * @param l the listener to add
         * @see #isValid
         */
        public void addChangeListener (ChangeListener l) {
        }

        /** Remove a listener to changes of the panel's validity.
         * @param l the listener to remove
         */
        public void removeChangeListener (ChangeListener l) {
        }

    }
    
    // --> REMOVED showSetupWizard() <--
    
}
