/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.autoupdate;

import java.io.File;
import java.beans.PropertyEditorSupport;
import java.util.*;
import java.text.SimpleDateFormat;



import org.openide.options.SystemOption;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.ServiceType;
import org.openide.TopManager;

/** Service type representing autoupdate server
*
* @author Ales Kemr
*/
public abstract class AutoupdateType extends org.openide.ServiceType
{

    /** serialVersionUID */
    static final long serialVersionUID = 362844512378569452L;
    
    static {        
        if ( java.beans.PropertyEditorManager.findEditor( ServiceType.class ) != null )
            java.beans.PropertyEditorManager.registerEditor( 
                AutoupdateType.class, 
                java.beans.PropertyEditorManager.findEditor( ServiceType.class ).getClass()
            );
    }
    
    public AutoupdateType() {
    }

    /** @return human presentable name */
    public String displayName() {
        return NbBundle.getBundle( AutoupdateType.class).getString("CTL_Settings_Name");
    }

    public HelpCtx getHelpCtx () {
        return new HelpCtx ( AutoupdateType.class );
    }

    public abstract Updates connectForUpdates();        
    
    /** Get all registered autoupdate types.
    * @return enumeration of <code>autoupdateType</code>s
    */
    public static Enumeration autoupdateTypes () {
        Enumeration en = TopManager.getDefault ().getServices ().services (AutoupdateType.class);
        java.util.List lst = new ArrayList();
        while (en.hasMoreElements()) {
            AutoupdateType at = (AutoupdateType)en.nextElement();
            if ( at.displayName().equals(
                    NbBundle.getBundle( Settings.class).getString("CTL_QuantumAutoupdateType_Name")))
                lst.add(0,at);
            else
                lst.add(at);             
        }
        return Collections.enumeration( lst );        
    }

    /** Find the
    * autoupdate type implemented as a given class, among the services registered to the
    * system.
    * <P>
    * This should be used during (de-)serialization
    * of the specific autoupdate type for a data object: only store its class name
    * and then try to find the autoupdate implemented by that class later.
    *
    * @param clazz the class of the autoupdate type looked for
    * @return the desired autoupdate type or <code>null</code> if it does not exist
    */
    public static AutoupdateType find (Class clazz) {
        ServiceType t = TopManager.getDefault ().getServices ().find (clazz);
        if (t instanceof AutoupdateType) {
            return (AutoupdateType)t;
        } else {
            return null;
        }
    }

    /** Find the
    * autoupdate with requested name, among the services registered to the
    * system.
    * <P>
    * This should be used during (de-)serialization
    * of the specific autoupdate type for a data object: only store its name
    * and then try to find the debugger later.
    *
    * @param name (display) name of autoupdate to find
    * @return the desired autoupdate or <code>null</code> if it does not exist
    */
    public static AutoupdateType find (String name) {
        ServiceType t = TopManager.getDefault ().getServices ().find (name);
        if (t instanceof AutoupdateType) {
            return (AutoupdateType)t;
        } else {
            return null;
        }
    }

    /** Gets the default autoupdate type in the system.
    */
    public static AutoupdateType getDefault () {
        return (AutoupdateType)autoupdateTypes ().nextElement ();
    }

}

