/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.autoupdate;

import java.util.Collection;
import javax.swing.*;
import org.openide.DialogDescriptor;
import org.openide.TopManager;
import org.openide.util.NbBundle;
import org.openide.util.HelpCtx;

/**
 *
 * @author 
 */
public class CertificateDialog extends javax.swing.JPanel {

    static final int CLOSE = 0;
    static final int ACCEPT = 1;
    static final int ALWAYS_ACCEPT = 2;
    static final int NOT_ALWAYS_ACCEPT = 3;
    static final int REJECT = 4;
    /** Result of the dialog */
    private static int result = CLOSE;

    static final int SIGNED_MODULE = 0;
    static final int TRUSTED_MODULE = 1;
    /** Type of the dialog */
    private static int type;

    /** The resource bundle */
    private static final java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("org/netbeans/modules/autoupdate/Bundle");

    /** Creates new form CertificateDialog */
    public CertificateDialog(Collection certs) {
        initComponents();
        certificateTextArea.setText( SignVerifier.formatCerts( certs ) );
        certificateTextArea.setCaretPosition(0);
        setPreferredSize(new java.awt.Dimension(350,220));
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        aboutTextArea = new javax.swing.JTextArea();
        jScrollPane1 = new javax.swing.JScrollPane();
        certificateTextArea = new javax.swing.JTextArea();
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(18, 18, 18, 18)));
        
        aboutTextArea.setLineWrap(true);
        aboutTextArea.setFont(new java.awt.Font ("Dialog", 0, 11));
        aboutTextArea.setText(bundle.getString(type == SIGNED_MODULE ? "CTL_Certificate_Signed_About" : "CTL_Certificate_Trusted_About"));
        aboutTextArea.setBackground(new java.awt.Color (204, 204, 204));
        aboutTextArea.setDisabledTextColor(java.awt.Color.black);
        aboutTextArea.setWrapStyleWord(true);
        aboutTextArea.setEnabled(false);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        add(aboutTextArea, gridBagConstraints1);
        
        
        
        certificateTextArea.setText("SDASDASD\nSASDA\nCVVX\nRTE\ngfhghf\nuytuiytui\nihlk\nhklhjkhk\nhjkhjk");
        certificateTextArea.setDisabledTextColor(java.awt.Color.black);
        certificateTextArea.setEnabled(false);
        jScrollPane1.setViewportView(certificateTextArea);
        
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(6, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(jScrollPane1, gridBagConstraints1);
        
    }//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextArea aboutTextArea;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTextArea certificateTextArea;
    // End of variables declaration//GEN-END:variables
    
    private static DialogDescriptor createDialog(Collection certs) {

        Object[] buttons;
        final JButton closeButton = new JButton(bundle.getString( "CTL_Certificate_Close" ) );
        final JButton acceptButton = new JButton(bundle.getString( "CTL_Certificate_Accept" ) );
        final JButton alwaysButton = new JButton(bundle.getString( "CTL_Certificate_AlwaysAccept" ) );
        final JButton notalwaysButton = new JButton(bundle.getString( "CTL_Certificate_NotAlwaysAccept" ) );
        final JButton rejectButton = new JButton(bundle.getString( "CTL_Certificate_Reject" ) );

        if (type == SIGNED_MODULE)
            buttons = new Object [] {
                acceptButton,
                alwaysButton,
                rejectButton,
                closeButton
            };
        else
            buttons = new Object [] {
                notalwaysButton,
                closeButton
            };
        
        DialogDescriptor dd;
        dd = new DialogDescriptor(
                 new CertificateDialog(certs),
                 bundle.getString( "CTL_Certificate_Title" ),
                 true,                                  // Modal
                 buttons,                               // Option list
                 closeButton,                           // Default
                 DialogDescriptor.DEFAULT_ALIGN,        // Align
                 new HelpCtx (CertificateDialog.class), // Help
                 new java.awt.event.ActionListener() {
                     public void actionPerformed( java.awt.event.ActionEvent evt ) {
                         if ( evt.getSource() == acceptButton )
                             result = ACCEPT;
                         else if ( evt.getSource() == alwaysButton )
                             result = ALWAYS_ACCEPT;
                         else if ( evt.getSource() == notalwaysButton )
                             result = NOT_ALWAYS_ACCEPT;
                         else if ( evt.getSource() == rejectButton )
                             result = REJECT;
                         else if ( evt.getSource() == closeButton )
                             result = CLOSE;                         
                     }
                 });

        dd.setClosingOptions( null );

        return dd;
    }
    
    static int showDialog(Collection certs, int type) {
        if (type == SignVerifier.TRUSTED)
            CertificateDialog.type = TRUSTED_MODULE;
        else
            CertificateDialog.type = SIGNED_MODULE;
        DialogDescriptor dd = createDialog(certs);
        java.awt.Dialog dialog = TopManager.getDefault().createDialog( dd );
        dialog.show();
        dialog.dispose();
        
        return result;
    }
}
