/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.autoupdate;

import javax.swing.*;
import org.openide.DialogDescriptor;
import org.openide.TopManager;
import org.openide.util.NbBundle;
import org.openide.util.HelpCtx;

class ConnectingErrorDialog extends javax.swing.JPanel {

    /** Type of the dialog */
    private static int type;
    private static String serverMessage;
    
    private javax.swing.JLabel titleLabel;
    private javax.swing.JTextArea textArea;
    private javax.swing.JLabel iconLabel;
    
    /** The resource bundle */
    private static final java.util.ResourceBundle bundle = 
        java.util.ResourceBundle.getBundle("org/netbeans/modules/autoupdate/Bundle");

    /** Creates new form ConnectingDialog */
    public ConnectingErrorDialog() {
        initComponents();
        setPreferredSize(new java.awt.Dimension(350,120));
    }

    /** This method is called from within the constructor to
     * initialize the form.
     */
    private void initComponents() {
        iconLabel = new javax.swing.JLabel();
        titleLabel = new javax.swing.JLabel();
        textArea = new javax.swing.JTextArea();
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints;

        iconLabel.setIcon(javax.swing.UIManager.getIcon("OptionPane.errorIcon")); // NOI18N
        titleLabel.setFont(new java.awt.Font ("Dialog", 1, 11));
        titleLabel.setForeground(java.awt.Color.black);
        if (type == Updates.UNKNOWN_ERROR)
            titleLabel.setText(bundle.getString("CTL_Unable_titleLabel"));
        else if (type == Updates.NO_SERVER_ERROR)
            titleLabel.setText(bundle.getString("CTL_NoServer_titleLabel"));
        else
            titleLabel.setText(bundle.getString("CTL_Invalid_titleLabel"));

        textArea.setFont(new java.awt.Font ("Dialog", 0, 11));
        textArea.setBackground(new java.awt.Color (204, 204, 204));
        textArea.setDisabledTextColor(java.awt.Color.black);
        textArea.setLineWrap(true);
        textArea.setWrapStyleWord(true);
        textArea.setEnabled(false);
        if ( type == Updates.UNKNOWN_ERROR && serverMessage != null )
            textArea.setText( serverMessage );
        else {
            if (type == Updates.UNKNOWN_ERROR)                
                textArea.setText(bundle.getString("CTL_Unable_textLabel"));
            else if (type == Updates.NO_SERVER_ERROR)
                textArea.setText(bundle.getString("CTL_NoServer_textLabel"));
            else
                textArea.setText(bundle.getString("CTL_Invalid_textLabel"));            
        }
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridheight = 0;
        gridBagConstraints.insets = new java.awt.Insets(18, 18, 0, 0);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        add(iconLabel, gridBagConstraints);

        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.gridwidth = 0;
        gridBagConstraints.insets = new java.awt.Insets(18, 12, 0, 18);
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weighty = 0.0;
        add(titleLabel, gridBagConstraints);

        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 0;
        gridBagConstraints.insets = new java.awt.Insets(0, 12, 0, 18);
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        add(textArea, gridBagConstraints);
    }

    private static DialogDescriptor createDialog() {

        final JButton okButton = new JButton(bundle.getString( "CTL_Error_OK" ) );

        DialogDescriptor dd;
        dd = new DialogDescriptor(
                 new ConnectingErrorDialog(),
                 bundle.getString( type == Updates.UNKNOWN_ERROR ? "CTL_Unable_Title" : "CTL_Invalid_Title" ),
                 true,                                  // Modal
                 new Object [] {
                     okButton,
                 },                                     // Option list
                 okButton,                              // Default
                 DialogDescriptor.DEFAULT_ALIGN,        // Align
                 new HelpCtx ( ConnectingErrorDialog.class ), // Help
                 new java.awt.event.ActionListener() {
                      public void actionPerformed( java.awt.event.ActionEvent evt ) {
                      }
                 });

        dd.setClosingOptions( null );

        return dd;
    }
    
    static void showDialog(int type, String serverMessage) {
        ConnectingErrorDialog.type = type;
        ConnectingErrorDialog.serverMessage = serverMessage;
        DialogDescriptor dd = createDialog();
        java.awt.Dialog dialog = TopManager.getDefault().createDialog( dd );
        dialog.show();
        dialog.dispose();
    }
}