/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */
package org.netbeans.modules.autoupdate;

import java.io.IOException;

import org.openide.actions.*;
import org.openide.filesystems.*;
import org.openide.loaders.*;
import org.openide.util.NbBundle;
import org.openide.util.actions.SystemAction;

/** Recognizes single files in the Repository as being of a certain type.
 *
 * @author Ales Kemr
 */
public class NbmDataLoader extends UniFileLoader {

    /** SUID */
    static final long serialVersionUID=-3599951493531214988L;
    
    public NbmDataLoader () {
        this (NbmDataObject.class);
    }

    public NbmDataLoader (Class recognizedObject) {
        super (recognizedObject);
    }

    protected void initialize () {

        super.initialize ();        

        ExtensionList extensions = new ExtensionList ();
        extensions.addExtension ("nbm");
        setExtensions (extensions);        
    }
    
    /** Get default actions.
    * @return array of default system actions or <CODE>null</CODE> if this loader does not have any
    *   actions
    */
    protected SystemAction[] defaultActions () {
        return new SystemAction[] {
                        SystemAction.get (OpenAction.class),
                        // SystemAction.get (CustomizeBeanAction.class),
                        SystemAction.get (FileSystemAction.class),
                        null,
                        /*
                        SystemAction.get (CompileAction.class),
                        null,
                        SystemAction.get (ExecuteAction.class),
                        null,
                        */
                        SystemAction.get (CutAction.class),
                        SystemAction.get (CopyAction.class),
                        SystemAction.get (PasteAction.class),
                        null,
                        SystemAction.get (DeleteAction.class),
                        SystemAction.get (RenameAction.class),
                        null,
                        SystemAction.get (SaveAsTemplateAction.class),
                        null,
                        SystemAction.get (ToolsAction.class),
                        SystemAction.get (PropertiesAction.class),
                    };
    }

    /** Get the default display name of this loader.
    * @return default display name
    */
    protected String defaultDisplayName () {
        return NbBundle.getMessage (NbmDataLoader.class, "LBL_loaderName");
    }
    
    protected MultiDataObject createMultiObject (FileObject primaryFile)
    throws DataObjectExistsException, IOException {
        return new NbmDataObject (primaryFile, this);
    }

}
