/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.autoupdate;

import java.io.File;
import java.io.FileFilter;
import java.awt.Dialog;
import java.util.ResourceBundle;
import java.text.MessageFormat;

import javax.swing.JFileChooser;
import javax.swing.DefaultListModel;
import javax.swing.event.ChangeListener;

import org.openide.DialogDescriptor;
import org.openide.NotifyDescriptor;
import org.openide.TopManager;
import org.openide.util.NbBundle;
import org.openide.util.HelpCtx;
import org.openide.util.RequestProcessor;
import org.openide.WizardDescriptor;
import org.openide.util.HelpCtx;

public class SelectModulesPanel extends javax.swing.JPanel {
    
    private static final String EMPTY_STRING = ""; // NOI18N

    private static final javax.swing.filechooser.FileFilter NBM_FILE_FILTER = new NbmFileFilter();

    /** The list model */
    private DefaultListModel modulesModel;

    /** Stores the last selected directory */
    File defaultDir = null;

    private Wizard.Validator validator;

    static final long serialVersionUID =8279789280479374665L;
    
    /** Creates new form SelectModulesPanel */
    public SelectModulesPanel(Wizard.Validator validator) {
        putClientProperty("WizardPanel_contentSelectedIndex", new Integer(1));
        setName(bundle.getString("LAB_Select"));
        initComponents();
        
        this.validator = validator;

        reset();
    }

    /** Overload getPreffered size to get a bit bigger dialog */
    public java.awt.Dimension getPreferredSize() {
        return new java.awt.Dimension( 350, super.getPreferredSize().height );
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jTextArea = new javax.swing.JTextArea();
        jList = new javax.swing.JList();
        addButton = new javax.swing.JButton();
        removeButton = new javax.swing.JButton();
        nextLabel = new javax.swing.JLabel();
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        jTextArea.setDisabledTextColor(java.awt.Color.black);
        jTextArea.setLineWrap(true);
        jTextArea.setFont(new java.awt.Font ("Dialog", 0, 11));
        jTextArea.setBackground(new java.awt.Color (204, 204, 204));
        jTextArea.setWrapStyleWord(true);
        jTextArea.setText(bundle.getString("LAB_21"));
        jTextArea.setEnabled(false);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.gridwidth = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 12, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(jTextArea, gridBagConstraints1);
        
        
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.gridheight = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(jList, gridBagConstraints1);
        
        
        addButton.setText(bundle.getString("BTN_Add"));
        addButton.setMnemonic(bundle.getString("BTN_Add").charAt(0));
        addButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addButtonActionPerformed(evt);
            }
        }
        );
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 12, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        add(addButton, gridBagConstraints1);
        
        
        removeButton.setText(bundle.getString("BTN_Remove"));
        removeButton.setMnemonic(bundle.getString("BTN_Remove").charAt(0));
        removeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                removeButtonActionPerformed(evt);
            }
        }
        );
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(12, 12, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        add(removeButton, gridBagConstraints1);
        
        
        nextLabel.setText(bundle.getString("LAB_22"));
        nextLabel.setForeground(java.awt.Color.black);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.gridwidth = 0;
        gridBagConstraints1.insets = new java.awt.Insets(12, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(nextLabel, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void removeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_removeButtonActionPerformed
        Object[] sel = jList.getSelectedValues();
        for (int i=0;i<sel.length;i++)
            modulesModel.removeElement(sel[i]);
        if ( modulesModel.getSize() == 0 )
            validator.setValid( false );
    }//GEN-LAST:event_removeButtonActionPerformed

    private void addButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addButtonActionPerformed
        JFileChooser chooser = new JFileChooser();
        chooser.setFileSelectionMode( JFileChooser.FILES_AND_DIRECTORIES );
        chooser.addChoosableFileFilter( NBM_FILE_FILTER );
        chooser.setFileFilter(NBM_FILE_FILTER);
        chooser.setDialogTitle(bundle.getString("CTL_FileChooser_Title"));

        if ( defaultDir != null) {
            chooser.setCurrentDirectory( defaultDir );
        }

        if (chooser.showDialog(TopManager.getDefault ().getWindowManager ().getMainWindow (),
                               bundle.getString("CTL_FileChooser_Approve_Button"))
                == JFileChooser.APPROVE_OPTION) {

            File dir = chooser.getSelectedFile();

            File[] files = null;

            if ( dir.isDirectory() ) {
                files = dir.listFiles( new FileFilter() {
                                           public boolean accept( File file ) {
                                               if ( !file.isDirectory() && file.getName().toUpperCase().endsWith(".NBM") ) // NOI18N
                                                   return true;
                                               else
                                                   return false;
                                           }
                                       } );
                if (files.length == 0)
                    TopManager.getDefault().notify(
                            new NotifyDescriptor.Message(bundle.getString("MSG_NoNbmInDir"),
                                                         NotifyDescriptor.WARNING_MESSAGE)
                        );
            }
            else {
                files = new File[] { dir };
            }

            for ( int i = 0; i < files.length; i++ ) {
                if ( i == 0 )
                    validator.setValid( true );

                if ( !selectedFile( files[i] ) ) {
                    modulesModel.addElement(files[i]);
                }
            }
        }

        defaultDir = chooser.getCurrentDirectory();
    }//GEN-LAST:event_addButtonActionPerformed
        
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextArea jTextArea;
    private javax.swing.JList jList;
    private javax.swing.JButton addButton;
    private javax.swing.JButton removeButton;
    private javax.swing.JLabel nextLabel;
    // End of variables declaration//GEN-END:variables

    private static final java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("org/netbeans/modules/autoupdate/Bundle");    
    
    private boolean selectedFile( File file ) {
        for ( int i = 0; i < modulesModel.size(); i++ ) {
            if ( ((File)modulesModel.elementAt(i)).compareTo( file ) == 0 )
                return true;
        }

        return false;
    }
    
    File[] getFiles() {
        File[] files = new File[ modulesModel.size() ];

        for ( int i = 0; i < modulesModel.size(); i ++ ) {
            files[i] = (File)modulesModel.elementAt( i );
        }

        return files;
    }

    /** Clears all modules in listbox, ceates new model,
     * if the model does not exist yet
     */
    void reset() {
        if ( modulesModel == null ) {
            modulesModel = new DefaultListModel();
            jList.setModel( modulesModel );
        }
        else {
            modulesModel.removeAllElements();
        }

    }

    private static class NbmFileFilter extends javax.swing.filechooser.FileFilter {
        public boolean accept( File f ) {
            return f.getName().toLowerCase().endsWith( ".nbm" ) || f.isDirectory(); // NOI18N
        }

        public String getDescription() {
            return bundle.getString( "CTL_FileFilterDescription" );
        }
    }    
    
}
