/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.autoupdate;

import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.beans.*;
import java.util.ArrayList;

import org.openide.WizardDescriptor;
import org.openide.explorer.propertysheet.PropertyPanel;
import org.openide.util.HelpCtx;

/**
 *
 * @author  akemr
 */
public class SetupPanel extends javax.swing.JPanel 
                        implements WizardDescriptor.FinishPanel,
                                   PropertyChangeListener {

    private static final java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("org/netbeans/modules/autoupdate/Bundle");
    
    private Settings settings;
    
    /** listeners on validation of panel changes */
    private ArrayList changeListeners = new ArrayList(1);
        
    private PropertyPanel lPanel;
    
    /** Creates new form SetupPanel */
    public SetupPanel() {
        settings = Settings.getShared();
        
        initComponents ();
        jLabel1.setDisplayedMnemonic (bundle.getString ("AutoCheckInfo.jLabel2.mnemonic").charAt (0));
        jLabel1.setLabelFor (periodComboBox);
        beforeCheckBox.setMnemonic (bundle.getString ("AutoCheckInfo.jCheckBox1.mnemonic").charAt (0));
        negativCheckBox.setMnemonic (bundle.getString ("AutoCheckInfo.jCheckBox2.mnemonic").charAt (0));
        setName(bundle.getString ("CTL_Settings_Name"));
        putClientProperty ("WizardPanel_contentData", new String[] {getName ()});
        putClientProperty ("WizardPanel_contentSelectedIndex", new Integer (0));

        String[] tags = (new Settings.PeriodPropertyEditor()).getTags();
        for ( int i = 0; i < tags.length; i++ )
            periodComboBox.addItem( tags[i] );

        initializePanel();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        periodPanel = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        periodComboBox = new javax.swing.JComboBox();
        checkPanel = new javax.swing.JPanel();
        beforeCheckBox = new javax.swing.JCheckBox();
        negativCheckBox = new javax.swing.JCheckBox();
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        periodPanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints2;
        jLabel1.setText(bundle.getString("AutoCheckInfo.jLabel2.text"));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 0;
        gridBagConstraints2.gridy = 0;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints2.ipadx = 8;
        gridBagConstraints2.ipady = 14;
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        periodPanel.add(jLabel1, gridBagConstraints2);
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 1;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        periodPanel.add(periodComboBox, gridBagConstraints2);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints1.weightx = 1.0;
        add(periodPanel, gridBagConstraints1);
        
        checkPanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints3;
        beforeCheckBox.setText(bundle.getString("AutoCheckInfo.jCheckBox1.text"));
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.gridx = 0;
        gridBagConstraints3.gridy = 0;
        gridBagConstraints3.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints3.insets = new java.awt.Insets(12, 20, 0, 0);
        gridBagConstraints3.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints3.weightx = 1.0;
        checkPanel.add(beforeCheckBox, gridBagConstraints3);
        
        negativCheckBox.setText(bundle.getString("AutoCheckInfo.jCheckBox2.text"));
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.gridx = 0;
        gridBagConstraints3.gridy = 1;
        gridBagConstraints3.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints3.insets = new java.awt.Insets(5, 20, 0, 0);
        gridBagConstraints3.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints3.weightx = 1.0;
        checkPanel.add(negativCheckBox, gridBagConstraints3);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(checkPanel, gridBagConstraints1);
        
    }//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel periodPanel;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JComboBox periodComboBox;
    private javax.swing.JPanel checkPanel;
    private javax.swing.JCheckBox beforeCheckBox;
    private javax.swing.JCheckBox negativCheckBox;
    // End of variables declaration//GEN-END:variables

    /** Initialize panel from NB setting
     */
    private void initializePanel() {
        settings = Settings.getShared();
        int period = settings.getPeriod();
        
        beforeCheckBox.setSelected( settings.isAskBefore() );
        negativCheckBox.setSelected( settings.isNegativeResults() );

        periodComboBox.setSelectedIndex( period );
        
        if ( lPanel != null ) checkPanel.remove(lPanel);
        lPanel = new PropertyPanel(settings, "autoupdateType", PropertyPanel.PREF_INPUT_STATE );
        
        java.awt.GridBagConstraints gridBagConstraints4 = new java.awt.GridBagConstraints();
        gridBagConstraints4.insets = new java.awt.Insets(16, 20, 0, 0);
        gridBagConstraints4.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints4.gridy = 2;
        gridBagConstraints4.weighty = 1.0;
        checkPanel.add(lPanel, gridBagConstraints4);        
    }
    
    /** Provides the wizard panel with the current data--either
    * the default data or already-modified settings, if the user used the previous and/or next buttons.
    * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
    * @param settings the object representing wizard panel state, as originally supplied to
    * {@link WizardDescriptor#WizardDescriptor(WizardDescriptor.Iterator,Object)}
    */
    public void readSettings (Object settings) {
        if (settings instanceof WizardDescriptor) {
            WizardDescriptor wd = (WizardDescriptor)settings;
            wd.addPropertyChangeListener(this);
            if (Boolean.TRUE.equals(wd.getProperty("initializePanel")))
                initializePanel();
        }
    }
    
    /** Handling of property changes in node structure and setup wizard descriptor
     */
    public void propertyChange(PropertyChangeEvent evt) {
        if ((evt.getSource() instanceof WizardDescriptor) 
            && (WizardDescriptor.PROP_VALUE.equals(evt.getPropertyName()))) {
                
            WizardDescriptor wd = (WizardDescriptor)evt.getSource();
            if (wd.getValue() == wd.FINISH_OPTION) {
                wd.removePropertyChangeListener(this);
                settings.setAskBefore( beforeCheckBox.isSelected() );
                settings.setNegativeResults( negativCheckBox.isSelected() );
                settings.setPeriod( periodComboBox.getSelectedIndex() );                
            }
        }
    }
    
    /** Get the component displayed in this panel.
    * @return the component
    */
    public java.awt.Component getComponent () {
        return this;
    }
    
    /** Help for this panel.
    * When the panel is active, this is used as the help for the wizard dialog.
    * @return the help or <code>null</code> if no help is supplied
    */
    public HelpCtx getHelp () {
        return null;
    }
    
    /** Provides the wizard panel with the opportunity to update the
    * settings with its current customized state.
    * Rather than updating its settings with every change in the GUI, it should collect them,
    * and then only save them when requested to by this method.
    * Also, the original settings passed to {@link #readSettings} should not be modified (mutated);
    * rather, the (copy) passed in here should be mutated according to the collected changes.
    * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
    * @param settings the object representing a settings of the wizard
    */
    public void storeSettings (Object settings) {
    }

    /** Test whether the panel is finished and it is safe to proceed to the next one.
    * If the panel is valid, the "Next" (or "Finish") button will be enabled.
    * @return <code>true</code> if the user has entered satisfactory information
    */
    public boolean isValid () {
        return true;
    }
    
    /** Add a listener to changes of the panel's validity.
    * @param l the listener to add
    * @see #isValid
    */
    public void addChangeListener (ChangeListener l) {
        if (!changeListeners.contains(l)) {
            changeListeners.add(l);
        }
    }

    /** Remove a listener to changes of the panel's validity.
    * @param l the listener to remove
    */
    public void removeChangeListener (ChangeListener l) {
        changeListeners.remove(l);
    }
    
    /** Notifyes listeners that panel's validation changed */
    private void fireStateChanged() {
        for(int i = 0; i < changeListeners.size(); i++)
            ((ChangeListener)changeListeners.get(i)).stateChanged(
                new ChangeEvent(this));
    }
}
