/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.autoupdate;

import java.util.ResourceBundle;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.util.Collection;
import java.util.Iterator;
import java.net.URL;
import java.net.MalformedURLException;
import javax.swing.border.TitledBorder;
import javax.swing.DefaultListModel;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ChangeListener;

import org.openide.util.HelpCtx;
import org.openide.WizardDescriptor;
import org.openide.NotifyDescriptor;
import org.openide.nodes.Node;
import org.openide.TopManager;
import org.openide.explorer.ExplorerPanel;
import org.openide.explorer.ExplorerManager;
import org.openide.explorer.view.BeanTreeView;

public class UpdatePanel extends javax.swing.JPanel {

    private static final String EMPTY_STRING = ""; // NOI18N
    
    private static final java.awt.Dimension PREFERRED_SIZE = new java.awt.Dimension( 600, 500 );

    private static final String SPACE = " "; //NOI18N

    /** Explorer tree */
    private ExplorerView explorerView;
    /** List model for listBox with selected modules */
    private DefaultListModel selectedListModel;
    /** Summation of lengths of all selected modules */
    private long totalSize;

    /** The collection of module updates */
    private Updates updates;

    private Wizard.Validator validator;
    
    private UpdateNode.Group allNodes;

    static final long serialVersionUID =897622109141801200L;
    /** Creates new form UpdatePanel */
    public UpdatePanel( Wizard.Validator validator ) {
        putClientProperty("WizardPanel_autoWizardStyle", new Boolean(true));
        putClientProperty("WizardPanel_contentDisplayed", new Boolean(true));
        putClientProperty("WizardPanel_contentNumbered", new Boolean(true));
        putClientProperty("WizardPanel_contentSelectedIndex", new Integer(1));
        putClientProperty("WizardPanel_contentData", new String[] {
            bundle.getString("WIZ_First"),            
            bundle.getString("WIZ_Select"),
            bundle.getString("WIZ_Download"),
            bundle.getString("WIZ_View")
        });
        initComponents();
        setName(bundle.getString("LAB_Select2"));
        
        this.validator = validator;

        // Add Explorer view
        explorerView = new ExplorerView();
        explorerView.requestFocus();

        explorerView.getExplorerManager().addPropertyChangeListener(
            new PropertyChangeListener() {
                public void propertyChange( PropertyChangeEvent evt ) {
                    if ( evt.getPropertyName() == ExplorerManager.PROP_SELECTED_NODES ) {
                        selectionChange();
                    }
                }
            } );                    

        java.awt.GridBagConstraints gridBagConstraints1 = new java.awt.GridBagConstraints ();
        gridBagConstraints1.gridwidth = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        gridBagConstraints1.insets = new java.awt.Insets (0, 3, 3, 3);
        jPanel1.add (explorerView, gridBagConstraints1);

        // Customize jList1

        jList1.setCellRenderer( new SelectedListCellRenderer() );
        selectedListModel = new DefaultListModel();
        jList1.setModel( selectedListModel );
        jList1.getSelectionModel().setSelectionMode( javax.swing.ListSelectionModel.SINGLE_SELECTION );
        jList1.getSelectionModel().addListSelectionListener(
            new ListSelectionListener() {
                public void valueChanged( javax.swing.event.ListSelectionEvent evt ) {
                    selectionChange();
                }
            }
        );
                
    }
    
    private boolean useRefresh() {
        ClassLoader cl = org.openide.TopManager.getDefault().systemClassLoader();
        try {
            Class qat = 
                Class.forName("org.netbeans.modules.autoupdate.QuantumAutoupdateType", true, cl);   //NOI18N
            if ( qat.isAssignableFrom(Settings.getShared().getAutoupdateType().getClass()) )
                return true;            
        } catch (Exception e) {            
        }
        return false;
    }
    
    /** Returns preferred size of the panel */
    public java.awt.Dimension getPreferredSize() {
        return PREFERRED_SIZE;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        panel1Label = new javax.swing.JLabel();
        jPanel1 = new javax.swing.JPanel();
        addButton = new javax.swing.JButton();
        removeButton = new javax.swing.JButton();
        addallButton = new javax.swing.JButton();
        removeallButton = new javax.swing.JButton();
        scrollpaneLabel = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        jList1 = new javax.swing.JList();
        refreshButton = new javax.swing.JButton();
        seeButton = new javax.swing.JButton();
        jPanel3 = new javax.swing.JPanel();
        totalsizeLabel = new javax.swing.JLabel();
        totalsizeTextField = new javax.swing.JTextField();
        jPanel2 = new javax.swing.JPanel();
        avLabel = new javax.swing.JLabel();
        avTextField = new javax.swing.JTextField();
        ivLabel = new javax.swing.JLabel();
        ivTextField = new javax.swing.JTextField();
        msLabel = new javax.swing.JLabel();
        msTextField = new javax.swing.JTextField();
        jScrollPane2 = new javax.swing.JScrollPane();
        jTextArea1 = new javax.swing.JTextArea();
        moreButton = new javax.swing.JButton();
        bottomLabel = new javax.swing.JLabel();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        panel1Label.setText(bundle.getString("LAB_221"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        add(panel1Label, gridBagConstraints1);
        
        jPanel1.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints2;
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.gridheight = 5;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(6, 0, 0, 0);
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(jPanel1, gridBagConstraints1);
        
        addButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/netbeans/modules/autoupdate/resources/add.gif")));
        addButton.setMnemonic(bundle.getString("BTN_221_Mnem").charAt(0));
        addButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 2;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(6, 12, 0, 12);
        add(addButton, gridBagConstraints1);
        
        removeButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/netbeans/modules/autoupdate/resources/rm.gif")));
        removeButton.setMnemonic(bundle.getString("BTN_222_Mnem").charAt(0));
        removeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                removeButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 2;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(6, 12, 0, 12);
        add(removeButton, gridBagConstraints1);
        
        addallButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/netbeans/modules/autoupdate/resources/add_all.gif")));
        addallButton.setMnemonic(bundle.getString("BTN_223_Mnem").charAt(0));
        addallButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addallButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 2;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(6, 12, 0, 12);
        add(addallButton, gridBagConstraints1);
        
        removeallButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/netbeans/modules/autoupdate/resources/rm_all.gif")));
        removeallButton.setMnemonic(bundle.getString("BTN_224_Mnem").charAt(0));
        removeallButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                removeallButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 2;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(6, 12, 0, 12);
        add(removeallButton, gridBagConstraints1);
        
        scrollpaneLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/autoupdate/Bundle").getString("LAB_222"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 3;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 0, 12);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        add(scrollpaneLabel, gridBagConstraints1);
        
        jList1.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                jList1MouseClicked(evt);
            }
        });
        
        jScrollPane1.setViewportView(jList1);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 3;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.gridwidth = 3;
        gridBagConstraints1.gridheight = 5;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(6, 0, 0, 0);
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(jScrollPane1, gridBagConstraints1);
        
        refreshButton.setMnemonic(bundle.getString("BTN_Ref_Mnem").charAt(0));
        refreshButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/autoupdate/Bundle").getString("BTN_Ref"));
        refreshButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                refreshButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 6;
        gridBagConstraints1.insets = new java.awt.Insets(12, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(refreshButton, gridBagConstraints1);
        
        seeButton.setMnemonic(bundle.getString("BTN_See_Mnem").charAt(0));
        seeButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/autoupdate/Bundle").getString("BTN_See"));
        seeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                seeButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 6;
        gridBagConstraints1.gridwidth = 3;
        gridBagConstraints1.insets = new java.awt.Insets(12, 12, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(seeButton, gridBagConstraints1);
        
        jPanel3.setLayout(new java.awt.GridLayout(1, 2, 6, 0));
        
        totalsizeLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/autoupdate/Bundle").getString("LAB_223"));
        totalsizeLabel.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jPanel3.add(totalsizeLabel);
        
        totalsizeTextField.setEditable(false);
        totalsizeTextField.setDisabledTextColor(java.awt.Color.black);
        totalsizeTextField.setMinimumSize(new java.awt.Dimension(6, 20));
        totalsizeTextField.setEnabled(false);
        jPanel3.add(totalsizeTextField);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 4;
        gridBagConstraints1.gridy = 6;
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(12, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints1.weightx = 1.0;
        add(jPanel3, gridBagConstraints1);
        
        jPanel2.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints3;
        
        jPanel2.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.TitledBorder(""), java.util.ResourceBundle.getBundle("org/netbeans/modules/autoupdate/Bundle").getString("TITLE22")));
        avLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/autoupdate/Bundle").getString("LAB_224"));
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.insets = new java.awt.Insets(0, 6, 0, 0);
        gridBagConstraints3.anchor = java.awt.GridBagConstraints.WEST;
        jPanel2.add(avLabel, gridBagConstraints3);
        
        avTextField.setEditable(false);
        avTextField.setDisabledTextColor(java.awt.Color.black);
        avTextField.setEnabled(false);
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.ipadx = 40;
        gridBagConstraints3.insets = new java.awt.Insets(0, 6, 0, 0);
        gridBagConstraints3.anchor = java.awt.GridBagConstraints.WEST;
        jPanel2.add(avTextField, gridBagConstraints3);
        
        ivLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/autoupdate/Bundle").getString("LAB_225"));
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.insets = new java.awt.Insets(0, 12, 0, 0);
        gridBagConstraints3.anchor = java.awt.GridBagConstraints.EAST;
        jPanel2.add(ivLabel, gridBagConstraints3);
        
        ivTextField.setEditable(false);
        ivTextField.setDisabledTextColor(java.awt.Color.black);
        ivTextField.setEnabled(false);
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints3.ipadx = 60;
        gridBagConstraints3.insets = new java.awt.Insets(0, 6, 0, 0);
        gridBagConstraints3.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints3.weightx = 1.0;
        jPanel2.add(ivTextField, gridBagConstraints3);
        
        msLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/autoupdate/Bundle").getString("LAB_226"));
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.insets = new java.awt.Insets(0, 12, 0, 0);
        gridBagConstraints3.anchor = java.awt.GridBagConstraints.EAST;
        jPanel2.add(msLabel, gridBagConstraints3);
        
        msTextField.setEditable(false);
        msTextField.setDisabledTextColor(java.awt.Color.black);
        msTextField.setMinimumSize(new java.awt.Dimension(6, 20));
        msTextField.setEnabled(false);
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints3.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints3.ipadx = 40;
        gridBagConstraints3.insets = new java.awt.Insets(0, 6, 0, 12);
        gridBagConstraints3.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints3.weightx = 1.0;
        jPanel2.add(msTextField, gridBagConstraints3);
        
        jScrollPane2.setMinimumSize(new java.awt.Dimension(22, 52));
        jTextArea1.setWrapStyleWord(true);
        jTextArea1.setLineWrap(true);
        jTextArea1.setEditable(false);
        jTextArea1.setRows(4);
        jTextArea1.setDisabledTextColor(java.awt.Color.black);
        jTextArea1.setEnabled(false);
        jScrollPane2.setViewportView(jTextArea1);
        
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.gridx = 0;
        gridBagConstraints3.gridy = 1;
        gridBagConstraints3.gridwidth = 6;
        gridBagConstraints3.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints3.insets = new java.awt.Insets(6, 6, 12, 0);
        gridBagConstraints3.weightx = 1.0;
        gridBagConstraints3.weighty = 1.0;
        jPanel2.add(jScrollPane2, gridBagConstraints3);
        
        moreButton.setMnemonic(bundle.getString("BTN_More_Mnem").charAt(0));
        moreButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/autoupdate/Bundle").getString("BTN_More"));
        moreButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                moreButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.gridx = 6;
        gridBagConstraints3.gridy = 1;
        gridBagConstraints3.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints3.insets = new java.awt.Insets(6, 12, 12, 12);
        gridBagConstraints3.anchor = java.awt.GridBagConstraints.SOUTH;
        jPanel2.add(moreButton, gridBagConstraints3);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 7;
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.gridheight = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(12, 0, 12, 0);
        add(jPanel2, gridBagConstraints1);
        
        bottomLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/autoupdate/Bundle").getString("LAB_227"));
        bottomLabel.setForeground(java.awt.Color.black);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 9;
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints1.weightx = 1.0;
        add(bottomLabel, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void seeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_seeButtonActionPerformed
        TopManager.getDefault().showUrl( getCompleteListURL() );
    }//GEN-LAST:event_seeButtonActionPerformed

    private void refreshButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_refreshButtonActionPerformed
        Downloader.deleteDownload();
        
        updates = Settings.getShared().getAutoupdateType().connectForUpdates();
        
        // refresh Wizard's updates
        if (validator instanceof Wizard.ConfigPanel) {
            ((Wizard.ConfigPanel)validator).setUpdates (updates);
        }
        
        updates.checkUpdates( new Wizard.Validator() {
                public void setValid( boolean valid ) {
                }
            });
            
        if ( updates.isError() )
            ConnectingErrorDialog.showDialog(updates.getError(),
                updates.getErrorMessage());                          
        else if ( updates.getModules() != null && updates.getModules().size() > 0 )
            Notification.performNotification( updates );                   
        
        setUpdates(updates, 0);
    }//GEN-LAST:event_refreshButtonActionPerformed

    private void removeallButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_removeallButtonActionPerformed
        for( int i = 0; i < selectedListModel.size(); i++ ) {
            ((ModuleUpdate)selectedListModel.get(i)).setSelected( false );
        }
        allNodes = new UpdateNode.Group( updates.getRootGroup());
        explorerView.setContext( allNodes );
        explorerView.expandAll();

        selectedListModel.clear();
        showSize();
        validator.setValid( selectedListModel.size() > 0 );
        
        buttonsCheck();
    }//GEN-LAST:event_removeallButtonActionPerformed

    private void moreButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_moreButtonActionPerformed
        ModuleUpdate mu = getSingleSelection();

        if ( mu != null && mu.getHomePage() != null ) {
            TopManager.getDefault().showUrl( mu.getHomePage() );
        }
    }//GEN-LAST:event_moreButtonActionPerformed

    private void addallButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addallButtonActionPerformed
        Collection modules = updates.getModules();

        Iterator it = modules.iterator();

        while ( it.hasNext() ) {
            if ( !addModule( (ModuleUpdate)it.next() ) ) {
                // Some licence rejected
                removeallButtonActionPerformed(evt);
                return;
            }
        }
        buttonsCheck();
        
    }//GEN-LAST:event_addallButtonActionPerformed

    private void removeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_removeButtonActionPerformed
        int index = jList1.getMinSelectionIndex();

        if ( index < 0 )
            return;

        removeModule( (ModuleUpdate)selectedListModel.get( index ) );
        validator.setValid( selectedListModel.size() > 0 );
        buttonsCheck();
    }//GEN-LAST:event_removeButtonActionPerformed

    private void addButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addButtonActionPerformed
        Node[] nodes = explorerView.getSelectedNodes();

        for ( int i = 0; i < nodes.length; i ++ ) {
            ModuleUpdate mu = (ModuleUpdate)nodes[i].getCookie( ModuleUpdate.class );
            if ( mu != null ) {
                if ( !addModule( mu ) ) {
                    return;
                }                                  
            }
        }
        buttonsCheck();
    }//GEN-LAST:event_addButtonActionPerformed

    private void jList1MouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jList1MouseClicked
        if ( evt.getClickCount() == 2 ) {
            int index = jList1.locationToIndex( evt.getPoint() );

            if ( index >= 0 && index < selectedListModel.size() ) {
                removeModule( (ModuleUpdate)selectedListModel.get( index ) );
                validator.setValid( selectedListModel.size() > 0 );
            }

        }
    }//GEN-LAST:event_jList1MouseClicked
     
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel panel1Label;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JButton addButton;
    private javax.swing.JButton removeButton;
    private javax.swing.JButton addallButton;
    private javax.swing.JButton removeallButton;
    private javax.swing.JLabel scrollpaneLabel;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JList jList1;
    private javax.swing.JButton refreshButton;
    private javax.swing.JButton seeButton;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JLabel totalsizeLabel;
    private javax.swing.JTextField totalsizeTextField;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JLabel avLabel;
    private javax.swing.JTextField avTextField;
    private javax.swing.JLabel ivLabel;
    private javax.swing.JTextField ivTextField;
    private javax.swing.JLabel msLabel;
    private javax.swing.JTextField msTextField;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JTextArea jTextArea1;
    private javax.swing.JButton moreButton;
    private javax.swing.JLabel bottomLabel;
    // End of variables declaration//GEN-END:variables
    
    private static final java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("org/netbeans/modules/autoupdate/Bundle");    
  
    void setUpdates( Updates updates, int wizardType ) {

        boolean firstTime = true;
        
        if (this.updates == updates)
            firstTime = false;        
            
        this.updates = updates;        
        
        if ( (wizardType == 1) || !useRefresh() ) {
            refreshButton.setVisible(false);
            seeButton.setVisible(false);
        } else {
            refreshButton.setVisible(true);
            seeButton.setVisible(true);
        }
            
            
        selectedListModel.clear();
        totalSize = 0;
        Iterator it = updates.getModules().iterator();

        while( it.hasNext() ) {
            ModuleUpdate mu = (ModuleUpdate)it.next();

            if ( mu.isSelected() ) {
                selectedListModel.addElement( mu );
                totalSize += mu.getDownloadSize();
            }
            
            if ( firstTime )
                if ( wizardType == 1 || mu.isPurchased() )
                    addModule(mu);                
        }
        validator.setValid( selectedListModel.size() > 0 );

        totalsizeTextField.setText( ( totalSize / 1024 ) + SPACE + bundle.getString("CTL_UpdatePanel_KB") );

        allNodes = new UpdateNode.Group( updates.getRootGroup());
        explorerView.setContext( allNodes );

        explorerView.expandAll();

        buttonsCheck();
        showSize();
    }

    // INNERCLAS ------------------------------------------------------
    static class ExplorerView extends ExplorerPanel  {

        private static ExplorerManager em;
        private BeanTreeView btv;

        static final long serialVersionUID =-5811911993587966912L;
        ExplorerView( ) {
            Node waitNode = new UpdateNode.Wait();
            createContent( waitNode );
        }

        private void createContent ( Node node ) {

            btv = new BeanTreeView ();
            btv.setPopupAllowed( false );
            btv.setDefaultActionAllowed( true );
            em = getExplorerManager ();

            //sp.add (new org.openide.explorer.view.ListView (), SplittedPanel.ADD_LEFT);
            setLayout (new java.awt.BorderLayout());
            add ( java.awt.BorderLayout.CENTER, btv );

            em.setRootContext ( node );
            em.setExploredContext( node );


            btv.setDefaultActionAllowed( true );
            btv.setRootVisible( false );
        }

        public java.awt.Dimension getPreferredSize () {
            java.awt.Dimension sup = super.getPreferredSize ();
            return new java.awt.Dimension ( Math.max (sup.width, 450), Math.max (sup.height, 300 ));
        }

        void expandAll() {
            btv.expandAll();
        }

        static Node[] getSelectedNodes() {
            return em.getSelectedNodes();
        }

        void setContext( Node node ) {
            em.setRootContext ( node );
            em.setExploredContext( node );
        }

    }

    private ModuleUpdate getSingleSelection() {
        if ( jList1.getSelectedIndex() >= 0 )
            // modules from right panel
            return ((ModuleUpdate)selectedListModel.get(jList1.getSelectedIndex()));
        else {
            // modules from left panel
            Node[] selectedNodes = explorerView.getSelectedNodes();
            if ( selectedNodes.length != 1 ) {
                return null;
            }
            else {
                return (ModuleUpdate)selectedNodes[0].getCookie( ModuleUpdate.class );
            }
        }
    }

    private void unselectLeft() {
        try {
            explorerView.getExplorerManager().setSelectedNodes( new Node[0]);
        } catch (java.beans.PropertyVetoException e) {
        }
    }
        
    /** Called when the selection in jList1 or explorerView changes */
    private void selectionChange() {
        ModuleUpdate mu;
        
        // prohibit selection on both left and right panels
        if ( jList1.hasFocus() && selectedListModel.getSize() > 0 ) {
            unselectLeft();            
        } else {
            jList1.clearSelection();            
        }
        mu = getSingleSelection();

        if ( mu == null ) {
            ivTextField.setText( EMPTY_STRING );
            avTextField.setText( EMPTY_STRING );
            msTextField.setText( EMPTY_STRING );
            jTextArea1.setText( EMPTY_STRING );
        }
        else {
            ivTextField.setText( mu.getLocalModule() == null ?
                                       bundle.getString( "CTL_UpdatePanel_ModuleNotInstalled" ) :
                                       mu.getLocalModule().getSpecVersion() );
            avTextField.setText( mu.getRemoteModule().getSpecVersion() );
            msTextField.setText( ( mu.getDownloadSize() / 1024 ) + SPACE + bundle.getString("CTL_UpdatePanel_KB") );
            jTextArea1.setText( mu.getDescription() );
        }
        
        buttonsCheck();
    }

    private void buttonsCheck() {
        if ( selectedListModel.getSize() > 0 ) {
            removeButton.setEnabled(true);
            removeallButton.setEnabled(true);
        } else {
            removeButton.setEnabled(false);
            removeallButton.setEnabled(false);
        }
        
        if ( existUnselectedModule() ) {
            addButton.setEnabled(true);
            addallButton.setEnabled(true);
        } else {
            addButton.setEnabled(false);
            addallButton.setEnabled(false);
        }

        ModuleUpdate mu = getSingleSelection();
        moreButton.setEnabled(mu != null && mu.getHomePage() != null);      
    }
    
    /** Adds module to selected list. Shows LicenceAgreement if needed.
     * Increases total size of download. Checks dependencies and adds 
     * all modules needed to satisfy all dependencies.
     *@return False if the licence was rejected
     */
    private boolean addModule( final ModuleUpdate mu ) {

        if ( selectedListModel.contains( mu ) )
            return true; // Already in download list

        // View module notification if exists
        if ( mu.getNotification() != null && mu.getNotification().length() > 0 
                && !mu.getNotificationAccepted() ) {
            if ( Notification.performModuleNotification( mu.getNotification() ) )
                mu.setNotificationAccepted( true );
            else
                return false;
        }
        
        // Module not selected yet and licence accepted

        selectedListModel.add( findPosition( mu ), mu );

        DependencyChecker dc = new DependencyChecker( updates );
        Collection modulesToAdd = dc.modulesToAdd( mu );
        Iterator it = modulesToAdd.iterator();
        while( it.hasNext() ) {
            ModuleUpdate addMu = (ModuleUpdate)it.next();
            if ( !addModule( addMu ) ) {
                selectedListModel.removeElement( mu );
                totalSize += mu.getDownloadSize();
                return false;
            }
        }

        showSize();
        validator.setValid( selectedListModel.size() > 0 );
        jList1.revalidate();
        
        mu.setSelected( true );
        allNodes = new UpdateNode.Group( updates.getRootGroup());
        explorerView.setContext( allNodes );        
        explorerView.expandAll();
        
        return true;
    }

    /** Shows LicenceAgreement if needed.
     *@return False if the licence was rejected
     */
    boolean checkLicencies() {
        ModuleUpdate mu;
        for( int j = 0; j < selectedListModel.size(); j++ ) {
            mu = (ModuleUpdate)selectedListModel.get(j);

            if ( mu.getLicenceText() != null && mu.getLicenceID() != null ) {

                // Test if the licence was already accepted
                boolean accepted = false;

                for( int i = 0; mu.getLicenceID() != null && i < j; i++ ) {

                    if( ((ModuleUpdate)selectedListModel.get(i)).getLicenceID() != null &&
                            ((ModuleUpdate)selectedListModel.get(i)).getLicenceID().equals( mu.getLicenceID() ) ) {
                        accepted = true;
                    }
                }

                if ( !accepted && !LicenceDialog.acceptLicence( mu.getLicenceText() ) ) {
                    return false;
                }
            }
        }
        return true;
    }
    
    /** Finds the right position of the module in the modules list */
    private int findPosition( ModuleUpdate mu ) {
        int pos = 0;

        Collection modules = updates.getModules();
        Iterator it = modules.iterator();
        while ( it.hasNext() ) {

            ModuleUpdate imu = ((ModuleUpdate)it.next());

            if ( mu == imu ) {
                return pos;
            }
            if ( selectedListModel.contains( imu ) ) {
                pos++;
            }
        }

        return pos;
    }

    /** Removes module at position <CODE>index</CODE>. Decreases total
     * size of download. Checks all dependencies and removes all dependent
     * modules.
     */   
    private void removeModule( final ModuleUpdate mu ) {

        if ( !selectedListModel.contains( mu ) )
            return; // is not selected

        final int index = selectedListModel.indexOf ( mu );
        totalSize -= mu.getDownloadSize();

        selectedListModel.remove( index );
        showSize();

        DependencyChecker dc = new DependencyChecker( updates );
        Collection modulesToRemove = dc.modulesToRemove( mu );
        Iterator it = modulesToRemove.iterator();
        while( it.hasNext() ) {
            ModuleUpdate removeMu = (ModuleUpdate)it.next();
            removeModule( removeMu );
        }

        mu.setSelected( false );
        allNodes = new UpdateNode.Group( updates.getRootGroup());
        explorerView.setContext( allNodes );
        explorerView.expandAll();

        validator.setValid( selectedListModel.size() > 0 );

    }

    void showSize() {
        totalSize = 0;

        for( int i = 0; i < selectedListModel.size(); i++ ) {
            if ( !((ModuleUpdate)selectedListModel.get(i)).isDownloadOK() ) {
                totalSize += ((ModuleUpdate)selectedListModel.get(i)).getDownloadSize();
            }
        }

        totalsizeTextField.setText( ( totalSize / 1024 ) + SPACE + bundle.getString("CTL_UpdatePanel_KB") );
    }

    int modulesToDownload() {
        int result = 0;

        for( int i = 0; i < selectedListModel.size(); i++ ) {
            if ( !((ModuleUpdate)selectedListModel.get(i)).isDownloadOK()
                  ||((ModuleUpdate)selectedListModel.get(i)).isNotChecked()) {
                result++;
            }
        }

        return result;
    }

    private boolean existUnselectedModule() {    
        Collection modules = updates.getModules();
        Iterator it = modules.iterator();
        while ( it.hasNext() )
            if ( ! ((ModuleUpdate)it.next()).isSelected() )
                return true;
        return false;
    }



    void markSelectedModules() {

        // We have firts to unmark all modules
        Collection modules = updates.getModules();
        Iterator it = modules.iterator();
        while ( it.hasNext() ) {
            ((ModuleUpdate)it.next()).setSelected( false );
        }

        // Then mark the selected

        for( int i = 0; i < selectedListModel.size(); i++ ) {
            ((ModuleUpdate)selectedListModel.get(i)).setSelected( true );
        }
    }

    private URL getCompleteListURL() {
        URL cURL = null;
        
        try {
            cURL = new URL( "http://www.sun.com/forte/ffj/resources/modules/index.html" );
        } catch ( MalformedURLException me ) {
        }
        return cURL;                
    }
}
