/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.autoupdate;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.ResourceBundle;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.text.ParsePosition;

import org.w3c.dom.*;

import org.openide.loaders.XMLDataObject;
import org.openide.TopManager;
import org.openide.modules.ModuleDescription;
import org.openide.util.*;
import org.openide.DialogDescriptor;
import org.openide.NotifyDescriptor;


/** Serves for building an UpdateCache from XML Document
 * @author  Petr Hrebejk
 */
public abstract class Updates extends Object {

    /** The ResourceBundle */
    private static final ResourceBundle bundle = NbBundle.getBundle( Updates.class );

    static final int NO_ERROR = 0;    
    static final int UNKNOWN_ERROR = 1;
    static final int AUTH_ERROR = 2;    
    static final int NO_SERVER_ERROR = 3;
    
    /** All installed modules */
    static protected ModuleDescription[] installedModules = Autoupdater.Support.getModuleDescriptions();
    
    /** Stops checking for new updates */
    protected boolean checkCanceled;    
    
    /** Checks for updates in separate thread. Displays progress in a dialog
     */
    public abstract void checkUpdates( final Wizard.Validator validator );

    public void cancelCheck() {
        checkCanceled = true;
    }

    /** Gets the root of the module/module group tree
     * @return The group in the root of the tree.
     */
    public abstract ModuleGroup getRootGroup();
        
    /** Gets the linear structure of all module updates i.e. Collection
     */
    public abstract Collection getModules();

    /** Gets the state of pError the file was not parsed */
    public abstract boolean isError();
    
    /** Gets the state of pError the file was not parsed */
    public abstract int getError();
    
    /** Gets the server error message if there is any */
    public abstract String getErrorMessage();

    /** Gets array of currently installed modules */
    public static ModuleDescription[] getInstalledModules() {
        return installedModules;
    }

    /** Gets Collection of patches installed in the system */
    public static ModuleDescription[] getInstalledPatches() {
        return PatchChecker.getPatches();
    }

    /** Returns the time stamp of the downloaded XML file */
    public abstract Date getTimeStamp();

    /** Returns notification text if specified otherwise null */
    public abstract String getNotificationText();

    /** Returns notification URL if specified otherwise null */
    public abstract URL getNotificationURL();
        
}
/*
 * Log
 *  17   Gandalf   1.16        2/23/00  Petr Hrebejk    Notifications added into
 *       autoupdate
 *  16   Gandalf   1.15        1/13/00  Petr Hrebejk    i18 mk3
 *  15   Gandalf   1.14        1/12/00  Petr Hrebejk    i18n mk2
 *  14   Gandalf   1.13        1/12/00  Petr Hrebejk    i18n
 *  13   Gandalf   1.12        1/10/00  Petr Hrebejk    Bug in setting last 
 *       stamp fixed
 *  12   Gandalf   1.11        1/3/00   Petr Hrebejk    Various bug fixes - 
 *       5097, 5098, 5110, 5099, 5108
 *  11   Gandalf   1.10        12/22/99 Petr Hrebejk    Various bugfixes
 *  10   Gandalf   1.9         12/20/99 Petr Hrebejk    Autocheck & security 
 *       finished
 *  9    Gandalf   1.8         12/1/99  Petr Hrebejk    Checkin signatures of 
 *       NBM files & automatic autoupdate check added
 *  8    Gandalf   1.7         11/8/99  Petr Hrebejk    Install of downloaded 
 *       modules added, Licenses in XML
 *  7    Gandalf   1.6         11/1/99  Petr Hrebejk    Remove of 
 *       org.netbeans.core.ModuleUpdater fixed
 *  6    Gandalf   1.5         10/22/99 Ian Formanek    NO SEMANTIC CHANGE - Sun
 *       Microsystems Copyright in File Comment
 *  5    Gandalf   1.4         10/11/99 Petr Hrebejk    Version before Beta 5
 *  4    Gandalf   1.3         10/10/99 Petr Hrebejk    AutoUpdate made to 
 *       wizard
 *  3    Gandalf   1.2         10/8/99  Petr Hrebejk    Next development version
 *  2    Gandalf   1.1         10/7/99  Petr Hrebejk    Next development version
 *  1    Gandalf   1.0         10/7/99  Petr Hrebejk    
 * $
 */
