/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/*
 * AddPropertyAction.java
 *
 * Created on 18. leden 2001, 13:26
 */

package org.netbeans.modules.beans;

import java.beans.Introspector;
import java.lang.reflect.Modifier;
import java.util.Iterator;

import org.openide.DialogDescriptor;
import org.openide.NotifyDescriptor;
import org.openide.TopManager;
import org.openide.nodes.Node;
import org.openide.util.NbBundle;
import org.openide.util.HelpCtx;
import org.openide.util.actions.CookieAction;
import org.openide.src.*;
//import org.openide.src.FieldElement;
import org.openide.cookies.SourceCookie;

import org.netbeans.modules.beans.*;

/**
 *
 * @author  Petr Suchomel
 * @version 0.1
 */
public class AddPropertyAction extends CookieAction {

    public java.lang.String getName() {
        return PatternNode.getString("CTL_MENU_GeneratePropertyFromField");
    }

    protected final Class[] cookieClasses () {
        return new Class[] { FieldElement.class, SourceCookie.Editor.class };
    }

    protected boolean enable ( Node[] activatedNodes ) {        
        if( activatedNodes != null && activatedNodes.length > 0 ){            
            for ( int i = 0; i < activatedNodes.length; i++ ) {                
                FieldElement fe = (FieldElement)activatedNodes[0].getCookie(FieldElement.class);
                if( fe == null || Modifier.isStatic(fe.getModifiers()) ){                                
                    return false;
                }
            }
            return true;
        }
        else
            return false;        
    }
    
    public org.openide.util.HelpCtx getHelpCtx() {
        return new HelpCtx (AddPropertyAction.class);
    }
    
    
    public int mode() {
        return MODE_ALL;//MODE_EXACTLY_ONE;
    }
    
    public void performAction( Node[] nodes ) {
        for ( int i = 0; i < nodes.length; i++ ) {
            FieldElement fe = (FieldElement)nodes[i].getCookie(FieldElement.class);        
            if( fe == null || Modifier.isStatic(fe.getModifiers()) )
                continue;
            PatternAnalyser analyser = new PatternAnalyser(fe.getDeclaringClass());        
            
            if( fe.getType().isArray() ){
                try {
                    String type = getType(fe.getType());
                    IdxPropertyPattern.create( analyser, Introspector.decapitalize(fe.getName().getName()) , type,
                            PropertyPattern.READ_ONLY, false, false, false, true, false, false, true, true, false, false);
                }catch(SourceException seEx){}
            }
            else {
                try {
                    PropertyPattern.create( analyser, fe.getName().getName() , fe.getType().getFullString(),
                            PropertyPattern.READ_ONLY, false, false, false, true, true, false);
                }catch(SourceException seEx){}
            }
            
            
            if( fe.getType().isArray() ){
                try {
                    String type = getType(fe.getType());
                    IdxPropertyPattern.create( analyser, Introspector.decapitalize(fe.getName().getName()) , type,
                            PropertyPattern.WRITE_ONLY, false, false, false, false, true, false, false, false, true, true);
                }catch(SourceException seEx){}
            }
            else {
                try {
                    PropertyPattern.create( analyser, fe.getName().getName() , fe.getType().getFullString(),
                            PropertyPattern.WRITE_ONLY, false, false, false, true, true, false);
                }catch(SourceException seEx){}
            }
        }
    }    
    
    private static String getType(org.openide.src.Type type){        
        if( type.isArray() ){
            return getType( type.getElementType() );
        }
        else{
            return type.getFullString();
        }        
    }

    protected String iconResource(){
        return "/org/netbeans/modules/beans/resources/generateRWfield.gif"; //NO I18N
    }
}
