/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.beaninfo.editors;

import java.util.*;
import java.awt.event.*;
import java.beans.*;
import java.io.File;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;

import org.openide.*;
import org.openide.explorer.*;
import org.openide.execution.*;
import org.openide.filesystems.FileSystem;
import org.openide.filesystems.LocalFileSystem;
import org.openide.filesystems.JarFileSystem;
import org.openide.nodes.*;
import org.openide.util.*;
import org.openide.util.datatransfer.NewType;
import org.openide.windows.TopComponent;
import org.openide.explorer.propertysheet.editors.EnhancedCustomPropertyEditor;
// TODO: check if we really need this
import org.netbeans.core.*;

/**
 * A panel capable of creating new FileSystem object.
 * @author  David Strupl
 */
public class FileSystemPanel extends TopComponent implements EnhancedCustomPropertyEditor {
    /** The new types of filesystems we are able to create. */
    private NewType newTypes[];
    /** The names displayed in typeComboBox */
    private String newTypeNames[];
    
    /** Remember last folder, which was explored in FileChooser.*/
    private static File lastDirFolder = null;
    /** Remember last folder, which was explored in FileChooser.*/
    private static File lastJarFolder = null;
    
    // remember both nodes and edited file systems?
    private FileSystem originalFS;
    private LocalFileSystem localFS;
    private JarFileSystem jarFS;
    private FileSystem otherFS;
    
    /** Currently edited file system*/
    private FileSystem fs;
    /** Node displayed in the propertySheet*/
    private Node activeNode;
    /** Node for the original value*/
    private Node originalNode;
    /** Node representing currently selected local file system. */
    private Node localFSNode;
    /** Node representing currently selected jar file system. */
    private Node jarFSNode;
    /** Node representing currently selected other file system. */
    private Node otherNode;
    /** Node displayed when no nodes are available*/
    private Node emptyNode;
    
    /** Creates new form FileSystemPanel */
    public FileSystemPanel() {
        // call the code generated by form editor
        initComponents ();
        // hand coded initiliazation
        initialize();
    }
    private void initComponents() {//GEN-BEGIN:initComponents
        group = new javax.swing.ButtonGroup();
        propertySheet1 = new org.openide.explorer.propertysheet.PropertySheet();
        dirRadioButton = new javax.swing.JRadioButton();
        jarRadioButton = new javax.swing.JRadioButton();
        otherRadioButton = new javax.swing.JRadioButton();
        originalRadioButton = new javax.swing.JRadioButton();
        dirLabel = new javax.swing.JLabel();
        jarLabel = new javax.swing.JLabel();
        createLabel = new javax.swing.JLabel();
        typeComboBox = new javax.swing.JComboBox();
        browseDirButton = new javax.swing.JButton();
        browseJarButton = new javax.swing.JButton();
        dirTextField = new javax.swing.JTextField();
        jarTextField = new javax.swing.JTextField();
        
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        setBorder(new javax.swing.border.CompoundBorder(new javax.swing.border.TitledBorder(java.util.ResourceBundle.getBundle("org/netbeans/beaninfo/editors/Bundle").getString("CTL_FileSystemPanel.Border_Title")), new javax.swing.border.EmptyBorder(new java.awt.Insets(8, 8, 8, 8))));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 7;
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(24, 0, 0, 0);
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(propertySheet1, gridBagConstraints1);
        
        dirRadioButton.setSelected(true);
        dirRadioButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/beaninfo/editors/Bundle").getString("CTL_FileSystemPanel.dirRadioButton.text"));
        dirRadioButton.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                dirRadioButtonItemStateChanged(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        add(dirRadioButton, gridBagConstraints1);
        
        jarRadioButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/beaninfo/editors/Bundle").getString("CTL_FileSystemPanel.jarRadioButton.text"));
        jarRadioButton.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                jarRadioButtonItemStateChanged(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        add(jarRadioButton, gridBagConstraints1);
        
        otherRadioButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/beaninfo/editors/Bundle").getString("CTL_FileSystemPanel.otherRadioButton.text"));
        otherRadioButton.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                otherRadioButtonItemStateChanged(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 5;
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        add(otherRadioButton, gridBagConstraints1);
        
        originalRadioButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/beaninfo/editors/Bundle").getString("CTL_FileSystemPanel.originalRadioButton.text"));
        originalRadioButton.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                originalRadioButtonItemStateChanged(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints1.weightx = 1.0;
        add(originalRadioButton, gridBagConstraints1);
        
        dirLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/beaninfo/editors/Bundle").getString("LAB_FileSystemPanel.dirLabel.text"));
        dirLabel.setLabelFor(dirTextField);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.insets = new java.awt.Insets(0, 24, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(dirLabel, gridBagConstraints1);
        
        jarLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/beaninfo/editors/Bundle").getString("LAB_FileSystemPanel.jarLabel.text"));
        jarLabel.setLabelFor(jarTextField);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.insets = new java.awt.Insets(0, 24, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(jarLabel, gridBagConstraints1);
        
        createLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/beaninfo/editors/Bundle").getString("LAB_FileSystemPanel.createLabel.text"));
        createLabel.setLabelFor(typeComboBox);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 6;
        gridBagConstraints1.insets = new java.awt.Insets(0, 24, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints1.weightx = 0.1;
        add(createLabel, gridBagConstraints1);
        
        typeComboBox.setPreferredSize(new java.awt.Dimension(200, 25));
        typeComboBox.setMinimumSize(new java.awt.Dimension(200, 25));
        typeComboBox.setEnabled(false);
        typeComboBox.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                typeComboBoxItemStateChanged(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 6;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints1.weightx = 0.8;
        add(typeComboBox, gridBagConstraints1);
        
        browseDirButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/beaninfo/editors/Bundle").getString("CTL_FileSystemPanel.browse"));
        browseDirButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                browseDirButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 2;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.insets = new java.awt.Insets(0, 8, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints1.weightx = 0.1;
        add(browseDirButton, gridBagConstraints1);
        
        browseJarButton.setToolTipText(java.util.ResourceBundle.getBundle("org/netbeans/beaninfo/editors/Bundle").getString("CTL_FileSystemPanel.browse"));
        browseJarButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/beaninfo/editors/Bundle").getString("CTL_FileSystemPanel.browse"));
        browseJarButton.setEnabled(false);
        browseJarButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                browseJarButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 2;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.insets = new java.awt.Insets(0, 8, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(browseJarButton, gridBagConstraints1);
        
        dirTextField.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                dirTextFieldFocusLost(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.weightx = 1.0;
        add(dirTextField, gridBagConstraints1);
        
        jarTextField.setEnabled(false);
        jarTextField.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                jarTextFieldFocusLost(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.weightx = 1.0;
        add(jarTextField, gridBagConstraints1);
        
    }//GEN-END:initComponents
    /** We check the path user typed in the jarTextField. If the path
     * is ok, we update the jarFS accordingly.
     */
  private void jarTextFieldFocusLost (java.awt.event.FocusEvent evt) {//GEN-FIRST:event_jarTextFieldFocusLost
            File f = new File(jarTextField.getText());
            if ((f != null) && (!f.exists())) {
                
                // TODO: prompt the user for jar file creation
                
                if (jarFS != null) {
                    jarTextField.setText(jarFS.getJarFile().getAbsolutePath());
                } else {
                    jarTextField.setText("");
                }
                return;
            }
            if ((f != null) && (f.isFile())) {
                try {
                    lastJarFolder = f.getParentFile();
                    jarFS = new ExJarFileSystem();
                    jarFS.setJarFile(f); 
                    createJarFSNode();
                    setActiveFS(jarFSNode, jarFS);
                    jarTextField.setText(f.getAbsolutePath());
                } catch (java.io.IOException e) {
                } catch (PropertyVetoException ex) {
                } catch (java.beans.IntrospectionException ex) {
                }
            } else {
                if (jarFS != null) {
                    jarTextField.setText(jarFS.getJarFile().getAbsolutePath());
                } else {
                    jarTextField.setText("");
                }
                // TODO: notify the user, that it is not a valid jar
            }

  }//GEN-LAST:event_jarTextFieldFocusLost
    /** We check the path user typed in the dirTextField. If the path
     * is ok, we update the localFS accordingly.
     */
  private void dirTextFieldFocusLost (java.awt.event.FocusEvent evt) {//GEN-FIRST:event_dirTextFieldFocusLost
            File f = new File(dirTextField.getText());
            if ((f != null) && (!f.exists())) {
                
                // TODO: prompt the user for creation
                
                if (localFS != null) {
                    dirTextField.setText(localFS.getRootDirectory().getAbsolutePath());
                } else {
                    dirTextField.setText(""); // NO-I18N
                }
                return;
            }
            if ((f != null) && (f.isDirectory())) {
                localFS = new org.netbeans.core.ExLocalFileSystem();
                try {
                    try {
                        f = f.getCanonicalFile ();
                    } catch (java.io.IOException ex) {
                        // ignore
                    }
                    lastDirFolder = f;

                    localFS.setRootDirectory (f);
                    createLocalFSNode();
                    setActiveFS(localFSNode, localFS);
                    dirTextField.setText(f.getAbsolutePath());
                } catch (java.io.IOException ex) {
                } catch (java.beans.PropertyVetoException ex) {
                } catch (java.beans.IntrospectionException ex) {
                }
            } else {
                if (localFS != null){
                    dirTextField.setText(localFS.getRootDirectory().getAbsolutePath());
                } else {
                    dirTextField.setText(""); // NO-I18N
                }
                
                // TODO: notify, that it is not a dir
                
            }

  }//GEN-LAST:event_dirTextFieldFocusLost

    /** If originalRadioButton is selected we display originalNode. */
  private void originalRadioButtonItemStateChanged (java.awt.event.ItemEvent evt) {//GEN-FIRST:event_originalRadioButtonItemStateChanged
        boolean selected = originalRadioButton.isSelected();
        if (selected) {
            if (originalNode == null) {
                setActiveFS(getEmptyNode(), null);
            } else {
                setActiveFS(originalNode, originalFS);
            }
        }
  }//GEN-LAST:event_originalRadioButtonItemStateChanged

    /** Brings up a JFileChooser to let the user select the jar file. */
  private void browseJarButtonActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_browseJarButtonActionPerformed

        JFileChooser chooser = new JFileChooser();
        chooser.setDialogType(JFileChooser.OPEN_DIALOG);
        HelpCtx.setHelpIDString (chooser, getHelpCtx ().getHelpID ());

        chooser.setFileFilter(new FileFilter() {
                                  public boolean accept(File f) {
                                      return (f.isDirectory() || f.getName().endsWith(".jar") || f.getName().endsWith(".zip")); // NOI18N
                                  }
                                  public String getDescription() {
                                      return getString("CTL_JarArchivesMask");
                                  }
                              });

        if (lastJarFolder != null) {
            chooser.setCurrentDirectory(lastJarFolder);
        }
        chooser.setDialogTitle(getString("CTL_FileSystemPanel.Jar_Dialog_Title"));
        while (Utilities.showJFileChooser(chooser, TopManager.getDefault ().getWindowManager ().getMainWindow (),
                               getString("CTL_FileSystemPanel.Approve_Button_Title"))
                == JFileChooser.APPROVE_OPTION) {
            File f = chooser.getSelectedFile();
            if ((f != null) && (f.isFile())) {
                try {
                    lastJarFolder = chooser.getCurrentDirectory();
                    jarFS = new ExJarFileSystem();
                    jarFS.setJarFile(f); 
                    createJarFSNode();
                    setActiveFS(jarFSNode, jarFS);
                    jarTextField.setText(f.getAbsolutePath());
                    return;
                } catch (java.io.IOException e) {
                } catch (PropertyVetoException ex) {
                } catch (java.beans.IntrospectionException ex) {
                }
            }
            TopManager.getDefault().notify(new DialogDescriptor.Message(getString("MSG_WrongFileName")));
        }

  }//GEN-LAST:event_browseJarButtonActionPerformed

    /** Brings up a JFileChooser to let the user select the directory. */
  private void browseDirButtonActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_browseDirButtonActionPerformed

        JFileChooser chooser = new JFileChooser();
        chooser.setDialogType(JFileChooser.OPEN_DIALOG);
        HelpCtx.setHelpIDString (chooser, getHelpCtx ().getHelpID ());
        chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        chooser.setDialogTitle(getString("CTL_FileSystemPanel.Local_Dialog_Title"));

        if (lastDirFolder != null) {
            chooser.setCurrentDirectory(lastDirFolder);
        }
        while (Utilities.showJFileChooser(chooser, TopManager.getDefault ().getWindowManager ().getMainWindow (),
                               getString("CTL_FileSystemPanel.Approve_Button_Title"))
                == JFileChooser.APPROVE_OPTION) {
            File f = chooser.getSelectedFile();
            if ((f != null) && (f.isDirectory())) {
                localFS = new org.netbeans.core.ExLocalFileSystem();
                try {
                    lastDirFolder = chooser.getCurrentDirectory();

                    try {
                        f = f.getCanonicalFile ();
                    } catch (java.io.IOException ex) {
                        // ignore
                    }

                    ((ExLocalFileSystem)localFS).setRootDirectory (f);
                    createLocalFSNode();
                    setActiveFS(localFSNode, localFS);
                    dirTextField.setText(f.getAbsolutePath());
                    return;
                } catch (java.io.IOException ex) {
                } catch (java.beans.PropertyVetoException ex) {
                } catch (java.beans.IntrospectionException ex) {
                }
            }
            TopManager.getDefault().notify(new DialogDescriptor.Message(getString("MSG_WrongFile")));
        }
//        chooser.dispose();
        
  }//GEN-LAST:event_browseDirButtonActionPerformed

    /** Display appropriate node (propertySheet). */
  private void jarRadioButtonItemStateChanged (java.awt.event.ItemEvent evt) {//GEN-FIRST:event_jarRadioButtonItemStateChanged
        boolean selected = jarRadioButton.isSelected();
        browseJarButton.setEnabled(selected);
        jarTextField.setEnabled(selected);
        if (selected) {
            if (jarFSNode == null) {
                setActiveFS(getEmptyNode(), null);
            } else {
                setActiveFS(jarFSNode, jarFS);
            }
        }

  }//GEN-LAST:event_jarRadioButtonItemStateChanged

    /** Display appropriate node (propertySheet). */
  private void dirRadioButtonItemStateChanged (java.awt.event.ItemEvent evt) {//GEN-FIRST:event_dirRadioButtonItemStateChanged
        boolean selected = dirRadioButton.isSelected();
        browseDirButton.setEnabled(selected);
        dirTextField.setEnabled(selected);
        if (selected) {
            if (localFSNode == null) {
                setActiveFS(getEmptyNode(), null);
            } else {
                setActiveFS(localFSNode, localFS);
            }
        }
  }//GEN-LAST:event_dirRadioButtonItemStateChanged

  private void typeComboBoxItemStateChanged (java.awt.event.ItemEvent evt) {//GEN-FIRST:event_typeComboBoxItemStateChanged
        otherNode = createOtherNode();
        if (otherNode != null) {
            setActiveFS(otherNode, otherFS);
        }
  }//GEN-LAST:event_typeComboBoxItemStateChanged

  private void otherRadioButtonItemStateChanged (java.awt.event.ItemEvent evt) {//GEN-FIRST:event_otherRadioButtonItemStateChanged
        if (otherRadioButton.isSelected()) {
            typeComboBox.setEnabled(true);
            if (otherNode == null) {
                otherNode = createOtherNode();
            }
            if (otherNode != null) {
                setActiveFS(otherNode, otherFS);
            }
        } else {
            typeComboBox.setEnabled(false);
        }
  }//GEN-LAST:event_otherRadioButtonItemStateChanged

  private void selectExisting (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_selectExisting
// Add your handling code here:
  }//GEN-LAST:event_selectExisting


  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.ButtonGroup group;
  private org.openide.explorer.propertysheet.PropertySheet propertySheet1;
  private javax.swing.JRadioButton dirRadioButton;
  private javax.swing.JRadioButton jarRadioButton;
  private javax.swing.JRadioButton otherRadioButton;
  private javax.swing.JRadioButton originalRadioButton;
  private javax.swing.JLabel dirLabel;
  private javax.swing.JLabel jarLabel;
  private javax.swing.JLabel createLabel;
  private javax.swing.JComboBox typeComboBox;
  private javax.swing.JButton browseDirButton;
  private javax.swing.JButton browseJarButton;
  private javax.swing.JTextField dirTextField;
  private javax.swing.JTextField jarTextField;
  // End of variables declaration//GEN-END:variables

  private static final java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("org/netbeans/beaninfo/editors/Bundle");  

    /** Sets the edited node and sets the fs variable.*/
    private void setActiveFS(Node n, FileSystem f) {
//        FileSystem old = fs;
        fs = f;
//        firePropertyChange("fileSystem", old, fs);// NO I18N
        activeNode = n;
        propertySheet1.setNodes(new Node[] { n } );
    }
    
    /** We remember just one emptyNode. */
    private Node getEmptyNode() {
        if (emptyNode == null) {
            emptyNode = new AbstractNode(Children.LEAF);
        }
        return emptyNode;
    }

    /** Finds types of actions available in typeComboBox.
     * Initiliazes newTypes, newTypeNames variables.
     * Called from the the constructor.
     */
    private void initialize() {
        // first find the newTypeNames 
        Node n = TopManager.getDefault().getPlaces ().nodes ().repository();
        newTypes = n.getNewTypes();
        Collection c = new ArrayList();
        for (int i = 0; i < newTypes.length; i++) {
            c.add(newTypes[i].getName());
        }
        newTypeNames = (String[])c.toArray(new String[c.size()]);
        // set the names in typeCombobox
        typeComboBox.setModel(new DefaultComboBoxModel(newTypeNames));
        
        group.add(originalRadioButton);
        group.add(dirRadioButton);
        group.add(jarRadioButton);
        group.add(otherRadioButton);
        
        if(typeComboBox.getItemCount() == 0)
            otherRadioButton.setEnabled(false);
        
        setOriginalValueVisibility();
    }

    /** Creates new node according to the state of typeComboBox */
    private Node createOtherNode() {
        int index = typeComboBox.getSelectedIndex();
        if ((index >= 0) && (index < newTypes.length)){
            if (newTypes[index] instanceof ModuleFSSection) {
                ModuleFSSection mfss = (ModuleFSSection)newTypes[index];
                try {
                    otherFS = mfss.createFileSystem();
                    return new BeanNode(otherFS);
                } catch (java.beans.IntrospectionException x) {
                    TopManager.getDefault().notifyException(x);
                } catch (InstantiationException x) {
                    TopManager.getDefault().notifyException(x);
                }
            } else {
//                System.out.println("New type is not ModuleFSSection.");
                // ignore ???
            }
        }
        return null;
    }
    
    private void createLocalFSNode() throws java.beans.IntrospectionException, 
                                        java.beans.IntrospectionException {
        localFSNode = new BeanNode(localFS);
        localFSNode.addPropertyChangeListener(new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent e) {
                setDirTextFieldText();
            }
        });
    }
    
    private void createJarFSNode() throws java.beans.IntrospectionException, 
                                        java.beans.IntrospectionException {
        jarFSNode = new BeanNode(jarFS);
        jarFSNode.addPropertyChangeListener(new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent e) {
                setJarTextFieldText();
            }
        });
    }
    
    /** Set the current file system. It tryies to avoid a cyclic
     * calls by detecting if the new one is the same as the old
     * one - in this case does nothing.
     */
    public void setFileSystem(FileSystem f) {
        try {
            if ((f != null) && (fs != f)){
                // now I hope that setFileSystem is called from outside world
                originalFS = f;
                originalNode = new BeanNode(originalFS);
                setActiveFS(originalNode, originalFS);
            }
        } catch (Exception x) {
            TopManager.getDefault().notifyException(x);
        }
        setOriginalValueVisibility();
    }
    
    /** Returns currently selected file system */
    public FileSystem getFileSystem() {
        return fs;
    }
    
    /** Method called from propertyChangeListener
     * It sets the dirTextField's text to the value obtained
     * from localFS.
     */
    private void setDirTextFieldText() {
        if (localFS != null) {
            dirTextField.setText(localFS.getRootDirectory().getAbsolutePath());
        }
    }
    
    /** Method called from propertyChangeListener
     * It sets the jarTextField's text to the value obtained
     * from jarFS.
     */
    private void setJarTextFieldText() {
        if (jarFS != null) {
            jarTextField.setText(jarFS.getJarFile().getAbsolutePath());
        }
    }
    
    /** Get the customized property value.
     * @return the property value
     * @exception InvalidStateException when the custom property editor does not contain a valid property value
     *           (and thus it should not be set)
     */
    public Object getPropertyValue() throws IllegalStateException {
        return getFileSystem();
    }

    private static final String getString(String s) {
        return NbBundle.getBundle(FileSystemPanel.class).getString(s);
    }

    /** If fs == null the ofiginal value will not be displayed */
    private void setOriginalValueVisibility() {
        if (originalFS == null) {
            originalRadioButton.setVisible(false);
        } else {
            originalRadioButton.setVisible(true);
        }
    }
}
