/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.beaninfo.swing;

import java.awt.Image;
import java.beans.*;
import java.util.ResourceBundle;

public class EtchedBorderBeanInfo extends SimpleBeanInfo {

    private static PropertyDescriptor propertyDescriptors[] = null;
    private static Image icon = null;
    private static Image icon32 = null;
    private static String path = "org/netbeans/beaninfo/swing/";

    private static void createPropertyDescriptors() {
        try {
            ResourceBundle bundle = ResourceBundle.getBundle(path + "Bundle");

            PropertyDescriptor pd1 = new PropertyDescriptor("etchType",
                                         javax.swing.border.EtchedBorder.class,
                                         "getEtchType", null);
            pd1.setDisplayName(bundle.getString("PROP_EtchType"));
            pd1.setShortDescription(bundle.getString("HINT_EtchType"));
            pd1.setPropertyEditorClass(EtchTypePropertyEditor.class);

            if (!System.getProperty("java.version", "1.3").startsWith("1.2")) {
                PropertyDescriptor pd2 = new PropertyDescriptor("highlightColor",
                                             javax.swing.border.EtchedBorder.class,
                                             "getHighlightColor", null);
                pd2.setDisplayName(bundle.getString("PROP_Highlight"));
                pd2.setShortDescription(bundle.getString("HINT_Highlight"));

                PropertyDescriptor pd3 = new PropertyDescriptor("shadowColor",
                                             javax.swing.border.EtchedBorder.class,
                                             "getShadowColor", null);
                pd3.setDisplayName(bundle.getString("PROP_Shadow"));
                pd3.setShortDescription(bundle.getString("HINT_Shadow"));

                propertyDescriptors = new PropertyDescriptor[3];
                propertyDescriptors[0] = pd1;
                propertyDescriptors[1] = pd2;
                propertyDescriptors[2] = pd3;
            }
            else {
                // highlightColor and shadowColor not in JDK 1.2
                propertyDescriptors = new PropertyDescriptor[1];
                propertyDescriptors[0] = pd1;
            }
        }
        catch(IntrospectionException ex) {
            ex.printStackTrace();
        }
    }

    public PropertyDescriptor[] getPropertyDescriptors() {
        if (propertyDescriptors == null)
            createPropertyDescriptors();
        return propertyDescriptors;
    }

    public Image getIcon(int type) {
        if (icon == null) {
            icon = loadImage("/" + path + "etchedBorder.gif");
            icon32 = loadImage("/" + path + "etchedBorder32.gif");
        }
        return type == ICON_COLOR_32x32 || type == ICON_MONO_32x32 ?
               icon32 : icon;
    }


    public static class EtchTypePropertyEditor extends PropertyEditorSupport {

        private static String tags[] = null;

        private static final int values[] = {
            javax.swing.border.EtchedBorder.LOWERED,
            javax.swing.border.EtchedBorder.RAISED
        };

        private static String[] javaInitStrings = {
            "javax.swing.border.EtchedBorder.LOWERED",
            "javax.swing.border.EtchedBorder.RAISED"
        };

        public String[] getTags() {
            if (tags == null) createTags();
            return tags;
        }

        public String getAsText() {
            Object valObj = getValue();
            if (valObj instanceof Integer) {
                if (tags == null) createTags();

                int value = ((Integer)valObj).intValue();
                for (int i = 0; i < values.length; i++)
                    if (value == values[i])
                        return tags[i];
            }
            return null;
        }

        public void setAsText(String str) {
            if (tags == null) createTags();

            int value = -1;

            for (int i = 0; i < tags.length; i++) {
                if (str.equals(tags[i])) {
                    value = values[i];
                    break;
                }
            }

            if (value != -1)
                setValue(new Integer(value));
        }

        public String getJavaInitializationString() {
            Object valObj = getValue();
            if (valObj instanceof Integer) {
                if (tags == null) createTags();

                int value = ((Integer)valObj).intValue();
                for (int i = 0; i < values.length; i++)
                    if (value == values[i])
                        return javaInitStrings[i];
            }
            return "???";
        }

        private static void createTags() {
            ResourceBundle bundle = ResourceBundle.getBundle(EtchedBorderBeanInfo.path + "Bundle");
            tags = new String[2];
            tags[0] = bundle.getString("VALUE_EtchLowered");
            tags[1] = bundle.getString("VALUE_EtchRaised");
        }
    }
}
