/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.beaninfo.swing;

import java.awt.Image;
import java.beans.*;
import java.util.ResourceBundle;

public class FlowLayoutBeanInfo extends SimpleBeanInfo {

    private static PropertyDescriptor propertyDescriptors[] = null;
    private static Image icon = null;
    private static Image icon32 = null;
    private static String path = "org/netbeans/beaninfo/swing/";

    private static void createPropertyDescriptors() {
        try {
            ResourceBundle bundle = ResourceBundle.getBundle(path + "Bundle");

            PropertyDescriptor pd1 = new PropertyDescriptor("alignment",
                                           java.awt.FlowLayout.class,
                                           "getAlignment", "setAlignment");
            pd1.setDisplayName(bundle.getString("PROP_Alignment"));
            pd1.setShortDescription(bundle.getString("HINT_Alignment"));
            pd1.setPropertyEditorClass(AlignmentPropertyEditor.class);

            PropertyDescriptor pd2 = new PropertyDescriptor("hgap",
                                           java.awt.FlowLayout.class,
                                           "getHgap", "setHgap");
            pd2.setDisplayName(bundle.getString("PROP_Hgap"));
            pd2.setShortDescription(bundle.getString("HINT_Hgap"));

            PropertyDescriptor pd3 = new PropertyDescriptor("vgap",
                                           java.awt.FlowLayout.class,
                                           "getVgap", "setVgap");
            pd3.setDisplayName(bundle.getString("PROP_Vgap"));
            pd3.setShortDescription(bundle.getString("HINT_Vgap"));

            propertyDescriptors = new PropertyDescriptor[3];
            propertyDescriptors[0] = pd1;
            propertyDescriptors[1] = pd2;
            propertyDescriptors[2] = pd3;
        }
        catch(IntrospectionException ex) {
            ex.printStackTrace();
        }
    }

    public PropertyDescriptor[] getPropertyDescriptors() {
        if (propertyDescriptors == null)
            createPropertyDescriptors();
        return propertyDescriptors;
    }

    public Image getIcon(int type) {
        if ( icon == null) {
            icon = loadImage("/" + path + "flowLayout.gif");
            icon32 = loadImage("/" + path + "flowLayout32.gif");
        }
        return type == ICON_COLOR_32x32 || type == ICON_MONO_32x32 ?
               icon32 : icon;
    }


    public static class AlignmentPropertyEditor extends PropertyEditorSupport {
        private static String[] tags;

        private static int[] values = {
            java.awt.FlowLayout.CENTER,
            java.awt.FlowLayout.LEFT,
            java.awt.FlowLayout.RIGHT
        };

        private static String[] javaInitStrings = {
            "java.awt.FlowLayout.CENTER",
            "java.awt.FlowLayout.LEFT",
            "java.awt.FlowLayout.RIGHT"
        };

        public String[] getTags() {
            if (tags == null) createTags();
            return tags;
        }

        public String getAsText() {
            Object valObj = getValue();
            if (valObj instanceof Integer) {
                if (tags == null) createTags();

                int value = ((Integer)valObj).intValue();
                for (int i = 0; i < values.length; i++)
                    if (value == values[i])
                        return tags[i];
            }
            return null;
        }

        public void setAsText(String str) {
            if (tags == null) createTags();

            int value = -1;

            for (int i = 0; i < tags.length; i++) {
                if (str.equals(tags[i])) {
                    value = values[i];
                    break;
                }
            }

            if (value != -1)
                setValue(new Integer(value));
        }

        public String getJavaInitializationString() {
            Object valObj = getValue();
            if (valObj instanceof Integer) {
                int value = ((Integer)valObj).intValue();
                for (int i = 0; i < values.length; i++)
                    if (value == values[i])
                        return javaInitStrings[i];
            }
            return "???";
        }

        private static void createTags() {
            ResourceBundle bundle = ResourceBundle.getBundle(FlowLayoutBeanInfo.path + "Bundle");
            tags = new String[3];
            tags[0] = bundle.getString("VALUE_AlignmentCenter");
            tags[1] = bundle.getString("VALUE_AlignmentLeft");
            tags[2] = bundle.getString("VALUE_AlignmentRight");
        }
    }
}
