/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.beaninfo.swing;

import java.awt.Image;
import java.beans.*;
import java.util.ResourceBundle;

public class LineBorderBeanInfo extends SimpleBeanInfo {

    private static PropertyDescriptor propertyDescriptors[] = null;
    private static Image icon = null;
    private static Image icon32 = null;
    private static String path = "org/netbeans/beaninfo/swing/";

    private static void createPropertyDescriptors() {
        try {
            ResourceBundle bundle = ResourceBundle.getBundle(path + "Bundle");

            PropertyDescriptor pd1 = new PropertyDescriptor("lineColor",
                                           javax.swing.border.LineBorder.class,
                                           "getLineColor", null);
            pd1.setDisplayName(bundle.getString("PROP_LineColor"));
            pd1.setShortDescription(bundle.getString("HINT_LineColor"));


            PropertyDescriptor pd2 = new PropertyDescriptor("thickness",
                                           javax.swing.border.LineBorder.class,
                                           "getThickness", null);
            pd2.setDisplayName(bundle.getString("PROP_Thickness"));
            pd2.setShortDescription(bundle.getString("HINT_Thickness"));

            if (!System.getProperty("java.version", "1.3").startsWith("1.2")) {
                PropertyDescriptor pd3 = new PropertyDescriptor("roundedCorners",
                                               javax.swing.border.LineBorder.class,
                                               "getRoundedCorners", null);
                pd3.setDisplayName(bundle.getString("PROP_RoundedCorners"));
                pd3.setShortDescription(bundle.getString("HINT_RoundedCorners"));

                propertyDescriptors = new PropertyDescriptor[3];
                propertyDescriptors[0] = pd1;
                propertyDescriptors[1] = pd2;
                propertyDescriptors[2] = pd3;
            }
            else {
                // roundedCorners not in JDK 1.2
                propertyDescriptors = new PropertyDescriptor[2];
                propertyDescriptors[0] = pd1;
                propertyDescriptors[1] = pd2;
            }
        }
        catch(IntrospectionException ex) {
            ex.printStackTrace();
        }
    }

    public PropertyDescriptor[] getPropertyDescriptors() {
        if (propertyDescriptors == null)
            createPropertyDescriptors();
        return propertyDescriptors;
    }

    public Image getIcon(int type) {
        if ( icon == null) {
            icon = loadImage("/" + path + "lineBorder.gif");
            icon32 = loadImage("/" + path + "lineBorder32.gif");
        }
        return type == ICON_COLOR_32x32 || type == ICON_MONO_32x32 ?
               icon32 : icon;
    }
}
