/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.beaninfo.swing;

import java.awt.Image;
import java.beans.*;
import java.util.ResourceBundle;

public class TitledBorderBeanInfo extends SimpleBeanInfo {
    
    private static PropertyDescriptor propertyDescriptors[] = null;
    private static Image icon = null;
    private static Image icon32 = null;
    private static String path = "org/netbeans/beaninfo/swing/";

    private static void createPropertyDescriptors() {
        try {
            ResourceBundle bundle = ResourceBundle.getBundle(path + "Bundle");

            PropertyDescriptor pd1 = new PropertyDescriptor("border",
                                         javax.swing.border.TitledBorder.class,
                                         "getBorder", "setBorder");
            pd1.setDisplayName(bundle.getString("PROP_Border"));
            pd1.setShortDescription(bundle.getString("HINT_Border"));

            PropertyDescriptor pd2 = new PropertyDescriptor("title",
                                         javax.swing.border.TitledBorder.class,
                                         "getTitle", "setTitle");
            pd2.setDisplayName(bundle.getString("PROP_Title"));
            pd2.setShortDescription(bundle.getString("HINT_Title"));

            PropertyDescriptor pd3 = new PropertyDescriptor("titleJustification",
                                         javax.swing.border.TitledBorder.class,
                                         "getTitleJustification", "setTitleJustification");
            pd3.setDisplayName(bundle.getString("PROP_TitleJustification"));
            pd3.setShortDescription(bundle.getString("HINT_TitleJustification"));
            pd3.setPropertyEditorClass(JustificationPropertyEditor.class);

            PropertyDescriptor pd4 = new PropertyDescriptor("titlePosition",
                                         javax.swing.border.TitledBorder.class,
                                         "getTitlePosition", "setTitlePosition");
            pd4.setDisplayName(bundle.getString("PROP_TitlePosition"));
            pd4.setShortDescription(bundle.getString("HINT_TitlePosition"));
            pd4.setPropertyEditorClass(PositionPropertyEditor.class);

            PropertyDescriptor pd5 = new PropertyDescriptor("titleColor",
                                         javax.swing.border.TitledBorder.class,
                                         "getTitleColor", "setTitleColor");
            pd5.setDisplayName(bundle.getString("PROP_TitleColor"));
            pd5.setShortDescription(bundle.getString("HINT_TitleColor"));

            PropertyDescriptor pd6 = new PropertyDescriptor("titleFont",
                                         javax.swing.border.TitledBorder.class,
                                         "getTitleFont", "setTitleFont");
            pd6.setDisplayName(bundle.getString("PROP_TitleFont"));
            pd6.setShortDescription(bundle.getString("HINT_TitleFont"));

            propertyDescriptors = new PropertyDescriptor[6];
            propertyDescriptors[0] = pd1;
            propertyDescriptors[1] = pd2;
            propertyDescriptors[2] = pd3;
            propertyDescriptors[3] = pd4;
            propertyDescriptors[4] = pd5;
            propertyDescriptors[5] = pd6;
        }
        catch(IntrospectionException ex) {
            ex.printStackTrace();
        }
    }

    public PropertyDescriptor[] getPropertyDescriptors() {
        if (propertyDescriptors == null)
            createPropertyDescriptors();
        return propertyDescriptors;
    }

    public Image getIcon(int type) {
        if (icon == null) {
            icon = loadImage("/" + path + "titledBorder.gif");
            icon32 = loadImage("/" + path + "titledBorder32.gif");
        }
        return type == ICON_COLOR_32x32 || type == ICON_MONO_32x32 ?
               icon32 : icon;
    }


    public static class PositionPropertyEditor extends PropertyEditorSupport {

        private static String tags[] = null;

        private static final int values[] = {
            javax.swing.border.TitledBorder.DEFAULT_POSITION,
            javax.swing.border.TitledBorder.ABOVE_TOP,
            javax.swing.border.TitledBorder.TOP,
            javax.swing.border.TitledBorder.BELOW_TOP,
            javax.swing.border.TitledBorder.ABOVE_BOTTOM,
            javax.swing.border.TitledBorder.BOTTOM,
            javax.swing.border.TitledBorder.BELOW_BOTTOM
        };

        private static String[] javaInitStrings = {
            "javax.swing.border.TitledBorder.DEFAULT_POSITION",
            "javax.swing.border.TitledBorder.ABOVE_TOP",
            "javax.swing.border.TitledBorder.TOP",
            "javax.swing.border.TitledBorder.BELOW_TOP",
            "javax.swing.border.TitledBorder.ABOVE_BOTTOM",
            "javax.swing.border.TitledBorder.BOTTOM",
            "javax.swing.border.TitledBorder.BELOW_BOTTOM"
        };

        public String[] getTags() {
            if (tags == null) createTags();
            return tags;
        }

        public String getAsText() {
            Object valObj = getValue();
            if (valObj instanceof Integer) {
                if (tags == null) createTags();

                int value = ((Integer)valObj).intValue();
                for (int i = 0; i < values.length; i++)
                    if (value == values[i])
                        return tags[i];
            }
            return null;
        }

        public void setAsText(String str) {
            if (tags == null) createTags();

            int value = -1;

            for (int i = 0; i < tags.length; i++) {
                if (str.equals(tags[i])) {
                    value = values[i];
                    break;
                }
            }

            if (value != -1)
                setValue(new Integer(value));
        }

        public String getJavaInitializationString() {
            Object valObj = getValue();
            if (valObj instanceof Integer) {
                if (tags == null) createTags();

                int value = ((Integer)valObj).intValue();
                for (int i = 0; i < values.length; i++)
                    if (value == values[i])
                        return javaInitStrings[i];
            }
            return "???";
        }

        private static void createTags() {
            ResourceBundle bundle = ResourceBundle.getBundle(TitledBorderBeanInfo.path + "Bundle");
            tags = new String[7];
            tags[0] = bundle.getString("VALUE_PosDefault");
            tags[1] = bundle.getString("VALUE_PosAboveTop");
            tags[2] = bundle.getString("VALUE_PosTop");
            tags[3] = bundle.getString("VALUE_PosBelowTop");
            tags[4] = bundle.getString("VALUE_PosAboveBottom");
            tags[5] = bundle.getString("VALUE_PosBottom");
            tags[6] = bundle.getString("VALUE_PosBelowBottom");
        }
    }

    public static class JustificationPropertyEditor extends PropertyEditorSupport {

        private static String tags[] = null;
        private static int values[] = null;
        private static String[] javaInitStrings = null;

        public String[] getTags() {
            if (tags == null) createTags();
            return tags;
        }

        public String getAsText() {
            Object valObj = getValue();
            if (valObj instanceof Integer) {
                if (tags == null) createTags();

                int value = ((Integer)valObj).intValue();
                for (int i = 0; i < values.length; i++)
                    if (value == values[i])
                        return tags[i];
            }
            return null;
        }

        public void setAsText(String str) {
            if (tags == null) createTags();

            int value = -1;

            for (int i = 0; i < tags.length; i++) {
                if (str.equals(tags[i])) {
                    value = values[i];
                    break;
                }
            }

            if (value != -1)
                setValue(new Integer(value));
        }

        public String getJavaInitializationString() {
            Object valObj = getValue();
            if (valObj instanceof Integer) {
                if (tags == null) createTags();

                int value = ((Integer)valObj).intValue();
                for (int i = 0; i < values.length; i++)
                    if (value == values[i])
                        return javaInitStrings[i];
            }
            return "???";
        }

        private static void createTags() {
            ResourceBundle bundle = ResourceBundle.getBundle(TitledBorderBeanInfo.path + "Bundle");
            if (System.getProperty("java.version", "1.3").startsWith("1.2")) {
                // there's no LEADING nor TRAILING until JDK 1.3
                tags = new String[4];
                tags[0] = bundle.getString("VALUE_JustDefault");
                tags[1] = bundle.getString("VALUE_JustLeft");
                tags[2] = bundle.getString("VALUE_JustCenter");
                tags[3] = bundle.getString("VALUE_JustRight");
                values = new int[4];
                values[0] = javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION;
                values[1] = javax.swing.border.TitledBorder.LEFT;
                values[2] = javax.swing.border.TitledBorder.CENTER;
                values[3] = javax.swing.border.TitledBorder.RIGHT;
                javaInitStrings = new String[4];
                javaInitStrings[0] = "javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION";
                javaInitStrings[1] = "javax.swing.border.TitledBorder.LEFT";
                javaInitStrings[2] = "javax.swing.border.TitledBorder.CENTER";
                javaInitStrings[3] = "javax.swing.border.TitledBorder.RIGHT";
            }
            else {
                tags = new String[6];
                tags[0] = bundle.getString("VALUE_JustDefault");
                tags[1] = bundle.getString("VALUE_JustLeft");
                tags[2] = bundle.getString("VALUE_JustCenter");
                tags[3] = bundle.getString("VALUE_JustRight");
                tags[4] = bundle.getString("VALUE_JustLeading");
                tags[5] = bundle.getString("VALUE_JustTrailing");
                values = new int[6];
                values[0] = javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION;
                values[1] = javax.swing.border.TitledBorder.LEFT;
                values[2] = javax.swing.border.TitledBorder.CENTER;
                values[3] = javax.swing.border.TitledBorder.RIGHT;
                values[4] = 4; // LEADING
                values[5] = 5; // TRAILING
                javaInitStrings = new String[6];
                javaInitStrings[0] = "javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION";
                javaInitStrings[1] = "javax.swing.border.TitledBorder.LEFT";
                javaInitStrings[2] = "javax.swing.border.TitledBorder.CENTER";
                javaInitStrings[3] = "javax.swing.border.TitledBorder.RIGHT";
                javaInitStrings[4] = "javax.swing.border.TitledBorder.LEADING";
                javaInitStrings[5] = "javax.swing.border.TitledBorder.TRAILING";
            }
        }
    }
}
