/*pr
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.core;

import java.awt.*;
import java.awt.datatransfer.Clipboard;
import java.awt.event.*;
import java.beans.*;
import java.io.*;
import java.net.URL;
import java.util.Enumeration;
import java.util.Vector;
import java.text.MessageFormat;
import javax.swing.*;
import javax.swing.border.*;

import org.openide.util.datatransfer.ExClipboard;
import org.openide.*;
import org.openide.cookies.ExecCookie;
import org.openide.loaders.*;
import org.openide.actions.*;
import org.openide.actions.PropertiesAction;
import org.openide.filesystems.*;
import org.openide.filesystems.FileSystem;
import org.openide.options.ControlPanel;
import org.openide.windows.*;
import org.openide.explorer.*;
import org.openide.explorer.view.BeanTreeView;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.RequestProcessor;
import org.openide.util.SharedClassObject;
import org.openide.util.UserCancelException;
import org.openide.util.actions.ActionPerformer;
import org.openide.util.actions.SystemAction;
import org.openide.util.io.*;
import org.openide.nodes.*;
import org.openide.util.Utilities;

import org.netbeans.core.actions.*;
import org.netbeans.core.windows.WindowManagerImpl;
import org.netbeans.core.windows.MainWindow;


/** This class is a TopManager for Corona environment.
*
* @author Ales Novak, Jaroslav Tulach, Ian Formanek, Petr Hamernik, Jan Jancura
*/
public final class Main extends NonGui {

  /** is there a splash screen or not */
  private static Splash.SplashOutput splash;

  /** Overrides the default status line method to display first in the splash
  * if there is one
  */
  public void setStatusText (String s) {
    Splash.SplashOutput sout = splash;
    if (sout != null) {
      sout.print (s);
    } else {
      super.setStatusText (s);
    }
  }

  /** Starts TopThreadGroup which properly overrides uncaughtException
   * Further - new thread in the group execs main
   */
  public static void start(String[] argv) {
    TopThreadGroup tg = new TopThreadGroup ("IDE Main", argv); // NOI18N - programatic name
    tg.start ();
  }


  /** Initialization code.
  */
  public void run () {

// -----------------------------------------------------------------------------------------------------
// 3. Initialization of L&F related things

    // if on window platform, set the WindowsLookAndFeel as the default
    // [IAN] Commented out accorning to HIIE request
/*    if (Utilities.isWindows ())  {
      try {
        uiClass = Class.forName (UIManager.getSystemLookAndFeelClassName ());
      } catch (ClassNotFoundException e) {
        uiClass = null;
      }
    } else {
      uiClass = null;
    }
*/
    if (uiClass != null) {
      try {
        LookAndFeel customUI = (javax.swing.LookAndFeel)uiClass.newInstance();
        UIManager.setLookAndFeel(customUI);
      } catch (Exception e) {
        System.out.println(getString("ERR_UIError"));
      }
    }
    else{
        NbTheme nbTheme = new NbTheme();
        javax.swing.plaf.metal.MetalLookAndFeel.setCurrentTheme(nbTheme);
        try {
                UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel"); // NOI18N
        } catch (Exception ex) {
                System.out.println("Failed loading Metal look and feel"); // NOI18N
                System.out.println(ex);
                uiFontSize=11;
        }
    }

    // Swing static initializations
//    ToolTipManager.sharedInstance ().setLightWeightPopupEnabled (false); // Deprecated code, no replacement available for now?

    // Modify default font size to the font size passed as a command-line parameter
    if(uiFontSize>0) {
            java.awt.Font nbDialogPlain = new java.awt.Font ("Dialog", java.awt.Font.PLAIN, uiFontSize); // NOI18N
            java.awt.Font nbSerifPlain = new java.awt.Font ("Serif", java.awt.Font.PLAIN, uiFontSize); // NOI18N
            java.awt.Font nbSansSerifPlain = new java.awt.Font ("SansSerif", java.awt.Font.PLAIN, uiFontSize); // NOI18N
            java.awt.Font nbMonospacedPlain = new java.awt.Font ("Monospaced", java.awt.Font.PLAIN, uiFontSize); // NOI18N
            UIManager.getDefaults ().put ("Button.font", nbDialogPlain); // NOI18N
            UIManager.getDefaults ().put ("ToggleButton.font", nbDialogPlain); // NOI18N
            UIManager.getDefaults ().put ("RadioButton.font", nbDialogPlain); // NOI18N
            UIManager.getDefaults ().put ("CheckBox.font", nbDialogPlain); // NOI18N
            UIManager.getDefaults ().put ("ColorChooser.font", nbDialogPlain); // NOI18N
            UIManager.getDefaults ().put ("ComboBox.font", nbDialogPlain); // NOI18N
            UIManager.getDefaults ().put ("Label.font", nbDialogPlain); // NOI18N
            UIManager.getDefaults ().put ("List.font", nbDialogPlain); // NOI18N
            UIManager.getDefaults ().put ("MenuBar.font", nbDialogPlain); // NOI18N
            UIManager.getDefaults ().put ("MenuItem.font", nbDialogPlain); // NOI18N
            UIManager.getDefaults ().put ("RadioButtonMenuItem.font", nbDialogPlain); // NOI18N
            UIManager.getDefaults ().put ("CheckBoxMenuItem.font", nbDialogPlain); // NOI18N
            UIManager.getDefaults ().put ("Menu.font", nbDialogPlain); // NOI18N
            UIManager.getDefaults ().put ("PopupMenu.font", nbDialogPlain); // NOI18N
            UIManager.getDefaults ().put ("OptionPane.font", nbDialogPlain); // NOI18N
            UIManager.getDefaults ().put ("Panel.font", nbDialogPlain); // NOI18N
            UIManager.getDefaults ().put ("ProgressBar.font", nbDialogPlain); // NOI18N
            UIManager.getDefaults ().put ("ScrollPane.font", nbDialogPlain); // NOI18N
            UIManager.getDefaults ().put ("Viewport.font", nbDialogPlain); // NOI18N
            UIManager.getDefaults ().put ("TabbedPane.font", nbDialogPlain); // NOI18N
            UIManager.getDefaults ().put ("Table.font", nbDialogPlain); // NOI18N
            UIManager.getDefaults ().put ("TableHeader.font", nbDialogPlain); // NOI18N
            UIManager.getDefaults ().put ("TextField.font", nbSansSerifPlain); // NOI18N
            UIManager.getDefaults ().put ("PasswordField.font", nbMonospacedPlain); // NOI18N
            UIManager.getDefaults ().put ("TextArea.font", nbMonospacedPlain); // NOI18N
            UIManager.getDefaults ().put ("TextPane.font", nbSerifPlain); // NOI18N
            UIManager.getDefaults ().put ("EditorPane.font", nbSerifPlain); // NOI18N
            UIManager.getDefaults ().put ("TitledBorder.font", nbDialogPlain); // NOI18N
            UIManager.getDefaults ().put ("ToolBar.font", nbDialogPlain); // NOI18N
            UIManager.getDefaults ().put ("ToolTip.font", nbSansSerifPlain); // NOI18N
            UIManager.getDefaults ().put ("Tree.font", nbDialogPlain); // NOI18N
    }

    // Ugly fix of bug #10487
    // Unregistering DesktopPane keystrokes due to debugger shortcuts!
    Object inputMap = UIManager.getDefaults ().get ("Desktop.ancestorInputMap"); // NOI18N
    if (inputMap != null) {
        // due to JDK 1.2.x, javax.swing.plaf.InputMapUIResource is since 1.3
        try {
            Class clazz = Class.forName ("javax.swing.plaf.InputMapUIResource");
            java.lang.reflect.Method m = clazz.getMethod ("remove", new Class [] { KeyStroke.class } );
            m.invoke(inputMap, new Object[] { KeyStroke.getKeyStroke (KeyEvent.VK_F5, KeyEvent.CTRL_MASK) } );
            m.invoke(inputMap, new Object[] { KeyStroke.getKeyStroke (KeyEvent.VK_F7, KeyEvent.CTRL_MASK) } );
            m.invoke(inputMap, new Object[] { KeyStroke.getKeyStroke (KeyEvent.VK_F8, KeyEvent.CTRL_MASK) } );
        } catch (ClassNotFoundException x) {
            // silently ignore all exceptions because it is not interesting if this does not succeed
        } catch (NoSuchMethodException x) {
        } catch (java.lang.reflect.InvocationTargetException x) {
        } catch (IllegalAccessException x) {
        }
    }

    // do the non gui initialization
    super.run ();


  }


  /** Method to initialize the main window.
  */
  protected void initializeMainWindow () {

// -----------------------------------------------------------------------------------------------------
// 11. Initialization of main window

    setStatusText (getString("MSG_MainWindowInit"));
    MainWindow mainWindow = WindowManagerImpl.mainWindow();
    mainWindow.initialize();

    // force to initialize timer
    // sometimes happened that the timer thread was initialized under
    // a TaskThreadGroup
    // such task never ends or, if killed, timer is over
    Timer timerInit = new Timer(0, new java.awt.event.ActionListener() {
          public void actionPerformed(java.awt.event.ActionEvent ev) { }
    });
    timerInit.setRepeats(false);
    timerInit.start();




    // prepare workspaces
    setStatusText (getString ("MSG_DefaultWorkspaceInit"));

    TopManager.getDefault().getWindowManager();

// -----------------------------------------------------------------------------------------------------
// 13. Initialize Shortcuts

    if (
        System.getProperty ("netbeans.kill") == null && // NOI18N
        System.getProperty ("netbeans.close") == null // NOI18N
    ) {
        // JST:
        // do not install the shortcut editor when the ide is started
        // for the first time
        ShortcutsFolder.initShortcuts();
    }

// -----------------------------------------------------------------------------------------------------
// 14. Open main window

    setStatusText (getString("MSG_WindowShowInit"));
    mainWindow.showWindow ();

  }

  /**
  * @exception SecurityException if it is called multiple times
  */
  public static void main(String[] args) throws SecurityException {
    long time = System.currentTimeMillis();
    if (System.getProperties ().get ("org.openide.TopManager") == null) { // NOI18N
      // update the top manager to our main if it has not been provided yet
      System.getProperties().put (
        "org.openide.TopManager", // NOI18N
        "org.netbeans.core.Main" // NOI18N
      );
    }
    
    // because of KL Group components, we define a property netbeans.design.time
    // which serves instead of Beans.isDesignTime () (which returns false in the IDE)
    System.getProperties ().put ("netbeans.design.time", "true"); // NOI18N

// JST commented out - Roger will do the initialization later when someone first access
//     the loger
//    org.netbeans.lib.logger.TraceLogger lgr = org.netbeans.lib.logger.TraceLogger.getTraceLogger();

    NonGui.parseCommandLine (args);

    // show the welcome screen
    if (!noSplash) {
      splash = Splash.showSplash ();
    }

// -----------------------------------------------------------------------------------------------------
// 4. Display Splash Screen & manager
    
    TopManager manager = TopManager.getDefault ();

    // finish starting
    if (splash != null) {
      Splash.hideSplash(splash);
      splash = null;
    }
    
    // welcome dialog
    if ((System.getProperty ("netbeans.full.hack") == null) && (System.getProperty ("netbeans.close") == null)) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                if (((IDESettings) SharedClassObject.findObject (IDESettings.class, true)).getShowTipsOnStartup()) {
                    org.netbeans.core.ui.WelcomePanel.showGettingStarted();
                }
            }
        });
    }

    measureStart(manager, time);
    
    if (System.getProperty ("netbeans.full.hack") == null) {
        // check the web for new modules
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                UpdateSupport.performUpdateCheck ();
            }
        });
    }

    // finish starting
    if (System.getProperty ("netbeans.kill") != null) {
      doExit (5);
    }

    // execute the resource
    if (System.getProperty ("netbeans.exec") != null) {
      startResource (System.getProperty ("netbeans.exec"));
    }
    // close IDE
    if (System.getProperty ("netbeans.close") != null) {
      org.openide.TopManager.getDefault().exit();
    }	
  }
  
    /** Prints out some info about time/heap if the measurement is enabled. */
    private static void measureStart(TopManager manager, long time) {
        org.netbeans.core.perftool.Util.waitForSystemThreads();
        org.netbeans.core.perftool.PerformanceMeter pm = org.netbeans.core.perftool.PerformanceMeterFactory.getPerformanceMeter();
        pm.notifyTime("Total startup time:", System.currentTimeMillis() - time);
        Runtime runtime = Runtime.getRuntime();
        System.gc();
        long mem = runtime.totalMemory();
        pm.notifyMemory("Total heap:", mem);
        pm.notifyMemory("Used heap:", mem - runtime.freeMemory());
    }
      
  /** Executes a resource.
  * @param res resource name to execute
  */
  private static void startResource (String res) {
    FileObject fo = TopManager.getDefault ().getRepository ().findResource(res);
    if (fo == null) {
      return;
    }

    try {
      DataObject obj = DataObject.find (fo);

      org.openide.actions.ExecuteAction.execute(
        new DataObject[] { obj }, true
      );
    } catch (DataObjectNotFoundException ex) {
      TopManager.getDefault ().getErrorManager ().notify (
        ErrorManager.INFORMATIONAL,
        ex
      );
    }
  }

  static String getString (String s) {
      return NbBundle.getMessage (Main.class, s);
  }
}
