/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.core;

import javax.swing.*;
import java.util.ArrayList;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.beans.*;

import org.openide.TopManager;
import org.openide.WizardDescriptor;
import org.openide.WizardDescriptor.Panel;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.netbeans.core.windows.WindowManagerImpl;
import org.netbeans.core.windows.UIModeManager;
import org.netbeans.core.ui.SetupWizard;
import org.netbeans.core.UpdateSupport;
import org.netbeans.core.IDESettings;

/** Wizard iterator used in setup wizard. Iterates through two panels - ModuleConfigPanel,
 * ModuleSelectionPanel.
 *
 * @author  Jan Rojcek
 */
public class ModuleConfigIterator implements WizardDescriptor.Iterator {

    private boolean customMode = false;
    private boolean configPanelCurrent = true;
    
    private ModuleSelectionPanel selectionPanel;
    private ModuleConfigPanel configPanel;
    
    private ChangeListener listener;
    
    /** Creates new ModuleConfigIterator */
    public ModuleConfigIterator() {
        selectionPanel = new ModuleSelectionPanel();
        configPanel = new ModuleConfigPanel(this);
        selectionPanel.putClientProperty("WizardPanel_contentData", new String[] { 
            configPanel.getName(),
            selectionPanel.getName()
        });
        selectionPanel.putClientProperty("WizardPanel_contentSelectedIndex", new Integer(1));
    }

    /** Test whether there is a next panel.
     * @return <code>true</code> if so
     */
    public boolean hasNext() {
        return configPanelCurrent && customMode;
    }
    
    /** Move to the previous panel.
     * I.e. decrement its index, need not actually change any GUI itself.
     * @exception NoSuchElementException if the panel does not exist
     */
    public void previousPanel() {
        configPanelCurrent = true;
    }
    
    /** Remove a listener to changes of the current panel.
     * @param l the listener to remove
     */
    public void removeChangeListener(ChangeListener l) {
    }
    
    /** Add a listener to changes of the current panel.
     * The listener is notified when the possibility to move forward/backward changes.
     * @param l the listener to add
     */
    public void addChangeListener(ChangeListener l) {
        listener = l;
        
        //this method is used as initialization place
        configPanelCurrent = true;
        configPanel.setIterator(this);
    }
    
    /** Get the name of the current panel.
     * @return the name
     */
    public String name() {
        return "";
    }
    
    /** Move to the next panel.
     * I.e. increment its index, need not actually change any GUI itself.
     * @exception NoSuchElementException if the panel does not exist
     */
    public void nextPanel() {
        configPanelCurrent = false;
    }
    
    /** Test whether there is a previous panel.
     * @return <code>true</code> if so
     */
    public boolean hasPrevious() {
        return !configPanelCurrent;
    }
    
    /** Get the current panel.
     * @return the panel
     */
    public Panel current() {
        return configPanelCurrent ? (Panel)configPanel : (Panel)selectionPanel;
    }
    
    /** This method is used by SetupWizard to pass its instance here.
     */
    public void registerWizard(SetupWizard sw) {
        configPanel.registerWizard(sw);
        selectionPanel.registerWizard(sw);
    }
    
    /* Called by ModuleConfigPanel
     */
    void setCustomMode(boolean custom, boolean notify) {
        customMode = custom;

        if (notify)
            listener.stateChanged(new ChangeEvent(this));
    }
}
