/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.core;

import javax.swing.*;
import javax.swing.table.TableModel;
import javax.swing.table.AbstractTableModel;
import javax.swing.event.ChangeListener;
import javax.swing.ListSelectionModel;
import javax.swing.event.*;
import java.util.*;
//import java.awt.*;
import java.net.URL;
import javax.swing.border.EmptyBorder;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import org.openide.WizardDescriptor;
import org.openide.NotifyDescriptor;
import org.openide.DialogDescriptor;
import org.openide.TopManager;
import org.openide.nodes.*;
import org.openide.util.HelpCtx;
import org.openide.util.RequestProcessor;
import org.openide.util.NbBundle;
import org.openide.modules.ModuleDescription;
import org.openide.explorer.ExplorerManager;
import org.netbeans.core.ModuleInstaller;
import org.netbeans.core.ui.SetupWizard;

/** Setup wizard panel. Allows to choose set of enabled modules.
 * Module sets are - basic (defined in bundle), complete (all installed modules), 
 * custom (ModuleSelectionPanel is shown as next step).
 * 
 * @author  Jan Rojcek
 */
public class ModuleConfigPanel extends javax.swing.JPanel implements org.openide.WizardDescriptor.FinishPanel, java.beans.PropertyChangeListener {

    private ArrayList ceItems = new ArrayList();

    private List customToDisable;
    private List customToEnable;

    private SetupWizard setupWizard;
    private ModuleConfigIterator iterator;
    
    /** Creates new form ModuleConfigurationPanel */
    public ModuleConfigPanel(ModuleConfigIterator iter) {
        iterator = iter;
        
        initComponents();
        setName(NbBundle.getMessage(ModuleConfigPanel.class, "LBL_ModuleConfigPanelName")); //NOI18N
        putClientProperty("WizardPanel_contentData", new String[] {getName()}); //NOI18N
        putClientProperty("WizardPanel_contentSelectedIndex", new Integer(0)); //NOI18N
        ButtonGroup bg = new ButtonGroup();
        bg.add(basicButton);
        bg.add(completeButton);
        bg.add(customButton);

        StringTokenizer st = new StringTokenizer(NbBundle.getMessage(ModuleConfigPanel.class, "BasicModuleNames"), ","); // NOI18N
        while (st.hasMoreTokens()) {
            ceItems.add(ModuleInstaller.getByName(st.nextToken()));
        }
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jTextArea1 = new javax.swing.JTextArea();
        jTextArea2 = new javax.swing.JTextArea();
        jTextArea3 = new javax.swing.JTextArea();
        jTextArea4 = new javax.swing.JTextArea();
        basicButton = new javax.swing.JRadioButton();
        completeButton = new javax.swing.JRadioButton();
        customButton = new javax.swing.JRadioButton();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        setPreferredSize(new java.awt.Dimension(550, 350));
        jTextArea1.setWrapStyleWord(true);
        jTextArea1.setLineWrap(true);
        jTextArea1.setEditable(false);
        jTextArea1.setFont(javax.swing.UIManager.getFont("Label.font"));
        jTextArea1.setText(org.openide.util.NbBundle.getMessage(ModuleConfigPanel.class, "LBL_ModuleConfigPanelDescr"));
        jTextArea1.setDisabledTextColor(javax.swing.UIManager.getColor("Label.foreground"));
        jTextArea1.setEnabled(false);
        jTextArea1.setOpaque(false);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 12, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        add(jTextArea1, gridBagConstraints1);
        
        jTextArea2.setWrapStyleWord(true);
        jTextArea2.setLineWrap(true);
        jTextArea2.setEditable(false);
        jTextArea2.setFont(javax.swing.UIManager.getFont("Label.font"));
        jTextArea2.setText(org.openide.util.NbBundle.getMessage(ModuleConfigPanel.class, "LBL_BasicDescr"));
        jTextArea2.setDisabledTextColor(javax.swing.UIManager.getColor("Label.foreground"));
        jTextArea2.setEnabled(false);
        jTextArea2.setOpaque(false);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 32, 12, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        add(jTextArea2, gridBagConstraints1);
        
        jTextArea3.setWrapStyleWord(true);
        jTextArea3.setLineWrap(true);
        jTextArea3.setEditable(false);
        jTextArea3.setFont(javax.swing.UIManager.getFont("Label.font"));
        jTextArea3.setText(org.openide.util.NbBundle.getMessage(ModuleConfigPanel.class, "LBL_CompleteDescr"));
        jTextArea3.setDisabledTextColor(javax.swing.UIManager.getColor("Label.foreground"));
        jTextArea3.setEnabled(false);
        jTextArea3.setOpaque(false);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 32, 12, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        add(jTextArea3, gridBagConstraints1);
        
        jTextArea4.setWrapStyleWord(true);
        jTextArea4.setLineWrap(true);
        jTextArea4.setEditable(false);
        jTextArea4.setFont(javax.swing.UIManager.getFont("Label.font"));
        jTextArea4.setText(org.openide.util.NbBundle.getMessage(ModuleConfigPanel.class, "LBL_CustomDescr"));
        jTextArea4.setDisabledTextColor(javax.swing.UIManager.getColor("Label.foreground"));
        jTextArea4.setEnabled(false);
        jTextArea4.setOpaque(false);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 6;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(0, 32, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(jTextArea4, gridBagConstraints1);
        
        basicButton.setText(org.openide.util.NbBundle.getMessage(ModuleConfigPanel.class, "LBL_BasicModulesSet"));
        basicButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                basicButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.insets = new java.awt.Insets(0, 12, 5, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(basicButton, gridBagConstraints1);
        
        completeButton.setText(org.openide.util.NbBundle.getMessage(ModuleConfigPanel.class, "LBL_CompleteModuleSet"));
        completeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                completeButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.insets = new java.awt.Insets(0, 12, 5, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(completeButton, gridBagConstraints1);
        
        customButton.setText(org.openide.util.NbBundle.getMessage(ModuleConfigPanel.class, "LBL_CustomModuleSet"));
        customButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                customButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 5;
        gridBagConstraints1.insets = new java.awt.Insets(0, 12, 5, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(customButton, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void customButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_customButtonActionPerformed
        if (setupWizard == null)
            return;
        if ((customToDisable != null) && (customToEnable != null)) {
            ModuleNode.updateLater(customToEnable, customToDisable);
            updateWizard(customToDisable);
        }
        updateState(true);
    }//GEN-LAST:event_customButtonActionPerformed

    private void completeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_completeButtonActionPerformed
        if (setupWizard == null)
            return;

        ArrayList all = new ArrayList(Arrays.asList(
                                        ModuleInstaller.getModuleItems(
                                            ModuleInstallerSupport.ENABLED_MODULE
                                            | ModuleInstallerSupport.DISABLED_MODULE)));
        ArrayList empty = new ArrayList();
        ModuleNode.updateLater(all, empty);
        updateWizard(empty);
        updateState(true);
    }//GEN-LAST:event_completeButtonActionPerformed

    private void basicButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_basicButtonActionPerformed
        if (setupWizard == null)
            return;

        ArrayList notCeItems = new ArrayList(Arrays.asList(
                                        ModuleInstaller.getModuleItems(
                                            ModuleInstallerSupport.ENABLED_MODULE
                                            | ModuleInstallerSupport.DISABLED_MODULE)));
        Collections.reverse(notCeItems);
        notCeItems.removeAll(ceItems);
        ModuleNode.updateLater(new ArrayList(ceItems), notCeItems);
        updateWizard(notCeItems);
        updateState(true);
    }//GEN-LAST:event_basicButtonActionPerformed

    /** Provides the wizard panel with the opportunity to update the
     * settings with its current customized state.
     * Rather than updating its settings with every change in the GUI, it should collect them,
     * and then only save them when requested to by this method.
     * Also, the original settings passed to {@link #readSettings} should not be modified (mutated);
     * rather, the (copy) passed in here should be mutated according to the collected changes.
     * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
     * @param settings the object representing a settings of the wizard
     */
    public void storeSettings(Object settings) {
    }    

    /** Help for this panel.
     * When the panel is active, this is used as the help for the wizard dialog.
     * @return the help or <code>null</code> if no help is supplied
     */
    public HelpCtx getHelp() {
        return null;
    }    

    /** Provides the wizard panel with the current data--either
     * the default data or already-modified settings, if the user used the previous and/or next buttons.
     * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
     * @param settings the object representing wizard panel state, as originally supplied to {@link WizardDescriptor#WizardDescriptor(WizardDescriptor.Iterator,Object)}
     * @exception IllegalStateException if the the data provided
     * by the wizard are not valid.
     */
    public void readSettings(Object settings) {
        if (settings instanceof WizardDescriptor) {
            WizardDescriptor wd = (WizardDescriptor)settings;
            wd.addPropertyChangeListener(this);
            if (customButton.isSelected() 
                || Boolean.TRUE.equals(wd.getProperty("initializePanel"))) { //NOI18N
                customToEnable = ModuleNode.getToEnableLater();
                customToDisable = ModuleNode.getToDisableLater();
            }
        }
    }
    
    /** Remove a listener to changes of the panel's validity.
     * @param l the listener to remove
     */
    public void removeChangeListener(ChangeListener l) {
    }
    
    /** Test whether the panel is finished and it is safe to proceed to the next one.
     * If the panel is valid, the "Next" (or "Finish") button will be enabled.
     * @return <code>true</code> if the user has entered satisfactory information
     */
    public boolean isValid() {
        return true;
    }

    /** Get the component displayed in this panel.
     * @return the component
     */
    public java.awt.Component getComponent() {
        return this;
    }

    /** Add a listener to changes of the panel's validity.
     * @param l the listener to add
     * @see #isValid
     */
    public void addChangeListener(ChangeListener l) {
    }

    /** Used by SetupWizard to pass its instance here.
     */
    void registerWizard(SetupWizard sw) {
        setupWizard = sw;
    }

    /** Helper method to update wizard steps.
     */
    private void updateWizard(List disable) {
        LinkedList disableList = new LinkedList();
        Iterator it = disable.iterator();
        while (it.hasNext()) {
            ModuleItem mi = (ModuleItem)it.next();
            disableList.add(mi.getDescription());
        }
        setupWizard.updateAdvancedWizard(disableList);
    }

    /** Used to pass iterator here.
     */
    void setIterator(ModuleConfigIterator iter) {
        iterator = iter;

        // Which radio button should be selected?
        ArrayList disabledItems = new ArrayList(Arrays.asList(ModuleInstaller.getModuleItems (ModuleInstallerSupport.DISABLED_MODULE)));
        ArrayList enabledItems = new ArrayList(Arrays.asList(ModuleInstaller.getModuleItems (ModuleInstallerSupport.ENABLED_MODULE)));
        if ((enabledItems.size() == ceItems.size()) && enabledItems.containsAll(ceItems)) {
            basicButton.setSelected(true);
        } else if (disabledItems.isEmpty()) {
            completeButton.setSelected(true);
        } else {
            customButton.setSelected(true);
        }
        updateState(false);
    }

    /** Update state of iterator - whether to show selection panel or not
     */
    private void updateState(boolean notify) {
        iterator.setCustomMode(customButton.isSelected(), notify);
        if (customButton.isSelected()) {
            putClientProperty("WizardPanel_contentData", new String[] { //NOI18N
                getName(),
                org.openide.util.NbBundle.getMessage(ModuleSelectionPanel.class, "LBL_SetupWizardModuleInstallation") //NOI18N
            });
        } else {
            putClientProperty("WizardPanel_contentData", new String[] {getName()}); //NOI18N
        }
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextArea jTextArea1;
    private javax.swing.JTextArea jTextArea2;
    private javax.swing.JTextArea jTextArea3;
    private javax.swing.JTextArea jTextArea4;
    private javax.swing.JRadioButton basicButton;
    private javax.swing.JRadioButton completeButton;
    private javax.swing.JRadioButton customButton;
    // End of variables declaration//GEN-END:variables

    /** Handling of property changes in node structure and setup wizard descriptor
     */
    public void propertyChange(PropertyChangeEvent evt) {
        if ((evt.getSource() instanceof WizardDescriptor) 
            && (WizardDescriptor.PROP_VALUE.equals(evt.getPropertyName()))) {
                
            WizardDescriptor wd = (WizardDescriptor)evt.getSource();
            if ((wd.getValue() == wd.CANCEL_OPTION) || (wd.getValue() == wd.CLOSED_OPTION)) {
                wd.removePropertyChangeListener(this);
                ModuleNode.clearUpdateLater();
                setupWizard = null;
            } else if (wd.getValue() == wd.FINISH_OPTION) {
                wd.removePropertyChangeListener(this);
                ModuleNode.performLateUpdate();
                setupWizard = null;
            }
        }
    }
}
