/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.core;

import javax.swing.*;
import javax.swing.table.TableModel;
import javax.swing.table.AbstractTableModel;
import javax.swing.event.ChangeListener;
import javax.swing.ListSelectionModel;
import javax.swing.event.*;
import java.util.*;
import java.awt.*;
import java.net.URL;
import javax.swing.border.EmptyBorder;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import org.openide.WizardDescriptor;
import org.openide.NotifyDescriptor;
import org.openide.DialogDescriptor;
import org.openide.TopManager;
import org.openide.nodes.*;
import org.openide.util.HelpCtx;
import org.openide.util.RequestProcessor;
import org.openide.util.NbBundle;
import org.openide.modules.ModuleDescription;
import org.openide.explorer.ExplorerManager;
import org.netbeans.core.ModuleInstaller;
import org.netbeans.core.ui.SetupWizard;


/** Module selection panel allows user to enable/disable modules in setup wizard
 *
 * @author cledantec, jrojcek
 */
public class ModuleSelectionPanel extends javax.swing.JPanel 
                                  implements WizardDescriptor.FinishPanel,
                                             PropertyChangeListener {

    /** See org.openide.WizardDescriptor.PROP_CONTENT_SELECTED_INDEX
     */
    private static final String PROP_CONTENT_SELECTED_INDEX = "WizardPanel_contentSelectedIndex"; // NOI18N
    /** See org.openide.WizardDescriptor.PROP_CONTENT_DATA
     */
    private static final String PROP_CONTENT_DATA = "WizardPanel_contentData"; // NOI18N

    /** Creates new form wizardFrame */
    public ModuleSelectionPanel() {
        initComponents();
        putClientProperty(PROP_CONTENT_SELECTED_INDEX, new Integer(0));
        putClientProperty(PROP_CONTENT_DATA, new String[] {getName()});

//        treeTableView.setPreferredSize(new Dimension(150, 50));
        treeTableView.setTreeViewPreferredSize(new Dimension (50, 50));
        treeTableView.setTableViewPreferredSize(new Dimension (100, 50));
        treeTableView.setProperties(
//           n.getChildren().getNodes()[0].getPropertySets()[0].getProperties());
            new Node.Property[]{
                new PropertySupport.ReadWrite (
                    "enabled", // NOI18N
                    Boolean.TYPE,
                    org.openide.util.NbBundle.getMessage (org.netbeans.core.Main.class, "PROP_modules_enabled"),
                    org.openide.util.NbBundle.getMessage (org.netbeans.core.Main.class, "HINT_modules_enabled")
                ) {
                    public Object getValue () {
                        return null;
                    }

                    public void setValue (Object o) {
                    }
                },
                new PropertySupport.ReadOnly (
                    "specVersion", // NOI18N
                    String.class,
                    org.openide.util.NbBundle.getMessage (org.netbeans.core.Main.class, "PROP_modules_specversion"),
                    org.openide.util.NbBundle.getMessage (org.netbeans.core.Main.class, "HINT_modules_specversion")
                ) {
                    public Object getValue () {
                        return null;
                    }
                }
            }
        );
        treeTableView.getTableColumn(0).setPreferredWidth(20);
        treeTableView.getTableColumn(1).setPreferredWidth(80);
        treeTableView.setPopupAllowed(false);

        manager = explorerPanel.getExplorerManager();
        ModuleNode node = new org.netbeans.core.ModuleNode();
        manager.setRootContext(node);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        descriptionPane = new javax.swing.JScrollPane();
        descriptionArea = new javax.swing.JTextArea();
        tableLabel = new javax.swing.JLabel();
        descriptionLabel = new javax.swing.JLabel();
        explorerPanel = new org.openide.explorer.ExplorerPanel();
        treeTableView = new org.netbeans.core.ui.TreeTableView();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        setPreferredSize(new java.awt.Dimension(530, 350));
        setName(org.openide.util.NbBundle.getMessage(ModuleSelectionPanel.class, "LBL_SetupWizardModuleInstallation"));
        descriptionPane.setPreferredSize(new java.awt.Dimension(50, 80));
        descriptionPane.setMinimumSize(new java.awt.Dimension(22, 80));
        descriptionArea.setWrapStyleWord(true);
        descriptionArea.setLineWrap(true);
        descriptionArea.setEditable(false);
        descriptionPane.setViewportView(descriptionArea);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        add(descriptionPane, gridBagConstraints1);
        
        tableLabel.setText(org.openide.util.NbBundle.getMessage(ModuleSelectionPanel.class, "LBL_SetupWizardCheckModules"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.insets = new java.awt.Insets(0, 1, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(tableLabel, gridBagConstraints1);
        
        descriptionLabel.setText(org.openide.util.NbBundle.getMessage(ModuleSelectionPanel.class, "LBL_SetupWizardDescription"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.insets = new java.awt.Insets(0, 1, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(descriptionLabel, gridBagConstraints1);
        
        treeTableView.setPreferredSize(new java.awt.Dimension(400, 400));
        treeTableView.setMinimumSize(new java.awt.Dimension(0, 0));
        explorerPanel.add(treeTableView, java.awt.BorderLayout.CENTER);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 11, 0);
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(explorerPanel, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void prevButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_prevButtonActionPerformed
        // Add your handling code here:
    }//GEN-LAST:event_prevButtonActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JScrollPane descriptionPane;
    private javax.swing.JTextArea descriptionArea;
    private javax.swing.JLabel tableLabel;
    private javax.swing.JLabel descriptionLabel;
    private org.openide.explorer.ExplorerPanel explorerPanel;
    private org.netbeans.core.ui.TreeTableView treeTableView;
    // End of variables declaration//GEN-END:variables

    private ExplorerManager manager;
    
    private boolean valid = true;
    
    private SetupWizard setupWizard;
    
    /** listeners on validation of panel changes */
    private ArrayList changeListeners = new ArrayList(1);

    /** Get the component displayed in this panel.
    * @return the component
    */
    public java.awt.Component getComponent () {
        return this;
    }

    /** Help for this panel.
    * When the panel is active, this is used as the help for the wizard dialog.
    * @return the help or <code>null</code> if no help is supplied
    */
    public HelpCtx getHelp () {
        return null;
    }

    /** Provides the wizard panel with the current data--either
    * the default data or already-modified settings, if the user used the previous and/or next buttons.
    * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
    * @param settings the object representing wizard panel state, as originally supplied to
    * {@link WizardDescriptor#WizardDescriptor(WizardDescriptor.Iterator,Object)}
    */
    public void readSettings (Object settings) {
        if (settings instanceof WizardDescriptor) {
            ((WizardDescriptor)settings).addPropertyChangeListener(this);
        }
        ModuleNode.setModuleUpdateListener(this);
        ModuleNode.setUpdateLater();
        manager.addPropertyChangeListener(this);
    }

    /** Provides the wizard panel with the opportunity to update the
    * settings with its current customized state.
    * Rather than updating its settings with every change in the GUI, it should collect them,
    * and then only save them when requested to by this method.
    * Also, the original settings passed to {@link #readSettings} should not be modified (mutated);
    * rather, the (copy) passed in here should be mutated according to the collected changes.
    * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
    * @param settings the object representing a settings of the wizard
    */
    public void storeSettings (Object settings) {
        ModuleNode.setModuleUpdateListener(null);
        manager.removePropertyChangeListener(this);
    }

    /** Test whether the panel is finished and it is safe to proceed to the next one.
    * If the panel is valid, the "Next" (or "Finish") button will be enabled.
    * @return <code>true</code> if the user has entered satisfactory information
    */
    public boolean isValid () {
        return valid;
    }
    
    /** Add a listener to changes of the panel's validity.
    * @param l the listener to add
    * @see #isValid
    */
    public void addChangeListener (ChangeListener l) {
        if (!changeListeners.contains(l)) {
            changeListeners.add(l);
        }
    }

    /** Remove a listener to changes of the panel's validity.
    * @param l the listener to remove
    */
    public void removeChangeListener (ChangeListener l) {
        changeListeners.remove(l);
    }
    
    /** Notifyes listeners that panel's validation changed */
    private void fireStateChanged() {
        for(int i = 0; i < changeListeners.size(); i++)
            ((ChangeListener)changeListeners.get(i)).stateChanged(
                new ChangeEvent(this));
    }
    
    /** Registers setup wizard instance which should be notified about panels state
     */
    public void registerWizard(SetupWizard sw) {
        setupWizard = sw;
    }

    /** Called when module set will be changed, we cannot move to next panel
     */
    void modulesUpdateScheduled() {
        valid = false;
        fireStateChanged();
    }

    /** Called when state of modules have been updated
     */
    void modulesUpdateFinished(LinkedList toDisable) {
        LinkedList disableList = new LinkedList();
        Iterator it = toDisable.iterator();
        while (it.hasNext()) {
            ModuleItem mi = (ModuleItem)it.next();
            disableList.add(mi.getDescription());
        }
        
        // tell to setup wizard that some of modules will be disabled later
        // so it should remove apropriate panels from wizard
        setupWizard.updateAdvancedWizard(disableList);
        valid = !ModuleNode.isUpdateScheduled();
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                fireStateChanged();
            }
        });
    }

    /** Handling of property changes in node structure and setup wizard descriptor
     */
    public void propertyChange(PropertyChangeEvent evt) {
        if ((evt.getSource() == manager) 
            && (manager.PROP_SELECTED_NODES.equals(evt.getPropertyName()))) {
                
            Node[] nodes = manager.getSelectedNodes();
            if ((nodes.length == 1) && (nodes[0] instanceof ModuleNode.Item)) {
                descriptionArea.setText(((ModuleNode.Item)nodes[0]).item.getDescription().getLongDescription());
                descriptionArea.setCaretPosition(0);
            } else 
                descriptionArea.setText("");
        } else if ((evt.getSource() instanceof WizardDescriptor) 
            && (WizardDescriptor.PROP_VALUE.equals(evt.getPropertyName()))) {
                
            WizardDescriptor wd = (WizardDescriptor)evt.getSource();
            if ((wd.getValue() == wd.CANCEL_OPTION) || (wd.getValue() == wd.CLOSED_OPTION)) {
                wd.removePropertyChangeListener(this);
                ModuleNode.clearUpdateLater();
                setupWizard = null;
            } else if (wd.getValue() == wd.FINISH_OPTION) {
                wd.removePropertyChangeListener(this);
                ModuleNode.performLateUpdate();
                setupWizard = null;
            }
        }
    }
    
}
