/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.core;

import java.awt.*;
import javax.swing.*;
import javax.swing.plaf.*;
import javax.swing.plaf.metal.*;
import org.openide.*;
public class NbTheme extends MetalTheme implements org.xml.sax.DocumentHandler {
    
    public static final String THEMEFILE_NAME = "Themes.xml"; // NOI18N
    
    private static final String MAIN_TAG = "MetalTheme"; // NOI18N
    private static final String MAIN_ACTIVATE_TAG = "activate"; // NOI18N
    private static final String THEME_TAG = "Theme"; // NOI18N
    private static final String THEME_NAME_TAG = "name"; // NOI18N
    
    private static final String R_TAG = "R"; // NOI18N
    private static final String G_TAG = "G"; // NOI18N
    private static final String B_TAG = "B"; // NOI18N
    private static final String PRIMARY1_TAG = "primary1"; // NOI18N
    private static final String PRIMARY2_TAG = "primary2"; // NOI18N
    private static final String PRIMARY3_TAG = "primary3"; // NOI18N
    private static final String SECONDARY1_TAG = "secondary1"; // NOI18N
    private static final String SECONDARY2_TAG = "secondary2"; // NOI18N
    private static final String SECONDARY3_TAG = "secondary3"; // NOI18N
    
    private static final String FONTNAME_TAG = "name"; // NOI18N
    private static final String FONTSTYLE_TAG = "style"; // NOI18N
    private static final String FONTSTYLE_PLAIN = "Plain"; // NOI18N
    private static final String FONTSTYLE_BOLD = "Bold"; // NOI18N
    private static final String FONTSTYLE_ITALIC = "Italic"; // NOI18N
    private static final String FONTSIZE_TAG = "size"; // NOI18N
    private static final String CONTROLFONT_TAG = "controlFont"; // NOI18N
    private static final String SYSTEMFONT_TAG = "systemFont"; // NOI18N
    private static final String USERFONT_TAG = "userFont"; // NOI18N
    private static final String SUBFONT_TAG = "subFont"; // NOI18N
    private static final String MENUFONT_TAG = "menuFont"; // NOI18N
    private static final String WINDOWTITLEFONT_TAG = "windowTitleFont"; // NOI18N
    
    
    // default Theme
    private ColorUIResource primary1 = new ColorUIResource(102, 102, 153);
    private ColorUIResource primary2 = new ColorUIResource(153, 153, 204);
    private ColorUIResource primary3 = new ColorUIResource(204, 204, 255);
    
    private ColorUIResource secondary1 = new ColorUIResource(102, 102, 102);
    private ColorUIResource secondary2 = new ColorUIResource(153, 153, 153);
    private ColorUIResource secondary3 = new ColorUIResource(204, 204, 204);
    
    private FontUIResource controlFont;
    private FontUIResource systemFont;
    private FontUIResource userFont;
    private FontUIResource subFont;
    private FontUIResource menuFont;
    private FontUIResource windowTitleFont;
    
    private String activeTag=""; // NOI18N
    private boolean activeTagFlag=false;
    
    public String getName(){ return "NetBeans"; } // NOI18N
    
    /** Creates new NBTheme */
    public NbTheme() {
        try{
            controlFont = new FontUIResource(new Font("Dialog", Font.PLAIN, 11)); // NOI18N
            systemFont = new FontUIResource(new Font("Dialog", Font.PLAIN, 11)); // NOI18N
            userFont = new FontUIResource(new Font("Dialog", Font.PLAIN, 11)); // NOI18N
            menuFont = new FontUIResource(new Font("Dialog", Font.PLAIN, 11)); // NOI18N
            windowTitleFont = new FontUIResource(new Font("Dialog", Font.PLAIN, 11)); // NOI18N
            subFont = new FontUIResource(new Font("Dialog", Font.PLAIN, 10)); // NOI18N            
            UIManager.getDefaults ().put ("List.font", controlFont); // NOI18N
            UIManager.getDefaults ().put ("Tree.font", controlFont); // NOI18N         
            UIManager.getDefaults ().put ("Panel.font", controlFont); // NOI18N
        } catch (Exception e){
            TopManager.getDefault().getErrorManager().notify(org.openide.ErrorManager.INFORMATIONAL, e);            
        }
        parseTheme();
    }
    
    protected ColorUIResource getPrimary1() { return primary1; }
    protected ColorUIResource getPrimary2() { return primary2; }
    protected ColorUIResource getPrimary3() { return primary3; }
    
    protected ColorUIResource getSecondary1() { return secondary1; }
    protected ColorUIResource getSecondary2() { return secondary2; }
    protected ColorUIResource getSecondary3() { return secondary3; }
    
    public FontUIResource getControlTextFont() { return controlFont; }
    public FontUIResource getSystemTextFont() { return systemFont; }
    public FontUIResource getUserTextFont() { return userFont; }
    public FontUIResource getMenuTextFont() { return menuFont; }
    public FontUIResource getWindowTitleFont() { return windowTitleFont; }
    public FontUIResource getSubTextFont() { return subFont; }
    
    private void parseTheme(){
        
        org.xml.sax.Parser p = org.openide.loaders.XMLDataObject.createParser();
        p.setDocumentHandler(this);
        java.io.File f = new java.io.File(NonGui.getSystemDir() + java.io.File.separator + THEMEFILE_NAME);
        java.io.Reader cs = null;
        boolean useDefaults = false;
        try{
            cs = new java.io.FileReader(f);
        }
        catch(java.io.FileNotFoundException e){
            useDefaults = true;
        }
        
        if(!useDefaults){
            org.xml.sax.InputSource is = new org.xml.sax.InputSource(cs);
            
            try{
                p.parse(is);
            }
            catch(java.io.IOException ie){
                TopManager.getDefault().getErrorManager().notify(org.openide.ErrorManager.INFORMATIONAL, ie);               
            }
            catch(org.xml.sax.SAXException se){
                TopManager.getDefault().getErrorManager().notify(org.openide.ErrorManager.INFORMATIONAL, se);              
            }
        }
    }
    
    public void endDocument() throws org.xml.sax.SAXException {
    }
    
    public void startDocument() throws org.xml.sax.SAXException {
    }
    
    public void startElement(java.lang.String p1,org.xml.sax.AttributeList atts) throws org.xml.sax.SAXException {
        int r = 0;
        int g = 0;
        int b = 0;
        String fontname = null;
        int fontstyle = 0;
        int fontsize = 0;
        
        if(p1.equals(MAIN_TAG) && atts.getLength()==1 && atts.getName(0).equals(MAIN_ACTIVATE_TAG)) {
            activeTag = atts.getValue(0);
        }
        else{
            if(p1.equals(THEME_TAG) && atts.getLength()==1 && atts.getName(0).equals(THEME_NAME_TAG) && atts.getValue(0).equals(activeTag)){
                activeTagFlag=true;
            }
            
            if(activeTagFlag){
                for (int i = 0; i < atts.getLength(); i++) {
                    if(atts.getName(i).equals(R_TAG)){
                        r = Integer.valueOf(atts.getValue(i)).intValue();
                    }
                    if(atts.getName(i).equals(G_TAG)){
                        g = Integer.valueOf(atts.getValue(i)).intValue();
                    }
                    if(atts.getName(i).equals(B_TAG)){
                        b = Integer.valueOf(atts.getValue(i)).intValue();
                    }
                    if(atts.getName(i).equals(FONTNAME_TAG)){
                        fontname = atts.getValue(i);
                    }
                    if(atts.getName(i).equals(FONTSTYLE_TAG)){
                        if(atts.getValue(i).equals(FONTSTYLE_PLAIN)) fontstyle = Font.PLAIN;
                        if(atts.getValue(i).equals(FONTSTYLE_BOLD)) fontstyle = Font.BOLD;
                        if(atts.getValue(i).equals(FONTSTYLE_ITALIC)) fontstyle = Font.ITALIC;
                    }
                    if(atts.getName(i).equals(FONTSIZE_TAG)){
                        fontsize = Integer.valueOf(atts.getValue(i)).intValue();
                    }
                }
                
                if(p1.equals(PRIMARY1_TAG)){
                    primary1 = new ColorUIResource(r, g, b);
                }
                if(p1.equals(PRIMARY2_TAG)){
                    primary2 = new ColorUIResource(r, g, b);
                }
                if(p1.equals(PRIMARY3_TAG)){
                    primary3 = new ColorUIResource(r, g, b);
                }
                
                if(p1.equals(SECONDARY1_TAG)){
                    secondary1 = new ColorUIResource(r, g, b);
                }
                if(p1.equals(SECONDARY2_TAG)){
                    secondary2 = new ColorUIResource(r, g, b);
                }
                if(p1.equals(SECONDARY3_TAG)){
                    secondary3 = new ColorUIResource(r, g, b);
                }
                if(p1.equals(CONTROLFONT_TAG)){
                    controlFont =  new FontUIResource(new Font(fontname, fontstyle, fontsize));
                    UIManager.getDefaults ().put ("List.font", controlFont); // NOI18N
                    UIManager.getDefaults ().put ("Tree.font", controlFont); // NOI18N
                    UIManager.getDefaults ().put ("Panel.font", controlFont); // NOI18N
                }
                if(p1.equals(SYSTEMFONT_TAG)){
                    systemFont =  new FontUIResource(new Font(fontname, fontstyle, fontsize));
                }
                if(p1.equals(USERFONT_TAG)){
                    userFont =  new FontUIResource(new Font(fontname, fontstyle, fontsize));
                }
                if(p1.equals(SUBFONT_TAG)){
                    subFont =  new FontUIResource(new Font(fontname, fontstyle, fontsize));
                }
                if(p1.equals(MENUFONT_TAG)){
                    menuFont =  new FontUIResource(new Font(fontname, fontstyle, fontsize));
                }
                if(p1.equals(WINDOWTITLEFONT_TAG)){
                    windowTitleFont =  new FontUIResource(new Font(fontname, fontstyle, fontsize));
                }
            }
        }
    }
    
    public void characters(char[] p1,int p2,int p3) throws org.xml.sax.SAXException {
    }
    
    public void setDocumentLocator(org.xml.sax.Locator p1) {
    }
    
    public void endElement(java.lang.String p1) throws org.xml.sax.SAXException {
        if(p1.equals(THEME_TAG)){
            activeTagFlag=false;
        }
        
    }
    
    public void ignorableWhitespace(char[] p1,int p2,int p3) throws org.xml.sax.SAXException {
    }
    
    public void processingInstruction(java.lang.String p1,java.lang.String p2) throws org.xml.sax.SAXException {
    }
    
    /*
    public ColorUIResource getSystemTextColor(){ return primary1; }
    public ColorUIResource getControlTextColor(){ return primary1; }
    public ColorUIResource getInactiveControlTextColor(){ return primary1; }
    public ColorUIResource getInactiveSystemTextColor(){ return primary1; }
    public ColorUIResource getUserTextColor(){ return primary1; }
    public ColorUIResource getTextHighlightColor(){ return primary1; }
    public ColorUIResource getHighlightedTextColor(){ return primary1; }
     */
}
