/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */
 
package org.netbeans.core;

import java.beans.*;
import java.util.*;

import org.openide.*;
import org.openide.nodes.*;

/** Show instances of one given subclass.
*
* @author  Jaroslav Tulach
*/
public class ServicesChildren extends Index.KeysChildren 
implements PropertyChangeListener, Runnable {
    public static final int INSTANCE = 0;
    public static final int TYPE = 1;
    
    /** type of the children */
    private int type;
    /** property to listen on change of */
    private String propName;
    
    /** Creates new ServicesInstanceChildren. 
    * @param list list of ServiceTypes 
    */
    public ServicesChildren(String propName, int type, List list) {
        super (list);
        this.type = type;
        this.propName = propName;       
    }
  
    /** Creates the nodes for given key (ServiceType).
    */
    protected Node[] createNodes (Object key) {
        try {
            switch (type) {
            case INSTANCE:
                return new Node[] { 
                    ServicesNode.createInstanceNode ((ServiceType)key)
                };
            case TYPE:
                return new Node[] { 
                    ServicesNode.createTypeNode ((Class)key)
                };
            }
        } catch (InstantiationException ex) {
              TopManager.getDefault ().getErrorManager ().notify (
                ErrorManager.INFORMATIONAL,
                ex
              );
        } catch (IntrospectionException ex) {
              TopManager.getDefault ().getErrorManager ().notify (
                ErrorManager.INFORMATIONAL,
                ex
              );
        }
        
        return new Node[0];
    }

    /** Updates the list */    
    public void propertyChange(PropertyChangeEvent p1) {
        if (propName.equals (p1.getPropertyName())) {
            org.openide.util.RequestProcessor.postRequest (this);
        }
    }
    
    public void run () {
        update ();
    }
    
    /** Lock on default instance of services.
     * @return the lock to use
     */
    protected Object lock () {
        return Services.getDefault ();
    }
}

