/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.core.actions;

import java.util.Map;
import java.util.HashMap;
import java.util.ResourceBundle;

import javax.swing.JButton;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.ButtonGroup;

import org.openide.TopManager;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.CallableSystemAction;
import org.openide.awt.JMenuPlus;
import org.openide.awt.Actions;

import org.netbeans.core.windows.UIModeManager;
import org.netbeans.core.windows.WindowManagerImpl;

/** Action that displays a submenu of radio buttons with choice which UI mode
 * you want the IDE be in.
 *
 * @author Dafe Simonek
 */
public class ChangeUIModeAction extends CallableSystemAction {
    static final long serialVersionUID = 3465637347523787865L;
    
    /** menu of the action */
    private JMenu menu;
    /** mapping <JRadioButtonMenuItem, Integer> between menu items and modes */
    private Map item2Mode;
    /** mapping <Integer, JRadioButtonMenuItem> between modes and menu items */
    private Map mode2Item;
    /** group of menu items */
    private ButtonGroup itemGroup;

    /** Not implemented. May only be used in a menu presenter, with the children performing the action. */
    public void performAction() {
        throw new UnsupportedOperationException();
    }
    
    public String getName () {
        return NbBundle.getMessage (ChangeUIModeAction.class, "CTL_ChangeUIMode");
    }

    /*protected String iconResource () {
        return "ChangeUIModeActionIcon.gif";
    }*/

    public HelpCtx getHelpCtx () {
        return HelpCtx.DEFAULT_HELP;
        // If you will provide context help then use:
        // return new HelpCtx (ChangeUIModeAction.class);
    }

    /** Perform extra initialization of this action's singleton.
     * PLEASE do not use constructors for this purpose!
    protected void initialize () {
	super.initialize ();
	putProperty ("someProp", value);
    }
    */

    /* Returns a JMenuItem that presents the Action, that implements this
    * interface, in a MenuBar.
    * @return the JMenuItem representation for the Action
    */
    public JMenuItem getMenuPresenter() {
        if (menu == null) {
            menu = new JMenuPlus();
            Actions.setMenuText(menu, getName(), true);
            item2Mode = new HashMap(7);
            mode2Item = new HashMap(7);
            itemGroup = new ButtonGroup();
            // ^ maps menu action listeners to UI IDE modes
            fillMenu(menu, item2Mode, mode2Item, itemGroup);
        }
        // update currently selected item
        JRadioButtonMenuItem selButton = (JRadioButtonMenuItem)mode2Item.get(
            new Integer(WindowManagerImpl.getDefault().getUIMode())
        );
        selButton.setSelected(true);
        return menu;
    }
    
    /** Creates and returns mapping between UI IDE modes and
     * their display names */
    private void fillMenu (JMenu menu, Map item2Mode, Map mode2Item, ButtonGroup itemGroup) {
        ResourceBundle myBundle = NbBundle.getBundle(UIModeManager.class);
        JRadioButtonMenuItem curItem = createMenuItem(myBundle.getString("CTL_SdiMode"));
        item2Mode.put(curItem, new Integer(UIModeManager.SDI_MODE));
        mode2Item.put(new Integer(UIModeManager.SDI_MODE), curItem);
        menu.add(curItem);
        itemGroup.add(curItem);
        curItem = createMenuItem(myBundle.getString("CTL_MdiMode"));
        item2Mode.put(curItem, new Integer(UIModeManager.MDI_MODE));
        mode2Item.put(new Integer(UIModeManager.MDI_MODE), curItem);
        menu.add(curItem);
        itemGroup.add(curItem);
        curItem = createMenuItem(myBundle.getString("CTL_SdiDialogMode"));
        item2Mode.put(curItem, new Integer(UIModeManager.DIALOG_SDI_MODE));
        mode2Item.put(new Integer(UIModeManager.DIALOG_SDI_MODE), curItem);
        menu.add(curItem);
        itemGroup.add(curItem);
    }

    /** Creates and returns menu item, filled with given display name and 
     * accompanied by proper action listener */ 
    private JRadioButtonMenuItem createMenuItem (String displayName) {
        JRadioButtonMenuItem menuItem = new JRadioButtonMenuItem();
        Actions.setMenuText(menuItem, displayName, true);
        HelpCtx.setHelpIDString (menuItem, ChangeUIModeAction.class.getName());
        menuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                Integer uiMode = (Integer)item2Mode.get(evt.getSource());
                WindowManagerImpl.getDefault().setUIMode(uiMode.intValue());
            }
        });
        return menuItem;
    }


}
