/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.core.actions;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.util.*;

import org.openide.TopManager;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.awt.JMenuPlus;
import org.openide.awt.Actions;
import org.openide.windows.*;
import org.openide.util.actions.*;

import org.netbeans.core.windows.*;
import org.netbeans.core.windows.frames.*;
import org.netbeans.core.windows.util.WindowUtils;

/** Takes care about deattaching of desktop-type frames.
 * Meaningful only in MDI, disabled in SDI.
 *
 * @author Dafe Simonek
 */
public final class DetachFrameAction extends CallableSystemAction 
                                     implements Presenter.Menu, Runnable {
    
    /** asociation with window manager implementation */
    private static WindowManagerImpl wm =
        (WindowManagerImpl)TopManager.getDefault().getWindowManager();

    /* Constructs a new action */
    public DetachFrameAction() {
    }

    /* Returns resource string pointing to actions' icon.
    * @return resource string of actions' icon
    */
    protected String iconResource () {
        return "/org/netbeans/core/resources/actions/detachFrame.gif"; // NOI18N
    }

    public HelpCtx getHelpCtx() {
        return new HelpCtx (AttachFrameAction.class);
    }

    /* @return Returns localized name of this action */
    public String getName() {
        return getBundleString("CTL_DetachFrameAction"); //NOI18N
    }

    /* Returns a submneu that will present this action in a Menu.
    * @return the JMenuItem representation for this action
    */
    public JMenuItem getMenuPresenter () {
        return createMenuPresenter(false);
    }
    
    /* Returns a submneu that will present this action in a PopupMenu.
    * @return the JMenuItem representation for this action
    */
    public JMenuItem getPopupPresenter() {
        return createMenuPresenter(true);
    }
    
    /** This action is active only in MDI mode for desktop-type frames */
    public boolean isEnabled () {
        if (!WindowUtils.isMdi()) {
            return false;
        }
        ModeImpl activeMode = WindowUtils.getActiveMode();
        return (activeMode != null) &&
               WindowTypesManager.DESKTOP_FRAME.equals(activeMode.getFrameType()); 
    }

    /* Detaches frame containing active top component and makes it floating
    */
    public void performAction () {
        if (SwingUtilities.isEventDispatchThread()) {
            run();
        } else {
            SwingUtilities.invokeLater(this);
        }
    }
        
    /** Implementation of runnable interface, runs the action command */
    public void run() {
        WindowUtils.changeActiveMode(
            null,
            WindowTypesManager.INTERNAL_FRAME
        );
    }
    
    private JMenuItem createMenuPresenter (boolean popup) {
        JMenuItem mainItem = new JMenuItem();
        Actions.connect(mainItem, this, popup);
//        HelpCtx.setHelpIDString (mainItem, DetachFrameAction.class.getName ());
        return mainItem;
    }
    
    /** @return returns string from bundle for given string pattern */
    private static final String getBundleString (String bundleStr) {
        return NbBundle.getBundle(DetachFrameAction.class).getString(bundleStr);
    }
    
}
