/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.core.actions;

import java.util.ResourceBundle;

import org.netbeans.core.ToolbarFolderNode;

import org.openide.TopManager;
import org.openide.util.actions.CallableSystemAction;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;

/** Displayes a toolbar tree in the explorer
*   @author Rostislav Levy
*/
public class DisplayToolbarsAction extends CallableSystemAction {

    /** Resource bundle */
    private static ResourceBundle bundle;

    /** Default constructor
     */
    public DisplayToolbarsAction() {
        bundle = null;
    }

    /* Human presentable name of the action. This should be
    *  presented as an item in a menu.
    *  @return the name of the action
    */
    public String getName () {
        return getBundle().getString("CTL_DisplayToolbars");
    }

    /* Resource name for the icon.
    *  @return resource name
    */
    protected String iconResource () {
        return null; // NOI18N
    }

    /* Help context where to find more about the action.
    *  @return the help context for this action
    */
    public HelpCtx getHelpCtx () {
        return new HelpCtx (DisplayToolbarsAction.class);
    }

    /** An action
     */
    public void performAction () {
        TopManager.getDefault().getNodeOperation ().explore(new ToolbarFolderNode());
    }

    /** Getter for a ResourceBundle
     *  @return a resource bundle
     */
    private ResourceBundle getBundle () {
        if(bundle == null)
            bundle = NbBundle.getBundle(DisplayToolbarsAction.class);
        return bundle;
    }
}
