/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.core.actions;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.openide.TopManager;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.WeakListener;
import org.openide.util.actions.CallableSystemAction;
import org.openide.windows.TopComponent;

import org.netbeans.core.windows.WindowManagerImpl;
import org.netbeans.core.windows.ModeImpl;
import org.netbeans.core.windows.frames.TopComponentContainer;

// [PENDING] This implementation is clumsy; assumes too much about how
// editor supports work. Better would be to make it check open top
// components for EditorCookie in their node selection?

/** The action which shows last activated editor component.
*
* @author Dafe Simonek
*/
public final class EditorViewAction extends CallableSystemAction {

    static final long serialVersionUID =-6878655133790342844L;

    /** the listener for this action */
    private static final String PROP_LISTENER = "listener"; // NOI18N

    /* Initialize the listener. */
    protected void initialize () {
        super.initialize();
        ActivationListener l = new ActivationListener(getClass());
        putProperty(PROP_LISTENER, l);
    }

    /* Adds listener to registry. */
    protected void addNotify () {
        // initializes the listener
        ActivationListener l = (ActivationListener)getProperty(PROP_LISTENER);
        l.setActive(true);
    }

    /* Removes listener to changes of activated nodes */
    protected void removeNotify () {
        ActivationListener l = (ActivationListener)getProperty(PROP_LISTENER);
        l.setActive(false);
    }

    /** Actually performs the action of showing last activated
    * editor component. */
    public void performAction () {
        /* bugfix #8970
        WindowManagerImpl wm = (WindowManagerImpl)TopManager.getDefault().
                               getWindowManager();
        wm.reactivateComponent(CloneableEditor.class);
         */
        TopComponent c = findEditorSelectedComponent();
        c.open();
        c.requestFocus();
    }

    public String getName() {
        return NbBundle.getBundle(EditorViewAction.class).getString("EditorView");
    }

    public HelpCtx getHelpCtx() {
        return new HelpCtx (EditorViewAction.class);
    }

    /** Resource name for the action's icon.
    * @return resource name
    */
    protected String iconResource () {
        return "/org/netbeans/core/resources/frames/editor.gif"; // NOI18N
    }

    private static TopComponent findEditorSelectedComponent() {
        TopComponentContainer container;
        TopComponent c = null;
        
        container = WindowManagerImpl.findContainer(org.netbeans.core.windows.WellKnownModeNames.EDITOR);
        if (null != container)
            c = container.getSelectedTopComponent();
        
        return c;
    }
    
    /** Node listener to check whether the action is enabled or not
    */
    private static final class ActivationListener implements PropertyChangeListener {
        /** the class we work with */
        private Class clazz;
        /** listener */
        private PropertyChangeListener listener;

        /** Constructor that checks the current state
        */
        public ActivationListener (Class clazz) {
            this.clazz = clazz;
        }

        /** Activates/passivates the listener. */
        void setActive (boolean active) {
            if (listener == null && active) {
                listener = WeakListener.propertyChange(this, TopComponent.getRegistry());
                TopComponent.getRegistry().addPropertyChangeListener(listener);
                checkEnabled(action());
                return;
            }

            if (listener != null && !active) {
                TopComponent.getRegistry().removePropertyChangeListener(listener);
                listener = null;
            }
        }

        /** Is the listener active?
        */
        boolean isActive () {
            return listener != null;
        }

        /** Property change listener. */
        public void propertyChange (PropertyChangeEvent ev) {
            EditorViewAction a = action();
            if (TopComponent.Registry.PROP_ACTIVATED.equals(ev.getPropertyName())) {
                // activated component changed, update action status
                checkEnabled(action());
                return;
            }
        }

        /** Obtains the action.
        * @return the node action we belong to or null if there is none
        */
        private EditorViewAction action () {
            return (EditorViewAction)findObject(clazz);
        }

        /** Checks the state of the action.
        */
        void checkEnabled (EditorViewAction a) {
            // no action no work
            if (a == null) return;
            // set state
            /* bugfix #8970 
            WindowManagerImpl wm = (WindowManagerImpl)TopManager.getDefault().
                                   getWindowManager();
            // [PENDING] Removed reference to deprecated
            // EditorSupport, so for EditorSupport.Editor components:
            // out of luck. Perhaps WindowManagerImpl.lastActivated
            // should check for subclasses too, in which case this
            // would be fine as EditorSupport.Editor extends
            // CloneableEditor.
            a.setEnabled(wm.lastActivated(CloneableEditor.class) != null);
             */
            
            a.setEnabled(null != findEditorSelectedComponent());
        }
    } // end of ActivationListener inner class

}
