/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.core.actions;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.util.*;

import org.openide.TopManager;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.windows.*;
import org.openide.util.actions.*;

import org.netbeans.core.windows.*;
import org.netbeans.core.windows.frames.PerimeterLayout;
import org.netbeans.core.windows.frames.PerimeterPane;
import org.netbeans.core.windows.frames.WindowTypesManager;
import org.netbeans.core.windows.util.WindowUtils;

/** Make the Mode dominant. Action is enabled only on desktop type of frames,
 * so meaningful and enabled only in MDI mode.
 *
 * @author Pavel Buzek, Dafe Simonek
 */
public final class MakeDominantAction extends CallableSystemAction 
                                      implements Runnable {
    
    /** asociation with window manager implementation */
    private static WindowManagerImpl wm =
        (WindowManagerImpl)TopManager.getDefault().getWindowManager();

    /* Constructs a new convert action */
    public MakeDominantAction() {
    }

    /* Returns resource string pointing to actions' icon.
    * @return resource string of actions' icon
    */
    protected String iconResource () {
        return "/org/netbeans/core/resources/actions/makeDominant.gif"; // NOI18N
    }

    public HelpCtx getHelpCtx() {
        return new HelpCtx (MakeDominantAction.class);
    }

    /* @return Returns localized name of this action */
    public String getName() {
        return NbBundle.getBundle(MakeDominantAction.class).getString("CTL_MakeDominanAction"); //NOI18N
    }

    
    /** Override initialization of the action. The default implementation just enabled it.
     * We enable it only in MDI mode
     */
    public boolean isEnabled() {
        if (!WindowUtils.isMdi()) {
            return false;
        }
        ModeImpl compMode = WindowUtils.getActiveMode();
        if ((compMode != null) &&
            WindowTypesManager.DESKTOP_FRAME.equals(compMode.getFrameType())) {
            Object constraint = compMode.getConstraints();
            return constraint.equals(PerimeterLayout.NORTH) ||
                   constraint.equals(PerimeterLayout.SOUTH) ||
                   constraint.equals(PerimeterLayout.WEST) ||
                   constraint.equals(PerimeterLayout.EAST);
        }
        return false;
    }
    
    /* Performs action - changes dominance for active mode in desktop pane.
    */
    public void performAction () {
        if (SwingUtilities.isEventDispatchThread()) {
            run();
        } else {
            SwingUtilities.invokeLater(this);
        }
    }
        
    /** Implementation of runnable interface, runs the action command */
    public void run() {
        WorkspaceImpl curWorkspace = (WorkspaceImpl)wm.getCurrentWorkspace();
        ModeImpl compMode = WindowUtils.getActiveMode();
        PerimeterPane perPane = (PerimeterPane) curWorkspace.desktopPane();
        perPane.setDominant((String)compMode.getConstraints());
    }
    
    /** @return returns string from bundle for given string pattern */
    private static final String getBundleString (String bundleStr) {
        return NbBundle.getBundle(MakeDominantAction.class).getString(bundleStr);
    }

}
