/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.core.actions;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JMenuItem;

import org.openide.TopManager;
import org.openide.awt.JInlineMenu;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.CallableSystemAction;
import org.openide.util.datatransfer.NewType;

import org.netbeans.core.ModuleFSSection;

/** The action that mounts new file system.
*
* @author Rostislav Levy
*/

public class MountAction extends CallableSystemAction {
    /** generated Serialized Version UID */
//    static final long serialVersionUID = -3249779028704471940L;

    /** URL to this action.
    * @return URL to the action icon
    */
    public String iconResource() {
        return "/org/openide/resources/actions/new.gif"; // NOI18N
    }

    public HelpCtx getHelpCtx() {
        return new HelpCtx (MountAction.class);
    }

    public String getName() {
        return NbBundle.getMessage (MountAction.class, "CTL_Mount");
    }

    public void performAction() {
    }
    
    public javax.swing.JMenuItem getPopupPresenter() {
        return new FS();
    }
    
    public javax.swing.JMenuItem getMenuPresenter() {
        return new FS();
    }
    
    private static class FS extends JInlineMenu {
        
        private transient boolean itemsAdded = false;
        
        public void addNotify () {
            if (itemsAdded) {
                super.addNotify();
                return;
            }
            NewType[] newtype=ModuleFSSection.listOfNewTypes(false);
            JMenuItem[] menuitem=new JMenuItem[newtype.length];
            for(int i=0;i<newtype.length;i++) {
                final NewType ntype=newtype[i];
                String name = NbBundle.getMessage (MountAction.class, "CTL_MountType", newtype[i].getName());
                menuitem[i]=new JMenuItem(name);
                menuitem[i].addActionListener(new ActionListener () {
                    public void actionPerformed (ActionEvent evt) {
                        try {
                            ntype.create();
                        }
                        catch (java.io.IOException exc) {
                            TopManager.getDefault().getErrorManager().notify(exc);
                        }
                    }});
            }
            setMenuItems(menuitem);
            itemsAdded = true;
            super.addNotify();
        }
    }
    
}
