/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.core.actions;

import java.util.ResourceBundle;

import org.openide.actions.NewTemplateAction;
import org.openide.util.actions.CallableSystemAction;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.SharedClassObject;

/** Creates a new object from template in the selected folder.
* @see DataObject#isTemplate
*
* @author Rostislav Levy
*/
public class NewTemplateCallableAction extends CallableSystemAction {

    /** Instance of a class NewTemplateAction */
    private static NewTemplateAction newTemplateActionInstance;
    /** Resource bundle */
    private static ResourceBundle bundle;

    /** Default constructor
     */
    public NewTemplateCallableAction() {
        newTemplateActionInstance = null;
        bundle = null;
    }

    /* Human presentable name of the action. This should be
    *  presented as an item in a menu.
    *  @return the name of the action
    */
    public String getName () {
        return getBundle().getString("NewTemplate");
    }

    /* Resource name for the icon.
    *  @return resource name
    */
    protected String iconResource () {
        return "/org/openide/resources/actions/new.gif"; // NOI18N
    }

    /* Help context where to find more about the action.
    *  @return the help context for this action
    */
    public HelpCtx getHelpCtx () {
        return new HelpCtx (NewTemplateAction.class);
    }

    /** An action
     */
    public void performAction () {
        getNewTemplateActionInstance().performAction();
    }

    /** Getter for a NewTemplateAction class instance
     *  @return an instance of NewTemplateAction class
     */
    private NewTemplateAction getNewTemplateActionInstance () {
        if(newTemplateActionInstance == null)
            newTemplateActionInstance = (NewTemplateAction)SharedClassObject.findObject(NewTemplateAction.class);
        return newTemplateActionInstance;
    }

    /** Getter for a ResourceBundle
     *  @return a resource bundle
     */
    private ResourceBundle getBundle () {
        if(bundle == null)
            bundle = NbBundle.getBundle(NewTemplateAction.class);
        return bundle;
    }
}
