/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.core.execution;

import java.util.ResourceBundle;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;

import org.openide.TopManager;
import org.openide.util.NbBundle;

/** A property editor for available workspaces.
* Actually edits the string name of the workspace, not
* the {@link Workspace} itself. Also permits user to specify
* "none" (i.e. no valid workspace).
* @see WorkspacePool#getWorkspaces
* @author   Jan Jancura
*/
public final class TabHandlerEditor extends java.beans.PropertyEditorSupport {
    
    private static final String keyTagNone = "TAG_None";
    private static final String keyTagReuse = "TAG_Reuse";
    private static final String keyTagReuseAndClear = "TAG_ReuseAndClear";
    
    private static final String[] keys = { keyTagNone, keyTagReuse, keyTagReuseAndClear };
    private static String[] locales = null;
    
    static final String[] getLocales() {
        if (locales == null) {
            ResourceBundle bundle = NbBundle.getBundle("org.netbeans.core.execution.Bundle");
            locales = new String[keys.length];
            for (int i = 0; i < locales.length; i++) {
                locales[i] = bundle.getString(keys[i]);
            }
        }
        
        return locales;
    }


    /*
    * @return The property value as a human editable string.
    * <p>   Returns null if the value can't be expressed as an editable string.
    * <p>   If a non-null value is returned, then the PropertyEditor should
    *       be prepared to parse that string back in setAsText().
    */
    public String getAsText() {
        int value = ((Integer) getValue()).intValue();
        return getLocales()[value];
    }

    /* Set the property value by parsing a given String.  May raise
    * java.lang.IllegalArgumentException if either the String is
    * badly formatted or if this kind of property can't be expressed
    * as text.
    * @param text  The string to be parsed.
    */
    public void setAsText(String text) throws java.lang.IllegalArgumentException {
        if (text == null) {
            throw new IllegalArgumentException();
        }
        int idx = name2index(text);
        setValue(new Integer(idx));
    }

    /*
    * If the property value must be one of a set of known tagged values,
    * then this method should return an array of the tag values.  This can
    * be used to represent (for example) enum values.  If a PropertyEditor
    * supports tags, then it should support the use of setAsText with
    * a tag value as a way of setting the value.
    *
    * @return The tag values for this property.  May be null if this
    *   property cannot be represented as a tagged value.
    *
    */
    public String[] getTags() {
        return getLocales();
    }

    /** @return one of ExecutionSettings.REUSE_AND_CLEAR, ExecutionSettings.REUSE, ExecutionSettings.NONE
     */
    private int name2index(String displayName) {
        
        String[] tags = getLocales();
        for (int i = tags.length; --i >= 0; ) {
            if (tags[i].equals(displayName)) {
                return i;
            }
        }
        // default
        return ExecutionSettings.REUSE_AND_CLEAR;
    }

}

/*
 * Log
 *  3    Gandalf-post-FCS1.1.1.0     3/15/00  David Simonek   japanese localization 
 *       now works correctly
 *  2    Gandalf   1.1         10/22/99 Ian Formanek    NO SEMANTIC CHANGE - Sun
 *       Microsystems Copyright in File Comment
 *  1    Gandalf   1.0         10/5/99  Ales Novak      
 * $
 * Beta Change History:
 *  0    Tuborg    0.11        --/--/98 anonymous       []
 */
