/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.core.lookup;

import java.util.*;
import java.lang.ref.WeakReference;
import java.lang.ref.Reference;

import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;

import org.openide.util.WeakListener;

/** Lookup based on real instances. Each time an instance is registered,
 * it is stored in class based tables, so it can be located quickly.
 *
 * @author  Jaroslav Tulach
 */
public final class InstanceLookup extends NbLookup {
    /** Creates empty lookup.
     */
    public InstanceLookup () {
    }
    
    /** The method to add instance to the lookup with.
     * @param inst instance 
     */
    public void add (Object inst) {
        addPair (new I (inst));
    }
    
    /** Remove instance.
     * @param obj instance
     */
    public void remove (Object inst) {
        removePair (new I (inst));
    }

    /** Instance of one item registered in the map.
     */
    final static class I extends Item {
        private Object obj;
        
        public I (Object obj) {
            this.obj = obj;
        }
        
        public Object getInstance () {
            return obj;
        }
        
        public Class getType () {
            return obj.getClass ();
        }
        
        public boolean equals (Object o) {
            if (o instanceof I) {
                return obj.equals (((I)o).obj);
            } else {
                return false;
            }
        }
        
        public int hashCode () {
            return obj.hashCode ();
        }
    }
}
