/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.core.lookup;

import java.util.*;

import org.openide.util.Lookup;
import org.openide.util.enum.AlterEnumeration;

/**
 * Root context delegating into a Lookup context. Inspirated by the 
 * work of Tomas and Petr, done by Yarda.
 *
 * @author Tomas Zezula, Petr Kuzel, Jaroslav Tulach
 */
public class Jndi extends Object 
implements javax.naming.Context, javax.naming.NameParser, 
javax.naming.spi.InitialContextFactory {
    // implements all three interfaces to lower the number of all classes
    
    /** Constructor.
     */
    public Jndi () {
    }
    
    /** Method to find lookup to use.
     */
    protected Lookup lookup () {
        return Lookup.getDefault ();
    }
    
    /** Class recognizer.
     * @param name name of the class
     * @return the class
     * @exception NamingException if the class is not found
     */
    protected Class clazz (String name) throws javax.naming.NamingException {
        try {
            return Class.forName (name);
        } catch (ClassNotFoundException ex) {
            javax.naming.NamingException e = new javax.naming.NamingException ();
            ex.printStackTrace();
            // PENDING annotate
            throw e;
        }
    }
    
    //
    // Factory method
    //
    

    /** This object is the context.
     *
     *  @param java.util.Hashtable env
     *  @return javax.naming.Context initial context
     *  @exception javax.naming.NamingException
     */
    public javax.naming.Context getInitialContext(java.util.Hashtable env) 
    throws javax.naming.NamingException {
        return this;
    }
    
    
    
    //
    // Parser implementation
    //
    
    
    private static java.util.Properties syntax = new java.util.Properties();
    static {
	syntax.put ("jndi.syntax.ignorecase","false");
    }

    /** parse
     *  Creates CompoundName representing given stringified name.
     *  @param String stringified name
     *  @return javax.naming.CompoundName
     *  @exception javax.naming.NamingException
     */
    public javax.naming.Name parse(String name)  throws javax.naming.NamingException {
	return new javax.naming.CompoundName (name, syntax);
    }

    
    
    
    //
    // Context implementation
    //
    
    /** 
     *  Returns name pareser for this service.
     *  @param String name
     *  @return javax.naming.NameParser
     */
    public javax.naming.NameParser getNameParser(java.lang.String name) 
    throws javax.naming.NamingException {
	return this;
    }
    
    /** 
     *  Returns name parser of this provider.
     *  @param javax.naming.Name name
     *  @return javax.naming.NameParser
     *  @exception javax.naming.NamingException
     */
    public javax.naming.NameParser getNameParser(javax.naming.Name name) throws javax.naming.NamingException {
	return this;
    }
    
    
    /** listBindings
     *  Lists bindings in context.
     *  @param javax.naming.String name
     *	@return javax.naming.NamingEnumeration of javax.naming.Binding objects
     *  @exception javax.naming.NamingException
     */
    public javax.naming.NamingEnumeration listBindings(java.lang.String name) {
        Lookup l = lookup ();
        
        Enumeration items;
        
        if (l instanceof NbLookup) {
            items = Collections.enumeration (
                ((NbLookup)l).lookup (new NbLookup.Template ()).allItems ()
            );
        } else {
            // [PENDING]
            // JST: For now just use instances
            items = Collections.enumeration (
                lookup ().lookup (new Lookup.Template ()).allInstances ()
            );
        }
        return new JiniBindingEnumeration(items);
    }
    
    /** list
     *  List names in context.
     *  @param String name
     *  @return javax.naming.NamingEnumeration of javax.naming.NameClassPair objects
     */
    public javax.naming.NamingEnumeration list (java.lang.String name)  {
        // Binding extends NameClassPair => we can return the same object
        return listBindings (name);
    }
    
    /** lookup
     *  Returns a CORBA reference to specified remote object.
     *  @param String old name
     *  @return Object object
     *  @exception javax.naming.NamingException
     */
    public java.lang.Object lookup(java.lang.String name) throws javax.naming.NamingException {
        Class clazz = clazz (name);
        return lookup ().lookup (clazz);
    }
    

    //
    // Delegating methods of the context
    //

    
    
    /** list
     *  Lists object in context.
     *  @param javax.naming.Name name
     *  @return javax.naming.NamingEnumeration enumeration of javax.naming.NameClassPair
     *  @exception javax.naming.NamingException
     */
    public javax.naming.NamingEnumeration list(javax.naming.Name name) throws javax.naming.NamingException {
	return this.list (name.toString());
    }
    
    /** lookupLink
     *  Looks up an object.
     *  @param String name
     *  @return Object
     *  @exception javax.naming.NamingException
     */    
    public java.lang.Object lookupLink(java.lang.String name) throws javax.naming.NamingException {
	return this.lookup (name);
    }
    
    
    /** listBindings
     *  List bindings in context.
     *  @param javax.naming.Name name
     *  @return javax.naming.NamingEnumeration enumeration of javax.naming.Binding
     *  @exception javax.naming.NamingException
     */
    public javax.naming.NamingEnumeration listBindings(javax.naming.Name name) throws javax.naming.NamingException {
	return this.listBindings (name.toString());
    }
    
    
    /** lookupLink
     *  Looks up an object.
     *  @param javax.naming.Name name
     *  @return Object
     *  @exception javax.naming.NamingException
     */
    public java.lang.Object lookupLink(javax.naming.Name name) throws javax.naming.NamingException {
	return lookup (name);
    }
    
    /** lookup
     *  Looks up an object.
     *  @param javax.naming.Name name
     *  @return Object
     *  @exception javax.naming.NamingException
     */
    public java.lang.Object lookup(javax.naming.Name name) throws javax.naming.NamingException {
	return this.lookup (name.toString());
    }
    
    
    //
    // Unimplemented methods of the context
    //
    
    
    private static javax.naming.NamingException noImpl () {
        return new javax.naming.NamingException ("Unimplemented operation, we are flat and read only"); // NOI18N
    }
    
    /** Not implemented.
     */
    public void rebind(java.lang.String name,java.lang.Object value) throws javax.naming.NamingException {
	throw noImpl ();
    }

    /** Not implemented.
     */
    public java.lang.String composeName(java.lang.String namePrefix,java.lang.String nameSuffix) throws javax.naming.NamingException {
	throw noImpl ();
    }

    /** Not implemented.
     */
    public javax.naming.Name composeName(javax.naming.Name name1,javax.naming.Name name2) throws javax.naming.NamingException {
	throw noImpl ();
    }

    /** Not implemented.
     */
    public void rename(java.lang.String oldName, java.lang.String newName) throws javax.naming.NamingException {
	throw noImpl ();
    }
    
    /** Not implemented.
     */
    public void rename(javax.naming.Name oldName, javax.naming.Name newName) throws javax.naming.NamingException {
	throw noImpl ();
    }
    
    
    /** Not implemented.
     */
    public void unbind(java.lang.String name) throws javax.naming.NamingException {
	throw noImpl ();
    }
    
    /** Not implemented.
     */
    public javax.naming.Context createSubcontext(javax.naming.Name name) throws javax.naming.NamingException {
	throw noImpl ();
    }
    
    /** Not implemented.
     */
    public void unbind(javax.naming.Name name) throws javax.naming.NamingException {
	throw noImpl ();
    }
    
    /** Not implemented.
     */
    public javax.naming.Context createSubcontext(java.lang.String name) throws javax.naming.NamingException {
	throw noImpl ();
    }
    
    /** Not implemented.
     */
    public void destroySubcontext(java.lang.String name) throws javax.naming.NamingException {
        throw noImpl ();
    }
    
    /** Not implemented.
     */
    public void destroySubcontext(javax.naming.Name name) throws javax.naming.NamingException {
	throw noImpl ();
    }
    
    
    /** Not implemented.
     */
    public void bind(java.lang.String name,java.lang.Object object) throws javax.naming.NamingException {
	throw noImpl ();
    }
    
    /** Not implemented.
     */
    public void rebind(javax.naming.Name name,java.lang.Object object) throws javax.naming.NamingException {
	throw noImpl ();
    }
    
    /** Not implemented.
     */
    public void bind(javax.naming.Name p1,java.lang.Object p2) throws javax.naming.NamingException {
	throw noImpl ();
    }
    
    
    /** 
     *  @return some silly hashtable with no usage at all
     */
    public java.util.Hashtable getEnvironment() 
    throws javax.naming.NamingException {
        // unreasonable implementation
	return (Hashtable) syntax.clone();
    }
    
    /** Does nothing.
     *  @param String property name
     *  @param Object property value
     *  @return null
     *  @exception javax.naming.NamingException
     */
    public java.lang.Object addToEnvironment(java.lang.String propertyName,java.lang.Object propertyValue) throws javax.naming.NamingException {
	return null;
    }
    
    /** No impl.
     */
    public java.lang.Object removeFromEnvironment(java.lang.String name) throws javax.naming.NamingException {
	return null;
    }
    
    /** getNameInNamespace
     *  Return its name
     *  @return String name
     *  @exception javax.naming.NamingException 
     */
    public java.lang.String getNameInNamespace() throws javax.naming.NamingException {
	return "";
    }
    
    
    /** Does nothing.
     *  @exception javax.naming.NamingException
     */
    public void close() throws javax.naming.NamingException {
    }

    
    /** An implementation of binding that delegates to Lookup.Item
     */
    private static class DelBind extends javax.naming.Binding {
        /** item */
        private NbLookup.Item item;
        
        public DelBind (NbLookup.Item item) {
            super (null, null);
            this.item = item;
        }
        
        /** The name of the class.
         */
        public java.lang.String getClassName() {
            return item.getType ().getName ();
        }

        /** @return the object
         */
        public java.lang.Object getObject() {
            return item.getInstance ();
        }
        
        public java.lang.String getName() {
            return item.getType ().getName ();
        }
        
    }
    
    /** Inner class representing NamingEnumeration for Bindings*/
    private static class JiniBindingEnumeration extends AlterEnumeration
    implements javax.naming.NamingEnumeration {
        /** @param classes classes of Lookup.Item */
	public JiniBindingEnumeration (Enumeration classes) {
            super (classes);
	}
	
	public boolean hasMore () {
	    return hasMoreElements ();
	}
        
        /** Converts the Lookup.Item into Binding.
         */
        protected Object alter (Object obj) {
            NbLookup.Item item;
            
            // PENDING
            // JST: Again just because we are missing Lookup.Item
            if (obj instanceof NbLookup.Item) {
                item = (NbLookup.Item)obj;
            } else {
                
                item = new InstanceLookup.I (obj);
            }
            
            return new DelBind (item);
        }
	
	public Object next () {
	    return nextElement();
	}

	public void close() {
	}
    }
}
