/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.core.lookup;

import java.lang.reflect.*;
import java.util.*;

import org.openide.TopManager;

/** Lookup that is based on the implementation of top manager.
 *
 * @author  Jaroslav Tulach
 */
public final class TMLookup extends NbLookup {
    /** Register all methods.
     */
    protected void initialize () {
        // first of all register ClassLoader
        addPair (new L ());
        
        // now register all methods from top manager that take no 
        // parameters
        try {
            Method[] arr = TopManager.class.getMethods();
            for (int i = 0; i < arr.length; i++) {
                Method m = arr[i];
                if (m.getParameterTypes ().length == 0) {
                    // ok, suitable method for registration
                    addPair (new I (m));
                }
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            throw new InternalError ();
        }
    }

    /** Method invokation Item.
     */
    private final static class I extends Item {
        private Method method;
        private Object value;
        
        /** @param method to call to get the value.
         */
        public I (Method m) {
            method = m;
        }
        
        public Class getType () {
            return method.getReturnType ();
        }
        
        public Object getInstance () {
            if (value != null) {
                return value;
            }
            
            try {
                value = method.invoke (TopManager.getDefault (), new Object[0]);
                return value;
            } catch (Exception ex) {
                InternalError e = new InternalError ();
                TopManager.getDefault ().getErrorManager ().copyAnnotation (e, ex);
                throw e;
            }
        }
    }
    
    /** Special item for system class loader (which is dynamic).
     */
    private final static class L extends Item {
        public Class getType () {
            return java.lang.ClassLoader.class;
        }
        
        public Object getInstance () {
            return TopManager.getDefault ().systemClassLoader ();
        }
    }
}
