/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.core.ui;

import javax.swing.*;
import java.util.ArrayList;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.beans.*;

import org.openide.TopManager;
import org.openide.WizardDescriptor;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.netbeans.core.windows.WindowManagerImpl;
import org.netbeans.core.windows.UIModeManager;
import org.netbeans.core.UpdateSupport;
import org.netbeans.core.IDESettings;

/** Setup wizard panel that represents general IDE settings
 *
 * @author  jrojcek
 */
public class IDESettingsPanel extends javax.swing.JPanel 
                              implements WizardDescriptor.FinishPanel,
                                         PropertyChangeListener {

    /** See org.openide.WizardDescriptor.PROP_CONTENT_SELECTED_INDEX
     */
    private static final String PROP_CONTENT_SELECTED_INDEX =  "WizardPanel_contentSelectedIndex"; // NOI18N
    /** See org.openide.WizardDescriptor.PROP_CONTENT_DATA
     */
    private static final String PROP_CONTENT_DATA = "WizardPanel_contentData"; // NOI18N

    java.util.ResourceBundle bundle = org.openide.util.NbBundle.getBundle("org/netbeans/core/ui/Bundle");
    
    /** Creates new form IDESettingsPanel */
    public IDESettingsPanel() {
        initComponents();

        serverLabel.setDisplayedMnemonic(bundle.getString("LBL_Server_Name_Mnemonic").toCharArray()[0]);
        portLabel.setDisplayedMnemonic(bundle.getString("LBL__Port_Mnemonic").toCharArray()[0]);
        proxyCheckBox.setMnemonic(bundle.getString("LBL_UseProxyServer_Mnemonic").toCharArray()[0]);
        sdiRadioButton.setMnemonic(bundle.getString("LBL_SDI_Mnemonic").toCharArray()[0]);
        mdiRadioButton.setMnemonic(bundle.getString("LBL_MDI_Mnemonic").toCharArray()[0]);

        javax.swing.ButtonGroup bg = new javax.swing.ButtonGroup();
        bg.add(mdiRadioButton);
        bg.add(sdiRadioButton);

        putClientProperty(PROP_CONTENT_SELECTED_INDEX, new Integer(0));
        putClientProperty(PROP_CONTENT_DATA, new String[] {getName()});
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        proxyCheckBox = new javax.swing.JCheckBox();
        serverField = new javax.swing.JTextField();
        portField = new javax.swing.JTextField();
        sdiRadioButton = new javax.swing.JRadioButton();
        mdiRadioButton = new javax.swing.JRadioButton();
        serverLabel = new javax.swing.JLabel();
        headLabel = new javax.swing.JLabel();
        portLabel = new javax.swing.JLabel();
        windowLabel = new javax.swing.JLabel();
        browserLabel = new javax.swing.JLabel();
        browserComboBox = new javax.swing.JComboBox();
        helpTextArea = new javax.swing.JTextArea();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        setPreferredSize(new java.awt.Dimension(550, 350));
        setName(bundle.getString("LBL_IDEPanelName"));
        proxyCheckBox.setText(bundle.getString("LBL_UseProxyServer"));
        proxyCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                changeProxyEnabled(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 5, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(proxyCheckBox, gridBagConstraints1);
        
        serverField.setPreferredSize(new java.awt.Dimension(300, 17));
        serverField.setMinimumSize(new java.awt.Dimension(300, 17));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 5, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(serverField, gridBagConstraints1);
        
        portField.setPreferredSize(new java.awt.Dimension(80, 17));
        portField.setMinimumSize(new java.awt.Dimension(80, 17));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 23, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(portField, gridBagConstraints1);
        
        sdiRadioButton.setText(bundle.getString("LBL_SDI"));
        sdiRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                changeWindowMode(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 5;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.insets = new java.awt.Insets(0, 12, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(sdiRadioButton, gridBagConstraints1);
        
        mdiRadioButton.setText(bundle.getString("LBL_MDI"));
        mdiRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                changeWindowMode(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 6;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.insets = new java.awt.Insets(0, 12, 23, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(mdiRadioButton, gridBagConstraints1);
        
        serverLabel.setText(bundle.getString("LBL_Server_Name"));
        serverLabel.setLabelFor(serverField);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.insets = new java.awt.Insets(0, 12, 5, 12);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(serverLabel, gridBagConstraints1);
        
        headLabel.setText(bundle.getString("LBL_Head"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 23, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(headLabel, gridBagConstraints1);
        
        portLabel.setText(bundle.getString("LBL_Port"));
        portLabel.setLabelFor(portField);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.insets = new java.awt.Insets(0, 12, 23, 12);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(portLabel, gridBagConstraints1);
        
        windowLabel.setText(bundle.getString("LBL_WindowMode"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 2, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(windowLabel, gridBagConstraints1);
        
        browserLabel.setText(bundle.getString ("CTL_BrowserLabel"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 7;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 0, 12);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(browserLabel, gridBagConstraints1);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 7;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(browserComboBox, gridBagConstraints1);
        
        helpTextArea.setWrapStyleWord(true);
        helpTextArea.setLineWrap(true);
        helpTextArea.setEditable(false);
        helpTextArea.setText(bundle.getString("CTL_HelpTextArea"));
        helpTextArea.setEnabled(false);
        helpTextArea.setOpaque(false);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 8;
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(12, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(helpTextArea, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void changeProxyEnabled(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_changeProxyEnabled
        setProxySettingEnabled(proxyCheckBox.isSelected());
    }//GEN-LAST:event_changeProxyEnabled

    private void setProxySettingEnabled(boolean b) {
        serverLabel.setEnabled(b);
        portLabel.setEnabled(b);
        serverField.setEnabled(b);
        portField.setEnabled(b);
    }
    
    private void changeWindowMode(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_changeWindowMode
        if (evt.getSource() == mdiRadioButton)
            uiMode = UIModeManager.MDI_MODE;
        else
            uiMode = UIModeManager.SDI_MODE;
    }//GEN-LAST:event_changeWindowMode


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JCheckBox proxyCheckBox;
    private javax.swing.JTextField serverField;
    private javax.swing.JTextField portField;
    private javax.swing.JRadioButton sdiRadioButton;
    private javax.swing.JRadioButton mdiRadioButton;
    private javax.swing.JLabel serverLabel;
    private javax.swing.JLabel headLabel;
    private javax.swing.JLabel portLabel;
    private javax.swing.JLabel windowLabel;
    private javax.swing.JLabel browserLabel;
    private javax.swing.JComboBox browserComboBox;
    private javax.swing.JTextArea helpTextArea;
    // End of variables declaration//GEN-END:variables

    private int uiMode;

    /** property editor for web browser customization */
    private PropertyEditor editor;

    /** listeners on validation of panel changes */
    private ArrayList changeListeners = new ArrayList(1);

    /** Get the component displayed in this panel.
    * @return the component
    */
    public java.awt.Component getComponent () {
        return this;
    }

    /** Help for this panel.
    * When the panel is active, this is used as the help for the wizard dialog.
    * @return the help or <code>null</code> if no help is supplied
    */
    public HelpCtx getHelp () {
        return null;
    }

    /** Provides the wizard panel with the current data--either
    * the default data or already-modified settings, if the user used the previous and/or next buttons.
    * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
    * @param settings the object representing wizard panel state, as originally supplied to
    * {@link WizardDescriptor#WizardDescriptor(WizardDescriptor.Iterator,Object)}
    */
    public void readSettings (Object settings) {
        if (settings instanceof WizardDescriptor) {
            WizardDescriptor wd = (WizardDescriptor)settings;
            wd.addPropertyChangeListener(this);
            if (Boolean.TRUE.equals(wd.getProperty("initializePanel")))
                initializePanel();
        }
    }

    /** Provides the wizard panel with the opportunity to update the
    * settings with its current customized state.
    * Rather than updating its settings with every change in the GUI, it should collect them,
    * and then only save them when requested to by this method.
    * Also, the original settings passed to {@link #readSettings} should not be modified (mutated);
    * rather, the (copy) passed in here should be mutated according to the collected changes.
    * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
    * @param settings the object representing a settings of the wizard
    */
    public void storeSettings (Object settings) {
    }

    /** Test whether the panel is finished and it is safe to proceed to the next one.
    * If the panel is valid, the "Next" (or "Finish") button will be enabled.
    * @return <code>true</code> if the user has entered satisfactory information
    */
    public boolean isValid () {
        return true;
    }
    
    /** Add a listener to changes of the panel's validity.
    * @param l the listener to add
    * @see #isValid
    */
    public void addChangeListener (ChangeListener l) {
        if (!changeListeners.contains(l)) {
            changeListeners.add(l);
        }
    }

    /** Remove a listener to changes of the panel's validity.
    * @param l the listener to remove
    */
    public void removeChangeListener (ChangeListener l) {
        changeListeners.remove(l);
    }
    
    /** Notifyes listeners that panel's validation changed */
    private void fireStateChanged() {
        for(int i = 0; i < changeListeners.size(); i++)
            ((ChangeListener)changeListeners.get(i)).stateChanged(
                new ChangeEvent(this));
    }
    
    /** Initialize advancedCheckBox
     * @param adv true to enable avanced checkbox
     */
    public void setAdvanced(boolean adv) {
    }

    /** Initialize panel from NB setting
     */
    private void initializePanel() {
        uiMode = WindowManagerImpl.getDefault().uiModeManager().getUIMode();
        sdiRadioButton.setSelected(uiMode == UIModeManager.SDI_MODE);
        mdiRadioButton.setSelected(uiMode == UIModeManager.MDI_MODE);

        proxyCheckBox.setSelected(UpdateSupport.isUseProxy());
        serverField.setText(UpdateSupport.getProxyHost());
        portField.setText(UpdateSupport.getProxyPort());
        setProxySettingEnabled(proxyCheckBox.isSelected());

        // web browser...
        browserLabel.setDisplayedMnemonic (bundle.getString ("CTL_BrowserLabel_mnemonic").charAt (0));
        browserLabel.setLabelFor (browserComboBox);
        helpTextArea.setFont (javax.swing.UIManager.getFont ("Label.font"));
        helpTextArea.setDisabledTextColor (javax.swing.UIManager.getColor ("Label.foreground"));
        // fill the web browser combo box
        try {
            IDESettings settings = (IDESettings)IDESettings.findObject (IDESettings.class, true);
            editor = PropertyEditorManager.findEditor (org.openide.awt.HtmlBrowser.Factory.class);
            editor.setValue (settings.getWWWBrowser ());
            String [] tags = editor.getTags ();
            browserComboBox.removeAllItems ();
            for (int j=0; j<tags.length; j++) {
                browserComboBox.insertItemAt (tags[j], j);
            }
            browserComboBox.setSelectedItem (editor.getAsText ());
        }
        catch (Exception ex) {
            TopManager.getDefault ().notifyException (ex);
        }
    }

    /** Handling of property changes in node structure and setup wizard descriptor
     */
    public void propertyChange(PropertyChangeEvent evt) {
        if ((evt.getSource() instanceof WizardDescriptor) 
            && (WizardDescriptor.PROP_VALUE.equals(evt.getPropertyName()))) {
                
            WizardDescriptor wd = (WizardDescriptor)evt.getSource();
            if (wd.getValue() == wd.FINISH_OPTION) {
                wd.removePropertyChangeListener(this);
                // set proxy
                UpdateSupport.setProxyConfiguration(
                    proxyCheckBox.isSelected(),
                    serverField.getText(),
                    portField.getText()
                );
                // set ui mode
                WindowManagerImpl.getDefault().uiModeManager().setUIMode(uiMode, true);
                // set web browser
                if (editor != null) {
                    try {
                        Object o = browserComboBox.getSelectedItem ();
                        if (o instanceof String && (o != null)) {
                            IDESettings settings = (IDESettings)IDESettings.findObject (IDESettings.class, true);
                            editor.setAsText ((String)o);
                            settings.setWWWBrowser ((org.openide.awt.HtmlBrowser.Factory)editor.getValue ());
                        }
                    }
                    catch (Exception ex) {
                        TopManager.getDefault ().notifyException (ex);
                    }
                }
            }
        }
    }
}
