/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.core.ui;

import java.beans.*;
import java.util.ArrayList;

import org.openide.TopManager;
import org.openide.WizardDescriptor;
import org.openide.awt.HtmlBrowser;

import org.netbeans.core.IDESettings;

/** Setup wizard panel for selecting web browser
 *
 * @author  gtichy
 */
public class WebBrowserPanel extends javax.swing.JPanel 
                             implements org.openide.WizardDescriptor.FinishPanel,
                                        PropertyChangeListener {

    java.util.ResourceBundle bundle = org.openide.util.NbBundle.getBundle ("org/netbeans/core/ui/Bundle");

    /** Creates new form WebBrowserPanel */
    public WebBrowserPanel () {
        initComponents();
        browserLabel.setDisplayedMnemonic (bundle.getString ("CTL_BrowserLabel_mnemonic").charAt (0));
        browserLabel.setLabelFor (browserComboBox);
        helpTextArea.setFont (javax.swing.UIManager.getFont ("Label.font"));
        helpTextArea.setDisabledTextColor (javax.swing.UIManager.getColor ("Label.foreground"));
        setName (bundle.getString ("LBL_WebBrowserPanelName"));
        
        // fill the combo box
        try {
            IDESettings settings = (IDESettings)IDESettings.findObject (IDESettings.class, true);
            editor = PropertyEditorManager.findEditor (org.openide.awt.HtmlBrowser.Factory.class);
            editor.setValue (settings.getWWWBrowser ());
            String [] tags = editor.getTags ();
            for (int j=0; j<tags.length; j++) {
                browserComboBox.insertItemAt (tags[j], j);
            }
            browserComboBox.setSelectedItem (editor.getAsText ());
        }
        catch (Exception ex) {
            TopManager.getDefault ().notifyException (ex);
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        browserLabel = new javax.swing.JLabel();
        browserComboBox = new javax.swing.JComboBox();
        helpTextArea = new javax.swing.JTextArea();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        browserLabel.setText(bundle.getString ("CTL_BrowserLabel"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(browserLabel, gridBagConstraints1);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.insets = new java.awt.Insets(0, 12, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(browserComboBox, gridBagConstraints1);
        
        helpTextArea.setWrapStyleWord(true);
        helpTextArea.setLineWrap(true);
        helpTextArea.setEditable(false);
        helpTextArea.setText(bundle.getString("CTL_HelpTextArea"));
        helpTextArea.setEnabled(false);
        helpTextArea.setOpaque(false);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(17, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(helpTextArea, gridBagConstraints1);
        
    }//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel browserLabel;
    private javax.swing.JComboBox browserComboBox;
    private javax.swing.JTextArea helpTextArea;
    // End of variables declaration//GEN-END:variables

    /** listeners on validation of panel changes */
    private ArrayList changeListeners = new ArrayList(1);

    /** property editor for web browser customization */
    private PropertyEditor editor;
    
    /** Get the component displayed in this panel.
    * @return the component
    */
    public java.awt.Component getComponent () {
        return this;
    }

    public org.openide.util.HelpCtx getHelp() {
        return null;
    }

    /** Test whether the panel is finished and it is safe to proceed to the next one.
    * If the panel is valid, the "Next" (or "Finish") button will be enabled.
    * @return <code>true</code> if the user has entered satisfactory information
    */
    public boolean isValid () {
        return true;
    }

    public void storeSettings(java.lang.Object obj) {
    }
    
    public void readSettings(java.lang.Object settings) {
        if (settings instanceof WizardDescriptor) {
            ((WizardDescriptor)settings).addPropertyChangeListener(this);
        }
    }
    
    public void addChangeListener(javax.swing.event.ChangeListener l) {
        if (!changeListeners.contains(l)) {
            changeListeners.add(l);
        }
    }

    public void removeChangeListener(javax.swing.event.ChangeListener l) {
        changeListeners.remove(l);
    }
    
    /** Handling of property changes in node structure and setup wizard descriptor
     */
    public void propertyChange(PropertyChangeEvent evt) {
        if ((evt.getSource() instanceof WizardDescriptor) 
            && (WizardDescriptor.PROP_VALUE.equals(evt.getPropertyName()))) {
                
            WizardDescriptor wd = (WizardDescriptor)evt.getSource();
            if (wd.getValue() == wd.FINISH_OPTION) {
                wd.removePropertyChangeListener(this);
                if (editor != null) {
                    try {
                        Object o = browserComboBox.getSelectedItem ();
                        if (o instanceof String && (o != null)) {
                            IDESettings settings = (IDESettings)IDESettings.findObject (IDESettings.class, true);
                            editor.setAsText ((String)o);
                            settings.setWWWBrowser ((HtmlBrowser.Factory)editor.getValue ());
                        }
                    }
                    catch (Exception ex) {
                        TopManager.getDefault ().notifyException (ex);
                    }
                }
            }
        }
    }
}
