/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.core.ui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.util.Map; // override javax.swing.html.Map
import java.text.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.text.html.parser.*;
import javax.swing.text.html.*;
import javax.swing.text.*;

import org.openide.*;
import org.openide.TopManager;
import org.openide.NotifyDescriptor;
import org.openide.loaders.*;
import org.openide.filesystems.*;
import org.openide.cookies.InstanceCookie;
import org.openide.nodes.*;
import org.openide.util.actions.*;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.SharedClassObject;

import org.netbeans.core.IDESettings;

/**
 * This class implements "Getting started" dialog.  HTML files are stored in
 * "org.netbeans.docs.startup.info#.html" files.  
 * It is also possible to determine (via checkbox) if the dialog
 * should appear the next time the IDE starts.
 *
 * @author  David Strupl
 */
public class WelcomePanel extends javax.swing.JPanel {

    private ResourceBundle bundle, bundle1;
    /** Where are the HTML files located.*/
    private static final String INFO_PREFIX = "org.netbeans.docs.startup.info"; // NOI18N
    /** Where the tips are located */
    private static final String TIPS_PREFIX = "org.netbeans.docs.tips.tip"; // NOI18N

    /** Property name for the property attached to the buttons. */
    private static final String BORDER_PROPERTY = "originalBorder"; // NOI18N
    /** Property name for the property attached to the buttons. */
    private static final String HELP_CTX_PROPERTY = "helpCtx"; // NOI18N
    /** Property name giving location of description for a component. */
    private static final String DESCRIPTION_PROPERTY = "welcomeDescription"; // NOI18N

    private static final Color BACKGROUND_COLOR_DEFAULT=new java.awt.Color(102, 102, 153),
                               BORDER_COLOR_HIGHLIGHT_DEFAULT=new java.awt.Color(255,255,255),
                               BORDER_COLOR_SHADOW_DEFAULT=new java.awt.Color(142,142,142);
    private static Color backgroundColor,borderShadowColor,borderHighlightColor;

    private IDESettings ideSettings;
    private int last = 1;

    /** Creates new form WelcomePanel */
    public WelcomePanel() {
        MouseListener mouselistenertipbutton;
        ideSettings = (IDESettings) SharedClassObject.findObject (IDESettings.class, true);
        bundle1 = bundle = NbBundle.getBundle(WelcomePanel.class);
        try {
            backgroundColor=new java.awt.Color(Integer.parseInt(bundle.getString("COLOR_WelcomeBACKGROUND"),16));
        } catch (Exception exc) {
            backgroundColor=BACKGROUND_COLOR_DEFAULT;
        }
        try {
            borderShadowColor=new java.awt.Color(Integer.parseInt(bundle.getString("COLOR_WelcomeBorderShadow"),16));
        } catch(Exception exc) {
            borderShadowColor=BORDER_COLOR_SHADOW_DEFAULT;
        }
        try {
            borderHighlightColor=new java.awt.Color(Integer.parseInt(bundle.getString("COLOR_WelcomeBorderHighlight"),16));
        } catch(Exception exc) {
            borderHighlightColor=BORDER_COLOR_HIGHLIGHT_DEFAULT;
        }
        initComponents ();
        setBackgroundColor(backgroundColor);
        showOnStartupCheckBox.setBorder(new EmptyBorder(5, 0, 0, 0));
        showOnStartupCheckBox.setSelected(ideSettings.getShowTipsOnStartup());
        showOnStartupCheckBox.addItemListener(new ItemListener() {
            public void itemStateChanged (ItemEvent e) {
                ideSettings.setShowTipsOnStartup(e.getStateChange() == ItemEvent.SELECTED);
            }
        });
        try {
            DataFolder df = TopManager.getDefault ().getPlaces ().folders ().welcome ();
            startLoading(df);
            java.net.URL url = null;
            url = NbBundle.getLocalizedFile(INFO_PREFIX, "html"); // NOI18N  }
            setBrowserPage(contextBrowser, url);
            setBrowserPage(tipsBrowser, url);
        } catch (Exception x) {
            TopManager.getDefault().notifyException(x);
        }
        HyperlinkListener hpl = new HyperlinkListener() {
            public void hyperlinkUpdate(HyperlinkEvent e) {
                if (e.getEventType () == HyperlinkEvent.EventType.ACTIVATED) {
                    TopManager tom = TopManager.getDefault();
                    tom.setStatusText(bundle.getString("CTL_OpeningBrowser"));
                    tom.showUrl(e.getURL());
                    TopManager.getDefault().setStatusText(""); // NOI18N
                }
             }
        };
        contextBrowser.addHyperlinkListener(hpl);
        tipsBrowser.addHyperlinkListener(hpl);
        mouselistenertipbutton=new MouseAdapter()
                                 {
                                   public void mouseEntered(MouseEvent e)
                                     {
                                       Object o = e.getSource();
                                       if(o instanceof JButton)
                                         ((JButton)o).setBorder(new BevelBorder(BevelBorder.RAISED));
                                     }
                                   public void mouseExited(MouseEvent e)
                                     {
                                       Object o = e.getSource();
                                       if(o instanceof JButton)
                                         {
                                           ((JButton)o).setBorder(new EmptyBorder(2,2,2,2));
                                           changeFocus();
                                         }
                                     }
                                 };
        nextTipButton.addMouseListener(mouselistenertipbutton);
        prevTipButton.addMouseListener(mouselistenertipbutton);
        restore();
        // TODO: refresh the layout somehow
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jPanelHtml = new javax.swing.JPanel();
        contextBrowserScroll = new javax.swing.JScrollPane();
        contextBrowser = new javax.swing.JEditorPane();
        jLabelHtmlTitle = new javax.swing.JLabel();
        buttonSetPanel = new javax.swing.JPanel();
        jPanelButton = new javax.swing.JPanel();
        showOnStartupCheckBox = new javax.swing.JCheckBox();
        closeButton = new javax.swing.JButton();
        jPanelTip = new javax.swing.JPanel();
        jLabelTipTitle = new javax.swing.JLabel();
        jPanelTipButton = new javax.swing.JPanel();
        nextTipButton = new javax.swing.JButton();
        prevTipButton = new javax.swing.JButton();
        tipsScrollPane = new javax.swing.JScrollPane();
        tipsBrowser = new javax.swing.JEditorPane();
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        setPreferredSize(new java.awt.Dimension(400, 300));
        setName("");
        setMinimumSize(new java.awt.Dimension(400, 300));
        
        jPanelHtml.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints2;
        jPanelHtml.setBorder(new javax.swing.border.EtchedBorder());
        jPanelHtml.setRequestFocusEnabled(false);
        
        contextBrowserScroll.setPreferredSize(new java.awt.Dimension(170, 140));
        contextBrowserScroll.setBorder(new javax.swing.border.EtchedBorder());
        contextBrowserScroll.setMinimumSize(new java.awt.Dimension(170, 140));
        contextBrowserScroll.setRequestFocusEnabled(false);
        
        contextBrowser.setPreferredSize(new java.awt.Dimension(170, 170));
        contextBrowser.setBorder(new javax.swing.border.EtchedBorder());
        contextBrowser.setEditable(false);
        contextBrowser.setText("Context browser");
        contextBrowser.setMinimumSize(new java.awt.Dimension(170, 170));
        contextBrowser.setRequestFocusEnabled(false);
        contextBrowser.setEditorKit(new javax.swing.text.html.HTMLEditorKit ());
        contextBrowserScroll.setViewportView(contextBrowser);
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 1;
        gridBagConstraints2.gridy = 1;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints2.insets = new java.awt.Insets(0, 12, 11, 11);
        gridBagConstraints2.weightx = 1.0;
        gridBagConstraints2.weighty = 1.0;
        jPanelHtml.add(contextBrowserScroll, gridBagConstraints2);
        
        
        jLabelHtmlTitle.setText(bundle.getString("LABEL_WelcomeHTMLPages"));
        jLabelHtmlTitle.setForeground(java.awt.Color.black);
        jLabelHtmlTitle.setFont(new java.awt.Font ("Dialog", 1, 11));
        jLabelHtmlTitle.setRequestFocusEnabled(false);
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridwidth = 2;
        gridBagConstraints2.insets = new java.awt.Insets(12, 12, 11, 11);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints2.weightx = 1.0;
        jPanelHtml.add(jLabelHtmlTitle, gridBagConstraints2);
        
        
        buttonSetPanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints3;
        buttonSetPanel.setPreferredSize(new java.awt.Dimension(170, 140));
        buttonSetPanel.setMinimumSize(new java.awt.Dimension(170, 140));
        buttonSetPanel.setMaximumSize(new java.awt.Dimension(170, 140));
        buttonSetPanel.setRequestFocusEnabled(false);
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 0;
        gridBagConstraints2.gridy = 1;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints2.insets = new java.awt.Insets(0, 12, 11, 12);
        gridBagConstraints2.weighty = 1.0;
        jPanelHtml.add(buttonSetPanel, gridBagConstraints2);
        
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(12, 12, 0, 11);
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(jPanelHtml, gridBagConstraints1);
        
        
        jPanelButton.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints4;
        jPanelButton.setPreferredSize(new java.awt.Dimension(278, 35));
        jPanelButton.setMinimumSize(new java.awt.Dimension(278, 35));
        jPanelButton.setMaximumSize(new java.awt.Dimension(32767, 100));
        jPanelButton.setRequestFocusEnabled(false);
        
        showOnStartupCheckBox.setText(bundle.getString("CTL_NEXTSTARTUP"));
        showOnStartupCheckBox.setNextFocusableComponent(closeButton);
        gridBagConstraints4 = new java.awt.GridBagConstraints();
        gridBagConstraints4.gridx = 0;
        gridBagConstraints4.gridy = 0;
        gridBagConstraints4.insets = new java.awt.Insets(17, 0, 11, 0);
        gridBagConstraints4.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints4.weightx = 1.0;
        jPanelButton.add(showOnStartupCheckBox, gridBagConstraints4);
        
        
        closeButton.setText(bundle.getString("CTL_CLOSE"));
        gridBagConstraints4 = new java.awt.GridBagConstraints();
        gridBagConstraints4.insets = new java.awt.Insets(17, 12, 11, 0);
        jPanelButton.add(closeButton, gridBagConstraints4);
        
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(17, 12, 11, 11);
        add(jPanelButton, gridBagConstraints1);
        
        
        jPanelTip.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints5;
        jPanelTip.setBorder(new javax.swing.border.EtchedBorder());
        jPanelTip.setRequestFocusEnabled(false);
        
        jLabelTipTitle.setText(bundle.getString("LABEL_WelcomeTipPages"));
        jLabelTipTitle.setForeground(java.awt.Color.black);
        jLabelTipTitle.setFont(new java.awt.Font ("Dialog", 1, 11));
        jLabelTipTitle.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        jLabelTipTitle.setRequestFocusEnabled(false);
        gridBagConstraints5 = new java.awt.GridBagConstraints();
        gridBagConstraints5.gridwidth = 2;
        gridBagConstraints5.insets = new java.awt.Insets(12, 12, 11, 11);
        gridBagConstraints5.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints5.weightx = 1.0;
        jPanelTip.add(jLabelTipTitle, gridBagConstraints5);
        
        
        jPanelTipButton.setLayout(new javax.swing.BoxLayout(jPanelTipButton, 1));
        jPanelTipButton.setRequestFocusEnabled(false);
        
        nextTipButton.setPreferredSize(new java.awt.Dimension(170, 27));
        nextTipButton.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(2, 2, 2, 2)));
        nextTipButton.setText(bundle1.getString("CTL_NEXT_BUTTON"));
        nextTipButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/netbeans/core/resources/actions/tipOfTheDay.gif")));
        nextTipButton.setMaximumSize(new java.awt.Dimension(170, 27));
        nextTipButton.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        nextTipButton.setMinimumSize(new java.awt.Dimension(170, 27));
        nextTipButton.setNextFocusableComponent(prevTipButton);
        nextTipButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                nextTipButtonActionPerformed(evt);
            }
        }
        );
        jPanelTipButton.add(nextTipButton);
        
        
        prevTipButton.setPreferredSize(new java.awt.Dimension(170, 27));
        prevTipButton.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(2, 2, 2, 2)));
        prevTipButton.setText(bundle1.getString("CTL_PREV_BUTTON"));
        prevTipButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/netbeans/core/resources/actions/tipOfTheDay.gif")));
        prevTipButton.setMaximumSize(new java.awt.Dimension(170, 27));
        prevTipButton.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        prevTipButton.setMinimumSize(new java.awt.Dimension(170, 27));
        prevTipButton.setNextFocusableComponent(showOnStartupCheckBox);
        prevTipButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                prevTipButtonActionPerformed(evt);
            }
        }
        );
        jPanelTipButton.add(prevTipButton);
        
        gridBagConstraints5 = new java.awt.GridBagConstraints();
        gridBagConstraints5.gridx = 0;
        gridBagConstraints5.gridy = 1;
        gridBagConstraints5.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints5.insets = new java.awt.Insets(0, 12, 11, 12);
        gridBagConstraints5.weighty = 1.0;
        jPanelTip.add(jPanelTipButton, gridBagConstraints5);
        
        
        tipsScrollPane.setMinimumSize(new java.awt.Dimension(22, 40));
        
        tipsBrowser.setPreferredSize(new java.awt.Dimension(377, 40));
        tipsBrowser.setFont(new java.awt.Font ("Dialog", 0, 11));
        tipsBrowser.setEditable(false);
        tipsBrowser.setMinimumSize(new java.awt.Dimension(173, 40));
        tipsBrowser.setRequestFocusEnabled(false);
        tipsBrowser.setEditorKit(new javax.swing.text.html.HTMLEditorKit ());
        tipsScrollPane.setViewportView(tipsBrowser);
        
        gridBagConstraints5 = new java.awt.GridBagConstraints();
        gridBagConstraints5.gridx = 1;
        gridBagConstraints5.gridy = 1;
        gridBagConstraints5.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints5.insets = new java.awt.Insets(0, 12, 11, 11);
        gridBagConstraints5.weightx = 1.0;
        gridBagConstraints5.weighty = 1.0;
        jPanelTip.add(tipsScrollPane, gridBagConstraints5);
        
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(0, 12, 0, 11);
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(jPanelTip, gridBagConstraints1);
        
    }//GEN-END:initComponents

  private void prevTipButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_prevTipButtonActionPerformed
        prevTip();
  }//GEN-LAST:event_prevTipButtonActionPerformed

  private void nextTipButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_nextTipButtonActionPerformed
        nextTip();
  }//GEN-LAST:event_nextTipButtonActionPerformed


  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JPanel jPanelHtml;
  private javax.swing.JScrollPane contextBrowserScroll;
  private javax.swing.JEditorPane contextBrowser;
  private javax.swing.JLabel jLabelHtmlTitle;
  private javax.swing.JPanel buttonSetPanel;
  private javax.swing.JPanel jPanelButton;
  private javax.swing.JCheckBox showOnStartupCheckBox;
  private javax.swing.JButton closeButton;
  private javax.swing.JPanel jPanelTip;
  private javax.swing.JLabel jLabelTipTitle;
  private javax.swing.JPanel jPanelTipButton;
  private javax.swing.JButton nextTipButton;
  private javax.swing.JButton prevTipButton;
  private javax.swing.JScrollPane tipsScrollPane;
  private javax.swing.JEditorPane tipsBrowser;
  // End of variables declaration//GEN-END:variables

    javax.swing.JButton getCloseButton() {
        return closeButton;
    }
    
    /** the folder which represents and loads content of the getting started button set */
    private GettingStartedFolder gettingStartedFolder;

    /** Blocks until the panel is completely created. */
    public void waitFinished () {
        gettingStartedFolder.instanceFinished();
    }

    /** Saves the contents of this object to the specified stream.
     *
     * @exception IOException Includes any I/O exceptions that may occur
     */
    public void writeExternal(ObjectOutput out) throws IOException {
        out.writeObject(gettingStartedFolder.getFolder());
    }

    /**
     * Restores contents of this object from the specified stream.
     *
     * @exception ClassNotFoundException If the class for an object being
     *              restored cannot be found.
     */
    public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
        startLoading((DataFolder)in.readObject());
    }

    /** Starts loading of this panel from a folder */
    void startLoading (final DataFolder folder) {
        gettingStartedFolder = new GettingStartedFolder(folder, buttonSetPanel);
    }

    /** Called when the tips dialog is about to be displayed to allow
     * the TipsOnStartup to perform initialization
     */
    void restore () {
        last = ideSettings.getLastTip();
        showOnStartupCheckBox.setSelected(ideSettings.getShowTipsOnStartup());
        nextTip();
    }

    /**
     * Shows next tip into given text area
     */
    void nextTip() {
        java.net.URL url;
        try {
            url = NbBundle.getLocalizedFile(TIPS_PREFIX + last++, "html"); // NOI18N
        } catch (MissingResourceException e) {
            last = 1;
            try {
                url = NbBundle.getLocalizedFile(TIPS_PREFIX + last++, "html"); // NOI18N
            } catch (MissingResourceException e2) {
                url = null;
            }
        }
        if (url == null) {
            TopManager.getDefault ().notify (
                new NotifyDescriptor.Message (
                    NbBundle.getBundle (WelcomePanel.class).getString ("ERR_NoTipsFound"),
                    NotifyDescriptor.WARNING_MESSAGE));
            return;
        }

        ideSettings.setLastTip (last);

        // Assertion.assert(url != null);
        try {
            setBrowserPage(tipsBrowser, url);
        } catch (IOException e) {
            // PENDING(david) EXC_URL_Not_found bundle
            TopManager.getDefault().notifyException(e);
        }
        changeFocus();
    }

    void prevTip () {
        java.net.URL url=null;
        last--;
        while(url==null) {
            if(last--<1)
                last=calculateCountTip();
            try {
                url=NbBundle.getLocalizedFile(TIPS_PREFIX+last,"html");
            }
            catch(MissingResourceException exc) {
                if(last==1) {
                      url=null;
                      break;
                }
            }
        }
        last++;
        if (url == null) {
            TopManager.getDefault ().notify (
                new NotifyDescriptor.Message (
                    NbBundle.getBundle (WelcomePanel.class).getString ("ERR_NoTipsFound"),
                    NotifyDescriptor.WARNING_MESSAGE));
            return;
        }

        ideSettings.setLastTip (last);

        // Assertion.assert(url != null);
        try {
            setBrowserPage(tipsBrowser, url);
        } catch (IOException e) {
            // PENDING(david) EXC_URL_Not_found bundle
            TopManager.getDefault().notifyException(e);
        }
        changeFocus();
    }

    int calculateCountTip () {
        int num=1;
        java.net.URL url=null;
        try {
            while(true)
                url=NbBundle.getLocalizedFile(TIPS_PREFIX+num++,"html");
        }
        catch(Exception exc) {
        }
        return num;
    }

    /** This class can be used to produce a <code>JPanel</code> instance
     * from the given <code>DataFolder</code>.
     */
    final class GettingStartedFolder extends FolderInstance
        implements NodeListener {

        /** the <code>JButton[]</code> to work with */
        private JPanel buttonPanel;

        /** Maps (String(action name) --> String (url) */
        private Map descriptionURLs;
        
        /**
         * Creates a panel on the specified <code>DataFolder</code>.
         * @param folder a <code>DataFolder</code> to work with
         * @param panel a <code>JPanel</code> containing the buttons
         */
        public GettingStartedFolder (final DataFolder folder, JPanel panel) {
            super(folder);
            buttonPanel = panel;
            recreate();
            instanceFinished();
        }

        /**
         * Creates a panel on the specified <code>DataFolder</code>.
         * @param folder a <code>DataFolder</code> to work with
         */
        public GettingStartedFolder (final DataFolder folder) {
            super(folder);
            recreate();
            instanceFinished();
        }

        /** Fired when a set of new children is added.
         * @param ev event describing the action
         */
        public void childrenAdded (NodeMemberEvent ev) {
        }

        /** Fired when a set of children is removed.
         * @param ev event describing the action
         */
        public void childrenRemoved (NodeMemberEvent ev) {
        }

        /** Fired when the order of children is changed.
         * @param ev event describing the change
         */
        public void childrenReordered(NodeReorderEvent ev) {
        }

        /** Fired when the node is deleted.
         * @param ev event describing the node
         */
        public void nodeDestroyed (NodeEvent ev) {
        }

        public void propertyChange(java.beans.PropertyChangeEvent e) {
        }

        /**
         * @return the name
         */
        public String instanceName () {
            return "GettingStarted"; // TODO : I18N
        }

        /** Returns the class of represented ????.
         * @return Object.class
         */
        public Class instanceClass ()
            throws java.io.IOException, ClassNotFoundException {
            return JPanel.class;
        }

        /**
         * Accepts only cookies that can provide <code>SystemAction</code>.
         * @param cookie an <code>InstanceCookie</code> to test
         * @return true if the cookie can provide accepted instances
         */
        protected InstanceCookie acceptCookie(InstanceCookie cookie)
            throws java.io.IOException, ClassNotFoundException {
            Class c = cookie.instanceClass();
            if (Presenter.Toolbar.class.isAssignableFrom (c)) {
                return cookie;
            } else if (JComponent.class.isAssignableFrom (c)) {
                return cookie;
            } else {
                return null;
            }
        }

        /**
         * Returns a <code>GettingStarted.GettingStartedFolder</code> cookie for the specified
         * <code>DataFolder</code>.
         * @param df a <code>DataFolder</code> to create the cookie for
         * @return a <code>GettingStarted.GettingStartedFolder</code> for the specified folder
         */
        protected InstanceCookie acceptFolder(DataFolder df) {
            return new GettingStartedFolder(df);
        }

        /** For outer class access to the data folder */
        DataFolder getFolder () {
            return folder;
        }

        protected InstanceCookie acceptDataObject(DataObject dob) {
            InstanceCookie cookie = super.acceptDataObject(dob);
            if (cookie != null) {
                URL descURL = (URL)dob.getPrimaryFile().getAttribute(DESCRIPTION_PROPERTY);
                if (descURL != null) {
                    getDescriptionURLs().put(cookie.instanceName(), descURL);
                } else {
                    TopManager.getDefault().getErrorManager().log(ErrorManager.WARNING, 
                       "WelcomePanel: " + dob.getPrimaryFile().getName() + " property welcomeDescription missing.");
                }
            }
            return cookie;
        }
        
        /** Updates the <code>JPanel</code> represented by this folder.
         * @param cookies array of instance cookies for the folder
         * @return the updated <code>JPanel</code> representee
         */
        protected Object createInstance(InstanceCookie[] cookies)
            throws java.io.IOException, ClassNotFoundException {
            // clear first
            if (buttonPanel != null) {
                buttonPanel.removeAll();
            } else {
                buttonPanel = new JPanel();
                buttonPanel.setLayout(new java.awt.GridBagLayout());
            }
            for (int i = 0; i < cookies.length; i++) {
                try {
                    Object obj = cookies[i].instanceCreate();
                    Object ob2 = obj;
                    if (obj instanceof Presenter.Toolbar) {
                        obj = ((Presenter.Toolbar)obj).getToolbarPresenter ();
                    }

                    if (obj instanceof JComponent) {
                        Object url = getDescriptionURLs().get(cookies[i].instanceName());
                        if (url != null) {
                            ((JComponent) obj).putClientProperty (DESCRIPTION_PROPERTY, url);
                        }
                        // [PENDING] should this be AbstractButton instead?
                        if (obj instanceof JButton) {
                            final JButton b = (JButton) obj;
                            if (ob2 instanceof SystemAction) {
                                // [PENDING] is this block necessary? The default toolbar button
                                // presenters for CallableSystemAction's should set both the text
                                // and the help client property automatically.
                                SystemAction sa = (SystemAction)ob2;
                                b.putClientProperty(HELP_CTX_PROPERTY, sa.getHelpCtx());
                                b.setText(sa.getName());
                            }
                            b.addMouseListener(new MouseAdapter() {
                                public void mouseEntered(MouseEvent e) {
                                    updateContextBrowser(e);
                                }
                                // restore the original border
                                public void mouseExited(MouseEvent e) {
                                    Border brdr = (Border) b.getClientProperty(BORDER_PROPERTY);
                                    if (brdr != null) {
                                        b.setBorder(brdr);
                                    }
                                    changeFocus();
                                }
                            });
                            b.addActionListener(new ActionListener () {
                                public void actionPerformed (ActionEvent evt) {
                                    changeFocus();
                                }
                            });
                            b.setBorder(new EmptyBorder(2,2,2,2));
                            b.putClientProperty(BORDER_PROPERTY, b.getBorder());
                            b.setBackground(backgroundColor);
                            b.setForeground(java.awt.Color.black);
                            b.setHorizontalAlignment(SwingConstants.LEFT);
                            String text = b.getText();
                            if (text.indexOf('&') >= 0) {
                                StringBuffer sb = new StringBuffer(text);
                                sb.deleteCharAt(text.indexOf('&'));
                                text = sb.toString();
                                b.setText(text);
                            }
                        } else {
                              if (obj instanceof JLabel) {
                                  // We don't want to display the labels, but want
                                  // it to have some height
                                  ((JLabel)obj).setText(" "); // NO-I18N
                              }
                        }
                        GridBagConstraints constraints = new java.awt.GridBagConstraints();
                        constraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
                        constraints.gridx = 0;
                        constraints.gridy = i;
                        constraints.weightx = 1.0;
                        constraints.weighty = 0.0;
                        constraints.insets = new java.awt.Insets(2, 0, 0, 10);
                        constraints.anchor = java.awt.GridBagConstraints.NORTH;
                        buttonPanel.add((Component)obj, constraints);
                    } 
                } catch (Exception exc) {
                    TopManager.getDefault().notifyException(exc);
                }
                // add an empty component to the buttom of the panel
                GridBagConstraints constraints = new java.awt.GridBagConstraints();
                constraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
                constraints.gridx = 0;
                constraints.gridy = cookies.length;
                constraints.weightx = 1.0;
                constraints.weighty = 1.0;
                constraints.insets = new java.awt.Insets(4, 0, 0, 10);
                constraints.anchor = java.awt.GridBagConstraints.SOUTH;
                buttonPanel.add(Box.createHorizontalStrut(1), constraints);
            }
            return buttonPanel;
        }
        /** Updates HTML page in the ContextBrowser and sets "nice" border
         * around the button
         */
        public void updateContextBrowser(MouseEvent ev) {
            Object o = ev.getSource();
            if (o instanceof JButton) {
                JButton jb = (JButton) o;
                jb.setToolTipText(null);
                jb.setBorder(new BevelBorder(BevelBorder.RAISED));
                URL url = (URL) jb.getClientProperty (DESCRIPTION_PROPERTY);
                if (url != null) {
                    try {
                        setBrowserPage(contextBrowser, url);
                        // originally:
                        // contextBrowser.setPage(url);
                    } catch (MissingResourceException mre) {
                        TopManager.getDefault().getErrorManager().annotate(mre, 
                                "WelcomePanel: "+ url + " cannot be loaded.");
                        TopManager.getDefault().getErrorManager().notify(ErrorManager.WARNING, mre);
                    } catch (IOException ioe) {
                        TopManager.getDefault().getErrorManager().annotate(ioe, 
                                "WelcomePanel: "+ url + " cannot be loaded.");
                        TopManager.getDefault().getErrorManager().notify(ErrorManager.WARNING, ioe);
                    }
                }
            }
        }
        
        /** Lazy initialization of a HashMap.*/
        private Map getDescriptionURLs() {
            if (descriptionURLs == null) {
                descriptionURLs = new HashMap(10);
                    // this magic number (10) should be greater than number of items
                    // in the welcome folder (currently 7) in order not to rehash
            }
            return descriptionURLs;
        }
    }
    
    void changeFocus () {
        closeButton.requestFocus();
    }

    /** Encoding extracted by EncParser. */
    private static String tmpEncoding = null;
    
    /* Try to determine the encoding of a HTML file from the META tag. 
     * Can return null if not succesfull.
     */
    private static String getEncodingFromMeta(URL url) {
        tmpEncoding = null;
        try {
            Parser p = new EncParser(DTD.getDTD("html32"));
            p.parse(new InputStreamReader(url.openStream()));
        } catch (IOException ex) {
            // if we are not susccesful just return null
        }
        return tmpEncoding;
    }

    /** HTML Parser that tries to find META tag with encoding.
     */
    private static class EncParser extends Parser {
        public EncParser(DTD dtd) {
            super(dtd);
        }

        protected void handleError(int ln, String msg) {
            // this is not empty in superclass (JDK1.2.2 Swing bug).
        }

        /** The META tag is handled by this method.*/
        protected void handleEmptyTag(TagElement tag) throws ChangedCharSetException {
            javax.swing.text.html.parser.Element elem = tag.getElement();
	    if (elem == dtd.meta) {
	        SimpleAttributeSet atts = getAttributes();
	        if (atts != null) {
		    String content = (String)atts.getAttribute(HTML.Attribute.CONTENT);
		    if (content != null) {
		        if ("content-type".equalsIgnoreCase((String)atts.getAttribute(HTML.Attribute.HTTPEQUIV))) {
                            if (content.indexOf("=") >=0) {
                                // searching for literal = becaues it is usally text/html; charset=XYZ
                                handleCharset(content.substring(content.indexOf("=")+1));
                            } else {
                                handleCharset(content);
                            }
		        } else if ("charset" .equalsIgnoreCase((String)atts.getAttribute(HTML.Attribute.HTTPEQUIV))) {
			    handleCharset(content);
		        }
		    }
	        }
	    }
        }
        /** Check the encoding and if it is valid sets tmpEncoding variable.*/
        private void handleCharset(String param) {
            try {
                InputStreamReader isr = new InputStreamReader(new ByteArrayInputStream(new byte[0]), param);
            } catch (UnsupportedEncodingException ex) {
                return; // in case the encoding does not exist do nothing
            }
            tmpEncoding = param;
        }
    }
    
    /** A replacement for the JEditorPane.setPage(URL url) method.
     * To make it work with swing JDK1.2.2. It reads the text
     * from the url and creates a string.
     */
    private static void setBrowserPage(JEditorPane browser, URL url) throws java.io.IOException {
        StringWriter w = new StringWriter();
        InputStreamReader isr = null;
        String enc = getEncodingFromMeta(url);
        if (enc != null) {
            isr = new InputStreamReader(url.openStream(), enc);
        } else {
            isr = new InputStreamReader(url.openStream());
        }
        int c = -1;
        while ( (c = isr.read()) != -1) {
            w.write(c);
        }
        isr.close();
        // ----------- replacement for setText ------------------------
        String t = w.toString();
        try {
	    javax.swing.text.Document doc = browser.getDocument();
	    doc.remove(0, doc.getLength());
            if (enc != null) {
                doc.putProperty("content-encoding", enc);
            }
            doc.putProperty("IgnoreCharsetDirective", Boolean.TRUE);
	    Reader r = new StringReader(t);
	    javax.swing.text.EditorKit kit = browser.getEditorKit();
            kit.read(r, doc, 0);
        } catch (javax.swing.text.BadLocationException ble) {
            // silently ignore?
	}
        //--------------------------------------------------------
        //        browser.setText(w.toString());
        browser.setCaretPosition(0);
    }
    
    // =======================================================================

    private static Dialog gettingStartedDialog = null;
    
    public static void showGettingStarted () {
        if (gettingStartedDialog == null) {
            final Object [] options = new Object [] {
            };
            final WelcomePanel gettingStartedComponent = new WelcomePanel();
            gettingStartedComponent.getCloseButton().addActionListener(new ActionListener () {
                    public void actionPerformed (ActionEvent evt) {
                        if (gettingStartedComponent.getCloseButton().equals (evt.getSource ())) {
                            gettingStartedDialog.setVisible (false);
                        }
                    }
                }
            );
            DialogDescriptor descriptor = new DialogDescriptor (
                gettingStartedComponent,
                NbBundle.getBundle((WelcomePanel.class)).getString("CTL_START_TITLE"),
                false,
                options,
                null, 
                DialogDescriptor.BOTTOM_ALIGN,
                null, // HelpCtx
                null // action listener
            );
            gettingStartedDialog = TopManager.getDefault ().createDialog (descriptor);
        }
        gettingStartedDialog.setSize(640,480);
        if(gettingStartedDialog instanceof JDialog) {
            ((JDialog)gettingStartedDialog).setLocationRelativeTo(null);
        }
        gettingStartedDialog.setBackground(backgroundColor);
        gettingStartedDialog.show();
    }
    // =============================================================
    public static void main(String args[]) {
        showGettingStarted ();
    }
    
    boolean setBackgroundColor (java.awt.Color color) {
        if(color==null)
            return(false);
        this.setBackground(color);
        jPanelHtml.setBackground(color);
        buttonSetPanel.setBackground(color);
        jPanelTip.setBackground(color);
        jPanelTipButton.setBackground(color);
        nextTipButton.setBackground(color);
        prevTipButton.setBackground(color);
        jPanelButton.setBackground(color);
        showOnStartupCheckBox.setBackground(color);
        closeButton.setBackground(color);
        if(borderHighlightColor!=null&&borderShadowColor!=null)
          {
            jPanelHtml.setBorder(new EtchedBorder(borderHighlightColor,borderShadowColor));
            jPanelTip.setBorder(new EtchedBorder(borderHighlightColor,borderShadowColor));
          }
        return(true);
    }

}
