/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/*
 * FSChooser.java
 *
 * Created on March 30, 2001, 11:32 AM
 */

package org.netbeans.core.upgrade;

import java.awt.Dimension;
import java.awt.Window;
import java.awt.Container;
import java.awt.Component;
import java.awt.event.KeyEvent;
import javax.swing.JFileChooser;

/** Class used for the choosing of filesystem (local or jar) 
 */
public class FSChooser extends JFileChooser implements java.awt.event.KeyListener {

    // parent window
    private java.awt.Window win;

    public FSChooser() {
        setBorder(new javax.swing.border.EmptyBorder(0, 8, 0, 8));
    }

    public Dimension getPreferredSize() {
        Dimension pref = super.getPreferredSize ();
        return new Dimension(Math.max (425, pref.width), Math.max (250, pref.height));
    }

    // JDK 1.3 - jdk 1.2 compatibility hack [TODO]
    private static Object getWindowAncestor;
    private static Object[] compRef;

    /** Catches top level window (JDialog) in which this chooser is displayed */
    public void addNotify() {
        super.addNotify();
        win = getWindowAncestor(this);
        win.addKeyListener(this);
        // following method does not work well in JDK 1.3 - so use KeyListener...
        // registerKeyboardAction(this, KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), JFileChooser.WHEN_IN_FOCUSED_WINDOW);
    }

    /** Disposes the JDialog in which this chooser was displayed */
    void dispose() {
        if (win instanceof javax.swing.JDialog) { // not null and not MDI
            win.setVisible(false);
            win.dispose();
            win.removeKeyListener(this);
            win = null;
        }
    }

    /**
     * @return the first Window ancestor of c, or null if component is not contained inside a window
     */
    public static Window getWindowAncestor(Component c) {
        for(Container p = c.getParent(); p != null; p = p.getParent()) {
            if (p instanceof Window) {
                return (Window)p;
            }
        }
        return null;
    }

    public void keyReleased(final java.awt.event.KeyEvent p1) {
    }

    public void keyPressed(final java.awt.event.KeyEvent ke) {
    }

    public void keyTyped(final java.awt.event.KeyEvent ke) {
        if (ke.getKeyChar() == KeyEvent.VK_ESCAPE) {
            dispose();
            ke.consume();
        }
    }

}
