/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.core.upgrade;

import java.io.*;
import java.util.*;
import java.beans.PropertyVetoException;
import org.openide.util.*;
import java.util.jar.*;

import org.openide.filesystems.*;
import org.openide.filesystems.FileSystem;
import org.openide.execution.NbClassPath;

/** Command line interface to the copy utilities.
 *
 * @author Petr Zajac
 * @version
 */
public final class FileCopy extends CopyUtil {
    /** Uses System.out.println
     * @param msg message to print
     */
    protected void message(String msg) {
        System.out.println(msg);
    }
    
    protected void progress(int current, int max) {
        System.out.print(".");
        if (current == max)
            System.out.println();
    }
    
    private static FileSystem getBackup (String backupDir) throws IOException,PropertyVetoException {
        File backupFile = new File(backupDir);
        if (backupFile.exists()) {
            if (backupFile.isDirectory () == false ) {
                System.out.println(getString("MSG_BACKUP_EXISTS"));
                return null;
            } else {
                if (backupFile.list().length > 0 ) {
                    System.out.println(getString("MSG_NO_BACKUP_EMPTY"));
                    return null;
                }
            }
        } else {
            if ( backupFile.mkdirs () == false ) {
                System.out.println(getString("MSG_NO_BACKUP_CREATED"));
                return null;
            }
        }
        LocalFileSystem fs = new LocalFileSystem();
        fs.setRootDirectory(backupFile);
        return fs;
    }
    
    public static void main (String args[]) {
        System.out.println(getString("MSG_TITLE"));
        try {
            if (args.length == 3 ) {
                String backupDir = args [0];
                String sourceDir = args[1];
                String nbSourceDir = args [2];
                File file = new File (sourceDir);
                if (!file.exists()) {
                    System.out.println(getString("MSG_NO_OLD_HOME"));
                    return;
                }
                
                FileCopy copy = new FileCopy ();
                
                // creates backup directory
                FileSystem backupSystem = getBackup (backupDir);
                // create netbeans/system FileSystem
                //
                File nbDirFile = new File (nbSourceDir);
                if (nbDirFile.exists() == false || nbDirFile.isDirectory() == false ) {
                    copy.message (getString("MSG_NO_NETBEANS_HOME") );
                    return;
                }
                File systemDirFile = new File (nbDirFile,"system"); //NOI18N
                if (systemDirFile.exists() == false || systemDirFile.isDirectory() == false ) {
                    copy.message (getString("MSG_NO_SYSTEM_DIRECTORY") + systemDirFile);
                    return;
                }
                LocalFileSystem fs = new LocalFileSystem();
                fs.setRootDirectory(systemDirFile);

                copy.updateIde (new File (sourceDir), fs, backupSystem);
                
                backupSystem.removeNotify ();
            } else {
                String scriptName = System.getProperty("os.name").startsWith("Windows")  ? //NOI18N
                "CTL_WINDOWS_SCRIPT" : "CTL_UNIX_SCRIPT";
                scriptName = getString(scriptName);
                System.out.println(getString("MSG_COMMAND_LINE",scriptName));
            }
        } catch (Exception e) {
            e.printStackTrace ();
        }
    }
    
    
}
