/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.core.upgrade;

import org.openide.filesystems.FileSystem;
import java.io.File;
import java.io.FileInputStream;
import java.util.*;

/** Graphical interface to the copy utilities.
 * 
 */
public final class InternalCopy extends CopyUtil {

    private Vector listeners = new Vector();
    
    /** Constructor
     */
    public InternalCopy() {
    }
    
    /** GUI print.
     * @param msg message to print
     */
    protected void message(String msg) {
        fireMessage(msg);
    }    
    
    protected void progress(int current, int max) {
        fireProgress(current, max);
    }

    public void addUpgradeProcessListener(UpgradeProcessListener lis) {
        listeners.add(lis);
    }
    
    public void removeUpgradeProcessListener(UpgradeProcessListener lis) {
        listeners.remove(lis);
    }

    public void upgrade (
        File sourceDir, FileSystem targetSystem, FileSystem backupSystem
    ) {
        boolean succeeded;
        
        fireStart();
        try {
            updateIde(sourceDir, targetSystem, backupSystem);
            succeeded = true;
        }
        catch (Exception e) {
            String msg = e.getMessage();
            if (null != msg && 0 != msg.length())
                fireMessage(e.getMessage());
            
            succeeded = false;
        }
        fireEnd(succeeded);
    }

    public void upgrade (
        String version, File sourceDir, FileSystem targetSystem, FileSystem backupSystem
    ) {
        boolean succeeded;
        
        fireStart();
        try {
            updateIde(CopyUtil.getIdeVersion(version), sourceDir, targetSystem, backupSystem);
            succeeded = true;
        }
        catch (Exception e) {
            String msg = e.getMessage();
            if (null != msg && 0 != msg.length())
                fireMessage(e.getMessage());
            
            succeeded = false;
        }
        fireEnd(succeeded);
    }
    
    private void fireMessage(String message) {
        Iterator i = listeners.iterator();
        while (i.hasNext()) {
            UpgradeProcessListener lis = (UpgradeProcessListener)i.next();
            lis.message(message);
        }
    }
    
    private void fireProgress(int current, int max) {
        Iterator i = listeners.iterator();
        while (i.hasNext()) {
            UpgradeProcessListener lis = (UpgradeProcessListener)i.next();
            lis.progress(current, max);
        }
    }

    private void fireStart() {
        Iterator i = listeners.iterator();
        while (i.hasNext()) {
            UpgradeProcessListener lis = (UpgradeProcessListener)i.next();
            lis.start();
        }
    }

    private void fireEnd(boolean status) {
        Iterator i = listeners.iterator();
        while (i.hasNext()) {
            UpgradeProcessListener lis = (UpgradeProcessListener)i.next();
            lis.done(status);
        }
    }
    
}
