/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */
/*
 * PreUpgradePanel.java
 *
 * Created on April 17, 2001, 5:02 PM
 */

package org.netbeans.core.upgrade;

import java.awt.Font;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.util.ResourceBundle;
import java.util.ArrayList;
import org.openide.WizardDescriptor;
import org.openide.util.HelpCtx;
import org.openide.TopManager;
import org.openide.DialogDescriptor;
import java.awt.Dialog;

/**
 *
 * @author  vs124454
 */
public class PreUpgradePanel extends javax.swing.JPanel 
    implements WizardDescriptor.Panel, UpgradeWizard.ExtendedPanel {

    public static void main(String args[]) {
        // display dialog
        DialogDescriptor descriptor = new DialogDescriptor (
            new PreUpgradePanel(),
            bundle.getString("LBL_UpgradePanel_Name")
        );
        Dialog dialog = TopManager.getDefault().createDialog(descriptor);
        dialog.show();
        dialog.dispose();
    }
        
    /** Creates new form PreUpgradePanel */
    public PreUpgradePanel() {
        initComponents();

        // set Bold font
        Font font = txtBottom.getFont();
        txtBottom.setFont(font.deriveFont(Font.BOLD));
        font = lblSelection.getFont();
        lblSelection.setFont(font.deriveFont(Font.BOLD));
    }

    /** Get the component displayed in this panel.
    * @return the component
    */
    public java.awt.Component getComponent () {
        return this;
    }

    /** Help for this panel.
    * When the panel is active, this is used as the help for the wizard dialog.
    * @return the help or <code>null</code> if no help is supplied
    */
    public HelpCtx getHelp () {
        return null;
    }

    /** Test whether the panel is finished and it is safe to proceed to the next one.
    * If the panel is valid, the "Next" (or "Finish") button will be enabled.
    * @return <code>true</code> if the user has entered satisfactory information
    */
    public boolean isValid () {
        return true;
    }

    /** Provides the wizard panel with the opportunity to update the
     * settings with its current customized state.
     * Rather than updating its settings with every change in the GUI, it should collect them,
     * and then only save them when requested to by this method.
     * Also, the original settings passed to {@link #readSettings} should not be modified (mutated);
     * rather, the (copy) passed in here should be mutated according to the collected changes.
     * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
     * @param settings the object representing a settings of the wizard
     */
    public void storeSettings(Object settings) {
        // no settings used
    }

    /** Provides the wizard panel with the current data--either
     * the default data or already-modified settings, if the user used the previous and/or next buttons.
     * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
     * @param settings the object representing wizard panel state, as originally supplied to {@link WizardDescriptor#WizardDescriptor(WizardDescriptor.Iterator,Object)}
     * @exception IllegalStateException if the the data provided 
     * by the wizard are not valid.
     */
    public void readSettings(Object settings) {
        // no settings used
    }

    /** Remove a listener to changes of the panel's validity.
     * @param l the listener to remove
     */
    public void removeChangeListener(ChangeListener l) {
        changeListeners.remove(l);
    }

    /** Add a listener to changes of the panel's validity.
     * @param l the listener to add
     * @see #isValid
     */
    public void addChangeListener(ChangeListener l) {
        if (!changeListeners.contains(l)) {
            changeListeners.add(l);
        }
    }

    public boolean hasNext() {
        return rbUpgrade.isSelected();
    }
    
    public boolean hasPrevious() {
        return true;
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        txtTop = new javax.swing.JTextArea();
        rbDontUpgrade = new javax.swing.JRadioButton();
        rbUpgrade = new javax.swing.JRadioButton();
        txtBottom = new javax.swing.JTextArea();
        jPanel1 = new javax.swing.JPanel();
        lblDummy = new javax.swing.JLabel();
        lblDummy.setVisible(false);
        lblSelection = new javax.swing.JLabel();
        lblExitWarning = new javax.swing.JLabel();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        setPreferredSize(new java.awt.Dimension(550, 350));
        setName(bundle.getString("LBL_PreUpgradePanel_Name"));
        txtTop.setWrapStyleWord(true);
        txtTop.setLineWrap(true);
        txtTop.setForeground(lblDummy.getForeground());
        txtTop.setText(bundle.getString("LBL_PreUpgradeText_Top"));
        txtTop.setBackground(lblDummy.getBackground());
        txtTop.setDisabledTextColor(lblDummy.getForeground());
        txtTop.setEnabled(false);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.weightx = 1.0;
        add(txtTop, gridBagConstraints1);
        
        rbDontUpgrade.setSelected(true);
        rbDontUpgrade.setText(bundle.getString("LBL_rbDontUpgrade"));
        rbDontUpgrade.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                rbDontUpgradeActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.insets = new java.awt.Insets(12, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(rbDontUpgrade, gridBagConstraints1);
        
        rbUpgrade.setText(bundle.getString("LBL_rbUpgrade"));
        rbUpgrade.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                rbUpgradeActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(rbUpgrade, gridBagConstraints1);
        
        txtBottom.setWrapStyleWord(true);
        txtBottom.setLineWrap(true);
        txtBottom.setForeground(lblDummy.getForeground());
        txtBottom.setText(bundle.getString("LBL_PreUpgradeText_Bottom"));
        txtBottom.setBackground(lblDummy.getBackground());
        txtBottom.setDisabledTextColor(lblDummy.getForeground());
        txtBottom.setEnabled(false);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(12, 0, 0, 0);
        gridBagConstraints1.weightx = 1.0;
        add(txtBottom, gridBagConstraints1);
        
        lblDummy.setText("lblDummy");
        jPanel1.add(lblDummy);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 5;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(jPanel1, gridBagConstraints1);
        
        lblSelection.setText(bundle.getString("LBL_lblSelection"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(12, 0, 0, 0);
        add(lblSelection, gridBagConstraints1);
        
        lblExitWarning.setText(bundle.getString("LBL_lblExitWarning"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 6;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(12, 0, 0, 0);
        add(lblExitWarning, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void rbDontUpgradeActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_rbDontUpgradeActionPerformed
        rbUpgrade.setSelected(false);
        rbDontUpgrade.setSelected(true);
        fireStateChanged();
    }//GEN-LAST:event_rbDontUpgradeActionPerformed

    private void rbUpgradeActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_rbUpgradeActionPerformed
        rbUpgrade.setSelected(true);
        rbDontUpgrade.setSelected(false);
        fireStateChanged();
    }//GEN-LAST:event_rbUpgradeActionPerformed

    /** Notifyes listeners that panel's validation changed */
    private void fireStateChanged() {
        for(int i = 0; i < changeListeners.size(); i++)
            ((ChangeListener)changeListeners.get(i)).stateChanged(
                new ChangeEvent(this));
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextArea txtTop;
    private javax.swing.JRadioButton rbDontUpgrade;
    private javax.swing.JRadioButton rbUpgrade;
    private javax.swing.JTextArea txtBottom;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JLabel lblDummy;
    private javax.swing.JLabel lblSelection;
    private javax.swing.JLabel lblExitWarning;
    // End of variables declaration//GEN-END:variables

    private static final ResourceBundle bundle = org.openide.util.NbBundle.getBundle("org.netbeans.core.upgrade.Bundle");
    private ArrayList changeListeners = new ArrayList(1);
}
