/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/*
 * TransferSettingsPanel.java
 *
 * Created on April 10, 2001, 2:32 PM
 */

package org.netbeans.core.upgrade;

import java.io.File;
import java.util.ResourceBundle;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.Timer;
import java.util.TimerTask;
import java.awt.event.WindowListener;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import javax.swing.SwingUtilities;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import org.openide.TopManager;
import org.openide.NotifyDescriptor;
import org.openide.WizardDescriptor;
import org.openide.filesystems.LocalFileSystem;
import org.openide.util.NbBundle;
import org.openide.util.HelpCtx;

/**
 *
 * @author  vs124454
 */
public final class TransferSettingsPanel extends javax.swing.JPanel 
    implements UpgradeProcessListener, WizardDescriptor.Panel, PropertyChangeListener {

    /** Creates new form TransferSettingsPanel */
    public TransferSettingsPanel() {
        initComponents();
        pbarProgress.setMinimum(0);

        // create and initialize upgrader
        upgrader = new InternalCopy();
        upgrader.addUpgradeProcessListener(this);
    }

    private Object lock = new Object();
    
    public int getStatus() {
        int ret;
        synchronized(lock) {
            ret = this.status;
        }
        return ret;
    }
    
    private void setStatus(int status) {
        synchronized(lock) {
            this.status = status;
        }
    }
    
    // ******* UpgradeProcessListener interface *******

    public void start() {
        setStatus(STATUS_RUNNING);
        log.clear();
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                progress = 0;
                pbarProgress.setMaximum(1000);
                lblStatus.setText(bundle.getString("MSG_Upgrade_Started"));
            }
        });
    }

    public void message(final String msg) {
        log.add(msg);
/*
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                lblStatus.setText(msg);
            }
        });
 */
    }

    public void done(final boolean status) {
        setStatus(status ? STATUS_FINISHED : STATUS_ERRORS);
        
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                StringBuffer sum = new StringBuffer();
                
                pbarProgress.setValue(1000);
                lblStatus.setText(bundle.getString("MSG_Upgrade_Finished"));

                if (status) {
                    sum.append(bundle.getString("MSG_Upgrade_Succeeded"));
                }
                else {
                    Iterator i = log.iterator();
                    while(i.hasNext()) {
                        String s = (String)i.next();
                        sum.append(s);
                        sum.append("\n");
                    }

                    sum.append("\n\n");
                    sum.append(bundle.getString("MSG_Upgrade_Failed"));
                }
                
                txtStatusSummary.setText(sum.toString());

                fireStateChanged();
            }
        });
    }

    public void progress(final int current, final int max) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                progress++;
                pbarProgress.setValue((int)(980.0 - 980.0/progress));
            }
        });
    }

    // ******* WizardDescriptor.Panel interface *******
    
    /** Get the component displayed in this panel.
    * @return the component
    */
    public java.awt.Component getComponent () {
        return this;
    }

    /** Help for this panel.
    * When the panel is active, this is used as the help for the wizard dialog.
    * @return the help or <code>null</code> if no help is supplied
    */
    public HelpCtx getHelp () {
        return null;
    }

    /** Test whether the panel is finished and it is safe to proceed to the next one.
    * If the panel is valid, the "Next" (or "Finish") button will be enabled.
    * @return <code>true</code> if the user has entered satisfactory information
    */
    public boolean isValid () {
        return STATUS_FINISHED == getStatus() || STATUS_ERRORS == getStatus();
    }

    /** Provides the wizard panel with the opportunity to update the
     * settings with its current customized state.
     * Rather than updating its settings with every change in the GUI, it should collect them,
     * and then only save them when requested to by this method.
     * Also, the original settings passed to {@link #readSettings} should not be modified (mutated);
     * rather, the (copy) passed in here should be mutated according to the collected changes.
     * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
     * @param settings the object representing a settings of the wizard
     */
    public void storeSettings(Object settings) {
        ((UpgradeWizardSettings)settings).read(this.settings);
    }

    /** Provides the wizard panel with the current data--either
     * the default data or already-modified settings, if the user used the previous and/or next buttons.
     * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
     * @param settings the object representing wizard panel state, as originally supplied to {@link WizardDescriptor#WizardDescriptor(WizardDescriptor.Iterator,Object)}
     * @exception IllegalStateException if the the data provided 
     * by the wizard are not valid.
     */
    public void readSettings(Object settings) {
        this.settings.read((UpgradeWizardSettings)settings);
    }

    /** Remove a listener to changes of the panel's validity.
     * @param l the listener to remove
     */
    public void removeChangeListener(ChangeListener l) {
        changeListeners.remove(l);
    }

    /** Add a listener to changes of the panel's validity.
     * @param l the listener to add
     * @see #isValid
     */
    public void addChangeListener(ChangeListener l) {
        if (!changeListeners.contains(l)) {
            changeListeners.add(l);
        }
    }

    // ******* WindowListener interface *******
    public void propertyChange(PropertyChangeEvent ev) {
        if (UpgradeWizard.PROP_CONTENT_PANEL_SELECTED.equals(ev.getPropertyName()) &&
            ev.getNewValue() == this) {
            
            tmr = new Timer();
            tmr.schedule(new UpgradeStarter(this), 200);
        }
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jLabel1 = new javax.swing.JLabel();
        lblStatus = new javax.swing.JLabel();
        pbarProgress = new javax.swing.JProgressBar();
        txtStatusSummary = new javax.swing.JTextArea();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        setPreferredSize(new java.awt.Dimension(550, 350));
        setName(bundle.getString("LBL_TransferSettingsPanel_Name"));
        setMinimumSize(new java.awt.Dimension(500, 279));
        jLabel1.setText(bundle.getString("LBL_lblUpgradeStatus"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(jLabel1, gridBagConstraints1);
        
        lblStatus.setText("...");
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 12, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(lblStatus, gridBagConstraints1);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(12, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints1.weightx = 1.0;
        add(pbarProgress, gridBagConstraints1);
        
        txtStatusSummary.setWrapStyleWord(true);
        txtStatusSummary.setLineWrap(true);
        txtStatusSummary.setEditable(false);
        txtStatusSummary.setForeground(lblStatus.getForeground());
        txtStatusSummary.setBackground(lblStatus.getBackground());
        txtStatusSummary.setDisabledTextColor(lblStatus.getForeground());
        txtStatusSummary.setEnabled(false);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(12, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(txtStatusSummary, gridBagConstraints1);
        
    }//GEN-END:initComponents

    /** Notifyes listeners that panel's validation changed */
    private void fireStateChanged() {
        for(int i = 0; i < changeListeners.size(); i++)
            ((ChangeListener)changeListeners.get(i)).stateChanged(
                new ChangeEvent(this));
    }

    private void notify(String message, int type) {
        NotifyDescriptor descr = new NotifyDescriptor.Message(message, type);
        TopManager.getDefault().notify(descr);
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel lblStatus;
    private javax.swing.JProgressBar pbarProgress;
    private javax.swing.JTextArea txtStatusSummary;
    // End of variables declaration//GEN-END:variables

    public static final int STATUS_NONE = 0;
    public static final int STATUS_RUNNING = 1;
    public static final int STATUS_FINISHED = 2;
    public static final int STATUS_ERRORS = 3;
    
    private static final ResourceBundle bundle = org.openide.util.NbBundle.getBundle("org.netbeans.core.upgrade.Bundle");
    private ArrayList changeListeners = new ArrayList(1);
    private LinkedList log = new LinkedList();
    private int progress = 0;
    private InternalCopy upgrader = null;
    private UpgradeWizardSettings settings = new UpgradeWizardSettings();
    private Timer tmr;
    private int status = STATUS_NONE;
    
    private class UpgradeStarter extends TimerTask {
        private TransferSettingsPanel panel;

        public UpgradeStarter(TransferSettingsPanel p) {
            super();
            panel = p;
        }

        public void run() {
            // suspend timer
            tmr.cancel();
            tmr = null;
            
            final LocalFileSystem   sys = new LocalFileSystem();
            final LocalFileSystem   tmp = new LocalFileSystem();

            File nbHomeDir = new File(System.getProperty("netbeans.user", ""));
            File nbSysDir = new File(nbHomeDir, "system");
            File nbBackupDir = new File(nbHomeDir, "system_backup");

            if (!nbSysDir.exists()) {
                String msg = NbBundle.getMessage(
                    UpgradePanel.class, "MSG_Sysdir_not_found", nbSysDir.getAbsolutePath());
                panel.notify(msg, NotifyDescriptor.ERROR_MESSAGE);
                return;
            }

            if (nbBackupDir.exists()) {
                String msg = NbBundle.getMessage(
                    UpgradePanel.class, "MSG_BackupDir_exists", nbBackupDir.getAbsolutePath());
                panel.notify(msg, NotifyDescriptor.ERROR_MESSAGE);
                return;
            }
            else {
                nbBackupDir.mkdirs();
            }

            try {
                sys.setRootDirectory(nbSysDir);
                tmp.setRootDirectory(nbBackupDir);
            }
            catch (Exception e) {
                panel.notify(bundle.getString("MSG_FS_failure"), NotifyDescriptor.ERROR_MESSAGE);
                return;
            }

            // run upgrade
            Thread t = new Thread() {
                public void run() {
                    upgrader.upgrade(settings.getIDEVersion(), settings.getIDELocation(), sys, tmp);
                }
            };

            t.start();
        }
    }
}
