/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/*
 * UpgradePanel.java
 *
 * Created on March 30, 2001, 2:26 PM
 */

package org.netbeans.core.upgrade;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import org.netbeans.core.*;
import org.openide.*;
import org.openide.cookies.ProjectCookie;
import org.openide.util.*;
import org.openide.util.NbBundle;
import org.openide.filesystems.*;


/**
 *
 * @author  vs124454
 */
public class UpgradePanel extends javax.swing.JPanel 
                          implements WizardDescriptor.Panel {

    public static void main(String args[]) {
        // display dialog
        DialogDescriptor descriptor = new DialogDescriptor (
            new UpgradePanel(),
            bundle.getString("LBL_UpgradePanel_Name")
        );
        Dialog dialog = TopManager.getDefault().createDialog(descriptor);
        dialog.show();
        dialog.dispose();
    }

    /** Creates new form UpgradePanel */
    public UpgradePanel() {
        initComponents();
        
        // set Bold font
        Font font = txtLableBottom.getFont();
        txtLableBottom.setFont(font.deriveFont(Font.BOLD));
    }

    /** Get the component displayed in this panel.
    * @return the component
    */
    public java.awt.Component getComponent () {
        return this;
    }

    /** Help for this panel.
    * When the panel is active, this is used as the help for the wizard dialog.
    * @return the help or <code>null</code> if no help is supplied
    */
    public HelpCtx getHelp () {
        return null;
    }

    /** Test whether the panel is finished and it is safe to proceed to the next one.
    * If the panel is valid, the "Next" (or "Finish") button will be enabled.
    * @return <code>true</code> if the user has entered satisfactory information
    */
    public boolean isValid () {
        return null != settings.getIDELocation();
    }

    /** Provides the wizard panel with the opportunity to update the
     * settings with its current customized state.
     * Rather than updating its settings with every change in the GUI, it should collect them,
     * and then only save them when requested to by this method.
     * Also, the original settings passed to {@link #readSettings} should not be modified (mutated);
     * rather, the (copy) passed in here should be mutated according to the collected changes.
     * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
     * @param settings the object representing a settings of the wizard
     */
    public void storeSettings(Object settings) {
        ((UpgradeWizardSettings)settings).read(this.settings);
    }

    /** Provides the wizard panel with the current data--either
     * the default data or already-modified settings, if the user used the previous and/or next buttons.
     * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
     * @param settings the object representing wizard panel state, as originally supplied to {@link WizardDescriptor#WizardDescriptor(WizardDescriptor.Iterator,Object)}
     * @exception IllegalStateException if the the data provided 
     * by the wizard are not valid.
     */
    public void readSettings(Object settings) {
        this.settings.read((UpgradeWizardSettings)settings);
    }

    /** Remove a listener to changes of the panel's validity.
     * @param l the listener to remove
     */
    public void removeChangeListener(ChangeListener l) {
        changeListeners.remove(l);
    }

    /** Add a listener to changes of the panel's validity.
     * @param l the listener to add
     * @see #isValid
     */
    public void addChangeListener(ChangeListener l) {
        if (!changeListeners.contains(l)) {
            changeListeners.add(l);
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        pnlUpgrade = new javax.swing.JPanel();
        lblLocation = new javax.swing.JLabel();
        txtLocation = new javax.swing.JTextField();
        cmdSelectDir = new javax.swing.JButton();
        lblVersion = new javax.swing.JLabel();
        txtVersion = new javax.swing.JTextField();
        txtLabelTop = new javax.swing.JTextArea();
        txtLableBottom = new javax.swing.JTextArea();
        jPanel1 = new javax.swing.JPanel();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        setPreferredSize(new java.awt.Dimension(550, 350));
        setName(bundle.getString("LBL_UpgradePanel_Name"));
        setMinimumSize(new java.awt.Dimension(500, 279));
        setMaximumSize(new java.awt.Dimension(123456, 123456));
        pnlUpgrade.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints2;
        
        lblLocation.setText(bundle.getString("LBL_lblLocation"));
        lblLocation.setHorizontalTextPosition(javax.swing.SwingConstants.LEFT);
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 0;
        gridBagConstraints2.gridy = 0;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        pnlUpgrade.add(lblLocation, gridBagConstraints2);
        
        txtLocation.setDisabledTextColor(java.awt.Color.black);
        txtLocation.setBorder(new javax.swing.border.EtchedBorder());
        txtLocation.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                txtLocationActionPerformed(evt);
            }
        });
        
        txtLocation.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyTyped(java.awt.event.KeyEvent evt) {
                txtLocationKeyTyped(evt);
            }
        });
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 1;
        gridBagConstraints2.gridy = 0;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.insets = new java.awt.Insets(0, 12, 0, 0);
        gridBagConstraints2.weightx = 1.0;
        pnlUpgrade.add(txtLocation, gridBagConstraints2);
        
        cmdSelectDir.setText(bundle.getString("LBL_cmdSelectDir"));
        cmdSelectDir.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cmdSelectDirActionPerformed(evt);
            }
        });
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 2;
        gridBagConstraints2.gridy = 0;
        gridBagConstraints2.insets = new java.awt.Insets(0, 12, 0, 0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.EAST;
        pnlUpgrade.add(cmdSelectDir, gridBagConstraints2);
        
        lblVersion.setText(bundle.getString("LBL_lblVersion"));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 0;
        gridBagConstraints2.gridy = 1;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.insets = new java.awt.Insets(12, 0, 0, 0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        pnlUpgrade.add(lblVersion, gridBagConstraints2);
        
        txtVersion.setEditable(false);
        txtVersion.setDisabledTextColor(java.awt.Color.black);
        txtVersion.setBorder(new javax.swing.border.EtchedBorder());
        txtVersion.setEnabled(false);
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 1;
        gridBagConstraints2.gridy = 1;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.insets = new java.awt.Insets(12, 12, 0, 0);
        gridBagConstraints2.weightx = 1.0;
        pnlUpgrade.add(txtVersion, gridBagConstraints2);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(12, 0, 0, 0);
        gridBagConstraints1.weightx = 1.0;
        add(pnlUpgrade, gridBagConstraints1);
        
        txtLabelTop.setWrapStyleWord(true);
        txtLabelTop.setLineWrap(true);
        txtLabelTop.setEditable(false);
        txtLabelTop.setForeground(lblLocation.getForeground());
        txtLabelTop.setText(bundle.getString("LBL_Upgrade_Description_Top"));
        txtLabelTop.setBackground(pnlUpgrade.getBackground());
        txtLabelTop.setDisabledTextColor(lblLocation.getForeground());
        txtLabelTop.setDoubleBuffered(true);
        txtLabelTop.setEnabled(false);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.weightx = 1.0;
        add(txtLabelTop, gridBagConstraints1);
        
        txtLableBottom.setWrapStyleWord(true);
        txtLableBottom.setLineWrap(true);
        txtLableBottom.setEditable(false);
        txtLableBottom.setForeground(lblLocation.getForeground());
        txtLableBottom.setBackground(pnlUpgrade.getBackground());
        txtLableBottom.setDisabledTextColor(lblLocation.getForeground());
        txtLableBottom.setEnabled(false);
        txtLableBottom.setAutoscrolls(false);
        txtLableBottom.setRequestFocusEnabled(false);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(12, 0, 0, 0);
        gridBagConstraints1.weightx = 1.0;
        add(txtLableBottom, gridBagConstraints1);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(jPanel1, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void txtLocationKeyTyped(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_txtLocationKeyTyped
        final UpgradePanel panel = this;
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                lastLocation = new File(txtLocation.getText());
                panel.checkLocation(lastLocation);
            }
        });
    }//GEN-LAST:event_txtLocationKeyTyped

    private void txtLocationActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_txtLocationActionPerformed
        lastLocation = new File(txtLocation.getText());
        checkLocation(lastLocation);
    }//GEN-LAST:event_txtLocationActionPerformed

private void cmdSelectDirActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cmdSelectDirActionPerformed
    FSChooser chooser = new FSChooser();
    chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
    chooser.setDialogTitle(bundle.getString("LBL_SelectDir_Dialog_Title"));

    if (lastLocation != null) {
        chooser.setCurrentDirectory(lastLocation);
    }

    if (Utilities.showJFileChooser(chooser, this, bundle.getString("LBL_Open_Button")) == JFileChooser.APPROVE_OPTION) {
        lastLocation = chooser.getSelectedFile();
        txtLocation.setText(lastLocation.getAbsolutePath());

        if (!checkLocation(lastLocation)) {
            notify(bundle.getString("MSG_Old_IDE_Location_Invalid"), NotifyDescriptor.ERROR_MESSAGE);
        }
    }
}//GEN-LAST:event_cmdSelectDirActionPerformed

    private boolean checkLocation(File oldIde) {
        int ideVer = CopyUtil.getIdeVersion(oldIde);
        
        if (-1 != ideVer) {
            // update settings
            settings.setIDELocation(oldIde);
            settings.setIDEVersion(CopyUtil.versions[ideVer]);

            // update GUI
            txtVersion.setText(CopyUtil.versions[ideVer]);
        }
        else {
            // reset settings
            settings.setIDELocation(null);
            settings.setIDEVersion(null);

            // update GUI
            txtVersion.setText("");
        }
        
        fireStateChanged();
        return -1 != ideVer;
    }
    
    /** Notifyes listeners that panel's validation changed */
    private void fireStateChanged() {
        for(int i = 0; i < changeListeners.size(); i++)
            ((ChangeListener)changeListeners.get(i)).stateChanged(
                new ChangeEvent(this));
    }

    private void notify(String message, int type) {
        NotifyDescriptor descr = new NotifyDescriptor.Message(message, type);
        TopManager.getDefault().notify(descr);
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel pnlUpgrade;
    private javax.swing.JLabel lblLocation;
    private javax.swing.JTextField txtLocation;
    private javax.swing.JButton cmdSelectDir;
    private javax.swing.JLabel lblVersion;
    private javax.swing.JTextField txtVersion;
    private javax.swing.JTextArea txtLabelTop;
    private javax.swing.JTextArea txtLableBottom;
    private javax.swing.JPanel jPanel1;
    // End of variables declaration//GEN-END:variables

    private File lastLocation = null;
    private static final ResourceBundle bundle = org.openide.util.NbBundle.getBundle("org.netbeans.core.upgrade.Bundle");
    private boolean upgradeStatus = false;
    private ArrayList changeListeners = new ArrayList(1);
    private UpgradeWizardSettings settings = new UpgradeWizardSettings();
}
