/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/*
 * UpgradeWizardSettings.java
 *
 * Created on April 10, 2001, 3:16 PM
 */

package org.netbeans.core.upgrade;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;

/**
 *
 * @author  vs124454
 * @version 
 */
public class UpgradeWizardSettings {
    
    private static final String UPGRADE_PROPERTIES = "upgrade.properties";
    private static final String PROP_VERSION = "VERSION";
    private static final String PROP_IDEDIR = "BACKUP_FOLDER";

    private File ideLocation = null;
    private String ideVersion = null;
    
    /** Creates new UpgradeWizardSettings */
    public UpgradeWizardSettings() {
    }

    public UpgradeWizardSettings(UpgradeWizardSettings settings) {
        read(settings);
    }

    public void read(UpgradeWizardSettings settings) {
        ideLocation = settings.getIDELocation();
        ideVersion = settings.getIDEVersion();
    }
    
    /**
     * @return location of previously installed IDE or null, when this property wasn't yet set
     */
    public File getIDELocation() {
        return ideLocation;
    }
    
    public void setIDELocation(File location) {
        ideLocation = location;
    }
    
    /**
     * @return string identification of IDE version or null when this property wasn't yet set
     */
    public String getIDEVersion() {
        return ideVersion;
    }
    
    public void setIDEVersion(String version) {
        ideVersion = version;
    }

    public static UpgradeWizardSettings getInstance() {
        File upgProps = getUpgradeProperties();
        UpgradeWizardSettings sett = new UpgradeWizardSettings();
        
        if (null == upgProps) {
            return sett;
        }
        
        FileInputStream fis = null;
        
        try {
            Properties prop = new Properties();
            fis = new FileInputStream(upgProps);
            prop.load(fis);
            sett.setIDELocation(new File(prop.getProperty(PROP_IDEDIR, "")));
            sett.setIDEVersion(prop.getProperty(PROP_VERSION, ""));
        } catch (IOException e) {
            // reset settings
            sett.setIDELocation(null);
            sett.setIDEVersion(null);
        } finally {
            if (null != fis) {
                try {
                    fis.close();
                } catch (IOException e) {};
            }
        }
        
        return sett;
    }
    
    /**
     * @return <netbeans.home>/upgrade.properties file or null if it doesn't exist
     */
    private static File getUpgradeProperties() {
        String nbhome;
        File upgProp;
        
        if (null == (nbhome = System.getProperty("netbeans.home")))
            return null;
        
        upgProp = new File(nbhome, UPGRADE_PROPERTIES);
        if (!upgProp.exists())
            return null;
        
        return upgProp;
    }
}
