/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.core.windows.frames;

import java.awt.LayoutManager;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Rectangle;

import javax.swing.JRootPane;

/**
 * ChildFrameTypeLayoutManager.java
 * 
 * This class is a hack for repostioning the title bar and JRootPane of a JInternalFrame
 * for the MDI implementation.  The problem is that the look and feel of Metal is not
 * well done in terms of reazl estate management, and hence the need for this class.  In
 * future it would be better for a completely new class ot be written to implement a
 * better internal frame.
 *
 * Created on October 20, 2000, 9:59 AM
 *
 * @author  Hippy
 * @version 
 *
 */
public class ChildFrameTypeLayoutManager extends Object implements LayoutManager {
    
    private static final int TITLE_BAR_SIZE = 15;

    /** 
     * Creates new ChildFrameTypeLayoutManager.
     */
    public ChildFrameTypeLayoutManager() {
    }

    public java.awt.Dimension preferredLayoutSize(final Container cont) {
        return cont.getSize();
    }
    
    public void removeLayoutComponent(final Component comp) {
    }
    public java.awt.Dimension minimumLayoutSize(final Container cont) {
        return cont.getMinimumSize();
    }
    
    public void addLayoutComponent(String str,final Component comp) {
    }
    
    /** 
     * This method lays out the internal frame in such a way that the title bar size is 
     * smaller and <code>JRootPane</code> is resized to enable more real estate for what
     * is to be displayed.  The size of the title bar is set so as not to interfere with
     * the default size of the title bar contol images or title text,
     */
    public void layoutContainer(final Container cont) {
        Component[] comps = cont.getComponents();
        JRootPane rootPane = (JRootPane) ((comps[0] instanceof JRootPane) ?
            comps[0] : comps[1]);
        Component comp = (Component) (!(comps[0] instanceof JRootPane) ?
            comps[0] : comps[1]);
        
        // get the cont size - insets
        Rectangle contRect = cont.getBounds();
        Insets insets = cont.getInsets();
        contRect.x = insets.left;
        contRect.width -= insets.left + insets.right;
        contRect.y = insets.top;
        contRect.height -= insets.top + insets.bottom;
        
        Rectangle compRect = comp.getBounds();
        compRect.width = contRect.width;
        compRect.height = TITLE_BAR_SIZE;
        compRect.setLocation(contRect.getLocation());
        comp.setBounds(compRect);
        
        Rectangle rootRect = rootPane.getBounds();
        rootRect.width = contRect.width;
        rootRect.height = contRect.height - TITLE_BAR_SIZE;
        rootRect.setLocation(contRect.x, contRect.y + TITLE_BAR_SIZE);
        rootPane.setBounds(rootRect);
    }
}