/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/ * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.core.windows.frames;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.AWTEvent;
import java.awt.event.MouseEvent;
import java.awt.event.AWTEventListener;
import java.awt.event.MouseMotionListener;
import java.beans.PropertyVetoException;
import java.util.EventListener;
import javax.swing.border.Border;
import javax.swing.border.BevelBorder;
import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.JInternalFrame;
import javax.swing.WindowConstants;
import javax.swing.event.EventListenerList;
import javax.swing.event.InternalFrameEvent;

import javax.swing.plaf.InternalFrameUI;
import javax.swing.plaf.basic.BasicInternalFrameUI;

import org.netbeans.core.windows.util.WindowUtils;

/**
 * This is the FrameType implementation of a desktop frame, one that resides
 * at the side of a desktop.
 *
 * @author  Hippy, Dafe Simonek
 * 
 * @see FrameType
 * 
 */
public class DesktopFrameTypeImpl extends InternalFrameTypeImpl
                                  implements WindowUtils.AWTEventProcessor {
    
    /** 
     * Creates new DesktopFrameImpl. 
     */
    public DesktopFrameTypeImpl() {
        this("");
    }

    /** 
     * Creates new DesktopFrameImpl while setting the title. 
     *
     * @param title    The title of the DektopFrame.
     */
    public DesktopFrameTypeImpl(String title) {
        super(title, false, true, false, false);
        Border border = BorderFactory.createEtchedBorder();
        setBorder(border);
        // remove the icon by default
        setFrameIcon(null);
    }

    /** 
     * Sets the state of the frame to only <code>FrameType#NORMAL</code>, and throws a 
     * PropertyVetoException exception otherwise since desktop frames can only be in one
     * state.
     *
     * @param state   The state to set the desktop frame to.
     * 
     * @throws PropertyVetoException If a state other than <code>FrameType#NORMAL</code>
     *         is tried.
     * 
     */
    public void setState(int state) throws PropertyVetoException {
        if (FrameType.NORMAL == state) {
            super.setState(state);
        }
        else {
            throw new IllegalStateException("Illegal state");
        }
    }
      
    /** 
     * Sets whether the desktop frame is selected or not.
     *
     * @param selected   True if selected.
     * 
     */
    public void setSelected(boolean selected) throws PropertyVetoException {
        super.setSelected(selected);
    }
    
    /** 
     * Get the type of frame.
     * 
     * @return Returns <code>FrameType#DESKTOP</code>.
     * 
     */
    public int getType() {
        return FrameType.DESKTOP;
    }
        
    /** 
     * Get's the frame's desktop icon. 
     * 
     * @return Returns null, since desktop frames cannot be minimized.
     * 
     */
    public Component getIcon() {
        return null;
    }

    /** When UI changes, install custom glass pane as a proxy for
     * trapping events to prevent this frame from being dragged
     */
    public void setUI(InternalFrameUI ui) {
        super.setUI(ui);
        // instals a glass pane over the title bar and traps events.
        // However, works only for UIs derived from BasicInternalFrameUI
        if (!(ui instanceof BasicInternalFrameUI)) {
            return;
        }
        BasicInternalFrameUI bui= (BasicInternalFrameUI)ui;
        boolean checkingEnabled = isRootPaneCheckingEnabled();
        try {
            setRootPaneCheckingEnabled(false);
            bui.setNorthPane(
                WindowUtils.createComponentProxy(bui.getNorthPane(), this)
            );
        } finally {
            setRootPaneCheckingEnabled(checkingEnabled);
        }
    }
    
    /** handles events of title bar (north pane of this frame)
     * @see #setUI
     */
    public boolean handleEvent (AWTEvent evt) {
        // mark drag events as processed, this will prevent this
        // frame from being dragged
        return evt.getID() == MouseEvent.MOUSE_DRAGGED;
    }

    
}
