/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Compuware Corporation and Sun Microsystems, Inc. Portions
 * Copyright 2000 Compuware corporation and Sun Microsystems, Inc.
 * All Rights Reserved.
 */

package org.netbeans.core.windows.frames;

import java.awt.AWTEvent;
import java.awt.event.WindowEvent;

/** 
 * Event definition for events produced by FrameType implementations.
 * Note that all event types are simply copied from good ol' Window events
 * since frames pretty much do the same thing.
 * 
 * @author  Evan 'Hippy' Slatis, Dafe Simonek
 */
public class FrameTypeEvent extends AWTEvent { 
    
    /**
     * Not a bit-head, chose increment at random, so someone fix this if they're smarter than
     * I am.  Mask for frameTypeEvent.
     */
    public static final int FRAMETYPE_EVENT = AWTEvent.RESERVED_ID_MAX + 1;
    
    /**
     * The frame activated event type.
     */
    public static final int FRAMETYPE_ACTIVATED = FRAMETYPE_EVENT | WindowEvent.WINDOW_ACTIVATED;
    
    /**
     * The frame closed event.
     */
    public static final int FRAMETYPE_CLOSED = FRAMETYPE_EVENT | WindowEvent.WINDOW_CLOSED;
    
    /**
     * The "frame is closing" event.
     */
    public static final int FRAMETYPE_CLOSING = FRAMETYPE_EVENT | WindowEvent.WINDOW_CLOSING;
 
    /**
     * The frame deactivated event type.
     */
    public static final int FRAMETYPE_DEACTIVATED = FRAMETYPE_EVENT | WindowEvent.WINDOW_DEACTIVATED;
    
    /**
     * The frame deiconified event type.
     */
    public static final int FRAMETYPE_DEICONIFIED = FRAMETYPE_EVENT | WindowEvent.WINDOW_DEICONIFIED;

    /**
     * The first number in the range of ids used for frame events.
     */ 
    public static final int FRAMETYPE_FIRST = FRAMETYPE_EVENT | WindowEvent.WINDOW_FIRST;
    
    /**
     * The frame iconified event.
     */
    public static final int FRAMETYPE_ICONIFIED = FRAMETYPE_EVENT | WindowEvent.WINDOW_ICONIFIED;
    
    /**
     * The last number in the range of ids used for frame events. 
     */
    public static final int FRAMETYPE_LAST = FRAMETYPE_EVENT | WindowEvent.WINDOW_LAST;
    
    /**
     * The frame opened event.
     */
    public static final int FRAMETYPE_OPENED = FRAMETYPE_EVENT | WindowEvent.WINDOW_OPENED;
    
    /**
     * The frame brought back from being maximized.
     */
    public static final int FRAMETYPE_NORMALIZED = FRAMETYPE_EVENT | (WindowEvent.WINDOW_LAST + 1);
    
    /**
     * The frame maximized event. 
     */
    public static final int FRAMETYPE_MAXIMIZED = FRAMETYPE_EVENT | (WindowEvent.WINDOW_LAST + 2);

    /**
     * Creates new FrameEvent, default constructor.
     * 
     * @param source  The source of the FrameEvent.
     * @param id     The event mask, as specified above.
     */
    public FrameTypeEvent(FrameType source, int id) {
        super(source, id);
    }
    
    /**
     * Returns the frame that generated this event.
     * 
     * @return Returns frame source of this event.
     * 
     */
    public FrameType getFrame () {
        return (FrameType)getSource();
    }

}
