/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.core.windows.frames;

import java.awt.Image;
import java.awt.Toolkit;
import java.awt.Container;
import java.util.ResourceBundle;
import java.beans.PropertyChangeListener;

import javax.swing.SwingUtilities;

import org.openide.util.NbBundle;
import org.openide.windows.TopComponent;
import org.openide.util.WeakListener;

import org.netbeans.core.windows.ModeImpl;


/** Class which responsibility is to provide support for common operations
 * happening in various frame type implementations.
 *
 * @author  Dafe Simonek
 */
final class FrameTypeSupport extends java.lang.Object {

    /** default image icon for multi tabbed frame */
    private static Image defaultIcon;
    
    /** Frame type we provide support for */
    FrameType ft;
    
    /** Creates new FrameTypeSupport */
    public FrameTypeSupport (FrameType ft) {
        this.ft = ft;
        NbFocusManager.install();
    }

    public void addNotify () {
        // set preferred size if dimensions are not defined yet
        Container container = ft.getRootPaneContainer();
        if ((container.getWidth() == 0) || (container.getHeight() == 0)) {
            container.setSize(container.getPreferredSize());
        }
        // if nothing has focus in the frame, then keyboard events are not
        // routed properly, keyboard shortcuts won't work.  This is actually
        // an AWT problem, but we must work around it, just give anything the
        // focus
        ft.getRootPane().requestFocus();
    }

    
}
