/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.core.windows.frames;

import javax.swing.event.InternalFrameListener;

/** Implementation of the window and component listener for SDI frame types.
 * It simply serves as an adapter, it adapts WindowListener interface to the
 * FrameTypeListener interface. Class takes window events and converts them 
 * to FrameEvents, which are then passed to asociated FrameTypeListener.
 *
 * @author Dafe Simonek
*/
final class InternalFrameListenerAdapter implements InternalFrameListener {

    /** Asociation with event source */
    FrameType source;
    /** Asociation with frame type listener */
    FrameTypeListener listener;

    /** Creates new MultiTabListenerImpl */
    public InternalFrameListenerAdapter(FrameType source,FrameTypeListener listener) {
        this.source = source;
        this.listener = listener;
    }

    public void internalFrameDeiconified(final javax.swing.event.InternalFrameEvent p1) {
        listener.frameDeiconified(new FrameTypeEvent(source, FrameTypeEvent.FRAMETYPE_DEICONIFIED));
    }
    
    public void internalFrameOpened(final javax.swing.event.InternalFrameEvent p1) {
        listener.frameOpened(new FrameTypeEvent(source, FrameTypeEvent.FRAMETYPE_OPENED));
    }
    
    public void internalFrameIconified(final javax.swing.event.InternalFrameEvent p1) {
        listener.frameIconified(new FrameTypeEvent(source, FrameTypeEvent.FRAMETYPE_ICONIFIED));
    }
    
    public void internalFrameClosing(final javax.swing.event.InternalFrameEvent p1) {
        listener.frameClosing(new FrameTypeEvent(source, FrameTypeEvent.FRAMETYPE_CLOSING));
    }
    
    public void internalFrameActivated(final javax.swing.event.InternalFrameEvent p1) {
        listener.frameActivated(new FrameTypeEvent(source, FrameTypeEvent.FRAMETYPE_ACTIVATED));
    }
    
    public void internalFrameDeactivated(final javax.swing.event.InternalFrameEvent p1) {
        listener.frameDeactivated(new FrameTypeEvent(source, FrameTypeEvent.FRAMETYPE_DEACTIVATED));
    }
    
    public void internalFrameClosed(final javax.swing.event.InternalFrameEvent p1) {
        listener.frameClosed(new FrameTypeEvent(source, FrameTypeEvent.FRAMETYPE_CLOSED));
    }
    
    public void internalFrameMaximized() {
        listener.frameMaximized(new FrameTypeEvent(source, FrameTypeEvent.FRAMETYPE_MAXIMIZED));
    }
    
    public void internalFrameNormalized() {
        listener.frameNormalized(new FrameTypeEvent(source, FrameTypeEvent.FRAMETYPE_NORMALIZED));
    }
}

/*
* Log
*  3    Gandalf   1.2         12/17/99 David Simonek   #1913, #2970
*  2    Gandalf   1.1         10/22/99 Ian Formanek    NO SEMANTIC CHANGE - Sun 
*       Microsystems Copyright in File Comment
*  1    Gandalf   1.0         7/11/99  David Simonek   
* $
*/