/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.core.windows.frames;

import javax.swing.RepaintManager;

/** RepaintManager allows switch off registering of dirty regions
*/
public final class PinRepaintRM extends RepaintManager {

    // fields
    /** state support */
    private boolean registeringAllowed;
    /** old manager */
    private RepaintManager replaced;

    /** Creates new PinRepaintRM
     *
     * @param replaced old repaint manager
     */
    public PinRepaintRM(RepaintManager replaced) {
        super();
        registeringAllowed = true;
        this.replaced = replaced;
    }

    /** Sets registeringAllowed flag
    */
    public void setRegisteringAllowed(boolean x) {
        if (java.awt.EventQueue.isDispatchThread()) {
            registeringAllowed = x;
        } else {
            // [TODO] throw new IllegalThreadStateException();
        }
    }

    /** @return registeringAllowed flag
    */
    public boolean getRegisteringAllowed() {
        return registeringAllowed;
    }

    /**
     * Add a component in the list of components that should be refreshed.
     * If <i>c</i> already has a dirty region, the rectangle <i>(x,y,w,h)</i> 
     * will be unioned with the region that should be redrawn. 
     * 
     * @see JComponent#repaint
     */
    public void addDirtyRegion(javax.swing.JComponent c, int x, int y, int w, int h) {
        if (registeringAllowed) {
            super.addDirtyRegion(c, x, y, w, h);
        }
    }

    /** Calls super and if invoked for the first time also
     * replaced->paintDirtyRegions() 
     */
    public void paintDirtyRegions() {
        if (replaced != null) {
            replaced.paintDirtyRegions();
            replaced = null;
        }
        super.paintDirtyRegions();
    }
}

/*
* Log
*  3    Jaga      1.2         4/12/00  Ales Novak      IllegalArgumentException 
*       commented out
*  2    Jaga      1.1         3/30/00  Ales Novak      custome RepaintManager
*  1    Jaga      1.0         3/28/00  Ales Novak      
* $
*/