/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.core.windows.frames;

import java.awt.*;
import java.util.*;
import java.awt.event.*;
import java.io.*;

import javax.swing.ImageIcon;
import javax.swing.JTabbedPane;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.plaf.TabbedPaneUI;
import javax.swing.plaf.basic.BasicTabbedPaneUI;

import org.openide.util.actions.SystemAction;
import org.openide.actions.NextTabAction;
import org.openide.actions.PreviousTabAction;
import org.openide.windows.TopComponent;
import org.openide.awt.MouseUtils;

import org.netbeans.core.ui.ArrowsTabbedPaneUI;
import org.netbeans.core.ui.PopupTabbedPaneUI;
import org.netbeans.core.windows.WindowManagerImpl;
import org.netbeans.core.windows.TabbedContainerUIManager;

/** Class that acts as serialization manager for split container. 
 * Don't rename this class, in spite of the fact that word "Splitted" is 
 * wrong. Renaming will break backward compatibility */
final class SplittedContainerImpl$SerializationReplacer implements Serializable {
    /** Unique ID */
    static final long serialVersionUID = 1234569191035648221L;
    /** Asociation with container which data needs to be serialized */
    private SplitContainerImpl source;

    private static final ObjectStreamField[] serialPersistentFields = {
        new ObjectStreamField(SplitContainerImpl.DEFAULT_CONTAINER, DefaultContainerImpl.class),
        new ObjectStreamField(SplitContainerImpl.CONSTRAINTS_LIST, ArrayList.class),
        new ObjectStreamField(SplitContainerImpl.TABBED_CONTAINERS_SIZE, ArrayList.class),
        new ObjectStreamField(SplitContainerImpl.INDEX_OF_SELECTED_TAB, ArrayList.class)
    };

    SplittedContainerImpl$SerializationReplacer (SplitContainerImpl source) {
        this.source = source;
    }

    /** Serialization of properties of split container - default
     * container plus array of side constraints. */
    private void writeObject (ObjectOutputStream oos)
    throws IOException {
        source.writeData(oos);
    }

    /** Deserialization of the workspace */
    private void readObject (ObjectInputStream ois)
    throws IOException, ClassNotFoundException {
        source = new SplitContainerImpl();
        source.readData(ois);
    }

    /** Resolves deserialized SerializationReplacer to the
     * SplitContainerImpl instance. */
    private synchronized Object readResolve ()
    throws ObjectStreamException {
        return source;
    }

}
