/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.core.windows.frames;

import java.awt.event.*;

/** Implementation of the window and component listener for SDI frame types.
 * It simply serves as an adapter, it adapts WindowListener interface to the
 * FrameTypeListener interface. Class takes window events and converts them 
 * to FrameEvents, which are then passed to asociated FrameTypeListener.
 *
 * @author Dafe Simonek
*/
final class WindowFrameListenerAdapter implements WindowListener {

    /** Asociation with event source */
    FrameType source;
    /** Asociation with frame type listener */
    FrameTypeListener listener;

    /** Creates new MultiTabListenerImpl */
    public WindowFrameListenerAdapter(FrameType source,FrameTypeListener listener) {
        this.source = source;
        this.listener = listener;
    }

    public void windowDeactivated (WindowEvent we) {
        listener.frameDeactivated(new FrameTypeEvent(source, FrameTypeEvent.FRAMETYPE_DEACTIVATED));
    }

    public void windowClosed (WindowEvent we) {
        listener.frameClosed(new FrameTypeEvent(source, FrameTypeEvent.FRAMETYPE_CLOSED));
    }

    public void windowDeiconified (WindowEvent we) {
        listener.frameDeiconified(new FrameTypeEvent(source, FrameTypeEvent.FRAMETYPE_DEICONIFIED));
    }

    public void windowOpened(WindowEvent we) {
        listener.frameOpened(new FrameTypeEvent(source, FrameTypeEvent.FRAMETYPE_OPENED));
    }

    public void windowIconified(WindowEvent we) {
        listener.frameIconified(new FrameTypeEvent(source, FrameTypeEvent.FRAMETYPE_ICONIFIED));
    }

    public void windowClosing(WindowEvent we) {
        listener.frameClosing(new FrameTypeEvent(source, FrameTypeEvent.FRAMETYPE_CLOSING));
    }

    public void windowActivated(WindowEvent we) {
        listener.frameActivated(new FrameTypeEvent(source, FrameTypeEvent.FRAMETYPE_ACTIVATED));
    }

}

/*
* Log
*  3    Gandalf   1.2         12/17/99 David Simonek   #1913, #2970
*  2    Gandalf   1.1         10/22/99 Ian Formanek    NO SEMANTIC CHANGE - Sun 
*       Microsystems Copyright in File Comment
*  1    Gandalf   1.0         7/11/99  David Simonek   
* $
*/