/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.core.windows.nodes;

import java.beans.PropertyEditorSupport;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.netbeans.core.windows.TabbedContainerUIManager;
import org.netbeans.core.windows.WindowManagerImpl;

/** This is a property editor for UI modes (BasicTabbedPaneUI, ArrowsTabbedPaneUI,
 *  PopupTabbedPaneUI) of containers include JTabbedPane
 *  (MultiTabbedContainerImpl, SplitContainerImpl).
 *  Data class is TabbedContainerUIManager.
 *
 *  @author Rostislav Levy
 */
public class TabbedContainerUIPropertyEditor extends PropertyEditorSupport {

    /** List of TabbedPaneUI modes (mode value, mode name)
     */
    private Map modes2Names;
    /** List of TabbedPaneUI modes (mode name, mode value)
     */
    private Map names2Modes;

    /** Default constructor
     */
    public TabbedContainerUIPropertyEditor() {
        modes2Names = null;
        names2Modes = null;
    }

    /** Reads the list of mode names
     *  return - the list of mode names
     */
    public String[] getTags () {
        return (String[])names2Modes().keySet().toArray(new String[names2Modes.size()]);
    }

    /** Reads name of actual mode
     *  return - name of actual mode
     */
    public String getAsText () {
        return (String)modes2Names().get(getValue());
    }

    /** Set actual mode by name
     *  text - name of setting mode
     */
    public void setAsText (String text) throws IllegalArgumentException {
        Object value = names2Modes().get(text);
        if (value != null) {
            setValue(value);
        } else {
            throw new IllegalArgumentException();
        }
    }

    /** Reads the list of TabbedPaneUI modes
     */
    private Map names2Modes () {
        if (names2Modes == null) {
            names2Modes = WindowManagerImpl.getDefault().tabbedContainerUIManager().getModeNames();
        }
        return names2Modes;
    }

    /** Reads the list of TabbedPaneUI modes
     */
    private Map modes2Names () {
        if (modes2Names == null) {
            Map names2Modes = names2Modes();
            modes2Names = new HashMap(10);
            Iterator iter = names2Modes.entrySet().iterator();
            Map.Entry curEntry = null;
            while (iter.hasNext()) {
                curEntry = (Map.Entry)iter.next();
                modes2Names.put(curEntry.getValue(), curEntry.getKey());
            }
        }
        return modes2Names;
    }
}
