/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.core.windows.toolbars;

import java.beans.PropertyEditor;
import java.beans.PropertyEditorSupport;

import org.openide.nodes.PropertySupport;
import org.openide.util.NbBundle;
import org.openide.awt.ToolbarPool;

import org.netbeans.core.windows.nodes.WorkspaceContext;

/** Property suport for workspaces' toolbar configuration.
 * @author Libor Kramolis
 */
public class ToolbarConfigurationPropertyEditor extends PropertySupport.ReadWrite {

    /** ResourceBundle to get localized strings. */
    private static final java.util.ResourceBundle bundle = NbBundle.getBundle (ToolbarConfigurationPropertyEditor.class);

    /** Default ToolbarPool instance. */
    private static ToolbarPool toolbarPool = ToolbarPool.getDefault();

    /** Specific WorkspaceContext. */
    private WorkspaceContext workspaceContext;

    /** Create new PropertyEditor for ToolbarConfiguration for specific WorkspaceContext.
     * @param wc Specific WorkspaceContext whose this PropertyEditor is.
     */
    public ToolbarConfigurationPropertyEditor (WorkspaceContext wc) {
        super (bundle.getString ("PROP_desk_config"), // NOI18N
               String.class,
               bundle.getString ("PROP_desk_config"), // NOI18N
               bundle.getString ("HINT_desk_config")); // NOI18N
        workspaceContext = wc;
    }

    /** Get the value.
     * @return the value of the property
     */
    public Object getValue () {
        int count;
        String value = workspaceContext.getToolbarConfigName();
        String [] tags = new ToolbarConfigurationPropertyEditorSupport().getTags();
        count = tags.length;
        for(int i = 0; i < count; i++)
            if(value.equalsIgnoreCase(tags[i]))
                return value;
        if(count == 0)
            return value;
        return tags[0];
    }

    /** Set the value.
     * @param val the new value of the property
     */
    public void setValue (Object n) {
        workspaceContext.setToolbarConfigName (n.toString());
    }

    /** Get a property editor for this property.
     * @return Get a property editor for this property.
     */
    public PropertyEditor getPropertyEditor () {
        return new ToolbarConfigurationPropertyEditorSupport();
    }


    /** This is a property editor for ToolbarConfiguration. */
    class ToolbarConfigurationPropertyEditorSupport extends PropertyEditorSupport {
        /** @return names of the toolbar configurations */
        public String[] getTags() {
            return toolbarPool.getConfigurations();
        }
    }
}

