/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger;

import java.beans.*;
import java.io.*;
import java.text.MessageFormat;
import java.util.*;
import javax.swing.JComponent;

import org.openide.loaders.DataObject;
import org.openide.text.Line;
import org.openide.TopManager;
import org.openide.NotifyDescriptor;
import org.openide.cookies.LineCookie;
import org.openide.debugger.*;
import org.openide.src.ConstructorElement;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.Repository;
import org.openide.filesystems.FileSystem;
import org.openide.filesystems.EnvironmentNotSupportedException;
import org.openide.windows.InputOutput;
import org.openide.windows.OutputWriter;
import org.openide.execution.NbProcessDescriptor;
import org.openide.util.NbBundle;
import org.openide.util.actions.ActionPerformer;
import org.openide.util.actions.CallbackSystemAction;
import org.openide.util.actions.SystemAction;
import org.openide.src.ClassElement;

import org.netbeans.modules.debugger.*;
import org.netbeans.modules.debugger.support.util.*;
import org.netbeans.modules.debugger.support.actions.*;

import javax.swing.SwingUtilities;

/**
* Main Corona debugger class
*
* @author   Jan Jancura
* @version  0.47, May 26, 1998
*/
public abstract class AbstractDebugger extends Debugger implements java.io.Serializable {
    

    /** Last action property constants for action Continue. */
    public static final int                   ACTION_GO = 5;
    /** Last action property constants for action StartDebugger. */
    public static final int                   ACTION_START = 6;
    
    /** Property name constant for last action property. */
    public static final String                PROP_LAST_ACTION = "lastAction"; // NOI18N

    
    /**
    * Suspends debugging.
    */
//    public void setSuspended (boolean suspended) {
//    }
    
    /**
    * Getter for last action property.
    *
    * @return one of XXX_ACTION constants
    */
    public abstract int getLastAction ();

    
    // DebuggerListener support .......................................................

    /**
    * This listener notificates about cahnges of breakpoints, watches and threads.
    *
    * @param l listener object.
    */
    public abstract void addDebuggerListener (DebuggerListener l);

    /**
    * Removes debugger listener.
    *
    * @param l listener object.
    */
    public abstract void removeDebuggerListener (DebuggerListener l);
}

