/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger;

import org.openide.debugger.*;

/**
*
* @author   Jan Jancura
*/
public abstract class AbstractDebuggerInfo extends DebuggerInfo {

    /**
    * Construct a new <code>DebuggerInfo</code> with the class to run and its parameters specified.
    * Sets class to stop in to be the class to run.
    *
    * @param className name of debugged class
    * @param argv command-line arguments used for debugging this class; may be empty but not <code>null</code>
    */
    public AbstractDebuggerInfo (
        String className,
        String[] argv
    ) {
        super (
            className,
            argv,
            className
        );
    }
    

    /**
    * Construct a new <code>DebuggerInfo</code> with the class to run, parameters, and a class to stop at.
    *
    * @param className name of debugged class
    * @param argv command-line arguments used for debugging this class; may be empty but not <code>null</code>
    * @param stopClassName name of class to stop in (may be <code>null</code>)
    */
    public AbstractDebuggerInfo (
        String className,
        String[] argv,
        String stopClassName
    ) {
        super (className, argv, stopClassName);
    }
    
    
    public abstract String getProcessName ();
    public abstract String getLocationName ();
}

