/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger;

import java.beans.*;
import java.util.*;

import javax.swing.SwingUtilities;

import org.openide.text.Line;
import org.openide.nodes.Node;
import org.openide.debugger.Debugger;
import org.openide.debugger.DebuggerException;

import org.netbeans.modules.debugger.support.util.Utils;

public abstract class AbstractThread {

    public static final String  PROP_NAME = "name";
    public static final String  PROP_CURRENT = "current";
    
    
    /**
    * Getter for the name of thread property.
    *
    * @throw DebuggerException if some problem occurs.
    * @return name of thread.
    */
    public abstract String getName ();
    
    /**
    * Returns true if this thread is current.
    */
    public abstract boolean isCurrent ();

    /**
    * Sets this thread current thread.
    * Calls debugger.setCurrentThread (this) and
    * marks current line in editor or show cursor on some stack frame
    * above, if current class is sourceless.
    *
    * @return true if the current thread switching is successfull.
    */
    public abstract void setCurrent (boolean current);

    /**
    * Adds property change listener for this thread. All property changes are fired
    * together in one PropertyChangeEvent with name of property == null.
    *
    * @param l listener to be added.
    */
    public abstract void addPropertyChangeListener (PropertyChangeListener l);

    /**
    * Removes property change listener from this thread.
    *
    * @param l listener to be removed.
    */
    public abstract void removePropertyChangeListener (PropertyChangeListener l);
}
