/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger;

import java.beans.PropertyChangeListener;

import org.openide.debugger.DebuggerException;


/**
*
* all properties fired at once.
* @author   Jan Jancura
*/
public interface AbstractVariable extends java.io.Serializable, Cloneable {


    /**
    * Returns the name of this variable.
    *
    * @return the name of this variable.
    */
    public String getVariableName ();

    /**
    * Getter for textual representation of the value. It converts
    * the value to a string representation. So if the watch represents
    * null reference, the returned string will be for example "null".
    * That is why null can be returned when the watch is not valid
    *
    * @return the value of this watch or null if the watch is not in the scope
    */
    public String getAsText ();

    /**
    * Setter that allows to change value of the watched variable.
    *
    * @param value text representation of the value
    * @exception DebuggerException if the value cannot be changed or the
    *    string does not represent valid value
    */
    public void setAsText (String value) throws DebuggerException;

    /**
    * Returns string representation of type of this variable.
    *
    * @return string representation of type of this variable.
    */
    public String getType ();

    /**
    * Adds listener on the property changing.
    */
    public void addPropertyChangeListener (PropertyChangeListener listener);

    /**
    * Removes listener on the property changing.
    */
    public void removePropertyChangeListener (PropertyChangeListener listener);
}
