/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger;

import java.beans.*;
import org.openide.debugger.Debugger;


/**
 * Debugger Core implementation should support installation of Debugger 
 * Implementation plugins. When some debugger session is started (instance
 * of Debugger is created), it delegates debugger functionality on it.
 *
 * @author Jan Jancura
 */
public interface CoreDebugger {

    public static final String PROP_CURRENT_DEBUGGER = "currentDebugger";

    public static final String PROP_DEBUGGERS = "debuggers";

    /**
     * Returns currently running debugger or null if debugger is not running.
     */
    public abstract Debugger getCurrentDebugger ();

    /**
     * Returns currently running debugger or null if debugger is not running.
     */
    public abstract Debugger[] getDebuggers ();
    
    /**
    * Adds listener on the property changing.
    */
    public void addPropertyChangeListener (PropertyChangeListener listener);

    /**
    * Removes listener on the property changing.
    */
    public void removePropertyChangeListener (PropertyChangeListener listener);
}

