/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger;

import org.openide.nodes.Node;
import java.util.*;
import java.lang.reflect.Constructor;

public class NodesRegistry {
    
    private static LinkedList      reg = new LinkedList ();
    private static LinkedList      nodes = new LinkedList ();
    private static HashMap         cache = new HashMap ();
    
    public static synchronized void register (Class document, Class node) {
        reg.add (new String [] {document.getName ()});
        nodes.add (node);
    }
    
    public static synchronized void register (Class[] document, Class node) {
        String[] s = new String [document.length];
        int i, k = document.length;
        for (i = 0; i < k; i++)
            s [i] = document [i].getName ();
        reg.add (s);
        nodes.add (node);
    }
    
    public static synchronized void register (String documentName, Class node) {
        reg.add (new String [] {documentName});
        nodes.add (node);
    }
    
    public static Node getNode (Object document) {
        return getNode (document, new Object [] {document});
    }
    
    public static Node getNode (Object document, Object[] params) {
        // 1. find node class (c) for given doc
        Class documentClass = document.getClass ();
        String cName = (String) cache.get (documentClass.getName ());
        Class c = null;
        if (cName != null) 
            try {
                c = Class.forName (cName);
            } catch (ClassNotFoundException ex) {
                ex.printStackTrace ();
                return null;
            }
        else {
            c = getNodeClass (documentClass);
            if (c == null) return null;
            cache.put (documentClass.getName (), c.getName ());
        }
        
        // create instance of c 
        return getNode (c, params);
    }
    
    public static Node getNode (String documentName, Object[] params) {
        // 1. find node class (c) for given doc
        String cName = (String) cache.get (documentName);
        Class c = null;
        if (cName != null) 
            try {
                c = Class.forName (cName);
            } catch (ClassNotFoundException ex) {
                return null;
            }
        else {
            c = getNodeString (documentName);
            if (c == null) return null;
            cache.put (documentName, c.getName ());
        }
        
        // create instance of c 
        return getNode (c, params);
    }
    
    private static Node getNode (Class c, Object[] params) {
        int i, k = params.length;
        Class[] par = new Class [k];
        for (i = 0; i < k; i++)
            par [i] = params [i].getClass ();
        
        try {
            Constructor constr = null;
            try {
                constr = c.getConstructor (par);
            } catch (NoSuchMethodException ex1) {
            }
            if (constr == null) {
                Constructor[] cs = c.getConstructors ();
                k = cs.length;
                for (i = 0; i < k; i++) {
                    Class[] parT = cs [i].getParameterTypes ();
                    if (parT.length != params.length) continue;
                    int j, jj = parT.length;
                    for (j = 0; j < jj; j++)
                        if (!compatible (parT [j], params [j]))
                            break;
                    if (j == jj) {
                        constr = cs [i];
                        break;
                    }
                }
            }
            return (Node) constr.newInstance (params);
        } catch (Exception ex) {
            ex.printStackTrace();
            return null;
        }
    }
    
    static HashMap helpT = new HashMap ();
    
    static {
        helpT.put (Integer.TYPE, Integer.class);
        helpT.put (Boolean.TYPE, Boolean.class);
        helpT.put (Character.TYPE, Character.class);
        helpT.put (Long.TYPE, Long.class);
        helpT.put (Byte.TYPE, Byte.class);
        helpT.put (Double.TYPE, Double.class);
        helpT.put (Short.TYPE, Short.class);
        helpT.put (Float.TYPE, Float.class);
    }
    
    static boolean compatible (Class a, Object b) {
        if (a.isInstance (b)) return true;
        return helpT.get (a).equals (b.getClass ());
    }
    
    private static Class getNodeString (String document) {
        LinkedList r = (LinkedList) reg.clone ();
        LinkedList n = (LinkedList) nodes.clone ();
        int i, k = r.size ();
        for (i = 0; i < k; i++) {
            String[] s = (String[]) r.get (i);
            if (s.length != 1) continue;
            if (s [0].equals (document))
                return (Class) nodes.get (i);
        }
        return null;
    }
    
    private static Class getNodeClass (Class document) {
        LinkedList r = (LinkedList) reg.clone ();
        LinkedList n = (LinkedList) nodes.clone ();
        int selected = -1;
        int count = -1;
        int rating = -1;
        Class[] oCls = null;
        int i, k = r.size ();
        for (i = 0; i < k; i++) {
            String[] clsName = (String[]) r.get (i);
            int j, jj = clsName.length;
            Class[] cls = new Class [jj];
            for (j = 0; j < jj; j++) {
                try {
                    cls [j] = Class.forName (clsName [j]);
                    if (!cls [j].isAssignableFrom (document))
                        break;
                } catch (ClassNotFoundException ex) {
                    return null;
                }
                
            }
            if (j < jj) continue;
            if ((jj == 1) && (cls [0].equals (document)))
                return (Class) nodes.get (i);
            if ( (selected == -1) ||
                 (count < jj)
            ) {
                selected = i;
                count = jj;
                oCls = cls;
                continue;
            }
            if (count > jj) continue;
            if (rating < 0) {
                rating = countRating (oCls, document, Integer.MAX_VALUE);
            }
            int newRating = countRating (cls, document, Integer.MAX_VALUE);//rating
            if (newRating < rating) {
                selected = i;
                count = jj;
                rating = newRating;
            }
        }
        if (selected < 0) return null;
        return (Class) nodes.get (selected);
    }
    
    private static int countRating (Class[] cls, Class o, int r) {
        LinkedList os = new LinkedList ();
        os.add (o);
        HashSet clss = new HashSet ();
        int j, jj = cls.length;
        for (j = 0; j < jj; j++)
            clss.add (cls [j]);
        return countRating (clss, os, r, 0, 0);
    }
    
    /**
     * Adds superclass and superinterfaces of c to o.
     */
    private static void addSuper (LinkedList o, Class c) {
        Class s = c.getSuperclass ();
        if (s != null)
            o.add (s);
        Class[] i = c.getInterfaces ();
        int j, jj = i.length;
        for (j = 0; j < jj; j++)
            o.add (i [j]);
    }
    
    private static int countRating (HashSet cls, LinkedList o, int r, int currentR, int depth) {
        int i, k = o.size ();
        for (i = 0; i < k; i++) {
            Class c = (Class) o.get (0);
            if (!cls.contains (c)) {
                addSuper (o, c);
                o.removeFirst ();
            } else {
                currentR += depth;
                if ( (cls.size () == 1) ||
                     (currentR >= r)
                ) return currentR;
                cls.remove (c);
            }
        }
        depth++;
        int od = depth * cls.size ();
        if (od >= r) return od;
        return countRating (cls, o, r, currentR, depth);
    }
}
