/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ResourceBundle;

import org.openide.util.NbBundle;
import org.openide.debugger.Watch;
import org.openide.debugger.DebuggerException;
import org.netbeans.modules.debugger.support.util.Validator;


/**
*
*
* @author   Jan Jancura
* @version  0.17, Apr 29, 1998
*/
public interface VariablesProducer {

    public static final String  PROP_VARIABLES = "variables";
    
    /**
    * Returns true if it has no subelements.
    *
    * @return true if it has no subelements.
    */
    public boolean isLeaf ();
    
    /**
    * Returns human-readable text info about currently returned variables 
    * or null. This text should be displayed with list of variables. 
    *
    * @return human-readable text info about currently returned variables 
    * or null
    */
    public String getInfo ();

    /**
    * If this AbstractVariable object represents instance of some class 
    * or array this method
    * returns variables (static and non-static) of this object.
    *
    * @return variables (static and non-static) of this object.
    */
    public AbstractVariable[] getVariables ();

    /**
    */
    public AbstractVariable[] getFilteredVariables ();

    /**
    * Adds listener on the property changing.
    */
    public void addPropertyChangeListener (PropertyChangeListener listener);

    /**
    * Removes listener on the property changing.
    */
    public void removePropertyChangeListener (PropertyChangeListener listener);
}
