/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.delegator;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;

import org.openide.debugger.DebuggerInfo;
import org.openide.util.NbBundle;
import org.netbeans.modules.debugger.*;
import org.netbeans.modules.debugger.support.DebuggerInfoProducer;



class ConnectorPanel extends JPanel implements DebuggerInfoProducer,
ActionListener {

    private JComboBox             cbConnectors;
    private DebuggerInfoProducer  producer;
    private boolean               doNotListen;
    private ArrayList             connectors = new ArrayList ();


    ConnectorPanel (java.util.List debuggerImpls)  {
        cbConnectors = new JComboBox ();
        int i, k = debuggerImpls.size ();
        for (i = 0; i < k; i++) {
            DebuggerImpl di = (DebuggerImpl) debuggerImpls.get (i);
            if (!(di instanceof ConnectSupport)) 
                break;
            JComponent c = ((ConnectSupport) di).getConnectPanel ();
            if (c == null)
                break;
            cbConnectors.addItem (di.getDisplayName ());
            connectors.add (c);
        }

        cbConnectors.setActionCommand ("SwitchMe!"); // NOI18N
        cbConnectors.addActionListener (this);

        setLayout (new GridBagLayout ());
        setBorder (new EmptyBorder (11, 11, 0, 10));
        refresh (0);
    }

    private void refresh (int index) {
        JLabel cbLabel = new JLabel (DelegatingDebugger.getLocString ("CTL_Connect_through"));
        cbLabel.setDisplayedMnemonic (DelegatingDebugger.getLocString ("CTL_Connect_through_Mnemonic").charAt(0));
        cbLabel.setLabelFor (cbConnectors);

        GridBagConstraints c = new GridBagConstraints ();
        c.insets = new Insets (0, 0, 6, 6);
        add (cbLabel, c);
        c = new GridBagConstraints ();
        c.weightx = 1.0;
        c.fill = java.awt.GridBagConstraints.HORIZONTAL;
        c.gridwidth = 0;
        c.insets = new Insets (0, 3, 6, 0);
        doNotListen = true;
        cbConnectors.setSelectedIndex (index);
        doNotListen = false;
        add (cbConnectors, c);

        c = new GridBagConstraints ();
        c.weightx = 1.0;
        c.weighty = 1.0;
        c.fill = java.awt.GridBagConstraints.BOTH;
        c.gridwidth = 0;
        JComponent comp = (JComponent) connectors.get (index);
        producer = (DebuggerInfoProducer) comp;
        add (comp, c);
    }

    /**
    * Returns DebuggerInfo
    */
    public DebuggerInfo getDebuggerInfo () {
        return producer.getDebuggerInfo ();
    }

    public void actionPerformed (ActionEvent e) {
        if (doNotListen) return;
        if (e.getActionCommand ().equals ("SwitchMe!")); // NOI18N
        removeAll ();
        refresh (((JComboBox) e.getSource ()).getSelectedIndex ());
        Component w = getParent ();
        while (!(w instanceof Window))
            w = w.getParent ();
        if (w != null) ((Window) w).pack (); // ugly hack...
        return;
    }
}
