/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.delegator;

/**
 *
 * @author  dprusa
 * @version 
 */

import java.util.Enumeration;
import java.util.Iterator;
import java.io.File;

import org.openide.TopManager;
import org.openide.execution.ExecInfo;
import org.openide.execution.NbProcessDescriptor;
import org.openide.debugger.DebuggerType;
import org.openide.debugger.DebuggerException;
import org.openide.debugger.DebuggerNotFoundException;
import org.openide.util.HelpCtx;
import org.openide.NotifyDescriptor;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;

import org.netbeans.modules.debugger.DebuggerImpl;
import org.netbeans.modules.debugger.support.ProcessDebuggerType;
import org.netbeans.modules.debugger.support.ProcessDebuggerInfo;
import org.netbeans.modules.debugger.support.util.Utils;

public class DefaultDebuggerType extends ProcessDebuggerType {

    static final long serialVersionUID = 5121438889855859123L;

    /** Property name of the debuggerType property */
    public static final String PROP_DEBUGGER_TYPE = "debuggerType"; // NOI18N

    // variables .................................................................

    /** debugger type */
    private String                selectedDebuggerType = null;


    // ...........................................................................

    /* Gets the display name for this debugger type. */
    public String displayName () {
        return org.openide.util.NbBundle.getBundle (
                   DefaultDebuggerType.class
               ).getString ("LAB_DefaultDebuggerType");
    }

    public HelpCtx getHelpCtx () {
        return new HelpCtx (DefaultDebuggerType.class);
    }

    /**
    * Setter for debuggerType property.
    */
    public void setDebuggerType (String debuggerType) {
        selectedDebuggerType = debuggerType;
        if (!setted) {
            boolean old = classic;
            classic = getClassicDefault ();
            firePropertyChange (PROP_CLASSIC, new Boolean (old), new Boolean (classic));
        }
    }

    /**
    * Getter for debuggerType property.
    */
    public String getDebuggerType () {
        Iterator list = DelegatingDebuggerImpl.getDelegatingImpl ().
            getDebuggerImpls ().iterator ();
        String def = null;
        boolean found = false;
        while (list.hasNext ()) {
            DebuggerImpl di = (DebuggerImpl) list.next ();
            if (di.getDisplayName ().equals (selectedDebuggerType)) {
                found = true;
                break;
            }
            String className = di.getClass ().getName ();
            if (className.equals ("org.netbeans.modules.debugger.jpda.JPDADebuggerImpl"))
                def = di.getDisplayName ();
            if ((def == null) && className.equals ("org.netbeans.modules.debugger.debug.ToolsDebuggerImpl"))
                def = di.getDisplayName ();           
        }
        if (!found)
            selectedDebuggerType = def;
        return selectedDebuggerType;
    }

    /**
    * Returns selected dubugger.
    */
    protected DebuggerImpl getDebuggerImpl () {
        Iterator list = DelegatingDebuggerImpl.getDelegatingImpl ().
            getDebuggerImpls ().iterator ();
        while (list.hasNext ()) {
            DebuggerImpl di = (DebuggerImpl) list.next ();
            if (di.getDisplayName ().equals (getDebuggerType ()))
                return di;
        }
        return null;
    }

    /**
    * Determines if classic switch will be used defaultly or not.
    */
    protected boolean getClassicDefault () {            
        getDebuggerType ();
        DebuggerImpl di = getDebuggerImpl ();
        if ((di != null) && 
            (di.getClass ().getName ().equals ("org.netbeans.modules.debugger.jpda.JPDADebuggerImpl"))) // NOI18N                 
            return Utils.classicPreferred (null);
        if (Utilities.getOperatingSystem () == Utilities.OS_MAC) 
            return false;
        return super.getClassicDefault ();
    }
    
    protected void startDebugger (
        String className,
        String[] arguments,
        String stopClassName,
        NbProcessDescriptor process,
        String classPath,
        String bootClassPath,
        String repositoryPath,
        String libraryPath,
        boolean classic,
        ExecInfo info,
        boolean stopOnMain
    ) throws DebuggerException {
        try {
            TopManager.getDefault ().getDebugger ().startDebugger (
                new DefaultDebuggerInfo (
                    getDebuggerImpl (),
                    className,
                    arguments,
                    stopClassName,
                    process,
                    classPath,
                    bootClassPath,
                    repositoryPath,
                    libraryPath,
                    getWorkingDirectory (), // workingDirectory
                    classic,
                    className
                )
            );
        }
        catch (DebuggerNotFoundException e) {
        }
    }
    
    /** Initialization of arguments.
     *
     * &return true if initialization pass without problems.
     */
/*    protected boolean prepareArguments () {
        if (!super.prepareArguments ()) return false;
        Class debuggerType = getDebuggerTypeClass ();
        if (debuggerType == null) {
            TopManager.getDefault ().notify (
                new NotifyDescriptor.Message (
                     NbBundle.getBundle (DefaultDebuggerType.class).
                     getString ("EXC_Debugger_not_installed")
                 ));
            return false;
        }
        return true;
    }*/
}