/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.delegator;

import java.util.*;
import java.beans.PropertyChangeEvent;
import javax.swing.SwingUtilities;

import org.openide.debugger.DebuggerException;

import org.netbeans.modules.debugger.*;
import org.netbeans.modules.debugger.support.*;


/**
* Core thread group delegates all functionality on inner instance of thread group.
*/
public class DelegatingThreadProducer extends ThreadPAdapter {

    // variables ...............................................................

    /** Delegating group */
    private ThreadsProducer      threadProducer;
    private Delegator            delegator = new Delegator ();


    // main methods ............................................................

    /**
    * Sets RemoteThreadGroup for this object.
    *
    * @param threadGroup RemoteThreadGroup which must be represented by this ToolsThread
    * instance. Id can be <CODE>null</CODE>.
    */
    public void setRemoteThreadGroup (ThreadsProducer threadProducer) {
        AbstractThread[] ts = null;
        int i, k;
        if (this.threadProducer != null) {
            this.threadProducer.removeThreadListener (delegator);
            ts = this.threadProducer.getThreads ();
            k = ts.length;
            for (i = 0; i < k; i++)
                removeThread (ts [i]);
        }
        ts = threadProducer.getThreads ();
        k = ts.length;
        for (i = 0; i < k; i++)
            addThread (ts [i]);
        this.threadProducer = threadProducer;
        threadProducer.addThreadListener (delegator);
    }


    // innerclasses ............................................................

    class Delegator implements ThreadListener {

        /**  Called when some thread is created in this thread group.
        *
        * @param t this new thread
        */
        public void threadCreated (AbstractThread t) {
            addThread (t);
        }
        
        /**  Called when some thread is destroyed in this thread group.
        *
        * @param t destroyed thread
        */
        public void threadDeath (AbstractThread t) {
            removeThread (t);
        }
    }
}
