/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.multisession;

import java.beans.*;
import java.awt.Image;
import java.util.ResourceBundle;

import org.openide.util.NbBundle;
import org.openide.execution.ProcessExecutor;
import org.openide.execution.Executor;
import org.openide.explorer.propertysheet.editors.FileOnlyEditor;


/** Object that provides beaninfo for {@link DebuggerExecutor}.
*
* @author Jan Jancura
*/
public class DebuggerExecutorBeanInfo extends SimpleBeanInfo {

    /** icon */
    private static Image icon;
    /** icon32 */
    private static Image icon32;

    private static BeanDescriptor descr;
    private static PropertyDescriptor[] prop;
    static {
        ResourceBundle bundle = NbBundle.getBundle (DebuggerExecutorBeanInfo.class);
        try {
            prop = new PropertyDescriptor[] {
                       new PropertyDescriptor (DebuggerExecutor.PROP_DEBUGGER_EXECUTOR_TYPE, DebuggerExecutor.class,
                                               "getDebuggerExecutorType", "setDebuggerExecutorType"), // 0 // NOI18N
                       new PropertyDescriptor (DebuggerExecutor.PROP_CLASSIC_SWITCH, DebuggerExecutor.class,
                                               "isClassic", "setClassic"), // 1 // NOI18N
                   };
            prop[0].setDisplayName (bundle.getString ("PROP_DEBUGGER_EXECUTOR_TYPE"));
            prop[0].setShortDescription (bundle.getString ("HINT_DEBUGGER_EXECUTOR_TYPE"));
            prop[0].setPropertyEditorClass (DebuggerExecutorEditor.class);
            prop[1].setDisplayName (bundle.getString ("PROP_CLASSIC_SWITCH"));
            prop[1].setShortDescription (bundle.getString ("HINT_CLASSIC_SWITCH"));
            prop[1].setExpert (true);
        } catch (IntrospectionException ex) {
            if (Boolean.getBoolean ("netbeans.debug.exceptions"))
                ex.printStackTrace ();
        }
        descr = new BeanDescriptor (DebuggerExecutor.class);
        descr.setName (bundle.getString ("CTL_DebuggerExecutorName"));
    }

    public BeanDescriptor getBeanDescriptor () {
        return descr;
    }

    /** Descriptor of valid properties
    * @return array of properties
    */
    public PropertyDescriptor[] getPropertyDescriptors () {
        return prop;
    }
    
    /**
    * Claim there are no icons available.  You can override
    * this if you want to provide icons for your bean.
    */
    public Image getIcon(int type) {
        if ((type == java.beans.BeanInfo.ICON_COLOR_16x16) || (type == java.beans.BeanInfo.ICON_MONO_16x16)) {
            if (icon == null) {
                icon = loadImage("/org/netbeans/modules/debugger/resources/debuggerExecution.gif");
            }
            return icon;
        } else { // 32
            if (icon32 == null) {
                icon32 = loadImage("/org/netbeans/modules/debugger/resources/debuggerExecution32.gif");
            }
            return icon32;
        }
    }

    public BeanInfo[] getAdditionalBeanInfo() {
        try {
            return new BeanInfo[] {
                       Introspector.getBeanInfo (ProcessExecutor.class)
                   };
        } catch (IntrospectionException ex) {
            return new BeanInfo[] {};
        }
    }
}
