/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.multisession;

import org.netbeans.modules.debugger.multisession.nodes.*;
import org.netbeans.modules.debugger.support.nodes.*;
import org.netbeans.modules.debugger.support.java.nodes.*;

import org.openide.actions.StartDebuggerAction;
import org.openide.util.actions.SystemAction;
import org.openide.TopManager;
import org.openide.loaders.*;
import org.openide.util.Utilities;

/**
* Module installation class for HtmlModule
*
* @author Jan Jancura
*/
public class EnterpriseModule extends org.netbeans.modules.debugger.support.DebuggerModule {

    // static ..............................................................................

    static final long serialVersionUID = -103749725432744558L;

    private static SessionRootNode sessionRootNode;
    private static SessionThreadRootNode sessionThreadsRootNode;


    // ModuleInstall implementation ........................................................

    /** Module installed for the first time. */
    public void installed () {
        super.installed ();
//        DATSupport.init ();
    }

    /** Module installed again. */
    public void restored () {
        super.restored ();
//        DATSupport.init ();
    }

    /** Module is being closed. */
    public boolean closing () {
        try {
            return ((EnterpriseDebugger) TopManager.getDefault ().
                getDebugger ()
            ).finishDebuggerIn ();
        } catch (Throwable e) {
        }
        return true; // agree to close
    }

    
    // main public methods ...................................................................

    public static SessionRootNode getSessionRootNode () {
        if (sessionRootNode == null) {
            sessionRootNode = new SessionRootNode ();
            addNode (sessionRootNode);
        }
        return sessionRootNode;
    }

    public static SessionThreadRootNode getSessionThreadsRootNode () {
        if (sessionThreadsRootNode == null) {
            sessionThreadsRootNode = new SessionThreadRootNode ();
            addNode (sessionThreadsRootNode);
        }
        return sessionThreadsRootNode;
    }


    // other methods .........................................................................

    protected void installActions () {
        super.installActions ();
        ((StartDebuggerAction) SystemAction.get (StartDebuggerAction.class)).setMultisession (true);
        /*
        try {

          // menu actions ...
          
          Utilities2.createAction (
            StartSessionAction.class, 
            DataFolder.create (
              TopManager.getDefault ().getPlaces ().folders ().menus (), 
              "Debug"
            ), 
            "Connect", true, false, false, false
          );
          
          // toolbars-debug actions ...
         
          Utilities2.createAction (
            StartSessionAction.class, 
            DataFolder.create (
              TopManager.getDefault ().getPlaces ().folders ().toolbars (), 
              "Debug"
            ), 
            "Connect", true, false, false, false
          );

          // toolbars-debug-full actions ...
          
          Utilities2.createAction (
            StartSessionAction.class, 
            DataFolder.create (
              TopManager.getDefault ().getPlaces ().folders ().toolbars (), 
              "DebugFull"
            ), 
            "Connect", true, false, false, false
          );
          
          // install into actions pool
          
          InstanceDataObject.create (
            DataFolder.create (
              TopManager.getDefault ().getPlaces ().folders ().actions (), 
              "Debug"
            ), 
            "StartSessionAction", 
            StartSessionAction.class.getName ()
          );
          
    } catch (Exception e) {
          if (System.getProperty ("netbeans.debug.exceptions") != null) {
            e.printStackTrace ();
          }
          // ignore failure to install
    }
        */
    }

    protected void uninstallActions () {
        super.uninstallActions ();
        ((StartDebuggerAction) SystemAction.get (StartDebuggerAction.class)).setMultisession (false);
        /*
        try {

          // menu actions ...

          Utilities2.removeAction (
            StartSessionAction.class, 
            DataFolder.create (
              TopManager.getDefault ().getPlaces ().folders ().menus (), 
              "Debug"
            )
          );

          // toolbars-debug actions ...
          
          Utilities2.removeAction (
            StartSessionAction.class, 
            DataFolder.create (
              TopManager.getDefault ().getPlaces ().folders ().toolbars (), 
              "Debug"
            )
          );
          
          // toolbars-debug-full actions ...
          
          Utilities2.removeAction (
            StartSessionAction.class, 
            DataFolder.create (
              TopManager.getDefault ().getPlaces ().folders ().toolbars (), 
              "Debug"
            )
          );
          
          // remove from actions pool
          
          InstanceDataObject.remove (
            DataFolder.create (
              TopManager.getDefault ().getPlaces ().folders ().actions (), 
              "Debug"
            ),
            "StartSessionAction", 
            StartSessionAction.class.getName ()
          );
          
    } catch (Exception e) {
          if (System.getProperty ("netbeans.debug.exceptions") != null) {
            e.printStackTrace ();
          }
          // ignore failure to uninstall
    }
        */
    }

    /**
    * Installation of debugger views.
    */
    protected void installViews () {
        addView (new DebuggerView (false, getSessionRootNode ()), false);
        addView (new DebuggerView (true, getBreakpointsRootNode ()), false);
        addView (new DebuggerView (true, getSessionThreadsRootNode ()), false);
        addView (new DebuggerView (true, getCallStackRootNode ()), false);
        addView (new DebuggerView (true, getWatchesRootNode ()), false);
        addView (new DebuggerView (true, getVariablesRootNode ()), false);
        addView (new DebuggerView (true, getClassesRootNode ()), false);
        //    addView (new ExpressionView ());
    }

    /**
    * Installation of debugger nodes.
    */
    protected void installNodes () {
        getSessionRootNode ();
        getBreakpointsRootNode ();
        getSessionThreadsRootNode ();
        getCallStackRootNode ();
        getWatchesRootNode ();
        getVariablesRootNode ();
        getClassesRootNode ();
    }
    
    protected void registerRootNodeClasses () {
        registerRootNodeClass (SessionRootNode.class);
        registerRootNodeClass (BreakpointsRootNode.class);
        registerRootNodeClass (SessionThreadRootNode.class);
        registerRootNodeClass (CallStackRootNode.class);
        registerRootNodeClass (WatchesRootNode.class);
        registerRootNodeClass (VariablesRootNode.class);
        registerRootNodeClass (ClassesRootNode.class);
    }
    
}

/*
* <<Log>>
* $
*/
