/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.multisession.actions;

import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.CookieAction;
import org.openide.nodes.Node;


/** SuspendResumeAction action.
* This class is final only for performance reasons,
* can be happily unfinaled if desired.
*
* @author   Jan Jancura
*/
public final class ConnectDisconnectAction extends CookieAction {
    /** generated Serialized Version UID */
    static final long serialVersionUID = 287995876365009335L;

    // static ..........................................................................................

    static boolean connect;

    // init ..........................................................................................

    /**
    * @param activatedNodes Currently activated nodes.
    */
    protected void performAction (final Node[] activatedNodes) {
        int i, k = activatedNodes.length;
        for (i = 0; i < k; i ++)
            ((SessionCookie) activatedNodes [i].getCookie (SessionCookie.class)).setConnected (connect);
    }

    /**
    * @param activatedNodes Currently activated nodes.
    */
    protected boolean enable (final Node[] activatedNodes) {
        if (activatedNodes == null) return false;
        int i, k = activatedNodes.length;
        if (k < 1) return false;
        SessionCookie sc = (SessionCookie) activatedNodes [0].getCookie (SessionCookie.class);
        if (sc == null) return false;
        connect = !sc.isConnected ();
        for (i = 1; i < k; i++)
            if ( (null == (sc = (SessionCookie) activatedNodes [i].getCookie (SessionCookie.class))) ||
                    (connect == sc.isConnected ())
               ) return false;
        return true;
    }

    /**
    * Returns MODE_ALL.
    */
    protected int mode () {
        return MODE_ALL;
    }

    /**
    * Returns SuspendCookie
    */
    protected Class[] cookieClasses () {
        return new Class [] {
                   SessionCookie.class
               };
    }

    /** @return the action's name */
    public String getName () {
        return connect == true
               ? NbBundle.getBundle (ConnectDisconnectAction.class).getString ("CTL_Connect")
               : NbBundle.getBundle (ConnectDisconnectAction.class).getString ("CTL_Disconnect");
    }

    /** @return the action's help context */
    public HelpCtx getHelpCtx () {
        return new HelpCtx (ConnectDisconnectAction.class);
    }

    /** The action's icon location.
    * @return the action's icon location
    */
    protected String iconResource () {
        return "/org/netbeans/modules/debugger/multisession/resources/disconnect.gif";
    }

    /**
    * Returns menu item which representates this submenu.
    *
    * @return <CODE>JMenuItem</CODE> Submenu representated with this context.
    */
    public javax.swing.JMenuItem getMenuPresenter () {
        setEnabled (enable (getActivatedNodes ()));
        return super.getMenuPresenter ();
    }
}

/*
* <<Log>>
*/
