/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.multisession.actions;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Dialog;
import java.beans.PropertyVetoException;
import java.util.Enumeration;
import java.util.ResourceBundle;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;

import org.openide.actions.StartDebuggerAction;
import org.openide.loaders.DataObject;
import org.openide.debugger.Debugger;
import org.openide.debugger.DebuggerNotFoundException;
import org.openide.DialogDescriptor;
import org.openide.NotifyDescriptor;
import org.openide.TopManager;
import org.openide.cookies.CompilerCookie;
import org.openide.cookies.DebuggerCookie;
import org.openide.windows.Workspace;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.NodeAction;
import org.openide.nodes.Node;
import org.openide.cookies.CompilerCookie;
import org.openide.compiler.Compiler;
import org.openide.compiler.CompilerJob;
import org.openide.compiler.CompilerTask;
import org.openide.windows.WindowManager;
import org.openide.windows.Workspace;

import org.netbeans.modules.debugger.multisession.EnterpriseDebugger;
import org.netbeans.modules.debugger.multisession.EnterpriseModule;

/**
* Starts a new debugger session.
* @see Debugger#startDebugger
* @see Debugger#go
* @see DebuggerCookie
*
* @author   Jan Jancura
*/
public class StartSessionAction extends NodeAction {


    // static ..........................................................................................

    static final long serialVersionUID = 3403920608369616710L;


    // other methods......................................................................................

    /* @return the action's icon */
    public String getName () {
        return NbBundle.getBundle (StartSessionAction.class).getString ("CTL_StartSessionAction");
    }

    /* @return the action's help context */
    public HelpCtx getHelpCtx() {
        return new HelpCtx (StartSessionAction.class);
    }

    /* @return the action's icon */
    protected String iconResource() {
        return "/org/netbeans/modules/debugger/multisession/resources/startSession.gif";
    }


    /* This performer starts a new debugger session.
    *
    * @param activatedNodes Currently activated nodes.
    */
    protected void performAction (final Node[] activatedNodes) {
        EnterpriseDebugger debugger = null;
        try {
            debugger = (EnterpriseDebugger) TopManager.getDefault ().getDebugger ();
        } catch (DebuggerNotFoundException e) {
            return;
        }

        /*    if (debugger.getState () == debugger.DEBUGGER_NOT_RUNNING) {
              // set workspace
              WorkspacePool dp = TopManager.getDefault ().getWorkspacePool ();
              Workspace d = dp.findWorkspace (StartDebuggerAction.getWorkspace());
              if (d != null) dp.setCurrentWorkspace (d);
            }*/

        // compile it!
        Node.Cookie cookie = activatedNodes[0].getCookie (CompilerCookie.Compile.class);
        if (cookie != null && StartDebuggerAction.getRunCompilation ()) {
            CompilerJob cj = new CompilerJob (Compiler.DEPTH_ZERO);
            ((CompilerCookie.Compile)cookie).addToJob (cj, Compiler.DEPTH_ZERO);
            // compile, wait for results and return if sometning went wrong
            CompilerTask ct = cj.start ();
            if (!ct.isSuccessful ()) {
                return;
            }
        }

        cookie = activatedNodes[0].getCookie (DebuggerCookie.class);
        if (cookie == null) {
            TopManager.getDefault().notify (
                new NotifyDescriptor.Message (
                    java.text.MessageFormat.format (
                        NbBundle.getBundle (StartDebuggerAction.class).getString ("FMT_MSG_CannotDebug"),
                        new Object[] {
                            activatedNodes[0].getDisplayName ()
                        }
                    ),
                    NotifyDescriptor.WARNING_MESSAGE)
            );
            return;
        }
        EnterpriseModule.changeWorkspace ();
        try {
            ((DebuggerCookie)cookie).debug (true);
        } catch (org.openide.debugger.DebuggerException e) {
            TopManager.getDefault().notify(
                new NotifyDescriptor.Exception(e.getTargetException() == null ? e : e.getTargetException(),
                                               NbBundle.getBundle (StartDebuggerAction.class).getString("EXC_Debugger") +
                                               ": " + e.getMessage())
            );
        }
    }

    /* Enables go action when only one node which supports
    * cookie action is selected.
    *
    * @param activatedNodes Currently activated nodes.
    */
    protected boolean enable (final Node[] activatedNodes) {
        if ((activatedNodes == null) || (activatedNodes.length != 1)) return false;
        return null != activatedNodes[0].getCookie (DebuggerCookie.class);
    }
}

/*
 * <<Log>>
 *  7    Gandalf   1.6         1/18/00  Daniel Prusa    GoAction replaced by 
 *       StartDebuggerAction
 *  6    Gandalf   1.5         10/23/99 Ian Formanek    NO SEMANTIC CHANGE - Sun
 *       Microsystems Copyright in File Comment
 *  5    Gandalf   1.4         8/17/99  Jan Jancura     Session nodes moved
 *  4    Gandalf   1.3         8/9/99   Jan Jancura     Functionality moved
 *  3    Gandalf   1.2         8/2/99   Jan Jancura     
 *  2    Gandalf   1.1         7/13/99  Jan Jancura     
 *  1    Gandalf   1.0         7/13/99  Jan Jancura     
 * $
 */
