/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.multisession.nodes;

import java.awt.Image;
import java.awt.Toolkit;
import java.beans.BeanInfo;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyEditor;
import java.lang.reflect.InvocationTargetException;
import java.util.ResourceBundle;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.Arrays;

import org.openide.TopManager;
import org.openide.actions.PropertiesAction;
import org.openide.actions.ToolsAction;
import org.openide.debugger.DebuggerException;
import org.openide.nodes.*;
import org.openide.explorer.propertysheet.editors.ChoicePropertyEditor;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.Mutex;
import org.openide.util.actions.SystemAction;

import org.netbeans.modules.debugger.*;
import org.netbeans.modules.debugger.support.actions.*;
import org.netbeans.modules.debugger.support.nodes.*;
import org.netbeans.modules.debugger.multisession.*;
import org.netbeans.modules.debugger.multisession.actions.*;


/**
* This class representates session as a Node.
*
* @author   Jan Jancura
*/
public final class SessionThreadNode extends FilterNode {
    /** generated Serialized Version UID */
    static final long                   serialVersionUID = -4736310787521117464L;


    // static ...........................................................................

    /** Property name */
    public static final String          PROP_SESSION_NAME        = "sessionName";
    /** Property name */
    public static final String          PROP_LOCATION_NAME       = "locationName";
    /** Property name */
    public static final String          PROP_PERSISTENT          = "persistent";
    
    protected static Image              sessionIcon;
    protected static Image              sessionCurrentIcon;

    /** System actions of this node */
    private static SystemAction[]       staticActions;


    // variables ...........................................................................

    /** Represented session. */
    private Session                     session;
    /** Listener for property current of session. */
    private SessionListener                listener;


    // init ...........................................................................

    /**
    * Creates empty SessionThreadNode.
    */
    SessionThreadNode (Session s) {
        super (DebuggerNode.createNode (s.getThreadsProducer ()));
        this.session = s;
        initialize ();
    }

    private void initialize () {
        //    setIconBase (ICON_BASE);
        String s = session.getLocationName ();
        String n = session.getSessionName () + (s.equals ("localhost") ? "" : (" : " + s));
        setDisplayName (n);
        setName (n);
        refreshIcon ();
        createProperties ();
        session.addPropertyChangeListener (
            listener = new SessionListener ()
        );
    }

    public HelpCtx getHelpCtx() {
        return new HelpCtx("NetbeansDebuggerThreadNode");        //NOI18N
    }


    // Node implementation ...........................................................................

    /** Creates property sets */
    private void createProperties () {
/*        final ResourceBundle bundle = NbBundle.getBundle (SessionThreadNode.class);
        // default sheet with "properties" property set
        Sheet sheet = getSheet ();
        Sheet.Set ps = sheet.get (Sheet.PROPERTIES);
        ps.put (new PropertySupport.ReadOnly (
            PROP_SESSION_NAME,
            String.class,
            bundle.getString ("PROP_session_name"),
            bundle.getString ("HINT_session_name")
        ) {
            public Object getValue () throws InvocationTargetException {
                return session.getSessionName ();
            }
        });
        ps.put (new PropertySupport.ReadOnly (
            PROP_LOCATION_NAME,
            String.class,
            bundle.getString ("PROP_location_name"),
            bundle.getString ("HINT_location_name")
        ) {
            public Object getValue () throws InvocationTargetException {
                return session.getLocationName ();
            }
        });
        ps.put (new PropertySupport.ReadOnly (
            PROP_PERSISTENT,
            Boolean.TYPE,
            bundle.getString ("PROP_persistent"),
            bundle.getString ("HINT_persistent")
        ) {
            public Object getValue () throws InvocationTargetException {
                return new Boolean (session.isPersistent ());
            }
        });
        // and set new sheet
        setSheet (sheet);*/
    }

    /**
    * Removes all listeners and destroyes node.
    */
    public void destroy () throws java.io.IOException {
        super.destroy ();
        session.removePropertyChangeListener (
            listener
        );
    }


    // private methods .....................................................................

    /**
    * Setsproper icon or session (session, current session).
    */
    protected void refreshIcon () {

/*        if (session.isCurrent ())
            setIconBase ("/org/netbeans/modules/debugger/multisession/resources/sessionCurrent");
        else
            setIconBase ("/org/netbeans/modules/debugger/multisession/resources/session");*/
    }


    // inner classes .......................................................

    /**
    * Listens on changing of current session.
    */
    private class SessionListener implements PropertyChangeListener {

        /**
        * Called when some property is changed in this thread group.
        *
        * @param e event
        */
        public void propertyChange (PropertyChangeEvent e) {
            if ( (e.getPropertyName () == null) &&
                 e.getPropertyName ().equals (Session.PROP_CURRENT)
               ) refreshIcon ();
        }
    }
}
