/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.support;

import org.openide.options.SystemOption;
//import org.openide.options.ContextSystemOption;
//import org.openide.actions.StartDebuggerAction;
import org.openide.debugger.DebuggerNotFoundException;
//import org.openide.execution.NbClassPath;
import org.openide.TopManager;
import org.openide.util.NbBundle;
import org.openide.util.HelpCtx;

/**
* Project Settings for java debugger.
*
* @author Jan Jancura, Ian Formanek
* @version 0.11, May 16, 1998
*/
public class DebuggerProjectSettings extends SystemOption {

    // static .....................................................................................

    /** generated Serialized Version UID */
    static final long serialVersionUID = 811103321325093629L;

    /** remoteDebugger property name */
    public static final String PROP_REMOTE_DEBUGGER = "remoteDebugger"; // NOI18N

    // SystemOption implementation ..................................................................

    /**
    * Returns name of this setings.
    */
    public String displayName () {
        return NbBundle.getBundle(DebuggerProjectSettings.class).getString("CTL_Debugger_project_settings");
    }

    public HelpCtx getHelpCtx () {
        return new HelpCtx (DebuggerProjectSettings.class);
    }

    public boolean isGlobal() {
        return false;
    }
    
    // SystemOption implementation ..................................................................

    // properties .................................................................................

    /**
    * Getter for remoteDebugger property. 
    */
    public DebuggerSupport getRemoteDebugger () throws DebuggerNotFoundException {
        return (DebuggerSupport) TopManager.getDefault ().getDebugger ();
    }

    /**
    * Setter for remoteDebugger property. 
    */
    public void setRemoteDebugger (DebuggerSupport javaDebugger) throws DebuggerNotFoundException {
        DebuggerSupport oldValue = getRemoteDebugger ();
        oldValue.setDebugger (javaDebugger);

        // fire the PropertyChange
        firePropertyChange (PROP_REMOTE_DEBUGGER, oldValue, javaDebugger);
    }

}