/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.support;

import java.awt.Image;
import java.awt.Toolkit;
import java.beans.*;
import java.util.ResourceBundle;

import org.openide.util.NbBundle;


/** A BeanInfor for DebuggerSettings.
*
* @author Jan Jancura
* @version 0.11, May 16, 1998
*/
public class DebuggerSettingsBeanInfo extends SimpleBeanInfo {

    /** Array of property descriptors. */
    private static PropertyDescriptor[] desc;
    private static BeanDescriptor descr;

    /** Icons */
    static Image icon;
    static Image icon32;

    // initialization of the array of descriptors
    static {
        descr = new BeanDescriptor (DebuggerSettings.class);
        ResourceBundle bundle = NbBundle.getBundle (DebuggerSettingsBeanInfo.class);
        descr.setDisplayName (bundle.getString ("CTL_Debugger_option"));
        descr.setShortDescription (bundle.getString ("HINT_Debugger_option"));

        try {
            desc = new PropertyDescriptor[] {
                       new PropertyDescriptor (DebuggerSettings.PROP_DESKTOP, DebuggerSettings.class,
                                               "getDesktop", "setDesktop"), // 0 // NOI18N
                       new PropertyDescriptor ("debugger", DebuggerSettings.class, // NOI18N
                                               "getRemoteDebugger", "setRemoteDebugger"), // 1 // NOI18N
                       new PropertyDescriptor (DebuggerSettings.PROP_RUN_COMPILATION, DebuggerSettings.class,
                                               "getRunCompilation","setRunCompilation"), // 2 // NOI18N
                       new PropertyDescriptor (DebuggerSettings.PROP_ACTION_ON_TRACE_INTO, DebuggerSettings.class,
                                               "getActionOnTraceInto", "setActionOnTraceInto"), // 3 // NOI18N
                       new PropertyDescriptor (DebuggerSettings.PROP_ACTION_ON_TRACE_INTO_SET, DebuggerSettings.class,
                                               "isActionOnTraceIntoSet", "setActionOnTraceIntoSet"), // 4 // NOI18N
                       new PropertyDescriptor (DebuggerSettings.PROP_SHOW_FINISH_DIALOG, DebuggerSettings.class,
                                               "getShowFinishDialog", "setShowFinishDialog"), // 5 // NOI18N
                       new PropertyDescriptor (DebuggerSettings.PROP_SHOW_RUN_MULTISESSION_DIALOG, DebuggerSettings.class,
                                               "getShowRunMultisessionDialog", "setShowRunMultisessionDialog"), // 6 // NOI18N
                       new PropertyDescriptor (DebuggerSettings.PROP_RUN_MULTISESSION, DebuggerSettings.class,
                                               "getRunMultisession", "setRunMultisession"), // 7 // NOI18N
                   };
            desc[0].setDisplayName (bundle.getString ("PROP_WORKSPACE"));
            desc[0].setShortDescription (bundle.getString ("HINT_WORKSPACE"));
            desc[0].setPropertyEditorClass (WorkspaceEditor.class);
            desc[1].setHidden (true);
            desc[2].setDisplayName (bundle.getString ("PROP_RunCompilation"));
            desc[2].setShortDescription (bundle.getString ("HINT_RunCompilation"));
            desc[3].setDisplayName (bundle.getString ("PROP_ActionOnTraceInto"));
            desc[3].setShortDescription (bundle.getString ("HINT_ActionOnTraceInto"));
            desc[3].setPropertyEditorClass (ActionTIEditor.class);
            desc[4].setDisplayName (bundle.getString ("PROP_ActionOnTraceIntoSet"));
            desc[4].setShortDescription (bundle.getString ("HINT_ActionOnTraceIntoSet"));
            desc[5].setDisplayName (bundle.getString ("PROP_ShowFinishDialog"));
            desc[5].setShortDescription (bundle.getString ("HINT_ShowFinishDialog"));
            desc[6].setDisplayName (bundle.getString ("PROP_ShowRunMultisessionDialog"));
            desc[6].setShortDescription (bundle.getString ("HINT_ShowRunMultisessionDialog"));
            desc[7].setDisplayName (bundle.getString ("PROP_RunMultisession"));
            desc[7].setShortDescription (bundle.getString ("HINT_RunMultisession"));
        } catch (IntrospectionException ex) {
            //throw new InternalError ();
            ex.printStackTrace (); //[PENDINGbeta comment out]
        }
    }

    /**
    * Returns BeanDescriptor.
    */
    public BeanDescriptor getBeanDescriptor () {
        return descr;
    }

    /** Descriptor of valid properties
    * @return array of properties
    */
    public PropertyDescriptor[] getPropertyDescriptors () {
        return desc;
    }

    /** Returns the DebuggerSettings' icon */
    public Image getIcon(int type) {
        if ((type == java.beans.BeanInfo.ICON_COLOR_16x16) || (type == java.beans.BeanInfo.ICON_MONO_16x16)) {
            if (icon == null)
                icon = loadImage("/org/netbeans/core/resources/debuggerSettings.gif"); // NOI18N
            return icon;
        } else {
            if (icon32 == null)
                icon32 = loadImage ("/org/netbeans/core/resources/debuggerSettings32.gif"); // NOI18N
            return icon32;
        }
    }
}

