/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.support;

import java.awt.Image;
import java.awt.Toolkit;
import java.beans.*;
import java.util.ResourceBundle;
import java.io.File;

import org.openide.util.NbBundle;
import org.openide.debugger.DebuggerType;
import org.openide.explorer.propertysheet.editors.DirectoryOnlyEditor;


/** A BeanInfo for ProcessDebuggerType.
*
* @author Jan Jancura
* @version 0.11, May 16, 1998
*/
public class ProcessDebuggerTypeBeanInfo extends SimpleBeanInfo {

    /** Array of property descriptors. */
    private static PropertyDescriptor[] desc;

    // initialization of the array of descriptors
    static {
        try {
            desc = new PropertyDescriptor[] {
                       new PropertyDescriptor (ProcessDebuggerType.PROP_DEBUGGER_PROCESS, ProcessDebuggerType.class,
                                               "getDebuggerProcess", "setDebuggerProcess"), // 0 // NOI18N
                       new PropertyDescriptor (ProcessDebuggerType.PROP_CLASSIC, ProcessDebuggerType.class,
                                               "isClassic", "setClassic"), // 1 // NOI18N
                       new PropertyDescriptor (ProcessDebuggerType.PROP_REPOSITORY, ProcessDebuggerType.class,
                                               "getRepositoryPath", null), // 2 // NOI18N
                       new PropertyDescriptor (ProcessDebuggerType.PROP_LIBRARY, ProcessDebuggerType.class,
                                               "getLibraryPath", null), // 3 // NOI18N
                       new PropertyDescriptor (ProcessDebuggerType.PROP_CLASSPATH, ProcessDebuggerType.class,
                                               "getClassPath", null), // 4 // NOI18N
                       new PropertyDescriptor (ProcessDebuggerType.PROP_BOOT_CLASSPATH, ProcessDebuggerType.class,
                                               "getBootClassPath", null), // 5 // NOI18N
                       null, // 6 .. created bellow ..
                   };
            ResourceBundle bundle = NbBundle.getBundle (ProcessDebuggerTypeBeanInfo.class);
            desc[0].setDisplayName (bundle.getString ("PROP_DEBUGGER_PROCESS"));
            desc[0].setShortDescription (bundle.getString ("HINT_DEBUGGER_PROCESS"));
            desc[1].setDisplayName (bundle.getString ("PROP_Classic"));
            desc[1].setShortDescription (bundle.getString ("HINT_Classic"));
            desc[2].setDisplayName (bundle.getString ("PROP_RepositoryPath"));
            desc[2].setShortDescription (bundle.getString ("HINT_RepositoryPath"));
            desc[2].setExpert (true);
            desc[3].setDisplayName (bundle.getString ("PROP_LibraryPath"));
            desc[3].setShortDescription (bundle.getString ("HINT_LibraryPath"));
            desc[3].setExpert (true);
            desc[4].setDisplayName (bundle.getString ("PROP_ClassPath"));
            desc[4].setShortDescription (bundle.getString ("HINT_ClassPath"));
            desc[4].setExpert (true);
            desc[5].setDisplayName (bundle.getString ("PROP_BootClassPath"));
            desc[5].setShortDescription (bundle.getString ("HINT_BootClassPath"));
            desc[5].setExpert (true);
            try {
                Runtime.class.getMethod ("exec", new Class[] { String[].class, String[].class, File.class }); // NOI18N
                desc[6] = new PropertyDescriptor (ProcessDebuggerType.PROP_WORKING_DIRECTORY, ProcessDebuggerType.class,
                                                  "getWorkingDirectory", "setWorkingDirectory"); // NOI18N
                desc[6].setShortDescription (bundle.getString ("HINT_WorkingDirectory_RW"));
            } catch (NoSuchMethodException nsme) {
                desc[6] = new PropertyDescriptor (ProcessDebuggerType.PROP_WORKING_DIRECTORY, ProcessDebuggerType.class,
                                                  "getWorkingDirectory", null); // NOI18N
                desc[6].setShortDescription (bundle.getString ("HINT_WorkingDirectory_RO"));
            }
            desc[6].setDisplayName (bundle.getString ("PROP_WorkingDirectory"));
            desc[6].setExpert (true);
            desc[6].setPropertyEditorClass (WorkDirEd.class);
            
        } catch (IntrospectionException ex) {
            if (Boolean.getBoolean ("netbeans.debug.exceptions"))
                ex.printStackTrace ();
        }
    }


    /** Descriptor of valid properties
    * @return array of properties
    */
    public PropertyDescriptor[] getPropertyDescriptors () {
        return desc;
    }

    public BeanInfo[] getAdditionalBeanInfo () {
        try {
            return new BeanInfo[] { Introspector.getBeanInfo (DebuggerType.class) };
        } catch (IntrospectionException ie) {
            if (Boolean.getBoolean ("netbeans.debug.exceptions"))
                ie.printStackTrace ();
            return null;
        }
    }

    /** @param type Desired type of the icon
    * @return returns the Executor's/Debugger Type's icon
    */
    public Image getIcon(final int type) {
        return getAdditionalBeanInfo()[0].getIcon(type);
    }

    // innerclasses ...................................................................
    
    public static class WorkDirEd extends DirectoryOnlyEditor {

        public String getAsText () {
            if (getValue () == null)
                return "null"; // NOI18N
            else
                return super.getAsText ();
        }

        public void setAsText (String text) {
            if ("null".equals (text)) // NOI18N
                setValue (null);
            else
                super.setAsText (text);
        }

    }
    
}
