/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.support;

import java.util.*;
import org.netbeans.modules.debugger.*;


public class ThreadPAdapter implements ThreadsProducer {

    private Vector                threads = new Vector ();
    // DebuggerListener support */
    private transient Vector      threadListener = new Vector ();
    
    
    /**
    * Returns array of thread in this thread group.
    *
    * @return array of thread in this thread group.
    */
    public AbstractThread[] getThreads () {
        AbstractThread[] t;
        synchronized (threads) {
            t = new AbstractThread [threads.size ()];
            threads.copyInto (t);
        }
        return t;
    }

    /**
    * Adds thread group listener for this thread group.
    *
    * @param l listener to be added.
    */
    public void addThreadListener (ThreadListener l) {
        threadListener.addElement (l);
    }

    /**
    * Removes thread group listener from this thread group.
    *
    * @param l listener to be removed.
    */
    public void removeThreadListener (ThreadListener l) {
        threadListener.removeElement (l);
    }
    
    /**
    * Notificates about creating a AbstractWatch.
    *
    * @param AbstractWatch
    */
    public void addThread (final AbstractThread thread) {
        threads.addElement (thread);
        Vector l = (Vector) threadListener.clone ();
        int i, k = l.size ();
        for (i = 0; i < k; i++)
            ((ThreadListener) l.elementAt (i)).threadCreated (thread);
    }

    /**
    * Notificates about removing a AbstractWatch.
    *
    * @param AbstractWatch
    */
    public void removeThread (final AbstractThread thread) {
        threads.removeElement (thread);
        Vector l = (Vector) threadListener.clone ();
        int i, k = l.size ();
        for (i = 0; i < k; i++)
            ((ThreadListener)l.elementAt (i)).threadDeath (thread);
    }
}
