/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.support.actions;

import java.awt.Dimension;
import java.awt.Window;
import java.awt.Component;
import javax.swing.SwingUtilities;

import org.openide.TopManager;
import org.openide.debugger.DebuggerNotFoundException;
import org.openide.util.NbBundle;

import org.netbeans.modules.debugger.*;
import org.netbeans.modules.debugger.CoreBreakpoint.Action;
import org.netbeans.modules.debugger.CoreBreakpoint.Event;

/**
* Customizer panel of breakpoint.
*
* @author  Jan Jacura
*/
public class AddBreakpointPanel extends javax.swing.JPanel {

    private CoreBreakpoint breakpoint;
    private boolean doNotRefresh = false;
    private CoreBreakpoint.Event[] events;

    /** Creates new form AddBreakpointPanel */
    public AddBreakpointPanel () {
        events = ((EventsProducer) Register.getDebuggerCoreImpl ()).
            getEvents ();
        initComponents ();
        //    setPreferredSize (new Dimension (520, 400));
    }

    private void initComponents () {
        setLayout (new java.awt.GridBagLayout ());
        java.awt.GridBagConstraints gridBagConstraints1;
//        setBorder (new javax.swing.border.EmptyBorder(new java.awt.Insets(12, 12, 0, 10)));

        jLabel1 = new javax.swing.JLabel ();
        cbEvents = new javax.swing.JComboBox ();

        jLabel1.setText (NbBundle.getBundle (AddBreakpointPanel.class).getString ("CTL_Breakpoint_type"));
        jLabel1.setLabelFor (cbEvents);
        jLabel1.setDisplayedMnemonic (NbBundle.getBundle (AddBreakpointPanel.class).getString ("CTL_Breakpoint_type_mnemonic").charAt (0));
        gridBagConstraints1 = new java.awt.GridBagConstraints ();
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.insets = new java.awt.Insets (12, 12, 0, 0);
        add (jLabel1, gridBagConstraints1);


        cbEvents.addActionListener (new java.awt.event.ActionListener () {
                                        public void actionPerformed (java.awt.event.ActionEvent evt) {
                                            cbEventsActionPerformed (evt);
                                        }
                                    }
                                   );


        gridBagConstraints1 = new java.awt.GridBagConstraints ();
        gridBagConstraints1.gridwidth = 0;
        gridBagConstraints1.insets = new java.awt.Insets (12, 12, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add (cbEvents, gridBagConstraints1);

        pEvent = new javax.swing.JPanel ();
        pEvent.setLayout (new java.awt.BorderLayout ());
        pEvent.setBorder (new javax.swing.border.CompoundBorder(
                              new javax.swing.border.TitledBorder(
                                  new javax.swing.border.EtchedBorder(), " " + NbBundle.getBundle (AddBreakpointPanel.class).getString ("CTL_Settings") + " "),
                              new javax.swing.border.EmptyBorder(new java.awt.Insets(0, 8, 0, 7))));


        gridBagConstraints1 = new java.awt.GridBagConstraints ();
        gridBagConstraints1.gridwidth = 0;
        gridBagConstraints1.insets = new java.awt.Insets (9, 9, 0, 9);
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add (pEvent, gridBagConstraints1);

        // ************************************

        pActionsOut = new javax.swing.JPanel ();
        pActionsOut.setLayout (new java.awt.BorderLayout ());
        pActionsOut.setBorder (new javax.swing.border.CompoundBorder(
                                   new javax.swing.border.TitledBorder(
                                       new javax.swing.border.EtchedBorder(), " " + NbBundle.getBundle (AddBreakpointPanel.class).getString ("CTL_Actions") + " "),
                                   new javax.swing.border.EmptyBorder(new java.awt.Insets(0, 8, 5, 7))));

        pActions = new javax.swing.JPanel ();
        pActions.setLayout (new javax.swing.BoxLayout (pActions, 1));

        pActionsOut.add (pActions, "North"); // NOI18N


        gridBagConstraints1 = new java.awt.GridBagConstraints ();
        gridBagConstraints1.gridwidth = 0;
        gridBagConstraints1.gridheight = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets (7, 9, 0, 9);
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 3.0;
        add (pActionsOut, gridBagConstraints1);

    }

    private void cbEventsActionPerformed (java.awt.event.ActionEvent evt) {
        // Add your handling code here:
        if (doNotRefresh) return;
        int j = cbEvents.getSelectedIndex ();
        if (j < 0) return;
        pEvent.removeAll ();
        pActions.removeAll ();
        setEvent (events [j]);
    }

    void setInitialEvent (Event e) {
        int i, k = events.length;
        for (i = 0; i < k; i++)
            if (events [i] == e) {
                cbEvents.setSelectedItem (events [i].getTypeDisplayName ());
            }
    }

    private void setEvent (Event ev) {
        Event newEvent;
        try {
            newEvent = (Event) ev.getClass ().newInstance ();
            breakpoint.setEvent (newEvent);
        } catch (Exception e) {
            e.printStackTrace ();
            return;
        }
        try {
            AbstractDebugger d = (AbstractDebugger) TopManager.getDefault ().getDebugger ();
            pEvent.add (breakpoint.getEvent (d).getCustomizer (), "Center"); // NOI18N
            Action[] a = newEvent.getActions ();
            int i, k = a.length;
            for (i = 0; i < k; i++) {
                Component c = a [i].getCustomizer ();
                if (c != null) pActions.add (c);
            }
        } catch (DebuggerNotFoundException ex) {
        }
        revalidate ();
        Window w = SwingUtilities.windowForComponent (this);
        if (w == null) return;
        w.pack ();
    }

    public void setBreakpoint (CoreBreakpoint breakpoint) {
        this.breakpoint = breakpoint;
        pEvent.removeAll ();
        pActions.removeAll ();
        if (cbEvents.getItemCount () > 0) cbEvents.removeAllItems ();
        int i, k = events.length;
        doNotRefresh = true;
        for (i = 0; i < k; i++)
            cbEvents.addItem (events [i].getTypeDisplayName ());
        doNotRefresh = false;
    }

    javax.swing.JLabel jLabel1;
    javax.swing.JComboBox cbEvents;
    javax.swing.JPanel pEvent;
    javax.swing.JPanel pActionsOut;
    javax.swing.JPanel pActions;
    javax.swing.JLabel jLabel2;
    javax.swing.JLabel jLabel3;
    javax.swing.JTextField tfCondition = null;

}

