/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.support.java;

import org.openide.debugger.DebuggerException;
import org.openide.text.Line;

import org.netbeans.modules.debugger.*;
import org.netbeans.modules.debugger.support.*;
import org.netbeans.modules.debugger.support.util.Utils;


/**
* Class representating one line of callstack.
*/
public abstract class CallStackFrame implements Location, VariablesProducer {

    /**
    * Returns line number of this frame in this callstack.
    *
    * @return Returns line number of this frame in this callstack.
    */
    public abstract int getLineNumber ();

    /**
    * Returns method name of this frame in this callstack.
    *
    * @return Returns method name of this frame in this callstack.
    */
    public abstract String getMethodName ();

    /**
    * Returns class name of this frame in this callstack.
    *
    * @return Returns class name of this frame in this callstack.
    */
    public abstract String getClassName ();

    /**
    * Returns name of file of this frame.
    *
    * @return Returns name of file of this frame.
    * @throws DebuggerException if informations about source are not included or some other error
    *   occurres.
    */
    public abstract String getSourceName () throws DebuggerException;

    /**
    * Returns true if it has no subelements.
    *
    * @return true if it has no subelements.
    */
    public boolean isLeaf () {
        return false;
    }
    
    /**
    */
    public AbstractVariable[] getFilteredVariables () {
        return getVariables ();
    }
    
    /**
    * Returns line object representing position in the editor where this thread 
    * callstack line is stopped.
    *
    * @throw DebuggerException if some problem occurs.
    * @return line object representing position where this callstack line stops.
    */
    public org.openide.text.Line getLine () {
        try {
            return Utils.getLineForSource (
                getClassName (),
                getSourceName (),
                getLineNumber ()
            );
        } catch (DebuggerException e) {
            return Utils.getLine (
                getClassName (),
                getLineNumber ()
            );
        }
    }
    
    public String getLocationName () {
        return getClassName () + '.' +
               getMethodName () + " : " + // NOI18N
               getLineNumber ();
    }
}
