/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.support.java;

import java.util.ResourceBundle;
import javax.swing.JComponent;

import org.openide.debugger.Debugger;
import org.openide.nodes.Node;
import org.openide.nodes.PropertySupport;
import org.openide.util.NbBundle;
import org.openide.util.actions.SystemAction;

import org.netbeans.modules.debugger.CoreBreakpoint;
import org.netbeans.modules.debugger.support.actions.AddBreakpointAction;


/**
* Abstract implementation of breakpoint event on class.
*
* @author   Jan Jancura
*/
public abstract class ClassBreakpointEvent extends CoreBreakpoint.Event {

    static final long               serialVersionUID = -4544769666886838818L;

    /** Property name constant */
    public static final String      PROP_CLASS_NAME = "className"; // NOI18N
    
    public static final String      TYPE_NAME = "Class";
    
    protected String                className = 
        ((AddBreakpointAction) SystemAction.get (AddBreakpointAction.class)).
        getCurrentClassName ();

    
    // Event implementation ....................................................................................

    /**
    * Returns specific properties of this event.
    */
    public Node.Property[] getProperties () {
        ResourceBundle bundle = NbBundle.getBundle (CoreBreakpoint.class);
        return new Node.Property[] {
           new PropertySupport.ReadWrite (
               PROP_CLASS_NAME,
               String.class,
               bundle.getString ("PROP_breakpoint_class_name"),
               bundle.getString ("HINT_breakpoint_class_name")
           ) {
               public Object getValue () {
                   return getClassName ();
               }
               public void setValue (Object val) throws IllegalArgumentException {
                   try {
                       setClassName (((String) val).trim ());
                   } catch (ClassCastException e) {
                       throw new IllegalArgumentException ();
                   }
               }
           }
       };
    }

    /**
    * Returns name of type of this event.
    */
    public String getTypeName () {
        return TYPE_NAME; // NOI18N
    }

    /**
    * Returns display name of this event.
    */
    public String getTypeDisplayName () {
        return NbBundle.getBundle (ClassBreakpointEvent.class).getString ("CTL_Class_event");
    }

    /**
    * Returns name of icon.
    */
    public String getIconBase () {
        return null;
    }

    /**
    * Returns customizer visual component.
    */
    public JComponent getCustomizer () {
        return new ClassBreakpointPanel (this);
    }


    // properties ........................................................................................

    /**
    * Get name of class to stop on.
    */
    public String getClassName () {
        return className;
    }

    /**
    * Set name of class to stop on.
    */
    public void setClassName (String cn) {
        if ( (cn == className) ||
             ((cn != null) && (className != null) && className.equals (cn))
        ) return;
        Object old = className;
        className = cn;
        firePropertyChange (PROP_CLASS_NAME, old, className);
    }
}
