/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is Forte for Java, Community Edition. The Initial
 * Developer of the Original Code is Sun Microsystems, Inc. Portions
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.support.java;

import java.util.ResourceBundle;
import java.util.List;
import java.util.Hashtable;

import org.openide.TopManager;
import org.openide.debugger.DebuggerNotFoundException;
import org.openide.util.NbBundle;
import org.openide.src.*;

import org.netbeans.modules.debugger.AbstractVariable;


/**
*
* @author   Daniel Prusa
*/
public abstract class JavaClass extends Object {

    // static final long serialVersionUID =-121348327319864863L;

    // variables ..........................................................................................

    private ClassElement          element = null;
    
    private Hashtable             innerClasses = new Hashtable ();
    
    // init ...............................................................................................

    // methods ............................................................................................
    
    /*
    * Returns the class name.
    */
    
    public abstract String getName ();
    
    public abstract JavaClass [] getInnerClasses ();
    
    // public abstract JavaClass findInnerClass (ClassElement el);
    
    public abstract MethodElement [] getMethods ();
    
    public abstract ConstructorElement [] getConstructors ();
    
    public abstract InitializerElement [] getInitializers ();
    
    public abstract FieldElement [] getFields ();
    
    public abstract boolean isClass ();
    
    public abstract int modifiers ();
    
    public abstract String classLoaderTypeName ();
    
    public abstract JavaVariable getClassLoader ();
    
    public abstract JavaVariable getStaticVariable (FieldElement element);
    
    public abstract boolean isPrepared ();
    
    public abstract Identifier getSuperClass ();
    
    public abstract Identifier [] getInterfaces ();
    
    public String getClassName () {
        String name = getName ();
        return name.substring (name.lastIndexOf ('.') + 1);
    }
    
    /**
    * Returns ClassElement created according to the JavaClass.
    */
    public ClassElement toClassElement () {
        
        if (element != null)
            return element;
        try {
            element = new ClassElement ();
            JavaClass [] inner = getInnerClasses ();
            innerClasses = new Hashtable ();
            ClassElement [] classes = new ClassElement [inner.length];
            for (int x = 0; x < inner.length; x++) {
                if ((classes [x] = inner [x].toClassElement ()) == null) {
                    classes [x] = new ClassElement ();
                }
                innerClasses.put (inner [x].getClassName (), inner [x]);
            }
            element.setName (Identifier.create (getName ()));
            element.setClasses (classes);
            element.setClassOrInterface (isClass ());
            Identifier superClass = getSuperClass ();
            if (superClass != null)
                element.setSuperclass (superClass);
            element.setInterfaces (getInterfaces ());
            if (isPrepared ()) {
                element.setConstructors (getConstructors ());
                element.setMethods (getMethods ());
                element.setInitializers (getInitializers ());
                element.setFields (getFields ());
            }
            element.setModifiers (modifiers ());
        } catch (SourceException e) {
            e.printStackTrace ();
        }
        return element;
    }

    public void updateClassElement (ClassElement element) {
        
        try {
            innerClasses = new Hashtable ();
            JavaClass [] inner = getInnerClasses ();
            ClassElement [] classes = new ClassElement [inner.length];
            for (int x = 0; x < inner.length; x++) {
                if ((classes [x] = inner [x].toClassElement ()) == null) {
                    classes [x] = new ClassElement ();
                }
                innerClasses.put (inner [x].getClassName (), inner [x]);
            }
            element.setClasses (classes);
            if (isPrepared ()) {
                element.setConstructors (getConstructors ());
                element.setMethods (getMethods ());
                element.setInitializers (getInitializers ());
                element.setFields (getFields ());
            }
            element.setModifiers (modifiers ());
        } catch (SourceException e) {
            e.printStackTrace ();
        }
    }
    
    public void updateInnerClasses (ClassElement element) {
        try {
            Hashtable temp = new Hashtable ();
            ClassElement [] last = element.getClasses ();
            JavaClass [] inner = getInnerClasses ();
            for (int x = 0; x < inner.length; x++) {
                ClassElement el = inner [x].toClassElement ();
                JavaClass clazz = (JavaClass) innerClasses.get (inner [x].getClassName ());
                if (clazz == null) {
                    innerClasses.put (inner [x].getClassName (), inner [x]);
                    element.addClass (el);
                }
                temp.put (inner [x].getClassName (), inner [x]);
            }
            for (int x = 0; x < last.length; x++) {
                if (temp.get (last [x].getName ().getName ()) == null)
                    element.removeClass (last [x]);
            }
            innerClasses = temp;
        } catch (SourceException e) {
            e.printStackTrace ();
        }
    }
 
    public JavaClass findInnerClass (ClassElement el) {
        return (JavaClass) innerClasses.get (el.getName ().getName ());
    }
    
}
