/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.support.java;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ResourceBundle;

import org.openide.util.NbBundle;
import org.openide.debugger.Watch;
import org.openide.debugger.DebuggerException;
import org.netbeans.modules.debugger.support.util.Validator;
import org.netbeans.modules.debugger.*;
import org.netbeans.modules.debugger.support.*;


/**
*
*
* @author   Jan Jancura
* @version  0.17, Apr 29, 1998
*/
public interface JavaVariable extends java.io.Serializable, AbstractVariable, 
Cloneable, Validator.Object, VariablesProducer {

    /**
    * Returns string representation of inner type of this variable.
    *
    * @return string representation of inner type of this variable.
    */
    public String getInnerType ();

    /**
    * Returns true, if this variable do not represents primitive type.
    *
    * @return true, if this variable do not represents primitive type.
    */
    public boolean isObject ();

    /**
    * Returns true, if this variable do not represents primitive type.
    *
    * @return true, if this variable do not represents primitive type.
    */
    public boolean isArray ();

    /**
    * Returns modifiers of this variable.
    *
    * @return modifiers of this variable.
    */
    public String getModifiers ();
    
    /**
     * If this AbstractVariable object represents instance of some array this method
     * returns (a part of) its items. This method does not check if the given interval
     * is inside the array!
     *
     * @param startindex index of the first item to be returned
     * @param numberofitems number of items to be returned
     * @return array of the items of the array
     */
    public AbstractVariable[] getVariables (int startindex, int numberofitems);
    
    /**
     * If this AbstractVariable object represents instance of some array this method
     * returns number of its items.
     *
     * @return number of items of the array
     */
    public int getVariablesNumber ();
    
    /**
     * Creates fixed variable pointing to the current instance.
     *
     * @return variable
     */
    public JavaVariable createFixedVariable ();
        
    /**
     * Tests if the variable is fixed or not.
     *
     * @return true if the variable is fixed
     */
    public boolean isFixedVariable ();
}

