/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.support.java;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import org.openide.TopManager;
import org.openide.debugger.Watch;
import org.openide.debugger.DebuggerNotFoundException;

import org.netbeans.modules.debugger.*;
import org.netbeans.modules.debugger.support.*;
import org.netbeans.modules.debugger.support.util.Validator;


/**
* Standart implementation of Watch interface.
* @see org.openide.debugger.Watch
*
* @author   Jan Jancura
* @version  0.18, Feb 23, 1998
*/
public abstract class JavaWatch extends AbstractWatch implements JavaVariable,
Validator.Object {

    /** generated Serialized Version UID */
    static final long               serialVersionUID = 3439367157517814302L;
    /** Property name constant */
    public static final String      PROP_IN_SCOPE = "inScope"; // NOI18N
    private transient PropertyChangeSupport pcs;
    /** Name of watch like xxx.yyy [2]. */
    protected String                displayName;
    private boolean                 isHidden = false;
    protected DebuggerSupport       debugger;
    protected transient Validator   validator;


    // init .....................................................................

    protected JavaWatch (DebuggerSupport debugger,boolean hidden) {
        this.debugger = debugger;
        isHidden = hidden;
        init ();
    }

    private void readObject(java.io.ObjectInputStream in)
    throws java.io.IOException, ClassNotFoundException {
        in.defaultReadObject ();
        // init () is called by the constructor, thus the following 5 lines are placed here
        // is it OK, Mr. Hanz ?
        try {
            debugger = (DebuggerSupport) TopManager.getDefault ().getDebugger ();
        } catch (DebuggerNotFoundException e) {
            throw new java.io.IOException ();
        }
        init ();
    }

    protected void init () {
        validator = ((ValidatorHolder) debugger).getValidator ();
        if (validator != null) validator.add (this);
        pcs = new PropertyChangeSupport (this);
    }


    // AbstractWatch implementation .........................................................

    /**
    * Returns the name of this watch.
    *
    * @return the name of this watch.
    */
    public String getVariableName () {
        return displayName;
    }

    /**
    * Sets variable name of this watch.
    *
    * @param name The name of variable of this watch.
    */
    public void setVariableName (String displayName) {
        String old = this.displayName;
        this.displayName = displayName;
        validate ();
    }

    /** Getter to test if the property is "hidden". If a property is hidden it
    * is not presented in the list of all properties. Such a property can be used
    * for private usage of the IDE, not displaying anything to user.
    * <P>
    * To create hidden watch, use <CODE>Debugger.createWatch ("name", true)</CODE> method.
    *
    * @return true if the watch is hidden, false otherwise
    */
    public boolean isHidden () {
        return isHidden;
    }

    /** Remove the watch from the list of all watches in the system.
     */
    public void remove () {
        debugger.removeWatch (this);
    }

    public synchronized void addPropertyChangeListener (PropertyChangeListener listener) {
        pcs.addPropertyChangeListener (listener);
    }

    public synchronized void removePropertyChangeListener (PropertyChangeListener listener) {
        pcs.removePropertyChangeListener (listener);
    }

    /**
    * Returns null. 
    *
    * @return null
    */
    public String getInfo () {
        return null;
    }

    
    // other public methods ....................................................

    /**
    * Returns true if this variable is in scope.
    *
    * @return true if this variable is in scope.
    */
    public abstract boolean isInScope ();

    /**
    * Create AbstractVariable object for this Watch. Can return null, if this Watch currently not
    * represents valide variable.
    *
    * @return AbstractVariable object for this class.
    */
    public abstract AbstractVariable getVariable ();

    /**
    * Returns error message if watch cannot be resolved or null.
    *
    * @return AbstractVariable object for this class.
    */
    public abstract String getErrorMessage ();


    // other methods ...........................................................

    protected void firePropertyChange (String s, Object o, Object n) {
        pcs.firePropertyChange (s, o, n);
    }
}

