/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.support.java;

import java.util.ResourceBundle;
import java.text.MessageFormat;
import javax.swing.JComponent;

import org.openide.debugger.Debugger;
import org.openide.nodes.Node;
import org.openide.nodes.PropertySupport;
import org.openide.util.NbBundle;
import org.openide.util.actions.SystemAction;

import org.netbeans.modules.debugger.support.actions.AddBreakpointAction;


/**
* Abstract implementation of breakpoint event on method.
*
* @author   Jan Jancura
*/
public abstract class MethodBreakpointEvent extends ClassBreakpointEvent {

    // static ..................................................................

    static final long               serialVersionUID =1311192209820078715L;

    /** Property name constant */
    public static final String      PROP_METHOD_NAME = "methodName"; // NOI18N
    
    public static final String      TYPE_NAME = "Method";

    
    // variables ...............................................................

    private String                  methodName = 
        ((AddBreakpointAction) SystemAction.get (AddBreakpointAction.class)).
        getCurrentMethodName ();
    
    
    // Event implementation ....................................................

    /**
    * Returns specific properties of this event.
    */
    public Node.Property[] getProperties () {
        ResourceBundle bundle = NbBundle.getBundle (MethodBreakpointEvent.class);
        return new Node.Property[] {
           new PropertySupport.ReadWrite (
               PROP_CLASS_NAME,
               String.class,
               bundle.getString ("PROP_breakpoint_class_name"),
               bundle.getString ("HINT_breakpoint_class_name")
           ) {
               public Object getValue () {
                   return getClassName ();
               }
               public void setValue (Object val) throws IllegalArgumentException {
                   try {
                       setClassName (((String)val).trim ());
                   } catch (ClassCastException e) {
                       throw new IllegalArgumentException ();
                   }
               }
           },
           new PropertySupport.ReadWrite (
               PROP_METHOD_NAME,
               String.class,
               bundle.getString ("PROP_breakpoint_method_name"),
               bundle.getString ("HINT_breakpoint_method_name")
           ) {
               public Object getValue () throws IllegalArgumentException {
                   return getMethodName ();
               }
               public void setValue (Object val) throws IllegalArgumentException {
                   try {
                       setMethodName (((String)val).trim ());
                   } catch (ClassCastException e) {
                       throw new IllegalArgumentException ();
                   }
               }
           }
       };
    }

    /**
    * Returns name of type of this event.
    */
    public String getTypeName () {
        return TYPE_NAME;
    }

    /**
    * Returns display name of this event.
    */
    public String getTypeDisplayName () {
        return NbBundle.getBundle (MethodBreakpointEvent.class).getString ("CTL_Method_event_type_name");
    }

    /**
    * Returns display name of this instance of event. It will be used
    * as the name of the breakpoint.
    */
    public String getDisplayName () {
        return new MessageFormat (
                   NbBundle.getBundle (MethodBreakpointEvent.class).getString ("CTL_Method_event_name")
               ).format (new Object[] {getClassName (), getMethodName ()});
    }

    /**
    * Returns name of icon.
    */
    public String getIconBase () {
        return "/org/netbeans/modules/debugger/resources/breakpointOnMethod"; // NOI18N
    }

    /**
    * Returns customizer visual component.
    */
    public JComponent getCustomizer () {
        return new MethodBreakpointPanel (this);
    }


    // properties ........................................................................................

    /**
    * Get name of method to stop on.
    */
    public String getMethodName () {
        return methodName;
    }

    /**
    * Set name of method to stop on.
    */
    public void setMethodName (String mn) {
        if ( (mn == methodName) ||
             ((mn != null) && (methodName != null) && methodName.equals (mn))
        ) return;
        String old = methodName;
        methodName = mn;
        firePropertyChange (PROP_METHOD_NAME, old, mn);
    }
}
