/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.support.java;

import org.openide.TopManager;
import org.openide.nodes.Node;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.CookieAction;
import org.openide.src.nodes.ConstructorElementNode;
import org.openide.src.ConstructorElement;
import org.openide.debugger.*;

import org.netbeans.modules.debugger.CoreBreakpoint;
import org.netbeans.modules.debugger.support.DebuggerSupport;
import org.netbeans.modules.debugger.support.util.Utils;


public class SetMethodBreakpointAction extends CookieAction {

    /** generated Serialized Version UID */
    static final long              serialVersionUID = -8487176709700045658L;

    /** @return the action's name */
    public String getName () {
        return NbBundle.getBundle (SetMethodBreakpointAction.class).getString ("CTL_add_method_breakpoint");
    }

    /** @return the action's help context */

    public HelpCtx getHelpCtx() {
        return new HelpCtx (SetMethodBreakpointAction.class);
    }

    public void performAction (Node [] activatedNodes) {
        try {
            SetMethodBreakpointCookie cookie = (SetMethodBreakpointCookie) activatedNodes [0].getCookie 
                (SetMethodBreakpointCookie.class);
            ConstructorElement method = cookie.getElement ();
            String className = cookie.getClassName ();
            DebuggerSupport debugger = (DebuggerSupport) TopManager.getDefault ().getDebugger ();
            CoreBreakpoint breakpoint = (CoreBreakpoint) debugger.createBreakpoint (method, false);
            MethodBreakpointEvent event = (MethodBreakpointEvent) breakpoint.getEvent ();
            event.setClassName (className);
        } catch (DebuggerNotFoundException ex) {
            return;
        }
    }

    protected int mode () {
        return MODE_EXACTLY_ONE;
    }

    protected Class[] cookieClasses () {
        return new Class [] {
                   SetMethodBreakpointCookie.class
               };
    }

}
