/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.support.java.nodes;

import java.awt.Image;
import java.awt.Toolkit;
import java.beans.BeanInfo;
import java.lang.reflect.InvocationTargetException;
import java.util.ResourceBundle;
import java.util.ArrayList;
import java.util.Arrays;

import org.openide.TopManager;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.text.Line;
import org.openide.cookies.LineCookie;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.Repository;
import org.openide.debugger.DebuggerException;
import org.openide.actions.PropertiesAction;
import org.openide.actions.ToolsAction;
import org.openide.util.NbBundle;
import org.openide.util.HelpCtx;
import org.openide.nodes.Node;
import org.openide.nodes.Sheet;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.PropertySupport;
import org.openide.util.actions.SystemAction;
import org.openide.util.actions.CallableSystemAction;

import org.netbeans.modules.debugger.support.*;
import org.netbeans.modules.debugger.support.nodes.LocationNode;
import org.netbeans.modules.debugger.support.actions.GoToSourceAction;
import org.netbeans.modules.debugger.support.actions.GoToSourceCookie;
import org.netbeans.modules.debugger.support.java.CallStackFrame;
import org.netbeans.modules.debugger.support.util.Utils;


/**
* This class callstack frame as a Node.
* This class is final only for performance reasons.
* Can be happily unfinaled if desired.
*
* @author   Jan Jancura
*/
public final class CallStackLineNode extends LocationNode {
    /** generated Serialized Version UID */
    static final long               serialVersionUID = -490111283333957792L;

    // static ....................................................................

    private static ResourceBundle   bundle;

    // variables .................................................................

    // init ......................................................................

    /** Creates call stack line with asociated stack frame
    */
    public CallStackLineNode (CallStackFrame stackFrame, int index) {
        super (stackFrame, index);
    }

    public HelpCtx getHelpCtx() {
        return new HelpCtx("NetbeansDebuggerCallStackLineNode");        //NOI18N
    }

    // Node implementation .......................................................

    /** Creates properties of this node */
    protected void createProperties () {
        // default sheet with "properties" property set // NOI18N
        Sheet sheet = Sheet.createDefault ();
        Sheet.Set props = sheet.get (Sheet.PROPERTIES);
        props.put (new PropertySupport.ReadOnly (
                       PROPERTY_CLASS_NAME,
                       String.class,
                       getLocalizedString ("PROP_call_stack_class_name"),
                       getLocalizedString ("HINT_call_stack_class_name")
                   ) {
                       public Object getValue () throws InvocationTargetException {
                           try {
                               return ((CallStackFrame) location).getClassName ();
                           } catch (Exception e) {
                               throw new InvocationTargetException (e);
                           }
                       }
                   });
        props.put (new PropertySupport.ReadOnly (
                       PROPERTY_METHOD_NAME,
                       String.class,
                       getLocalizedString ("PROP_call_stack_method_name"),
                       getLocalizedString ("HINT_call_stack_method_name")
                   ) {
                       public Object getValue () throws InvocationTargetException {
                           try {
                               return ((CallStackFrame) location).getMethodName ();
                           } catch (Exception e) {
                               throw new InvocationTargetException (e);
                           }
                       }
                   });
        props.put (new PropertySupport.ReadOnly (
                       PROPERTY_LINE_NUMBER,
                       Integer.TYPE,
                       getLocalizedString ("PROP_call_stack_line_number"),
                       getLocalizedString ("HINT_call_stack_line_number")
                   ) {
                       public Object getValue () throws InvocationTargetException {
                           try {
                               return new Integer (((CallStackFrame) location).getLineNumber ());
                           } catch (Exception e) {
                               throw new InvocationTargetException (e);
                           }
                       }
                   });
        // set the new sheet
        setSheet (sheet);
    }
    
    // helper methods ............................................................

    /**
    * @return localized string.
    */
    static String getLocalizedString (String s) {
        if (bundle == null)
            bundle = NbBundle.getBundle (CallStackLineNode.class);
        return bundle.getString (s);
    }

}