/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.support.java.nodes;

import java.util.ResourceBundle;

import org.openide.nodes.Node;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;

import org.netbeans.modules.debugger.AbstractThread;

import org.netbeans.modules.debugger.support.java.*;

public class ContendedMonitorNode extends AbstractNode {

    // static variables ..........................................................

    /** generated Serialized Version UID */
    static final long serialVersionUID = -5346121323458112148L;
    
    private static String              ICON_BASE =
        "/org/netbeans/modules/debugger/resources/contendedMonitor"; // NOI18N
    
    private static ResourceBundle     bundle;

    /**
    * @return localized string.
    */
    static String getLocalizedString (String s) {
        if (bundle == null)
            bundle = NbBundle.getBundle (ContendedMonitorNode.class);
        return bundle.getString (s);
    }
    
    // variables .................................................................
    
    JavaThread thread;
    
    private String monitorStr;
    
    // init ......................................................................

    ContendedMonitorNode (JavaThread thread) {
        super (new ContendedMonitorChildren ());
        this.thread = thread;
        String s = getLocalizedString ("CTL_Contended_monitor");
        monitorStr = s;
        setDisplayName (s);
        setName (s);
        setIconBase (ICON_BASE);
        update ();
    }
    
    public HelpCtx getHelpCtx() {
        return new HelpCtx("NetbeansDebuggerContendedMonitorNode");        //NOI18N
    }

    public void update () {
        Monitor monitor = ((MonitorInfoProducer) thread).contendedMonitor ();
        if (monitor == null) {
            setDisplayName (monitorStr);
        }
        else {
            setDisplayName (monitorStr + ": " + monitor.name ());
            JavaThread owner = monitor.owningThread ();
            JavaThread [] waitThreads = monitor.waitingThreads ();
            int index = (owner != null) ? 1 : 0; 
            ThreadHolder [] keys = new ThreadHolder [index + waitThreads.length];
            if (owner != null) // !?!
                keys [0] = new ThreadHolder (owner, true);
            for (int x = 0; x < waitThreads.length; x++)
                keys [index + x] = new ThreadHolder (waitThreads [x], false);
            ((ContendedMonitorChildren) getChildren ()).setMyKeys (keys);
        }
    }
    
    // innerclasses ..............................................................

    private static final class ContendedMonitorChildren extends Children.Keys {

        /** Creates nodes for given key.
        * @param key the key that is used
        * @return array of nodes representing the key
        */
        protected Node[] createNodes (final Object key) {
            if (((ThreadHolder) key).isOwner)
                return new Node[] {
                    new JavaThreadNode (
                        (JavaThread)((ThreadHolder) key).thread,
                        getLocalizedString ("CTL_Owning_thread_prefix")
                    )
                };
            return new Node[] {
                new JavaThreadNode (
                    (JavaThread)((ThreadHolder) key).thread
                )
            };    
        }

        /** Accessor for LocalesRootNode outer class */
        private void setMyKeys (final Object[] keys) {
            setKeys (keys);
        }

    }

    private static final class ThreadHolder {
        
        public boolean isOwner;
        
        public JavaThread thread;
        
        public ThreadHolder (JavaThread thread, boolean isOwner) {
            this.isOwner = isOwner;
            this.thread = thread;
        }
 
        public int hashCode () {
            return thread.hashCode ();
        }
        
        public boolean equals (Object o) {
            if (! (o instanceof ThreadHolder))
                return false;
            if (thread == null)
                return ((ThreadHolder) o).thread == null;
            return thread.equals (((ThreadHolder) o).thread);
        }
        
    }
    
}
