/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.support.java.nodes;

import java.util.ResourceBundle;

import org.openide.nodes.AbstractNode;
import org.openide.nodes.Node;
import org.openide.nodes.Children;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;

import org.netbeans.modules.debugger.AbstractThread;

import org.netbeans.modules.debugger.support.java.*;

public class MonitorNode extends AbstractNode {

    // static variables ..........................................................

    /** generated Serialized Version UID */
    static final long serialVersionUID = -6346125209458112778L;
    
    private static String              ICON_BASE =
        "/org/netbeans/modules/debugger/resources/monitor"; // NOI18N
    
    private static ResourceBundle     bundle;

    /**
    * @return localized string.
    */
    static String getLocalizedString (String s) {
        if (bundle == null)
            bundle = NbBundle.getBundle (MonitorNode.class);
        return bundle.getString (s);
    }
    
    // variables .................................................................
    
    Monitor monitor;
    
    private String monitorStr;
    
    // init ......................................................................

    MonitorNode (Monitor monitor) {
        super (new MonitorChildren ());
        this.monitor = monitor;
        monitorStr = getLocalizedString ("CTL_Monitor");
        setDisplayName (monitorStr);
        setName (monitorStr);
        setIconBase (ICON_BASE);
        update ();
    }
    
    public void update () {
        if (monitor != null) {
            setDisplayName (monitorStr + ": " + monitor.name ());
        }
        else
            setDisplayName (monitorStr);
        ((MonitorChildren) getChildren ()).setMyKeys (monitor.waitingThreads ());
    }
    
    public HelpCtx getHelpCtx() {
        return new HelpCtx("NetbeansDebuggerMonitorNode");        //NOI18N
    }

    // innerclasses ..............................................................

    private static final class MonitorChildren extends Children.Keys {

        /** Creates nodes for given key.
        * @param key the key that is used
        * @return array of nodes representing the key
        */
        protected Node[] createNodes (final Object key) {
            
            return new Node[] {
                       new JavaThreadNode ((JavaThread) key)
                   };
        }

        /** Accessor for LocalsRootNode outer class */
        private void setMyKeys (final Object[] keys) {
            setKeys (keys);
        }

    }
    
}
