/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.support.java.nodes;

import java.util.ResourceBundle;

import org.openide.nodes.AbstractNode;
import org.openide.nodes.Node;
import org.openide.nodes.Children;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;

import org.netbeans.modules.debugger.AbstractThread;

import org.netbeans.modules.debugger.support.java.Monitor;
import org.netbeans.modules.debugger.support.java.MonitorInfoProducer;

public class OwnedMonitorsNode extends AbstractNode {

    // static variables ..........................................................

    /** generated Serialized Version UID */
    static final long serialVersionUID = -6111221209458111328L;
    
    private static String              ICON_BASE =
        "/org/netbeans/modules/debugger/resources/ownedMonitors"; // NOI18N
    
    private static ResourceBundle     bundle;
    
    /**
    * @return localized string.
    */
    static String getLocalizedString (String s) {
        if (bundle == null)
            bundle = NbBundle.getBundle (OwnedMonitorsNode.class);
        return bundle.getString (s);
    }
    
    // variables .................................................................
    
    private AbstractThread thread;
    
    // init ......................................................................

    OwnedMonitorsNode (AbstractThread thread) {
        super (new OwnedMonitorsChildren ());
        this.thread = thread;
        String s = getLocalizedString ("CTL_Owned_monitors");
        setDisplayName (s);
        setName (s);
        setIconBase (ICON_BASE);
        update ();
    }
    
    public void update () {
        Monitor [] monitor = ((MonitorInfoProducer) thread).ownedMonitors ();
        ((OwnedMonitorsChildren) getChildren ()).setMyKeys (monitor);
    }
    
    public HelpCtx getHelpCtx() {
        return new HelpCtx("NetbeansDebuggerOwnedMonitorsNode");        //NOI18N
    }

    // innerclasses ..............................................................

    static final class OwnedMonitorsChildren extends Children.Keys {

        /** Creates nodes for given key.
        * @param key the key that is used
        * @return array of nodes representing the key
        */
        protected Node[] createNodes (final Object key) {
            return new Node[] {
                       new MonitorNode ((Monitor) key)
                   };
        }

        /** Accessor for LocalsRootNode outer class */
        private void setMyKeys (final Object[] keys) {
            setKeys (keys);
        }

    }
}
