/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is Forte for Java, Community Edition. The Initial
 * Developer of the Original Code is Sun Microsystems, Inc. Portions
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 */
package org.netbeans.modules.debugger.support.java.nodes;

import java.util.ResourceBundle;
import java.util.HashMap;
import java.util.Arrays;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import org.openide.TopManager;
import org.openide.nodes.Node;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.debugger.DebuggerException;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.Mutex;
import org.openide.src.ClassElement;
import org.openide.src.nodes.ClassElementNode;
import org.openide.src.nodes.ClassChildren;

import org.netbeans.modules.debugger.support.java.*;
import org.netbeans.modules.debugger.support.util.Utils;

import java.util.Comparator;

public class PackageNode extends AbstractNode {


    // static variables ..........................................................

    /** generated Serialized Version UID */
    static final long                  serialVersionUID = -5432332123422231L;

    private static String              ICON_BASE =
        "/org/openide/resources/defaultFolder"; // NOI18N
    private static String              ICON_OPENED =
        "/org/openide/resources/defaultFolderOpen"; // NOI18N

    private String                     name;
    
    private PackageContent             content;
    
    // comparator for class and package subnodes sorting
    private static Comparator comparator = new Comparator () {
        public int compare (Object o1, Object o2) {
            if (o1 instanceof JavaClass) {
                if (o2 instanceof PackageContent)
                    return 1;
                else
                    return ((JavaClass) o1).getClassName ().compareToIgnoreCase 
                        (((JavaClass) o2).getClassName ());
            }
            else {
                if (o2 instanceof JavaClass)
                    return -1;
                else
                    return ((PackageContent) o1).getName ().compareToIgnoreCase 
                        (((PackageContent) o2).getName ());
            }
        }
    };
    
    //private static ResourceBundle    bundle;

    // init ......................................................................

    /**
    * Creates empty VariablesContext.
    */
    public PackageNode (PackageContent content) {
        super (new PackageChildren ());
        this.content = content;
        init ();
    }

    public String getName () {
        return name;
    }
    
    protected void init () {
        name = content.getName ();
        setDisplayName (name);
        setName (name);
        setIconBase (ICON_BASE);
        changeChildren (content);
    }
    
    public HelpCtx getHelpCtx() {
        return new HelpCtx("NetbeansDebuggerPackageNode");        //NOI18N
    }

    // helper methods ............................................................

    /**
    * @return localized string.
    */
    /*
    static String getLocalizedString (String s) {
        if (bundle == null)
            bundle = NbBundle.getBundle (PackageNode.class);
        return bundle.getString (s);
    }
    */

    void changeChildren (PackageContent content) {
        ((PackageChildren)getChildren ()).setContent (content);
    }
    
    // innerclasses ..............................................................

    static final class PackageChildren extends Children.Keys {

        private boolean                 inited = false;
        
        PackageChildren () {
        }

        /** Creates nodes for given key.
        * @param key the key that is used
        * @return array of nodes representing the key
        */
        protected Node[] createNodes (final Object key) {
            if (key instanceof PackageContent)
                return new Node[] {new PackageNode ((PackageContent) key)};
            if (key instanceof JavaClass) {
                //ClassElement element = ((JavaClass) key).toClassElement ();
                return new Node[] {new ClassNode ((JavaClass) key, 
                    ((JavaClass) key).toClassElement (), 
                    new LoadedClassFactory ((JavaClass) key))};
            }
            return new Node[] {};
        }

        protected void addNotify() {
            super.addNotify ();
            inited = true;
        }
        
        /** Accessor for ClassesRootNode outer class */
        private void setMyKeys (final Object[] keys) {
            setKeys (keys);
        }

        void setContent (PackageContent content) {
            if (content.changed ()) {
                Object [] keys = content.getContent ();
                Arrays.sort (keys, comparator);
                setKeys (keys);
            }
            content.clearChangedFlag ();
            if (inited) {
                Node [] nodes = (Node []) Children.MUTEX.writeAccess (new Mutex.Action () {
                    public Object run () {
                        return getNodes ();
                    }
                });
                for (int x = 0; x < nodes.length; x++) {
                    if (nodes [x] instanceof PackageNode) {
                        PackageNode node = (PackageNode) nodes [x];
                        node.changeChildren (content.getSubContent (node.getName ()));
                    }
                    else if (nodes [x] instanceof ClassNode)
                        ((ClassNode) nodes [x]).update ();
                }
            }
        }
        
    } // end of PackageChildren inner class

}
