/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.support.util;

import java.beans.PropertyChangeListener;

import org.netbeans.modules.debugger.support.java.CallStackFrame;
import org.netbeans.modules.debugger.AbstractVariable;

import org.openide.debugger.DebuggerException;
import org.openide.text.Line;


/**
* Class representating one line of callstack.
*/
public class CallStackFrameMemoryImpl extends CallStackFrame {
    
    private String              className;
    private Line                line;
    private int                 lineNumber;
    private AbstractVariable[]  locals;
    private String              methodName;
    private String              sourceName;
    private String              info;
    
    public CallStackFrameMemoryImpl (
      CallStackFrame callStackFrame
    ) {
        this.className = callStackFrame.getClassName ();
        this.line = callStackFrame.getLine ();
        this.lineNumber = callStackFrame.getLineNumber ();
        this.locals = callStackFrame.getVariables ();
        this.methodName = callStackFrame.getMethodName ();
        this.info = callStackFrame.getInfo ();
        try {
            this.sourceName = callStackFrame.getSourceName ();
        } catch (DebuggerException e) {
        }
    }
    
    /**
    * Returns line number of this frame in this callstack.
    *
    * @return Returns line number of this frame in this callstack.
    */
    public int getLineNumber () {
        return lineNumber;
    }

    /**
    * Returns method name of this frame in this callstack.
    *
    * @return Returns method name of this frame in this callstack.
    */
    public String getMethodName () {
        return methodName;
    }

    /**
    * Returns class name of this frame in this callstack.
    *
    * @return Returns class name of this frame in this callstack.
    */
    public String getClassName () {
        return className;
    }

    /**
    * Returns human-readable text info about currently returned variables 
    * or null. This text should be displayed with list of variables. 
    *
    * @return human-readable text info about currently returned variables 
    * or null
    */
    public String getInfo () {
        return info;
    }
    
    /**
    * Returns current local variables for this frame stack.
    *
    * @return Current local variables for this frame stack.
    */
    public AbstractVariable[] getVariables () {
        return locals;
    }

    /**
    * Returns current local variables for this frame stack.
    *
    * @return Current local variables for this frame stack.
    */
    public AbstractVariable[] getFilteredVariables () {
        return locals;
    }
    
    /**
    * Returns name of file of this frame.
    *
    * @return Returns name of file of this frame.
    * @throws DebuggerException if informations about source are not included or some other error
    *   occurres.
    */
    public String getSourceName () throws DebuggerException {
        return sourceName;
    }

    /**
    * Returns line object representing position in the editor where this thread 
    * callstack line is stopped.
    *
    * @throw DebuggerException if some problem occurs.
    * @return line object representing position where this callstack line stops.
    */
    public org.openide.text.Line getLine () {
        return line;
    }
    
    /**
    * Adds listener on the property changing.
    */
    public void addPropertyChangeListener (PropertyChangeListener listener) {
    }

    /**
    * Removes listener on the property changing.
    */
    public void removePropertyChangeListener (PropertyChangeListener listener) {
    }
}

