/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.support.util;

import java.util.*;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

//import org.openide.util.RequestProcessor.Task;
//import org.openide.util.RequestProcessor;


/**
* Keeps weakly objects, and calls off-line their validate () method.
*
* @author Jan Jancura
*/
public class Validator {

    // variables .................................................................

    private WeakHashMap                 objects = new WeakHashMap ();
    private HashMap                     tasks = new HashMap ();
    private int                         waitingTasks = 0;
    private PropertyChangeSupport       pcs;

    // init ......................................................................

    public Validator () {
        pcs = new PropertyChangeSupport (this);
    }


    // main methods ..............................................................

    public void add (Object object) {
        //if (objects.containsKey (object)) return;

        //S ystem.out.println("add " + ((AbstractVariable) object).getVariableName () + " : " + object);
        synchronized (objects) {
            objects.put (object, null);
        }
    }

    public void remove (Object object) {
        synchronized (objects) {
            objects.remove (object);
        }
    }

    public synchronized void validate () {
        LinkedList list = new LinkedList ();
        Iterator i;
        synchronized (objects) {
            Iterator keys = objects.keySet ().iterator ();            
            try {
                while (keys.hasNext ())
                    list.add (keys.next ());
            } catch (NoSuchElementException e) {
            }
        }
        i = list.iterator ();
        while (i.hasNext ()) {
            final Object o = (Object) i.next ();
            if (o == null) continue;
            synchronized (o) {
                Task task = (Task) tasks.get (o);
                if (task != null) {
                    //S ystem.out.println ("Validator.task living! " + ((AbstractVariable) o).getVariableName () + " : " + o);
                    continue;
                }
                task = RequestProcessor.postRequest (new Runnable () {
                    public void run () {
                        synchronized (o) {
                            o.validate ();
                            //S ystem.out.println ("Validator.done task! " + ((AbstractVariable) o).getVariableName () + " : " + o);
                            removeTask (o);
                        }
                    }
                });
                //S ystem.out.println ("Validator.add task! " + ((AbstractVariable) o).getVariableName () + " : " + o);
                addTask (o, task);
            }
        }
        //S ystem.out.println("=========================================================");
    }

    public synchronized void clear () {
        Iterator i;
        synchronized (objects) {
            i = new ArrayList (objects.keySet ()).iterator ();
        }
        while (i.hasNext ()) {
            final Object o = (Object) i.next ();
            if (o == null) continue;
            synchronized (o) {
                if (o.canRemove ()) {
                    RequestProcessor.Task task = (RequestProcessor.Task) tasks.get (o);
                    if (task != null) {
                        task.cancel ();
                        removeTask (o);
                    }
                    remove (o);
                }
            }
        }
    }

    public boolean isValidated () {
        return waitingTasks == 0;
    }

    public void addPropertyChangeListener (PropertyChangeListener pcl) {
        pcs.addPropertyChangeListener (pcl);
    }

    public void removePropertyChangeListener (PropertyChangeListener pcl) {
        pcs.removePropertyChangeListener (pcl);
    }


    // helper methods ............................................................

    private void addTask (Object object, Task task) {
        tasks.put (object, task);
        waitingTasks++;
        if (waitingTasks == 1)
            pcs.firePropertyChange (null, null, null);
    }

    private void removeTask (Object object) {
        tasks.remove (object);
        waitingTasks--;
        if (waitingTasks == 0)
            pcs.firePropertyChange (null, null, null);
    }


    // innerclasses ..............................................................

    public interface Object {
        public void validate ();
        //public boolean canValidate ();
        public boolean canRemove ();
    }

}
