/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.jpda;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;

import org.openide.debugger.Debugger;
import org.openide.debugger.DebuggerInfo;

import org.netbeans.modules.debugger.support.DebuggerInfoProducer;

import com.sun.jdi.Bootstrap;
import com.sun.jdi.VirtualMachineManager;
import com.sun.jdi.connect.AttachingConnector;
import com.sun.jdi.connect.Connector.Argument;


/**
 * This class represents one type of debugger - Debugger Implementation.
 *
 * @author Jan Jancura
 */
class ConnectPanel extends JPanel implements DebuggerInfoProducer,
ActionListener {

    private JComboBox             cbConnectors;
    private Map                   args;
    private java.util.List        acs;
    private JTextField[]          tfParams;
    private AttachingConnector    ac;


    ConnectPanel () {
        VirtualMachineManager vmm = Bootstrap.virtualMachineManager ();
        acs = vmm.attachingConnectors ();
        setLayout (new GridBagLayout ());
        refresh (0);
    }

    private void refresh (int index) {
        GridBagConstraints c = new GridBagConstraints ();

        // No connector ................
        if (acs.size () == 0) {
            add (new JLabel (JPDADebugger.getLocString ("CTL_No_Connector")), c);
            return;
        }

        // Connector switch ................
        if (acs.size () > 1) {
            c.insets = new Insets (0, 0, 3, 3);
            add (new JLabel (
                JPDADebugger.getLocString ("CTL_Connector")
            ), c);

            cbConnectors = new JComboBox ();
            int i, k = acs.size ();
            for (i = 0; i < k; i++) {
                AttachingConnector ac = (AttachingConnector) acs.get (i);
                int jj = ac.name ().lastIndexOf ('.');
                String s = (jj < 0) ? ac.name () : ac.name ().substring (jj + 1);
                cbConnectors.addItem (s + " (" + ac.description () + ")");
            }
            c = new GridBagConstraints ();
            c.insets = new Insets (0, 3, 3, 0);
            c.weightx = 1.0;
            c.fill = java.awt.GridBagConstraints.HORIZONTAL;
            c.gridwidth = 0;
            cbConnectors.setSelectedIndex (index);
            cbConnectors.setActionCommand ("SwitchMe!");
            cbConnectors.addActionListener (this);
            add (cbConnectors, c);
        }

        ac = (AttachingConnector) acs.get (index);

        // Transport ................
        JTextField tfTransport = new JTextField (ac.transport ().name ());
        JLabel tfLabel = new JLabel (JPDADebugger.getLocString ("CTL_Transport"));
        tfLabel.setDisplayedMnemonic (JPDADebugger.getLocString ("CTL_Transport_Mnemonic").charAt(0));
        tfLabel.setLabelFor (tfTransport);
        
        c = new GridBagConstraints ();
        c.insets = new Insets (3, 0, 0, 6);
        add (tfLabel, c);

        tfTransport.setEnabled (false);
        c = new GridBagConstraints ();
        c.gridwidth = 0;
        c.insets = new Insets (3, 3, 0, 0);
        c.fill = java.awt.GridBagConstraints.HORIZONTAL;
        c.weightx = 1.0;
        add (tfTransport, c);

        // Other params ................
        args = ac.defaultArguments ();
        tfParams = new JTextField [args.size ()];
        Iterator it = args.keySet ().iterator ();
        int i = 0;
        while (it.hasNext ()) {
            String name = (String) it.next ();
            Argument a = (Argument) args.get (name);

            c = new GridBagConstraints ();
            c.insets = new Insets (6, 0, 0, 3);
            c.anchor = GridBagConstraints.WEST;
            JLabel iLabel = new JLabel (a.label () + ": ");
            iLabel.setDisplayedMnemonic (a.label ().charAt(0));
            add (iLabel, c);

            JTextField tfParam = new JTextField (a.value ());
            iLabel.setLabelFor(tfParam);
            tfParams [i ++] = tfParam;
            tfParam.setName (name);
            c = new GridBagConstraints ();
            c.gridwidth = 0;
            c.insets = new Insets (6, 3, 0, 0);
            c.fill = java.awt.GridBagConstraints.HORIZONTAL;
            c.weightx = 1.0;
            add (tfParam, c);
        }

        c = new GridBagConstraints ();
        c.weighty = 1.0;
        JPanel p = new JPanel ();
        p.setPreferredSize (new Dimension (1, 1));
        add (p, c);
    }

    /**
    * Returns DebuggerInfo.
    */
    public DebuggerInfo getDebuggerInfo () {
        int i, k = tfParams.length;
        for (i = 0; i < k; i++) {
            Argument a = (Argument) args.get (tfParams [i].getName ());
            a.setValue (tfParams [i].getText ());
        }
        return new RemoteDebuggerInfo (ac, args);
    }

    public void actionPerformed (ActionEvent e) {
        removeAll ();
        refresh (((JComboBox) e.getSource ()).getSelectedIndex ());
        Component w = getParent ();
        while (!(w instanceof Window))
            w = w.getParent ();
        if (w != null) ((Window) w).pack (); // ugly hack...
        return;
    }

    private String translate (String name) {
        return name;
    }
}



