/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.jpda;

import java.awt.Dialog;
import java.awt.event.*;
import javax.swing.*;
import java.net.URL;
import java.net.MalformedURLException;
import java.io.*;

import org.openide.TopManager;
import org.openide.DialogDescriptor;
import org.openide.util.NbBundle;

import org.netbeans.modules.debugger.Register;
import org.netbeans.modules.debugger.support.DebuggerSettings;


/**
* Module installation class for JPDADebugger Module
*
* @author Jan Jancura
*/
public class JPDADebuggerModule extends org.openide.modules.ModuleInstall {

    // static ..................................................................
    
    static final long           serialVersionUID = -2272025566936592771L;
    private static boolean      previouslyInstalled;
    private static int          counter;
    static boolean              installed;
    private static JPDADebuggerImpl jpdaDebuggerImpl = new JPDADebuggerImpl ();


    static {
        counter = 0;
        installed = false;
    }
    
    static JPDADebuggerImpl getJPDADebuggerImpl () {
        return jpdaDebuggerImpl;
    }
    

    // variables ...............................................................
    
    transient private Dialog    dialog;

    
    // init ....................................................................
    
    public JPDADebuggerModule () {
    }
    
    public JPDADebuggerModule (int counter, boolean installed) {
        this.counter = counter;
        this.installed = installed;
    }

    protected Object writeReplace () {
        return new JPDADebuggerModuleLoader (counter, installed);
    }

    
    // ModuleInstall ...........................................................
    
    /** Module installed for the first time. */
    public void installed () {
        restored ();
    }

    /** Module installed again. */
    public void restored () {
        previouslyInstalled = installed;
        try {
            Class.forName ("com.sun.jdi.VirtualMachineManager"); // NOI18N
            installed = true;
        } catch (ClassNotFoundException e) {
            installed = false;
        }
        if (installed) {
            DebuggerSettings ds = (DebuggerSettings) DebuggerSettings.findObject (
                                       DebuggerSettings.class, true
                                   );

            JPDADebugger.ACTION_ON_TI_FIND = ds.putActionTI (
                                                NbBundle.getBundle (JPDADebuggerModule.class).getString
                                                ("CTL_Action_on_TI_find")
                                             );

            Register.registerDebuggerImpl (
                jpdaDebuggerImpl
            );
        }
        else
            if (previouslyInstalled || (counter < 2))
                showWarning ();
        if (counter < 2)
            counter ++;
    }

    /** Module was uninstalled. */
    public void uninstalled () {
        if (installed)
            try {
                Register.unregisterDebuggerImpl (
                    jpdaDebuggerImpl
                );
            } catch (RuntimeException e) {
            }
    }

    /**
    * Shows JPDA not installed warning.
    */
    private void showWarning () {
        final JButton ok = new JButton (
                               NbBundle.getBundle (JPDADebuggerModule.class).getString ("PROP_OK")
                           );
        final JButton url = new JButton (
                                NbBundle.getBundle (JPDADebuggerModule.class).getString ("PROP_URL")
                            );
        JOptionPane pane = new JOptionPane (
                               NbBundle.getBundle (JPDADebuggerModule.class).getString ("EXC_JPDA_not_found"),
                               JOptionPane.INFORMATION_MESSAGE,
                               JOptionPane.DEFAULT_OPTION
                           );
        pane.setOptions (new Object[] {});
        DialogDescriptor desc = new DialogDescriptor (
            pane,
            NbBundle.getBundle (JPDADebuggerModule.class).getString ("MSG_MissingJPDATitle"),
            false,
            DialogDescriptor.DEFAULT_OPTION,
            DialogDescriptor.OK_OPTION,
            new ActionListener () {
                public void actionPerformed (ActionEvent ev) {
                    dialog.setVisible (false);
                    dialog.dispose ();
                    dialog = null;
                    if (ev.getSource () == url) {
                        // display www browser
                        try {
                            TopManager.getDefault ().showUrl (new URL (
                                                                  NbBundle.getBundle (JPDADebuggerModule.class).getString ("PROP_JPDA_URL")
                                                              ));
                        } catch (MalformedURLException ex) {
                            // Bad URL means no browsing ...
                        }
                    }
                }
            }
        );
        desc.setOptions (new Object[] {ok, url});
        dialog = TopManager.getDefault ().createDialog (desc);
        dialog.show ();
    }

}

// helper loader class ...............................................................

class JPDADebuggerModuleLoader implements Externalizable {

    private int                   counter;
    private boolean               installed;
    /** Current serialization version. */
    private static final int      SERIAL_VERSION = 1;

    static final long serialVersionUID = 4763887899005102648L;

    public JPDADebuggerModuleLoader () {
    }

    public JPDADebuggerModuleLoader (int counter, boolean installed) {
        this.counter = counter;
        this.installed = installed;
    }

    public void readExternal (ObjectInput oi) throws IOException, ClassNotFoundException {
        oi.readInt (); // SERIAL_VERSION
        counter = oi.readInt ();
        installed = oi.readBoolean ();
    }

    private Object readResolve () {
        return new JPDADebuggerModule (counter, installed);
    }

    public void writeExternal (ObjectOutput oo) throws IOException {
        oo.writeInt (SERIAL_VERSION);
        oo.writeInt (counter);
        oo.writeBoolean (installed);
    }
}
