/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.jpda;

import java.util.Enumeration;
import java.io.File;

import org.openide.TopManager;
import org.openide.execution.ExecInfo;
import org.openide.debugger.DebuggerType;
import org.openide.debugger.DebuggerException;
import org.openide.util.HelpCtx;
import org.openide.NotifyDescriptor;
import org.openide.util.NbBundle;

import org.netbeans.modules.debugger.support.ProcessDebuggerType;
import org.netbeans.modules.debugger.support.ProcessDebuggerInfo;


/**
* Default debugger type for JPDA debugger.
*/
public class JPDADebuggerType extends ProcessDebuggerType {

    static final long serialVersionUID = 5253430489855859777L;

    private Object readResolve () {
        return new ProcessDebuggerType ();
    }

    /* Gets the display name for this debugger type. */
    public String displayName () {
        return org.openide.util.NbBundle.getBundle (
                   JPDADebuggerType.class
               ).getString ("LAB_JPDADebuggerType");
    }

    public HelpCtx getHelpCtx () {
        return new HelpCtx (JPDADebuggerType.class);
    }

    /* Starts the debugger. */
    public void startDebugger (ExecInfo info, boolean stopOnMain) throws DebuggerException {
        if (!JPDADebuggerModule.installed) {
            TopManager.getDefault ().notify (new NotifyDescriptor.Message (
                                                 NbBundle.getBundle (JPDADebuggerModule.class).getString ("EXC_JPDA_not_installed")
                                             ));
            TopManager.getDefault ().getDebugger ().startDebugger (
                new ProcessDebuggerInfo (
                    info.getClassName (),
                    info.getArguments (),
                    stopOnMain ? info.getClassName () : null,
                    getDebuggerProcess (),
                    getClassPath (),
                    getBootClassPath (),
                    getRepositoryPath (),
                    getLibraryPath (),
                    getWorkingDirectory (),
                    isClassic (),
                    info.getClassName ()
                )
            );
            return;
        }
        TopManager.getDefault ().getDebugger ().startDebugger (
            new JPDADebuggerInfo (
                info.getClassName (),
                info.getArguments (),
                stopOnMain ? info.getClassName () : null,
                getDebuggerProcess (),
                getClassPath (),
                getBootClassPath (),
                getRepositoryPath (),
                getLibraryPath (),
                getWorkingDirectory (),
                isClassic (),
                info.getClassName ()
            )
        );
    }
}

