/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.jpda;

import java.util.*;

import com.sun.jdi.ThreadGroupReference;
import com.sun.jdi.ThreadReference;

import javax.swing.SwingUtilities;

import org.openide.debugger.DebuggerException;

import org.netbeans.modules.debugger.*;
import org.netbeans.modules.debugger.support.*;
import org.netbeans.modules.debugger.support.java.*;


/**
* JPDA Implementation of debugger thread group.
*/
public class JPDAThreadGroup extends JavaThreadGroup {

    // variables .................................................................

    /** Thread group which are representated by this AbstractThreadGroup instance */
    private ThreadGroupReference    threadGroup;


    // init ............................................................................

    /**
    * Creates ThreadGroup for some remote thread group.
    */
    JPDAThreadGroup (JPDAThreadGroup parentThreadGroup) {
        this (parentThreadGroup, null);
    }

    /**
    * Creates ThreadGroup for some remote thread group.
    */
    JPDAThreadGroup (JPDAThreadGroup parentThreadGroup, ThreadGroupReference threadGroup) {
        super (parentThreadGroup);
        this.threadGroup = threadGroup;
    }


    // implementation of AbstractThreadGroup .............................................

    /**
    * Getter for the name of thread group property.
    *
    * @return name of thread.
    */
    public String getName () {
        try {
            return threadGroup.name ();
        } catch (Exception e) {
        }
        return "Thread"; // NOI18N
    }


    // helper methods .....................................................................

    /**
    * Marks as suspended/resumed all threads in this thread group recursivelly.
    */
    void markSuspended (boolean suspended) {
        AbstractThread[] threads = getJavaThreads ();
        int i, k = threads.length;
        for (i = 0; i < k; i++)
            try {
                ((JPDAThread)threads [i]).markSuspended (suspended);
            } catch (DebuggerException e) { 
            }
        JPDAThreadGroup[] groups = (JPDAThreadGroup[]) getThreadGroups ();
        k = groups.length;
        for (i = 0; i < k; i++)
            ((JPDAThreadGroup)groups [i]).markSuspended (suspended);
    }
    
    ThreadGroupReference getThreadGroupReference () {
        return threadGroup;
    }
    
    public String toString () {
        try {
            return "Thread Group: " + getName () + " (" + super.toString () + ")"; // NOI18N
        } catch (Exception e) {
            return super.toString ();
        }
    }

    /**
    * Adds given thread group to this threadgroup.
    */
    void addThreadGroup (JPDAThreadGroup threadGroup) {
        super.addThreadGroup (threadGroup);
    }

    /**
    * Removes given thread group from this thread group.
    */
    void removeThreadGroup (JPDAThreadGroup threadGroup) {
        super.removeThreadGroup (threadGroup);
    }

    /**
    * Adds given thread to this threadgroup.
    */
    void addThread (JPDAThread thread) {
        super.addThread (thread);
    }

    /**
    * Removes given thread from this thread group.
    */
    void removeThread (JPDAThread thread) {
        super.removeThread (thread);
    }

    void removeAllD () {
        removeAll ();
    }

    /**
    * Refresh of thread group properties - fires changes.
    */
    void refresh () {
        firePropertyChange (null, null, null);
    }
}
