/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.debugger.jpda.util;

import java.util.Vector;

import org.openide.debugger.*;
import org.openide.text.Line;
import org.openide.src.ConstructorElement;

import org.netbeans.modules.debugger.CoreBreakpoint;
import org.netbeans.modules.debugger.CoreBreakpoint.Event;
import org.netbeans.modules.debugger.jpda.*;


/**
* Helper methods for debugger.
*
* @author  Jan Jancura
*/
public class JPDAFind {
    
    /** Find an breakpoint assigned to a specific line.
    * The line is represented by a line object that can change its
    * position as the text is modified.
    *
    * @param l line to find breakpoint at
    * @param endbled set true to find enabled breakpoints
    * @param disabled set true to find disabled breakpoints
    * @param visible set true to find visible breakpoints
    * @param hidden set true to find hidden breakpoints
    * @return the breakpoint or null
    */
    public static CoreBreakpoint[] variableBreakpoint (
        Debugger d, 
        String className, 
        String fieldName, 
        boolean enabled,
        boolean disabled,
        boolean visible,
        boolean hidden
    ) {
        Breakpoint[] b = d.getBreakpoints ();
        Vector br = new Vector ();
        int i, k = b.length;
        for (i = 0; i < k; i++) {
            if (!(b [i] instanceof CoreBreakpoint))
                continue;
            Event e = ((CoreBreakpoint) b [i]).getEvent ();
            if (e == null) 
                continue;
            if (!(e instanceof VariableBreakpoint)) 
                continue;
            String cn = ((VariableBreakpoint) e).getClassName ();
            String fn = ((VariableBreakpoint) e).getFieldName ();
            if ( (cn != null) && 
                 cn.equals (className) && 
                 (fn != null) && 
                 fn.equals (fieldName) && 
                 ( (enabled & b [i].isEnabled ()) ||
                   (disabled & !b [i].isEnabled ())) &&
                 ( (hidden & b [i].isHidden ()) ||
                   (visible & !b [i].isHidden ()))
            ) br.add (b [i]);
        }
        CoreBreakpoint[] res = new CoreBreakpoint [br.size ()];
        br.copyInto (res);
        return res;
    }
    
    /** Find an breakpoint assigned to a specific line.
    * The line is represented by a line object that can change its
    * position as the text is modified.
    *
    * @param l line to find breakpoint at
    * @param endbled set true to find enabled breakpoints
    * @param disabled set true to find disabled breakpoints
    * @param visible set true to find visible breakpoints
    * @param hidden set true to find hidden breakpoints
    * @return the breakpoint or null
    */
    public static CoreBreakpoint[] exceptionBreakpoint (
        Debugger d, 
        String exceptionName, 
        boolean enabled,
        boolean disabled,
        boolean visible,
        boolean hidden
    ) {
        Breakpoint[] b = d.getBreakpoints ();
        Vector br = new Vector ();
        int i, k = b.length;
        for (i = 0; i < k; i++) {
            if (!(b [i] instanceof CoreBreakpoint))
                continue;
            Event e = ((CoreBreakpoint) b [i]).getEvent ();
            if (e == null) 
                continue;
            if (!(e instanceof ExceptionBreakpoint)) 
                continue;
            String en = ((ExceptionBreakpoint) e).getClassName ();
            if ( (en != null) && 
                 en.equals (exceptionName) && 
                 ( (enabled & b [i].isEnabled ()) ||
                   (disabled & !b [i].isEnabled ())) &&
                 ( (hidden & b [i].isHidden ()) ||
                   (visible & !b [i].isHidden ()))
            ) br.add (b [i]);
        }
        CoreBreakpoint[] res = new CoreBreakpoint [br.size ()];
        br.copyInto (res);
        return res;
    }
}
