/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.editor;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Dialog;
import javax.swing.*;
import java.awt.event.*;
import java.util.HashMap;
import java.util.Map;
import java.util.List;


/** The support for creating macros.
 *
 * @author  Petr Nejedly
 * @version 1.0
 */
public class MacroDialogSupport implements ActionListener {

    JButton okButton;
    JButton cancelButton;

    MacroSavePanel panel;
    Dialog macroDialog;
    Class kitClass;
    
    /** Creates new MacroDialogSupport */
    public MacroDialogSupport( Class kitClass ) {
        this.kitClass = kitClass;
        panel = new MacroSavePanel(kitClass);
        okButton = new JButton(LocaleSupport.getString("MDS_ok"));
        okButton.setEnabled(false);
        cancelButton = new JButton(LocaleSupport.getString("MDS_cancel"));        

        panel.addPropertyChangeListener( new java.beans.PropertyChangeListener() {
                 public void propertyChange( java.beans.PropertyChangeEvent evt ) {
                     if( MacroSavePanel.PROP_MACRONAME == evt.getPropertyName() )
                        okButton.setEnabled( panel.getName().length() > 0 );
                 }
             } );
    }
    

    public void setBody( String body ) {
        panel.setBody( body );
    }
    
    public void showMacroDialog() {
        macroDialog = DialogSupport.createDialog(
                LocaleSupport.getString("MDS_title"),
                panel, true, new JButton[] { okButton, cancelButton }, false, 0, 1, this );

        macroDialog.pack();
        panel.popupNotify();
        macroDialog.requestFocus();
        macroDialog.show();
    }
    
    public void actionPerformed(java.awt.event.ActionEvent evt ) {
        Object source = evt.getSource();
        if( source == okButton ) {
            Map macroMap = (Map)Settings.getValue( kitClass, SettingsNames.MACRO_MAP);
            Map newMap = new HashMap( macroMap );
            
            if (newMap.get( panel.getName() ) != null)
            {
                // PENDING: will be solved in version 3.3
                // it requires extension of class DialogSupport
                // user must confirm here that wants to overwrite already existing macro
            }
            
            newMap.put( panel.getName(), panel.getBody() );
            Settings.setValue( kitClass, SettingsNames.MACRO_MAP, newMap );
            
            List listBindings = panel.getKeySequences();

              // insert listBindings into keybindings
            if (listBindings.size() > 0)
            {
                List keybindings = SettingsUtil.getClonedList( kitClass, SettingsNames.KEY_BINDING_LIST );
                String actionName = new String(BaseKit.macroActionPrefix + panel.getName());
                for (int i = 0; i < listBindings.size(); i++)
                {
                    KeyStroke[] keyStrokes = (KeyStroke[])listBindings.get(i);
                    MultiKeyBinding multiKey = new MultiKeyBinding(keyStrokes, actionName);
                    keybindings.add(multiKey);
                }

                // set new KEY_BINDING_LIST
                Settings.setValue( kitClass, SettingsNames.KEY_BINDING_LIST, keybindings);
            }
        }
        macroDialog.setVisible( false );
        macroDialog.dispose();        
    }
    
}
