/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.editor.ext.java;


import java.awt.Component;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;

import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;
import javax.swing.JLabel;
import javax.swing.SwingUtilities;
import javax.swing.ListCellRenderer;
import javax.swing.border.*;
import java.util.*;

import org.netbeans.editor.LocaleSupport;

/**
 * The panel for opening java classes.
 * 
 * @author Miloslav Metelka
 * @version 1.0
 */
class JavaFastOpenPanel extends javax.swing.JPanel {
   
    private JavaFastOpen jfo;
    
    /** Creates new form SaveMacroPanel */
    public JavaFastOpenPanel(JavaFastOpen jfo) {
        this.jfo = jfo;
        
        initComponents ();
        
        listScrollPane.setViewportView(jfo.getResultList());
        
        setMaximumSize( new Dimension( 400, 200 ) );

        expLabel.setDisplayedMnemonic (LocaleSupport.getString ("JFOP_expLabelMnemonic", "C").charAt (0));
        expField.addKeyListener(
            new KeyListener() {
                public void keyPressed(KeyEvent evt) {
                    change();
                }
                public void keyReleased(KeyEvent evt) {
                    change();
                }
                public void keyTyped(KeyEvent evt) {
                    change();
                }
                
                private void change() {
                    JavaFastOpenPanel.this.jfo.postUpdate(expField.getText());
                }
            }
        );
    }

    public Dimension getPreferredSize() {
        Dimension pref = super.getPreferredSize();
        Dimension max = getMaximumSize();
        if( pref.width > max.width ) pref.width = max.width;
        if( pref.height > max.height ) pref.height = max.height;
	return pref;
    }
    
    void popupNotify() {
        expField.requestFocus();
    }
    
    private void initComponents() {//GEN-BEGIN:initComponents
        queryPanel = new javax.swing.JPanel();
        expLabel = new javax.swing.JLabel();
        expField = new javax.swing.JTextField();
        listPanel = new javax.swing.JPanel();
        listScrollPane = new javax.swing.JScrollPane();
        
        setLayout(new java.awt.BorderLayout());
        
        setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(12, 12, 0, 11)));
        queryPanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        expLabel.setText(LocaleSupport.getString( "JFOP_expLabel", "Class Name:" ));
        expLabel.setLabelFor(expField);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.insets = new java.awt.Insets(2, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHEAST;
        queryPanel.add(expLabel, gridBagConstraints1);
        
        expField.setPreferredSize(new java.awt.Dimension(100, 21));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 12, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints1.weightx = 1.0;
        queryPanel.add(expField, gridBagConstraints1);
        
        add(queryPanel, java.awt.BorderLayout.NORTH);
        
        listPanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints2;
        
        listPanel.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(12, 0, 0, 0)));
        listScrollPane.setPreferredSize(new java.awt.Dimension(200, 100));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridheight = 3;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints2.weightx = 1.0;
        gridBagConstraints2.weighty = 1.0;
        listPanel.add(listScrollPane, gridBagConstraints2);
        
        add(listPanel, java.awt.BorderLayout.CENTER);
        
    }//GEN-END:initComponents
    

    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel queryPanel;
    private javax.swing.JLabel expLabel;
    private javax.swing.JTextField expField;
    private javax.swing.JPanel listPanel;
    private javax.swing.JScrollPane listScrollPane;
    // End of variables declaration//GEN-END:variables

}
