/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.editor;

import org.openide.filesystems.RepositoryEvent;
import org.openide.filesystems.RepositoryReorderedEvent;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.TopManager;
import org.netbeans.modules.editor.java.JCUpdater;
import org.openide.nodes.Node;
import org.openide.NotifyDescriptor;
import org.openide.util.NbBundle;
import java.text.MessageFormat;
import org.netbeans.modules.editor.java.JCStorage;


/**
 * Listener for Repository events.
 *
 * This class listen on Repository changes and if new FileSystem is
 * added to Repository, user will be asked, whether he wants to add
 * this element to parser DB.
 *
 * @author  Martin Roskanin
 */
class RepositListener implements org.openide.filesystems.RepositoryListener {
    
    /** Creates new RepositListener */
    public RepositListener() {
    }
    
    public void fileSystemAdded(RepositoryEvent ev){
        org.openide.windows.WindowManager wm=TopManager.getDefault().getWindowManager();
        if(wm!=null){
            java.awt.Frame frm = wm.getMainWindow();
            if(frm!=null){
                if(frm.isVisible()){
                    FileObject fo = ev.getFileSystem().getRoot();
                    // ignoring JavaDoc (if FS is hidden), ignoring FS already parsed
                    if ((fo!=null) && (!ev.getFileSystem().isHidden()) &&
                    (JCStorage.getStorage().findFileSystemElement(ev.getFileSystem().getSystemName())==null)){                    
                        try{
                            DataObject dataObj = DataObject.find(fo);
                            Node[] node = new Node[1];
                            node[0] = dataObj.getNodeDelegate();
                            NotifyDescriptor NDConfirm = new NotifyDescriptor.Confirmation(
                            MessageFormat.format(
                            NbBundle.getBundle( org.netbeans.modules.editor.java.JCUpdater.class ).getString("ask_update"),
                            new Object[] {ev.getFileSystem().getDisplayName()}),
                            NotifyDescriptor.YES_NO_OPTION
                            );                            
                            
                            TopManager.getDefault().notify(NDConfirm);
                            if (NDConfirm.getValue()==NDConfirm.YES_OPTION){
                                (new JCUpdater(node)).run();
                            }
                        }catch(DataObjectNotFoundException notFound){
                        }
                    }
                }
            }
        }
    }
    
    public void fileSystemRemoved(RepositoryEvent ev){
    }
    
    public void fileSystemPoolReordered(RepositoryReorderedEvent ev){
    }
    
}
