/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.editor.java;

import javax.swing.ButtonGroup;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;
import org.openide.util.NbBundle;
import org.netbeans.editor.ext.java.JavaCompletion;

/**
 *
 * @author  Miloslav Metelka
 * @version 1.0
 */
class JCLevelPanel extends javax.swing.JPanel {

    static final long serialVersionUID =-1647135977670510564L;
    /** Creates new form JCLevelPanel */
    JCLevelPanel(String title) {
        initComponents ();
        jLabel1.setText(title);

        ButtonGroup bg = new ButtonGroup();
        bg.add(publicLevel);
        bg.add(protectedLevel);
        bg.add(packageLevel);
        bg.add(privateLevel);

    }

    public int getLevel() {
        if (publicLevel.isSelected()) {
            return JavaCompletion.PUBLIC_LEVEL;
        } else if (protectedLevel.isSelected()) {
            return JavaCompletion.PROTECTED_LEVEL;
        } else if (packageLevel.isSelected()) {
            return JavaCompletion.PACKAGE_LEVEL;
        } else {
            return JavaCompletion.PRIVATE_LEVEL;
        }
    }

    public void setLevel(int level) {
        if (level == JavaCompletion.PUBLIC_LEVEL) {
            publicLevel.setSelected(true);
        } else if (level == JavaCompletion.PROTECTED_LEVEL) {
            protectedLevel.setSelected(true);
        } else if (level == JavaCompletion.PACKAGE_LEVEL) {
            packageLevel.setSelected(true);
        } else {
            privateLevel.setSelected(true);
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        publicLevel = new javax.swing.JRadioButton();
        protectedLevel = new javax.swing.JRadioButton();
        packageLevel = new javax.swing.JRadioButton();
        privateLevel = new javax.swing.JRadioButton();
        jLabel1 = new javax.swing.JLabel();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        publicLevel.setText(NbBundle.getBundle(getClass()).getString("CTL_JC_level_public"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(5, 12, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(publicLevel, gridBagConstraints1);
        
        protectedLevel.setSelected(true);
        protectedLevel.setText(NbBundle.getBundle(getClass()).getString("CTL_JC_level_protected"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(5, 12, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(protectedLevel, gridBagConstraints1);
        
        packageLevel.setText(NbBundle.getBundle(getClass()).getString("CTL_JC_level_package"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(5, 12, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(packageLevel, gridBagConstraints1);
        
        privateLevel.setText(NbBundle.getBundle(getClass()).getString("CTL_JC_level_private"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.gridheight = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(5, 12, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(privateLevel, gridBagConstraints1);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(jLabel1, gridBagConstraints1);
        
    }//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JRadioButton publicLevel;
    private javax.swing.JRadioButton protectedLevel;
    private javax.swing.JRadioButton packageLevel;
    private javax.swing.JRadioButton privateLevel;
    private javax.swing.JLabel jLabel1;
    // End of variables declaration//GEN-END:variables

}
