/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.editor.java;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.util.ResourceBundle;
import org.openide.util.NbBundle;
import org.openide.util.HelpCtx;

/**
 *
 * @author  Miloslav Metelka
 * @version 1.0
 */
public class JCProviderPanel extends javax.swing.JPanel {

    String BUNDLE_DB_PREFIX = "JC_db_prefix";  // NOI18N
    String BUNDLE_DB_PREFIX_MN = "JC_db_prefix.mnemonic";  // NOI18N
    String BUNDLE_TOOLTIP = "JC_title_tooltip"; // NOI18N    
    String BUNDLE_STORAGE_LEVELS = "JC_storage_levels"; // NOI18N
    String BUNDLE_CLASS_LEVEL = "JC_class_level"; // NOI18N
    String BUNDLE_FIELD_LEVEL = "JC_field_level"; // NOI18N
    String BUNDLE_METHOD_LEVEL = "JC_method_level"; // NOI18N

    private static ResourceBundle bundle;

    private JCLevelPanel classLevel;
    private JCLevelPanel fieldLevel;
    private JCLevelPanel methodLevel;

    private static final String HELP_ID = "editing.parserdatabase"; // !!! NOI18N
    
    static final long serialVersionUID =1588903994175008478L;
    /** Creates new form JCProviderPanel */
    public JCProviderPanel() {
        initComponents ();

        titleLabel.setText(getBundleString(BUNDLE_DB_PREFIX) + ":"); // NOI18N
        titleLabel.setToolTipText(getBundleString(BUNDLE_TOOLTIP));  
        titleLabel.setDisplayedMnemonic(getBundleString(BUNDLE_DB_PREFIX_MN).charAt(0));
        titleLabel.setLabelFor(namePrefix);
        
        jLabel1.setText(getBundleString(BUNDLE_STORAGE_LEVELS)); // NOI18N
        
        namePrefix.setToolTipText(getBundleString(BUNDLE_TOOLTIP));

        classLevel = new JCLevelPanel(getBundleString(BUNDLE_CLASS_LEVEL));
        fieldLevel = new JCLevelPanel(getBundleString(BUNDLE_FIELD_LEVEL));
        methodLevel = new JCLevelPanel(getBundleString(BUNDLE_METHOD_LEVEL));
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.insets = new Insets(5, 15, 10, 15);
        gbc.anchor = GridBagConstraints.NORTHWEST;
        levelPanel.add(classLevel, gbc);
        levelPanel.add(fieldLevel, gbc);
        levelPanel.add(methodLevel, gbc);

        HelpCtx.setHelpIDString (this, HELP_ID);
    }

    public String getNamePrefix() {
        return namePrefix.getText();
    }

    public void setNamePrefix(String s) {
        namePrefix.setText(s);
    }

    public int getClassLevel() {
        return classLevel.getLevel();
    }

    public int getFieldLevel() {
        return fieldLevel.getLevel();
    }

    public int getMethodLevel() {
        return methodLevel.getLevel();
    }

    public void setClassLevel(int level) {
        classLevel.setLevel(level);
    }

    public void setFieldLevel(int level) {
        fieldLevel.setLevel(level);
    }

    public void setMethodLevel(int level) {
        methodLevel.setLevel(level);
    }

    String getBundleString(String s) {
        if (bundle == null) {
            bundle = NbBundle.getBundle(JCProviderPanel.class);
        }
        return bundle.getString(s);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        namePrefix = new javax.swing.JTextField();
        titleLabel = new javax.swing.JLabel();
        jLabel1 = new javax.swing.JLabel();
        levelPanel = new javax.swing.JPanel();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentShown(java.awt.event.ComponentEvent evt) {
                formComponentShown(evt);
            }
        });
        
        namePrefix.setText("pd_prefix");
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(12, 5, 0, 12);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints1.weightx = 1.0;
        add(namePrefix, gridBagConstraints1);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.insets = new java.awt.Insets(12, 12, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(titleLabel, gridBagConstraints1);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(12, 12, 0, 12);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints1.weightx = 1.0;
        add(jLabel1, gridBagConstraints1);
        
        levelPanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints2;
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.gridheight = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.insets = new java.awt.Insets(12, 12, 17, 12);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints1.weighty = 1.0;
        add(levelPanel, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void formComponentShown(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_formComponentShown
        // Add your handling code here:
        namePrefix.requestFocus();
    }//GEN-LAST:event_formComponentShown


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextField namePrefix;
    private javax.swing.JLabel titleLabel;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JPanel levelPanel;
    // End of variables declaration//GEN-END:variables

}
