/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.editor.java;

import java.awt.Dialog;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.text.MessageFormat;
import java.util.List;
import java.util.Map;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.ResourceBundle;
import javax.swing.JButton;
import javax.swing.SwingUtilities;
import org.netbeans.editor.ext.java.JavaCompletion;
import org.netbeans.editor.ext.java.JCClass;
import org.netbeans.editor.ext.java.JCUtilities;
import org.netbeans.editor.ext.java.JCFileProvider;
import org.openide.src.ClassElement;
import org.openide.src.ConstructorElement;
import org.openide.src.MethodElement;
import org.openide.src.FieldElement;
import org.openide.src.Identifier;
import org.openide.src.SourceElement;
import org.openide.src.Type;
import org.openide.src.MethodParameter;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataFolder;
import org.openide.nodes.Node;
import org.openide.cookies.SourceCookie;
import org.openide.filesystems.FileObject;
import org.openide.TopManager;
import org.openide.DialogDescriptor;
import org.openide.NotifyDescriptor;
import org.openide.util.NbBundle;
import org.openide.filesystems.FileStateInvalidException;
import org.openide.filesystems.FileSystem;
import org.openide.util.enum.QueueEnumeration;

/**
* Updating of JC classes
*
* @author Miloslav Metelka
* @version 1.00
*/

public class JCUpdater extends Thread {

    private static ResourceBundle bundle;

    private static final String BUNDLE_TITLE = "JC_title"; // NOI18N
    private static final String BUNDLE_INSPECTING = "JC_inspecting"; // NOI18N
    private static final String BUNDLE_BUILDING_CLASS = "JC_building_class"; // NOI18N
    private static final String BUNDLE_INIT_PARSER = "JC_init_parser"; // NOI18N
    private static final String BUNDLE_UPDATING_DB = "JC_updating_db"; // NOI18N
    private static final String BUNDLE_UPDATING_MEMORY = "JC_updating_memory"; // NOI18N
    private static final String BUNDLE_ENTRIES = "JC_entries"; // NOI18N
    private static final String BUNDLE_CLASSES_FOUND = "JC_classes_found"; // NOI18N
    private static final String BUNDLE_CLASSES_DONE = "JC_classes_done"; // NOI18N
    private static final String BUNDLE_CLASSES_DONE_OF = "JC_classes_done_of"; // NOI18N

    Node[] activatedNodes;

    static ArrayList processedNodes = new ArrayList();    

    Map lookupCache;

    int doneCnt;
    
    public JCUpdater(){
        setName("ParserDB Updater"); // NOI18N
    }

    public JCUpdater(Node[] activatedNodes) {
        this.activatedNodes = activatedNodes;

        //    setPriority(Thread.MIN_PRIORITY);
        setName("ParserDB Updater"); // NOI18N
    }

    public void run() {
        ProgressPanel progress = new ProgressPanel();
        try {
            progress.setDialogVisible(true);

            lookupCache = new HashMap(89);

            Node[] nodes = (Node[])activatedNodes.clone();
            activatedNodes = null;

            for (int i = 0; i < nodes.length; i++) {
                Node n = nodes[i];

                progress.invokeSetBuilding(getBundleString(BUNDLE_INSPECTING));
                progress.invokeSetCurrentClass(""); // NOI18N
                progress.invokeSetDoneString(""); // NOI18N

                if (!processNode(n, progress)) {
                    break;
                }

                nodes[i] = null;
            }

            lookupCache.clear();
            lookupCache = null;

            progress.destroy();
            progress = null;
        } catch (Throwable t) {
            System.err.println("Exception occurred during parser database rebuilding"); // NOI18N
            t.printStackTrace();
            progress.destroy();
            progress = null;
        }
    }

    public synchronized boolean processNode(Node n, ProgressPanel progress) {
        QueueEnumeration qe = new QueueEnumeration();
        int cnt = 0;
        boolean root = false;
        String fsName = null;
        DataFolder df = (DataFolder)n.getCookie(DataFolder.class);
        if (processedNodes.contains(n)) return true;
        try{
            processedNodes.add(n);
            try {
                if (df != null) { // data folder
                    if ( inspectFolder(df, qe, progress) == false ){
                        return false;
                    }
                    cnt = doneCnt;
                    doneCnt = 0;
                    FileObject fo = df.getPrimaryFile();
                    fsName = fo.getFileSystem().getSystemName();
                    root = fo.isRoot();
                    
                } else {
                    SourceCookie sc = (SourceCookie)n.getCookie(SourceCookie.class);
                    if (sc != null) {
                        cnt = 1;
                        // Updating has been invoked on Java source object.
                        // This happens mainly during automatic parsing of modified sources
                        qe.put(sc);
                        fsName = ((DataObject)n.getCookie(DataObject.class)).getPrimaryFile(
                        ).getFileSystem().getSystemName();
                    }
                }
            } catch (FileStateInvalidException e) {
                // fsName will be null
            }
            
            // process queue
            if (cnt > 0 && fsName != null) {
                if (progress!=null)
                    return progress.cancelled ? false : updateProvider(qe, cnt, fsName, root, progress);
                else
                    return updateProviderOnBackground(qe, cnt, fsName, root);
            }
        }finally{
            processedNodes.remove(n);
        }
        return true;
    }

    boolean inspectFolder(DataFolder df, QueueEnumeration qe, ProgressPanel progress) {
        DataObject[] children = df.getChildren();
        children = (DataObject[])children.clone();
        boolean validFolder = false;
        for (int i = 0; i < children.length; i++) {
            if ((progress!=null) && (progress.cancelled)) {
                return false;
            }
            DataObject dob = children[i];
            if (dob instanceof DataFolder) {
                if (progress!=null){
                    progress.invokeSetCurrentClass(Integer.toString(doneCnt) + " " + getBundleString(BUNDLE_CLASSES_FOUND));
                }
                if ( inspectFolder((DataFolder)dob, qe, progress) == false ){
                    return false;
                }
            } else {
                    SourceCookie sc = (SourceCookie)dob.getCookie(SourceCookie.class);
                    if (sc != null) {
                        validFolder = true;
                    }
                }
            children[i] = null;
        }
        if (validFolder) {
            // we will take into the consideration only valid folders (which contains JAVA sources... )
            qe.put(df);
            doneCnt++;
        }
        
        return true;
    }

    private boolean appendSource(SourceCookie sc, JCStorageElement element, ProgressPanel progress){
        final int classLevel = element.getClassLevel();
        final int fieldLevel = element.getFieldLevel();
        final int methodLevel = element.getMethodLevel();
        SourceElement se = sc.getSource();
        ClassElement ce[] = se.getAllClasses();
        try{
            if (ce != null) {
                for (int i = 0; i < ce.length; i++) {
                    if (JavaCompletion.getLevel(ce[i].getModifiers()) >= classLevel){
                        JCClass classToAppend = JCExtension.parseClassElement(ce[i], classLevel,
                        fieldLevel, methodLevel, lookupCache, false);
                        if (classToAppend != null) {
                            element.getProvider().append(new JavaCompletion.SingleProvider(classToAppend));
                            classToAppend = null;
                            if ((progress!=null) && (progress.cancelled)) {
                                return false;
                            }
                        }
                    }
                }
            }
        }catch(OutOfMemoryError oome){
            return false;
        }
        return true;
    }

    boolean updateProvider(final QueueEnumeration qe, final int classCnt,
                           String fsName, boolean isRoot, final ProgressPanel progress) {
         
        // Get provider
        JCStorage storage = JCStorage.getStorage();
        JCStorageElement element = storage.findFileSystemElement(fsName);

        if (isRoot || element == null) { // not yet created
            progress.setDialogVisible(false);
            
            // Display dialog with options
            JCProviderPanel pp = new JCProviderPanel();
            if (element != null) {
                pp.setNamePrefix(element.getName());
                pp.setClassLevel(element.getClassLevel());
                pp.setFieldLevel(element.getFieldLevel());
                pp.setMethodLevel(element.getMethodLevel());
            }
            boolean overwrite;
            do{
                overwrite = true;
                DialogDescriptor dd = new DialogDescriptor(pp,
                getBundleString(BUNDLE_TITLE) + ' ' + fsName);
                Dialog d = TopManager.getDefault().createDialog(dd);
                d.pack();
                d.setVisible(true);
                Object o = dd.getValue();
                if (o == DialogDescriptor.OK_OPTION) {
                    JCStorageElement jcse[] = storage.getElements();
                    for(int m=0; m<jcse.length; m++){
                        // if DB file with entered prefix exists and fileSystem is different,
                        // means that user can lost previous data
                        if ((pp.getNamePrefix().equals(jcse[m].getName())) && (!fsName.equals(jcse[m].getFileSystemName()))){
                            NotifyDescriptor NDConfirm = new NotifyDescriptor.Confirmation(
                            MessageFormat.format(
                            NbBundle.getBundle( org.netbeans.modules.editor.java.JCUpdater.class ).getString("ask_overwrite"),
                            new Object[] {pp.getNamePrefix()}),
                            NotifyDescriptor.YES_NO_OPTION,
                            NotifyDescriptor.WARNING_MESSAGE
                            );
                            
                            TopManager.getDefault().notify(NDConfirm);
                            if (NDConfirm.getValue()==NDConfirm.YES_OPTION){
                                overwrite = true; 
                            }else{
                                overwrite = false;
                            }
                        }
                    }
                    if (overwrite){
                        element = storage.addElement(pp.getNamePrefix(), fsName,
                        pp.getClassLevel(), pp.getFieldLevel(), pp.getMethodLevel());
                    }
                } else { // cancel pressed
                    d.setVisible(false);
                    d.dispose();
                    d = null;
                    return false;
                }
            } while(!overwrite);

            progress.setDialogVisible(true);
        }

        final int classLevel = element.getClassLevel();
        final int fieldLevel = element.getFieldLevel();
        final int methodLevel = element.getMethodLevel();
        storage.checkProvider(element);

        try {
            
            if (isRoot){
                JCStorage.getStorage().addProcessedElement(element);
                JCStorage.getStorage().refresh();
            }
            
            // Update provider classes
            final String doneString = " " + getBundleString(BUNDLE_CLASSES_DONE); // NOI18N
            final String ofString = " " + getBundleString(BUNDLE_CLASSES_DONE_OF) + " "; // NOI18N
            
            // Build class list
            progress.invokeSetBuilding(getBundleString(BUNDLE_BUILDING_CLASS));
            progress.invokeSetDoneString(getBundleString(BUNDLE_INIT_PARSER));
            
            boolean ok = true;
            doneCnt = 0;
            
            while (ok && qe.hasMoreElements()) {
                Object ob = qe.nextElement();
                if (ob instanceof DataFolder){
                    DataFolder dfo = ((DataFolder)(ob));
                    DataObject[] children = dfo.getChildren();
                    progress.invokeSetCurrentClass(dfo.getPrimaryFile().getPackageName('.'));
                    doneCnt++;
                    progress.invokeSetDoneString(doneCnt + ofString + classCnt + doneString);
                    
                    for (int j = 0; j < children.length; j++) {
                        if ((progress!=null) && (progress.cancelled)) {
                            return false;
                        }
                        DataObject dob = children[j];
                        if (!(dob instanceof DataFolder)) {
                            SourceCookie sc = (SourceCookie)dob.getCookie(SourceCookie.class);
                            if (sc!=null){
                                if(!appendSource(sc, element, progress)){
                                    ok=false;
                                    break;
                                }
                            }
                        }
                    }
                }else if (ob instanceof SourceCookie){
                    SourceCookie sc = (SourceCookie)ob;
                    if(!appendSource(sc, element, progress)){
                        ok=false;
                        break;
                    }
                }
            }
            
            if (!ok) {
                return false;
            }
        }finally{
            JCStorage.getStorage().removeProcessedElement(element);
        }

        return true;
    }

    /** Update memory and parser database after save document event */
    boolean updateProviderOnBackground(final QueueEnumeration qe, final int classCnt,
    String fsName, boolean isRoot) {
        
        lookupCache = new HashMap(89);
        
        // Get provider
        JCStorage storage = JCStorage.getStorage();
        if (storage == null) return false;  // bugfix of 12661. storage can be null since bugfix of 12118 ...
        JCStorageElement element = storage.findFileSystemElement(fsName);
        if(element==null){
            // parser DB for this element doesn't exist
            return false;
        }
        
        final int classLevel = element.getClassLevel();
        final int fieldLevel = element.getFieldLevel();
        final int methodLevel = element.getMethodLevel();
        
        if(isRoot){
            element = storage.addElement(element.getName(), fsName,
            classLevel, fieldLevel, methodLevel); 
        }
        
        storage.checkProvider(element);
        
        // Update provider classes
        final String doneString = " " + getBundleString(BUNDLE_CLASSES_DONE); // NOI18N
        final String ofString = " " + getBundleString(BUNDLE_CLASSES_DONE_OF) + " "; // NOI18N
        
        boolean ok = true;
        doneCnt = 0;
        while (ok && qe.hasMoreElements()) {
            Object ob = qe.nextElement();
            if (ob instanceof DataFolder){
                DataObject[] children = ((DataFolder)(ob)).getChildren();
                doneCnt++;
                for (int j = 0; j < children.length; j++) {
                    DataObject dob = children[j];
                    if (!(dob instanceof DataFolder)) {
                        SourceCookie sc = (SourceCookie)dob.getCookie(SourceCookie.class);
                        if (sc!=null){
                            if(!appendSource(sc, element, null)){
                                ok=false;
                                break;
                            }
                        }
                    }
                }
            }else if (ob instanceof SourceCookie){
                SourceCookie sc = (SourceCookie)ob;
                if(!appendSource(sc, element, null)){
                    ok=false;
                    break;
                }
            }
        }

        if (!ok) {
            return false;
        }
        
        lookupCache.clear();
        lookupCache=null;
        return true;
    }

     
    String getBundleString(String s) {
        if (bundle == null) {
            bundle = NbBundle.getBundle(JCProviderPanel.class);
        }
        return bundle.getString(s);
    }


    class ProgressPanel extends JCProgressPanel
        implements ActionListener {

        private DialogDescriptor descriptor;

        Dialog dialog;

        boolean cancelled;

        static final long serialVersionUID =6306529202240892814L;
        ProgressPanel() {
        }

        DialogDescriptor getDescriptor() {
            if (descriptor == null) {
                descriptor = createDescriptor();
            }
            return descriptor;
        }

        DialogDescriptor createDescriptor() {
            return new DialogDescriptor(this,
                                        getBundleString("JC_progress_title"), // NOI18N
                                        false,
                                        new Object[] { DialogDescriptor.CANCEL_OPTION },
                                        DialogDescriptor.CANCEL_OPTION, DialogDescriptor.BOTTOM_ALIGN,
                                        null,
                                        this
                                       );
        }
        
        public void actionPerformed(ActionEvent evt){
        }

        void cancelPressed() {
            cancelled = true;
            setDialogVisible(false);
        }

        void setDialogVisible(final boolean visible) {
            SwingUtilities.invokeLater(
                new Runnable() {
                    public void run() {
                        if (visible && dialog == null) {
                            dialog = TopManager.getDefault().createDialog(getDescriptor());
                            dialog.setSize(300, 200);
                            descriptor.setButtonListener(new ActionListener(){
                                public void actionPerformed(ActionEvent evt){
                                    if (evt.getSource() == NotifyDescriptor.CANCEL_OPTION) {
                                        cancelPressed();
                                    }
                                }
                            }
                            );
                        }

                        if (dialog != null) {
                            dialog.setVisible(visible);
                        }
                    }
                }
            );
        }

        void destroy() {
            if (dialog != null) {
                dialog.setVisible(false);
                dialog.dispose();
                dialog = null;
            }
            descriptor = null;
        }
    }

}
