/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.editor.java;

import java.text.MessageFormat;
import javax.swing.ListCellRenderer;
import org.netbeans.editor.ext.java.JCClass;
import org.netbeans.editor.ext.java.JavaFastOpen;
import org.netbeans.modules.editor.NbEditorUtilities;
import org.openide.TopManager;
import org.openide.NotifyDescriptor;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.cookies.OpenCookie;
import org.openide.util.NbBundle;

/**
 * Open java source using Repository.
 *
 * @author Miloslav Metelka
 * @version 1.0
 */

public class NbJavaFastOpen extends JavaFastOpen {
    
    protected void openSource(Object item) {
        if (item instanceof JCClass) {
            JCClass cls = (JCClass)item;
            TopManager tm = NbEditorUtilities.getTopManager();
            if (tm != null) {
                FileObject fo = tm.getRepository().find(
                    cls.getPackageName(), cls.getName(), "java");

                if (fo != null) {
                    DataObject dob;
                    try {
                        dob = DataObject.find(fo);
                    } catch (DataObjectNotFoundException e) {
                        dob = null;
                    }

                    if (dob != null) {
                        OpenCookie oc = (OpenCookie)dob.getCookie(OpenCookie.class);
                        if (oc != null) {
                            oc.open();
                        }
                    }

                } else { // Class not found in filesystems
                    String msg = NbBundle.getBundle(JavaKit.class).getString(
                        "goto_source_source_not_found"); // NOI18N
                    msg = MessageFormat.format(msg, new Object [] { cls.getFullName() } );
                    tm.notify(new NotifyDescriptor.Message(msg));
                }
            }
        }
    }
    
    protected ListCellRenderer createCellRenderer() {
        NbJCCellRenderer rr = new NbJCCellRenderer();
        rr.setClassDisplayFullName(true);
        return rr;
    }
    
    
}
