/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.extbrowser;

import org.openide.options.SystemOption;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;

/** Options for something or other.
 *
 * @author  rkubacki
 */
public class ExtBrowserSettings extends SystemOption {

    // static final long serialVersionUID = ...;

    public static final String PROP_BROWSER = "prop_browser";
    public static final String PROP_START = "prop_startbrowser";
    public static final String PROP_CLOSE = "prop_closebrowser";

    /** DDE server names */
    public static final String NETSCAPE = "NETSCAPE";
    public static final String IEXPLORE = "IEXPLORE";

    // No constructor please!

    protected void initialize () {
        setBrowser (NETSCAPE);
        setStartWhenNotRunning (true);
        setCloseOnExit (true);
    }

    public String displayName () {
        return NbBundle.getMessage (ExtBrowserSettings.class, "LBL_settings");
    }

    public HelpCtx getHelpCtx () {
        return HelpCtx.DEFAULT_HELP;
        // If you provide context help then use:
        // return new HelpCtx (ExtBrowserSettings.class);
    }

    /** Default instance of this system option, for the convenience of associated classes. */
    static final ExtBrowserSettings OPTIONS = (ExtBrowserSettings) findObject (ExtBrowserSettings.class, true);

    public String getBrowser () {
        return (String) getProperty (PROP_BROWSER);
    }

    public void setBrowser (String Browser) {
        // Automatically fires property changes if needed etc.:
        putProperty (PROP_BROWSER, Browser, true);
        // If you need to start some service, or do something else with
        // an external effect, you should not use putProperty(...): keep
        // the data as a private static member, and manually modify that
        // variable and use firePropertyChange(...). Because if putProperty(...)
        // is used, getters and setters will be skipped during project save
        // and restore, which may cause problems.
    }

    public boolean isStartWhenNotRunning () {
        return ((Boolean) getProperty (PROP_START)).booleanValue ();
    }

    public void setStartWhenNotRunning (boolean s) {
        putProperty (PROP_START, new Boolean (s), true);
    }

    public boolean isCloseOnExit () {
        return ((Boolean) getProperty (PROP_CLOSE)).booleanValue ();
    }

    public void setCloseOnExit (boolean s) {
        putProperty (PROP_CLOSE, new Boolean (s), true);
    }

}
