/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.extbrowser;

import java.awt.Image;
import java.beans.*;

import org.openide.util.NbBundle;

/** Description of {@link ExtBrowserSettings}.
 *
 * @author Radim Kubacki
 */
public class ExtBrowserSettingsBeanInfo extends SimpleBeanInfo {

    public PropertyDescriptor[] getPropertyDescriptors () {
        try {
            PropertyDescriptor propone = new PropertyDescriptor ("browser", ExtBrowserSettings.class, "getBrowser", "setBrowser");    // NOI18N
            propone.setDisplayName (NbBundle.getMessage (ExtBrowserSettingsBeanInfo.class, "PROP_browser"));
            propone.setShortDescription (NbBundle.getMessage (ExtBrowserSettingsBeanInfo.class, "HINT_browser"));
            propone.setPropertyEditorClass (BrowserPropertyEditor.class);
            
            PropertyDescriptor proptwo = new PropertyDescriptor ("startWhenNotRunning", ExtBrowserSettings.class);    // NOI18N
            proptwo.setDisplayName (NbBundle.getMessage (ExtBrowserSettingsBeanInfo.class, "PROP_start"));
            proptwo.setShortDescription (NbBundle.getMessage (ExtBrowserSettingsBeanInfo.class, "HINT_start"));
            proptwo.setPropertyEditorClass (ProptwoEd.class);
            
            PropertyDescriptor prop3 = new PropertyDescriptor ("closeOnExit", ExtBrowserSettings.class);    // NOI18N
            prop3.setDisplayName (NbBundle.getMessage (ExtBrowserSettingsBeanInfo.class, "PROP_close"));
            prop3.setShortDescription (NbBundle.getMessage (ExtBrowserSettingsBeanInfo.class, "HINT_close"));
            // proptwo.setExpert (true);
            proptwo.setPropertyEditorClass (ProptwoEd.class);
            return new PropertyDescriptor[] { propone, proptwo, prop3 };
        } catch (IntrospectionException ie) {
            if (Boolean.getBoolean ("netbeans.debug.exceptions"))
                ie.printStackTrace ();
            return null;
        }
    }

    private static Image icon, icon32;
    public Image getIcon (int type) {
        if (type == BeanInfo.ICON_COLOR_16x16 || type == BeanInfo.ICON_MONO_16x16) {
            if (icon == null)
                icon = loadImage ("ExtBrowserSettingsIcon.gif");
            return icon;
        } else {
            if (icon32 == null)
                icon32 = loadImage ("ExtBrowserSettingsIcon32.gif");
            return icon32;
        }
    }

    public static class ProptwoEd extends PropertyEditorSupport {

        private static final String[] tags = {
            NbBundle.getMessage (ExtBrowserSettingsBeanInfo.class, "LBL_true"),
            NbBundle.getMessage (ExtBrowserSettingsBeanInfo.class, "LBL_false")
	};

        public String[] getTags () {
            return tags;
        }

        public String getAsText () {
            return tags[((Boolean) getValue ()).booleanValue () ? 0 : 1];
        }

        public void setAsText (String text) throws IllegalArgumentException {
            if (tags[0].equals (text))
                setValue (Boolean.TRUE);
            else if (tags[1].equals (text))
                setValue (Boolean.FALSE);
            else
                throw new IllegalArgumentException ();
        }

    }

}
