/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.extbrowser;

import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.util.Properties;
import javax.swing.*;

import org.openide.TopManager;
import org.openide.awt.*;
import org.openide.cookies.InstanceCookie;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObject;
import org.openide.modules.*;
import org.openide.util.Utilities;

/**
 * Factory for external browser.
 *
 * It uses native interface to make DDE communication on Windows platform.
 * On UNIX there are some command line paramenters
 *
 * @author Radim Kubacki
 */
public class ExtHtmlBrowser extends ModuleInstall {

    static final long serialVersionUID = -2250739874462641437L;
    
    /** 
     * Determines order of platform dependent browsers.
     *
     * @return true if Unix is current platform and Unix browser should be before 
     *              the Windows browser
     *         false otherwise
     */
    public static Boolean isUnixFirst () {
        return new Boolean (Utilities.isUnix ());
    }
    
    /** Module installed for the first time. */
    public void installed () {
        // updateBrowsers (true);
    }

    /** Module installed again. */
    public void restored () {
        // updateBrowsers (true);
    }

    /**
     * Called when the module is uninstalled (from a running IDE).
     * Should remove whatever functionality from the IDE that it had registered.
     */
    public void uninstalled () {
        // updateBrowsers (false);
    }
    
    /** 
     * 
     */
    private static void updateBrowsers (boolean installed) {
        try {
            FileObject fo = TopManager.getDefault ().getRepository ()
                .getDefaultFileSystem ().findResource ("Services/Browsers");   // NOI18N
            DataFolder folder = DataFolder.findFolder (fo);
            DataObject [] dobjs = folder.getChildren ();
            System.out.println("updateBrowsers ()"+Utilities.isUnix ()+", "+Utilities.isWindows ());
            for (int i = 0; i<dobjs.length; i++) {
                Object o = ((InstanceCookie)dobjs[i].getCookie (InstanceCookie.class)).instanceCreate ();
                if (o != null) {
                    System.out.print("Browser "+o.toString ());
                    if (o instanceof WinWebBrowser && !Utilities.isWindows ()) 
                        System.out.println("1");
                    if (o instanceof WinWebBrowser) 
                        System.out.println("2");
                    if ((o instanceof UnixWebBrowser && !Utilities.isUnix ())
                    ||  (o instanceof WinWebBrowser && !Utilities.isWindows ())
                    ||  !installed) {
                        System.out.println(" hide");
                        dobjs[i].getPrimaryFile ().setAttribute ("hidden", Boolean.TRUE);
                    }
                    else {
                        System.out.println(" show");
                        dobjs[i].getPrimaryFile ().setAttribute ("hidden", Boolean.FALSE);
                    }
                }
            }
        }
        catch (Exception ex) {
            TopManager.getDefault().notifyException(ex);
        }
        
    }
    
    // debug ..................................................................
    
    public static void main (String [] arg) {
        // installBrowser ();
        
        HtmlBrowser htmlViewer = new HtmlBrowser ();
        JDialog w = new JDialog ();
        w.addWindowListener (new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.out.println("Got windowClosing");
            }
            
            public void windowClosed(WindowEvent e) {
                System.out.println ("Got windowClosed");
                System.exit (0);
            }
        });
        w.getContentPane().setLayout(new BorderLayout ());
        w.getContentPane ().add (htmlViewer, "Center"); // NOI18N
        htmlViewer.setVisible(true);
        w.setName ("External Web Browser");
        w.pack ();
        w.show ();
        htmlViewer.setURL ("http://www.netbeans.org");

        return;
    }
    
}
