/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.extbrowser;

import java.io.ObjectStreamException;

import org.openide.TopManager;
import org.openide.NotifyDescriptor;
import org.openide.awt.HtmlBrowser;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;
/**
 * Browser factory for UNIX like browser.
 *
 * @author  Radim Kubacki
 */
public class UnixWebBrowser extends ExtWebBrowser {

    private static final long serialVersionUID = -7317179197254112564L;
    
    private static final String DEFAULT_EXECUTABLE = "netscape";    // NOI18N
    
    /** 
     * Determines if this browser is hidden to user.
     *
     * @return false on Unix system otherwise false
     */
    public static Boolean isHidden () {
        return new Boolean (!org.openide.util.Utilities.isUnix ());
    }
    
    /** Creates new UnixWebBrowser */
    public UnixWebBrowser () {
    }

    /**
     * Returns a new instance of BrowserImpl implementation.
     */
    public HtmlBrowser.Impl createHtmlBrowserImpl() {
        ExtBrowserImpl impl = null;

        if (org.openide.util.Utilities.isUnix ())
            impl = new UnixBrowserImpl (this);
        else 
            throw new UnsupportedOperationException (NbBundle.getMessage (WinWebBrowser.class, "MSG_CannotUseBrowser"));
        
        return impl;
    }
    
    public String getExecutable () {
        String retValue = super.getExecutable ();
        if (retValue == null)
            return DEFAULT_EXECUTABLE;
        return retValue;
    }
    
    /*
    private Object readResolve () 
    throws ObjectStreamException {
        System.out.println("resolving UnixWebBrowser");
        if (!org.openide.util.Utilities.isUnix () || Boolean.getBoolean ("abc"))
            return null;
        
        return this;
    }
     */
}
