/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.extbrowser;

import java.io.ObjectStreamException;

import org.openide.TopManager;
import org.openide.NotifyDescriptor;
import org.openide.awt.HtmlBrowser;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;
/**
 * Browser factory for UNIX like browser.
 *
 * @author  Radim Kubacki
 */
public class WinWebBrowser extends ExtWebBrowser {

    private static final long serialVersionUID = -7317179197254112564L;
    
    /** DDE server proerty name */
    public static final String PROP_DDESERVER = "DDE_SERVER";   // NOI18N
    
    public static Boolean isHidden () {
        return new Boolean (!org.openide.util.Utilities.isWindows ());
    }
    
    /** storage for DDE server property */
    private String ddeServer;
    
    /** Creates new WinWebBrowser */
    public WinWebBrowser () {
    }

    /**
     * Returns a new instance of BrowserImpl implementation.
     */
    public HtmlBrowser.Impl createHtmlBrowserImpl() {
        ExtBrowserImpl impl = null;

        if (org.openide.util.Utilities.isWindows ())
            impl = new NbDdeBrowserImpl (this);
        else 
            throw new UnsupportedOperationException (NbBundle.getMessage (WinWebBrowser.class, "MSG_CannotUseBrowser"));
        
        return impl;
    }
    
    /**
     * Gets DDE server name
     */
    public String getDDEServer () {
        /*
        if (ddeServer == null) {
            ExtBrowserSettings settings = (ExtBrowserSettings)ExtBrowserSettings.findObject (ExtBrowserSettings.class);
            ddeServer = settings.getBrowser ();
        }
         */
        return ddeServer;
    }
    
    /** 
     * Sets DDE server name
     */
    public void setDDEServer (String ddeServer) {
        if (((ddeServer == null) && (this.ddeServer != null))
        ||  !ddeServer.equals (this.ddeServer)) {
            String old = this.ddeServer;
            this.ddeServer = ddeServer;
            pcs.firePropertyChange (PROP_DDESERVER, old, ddeServer);
        }
    }
    
    /*
    private Object readResolve () 
    throws ObjectStreamException {
        System.out.println("resolving UnixWebBrowser");
        if (!org.openide.util.Utilities.isUnix () || Boolean.getBoolean ("abc"))
            return null;
        
        return this;
    }
     */
}
