/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.form;

import java.beans.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import org.openide.*;
import org.openide.util.HelpCtx;
import org.openide.util.Utilities;


/** The first panel of connection wizard - for selecting the activation event
 * on the source component and handler for the event (where the connection code
 * will be generated).
 *
 * @author  Ian Formanek, Tomas Pavek
 */
public class ConnectionPanel1 extends javax.swing.JPanel implements WizardDescriptor.Panel
{
    private Event selectedEvent = null;

    private RADComponent sourceComponent;

    private EventListenerList listenerList = null;

    /** Creates new form ConnectionPanel1 */
    public ConnectionPanel1(RADComponent source) {
        sourceComponent = source;

        initComponents();

        setName(FormEditor.getFormBundle().getString("CTL_CW_Step1_Title"));
        sourceComponentName.setText(source.getName());

        eventNameCombo.setEnabled(selectedEvent != null);

        eventNameCombo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent e) {
                fireStateChanged();
            }
        });

        eventNameCombo.getEditor().addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent e) {
                fireStateChanged();
            }
        });

         // populate event tree
        final Vector eventNodes = new Vector();
        TreeNode rootNode = new TreeNode() {
            public TreeNode getChildAt(int childIndex) {
                return(TreeNode) eventNodes.elementAt(childIndex);
            }
            public int getChildCount() {
                return eventNodes.size();
            }
            public TreeNode getParent() {
                return null;
            }
            public int getIndex(TreeNode node) {
                return eventNodes.indexOf(node);
            }
            public boolean getAllowsChildren() {
                return true;
            }
            public boolean isLeaf() {
                return false;
            }
            public Enumeration children() {
                return eventNodes.elements();
            }
        };

        EventSet[] setHandlers = source.getEventHandlers().getEventSets();
        for (int i = 0; i < setHandlers.length; i++) {
            Event[] events = setHandlers[i].getEvents();
            Vector eventsVector = new Vector();
            EventSetNode esn = new EventSetNode(rootNode, setHandlers[i], eventsVector);
            for (int j = 0; j < events.length; j++) {
                eventsVector.addElement(new EventNode(esn, events[j]));
            }
            eventNodes.addElement(esn);
        }

        DefaultTreeSelectionModel treeSelectionModel = new DefaultTreeSelectionModel();
        treeSelectionModel.addTreeSelectionListener(new TreeSelectionListener() {
            public void valueChanged(TreeSelectionEvent evt) {
                TreePath[] paths = eventSelectTree.getSelectionPaths();
                if ((paths != null) &&(paths.length == 1)) {
                    TreeNode node =(TreeNode) paths[0].getLastPathComponent();
                    if ((node != null) &&(node instanceof EventNode)) {
                        setSelectedEvent(((EventNode)node).getEvent());
                        return;
                    }
                }
                setSelectedEvent(null);
            }
        });

        treeSelectionModel.setSelectionMode(DefaultTreeSelectionModel.SINGLE_TREE_SELECTION);
        eventSelectTree.setModel(new DefaultTreeModel(rootNode));
        eventSelectTree.setSelectionModel(treeSelectionModel);

        // localization code
        insidePanel.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(8, 4, 0, 4)));
        sourcePanel.setBorder(new javax.swing.border.CompoundBorder(
            new javax.swing.border.EmptyBorder(new java.awt.Insets(0, 0, 8, 0)),
            new javax.swing.border.CompoundBorder(
                new javax.swing.border.TitledBorder(
                    new javax.swing.border.EtchedBorder(), FormEditor.getFormBundle().getString("CTL_CW_ConnectionSource")),
                new javax.swing.border.EmptyBorder(new java.awt.Insets(5, 5, 5, 5)))));
        sourceNameLabel.setText(FormEditor.getFormBundle().getString("CTL_CW_SourceComponent"));
        eventSelectLabel.setText(FormEditor.getFormBundle().getString("CTL_CW_Event"));
        eventHandlerPanel.setBorder(new javax.swing.border.CompoundBorder(
            new javax.swing.border.EmptyBorder(new java.awt.Insets(0, 0, 4, 0)),
            new javax.swing.border.CompoundBorder(
                new javax.swing.border.TitledBorder(
                    new javax.swing.border.EtchedBorder(), FormEditor.getFormBundle().getString("CTL_CW_EventHandlerMethod")),
                new javax.swing.border.EmptyBorder(new java.awt.Insets(5, 5, 5, 5)))));
        eventNameLabel.setText(FormEditor.getFormBundle().getString("CTL_CW_MethodName"));
        
        putClientProperty("WizardPanel_contentSelectedIndex", new Integer(0)); // NOI18N
    }

    public java.awt.Dimension getPreferredSize() {
        return new java.awt.Dimension(450, 300);
    }

    String getEventName() {
        return (String) eventNameCombo.getEditor().getItem();
    }

    Event getSelectedEvent() {
        return selectedEvent;
    }

    boolean handlerAlreadyExists() {
        String handlerName = (String) eventNameCombo.getEditor().getItem();

        if (!"".equals(handlerName) && selectedEvent != null && selectedEvent.getHandlers().size() > 0) {
            for (java.util.Iterator it = selectedEvent.getHandlers().iterator(); it.hasNext(); ) {
                EventHandler eh = (EventHandler) it.next();
                if (eh.getName().equals(handlerName))
                    return true;
            }
        }

        return false;
    }

    private void setSelectedEvent(Event event) {
        selectedEvent = event;
        eventNameCombo.removeAllItems();

        if (selectedEvent != null) {
            eventNameCombo.setEnabled(true);

            if (selectedEvent.getHandlers().size() > 0) {
                for (java.util.Iterator it = selectedEvent.getHandlers().iterator(); it.hasNext(); ) {
                    EventHandler eh = (EventHandler) it.next();
                    eventNameCombo.addItem(eh.getName());
                }
            }
            else {
                eventNameCombo.addItem(FormUtils.getDefaultEventName(sourceComponent, selectedEvent.getListenerMethod()));
            }
            eventNameCombo.setSelectedIndex(0);
        }
        else eventNameCombo.setEnabled(false);
        fireStateChanged();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        insidePanel = new javax.swing.JPanel();
        sourcePanel = new javax.swing.JPanel();
        sourceInfoPanel = new javax.swing.JPanel();
        sourceNamePanel = new javax.swing.JPanel();
        sourceNameLabel = new javax.swing.JLabel();
        sourceComponentName = new javax.swing.JLabel();
        eventSelectLabelPanel = new javax.swing.JPanel();
        eventSelectLabel = new javax.swing.JLabel();
        eventSelectScroll = new javax.swing.JScrollPane();
        eventSelectTree = new javax.swing.JTree();
        eventHandlerPanel = new javax.swing.JPanel();
        eventNameLabel = new javax.swing.JLabel();
        eventNameCombo = new javax.swing.JComboBox();
        setLayout(new java.awt.BorderLayout());
        
        insidePanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        sourcePanel.setLayout(new java.awt.BorderLayout());
        
        sourceInfoPanel.setLayout(new java.awt.GridLayout(2, 1));
        
        sourceNamePanel.setLayout(new java.awt.FlowLayout(0, 0, 0));
        
        sourceNameLabel.setText(FormEditor.getFormBundle ().getString ("CTL_CW_SourceComponent"));
        sourceNameLabel.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(0, 0, 0, 6)));
        sourceNamePanel.add(sourceNameLabel);
        
        
        sourceNamePanel.add(sourceComponentName);
        
        sourceInfoPanel.add(sourceNamePanel);
        
        
        eventSelectLabelPanel.setLayout(new java.awt.FlowLayout(0, 0, 3));
        
        eventSelectLabel.setText(FormEditor.getFormBundle ().getString ("CTL_CW_Event"));
        eventSelectLabelPanel.add(eventSelectLabel);
        
        sourceInfoPanel.add(eventSelectLabelPanel);
        
        sourcePanel.add(sourceInfoPanel, java.awt.BorderLayout.NORTH);
        
        
        eventSelectScroll.setMaximumSize(new java.awt.Dimension(32767, 100));
        
        eventSelectTree.setShowsRootHandles(true);
        eventSelectTree.setRootVisible(false);
        eventSelectScroll.setViewportView(eventSelectTree);
        
        sourcePanel.add(eventSelectScroll, java.awt.BorderLayout.CENTER);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridwidth = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 0.5;
        insidePanel.add(sourcePanel, gridBagConstraints1);
        
        
        eventHandlerPanel.setLayout(new java.awt.BorderLayout(8, 0));
        
        eventNameLabel.setText(FormEditor.getFormBundle ().getString ("CTL_CW_MethodName"));
        eventHandlerPanel.add(eventNameLabel, java.awt.BorderLayout.WEST);
        
        
        eventNameCombo.setEditable(true);
        eventHandlerPanel.add(eventNameCombo, java.awt.BorderLayout.CENTER);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.weightx = 1.0;
        insidePanel.add(eventHandlerPanel, gridBagConstraints1);
        
        
        add(insidePanel, java.awt.BorderLayout.CENTER);
        
    }//GEN-END:initComponents

    // ------------------------------------------------------------------------
    // WizardDescriptor.Panel implementation
    
    public java.awt.Component getComponent() {
        return this;
    }

    public HelpCtx getHelp() {
        return new HelpCtx("gui.connecting.source"); // NOI18N
    }

    public boolean isValid() {
        String eventName = getEventName();
        return selectedEvent != null
               && eventName != null && !"".equals(eventName) 
               && Utilities.isJavaIdentifier(eventName);
    }

    public void readSettings(java.lang.Object settings) {
    }

    public void storeSettings(java.lang.Object settings) {
    }

    public void addChangeListener(ChangeListener listener) {
        if (listenerList == null ) {
            listenerList = new EventListenerList();
        }
        listenerList.add(ChangeListener.class, listener);
    }

    public void removeChangeListener(ChangeListener listener) {
        if (listenerList != null) {
            listenerList.remove(ChangeListener.class, listener);
        }
    }

    // ------------------------------------------------------------------------

    private void fireStateChanged() {
        if (listenerList == null) return;

        ChangeEvent e = null;
        Object[] listeners = listenerList.getListenerList();
        for (int i = listeners.length-2; i>=0; i-=2) {
            if (listeners[i] == ChangeListener.class) {
                if (e == null)
                    e = new ChangeEvent(this);
                ((ChangeListener)listeners[i+1]).stateChanged(e);
            }
        }
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel insidePanel;
    private javax.swing.JPanel sourcePanel;
    private javax.swing.JPanel sourceInfoPanel;
    private javax.swing.JPanel sourceNamePanel;
    private javax.swing.JLabel sourceNameLabel;
    private javax.swing.JLabel sourceComponentName;
    private javax.swing.JPanel eventSelectLabelPanel;
    private javax.swing.JLabel eventSelectLabel;
    private javax.swing.JScrollPane eventSelectScroll;
    private javax.swing.JTree eventSelectTree;
    private javax.swing.JPanel eventHandlerPanel;
    private javax.swing.JLabel eventNameLabel;
    private javax.swing.JComboBox eventNameCombo;
    // End of variables declaration//GEN-END:variables

    // ----------------------------------------------------------------------------
    // Innerclasses

    class EventSetNode implements TreeNode {
        private TreeNode parent;
        private EventSet eventSet;
        private Vector subNodes;

        public EventSetNode(TreeNode parent, EventSet eventSet, Vector subNodes) {
            this.parent = parent;
            this.eventSet = eventSet;
            this.subNodes = subNodes;
        }

        public TreeNode getChildAt(int childIndex) {
            return(TreeNode) subNodes.elementAt(childIndex);
        }
        public int getChildCount() {
            return subNodes.size();
        }
        public TreeNode getParent() {
            return null;
        }
        public int getIndex(TreeNode node) {
            return subNodes.indexOf(node);
        }
        public boolean getAllowsChildren() {
            return true;
        }
        public boolean isLeaf() {
            return false;
        }
        public Enumeration children() {
            return subNodes.elements();
        }
        public String toString() {
            return eventSet.getName();
        }
    }

    class EventNode implements TreeNode {
        private TreeNode parent;
        private Event event;
        public EventNode(TreeNode parent, Event event) {
            this.parent = parent;
            this.event = event;
        }
        public TreeNode getChildAt(int childIndex) {
            return null;
        }
        public int getChildCount() {
            return 0;
        }
        public TreeNode getParent() {
            return parent;
        }
        public int getIndex(TreeNode node) {
            return -1;
        }
        public boolean getAllowsChildren() {
            return false;
        }
        public boolean isLeaf() {
            return true;
        }
        public Enumeration children() {
            return null;
        }
        public String toString() {
            if (event.getHandlers().size() == 0)
                return event.getName();
            if (event.getHandlers().size() == 1)
                return event.getName() + " ["+((EventHandler) event.getHandlers().get(0)).getName()+"]"; // NOI18N
            return event.getName() + " [...]"; // NOI18N
        }
        Event getEvent() {
            return event;
        }
    }
}
