/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.form;

import java.beans.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;

import org.openide.*;
import org.openide.util.HelpCtx;
import org.openide.util.Utilities;


/** The second panel of connection wizard - for selecting what to perform on
 * the target component (set a property, call a method or execute some user code).
 *
 * @author  Ian Formanek, Tomas Pavek
 */
public class ConnectionPanel2 extends javax.swing.JPanel implements WizardDescriptor.Panel
{
    public static final int METHOD_TYPE = 0;
    public static final int PROPERTY_TYPE = 1;
    public static final int CODE_TYPE = 2;

    private Object[] propertyListData;
    private Object[] methodListData;
    private MethodDescriptor[] methodDescriptors;
    private PropertyDescriptor[] propDescriptors;

    private RADComponent targetComponent;

    private EventListenerList listenerList = null;

    /** Creates new form ConnectionPanel2 */
    public ConnectionPanel2(RADComponent target) {
        targetComponent = target;

        initComponents();

        setName(FormEditor.getFormBundle().getString("CTL_CW_Step2_Title"));

        javax.swing.ButtonGroup gr = new javax.swing.ButtonGroup();
        gr.add(propertyButton);
        gr.add(methodButton);
        gr.add(codeButton);

        targetComponentName.setText(target.getName());

        actionList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        actionList.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent evt) {
                if (!evt.getValueIsAdjusting())
                    fireStateChanged();
            }
        });

        updateActionList();

        // localization code
        targetPanel.setBorder(new javax.swing.border.CompoundBorder(
            new javax.swing.border.TitledBorder(
                new javax.swing.border.EtchedBorder(), FormEditor.getFormBundle().getString("CTL_CW_ConnectionTarget")), // "Operation on Target"
            new javax.swing.border.EmptyBorder(new java.awt.Insets(5, 5, 5, 5))));
        targetNameLabel.setText(FormEditor.getFormBundle().getString("CTL_CW_TargetComponent"));                          // "Target Component:"
        propertyButton.setText(FormEditor.getFormBundle().getString("CTL_CW_SetProperty"));                               // "Set Property"
        methodButton.setText(FormEditor.getFormBundle().getString("CTL_CW_MethodCall"));                                  // "Method Call"
        codeButton.setText(FormEditor.getFormBundle().getString("CTL_CW_UserCode"));                                      // "User Code"

        insidePanel.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(8, 4, 4, 4)));
        targetNamePanel.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(0, 0, 5, 0)));

        putClientProperty("WizardPanel_contentSelectedIndex", new Integer(1)); // NOI18N
    }

    public java.awt.Dimension getPreferredSize() {
        return new java.awt.Dimension(450, 300);
    }


    int getActionType() {
        if (methodButton.isSelected())
            return METHOD_TYPE;
        else if (propertyButton.isSelected())
            return PROPERTY_TYPE;
        else
            return CODE_TYPE;
    }

    MethodDescriptor getSelectedMethod() {
        if (!methodButton.isSelected()) return null;
        if (actionList.getSelectedIndex() == -1) return null;
        return methodDescriptors [actionList.getSelectedIndex()];
    }

    PropertyDescriptor getSelectedProperty() {
        if (!propertyButton.isSelected()) return null;
        if (actionList.getSelectedIndex() == -1) return null;
        return propDescriptors [actionList.getSelectedIndex()];
    }

    private void updateActionList() {
        if (codeButton.isSelected()) {
            actionList.setListData(new String [] {
                FormEditor.getFormBundle().getString("CTL_CW_UserCodeText1"),    // "After clicking Finish, go to the editor to enter the",
                FormEditor.getFormBundle().getString("CTL_CW_UserCodeText2") }); // "custom code for handling the event.";
            actionList.setEnabled(false);
        } 
        else if (propertyButton.isSelected()) {
            // properties list
            actionList.setEnabled(true);
            if (propertyListData == null) {
                BeanInfo targetBeanInfo = targetComponent.getBeanInfo();
                PropertyDescriptor[] descs = targetBeanInfo.getPropertyDescriptors();

                // filter out read-only properties // [FUTURE: provide also indexed properties]
                ArrayList list = new ArrayList();
                for (int i = 0; i < descs.length; i++) {
                    if (descs[i].getWriteMethod() != null) {
                        list.add(descs[i]);
                    }
                }

                // sort the properties by name
                Collections.sort(list, new Comparator() {
                    public int compare(Object o1, Object o2) {
                        return((PropertyDescriptor)o1).getName().compareTo(((PropertyDescriptor)o2).getName());
                    }
                });

                propDescriptors = new PropertyDescriptor [list.size()];
                list.toArray(propDescriptors);

                propertyListData = new String [propDescriptors.length];
                for (int i = 0; i < propDescriptors.length; i++) {
                    propertyListData [i] = propDescriptors [i].getName();
                }
            }
            actionList.setListData(propertyListData);
        } 
        else {
            // methods list
            actionList.setEnabled(true);
            if (methodListData == null) {
                BeanInfo targetBeanInfo = targetComponent.getBeanInfo();
                methodDescriptors = targetBeanInfo.getMethodDescriptors();
                ArrayList list = new ArrayList();
                for (int i = 0; i < methodDescriptors.length; i++) {
                    list.add(methodDescriptors[i]);
                }

                // sort the methods by name
                Collections.sort(list, new Comparator() {
                    public int compare(Object o1, Object o2) {
                        return((MethodDescriptor)o1).getName().compareTo(((MethodDescriptor)o2).getName());
                    }
                });

                // copy it back to the array as it is used later
                list.toArray(methodDescriptors);

                methodListData = new String [list.size()];
                int i = 0;
                for (Iterator it = list.iterator(); it.hasNext();) {
                    methodListData [i++] = getMethodName((MethodDescriptor)it.next());
                }
            }
            actionList.setListData(methodListData);
        }
        actionList.revalidate();
        actionList.repaint();
    }

    private static String getMethodName(MethodDescriptor desc) {
        StringBuffer sb = new StringBuffer(desc.getName());
        Class[] params = desc.getMethod().getParameterTypes();
        if ((params == null) ||(params.length == 0)) {
            sb.append("()"); // NOI18N
        } else {
            for (int i = 0; i < params.length; i++) {
                if (i == 0) sb.append("("); // NOI18N
                else sb.append(", "); // NOI18N
                sb.append(Utilities.getShortClassName(params[i]));
            }
            sb.append(")"); // NOI18N
        }

        return sb.toString();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        insidePanel = new javax.swing.JPanel();
        targetPanel = new javax.swing.JPanel();
        targerInfoPanel = new javax.swing.JPanel();
        targetNamePanel = new javax.swing.JPanel();
        targetNameLabel = new javax.swing.JLabel();
        targetComponentName = new javax.swing.JLabel();
        actionTypePanel = new javax.swing.JPanel();
        propertyButton = new javax.swing.JRadioButton();
        methodButton = new javax.swing.JRadioButton();
        codeButton = new javax.swing.JRadioButton();
        actionPanel = new javax.swing.JScrollPane();
        actionList = new javax.swing.JList();
        setLayout(new java.awt.BorderLayout());
        
        insidePanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        targetPanel.setLayout(new java.awt.BorderLayout());
          
          targerInfoPanel.setLayout(new java.awt.GridLayout(2, 1));
            
            targetNamePanel.setLayout(new java.awt.FlowLayout(0, 0, 0));
              
              targetNameLabel.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(0, 0, 0, 6)));
                targetNameLabel.setText(FormEditor.getFormBundle ().getString ("CTL_CW_TargetComponent"));
                targetNamePanel.add(targetNameLabel);
                
                
              targetNamePanel.add(targetComponentName);
                
                targerInfoPanel.add(targetNamePanel);
              
              
            actionTypePanel.setLayout(new java.awt.FlowLayout(0, 0, 0));
              
              propertyButton.setSelected(true);
                propertyButton.setText(FormEditor.getFormBundle ().getString ("CTL_CW_SetProperty"));
                propertyButton.setMargin(new java.awt.Insets(2, 2, 2, 10));
                propertyButton.addActionListener(new java.awt.event.ActionListener() {
                    public void actionPerformed(java.awt.event.ActionEvent evt) {
                        actionTypeButtonPressed(evt);
                    }
                }
                );
                actionTypePanel.add(propertyButton);
                
                
              methodButton.setText(FormEditor.getFormBundle ().getString ("CTL_CW_MethodCall"));
                methodButton.setMargin(new java.awt.Insets(2, 2, 2, 10));
                methodButton.addActionListener(new java.awt.event.ActionListener() {
                    public void actionPerformed(java.awt.event.ActionEvent evt) {
                        actionTypeButtonPressed(evt);
                    }
                }
                );
                actionTypePanel.add(methodButton);
                
                
              codeButton.setText(FormEditor.getFormBundle ().getString ("CTL_CW_UserCode"));
                codeButton.addActionListener(new java.awt.event.ActionListener() {
                    public void actionPerformed(java.awt.event.ActionEvent evt) {
                        actionTypeButtonPressed(evt);
                    }
                }
                );
                actionTypePanel.add(codeButton);
                
                targerInfoPanel.add(actionTypePanel);
              
              targetPanel.add(targerInfoPanel, java.awt.BorderLayout.NORTH);
            
            
          
            actionPanel.setViewportView(actionList);
              
              targetPanel.add(actionPanel, java.awt.BorderLayout.CENTER);
            
            gridBagConstraints1 = new java.awt.GridBagConstraints();
          gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
          gridBagConstraints1.weightx = 1.0;
          gridBagConstraints1.weighty = 0.5;
          insidePanel.add(targetPanel, gridBagConstraints1);
          
          
        add(insidePanel, java.awt.BorderLayout.CENTER);
        
    }//GEN-END:initComponents

    private void actionTypeButtonPressed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_actionTypeButtonPressed
        updateActionList();
        fireStateChanged();
    }//GEN-LAST:event_actionTypeButtonPressed

    // ------------------------------------------------------------------------
    // WizardDescriptor.Panel implementation
    
    public java.awt.Component getComponent() {
        return this;
    }

    public HelpCtx getHelp() {
        return new HelpCtx("gui.connecting.target"); // NOI18N
    }

    public boolean isValid() {
        if (getActionType() == METHOD_TYPE) {
            if (!methodButton.isSelected() || actionList.getSelectedIndex() == -1) return false;
        }
        else if (getActionType() == PROPERTY_TYPE) {
            if (!propertyButton.isSelected() || actionList.getSelectedIndex() == -1) return false;
        }

        return true;
    }

    public void readSettings(java.lang.Object settings) {
    }

    public void storeSettings(java.lang.Object settings) {
    }

    public void addChangeListener(ChangeListener listener) {
        if (listenerList == null ) {
            listenerList = new EventListenerList();
        }
        listenerList.add(ChangeListener.class, listener);
    }

    public void removeChangeListener(ChangeListener listener) {
        if (listenerList != null) {
            listenerList.remove(ChangeListener.class, listener);
        }
    }

    // ------------------------------------------------------------------------

    private void fireStateChanged() {
        if (listenerList == null) return;

        ChangeEvent e = null;
        Object[] listeners = listenerList.getListenerList();
        for (int i = listeners.length-2; i>=0; i-=2) {
            if (listeners[i] == ChangeListener.class) {
                if (e == null)
                    e = new ChangeEvent(this);
                ((ChangeListener)listeners[i+1]).stateChanged(e);
            }
        }
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel insidePanel;
    private javax.swing.JPanel targetPanel;
    private javax.swing.JPanel targerInfoPanel;
    private javax.swing.JPanel targetNamePanel;
    private javax.swing.JLabel targetNameLabel;
    private javax.swing.JLabel targetComponentName;
    private javax.swing.JPanel actionTypePanel;
    private javax.swing.JRadioButton propertyButton;
    private javax.swing.JRadioButton methodButton;
    private javax.swing.JRadioButton codeButton;
    private javax.swing.JScrollPane actionPanel;
    private javax.swing.JList actionList;
    // End of variables declaration//GEN-END:variables
}
