/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.form;

import javax.swing.*;
import javax.swing.event.*;
import java.lang.reflect.Method;

import org.openide.*;
import org.openide.util.HelpCtx;
import org.openide.util.Utilities;


/** The third panel of connection wizard - for entering parameters for method
 * which will be called on the target component (specified in step 2).
 *
 * @author  Ian Formanek, Tomas Pavek
 */
public class ConnectionPanel3 extends javax.swing.JPanel implements WizardDescriptor.Panel
{
    private Class[] parameters;
    private ParametersPicker[] pickers;
    private FormModel formModel;
    private RADComponent sourceComponent;
    private boolean valid = false;

    private ChangeListener paramsChangeListener = null;

    private EventListenerList listenerList = null;

    /** Creates new form ConnectionPanel3 */
    public ConnectionPanel3(FormModel model, RADComponent source) {
        formModel = model;
        sourceComponent = source;

        initComponents ();
        
        setName(FormEditor.getFormBundle().getString("CTL_CW_Step3_Title"));

        paramsChangeListener = new ChangeListener() {
            public void stateChanged(ChangeEvent evt) {
                updatePreview();
            }
        };

        previewLabel.setText(FormEditor.getFormBundle().getString("CTL_CW_Preview")); // "Preview"
        previewLabel.setBorder(new javax.swing.border.CompoundBorder(
            new javax.swing.border.TitledBorder(
                new javax.swing.border.EtchedBorder(), " " + FormEditor.getFormBundle().getString("CTL_CW_GeneratedPreview") +" "), // "Generated Parameters Preview:"
            new javax.swing.border.EmptyBorder(new java.awt.Insets(5, 5, 5, 5))));

        insidePanel.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(8, 4, 4, 4)));

        putClientProperty("WizardPanel_contentSelectedIndex", new Integer(2)); // NOI18N
    }

    public java.awt.Dimension getPreferredSize() {
        return new java.awt.Dimension(450, 300);
    }

    void setMethod(Method m) {
        parameterTabs.removeChangeListener(paramsChangeListener);
        parameterTabs.removeAll();

        parameters = m.getParameterTypes();
        pickers = new ParametersPicker[parameters.length];
        for (int i = 0; i < parameters.length; i++) {
            pickers[i] = new ParametersPicker(formModel, /*sourceComponent,*/ parameters[i]);
            pickers[i].addChangeListener(new ChangeListener() {
                public void stateChanged(ChangeEvent evt) {
                    updatePreview();
                }
            });

            parameterTabs.addTab(Utilities.getShortClassName(parameters[i]),
                                 null, pickers[i], parameters[i].getName());
        }

        valid = isValid();
        parameterTabs.addChangeListener(paramsChangeListener);
        updatePreview();
    }

    String getParametersText() {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < pickers.length; i++) {
            buf.append(pickers[i].getText());
            if (i != pickers.length - 1)
                buf.append(", "); // NOI18N
        }
        return buf.toString();
    }

    Object[] getParameters() {
        try {
            Object values[] = new Object [pickers.length];
            for (int i = 0; i < pickers.length; i++) {
                values[i] = pickers[i].getPropertyValue();
            }
            return values;
        } catch (IllegalStateException e) {
            e.printStackTrace();
            return null;
        }
    }

    private String getPreviewText() {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < pickers.length; i++) {
            buf.append(pickers[i].getPreviewText());
            if (i != pickers.length - 1)
                buf.append(", "); // NOI18N
        }
        return buf.toString();
    }

    private void updatePreview() {
        previewLabel.setText(getPreviewText());

        boolean now = isValid();
        if (now != valid) {
            valid = now;
            fireStateChanged();
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        insidePanel = new javax.swing.JPanel();
        parameterTabs = new javax.swing.JTabbedPane();
        previewLabel = new javax.swing.JLabel();
        setLayout(new java.awt.BorderLayout());
        
        insidePanel.setLayout(new java.awt.BorderLayout(0, 5));
        
        insidePanel.add(parameterTabs, java.awt.BorderLayout.CENTER);
        
        
        previewLabel.setText(FormEditor.getFormBundle ().getString ("CTL_CW_Preview"));
        insidePanel.add(previewLabel, java.awt.BorderLayout.SOUTH);
        
        
        add(insidePanel, java.awt.BorderLayout.CENTER);
        
    }//GEN-END:initComponents

    // ------------------------------------------------------------------------
    // WizardDescriptor.Panel implementation

    public java.awt.Component getComponent() {
        return this;
    }

    public HelpCtx getHelp() {
        return new HelpCtx("gui.connecting.code"); // NOI18N
    }

    public boolean isValid() {
        boolean flag = true;
        for (int i = 0; i < pickers.length; i++)
            flag = flag && pickers[i].isFilled();
        return flag;
    }

    public void readSettings(java.lang.Object settings) {
    }

    public void storeSettings(java.lang.Object settings) {
    }

    public void addChangeListener(ChangeListener listener) {
        if (listenerList == null ) {
            listenerList = new EventListenerList();
        }
        listenerList.add(ChangeListener.class, listener);
    }

    public void removeChangeListener(ChangeListener listener) {
        if (listenerList != null) {
            listenerList.remove(ChangeListener.class, listener);
        }
    }

    // ------------------------------------------------------------------------

    private void fireStateChanged() {
        if (listenerList == null) return;

        ChangeEvent e = null;
        Object[] listeners = listenerList.getListenerList();
        for (int i = listeners.length-2; i>=0; i-=2) {
            if (listeners[i] == ChangeListener.class) {
                if (e == null)
                    e = new ChangeEvent(this);
                ((ChangeListener)listeners[i+1]).stateChanged(e);
            }
        }
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel insidePanel;
    private javax.swing.JTabbedPane parameterTabs;
    private javax.swing.JLabel previewLabel;
    // End of variables declaration//GEN-END:variables
}
