/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.form;

import org.openide.*;

/** A Dialog which is displayed if some errors or warnings are issued during
 * loading of a form.  It displays the list of errors and allows to display
 * details on each item.
 *
 * @author  Ian Formanek
 */

public class ErrorLogDialog extends javax.swing.JPanel {
    private FormEditor.ErrorLogItem[] errors;

    /** Creates new form ErrorLogDialog */
    public ErrorLogDialog(FormEditor.ErrorLogItem[] errors) {
        this.errors = errors;
        initComponents();
        textLabel1.setText(FormEditor.getFormBundle().getString("CTL_ErrorLogText1"));
        textLabel1.setDisabledTextColor(textLabel1.getForeground());
        textLabel2.setText(FormEditor.getFormBundle().getString("CTL_ErrorLogText2"));
        textLabel2.setDisplayedMnemonic(FormEditor.getFormBundle().getString("CTL_ErrorLogText2_Mnemonic").charAt(0));
        textLabel2.setLabelFor(errorsList);
        detailsButton.setText(FormEditor.getFormBundle().getString("CTL_DETAILS"));
        detailsButton.setMnemonic(FormEditor.getFormBundle().getString("CTL_DETAILS_Mnemonic").charAt(0));
        errorsList.setCellRenderer(new ErrorItemRenderer());
        errorsList.setListData(errors);
        errorsList.getSelectionModel().setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        updateDetailsButton();
    }

    public java.awt.Dimension getPreferredSize() {
        java.awt.Dimension inh = super.getPreferredSize();
        return new java.awt.Dimension(Math.min(inh.width, 550), Math.min(inh.height, 200));
    }

    private void updateDetailsButton() {
        detailsButton.setEnabled(errorsList.getSelectedIndices().length == 1);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        textPanel = new javax.swing.JPanel();
        textLabel2 = new javax.swing.JLabel();
        textLabel1 = new javax.swing.JTextArea();
        errorsScrollPane = new javax.swing.JScrollPane();
        errorsList = new javax.swing.JList();
        detailsPanel = new javax.swing.JPanel();
        detailsButton = new javax.swing.JButton();
        
        setLayout(new java.awt.BorderLayout(11, 5));
        
        setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(12, 12, 0, 11)));
        textPanel.setLayout(new java.awt.BorderLayout(0, 12));
        
        textPanel.add(textLabel2, java.awt.BorderLayout.CENTER);
        
        textLabel1.setWrapStyleWord(true);
        textLabel1.setLineWrap(true);
        textLabel1.setEditable(false);
        textLabel1.setEnabled(false);
        textLabel1.setOpaque(false);
        textPanel.add(textLabel1, java.awt.BorderLayout.NORTH);
        
        add(textPanel, java.awt.BorderLayout.NORTH);
        
        errorsList.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                errorsListValueChanged(evt);
            }
        });
        
        errorsScrollPane.setViewportView(errorsList);
        
        add(errorsScrollPane, java.awt.BorderLayout.CENTER);
        
        detailsPanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        detailsButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                detailsButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTH;
        gridBagConstraints1.weighty = 1.0;
        detailsPanel.add(detailsButton, gridBagConstraints1);
        
        add(detailsPanel, java.awt.BorderLayout.EAST);
        
    }//GEN-END:initComponents

    private void detailsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_detailsButtonActionPerformed
        FormEditor.ErrorLogItem eli =(FormEditor.ErrorLogItem)errorsList.getSelectedValue();
        if (eli != null) {
            ErrorDetail ed = new ErrorDetail(eli.getThrowable());
            detDlg = TopManager.getDefault().createDialog(new DialogDescriptor(
                ed,
                FormEditor.getFormBundle().getString("CTL_ErrorDetailTitle"),
                true,
                new Object[] { FormEditor.getFormBundle().getString("CTL_CLOSE") },
                FormEditor.getFormBundle().getString("CTL_CLOSE"),
                DialogDescriptor.BOTTOM_ALIGN,
                null,
                new java.awt.event.ActionListener() {
                    public void actionPerformed(java.awt.event.ActionEvent ev) {
                        detDlg.setVisible(false);
                    }
                }
                )
                                                          );
            detDlg.show();

        }
    }//GEN-LAST:event_detailsButtonActionPerformed

    private void errorsListValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_errorsListValueChanged
        updateDetailsButton();
    }//GEN-LAST:event_errorsListValueChanged


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel textPanel;
    private javax.swing.JLabel textLabel2;
    private javax.swing.JTextArea textLabel1;
    private javax.swing.JScrollPane errorsScrollPane;
    private javax.swing.JList errorsList;
    private javax.swing.JPanel detailsPanel;
    private javax.swing.JButton detailsButton;
    // End of variables declaration//GEN-END:variables

    private java.awt.Dialog detDlg;

    class ErrorItemRenderer extends javax.swing.DefaultListCellRenderer {
        public java.awt.Component getListCellRendererComponent(javax.swing.JList list, Object value,
                                                               int index, boolean isSelected, boolean cellHasFocus) {
            super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);

            FormEditor.ErrorLogItem item =(FormEditor.ErrorLogItem)value;
            String text;
            if (item.getType() == FormEditor.ErrorLogItem.WARNING) {
                text = java.text.MessageFormat.format(
                    FormEditor.getFormBundle().getString("FMT_WarningItem"),
                    new Object[] { item.getDescription() }
                    );
            } else {
                text = java.text.MessageFormat.format(
                    FormEditor.getFormBundle().getString("FMT_ErrorItem"),
                    new Object[] { item.getDescription() }
                    );
            }
            setText(" " + text); // the leading space provides empty border on the left side of the list // NOI18N

            return this;
        }
    }

    static private class ErrorDetail extends javax.swing.JPanel {
        ErrorDetail(Throwable t) {
            setBorder(new javax.swing.border.EmptyBorder(12, 12, 0, 11));
            setLayout(new java.awt.BorderLayout(0, 8));
            javax.swing.JTextArea detailArea = new javax.swing.JTextArea();
            //      detailArea.setBackground((java.awt.Color) javax.swing.UIManager.getDefaults().get("Label.background")); // NOI18N
            java.io.StringWriter sw = new java.io.StringWriter();
            t.printStackTrace(new java.io.PrintWriter(sw));
            detailArea.setText(sw.toString());
            javax.swing.JScrollPane jScrollPane1 = new javax.swing.JScrollPane();
            jScrollPane1.setViewportView(detailArea);
            add(jScrollPane1, java.awt.BorderLayout.CENTER);
        }

        public java.awt.Dimension getPreferredSize() {
            java.awt.Dimension inh = super.getPreferredSize();
            return new java.awt.Dimension(Math.min(inh.width, 630), Math.min(inh.height, 300));
        }
    }

}
