/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.form;

import org.netbeans.modules.form.forminfo.*;

import java.awt.Container;

/**
 *
 * @author Ian Formanek
 */
public class RADForm {

    private FormInfo formInfo;
    private RADComponent topRADComponent;

    public RADForm(FormInfo formInfo) {
        this.formInfo = formInfo;
    }

    /** Called by form manager to attach this RADForm to the form it manages */
    public void initialize(FormModel formModel) {
        if (formInfo.getTopContainer() != null) {
            topRADComponent = new RADVisualFormContainer(formInfo);
        } else {
            topRADComponent = new RADFormContainer(formInfo);
        }
        topRADComponent.initialize(formModel);
        topRADComponent.setComponent(formInfo.getFormInstance().getClass());
        topRADComponent.setName(formModel.getFormDataObject().getName()); // [PENDING - message format]
        ((ComponentContainer)topRADComponent).initSubComponents(new RADComponent[0]);
    }

    public FormInfo getFormInfo() {
        return formInfo;
    }

    public RADComponent getTopLevelComponent() {
        return topRADComponent;
    }

    public FormContainer getFormContainer() {
        return(FormContainer)topRADComponent;
    }

    public boolean allowsVisualComponents() {
        return(formInfo.getTopContainer() != null);
    }


    // Ideas:
    // 1. Dialog, Frame, ... - will be synthesized, not real instance will be created until TestMode
    // 2. Panel, JInternalFrame
    //    - real top-level component instance will be created
    //    - it is necessary to combine synthesized properties(size, ...) with real(live) properties
}
