/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.form.compat2;

import org.netbeans.modules.form.util2.NbVersion;
import org.netbeans.modules.form.util2.NbVersionNotCompatibleException;
import org.netbeans.modules.form.FormDataObject;

/** The DesignForm is an abstract superclass of all different
 * Form types supported by the FormEditor.
 *
 * @author  Ian Formanek
 */
public abstract class DesignForm extends Object implements java.io.Externalizable {
    /** A JDK 1.1 serial version UID */
    static final long serialVersionUID = 6521541306176410181L; // fixed number to provide backward compatibility

    /** Netbeans class version */
    public static final NbVersion nbClassVersion = new NbVersion(1, 0);

    // --------------------------------------------------------------------------------------
    // private area

    /** The manager that manages a design-time behaviour of this form */
    public FormManager formManager;

    // -----------------------------------------------------------------------------
    // FINALIZE DEBUG METHOD

    public void finalize() throws Throwable {
        super.finalize();
        if (System.getProperty("netbeans.debug.form.finalize") != null) {
            System.out.println("finalized: "+this.getClass().getName()+", instance: "+this); // NOI18N
        }
    }

    // --------------------------------------------------------------------------------------
    // Serialization

    /** Fields to restore before calling this method:
     *
     *     formModel [FormManager]
     *
     */
    public void writeExternal(java.io.ObjectOutput oo) throws java.io.IOException {
        // store the version
        oo.writeObject(nbClassVersion);

        oo.writeObject(formManager);
    }

    /** Reads this object from the specified stream */
    public void readExternal(java.io.ObjectInput oi) throws java.io.IOException, ClassNotFoundException {
        org.netbeans.modules.form.FormUtils.DEBUG(">> Design Form: readExternal: START"); // NOI18N
        org.netbeans.modules.form.FormUtils.DEBUG("?? Design Form: readExternal: expecting NbVersion"); // NOI18N
        // check the version
        NbVersion classVersion =(NbVersion) oi.readObject();
        if (!nbClassVersion.isCompatible(classVersion))
            throw new NbVersionNotCompatibleException(classVersion, nbClassVersion);
        org.netbeans.modules.form.FormUtils.DEBUG("** Design Form: readExternal: loaded: "+classVersion); // NOI18N
        org.netbeans.modules.form.FormUtils.DEBUG("?? Design Form: readExternal: expecting FormManager"); // NOI18N
        formManager =(FormManager) oi.readObject();
        org.netbeans.modules.form.FormUtils.DEBUG("** Design Form: readExternal: loaded: "+formManager); // NOI18N
        org.netbeans.modules.form.FormUtils.DEBUG("<< Design Form: readExternal: END"); // NOI18N
    }

}
