/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.form.compat2;

import org.netbeans.modules.form.util2.NbVersion;
import org.netbeans.modules.form.util2.NbVersionNotCompatibleException;
import org.netbeans.modules.form.util2.node.NodeArray;

/** The NonVisualsNode is a Node that holds all the nonvisual components of the Form.
 *
 * @author Ian Formanek
 */
public class NonVisualsNode implements java.io.Externalizable, RADContainer {
    /** A JDK 1.1 serial version UID */
    static final long serialVersionUID = 3159852954546091927L;

    /** Netbeans class version */
    public static final NbVersion nbClassVersion = new NbVersion(1, 0);

    // -----------------------------------------------------------------------------
    // Private area

    public RADNodeArray nodeArray;

    public FormManager formManager;

    // FINALIZE DEBUG METHOD
    public void finalize() throws Throwable {
        super.finalize();
        if (System.getProperty("netbeans.debug.form.finalize") != null) {
            System.out.println("finalized: "+this.getClass().getName()+", instance: "+this); // NOI18N
        }
    } // FINALIZE DEBUG METHOD

    /** For externalization only */
    public NonVisualsNode() {
    }

    // ------------------------------------------------------
    // Serialization

    /** Fields to restore before calling this method:
     *
     *     formModel [FormManager]
     *     nodeArray [RADNodeArray]
     *
     */
    public void writeExternal(java.io.ObjectOutput oo)
        throws java.io.IOException {
        // store the version
        oo.writeObject(nbClassVersion);

        oo.writeObject(formManager);
        oo.writeObject(nodeArray);
    }

    /** Reads the object from stream.
     * @param ois input stream to read from
     * @exception IOException on error
     * @exception ClassNotFoundException if the class of the read object is not found
     */
    public void readExternal(java.io.ObjectInput oi)
        throws java.io.IOException, ClassNotFoundException {
        org.netbeans.modules.form.FormUtils.DEBUG(">> NonVisualsNode: readExternal: START"); // NOI18N
        // check the version
        NbVersion classVersion =(NbVersion) oi.readObject();
        if (!nbClassVersion.isCompatible(classVersion))
            throw new NbVersionNotCompatibleException(classVersion, nbClassVersion);

        formManager =(FormManager)oi.readObject();
        Object o = oi.readObject();
        nodeArray =(RADNodeArray) o;
        /*    Node[] children = nodeArray.toArray();

        for (int i = 0; i < children.length; i++) 
        ((RADNode)children[i]).setParentNodeWithoutNotify(this);
            
        setDisplayName(FormEditor.getFormBundle().getString("CTL_NonVisualsNode")); */
        org.netbeans.modules.form.FormUtils.DEBUG("<< NonVisualsNode: readExternal: END"); // NOI18N
    }


}
