/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.form.compat2;

import org.netbeans.modules.form.util2.NbVersion;
import org.netbeans.modules.form.util2.NbVersionNotCompatibleException;

/** A Node that encapsulates the whole form(and its top-level container).
 *
 * @author Ian Formanek, Petr Hamernik
 */
final public class RADFormNode extends RADContainerNode {
    /** A JDK 1.1 serial version UID */
    static final long serialVersionUID = 846459318776993001L;

    /** Netbeans class version */
    public static final NbVersion nbClassVersion = new NbVersion(1, 0);


    /** For externalization only. */
    public RADFormNode() {
    }

    // ------------------------------------------------------
    // Private area

    /** The Form represented by this RADFormNode */
    public DesignForm form;

    /** The context for nonvisual components */
    public NonVisualsNode nonVisualsNode;

    /** The current menu - name of the variable. */
    public String menu = null;

    // ------------------------------------------------------
    // serialization

    /** Fields to restore before calling this method:
     *
     *     form [DesignForm]
     *     nonVisualsNode [NonVisualsNode]
     *     menu [String](null for no menu)
     *
     */
    protected void writeExternalImpl(java.io.ObjectOutput oo)
        throws java.io.IOException {
        super.writeExternalImpl(oo);

        // store the version
        oo.writeObject(nbClassVersion);

        oo.writeObject(form);
        oo.writeObject(nonVisualsNode);
        oo.writeObject(menu);
    }

    /** Reads the object from stream.
     * @param ois input stream to read from
     * @exception IOException on error
     * @exception ClassNotFoundException if the class of the read object is not found
     */
    protected void readExternalImpl(java.io.ObjectInput oi)
        throws java.io.IOException, ClassNotFoundException {
        super.readExternalImpl(oi);

        org.netbeans.modules.form.FormUtils.DEBUG(">> RADFormNode: readExternal: START"); // NOI18N
        // check the version
        NbVersion classVersion =(NbVersion) oi.readObject();
        if (!nbClassVersion.isCompatible(classVersion))
            throw new NbVersionNotCompatibleException(classVersion, nbClassVersion);

        form =(DesignForm) oi.readObject();
        nonVisualsNode =(NonVisualsNode) oi.readObject();
        menu =(String) oi.readObject();
        org.netbeans.modules.form.FormUtils.DEBUG("<< RADFormNode: readExternal: END"); // NOI18N
    }

}
