/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.form.compat2;

import org.netbeans.modules.form.util2.NbVersion;
import org.netbeans.modules.form.util2.NbVersionNotCompatibleException;
import org.netbeans.modules.form.util2.node.NodeArray;

/** The RADMenuNode is a Node that represents one component placed on the Form.
 *
 * @author Petr Hamernik, Ian Formanek
 */
public class RADMenuNode extends RADMenuItemNode implements RADContainer {

    /** A JDK 1.1 serial version UID */
    static final long serialVersionUID = -6333847833552116544L;

    /** Netbeans class version */
    public static final NbVersion nbClassVersion = new NbVersion(1, 0);

    /** array of subnodes representing menuitems */
    public RADNodeArray nodeArray;

    /** For externalization only. */
    public RADMenuNode() {
    }

    public Object[] getSubNodes() {
        if (nodeArray == null) return new RADNode [0];
        return nodeArray.array;
    }

    //-------------------------------------------------------------------------------
    // Menu Separator

    /** Node which represents the separator in the tree of menu components */
    public static class MenuSeparatorNode implements java.io.Externalizable {
        /** A JDK 1.1 serial version UID */
        static final long serialVersionUID = -6333847833552116000L;

        /** Netbeans class version of separator nodes. */
        public static final NbVersion nbClassVersionSeparator = new NbVersion(1, 0);

        /** For externalization only. */
        public MenuSeparatorNode() {
        }

        /** serializes object */
        public void writeExternal(java.io.ObjectOutput oo)
            throws java.io.IOException {
            // store the version
            oo.writeObject(nbClassVersionSeparator);
        }

        /** deserializes object */
        public void readExternal(java.io.ObjectInput oi)
            throws java.io.IOException, ClassNotFoundException {
            org.netbeans.modules.form.FormUtils.DEBUG(">> RADMenuNode.MenuSeparatorNode: readExternal: START"); // NOI18N
            // check the version
            NbVersion classVersion =(NbVersion) oi.readObject();
            if (!nbClassVersionSeparator.isCompatible(classVersion))
                throw new NbVersionNotCompatibleException(classVersion, nbClassVersionSeparator);
            org.netbeans.modules.form.FormUtils.DEBUG("<< RADMenuNode.MenuSeparatorNode: readExternal: END"); // NOI18N
        }
    }

    // ------------------------------------------------------
    // serialization

    protected void writeExternalImpl(java.io.ObjectOutput oo)
        throws java.io.IOException {
        super.writeExternalImpl(oo);

        // store the version
        oo.writeObject(nbClassVersion);

        oo.writeObject(nodeArray);
    }

    /** Reads the object from stream.
     * @param ois input stream to read from
     * @exception IOException on error
     * @exception ClassNotFoundException if the class of the read object is not found
     */
    protected void readExternalImpl(java.io.ObjectInput oi)
        throws java.io.IOException, ClassNotFoundException {
        super.readExternalImpl(oi);
        org.netbeans.modules.form.FormUtils.DEBUG(">> RADMenuNode: readExternal: START"); // NOI18N

        // check the version
        NbVersion classVersion =(NbVersion) oi.readObject();
        if (!nbClassVersion.isCompatible(classVersion))
            throw new NbVersionNotCompatibleException(classVersion, nbClassVersion);

        nodeArray =(RADNodeArray) oi.readObject();

        org.netbeans.modules.form.FormUtils.DEBUG("<< RADMenuNode: readExternal: END"); // NOI18N
    }
}
