/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.form.compat2.layouts.support;

import javax.swing.JLayeredPane;

import org.netbeans.modules.form.util2.NbVersion;
import org.netbeans.modules.form.util2.NbVersionNotCompatibleException;
import org.netbeans.modules.form.*;
import org.netbeans.modules.form.compat2.layouts.DesignLayout;

/** A design-time support layout for JLayeredPane component.
 *
 * @author   Ian Formanek
 */
final public class JLayeredPaneSupportLayout extends PlainDesignSupportLayout {
    /** A JDK 1.1 serial version UID */
    static final long serialVersionUID = -8907380680799986131L;

    /** Returns a constraint to be used for adding a component to the
     * specified position(e.g. in BorderLayout, the "North", "South", ...
     * will be determined by the position in the container).
     * The default implementation just returns the default constraints.
     * A special constraints object will be returned for layouts that
     * wish to set the position&size of the components rather than add
     * with constraints.
     * @param position The position within the container for which the
     *                 constraints should be returned.
     */
    public DesignLayout.ConstraintsDescription getConstraintsDescription(
                                                 java.awt.Point position) {
        return new JLayeredPaneConstraintsDescription(
                       JLayeredPane.DEFAULT_LAYER.intValue(),
                       -1, position.x, position.y, -1, -1);
    }

    /** Generates the code for adding specified component to this layout.
     * @param comp   The component to be added to this layout
     * @param cont   The container that is managed by this layout
     */
    public String generateComponentCode(RADVisualContainer cont, RADVisualComponent comp) {
        return null;
    }

    // -----------------------------------------------------------------------------
    // constraints innerclass

    /** The ConstraintsDescription class encapsulates constraints data and
     * operations on a constraints that will be used for adding components
     * to the layout.
     */
    final public static class JLayeredPaneConstraintsDescription extends PlainDesignSupportLayout.PlainSupportConstraintsDescription {
        /** A JDK 1.1 serial version UID */
        static final long serialVersionUID = 731223873106449595L;

        /** Netbeans class version */
        public static final NbVersion nbClassVersion = new NbVersion(1, 0); // saved from Beta3(Build 129)

        public JLayeredPaneConstraintsDescription() {
        }

        public JLayeredPaneConstraintsDescription(int layer, int position, int x, int y, int width, int height) {
            this.layer = layer;
            this.position = position;
            this.x = x;
            this.y = y;
            this.width = width;
            this.height = height;
        }

        public int getLayer() {
            return layer;
        }

        public int getPosition() {
            return position;
        }

        public java.awt.Rectangle getBounds() {
            return new java.awt.Rectangle(x, y, width, height);
        }

        public boolean equals(Object o) {
            if (!(o instanceof JLayeredPaneConstraintsDescription)) return false;
            JLayeredPaneConstraintsDescription cd =(JLayeredPaneConstraintsDescription) o;
            return(layer == cd.layer) &&
                (position == cd.position) &&
                (x == cd.x) &&
                (y == cd.y) &&
                (width == cd.width) &&
                (height == cd.height);
        }

        public int hashCode() {
            return layer*32 + position*16 + x*8 + y*4 + width*2 + height;
        }

        public String getConstraintsString() {
            if ((width == -1) &&(height == -1))
                return "[layer="+layer+", position="+position+", x="+x+", y="+y+"]"; // NOI18N
            else
                return "[layer="+layer+", position="+position+", x="+x+", y="+y+", width="+width+", height="+height+"]"; // NOI18N
        }


        // -----------------------------------------------------------------------------
        // Serialization

        /** Writes the object to the stream.
         * @param oo output stream to write to
         * @exception IOException Includes any I/O exceptions that may occur
         */
        public void writeExternal(java.io.ObjectOutput oo)
            throws java.io.IOException {
            // store version
            oo.writeObject(nbClassVersion);

            oo.writeInt(layer);
            oo.writeInt(position);
            oo.writeInt(x);
            oo.writeInt(y);
            oo.writeInt(width);
            oo.writeInt(height);
        }

        /** Reads the object from stream.
         * @param oi input stream to read from
         * @exception IOException Includes any I/O exceptions that may occur
         * @exception ClassNotFoundException if the class of the read object is not found
         */
        public void readExternal(java.io.ObjectInput oi)
            throws java.io.IOException, ClassNotFoundException {
            org.netbeans.modules.form.FormUtils.DEBUG(">> JLayeredPaneConstraintsDescription: readExternal: START"); // NOI18N
            // check the version
            NbVersion classVersion =(NbVersion) oi.readObject();
            if (!nbClassVersion.isCompatible(classVersion))
                throw new NbVersionNotCompatibleException(classVersion, nbClassVersion);

            layer = oi.readInt();
            position = oi.readInt();
            x = oi.readInt();
            y = oi.readInt();
            width = oi.readInt();
            height = oi.readInt();
            org.netbeans.modules.form.FormUtils.DEBUG("<< JLayeredPaneConstraintsDescription: readExternal: END"); // NOI18N
        }

        // -----------------------------------------------------------------------------
        // XML Persistence

        /** Called to load property value from specified XML subtree. If succesfully loaded,
         * the value should be available via the getValue method.
         * An IOException should be thrown when the value cannot be restored from the specified XML element
         * @param element the XML DOM element representing a subtree of XML from which the value should be loaded
         * @exception IOException thrown when the value cannot be restored from the specified XML element
         */
        public void readFromXML(org.w3c.dom.Node element) throws java.io.IOException {
            if (!XML_LAYERED_PANE_CONSTRAINTS.equals(element.getNodeName())) {
                throw new java.io.IOException();
            }
            org.w3c.dom.NamedNodeMap attributes = element.getAttributes();
            org.w3c.dom.Node node;
            node = attributes.getNamedItem(ATTR_X); if (node != null) x = Integer.parseInt(node.getNodeValue());
            node = attributes.getNamedItem(ATTR_Y); if (node != null) y = Integer.parseInt(node.getNodeValue());
            node = attributes.getNamedItem(ATTR_W); if (node != null) width = Integer.parseInt(node.getNodeValue());
            node = attributes.getNamedItem(ATTR_H); if (node != null) height = Integer.parseInt(node.getNodeValue());
            node = attributes.getNamedItem(ATTR_LAYER); if (node != null) layer = Integer.parseInt(node.getNodeValue());
            node = attributes.getNamedItem(ATTR_POSITION); if (node != null) position = Integer.parseInt(node.getNodeValue());
        }

        /** Called to store current property value into XML subtree. The property value should be set using the
         * setValue method prior to calling this method.
         * @param doc The XML document to store the XML in - should be used for creating nodes only
         * @return the XML DOM element representing a subtree of XML from which the value should be loaded or null 
         *         if the value does not need to save any additional data and can be created using the default constructor
         */
        public org.w3c.dom.Node storeToXML(org.w3c.dom.Document doc) {
            org.w3c.dom.Element el = doc.createElement(XML_LAYERED_PANE_CONSTRAINTS);
            el.setAttribute(ATTR_X, Integer.toString(x));
            el.setAttribute(ATTR_Y, Integer.toString(y));
            el.setAttribute(ATTR_W, Integer.toString(width));
            el.setAttribute(ATTR_H, Integer.toString(height));
            el.setAttribute(ATTR_LAYER, Integer.toString(layer));
            el.setAttribute(ATTR_POSITION, Integer.toString(position));
            return el;
        }

        public static final String XML_LAYERED_PANE_CONSTRAINTS = "JLayeredPaneConstraints"; // NOI18N
        public static final String ATTR_X = "x"; // NOI18N
        public static final String ATTR_Y = "y"; // NOI18N
        public static final String ATTR_W = "width"; // NOI18N
        public static final String ATTR_H = "height"; // NOI18N
        public static final String ATTR_LAYER = "layer"; // NOI18N
        public static final String ATTR_POSITION = "position"; // NOI18N

        /** The layer represented by this JLayeredPaneConstraintsDescription class */
        int layer = javax.swing.JLayeredPane.DEFAULT_LAYER.intValue();

        /** The psoition represented by this JLayeredPaneConstraintsDescription class */
        int position = -1;

        int x = 0;
        int y = 0;
        int width = -1;
        int height = -1;
    }
}
