/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.form.layoutsupport;

import java.awt.*;
import java.beans.*;
import java.util.*;
import org.openide.nodes.Node;
import org.netbeans.modules.form.*;

/**
 *
 * @author Tran Duc Trung, Tomas Pavek
 */

public class CardLayoutSupport extends AbstractLayoutSupport
                               implements LayoutSupportArranging
{
    private CardConstraintsDesc currentCard;


    public Class getLayoutClass() {
        return CardLayout.class;
    }

    public void addComponent(RADVisualComponent component, ConstraintsDesc desc) {
        if (desc == null) desc = new CardConstraintsDesc(component.getName());
        super.addComponent(component, desc);
        if (currentCard == null && desc instanceof CardConstraintsDesc)
            currentCard = (CardConstraintsDesc)desc;
    }

    public int getNewIndex(Container container, Point posInCont,
                           Component component, Point posInComp) {
        if (!(container.getLayout() instanceof CardLayout))
            return -1;
        return container.getComponentCount();
    }

    public boolean paintDragFeedback(Container container,
                                     Component component,
                                     ConstraintsDesc newConstraints,
                                     int newIndex,
                                     Graphics g) {
        if (!(container.getLayout() instanceof CardLayout))
            return false;

        Dimension sz = container.getSize();
        Insets insets = container.getInsets();
        sz.width -= insets.left + insets.right;
        sz.height -= insets.top + insets.bottom;
        
        g.drawRect(0, 0, sz.width, sz.height);
        return true;
    }
    
    
    // ------------------------
    // LayoutSupportArranging

    public void processMouseClick(Point p, Container cont) {
    }

    public void selectComponent(RADVisualComponent comp) {
        ConstraintsDesc desc = getConstraints(comp);
        if (desc instanceof CardConstraintsDesc)
            currentCard = (CardConstraintsDesc) desc;
    }

    public void arrangeContainer(Container cont) {
        LayoutManager lm = cont.getLayout();
        if (!(lm instanceof CardLayout)
                || currentCard == null) return;

        ((CardLayout)lm).show(cont, (String)currentCard.getConstraintsObject());
    }

    // ----------------

    public static class CardConstraintsDesc
            implements LayoutSupport.ConstraintsDesc
    {
        private String card;

        transient private Node.Property[] properties;

        public CardConstraintsDesc(String card) {
            this.card = card;
        }

        public Node.Property[] getProperties() {
            if (properties == null)
                properties = new Node.Property[] {
                    new FormProperty("cardName", // NOI18N
                                     String.class,
                                 getBundle().getString("PROP_cardName"), // NOI18N
                                 getBundle().getString("HINT_cardName")) { // NOI18N

                        public Object getTargetValue() {
                            return card;
                        }

                        public void setTargetValue(Object value) {
                            card = (String)value;
                        }
                    }
                };

            return properties;
        }

        public Object getConstraintsObject() {
            return card;
        }

        public String getJavaInitializationString() {
            return ((FormProperty)getProperties()[0]).getJavaInitializationString();
        }

        public void readFromXML(org.w3c.dom.Node element)
        throws java.io.IOException {
        }

        public org.w3c.dom.Node storeToXML(org.w3c.dom.Document doc) {
            return null;
        }
    }
}
