/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.form.layoutsupport;

import java.awt.*;

import org.openide.nodes.Node;

import org.netbeans.modules.form.compat2.layouts.*;
import org.netbeans.modules.form.compat2.layouts.support.*;
import org.netbeans.modules.form.layoutsupport.dedicated.*;


/**
 *
 * @author Tran Duc Trung, Tomas Pavek
 */

public class Compat31LayoutFactory
{
    private static final String[] designLayoutNames = {
        "org.netbeans.modules.form.compat2.layouts.DesignBorderLayout",
        "org.netbeans.modules.form.compat2.layouts.DesignFlowLayout",
        "org.netbeans.modules.form.compat2.layouts.DesignBoxLayout",
        "org.netbeans.modules.form.compat2.layouts.DesignGridBagLayout",
        "org.netbeans.modules.form.compat2.layouts.DesignGridLayout",
        "org.netbeans.modules.form.compat2.layouts.DesignCardLayout",
        "org.netbeans.modules.form.compat2.layouts.DesignAbsoluteLayout",
        "org.netbeans.modules.form.compat2.layouts.DesignAbsoluteLayout",
        "org.netbeans.modules.form.compat2.layouts.support.JScrollPaneSupportLayout",
        "org.netbeans.modules.form.compat2.layouts.support.JSplitPaneSupportLayout",
        "org.netbeans.modules.form.compat2.layouts.support.JTabbedPaneSupportLayout",
        "org.netbeans.modules.form.compat2.layouts.support.JLayeredPaneSupportLayout",
        "org.netbeans.modules.form.compat2.layouts.DesignBoxLayout"
    };

    private static final String[] layoutSupportNames = {
        "org.netbeans.modules.form.layoutsupport.BorderLayoutSupport",
        "org.netbeans.modules.form.layoutsupport.FlowLayoutSupport",
        "org.netbeans.modules.form.layoutsupport.BoxLayoutSupport",
        "org.netbeans.modules.form.layoutsupport.GridBagLayoutSupport",
        "org.netbeans.modules.form.layoutsupport.GridLayoutSupport",
        "org.netbeans.modules.form.layoutsupport.CardLayoutSupport",
        "org.netbeans.modules.form.layoutsupport.AbsoluteLayoutSupport",
        "org.netbeans.modules.form.layoutsupport.NullLayoutSupport",
        "org.netbeans.modules.form.layoutsupport.dedicated.JScrollPaneSupport",
        "org.netbeans.modules.form.layoutsupport.dedicated.JSplitPaneSupport",
        "org.netbeans.modules.form.layoutsupport.dedicated.JTabbedPaneSupport",
        "org.netbeans.modules.form.layoutsupport.dedicated.JLayeredPaneSupport",
        "org.netbeans.modules.form.layoutsupport.dedicated.JToolBarSupport"
    };


    private Compat31LayoutFactory() {}

    /** Converts DesignLayout to compatible LayoutSupport.
     */
    public static LayoutSupport createCompatibleLayoutSupport(DesignLayout dl) {
        if (dl instanceof DesignBorderLayout) {
            BorderLayoutSupport laySup = new BorderLayoutSupport();
            if (dl.getRADContainer() == null) return laySup;

            laySup.initialize(dl.getRADContainer());
            java.util.Iterator properties = dl.getChangedProperties().iterator();
            while (properties.hasNext()) {
                try {
                    Node.Property dlProp = (Node.Property) properties.next();
                    if (DesignBorderLayout.PROP_HGAP.equals(dlProp.getName())) {
                        Node.Property lsProp = laySup.getProperty("hgap"); // NOI18N
                        if (lsProp != null)
                            lsProp.setValue(dlProp.getValue());
                    }
                    else if (DesignBorderLayout.PROP_VGAP.equals(dlProp.getName())) {
                        Node.Property lsProp = laySup.getProperty("vgap"); // NOI18N
                        if (lsProp != null)
                            lsProp.setValue(dlProp.getValue());
                    }
                }
                catch (Exception ex) {} // should not happen
            }

            return laySup;
        }

        if (dl instanceof DesignFlowLayout) {
            FlowLayoutSupport laySup = new FlowLayoutSupport();
            if (dl.getRADContainer() == null) return laySup;

            laySup.initialize(dl.getRADContainer());
            java.util.Iterator properties = dl.getChangedProperties().iterator();
            while (properties.hasNext()) {
                try {
                    Node.Property dlProp = (Node.Property) properties.next();
                    if (DesignFlowLayout.PROP_ALIGNMENT.equals(dlProp.getName())) {
                        Node.Property lsProp = laySup.getProperty("alignment"); // NOI18N
                        if (lsProp != null)
                            lsProp.setValue(dlProp.getValue());
                    }
                    else if (DesignFlowLayout.PROP_HGAP.equals(dlProp.getName())) {
                        Node.Property lsProp = laySup.getProperty("hgap"); // NOI18N
                        if (lsProp != null)
                            lsProp.setValue(dlProp.getValue());
                    }
                    else if (DesignFlowLayout.PROP_VGAP.equals(dlProp.getName())) {
                        Node.Property lsProp = laySup.getProperty("vgap"); // NOI18N
                        if (lsProp != null)
                            lsProp.setValue(dlProp.getValue());
                    }
                }
                catch (Exception ex) {} // should not happen
            }

            return laySup;
        }

        if (dl instanceof DesignBoxLayout) {
            BoxLayoutSupport laySup = new BoxLayoutSupport();
            if (dl.getRADContainer() == null) return laySup;

            laySup.initialize(dl.getRADContainer());
            java.util.Iterator properties = dl.getChangedProperties().iterator();
            while (properties.hasNext()) {
                try {
                    Node.Property dlProp = (Node.Property) properties.next();
                    if (DesignBoxLayout.PROP_AXIS.equals(dlProp.getName())) {
                        Node.Property lsProp = laySup.getProperty("axis"); // NOI18N
                        if (lsProp != null)
                            lsProp.setValue(dlProp.getValue());
                    }
                }
                catch (Exception ex) {} // should not happen
            }

            return laySup;
        }

        if (dl instanceof DesignGridBagLayout) {
            GridBagLayoutSupport laySup = new GridBagLayoutSupport();
            if (dl.getRADContainer() != null)
                laySup.initialize(dl.getRADContainer());
            return laySup;
        }

        if (dl instanceof DesignGridLayout) {
            GridLayoutSupport laySup = new GridLayoutSupport();
            if (dl.getRADContainer() == null) return laySup;

            laySup.initialize(dl.getRADContainer());
            Node.Property lsProp;
            try { // set default DesignGridLayout columns and rows
                lsProp = laySup.getProperty("rows"); // NOI18N
                if (lsProp != null) lsProp.setValue(new Integer(2));
                lsProp = laySup.getProperty("columns"); // NOI18N
                if (lsProp != null) lsProp.setValue(new Integer(3));
            }
            catch (Exception ex) {} // should not happen

            java.util.Iterator properties = dl.getChangedProperties().iterator();
            while (properties.hasNext()) {
                try {
                    Node.Property dlProp = (Node.Property) properties.next();
                    if (DesignGridLayout.PROP_ROWS.equals(dlProp.getName())) {
                        lsProp = laySup.getProperty("rows"); // NOI18N
                        if (lsProp != null)
                            lsProp.setValue(dlProp.getValue());
                    }
                    else if (DesignGridLayout.PROP_COLUMNS.equals(dlProp.getName())) {
                        lsProp = laySup.getProperty("columns"); // NOI18N
                        if (lsProp != null)
                            lsProp.setValue(dlProp.getValue());
                    }
                    else if (DesignGridLayout.PROP_HGAP.equals(dlProp.getName())) {
                        lsProp = laySup.getProperty("hgap"); // NOI18N
                        if (lsProp != null)
                            lsProp.setValue(dlProp.getValue());
                    }
                    else if (DesignGridLayout.PROP_VGAP.equals(dlProp.getName())) {
                        lsProp = laySup.getProperty("vgap"); // NOI18N
                        if (lsProp != null)
                            lsProp.setValue(dlProp.getValue());
                    }
                }
                catch (Exception ex) {} // should not happen
            }

            return laySup;
        }

        if (dl instanceof DesignCardLayout) {
            CardLayoutSupport laySup = new CardLayoutSupport();
            if (dl.getRADContainer() == null) return laySup;

            laySup.initialize(dl.getRADContainer());
            java.util.Iterator properties = dl.getChangedProperties().iterator();
            while (properties.hasNext()) {
                try {
                    Node.Property dlProp = (Node.Property) properties.next();
                    if (DesignCardLayout.PROP_HGAP.equals(dlProp.getName())) {
                        Node.Property lsProp = laySup.getProperty("hgap"); // NOI18N
                        if (lsProp != null)
                            lsProp.setValue(dlProp.getValue());
                    }
                    else if (DesignCardLayout.PROP_VGAP.equals(dlProp.getName())) {
                        Node.Property lsProp = laySup.getProperty("vgap"); // NOI18N
                        if (lsProp != null)
                            lsProp.setValue(dlProp.getValue());
                    }
                }
                catch (Exception ex) {} // should not happen
            }

            return laySup;
        }

        if (dl instanceof DesignAbsoluteLayout) {
            return ((DesignAbsoluteLayout)dl).isNullLayout() ?
                   new NullLayoutSupport() : new AbsoluteLayoutSupport();
        }

        if (dl instanceof JScrollPaneSupportLayout) {
            return new JScrollPaneSupport();
        }

        if (dl instanceof JSplitPaneSupportLayout) {
            return new JSplitPaneSupport();
        }

        if (dl instanceof JTabbedPaneSupportLayout) {
            return new JTabbedPaneSupport();
        }

        if (dl instanceof JLayeredPaneSupportLayout) {
            return new JLayeredPaneSupport();
        }

        // no compatible LayoutSupport, use the "bridge" Compat31LayoutSupport
        return new Compat31LayoutSupport(dl);
    }

    /** Converts LayoutSupport to compatible DesignLayout.
     */
    public static DesignLayout createCompatibleDesignLayout(LayoutSupport ls) {
        if (ls instanceof BorderLayoutSupport) {
            AbstractLayoutSupport als = (AbstractLayoutSupport)ls;
            int hgap = 0, vgap = 0;

            try {
                Node.Property prop;
                prop = als.getProperty("hgap"); // NOI18N
                if (prop != null)
                    hgap = ((Integer)prop.getValue()).intValue();
                prop = als.getProperty("vgap"); // NOI18N
                if (prop != null)
                    vgap = ((Integer)prop.getValue()).intValue();
            }
            catch (Exception ex) {} // should not happen

            return new DesignBorderLayout(hgap, vgap);
        }

        if (ls instanceof FlowLayoutSupport) {
            AbstractLayoutSupport als = (AbstractLayoutSupport)ls;
            int alignment = java.awt.FlowLayout.CENTER;
            int hgap = 5, vgap = 5;

            try {
                Node.Property prop;
                prop = als.getProperty("alignment"); // NOI18N
                if (prop != null)
                    alignment = ((Integer)prop.getValue()).intValue();
                prop = als.getProperty("hgap"); // NOI18N
                if (prop != null)
                    hgap = ((Integer)prop.getValue()).intValue();
                prop = als.getProperty("vgap"); // NOI18N
                if (prop != null)
                    vgap = ((Integer)prop.getValue()).intValue();
            }
            catch (Exception ex) {} // should not happen

            return new DesignFlowLayout(alignment, hgap, vgap);
        }

        if (ls instanceof BoxLayoutSupport) {
            AbstractLayoutSupport als = (AbstractLayoutSupport)ls;
            int axis = javax.swing.BoxLayout.X_AXIS;

            try {
                Node.Property prop;
                prop = als.getProperty("axis"); // NOI18N
                if (prop != null)
                    axis = ((Integer)prop.getValue()).intValue();
            }
            catch (Exception ex) {} // should not happen

            return new DesignBoxLayout(axis);
        }

        if (ls instanceof GridBagLayoutSupport) {
            return new DesignGridBagLayout();
        }

        if (ls instanceof GridLayoutSupport) {
            AbstractLayoutSupport als = (AbstractLayoutSupport)ls;
            int rows = 2, columns = 3;
            int hgap = 0, vgap = 0;

            try {
                Node.Property prop;
                prop = als.getProperty("rows"); // NOI18N
                if (prop != null)
                    rows = ((Integer)prop.getValue()).intValue();
                prop = als.getProperty("columns"); // NOI18N
                if (prop != null)
                    columns = ((Integer)prop.getValue()).intValue();
                prop = als.getProperty("hgap"); // NOI18N
                if (prop != null)
                    hgap = ((Integer)prop.getValue()).intValue();
                prop = als.getProperty("vgap"); // NOI18N
                if (prop != null)
                    vgap = ((Integer)prop.getValue()).intValue();
            }
            catch (Exception ex) {} // should not happen

            return new DesignGridLayout(rows, columns, hgap, vgap);
        }

        if (ls instanceof CardLayoutSupport) {
            AbstractLayoutSupport als = (AbstractLayoutSupport)ls;
            int hgap = 0, vgap = 0;

            try {
                Node.Property prop;
                prop = als.getProperty("hgap"); // NOI18N
                if (prop != null)
                    hgap = ((Integer)prop.getValue()).intValue();
                prop = als.getProperty("vgap"); // NOI18N
                if (prop != null)
                    vgap = ((Integer)prop.getValue()).intValue();
            }
            catch (Exception ex) {} // should not happen

            return new DesignCardLayout(hgap, vgap);
            // current card??
        }

        if (ls instanceof JLayeredPaneSupport) {
            // JLayeredPaneSupport is subclass of AbsoluteLayoutSupport
            return new JLayeredPaneSupportLayout();
        }

        if (ls instanceof NullLayoutSupport) {
            // NullLayoutSupport is subclass of AbsoluteLayoutSupport
            return new DesignAbsoluteLayout(true);
        }

        if (ls instanceof AbsoluteLayoutSupport) {
            return new DesignAbsoluteLayout(false);
        }

        if (ls instanceof JScrollPaneSupport) {
            return new JScrollPaneSupportLayout();
        }

        if (ls instanceof JSplitPaneSupport) {
            return new JSplitPaneSupportLayout();
        }

        if (ls instanceof JTabbedPaneSupport) {
            return new JTabbedPaneSupportLayout();
        }

        if (ls instanceof JToolBarSupport) {
            return new DesignBoxLayout();
        }

        if (ls instanceof Compat31LayoutSupport) {
            return ((Compat31LayoutSupport)ls).getDesignLayout();
        }

        return null;
    }

    public static String getCompatibleLayoutSupportName(String dlName) {
        for (int i=0; i < designLayoutNames.length; i++)
            if (designLayoutNames[i].equals(dlName))
                return layoutSupportNames[i];
        return null;
    }

    public static String getCompatibleDesignLayoutName(String lsName) {
        for (int i=0; i < layoutSupportNames.length; i++)
            if (layoutSupportNames[i].equals(lsName))
                return designLayoutNames[i];
        return null;
    }

    public static LayoutSupport.ConstraintsDesc createCompatibleConstraints(
                                    DesignLayout.ConstraintsDescription desc) {
        // constraints of BorderLayout
        if (desc instanceof DesignBorderLayout.BorderConstraintsDescription) {
            String str = (String) desc.getConstraintsObject();
            return new BorderLayoutSupport.BorderConstraintsDesc(str);
        }

        // constraints of GridBagLayout
        if (desc instanceof DesignGridBagLayout.GridBagConstraintsDescription) {
            java.awt.GridBagConstraints constraints = (java.awt.GridBagConstraints)
                                                   desc.getConstraintsObject();
            return new GridBagLayoutSupport.GridBagConstraintsDesc(constraints);
        }

        // constraints of CardLayout
        if (desc instanceof DesignCardLayout.CardConstraintsDescription) {
            String str = (String) desc.getConstraintsObject();
            return new CardLayoutSupport.CardConstraintsDesc(str);
        }

        // constraints of AbsoluteLayout
        if (desc instanceof DesignAbsoluteLayout.AbsoluteConstraintsDescription) {
            Point position =
                ((DesignAbsoluteLayout.AbsoluteConstraintsDescription)desc)
                    .getPosition();
            Dimension size =
                ((DesignAbsoluteLayout.AbsoluteConstraintsDescription)desc)
                    .getSize();
            return new AbsoluteLayoutSupport.AbsoluteConstraintsDesc(
                            position.x, position.y, size.width, size.height);
        }

        // constraints of JSplitPane
        if (desc instanceof JSplitPaneSupportLayout.JSplitPaneConstraintsDescription) {
            int splitPos =
                ((JSplitPaneSupportLayout.JSplitPaneConstraintsDescription)desc)
                    .getSplitPos();
            String position;

            if (splitPos == JSplitPaneSupportLayout.POS_TOP)
                position = javax.swing.JSplitPane.TOP;
            else if (splitPos == JSplitPaneSupportLayout.POS_BOTTOM)
                position = javax.swing.JSplitPane.BOTTOM;
            else if (splitPos == JSplitPaneSupportLayout.POS_LEFT)
                position = javax.swing.JSplitPane.LEFT;
            else
                position = javax.swing.JSplitPane.RIGHT;

            return new JSplitPaneSupport.SplitConstraintsDesc(position);
        }

        // constraints of JTabbedPane
        if (desc instanceof JTabbedPaneSupportLayout.JTabbedPaneConstraintsDescription) {
            JTabbedPaneSupportLayout.JTabbedPaneConstraintsDescription tabDesc =
                (JTabbedPaneSupportLayout.JTabbedPaneConstraintsDescription)desc;
            return new JTabbedPaneSupport.TabConstraintsDesc(
                tabDesc.getTabName(), tabDesc.getImage(), tabDesc.getToolTip());
        }

        // constraints of JLayeredPane
        if (desc instanceof JLayeredPaneSupportLayout.JLayeredPaneConstraintsDescription) {
            int layer =
                ((JLayeredPaneSupportLayout.JLayeredPaneConstraintsDescription)desc)
                    .getLayer();
            Rectangle bounds =
                ((JLayeredPaneSupportLayout.JLayeredPaneConstraintsDescription)desc)
                    .getBounds();
            return new JLayeredPaneSupport.LayeredConstraintsDesc(
                layer, bounds.x, bounds.y, bounds.width, bounds.height);
        }

        return null;
    }

    public static DesignLayout.ConstraintsDescription createCompatibleConstraints(
                                          LayoutSupport.ConstraintsDesc desc) {
        // constraints of BorderLayout
        if (desc instanceof BorderLayoutSupport.BorderConstraintsDesc) {
            String str = (String) desc.getConstraintsObject();
            return new DesignBorderLayout.BorderConstraintsDescription(str);
        }

        // constraints of GridBagLayout
        if (desc instanceof GridBagLayoutSupport.GridBagConstraintsDesc) {
            java.awt.GridBagConstraints constraints = (java.awt.GridBagConstraints)
                                                    desc.getConstraintsObject();
            return new DesignGridBagLayout.GridBagConstraintsDescription(constraints);
        }

        // constraints of CardLayout
        if (desc instanceof CardLayoutSupport.CardConstraintsDesc) {
            String str = (String) desc.getConstraintsObject();
            return new DesignCardLayout.CardConstraintsDescription(str);
        }

        // constraints of JLayeredPane (must be tested before AbsoluteLayout)
        if (desc instanceof JLayeredPaneSupport.LayeredConstraintsDesc) {
            int layer =
                ((JLayeredPaneSupport.LayeredConstraintsDesc)desc)
                    .getLayer();
            Rectangle bounds =
                ((JLayeredPaneSupport.LayeredConstraintsDesc)desc)
                    .getBounds();
            return new JLayeredPaneSupportLayout.JLayeredPaneConstraintsDescription(
                layer, -1, bounds.x, bounds.y, bounds.width, bounds.height);
        }

        // constraints of AbsoluteLayout
        if (desc instanceof AbsoluteLayoutSupport.AbsoluteConstraintsDesc) {
            Rectangle r = ((AbsoluteLayoutSupport.AbsoluteConstraintsDesc)desc)
                              .getBounds();
            return new DesignAbsoluteLayout.AbsoluteConstraintsDescription(
                                    new Point(r.x, r.y),
                                    new Dimension(r.width, r.height));
        }

        // constraints of JSplitPane
        if (desc instanceof JSplitPaneSupport.SplitConstraintsDesc) {
            Object position = desc.getConstraintsObject();
            int splitPos;

            if (javax.swing.JSplitPane.TOP.equals(position))
                splitPos = JSplitPaneSupportLayout.POS_TOP;
            else if (javax.swing.JSplitPane.BOTTOM.equals(position))
                splitPos = JSplitPaneSupportLayout.POS_BOTTOM;
            else if (javax.swing.JSplitPane.LEFT.equals(position))
                splitPos = JSplitPaneSupportLayout.POS_LEFT;
            else
                splitPos = JSplitPaneSupportLayout.POS_RIGHT;

            return new JSplitPaneSupportLayout.JSplitPaneConstraintsDescription(splitPos);
        }

        // constraints of JTabbedPane
        if (desc instanceof JTabbedPaneSupport.TabConstraintsDesc) {
            JTabbedPaneSupport.TabConstraintsDesc tabDesc =
                (JTabbedPaneSupport.TabConstraintsDesc) desc;
            return new JTabbedPaneSupportLayout.JTabbedPaneConstraintsDescription(
                tabDesc.getTitle(), tabDesc.getIcon(), tabDesc.getToolTip());
        }

        return null;
    }
}
