/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.form.layoutsupport;

import java.awt.*;
import java.beans.*;

import org.openide.nodes.Node;

import org.netbeans.modules.form.*;
import org.netbeans.modules.form.compat2.layouts.DesignLayout;

/**
 *
 * @author Tran Duc Trung, Tomas Pavek
 */

public class Compat31LayoutSupport implements LayoutSupport
{
    private DesignLayout designLayout;
    private BeanInfo layoutBeanInfo;

    Compat31LayoutSupport(DesignLayout dl) {
        setDesignLayout(dl);
    }

    Compat31LayoutSupport() {
    }

    public void setDesignLayout(DesignLayout dl) {
        designLayout = dl;
    }

    public DesignLayout getDesignLayout() {
        return designLayout;
    }

    // ----------------

    public void initialize(RADVisualContainer cont) {
        designLayout.setRADContainer(cont);
    }

    public String getDisplayName() {
        return designLayout.getDisplayName();
    }

    public Image getIcon(int type) {
        return designLayout.getIcon(type);
    }
    
    public Class getLayoutClass() {
        return designLayout.getLayoutClass();
    }

    public LayoutManager createDefaultLayoutInstance(Container cont) {
        return (LayoutManager) BeanSupport.createBeanInstance(getLayoutClass());
    }

    public LayoutManager cloneLayoutInstance(Container cont) {
        RADVisualContainer container = designLayout.getRADContainer();
        LayoutManager lm =
            container.getContainerDelegate(container.getBeanInstance()).getLayout();
        if (lm == null) return null;

        if (layoutBeanInfo == null) {
            layoutBeanInfo = BeanSupport.createBeanInfo(lm.getClass());
            if (layoutBeanInfo == null) return null;
        }

        try {
            return (LayoutManager) FormUtils.cloneBeanInstance(lm, layoutBeanInfo);
        }
        catch (CloneNotSupportedException e) { // ignore
            return null;
        }
    }

    public RADVisualContainer getContainer() {
        return designLayout.getRADContainer();
    }

    public Node.PropertySet[] getPropertySets() {
        return designLayout.getPropertySet();
    }

    public Class getCustomizerClass() {
        return designLayout.getCustomizerClass();
    }

    public ConstraintsDesc getNewConstraints(Container cont, Point posInCont,
                                             Component comp, Point posInComp) {
        DesignLayout.ConstraintsDescription constr =
                              designLayout.getConstraintsDescription(posInCont);
        return constr == null ? null :
                                new Compat31Constraints(constr, null);
    }

    public int getNewIndex(Container cont, Point posInCont,
                           Component comp, Point posInComp) {
        return -1; //designLayout.getRADContainer().getSubComponents().length;
    }

    public boolean paintDragFeedback(Container container,
                                     Component component,
                                     ConstraintsDesc newConstraints,
                                     int newIndex,
                                     Graphics g) {
        return false;
    }

    public int getResizableDirections(Component component) {
        return 0;
    }

    public ConstraintsDesc getResizedConstraints(Component component,
                                                 Insets sizeChanges) {
        return null;
    }

    public void addComponent(RADVisualComponent component, ConstraintsDesc desc) {
        if (desc instanceof Compat31Constraints)
            designLayout.addComponent(component,
                              ((Compat31Constraints)desc).designConstraints);
        else
            designLayout.addComponent(component);
    }

    public void removeComponent(RADVisualComponent component) {
        designLayout.removeComponent(component);
    }

    public ConstraintsDesc getConstraints(RADVisualComponent component) {
        DesignLayout.ConstraintsDescription constr =
                             component.getConstraints(designLayout.getClass());
        return constr == null ? null :
                                new Compat31Constraints(constr, component);
    }

    public ConstraintsDesc fixConstraints(ConstraintsDesc constraintsDesc) {
        return constraintsDesc;
    }

    public ConstraintsDesc[] convertConstraints(ConstraintsDesc[] constraints,
                                                Component[] components) {
        return null;
    }

    public String getJavaSetLayoutString(/*RADVisualContainer cont*/) {
        return designLayout.generateInitCode(getContainer()); // cont
    }

    public String getJavaAddComponentString(/*RADVisualContainer cont,*/
                                            RADVisualComponent comp) {
        return designLayout.generateComponentCode(getContainer(), comp); // cont
    }

    // ------------
    // Class for accessing DesignLayout.ConstraintsDescription
    // as LayoutSupport.ConstraintsDesc. Instance of this class should
    // be never set directly to RADVisualComponent - it is created
    // on DesignLayout.ConstraintsDescription instance when needed.

    class Compat31Constraints implements LayoutSupport.ConstraintsDesc {
        private DesignLayout.ConstraintsDescription designConstraints;
        private RADVisualComponent component;

        Compat31Constraints(DesignLayout.ConstraintsDescription dConstr,
                            RADVisualComponent comp) {
            designConstraints = dConstr;
            component = comp;
        }

        public Node.Property[] getProperties() {
            return component == null ? null :
                       designLayout.getComponentProperties(component);
        }

        public Object getConstraintsObject() {
            return designConstraints.getConstraintsObject();
        }

        public String getJavaInitializationString() {
            return designConstraints.getConstraintsString();
        }

        public void readFromXML(org.w3c.dom.Node element)
        throws java.io.IOException {
            designConstraints.readFromXML(element);
        }

        public org.w3c.dom.Node storeToXML(org.w3c.dom.Document doc) {
            return designConstraints.storeToXML(doc);
        }
    }
}
