/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.form.layoutsupport;

import java.awt.*;
import java.util.*;

/**
 *
 * @author Tran Duc Trung
 */

public class GridLayoutSupport extends AbstractLayoutSupport
{
    public Class getLayoutClass() {
        return java.awt.GridLayout.class;
    }

    public int getNewIndex(Container container, Point posInCont,
                           Component component, Point posInComp) {
        if (!(container.getLayout() instanceof GridLayout))
            return -1;

        Component[] components = container.getComponents();
        GridLayout layout = (GridLayout) container.getLayout();
        int nrows = layout.getRows();
        int ncols = layout.getColumns();

        if (nrows <= 0 && ncols <= 0 || components.length == 0)
            return components.length;
        
        if (nrows != 0) {
            ncols = (components.length + nrows - 1) / nrows;
        }
        else {
            nrows = (components.length + ncols - 1) / ncols;
        }

        Dimension sz = container.getSize();
        Insets insets = container.getInsets();
        sz.width -= insets.left + insets.right;
        sz.height -= insets.top + insets.bottom;

        int colwidth = sz.width / ncols;
        if (colwidth <= 0)
            return components.length;
        int col = (posInCont.x + colwidth / 2) / colwidth;
        
        int rowheight = sz.height / nrows;
        if (rowheight <= 0)
            return components.length;
        int row = posInCont.y / rowheight;

        int index = row * ncols + col;
        return index >= components.length ? components.length : index;
    }

    public boolean paintDragFeedback(Container container,
                                     Component component,
                                     ConstraintsDesc newConstraints,
                                     int newIndex,
                                     Graphics g) {
        if (!(container.getLayout() instanceof GridLayout))
            return false;

        Component[] components = container.getComponents();
        GridLayout layout = (GridLayout) container.getLayout();
        int hgap = layout.getHgap();

        int x = 0, y1 = 0, y2 = 0;
        
        if (newIndex <= 0) {
            if (components.length > 0) {
                Rectangle b = components[0].getBounds();
                x = b.x;
                y1 = b.y;
                y2 = b.y + b.height;
            }
            else {
                x = 0;
                y1 = 0;
                y2 = 20;
            }
        }
        else if (newIndex >= components.length) {
            Rectangle b = components[components.length-1].getBounds();
            x = b.x + b.width;
            y1 = b.y;
            y2 = b.y + b.height;
        }
        else {
            Rectangle b1 = components[newIndex].getBounds();
            x = b1.x;
            y1 = b1.y;
            y2 = b1.y + b1.height;
        }
        g.drawRect(x - 10 - hgap / 2, y1, 20, y2 - y1);
        return true;
    }
}
