/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.form.layoutsupport;

import java.awt.*;
import java.beans.*;
import java.util.Iterator;

import org.openide.nodes.Node;
import org.netbeans.modules.form.*;

/**
 *
 * @author Tran Duc Trung, Tomas Pavek
 */

public interface LayoutSupport
{
    // possible component resizing directions (bit flag constants)
    
    public final int RESIZE_UP = 1;
    public final int RESIZE_DOWN = 2;
    public final int RESIZE_LEFT = 4;
    public final int RESIZE_RIGHT = 8;

    // connection to meta-container
    
    public void initialize(RADVisualContainer container);
    public RADVisualContainer getContainer();

    // node presentation
    
    public String getDisplayName();
    public Image getIcon(int type);

    // LayoutManager operations
    
    public Class getLayoutClass();
    public LayoutManager createDefaultLayoutInstance(Container container);
    public LayoutManager cloneLayoutInstance(Container container);

    // properties and customizer
    
    public Node.PropertySet[] getPropertySets();
    public Class getCustomizerClass();

    // drag and drop support
    
    public ConstraintsDesc getNewConstraints(Container container,
                                             Point posInCont,
                                             Component component,
                                             Point posInComp);
    public int getNewIndex(Container container,
                           Point posInCont,
                           Component component,
                           Point posInComp);
    
    public boolean paintDragFeedback(Container container, 
                                     Component component,
                                     ConstraintsDesc newConstraints,
                                     int newIndex,
                                     Graphics g);

    // resizing support
    
    public int getResizableDirections(Component component);
    public ConstraintsDesc getResizedConstraints(Component component,
                                                 Insets sizeChanges);

    // components manipulation
    
    public void addComponent(RADVisualComponent component,
                             ConstraintsDesc desc);
    public void removeComponent(RADVisualComponent component);

    // constraints handling

    public ConstraintsDesc getConstraints(RADVisualComponent component);
    public ConstraintsDesc fixConstraints(ConstraintsDesc constraints);
    public ConstraintsDesc[] convertConstraints(ConstraintsDesc[] constraints,
                                                Component[] components);

    // code generation
    
    public String getJavaSetLayoutString();
    public String getJavaAddComponentString(RADVisualComponent component);

    // ------

    public interface ConstraintsDesc extends java.io.Serializable
    {
        public Node.Property[] getProperties();

        public Object getConstraintsObject();
        public String getJavaInitializationString();
        public void readFromXML(org.w3c.dom.Node element) throws java.io.IOException;
        public org.w3c.dom.Node storeToXML(org.w3c.dom.Document doc);
    }
}
