/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.form.layoutsupport;

import java.awt.*;
import java.beans.*;
import java.util.*;
import org.openide.nodes.*;
import org.netbeans.modules.form.*;

/**
 *
 * @author Tomas Pavek
 */

public class NullLayoutSupport extends AbsoluteLayoutSupport {

    private static Image layoutIcon =
        Toolkit.getDefaultToolkit().getImage(
            AbstractLayoutSupport.class.getResource(
                "resources/NullLayout.gif")); // NOI18N
    private static Image layoutIcon32 =
        Toolkit.getDefaultToolkit().getImage(
            AbstractLayoutSupport.class.getResource(
                "resources/NullLayout32.gif")); // NOI18N

    public Image getIcon(int type) {
        switch (type) {
            case BeanInfo.ICON_COLOR_16x16:
            case BeanInfo.ICON_MONO_16x16:
                return layoutIcon;
            default:
                return layoutIcon32;
        }
    }

    public Class getLayoutClass() {
        return null;
    }

    public void initialize(RADVisualContainer container) {
        super.initialize(container);

        Container cont = container.getContainerDelegate(
            container.getBeanInstance());
        cont.setLayout(null);
    }
    
    public void addComponent(RADVisualComponent component, ConstraintsDesc desc) {
        if (desc == null)
            desc = new AbsoluteConstraintsDesc(0, 0, -1, -1);

        Container cont = getContainer().getContainerDelegate(
                            getContainer().getBeanInstance());
        if (cont != null) {
            Component comp = component.getComponent();
//            Rectangle bounds = ((AbsoluteConstraintsDesc)desc).getBounds();
//            if (bounds.width == -1)
//                bounds.width = comp.getPreferredSize().width;
//            if (bounds.height == -1)
//                bounds.height = comp.getPreferredSize().height;
            cont.add(comp);
//            comp.setBounds(bounds);
        }

        component.setConstraintsDesc(AbsoluteLayoutSupport.class, desc);
    }

    public ConstraintsDesc getConstraints(RADVisualComponent component) {
        return (ConstraintsDesc)
               component.getConstraintsDesc(AbsoluteLayoutSupport.class);
    }

    public String getJavaSetLayoutString() {
        StringBuffer buf = new StringBuffer();
        String delegate = getContainer().getJavaContainerDelegateString();
        if (!"".equals(delegate)) { // NOI18N
            buf.append(delegate);
            buf.append("."); // NOI18N
        }
        buf.append("setLayout(null);\n"); // NOI18N
        return buf.toString();
    }

    public String getJavaAddComponentString(RADVisualComponent comp) {
        StringBuffer buf = new StringBuffer();
        String compName = comp.getName();

        String delegate = getContainer().getJavaContainerDelegateString();
        if (!"".equals(delegate)) { // NOI18N
            buf.append(delegate);
            buf.append("."); // NOI18N
        }

        buf.append("add("); // NOI18N
        buf.append(compName);
        buf.append(");\n"); // NOI18N

        ConstraintsDesc desc = getConstraints(comp);
        if (desc instanceof AbsoluteConstraintsDesc) {
            buf.append(compName);

            Node.Property[] props = desc.getProperties();
            FormProperty xProp = (FormProperty) props[0];
            FormProperty yProp = (FormProperty) props[1];
            FormProperty wProp = (FormProperty) props[2];
            FormProperty hProp = (FormProperty) props[3];

            buf.append(".setBounds("); // NOI18N
            buf.append(xProp.getJavaInitializationString());
            buf.append(", "); // NOI18N
            buf.append(yProp.getJavaInitializationString());
            buf.append(", "); // NOI18N

            Dimension pref = null;

            if (wProp.isChanged())
                buf.append(wProp.getJavaInitializationString());
            else {
                if (pref == null)
                    pref = comp.getComponent().getPreferredSize();
                buf.append(Integer.toString(pref.width));
            }
            buf.append(", "); // NOI18N

            if (hProp.isChanged())
                buf.append(hProp.getJavaInitializationString());
            else {
                if (pref == null)
                    pref = comp.getComponent().getPreferredSize();
                buf.append(Integer.toString(pref.height));
            }

            buf.append(");\n"); // NOI18N
        }

        return buf.toString();
    }
}
