/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.form.layoutsupport.dedicated;

import java.awt.*;
import javax.swing.*;
import java.beans.*;

import org.openide.nodes.*;
import org.openide.explorer.propertysheet.editors.EnhancedPropertyEditor;
import org.netbeans.modules.form.*;
import org.netbeans.modules.form.layoutsupport.*;

/**
 * @author Tomas Pavek
 */

public class JLayeredPaneSupport extends AbsoluteLayoutSupport {

    public Class getLayoutClass() {
        return null;
    }

    public void addComponent(RADVisualComponent component, ConstraintsDesc desc) {
        if (desc == null)
            desc = new LayeredConstraintsDesc(0, 0, 0, -1, -1);
        super.addComponent(component, desc);

        Component comp = component.getComponent();
        Rectangle bounds = ((AbsoluteConstraintsDesc)desc).getBounds();
        if (bounds.width == -1)
            bounds.width = comp.getPreferredSize().width;
        if (bounds.height == -1)
            bounds.height = comp.getPreferredSize().height;
        comp.setBounds(bounds);
    }

    public ConstraintsDesc[] convertConstraints(ConstraintsDesc[] constraints,
                                                Component[] components) {
        return null; // not needed here
    }

    public String getJavaAddComponentString(RADVisualComponent comp) {
        StringBuffer buf = new StringBuffer(
                               super.getJavaAddComponentString(comp));

        ConstraintsDesc desc = getConstraints(comp);
        if (desc instanceof LayeredConstraintsDesc) {
            LayeredConstraintsDesc lcd = (LayeredConstraintsDesc) desc;
            lcd.comp = comp;
            buf.append(lcd.getAdditionalJavaInitString());
        }

        return buf.toString();
    }

    // ----------

    protected ConstraintsDesc createNewConstraints(ConstraintsDesc currentConstr,
                                                   int x, int y, int w, int h) {
        int layer = currentConstr instanceof LayeredConstraintsDesc ?
                    ((LayeredConstraintsDesc)currentConstr).getLayer() : 0;

        return new LayeredConstraintsDesc(layer, x, y, w, h);
    }

    // ----------

    public static class LayeredConstraintsDesc
            extends AbsoluteConstraintsDesc
    {
        private int layer;

        transient private RADVisualComponent comp;

        public LayeredConstraintsDesc(int layer, int x, int y, int w, int h) {
            super(x, y, w, h);
            this.layer = layer;
        }

        public int getLayer() {
            return layer;
        }

        // ------

        public Object getConstraintsObject() {
            return new Integer(layer);
        }

        public String getJavaInitializationString() {
            StringBuffer buf = new StringBuffer();
            FormProperty layerProp = (FormProperty) getProperties()[0];
            buf.append(layerProp.getJavaInitializationString());
            return buf.toString();
        }

        // -------

        public String getAdditionalJavaInitString() {
            if (comp == null) return null;

            StringBuffer buf = new StringBuffer();
            buf.append(comp.getName());

            Node.Property[] props = getProperties();
            FormProperty xProp = (FormProperty) props[1];
            FormProperty yProp = (FormProperty) props[2];
            FormProperty wProp = (FormProperty) props[3];
            FormProperty hProp = (FormProperty) props[4];

            buf.append(".setBounds("); // NOI18N
            buf.append(xProp.getJavaInitializationString());
            buf.append(", "); // NOI18N
            buf.append(yProp.getJavaInitializationString());
            buf.append(", "); // NOI18N

            Dimension pref = null;

            if (wProp.isChanged())
                buf.append(wProp.getJavaInitializationString());
            else {
                if (pref == null)
                    pref = comp.getComponent().getPreferredSize();
                buf.append(Integer.toString(pref.width));
            }
            buf.append(", "); // NOI18N

            if (hProp.isChanged())
                buf.append(hProp.getJavaInitializationString());
            else {
                if (pref == null)
                    pref = comp.getComponent().getPreferredSize();
                buf.append(Integer.toString(pref.height));
            }

            buf.append(");\n"); // NOI18N
            return buf.toString();
        }

        protected Node.Property[] createProperties() {
            Node.Property[] props = super.createProperties();
            Node.Property[] layeredProps = new Node.Property[props.length + 1];

            layeredProps[0] =
                new FormProperty("layer", // NOI18N
                                 Integer.TYPE,
                             getBundle().getString("PROP_layer"), // NOI18N
                             getBundle().getString("HINT_layer")) { // NOI18N

                    public Object getTargetValue() {
                        return new Integer(layer);
                    }
                    public void setTargetValue(Object value) {
                        layer = ((Integer)value).intValue();
                    }
                    public boolean supportsDefaultValue () {
                        return true;
                    }
                    public Object getDefaultValue() {
                        return new Integer(0);
                    }
                    public PropertyEditor getExpliciteEditor() {
                        return new LayerEditor();
                    }
                };

            for (int i=0; i < props.length; i++)
                layeredProps[i+1] = props[i];

            return layeredProps;
        }
    }

    // ---------

    public static final class LayerEditor extends PropertyEditorSupport
                                          implements EnhancedPropertyEditor {
        final String[] tags = {
            "DEFAULT_LAYER", // NOI18N
            "PALETTE_LAYER", // NOI18N
            "MODAL_LAYER", // NOI18N
            "POPUP_LAYER", // NOI18N
            "DRAG_LAYER" // NOI18N
        };

        final Integer[] values = {
            JLayeredPane.DEFAULT_LAYER,
            JLayeredPane.PALETTE_LAYER,
            JLayeredPane.MODAL_LAYER,
            JLayeredPane.POPUP_LAYER,
            JLayeredPane.DRAG_LAYER
        };

        final String[] javaInitStrings = {
            "javax.swing.JLayeredPane.DEFAULT_LAYER", // NOI18N
            "javax.swing.JLayeredPane.PALETTE_LAYER", // NOI18N
            "javax.swing.JLayeredPane.MODAL_LAYER", // NOI18N
            "javax.swing.JLayeredPane.POPUP_LAYER", // NOI18N
            "javax.swing.JLayeredPane.DRAG_LAYER" // NOI18N
        };

        public String[] getTags() {
            return tags;
        }

        public String getAsText() {
            Object value = getValue();
            for (int i=0; i < values.length; i++)
                if (values[i].equals(value))
                    return tags[i];

            return value.toString();
        }

        public void setAsText(String str) {
            for (int i=0; i < tags.length; i++)
                if (tags[i].equals(str)) {
                    setValue(values[i]);
                    return;
                }

            try {
                setValue(new Integer(Integer.parseInt(str)));
            } 
            catch (NumberFormatException e) {} // ignore
        }

        public String getJavaInitializationString() {
            Object value = getValue();
            for (int i=0; i < values.length; i++)
                if (values[i].equals(value))
                    return javaInitStrings[i];

            return value != null ? 
                       "new Integer(" + value.toString() + ")" // NOI18N
                       : null;
        }

        public Component getInPlaceCustomEditor() {
            return null;
        }
        public boolean hasInPlaceCustomEditor() {
            return false;
        }
        public boolean supportsEditingTaggedValues() {
            return true;
        }
    }
}
