/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.form.layoutsupport.dedicated;

import java.awt.*;
import javax.swing.*;

import org.netbeans.modules.form.*;
import org.netbeans.modules.form.layoutsupport.*;
import org.netbeans.modules.form.fakepeer.FakePeerSupport;

/**
 * @author Tomas Pavek
 */

public class JScrollPaneSupport extends AbstractLayoutSupport {

    public Class getLayoutClass() {
        return null;
    }

    public int getNewIndex(Container container, Point posInCont,
                           Component component, Point posInComp) {
        if (container.getComponentCount() > 1)
            return -1;
        return 0;
    }

    public boolean paintDragFeedback(Container container,
                                     Component component,
                                     ConstraintsDesc newConstraints,
                                     int newIndex,
                                     Graphics g) {
        Dimension sz = container.getSize();
        Insets insets = container.getInsets();
        sz.width -= insets.left + insets.right;
        sz.height -= insets.top + insets.bottom;
        
        g.drawRect(0, 0, sz.width, sz.height);
        return true;
    }

    public void addComponent(RADVisualComponent component, ConstraintsDesc desc) {
        Object cont = getContainer().getBeanInstance();
        if (cont instanceof JScrollPane) {
            ((JScrollPane)cont).setViewportView(component.getComponent());
        }
        else if (cont instanceof ScrollPane) {
            ScrollPane scroll = (ScrollPane) cont;
            Component removedComp = null;
            if (scroll.getComponentCount() > 0)
                removedComp = scroll.getComponent(0);
            scroll.add(component.getComponent());
            // hack for AWT components - we must attach the fake peer again
            // to the component that was removed by adding new component
            ensureFakePeerAttached(removedComp);
        }
    }

    public void removeComponent(RADVisualComponent component) {
        RADVisualComponent[] comps = getContainer().getSubComponents();
        int i = getContainer().getIndexOf(component);
        if (i < comps.length-1)
            i = comps.length - 1;
        else if (i > 0)
            i = 0;
        else i = -1;

        Component setComp;
        setComp = i != -1 ? comps[i].getComponent() : null;

        Object cont = getContainer().getBeanInstance();
        if (cont instanceof JScrollPane) {
            ((JScrollPane)cont).setViewportView(setComp);
        }
        else if (cont instanceof ScrollPane) {
            ScrollPane scroll = (ScrollPane) cont;
            Component removedComp = component.getComponent();
            if (setComp != null)
                scroll.add(setComp);
            else
                scroll.remove(removedComp);
            // hack for AWT components - we must attach the fake peer again
            // to the removed component
            ensureFakePeerAttached(removedComp);
        }
    }

    public String getJavaAddComponentString(RADVisualComponent comp) {
        StringBuffer buf = new StringBuffer();

        String delegate = getContainer().getJavaContainerDelegateString();
        if (!"".equals(delegate)) { // NOI18N
            buf.append(getContainer().getName());
            buf.append("."); // NOI18N
        }

        if (getContainer().getBeanInstance() instanceof JScrollPane)
            buf.append("setViewportView("); // NOI18N
        else
            buf.append("add("); // NOI18N
        buf.append(comp.getName());
        buf.append(");\n"); // NOI18N

        return buf.toString();
    }
}
