/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.form.layoutsupport.dedicated;

import java.awt.*;
import java.beans.*;
import java.util.*;
import javax.swing.JSplitPane;

import org.openide.nodes.Node;
import org.netbeans.modules.form.*;
import org.netbeans.modules.form.layoutsupport.*;

/**
 *
 * @author Tomas Pavek
 */

public class JSplitPaneSupport extends AbstractLayoutSupport {

    public Class getLayoutClass() {
        return null;
    }
    
    public ConstraintsDesc getNewConstraints(Container container,
                                             Point posInCont,
                                             Component component,
                                             Point posInComp) {
        if (!(container instanceof JSplitPane))
            return null;

        JSplitPane splitPane = (JSplitPane) container;
        Dimension sz = splitPane.getSize();
        int orientation = splitPane.getOrientation();
        Component left = splitPane.getLeftComponent();
        Component right = splitPane.getRightComponent();

        String freePosition = findFreePosition();
        
        if (left == null || right == null) {
            if (orientation == JSplitPane.HORIZONTAL_SPLIT) {
                if (posInCont.x <= sz.width / 2)
                    freePosition = JSplitPane.LEFT;
                else
                    freePosition = JSplitPane.RIGHT;
            }
            else {
                if (posInCont.x <= sz.height / 2)
                    freePosition = JSplitPane.TOP;
                else
                    freePosition = JSplitPane.BOTTOM;
            }
        }

        return new SplitConstraintsDesc(freePosition);
    }

    public boolean paintDragFeedback(Container container,
                                     Component component,
                                     ConstraintsDesc newConstraints,
                                     int newIndex,
                                     Graphics g) {
        if (!(container instanceof JSplitPane))
            return false;

        String position = (String) newConstraints.getConstraintsObject();
        if (position == null)
            return false;
        
        JSplitPane splitPane = (JSplitPane) container;
        int orientation = splitPane.getOrientation();

        Dimension sz = splitPane.getSize();
        Insets insets = container.getInsets();
        sz.width -= insets.left + insets.right;
        sz.height -= insets.top + insets.bottom;

        Rectangle rect = new Rectangle(insets.left, insets.top, sz.width, sz.height);

        if (orientation == JSplitPane.HORIZONTAL_SPLIT) {
            Component left = splitPane.getLeftComponent();
            Component right = splitPane.getRightComponent();
            
            if (position == JSplitPane.LEFT) {
                if (right == null) {
                    rect.width = sz.width / 2;
                }
                else {
                    rect.width = right.getBounds().x - rect.x;
                }
            }
            else {
                if (left == null) {
                    rect.x = insets.left + sz.width / 2;
                    rect.width = sz.width - rect.x;
                }
                else {
                    rect.x = left.getBounds().x + left.getBounds().width;
                    rect.width = sz.width - rect.x;
                }
            }
        }
        else {
            Component top = splitPane.getTopComponent();
            Component bottom = splitPane.getBottomComponent();
            
            if (position == JSplitPane.TOP) {
                if (bottom == null) {
                    rect.height /= 2;
                }
                else {
                    rect.height = bottom.getBounds().y - rect.y;
                }
            }
            else {
                if (top == null) {
                    rect.y = insets.top + sz.height / 2;
                    rect.height = sz.height - rect.y;
                }
                else {
                    rect.y = top.getBounds().y + top.getBounds().height;
                    rect.height = sz.height - rect.y;
                }
            }
        }
        g.drawRect(rect.x, rect.y, rect.width, rect.height);
        return true;
    }

    public void addComponent(RADVisualComponent component, ConstraintsDesc desc) {
        if (desc == null)
            desc = new SplitConstraintsDesc(findFreePosition());
        component.setConstraintsDesc(getClass(), desc);

        Container cont =
            getContainer().getContainerDelegate(getContainer().getBeanInstance());

        if (cont instanceof JSplitPane) {
            int descPos = convertPosition(desc);
            if (descPos == 0)
                ((JSplitPane)cont).setLeftComponent(component.getComponent());
            else if (descPos == 1)
                ((JSplitPane)cont).setRightComponent(component.getComponent());
        }
    }

    public void removeComponent(RADVisualComponent component) {
        ConstraintsDesc desc = getConstraints(component);
        if (!(desc instanceof SplitConstraintsDesc)) return;
        int descPos = convertPosition(desc);

        Component comp = null;
        RADVisualComponent[] comps = getContainer().getSubComponents();
        for (int i= comps.length-1; i >= 0; i--) {
            if (comps[i] == component) continue;
            ConstraintsDesc cd = getConstraints(comps[i]);
            if (!(cd instanceof SplitConstraintsDesc)) continue;
            if (descPos == convertPosition(cd)) {
                comp = comps[i].getComponent();
                break;
            }
        }

        Container cont =
            getContainer().getContainerDelegate(getContainer().getBeanInstance());

        if (cont instanceof JSplitPane) {
            if (descPos == 0)
                ((JSplitPane)cont).setLeftComponent(comp);
            else if (descPos == 1)
                ((JSplitPane)cont).setRightComponent(comp);
        }
    }

    public String getJavaAddComponentString(/*RADVisualContainer cont,*/
                                            RADVisualComponent comp) {
        StringBuffer buf = new StringBuffer();

        String delegate = getContainer().getJavaContainerDelegateString();
        if (!"".equals(delegate)) { // NOI18N
            buf.append(delegate);
            buf.append("."); // NOI18N
        }

        ConstraintsDesc desc = getConstraints(comp);
        Object position = desc instanceof SplitConstraintsDesc ?
            desc.getConstraintsObject() : findFreePosition();

        if (JSplitPane.LEFT.equals(position))
            buf.append("setLeftComponent("); // NOI18N
        else if (JSplitPane.RIGHT.equals(position))
            buf.append("setRightComponent("); // NOI18N
        else if (JSplitPane.TOP.equals(position))
            buf.append("setTopComponent("); // NOI18N
        else if (JSplitPane.BOTTOM.equals(position))
            buf.append("setBottomComponent("); // NOI18N
        else
            buf.append("add("); // NOI18N

        buf.append(comp.getName());
        buf.append(");\n"); // NOI18N

        return buf.toString();
    }

    // ------------

    private int convertPosition(ConstraintsDesc desc) {
        Object position = desc.getConstraintsObject();
        if (JSplitPane.LEFT.equals(position) || JSplitPane.TOP.equals(position))
            return 0;
        if (JSplitPane.RIGHT.equals(position) || JSplitPane.BOTTOM.equals(position))
            return 1;
        return -1;
    }

    private String findFreePosition() {
        int leftTop = 0, rightBottom = 0;
        int orientation = JSplitPane.HORIZONTAL_SPLIT;
        RADVisualComponent[] comps = getContainer().getSubComponents();

        for (int i=0; i < comps.length; i++) {
            ConstraintsDesc desc = getConstraints(comps[i]);
            if (!(desc instanceof SplitConstraintsDesc)) continue;

            int descPos = convertPosition(desc);
            if (descPos == 0) leftTop++;
            else if (descPos == 1) rightBottom++;
        }

        if (leftTop == 0 || leftTop < rightBottom)
            return orientation == JSplitPane.HORIZONTAL_SPLIT ?
                JSplitPane.LEFT : JSplitPane.TOP;
        else
            return orientation == JSplitPane.HORIZONTAL_SPLIT ?
                JSplitPane.RIGHT : JSplitPane.BOTTOM;
    }

    // -----------

    public static class SplitConstraintsDesc
            implements LayoutSupport.ConstraintsDesc
    {
        private String position;

        transient private Node.Property[] properties;

        public SplitConstraintsDesc(String position) {
            this.position = position;
        }

        public Node.Property[] getProperties() {
            if (properties == null)
                properties = new Node.Property[] {
                    new FormProperty("splitPosition", // NOI18N
                                     String.class,
                                 getBundle().getString("PROP_splitPos"), // NOI18N
                                 getBundle().getString("HINT_splitPos")) { // NOI18N

                        public Object getTargetValue() {
                            return position;
                        }

                        public void setTargetValue(Object value) {
                            position = (String)value;
                        }

                        public PropertyEditor getExpliciteEditor() {
                            return new SplitPositionEditor();
                        }
                    }
                };

            return properties;
        }

        public Object getConstraintsObject() {
            return position;
        }

        public String getJavaInitializationString() {
            return null;
        }

        public void readFromXML(org.w3c.dom.Node element)
        throws java.io.IOException {
        }

        public org.w3c.dom.Node storeToXML(org.w3c.dom.Document doc) {
            return null;
        }
    }

    static class SplitPositionEditor extends PropertyEditorSupport {
        private final String[] values = {
            JSplitPane.LEFT,
            JSplitPane.RIGHT,
            JSplitPane.TOP,
            JSplitPane.BOTTOM
        };

        public String[] getTags() {
            return values;
        }

        public String getAsText() {
            return (String)getValue();
        }

        public void setAsText(String str) {
            for (int i = 0; i < values.length; i++)
                if (str.equals(values[i])) {
                    setValue(str);
                    break;
                }
        }
    }
}
