/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.form.palette;

import java.awt.*;
import javax.swing.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.datatransfer.*;
import java.util.ResourceBundle;
import java.text.MessageFormat;

import org.openide.util.datatransfer.*;
import org.openide.awt.JPopupMenuPlus;
import org.openide.nodes.*;
import org.openide.*;


/** Popup menu for palette category.
 *
 * @author Tomas Pavek
 */
class CategoryPopupMenu extends JPopupMenuPlus {

    private Node catNode;

    public CategoryPopupMenu(Node node) {
        catNode = node;

        JMenuItem menuItem;
        Node parent = catNode.getParentNode();
        Index indexCookie = parent != null ?
                        (Index)parent.getCookie(Index.class) : null;

        ResourceBundle bundle = ComponentPalette.getBundle();

        menuItem = new JMenuItem(bundle.getString("CTL_MoveLeft")); // NOI18N
        menuItem.setEnabled(indexCookie != null
                                && indexCookie.indexOf(catNode) > 0);
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                moveLeft();
            }
        });
        add(menuItem);

        menuItem = new JMenuItem(bundle.getString("CTL_MoveRight")); // NOI18N
        if (indexCookie != null) {
            int ind = indexCookie.indexOf(catNode);
            menuItem.setEnabled(ind >= 0 && ind+1 < indexCookie.getNodesCount());
        }
        else menuItem.setEnabled(false);
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                moveRight();
            }
        });
        add(menuItem);

/*        addSeparator();

        menuItem = new JMenuItem(bundle.getString("CTL_OrderCategories")); // NOI18N
        menuItem.setEnabled(parent.getCookie(Index.class) != null);
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                reorderCategories();
            }
        });
        add(menuItem); */

        menuItem = new JMenuItem(bundle.getString("CTL_OrderBeans")); // NOI18N
        menuItem.setEnabled(catNode.getCookie(Index.class) != null);
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                reorderBeans();
            }
        });
        add(menuItem);

        addSeparator();

        menuItem = new JMenuItem(bundle.getString("CTL_Paste")); // NOI18N
        menuItem.setEnabled(getPasteType() != null);
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                pasteBean();
            }
        });
        add(menuItem);

        addSeparator();

/*        menuItem = new JMenuItem(bundle.getString("CTL_CreateCategory")); // NOI18N
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                createCategory();
            }
        });
        add(menuItem); */

        menuItem = new JMenuItem(bundle.getString("CTL_DeleteCategory")); // NOI18N
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                deleteNode();
            }
        });
        add(menuItem);

        menuItem = new JMenuItem(bundle.getString("CTL_RenameCategory")); // NOI18N
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                renameNode();
            }
        });
        add(menuItem);
    }

    private void pasteBean() {
        PasteType type = getPasteType();
        if (type != null) {
            try {
                Transferable trans = type.paste();
                if (trans != null) {
                    ClipboardOwner owner = trans instanceof ClipboardOwner ?
                                             (ClipboardOwner)trans :
                                             new StringSelection(""); // NOI18N
                    Clipboard clipboard = TopManager.getDefault().getClipboard();
                    clipboard.setContents(trans, owner);
                }
            }
            catch (java.io.IOException e) {
                if (Boolean.getBoolean("netbeans.debug.exceptions")) // NOI18N
                    e.printStackTrace();
                TopManager.getDefault().getErrorManager().notify(e);
            }
        }
    }

    private PasteType getPasteType() {
        Transferable trans = TopManager.getDefault().getClipboard().getContents(
                                                       TopManager.getDefault());
        if (trans != null) {
            PasteType[] pasteTypes = catNode.getPasteTypes(trans);
            if (pasteTypes != null && pasteTypes.length != 0)
                return pasteTypes[0];
        }
        return null;
    }

    private void moveLeft() {
        Node parent = catNode.getParentNode();
        if (parent == null) return;
        Index indexCookie = (Index)parent.getCookie(Index.class);
        if (indexCookie == null) return;
        int ind = indexCookie.indexOf(catNode);
        if (indexCookie != null && indexCookie.indexOf(catNode) > 0)
            indexCookie.moveUp(ind);
    }

    private void moveRight() {
        Node parent = catNode.getParentNode();
        if (parent == null)
            return;
        Index indexCookie = (Index)parent.getCookie(Index.class);
        if (indexCookie == null)
            return;
        int ind = indexCookie.indexOf(catNode);
        if (ind >=0 && ind+1 < indexCookie.getNodesCount())
            indexCookie.moveDown(ind);
    }

/*    private void reorderCategories() {
        Node parent = catNode.getParentNode();
        Index order = parent != null ? 
                          (Index)parent.getCookie(Index.class) : null;
        if (order != null)
            order.reorder();
    } */

    private void reorderBeans() {
        Index order = (Index)catNode.getCookie(Index.class);
        if (order != null)
            order.reorder();
    }

/*    private void createCategory() {
        try {
            PaletteNode.getPaletteNode().createNewCategory();
        }
        catch (java.io.IOException e) {
            if (Boolean.getBoolean("netbeans.debug.exceptions")) // NOI18N
                e.printStackTrace();
            TopManager.getDefault().getErrorManager().notify(e);
        }
    } */

    private void deleteNode() {
        // first user confirmation...
        String message = MessageFormat.format(
            ComponentPalette.getBundle().getString("FMT_ConfirmCategoryDelete"), // NOI18N
            new Object [] { catNode.getName() });

        NotifyDescriptor desc = new NotifyDescriptor.Confirmation(message,
                    ComponentPalette.getBundle().getString("CTL_ConfirmCategoryTitle"), // NOI18N
                    NotifyDescriptor.YES_NO_OPTION);

        if (NotifyDescriptor.YES_OPTION.equals(TopManager.getDefault().notify(desc))) {
            try {
                catNode.destroy();
            } 
            catch (java.io.IOException e) {
                if (Boolean.getBoolean("netbeans.debug.exceptions")) // NOI18N
                    e.printStackTrace();
                TopManager.getDefault().getErrorManager().notify(e);
            }
        }
    }

    private void renameNode() {
        ResourceBundle bundle = ComponentPalette.getBundle();
        NotifyDescriptor.InputLine dlg = new NotifyDescriptor.InputLine(
                                           bundle.getString("CTL_NewName"), // NOI18N
                                           bundle.getString("CTL_Rename")); // NOI18N
        dlg.setInputText(catNode.getName());

        if (NotifyDescriptor.OK_OPTION.equals(TopManager.getDefault().notify(dlg))) {
            String newName = null;
            try {
                newName = dlg.getInputText();
                if (!"".equals(newName)) // NOI18N
                    catNode.setName(newName);
            }
            catch (IllegalArgumentException e) {
                if (Boolean.getBoolean("netbeans.debug.exceptions")) // NOI18N
                    e.printStackTrace();
            }
        }
    }
}
