/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.form.palette;

import org.openide.util.HelpCtx;
import org.openide.util.actions.SystemAction;
import org.openide.util.actions.Presenter;
import org.openide.nodes.Node;

/**
 * The PaletteAction is an action that has a state represented by a PaletteNode
 * value. The state is a PaletteNode that represents a Javabean to be added as
 * a new component to the Form, or null for Selection mode. R/W property
 * addMode representates this state...  PaletteAction has table of all
 * currently used DesignLayouts, and It can find proper DesignLayout for given
 * class of LayoutManager(see method getDesignLayout()).
 *
 * @author   Ian Formanek, Jan Jancura
 */

public class PaletteAction
    extends SystemAction implements Presenter.Toolbar //, Presenter.Menu
{
    /** generated Serialized Version UID */
    static final long serialVersionUID = -6466826799827497471L;

    /** In selection mode, clicking on a form selects components */
    public final static int MODE_SELECTION = 0;

    /** In connection mode, clicking on a form connects components(event ->
     * method,property) */
    public final static int MODE_CONNECTION = 1;
    
    /** In add mode, clicking on a form adds a new component to the form */
    public final static int MODE_ADD = 2;

    // SystemAction interface

    /** Human presentable name of the action. This should be presented as an
     * item in a menu.
     * @return the name of the action
     */
    public String getName() {
        return org.openide.util.NbBundle.getBundle(PaletteAction.class).getString("CTL_Component_palette");
    }

    /** Help context where to find more about the action.
     * @return the help context for this action
     */
    public HelpCtx getHelpCtx() {
        return new HelpCtx("gui.components.palette"); // NOI18N
    }

    /** @return resource for the action icon */
//    protected String iconResource() {
//        return "/org/netbeans/modules/form/resources/paletteAction.gif"; // NOI18N
//    }

    /** Returns a JMenuItem that presents the Action, that implements this
     * interface, in a MenuBar.
     * @return the JMenuItem representation for the Action
     */
    public javax.swing.JMenuItem getMenuPresenter() {
        return null;
    }

    /** Returns a Component that presents the Action, that implements this
     * interface, in a ToolBar.
     * @return the Component representation for the Action
     */
    public java.awt.Component getToolbarPresenter() {
        return ComponentPalette.getDefault();
    }


    // Palette methods ...................................................................

    /** Getter method for the paletteMode property
     * @return Current Mode property value - one of MODE_SELECTION, MODE_CONNECTION, MODE_ADD constants
     *
     public int getPaletteMode() {
     return getComponentPalette().getMode();
     }

     /** Setter method for the paletteMode property
     * @param mode the desired new Mode property value - one of MODE_SELECTION, MODE_CONNECTION, MODE_ADD constants
     *
     public void setPaletteMode(int mode) {
     getComponentPalette().setMode(mode);
     }

     /** Do nothing. */
    public void actionPerformed(java.awt.event.ActionEvent ev) {}

    /** Getter method for the AddComponent property.
     * @return Current Mode property value
     */
    /*  public PaletteNode getAddComponent() {
        return getComponentPalette().getAddNode();
        } */

    /** Sets the node that would be added as a new component to the form
     * @param value The PaletteNode that represents the addComponent
     *              or null for selection mode.
     */
    /*  public void setAddComponent(PaletteNode value) {
        getComponentPalette().setAddNode(value);
        } */

    /** Returns the proper DesignLayout class for the LayoutManager class.
     * @param layoutManagerClass The DesignLayout class for whis the LayoutManager class is looking
     *  for.
     * @return Class of proper DesignLayout.
     */
    /*  public Class getDesignLayout(Class layoutManagerClass) {
        return PaletteContext.getPaletteContext(). getDesignLayout(layoutManagerClass);
        } */

    /** Returns the proper DesignLayout class for the LayoutManager class.
     * @param layoutManagerClass The DesignLayout class for whis the LayoutManager class is looking
     *  for.
     * @return Class of proper DesignLayout.
     */
    /*  public java.awt.Image getIconForClass(Class cl, int iconType) {
        PaletteNode node = getComponentPalette().getNodeForClass(cl);
        if (node == null)
        return null;
        return node.getIcon(iconType);
        } */

}
