/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.form.palette;

import java.awt.*;
import java.beans.*;
import javax.swing.*;

import org.openide.*;
import org.openide.awt.*;
import org.openide.loaders.InstanceSupport;
import org.openide.nodes.*;
import org.openide.util.HelpCtx;
import org.openide.util.WeakListener;
import org.openide.util.RequestProcessor;

/**
 * Button that displays on node in component palette.
 * @author Jaroslav Tulach
 */

class PaletteButton extends ToolbarToggleButton
    implements NodeListener //, Runnable
{
    static final long serialVersionUID = -8524018463476326691L;

    private PaletteItem palItem;

    /** Creates new PaletteButton */

    public PaletteButton(PaletteItem palItem) {
        super(new ImageIcon(palItem.getItemNode().getIcon(BeanInfo.ICON_COLOR_16x16)),
              false);
        this.palItem = palItem;

        setMargin(new Insets(1, 1, 0, 1));
        setToolTipText(palItem.getDisplayName().replace('-', '.'));

        // listening to change of icon
        palItem.getItemNode().addNodeListener(WeakListener.node(this, 
                                                       palItem.getItemNode()));
        
        HelpCtx.setHelpIDString(this, "gui.components.adding"); // NOI18N
//        RequestProcessor.postRequest(this);
    }

    /** Later initialization of the help. */

/*    public void run() {
        // JST: Found too deadlocking ;-) => moved to request processor
        // help
        // The following may be commented out if found to be slow(should not be):

        String help = null;
        if (!palItem.getItemClass().getName().startsWith(
                "org.netbeans.modules.form.compat2")) {
            HelpCtx hc = InstanceSupport.findHelp(palItem.getInstanceCookie());

            if (hc != null) {
                help = hc.getHelpID();
            }
        }

        if (help == null) {
            help = ComponentPalette.class.getName() + ".paletteItem"; // NOI18N
        }

        HelpCtx.setHelpIDString(this, help);
    } */

    PaletteItem getItem() {
        return palItem;
    }

    Node getItemNode() {
        return palItem.getItemNode(); 
    }

    Node getCategoryNode() {
        return palItem.getCategoryNode();
    }

//    public String getToolTipText() {
//        return palItem.getDisplayName();
//    }

    public void propertyChange(PropertyChangeEvent ev) {
        if (Node.PROP_ICON.equals(ev.getPropertyName())) {
            setIcon(new ImageIcon(palItem.getItemNode().getIcon(
                                                  BeanInfo.ICON_COLOR_16x16)));
        }
    }

    public void childrenAdded(NodeMemberEvent ev) {
    }
    public void childrenRemoved(NodeMemberEvent ev) {
    }
    public void childrenReordered(NodeReorderEvent ev) {
    }
    public void nodeDestroyed(NodeEvent ev) {
    }
}
