/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.form.palette;

import org.openide.actions.*;
import org.openide.cookies.InstanceCookie;
import org.openide.loaders.*;
import org.openide.nodes.*;
import org.openide.util.actions.SystemAction;
import org.netbeans.modules.form.BeanSupport;

/** The PaletteItemNode is a Node representing the ComponentPaletteItem
 * in the tree under Environment.
 *
 * @author   Ian Formanek
 */
public class PaletteItemNode extends FilterNode {
    /** generated Serialized Version UID */
    //  static final long serialVersionUID = -2098259549820241091L;

    /** Name of the template property. */
    public static final String PROP_IS_CONTAINER = "isContainer"; // NOI18N



    // -----------------------------------------------------------------------------
    // Static variables

    private static SystemAction[] staticActions;

    // -----------------------------------------------------------------------------
    // Constructors

    /** Creates a new palette node */
    public PaletteItemNode(Node original) {
        super(original, Children.LEAF);
    }

    // -----------------------------------------------------------------------------
    // Other methods

    public String getDisplayName() {
        String expName = getExplicitDisplayName();
        if (expName != null)
            return expName;

        InstanceCookie ic =
            (InstanceDataObject)getCookie(InstanceDataObject.class);
        return ic != null ? ic.instanceName() : super.getDisplayName();
    }

    String getExplicitDisplayName() {
        String name = getOriginal().getName();
        String displayName = getOriginal().getDisplayName();
        if (DataNode.getShowFileExtensions()) {
            DataObject obj = (DataObject)getOriginal().getCookie(DataObject.class);
            if (obj != null) {
                String ext = "." + obj.getPrimaryFile().getExt();
                if (displayName.endsWith(ext))
                    displayName = displayName.substring(0,
                                      displayName.length() - ext.length());
            }
        }
        return name.equals(displayName) ? null : displayName;
    }

    /** Creates properties for this node */
    public Node.PropertySet[] getPropertySets() {
        java.util.ResourceBundle bundle = org.openide.util.NbBundle.getBundle(PaletteItemNode.class);

        // default sheet with "properties" property set // NOI18N
        Sheet sheet = Sheet.createDefault();
        if (canBeContainer((InstanceCookie)getCookie(InstanceCookie.class))) {
            final DataObject obj =(DataObject)getCookie(DataObject.class);
            sheet.get(Sheet.PROPERTIES).put(
                new PropertySupport.ReadWrite(
                    PROP_IS_CONTAINER,
                    Boolean.TYPE,
                    bundle.getString("PROP_isContainer"),
                    bundle.getString("HINT_isContainer")
                    ) {
                    public Object getValue() {
                        Object val = obj.getPrimaryFile().getAttribute(PaletteItem.ATTR_IS_CONTAINER);
                        if (val instanceof Boolean) {
                            return val;
                        } else {
                            return Boolean.TRUE;
                        }
                    }

                    public void setValue(Object val) throws IllegalAccessException,
                                                            IllegalArgumentException, java.lang.reflect.InvocationTargetException {
                        if (!(val instanceof Boolean))
                            throw new IllegalArgumentException();
                        try {
                            if (!((Boolean)val).booleanValue()) {
                                obj.getPrimaryFile().setAttribute(PaletteItem.ATTR_IS_CONTAINER, val);
                            } else { // true is default
                                obj.getPrimaryFile().setAttribute(PaletteItem.ATTR_IS_CONTAINER, null);
                            }
                        } catch (java.io.IOException e) {
                            // silently ignore - the property is just not changed
                        }
                    }

                }
                );
        }
        return sheet.toArray();
    }

    static boolean canBeContainer(InstanceCookie ic) {
//        InstanceCookie ic =(InstanceCookie)getCookie(InstanceCookie.class);
        if (ic != null) {
            try {
                Class instClass = ic.instanceClass();
                if (java.awt.Container.class.isAssignableFrom(instClass)) {
                    Object isContainerValue = BeanSupport.createBeanInfo(instClass).getBeanDescriptor().getValue("isContainer"); // NOI18N
                    if (isContainerValue == null) return true; // containers without the isCOntainer special flag are real containers
                    if (isContainerValue instanceof Boolean) {
                        return((Boolean)isContainerValue).booleanValue();
                    }
                }
            } catch (Exception e) {
                // in such case return false
                return false;
            }
        }
        return false;
    }

    public boolean canRename() {
        // ".instance" nodes should not be renamed
        // ".shadow" nodes might be renamed
        DataObject dobj = (DataObject)getCookie(DataObject.class);
        return dobj != null && !(dobj instanceof InstanceDataObject);
    }

    /** Actions.
     * @return array of actions for this node
     */
    public SystemAction[] getActions() {
        if (staticActions == null)
            staticActions = new SystemAction [] {
                SystemAction.get(CustomizeBeanAction.class),
                null,
                SystemAction.get(MoveUpAction.class),
                SystemAction.get(MoveDownAction.class),
                null,
                SystemAction.get(CutAction.class),
                SystemAction.get(CopyAction.class),
                null,
                SystemAction.get(DeleteAction.class),
                null,
                SystemAction.get(ToolsAction.class),
                SystemAction.get(PropertiesAction.class),
            };
        return staticActions;
    }

}
