/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.httpserver;           

import java.io.InputStream;
import java.util.Enumeration;

import javax.servlet.*;
import javax.servlet.http.*;

import org.openide.execution.NbfsStreamHandlerFactory;
import org.openide.execution.NbfsURLConnection;
import org.openide.filesystems.*;
import org.openide.TopManager;
import org.openide.util.NbBundle;

/** This servlet is used to give access to documentation previously 
 *  displayed through nbfs: protocol
 *
 * @author  rkubacki
 * @version 
 */
public class JavadocServlet extends HttpServlet {
   
    /** Initializes the servlet.
    */  
    public void init(ServletConfig config) throws ServletException {
        super.init(config);
    }

    /** Destroys the servlet.
    */  
    public void destroy() {

    }

    /** Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
    * @param request servlet request
    * @param response servlet response
    */
    protected void processRequest (HttpServletRequest request, HttpServletResponse response)
    throws ServletException, java.io.IOException {
        response.setContentType("text/html");   // NOI18N
        java.io.PrintWriter out = response.getWriter();
        
        // output your page here
        String path = request.getPathInfo ();
        try {
            
            // resource name
            if (path.startsWith ("/")) path = path.substring (1); // NOI18N
            
            if (path.length () == 0) {
                // show all javadoc trees
                out.println ("<HTML><HEAD><TITLE>List of javadocs</TITLE></HEAD><BODY>");
                out.println ("<H2>List of javadocs:</H2>");
                Enumeration en = TopManager.getDefault ().getRepository ().getFileSystems ();
                while (en.hasMoreElements ()) {
                    FileSystem fs = (FileSystem)en.nextElement ();
                    if (fs.getCapability ().capableOf (FileSystemCapability.DOC))
                        out.println ("<P><A HREF=\""+HttpServerSettings.mangle (fs.getSystemName ())
                        //fs.getRoot ().getURL ()
                        +"/\">"+fs.getDisplayName ()+"</A></P>");
                }
                
                out.println ("</BODY></HTML>");
                out.close ();
                return;
            }
            else {

                // first part is FS name
                int first = path.indexOf ('/');
                if (first == -1)
                    throw new java.io.IOException ();

                String fileSystemName = HttpServerSettings.demangle (path.substring (0, first));
                path = path.substring (first);
                FileSystem fsys = TopManager.getDefault().getRepository ().findFileSystem(fileSystemName);
                FileObject fo = fsys.findResource (path);
                if (fo == null) 
                    throw new java.io.IOException ();

                if (fo.isFolder () || fo.isRoot ()) {
                    // show all javadoc trees
                    out.println ("<HTML><HEAD><TITLE>List of javadocs</TITLE></HEAD><BODY>");
                    out.println ("<TT>Directory listing:</TT>");
                    if (!fo.isRoot ())
                        out.println ("<P><A HREF=\"..\">Parent directory</A></P><BR>");
                        
                    Enumeration en = fo.getChildren (false);
                    while (en.hasMoreElements ()) {
                        FileObject child = (FileObject)en.nextElement ();
                        if (child.isFolder ())
                            out.println ("<P><A HREF=\""+child.getName ()+"/\">"+child.getName ()+"</A></P>");
                        else
                            out.println ("<P><A HREF=\""+child.getNameExt ()+"\">"+child.getNameExt ()+"</A></P>");
                    }

                    out.println ("</BODY></HTML>");
                    out.close ();
                    return;
                }
                else {
                    java.net.URLConnection conn = fo.getURL ().openConnection ();
                    InputStream in = conn.getInputStream ();
                    byte [] buff = new byte [256];
                    int len;

                    while ((len = in.read (buff)) != -1) {
                        out.print (new String (buff, 0, len));
                    }
                    in.close ();
                }
            }
        }
        catch (java.net.MalformedURLException ex) {
            ex.printStackTrace (out);
        }
        catch (java.io.IOException ex) {
            ex.printStackTrace (out);
        }
        
        out.close();
    }

    /** Handles the HTTP <code>GET</code> method.
    * @param request servlet request
    * @param response servlet response
    */
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, java.io.IOException {
        processRequest(request, response);
    } 

    /** Handles the HTTP <code>POST</code> method.
    * @param request servlet request
    * @param response servlet response
    */
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, java.io.IOException {
        processRequest(request, response);
    }

    /**
    * Returns a short description of the servlet.
    */
    public String getServletInfo() {
        return NbBundle.getBundle(ClasspathServlet.class).getString("MSG_JavadocServletDescr"); // NOI18N
    }

}