/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.i18n;


import java.awt.Component;
import javax.swing.JMenuItem;
import javax.swing.JToolBar;

import org.openide.util.actions.Presenter;
import org.openide.util.actions.SystemAction;


/** 
 * I18n group action which is always present under Menu | Tools and could be also in toolbar. 
 *
 * @author Peter Zavadsky
 */
public class I18nGroupMenuAction extends I18nGroupAction implements Presenter.Menu, Presenter.Toolbar {

    /** Generated serialVersionUID. */
    static final long serialVersionUID = 2721276246560040431L;
    

    /** Implements <code>Presenter.Menu</code> interface. */
    public JMenuItem getMenuPresenter() {
        return createLazyPopup(true);
    }

    /** Implements <code>Presenter.Toolbar</code> interface. */
    public Component getToolbarPresenter() {
        // In jdk1.3 could be used new JToolBar(getName()).
        JToolBar toolbar = new JToolBar ();
        toolbar.setName(getName());

        for(int i=0; i<i18nActions.length; i++) {
            SystemAction action = i18nActions[i];

            if(action == null) {
                toolbar.addSeparator();
            } else if(action instanceof Presenter.Toolbar) {
                toolbar.add(((Presenter.Toolbar)action).getToolbarPresenter());
            }
        }

        return toolbar;
    }
} // End of Menu class.

