/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.i18n;


import java.awt.Image;
import java.beans.BeanDescriptor;
import java.beans.BeanInfo;
import java.beans.Introspector;
import java.beans.IntrospectionException;
import java.beans.PropertyDescriptor;
import java.beans.SimpleBeanInfo;

import org.openide.execution.ProcessExecutor;
import org.openide.util.NbBundle;


/**
 * Bean info for <code>LocaleExecutor</code> class.
 *
 * @author  Peter Zavadsky
 */
public class LocaleExecutorBeanInfo extends SimpleBeanInfo {

    /** Bean descriptor for returned by this bean info. */
    private static BeanDescriptor beanDescriptor;
    
    /** Property desriptors returned by this bean info. */
    private static PropertyDescriptor[] propertyDescriptors;

    
    /** Gets bean descriptor. Overrides superclass method. */
    public BeanDescriptor getBeanDescriptor () {
        if(beanDescriptor == null) {
            beanDescriptor = new BeanDescriptor(LocaleExecutor.class);
            beanDescriptor.setValue("version", "1.1"); // NOI18N
            beanDescriptor.setName(NbBundle.getBundle(LocaleExecutorBeanInfo.class).getString("TXT_BeanDescriptorName"));
        }
        
        return beanDescriptor;
    }

    /** Gets property descriptors. Overrides superclass method. */
    public PropertyDescriptor[] getPropertyDescriptors () {
        if(propertyDescriptors == null) {
            try {
                propertyDescriptors = new PropertyDescriptor[1];

                propertyDescriptors[0] = new PropertyDescriptor("locale", LocaleExecutor.class, "getLocale", "setLocale"); // NOI18N
                propertyDescriptors[0].setDisplayName(NbBundle.getBundle(LocaleExecutorBeanInfo.class).getString("TXT_LocalePropertyDisplayName"));
                propertyDescriptors[0].setShortDescription(NbBundle.getBundle(LocaleExecutorBeanInfo.class).getString("TXT_LocalePropertyShortDescription"));
                propertyDescriptors[0].setBound(true);
                propertyDescriptors[0].setPropertyEditorClass(LocalePropertyEditor.class);
            } catch(IntrospectionException ie) {
                if(Boolean.getBoolean ("netbeans.debug.exceptions")) // NOI18N
                    ie.printStackTrace ();
                
                return null;
            }
        }
        
        return propertyDescriptors;
    }

    /** Gets additional bean info. Overrides superclass method. */
    public BeanInfo[] getAdditionalBeanInfo () {
        try {
            return new BeanInfo[] {Introspector.getBeanInfo(ProcessExecutor.class)};
        } catch(IntrospectionException ie) {
            if(Boolean.getBoolean ("netbeans.debug.exceptions")) // NOI18N
                ie.printStackTrace ();
            
            return null;
        }
    }

    /** Gets icon. Overrides superclass method. */
    public Image getIcon(int type) {
        BeanInfo[] additionalBeanInfo = getAdditionalBeanInfo();
        
        if(additionalBeanInfo == null)
            return null;
        
        return additionalBeanInfo[0].getIcon(type);
    }
    
}
