/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.i18n.form;


import org.netbeans.modules.form.FormDataObject;
import org.netbeans.modules.form.FormPropertyEditorManager;
import org.netbeans.modules.form.PersistenceObjectRegistry;
import org.netbeans.modules.i18n.FactoryRegistry;

import org.openide.modules.ModuleInstall;


/** 
 * Installation class for i18n to form cross dependency module.
 * It registers <code>FormI18nStringEditor</code> to form property editors
 * and <code>FormI18nSupport.Factory</code> to i18n module.
 *
 * @author Peter Zavadsky
 */
public class I18nFormCrossModule extends ModuleInstall {

    /** Generated serial version UID. */
    static final long serialVersionUID =2673802750264867L;

    /** Registers property editor in form module and factory in i18n module. */
    public void restored() {
        registerFormPropertyEditor();
        registerFormI18nSupport();
    }
    
    /** Unregisters i18n factory from i18n module. */
    public void uninstalled() {
        FactoryRegistry.unregisterSupport(FormDataObject.class.getName());
    }

    /** Registers <code>FormI18nStringEditor</code> form property editor to form module. */
    private void registerFormPropertyEditor() {
        Class newEditorClass = FormI18nStringEditor.class;
        
        // Set alias to very old property editor name.
        PersistenceObjectRegistry.registerAlias(
            newEditorClass.getName(),
            "org.netbeans.modules.i18n.ResourceBundleStringFormEditor"); // NOI18N

        // Set alias to older property editor name.
        PersistenceObjectRegistry.registerAlias(
            newEditorClass.getName(),
            "org.netbeans.modules.i18n.cross.FormI18nStringEditor"); // NOI18N
        
        // Register new property editor.
        FormPropertyEditorManager.registerEditor (String.class, newEditorClass);
    }

    /** Registers <code>FormI18nSupport.Factory</code> to i18n module. */
    private void registerFormI18nSupport() {
        FactoryRegistry.registerSupport(FormDataObject.class.getName(), new FormI18nSupport.Factory());
    }
}
