/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.i18n.wizard; 


import java.util.EventObject;
import javax.swing.CellEditor;
import javax.swing.event.*;


/**
 * This class is only due compatibility with jdk1.2. Remove later
 * when not supported.
 * 
 * @author Peter Zavadsky
 */

public abstract class AbstractCellEditor implements CellEditor {

    protected EventListenerList listenerList = new EventListenerList();
    transient protected ChangeEvent changeEvent = null;

    /** Gets if cell is editable. 
     * @return <code>true</code> */
    public boolean isCellEditable(EventObject e) { 
	return true; 
    } 

    /** Gets if should be cell selected.
     * @return <code>true</code> */
    public boolean shouldSelectCell(EventObject anEvent) { 
	return true; 
    }
    
    /** Stops editing instanceof cell. */
    public boolean stopCellEditing() { 
	fireEditingStopped(); 
	return true;
    }

    /** Cancels editing. */
    public void  cancelCellEditing() { 
	fireEditingCanceled(); 
    }

    /** Adds cell editor listener. */
    public void addCellEditorListener(CellEditorListener l) {
	listenerList.add(CellEditorListener.class, l);
    }

    /** Removes cell editor listener. */
    public void removeCellEditorListener(CellEditorListener l) {
	listenerList.remove(CellEditorListener.class, l);
    }

    /** Fires event when stopped editing. */
    protected void fireEditingStopped() {
	// Guaranteed to return a non-null array
	Object[] listeners = listenerList.getListenerList();
	// Process the listeners last to first, notifying
	// those that are interested in this event
	for (int i = listeners.length-2; i>=0; i-=2) {
	    if (listeners[i]==CellEditorListener.class) {
		// Lazily create the event:
		if (changeEvent == null)
		    changeEvent = new ChangeEvent(this);
		((CellEditorListener)listeners[i+1]).editingStopped(changeEvent);
	    }	       
	}
    }

    /** Fires event when editing cancelled. */
    protected void fireEditingCanceled() {
	// Guaranteed to return a non-null array
	Object[] listeners = listenerList.getListenerList();
	// Process the listeners last to first, notifying
	// those that are interested in this event
	for (int i = listeners.length-2; i>=0; i-=2) {
	    if (listeners[i]==CellEditorListener.class) {
		// Lazily create the event:
		if (changeEvent == null)
		    changeEvent = new ChangeEvent(this);
		((CellEditorListener)listeners[i+1]).editingCanceled(changeEvent);
	    }	       
	}
    }
}
