/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.icebrowser;

import ice.storm.Pilot;
import ice.storm.Viewport;
import ice.pilots.html4.Search;
import ice.pilots.html4.ThePilot;

import java.awt.Dialog;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.ResourceBundle;
import javax.swing.JPanel;

import org.openide.DialogDescriptor;
import org.openide.TopManager;
import org.openide.NotifyDescriptor;
import org.openide.util.actions.ActionPerformer;
import org.openide.util.actions.SystemAction;
import org.openide.util.NbBundle;
import org.openide.util.HelpCtx;


/**
* Support for Find Dialog for Ice browser.
*
* @author Jan Jancura
*/
class FindManager implements ActionListener, ActionPerformer {

    private static String         FIND_BUTT = NbBundle.getBundle (
                FindManager.class).getString ("CTL_Find");


    // variables .................................................................

    private Dialog                dialog;
    private String                previous = null;
    private FindDialogPanel       panel;
    private IceBrowserImpl        browser;
    private Search                s;

    // init ......................................................................

    FindManager (IceBrowserImpl impl) {
        browser = impl;
    }


    // ActionPerformer implementation ............................................

    public void performAction (SystemAction action) {
        if (dialog == null)
            dialog = createDialog ();
        
        s = null;
        Pilot pilot = browser.getViewport ().getPilot ();
        if (pilot instanceof ThePilot) 
            s = ((ThePilot)pilot).getSearch ();
        if (s == null) {
            TopManager.getDefault ().notify (new NotifyDescriptor.Message (
                NbBundle.getBundle (FindManager.class).getString ("MSG_FindNotSupported"),  // NOI18N
                NotifyDescriptor.INFORMATION_MESSAGE)
            );
            return;
        }
        previous = null;
        dialog.show ();
    }


    // ActionListener implementation ............................................

    /**
    * Called when some dialog button was pressed 
    */
    public void actionPerformed (ActionEvent evt) {
        if (FIND_BUTT.equals (evt.getActionCommand ())) {
            String text = ((String) panel.findWhat.getSelectedItem ()).trim ();
            put (text);
            
            if (!text.equals (previous))
                s.setSearchText (text);
                
            if (!s.findNext ()) {
                TopManager.getDefault ().notify (new NotifyDescriptor.Message (
                    NbBundle.getBundle (FindManager.class).getString ("MSG_NotFound"),  // NOI18N
                    NotifyDescriptor.INFORMATION_MESSAGE)
                );
            }
        } else {
            dialog.setVisible (false);
            s = null;
        }
        // dialog.dispose ();
    }


    // other methods .....................................................................

    /**
    * Adds new string to find dialog's ComboBox.
    */
    private void put (String s) {
        int i, k = panel.findWhat.getItemCount ();
        for (i = 0; i < k; i++)
            if (panel.findWhat.getItemAt (i).equals (s))
                return;
        panel.findWhat.addItem (s);
    }

    /** Constructs managed dialog instance using TopManager.createDialog
    * and returnrs it */
    private Dialog createDialog () {
        ResourceBundle bundle = NbBundle.getBundle (FindManager.class);

        panel = new FindDialogPanel ();
        // create dialog descriptor, create & return the dialog
        DialogDescriptor dialogDesc = new DialogDescriptor (
                                            panel,
                                            bundle.getString ("CTL_Find_Dialog_Title"),
                                            true,                                      // modal
                                            new Object[] {
                                              bundle.getString ("CTL_Find"),
                                              DialogDescriptor.CANCEL_OPTION           // buttons
                                            },
                                            bundle.getString ("CTL_Find"),             // init. value
                                            DialogDescriptor.RIGHT_ALIGN,              // buttons align
                                            new HelpCtx (FindManager.class.getName () + ".dialog"), // NOI18N
                                            this                                       // action list.
        );
        Dialog dialog = TopManager.getDefault ().createDialog (dialogDesc);
        dialog.pack ();
        return dialog;
    }
}
