/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.icebrowser;

import org.openide.options.SystemOption;
import org.openide.actions.GoAction;
import org.openide.TopManager;
import org.openide.util.NbBundle;
import org.openide.util.HelpCtx;

import java.awt.Font;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Enumeration;


/**
* Settings for ICE Browser.
*
* @author Jan Jancura
*/
public class ICEBrowserSettings extends SystemOption {

    // static .....................................................................................

    /** generated Serialized Version UID */
    static final long serialVersionUID = 833203088075073629L;

    /** defaultBackground property name */
    public static final String PROP_DEFAULT_BACKGROUND = "defaultBackground"; // NOI18N
    /** fixedFont property name */
    public static final String PROP_FIXED_FONT = "fixedFont"; // NOI18N
    /** proportionalFont property name */
    public static final String PROP_PROPORTIONAL_FONT = "proportionalFont"; // NOI18N
    /** encoding property name */
    public static final String PROP_ENCODING = "encoding"; // NOI18N
    /** image animation property name */
    public static final String PROP_ANIMATION = "imageAnimation"; // NOI18N


    // init ........................................................................................

    public ICEBrowserSettings () {
        super ();
        // nothing to do here 
    }

    /** Initialize properties
     */
    private void initializeProperties () {
        if (getProperty (PROP_DEFAULT_BACKGROUND) == null) {
            putProperty (PROP_DEFAULT_BACKGROUND, java.awt.Color.white, false);
        }
        if (getProperty (PROP_FIXED_FONT) == null) {
            putProperty (
                PROP_FIXED_FONT,
                Font.decode (NbBundle.getBundle (ICEBrowserSettings.class).getString ("DEFAULT_FIXED_FONT")),
                false
            ); // NOI18N
        }
        if (getProperty (PROP_PROPORTIONAL_FONT) == null) {
            putProperty (
                PROP_PROPORTIONAL_FONT,
                Font.decode (NbBundle.getBundle (ICEBrowserSettings.class).getString ("DEFAULT_PROPORTIONAL_FONT")),
                false
            ); // NOI18N
        }
        if (getProperty (PROP_ENCODING) == null) {
            String s = NbBundle.getBundle (ICEBrowserSettings.class).getString ("DEFAULT_ENCODING"); // NOI18N
            if ( (s != null) && (s.trim ().length () > 0))
                putProperty (PROP_ENCODING, s, false);
            else {
                /*        s = System.getProperties ().getProperty ("sun.io.unicode.encoding"); // NOI18N
                        if (s != null) 
                          putProperty (PROP_ENCODING, s, false);
                        else    Do not work...*/
                putProperty (PROP_ENCODING, "", false);
            }
        }
        if (getProperty (PROP_ANIMATION) == null) {
            putProperty (PROP_ANIMATION, new Boolean (false), false);
        }
    }


    // SystemOption implementation ..................................................................

    /**
    * Returns name of this setings.
    */
    public String displayName () {
        return NbBundle.getBundle (ICEBrowserSettings.class).getString ("CTL_ICE_Browser_settings");
    }

    public HelpCtx getHelpCtx () {
        return new HelpCtx (ICEBrowserSettings.class);
    }


    // properties .................................................................................

    /**
    * @returns default color of browser background.
    */
    public java.awt.Color getDefaultBackground () {
        initializeProperties ();
        return (java.awt.Color) getProperty (PROP_DEFAULT_BACKGROUND);
    }

    /**
    * Set default color of browser background.
    */
    public void setDefaultBackground (java.awt.Color color) {
        putProperty (PROP_DEFAULT_BACKGROUND, color, true);
    }

    /**
    * @returns default fixed font for HTML browser.
    */
    public Font getFixedFont () {
        initializeProperties ();
        return (Font) getProperty (PROP_FIXED_FONT);
    }

    /**
    * Set default fixed font for HTML browser.
    */
    public void setFixedFont (Font font) {
        putProperty (PROP_FIXED_FONT, font, true);
    }

    /**
    * @returns default proportional font for HTML browser.
    */
    public Font getProportionalFont () {
        initializeProperties ();
        return (Font) getProperty (PROP_PROPORTIONAL_FONT);
    }

    /**
    * Set default proportional font for HTML browser.
    */
    public void setProportionalFont (Font font) {
        putProperty (PROP_PROPORTIONAL_FONT, font, true);
    }

    /**
    * @returns history for HTML browser.
    */
    public java.util.ArrayList getHistory () {
        // return IceBrowserImpl.history;
        return new java.util.ArrayList ();
    }

    /**
    * Set history for HTML browser.
    */
    public void setHistory (java.util.ArrayList history) {
        // IceBrowserImpl.history = history;
    }

    /**
    * @returns encoding for browser.
    */
    public String getEncoding () {
        initializeProperties ();
        return (String) getProperty (PROP_ENCODING);
    }

    /**
    * Set encoding for browser.
    */
    public void setEncoding (String encoding) {
        putProperty (PROP_ENCODING, encoding, true);
    }
    
    /**
    * @returns encoing for browser.
    */
    public boolean isAnimation () {
        initializeProperties ();
        return ((Boolean)getProperty (PROP_ANIMATION)).booleanValue ();
    }

    /**
    * Set encoing for browser.
    */
    public void setAnimation (boolean animation) {
        putProperty (PROP_ANIMATION, new Boolean (animation), true);
    }
}
