/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.icebrowser;

import java.awt.Image;
import java.beans.*;
import java.util.ResourceBundle;

import org.openide.util.NbBundle;

public class IceBrowserBeanInfo extends SimpleBeanInfo {

    private static BeanDescriptor descr;

    static {
        descr = new BeanDescriptor (IceBrowser.class);
        descr.setDisplayName (NbBundle.getMessage (IceBrowserBeanInfo.class, "CTL_IceBrowserName"));
        descr.setShortDescription (NbBundle.getMessage (IceBrowserBeanInfo.class, "HINT_IceBrowserName"));
    }
    
    private static PropertyDescriptor[] properties = null; 

    public BeanDescriptor getBeanDescriptor () {
        return descr;
    }

    /**
     * Gets the bean's <code>PropertyDescriptor</code>s.
     * 
     * @return An array of PropertyDescriptors describing the editable
     * properties supported by this bean.  May return null if the
     * information should be obtained by automatic analysis.
     * <p>
     * If a property is indexed, then its entry in the result array will
     * belong to the IndexedPropertyDescriptor subclass of PropertyDescriptor.
     * A client of getPropertyDescriptors can use "instanceof" to check
     * if a given PropertyDescriptor is an IndexedPropertyDescriptor.
     */
    public PropertyDescriptor[] getPropertyDescriptors() {
        if (properties == null) {
            try {
                properties = new PropertyDescriptor [] {
                    new PropertyDescriptor ("name", IceBrowser.class, "getName", null),    // NOI18N
                    new PropertyDescriptor (
                    ICEBrowserSettings.PROP_DEFAULT_BACKGROUND, IceBrowser.class,
                    "getDefaultBackground", "setDefaultBackground"), // 1 // NOI18N
                    new PropertyDescriptor (
                    ICEBrowserSettings.PROP_FIXED_FONT, IceBrowser.class,
                    "getFixedFont", "setFixedFont"), // 2 // NOI18N
                    new PropertyDescriptor (
                    ICEBrowserSettings.PROP_PROPORTIONAL_FONT, IceBrowser.class,
                    "getProportionalFont", "setProportionalFont"), // 3 // NOI18N
                    new PropertyDescriptor (
                    "encoding", IceBrowser.class, // NOI18N
                    "getEncoding", "setEncoding"), // 4 // NOI18N
                    new PropertyDescriptor (
                    "imageAnimation", IceBrowser.class, // NOI18N
                    "isAnimation", "setAnimation"), // 5 // NOI18N
                };
                ResourceBundle bundle = NbBundle.getBundle (IceBrowserBeanInfo.class);
                properties[0].setDisplayName (bundle.getString ("PROP_browserName"));
                properties[0].setShortDescription (bundle.getString ("HINT_browserName"));
                properties[1].setDisplayName (bundle.getString ("PROP_DEFAULT_BACKGROUND"));
                properties[1].setShortDescription (bundle.getString ("HINT_DEFAULT_BACKGROUND"));
                properties[2].setDisplayName (bundle.getString ("PROP_FIXED_FONT"));
                properties[2].setShortDescription (bundle.getString ("HINT_FIXED_FONT"));
                properties[3].setDisplayName (bundle.getString ("PROP_PROPORTIONAL_FONT"));
                properties[3].setShortDescription (bundle.getString ("HINT_PROPORTIONAL_FONT"));
                properties[4].setDisplayName (bundle.getString ("PROP_ENCODING"));
                properties[4].setShortDescription (bundle.getString ("HINT_ENCODING"));
                properties[4].setPropertyEditorClass (EncodingEditor.class);
                properties[5].setDisplayName (bundle.getString ("PROP_ANIMATION"));
                properties[5].setShortDescription (bundle.getString ("HINT_ANIMATION"));
            } catch (IntrospectionException ie) {
                if (Boolean.getBoolean ("netbeans.debug.exceptions"))   // NOI18N
                    ie.printStackTrace ();
                return null;
            }
        }
        return properties;
    }

    /**
    * Returns the IceBrowserSettings' icon. 
    */
    public Image getIcon (int type) {
        return loadImage("/org/openide/resources/html/htmlView.gif"); // NOI18N
    }
}
