/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.icebrowser;

import ice.net.*;
import ice.storm.*;
import ice.pilots.html4.*;

import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.awt.Component;
import java.awt.Container;
import java.awt.Point;
import java.awt.event.*;
import java.io.StringBufferInputStream;
import java.io.StringReader;
import java.net.URL;
import java.lang.reflect.Field;
import java.text.DateFormat;
import java.util.Hashtable;
import java.util.ArrayList;
import java.util.Date;
import java.util.ResourceBundle;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import org.openide.TopManager;
import org.openide.NotifyDescriptor;
import org.openide.actions.FindAction;
import org.openide.awt.HtmlBrowser;
import org.openide.execution.NbfsStreamHandlerFactory;
import org.openide.util.NbBundle;
import org.openide.util.SharedClassObject;

/**
* ICE Browser support.
*
* @author Jan Jancura
*/
public class IceBrowserImpl extends HtmlBrowser.Impl {

    // static ....................................................................
    static ArrayList              history = new ArrayList ();

    private FindManager    findManager;
    /** browser controller */
    static StormBase              browserBase;
    /** mapping between viewport IDs and components */
    private static Hashtable      windows = new Hashtable();
    
    // variables .................................................................

    /** browser visual component */
    private Container               browserCont;
    /** viewport where content is rendered */
    private Viewport                viewport;
    
    /** current URL */
    private URL                     url;
    /** standart helper variable */
    private PropertyChangeSupport   pcs;
    /** Current status message. */
    private String                  statusMessage = ""; // NOI18N
    /** Helper variable for reverting status message to default one. */
    private String                  oldStatus;
    /** Helper variable to keep status of loaded page */
    private boolean contentLoadingFinished = false;
    /** Helper variable to keep status of loaded page */
    private boolean contentRenderingFinished = false;

    /** Current value of title property. */
    private String                  title = ""; // NOI18N
    /** Helper instance of settings. */
    private IceBrowser              settings = null;
    /** Forward property value. */
    private boolean                 forward;
    /** Backward property value. */
    private boolean                 backward;

    // init ......................................................................

    /**
    * Creates instance of Ice Browser.
    */
    public IceBrowserImpl () {
        init (null, new Viewport ("view"+this.toString ()));
    }

    /**
     * Creates instance of Ice Browser.
     * 
     * Keeps reference to factory that contains settings.
     */
    public IceBrowserImpl (IceBrowser fact) {
        init (fact, new Viewport ("view"+this.toString ()));
    }
    
    /**
     * Creates instance of Ice Browser.
     * 
     * Called when user click on link that should be displayed in new frame/window.
     */
    public IceBrowserImpl (Viewport view) {
        init (null, view);
    }
    
    
    /**
     * Creates instance of Ice Browser.
     * 
     * Keeps reference to factory that contains settings.
     * Called when user click on link that should be displayed in new frame/window.
     */
    public IceBrowserImpl (IceBrowser fact, Viewport view) {
        init (fact, view);
    }
    
    
    private void init (IceBrowser fact, Viewport view) {
        settings = (fact == null)? new IceBrowser (): fact;
        pcs = new PropertyChangeSupport (this);
        findManager = new FindManager (this);
        
        try {
            if (browserBase == null) {
                browserBase = new StormBase ();
                browserBase.setCallback (new IceBrowserImpl.IceCallback ());
                browserBase.setHistoryManager (new PseudoHistoryManager (browserBase));
                browserBase.setComponentToolkitName ("swing"); // NOI18N
                browserBase.setURLClassLoaderFactory (new NbClassLoaderFactory ());
                
                ProxyManager proxyMan = new ProxyManager ();
                proxyMan.setProxyUse (true);
                ice.net.HttpURLConnection.setProxyManager (proxyMan);
                
                NbfsStreamHandlerFactory.getDefault ().register ("http", new ice.net.HttpURLStreamHandler ());  // NOI18N
                NbfsStreamHandlerFactory.getDefault ().register ("image", new ice.storm.ImageHandler ());  // NOI18N
                // NbfsStreamHandlerFactory.getDefault ().register ("https", new ice.net.HttpsURLStreamHandler ());
                NbfsStreamHandlerFactory.getDefault ().register ("mailto", new ice.net.mailto.Handler ());
                // NbfsStreamHandlerFactory.getDefault ().register ("dom", new ice.pilots.domviewer.Handler ());
                // NbfsStreamHandlerFactory.getDefault ().register ("sitemap", new ice.pilots.sitemap.Handler ());
                NbfsStreamHandlerFactory.getDefault ().register ("about", new ice.net.about.Handler ());  // NOI18N
                // NbfsStreamHandlerFactory.getDefault ().register ("doc", new ice.net.about.Handler ());
                // NbfsStreamHandlerFactory.getDefault ().register ("doc", new ice.net.doc.Handler ());
            }
            browserCont = new IceBrowserImpl.IceBrowserContainer ();
            
// System.out.println("windows.put () for "+view.getId ()+ " ("+view.getName ()+")");
            viewport = view;
            windows.put (view.getName (), this);

            System.getProperties ().put ("ice.pilots.html4.defaultEncoding",settings.getEncoding ());   // NOI18N
            System.getProperties ().put ("ice.pilots.html4.imageAnimation",(new Boolean (settings.isAnimation ())).toString ());   // NOI18N
            
            IceListener lsnr = new IceBrowserImpl.IceListener (this);
            viewport.addPropertyChangeListener (lsnr);
            settings.addPropertyChangeListener (lsnr);
           
            setStyleSheet ();
        /*
            browser.setDefaultBackground (settings.getDefaultBackground ());
            browser.setFixedFont (settings.getFixedFont ());
            browser.setProportionalFont (settings.getProportionalFont ());
         */
        }
        catch (Exception ex) {
            ex.printStackTrace ();
        }
    }
    
    /**
     * Sets viewport to one that is really used.
     * This means that this viewport is passed as argument to createTopLevelContainer ().
     */
    void setViewport (Viewport v) {
        if (windows.get (viewport.getName ()) != null)
            windows.remove (viewport.getName ());
            
        viewport = v;
        windows.put (viewport.getName (), this);
        IceListener lsnr = new IceBrowserImpl.IceListener (this);
        viewport.addPropertyChangeListener (lsnr);
        // settings.addPropertyChangeListener (lsnr);
        
        return;
    }
     // */


    // HtmpBrowser.Impl implementation ...........................................

    /**
    * Returns visual component of html browser.
    *
    * @return visual component of html browser.
    */
    public java.awt.Component getComponent () {
        return browserCont;
    }

    /**
    * Reloads current html page.
    */
    public void reloadDocument () {
        // browserBase.unload ();  // Unload/dispose all static caches
        try {
            browserBase.reload (viewport.getId ());
        }
        catch (Exception ex) {
            // icebrowser is buggy :-(
            if (Boolean.getBoolean ("netbeans.debug.exception")) {
                ex.printStackTrace ();
            }
        }
    }

    /**
    * Stops loading of current html page.
    */
    public void stopLoading () {
        browserBase.stopLoading (viewport.getName ());
    }

    /**
    * Sets current URL.
    *
    * @param url URL to show in the browser.
    */
    public void setURL (URL url) {
        if (url == null) 
            return;
        
// System.out.println("setURL("+url.toString ()+") in "+viewport.getName ()+", "+viewport.getId ());
        URL old = getURL ();
        this.url = url;
        browserBase.renderContent (url.toString (), null, viewport.getName ());
        // put (url);
        update ();
        pcs.firePropertyChange (PROP_URL, old, url);
    }

    /**
    * Returns current URL.
    *
    * @return current URL.
    */
    public URL getURL () {
        return url;
    }

    /**
    * Returns status message representing status of html browser.
    *
    * @return status message.
    */
    public String getStatusMessage () {
        return statusMessage;
    }

    /**
    * Shows given string in the html browser.
    *
    * @param str String to show in html browser.
    */
    public void showString (String str) {
        // browser.htmlClear ();
        // browser.htmlAppend (str);
    }

    /** Returns title of the displayed page.
    * @return title 
    */
    public String getTitle () {
        return title;
    }

    /** Is forward button enabled?
    * @return true if it is
    */
    public boolean isForward () {
        return forward;
    }

    /** Moves the browser forward. Failure is ignored.
    */
    public void forward () {
        try {
            URL old = getURL ();
            browserBase.goForward (viewport.getId ());
            String sUrl = viewport.getPilot ().getCurrentLocation ();
            url = new URL (sUrl);
            pcs.firePropertyChange (PROP_URL, old, url);
            update ();
        } catch (java.net.MalformedURLException e) {
        }
        catch (NullPointerException ex) {
            ex.printStackTrace ();
        }
    }

    /** Is backward button enabled?
    * @return true if it is
    */
    public boolean isBackward () {
        return backward;
    }

    /** Moves the browser forward. Failure is ignored.
    */
    public void backward () {
        try {
            URL old = getURL ();
            browserBase.goBack (viewport.getId ());
            String sUrl = viewport.getPilot ().getCurrentLocation ();
            url = new URL (sUrl);
            pcs.firePropertyChange (PROP_URL, old, url);
            update ();
        } catch (java.net.MalformedURLException e) {
        }
        catch (NullPointerException ex) {
            ex.printStackTrace ();
        }
    }

    /** Is history button enabled?
    * @return true if it is
    */
    public boolean isHistory () {
        return true;
    }

    /** Invoked when the history button is pressed.
    */
    public void showHistory () {
        ResourceBundle bundle = NbBundle.getBundle (FindManager.class);
        StringBuffer sb = new StringBuffer ();
        

        String [] urls;
        try {
            urls = browserBase.getHistoryManager ().getBackHistory (viewport.getName ());
        }
        catch (Exception ex) {
            // sometimes IB history manager fails (not inited?)
            urls = null;
        }
        sb.append ("<HTML><HEAD><TITLE>"). // NOI18N
        append (bundle.getString ("CTL_History")).
        append ("</TITLE></HEAD>\n<BODY><H2>"). // NOI18N
        append (bundle.getString ("CTL_History")).
        append (":</H2>\n"); // NOI18N
        if (urls == null) {
            sb.append ("No history is available.");
        }
        else {
            for (int i = 0; i < urls.length; i++) {
                sb.append ("<BR>"). // NOI18N
                append ("<A HREF=\""). // NOI18N
                append (urls[i]).
                append ("\">"). // NOI18N
                append (urls[i]).
                append ("</A> "). // NOI18N
                append ("\n") ; // NOI18N
            }
            // current URL is not returned from history, append it
            sb.append ("<BR>"). // NOI18N
            append ("<A HREF=\""). // NOI18N
            append (url.toString ()).
            append ("\">"). // NOI18N
            append (url.toString ()).
            append ("</A> "). // NOI18N
            append ("\n") ; // NOI18N
        }
        sb.append ("</BODY></HTML>"); // NOI18N

        
        StringBufferInputStream in = new StringBufferInputStream (new String(sb));
        ContentLoader cl = new ContentLoader (null, in, "text/html");

        browserBase.renderContent(cl, viewport.getName ());
    }

    /**
    * Adds PropertyChangeListener to this browser.
    *
    * @param l Listener to add.
    */
    public void addPropertyChangeListener (PropertyChangeListener l) {
        pcs.addPropertyChangeListener (l);
    }

    /**
    * Removes PropertyChangeListener from this browser.
    *
    * @param l Listener to remove.
    */
    public void removePropertyChangeListener (PropertyChangeListener l) {
        pcs.removePropertyChangeListener (l);
    }


    // other methods ..............................................................

    /**
    * Clear all items from history.
    */
    private void clearHistory () {
        // history = new ArrayList ();
        // browser.htmlInterrupt ();
        reloadDocument ();
    }

    /**
    * Update values of backward & forward properties.
    */
    private void update () {
        try {
            boolean nBackward = browserBase.canGoBack (viewport.getName ());
            boolean nForward = browserBase.canGoForward (viewport.getName ());
            if (nBackward != backward) {
                backward = nBackward;
                pcs.firePropertyChange (
                    PROP_BACKWARD,
                    new Boolean (!backward),
                    new Boolean (backward)
                );
            }
            if (nForward != forward) {
                forward = nForward;
                pcs.firePropertyChange (
                    PROP_FORWARD,
                    new Boolean (!forward),
                    new Boolean (forward)
                );
            }
        }
        catch (NullPointerException ex) {
            ex.printStackTrace ();
        }
    }
    
    /**
     *  Getter for viewport associated with this broeser window.
     *
     * @return associated viewport
     */
    public Viewport getViewport () {
        return viewport;
    }
    
    /**
     * Access to StormBase object
     */
    public static StormBase getStormBase () {
        return browserBase;
    }
    
    void setStyleSheet () {
        try {
            StringBuffer style = new StringBuffer ("BODY {font-family: "); // NOI18N
            style.append (settings.getProportionalFont ().getFamily ()).
            append (";"). // NOI18N
            append (" font-style: "). // NOI18N
            append (settings.getProportionalFont ().isItalic ()? "italic": "normal").
            append (";"). // NOI18N
            append (" font-weight: "). // NOI18N
            append (settings.getProportionalFont ().isBold ()? "bold": "normal").
            append (";"). // NOI18N
            append (" font-size: "). // NOI18N
            append (settings.getProportionalFont ().getSize ()).
            append ("pt;"). // NOI18N
            append (" background-color: rgb(").   // NOI18N
            append (settings.getDefaultBackground ().getRed ()).append (",").   // NOI18N
            append (settings.getDefaultBackground ().getGreen ()).append (",").   // NOI18N
            append (settings.getDefaultBackground ().getBlue ()).append (")").   // NOI18N
            append (" } "). // NOI18N
            append ("PRE, TT, CODE, KBD, SAMP, LISTING, PLAINTEXT, XMP {font-family: "). // NOI18N
            append (settings.getFixedFont ().getFamily ()).
            append (";"). // NOI18N
            append (" font-style: "). // NOI18N
            append (settings.getFixedFont ().isItalic ()? "italic": "normal").
            append (";"). // NOI18N
            append (" font-weight: "). // NOI18N
            append (settings.getFixedFont ().isBold ()? "bold": "normal").
            append (";"). // NOI18N
            append (" font-size: "). // NOI18N
            append (settings.getFixedFont ().getSize ()).
            append ("pt;"). // NOI18N
            append (" }"); // NOI18N
// System.out.println("setting font: "+style);
            DStyleSheet us = ThePilot.getUserStyleSheet();
            CSSBuilder b = new CSSBuilder();
            b.build(us,new StringReader (new String (style)),null);
        }
        catch (Exception ex) {
            if (Boolean.getBoolean ("netbeans.debug.exceptions"))   // NOI18N
                ex.printStackTrace();
        }
    }

    /**
    * Represents one position in history. Only for compatibility with older version.
    */
    private static class Node implements java.io.Serializable {

        /** serialVersionUID */
        private static final long serialVersionUID = -6252984103555792114L;

        Point                   position;
        Date                    date;
        String                  url;

        Node (
            String   url
        ) {
            this.position = position;
            this.url = url;
            this.date = new Date ();
        }
    }

    /** container which holds ICEBrowser content */
    private class IceBrowserContainer extends JPanel {
        
        public IceBrowserContainer () {
            super ();
            
            setRequestFocusEnabled (true);
        }
        
        /** adds window listener which can close
         *  assocciated window when closing this window
         * /
        public void addNotify () {
            super.addNotify ();
            
            Component w;
            for (w=this; !((w instanceof java.awt.Window) || (w == null)); w = w.getParent ())
                ;   // empty loop body
            
            if (w == null) return;
            
            ((java.awt.Window)w).addWindowListener (new WindowAdapter () {
                public void windowClosing (WindowEvent e) {
                    try {
                        // browserBase.removePropertyChangeListener (lsnr, viewportName);
                        // browserBase.closeViewport (viewport.getId ());
                    }
                    catch (NullPointerException ex) {
                        ex.printStackTrace ();
                    }
                    try {
                        // System.out.println("Closing StormBase");
                        // browserBase.dispose ();
                        // System.out.println ("Closed");
                    }
                    catch (NullPointerException ex) {
                        ex.printStackTrace ();
                    }
                }
                
            });
        }
         */
        
        public void requestFocus () {
            ((FindAction) FindAction.get (FindAction.class)).setActionPerformer (
            findManager
            );
            super.requestFocus ();
        }
    }
    
    /**
     * Listens on Ice browser on property changes and
     */
    private static class IceListener implements PropertyChangeListener {
        IceBrowserImpl impl;
        
        public IceListener(IceBrowserImpl impl) {
            this.impl = impl;
        }
        
        /**
         * Listens to browser events.
         */
        public void propertyChange (PropertyChangeEvent e) {
// System.out.println("propertyChange "+e.getPropertyName ()+
// " from "+e.getSource ().toString () +" received ("+(String)e.getNewValue ()+")");
            if (e.getSource () instanceof Viewport) {
                // The source of the event
                Viewport v = (Viewport)e.getSource ();

                // Verify that it is the right Viewport
                if(v.getName ().equals (impl.viewport.getName ())) {

                    // The details of the event
                    String name = e.getPropertyName();
                    if (name == null)
                        return;
                    String newValue = (String)e.getNewValue();

                    // Loading a new page resets flags
                    if(name.equals("contentLoading") && newValue.equals("connect")) { // NOI18N
                        impl.contentLoadingFinished = false;
                        impl.contentRenderingFinished = false;
                        String old = impl.statusMessage;
                        impl.pcs.firePropertyChange (PROP_STATUS_MESSAGE, old, impl.statusMessage = "Loading document...");
                        impl.oldStatus = impl.statusMessage;
                    }

                    // Loading a new page resets flags
                    if(name.equals("contentLoading") && newValue.equals("error")) { // NOI18N
                        impl.contentLoadingFinished = false;
                        impl.contentRenderingFinished = false;
                        String old = impl.statusMessage;
                        ContentLoader cl = (ContentLoader)e.getOldValue ();
                        impl.statusMessage = (cl == null)? "Loading error...": cl.getException ().getMessage ();
                        impl.pcs.firePropertyChange (PROP_STATUS_MESSAGE, old, impl.statusMessage);
                        impl.oldStatus = impl.statusMessage;
                    }

                    // The first check
                    if(name.equals("contentLoading") && newValue.equals("end")) { // NOI18N
                        impl.contentLoadingFinished = true;
                        String old = impl.statusMessage;
                        impl.pcs.firePropertyChange (PROP_STATUS_MESSAGE, old, impl.statusMessage = "Document loaded...");
                        impl.oldStatus = impl.statusMessage;
                    }

                    // Second check
                    if(name.equals("contentRendering") && newValue.equals("end") // NOI18N
                    && (impl.contentLoadingFinished)) {
                        impl.contentRenderingFinished = true;
                        String old = impl.statusMessage;
                        impl.pcs.firePropertyChange (PROP_STATUS_MESSAGE, old, impl.statusMessage = "Document rendered...");
                        impl.oldStatus = impl.statusMessage;
                    }

                    // Final check
                    if(name.equals("outstandingImages") && newValue.equals("0") // NOI18N
                    && (impl.contentLoadingFinished) && (impl.contentRenderingFinished)) {
                        String old = impl.statusMessage;
                        impl.pcs.firePropertyChange (PROP_STATUS_MESSAGE, old, impl.statusMessage = null);
                        impl.oldStatus = impl.statusMessage;
                        // Any other appropriate action should be added here
                    }
                    if (e.getPropertyName ().equals ("title")) {
                        String old = impl.title;
                        impl.pcs.firePropertyChange (PROP_TITLE, old, impl.title = (String) e.getNewValue ());
                    } else if (e.getPropertyName ().equals ("location")) {
                        try {
                            URL old = impl.url;
                            impl.url = new URL (newValue);
                            impl.pcs.firePropertyChange (PROP_URL, old, newValue);
                            impl.browserBase.getHistoryManager ().goingSomewhere (newValue, v.getId ());
                        }
                        catch (java.net.MalformedURLException ex) {
                            // unknown location URL - ignore this
                        }
                        impl.update ();
                    } else if (e.getPropertyName ().equals ("statusLine")) {
                        String old = impl.statusMessage;
                        impl.pcs.firePropertyChange (PROP_STATUS_MESSAGE, old, impl.statusMessage = (String) e.getNewValue ());
                        impl.oldStatus = impl.statusMessage;
                    } 
                }
            }
            else if (e.getSource () == impl.settings) {
                // changes of settings
                if (e.getPropertyName () == null) {
                    impl.setStyleSheet ();
                    System.getProperties ().put ("ice.pilots.html4.defaultEncoding", impl.settings.getEncoding ());   // NOI18N
                    System.getProperties ().put ("ice.pilots.html4.imageAnimation",(new Boolean (impl.settings.isAnimation ())).toString ());   // NOI18N
                    
                }
                else if (e.getPropertyName ().equals (ICEBrowserSettings.PROP_DEFAULT_BACKGROUND)) {
                    impl.setStyleSheet ();
                } 
                else if (e.getPropertyName ().equals (ICEBrowserSettings.PROP_FIXED_FONT)) {
                    impl.setStyleSheet ();
                } 
                else if (e.getPropertyName ().equals (ICEBrowserSettings.PROP_PROPORTIONAL_FONT)) {
                    impl.setStyleSheet ();
                } 
                else if (e.getPropertyName ().equals (ICEBrowserSettings.PROP_ENCODING)) {
                    System.getProperties ().put ("ice.pilots.html4.defaultEncoding", impl.settings.getEncoding ());   // NOI18N
                }
                else if (e.getPropertyName ().equals (ICEBrowserSettings.PROP_ANIMATION)) {
                    System.getProperties ().put ("ice.pilots.html4.imageAnimation",(new Boolean (impl.settings.isAnimation ())).toString ());   // NOI18N
                }
            }
        }
    }
    
    public static class IceCallback extends java.lang.Object implements ice.storm.StormCallback {
        
        private StormBase base;

        public void init(StormBase base) {
            this.base=base;
        }

        public Container createTopLevelContainer(Viewport view) {
            try {
                IceBrowserImpl impl = (IceBrowserImpl)windows.get(view.getName ());
                if (impl == null) {
// System.out.println("createTopLevelContainer for "+view.getId ()+ " ("+view.getName ()+")");
                    impl = IceBrowserModule.getIceBrowserFactory ().prebuildHtmlBrowserImpl (view);
                    HtmlBrowser.BrowserComponent br = new HtmlBrowser.BrowserComponent ();
                    br.open ();
                }
                Container w = (Container)impl.getComponent ();
                if (w != null) {
                    w.setVisible(true);
                }
               
                impl.setViewport (view);
                return w;
            }
            catch (NullPointerException ex) {
                ex.printStackTrace ();
            }
            catch (ClassCastException ex) {
                ex.printStackTrace ();
            }
            return null;
        }

        public void disposeTopLevelContainer(final ice.storm.Viewport view) {

            IceBrowserImpl impl = (IceBrowserImpl)windows.get(view.getName ());
            Container w = (Container)impl.getComponent ();
            if (w!=null) {
                windows.remove (view.getName ());
//                base.closeViewport (view.getId());
            }
        }
        
        public java.lang.String prompt(final ice.storm.Viewport viewport,String msg,String dflt) {
            NotifyDescriptor.InputLine desc = new NotifyDescriptor.InputLine (msg, "Browser request");
            desc.setInputText (dflt);
            TopManager.getDefault ().notify (desc);
            return desc.getInputText ();
        }
        
        public void message(final ice.storm.Viewport viewport,String msg) {
            TopManager.getDefault ().notify (new NotifyDescriptor.Message (msg));
        }
        
        public boolean confirm(final ice.storm.Viewport viewport,String str) {
            NotifyDescriptor.Confirmation desc = new NotifyDescriptor.Confirmation (str);
            return (TopManager.getDefault ().notify (desc) == NotifyDescriptor.OK_OPTION);
        }
        
    }
    
}
