/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.icebrowser;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import ice.storm.Viewport;

import org.openide.awt.HtmlBrowser;
import org.openide.modules.ModuleInstall;
import java.util.Properties;

/**
* Module installation class for Ice Browser
*
* @author Jan Jancura
*/
public class IceBrowserModule extends ModuleInstall {

    static final long serialVersionUID =1906033893126812896L;
    
    private static IceBrowserFactory fact = null;
    
    /** Module installed for the first time. */
    public void installed () {
        installIceBrowser ();
    }

    /** Module installed again. */
    public void restored () {
        installIceBrowser ();
    }

    private void installIceBrowser () {
        Properties p = System.getProperties ();
        p.put ("ice.browser.verbose", (Boolean.getBoolean ("netbeans.debug.exceptions")? "true": "false")); // NOI18N
        // p.put ("ice.iblite.installSecurityManager", "false"); // NOI18N
        // p.put ("ice.iblite.verbose", "false"); // NOI18N
        System.setProperties (p);

        try {
            Class.forName ("ice.storm.StormBase"); // NOI18N
        } catch (ClassNotFoundException e) {
            org.openide.TopManager.getDefault ().notifyException (e);
        }
        HtmlBrowser.setFactory (fact = new IceBrowserFactory () );
    }
    
    static IceBrowserFactory getIceBrowserFactory () {
        return fact;
    }
    
    public class IceBrowserFactory implements HtmlBrowser.Factory {
        
        IceBrowserImpl impl = null;
        /**
          * Returns a new instance of BrowserImpl implementation.
          */
        public HtmlBrowser.Impl createHtmlBrowserImpl () {
            IceBrowserImpl result = (impl == null)? new IceBrowserImpl (): impl;
            impl  = null;
            return result;
        }
        
        public IceBrowserImpl prebuildHtmlBrowserImpl (Viewport v) {
            impl = new IceBrowserImpl (v);
            return impl;
        }
    }

    // debug ..................................................................
    
    public static void main (String [] arg) {
        IceBrowserModule mod = new IceBrowserModule ();
        mod.installIceBrowser ();
        
        HtmlBrowser htmlViewer = new HtmlBrowser ();
        JDialog w = new JDialog ();
        w.addWindowListener (new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.out.println("Got windowClosing");
            }
            
            public void windowClosed(WindowEvent e) {
                System.out.println ("Got windowClosed");
                // System.exit (0);
            }
        });
        w.getContentPane().setLayout(new BorderLayout ());
        w.getContentPane ().add (htmlViewer, "Center"); // NOI18N
        htmlViewer.setVisible(true);
        w.setName ("External Web Browser");
        w.pack ();
        w.show ();
        htmlViewer.setURL ("http://www.netbeans.org");

        return;
    }
    
}
