/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.jarpackager;

import java.net.URL;
import java.util.jar.Manifest;
import java.util.Map;
import java.util.List;

import org.openide.loaders.DataObject;

/** Interface that abstracts content of jar archive file.
 *
 * @author  Dafe Simonek
 */
public interface ArchiveDescriptor extends java.io.Serializable {

    /** @return current list of contents */
    public ContentMemberList getContentList();
    
    /** @return currently asociated file object filter. */
    public FileObjectFilter getFilter ();

    /** Sets new filter for filtering file objects in archive.
    * @filter New filter */
    public void setFilter (FileObjectFilter filter);

    /** @return resource which contains jar
    * (or will contain jar if it is not created yet) */
    public URL getTargetResource ();

    /** Sets new target resource.
    * @param url new target URL */
    public void setTargetResource (URL url);

    /** @return current compression level. */
    public CompressionLevel getCompLevel ();

    /** Sets new compression level.
    * @compressionLevel new compression level
    */
    public void setCompLevel (CompressionLevel compressionLevel);

    /** @return the manifest of archive described by this descriptor.
    *
    * Note that when you modify the manifest returned from this method,
    * you will also update manifest of this desriptor.
    */
    public Manifest getManifest ();

    /** Sets the manifest of the archive.
    * @param manifest new manifest instance
    */
    public void setManifest (Manifest manifest);

    /** @return the map of extended properties */
    public Map getExtendedProperties();

}
