/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.jarpackager;

import java.util.zip.Deflater;
import org.netbeans.modules.jarpackager.options.JarPackagerOption;

/** Helper 'marker' class. We need special type of class for compression level,
* because we would like to be usable through PropertyPanel, for which
* special class type is neccessary. Functionality of this class
* is itentical to Integer class, only checking for validity is added.
*
* @author  Dafe Simonek
*/ 
public class CompressionLevel extends Object implements java.io.Serializable {

    static final long serialVersionUID = 4631233450985777767L;
    
    /** holds integer value - actual compression level */
    private Integer value;

    /** Constructs default compression level */
    public CompressionLevel () {
        this(JarPackagerOption.DEFAULT_COMPRESSION_LEVEL);
    }

    /** Constructs compression level according to given number */
    public CompressionLevel (int number) {
        value = new Integer(number);
        checkValue(number); 
    }

    /** Constructs compression level according to given integer string */
    public CompressionLevel (String string) throws NumberFormatException {
        this.value = new Integer(string);
        checkValue(value.intValue()); 
    }

    /** Constructs compression level from another instance - copy constuctor */
    public CompressionLevel (CompressionLevel other) {
        this.value = other.getValue();
    }
    
    
    public Integer getValue () {
        return value;
    }

    public void setValue(Integer value) {
    	this.value = value;
    }

    /** @return true if compression level is set to some compression level,
     * false when this instance represents no compression value 
     */
    public boolean isCompressed () {
        return value.intValue() != Deflater.NO_COMPRESSION;
    }

    private static void checkValue (int value) {
        if ((value < Deflater.NO_COMPRESSION) || (value > Deflater.BEST_COMPRESSION)) {
            throw new IllegalArgumentException("Invalid compression level"); //NOI18N
        }
    }

}
