/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.jarpackager;

import java.io.Serializable; 
import org.netbeans.modules.jarpackager.api.*;
import org.netbeans.modules.jarpackager.util.JarUtils;
import org.openide.loaders.DataObject;
import org.openide.TopManager;
import org.openide.filesystems.*;

/** This is one mwmber of a jar contents file
 */
public class ContentMember implements Serializable, ArchiveMember {

    static final long serialVersionUID = -7995415137147855602L;

    /** the DataObject to be placed in the jar file */
    DataObject dataObject;

    /** the filter specific to this member */
    ArchiveFilter filter;

    /** get the target directory within the jar */
    String targetDirectory;

    /** Whether to expand recursively (only applies if the DataObject is 
     *  a DataFolder) 
     */
    boolean isRecursive;

    /** Default constructor */
    public ContentMember() {
    }

    /** Construct from DataObject */
    public ContentMember(DataObject dataObj) {
    	dataObject = dataObj;
    }

    /** Construct from fields */
    public ContentMember(
      DataObject dataObject, ArchiveFilter filter, String targetDirectory) {
    	this.dataObject = dataObject;
    	this.targetDirectory = targetDirectory;
    	this.filter = filter;
	this.isRecursive = false;
    }

    /** Construct from fields */
    public ContentMember(
      DataObject dataObject, ArchiveFilter filter, String targetDirectory,
      boolean recursive) {
    	this.dataObject = dataObject;
    	this.targetDirectory = targetDirectory;
    	this.filter = filter;
	this.isRecursive = recursive;
    }

    /* ArchiveMember methods */
    /** Resolve any links */
    public ContentMember resolve() {
    	DataObject resolved = JarUtils.resolve(dataObject);
	if (resolved == dataObject) {
	    return this;
	}
	else {
	    return new ContentMember(resolved, this.filter,
		this.targetDirectory, this.isRecursive);
	}
    }

    /** return the entry's DataObject
    * @return the DataObject
    */
    public DataObject getDataObject() {
	return dataObject;
    }

    /** return the entry-specific filter 
    * @return the filter
    */
    public ArchiveFilter getFilter() {
	return filter;
    }

    /** return the target directory
    * @return the target directory
    */
    public String getTargetDirectory() {
	return targetDirectory;
    }

    /* Bean methods (for archiving) */

    /** Set the target directory
    * @param dir target directory
    */
    public void setTargetDirectory(String dir) {
	targetDirectory = dir;
    }

    /** Set the filter
    * @param f the filter
    */
    public void setFilter(ArchiveFilter f) {
	filter = f;
    }

    /* Bean methods */

    /** @return whether a folder is recursive
    */
    boolean isRecursive() {
	//System.out.println("returning " + isRecursive);
    	return isRecursive;
    }

    /** set whether a folder is recursive
    */
    void setRecursive(boolean r) {
    	isRecursive = r;
    }

    /** Save a data object as its primary file */
    String[] saveDataObject() {
	String result[];
    	try {
	    result = new String[2];
	    FileObject primary = dataObject.getPrimaryFile();
	    result[0] = primary.getFileSystem().getSystemName();
	    result[1] = primary.toString();
	    //System.out.println("returning " + result[0] + " " + result[1]);
	    return result;
	}
	catch (Exception ex) {
	    ex.printStackTrace();
	    result = new String[0];
	}

	return result;
    }

    /** Turn saved primary file name into DataObject */
    void restoreDataObject(String[] name) {
	if (name.length != 2) {
	    dataObject = null;
	}
	else {
	    try {
		Repository rep = TopManager.getDefault().getRepository();
		FileSystem fs = rep.findFileSystem (name[0]);
		FileObject fo = null;
		if (fs != null) {
		    // scan desired system
		    fo = fs.findResource(name[1]);
		}

		if (fo == null) {
		    // scan all systems
		    fo = rep.findResource(name[1]);
		}
	    }
	    catch (Exception ex) {
		ex.printStackTrace();
	    	dataObject = null;
	    }
	}
    }

    /** return a String representation.  In this case, just return the
    * name of the DataObject.
    */
    public String toString() {
    	return dataObject == null ? "<NULL>" : dataObject.getName();
    }
	
}
