/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.jarpackager;

import java.awt.Component;
import java.io.*;
import java.text.*;
import java.util.List;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Arrays;
import java.beans.PropertyEditor;
import java.beans.BeanInfo;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import java.awt.event.*;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

import javax.swing.AbstractListModel;
import javax.swing.ListSelectionModel;
import javax.swing.table.*;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ImageIcon;
import javax.swing.DefaultCellEditor;
import javax.swing.event.ChangeEvent;

import org.openide.util.NbBundle;
import org.openide.explorer.ExplorerPanel;
import org.openide.explorer.ExplorerManager;
import org.openide.explorer.view.BeanTreeView;
import org.openide.filesystems.FileObject;
import org.openide.TopManager;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataNode;
import org.openide.loaders.DataFolder;

import org.openide.explorer.view.NodeRenderer;
import org.openide.nodes.Node;
import org.openide.nodes.FilterNode;
import org.openide.util.HelpCtx;


import org.netbeans.modules.jarpackager.util.JarUtils;
import org.netbeans.modules.jarpackager.api.ArchiveMember;

/** One of default tabs contained in packaging panel. Allows users to choose
 * data objects they want to include to the archive being created or managed.
 * Works also as property editor for indexed property "pile of data objects"
 *
 * @author  Dafe Simonek
 * @version 
 */
public class DataObjectListEditor extends javax.swing.JPanel implements PropertyChangeListener, PropertyEditor,TableModelListener {


    
    private static java.util.ResourceBundle bundle = NbBundle.getBundle(JarPackagerModule.class);    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        sourcePanel = new javax.swing.JPanel();
        jPanel3 = new javax.swing.JPanel();
        jPanel1 = new javax.swing.JPanel();
        upButton = new javax.swing.JButton();
        rootButton = new javax.swing.JButton();
        buttonsPanel = new javax.swing.JPanel();
        addButton = new javax.swing.JButton();
        removeButton = new javax.swing.JButton();
        clearButton = new javax.swing.JButton();
        dummyPanel = new javax.swing.JPanel();
        chosenContentPanel = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        chosenContent = new javax.swing.JTable();
        jPanel2 = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        prefix = new javax.swing.JComboBox();
        featuresPanel = new javax.swing.JPanel();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        sourcePanel.setLayout(new java.awt.BorderLayout());
        
        sourcePanel.setPreferredSize(new java.awt.Dimension(250, 350));
        jPanel3.setLayout(new java.awt.BorderLayout());
        
        sourcePanel.add(jPanel3, java.awt.BorderLayout.CENTER);
        
        jPanel1.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT, 4, 4));
        
        upButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/netbeans/modules/jarpackager/resources/upFolder.gif")));
        upButton.setToolTipText(org.openide.util.NbBundle.getBundle(DataObjectListEditor.class).getString("CTL_UpButtonTip"));
        upButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        upButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                upButtonActionPerformed(evt);
            }
        });
        
        jPanel1.add(upButton);
        
        rootButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/netbeans/modules/jarpackager/resources/rootFolder.gif")));
        rootButton.setToolTipText(org.openide.util.NbBundle.getBundle(DataObjectListEditor.class).getString("CTL_RootButtonTip"));
        rootButton.setAlignmentY(0.0F);
        rootButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        rootButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                rootButtonActionPerformed(evt);
            }
        });
        
        jPanel1.add(rootButton);
        
        sourcePanel.add(jPanel1, java.awt.BorderLayout.NORTH);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(sourcePanel, gridBagConstraints1);
        
        buttonsPanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints2;
        
        addButton.setToolTipText(NbBundle.getBundle(DataObjectListEditor.class).getString("CTL_AddButtonTip"));
        addButton.setText(NbBundle.getBundle(DataObjectListEditor.class).getString("CTL_AddButton"));
        addButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.insets = new java.awt.Insets(30, 8, 8, 8);
        buttonsPanel.add(addButton, gridBagConstraints2);
        
        removeButton.setToolTipText(NbBundle.getBundle(DataObjectListEditor.class).getString("CTL_RemoveButtonTip"));
        removeButton.setText(NbBundle.getBundle(DataObjectListEditor.class).getString("CTL_RemoveButton"));
        removeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                removeButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.insets = new java.awt.Insets(8, 8, 8, 8);
        buttonsPanel.add(removeButton, gridBagConstraints2);
        
        clearButton.setToolTipText(NbBundle.getBundle(DataObjectListEditor.class).getString("CTL_ClearButtonTip"));
        clearButton.setText(NbBundle.getBundle(DataObjectListEditor.class).getString("CTL_ClearButton"));
        clearButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                clearButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.insets = new java.awt.Insets(8, 8, 8, 8);
        buttonsPanel.add(clearButton, gridBagConstraints2);
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints2.gridheight = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints2.weightx = 1.0;
        gridBagConstraints2.weighty = 1.0;
        buttonsPanel.add(dummyPanel, gridBagConstraints2);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 0;
        add(buttonsPanel, gridBagConstraints1);
        
        chosenContentPanel.setLayout(new java.awt.BorderLayout());
        
        chosenContentPanel.setPreferredSize(new java.awt.Dimension(250, 350));
        jScrollPane1.setPreferredSize(new java.awt.Dimension(200, 250));
        chosenContent.setBorder(new javax.swing.border.LineBorder(java.awt.Color.black));
        chosenContent.setModel(new javax.swing.table.DefaultTableModel(
        new Object [][] {
            {null, null, null, null},
            {null, null, null, null},
            {null, null, null, null},
            {null, null, null, null}
        },
        new String [] {
            "Title 1", "Title 2", "Title 3", "Title 4"
        }
        ) {
            Class[] types = new Class [] {
                java.lang.Object.class, java.lang.Object.class, java.lang.Object.class, java.lang.Object.class
            };
            
            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }
        });
        //chosenContent.setPreferredSize(new java.awt.Dimension(500, 400));
        //chosenContent.setRowHeight(14);
        //chosenContent.setPreferredScrollableViewportSize(new java.awt.Dimension(500, 400));
        //chosenContent.setMinimumSize(new java.awt.Dimension(500, 400));
        chosenContent.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                chosenContentFocusLost(evt);
            }
        });
        
        jScrollPane1.setViewportView(chosenContent);
        
        chosenContentPanel.add(jScrollPane1, java.awt.BorderLayout.CENTER);
        
        jPanel2.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
        
        jLabel2.setText(NbBundle.getBundle(DataObjectListEditor.class).getString("CTL_DirectoryPrefix"));
        jLabel2.setToolTipText(org.openide.util.NbBundle.getBundle(DataObjectListEditor.class).getString("CTL_DirectoryPrefixTip"));
        jPanel2.add(jLabel2);
        
        prefix.setToolTipText(org.openide.util.NbBundle.getBundle(DataObjectListEditor.class).getString("CTL_DirectoryPrefixTip"));
        prefix.setEditable(true);
        prefix.setPreferredSize(new java.awt.Dimension(140, 25));
        prefix.setName("Prefix");
        prefix.setMinimumSize(new java.awt.Dimension(140, 25));
        prefix.setMaximumSize(new java.awt.Dimension(140, 25));
        prefix.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                prefixActionPerformed(evt);
            }
        });
        
        jPanel2.add(prefix);
        
        chosenContentPanel.add(jPanel2, java.awt.BorderLayout.NORTH);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 2;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(chosenContentPanel, gridBagConstraints1);
        
        featuresPanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints3;
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(8, 0, 0, 0);
        gridBagConstraints1.weightx = 1.0;
        add(featuresPanel, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void chosenContentFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_chosenContentFocusLost
        // Add your handling code here:
             TableCellEditor editor = chosenContent.getCellEditor();
                 //System.out.println("FocusLost");
            if (editor != null) {
                Object value = editor.getCellEditorValue();
                //System.out.println("editing" + " " +value);
                model.setValueAt(value, chosenContent.getEditingRow(), chosenContent.getEditingColumn());
              }
       
    }//GEN-LAST:event_chosenContentFocusLost

    protected void prefixActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_prefixActionPerformed
        // Add your handling code here:
        //need to do validation
        prefix.addItem(prefix.getSelectedItem());
        
    }//GEN-LAST:event_prefixActionPerformed

    protected void upButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_upButtonActionPerformed
        // Add your handling code here:
        Node currentRoot = sourceExplorer.getExplorerManager().getRootContext();
        Node parentNode = currentRoot.getParentNode();
        if (parentNode != null)
                 sourceExplorer.getExplorerManager().setRootContext(parentNode);
       updateDirectoryButtons();
    }//GEN-LAST:event_upButtonActionPerformed

    protected void rootButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_rootButtonActionPerformed
        // Add your handling code here:
        Node[] selNodes = sourceExplorer.getExplorerManager().getSelectedNodes();
        if (selNodes.length == 1) {
            DataObject dataObj = (DataObject)selNodes[0].getCookie(DataObject.class);
            if (dataObj instanceof DataFolder)
                sourceExplorer.getExplorerManager().setRootContext(selNodes[0]);
        }
        updateDirectoryButtons();
    }//GEN-LAST:event_rootButtonActionPerformed

  protected void clearButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_clearButtonActionPerformed
// Add your handling code here:
        model.clear();
        chosenContent.clearSelection();
        support().firePropertyChange("", null, null);
 
        clearButton.setEnabled(false);
        // remove saving flag * from window title
  }//GEN-LAST:event_clearButtonActionPerformed

  protected void removeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_removeButtonActionPerformed
// Add your handling code here:
        int[] selVals = chosenContent.getSelectedRows();
        if (selVals.length <= 0)
            return;
        // obtain data objects and remove them from data model
        for (int i = 0; i < selVals.length; i++) {
            model.remove(selVals[i]);
        }
        chosenContent.clearSelection();
        support().firePropertyChange("", null, null);
 
        // setting, removing saving flag * from window title
  }//GEN-LAST:event_removeButtonActionPerformed

  protected void addButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addButtonActionPerformed
// Add your handling code here:
        Node[] selNodes = sourceExplorer.getExplorerManager().getSelectedNodes();
        // obtain data objects and add them to the data model
        Node rootNode = sourceExplorer.getExplorerManager().getRootContext();
        DataObject rootDataObject = (DataObject) rootNode.getCookie(DataObject.class);
        FileObject folder= null;
        int trim = 0;
        if (rootDataObject!= null && rootDataObject.getPrimaryFile()!=null)
            trim= rootDataObject.getPrimaryFile().getPackageName('/').length();
        if (trim!=0)
            trim +=1;
        boolean canADD;
        String dirPrefix = (String) prefix.getSelectedItem();
        for (int i = 0; i < selNodes.length; i++) {
             DataObject dataObj = (DataObject)selNodes[i].getCookie(DataObject.class);
             String target = "";
	     if (trim != 0 || (dirPrefix != null && dirPrefix.length() != 0)) {
		 FileObject theFile = dataObj.getPrimaryFile();
		 FileObject thePackage = 
		    (theFile.isFolder()) ? theFile : theFile.getParent();
		 if (!dataObj.equals(rootDataObject)) {
		    String packageName = thePackage.getPackageName('/');
		    if (trim < packageName.length()) {
		    	target = thePackage.getPackageName('/').substring(trim);
		    }
		 }
		 if (dirPrefix!=null) {
		    target = dirPrefix + target;
		 }
		 if (trim > 0 && target.length() == 0) {
		    target = ArchiveMember.ROOT;
		 }
	     }
             model.add(dataObj, target);
      }        
         
      if(model.getData().toArray().length>0) clearButton.setEnabled(true);
      support().firePropertyChange("", null, null);
   
  }//GEN-LAST:event_addButtonActionPerformed

  protected void chosenContentValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_chosenContentValueChanged
// Add your handling code here:
    removeButton.setEnabled(chosenContent.getSelectedRows().length > 0);  
    clearButton.setEnabled(model.getData().toArray().length > 0);
  }//GEN-LAST:event_chosenContentValueChanged

  public void updateDirectoryButtons()
  {
        Node currentRoot = sourceExplorer.getExplorerManager().getRootContext();
        Node parentNode = currentRoot.getParentNode();
        if (parentNode != null)
                 upButton.setEnabled(true);
        else
                 upButton.setEnabled(false);

        Node[] selNodes = sourceExplorer.getExplorerManager().getSelectedNodes();
        if (selNodes.length == 1) {
            DataObject dataObj = (DataObject)selNodes[0].getCookie(DataObject.class);
            if (dataObj instanceof DataFolder)
                rootButton.setEnabled(true);
            else
                rootButton.setEnabled(false);
        }
        else
           rootButton.setEnabled(false);
               

  }
  
  // Variables declaration - do not modify//GEN-BEGIN:variables
  protected javax.swing.JPanel sourcePanel;
  protected javax.swing.JPanel jPanel3;
  protected javax.swing.JPanel jPanel1;
  protected javax.swing.JButton upButton;
  protected javax.swing.JButton rootButton;
  protected javax.swing.JPanel buttonsPanel;
  protected javax.swing.JButton addButton;
  protected javax.swing.JButton removeButton;
  protected javax.swing.JButton clearButton;
  protected javax.swing.JPanel dummyPanel;
  protected javax.swing.JPanel chosenContentPanel;
  protected javax.swing.JScrollPane jScrollPane1;
  protected javax.swing.JTable chosenContent;
  protected javax.swing.JPanel jPanel2;
  protected javax.swing.JLabel jLabel2;
  protected javax.swing.JComboBox prefix;
  protected javax.swing.JPanel featuresPanel;
  // End of variables declaration//GEN-END:variables

    // Attributes

    // Associations
    /** support for firing */
    PropertyChangeSupport support;
    /** explorer for source */
    protected ExplorerPanel sourceExplorer;
    /** data model which holds chosen data objects */
    public ContentMemberListModel model;
    /** jar content we ae editing */
    JarContent jc;

    // Operation

    /** Creates new form ContentTab */
    public DataObjectListEditor() {
        initComponents();
        additionalInitialization();
        HelpCtx.setHelpIDString (this, DataObjectListEditor.class.getName ());

    }

    /** Helper method, initializes gui things that is not possible to do
    * with form editor */
    void additionalInitialization () {
        // set borders for source and chosen content panels
        // source panel title
        sourcePanel.setBorder (new javax.swing.border.TitledBorder(
                               new javax.swing.border.EtchedBorder(
                                   java.awt.Color.white,
                                   new java.awt.Color (134, 134, 134)),
                               NbBundle.getBundle(DataObjectListEditor.class).getString("CTL_SourcePanel")
                           ));
        // chosen content title
        chosenContentPanel.setBorder (new javax.swing.border.CompoundBorder(
                                new javax.swing.border.TitledBorder(
                                    new javax.swing.border.EtchedBorder(
                                        java.awt.Color.white,
                                        new java.awt.Color (134, 134, 134)),
                                    NbBundle.getBundle(DataObjectListEditor.class).getString("CTL_ChosenContent")
                                ),
                                new javax.swing.border.EmptyBorder(new java.awt.Insets(3, 3, 3, 3))));
        // source panel
        sourceExplorer = new ExplorerPanel();
        ExplorerManager manager = sourceExplorer.getExplorerManager();
        manager.setRootContext(
            TopManager.getDefault().getPlaces().nodes().repository());
        manager.addPropertyChangeListener(this);
        BeanTreeView treeView = new BeanTreeView();
        treeView.setPopupAllowed(false);
        sourceExplorer.setLayout(new java.awt.BorderLayout());
        sourceExplorer.add(treeView, java.awt.BorderLayout.CENTER);
        sourceExplorer.setPreferredSize(new java.awt.Dimension(200, 250));
        sourcePanel.add(sourceExplorer, java.awt.BorderLayout.CENTER);
        // initialize target list data model and cell renderer
        ListSelectionModel rowSM = chosenContent.getSelectionModel();
        rowSM.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                chosenContentValueChanged(e);
             }
        });
        initializeChosenContentTable();
        removeButton.setEnabled(false);
        addButton.setEnabled(false);   
        clearButton.setEnabled(false);
        upButton.setEnabled(false);
        rootButton.setEnabled(false);
    }

    public void initializeChosenContentTable(){
        model = new ContentMemberListModel();
        chosenContent.setModel(model);
        model.addTableModelListener(this);
        chosenContent.setDefaultRenderer(ContentMember.class, new ContentMemberTableRenderer());
        
        DefaultTableCellRenderer rend = new ContentMemberTableRenderer();
        rend.setToolTipText(bundle.getString("CTL_SourceTip"));
        chosenContent.getColumnModel().getColumn(ContentMemberListModel.SOURCE_COLUMN).setCellRenderer(rend);
        
        DefaultCellEditor editor = new DefaultCellEditor(new JTextField());
        rend = new DefaultTableCellRenderer();
        rend.setToolTipText(bundle.getString("CTL_DestinationTip"));
        chosenContent.getColumnModel().getColumn(ContentMemberListModel.DESTINATION_COLUMN).setCellEditor(editor);
        chosenContent.getColumnModel().getColumn(ContentMemberListModel.DESTINATION_COLUMN).setCellRenderer(rend);
        
        editor = new ContentMemberCellEditor(new JTextField());
        rend = new DefaultTableCellRenderer();
        rend.setToolTipText(bundle.getString("CTL_RegExpFieldTip"));
        chosenContent.getColumnModel().getColumn(ContentMemberListModel.FILTER_COLUMN).setCellEditor(editor);
        chosenContent.getColumnModel().getColumn(ContentMemberListModel.FILTER_COLUMN).setCellRenderer(rend);
      
    }
    /** implementation of property change listener, updates state
    * of buttons */
    public void propertyChange (java.beans.PropertyChangeEvent evt) {
        if (evt.getPropertyName().equals(ExplorerManager.PROP_SELECTED_NODES)) {
	    File target = jc.getTargetFile();
	    addButton.setEnabled(
		JarUtils.canAdd(jc, 
		(Node[])evt.getNewValue(),
		(target == null) ? null : target.getPath())
	    );
	    updateDirectoryButtons();
	 }
    }

    /******** implementation of PropertyEditor interface *********/

    public String getJavaInitializationString () {
        return null; // no code generation
    }

    public boolean supportsCustomEditor () {
        return true;
    }

    public java.awt.Component getCustomEditor() {
        return this;
    }

    public boolean isPaintable() {
        return false;
    }

    public void paintValue(java.awt.Graphics p1, java.awt.Rectangle p2) {
    }

    public String[] getTags() {
        // we don't support tags
        return null;
    }

    public void addPropertyChangeListener (PropertyChangeListener pchl) {
        support().addPropertyChangeListener(pchl);
    }

    public void removePropertyChangeListener(PropertyChangeListener pchl) {
        support().addPropertyChangeListener(pchl);
    }

    public void setValue (Object value) {
        if (!(value instanceof JarContent)) {
	    throw new IllegalArgumentException(value.toString());
	}
	jc = (JarContent)value;
        // create our value for modification instead of modifying given one
        model.clear();
        model.addList(jc.getContentList());
        support().firePropertyChange("", null, null);
    }

    public Object getValue () {
 	jc.setContentList(model.getData());
	return jc;
    }

     
    public void setAsText (String text) throws IllegalArgumentException {
        // not supported
        throw new IllegalArgumentException();
    }

    public String getAsText() {
	Integer size = new Integer(model.getData().size());
	return MessageFormat.format(
		bundle.getString("FMT_ContentMemberListSize"), 
		new Object[] {size} );
    }
    
    public ContentMemberListModel getModel(){
        return model;
    }

    public void clearContent(){
        model.clear();
        chosenContent.clearSelection();
        removeButton.setEnabled(false);  
    }
    
    /** Safe getter for property change support with lazy initialization */
    private PropertyChangeSupport support () {
        if (support == null) {
            support = new PropertyChangeSupport(this);
        }
        return support;
    }

    public void tableChanged(javax.swing.event.TableModelEvent p1) {
               support().firePropertyChange("", null, null);
    }    
   
    /** The data model for the list showing chosen content of the jar archive.
    * Simply as a list of root data objects 
    */
    public static class ContentMemberListModel extends AbstractListModel implements TableModel {
        /** List holding chosen data objects */
        public static final int SOURCE_COLUMN = 0;
        public static final int DESTINATION_COLUMN = 1;
        public static final int FILTER_COLUMN = 2;
        
        private static java.util.ResourceBundle bundle = NbBundle.getBundle(JarPackagerModule.class);    
        
        ContentMemberList dataObjs;
        String[] columnNames = {bundle.getString("PROP_Source"),bundle.getString("PROP_Destination"),bundle.getString("PROP_Filter")};
        /** Creates the model with default collection size */
        public ContentMemberListModel () {
            this(10);
        }

        /** Creates the model with specified collection size */
        public ContentMemberListModel (int size) {
            dataObjs = new ContentMemberList(size);
        }
        
        
        /* methods that implement TableModel */        
        public int getColumnCount() {
            return 3;
        }
        public int getRowCount() {
            return getSize();
        }
        
        public String getColumnName(int columnIndex) {    
            return columnNames[columnIndex];   
        }
        
        public Class getColumnClass(int columnIndex) {    
            return ContentMember.class;   
        }
       
        public Object getValueAt (int rowIndex, int columnIndex) {
            if (columnIndex == DESTINATION_COLUMN)
                return((ContentMember)dataObjs.get(rowIndex)).getTargetDirectory();
            if (columnIndex == FILTER_COLUMN)
                return((ContentMember)dataObjs.get(rowIndex)).getFilter();
            return((ContentMember)dataObjs.get(rowIndex));
        }
        
        public void setValueAt (Object aValue, int rowIndex, int columnIndex) {
            if (columnIndex == DESTINATION_COLUMN && aValue instanceof String) {
                ((ContentMember)dataObjs.get(rowIndex)).setTargetDirectory((String) aValue);
                fireTableChanged();
            }
            if (columnIndex == FILTER_COLUMN && aValue instanceof String) {
                try {
                    ((ContentMember)dataObjs.get(rowIndex)).setFilter(new REFilter((String) aValue));
                    fireTableChanged();
                }
                catch (Exception e)
                {
                    //fixme
                }
            }
        }
        
        public boolean isCellEditable(int rowIndex, int columnIndex) { 
            if (columnIndex == DESTINATION_COLUMN)
                return true;
            if (columnIndex == FILTER_COLUMN) {
               ContentMember member = (ContentMember)dataObjs.get(rowIndex);
               if ((member.getFilter()==null) || member.getFilter() instanceof REFilter)
                   return true;
               else
                   return false;
            }
            return false;
        }
        
        public void addTableModelListener(TableModelListener l) {
                 listenerList.add(TableModelListener.class, l);
        }
        public void removeTableModelListener(TableModelListener l){
                listenerList.remove(TableModelListener.class, l);
        }
        
         protected void fireTableChanged(TableModelEvent tableEvent) {
            // Guaranteed to return a non-null array
            Object[] listeners = listenerList.getListenerList();
            // Process the listeners last to first, notifying
            // those that are interested in this event
             for (int i = listeners.length-2; i>=0; i-=2) {
                 if (listeners[i]==TableModelListener.class) {
                        ((TableModelListener)listeners[i+1]).tableChanged(tableEvent);
                  }
              }
        }
        
        protected void fireTableChanged() {
            fireTableChanged(new TableModelEvent(this));
        }
        /* end methods that implement TableModel */        
  
        /** Returns the value at the specified index. */
        public Object getElementAt (int index) {
            return new XDataNode(((ContentMember)dataObjs.get(index)).getDataObject().getNodeDelegate());
        }

        /** Returns the length of the list. */
        public int getSize () {
            return dataObjs.size();
        }

        /** Adds specified data object to the model */
        boolean add (DataObject dataObj) {
            if (dataObjs.addMember(dataObj)) {
                int size = dataObjs.size();
                fireContentsChanged(this, size - 1, size - 1);
                fireTableChanged();
                return true;
            }
            return false;
        }
       boolean add (DataObject dataObj,String target) {
            if (dataObjs.addMember(dataObj,target)) {
                int size = dataObjs.size();
                fireContentsChanged(this, size - 1, size - 1);
                fireTableChanged();
                return true;
            }
            return false;
        }

        /** Removes specified data object from the model */
        boolean remove (DataObject dataObj) {
            boolean result = false;
            int index=dataObjs.findDataObject(dataObj);
             if (index >= 0 && dataObjs.removeMember(dataObj)) {
                result = true;
                int size = dataObjs.size();
               fireContentsChanged(this, index, size - 1);
               fireTableChanged();
           }
            return result;
        }
        boolean remove (int index) {
            boolean result = false;
             if (dataObjs.removeMember(index)) {
                result = true;
                int size = dataObjs.size();
               fireContentsChanged(this, index, size - 1);
               fireTableChanged();
           }
            return result;
        }

        /** Adds all items in given collection */
        void addList (ArrayList list) {
            int prevSize = dataObjs.size();
            dataObjs.addList(list);
            fireContentsChanged(this, Math.max(prevSize - 1, 0), dataObjs.size() - 1);
            fireTableChanged();

        }
        
 
        /** Clears all data in the model */
        void clear () {
            int prevSize = dataObjs.size();
            dataObjs.clear();
            fireContentsChanged(this, 0, Math.max(prevSize - 1, 0));
            fireTableChanged();
        }

        ContentMemberList getData () {
            return dataObjs;
        }         
    }
    
   static class XDataNode extends FilterNode {
       
        public XDataNode (Node orig) {
           super (orig);
           disableDelegation(FilterNode.DELEGATE_SET_DISPLAY_NAME | FilterNode.DELEGATE_GET_DISPLAY_NAME);
          
           DataObject dataObj = ((DataNode)orig).getDataObject();
           setDisplayName(dataObj.getPrimaryFile().getPackageName('/'));
        }
        
         public XDataNode (Node orig, ContentMember member) {
           this (orig);
           if (member.getTargetDirectory() != null)
                setDisplayName(member.getTargetDirectory());
        }
 
   }

   public static class ContentMemberCellEditor extends DefaultCellEditor implements FocusListener {
       
       ContentMemberCellEditor(JTextField textField) {
           super(textField);
           textField.addFocusListener(this);
       }    
       public void focusGained(FocusEvent e) {
       }
       
       public void focusLost(FocusEvent e) {
           fireEditingStopped();
       }
   }    
   public static class ContentMemberTableRenderer extends DefaultTableCellRenderer {
       public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus,int row, int column) {
                                                   
           if (value instanceof ContentMember){
               if(column==ContentMemberListModel.SOURCE_COLUMN) {
                  ContentMember member = (ContentMember) value;
                  setIcon( new ImageIcon(member.getDataObject().getNodeDelegate().getIcon(BeanInfo.ICON_COLOR_16x16)));
                  String name = member.getDataObject().getNodeDelegate().getDisplayName();
                  try {
                     if ("".equals(name) && member.getDataObject().getPrimaryFile().isRoot()){
                     // special case root filesystems
                        name =  member.getDataObject().getPrimaryFile().getFileSystem().getDisplayName();
                     }
                  }
                  catch (Exception e)
                  {
                      // do nothing
                  }
                  return super.getTableCellRendererComponent(table,name,isSelected,hasFocus,row,column);
               }
               else if (column == ContentMemberListModel.DESTINATION_COLUMN){
                 ContentMember member = (ContentMember) value;
                 return super.getTableCellRendererComponent(table,member.getTargetDirectory(),isSelected,hasFocus,row,column);
               }
               else if (column == ContentMemberListModel.FILTER_COLUMN){
                 ContentMember member = (ContentMember) value;
                 return super.getTableCellRendererComponent(table,member.getFilter(),isSelected,hasFocus,row,column);
               }
                  
                  
           }
         return super.getTableCellRendererComponent(table,value,isSelected,hasFocus,row,column);
       }                                                
   }

}
