/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.jarpackager;

import java.util.*;
import java.beans.*;
import java.awt.Image;

import org.openide.util.NbBundle;
import org.openide.loaders.MultiFileLoader;

/** BeanInfo for java source loader.
*
* @author Petr Hamernik, Dafe Simonek
*/
public final class JarDataLoaderBeanInfo extends SimpleBeanInfo {
    /** Prefix of the icon location. */
    private static String ICON_PREFIX = "/org/netbeans/modules/jarpackager/resources/jarObject"; // NOI18N

    
    public BeanInfo[] getAdditionalBeanInfo () {
        try {
            return new BeanInfo[] { Introspector.getBeanInfo (MultiFileLoader.class) };
        } catch (IntrospectionException ie) {
            if (Boolean.getBoolean ("netbeans.debug.exceptions")) // NOI18N
                ie.printStackTrace ();
            return null;
        }
    }

    /** Icons for compiler settings objects. */
    private static Image icon;
    private static Image icon32;

    /** @param type Desired type of the icon
    * @return returns the Jarpackager loader's icon
    */
    public Image getIcon(final int type) {
        if ((type == BeanInfo.ICON_COLOR_16x16) || (type == BeanInfo.ICON_MONO_16x16)) {
            if (icon == null)
                icon = loadImage(ICON_PREFIX + ".gif"); // NOI18N
            return icon;
        }
        else {
            if (icon32 == null)
                icon32 = loadImage(ICON_PREFIX + "32.gif"); // NOI18N
            return icon32;
        }
    }
    
    private static PropertyDescriptor[] descriptors;
    
    // Return bean descriptor for versioning.  Note that "1.1"
    // means NetBeans 3.2
    static BeanDescriptor bdesc;
    static {
	ResourceBundle bundle = 
		NbBundle.getBundle(JarDataLoaderBeanInfo.class);

        bdesc = new BeanDescriptor (JarDataLoader.class);
        bdesc.setDisplayName (bundle.getString("CTL_JarPackager_Name"));
        bdesc.setValue ("version", "1.1"); // NOI18N
    }

    public BeanDescriptor getBeanDescriptor () {
        return bdesc;
    }


    public PropertyDescriptor[] getPropertyDescriptors() {
        if (descriptors != null)
            return descriptors;

        try {
	    PropertyDescriptor extension = new PropertyDescriptor("extension", JarDataLoader.class);
            PropertyDescriptor archiveExt = new PropertyDescriptor("archiveExt", JarDataLoader.class);
	    
	    ResourceBundle bundle = NbBundle.getBundle(JarDataLoader.class);
            extension.setDisplayName(bundle.getString("PROP_JarLoaderExtension"));
	    extension.setShortDescription(bundle.getString("HINT_JarLoaderExtension"));

            archiveExt.setDisplayName (bundle.getString("PROP_JarLoaderArchiveExt"));
            archiveExt.setShortDescription (bundle.getString("HINT_JarLoaderArchiveExt"));

            descriptors = new PropertyDescriptor[] { extension, archiveExt };
	} catch (IntrospectionException ex) {
	    if (Boolean.getBoolean("netbeans.debug.exceptions"))
		ex.printStackTrace();
	    return null;
	}
        return descriptors;
    }
}
