/*
*                 Sun Public License Notice
* 
* The contents of this file are subject to the Sun Public License
* Version 1.0 (the "License"). You may not use this file except in
* compliance with the License. A copy of the License is available at
* http://www.sun.com/
* 
* The Original Code is NetBeans. The Initial Developer of the Original
* Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
* Microsystems, Inc. All Rights Reserved.
*/

package org.netbeans.modules.jarpackager;

import java.awt.Image;
import java.awt.Toolkit;
import java.beans.*;
import java.io.*;
import java.lang.reflect.InvocationTargetException;
import java.util.Enumeration;
import java.net.URL;
import java.net.MalformedURLException;
import java.util.jar.Manifest;
import java.util.*;

import org.openide.*;
import org.openide.cookies.*;
import org.openide.compiler.Compiler;
import org.openide.compiler.CompilerJob;
import org.openide.execution.*;
import org.openide.explorer.propertysheet.editors.FileOnlyEditor;
import org.openide.filesystems.*;
import org.openide.loaders.*;
import org.openide.windows.*;
import org.openide.actions.*;
import org.openide.text.*;
import org.openide.util.*;
import org.openide.util.actions.*;
import org.openide.nodes.Node;
import org.openide.nodes.*;
import org.openide.nodes.Sheet;
import org.openide.nodes.Children;

import org.netbeans.modules.jarpackager.options.JarPackagerOption;
import org.netbeans.modules.jarpackager.util.JarUtils;

import org.netbeans.modules.jarpackager.api.*;

/*****
import com.sun.java.beans.XMLEncoder;
*****/

/** Object representing one JAR file that can be updated.
*
* @author Dafe Simonek, Jaroslav Tulach
*/
public class JarDataObject extends MultiDataObject
	implements OpenCookie,
	CompilerCookie.Compile,
	CompilerCookie.Build {
    /** generated Serialized Version UID */
    static final long serialVersionUID = 2983457824351249776L;

    static final String EA_JAR_CONTENT = "NetBeans-JarContent"; // NOI18N

    transient JarContent jarContent;

    /** Constructs new JarDataObject.
    */
    public JarDataObject (FileObject obj, MultiFileLoader loader) 
    throws DataObjectExistsException {
	super (obj, loader);
	getCookieSet().add(new JarExecSupport(getPrimaryEntry()));	
	//System.out.println("new JarDataObject");
    }

    /** Opens the Jar file. (Implementation of the open cookie)
    */
    public void open () {
	// PENDING - FileObject ---> File --> if exists ---> add to repository
    }

    /** mark unmodified, so we don't stupidly get propted to save it on exit */
    protected void handleDelete() throws IOException {
	super.handleDelete();
	setModified(false);
    }

    /** in addition to usual DataObject stuff, enable and disable Save cookie */
    public void setModified(boolean mod) {
	CookieSet cookies = getCookieSet();
	Save saveCookie = (Save)cookies.getCookie(Save.class);
	if (mod && saveCookie == null) {
	    cookies.add(new Save());
	}
	else if (!mod && saveCookie != null) {
	    cookies.remove(saveCookie);
	}
    	super.setModified(mod);
    }

    public void save() throws IOException {
	writeJarContent(getJarContent());
	this.setModified(false);
    }

    private class Save implements SaveCookie {
    	public void save() throws IOException {
	    JarDataObject.this.save();
	}
    }

    /** Accepts depth one.
    * @return <code>true</code> if this cookie supports depth one
    */
    public boolean isDepthSupported (Compiler.Depth depth) {
	return Compiler.DEPTH_ONE.equals(depth);
    }

    /** Creates jar compiler with given job.
    */
    public void addToJob (CompilerJob job, Compiler.Depth depth) {
	//System.out.println("JDO " + getPrimaryFile() + " being added to job...");
	//Thread.dumpStack();
	JarContent jc = getJarContent();
	// do not compile if jar content is not valid
	if (jc == null) {
	    //System.out.println("jc is null");
	    return;
	}
	// obtain archive file (create if not exist)
	File archiveFile = jc.getTargetFile();
	if (archiveFile == null) {
	    FileObject contentFo = findContentFile();
	    String archiveExt = ((JarDataLoader)getLoader()).getArchiveExt();
	    FileObject fo = FileUtil.findBrother(contentFo, archiveExt);
	    if (fo == null) {
		try {
		    fo = contentFo.getParent().createData(
		    			contentFo.getName(), archiveExt);
		} catch (IOException exc) {
		    TopManager.getDefault().notify(
		    	new NotifyDescriptor.Exception(
			   exc,
			   NbBundle.getBundle(JarDataObject.class).
			   getString("MSG_CreateArchiveError"))
			);
		    return;
		}
	    }
	    jc.setTargetFile(NbClassPath.toFile(fo));
	    this.setModified(true);
	}
	// update manifest
	//JarUtils.updateManifest(jc);
	// add compiler which manages compilation
	job.add(new JarCompiler(jc,this));
    }

    /** Sets the description of the jar content to given filedata object.
    * Jar content description will be stored in the extended attributes,
    * allowing to repackage archive whenever needed.
    */   
    public void writeJarContent (JarContent jarContent) throws IOException {
	FileObject jcFile = findContentFile();
	// write jar content
	FileObject pfo = getPrimaryFile();
	FileLock lock = null;
	try {
	    if (jcFile == null) {
		// create jar content information file
		jcFile = pfo.getParent().createData(
		    pfo.getName(),
		    JarPackagerOption.singleton().getContentExt()
		);
	    }
	    lock = jcFile.lock();
	    ObjectOutputStream oos =
		new ObjectOutputStream(jcFile.getOutputStream(lock));
	    try {
		jarContent.writeContent(oos);
	    } finally {
		oos.close();
	    }
	} finally {
	    if (lock != null)
		lock.releaseLock();
	}

	/*****************
	ClassLoader ctx = Thread.currentThread().getContextClassLoader();
	try {
	    Thread.currentThread().setContextClassLoader(
	    JarContent.class.getClassLoader());
	    String ext = JarPackagerOption.singleton().getContentExt() + ".xml";
	    FileObject xmlFile = 
		pfo.getParent().getFileObject(pfo.getName(), ext);
	    if (xmlFile != null) {
		try {
		    lock = xmlFile.lock();
		    xmlFile.delete(lock);
		}
		finally {
		    lock.releaseLock();
		}
	    }

	    xmlFile = pfo.getParent().createData(pfo.getName(), ext);
	    lock = xmlFile.lock();
	    XMLEncoder enc = new XMLEncoder(xmlFile.getOutputStream(lock));
	    enc.writeObject(jarContent);
	    enc.close();
	}
	catch(Exception ex) {
	    ex.printStackTrace();
	}
	finally {
	    lock.releaseLock();
	    Thread.currentThread().setContextClassLoader(ctx);
	}
	*****************/
    }

    /** @return new instance of current description of the jar content.
    * or null if jar content cannot be found for some reason.
    * Note that if you wish to modify jar content, you will have to call
    * setModified(true)() after modification */
    public JarContent getJarContent () {

	if (jarContent == null) {
	    FileObject jcFile = findContentFile();
	    if (jcFile == null) {
		return null;
	    }
	    jarContent = JarContent.readFromFile(jcFile);
	}
	return jarContent;
    }

    /** @return file object of jar content file describing archive
    * or null if no content file can be found.
    * Subclasses can override this to provide their own
    * content file */
    protected FileObject findContentFile () {
	return getPrimaryFile();
    }

    /** Utility method, returns file object of the archive or
    * null if no archive exist */
    public FileObject findArchiveFile () {
	FileObject contentFo = findContentFile();
	return FileUtil.findBrother(
	    contentFo,
	    ((JarDataLoader)getLoader()).getArchiveExt()
	);
    }

    /** construct default file name for archive */
    public File defaultArchiveFile() {
	return defaultArchiveFile(NbClassPath.toFile(getPrimaryFile()), 
				 ((JarDataLoader)getLoader()).getArchiveExt());
    }

    /** construct default file name for archive */
    public static File defaultArchiveFile(File content, String extension) {
	if (content != null) {
	    String jarContentName;
	    try {
		jarContentName = content.getCanonicalPath();
	    }
	    catch (IOException ex) {
	    	return null;
	    }
	    String jarBase = jarContentName;
	    JarDataLoader loader = 
		(JarDataLoader)JarDataLoader.findObject(JarDataLoader.class);
	    int ext = 
		jarContentName.lastIndexOf("." + loader.getExtension());
	    if (ext > 0) {
		jarBase = jarContentName.substring(0, ext);
	    }
	    
	    return new File(jarBase + "." + extension);
	}

	return null;
    }

    /** is the default archive file used? */
    public boolean usesDefaultArchiveFile() {
    	File target = getJarContent().getTargetFile();
	File def = defaultArchiveFile();
	try {
	    return target != null && 
	       target.getCanonicalPath().equals(def.getPath());
	}
	catch (IOException ex) {
	    return false;
	}
    }

    /** Provides node that should represent this data object. When a node for 
    * representation
    * in a parent is requested by a call to getNode (parent) 
    * it is the exact copy of this node
    * with only parent changed. This implementation creates instance
    * <CODE>DataNode</CODE>.
    * <P>
    * This method is called only once.
    *
    * @return the node representation for this data object
    * @see DataNode
    */
    protected Node createNodeDelegate () {
	return new JarNode(this);
    }

    /** Help context for this object.
    * @return help context
    */
    public HelpCtx getHelpCtx () {
	return new HelpCtx(JarDataObject.class);
    }


    /** Process Copy command
    */
    protected synchronized DataObject handleCopy (DataFolder df) throws IOException {
        DataObject obj = super.handleCopy (df);
        try {
	    // Some DataLoaders (in the web module, for instance) create
	    // a different sort of DataObject for jar files. Remove the
	    // JarDataObject created by copy and let the DataLoader create
	    // the proper sort.
            obj.setValid (false);
        } catch (PropertyVetoException ex) {
            //System.err.println("Exception caught: " + ex);
        }
        return obj;
    }

    /** Jar data Node implementation.
    * Leaf node, icons and name redefined.
    */
    public static class JarNode extends DataNode {
	/** Icon base for the JarNode node */
	static final String JAR_ICON_BASE =
	    "org/netbeans/modules/jarpackager/resources/jarObject"; // NOI18N
	private static java.util.ResourceBundle bundle = 
	    NbBundle.getBundle(JarPackagerModule.class);

	/** Default constructor, constructs node */
	public JarNode (DataObject dataObject) {
	    this(dataObject, Children.LEAF);
	}

	/** Constructs node with specified data object and with given
	* children */
	public JarNode (DataObject dataObject, Children children) {
	    super(dataObject, children);
	    setIconBase (JAR_ICON_BASE);
	    setDefaultAction (SystemAction.get(PropertiesAction.class));
	}

	/** Create the property sheet for this node.
	* @return the sheet
	*/
	protected Sheet createSheet () {
	    Sheet sheet = super.createSheet();
	    sheet.get(Sheet.PROPERTIES).put(createLocationProperty());
	    sheet.get(Sheet.PROPERTIES).put(createCompressionProperty());
	    sheet.get(Sheet.PROPERTIES).put(createFilterProperty());
	    sheet.get(Sheet.PROPERTIES).put(createManifestProperty());
	    sheet.get(Sheet.PROPERTIES).put(createContentsProperty());
	    addExtendedProperties(sheet);
	    // create execution set
	    Sheet.Set ps = new Sheet.Set ();
	    ps.setName(
		NbBundle.getBundle(JarDataObject.class).getString(
							"CTL_ExecutionSet")
	    );
	    ps.setDisplayName(NbBundle.getBundle(JarDataObject.JarNode.class).
	    getString("PROP_executionSetName"));
	    ps.setShortDescription(
	    	NbBundle.getBundle(JarDataObject.JarNode.class).
	    getString("HINT_executionSetName"));
	    // fill execution properties
            // #10827: subclasses might not have JarExecSupport cookie
	    //((JarExecSupport)getCookie(JarExecSupport.class)).addProperties(ps);
            ExecSupport exec = (ExecSupport)getCookie(ExecSupport.class);
            if (exec != null) {
                exec.addProperties(ps);
                ps.remove(ExecSupport.PROP_DEBUGGER_TYPE);
            }
	    sheet.put(ps);
	    return sheet;
	}

	public JarContent getJarContent() {
	    return ((JarDataObject)getDataObject()).getJarContent();
	}

	public void setModified (boolean mod)  {
	    ((JarDataObject) getDataObject()).setModified(mod);
	}

	public FileObject getJarContentFile() {
	    return getDataObject().getPrimaryFile();
	}

	public void addExtendedProperties(Sheet sheet) {

	    Map definedProps = JarCreator.getExtendedProperties();
	    if (definedProps == null){
		return;
	    }
	    Iterator props = definedProps.values().iterator();
	    while (props.hasNext()) {
		ExtendedPropertyFactory fact = 
		    (ExtendedPropertyFactory)props.next();
		Node.Property prop = 
			fact.createProperty(getDataObject(), getJarContent());
		sheet.get(Sheet.PROPERTIES).put(prop);
	    }

	}

	public Node.Property createLocationProperty() {
	    return new PropertySupport.ReadWrite
		("location", 
		File.class,
		bundle.getString("PROP_Location"),
		bundle.getString("CTL_LocationTip")) {

		    public Object getValue () {
			return getJarContent().getTargetFile();
		    }

		    public void setValue (final Object val) 
			throws IllegalArgumentException, IllegalAccessException,
			InvocationTargetException {
			if (! (val instanceof File)) {
			    throw new IllegalArgumentException ();
			}
			try {
			    JarContent jc = getJarContent();

			    jc.setTargetFile((File) val);
			    setModified(true);                 
			}
			catch (Exception e) {
			    throw new IllegalArgumentException ();
			}
		    }

		    public PropertyEditor getPropertyEditor () {
			return new FileOnlyEditor ();
		    }

		};

	}	

	public Node.Property createCompressionProperty() {
	    return new PropertySupport.ReadWrite
		("compression", 
		CompressionLevel.class,
		bundle.getString("PROP_Compression"),
		bundle.getString("CTL_CompressionLevelTip")) {

		    public Object getValue () {
			return getJarContent().getCompLevel();
		    }

		    public void setValue (final Object val) 
			throws IllegalArgumentException, IllegalAccessException,
			InvocationTargetException {
			if (! (val instanceof CompressionLevel)) {
			    throw new IllegalArgumentException ();
			}
			try {
			    JarContent jc = getJarContent();
			    jc.setCompLevel( (CompressionLevel) val);
			    setModified(true);
			}
			catch (Exception e) {
			    throw new IllegalArgumentException ();
			}

		    }

		    public PropertyEditor getPropertyEditor () {
			return new CompressionEditor ();
		    }

		};

	}	
	public Node.Property createFilterProperty() {
	    Node.Property filterProp = new PropertySupport.ReadWrite
		("filter", 
		FileObjectFilter.class,
		bundle.getString("PROP_Filter"),
		bundle.getString("CTL_FilterTip")) {

		    public Object getValue () {
			return getJarContent().getFilter();
		    }

		    public void setValue (final Object val) 
		    throws IllegalArgumentException, IllegalAccessException,
		    InvocationTargetException {
			if (! (val instanceof FileObjectFilter)) {
				throw new IllegalArgumentException ();
			}
			try {
			    JarContent jc = getJarContent();
			    jc.setFilter( (FileObjectFilter) val);
			    setModified(true);

			}
			catch (Exception e) {
			    throw new IllegalArgumentException (); 
			} 
		    }

		    public PropertyEditor getPropertyEditor () {
			return new FilterEditor ();
		    }

		};

	    filterProp.setValue("canEditAsText", Boolean.FALSE);// NOI8N
	    return filterProp;
	}	

	public Node.Property createManifestProperty() {
	    Node.Property manifestProp = 
	    new PropertySupport.ReadWrite
		("manifest", 
		Manifest.class,
		bundle.getString("PROP_Manifest"),
		bundle.getString("CTL_ManifestTip")) {

		    public Object getValue () {
			return getJarContent().getManifest();
		    }

		    public void setValue (final Object val) 
		    throws IllegalArgumentException, IllegalAccessException,
		    InvocationTargetException {
			if (! (val instanceof Manifest)) {
			    throw new IllegalArgumentException ();
			}
			try {
			    JarContent jc = getJarContent();
			    jc.setManifest( (Manifest) val);
			    setModified(true);
			}
			catch (Exception e) {
			    throw new IllegalArgumentException ();
			}
		    }

		    public PropertyEditor getPropertyEditor () {
			ManifestTab editor = new ManifestTab();
			editor.setJarContentFile(
			    getJarContentFile().getPackageNameExt('/', '.'));
			return editor;
		    }

		};
		manifestProp.setValue("canEditAsText", Boolean.FALSE);// NOI8N
		return manifestProp;

	}	

	public Node.Property createContentsProperty() {
	    Node.Property contentsProp = new PropertySupport.ReadWrite
		("contents", 
		JarContent.class,
		bundle.getString("PROP_Contents"),
		bundle.getString("CTL_ContentsTip")) {

		    public Object getValue () {
			return getJarContent().makeCopy();
		    }

		    public void setValue (final Object val) 
		    throws IllegalArgumentException, IllegalAccessException,
		    InvocationTargetException {
			if (! (val instanceof JarContent)) {
			    throw new IllegalArgumentException ();
			}
			try {
			    getJarContent().setContentList(
			    	((JarContent)val).getContentList());
			    setModified(true);
			}
			catch (Exception e) {
			    throw new IllegalArgumentException ();
			}
		    }

		    public PropertyEditor getPropertyEditor () {
			return new DataObjectListEditor ();
		    }

		};
	    contentsProp.setValue("canEditAsText", Boolean.FALSE);// NOI8N
	    return contentsProp;

	}	

    } // end of JarNode inner class

    static class JarExecSupport extends ExecSupport {

       /** Create new support for given entry. The file is taken from the
	* entry and is updated if the entry moves or renames itself.
	* @param entry entry to create instance from
	*/
	public JarExecSupport (MultiDataObject.Entry entry) {
	    super(entry);
	}

	    protected Executor defaultExecutor () {
	    return new JarExecutor();
	}
    }

}

/*
* <<Log>>
*  25   Jaga      1.23.1.0    4/20/00  David Simonek   
*  24   Gandalf   1.23        1/25/00  David Simonek   Various bugfixes and 
*       i18n
*  23   Gandalf   1.22        1/16/00  David Simonek   i18n
*  22   Gandalf   1.21        11/9/99  David Simonek   addToJob bug fixed 
*       (concerning null jar content)
*  21   Gandalf   1.20        10/23/99 Ian Formanek    NO SEMANTIC CHANGE - Sun
*       Microsystems copyright in file comment
*  20   Gandalf   1.19        10/14/99 David Simonek   manifest updating 
*       bugfixes
*  19   Gandalf   1.18        10/13/99 David Simonek   jar content now primary 
*       file, other small changes
*  18   Gandalf   1.17        10/7/99  Jesse Glick     Fixed problem with 
*       execution sheet being blank.
*  17   Gandalf   1.16        10/4/99  David Simonek   
*  16   Gandalf   1.15        9/16/99  David Simonek   a lot of bugfixes (RE 
*       filters, empty jar content etc)  added templates
*  15   Gandalf   1.14        9/13/99  David Simonek   bugfixes, compressed 
*       on/off support fixed
*  14   Gandalf   1.13        9/8/99   David Simonek   new version of jar 
*       packager
*  13   Gandalf   1.12        8/17/99  David Simonek   installations of 
*       actions, icon changing
*  12   Gandalf   1.11        8/1/99   David Simonek   automatic file list 
*       generation to the manifest added
*  11   Gandalf   1.10        7/11/99  David Simonek   
*  10   Gandalf   1.9         6/10/99  David Simonek   progress indocator + 
*       minor bugfixes....
*  9    Gandalf   1.8         6/10/99  David Simonek   progress dialog now 
*       functional
*  8    Gandalf   1.7         6/9/99   Ian Formanek    ---- Package Change To 
*       org.openide ----
*  7    Gandalf   1.6         6/8/99   David Simonek   
*  6    Gandalf   1.5         6/8/99   David Simonek   bugfixes....
*  5    Gandalf   1.4         6/4/99   Petr Hamernik   temporary version
*  4    Gandalf   1.3         6/4/99   David Simonek   executor properties set 
*       added
*  3    Gandalf   1.2         6/4/99   David Simonek   
*  2    Gandalf   1.1         6/4/99   David Simonek   compile action on jar 
*       data object
*  1    Gandalf   1.0         6/4/99   Jaroslav Tulach 
* $
* Beta Change History:
*  0    Tuborg    0.54        --/--/98 Jan Formanek    reflecting changes in cookies
*  0    Tuborg    0.55        --/--/98 Jan Formanek    templates
*  0    Tuborg    0.56        --/--/98 Petr Hamernik   discarding doc.
*  0    Tuborg    0.57        --/--/98 Petr Hamernik   locking changed
*/
