/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.jarpackager;

import java.awt.Component;
import java.beans.PropertyEditor;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.jar.Manifest;
import java.util.jar.Attributes;
import java.util.*;
import java.io.*;
import java.text.MessageFormat;

import javax.swing.filechooser.FileFilter;
import javax.swing.JFileChooser;
import javax.swing.text.EditorKit;
import javax.swing.DefaultComboBoxModel;

import org.openide.util.NbBundle;
import org.openide.ErrorManager;
import org.openide.TopManager;

import org.netbeans.modules.jarpackager.util.JarUtils;
import org.openide.util.HelpCtx;
import org.openide.explorer.propertysheet.ExPropertyEditor;


/** Property editor for Manifest. Also serves as one of default tabs
 * in packaging panel.
 *
 * @author  Dafe Simonek
 * @version 
 */
public class ManifestTab extends javax.swing.JPanel implements PropertyEditor {

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jScrollPane1 = new javax.swing.JScrollPane();
        manifestPane = new javax.swing.JEditorPane();
        featuresPanel = new javax.swing.JPanel();
        loadButton = new javax.swing.JButton();
        generationPanel = new javax.swing.JPanel();
        generationComboBox = new javax.swing.JComboBox();
        generateButton = new javax.swing.JButton();
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        jScrollPane1.setPreferredSize(new java.awt.Dimension(418, 200));
        jScrollPane1.setMinimumSize(new java.awt.Dimension(418, 200));
        
        manifestPane.setPreferredSize(new java.awt.Dimension(400, 200));
        manifestPane.setMinimumSize(new java.awt.Dimension(400, 200));
        manifestPane.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                manifestPaneFocusLost(evt);
            }
        }
        );
        manifestPane.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                manifestPaneKeyPressed(evt);
            }
            public void keyReleased(java.awt.event.KeyEvent evt) {
                manifestPaneKeyReleased(evt);
            }
        }
	);
        manifestPane.addPropertyChangeListener(new java.beans.PropertyChangeListener() {
            public void propertyChange(java.beans.PropertyChangeEvent evt) {
                manifestPanePropertyChange(evt);
            }
        }
        );
        jScrollPane1.setViewportView(manifestPane);
        
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.gridwidth = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 8, 0);
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(jScrollPane1, gridBagConstraints1);
        
        
        featuresPanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints2;
        
        loadButton.setToolTipText(bundle.getString("CTL_LoadManifestTip"));
        loadButton.setText(bundle.getString("CTL_LoadManifest"));
        loadButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loadButtonActionPerformed(evt);
            }
        }
        );
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 1;
        gridBagConstraints2.gridy = 0;
        gridBagConstraints2.gridwidth = 0;
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints2.weightx = 1.0;
        featuresPanel.add(loadButton, gridBagConstraints2);
        
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.gridwidth = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 10, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.SOUTH;
        gridBagConstraints1.weightx = 2.0;
        add(featuresPanel, gridBagConstraints1);
        
        
        generationPanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints3;
        generationPanel.setBorder(new javax.swing.border.CompoundBorder(
        new javax.swing.border.TitledBorder("Content Generation"),
        new javax.swing.border.EmptyBorder(new java.awt.Insets(0, 5, 4, 5))));
        
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints3.insets = new java.awt.Insets(0, 0, 0, 8);
        gridBagConstraints3.weightx = 1.0;
        generationPanel.add(generationComboBox, gridBagConstraints3);
        
        
        generateButton.setToolTipText(bundle.getString("CTL_GenerationButtonTip"));
        generateButton.setText(bundle.getString("CTL_GenerationButton"));
        generateButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                generateButtonActionPerformed(evt);
            }
        }
        );
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        generationPanel.add(generateButton, gridBagConstraints3);
        
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.weightx = 1.0;
        add(generationPanel, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void manifestPanePropertyChange(java.beans.PropertyChangeEvent evt) {//GEN-FIRST:event_manifestPanePropertyChange
        // Add your handling code here:
        //updateValue;
    }//GEN-LAST:event_manifestPanePropertyChange

    private void manifestPaneFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_manifestPaneFocusLost
        // Add your handling code here:
        try {
            updateValue();
        }
        catch (Exception e)
        {
         }
    }//GEN-LAST:event_manifestPaneFocusLost

    private void manifestPaneKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_manifestPaneKeyPressed
        // Add your handling code here:
    }//GEN-LAST:event_manifestPaneKeyPressed

    private void manifestPaneKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_manifestPaneKeyReleased
        // Add your handling code here:
	checkValid();
    }//GEN-LAST:event_manifestPaneKeyReleased

  private void generateButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_generateButtonActionPerformed
// Add your handling code here:
        // get currently selected generator
        Generator generator = (Generator)generationComboBox.getSelectedItem();
        if (generator == null) {
            return;
        }
        // and let it modify manifest
        generator.generate(manifest);
        setValue(manifest);
	checkValid();
  }//GEN-LAST:event_generateButtonActionPerformed

  private void loadButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loadButtonActionPerformed
// Add your handling code here:
        JFileChooser chooser = new JFileChooser();
        // show mf and txt files only by default
        chooser.setFileFilter(new MfFileFilter());
        if (lastManifestFolder != null) {
            chooser.setCurrentDirectory(lastManifestFolder);
        }
        chooser.setDialogTitle(NbBundle.getBundle(ManifestTab.class).
                               getString("CTL_LoadManifestTitle"));
        if (chooser.showDialog(
                    TopManager.getDefault().getWindowManager().getMainWindow (),
                    NbBundle.getBundle(ManifestTab.class).getString("CTL_BrowseOk"))
                == JFileChooser.APPROVE_OPTION) {
            File f = chooser.getSelectedFile();
            if (f != null) {
                lastManifestFolder = chooser.getCurrentDirectory();
                // load the content of the file to the manifest text area
                try {
                    BufferedInputStream bufIs = new BufferedInputStream(new FileInputStream(f));
                    try {
                        setValue(new Manifest(bufIs));
                    } finally {
                        bufIs.close();
			checkValid();
                    }
                } catch (IOException exc) {
                    // value update is not possible, we will have to 
                    // continue with old value and notify user
                    ErrorManager errMgr = TopManager.getDefault().getErrorManager();
                    errMgr.notify(
                        ErrorManager.ERROR,
                        errMgr.annotate(
                            exc, NbBundle.getBundle(ManifestTab.class).getString("MSG_LoadManifestError")
                        )
                    );
                }
            }
        }
  }//GEN-LAST:event_loadButtonActionPerformed


  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JScrollPane jScrollPane1;
  private javax.swing.JEditorPane manifestPane;
  private javax.swing.JPanel featuresPanel;
  private javax.swing.JButton loadButton;
  private javax.swing.JPanel generationPanel;
  private javax.swing.JComboBox generationComboBox;
  private javax.swing.JButton generateButton;
  // End of variables declaration//GEN-END:variables

    /** Last folder where manifest was loaded from */
    private static File lastManifestFolder = null;

    /** Holds value of the property */
    private Manifest manifest;
    /** firing support */
    private PropertyChangeSupport support;
    /** helper flag, false if editor kit is not initialized, true otherwise */
    private boolean editorKitInitialized;
    /** Data model for generation combo box */
    private DefaultComboBoxModel model;
    /** attributes generator */
    BasicAttributesGenerator baGen;

    private static java.util.ResourceBundle bundle = NbBundle.getBundle(JarPackagerModule.class);
    
    /** property which points bace to jar content file */

    public static final String SPECIFIED_BY = new String("Specified-By"); //NOI18N
    /** Creates new form ManifestTab */
    public ManifestTab() {
        initComponents();
        additionalInitialization();
        HelpCtx.setHelpIDString (this, ManifestTab.class.getName ());
    }

    public void setJarContentFile(String jarContentFile) {
	if (baGen != null) {
	    baGen.setJarContentFile(jarContentFile);
	}
    }
    	
    private void additionalInitialization () {
        // set border for generation
        generationPanel.setBorder(new javax.swing.border.CompoundBorder(
            new javax.swing.border.TitledBorder(
                NbBundle.getBundle(ManifestTab.class).getString("CTL_GenerationBorder")
            ), 
            new javax.swing.border.EmptyBorder(new java.awt.Insets(0, 5, 4, 5)))
        );
        // install some manifest attribute generators
        model = new DefaultComboBoxModel();
	baGen = new BasicAttributesGenerator();
        model.addElement(baGen);
        generationComboBox.setModel(model);
        initContentType ();
    }

    /** This method will be called every time when we will need
     * to update contained tabs according to new content value. Implementors
     * should read the data from content and update its visual component.
     *
     * @param content New content.
     */
    public void readSettings(ArchiveDescriptor content) throws IOException {
        doSetValue(content.getManifest());
        updateGui(manifest);
    }

    /** Allows implementors to store their current setting to the given
     * content. This method will be called for example when user starts jar
     * creation or when user switches the tab.
     *
     * @param content Content to store current state into.
     */
    public void storeSettings(ArchiveDescriptor content) throws IOException {
        updateValue();
        content.setManifest(manifest);
        support().firePropertyChange("",null,null);
    }

    /** Return the component which will serve as visual presentation of 
     * packaging tab. Returned component will be contained in PackaginPanel in
     * some way.
     *
     * @return Visual presentation of the tab.
     */
    public Component getComponent() {
        return this;
    }

    /** Notifies controller that archive is being processed
     * (either pre-processed if registered via JarCreator.addPreController or
     * post-processed if registered via JarCreator.addPostController).
     * @param driver 'context' object that provides both informations about
     * archive and methods to manipulate and customize archive being created
     * @see ArchiveDriver
    public void processArchive(ArchiveDriver driver) {
    }
     */

    /** Unique programmatic name of this packaging tab.
    * @return Programmatic name of this packaging tab
    */
    public String getName () {
        return "manifestTab"; // NO I18N
    }

    /** Display name of this packaging tab, should be localized properly.
     * @return Name of this packaging tab
     */
    public String getDisplayName () {
        return NbBundle.getBundle(ManifestTab.class).getString("CTL_ManifestTab");
    }

    /** Short description of this packaging tab - should be localized.
     * Can be used in tooltips etc. by the container.
     * @return Short string description of this packaging tab or null.
     */
    public String getShortDescription() {
        return NbBundle.getBundle(ManifestTab.class).getString("HINT_ManifestTab");
    }

    public String getJavaInitializationString () {
        return null; // no code generation
    }

    public boolean supportsCustomEditor () {
        return true;
    }

    public java.awt.Component getCustomEditor() {
        return this;
    }

    public boolean isPaintable() {
        return false;
    }

    public void paintValue(java.awt.Graphics p1, java.awt.Rectangle p2) {
    }

    public String[] getTags() {
        // we don't support tags
        return null;
    }

    public void addPropertyChangeListener (PropertyChangeListener pchl) {
        support().addPropertyChangeListener(pchl);
    }

    public void removePropertyChangeListener(PropertyChangeListener pchl) {
        support().addPropertyChangeListener(pchl);
    }

    public void setValue (Object value) {
        try {
            doSetValue(value);
            updateGui(value);
        } catch (IOException exc) {
            // value update is not possible, we will have to 
            // continue with old value and notify user
            ErrorManager errMgr = TopManager.getDefault().getErrorManager();
            errMgr.notify(
                ErrorManager.WARNING,
                errMgr.annotate(
                    exc, NbBundle.getBundle(ManifestTab.class).getString("MSG_CanNotReadManifest")
                )
            );
        }
    }

    public Object getValue() {
        return manifest;
    }

    public void setAsText (String text) throws IllegalArgumentException {
        // not supported
        // PENDING
        throw new IllegalArgumentException();
    }

    public String getAsText() {
	Integer mainAttrs = new Integer(manifest.getMainAttributes().size());
	Integer entries = new Integer(manifest.getEntries().size());
        return MessageFormat.format(
		bundle.getString("FMT_ManifestSize"),
		new Object[] {mainAttrs, entries} );

    }

    /** Safe getter for property change support with lazy initialization */
    private PropertyChangeSupport support () {
        if (support == null) {
            support = new PropertyChangeSupport(this);
        }
        return support;
    }

    /** Utility method, update property editor's value to be synchronized
    * with current content of visual controls */
    void updateValue () throws IOException {
         doSetValue(makeManifest());
     }

    /** previous state of validity */
    private boolean isValid = true;

    /** return current validity status */
    public boolean isValid() {
    	return isValid;
    }

    /** Utility method, check if current value is a valid manifest */
    private boolean checkValid() {
	boolean nowValid = true;
    	try {
	    makeManifest();
	}
	catch (IOException ex) {
	    nowValid = false;
	}
	if (isValid != nowValid) {
	    isValid = nowValid;
	    support().firePropertyChange(ExPropertyEditor.PROP_VALUE_VALID,
					 !isValid, isValid);
	}
	return isValid;
    }

    /** Utility method to make new manifest */
    private Manifest makeManifest() throws IOException {
	byte panelBytes[] = manifestPane.getText().getBytes();
	InputStream str = 
	    new TerminateLineInputStream(new ByteArrayInputStream(panelBytes));
    	return new Manifest(str);
    }

    /** Utility method, update property editor's value to be synchronized
    * with current content of visual controls */
    void doSetValue (Object value) {
        if (!(value instanceof Manifest)) throw new IllegalArgumentException(value.toString());
        manifest = (Manifest)value;
        support().firePropertyChange("", null, null);

    }
    

    /** Utility method, update gui of this custom property editor according
    * to given property value. */
    void updateGui (Object value) throws IOException {
        Manifest manifest = (Manifest)value;
        manifestPane.setText(""); // NOI18N
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        try {
            manifest.write(baos);
        } finally {
            baos.close();
        }
        manifestPane.setText(baos.toString().trim());
	manifestPane.requestFocus();
    }

    /** Resds the content of specified etxtual file and returns
    * its content as a string */
    String readManifestFile (File file) throws IOException {
        BufferedReader br = new BufferedReader(new FileReader(file));
        StringBuffer sb = new StringBuffer();
        String curLine;
        try {
            while ((curLine = br.readLine()) != null) {
                sb.append(curLine).append("\n"); // NOI18N
            }
        } finally {
            br.close();
        }
        return sb.toString();
    }

    /** force right editor kit to be used for manifest editing */
    void initContentType () {
        EditorKit kit = manifestPane.getEditorKitForContentType("text/plain"); // NOI18N
        if (kit != null) {
            String content = manifestPane.getText();
            manifestPane.setContentType("text/plain"); // NOI18N
            manifestPane.setEditorKit(kit);
            // re-set the content again
            manifestPane.setText(content);
        }
    }
    
    /** Completes main attributes of the manifest.
    * Adds version and created-by attributes if they are not present.
    */
    static void completeMainAttributes(
    	Attributes mainAttr, String jarContentFile) {
        // add version if missing
        if (mainAttr.getValue(Attributes.Name.MANIFEST_VERSION) == null)
            mainAttr.put(Attributes.Name.MANIFEST_VERSION, JarUtils.MANIFEST_VERSION);
        // add created-by if missing
        Attributes.Name cbAttr = new Attributes.Name("Created-By"); // NOI18N
        if (mainAttr.getValue(cbAttr) == null) {
            // PENDING - should be changed with something like getProductName()
            mainAttr.put(cbAttr, NbBundle.getBundle(ManifestTab.class).getString("CTL_IDEName")); // NOI18N
	}
        // add specified-by if missing and if we have a jar content file name
	if (jarContentFile != null) {
	    Attributes.Name sbAttr = new Attributes.Name(SPECIFIED_BY);
	    if (mainAttr.getValue(sbAttr) == null) {
		mainAttr.put(sbAttr, jarContentFile);
	    }
        }
    }

    /** Implementation of TxtFileFilter interface which accepts mf
    * and txt files. */
    static final class MfFileFilter extends FileFilter {
        /** Accepts only directories and archives */
        public boolean accept (File f) {
            return f.isDirectory() || f.getName().endsWith(".mf") || // NOI18N
                   f.getName().endsWith(".txt") || // NOI18N
                   f.getName().endsWith(".MF"); // NOI18N
        }
        /** Returns a description of this filter */
        public String getDescription () {
            return NbBundle.getBundle(ManifestTab.class).
                   getString("CTL_MfFiles");
        }
    } // end of TxtFileFilter inner class


    /** Interface for automatic generation of manifest attributes */
    interface Generator {
        
        /** Implementors should generate attributes and modify given
         * manifest instance in this method */  
        public void generate (Manifest manifest);
        
        /** @return localized name of the generator */
        public String getDisplayName ();
        
        /** @return localized short description of the generator or null if
         * no description is needed. To be used in tooltips etc. */
        public String getShortDescription ();
        
    } // end of inner interface Generator

    /** Automatic generation of basic main manifest attributes */ 
    final static class BasicAttributesGenerator implements Generator {

       private String jarContentFile;

       BasicAttributesGenerator() {
       }

       public void setJarContentFile(String jarContentFile) {
	    this.jarContentFile = jarContentFile;
       }

       /** Implementors should generate attributes and modify given
         * manifest instance in this method */  
        public void generate (Manifest manifest) {
            completeMainAttributes(
	    	manifest.getMainAttributes(), jarContentFile);
        }
        
        /** @return localized name of the generator */
        public String getDisplayName () {
            return NbBundle.getBundle(ManifestTab.class).getString("CTL_BasicAttributes");
        }

        /** @return localized short description of the generator or null if
         * no description is needed. To be used in tooltips etc. */
        public String getShortDescription () {
            return NbBundle.getBundle(ManifestTab.class).getString("CTL_BasicAttributesTip");
        }
        
        /** For use in DefaultComboBoxModel */
        public String toString () {
            return getDisplayName();
        }
        
    } // end of inner class BasicAttributesGenerator

    /** filter which terminates unterminated lines */
    static class TerminateLineInputStream extends FilterInputStream {
    	
	byte last = 0;
	boolean terminated = true;

	/** create filter */
	TerminateLineInputStream(InputStream in) {
	    super(in);
	}

	/** read one character */
	public int read() throws IOException {
	    int val = in.read();
	    if (val < 0) {
	    	if (last == '\n' || terminated) {
		    return val;
		}
		else {
		    terminated = true;
		    return '\n';
		}
	    }
	    else {
		terminated = false;
	    	last = (byte)val;
		return val;
	    }
	}

	/** read an array of characters */
	public int read(byte b[], int off, int len) throws IOException {
	    int size = in.read(b, off, len);
	    if (size < 0) {
	    	if (last == (byte) '\n' || terminated) {
		    return size;
		}
		else {
		    b[off] = (byte) '\n';
		    terminated = true;
		    return 1;
		}
	    }
	    else if (size > 0) {
		terminated = false;
	    	last = b[off + size - 1];
	    }
	    return size;
	}
    }
}
