/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.jarpackager.api;

import java.io.*;
import java.util.*;
import org.openide.nodes.Node;

/** Interface for controlling packaging process. Anyone interested in
* controlling packaging process in any way should implement this interface
* and register the implementation via JarCreator.addBuilder method.  The
* ArchiveBuilder will be called back during packaging, to create an
* ArchiveController, which can affect the packaging of a specific jar file.
*
* @author Mike Schilling
*/
public interface ArchiveBuilder extends Node.Cookie {



    /** Phase 0.  Called at the start of packaging a jar.  This lets
     * each builder see which jar is being constructed and the values of its 
     * extended properties, and determine if it needs to be involved in the 
     * packaging. 
     * @param target the jar file being produced
     * @param options the extended properties for this jar.  Should be treated
     * as readonly by the called method.
     * @return ArchiveController object for further callbacks.  Null if the
     * builder does not want to affect the building of this jar
     */
    ArchiveController initController(java.io.File target, Map options);

}
