/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.jarpackager.api;

import java.io.*;
import java.util.*;
import org.openide.nodes.Node;

/** A convenience superclass for ArchiveController implementations.
* All methods do nothing.  To make a useful ArchiveController, create a subclass
* and override the necessary methods.
*
* @author Mike Schilling
*/
public abstract class ArchiveControllerSupport implements ArchiveController {

    /** Phase 1. Called to let each archive controller modify the set
     * of data objects which are inputs to the jar.
     * @param context context returned from initController
     * @param ArchiveMemberSet current list of inputs, which this controller
     * can modify
     */
    public void processMemberSet(ArchiveMemberSet set) {
    }

    /** Phase 2.  Called to allow each archive controller to add additional
     * ArchiveEntry's as inputs to the jar file.
     * @param context context returned from initController
     * @return array of entries to be added
     */
    public ArchiveEntry[] addEntry() {
    	return null;
    }

    /** Phase 3.  Called to allow each archive controller to express interest
     * in each ArchiveEntry.  "Interest", in this case, means a desire to 
     * process the contents of the ArchiveEntry before it is written
     * to the jar file.  The controller indicates its interest by returning 
     * one of the POSTPROCESS constants.  Note that if more than one controller
     * needs to be the first postprocessor for any entry (or more than one the
     * last), the packaging fails.
     * @param entry the ArchiveEntry which the controller may want to process
     * @return the controller's interest in this entry.
     */
    public int willPostprocess(ArchiveEntry entry) {
    	return WILL_NOT_POSTPROCESS;
    }

    /** Phase 4.  Called to allow ArchiveController's to postprocess 
     * ArchiveEntry's.  The ArchiveController replaces the InputStream
     * passed in with one of its own, usually by filtering it.  Note that
     * the ArchiveController should being with the InputStream argument, which
     * may be the result of a previous postprocessor, not with the result of
     * ArchiveEntry.createInputStream().
     */
    public InputStream postProcess(ArchiveEntry entry, InputStream input) {
    	return input;
    }
}
