/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.jarpackager.api;

import java.io.InputStream;
import java.io.IOException;

/* Each extra entry holds entry name which is the name of the entry
* in the resulting archive, the input stream is an instance of
* InputStream class, which will be used to read extra info data.
* size is the size of extra data in entry.
*/
public interface ArchiveEntry {

    /** Name of this entry, will be used as entry name in the archive.
    * Entry name should be unique in context of an archive.
    * (So implementators should consider implementing equals method
    * as a name comparison)
    * @return name of the entry 
    */
    public String getName ();

    /** Data of the entry. Returned stream will be used for reading data
     * of this entry. Caller is responsible for proper clean-up.
     * @throws IOException if input stream cannot be created for some reason
     * @return Newly created input stream
     */
    public InputStream createInputStream () throws IOException;
}
