/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.jarpackager.api;

import org.openide.loaders.DataObject;
import org.openide.util.NbBundle;

/** This interface represents one DataObject whose members will be written
 * to a jar file.  
 * <p>
 * The targetDirectory property controls which directory in the jar file the
 * files from a DataObject are written to.  The rules are:
 * </ul>
 * <li> First, for a DataObject which is not a folder.  Assume an ArchiveMember
 * with DataObject org/mycompany/Top which contains an Entry Top.class.
 * <ul>
 * <li> If the targetDirectory is null, the files are written to the 
 * same directory as the DataObject.  The jar file will contain 
 * org/mycompany/Top.class
 * <li> If the targetDirectory is ArchiveMember.ROOT, the files are always 
 * written to the jar file's root directory.  The jar file will contain
 * Top.class.
 * <li> Otherwise, the value of the targetDirectory property is the directory
 * name.  For instance, after setTargetDirectory("classes/archive")
 * the jar file will contain classes/archive/Top.class.
 * </ul>
 * <li> For a Data Object which is a folder, the folder itself is not written
 * the to jar file; its children are.  This time assume the DataObject is the
 * org folder.
 * <ul>
 * <li> If the targetDirectory is null, the files are written to the 
 * same directory as the DataObject.  That is, the jar file will contain
 * org/mycompany/Top.class
 * <li> If the targetDirectory is ArchiveMember.ROOT, the DataObject is
 * mapped to the root.  The jar file will contain
 * mycompany/Top.class.
 * from org.netbeans.openide become *.* in the jar file.
 * <li> Otherwise, the folder is mapped to  the value of the targetDirectory 
 * property. For instance, after setTargetDirectory("classes/archive"), 
 * the jar file will contain classes/archive/company/Top.class.
 * </ul>
 * </ul>
 * 
 */
public interface ArchiveMember {

    /** Set targetDirectory to this to force a data object to be put in
    * the root directory */
    public static String ROOT = "<root>";

    /** get the DataObject
    * @return the member's DataObject 
    */
    DataObject getDataObject();

    /** get the member-specific filter, if any
    * @return the filter.  Returns null if there is none
    */
    ArchiveFilter getFilter();

    /** set a new filter
    * @param the new filter.  null to turn off member-specific filtering
    */
    void setFilter(ArchiveFilter filter);

    /** get the target directory within the jar
    * @return target directory
    */
    String getTargetDirectory();

    /** set a new target directory
    * @param directory new target directory.  null for the target directory
    * to be the same as the DataObject's directory.
    */
    void setTargetDirectory(String directory);
}

