/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.jarpackager.api;

import org.openide.loaders.DataObject;
import org.openide.loaders.DataFolder;

/** This interface represents the set of archive members to be written to
 * a jar file.  It allows that set to be modified.  Note that this isn't
 * a java.util.Set.
 */
public interface ArchiveMemberSet {
    
    /** return all members of the set as an array. This array does not track 
     * additions to and removals from the set, though the individual objects 
     * within the array do track modifications to their own state.
     * @return an array of all current members of the set.
     */
    ArchiveMember[] allMembers();

    /** add a new member to the set.  If the specified DataObject is a folder,
     * all of the folder's non-folder children are added (though not its
     * children's children).
     * @param dataObj the DataObject for the new member.  If there was already
     * a member with this DataObject, it will not be added.
     * @param filter a member-specific filter.  This may be null.
     * @param traget the target directory in the jar.  If this is null, the
     * target is determined by the DataObject.
     * @return the number of members added
     */
    int addMember(
    	DataObject dataObj, ArchiveFilter filter, String targetDirectory);

    /** remove a member from the set.  If the DataObject is a folder, it 
     * removes all of that folder's children from the set
     * @param dataObj the DataObject for the member to be removed
     * @return the number of members removed
     */
    int removeMember(DataObject dataobj);

    /** get a member from the set.  There will be at most one.
     * Note that folders are never members
     * of the set.
     * @param dataObj DataObject of member to get.
     * @return the member for that DataObject or null.
     */
    ArchiveMember getMember(DataObject dataObj);

    /** get the members which are children of a specified folder.
     * @param dataFolder the folder containing the DataObjects for the
     * returned members.
     * @return an array of members (never null)
     */
    ArchiveMember[] getMembers(DataFolder dataFolder);
}
