/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.jarpackager.api;

import java.util.*;

import org.openide.nodes.Node;
import org.openide.loaders.DataObject;
import org.netbeans.modules.jarpackager.*;

/** This is a factory interface.  The factory classes which implement it
 *  are used to create the Node.Property objects
 *  which allow extended properties of jar files to be processed.
 *  @see org.netbeans.modules.jarpackager.JarCreator#addExtendedProperty
 */
public interface ExtendedPropertyFactory {

    /** @return a Node.Property which allows processing of a jar
     * @param dataObj the DataObject whose properties are being edited.  
     * This may be null if no property sheet is being displayed, for instance 
     * during serialization and deserialization.
     * @param jarContent the jar content object containing the extended
     * property
     */
    public Node.Property createProperty(DataObject dataObj, JarContent jarContent);

    /** @return the name of the extended property this factory 
     * creates.  Must be the same as the return from Node.Property.getName()
     */
    public String getName();
}

