/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.jarpackager.api;

import java.io.InputStream;
import java.io.IOException;

import org.openide.filesystems.FileObject;

/** Default implementation of ArchiveEntry, which presents given
* file object as an archive entry */
public class FOArchiveEntry extends Object implements ArchiveEntry {

    /** Asociation with file object, to which nearly all
    * funtionality is delegated */
    private FileObject fo;
    /** name of this entry */
    private String name;

    /* Default contructor, constructs an entry with given file object as
    * its iunderlying representation. Name of the entry will be set to
    * package name of the file object. */
    public FOArchiveEntry (FileObject fo) {
        this(fo, fo.getPackageNameExt('/', '.'));
    }

    /* Constructs an entry with given file object as
    * its iunderlying representation and with given name */
    public FOArchiveEntry (FileObject fo, String name) {
        this.fo = fo;
        this.name = name;
    }

    /** @return Name of the entry */
    public String getName () {
        return name;
    }

    /** @return input stream of asociated file object */
    public InputStream createInputStream () throws IOException {
        return fo.getInputStream();
    }

    /** @return Size of file represented by file object */
    public long getSize () {
        return fo.getSize();
    }

    /** Getter for asociated file object
    * @return asociated file object. */
    public FileObject getFileObject () {
        return fo;
    }

    public int hashCode() {
        // FileObject doesn't have hashCode so use file name
        String thisFO = this.getFileObject().getPackageNameExt('/', '.');
        return thisFO.hashCode();
    }

    public boolean equals(Object other) {
        if (other instanceof FOArchiveEntry) {
            FOArchiveEntry otherEntry = (FOArchiveEntry) other;
            String thisFO = this.getFileObject().getPackageNameExt('/', '.');
            String otherFO = otherEntry.getFileObject().getPackageNameExt('/', '.');
            if (thisFO.equals(otherFO)) {
                return this.getName().equals(otherEntry.getName());
            }
        }
        return false;
    }


}
