/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.jarpackager.wizard;

import java.awt.Component;

import org.openide.loaders.TemplateWizard;
import org.openide.util.NbBundle;
import org.openide.WizardDescriptor;


import org.netbeans.modules.jarpackager.DataObjectListEditor;
import org.netbeans.modules.jarpackager.ContentMemberList;
import org.netbeans.modules.jarpackager.JarContent;

/**
* This embeds the DataObjectListEditor in a wizard panel.
*/

class ContentWizardPanel 
	extends PropertyEditorPanel 
	implements WizardDescriptor.FinishPanel, 
	    JarContentsWizardIterator.JarContentEditor	{

    private DataObjectListEditor editor;

    /**
    * @param order order of this panel in the wizard
    */
    public ContentWizardPanel(int order) {
	super(new DataObjectListEditor());
	editor = (DataObjectListEditor)super.editor;
    	editor.setName(NbBundle.getBundle(ContentWizardPanel.class).
						getString("TIT_ContentPanel"));
	editor.putClientProperty("WizardPanel_contentSelectedIndex", new Integer(order));
    }

    /**
    * no further validation required
    */
    public boolean isValid() {
	return true;
    }

    /**
    * called to initialize (or reinitialize) panel
    * @param settings the TemplateWizard object for the parent wizard
    */
    public void readSettings (Object settings) {
	JarContent fromWizard = 
	    JarContentsWizardIterator.getJarContent(
					    (TemplateWizard)settings);
	//System.out.println("readSettings " + fromWizard);
	editor.setValue(fromWizard);
    }

    /**
    * called when exiting the panel
    */
    public void storeSettings(Object settings) {
	JarContent fromWizard = 
	    JarContentsWizardIterator.getJarContent(
					    (TemplateWizard)settings);
	//System.out.println("storeSettings " + fromWizard);
	JarContent current = (JarContent)editor.getValue();
	fromWizard.setContentList(
		new ContentMemberList(current.getContentList()));
    }

    /** returns the awt component for the panel
    */
    public Component getComponent() {
	return editor;
    }

    /** apply changes to template-generated object */
    public void applyChanges(JarContent fromTemplate, JarContent fromWizard) {
    	fromTemplate.setContentList(fromWizard.getContentList());
    }
}

