/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.jarpackager.wizard;

import java.io.*;
import java.util.jar.Manifest;
import java.util.jar.Attributes;

import java.awt.Component;

import org.openide.loaders.TemplateWizard;
import org.openide.util.NbBundle;
import org.openide.WizardDescriptor;

import org.netbeans.modules.jarpackager.JarContent;
import org.netbeans.modules.jarpackager.ManifestTab;

/** embed the manifest editor in a wizard panel */
class ManifestWizardPanel 
	extends PropertyEditorPanel
	implements WizardDescriptor.FinishPanel,
		JarContentsWizardIterator.JarContentEditor	{

    ManifestTab editor;

    /** Create panel
     * @param order order of this panel in wizard 
     */
    public ManifestWizardPanel(int order) {
	super(new ManifestTab());
	editor = (ManifestTab)super.editor;
	editor.putClientProperty(
	    "WizardPanel_contentSelectedIndex", new Integer(order));
	editor.setName(NbBundle.getBundle(ManifestWizardPanel.class)
					.getString("TIT_ManifestPanel"));

    }

   /** editor knows if it's valid */
   public boolean isValid() {
	return editor.isValid();
    }

   /** called when initializing or reinitializing panel
    * @param settings the parent TemplateWizard
    */
   public void readSettings (Object settings) {
	TemplateWizard wizard = (TemplateWizard)settings;
	JarContent fromWizard  =
	    JarContentsWizardIterator.getJarContent(wizard);
	String content = 
	    JarContentsWizardIterator.getJarContentFile(wizard);
	//System.out.println("content is " + content);
	if (content != null) {
	    editor.setJarContentFile(content);
	}
	Manifest manifest = fromWizard.getManifest();
	// update "Specified-By", in case target has changed since it
	// was generated
	if (manifest != null) {
	    Attributes mainAttr = manifest.getMainAttributes();
	    Attributes.Name sbAttr = 
	    	new Attributes.Name(ManifestTab.SPECIFIED_BY);
	    if (mainAttr.getValue(sbAttr) != null) {
	    	mainAttr.put(sbAttr, content);
	    }
	}
	editor.setValue(manifest);
    }

    /** called when exiting panel
     * @param settings the parent TemplateWizard
     */
    public void storeSettings(Object settings) {
	JarContent fromWizard = 
	    JarContentsWizardIterator.getJarContent(
					    (TemplateWizard)settings);
	Manifest manifest = (Manifest)editor.getValue();
	fromWizard.setManifest(manifest);
    }

    /** return associated awt component */
    public Component getComponent() {
	return editor;
    }

    /** apply changes to template-generated object */
    public void applyChanges(JarContent fromTemplate, JarContent fromWizard) {
    	fromTemplate.setManifest(fromWizard.getManifest());
    }
}

