/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.jarpackager.wizard;

import java.awt.Component;
import java.beans.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;

import org.openide.util.HelpCtx;
import org.openide.WizardDescriptor;

import org.netbeans.modules.jarpackager.JarContent;

public abstract class PropertyEditorPanel 
	implements WizardDescriptor.Panel, PropertyChangeListener {
    private ArrayList listeners_ = new ArrayList();

    protected PropertyEditor editor;

    /** create a wizard panel from the supplied property editor */
    protected PropertyEditorPanel(PropertyEditor editor) {
    	this.editor = editor;
	editor.addPropertyChangeListener(this);
    }

    /** Get the component displayed in this panel.
    * @return the component
    */
    public abstract Component getComponent ();

    /** Help for this panel.
    * When the panel is active, this is used as the help for the wizard dialog.
    * @return the help or <code>null</code> if no help is supplied
    */
    public HelpCtx getHelp () {
    	return HelpCtx.findHelp(getComponent());
    }

    /** Provides the wizard panel with the current data--either
    * the default data or already-modified settings, if the user used the previous and/or next buttons.
    * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
    * @param settings the object representing wizard panel state, as originally supplied to {@link WizardDescriptor#WizardDescriptor(WizardDescriptor.Iterator,Object)}
    * @exception IllegalStateException if the the data provided 
    * by the wizard are not valid.
    */
    public abstract void readSettings (Object settings);

    /** Provides the wizard panel with the opportunity to update the
    * settings with its current customized state.
    * Rather than updating its settings with every change in the GUI, it should collect them,
    * and then only save them when requested to by this method.
    * Also, the original settings passed to {@link #readSettings} should not be modified (mutated);
    * rather, the (copy) passed in here should be mutated according to the collected changes.
    * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
    * @param settings the object representing a settings of the wizard
    */
    public abstract void storeSettings (Object settings);

    /** Test whether the panel is finished and it is safe to proceed to the next one.
    * If the panel is valid, the "Next" (or "Finish") button will be enabled.
    * @return <code>true</code> if the user has entered satisfactory information
    */
    public abstract boolean isValid ();

    /** Add a listener to changes of the panel's validity.
    * @param l the listener to add
    * @see #isValid
    */
    public void addChangeListener (ChangeListener l) {
    	this.listeners_.add (l);
    }

    /** Remove a listener to changes of the panel's validity.
    * @param l the listener to remove
    */
    public void removeChangeListener (ChangeListener l) {
    	this.listeners_.remove (l);
    }

    /** Translate property-editor-like change notification to 
     * wizard-like notification.
     */
    public void propertyChange(PropertyChangeEvent evt) {
	ChangeEvent event = new ChangeEvent(evt.getSource());
        Iterator iterator;
        synchronized (this) {
            iterator = ((ArrayList)listeners_.clone()).iterator();
        }
        while (iterator.hasNext ()) {
            ((ChangeListener)iterator.next()).stateChanged(event);
        }
    }

}

