package org.netbeans.modules.jarpackager;

import java.util.*;
import java.io.*;
import org.openide.loaders.*;
import org.openide.filesystems.*;
import org.netbeans.modules.jarpackager.api.*;
/**
* Simple example of archive builder and controller
* Uppercases all files whose names contain the string 'file3'
*/

public class Controller2 implements ArchiveBuilder {
	
    private PrintStream out;

    public Controller2(PrintStream out) {
    	this.out = out;
    }
    /** we have no jar-specific context , and so can just return this */
    public ArchiveController initController(java.io.File target, Map options) {
    	return new TheController(out);
    }

    static class TheController implements ArchiveController {
	private PrintStream out;

	TheController(PrintStream out) {
	    this.out = out;
	}

	/** Nothing to do */
	public void processMemberSet(ArchiveMemberSet set) {
	}

	/** Add a member that indicates we were here */
	public ArchiveEntry[] addEntry() {
	    ArchiveEntry sae = 
		new StringArchiveEntry( "/fake/controller2", "Kilroy 2 was here\n");
	    return new ArchiveEntry[]{sae};
	}

	/** No postprocessing */
	public int willPostprocess(ArchiveEntry entry) {
	    if (entry.getName().indexOf("file3") >= 0) {
		//out.println("Will postprocess " + entry.getName());
		return ArchiveController.WILL_POSTPROCESS;
	    }
	    //out.println("Will not postprocess " + entry.getName());
	    return ArchiveController.WILL_NOT_POSTPROCESS;
	}

	/** No postprocessing */
	public InputStream postProcess(
	    ArchiveEntry entry, InputStream input) {
		//out.println("Creating stream for " + entry.getName());
		return new UppercaseStream(input);
	}

	class UppercaseStream extends FilterInputStream {
	    
	    public UppercaseStream(InputStream in) {
		super(in);
	    }

	    public int read() throws IOException {
		int val = in.read();
		if (val < 0)
		    return val;
		    
		return (int)Character.toUpperCase((char)val);
	    }

	    public int read(byte b[]) throws IOException {
		return read(b, 0, b.length);
	    }

	    public int read(byte b[], int off, int len) throws IOException {
		byte buffer[] = new byte[len];
		int retval = in.read(buffer, 0, len);
		for (int i = 0; i < retval; i++) {
		    b[off + i] = (byte)Character.toUpperCase((char)buffer[i]);
		}
		return retval;
	    }

       }
    }
}

