/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java;

import java.util.*;

import org.openide.compiler.Compiler;
import org.openide.compiler.CompilerGroup;
import org.openide.compiler.ErrorEvent;
import org.openide.filesystems.FileObject;
import org.openide.ErrorManager;
import org.openide.util.Lookup;

public final class BogusCompiler extends Compiler {
    Throwable   error;
    FileObject  fo;
    
    BogusCompiler(FileObject fo, Throwable err) {
        this.error = err;
        this.fo = fo;
    }
    
    public Class compilerGroupClass() {
        return Group.class;
    }
    
    /**
     * BogusCompilers always return false, because they need to be processed 
     * (and error messages need to be printed)
     */
    protected boolean isUpToDate() {
        return false;
    }
    
    public final static class Group extends CompilerGroup {
        private static final ErrorManager errMan = (ErrorManager)
            Lookup.getDefault().lookup(ErrorManager.class);
        
        Collection  compilers = new LinkedList();
        
        public void add(Compiler c) throws IllegalArgumentException {
            if (!(c instanceof BogusCompiler))
                throw new IllegalArgumentException("Incorrect compiler type: " +
                    c);
            compilers.add(c);
        }
        
        public boolean start() {
            for (Iterator it = compilers.iterator(); it.hasNext(); ) {
                BogusCompiler comp = (BogusCompiler)it.next();
                
                Throwable t = comp.error;
                FileObject f = comp.fo;
                ErrorManager.Annotation[] anns = errMan.findAnnotations(t);
                boolean shown = false;
                
                for (int i = 0; i < anns.length; i++) {
                    String lm = anns[i].getLocalizedMessage();
                    if (lm != null) {
                        fireErrorEvent(new ErrorEvent(
                            this, f, 0, 0, lm, null));
                        shown = true;
                    }
                }
                if (!shown)
                    fireErrorEvent(new ErrorEvent(this, f, 0, 0,
                        t.toString(), null));
            }
            return false;
        }
    }
}

