/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java;

import java.io.File;
import java.io.ObjectInputStream;
import java.io.IOException;

import org.openide.loaders.DataObject;
import org.openide.compiler.Compiler;
import org.openide.compiler.CompilerType;
import org.openide.compiler.CompilerJob;
import org.openide.compiler.ExternalCompiler;
import org.openide.compiler.ExternalCompiler.ErrorExpression;
import org.openide.compiler.ExternalCompilerGroup;
import org.openide.execution.NbProcessDescriptor;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;
import org.openide.filesystems.FileObject;

import org.netbeans.modules.java.settings.JavaSettings;

/**
*
* @author Ales Novak
*/
public class FastJavacCompilerType extends JavaExternalCompilerType {
    /** generated Serialized Version UID */
    private static final long serialVersionUID = -3875747959787225041L;
    
    /** Fully qualified name of the binary according to the host OS */
    private static String wiredFullName;
    
    /** base name of the binary, dependent on the host OS */
    private static String wiredName;

    public FastJavacCompilerType() {
        super();
    }

    /** overriden */
    NbProcessDescriptor createDescriptor() {
        return new NbProcessDescriptor(
            "{" + JExternalCompilerGroup.JFormat.TAG_IDEHOME + "}{" + ExternalCompilerGroup.Format.TAG_SEPARATOR + "}bin{" + // NOI18N
            ExternalCompilerGroup.Format.TAG_SEPARATOR + "}fastjavac{" + ExternalCompilerGroup.Format.TAG_SEPARATOR + "}{" + // NOI18N
		JExternalCompilerGroup.JFormat.TAG_FASTJAVAC_BINARY + "}",  // NOI18N
            //"-msgfile {" + JExternalCompilerGroup.JFormat.TAG_MSGFILE + "} " + // NOI18N
            "{" + JExternalCompilerGroup.JFormat.TAG_DEBUGINFO + "} " + // NOI18N
            "{" + JExternalCompilerGroup.JFormat.TAG_DEPRECATION + "} " + // NOI18N
            "{" + JExternalCompilerGroup.JFormat.TAG_OPTIMIZE + "} " + // NOI18N
            "{" + JExternalCompilerGroup.JFormat.TAG_OUTDIR + "} " + // NOI18N
            "{" + JExternalCompilerGroup.JFormat.TAG_ENCODING + "} " + // NOI18N
            "{" + JExternalCompilerGroup.JFormat.TAG_OPTION_BOOTCLASSPATH + "}" + // NOI18N
            "{" + JExternalCompilerGroup.JFormat.TAG_BOOTCLASSPATH + "} " + // NOI18N
            "-jdk {" + JExternalCompilerGroup.JFormat.TAG_QUOTE + "}{" + // NOI18N
                ExternalCompilerGroup.Format.TAG_JDKHOME + "}{" + // NOI18N
                JExternalCompilerGroup.JFormat.TAG_QUOTE + "} " + // NOI18N
            "-classpath {" + ExternalCompilerGroup.Format.TAG_REPOSITORY + "}{" + ExternalCompilerGroup.Format.TAG_PATHSEPARATOR + "}" + // NOI18N
            "{" + ExternalCompilerGroup.Format.TAG_CLASSPATH + "} " + // NOI18N
            "{" + ExternalCompilerGroup.Format.TAG_FILES + "}", // NOI18N
            getAdditionalHint() +  Util.getString("MSG_FastCompilerHint")
        );
    }
    
    /** human presentable name */
    public String displayName() {
        // Default only matters for regular external, which is
        // instantiated using Class.newInstance:
        return getString("CTL_FastCompilerType");
        // In the case of the FastJavac, the .ser is given the token
        // name; when initially loaded from manifest, readObject
        // gives it a localized name. Thereafter the localized name
        // is serialized in the project.
    }

    public HelpCtx getHelpCtx () {
        return new HelpCtx(FastJavacCompilerType.class);
    }

    
    /*public static boolean isFastJavacPlatform() {
        return ((Utilities.getOperatingSystem() &
                 (Utilities.OS_WINDOWS_MASK | Utilities.OS_SOLARIS)) != 0) ||
               isX86Linux();
    }*/

    private static boolean isX86() {
	String arch = System.getProperty("os.arch"); // NOI18N
	return (arch.indexOf("i386") >= 0 || // NOI18N
	    arch.indexOf("x86") >= 0); // NOI18N
    }
    
    private static boolean isX86Linux() {
        return isX86() && ((Utilities.getOperatingSystem() & Utilities.OS_LINUX) != 0);
    }
    public static boolean isFastJavacPlatform() {
        return getWiredFullName() != null;
    }
    
    private static boolean isWin() {
        return ((Utilities.getOperatingSystem() & Utilities.OS_WINDOWS_MASK) != 0);
    }
    
    private static String findFastJavac(String rootName) {
	String name = getWiredName();

	if (name == null)
	    // unknown architecture ?!?
	    return null;
	    
        File root = new java.io.File(rootName);
        File dir = new File(new java.io.File(root, "bin"), "fastjavac");
        File f1 = new File(dir, getWiredName());
        if (f1.exists()) {
            try {
                return f1.getCanonicalPath();
            } catch (IOException ex) {
                // next try
            }
        } 
        f1 = new File(dir, "fastjavac");
        if (f1.exists()) {
            try {
                return f1.getCanonicalPath();
            } catch (IOException ex) {
                // next try
            }
        }
        return null;
    }
    
    static String getWiredFullName() {
        if (wiredFullName != null) {
            if (wiredFullName.length() == 0)
                return null;
            return wiredFullName;
        }
        String nb = System.getProperty(JExternalCompilerGroup.JFormat.TAG_NBHOME);
        String fjpath = findFastJavac(nb);
        if (fjpath == null) {
            String user = System.getProperty("netbeans.user"); // NOI18N
            if (!user.equals(nb)) {
                fjpath = findFastJavac(user);
            }
        }
        if (fjpath != null) {
            wiredFullName = fjpath;
            return fjpath;
        }
        wiredFullName = ""; // NOI18N
        return null;
    }
    
    /** @return one of fastjavac.sun, fastjavac.sun.intel, fastjavac.exe
     * fastjavac.linux, null
     */
    static String getWiredName() {
        if (wiredName != null)
            return wiredName;
        int OS = Utilities.getOperatingSystem();
        if ((OS & Utilities.OS_SOLARIS) != 0) {
            if (System.getProperty("os.arch").indexOf("sparc") >= 0) { // NOI18N
                wiredName = "fastjavac.sun"; // NOI18N
            } else {
                wiredName = "fastjavac.sun.intel"; // NOI18N
            }
        } else if ((OS & Utilities.OS_WINDOWS_MASK) != 0) {
            wiredName = "fastjavac.exe";  // NOI18N
        } else if (isX86Linux()) {
            wiredName = "fastjavac.linux";
        }
        
        return wiredName;
    }
}
