/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java;


import org.openide.cookies.CompilerCookie;
import org.openide.compiler.ExternalCompiler;
import org.openide.compiler.CompilerJob;
import org.openide.filesystems.FileSystem;
import org.openide.filesystems.FileObject;
import org.openide.execution.NbProcessDescriptor;

/** 
*
* @author Ales Novak
*/
class JExternalCompiler extends ExternalCompiler {
    /** Value of my compiler type */
    private JavaExternalCompilerType myType;

    /** Type of compilation */
    protected Object type;
    
    /** Target filesystem, initially null.
     */
    private FileSystem target;

    /** Create an external compiler.
    * @param fo a file to compile
    * @param type the type of compilation ({@link #COMPILE}, {@link #BUILD}, or {@link #CLEAN})
    * @param nbDescriptor a description of an external compiler executable
    * @param err a regular expression to scan for compiler errors
    * @exception IllegalArgumentException if the file object is invalid
    */
    public JExternalCompiler(FileObject fo,Object type,
        NbProcessDescriptor nbDescriptor,ErrorExpression err,
        JavaExternalCompilerType ctype, FileSystem targetFS) {
        super (fo, type, nbDescriptor, err);
        this.myType = ctype;
        this.type = type;
        this.target = targetFS;
    }

    /** @return ExternalCompilerGroup
    */
    public Class compilerGroupClass() {
        return JExternalCompilerGroup.class;
    }

    /** @return JavaExternalCompilerTyep
    */
    public Object compilerGroupKey() {
        return myType;
    }

    JavaExternalCompilerType getMyType() {
        return myType;
    }

    /** @return <tt>true</tt> if the compilers are equal */
    public boolean equals(Object o) {
        if (o instanceof JExternalCompiler) {
            JExternalCompiler him = (JExternalCompiler) o;
            return (super.equals(o) &&
                    (him.getFileObject() == getFileObject()) && 
                    (him.myType == myType));
        } else {
            return false;
        }
    }
    
    public int hashCode() {
        FileObject fo = getFileObject();
        return ((fo == null) ? 0 : fo.hashCode()) ^ ((myType == null) ? 0 : myType.hashCode());
    }

    /* inherited */
    public boolean isUpToDate() {
        if (type == BUILD) {
            return false;
        }
        FileObject fo = getFileObject();
        boolean status = fo == null || JavaCompilerType.isUpToDate(fo, target);
        return status;
    }  
    
    final FileObject getFileObject0() {
        return getFileObject();
    }
}

