/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.codegen;

import org.openide.text.PositionRef;
import org.openide.text.PositionBounds;
/**
 * Helps with sorting elements according to their positions.
 *
 * @author  Svatopluk Dedic <mailto:sdedic@netbeans.org>
 * @version 0.1
 */
class PositionComparator implements java.util.Comparator {
    public int compare(java.lang.Object p1, java.lang.Object p2) {
        if (p1 instanceof Integer) {
            return comparePosToBinding((Integer)p1, (ElementBinding)p2);
        } else if (p2 instanceof PositionRef) {
            return -comparePosToBinding((Integer)p2, (ElementBinding)p1);
        }
        ElementBinding b1 = (ElementBinding)p1;
        ElementBinding b2 = (ElementBinding)p2;
        
        return b1.wholeBounds.getBegin().getOffset() -
               b2.wholeBounds.getBegin().getOffset();
    }
    
    private int comparePosToBinding(Integer iPos, ElementBinding b) {
        int off = iPos.intValue();
        PositionBounds w = b.wholeBounds;
        if (w.getBegin().getOffset() <= off &&
            w.getEnd().getOffset() > off)
            return 0;
        return off - w.getBegin().getOffset();
    }
    
    public boolean equals(java.lang.Object p1) {
        return (p1 instanceof PositionComparator);
    }
}
