/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/*
 * IdentifierNode.java
 *
 * Created on February 27, 2001, 10:57 AM
 */

package org.netbeans.modules.java.imptool;

import org.openide.nodes.*;
import java.util.ArrayList;
import java.util.Collection;
import java.beans.PropertyEditor;
import java.beans.PropertyEditorManager;
/**
 *
 * @author  thurka
 * @version 
 */
public class IdentifierNode extends AbstractNode {
    private Collection sourceIds;
    private SourceIdentifier sourceId;
    private static PropertyEditor choiceEditor1;
    
    /** Creates new IdentifierNode */
    public IdentifierNode(SourceIdentifier srcId) {
        super(Children.LEAF);
        sourceIds=new ArrayList();
        sourceIds.add(srcId);
        sourceId=srcId;
        setName(srcId.getFQN());
        setDisplayName(srcId.getName());
        setIconBase("/org/netbeans/modules/java/resources/identifier");
    }
    
    void add(SourceIdentifier srcId) {
        sourceIds.add(srcId);
    }
    
    /* Creates property set for this node */
    protected Sheet createSheet () {
        Sheet sheet = super.createSheet();
        Sheet.Set ps = Sheet.createPropertiesSet();
        
        sheet.put(ps);
        ps.put(new FQNProp());
        ps.put(new SourceNameProp());
        ps.put(new PackageProp());
        return sheet;
    }

    class FQNProp extends AbstractProp {
        
        FQNProp() {
            super("FQN",String.class,false);    // NOI18N
        }
        public Object getValue ()  {
            return sourceId.getFQN();
        }
    }

    class PackageProp extends AbstractProp {
        
        PackageProp() {
            super("PACKAGE",String.class,true);   // NOI18N
        }
        public Object getValue ()  {
            String src=sourceId.getPackage();
            
            return src==null?"":src;
        }
        public void setValue (Object val)  {
           ImpDataSource ds=((ImpDataSourceNode)IdentifierNode.this.getParentNode()).getDataSource();
           String str=(String)val;
           
           firePropertyChange(Node.PROP_PROPERTY_SETS,null,null);
           ds.setPackageForArray(str.length()==0?null:str,sourceIds);
        }

        public PropertyEditor getPropertyEditor () {
            if (choiceEditor1==null) {
                PropertyEditor editor=PropertyEditorManager.findEditor(getValueType());
                choiceEditor1= editor; 
            }
            return choiceEditor1;
        }
    }
    class SourceNameProp extends AbstractProp {
        
        SourceNameProp() {
            super("SOURCE_NAME",String.class,false);    // NOI18N
        }
        public Object getValue ()  {
            return sourceId.getSourceText();
        }
    }
}
