/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/*
 * ImpDataSource.java
 *
 * Created on February 23, 2001, 7:32 PM
 */

package org.netbeans.modules.java.imptool;

import org.netbeans.modules.java.JavaDataObject;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import java.util.*;
import org.openide.src.*;
import org.openide.util.SharedClassObject;
import org.openide.cookies.SourceCookie;
import org.netbeans.modules.java.JavaEditor;
import org.openide.text.PositionBounds;
import java.io.IOException;
import javax.swing.text.BadLocationException;

/**
 *
 * @author  thurka
 * @version 
 */
public class ImpDataSource extends Object {
    private HashMap sourceIDMap;
    private HashMap packageMap;
    private HashMap idNodes;
    private ImpToolSettings impSetting;
    private JavaDataObject dataObject;
    private PackageResolver packageResolver;
    private SourceCookie.Editor sourceEditor;
    private JavaEditor javaEditor;
    private SourceElement sourceElement;
    private String sourceFilePackageName;
    private int stage=0;
    
    private static final int FQN_STAGE=1;
    private static final int IMPORT_STAGE=2;
    private static final int RESOLVE_STAGE=3;
    private static final int COMMIT_STAGE=4;
    
    /** Creates new ImpDataSource */
    ImpDataSource(JavaDataObject jdo,Collection bodyIdentifiers) {
        ClassElement classArray[];
        Import imports[];
        int i;
        Iterator bodyIdIt=bodyIdentifiers.iterator();
        Identifier sourceFilePackage;
        
        dataObject=jdo;
        sourceEditor=(SourceCookie.Editor)jdo.getCookie(SourceCookie.Editor.class);
        javaEditor=(JavaEditor)jdo.getCookie(JavaEditor.class);
        sourceElement=jdo.getSource();
        sourceIDMap=new HashMap();
        packageMap=new HashMap();
        idNodes=new HashMap();
        impSetting=(ImpToolSettings)SharedClassObject.findObject(ImpToolSettings.class, true);
        classArray=sourceElement.getAllClasses();
        sourceFilePackage=sourceElement.getPackage();
        if (sourceFilePackage!=null)
            sourceFilePackageName=sourceFilePackage.getSourceName();
        for(i=0;i<classArray.length;i++) {
            ClassElement clsEl=classArray[i];
            FieldElement[] fields=clsEl.getFields();
            ConstructorElement[] constructors=clsEl.getConstructors();
            MethodElement[] methods=clsEl.getMethods();
            Identifier superClass=clsEl.getSuperclass();
            int j;
            
            if (!isGuarded(clsEl)) {
                if (superClass!=null)
                    addIdentifier(superClass,SourceIdentifier.TYPE_SUPERCLASS,clsEl);
                addIdentifierFromArray(clsEl.getInterfaces(),SourceIdentifier.TYPE_INTERFACE,clsEl);
            }
            for (j=0;j<fields.length;j++) {
                FieldElement fEl=fields[j];
                if (!isGuarded(fEl))
                    addIdentifierFromType(fEl.getType(),fEl);
            }
            for(j=0;j<constructors.length;j++) {
                ConstructorElement constructor=constructors[j];
                
                if (!isGuarded(constructor)) {
                    addIdentifierFromMethodParams(constructor);
                    addIdentifierFromArray(constructor.getExceptions(),SourceIdentifier.TYPE_EXCEPTION,constructor);
                }
            }
            for(j=0;j<methods.length;j++) {
                MethodElement method=methods[j];
               
                if (!isGuarded(method)) {
                    addIdentifierFromMethodParams(method);
                    addIdentifierFromArray(method.getExceptions(),SourceIdentifier.TYPE_EXCEPTION,method);
                    addIdentifierFromType(method.getReturn(),method);
                }
            }
        }
        while(bodyIdIt.hasNext()) {
            BodyIdentifier bId=(BodyIdentifier)bodyIdIt.next();
            
            if (!isGuardedIndetifier(bId))
                addIdentifier(bId.getIdentifier(),SourceIdentifier.TYPE_BODY,bId);
        }
        imports=sourceElement.getImports();
        for(i=0;i<imports.length;i++) {
            SourceImport srcImp=new SourceImport(imports[i]);
            
            srcImp.setUsed(!impSetting.isRemoveImport());
            addImport(srcImp);
        }
        if (impSetting.getResolveStrategy()==ImpToolSettings.RESOLVE_WELLKNOWN) {
            tryResolveIdetifiers();
        }
    }
    
    private void addIdentifierFromType(Type type,Object owner) {
        if (type.isClass())
            addIdentifier(type.getTypeIdentifier(),SourceIdentifier.TYPE_TYPE,owner);
        else if (type.isArray())
            addIdentifierFromType(type.getElementType(),owner);
    }

    private void addIdentifierFromMethodParams(ConstructorElement el) {
        MethodParameter pars[]=el.getParameters();
        int i;
        
        for(i=0;i<pars.length;i++) {
            MethodParameterOwner owner=new MethodParameterOwner(el,i);
            
            addIdentifierFromType(pars[i].getType(),owner);
        }
    }
    
    private void addIdentifierFromArray(Identifier[] ids, int type, Element owner) {
        int i;
        
        for (i=0;i<ids.length;i++)
            addIdentifier(ids[i],type,owner);
    }
    
    private void addIdentifier(Identifier id, int type, Object owner) {
        SourceIdentifier srcid=new SourceIdentifier(id,type,owner);
        
        addObjectToMap(sourceIDMap,srcid.getPackage(),srcid.getName(),srcid);
        if (id.getResolutionStatus()==Identifier.UNRESOLVED) {
            String idKey=id.getSourceName();
            IdentifierNode idNode=(IdentifierNode)idNodes.get(idKey);
            
            if (idNode==null) {
                idNode=new IdentifierNode(srcid);
                idNodes.put(idKey,idNode);
            } else
                idNode.add(srcid);
        }
    }

    private void addImport(SourceImport imp) {
        String packageName=imp.getPackage();
        String className;
        
        if (imp.isPackage()) {
            className=SourceImport.WHOLE_PACKAGE_ID;
        }
        else {
            className=imp.getName();
        }
        addObjectToMap(packageMap, packageName, className, imp);
    }
    
    private void addObjectToMap(HashMap map,String packageName, String className, Object o) {
        HashMap nameMap=(HashMap)map.get(packageName);
        ArrayList objs;

        if (nameMap==null) {
            nameMap=new HashMap();                
            map.put(packageName,nameMap);
        }
        objs=(ArrayList)nameMap.get(className);
        if (objs==null) {
            objs=new ArrayList();
            nameMap.put(className,objs);
        }
        objs.add(o);
    }
    
    ImpDataSourceNode createRootFQNNode() {
        Node template[]=new Node[0];
        Children ch=new Children.Array();
        
        ch.add((Node[])idNodes.values().toArray(template));
        stage=FQN_STAGE;
        return new ImpDataSourceNode(this,ch,ImpDataSourceNode.CLASS_NODE);
    }
    
    ImpDataSourceNode createRootImportNode() {
        Node template[]=new Node[0];
        Children ch=new Children.Array();
        Iterator packageNameIt=sourceIDMap.keySet().iterator();
        ArrayList packageArray=new ArrayList();
        
        while(packageNameIt.hasNext()) {
            HashMap nameMap;
            Iterator nameIt;
            String packageName=(String)packageNameIt.next();
            boolean isSpecialPackage;
            
            if (packageName==null)
                continue;
            isSpecialPackage=packageName.equals("java.lang") || packageName.equals(sourceFilePackageName); // NOI18N
            nameMap=(HashMap)sourceIDMap.get(packageName);
            nameIt=nameMap.values().iterator();
            
            while(nameIt.hasNext()) {
                Collection array=(Collection)nameIt.next();

                packageArray.add(new PackageNode(array));
                if (!isSpecialPackage)
                    setFQNDefaultFlagFor(array);
            }
        }
        ch.add((Node[])packageArray.toArray(template));
        stage=IMPORT_STAGE;       
        return new ImpDataSourceNode(this,ch,ImpDataSourceNode.PACKAGE_NODE);
    }

    ImpDataSourceNode createRootResolveNode() {
        Collection packageArray;
        Iterator pckIt;
        Node packageNodes[];
        Children ch=new Children.Array();
        int i=0;
        
        mergeSourceIDToPackages();
        packageArray=packageMap.values();
        pckIt=packageArray.iterator();
        packageNodes=new Node[packageArray.size()];
        while(pckIt.hasNext()) {
            HashMap nameMap=(HashMap)pckIt.next();
            
            packageNodes[i++]=ImportLineNode.createPackageNode(nameMap.values());
        }
        ch.add(packageNodes);
        stage=RESOLVE_STAGE;       
        return new ImpDataSourceNode(this,ch,ImpDataSourceNode.RESOLVE_NODE);
    }
    
    void setPackageForArray(String packageName,Collection srcIds) {
        Iterator idIt=srcIds.iterator();
        
        while(idIt.hasNext()) {
            SourceIdentifier srcId=(SourceIdentifier)idIt.next();
            
            setPackage(packageName,srcId);
        }
    }
    
    private void setPackage(String packageName,SourceIdentifier srcId) {
        HashMap nameMap=(HashMap)sourceIDMap.get(srcId.getPackage());
        Collection array;
        String className=srcId.getName();
        
        if (nameMap==null)
            throw new IllegalArgumentException("Package HashMap mismatch"); // NOI18N
        array=(Collection)(nameMap.get(className));
        if (array==null)
            throw new IllegalArgumentException("Name HashMap mismatch"); // NOI18N
        if (!array.remove(srcId))
            throw new IllegalArgumentException("Array mismatch"); // NOI18N
        if (array.size()==0)
            nameMap.remove(className);
        srcId.setPackage(packageName);
        addObjectToMap(sourceIDMap,packageName,className,srcId);
    }
    
    void setFQNFlagFor(Collection srcArray,int fqnFlag) {
        Iterator it=srcArray.iterator();

        while(it.hasNext()) {
            SourceIdentifier srcId=(SourceIdentifier)it.next();
            
            srcId.setUseFQN(fqnFlag);
        }
    }
    
    int getFQNFlagFor(Collection srcArray) {
        Iterator it=srcArray.iterator();
        
        if(it.hasNext()) {
            SourceIdentifier srcId=(SourceIdentifier)it.next();
            
            return srcId.getUseFQN();
        }
        return SourceIdentifier.USE_FQN_NO_CHANGE;
    }

    void setFQNDefaultFlagFor(Collection srcArray) {
        int size=srcArray.size();
        int fqnFlag;
        
        if (size>=impSetting.getMaxFqn()) 
            fqnFlag=SourceIdentifier.USE_FQN_FALSE;
        else
            fqnFlag=SourceIdentifier.USE_FQN_TRUE;
        setFQNFlagFor(srcArray,fqnFlag);
    }

    void mergeSourceIDToPackages() {
        Iterator packageNameIt=sourceIDMap.keySet().iterator();
        SourceImport srcImp;
        Iterator importMapIt;
        
        while(packageNameIt.hasNext()) {
            HashMap nameImportMap;
            HashMap nameMap;
            Iterator nameIt;
            String packageName=(String)packageNameIt.next();

            if (packageName==null || packageName.equals("java.lang") || packageName.equals(sourceFilePackageName)) // NOI18N
                continue;
            nameImportMap=(HashMap)packageMap.get(packageName);
            nameMap=(HashMap)sourceIDMap.get(packageName);
            nameIt=nameMap.keySet().iterator();
            while(nameIt.hasNext()) {
                String className=(String)(nameIt.next());
                Iterator idIt=((Collection)nameMap.get(className)).iterator();
                
                while(idIt.hasNext()) {
                    SourceIdentifier srcId=(SourceIdentifier)idIt.next();
                    
                    if (srcId.reallyUseFQN())
                        continue;
                    if (nameImportMap!=null) {
                        ArrayList impArray=(ArrayList)nameImportMap.get(className);

                        if (impArray!=null && !impArray.isEmpty()) {
                            srcImp=(SourceImport)impArray.get(0);
                            srcImp.setUsed(true);
                            continue;
                        }
                    }
                    srcImp=new SourceImport(packageName,className);
                    srcImp.setUsed(true);
                    addImport(srcImp);
                }
            }
        }
        importMapIt=packageMap.keySet().iterator();
        while(importMapIt.hasNext()) {
            String packageName=(String)importMapIt.next();
            HashMap nameImportMap=(HashMap)packageMap.get(packageName);
            
            if (nameImportMap.get(SourceImport.WHOLE_PACKAGE_ID)==null) {
                srcImp=new SourceImport(packageName,SourceImport.WHOLE_PACKAGE_ID);
                srcImp.setUsed(false);
                addImport(srcImp);
            }
            setUseDefaultFlagFor(nameImportMap);
        }
    }
    
    void setUseDefaultFlagFor(HashMap nameImportMap) {
        Iterator impIt=nameImportMap.values().iterator();
        int usedImports=0;
        SourceImport packageImport=null;
        
        while(impIt.hasNext()) {
            ArrayList impArray=(ArrayList)impIt.next();
            
            if (!impArray.isEmpty()) {
                SourceImport srcImp=(SourceImport)impArray.get(0);
                
                if (srcImp.isPackage())
                    packageImport=srcImp;
                else if (srcImp.isUse())
                    usedImports++;
            }
        }
        if (packageImport!=null) {
            if (usedImports>impSetting.getSingleImports())
                packageImport.setUse(true);
            else
                packageImport.setUse(false);
        }
    }
    
    void commitIdentifiers() throws SourceException,IOException,BadLocationException {
        Iterator classMapIt=sourceIDMap.values().iterator();
        
        while(classMapIt.hasNext()) {
            Iterator classArrayIt=((HashMap)classMapIt.next()).values().iterator();
            
            while(classArrayIt.hasNext()) {
                ArrayList srcIdList=(ArrayList)classArrayIt.next();
                int i;
                
                for (i=0;i<srcIdList.size();i++)
                    ((SourceIdentifier)srcIdList.get(i)).commit();
            }
        }
    }

    void commitImports() throws SourceException {
        Iterator classMapIt=packageMap.values().iterator();
        ArrayList allImports=new ArrayList();
        Import impArray[]=new Import[0];
        
        while(classMapIt.hasNext()) {
            HashMap nameImportMap=(HashMap)classMapIt.next();
            ArrayList srcImpArr=(ArrayList)nameImportMap.get(SourceImport.WHOLE_PACKAGE_ID);
            
            if (!addImportToList(allImports,srcImpArr)) {
                Iterator impArrayIt=nameImportMap.values().iterator();
                
                while(impArrayIt.hasNext()) {                    
                    addImportToList(allImports,(ArrayList)impArrayIt.next());
               }
            }
        }
        impArray=(Import[])allImports.toArray(impArray);
        dataObject.getSource().setImports(impArray);
    }
    
    private void commitAllChanges() {
	final Exception ex[] = { null };

        dataObject.suspendSupports();
        try {
            sourceElement.runAtomicAsUser(new Runnable() {
                public void run() {
                    try {
                        commitIdentifiers();
                        commitImports();
                    } catch (Exception e) {
                        ex[0]=e;
                    }    
                }
            });
        } catch (SourceException e) {
            ex[0]=e;
        }        
        dataObject.resumeSupports();
	if (ex[0] != null) {
            org.openide.TopManager.getDefault().notifyException(ex[0]);
        }
    }
    
    private boolean addImportToList(ArrayList allImports,ArrayList imps) {
        if (imps!=null && imps.size()>0) {
            SourceImport srcImp=(SourceImport)imps.get(0);

            if (srcImp.isUse()) {
                allImports.add(srcImp.getImport());
                return true;
            }
        }
        return false;
    }
    
    private Collection getAllPackages() {
        String defaultPackages[]=impSetting.getResolvePlaces();
        Set allPackages=new HashSet(sourceIDMap.keySet());
        Set impPackages=packageMap.keySet();
        int i;
        
        allPackages.remove(null);
        impPackages.remove(null);
        allPackages.addAll(impPackages);
        for(i=0;i<defaultPackages.length;i++) {
            allPackages.add(defaultPackages[i]);
        }
        return allPackages;
    }
    
    PackageResolver getPackageResolver() {
        if (packageResolver==null)
            packageResolver=new PackageResolver(getAllPackages());
        return packageResolver;
    }
    
    private void tryResolveIdetifiers() {
        HashMap classMap=(HashMap)sourceIDMap.get(null);
        
        if (classMap!=null) {
            Iterator arrIt;
            PackageResolver res;
            
            classMap=(HashMap)classMap.clone();
            arrIt=classMap.values().iterator();
            res=getPackageResolver();            
            while(arrIt.hasNext()) {
                ArrayList arr=(ArrayList)((ArrayList)arrIt.next()).clone();
                int i;
                
                for (i=0;i<arr.size();i++) {
                    SourceIdentifier srcId=(SourceIdentifier)arr.get(i);
                    String packageName=res.getPackageForClass(srcId.getName());
                    
                    if (packageName!=null)
                        setPackage(packageName,srcId);
                }
            }
        }
    }

    void runSilentFromNextStage() {
        switch (stage+1) {
            case FQN_STAGE:
                createRootFQNNode();
            case IMPORT_STAGE:
                createRootImportNode();
            case RESOLVE_STAGE:
                createRootResolveNode();
            case COMMIT_STAGE:
                commitAllChanges();                
        }
    }
    
    private boolean isGuarded(Element el) {
        if (sourceEditor!=null && javaEditor!=null) {
            javax.swing.text.Element textEl=sourceEditor.sourceToText(el);
            int start=textEl.getStartOffset();
            int end=textEl.getEndOffset();
            PositionBounds bounds;

            if (!textEl.isLeaf()) {
                javax.swing.text.Element firstTextEl=textEl.getElement(0);         
                end=firstTextEl.getStartOffset()-1;
            }
            bounds=javaEditor.createBounds(start,end,true);
            return !javaEditor.testOverlap(bounds);
        }
        return false;
    }
    
    private boolean isGuardedIndetifier(BodyIdentifier bId) {
        if (javaEditor!=null) {
            return !javaEditor.testOverlap(bId.getBounds());
        }
        return false;
    }
}
