/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.imptool;

import org.openide.util.actions.CookieAction;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.nodes.Node;
import org.openide.loaders.DataNode;
import org.netbeans.modules.java.JavaDataObject;
import org.openide.util.Task;
import org.openide.util.TaskListener;
import org.openide.util.RequestProcessor;

/** This action starts the Import Management Tool.
*
* @author Tomas Hurka
*/
public final class ImpToolAction extends CookieAction {
    /** generated Serialized Version UID */
    static final long serialVersionUID = 11111111111111L;
    private boolean imtRunning=false;
    
    /** Creates and starts a thread for generating documentation
    */
    protected void performAction(Node[] activatedNodes) {
        JavaDataObject[] jdos=new JavaDataObject[activatedNodes.length];
        Task impToolTask;
        
        imtRunning=true;
        for (int i = 0; i < activatedNodes.length; i++) {    
            jdos[i]=(JavaDataObject)activatedNodes[i].getCookie(JavaDataObject.class);
        }
        impToolTask=new Task(new ImpToolExecutor(jdos));

        impToolTask.addTaskListener(new TaskListener() {
            public void taskFinished(Task t) {
                t.removeTaskListener(this);
                notifyImtDone(t);
            }
        }
        );
        RequestProcessor.postRequest(impToolTask);
    }

    private void notifyImtDone(Task t) {
        imtRunning=false;
    }


    protected boolean enable(Node[] activNodes) {
        boolean ena=super.enable(activNodes);
        
        if (ena)
            return !imtRunning;
        return ena;
    }
    
    /** Cookie classes contains one class returned by cookie () method.
    */
    protected final Class[] cookieClasses () {
        return new Class[] { JavaDataObject.class };
    }

    /** All must be DataFolders or JavaDataObjects
    */
    protected int mode () {
        return MODE_ALL;
    }

    /* Help context where to find more about the action.
    * @return the help context for this action
    */
    public HelpCtx getHelpCtx() {
        return new HelpCtx (ImpToolAction.class);
    }

    /* Human presentable name of the action. This should be
    * presented as an item in a menu.
    * @return the name of the action
    */
    public String getName() {
        return Util.getString("LAB_ImpToolAction");    // NOI18N
    }
    
    /** The action's icon location.
    * @return the action's icon location
    */
    protected String iconResource () {
        return "/org/netbeans/modules/java/resources/impTool.gif"; // NOI18N
    }
    
}

/*
 * Log
 */
