/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/*
 * ImpToolExecutor.java
 *
 * Created on February 21, 2001, 3:18 PM
 */

package org.netbeans.modules.java.imptool;

import org.netbeans.modules.java.JavaDataObject;
import org.openide.util.Task;
import org.openide.util.TaskListener;
import org.netbeans.modules.java.parser.JavaParser;
import java.util.*;
import org.openide.src.*;
import java.awt.Dialog;
import org.openide.WizardDescriptor;
import org.openide.TopManager;

/**
 *
 * @author  thurka
 * @version 
 */
class ImpToolExecutor extends Object implements java.lang.Runnable {
    /** See org.openide.WizardDescriptor.PROP_AUTO_WIZARD_STYLE
     */
    private static final String PROP_AUTO_WIZARD_STYLE = "WizardPanel_autoWizardStyle"; // NOI18N
    /** See org.openide.WizardDescriptor.PROP_CONTENT_DISPLAYED
     */
    private static final String PROP_CONTENT_DISPLAYED = "WizardPanel_contentDisplayed"; // NOI18N
    /** See org.openide.WizardDescriptor.PROP_CONTENT_NUMBERED
     */
    private static final String PROP_CONTENT_NUMBERED = "WizardPanel_contentNumbered"; // NOI18N
    /** See org.openide.WizardDescriptor.PROP_CONTENT_SELECTED_INDEX
     */
    private static final String PROP_CONTENT_SELECTED_INDEX = "WizardPanel_contentSelectedIndex"; // NOI18N
    /** See org.openide.WizardDescriptor.PROP_CONTENT_DATA
     */
    private static final String PROP_CONTENT_DATA = "WizardPanel_contentData"; // NOI18N

    private JavaDataObject[] jdoArray;
    
    /** Creates new ImpToolExecutor */
    public ImpToolExecutor(JavaDataObject[] jdos) {
        jdoArray=jdos;
    }

    public void run() {
        int i;
        
        for(i=0;i<jdoArray.length;i++) {
            JavaDataObject jdo=jdoArray[i];
            JavaParser parser=(JavaParser)jdo.getCookie(JavaParser.class);
            Task parserTask;
            Dialog imtDialog;
            WizardDescriptor imtWd;
            RootPanel panels[];
            ImpDataSource dataSource;
            Object option;
            ParseIdentifiersRequest request=new ParseIdentifiersRequest();
            
            parserTask = parser.parse(parser.PRIORITY_NORMAL, true, false,request);
            parserTask.waitFinished();
            
            dataSource=new ImpDataSource(jdo,((IdentifierBuilder)request.getFactory()).bodyIdentifiers());
            panels=getIMTPanels();
            imtWd=new WizardDescriptor(panels,dataSource);
            imtWd.putProperty(PROP_AUTO_WIZARD_STYLE, new Boolean(true));
            imtWd.putProperty(PROP_CONTENT_DISPLAYED, new Boolean(true));
            imtWd.putProperty(PROP_CONTENT_NUMBERED, new Boolean(true));
            imtWd.putProperty(PROP_CONTENT_DATA, getIMTPanelNames(panels));
            imtWd.setModal(true);
            imtWd.setTitle(Util.getString("CTL_IMT_TITLE"));    // NOI18N
            imtWd.setTitleFormat(new java.text.MessageFormat("{0}")); // NOI18N
            imtDialog = TopManager.getDefault().createDialog(imtWd);
            imtDialog.show();
            imtDialog.dispose();
            if (imtWd.getValue()==WizardDescriptor.FINISH_OPTION) {
                dataSource.runSilentFromNextStage();
            }
       } 
    }
    
    RootPanel[] getIMTPanels() {
        RootPanel panels[]=new RootPanel[] { new FqnPanel(), new ImportPanel(), new ResolvePanel() };
        int i;
        
        for (i=0;i<panels.length;i++)
            panels[i].setHighlight(i);
        return panels;
    }
    
    String[] getIMTPanelNames(RootPanel panels[]) {
        String names[]=new String[panels.length];
        int i;
        
        for(i=0;i<panels.length;i++)
            names[i]=panels[i].getName();
        return names;
    }
}
