/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.imptool;

import java.beans.*;
import java.util.HashMap;

import org.openide.options.SystemOption;
import org.openide.src.Type;
import org.openide.util.HelpCtx;

/** A settings for Import Management Tool.
*
* @author Tomas Hurka
*/
public class ImpToolSettings extends SystemOption {
    /** generated Serialized Version UID */
    static final long serialVersionUID = 4521215457878L;

    public static final String PROP_MAX_FQN = "maxFqn"; // NOI18N
    public static final String PROP_SINGLE_IMPORTS = "singleImports"; // NOI18N
    public static final String PROP_REMOVE_IMPORT = "removeImport"; // NOI18N
    public static final String PROP_RESOLVE_STRATEGY = "resolveStrategy"; // NOI18N
    public static final String PROP_RESOLVE_PLACES = "resolvePlaces"; // NOI18N

    public static final int RESOLVE_NOTHING = 0;
    public static final int RESOLVE_WELLKNOWN = 1;
    public static final int RESOLVE_TAGS = 2;
    
    private static int maxFqn=4;
    private static int singleImports=4;
    private static int resolveStrategy=RESOLVE_NOTHING;
    private static boolean removeImport = true;
    private static String resolvePlaces[] = new String[] {};  // NOI18N

    /** human presentable name */
    public String displayName() {
        return Util.getString("CTL_IMT_Settings");  // NOI18N
    }

    public HelpCtx getHelpCtx () {
        return new HelpCtx (ImpToolSettings.class);
    }

    public int getSingleImports() {
        return singleImports;
    }

    public void setSingleImports(int val) {
        if (singleImports != val) {
            int old = singleImports;
            singleImports = val;
            firePropertyChange(PROP_SINGLE_IMPORTS, new Integer(old), new Integer(val));
        }
    }
    
    public int getMaxFqn() {
        return maxFqn;
    }

    public void setMaxFqn(int val) {
        if (maxFqn != val) {
            int old = maxFqn;
            maxFqn = val;
            firePropertyChange(PROP_MAX_FQN, new Integer(old), new Integer(val));
        }
    }

    public boolean isRemoveImport() {
        return removeImport;
    }

    public void setRemoveImport(boolean val) {
        if (val != removeImport) {
            removeImport = val;
            if (val)
                firePropertyChange(PROP_REMOVE_IMPORT, Boolean.FALSE, Boolean.TRUE);
            else
                firePropertyChange(PROP_REMOVE_IMPORT, Boolean.TRUE, Boolean.FALSE);
        }
    }    
    
    public String[] getResolvePlaces() {
        return resolvePlaces;
    }

    public void setResolvePlaces(String newPlaces[]) {
        if (resolvePlaces != newPlaces) {
            String old[] = resolvePlaces;
            resolvePlaces = newPlaces; 
            firePropertyChange(PROP_RESOLVE_STRATEGY, old, newPlaces);
        }
    }

    public int getResolveStrategy() {
        return resolveStrategy;
    }

    public void setResolveStrategy(int val) {
        if (resolveStrategy != val) {
            int old = resolveStrategy;
            resolveStrategy = val;
            firePropertyChange(PROP_RESOLVE_PLACES, new Integer(old), new Integer(val));
        }
    }

    public boolean isGlobal() {
        return false;
    }
}




