/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.imptool;

import java.awt.Image;
import java.beans.*;
import java.util.ResourceBundle;
import org.netbeans.modules.java.settings.JavaSettings;

/** BeanInfo for class ImpToolSettings.
* It describes its properties.
*
* @author Petr Hamernik
*/
public class ImpToolSettingsBeanInfo extends SimpleBeanInfo {

    /** Icons for java synchronization settings objects. */
    private Image icon;
    private Image icon32;

    /** Array of property descriptors. */
    private static PropertyDescriptor[] desc;

    /**
    * loads icons
    */
    public ImpToolSettingsBeanInfo() {
    }

    /** Returns the ExternalCompilerSettings' icon */
    public Image getIcon(int type) {
        if ((type == java.beans.BeanInfo.ICON_COLOR_16x16) || (type == java.beans.BeanInfo.ICON_MONO_16x16)) {
            if (icon == null)
                icon = loadImage("/org/netbeans/modules/java/resources/impToolSettings.gif"); // NOI18N
            return icon;
        } else {
            if (icon32 == null)
                icon32 = loadImage ("/org/netbeans/modules/java/resources/impToolSettings32.gif"); // NOI18N
            return icon32;
        }
    }

    /** Descriptor of valid properties
    * @return array of properties
    */
    public PropertyDescriptor[] getPropertyDescriptors () {
        if (desc==null) {
            try {
                desc = new PropertyDescriptor[] {
                       new PropertyDescriptor(ImpToolSettings.PROP_MAX_FQN,ImpToolSettings.class), 
                       new PropertyDescriptor(ImpToolSettings.PROP_SINGLE_IMPORTS,ImpToolSettings.class), 
                       new PropertyDescriptor(ImpToolSettings.PROP_REMOVE_IMPORT,ImpToolSettings.class), 
                       new PropertyDescriptor(ImpToolSettings.PROP_RESOLVE_STRATEGY,ImpToolSettings.class), 
                       new PropertyDescriptor(ImpToolSettings.PROP_RESOLVE_PLACES,ImpToolSettings.class) 
                       };
                desc[0].setDisplayName(Util.getString("PROP_MAX_FQN"));  // NOI18N
                desc[0].setShortDescription(Util.getString("HINT_MAX_FQN"));  // NOI18N
                desc[1].setDisplayName(Util.getString("PROP_SINGLE_IMPORTS"));  // NOI18N
                desc[1].setShortDescription(Util.getString("HINT_SINGLE_IMPORTS"));  // NOI18N
                desc[2].setDisplayName(Util.getString("PROP_REMOVE_IMPORT"));  // NOI18N
                desc[2].setShortDescription(Util.getString("HINT_REMOVE_IMPORT"));  // NOI18N
                desc[3].setDisplayName(Util.getString("PROP_RESOLVE_STRATEGY"));  // NOI18N
                desc[3].setShortDescription(Util.getString("HINT_RESOLVE_STRATEGY"));  // NOI18N
                desc[3].setPropertyEditorClass(ResStrategyEditor.class);
                desc[4].setDisplayName(Util.getString("PROP_RESOLVE_PLACES"));  // NOI18N
                desc[4].setShortDescription(Util.getString("HINT_RESOLVE_PLACES"));  // NOI18N
 
            }
            catch (IntrospectionException ex) {
                ex.printStackTrace ();
            }    
        }
        return desc;
    }

    public BeanDescriptor getBeanDescriptor() {
        BeanDescriptor javaDesc = new BeanDescriptor(JavaSettings.class);
        javaDesc.setDisplayName(Util.getString("CTL_IMT_Settings"));  // NOI18N
        return javaDesc;
    }

    /** Simple property editor for two-item pulldown. */
    public static class ResStrategyEditor extends PropertyEditorSupport {

        private static String resolveTags[];
        
        static void initResolveStrategyTags() {
            if (resolveTags==null) {
                resolveTags=new String[ImpToolSettings.RESOLVE_TAGS];

                resolveTags[ImpToolSettings.RESOLVE_NOTHING]=Util.getString("CTL_RESOLVE_NOTHING");  // NOI18N
                resolveTags[ImpToolSettings.RESOLVE_WELLKNOWN]=Util.getString("CTL_RESOLVE_WELLKNOWN");  // NOI18N
            }
        }
        
        public ResStrategyEditor() {
            initResolveStrategyTags();
        }
        
        public String[] getTags () {
            return resolveTags;
        }

        public String getAsText () {
            return resolveTags[((Integer) getValue ()).intValue ()];
        }

        public void setAsText (String text) throws IllegalArgumentException {
            for (int i = 0; i < resolveTags.length; i++) {
                if (resolveTags[i].equals (text)) {
                    setValue (new Integer (i));
                    return;
                }
            }
            throw new IllegalArgumentException ();
        }
    }
}
