/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/*
 * SinglePackageNode.java
 *
 * Created on March 1, 2001, 11:46 AM
 */

package org.netbeans.modules.java.imptool;

import org.openide.nodes.*;
import org.openide.src.Identifier;
import java.util.*;
import java.beans.PropertyEditor;
import org.openide.explorer.propertysheet.editors.ChoicePropertyEditor;

/**
 *
 * @author  thurka
 * @version 
 */
public class ImportLineNode extends AbstractNode {
    private SourceImport sourceImp;
    private static PropertyEditor choiceEditor1,choiceEditor2,choiceEditor3;

    /** Creates new SinglePackageNode */
    public ImportLineNode(SourceImport imp) {
        this(imp,Children.LEAF);
    }

    public ImportLineNode(SourceImport imp,Children ch) {
        super(ch);
        Identifier id=imp.getIdentifier();
        String name=imp.getName();
        sourceImp=imp;
        setName(id.getFullName());
        if (imp.isPackage()) {
            name=imp.getPackage();
            setIconBase("/org/netbeans/modules/java/resources/importPackage");
        } else
            setIconBase("/org/netbeans/modules/java/resources/importLine");
        setDisplayName(name);
    }

    static ImportLineNode createPackageNode(Collection impArray) {
        Iterator impIt=impArray.iterator();
        Children ch=new Children.Array();
        int i=0;
        SourceImport packageImp=null;
        Node childrens[];
        
        if (impArray.isEmpty())
            throw new java.lang.IllegalArgumentException();
        childrens=new Node[impArray.size()-1];
        while(impIt.hasNext()) {
            SourceImport imp=(SourceImport)((ArrayList)impIt.next()).get(0);
            
            if (!imp.isPackage()) {
                childrens[i++]=new ImportLineNode(imp);
            } else
                packageImp=imp;
        }
        if (childrens.length==0)
            ch=Children.LEAF;
        else
            ch.add(childrens);
        return new ImportLineNode(packageImp,ch);
    }
    
    /* Creates property set for this node */
    protected Sheet createSheet () {
        Sheet sheet = super.createSheet();
        Sheet.Set ps = Sheet.createPropertiesSet();
        
        sheet.put(ps);
        if (sourceImp.isClass()) {
            if (sourceImp.isUsed())
                ps.put(new NoActionProp());
            else
                ps.put(new AddRemoveActionProp());
        }
        else 
            ps.put(new PackageActionProp());
        return sheet;
    }

    class AddRemoveActionProp extends AbstractProp {
        
        AddRemoveActionProp() {
            super("RESOLVE_ACTION",Integer.TYPE,true);  // NOI18N
        }        

        public Object getValue ()  {
            return new Integer(sourceImp.isUse()?1:0); 
        }

        public void setValue (Object val)  {
            firePropertyChange(Node.PROP_PROPERTY_SETS,null,null);
            sourceImp.setUse(((Integer)val).intValue()!=0?true:false);
        }

        public PropertyEditor getPropertyEditor () {
            if (choiceEditor1==null) {
               choiceEditor1= new AddRemoveActionEditor(); 
            }
            return choiceEditor1;
        }
    }

    class NoActionProp extends AbstractProp {
        
        NoActionProp() {
            super("RESOLVE_ACTION",Integer.TYPE,false);  // NOI18N
        }        

        public Object getValue ()  {
            return new Integer(0); 
        }


        public PropertyEditor getPropertyEditor () {
            if (choiceEditor2==null) {
               choiceEditor2= new NoActionEditor(); 
            }
            return choiceEditor2;
        }
    }    

    class PackageActionProp extends AbstractProp {
        
        PackageActionProp() {
            super("RESOLVE_ACTION",Integer.TYPE,true);  // NOI18N
        }        

        public Object getValue ()  {
            return new Integer(sourceImp.isUse()?1:0); 
        }

        public void setValue (Object val)  {
            firePropertyChange(Node.PROP_PROPERTY_SETS,null,null);
            sourceImp.setUse(((Integer)val).intValue()!=0?true:false);
        }

        public PropertyEditor getPropertyEditor () {
            if (choiceEditor3==null) {
                choiceEditor3= new PackageActionEditor(); 
            }
            return choiceEditor3;
        }
    }
    
    static public class AddRemoveActionEditor extends ChoicePropertyEditor {
        
        public AddRemoveActionEditor() {
            super(new int[] {0,1},
                  new String[] {Util.getString("PROP_RESOLVE_ACTION_REMOVE"),  // NOI18N
                                Util.getString("PROP_RESOLVE_ACTION_USE")}  // NOI18N
            );
        }
    }

    static public class PackageActionEditor extends ChoicePropertyEditor {
        
        public PackageActionEditor() {
            super(new int[] {0,1},
                  new String[] {Util.getString("PROP_RESOLVE_ACTION_SINGLE"),  // NOI18N
                                Util.getString("PROP_RESOLVE_ACTION_PACKAGE")}  // NOI18N
            );
        }
    }    
    
    static public class NoActionEditor extends ChoicePropertyEditor {
        
        public NoActionEditor() {
            super(new int[] {0},
                  new String[] {Util.getString("PROP_RESOLVE_ACTION_NOACTION")}  // NOI18N
            );
        }
    }    
}
