/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/*
 * PackageNode.java
 *
 * Created on February 27, 2001, 4:57 PM
 */

package org.netbeans.modules.java.imptool;

import org.openide.nodes.*;
import java.util.*;
import java.beans.PropertyEditor;
import org.openide.explorer.propertysheet.editors.ChoicePropertyEditor;
/**
 *
 * @author  thurka
 * @version 
 */
public class PackageNode extends AbstractNode {
    private Collection identifierArray;
    private static PropertyEditor choiceEditor;
    
    /** Creates new PackageNode */
    public PackageNode(Collection idArray) {
        super(Children.LEAF);
        identifierArray=idArray;
        Iterator it=idArray.iterator();
        
        if (it.hasNext()) {
            SourceIdentifier srcId=(SourceIdentifier)it.next();
            String fqn=srcId.getFQN();
            
            setName(fqn);
            setDisplayName(fqn);
        } else {
            setName("Empty");  // NOI18N
            setDisplayName("Empty");  // NOI18N
        }
        setIconBase("/org/netbeans/modules/java/resources/identifier");
    }
    
    /* Creates property set for this node */
    protected Sheet createSheet() {
        Sheet sheet = super.createSheet();
        Sheet.Set ps = Sheet.createPropertiesSet();
        
        sheet.put(ps);
        ps.put(new CountProp());
        ps.put(new CountFQNProp());
        ps.put(new UseFQNProp());
        return sheet;
    }

    class CountProp extends AbstractProp {
        
        CountProp() {
            super("COUNT_IDS",Integer.TYPE,false);  // NOI18N
        }
        public Object getValue ()  {
            return new Integer(identifierArray.size()); 
        }
    }
    
    class CountFQNProp extends AbstractProp {
        
        CountFQNProp() {
            super("FQN_COUNT_IDS",Integer.TYPE,false);  // NOI18N
        }
        public Object getValue ()  {
            Iterator it=identifierArray.iterator();
            int count=0;
            
            while (it.hasNext()) {
                SourceIdentifier srcId=(SourceIdentifier)it.next();
                
                if (srcId.reallyUseFQN()) {
                    count++;
                }
            }
            return new Integer(count); 
        }
    }
    class UseFQNProp extends AbstractProp {
        private int fqnValue;
        ImpDataSource dataSource;
        
        UseFQNProp() {
            super("USE_FQN",Integer.TYPE,true);  // NOI18N
            dataSource=((ImpDataSourceNode)(PackageNode.this.getParentNode())).getDataSource();
        }

        public PropertyEditor getPropertyEditor () {
            if (choiceEditor==null) {
               choiceEditor= new FQNChoicePropertyEditor(); 
            }
            return choiceEditor;
        }
        

        public Object getValue ()  {
            return new Integer(dataSource.getFQNFlagFor(identifierArray)); 
        }

        public void setValue (Object val)  {
            firePropertyChange(Node.PROP_PROPERTY_SETS,null,null);
            dataSource.setFQNFlagFor(identifierArray,((Integer)val).intValue());
        }

    }
    static public class FQNChoicePropertyEditor extends ChoicePropertyEditor {
        
        public FQNChoicePropertyEditor() {
            super(new int[] {SourceIdentifier.USE_FQN_NO_CHANGE,
                             SourceIdentifier.USE_FQN_TRUE,
                             SourceIdentifier.USE_FQN_FALSE},
                  new String[] {Util.getString("PROP_USE_FQN_NO_CHANGE"),  // NOI18N
                                Util.getString("PROP_USE_FQN_TRUE"),  // NOI18N
                                Util.getString("PROP_USE_FQN_FALSE")}  // NOI18N
            );
        }
    }
}
