/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.imptool;

import java.util.*;
/** PackageResolver
 *  
 *  Package name resolver for the class name
 *
 * @author msedliak
 * @version
 */
public class PackageResolver extends Object {

    private static final String DOT_SEPARATOR = ".";
    private Map resolvedClasses = null;
    private Collection packageNames = null;

    /** Creates new PackageResolver 
     	@param packages names of the packages where to look for the classes
     */
    PackageResolver(Collection packages) {
        initialize();
        packageNames=packages;
    }

    /** Finds the package name for a class name
	@param className name of the class
    */
    String getPackageForClass(String className) {
        String result = null;
        if ( (packageNames != null) && (resolvedClasses != null) ) {
            if (resolvedClasses.containsKey( className ))
                result = (String)resolvedClasses.get( className );
            else
                result = getPackageForClassName( className );
        }
        if ( (result != null) && (!resolvedClasses.containsKey( className ) ) ) resolvedClasses.put( className, result );
        return result;
    }

    /** Sets already resolved class names
	@param className name of the class
        @param packageName name of the package
    */
    void setResolvedClasses(String className, String packageName) {
        if (resolvedClasses == null) initialize();
        if (!resolvedClasses.containsKey( className ) ) resolvedClasses.put( className, packageName );
    }

    /** Initialization
    */
    private void initialize() {
        if (resolvedClasses == null) resolvedClasses = new HashMap();
        if (packageNames == null) packageNames = new ArrayList(10);
        resolvedClasses.clear();
        packageNames.clear();
    }

    /** Looks package for not found class
	@param className name of the class
    */
    private String getPackageForClassName(String className) {
        String result = null;
        String packageName = null;
        for (Iterator it = packageNames.iterator(); it.hasNext(); ) {
            packageName = (String)it.next();
            result = resolvePackageForClass( packageName, className );
            if (result != null) break;
        }
        return result;
    }

    /** Executive method
	@param className name of the class
        @param packageName name of the package
    */
    private String resolvePackageForClass(String packageName, String className) {
        String result = null;
        //Class clazz = Class.forName( packageName + DOT_SEPARATOR + className ); --> changed to openide equivalent :
        org.openide.src.ClassElement classElement = org.openide.src.ClassElement.forName( packageName + DOT_SEPARATOR + className );
        //ATTENTION !!! : LinkageError is treated as NOT FOUND in superclass - ClassElement.forName() without any chance to treat it different way
        if (classElement != null) result = packageName;
        classElement = null;
        return result;
    }
}