/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.java.imptool;

import java.awt.*;
import javax.swing.*;
import org.openide.WizardDescriptor;
import org.openide.nodes.Node;
import org.openide.nodes.Sheet;
import javax.swing.event.ChangeListener;
import org.openide.TopManager;
import org.openide.explorer.*;

/**
 *
 * @author  thurka
 */
abstract class RootPanel extends JPanel implements WizardDescriptor.Panel, WizardDescriptor.FinishPanel {
    ImpDataSource dataSource;
    boolean initialized=false;

    /** See org.openide.WizardDescriptor.PROP_CONTENT_SELECTED_INDEX
     */
    private static final String PROP_CONTENT_SELECTED_INDEX = "WizardPanel_contentSelectedIndex"; // NOI18N

    /** Creates new form importPanel */
    public RootPanel() {
    }

    void initialize() {
        if (!initialized) {
            initAllComponents();
            setPreferredSize(new Dimension(560, 350));
            initialized = true;
        }
    }
    
    public java.awt.Component getComponent() {
      initialize();
      return this;
    }
  
    /** Help for this panel.
     * When the panel is active, this is used as the help for the wizard dialog.
     * @return the help or <code>null</code> if no help is supplied
     */
    public org.openide.util.HelpCtx getHelp() {
        return new org.openide.util.HelpCtx("java.import");
    }

    /** Provides the wizard panel with the current data--either
     * the default data or already-modified settings, if the user used the previous and/or next buttons.
     * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
     * @param settings the object representing wizard panel state, as originally supplied to {@link WizardDescriptor#WizardDescriptor(WizardDescriptor.Iterator,Object)}
     */
    public void readSettings(Object settings) {
        dataSource=(ImpDataSource)settings;
    }
     /** Provides the wizard panel with the opportunity to update the
     * settings with its current customized state.
     * Rather than updating its settings with every change in the GUI, it should collect them,
     * and then only save them when requested to by this method.
     * Also, the original settings passed to {@link #readSettings} should not be modified (mutated);
     * rather, the (copy) passed in here should be mutated according to the collected changes.
     * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
     * @param settings the object representing a settings of the wizard
     */
    public void storeSettings(Object settings) {
    }
  
    /** Test whether the panel is finished and it is safe to proceed to the next one.
     * If the panel is valid, the "Next" (or "Finish") button will be enabled.
     * @return <code>true</code> if the user has entered satisfactory information
     */
    public boolean isValid() {
      return true;
    }
    /** Add a listener to changes of the panel's validity.
     * @param l the listener to add
     * @see #isValid
     */
    public void addChangeListener(ChangeListener l) {
    }
    /** Remove a listener to changes of the panel's validity.
     * @param l the listener to remove
     */
    public void removeChangeListener(ChangeListener l) {
    }
  
    public void setHighlight(int number) {
        putClientProperty(PROP_CONTENT_SELECTED_INDEX,new Integer(number));
    }


    void addTableView(Node masterNode,JScrollPane tv) {
        ExplorerManager em;
        ExplorerPanel ep;
        
        em = new ExplorerManager();
        em.setRootContext(masterNode);
        ep = new ExplorerPanel (em);
        ep.setLayout (new BorderLayout ());
        ep.add ("Center", tv);  // NOI18N
        add(ep,BorderLayout.CENTER);
    }
    
    void initListTableView(Node masterNode) {
        
        if (masterNode.getChildren().getNodesCount()!=0) {
            Node.Property props[];
            ListTableView ltv= new ListTableView();

            ltv.setPopupAllowed(false);
            ltv.setListViewPreferredSize(new Dimension(200, 0));
    //        ltv.setTableViewPreferredSize(new Dimension(250, 0));
    //        ltv.setPreferredSize(new Dimension(450, 0));
            ltv.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
            props=getPropertiesFrom(masterNode);
            if (props!=null)
                ltv.setProperties(props);
            addTableView(masterNode,ltv);
        } else {
            addNoRowsLabel();
        }
    }

    void initTreeTableView(Node masterNode) {
        
        if (masterNode.getChildren().getNodesCount()!=0) {
            Node.Property props[];
            TreeTableView ttv= new TreeTableView();

            ttv.setPopupAllowed(false);
            ttv.setTreeViewPreferredSize(new Dimension(250, 0));
    //        ttv.setTableViewPreferredSize(new Dimension(250,0));
    //        ttv.setPreferredSize(new Dimension(450, 0));
            ttv.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
    //        ttv.setRootVisible(false);
            props=getPropertiesFrom(masterNode);
            if (props!=null)
                ttv.setProperties(props);
            addTableView(masterNode,ttv);
        } else {
            addNoRowsLabel();
        }
    }
    
    Node.Property[] getPropertiesFrom(Node masterNode) {
        Node nodes[]=masterNode.getChildren().getNodes();
        
        if (nodes.length>0) {
            Node.PropertySet sets[]=(Node.PropertySet[])nodes[0].getPropertySets();
            
            if (sets.length>0)
                return sets[0].getProperties();
        }
        return null;
    }
    
    private void addNoRowsLabel() {
        JLabel noRowText = new JLabel();

        noRowText.setText(Util.getString(noRowsText()));
        add(noRowText,BorderLayout.NORTH);
    }
    
    abstract void initAllComponents();
    
    abstract public String getName();

    abstract String noRowsText();
    
}
