/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/*
 * SourceIdentifier.java
 *
 * Created on February 23, 2001, 7:14 PM
 */

package org.netbeans.modules.java.imptool;

import org.openide.src.*;
import java.io.IOException;
import javax.swing.text.BadLocationException;

/**
 *
 * @author  thurka
 * @version 
 */
public class SourceIdentifier extends Object {
    private Identifier identifier;
    private int type;
    private Object owner;
    private String resolvedPackage;
    private int useFQN=USE_FQN_NO_CHANGE;
    
    static final int USE_FQN_NO_CHANGE=0;
    static final int USE_FQN_TRUE=1;
    static final int USE_FQN_FALSE=2;

    static final int TYPE_TYPE = 0;
    static final int TYPE_SUPERCLASS = 1;
    static final int TYPE_INTERFACE = 2;
    static final int TYPE_EXCEPTION = 3;
    static final int TYPE_BODY = 4;

    /** Creates new SourceIdentifier */
    public SourceIdentifier(Identifier id,int t,Object o) {
        identifier=id;
        type=t;
        owner=o;
    }
    
    void commit() throws SourceException,IOException,BadLocationException {
        Identifier newId=null;
        
        if (resolvedPackage==null && useFQN==USE_FQN_NO_CHANGE)
            return;
        newId=Identifier.create(getFQN(),getSourceText());
        if (owner instanceof ClassElement) {
            ClassElement clEl=(ClassElement)owner;

            switch (type) {
                case TYPE_SUPERCLASS:
                    clEl.setSuperclass(newId);
                    break;
                case TYPE_INTERFACE:
                    clEl.removeInterface(identifier);
                    clEl.addInterface(newId);
                    break;
            }
        } else if (owner instanceof MethodElement) {
            MethodElement mEl=(MethodElement)owner;

            switch (type) {
                case TYPE_EXCEPTION:
                    int i;
                    Identifier exs[]=mEl.getExceptions();

                    replaceIdentifierIn(exs,newId);
                    mEl.setExceptions(exs);
                    break;
                case TYPE_TYPE:
                    Type newType=createTypeFrom(mEl.getReturn(),newId);

                    mEl.setReturn(newType);
                    break;
            }            
        } else if (owner instanceof ConstructorElement) {
            ConstructorElement cEl=(ConstructorElement)owner;

            switch (type) {
                case TYPE_EXCEPTION:
                    Identifier exs[]=cEl.getExceptions();

                    replaceIdentifierIn(exs,newId);
                    cEl.setExceptions(exs);
                    break;
            }   
        } else if (owner instanceof FieldElement) {
            FieldElement fEl=(FieldElement)owner;
            Type newType=createTypeFrom(fEl.getType(),newId);

            fEl.setType(newType);
        } else if (owner instanceof MethodParameterOwner) {
            MethodParameterOwner mpo=(MethodParameterOwner)owner;            
            Type newType=createTypeFrom(mpo.getMethodParameterType(),newId);

            mpo.setMethodParameterType(newType);
        } else if (owner instanceof BodyIdentifier) {
            BodyIdentifier bId=(BodyIdentifier)owner;

            bId.getBounds().setText(getSourceText());
        } else
            throw new java.lang.IllegalArgumentException();
    }

    void rollback() {
       resolvedPackage=null;
       useFQN=USE_FQN_NO_CHANGE;
    }
    
    int resolve(int type) {
        return 0;
    }
    
    String getSourceText() {
        if (resolvedPackage!=null) {
            if (useFQN==USE_FQN_TRUE) return getFQN();
            else return getName();
        }
        if (useFQN==USE_FQN_TRUE)
            return identifier.getFullName();
        else if (useFQN==USE_FQN_FALSE)
            return identifier.getName();
        return identifier.getSourceName();
    }
    
    String getFQN() {
        if (resolvedPackage!=null) {
            return resolvedPackage.concat(".").concat(getName());    // NOI18N
        }
        return identifier.getFullName();
    }
    
    String getPackage() {
        String pck;
        
        if (resolvedPackage!=null) {
            return resolvedPackage;  
        }
        if (identifier.getResolutionStatus()==Identifier.UNRESOLVED)
            return null;
        pck=identifier.getQualifier();
        if (pck!=null && pck.length()>0)
            return pck;
        return null;
    }

    void setPackage(String packageName) {
        resolvedPackage=packageName;
    }

    String getName() {
        return identifier.getName();
    }
    
    int getResolveStatus() {
        return 0;
    }
    
    /** Getter for property useFQN.
     * @return Value of property useFQN.
    */
    int getUseFQN() {
        return useFQN;
    }
    
    /** Setter for property useFQN.
     * @param useFQN New value of property useFQN.
    */
    void setUseFQN(int use) {
        useFQN = use;
    }
    
    boolean reallyUseFQN() {        
       return getSourceText().equals(getName())?false:true;
    }
    
    private Type createTypeFrom(Type type,Identifier newId) {
        if (type.isClass()) {
            if (type.getTypeIdentifier().equals(identifier)) {
               return Type.createClass(newId); 
            } else
               throw new java.lang.IllegalArgumentException();       
        }
        else if (type.isArray())
            return Type.createArray(createTypeFrom(type.getElementType(),newId));
        throw new java.lang.IllegalArgumentException();
    }
    
    private void replaceIdentifierIn(Identifier ids[],Identifier newId) {
        int i;
        
        for (i=0;i<ids.length;i++) {
            if (ids[i].equals(identifier)) {
                ids[i]=newId;
                break;
            }
        }
    }    
}
