/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/*
 * SourceImport.java
 *
 * Created on February 28, 2001, 5:12 PM
 */

package org.netbeans.modules.java.imptool;

import org.openide.src.Import;
import org.openide.src.Identifier;
/**
 *
 * @author  thurka
 * @version 
 */
class SourceImport extends Object {
    private Import originalImport;
    private boolean used;
    private boolean created;
    private boolean use;
    static final String WHOLE_PACKAGE_ID = "*";   // NOI18N
    
    /** Creates new SourceImport */
    SourceImport(Import imp) {
        originalImport=imp;
        used=false;
        created=false;
    }
    
    SourceImport(String packageName,String className) {
        boolean wholePackage=className.equals(WHOLE_PACKAGE_ID);
        Identifier id;
        
        if (wholePackage)
            id=Identifier.create(packageName);
        else
            id=Identifier.create(packageName+"."+className);  // NOI18N
        originalImport=new Import(id,wholePackage);
        used=true;
        created=true;
    }
    
    /** Getter for property used.
     * @return Value of property used.
    */
    boolean isUsed() {
        return used;
    }
    
    /** Setter for property used.
     * @param used New value of property used.
    */
    void setUsed(boolean val) {
        used = val;
        setUse(val);
    }
    
    /** Getter for property created.
     * @return Value of property created.
    */
    boolean isCreated() {
        return created;
    }
    
    /** Setter for property created.
     * @param created New value of property created.
    */
    void setCreated(boolean val) {
        created = val;
    }

    boolean isClass() {
        return originalImport.isClass();
    }

    boolean isPackage() {
        return originalImport.isPackage();
    }
    
    String getPackage() {
        Identifier id=getIdentifier();
        if (isPackage())
            return id.getFullName();
        return id.getQualifier();
    }
    
    String getName() {
        return getIdentifier().getName();
    }
    
    Identifier getIdentifier() {
        return originalImport.getIdentifier();
    }
    
    /** Getter for property use.
     * @return Value of property use.
    */
    boolean isUse() {
        return use;
    }
    
    /** Setter for property use.
     * @param use New value of property use.
    */
    void setUse(boolean use) {
        this.use = use;
    }
    
    Import getImport() {
        return originalImport;
    }
}
