/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.model;

import java.lang.reflect.Modifier;
import java.beans.PropertyChangeEvent;

import org.openide.src.*;

/**
 *
 * @author  sdedic
 * @version 
 */
class ConstructorElementImpl extends CallableImpl {
    
    ConstructorElementImpl(DefaultLangModel model) {
        super(model);
    }
    
    protected Binding createBinding(Element el) {
        return getModelImpl().getBindingFactory().bindConstructor((ConstructorElement)el);
    }
    
    public Object readResolve() {
        return null;
    }
    
    protected Identifier createName(String dummy) {
        if (getDeclaringImpl() != null)
            return Identifier.create(getDeclaringImpl().getName().getSourceName());
        else
            return super.createName(dummy);
    }
    
    /** Imposes additional constraint on the constructor: the declaring class'
     * name must match the constructor's name. 
     */
    protected void checkNameConstraints(Identifier name) throws SourceException {
        super.checkNameConstraints(name);
        // additional constraint:
        // PENDING: do not check depending on model lock type.
        if (!name.getName().equals(getDeclaringImpl().getName().getName())) {
            throw new SourceException("Invalid constructor name: " + name);
        }
    }    
    
    /** Check for modifiers disallowed on a constructor: static, strictfp, final,
     * synchronized, abstract
     */
    protected void checkModifierConstraints(int newMods) throws SourceException {
        super.checkModifierConstraints(newMods);
        // PENDING: constraint checking depending on the lock type 
        if ((newMods & (Modifier.STATIC | Modifier.STRICT | Modifier.FINAL |
            Modifier.ABSTRACT | Modifier.SYNCHRONIZED)) > 0) {
            throw new SourceException("Invalid constructor modifiers : " + 
                Modifier.toString(newMods));
        }
    }

    public String toString() {
        return "ConstructorElementImpl[" + getName().getSourceName() + ", " +
            getParameters().length + " args]";
    }
    
    protected Element cloneSelf() {
        ConstructorElement clone = new ConstructorElement();
        copyCallableProperties(clone);
        return clone;
    }
}
