/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.model;

import org.openide.src.*;

/**
 *
 * @author  sdedic
 * @version 
 */
class FieldCollection extends PartialCollection {

    static final FieldElement[] EMPTY = new FieldElement[0];
    
    public FieldCollection(ElementEvents events, ElementCreator creator, 
            MemberCollection allMembers) {
        super(events, creator, allMembers, ElementProperties.PROP_FIELDS);
    }

    public Class getElementImplClass() {
        return FieldElementImpl.class;
    }

    protected ElementImpl createElement(Element parent) {
        FieldElementImpl fimpl = creator.createField((ClassElement)parent);
        return fimpl;
    }

    protected Object[] createEmpty() {
        return EMPTY;
    }

    protected Element[] createEmpty(int size) {
        return new FieldElement[size];
    }

    /**
     * Attempts to find a field with the specified name. If more than one such field
     * exist, it returns one of them.
     * @param name name of the field to search for.
     * @return field element instance, or null if not found.
     */
    public FieldElement getField(Identifier name) {
        Element[] elems = getElements();
        String srcName = name.getSourceName();
        
        for (int i = 0; i < elems.length; i++) {
            FieldElement el = (FieldElement)elems[i];
            if (el.getName().getSourceName().equals(srcName)) {
                return el;
            }
        }
        return null;
    }
}
