/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.model;

import java.beans.PropertyChangeEvent;

import org.openide.src.*;

/**
 *
 * @author  sdedic
 * @version 
 */
public class ImportImpl extends ElementImpl implements ImportElement.Impl {
    Import      def;

    SourceElementImpl   sourceImpl;
    
    /** Creates new ImportImpl */
    ImportImpl(Import d) {
        super(null);
        this.def = d;
    }
    
    ImportImpl(DefaultLangModel model) {
        super(model);
    }
    
    protected void createFromModel(Element model) throws SourceException {
        ImportElement i = (ImportElement)model;
        
        setImport(i.getImport());
    }

    protected final Binding createBinding(Element el) {
        return getModelImpl().getBindingFactory().bindImport((ImportElement)el);
    }
    
    public void setParent(ElementImpl impl) {
        sourceImpl = (SourceElementImpl)impl;
    }
    
    public Import   getImport() {
        return def;
    }
    
    public SourceElementImpl findSource() {
        return sourceImpl;
    }
    
    private Binding.Import getImportBinding() {
        return (Binding.Import)getBinding();
    }
    
    public void setImport(Import i) throws SourceException {
        Object token = takeLock();
        try {
            if (def != null && def.isPackage() == i.isPackage() &&
                def.getIdentifier().getSourceName().equals(
                i.getIdentifier().getSourceName()))
                return;
            PropertyChangeEvent evt = new PropertyChangeEvent(
                getElement(), "import", def, i); // NOI18N
            checkVetoablePropertyChange(evt);
            getImportBinding().changeImport(i);
            this.def = i;
            fireOwnPropertyChange(evt);
            if (!isCreated())
                sourceImpl.imports.importsChanged();
        } finally {
            releaseLock(token);
        }
    }

    public Object readResolve() {
        return null;
    }
    
    public Object writeReplace() {
        return null;
    }
    
    protected boolean parentValid() {
        return sourceImpl != null && sourceImpl.isValid();
    }
}
