/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.model;

import org.openide.src.*;

/**
 *
 * @author  sdedic
 * @version 
 */
class InnerClassCollection extends PartialCollection {
    static final ClassElement[] EMPTY = new ClassElement[0];
    
    public InnerClassCollection(ElementEvents events, ElementCreator creator, 
            MemberCollection allMembers) {
        super(events, creator, allMembers, ElementProperties.PROP_CLASSES);
    }

    public Class getElementImplClass() {
        return ClassElementImpl.class;
    }

    public Element wrapElement(ElementImpl impl) {
        return this.creator.getWrapper().wrapClass((ClassElementImpl)impl,
            (ClassElement)events.getEventSource());
    }

    protected ElementImpl createElement(Element parent) {
        ClassElementImpl cimpl = creator.createInnerClass((ClassElement)parent);
        return cimpl;
    }

    protected Object[] createEmpty() {
        return EMPTY;
    }

    protected Element[] createEmpty(int size) {
        return new ClassElement[size];
    }
    
    public ClassElement getInnerClass(Identifier name) {
        ClassElement[] elems = (ClassElement[])getElements();
        String srcName = name.getSourceName();
        
        for (int i = 0; i < elems.length; i++) {
            ClassElement el = elems[i];
            if (el.getName().getSourceName().equals(srcName)) {
                String fname = name.getFullName();
                if (!fname.equals(srcName) &&
                    !fname.equals(el.getName().getFullName()))
                    return null;
                return el;
            }
        }
        return null;
    }
}
