/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.model;

import org.openide.src.*;

/**
 *
 * @author  sdedic
 * @version 
 */
class MethodCollection extends PartialCollection {
    static final MethodElement[] EMPTY = new MethodElement[0];
    static final Type[] NO_TYPES = new Type[0];
    
    public MethodCollection(ElementEvents events, ElementCreator creator, 
            MemberCollection allMembers) {
        super(events, creator, allMembers, ElementProperties.PROP_METHODS);
    }

    public Class getElementImplClass() {
        return MethodElementImpl.class;
    }

    protected ElementImpl createElement(Element parent) {
        MethodElementImpl mimpl = creator.createMethod((ClassElement)parent);
        return mimpl;
    }

    protected Object[] createEmpty() {
        return EMPTY;
    }

    protected Element[] createEmpty(int size) {
        return new MethodElement[size];
    }
    
    public MethodElement getMethod(Identifier name, Type[] argtypes) {
        if (argtypes == null)
            argtypes = NO_TYPES;
        MethodElement[] elems = (MethodElement[])getElements();
        String srcName = name.getSourceName();
        
        method:
        for (int i = 0; i < elems.length; i++) {
            MethodElement el = elems[i];
            if (!el.getName().getSourceName().equals(srcName))
                continue;
            MethodParameter[] params = el.getParameters();
            if (params.length != argtypes.length)
                continue;
            
            for (int j = 0; j < params.length; j++) {
                if (!params[j].getType().equals(argtypes[j]))
                    continue method;
            }
            return el;
        }
        return null;
    }
}
