/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.model;

import java.beans.*;
import org.openide.src.*;

/**
 * The class implements properties specific to a method.
 *
 * @author  Svatopluk Dedic, Petr Hamernik, Jaroslav Tulach
 * @version 0.1
 * @since 24/11/2000
 */
class MethodElementImpl extends CallableImpl implements MethodElement.Impl {
    /** Return type for the method. The type can be null or errorenous. Special case:
     * constructors have return type Type.NONE
     */
    private Type                returnType;

    // Construction
    ///////////////////////////////////////////////////////////////////////////////////

    /*
    MethodElementImpl(Identifier name, int modifiers, Type returnType,
        MethodParameter[] params, Identifier[] exceptions) {
        super(name, modifiers, params, exceptions);
        this.returnType = returnType;
    }
    */
    
    MethodElementImpl(DefaultLangModel model) {
        super(model);
    }
    
    protected void createFromModel(Element el) throws SourceException {
        super.createFromModel(el);
        MethodElement m = (MethodElement)el;
        setReturn(m.getReturn());
    }

    protected final Binding createBinding(Element el) {
        return getModelImpl().getBindingFactory().bindMethod((MethodElement)el);
    }

    // Getters
    ///////////////////////////////////////////////////////////////////////////////
    
    /**
     * Retrieves the return type of the method.
     * @return the return type description.
     */
    public Type getReturn() {
        return this.returnType;
    }
    
    // Setters
    ///////////////////////////////////////////////////////////////////////////////

    /**
     * Sets the return type.
     */
    public void setReturn(Type newType) throws SourceException {
        Object token = takeLock();
        try {
            newType = resolveType(newType);
            if (compareSourceTypes(this.returnType, newType)) {
                return;
            }
            Type old = this.returnType;
            PropertyChangeEvent evt = new PropertyChangeEvent(getElement(), PROP_RETURN, old, newType);

            checkReturnTypeConstraints(newType);
            checkVetoablePropertyChange(evt);
            getMethodBinding().changeReturnType(newType);
            fireOwnPropertyChange(evt);
            this.returnType = newType;
            commit();
        } finally {
            releaseLock(token);
        }
    }
    
    // Utility and helper methods.
    ///////////////////////////////////////////////////////////////////////////////
    protected final Element createWrapperElement() {
        return new MethodElement(this, getDeclaringClass());
    }
    
    protected void checkReturnTypeConstraints(Type t) throws SourceException {
        // no checking, no constraings on the rettype.
    }

    // Serialization support
    ///////////////////////////////////////////////////////////////////////////////
    public Object readResolve() {
        return null;
    }
    
    public String toString() {
        return "MethodElementImpl[" + getName().getSourceName() + ", " +
            getParameters().length + " args]";
    }
    
    protected Element cloneSelf() {
        MethodElement clone = new MethodElement();
        copyCallableProperties(clone);
        try {
            clone.setReturn(getReturn());
        } catch (SourceException ex) {
        }
        return clone;
    }
}
