/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.model;

import org.openide.src.*;

/**
 *
 * @author  sdedic
 * @version 
 */
class TopClassCollection extends ObjectCollection implements Positioner.Acceptor {
    Positioner      positioner;
    
    public TopClassCollection(Binding.Container cont, ElementCreator creator, 
        ElementEvents events) {
        super(cont, creator, events, ElementProperties.PROP_CLASSES);
        positioner = DefaultLangModel.DEFAULT_POSITIONER;
    }
    
    public void setPositioner(Positioner pos) {
        this.positioner = pos;
    }
    
    public ClassElement getClass(Identifier id) {
        ClassElement[] els = (ClassElement[])getElements();
        String srcName = id.getSourceName();
        
        for (int i = 0; i < els.length; i++) {
            if (els[i].getName().getSourceName().equals(srcName)) {
                if (srcName.equals(id.getFullName()) ||
                    els[i].getName().getFullName().equals(id.getFullName()))
                    return els[i];
                else
                    return null;
            }
        }
        return null;
    }

    public ElementImpl createElement(Element parent) {
        ClassElementImpl cimpl = creator.createTopClass((SourceElement)parent);
        return cimpl;
    }
    
    public Element[] createEmpty(int size) {
        return size == 0 ? InnerClassCollection.EMPTY : new ClassElement[size];
    }
    
    public Element[] findPositions(Element[] elements) {
        return positioner.findInsertPositions(
            (Element)events.getEventSource(), elements, this);
    }
    
    public boolean canInsertAfter(Element el) {
        ElementImpl impl;
        
        // special case -- element is just being created.
        if (events.getElementImpl().isCreated())
            return true;
        
        if (el != null && el != Positioner.FIRST) {
            impl =  (ElementImpl)el.getCookie(ElementImpl.class);
            if (impl == null) {
                throw new IllegalArgumentException("Unsupported element implementation");
            }
        } else
            impl = null;
        return containerBinding.canInsertAfter(impl == null ? null : impl.getBinding());
    }
}
