/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.parser;

import org.openide.src.*;

import org.netbeans.modules.java.model.ElementImpl;
import org.netbeans.modules.java.model.LangModel;

/**
 *
 * @author  svata
 * @version  
 */
public class MethodInfo extends BaseElementInfo {
    Type                retType;
    MethodParameter[]   params;
    Identifier[]        exceptions;
    boolean             constructor;
    
    public MethodInfo(String name, int mods) {
        super(name, mods);
    }
    
    public Element createModelImpl(LangModel.Updater model, Element parent) {
        if (constructor) {
            return ((ElementImpl)model.createConstructor((ClassElement)parent)).getElement();
        } else {
            return ((ElementImpl)model.createMethod((ClassElement)parent)).getElement();
        }
    }

    public void updateElement(LangModel.Updater model, Element target) 
    throws SourceException {
        super.updateElement(model, target);
        super.updateBase(target);

        ConstructorElement c = (ConstructorElement)target;
        
        ClassElement decl = c.getDeclaringClass();
        c.setParameters(mapParameters(model, c));
        c.setExceptions(mapIdentifiers(model, exceptions, decl));
        updateJavaDoc(c.getJavaDoc());
        model.updateBody(c, bodyContent);
        if (!this.constructor) {
            ((MethodElement)c).setReturn(createContextType(model, decl, retType));
        }
    }
    
    private MethodParameter[] mapParameters(LangModel.Updater model, ConstructorElement t) {
        if (params.length == 0)
            return params;
        MethodParameter[] pars = new MethodParameter[params.length];
        Element clazz = t.getDeclaringClass();
        for (int i = 0; i < params.length; i++) {
            pars[i] = new MethodParameter(
                params[i].getName(),
                createContextType(model, clazz, params[i].getType()),
                params[i].isFinal());
        }
        return pars;
    }
    
    public static class Finder extends BaseElementInfo.NameFinder {
        private final boolean compareName;
        private final boolean compareTypes;
        
        public Finder(boolean names, boolean types) {
            compareName = names;
            compareTypes = types;
        }
        
        protected boolean matches(BaseElementInfo baseInfo, Element old) {
            MethodInfo info = (MethodInfo)baseInfo;
            ConstructorElement oldElem = (ConstructorElement)old;
            
            if (compareName && !super.matches(baseInfo, old))
                return false;
            if (compareTypes && !compareTypes(info, oldElem))
                return false;
            return true;
        }
        
        private boolean compareTypes(MethodInfo info, ConstructorElement old) {
            MethodParameter[] params1 = info.params;
            MethodParameter[] params2 = old.getParameters();
            
            if (params1.length != params2.length)
                return false;
            for (int i = 0; i < params1.length; i++) {
                if (!compareTypes(params1[i].getType(), params2[i].getType()))
                    return false;
            }
            return true;
        }
        
        private boolean compareTypes(Type t1, Type t2) {
            return t1.getSourceString().equals(t2.getSourceString());
        }
    }
}
