/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.parser;

import java.util.*;

import org.openide.src.*;
import org.netbeans.modules.java.model.ImportElement;

import org.openide.text.PositionBounds;

import org.netbeans.modules.java.model.ElementImpl;
import org.netbeans.modules.java.model.LangModel;
import org.netbeans.modules.java.model.Binding;
import org.netbeans.modules.java.model.SourceElementImpl;

import org.netbeans.modules.java.codegen.DocumentBinding;
import org.netbeans.modules.java.codegen.TextBinding;

/**
 *
 * @author  svata
 * @version 
 */
public class SourceInfo extends BaseElementInfo implements DocumentModelUpdater {
    ChildCollection     imports;
    Identifier          packageDecl;
    PositionBounds      packageBounds;
    ChildCollection     classes;
    SourceElement.Impl  sourceImpl;
    LangModel.Updater   updater;
    
    Element[]           newImports, newClasses;
    
    private static final ImportElement[] NO_IMPORTS = new ImportElement[0];
    private static final ClassElement[] NO_CLASSES = new ClassElement[0];
    
    private static final ElementMatch.Finder[] CLASS_FINDERS = 
        ClassInfo.DEFAULT_CLASS_FINDERS;
    
    private static final ElementMatch.Finder[] IMPORT_FINDERS = {
        //new TextPositionMatch(),
        new ImportFinder()
    };
    
    SourceInfo() {
        super(null, 0);
    }
    
    public Element createModelImpl(LangModel.Updater model, Element parent) {
        return null;
    }

    public void addClass(ClassInfo cls) {
        if (classes == null) {
            classes = new ChildCollection(CLASS_FINDERS, ClassElement.class);
        }
        classes.addChild(cls, -1);
    }
    
    public void addImport(ImportInfo info) {
        if (imports == null)
            imports = new ChildCollection(IMPORT_FINDERS, 
                org.netbeans.modules.java.model.ImportElement.class);
        imports.addChild(info, -1);
    }
    
    public void updateBinding(DocumentBinding updater, TextBinding target) {
        Binding.Source sb = (Binding.Source)target;
        
        target.updateBounds(TextBinding.BOUNDS_PACKAGE, 
            packageDecl == null ? null : packageBounds);
        if (classes != null) {
            updateChildren(updater, classes.getChildren(), 
                (TextBinding.Container)sb.getClassSection());
        }
        if (imports != null) {
            updateChildren(updater, imports.getChildren(), 
                (TextBinding.Container)sb.getImportsSection());
        }
    }
    
    public void updateModel(LangModel.Updater updater, SourceElement src, 
        SourceElement.Impl impl) throws SourceException {
        updateElement(updater, src);
        this.sourceImpl = impl;
        
        impl.setPackage(packageDecl);
        
        int[] importMap = null, classMap = null;
        if (imports == null)
            newImports = NO_IMPORTS;
        else {
            newImports = (Element[])imports.updateChildren(src, updater, 
                ((SourceElementImpl)impl).getImportElements());
            importMap = imports.getResultMap();
        }
        if (classes == null)
            newClasses = NO_CLASSES;
        else {
            newClasses = (Element[])classes.updateChildren(src, updater, impl.getClasses());
            classMap = classes.getResultMap();
        }
        Util.log("Updating imports");
        updater.updateMembers(src, ElementProperties.PROP_IMPORTS, newImports, null,
            importMap);
        Util.log("Updating toplevel classes");
        updater.updateMembers(src, ElementProperties.PROP_CLASSES, newClasses, null,
            classMap);
    }

    private static final class ImportFinder extends ElementMatch.AbstractFinder {
        protected boolean matches(BaseElementInfo info, Element el) {
            ImportInfo iinfo = (ImportInfo)info;
            ImportElement ie = (ImportElement)el;
            
            return ie.getImport().equals(iinfo.statement);
        }
    }
}
