/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.settings;

import java.awt.Image;
import java.awt.Toolkit;
import java.beans.SimpleBeanInfo;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;
import java.beans.BeanDescriptor;

/**
*
* @author Ales Novak
*/
public class JavaSettingsBeanInfo extends SimpleBeanInfo {
    /** Array of property descriptors. */
    private static PropertyDescriptor[] desc;

    // initialization of the array of descriptors
    static {
        try {
            desc = new PropertyDescriptor[] {
                       new PropertyDescriptor (JavaSettings.PROP_REPLACEABLE_STRINGS_TABLE, JavaSettings.class),
                       new PropertyDescriptor (JavaSettings.PROP_AUTO_PARSING_DELAY, JavaSettings.class),
                       new PropertyDescriptor (JavaSettings.PROP_COMPILER, JavaSettings.class),
                       new PropertyDescriptor (JavaSettings.PROP_DEBUGGER, JavaSettings.class),
                       new PropertyDescriptor (JavaSettings.PROP_EXECUTOR, JavaSettings.class),
                       new PropertyDescriptor (JavaSettings.PROP_PRESCAN_SOURCES, JavaSettings.class),
                       new PropertyDescriptor (JavaSettings.PROP_SHOW_COMPILE_STATUS, JavaSettings.class, "isCompileStatusEnabled", "enableCompileStatus"),
                       new PropertyDescriptor (JavaSettings.PROP_PARSER_BOOTPATH, JavaSettings.class),
                       new PropertyDescriptor (JavaSettings.PROP_PARSER_CLASSPATH, JavaSettings.class),
                       new PropertyDescriptor (JavaSettings.PROP_SHOW_TOOLBAR, JavaSettings.class),
                   };

            desc[0].setDisplayName (JavaSettings.getString("PROP_REPLACEABLE_STRINGS"));
            desc[0].setShortDescription (JavaSettings.getString("HINT_REPLACEABLE_STRINGS"));
            desc[1].setDisplayName (JavaSettings.getString("PROP_AUTO_PARSING_DELAY"));
            desc[1].setShortDescription (JavaSettings.getString("HINT_AUTO_PARSING_DELAY"));
            desc[2].setDisplayName (JavaSettings.getString("PROP_COMPILER"));
            desc[2].setShortDescription (JavaSettings.getString("HINT_COMPILER"));
            desc[3].setDisplayName (JavaSettings.getString("PROP_DEBUGGER"));
            desc[3].setShortDescription (JavaSettings.getString("HINT_DEBUGGER"));
            desc[4].setDisplayName (JavaSettings.getString("PROP_EXECUTOR"));
            desc[4].setShortDescription (JavaSettings.getString("HINT_EXECUTOR"));
            desc[5].setDisplayName(JavaSettings.getString("PROP_PRESCAN_SOURCES"));
            desc[5].setShortDescription(JavaSettings.getString("HINT_PRESCAN_SOURCES"));
            desc[6].setDisplayName(JavaSettings.getString("PROP_SHOW_COMPILE_STATUS"));
            desc[6].setShortDescription(JavaSettings.getString("HINT_SHOW_COMPILE_STATUS"));
            desc[7].setExpert(true);
            desc[7].setDisplayName(JavaSettings.getString("PROP_PARSER_BOOTPATH"));
            desc[7].setShortDescription(JavaSettings.getString("HINT_PARSER_BOOTPATH"));
            desc[8].setExpert(true);
            desc[8].setDisplayName(JavaSettings.getString("PROP_PARSER_CLASSPATH"));
            desc[8].setShortDescription(JavaSettings.getString("HINT_PARSER_CLASSPATH"));
            desc[9].setDisplayName(JavaSettings.getString("PROP_SHOW_TOOLBAR"));
            desc[9].setShortDescription(JavaSettings.getString("HINT_SHOW_TOOLBAR"));
        } catch (IntrospectionException ex) {
            throw new InternalError ();
        }
    }

    public BeanDescriptor getBeanDescriptor() {
        BeanDescriptor desc = new BeanDescriptor(JavaSettings.class);
        desc.setDisplayName(JavaSettings.getString("CTL_Java_option"));
        /* for POST-FCS desc.setShortDescription(JavaSettings.getString("HINT_Java_option")); */
        return desc;
    }


    /** Descriptor of valid properties
    * @return array of properties
    */
    public PropertyDescriptor[] getPropertyDescriptors () {
        return desc;
    }

    /**
    * Claim there are no icons available.  You can override
    * this if you want to provide icons for your bean.
    */
    public Image getIcon(int type) {
        if ((type == java.beans.BeanInfo.ICON_COLOR_16x16) || (type == java.beans.BeanInfo.ICON_MONO_16x16)) {
            return org.openide.util.Utilities.loadImage("/org/netbeans/modules/java/resources/class.gif");
        } else { // 32
            return org.openide.util.Utilities.loadImage("/org/netbeans/modules/java/resources/class32.gif");
        }
    }
}
