/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.tools;

import java.text.MessageFormat;
import java.util.*;

import org.openide.*;
import org.openide.nodes.*;
import org.openide.util.actions.CookieAction;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;

import org.openide.src.*;
import org.openide.src.nodes.SourceOptions;

/** This action allows the user to override some of the methods present in the
 * selected class.
 * The action displays a modal dialog with several lists, then overrides
 * one or more methods the user selects.
 */
public class OverrideAction extends CookieAction {
  private static ResourceBundle bundle;
  
  static ResourceBundle getBundle() {
      if (bundle != null)
          return bundle;
      synchronized (OverrideAction.class) {
          if (bundle == null)
	      bundle = NbBundle.getBundle(OverrideAction.class);
      }
      return bundle;
  }

  public HelpCtx getHelpCtx() {
    return null;
  }
  
  public String getName() {
    return getBundle().getString("LAB_OverrideTool");
  }
  
  /** Only allow overriding in one class at a time.
  */
  public int mode() {
    return MODE_ONE;
  }
  
  /** This action is only valid for classes
   */
  public Class[] cookieClasses() {
    return new Class[] { ClassElement.class };
  }
   
  public void performAction(Node[] activatedNodes) {
    ClassElement el;
    
    el = (ClassElement)activatedNodes[0].getCookie(Element.class);
    InheritanceSupport support = new InheritanceSupport();
    
    support.setSourceClass(el);
    OverrideActionPanel panel = new OverrideActionPanel(support);
 
    String title = MessageFormat.format(getBundle().getString("LAB_OverrideDialogName"),
      new Object[] { el.getName().getFullName() } );

    DialogDescriptor desc = new DialogDescriptor(panel, title, true,  null);
    desc.setHelpCtx(new org.openide.util.HelpCtx("java.override")); // NOI18N
    TopManager.getDefault().createDialog(desc).show();
    if (desc.getValue().equals(DialogDescriptor.OK_OPTION)) {
      try {
          Collection methodsAdd = panel.getMethodsToAdd();
          Collection methodsRemove = panel.getMethodsToRemove();
          Iterator it;
          
          // add only -- for now:
          if (methodsAdd != null) {
              for (it = methodsAdd.iterator(); it.hasNext(); ) {
                  MethodElement m = (MethodElement)it.next();
                  m = support.overrideMethod(m, panel.isGenerateSuperCall());
              }
          }
      } catch (SourceException ex) {
          TopManager.getDefault().notifyException(ex);
      }
    }
  }
  
  static String getString(String key) {
      return getBundle().getString(key);
  }
}