/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.tools;

import java.util.*;

import org.openide.src.*;
/**
 *
 */
public class OverrideActionPanel extends org.netbeans.modules.java.tools.OverridePanelBase {
    /** Methods that should be removed from the class.
     */
    private Map removeMethods;
    
    /** Methods that should be added to the class.
     */
    private Map addMethods;
    
    /** Creates new OverrideActionPanel */
    public OverrideActionPanel(InheritanceSupport support) {
        super(support);
        initializeGUI();
    }
    
    /**
     * Checks whether the method in question is already in the customized class.
     */
    private boolean isInSource(MethodElement method) {
	ClassElement target = getSupport().getSourceClass();
	MethodParameter[] mp = method.getParameters();
	Type[] paramTypes = new Type[mp.length];
	for (int i = 0; i < mp.length; i++) {
	    paramTypes[i] = mp[i].getType();
	}
	return target.getMethod(method.getName(), paramTypes) != null;
    }
    
    protected void overrideMethod(MethodElement source) throws SourceException {
        int i;

        MethodElement.Key k = new MethodElement.Key(source);

        if (removeMethods != null && removeMethods.containsKey(k)) {
            removeMethods.remove(k);
        } else {
	    if (isInSource(source)) {
		throwAddException(source);
	    }
            if (addMethods == null) {
                addMethods = new HashMap(17);
            } else if (addMethods.containsKey(k)) {
		throwAddException(source);
            }
            addMethods.put(k, source);
        }
        // actually display the thing:
        super.overrideMethod(source);
    }
    
    private void throwAddException(MethodElement m) throws SourceException {
      String msg;
                  msg = java.text.MessageFormat.format(OverrideAction.getString("FMT_METHOD_PRESENT"),
                    new Object[] {
                	m.getName().getName()
                    });
                throw new SourceException(msg);
    }	
    
    public Collection getMethodsToAdd() {
        if (addMethods == null)
            return null;
        return addMethods.values();
    }
    
    public Collection getMethodsToRemove() {
        if (removeMethods == null)
            return null;
        return removeMethods.values();
    }
    
    protected void removeMethod(MethodElement m) throws SourceException {
        MethodElement.Key k = new MethodElement.Key(m);
        
        if (addMethods == null || !addMethods.containsKey(k)) {
            if (removeMethods == null)
                removeMethods = new HashMap(17);
            removeMethods.put(k, m);
        } else {
            addMethods.remove(k);
        }
        super.removeMethod(m);
    }
}