/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.java.tools;

import java.util.*;

import java.awt.BorderLayout;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Node;
import org.openide.nodes.Children;
import org.openide.explorer.view.ListView;
import org.openide.explorer.ExplorerManager;

import org.openide.src.ClassElement;

public class SuperclassList extends javax.swing.JPanel implements ExplorerManager.Provider {
    /** Object that collects class inheritance data.
     */
    private InheritanceSupport data;

    /** Explorer manager provided to the list view.
     */
    private ExplorerManager manager;
    
    /** List view that contains superclass/superinterface list
     */
    private ListView list;
    
    /** Comparator for proper ordering of displayed elements
     */
    private Comparator comparator;

    /** Creates new form SuperclassList */
    public SuperclassList() {
        initComponents ();
        list = new ListView();
        list.setPopupAllowed(false);
        list.setDragSource(false);
        list.setDropTarget(false);
        comparator = new Comparator();
        add(list, BorderLayout.CENTER);
    }

    public ExplorerManager getExplorerManager() {
        if (manager == null) {
            manager = new ExplorerManager();
        }
        return manager;
    }

    public void setData(InheritanceSupport data) {
        this.data = data;
    }
    
    public void addNotify() {
        super.addNotify();
        Children c = new InheritanceChildren(data, comparator);
        ExplorerManager m = getExplorerManager();
        m.setRootContext(new AbstractNode(c));
    }

    public Collection getSelectedClasses() {
        Node[] n = getExplorerManager().getSelectedNodes();
        if (n == null || n.length == 0) {
            return new LinkedList();
        }
        Collection c = new ArrayList(n.length);
        for (int i = 0; i < n.length; i++) {
            c.add(n[i].getCookie(ClassElement.class));
        }
        return c;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        setLayout(new java.awt.BorderLayout());
    }//GEN-END:initComponents

// Variables declaration - do not modify//GEN-BEGIN:variables
// End of variables declaration//GEN-END:variables

    private static class Comparator implements java.util.Comparator {
        public int compare(Object value1, Object value2) {
            ClassElement v1 = (ClassElement)value1;
            ClassElement v2 = (ClassElement)value2;
            
            if (v1.isClassOrInterface() != v2.isClassOrInterface()) {
                return v1.isClassOrInterface() ? -1 : 1;
            }
            return v1.getName().getFullName().compareTo(v2.getName().getFullName());
        }
        
        public boolean equals(final java.lang.Object p1) {
            return p1 instanceof Comparator;
        }
    }
}
