/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.ui.wizard;

import java.lang.reflect.Modifier;

/**
 *
 * @author jrojcek
 */
public class AccessLevelPanel extends javax.swing.JPanel {

    public static final String PROP_ACCESS_LEVEL = "accessLevel"; // NOI18N
    
    /** Creates new form AccessLevelPanel
    */
    public AccessLevelPanel(int allowed, int mod) {
        initComponents();
        setAllowedAccessLevel(allowed);
        setAccessLevel(mod);
        accessLabel.setDisplayedMnemonic(JavaWizardIterator.getString("LBL_Access_Level_Mnemonic").charAt(0));
    }

    public void setAllowedAccessLevel(int mod) {
        accessCombo.addItem("<default>"); // NOI18N
	if (Modifier.isPublic(mod)) accessCombo.addItem("public"); // NOI18N
	if (Modifier.isPrivate(mod)) accessCombo.addItem("private"); // NOI18N
	if (Modifier.isProtected(mod)) accessCombo.addItem("protected"); // NOI18N
    }

    public void setAccessLevel(int mod) {
        String item = Modifier.toString(mod);
        accessCombo.setSelectedItem("".equals(item) ? "<default>" : item); // NOI18N
    }
    
    public int getAccessLevel() {
        return toBin(accessCombo.getSelectedItem());
    }
    
    private int toBin(Object mod) {
        if ("public".equals(mod)) return Modifier.PUBLIC; // NOI18N
        if ("private".equals(mod)) return Modifier.PRIVATE; // NOI18N
        if ("protected".equals(mod)) return Modifier.PROTECTED; // NOI18N
        return 0;
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        accessLabel = new javax.swing.JLabel();
        accessCombo = new javax.swing.JComboBox();
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        accessLabel.setText(JavaWizardIterator.getString("LBL_Access_Level"));
        accessLabel.setLabelFor(accessCombo);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 2, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(accessLabel, gridBagConstraints1);
        
        
        accessCombo.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                changeAccessLevel(evt);
            }
        }
        );
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.weightx = 1.0;
        add(accessCombo, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void changeAccessLevel(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_changeAccessLevel
        super.firePropertyChange(PROP_ACCESS_LEVEL, null, null);
    }//GEN-LAST:event_changeAccessLevel
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel accessLabel;
    private javax.swing.JComboBox accessCombo;
    // End of variables declaration//GEN-END:variables

}
