/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.ui.wizard;

import java.beans.PropertyEditor;
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.text.MessageFormat;
import java.lang.reflect.Modifier;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import org.openide.src.*;
import org.openide.explorer.propertysheet.*;
import org.openide.util.Utilities;
import org.openide.ErrorManager;
import org.openide.NotifyDescriptor;
import org.openide.TopManager;

import org.openide.loaders.TemplateWizard;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataFolder;
import org.openide.WizardDescriptor;

/**
 *
 * @author  jrojcek
 */
public class ClassCustomizer extends javax.swing.JPanel implements PropertyChangeListener {

    /** Class element whose properties are being customized. This member is updated
     * from JavaWizardData every time the panel is displayed.
     */
    private ClassElement clazz;

    /** Predefined types in the type combo */
    private static final String[] COMMON_TYPES = {
        "java.lang.Object", // NOI18N
        "java.awt.Component", // NOI18N
        "javax.swing.JComponent", // NOI18N
        "javax.swing.JPanel" // NOI18N
    };

    /** This is set to true if the user has been warned about changing superclass
     * specified in the template.
     * The warning is not issued on subsequent changes.
     */
    private boolean superclassChangeWarning = false;
    
    public ClassCustomizer(ClassElement clazz) {
        initComponents();

        interfacesPanel = new PropertyPanel (
            new ClassElement(),
            ElementProperties.PROP_INTERFACES,
            PropertyPanel.PREF_INPUT_STATE
        );

        java.awt.GridBagConstraints gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 9;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 0, 11);
//        gridBagConstraints1.weightx = 1.0;
        add(interfacesPanel, gridBagConstraints1);

        nameLabel.setDisplayedMnemonic(JavaWizardIterator.getString("LBL_Name_Mnemonic").charAt(0));
        packageLabel.setDisplayedMnemonic(JavaWizardIterator.getString("LBL_Package_Mnemonic").charAt(0));
        superClassLabel.setDisplayedMnemonic(JavaWizardIterator.getString("LBL_Superclass_Mnemonic").charAt(0));
        classDeclarationLabel.setDisplayedMnemonic(JavaWizardIterator.getString("LBL_Class_Declaration_Mnemonic").charAt(0));
        interfacesLabel.setDisplayedMnemonic(JavaWizardIterator.getString("LBL_Interfaces_Implemented_Mnemonic").charAt(0));
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        nameField = new javax.swing.JTextField();
        packageField = new javax.swing.JTextField();
        classDeclarationField = new javax.swing.JTextField();
        nameLabel = new javax.swing.JLabel();
        packageLabel = new javax.swing.JLabel();
        superClassLabel = new javax.swing.JLabel();
        classDeclarationLabel = new javax.swing.JLabel();
        interfacesLabel = new javax.swing.JLabel();
        superclassCombo = new javax.swing.JComboBox(COMMON_TYPES);
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        setName(JavaWizardIterator.getString("TIT_InheritanceWizardPanel"));
        nameField.setEditable(false);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 11, 11);
        add(nameField, gridBagConstraints1);
        
        packageField.setEditable(false);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 11, 0);
        add(packageField, gridBagConstraints1);
        
        classDeclarationField.setEditable(false);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.gridwidth = 3;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 11, 0);
        gridBagConstraints1.weightx = 1.0;
        add(classDeclarationField, gridBagConstraints1);
        
        nameLabel.setText(JavaWizardIterator.getString("LBL_Name"));
        nameLabel.setLabelFor(nameField);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 2, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(nameLabel, gridBagConstraints1);
        
        packageLabel.setText(JavaWizardIterator.getString("LBL_Package"));
        packageLabel.setLabelFor(packageField);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 2, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(packageLabel, gridBagConstraints1);
        
        superClassLabel.setText(JavaWizardIterator.getString("LBL_Superclass"));
        superClassLabel.setLabelFor(superclassCombo);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 5;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 2, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(superClassLabel, gridBagConstraints1);
        
        classDeclarationLabel.setText(JavaWizardIterator.getString("LBL_Class_Declaration"));
        classDeclarationLabel.setLabelFor(classDeclarationField);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 2, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(classDeclarationLabel, gridBagConstraints1);
        
        interfacesLabel.setText(JavaWizardIterator.getString("LBL_Interfaces_Implemented"));
        interfacesLabel.setLabelFor(interfacesPanel);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 7;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 2, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(interfacesLabel, gridBagConstraints1);
        
        superclassCombo.setEditable(true);
        superclassCombo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                superclassComboActionPerformed(evt);
            }
        });
        
        superclassCombo.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                superclassComboFocusLost(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 6;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 11, 11);
        gridBagConstraints1.weightx = 1.0;
        add(superclassCombo, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void superclassComboFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_superclassComboFocusLost
        updateSuperclass();
    }//GEN-LAST:event_superclassComboFocusLost

    private void superclassComboActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_superclassComboActionPerformed
        updateSuperclass();
    }//GEN-LAST:event_superclassComboActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextField nameField;
    private javax.swing.JTextField packageField;
    private javax.swing.JTextField classDeclarationField;
    private javax.swing.JLabel nameLabel;
    private javax.swing.JLabel packageLabel;
    private javax.swing.JLabel superClassLabel;
    private javax.swing.JLabel classDeclarationLabel;
    private javax.swing.JLabel interfacesLabel;
    private javax.swing.JComboBox superclassCombo;
    // End of variables declaration//GEN-END:variables
    private AccessLevelPanel accessLevelPanel;
    private ClassTypePanel classTypePanel;
    private PropertyPanel interfacesPanel;
    
    private void updateSuperclass() {
        // in case that the template originally specified an ancestor (not Object) for the class,
        // warn the user once.
        Identifier superid = clazz.getSuperclass();
        String oldValueStr = superid == null ? "" : superid.getFullName();
        Object selItem = superclassCombo.getSelectedItem();
        
        if (selItem == null) {
            superclassCombo.setSelectedItem(oldValueStr);
            return;
        }
        
        String newValueStr = selItem.toString();
        
        if (superclassChangeWarning == false) {
            // test if there's a superclass other than the Object.
            if (superid != null && ClassElement.ROOT_OBJECT.equals(superid)) {
                if (warnSuperclassChange()) {
                    // revert the changes back...
                    superclassCombo.setSelectedItem(superid);
                    return;
                }
            }
            superclassChangeWarning = true;
        }
        
        boolean ok = false;
        try {
            if (!oldValueStr.equals(newValueStr)) {
                try {
                    Identifier id;
                    if (newValueStr.equals("")) { //NOI18N
                        id = null;
                    } else {
                        id = Identifier.create(newValueStr);
                    }
                    clazz.setSuperclass(id);
                    ok = true;
                }
                catch (SourceException e) {
                    TopManager.getDefault().notifyException(e);
                }
            }
        }
        catch (IllegalArgumentException e) {
            TopManager.getDefault().notify(
            new NotifyDescriptor.Message(
            JavaWizardIterator.getString("ERR_InvalidClassIdentifier"),
            NotifyDescriptor.ERROR_MESSAGE
            )
            );
        }
        if (!ok) {
            superclassCombo.setSelectedItem(oldValueStr);
        }
    }
    
    private void updateClassDeclaration() {
        String sumText = Modifier.toString(clazz.getModifiers());
        if (sumText.length() > 0) 
            sumText += " "; // NOI18N
        if (clazz.isClass())
            sumText += "class "; // NOI18N
        else
            sumText += "interface "; // NOI18N
        sumText += clazz.getName();
        if (clazz.getSuperclass() != null)
            sumText += " extends " + clazz.getSuperclass().getName(); // NOI18N
        if (clazz.getInterfaces().length > 0) {
            sumText += (clazz.isClass()) ? " implements " : " extends "; // NOI18N
            sumText += interfacesPanel.getPropertyEditor().getAsText();
        }
        classDeclarationField.setText(sumText);
    }

    /** This method warns that the superclass is being changed. It should prompt
     * the user with a question if he really wants to do it since it might break
     * compilability of the Template.
     * @return true, if the action was cancelled. False if user confirmed.
     */
    private boolean warnSuperclassChange() {
        return false;
    }

    public void propertyChange(PropertyChangeEvent evt) {
        String propName = evt.getPropertyName();
        if ((ElementProperties.PROP_NAME.equals(propName))
            || (ElementProperties.PROP_SUPERCLASS.equals(propName))
            || (ElementProperties.PROP_MODIFIERS.equals(propName))
            || (ElementProperties.PROP_INTERFACES.equals(propName))) {
            updateClassDeclaration();
        }
    }
    
    public void addNotify() {
        super.addNotify();
        if (clazz != null)
            clazz.addPropertyChangeListener(this);
    }

    public void removeNotify() {
        super.removeNotify();
        if (clazz != null)
            clazz.removePropertyChangeListener(this);
    }
    
    /////////////////////////////////////////////////////////////////////////////
    public org.openide.util.HelpCtx getHelp() {
        return null;
    }
    
    /** Sets the new class element for all interacting property editors and
     * panels.
     */
    public void setObject(ClassElement clazz) {
        if (this.clazz == clazz)
            return;
        if (this.clazz != null)
            this.clazz.removePropertyChangeListener(this);
        this.clazz = clazz;
        if (this.clazz != null) {
            this.clazz.addPropertyChangeListener(this);
            updateClassDeclaration();
        }

        String qualifier = clazz.getName().isQualified() ?
        clazz.getName().getQualifier() : JavaWizardIterator.getString("TXT_DefaultPackageName");

        packageField.setText(qualifier);
        nameField.setText(clazz.getName().getSourceName());

        superclassCombo.setEnabled(clazz.isClass());
        String superName = (clazz.isInterface() || clazz.getSuperclass() == null) ? "" : clazz.getSuperclass().getFullName();
        superclassCombo.setSelectedItem(superName);

        interfacesPanel.setModel(new DefaultPropertyModel(
            clazz,
            ElementProperties.PROP_INTERFACES
        ));
        interfacesPanel.getPropertyEditor().setValue(clazz.getInterfaces());

        if (accessLevelPanel != null)
            remove(accessLevelPanel);
        if (classTypePanel != null)
            remove(classTypePanel);
        
        PropertyPanel modifPanel = new PropertyPanel(
            new TempPropertyModel(
                clazz,
                ElementProperties.PROP_MODIFIERS,
                clazz.isInterface()
            ),
            PropertyPanel.PREF_CUSTOM_EDITOR
        );

        PropertyEditor propEdit = modifPanel.getPropertyEditor();
        if (propEdit instanceof ModifierEditor) {
            accessLevelPanel = ((ModifierEditor)propEdit).getAccessLevelPanel();
            classTypePanel = (ClassTypePanel)((ModifierEditor)propEdit).getModifierPanel();
        }

        java.awt.GridBagConstraints gridBagConstraints1;
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 2;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.gridheight = 5;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.VERTICAL;
        add(classTypePanel, gridBagConstraints1);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 11, 11);
        gridBagConstraints1.weightx = 1.0;
        add(accessLevelPanel, gridBagConstraints1);
    }

    /////////////////////////////////////////////////////////////////////////////

    static class TempPropertyModel extends DefaultPropertyModel {
    
        boolean inter;
        
        public TempPropertyModel(Object bean, String propertyName, boolean inter) throws IllegalArgumentException {
            super(bean, propertyName);
            this.inter = inter;
        }
        /** The class of the property editor or <CODE>null</CODE>
         * if default property editor should be used.
         */
        public Class getPropertyEditorClass() {
            return inter ? InterfaceModifierEditor.class : ClassModifierEditor.class;
        }
    }
    
    public static class ClassModifierEditor extends ModifierEditor {
        public ClassModifierEditor() {
            super(new AccessLevelPanel(Modifier.PUBLIC, 0),
                  new ClassTypePanel(false));
        }
    }

    public static class InterfaceModifierEditor extends ModifierEditor {
        public InterfaceModifierEditor() {
            super(new AccessLevelPanel(Modifier.PUBLIC, 0),
                  new ClassTypePanel(true));
        }
    }
}
