/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.ui.wizard;

import java.beans.PropertyEditor;
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.text.MessageFormat;
import java.lang.reflect.Modifier;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import org.openide.src.*;
import org.openide.explorer.propertysheet.*;
import org.openide.util.Utilities;
import org.openide.ErrorManager;
import org.openide.NotifyDescriptor;
import org.openide.TopManager;

import org.openide.loaders.TemplateWizard;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataFolder;
import org.openide.WizardDescriptor;

/**
 *
 * @author  jrojcek
 */
public class ClassPanel extends javax.swing.JPanel implements JavaPanel {

    /** Reference to the data holder.
     */
    private JavaWizardData wizardData;
    
    /** Class customizer which interacts with user
     */
    private ClassCustomizer classCustomizer;
    
    public void initialize(JavaWizardData data) {
        wizardData = data;
        putClientProperty("WizardPanel_contentSelectedIndex", new Integer(1));
        initComponents();
        classCustomizer = new ClassCustomizer(wizardData.getCustomizedClass());
        java.awt.GridBagConstraints gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(classCustomizer, gridBagConstraints1);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        headLabel = new javax.swing.JLabel();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        setName(JavaWizardIterator.getString("TIT_ImplementInterface"));
        headLabel.setText(JavaWizardIterator.getString("LBL_Basic_class_characteristics"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 12, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        add(headLabel, gridBagConstraints1);
        
    }//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel headLabel;
    // End of variables declaration//GEN-END:variables

    public void addNotify() {
        super.addNotify();
        classCustomizer.setObject(wizardData.getCustomizedClass());
    }
    
    /////////////////////////////////////////////////////////////////////////////
    public org.openide.util.HelpCtx getHelp() {
        return org.openide.util.HelpCtx.DEFAULT_HELP;
    }
    
    public void readSettings(Object settings) {
      if (settings instanceof TemplateWizard) {
          TemplateWizard wizardInstance = (TemplateWizard)settings;
          
          DataObject dobj = null;
          DataFolder target = null;
          
          try {
              target = wizardInstance.getTargetFolder();
              dobj = wizardInstance.getTemplate();
          } catch (java.io.IOException ex) {
          }
          
          String name = wizardInstance.getTargetName();
          if (dobj == null || target == null) {
              throw new IllegalStateException("Expected target folder & template, got null(s)"); // NOI18N
          }
          // checking for java-compatible name - both the folder name and the target name
          // must be acceptable.
          String fullTarget = target.getPrimaryFile().getPackageNameExt('.', '.');
          if (!Util.isValidPackageName(fullTarget)) {
              String msg = MessageFormat.format(JavaWizardIterator.getString("FMTERR_IllegalFolderName"),
              new Object[] {
                  target.getPrimaryFile().getPackageNameExt('/', '.'),
                  fullTarget
              });
              throw (IllegalArgumentException)TopManager.getDefault().getErrorManager().annotate(
                new IllegalArgumentException("Illegal folder name"),
                ErrorManager.USER, null, msg,
                null, null
              );
          }
          if (name == null) {
              // make up an name using the Template's name
              name = org.openide.filesystems.FileUtil.findFreeFileName(target.getPrimaryFile(),
              dobj.getName(), "java");
          } else if (!Utilities.isJavaIdentifier(name)) {
              String msg = MessageFormat.format(JavaWizardIterator.getString("FMTERR_IllegalTargetName"),
              new Object[] {
                  name
              });
              throw (IllegalArgumentException)TopManager.getDefault().getErrorManager().annotate(
                new IllegalArgumentException("Illegal target name"),
                ErrorManager.USER, null, msg,
                null, null
              );
          }
          wizardData.setTemplate(dobj);
          wizardData.setTarget(target.getPrimaryFile(), name);
      }
    }
    
    public void storeSettings(Object settings) {
    }

    public boolean isValid() {
        return true;
    }
    
    public void addChangeListener(javax.swing.event.ChangeListener l) {
    }

    public void removeChangeListener(javax.swing.event.ChangeListener l) {
    }
    
    public java.awt.Component getComponent() {
        return this;
    }
    
    /////////////////////////////////////////////////////////////////////////////
}
