/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.ui.wizard;

import javax.swing.ButtonGroup;
import java.lang.reflect.Modifier;
import java.awt.Component;

/**
 *
 * @author  jrojcek
 */
public class ClassTypePanel extends javax.swing.JPanel 
                            implements ModifierEditor.ModifierPanel {

    boolean inter;
    /** Creates new form ClassModifiersPanel */
    public ClassTypePanel(boolean inter) {
        this.inter = inter;
        initComponents();
        setBorder(new javax.swing.border.TitledBorder(
            new javax.swing.border.EtchedBorder(), 
            org.openide.util.NbBundle.getBundle(MethodTypePanel.class).getString("LBL_ClassType")));
        
        ButtonGroup bg = new ButtonGroup();
        bg.add(regularRadioButton);
        bg.add(finalRadioButton);
        bg.add(abstractRadioButton);
        regularRadioButton.setSelected(true);

        regularRadioButton.setMnemonic('l');
        finalRadioButton.setMnemonic('i');
        abstractRadioButton.setMnemonic('a');
    }
    
    public void setTypeModifier(int mod) {
        if (Modifier.isFinal(mod)) finalRadioButton.setSelected(true);
        else if (Modifier.isAbstract(mod)) abstractRadioButton.setSelected(true);
        else regularRadioButton.setSelected(true);
    }
    
    public int getTypeModifier() {
        if (finalRadioButton.isSelected()) return Modifier.FINAL;
        if (abstractRadioButton.isSelected()) return Modifier.ABSTRACT;
        return 0;
    }
    
    public int getMask() {
        return Modifier.ABSTRACT | Modifier.FINAL;
    }
    
    public Component getComponent() {
        return this;
    }
    
    public void addNotify() {
        super.addNotify();
        finalRadioButton.setEnabled(! inter);
    }
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        regularRadioButton = new javax.swing.JRadioButton();
        finalRadioButton = new javax.swing.JRadioButton();
        abstractRadioButton = new javax.swing.JRadioButton();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        setBorder(new javax.swing.border.TitledBorder("Class Type"));
        regularRadioButton.setText("<default>");
        regularRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                changeClassType(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.insets = new java.awt.Insets(0, 12, 0, 8);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(regularRadioButton, gridBagConstraints1);
        
        finalRadioButton.setText("final");
        finalRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                changeClassType(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.insets = new java.awt.Insets(0, 12, 0, 8);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        add(finalRadioButton, gridBagConstraints1);
        
        abstractRadioButton.setText("abstract");
        abstractRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                changeClassType(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.insets = new java.awt.Insets(0, 12, 0, 8);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        add(abstractRadioButton, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void changeClassType(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_changeClassType
        super.firePropertyChange(PROP_TYPE_MODIFIER, null, null);
    }//GEN-LAST:event_changeClassType


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JRadioButton regularRadioButton;
    private javax.swing.JRadioButton finalRadioButton;
    private javax.swing.JRadioButton abstractRadioButton;
    // End of variables declaration//GEN-END:variables

}
