/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.ui.wizard;

import java.lang.reflect.Modifier;
import java.awt.Container;
import java.awt.Component;

import org.openide.explorer.*;
import org.openide.nodes.*;
import org.openide.src.*;
import org.openide.explorer.propertysheet.*;
import org.openide.util.Utilities;
import org.openide.NotifyDescriptor;
import org.openide.TopManager;

/**
 *
 * @author  jrojcek
 */
public class FieldCustomizer extends javax.swing.JPanel {

    private static final FieldElement EMPTY_ELEMENT;

    private FieldElement field;
    
    static {
        EMPTY_ELEMENT = new FieldElement();
        try {
            EMPTY_ELEMENT.setType(Type.INT);
        } catch (SourceException ex) {
            // never happens on memory representation.
        }
    }

    public FieldCustomizer(FieldElement field) {
        this.field = (field != null) ? field : EMPTY_ELEMENT;
        
        initComponents();

        nameField.setText(this.field.getName().toString());

        typePanel = new PropertyPanel (
            new TempPropertyModel(
                this.field,
                ElementProperties.PROP_TYPE,
                FieldTypeEditor.class),
            PropertyPanel.PREF_INPUT_STATE
        );

        java.awt.GridBagConstraints gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 11, 11);
        gridBagConstraints1.weightx = 1.0;
        add(typePanel, gridBagConstraints1);

        initPanel = new PropertyPanel (
            this.field,
            ElementProperties.PROP_INIT_VALUE,
            PropertyPanel.PREF_INPUT_STATE
        );

        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 5;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 0, 11);
        gridBagConstraints1.weightx = 1.0;
        add(initPanel, gridBagConstraints1);

        modifPanel = new PropertyPanel (
            new TempPropertyModel(
                this.field,
                ElementProperties.PROP_MODIFIERS,
                FieldModifierEditor.class),
            PropertyPanel.PREF_CUSTOM_EDITOR
        );
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.gridheight = 6;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.VERTICAL;
        add(modifPanel, gridBagConstraints1);
        
        initialLabel.setDisplayedMnemonic(JavaWizardIterator.getString("LBL_Initial_Value_Mnemonic").charAt(0));
        typeLabel.setDisplayedMnemonic(JavaWizardIterator.getString("LBL_Type_Mnemonic").charAt(0));
        nameLabel.setDisplayedMnemonic(JavaWizardIterator.getString("LBL_Name_Mnemonic").charAt(0));
    }

    /** This field is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this field is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        initialLabel = new javax.swing.JLabel();
        typeLabel = new javax.swing.JLabel();
        nameLabel = new javax.swing.JLabel();
        nameField = new javax.swing.JTextField();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        setName(JavaWizardIterator.getString("TIT_AddFieldsWizardPanel"));
        initialLabel.setText(JavaWizardIterator.getString("LBL_Initial_Value"));
        initialLabel.setLabelFor(initPanel);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 2, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(initialLabel, gridBagConstraints1);
        
        typeLabel.setText(JavaWizardIterator.getString("LBL_Type"));
        typeLabel.setLabelFor(typePanel);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 2, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(typeLabel, gridBagConstraints1);
        
        nameLabel.setText(JavaWizardIterator.getString("LBL_Name"));
        nameLabel.setLabelFor(nameField);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 2, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(nameLabel, gridBagConstraints1);
        
        nameField.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                nameFieldFocusLost(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 11, 11);
        gridBagConstraints1.weightx = 1.0;
        add(nameField, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void nameFieldFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_nameFieldFocusLost
        if (evt.isTemporary())
            return;

        String newName = nameField.getText();
        String oldName = field.getName().toString();
        boolean ok = false;
        if (Utilities.isJavaIdentifier(newName)) {
            if (!oldName.equals(newName)) {
                final Identifier id = Identifier.create(newName);
                try {
                    field.setName(id);
                    ok = true;
                }
                catch (SourceException e) {
                    TopManager.getDefault().notifyException(e);
                }
            }
        }
        else {
            TopManager.getDefault().notify(
                new NotifyDescriptor.Message(
                    JavaWizardIterator.getString("MSG_Not_Valid_Identifier"),
                    NotifyDescriptor.ERROR_MESSAGE
                )
            );
        }
        if (!ok)
            nameField.setText(oldName);
    }//GEN-LAST:event_nameFieldFocusLost


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel initialLabel;
    private javax.swing.JLabel typeLabel;
    private javax.swing.JLabel nameLabel;
    private javax.swing.JTextField nameField;
    // End of variables declaration//GEN-END:variables
    private PropertyPanel modifPanel;
    private PropertyPanel initPanel;
    private PropertyPanel typePanel;

    public void setEnabled(boolean enable) {
        nameField.setEnabled(enable);
        modifPanel.setEnabled(enable);
        initPanel.setEnabled(enable);
        typePanel.setEnabled(enable);
        super.setEnabled(enable);
    }
    
    /** Sets the new class element for all interacting property editors and
     * panels.
     */
    public void setObject(FieldElement field) {
        if (this.field == field)
            return;
        this.field = (field != null) ? field : EMPTY_ELEMENT;
        resetPanels();
        revalidate();
    }

    private void resetPanels() {
        nameField.setText(field.getName().toString());

        typePanel.setModel(new TempPropertyModel(
            this.field,
            ElementProperties.PROP_TYPE,
            FieldTypeEditor.class
        ));

        initPanel.setModel(new DefaultPropertyModel(
            this.field,
            ElementProperties.PROP_INIT_VALUE
        ));
        
        modifPanel.setModel(new TempPropertyModel(
            this.field,
            ElementProperties.PROP_MODIFIERS,
            FieldModifierEditor.class
        ));

    }

    public void addNotify() {
        super.addNotify();
//        if (field != null)
//            field.addPropertyChangeListener(this);
    }

    public void removeNotify() {
        super.removeNotify();
//        if (field != null)
//            field.removePropertyChangeListener(this);
    }
    /////////////////////////////////////////////////////////////////////////////
    
    public org.openide.util.HelpCtx getHelp() {
        return null;
    }
    
    public void readSettings(Object settings) {
    }
    
    public void storeSettings(Object settings) {
    }

    public boolean isValid() {
        return true;
    }
    
    public void addChangeListener(javax.swing.event.ChangeListener l) {
    }

    public void removeChangeListener(javax.swing.event.ChangeListener l) {
    }
    
    public java.awt.Component getComponent() {
        return this;
    }
    
    /////////////////////////////////////////////////////////////////////////////

    static class TempPropertyModel extends DefaultPropertyModel {
    
        Class editor;
        
        public TempPropertyModel(Object bean, String propertyName, Class editorClass) throws IllegalArgumentException {
            super(bean, propertyName);
            editor = editorClass;
        }
        /** The class of the property editor or <CODE>null</CODE>
         * if default property editor should be used.
         */
        public Class getPropertyEditorClass() {
            return editor;
        }
    }
    
    public static class FieldModifierEditor extends ModifierEditor {
        public FieldModifierEditor() {
            super(new AccessLevelPanel(Modifier.PUBLIC 
                                       | Modifier.PRIVATE 
                                       | Modifier.PROTECTED, 0),
                  new FieldTypePanel());
        }
    }
    
    public static class FieldTypeEditor extends org.openide.explorer.propertysheet.editors.TypeEditor {
        
        public String[] getTags () {
            java.util.ArrayList tags = new java.util.ArrayList(java.util.Arrays.asList(super.getTags()));
            tags.remove("void");
            return (String[])tags.toArray(new String[tags.size()]);
        }
        
    }
}
