/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.ui.wizard;

import org.openide.nodes.*;
import org.openide.src.*;
import org.openide.src.nodes.FieldElementNode;

import org.netbeans.modules.java.ui.IndexedPropertyChildren;

/**
 * Specialization of IndexedPropertyChildren that handles FieldElement nodes.
 */
class FieldsChildren extends IndexedPropertyChildren {
    /**
     * Default format for displaying elements.
     * PENDING: shouldn't the format be obtained from global options to reflect
     * user preferences ?
     */
    private static ElementFormat FIELD_FORMAT = new ElementFormat("{n}");

    /**
     * Constructs a Children over the specified class description.
     */
    public FieldsChildren(ClassElement clazz) {
        super(clazz, ElementProperties.PROP_FIELDS);
    }

    protected Node[] createNodes(Object key) {
        return new Node[] {
            createFieldNode((FieldElement)key)
        };
    }

    /**
     * Constructs a standard node for a MethodElement and sets the predefined
     * format to it.
     */
    private Node createFieldNode(FieldElement f) {
        FieldElementNode mn = new FieldElementNode(f, true);
        mn.setElementFormat(FIELD_FORMAT);
        return mn;
    }
}
