/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.ui.wizard;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.openide.explorer.*;
import org.openide.nodes.*;
import org.openide.src.*;

import org.netbeans.modules.java.ui.IndexedRoot;

/**
 *
 * @author  jrojcek
 */
public class MemberListPanel extends javax.swing.JPanel {
    
    private ElementCreator elementCreator;
    private ExplorerManager manager;
    private PropertyChangeListener managerListener;
    private Index indexableParent;

    private static final Node[] EMPTY_SELECTION = new Node[0];

    public MemberListPanel(ElementCreator creator) {
        initComponents();
        elementCreator = creator;

        newButton.setMnemonic(JavaWizardIterator.getString("LBL_New_Mnemonic").charAt(0));
        duplicateButton.setMnemonic(JavaWizardIterator.getString("LBL_Duplicate_Mnemonic").charAt(0));
        deleteButton.setMnemonic(JavaWizardIterator.getString("LBL_Delete_Mnemonic").charAt(0));
        upButton.setMnemonic(JavaWizardIterator.getString("LBL_Up_Mnemonic").charAt(0));
        downButton.setMnemonic(JavaWizardIterator.getString("LBL_Down_Mnemonic").charAt(0));
        
        listView.setPopupAllowed(false);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        newButton = new javax.swing.JButton();
        duplicateButton = new javax.swing.JButton();
        deleteButton = new javax.swing.JButton();
        upButton = new javax.swing.JButton();
        downButton = new javax.swing.JButton();
        listView = new org.openide.explorer.view.ListView();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        newButton.setText(JavaWizardIterator.getString("LBL_New"));
        newButton.setMargin(new java.awt.Insets(2, 8, 2, 8));
        newButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                newButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 5, 0);
        add(newButton, gridBagConstraints1);
        
        duplicateButton.setText(JavaWizardIterator.getString("LBL_Duplicate"));
        duplicateButton.setMargin(new java.awt.Insets(2, 8, 2, 8));
        duplicateButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                duplicateButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 5, 0);
        add(duplicateButton, gridBagConstraints1);
        
        deleteButton.setText(JavaWizardIterator.getString("LBL_Delete"));
        deleteButton.setMargin(new java.awt.Insets(2, 8, 2, 8));
        deleteButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                deleteButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 11, 0);
        add(deleteButton, gridBagConstraints1);
        
        upButton.setText(JavaWizardIterator.getString("LBL_Up"));
        upButton.setMargin(new java.awt.Insets(2, 8, 2, 8));
        upButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                upButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 5, 0);
        add(upButton, gridBagConstraints1);
        
        downButton.setText(JavaWizardIterator.getString("LBL_Down"));
        downButton.setMargin(new java.awt.Insets(2, 8, 2, 8));
        downButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                downButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 11, 0);
        add(downButton, gridBagConstraints1);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.gridheight = 6;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 11, 11);
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(listView, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void duplicateButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_duplicateButtonActionPerformed
        Node[] n = manager.getSelectedNodes();
        try {
            if (n.length > 0)
                updateSelection(new Node[] {
                    manager.getExploredContext().getChildren().findChild(
                        elementCreator.duplicateElement(
                            (MemberElement)n[0].getCookie(MemberElement.class)
                        ).getName().getSourceName()
                    )
                });
        } catch (SourceException ex) {
            // silently
        }
    }//GEN-LAST:event_duplicateButtonActionPerformed

    private void deleteButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_deleteButtonActionPerformed
        Node[] n = manager.getSelectedNodes();
        try {
            for (int i = 0; i < n.length; i++) {
                MemberElement el = (MemberElement)n[i].getCookie(MemberElement.class);
                if (elementCreator.canDelete(el)) elementCreator.deleteElement(el);
            }
        } catch (SourceException ex) {
            // silently
        }
        updateSelection(EMPTY_SELECTION);
        updateButtons();
    }//GEN-LAST:event_deleteButtonActionPerformed

    private void downButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_downButtonActionPerformed
        Node[] n = manager.getSelectedNodes();
        indexableParent.moveDown(indexableParent.indexOf(n[0]));
        updateSelection(EMPTY_SELECTION);
        updateSelection(new Node[] {n[0]});
        updateButtons();
    }//GEN-LAST:event_downButtonActionPerformed

    private void upButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_upButtonActionPerformed
        Node[] n = manager.getSelectedNodes();
        indexableParent.moveUp(indexableParent.indexOf(n[0]));
        updateSelection(EMPTY_SELECTION);
        updateSelection(new Node[] {n[0]});
        updateButtons();
    }//GEN-LAST:event_upButtonActionPerformed

    private void newButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_newButtonActionPerformed
        try {
            updateSelection(new Node[] {
                manager.getExploredContext().getChildren().findChild(
                    elementCreator.createElement().getName().getSourceName())
            });
        } catch (SourceException ex) {
            // silently
        }
    }//GEN-LAST:event_newButtonActionPerformed
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton newButton;
    private javax.swing.JButton duplicateButton;
    private javax.swing.JButton deleteButton;
    private javax.swing.JButton upButton;
    private javax.swing.JButton downButton;
    private org.openide.explorer.view.ListView listView;
    // End of variables declaration//GEN-END:variables

    
    public void addNotify() {
        super.addNotify();
        if (manager == null)
            manager = ExplorerManager.find(this);
        if (managerListener == null)
            managerListener = new java.beans.PropertyChangeListener() {
                public void propertyChange(PropertyChangeEvent evt) {
                    if (ExplorerManager.PROP_SELECTED_NODES.equals(evt.getPropertyName())) {
                        updateButtons();
                    }
                    if (ExplorerManager.PROP_EXPLORED_CONTEXT.equals(evt.getPropertyName())) {
                        indexableParent = (Index)((Node)evt.getNewValue()).getCookie(Index.class);
                        updateButtons();
                    }
                }
            };
        manager.addPropertyChangeListener(managerListener);
        
        Node n = manager.getExploredContext();
        if (n != null)
            indexableParent = (Index)n.getCookie(Index.class);
        updateButtons();
    }
    
    public void removeNotify() {
        manager.removePropertyChangeListener(managerListener);
    }

    private void updateSelection(final Node node[]) {
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                try {
                    manager.setSelectedNodes(node);
                } catch (java.beans.PropertyVetoException ex) {
                    // silently ignore.
                }
            }
        });
    }

    private void updateButtons() {
        ExplorerManager m = manager;
        Node[] nodes = m.getSelectedNodes();   
        boolean singleSel = nodes.length == 1;

        // these buttons are valid only for single selection
        if (indexableParent != null && singleSel) {
            // If the nodes provide ReorderCookie, they can be reordered inside their parent.
            upButton.setEnabled(!isFirst(nodes[0]));
            downButton.setEnabled(!isLast(nodes[0]));
        } else {
            upButton.setEnabled(false);
            downButton.setEnabled(false);
        }
        
        boolean canRemove = false;
        for (int i = 0; i < nodes.length; i++) {
            if ((nodes[i].canDestroy()) 
                && (elementCreator.canDelete((MemberElement)nodes[i].getCookie(MemberElement.class)))) {
                canRemove = true;
                break;
            }
        }
        deleteButton.setEnabled(canRemove);
        duplicateButton.setEnabled(singleSel);
    }

    protected boolean isFirst(Node n) {
        return indexableParent.indexOf(n) == 0;
    }

    protected boolean isLast(Node n) {
        return indexableParent.indexOf(n) == indexableParent.getNodesCount() - 1;
    }

    protected void removeNode(Node n) {
        try {
            n.destroy();
        } catch (java.io.IOException e) {
        }
    }

    protected void createNewNode(Node parent) {
        try {
            parent.getNewTypes()[0].create();
        } catch (java.io.IOException e) {
        }
    }

    public static interface ElementCreator {
        public MemberElement createElement() throws SourceException;
        public MemberElement duplicateElement(MemberElement el) throws SourceException;
        public void deleteElement(MemberElement el) throws SourceException;
        public boolean canDelete(MemberElement el);
    }
}
