/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.ui.wizard;

import java.lang.reflect.Modifier;
import java.awt.Container;
import java.awt.Component;

import org.openide.explorer.*;
import org.openide.nodes.*;
import org.openide.src.*;
import org.openide.explorer.propertysheet.*;
import org.openide.util.Utilities;
import org.openide.NotifyDescriptor;
import org.openide.TopManager;

/**
 *
 * @author  jrojcek
 */
public class MethodCustomizer extends javax.swing.JPanel {

    private static final MethodElement EMPTY_ELEMENT = new MethodElement();

    private MethodElement method;
    
    
    public MethodCustomizer(MethodElement method) {
        this.method = (method != null) ? method : EMPTY_ELEMENT;

        initComponents();

        nameField.setText(this.method.getName().toString());

        typePanel = new PropertyPanel (
            this.method,
            ElementProperties.PROP_RETURN,
            PropertyPanel.PREF_INPUT_STATE
        );

        java.awt.GridBagConstraints gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 11, 11);
        gridBagConstraints1.weightx = 1.0;
        add(typePanel, gridBagConstraints1);
        
        paramPanel = new PropertyPanel (
            this.method,
            ElementProperties.PROP_PARAMETERS,
            PropertyPanel.PREF_INPUT_STATE
        );

        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 5;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 11, 11);
        gridBagConstraints1.weightx = 1.0;
        add(paramPanel, gridBagConstraints1);
        
        exceptPanel = new PropertyPanel (
            this.method,
            ElementProperties.PROP_EXCEPTIONS,
            PropertyPanel.PREF_INPUT_STATE
        );

        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 7;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 0, 11);
        gridBagConstraints1.weightx = 1.0;
        add(exceptPanel, gridBagConstraints1);

        modifPanel = new PropertyPanel (
            new TempPropertyModel(
                this.method,
                ElementProperties.PROP_MODIFIERS),
            PropertyPanel.PREF_CUSTOM_EDITOR
        );

        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.gridheight = 8;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        add(modifPanel, gridBagConstraints1);
        
        nameLabel.setDisplayedMnemonic(JavaWizardIterator.getString("LBL_Name_Mnemonic").charAt(0));
        exceptionsLabel.setDisplayedMnemonic(JavaWizardIterator.getString("LBL_Exceptions_Mnemonic").charAt(0));
        parametersLabel.setDisplayedMnemonic(JavaWizardIterator.getString("LBL_Parameters_Mnemonic").charAt(0));
        returnTypeLabel.setDisplayedMnemonic(JavaWizardIterator.getString("LBL_Return_Type_Mnemonic").charAt(0));
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        nameLabel = new javax.swing.JLabel();
        exceptionsLabel = new javax.swing.JLabel();
        parametersLabel = new javax.swing.JLabel();
        returnTypeLabel = new javax.swing.JLabel();
        nameField = new javax.swing.JTextField();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        nameLabel.setText(JavaWizardIterator.getString("LBL_Name"));
        nameLabel.setLabelFor(nameField);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 2, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(nameLabel, gridBagConstraints1);
        
        exceptionsLabel.setText(JavaWizardIterator.getString("LBL_Exceptions"));
        exceptionsLabel.setLabelFor(exceptPanel);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 6;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 2, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(exceptionsLabel, gridBagConstraints1);
        
        parametersLabel.setText(JavaWizardIterator.getString("LBL_Parameters"));
        parametersLabel.setLabelFor(paramPanel);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 2, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(parametersLabel, gridBagConstraints1);
        
        returnTypeLabel.setText(JavaWizardIterator.getString("LBL_Return_Type"));
        returnTypeLabel.setLabelFor(typePanel);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 2, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(returnTypeLabel, gridBagConstraints1);
        
        nameField.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                nameFieldFocusLost(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 11, 11);
        gridBagConstraints1.weightx = 1.0;
        add(nameField, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void nameFieldFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_nameFieldFocusLost
        if (evt.isTemporary())
            return;

        String newName = nameField.getText();
        String oldName = method.getName().toString();
        boolean ok = false;
        if (Utilities.isJavaIdentifier(newName)) {
            if (!oldName.equals(newName)) {
                final Identifier id = Identifier.create(newName);
                try {
                    method.setName(id);
                    ok = true;
                }
                catch (SourceException e) {
                    TopManager.getDefault().notifyException(e);
                }
            }
        }
        else {
            TopManager.getDefault().notify(
                new NotifyDescriptor.Message(
                    JavaWizardIterator.getString("MSG_Not_Valid_Identifier"),
                    NotifyDescriptor.ERROR_MESSAGE
                )
            );
        }
        if (!ok)
            nameField.setText(oldName);
    }//GEN-LAST:event_nameFieldFocusLost


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel nameLabel;
    private javax.swing.JLabel exceptionsLabel;
    private javax.swing.JLabel parametersLabel;
    private javax.swing.JLabel returnTypeLabel;
    private javax.swing.JTextField nameField;
    // End of variables declaration//GEN-END:variables
    private PropertyPanel modifPanel;
    private PropertyPanel paramPanel;
    private PropertyPanel typePanel;
    private PropertyPanel exceptPanel;

    public void setEnabled(boolean enable) {
        nameField.setEnabled(enable);
        modifPanel.setEnabled(enable);
        paramPanel.setEnabled(enable);
        typePanel.setEnabled(enable);
        exceptPanel.setEnabled(enable);
        super.setEnabled(enable);
    }
    
    /** Sets the new class element for all interacting property editors and
     * panels.
     */
    public void setObject(MethodElement method) {
//        if (this.method == method)
//            return;
        this.method = (method != null) ? method : EMPTY_ELEMENT;
        resetPanels();
        revalidate();
    }

    private void resetPanels() {
        nameField.setText(this.method.getName().toString());

        typePanel.setModel(new DefaultPropertyModel(
            this.method,
            ElementProperties.PROP_RETURN
        ));

        paramPanel.setModel(new DefaultPropertyModel(
            this.method,
            ElementProperties.PROP_PARAMETERS
        ));

        exceptPanel.setModel(new DefaultPropertyModel(
            this.method,
            ElementProperties.PROP_EXCEPTIONS
        ));
        
        modifPanel.setModel(new TempPropertyModel(
            this.method,
            ElementProperties.PROP_MODIFIERS
        ));
    }

    public void addNotify() {
        super.addNotify();
//        if (method != null)
//            method.addPropertyChangeListener(this);
    }

    public void removeNotify() {
        super.removeNotify();
//        if (method != null)
//            method.removePropertyChangeListener(this);
    }
    /////////////////////////////////////////////////////////////////////////////
    
    public org.openide.util.HelpCtx getHelp() {
        return null;
    }
    
    public void readSettings(Object settings) {
    }
    
    public void storeSettings(Object settings) {
    }

    public boolean isValid() {
        return true;
    }
    
    public void addChangeListener(javax.swing.event.ChangeListener l) {
    }

    public void removeChangeListener(javax.swing.event.ChangeListener l) {
    }
    
    public java.awt.Component getComponent() {
        return this;
    }
    
    /////////////////////////////////////////////////////////////////////////////
    static class TempPropertyModel extends DefaultPropertyModel {
    
        public TempPropertyModel(Object bean, String propertyName) throws IllegalArgumentException {
            super(bean, propertyName);
        }
        /** The class of the property editor or <CODE>null</CODE>
         * if default property editor should be used.
         */
        public Class getPropertyEditorClass() {
            return MethodModifierEditor.class;
        }
    }
    
    public static class MethodModifierEditor extends ModifierEditor {
        public MethodModifierEditor() {
            super(new AccessLevelPanel(Modifier.PUBLIC 
                                       | Modifier.PRIVATE 
                                       | Modifier.PROTECTED, 0),
                  new MethodTypePanel());
        }
    }
}
