/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.ui.wizard;

import javax.swing.ButtonGroup;
import java.lang.reflect.Modifier;
import java.awt.Component;

/**
 *
 * @author  jrojcek
 */
public class MethodTypePanel extends javax.swing.JPanel
                             implements ModifierEditor.ModifierPanel {

    /** Creates new form MethodModifiersPanel */
    public MethodTypePanel() {
        initComponents();
        setBorder(new javax.swing.border.TitledBorder(
            new javax.swing.border.EtchedBorder(), 
            org.openide.util.NbBundle.getBundle(MethodTypePanel.class).getString("LBL_MethodType")));

        ButtonGroup bg = new ButtonGroup();
        bg.add(regularRadioButton);
        bg.add(abstractRadioButton);
        regularRadioButton.setSelected(true);

        regularRadioButton.setMnemonic('l');
        abstractRadioButton.setMnemonic('a');
        finalCheckBox.setMnemonic('i');
        nativeCheckBox.setMnemonic('v');
        staticCheckBox.setMnemonic('t');
        synchronizedCheckBox.setMnemonic('z');
    }
    
    public void setTypeModifier(int mod) {
        abstractRadioButton.setSelected(Modifier.isAbstract(mod));
        regularRadioButton.setSelected(!Modifier.isAbstract(mod));
        finalCheckBox.setSelected(Modifier.isFinal(mod));
        nativeCheckBox.setSelected(Modifier.isNative(mod));
        staticCheckBox.setSelected(Modifier.isStatic(mod));
        synchronizedCheckBox.setSelected(Modifier.isSynchronized(mod));
    }

    public int getTypeModifier() {
        if (abstractRadioButton.isSelected()) return Modifier.ABSTRACT;
        int mod = 0;
        if (finalCheckBox.isSelected()) mod |= Modifier.FINAL;
        if (nativeCheckBox.isSelected()) mod |= Modifier.NATIVE;
        if (staticCheckBox.isSelected()) mod |= Modifier.STATIC;
        if (synchronizedCheckBox.isSelected()) mod |= Modifier.SYNCHRONIZED;
        return mod;
    }
    
    public int getMask() {
        return Modifier.FINAL | Modifier.STATIC
               | Modifier.NATIVE | Modifier.SYNCHRONIZED 
               | Modifier.ABSTRACT;
    }
    
    public Component getComponent() {
        return this;
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        regularRadioButton = new javax.swing.JRadioButton();
        abstractRadioButton = new javax.swing.JRadioButton();
        finalCheckBox = new javax.swing.JCheckBox();
        nativeCheckBox = new javax.swing.JCheckBox();
        staticCheckBox = new javax.swing.JCheckBox();
        synchronizedCheckBox = new javax.swing.JCheckBox();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        setBorder(new javax.swing.border.TitledBorder("Method Type"));
        regularRadioButton.setText("<default>");
        regularRadioButton.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                regularRadioButtonStateChanged(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.insets = new java.awt.Insets(6, 6, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(regularRadioButton, gridBagConstraints1);
        
        abstractRadioButton.setText("abstract");
        abstractRadioButton.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                abstractRadioButtonStateChanged(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.insets = new java.awt.Insets(0, 6, 5, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(abstractRadioButton, gridBagConstraints1);
        
        finalCheckBox.setText("final");
        finalCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                checkBoxActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.insets = new java.awt.Insets(0, 18, 0, 6);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(finalCheckBox, gridBagConstraints1);
        
        nativeCheckBox.setText("native");
        nativeCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                checkBoxActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.insets = new java.awt.Insets(0, 18, 0, 6);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(nativeCheckBox, gridBagConstraints1);
        
        staticCheckBox.setText("static");
        staticCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                checkBoxActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 0, 5);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(staticCheckBox, gridBagConstraints1);
        
        synchronizedCheckBox.setText("synchronized");
        synchronizedCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                checkBoxActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 0, 5);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(synchronizedCheckBox, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void abstractRadioButtonStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_abstractRadioButtonStateChanged
        changeChecksEnabled(!abstractRadioButton.isSelected());
        firePropertyChange(PROP_TYPE_MODIFIER, null, null);
    }//GEN-LAST:event_abstractRadioButtonStateChanged

    private void regularRadioButtonStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_regularRadioButtonStateChanged
        changeChecksEnabled(regularRadioButton.isSelected());
        firePropertyChange(PROP_TYPE_MODIFIER, null, null);
    }//GEN-LAST:event_regularRadioButtonStateChanged

    private void checkBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_checkBoxActionPerformed
        firePropertyChange(PROP_TYPE_MODIFIER, null, null);
    }//GEN-LAST:event_checkBoxActionPerformed

    private void changeChecksEnabled(boolean b) {
        finalCheckBox.setEnabled(b);
        staticCheckBox.setEnabled(b);
        nativeCheckBox.setEnabled(b);
        synchronizedCheckBox.setEnabled(b);
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JRadioButton regularRadioButton;
    private javax.swing.JRadioButton abstractRadioButton;
    private javax.swing.JCheckBox finalCheckBox;
    private javax.swing.JCheckBox nativeCheckBox;
    private javax.swing.JCheckBox staticCheckBox;
    private javax.swing.JCheckBox synchronizedCheckBox;
    // End of variables declaration//GEN-END:variables

}
