/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.ui.wizard;

import java.text.MessageFormat;
import java.util.ResourceBundle;
import java.util.Arrays;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import org.openide.explorer.*;
import org.openide.nodes.*;
import org.openide.src.*;
import org.openide.util.NbBundle;

import org.netbeans.modules.java.ui.IndexedRoot;

/**
 *
 * @author  jrojcek
 */
public class MethodsPanel extends javax.swing.JPanel 
                          implements JavaPanel, MemberListPanel.ElementCreator, ExplorerManager.Provider {


    private static ResourceBundle bundle = NbBundle.getBundle(MethodsPanel.class);
    private static final MessageFormat METHOD_NAME_FORMAT = 
        new java.text.MessageFormat(bundle.getString("FMT_NewMethodName"));

    /**
     * Reference to the data holder.
     */
    private JavaWizardData wizardData;

    /**
     * Reference to the data holder.
     */
    private MethodElement method;
    
    /** Class customizer which interacts with user
     */
    private MethodCustomizer methodCustomizer;
    private MemberListPanel listPanel;
    private ExplorerManager manager;
    private PropertyChangeListener managerListener;
    private int newElementCount = 0;

    public void initialize(JavaWizardData data) {
        wizardData = data;
        initComponents();
        putClientProperty("WizardPanel_contentSelectedIndex", new Integer(4));

        listPanel = new MemberListPanel(this);
        
        java.awt.GridBagConstraints gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 11, 0);
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(listPanel, gridBagConstraints1);

        methodCustomizer = new MethodCustomizer(method);

        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        add(methodCustomizer, gridBagConstraints1);

        Index.KeysChildren c = new MethodsChildren(data.getCustomizedClass());
        Node n = new IndexedRoot(c);
        getExplorerManager().setRootContext(n);

        methodsLabel.setDisplayedMnemonic(JavaWizardIterator.getString("LBL_Methods_Mnemonic").charAt(0));
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        methodsLabel = new javax.swing.JLabel();
        headLabel = new javax.swing.JLabel();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        setName(JavaWizardIterator.getString("TIT_AddMethodsWizardPanel"));
        methodsLabel.setText(JavaWizardIterator.getString("LBL_Methods"));
        methodsLabel.setLabelFor(listPanel);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 2, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(methodsLabel, gridBagConstraints1);
        
        headLabel.setText(JavaWizardIterator.getString("LBL_Add_method"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 12, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(headLabel, gridBagConstraints1);
        
    }//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel methodsLabel;
    private javax.swing.JLabel headLabel;
    // End of variables declaration//GEN-END:variables

    public void addNotify() {
        super.addNotify();
        if (managerListener == null) 
            managerListener = new PropertyChangeListener() {
                public void propertyChange(PropertyChangeEvent evt) {
                    if (ExplorerManager.PROP_SELECTED_NODES.equals(evt.getPropertyName())) {
                        handleSelection((Node[])evt.getNewValue());
                    }
                }
            };
        manager.addPropertyChangeListener(managerListener);
        
        handleSelection(manager.getSelectedNodes());
    }
    
    private void handleSelection(Node[] nodes) {
        MethodElement me = null;
        boolean enable = false;
        if (nodes.length == 1) {
            me = (MethodElement)nodes[0].getCookie(MethodElement.class);
            enable = ! containsOriginal(me);
        }
        customizeMethod(me, enable);
    }
    
    private void customizeMethod(MethodElement me, boolean enable) {
        if (enable) {
            methodCustomizer.setEnabled(true);
            methodCustomizer.setObject(me);
        } else {
            methodCustomizer.setObject(me);
            methodCustomizer.setEnabled(false);
        }
    }
    
    public void removeNotify() {
        super.removeNotify();
        manager.removePropertyChangeListener(managerListener);
    }

    public MemberElement createElement() throws SourceException {
        MethodElement el = new MethodElement();
        createName(el);
        wizardData.getCustomizedClass().addMethod(el);
        return el;
    }

    public MemberElement duplicateElement(MemberElement template) throws SourceException {
        MethodElement el;
        try {
            el = (MethodElement)template.clone();
            createName(el);
        }
        catch (java.lang.CloneNotSupportedException ex) {
            System.err.println(ex);
            el = (MethodElement)createElement();
        }
        wizardData.getCustomizedClass().addMethod(el);
        return el;
    }
    
    private void createName(MethodElement el) throws SourceException {
        String name = METHOD_NAME_FORMAT.format(
                new Object[] {
                    new Integer(newElementCount)
                });
        el.setName(Identifier.create(name));
        newElementCount++;
    }

    public void deleteElement(MemberElement el) throws SourceException {
        wizardData.getCustomizedClass().removeMethod((MethodElement)el);
    }

    public boolean canDelete(MemberElement el) {
        return !containsOriginal((MethodElement)el);
    }

    private boolean elementEquals(MemberElement e1, MemberElement e2) {
        MethodElement.Key k1 = new MethodElement.Key((MethodElement)e1);
        MethodElement.Key k2 = new MethodElement.Key((MethodElement)e2);
        return k1.equals(k2);
    }
    
    private boolean containsOriginal(MethodElement m) {
        MethodElement[] me = wizardData.getOriginalClass().getMethods();
        for (int i = 0; i < me.length; i++) {
            if (elementEquals(m, me[i]))
                return true;
        }
        return false;
    }

    /** Get the explorer manager.
     * @return the manager
     */
    public ExplorerManager getExplorerManager() {
        if (manager == null)
            manager = new ExplorerManager();
        return manager;
    }
    /////////////////////////////////////////////////////////////////////////////
    
    public org.openide.util.HelpCtx getHelp() {
        return org.openide.util.HelpCtx.DEFAULT_HELP;
    }
    
    public void readSettings(Object settings) {
    }
    
    public void storeSettings(Object settings) {
    }

    public boolean isValid() {
        return true;
    }
    
    public void addChangeListener(javax.swing.event.ChangeListener l) {
    }

    public void removeChangeListener(javax.swing.event.ChangeListener l) {
    }
    
    public java.awt.Component getComponent() {
        return this;
    }
    
    /////////////////////////////////////////////////////////////////////////////
}
