/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.ui.wizard;

import java.util.*;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import org.openide.nodes.*;
import org.openide.src.*;
import org.openide.src.nodes.*;
import org.openide.explorer.ExplorerManager;

import org.netbeans.modules.java.tools.ElementFilter;
/**
 *
 * @author  jrojcek
 */
public class OverridePanel extends javax.swing.JPanel implements JavaPanel {
    /**
     * Reference to the data holder.
     */
    private JavaWizardData wizardData;
    private PropertyChangeListener managerListener;
    
    public void initialize(JavaWizardData data) {
        wizardData = data;
        initComponents();
        putClientProperty("WizardPanel_contentSelectedIndex", new Integer(3)); // NOI18N

        Children.Keys lc = new InheritanceChildren(data.getInheritance(), 
                                                  new ClassComparator(),
                                                  new MethodComparator(),
                                                  getAvailableMethodsFilter(true, false));
        Node treeNode = new AbstractNode(lc);

        treeExplorerPanel = new ExPanel();
        treeExplorerPanel.setLayout(new java.awt.BorderLayout());
        treeExplorerPanel.getExplorerManager().setRootContext(treeNode);

        beanTreeView = new org.openide.explorer.view.BeanTreeView();
        beanTreeView.setRootVisible(false);
        beanTreeView.setPopupAllowed(false);
        treeExplorerPanel.add("Center", beanTreeView); // NOI18N
        
        java.awt.GridBagConstraints gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.gridheight = 5;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 5, 11);
        gridBagConstraints1.weightx = 0.2;
        gridBagConstraints1.weighty = 1.0;
        add(treeExplorerPanel, gridBagConstraints1);

        InheritanceChildren.OverridenMethods methodsToOverride = new InheritanceChildren.OverridenMethods(
            data.getCustomizedClass(),
            data.getInheritance(), 
            new MethodComparator()
        );
        Node listNode = new AbstractNode(methodsToOverride);

        listExplorerPanel = new ExPanel();
        listExplorerPanel.setLayout(new java.awt.BorderLayout());
        listExplorerPanel.getExplorerManager().setRootContext(listNode);

        listView = new org.openide.explorer.view.ListView();
        listView.setPopupAllowed(false);
        listExplorerPanel.add("Center", listView); // NOI18N
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 3;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.gridheight = 5;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 5, 0);
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(listExplorerPanel, gridBagConstraints1);

        avaliableMethodsLabel.setDisplayedMnemonic(JavaWizardIterator.getString("LBL_Available_superclass_Mnemonic").charAt(0));
        overridenMethodsLabel.setDisplayedMnemonic(JavaWizardIterator.getString("LBL_Methods_overridden_Mnemonic").charAt(0));

        inheritedCheckBox.setMnemonic(JavaWizardIterator.getString("LBL_Show_inherited_Mnemonic").charAt(0));
        abstractCheckBox.setMnemonic(JavaWizardIterator.getString("LBL_Only_show_abstract_Mnemonic").charAt(0));
        addButton.setMnemonic(JavaWizardIterator.getString("LBL_Add_Mnemonic").charAt(0));
        removeButton.setMnemonic(JavaWizardIterator.getString("LBL_Remove_Mnemonic").charAt(0));
        addAllButton.setMnemonic(JavaWizardIterator.getString("LBL_Add_All_Mnemonic").charAt(0));
        removeAllButton.setMnemonic(JavaWizardIterator.getString("LBL_Remove_All_Mnemonic").charAt(0));
        superCheckBox.setMnemonic(JavaWizardIterator.getString("LBL_Generate_super_calls_Mnemonic").charAt(0));
        updateButtons();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        headLabel = new javax.swing.JLabel();
        avaliableMethodsLabel = new javax.swing.JLabel();
        overridenMethodsLabel = new javax.swing.JLabel();
        inheritedCheckBox = new javax.swing.JCheckBox();
        abstractCheckBox = new javax.swing.JCheckBox();
        addButton = new javax.swing.JButton();
        removeButton = new javax.swing.JButton();
        addAllButton = new javax.swing.JButton();
        removeAllButton = new javax.swing.JButton();
        superCheckBox = new javax.swing.JCheckBox();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        setName(JavaWizardIterator.getString("TIT_OverrideWizardPanel"));
        headLabel.setText(JavaWizardIterator.getString("LBL_Select_methods"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridwidth = 4;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 11, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints1.weightx = 1.0;
        add(headLabel, gridBagConstraints1);
        
        avaliableMethodsLabel.setText(JavaWizardIterator.getString("LBL_Available_superclass"));
        avaliableMethodsLabel.setLabelFor(beanTreeView);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 2, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(avaliableMethodsLabel, gridBagConstraints1);
        
        overridenMethodsLabel.setText(JavaWizardIterator.getString("LBL_Methods_overridden"));
        overridenMethodsLabel.setLabelFor(listView);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 3;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 2, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(overridenMethodsLabel, gridBagConstraints1);
        
        inheritedCheckBox.setText(JavaWizardIterator.getString("LBL_Show_inherited"));
        inheritedCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                inheritedCheckBoxActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 7;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(inheritedCheckBox, gridBagConstraints1);
        
        abstractCheckBox.setText(JavaWizardIterator.getString("LBL_Only_show_abstract"));
        abstractCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                abstractCheckBoxActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 7;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(abstractCheckBox, gridBagConstraints1);
        
        addButton.setText(JavaWizardIterator.getString("LBL_Add"));
        addButton.setMargin(new java.awt.Insets(2, 8, 2, 8));
        addButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 2;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 5, 11);
        add(addButton, gridBagConstraints1);
        
        removeButton.setText(JavaWizardIterator.getString("LBL_Remove"));
        removeButton.setMargin(new java.awt.Insets(2, 8, 2, 8));
        removeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                removeButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 2;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 11, 11);
        add(removeButton, gridBagConstraints1);
        
        addAllButton.setText(JavaWizardIterator.getString("LBL_Add_All"));
        addAllButton.setMargin(new java.awt.Insets(2, 8, 2, 8));
        addAllButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addAllButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 2;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 5, 11);
        add(addAllButton, gridBagConstraints1);
        
        removeAllButton.setText(JavaWizardIterator.getString("LBL_Remove_All"));
        removeAllButton.setMargin(new java.awt.Insets(2, 8, 2, 8));
        removeAllButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                removeAllButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 2;
        gridBagConstraints1.gridy = 5;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 5, 11);
        add(removeAllButton, gridBagConstraints1);
        
        superCheckBox.setText(JavaWizardIterator.getString("LBL_Generate_super_calls"));
        superCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                superCheckBoxActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 3;
        gridBagConstraints1.gridy = 7;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 0, 11);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(superCheckBox, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void superCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_superCheckBoxActionPerformed
        Collection override = collectMethodsToOverride(
            new Node[] {listExplorerPanel.getExplorerManager().getRootContext()});
        undoOverrideMethods(override);
        overrideMethods(override);
    }//GEN-LAST:event_superCheckBoxActionPerformed

    private void removeAllButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_removeAllButtonActionPerformed
        undoOverrideMethods(
            collectMethodsToOverride(
                new Node[] {listExplorerPanel.getExplorerManager().getRootContext()}
            )
        );
        updateButtons();
    }//GEN-LAST:event_removeAllButtonActionPerformed

    private void removeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_removeButtonActionPerformed
        undoOverrideMethods(
            collectMethodsToOverride(
                listExplorerPanel.getExplorerManager().getSelectedNodes()
            )
        );
        updateButtons();
    }//GEN-LAST:event_removeButtonActionPerformed

    private void addAllButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addAllButtonActionPerformed
        overrideMethods(
            collectMethodsToOverride(
                new Node[] {treeExplorerPanel.getExplorerManager().getRootContext()}
            )
        );
        updateButtons();
    }//GEN-LAST:event_addAllButtonActionPerformed

    private void addButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addButtonActionPerformed
        overrideMethods(
            collectMethodsToOverride(
                treeExplorerPanel.getExplorerManager().getSelectedNodes()
            )
        );
        updateButtons();
    }//GEN-LAST:event_addButtonActionPerformed

    private void abstractCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_abstractCheckBoxActionPerformed
        InheritanceChildren children = (InheritanceChildren)treeExplorerPanel
                                        .getExplorerManager().getRootContext().getChildren();
        children.setFilter(getAvailableMethodsFilter(true, abstractCheckBox.isSelected()));
        updateButtons();
    }//GEN-LAST:event_abstractCheckBoxActionPerformed

    private void inheritedCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_inheritedCheckBoxActionPerformed
        InheritanceChildren children = (InheritanceChildren)treeExplorerPanel
                                        .getExplorerManager().getRootContext().getChildren();
        children.setMethodsOnly(inheritedCheckBox.isSelected());
        updateButtons();
    }//GEN-LAST:event_inheritedCheckBoxActionPerformed

    private void undoOverrideMethods(Collection methods) {
        Iterator iter = methods.iterator();
        while (iter.hasNext()) {
            MethodElement me = (MethodElement)iter.next();
            try {
                if (!containsOriginal(me))
                    wizardData.getInheritance().getSourceClass().removeMethod(me);
            } catch (SourceException ex) {
//                System.err.println(ex);
            }
        }
    }

    private void overrideMethods(Collection methods) {
        Collection newMethods = new ArrayList();
        Iterator iter = methods.iterator();
        while (iter.hasNext()) {
            MethodElement me = (MethodElement)iter.next();
            try {
                newMethods.add(wizardData.getInheritance().overrideMethod(me, superCheckBox.isSelected()));
            } catch (SourceException ex) {
//                System.err.println(ex);
            }
        }
    }
        
    private Collection collectMethodsToOverride(Node[] n) {
        Collection list = new ArrayList();
        for (int i = 0; i < n.length; i++) {
            if (!n[i].isLeaf()) {
                list.addAll(collectMethodsToOverride(n[i].getChildren().getNodes()));
            } else {
                MemberElement me = (MemberElement)n[i].getCookie(MemberElement.class);
                if (me instanceof MethodElement) {
                    list.add(me);
                }
            }
        }
        return list;
    }

    private void updateButtons() {
        addButton.setEnabled(treeExplorerPanel.getExplorerManager().getSelectedNodes().length > 0);
        addAllButton.setEnabled(treeExplorerPanel.getExplorerManager().getRootContext().getChildren().getNodes().length > 0);
        removeAllButton.setEnabled(listExplorerPanel.getExplorerManager().getRootContext().getChildren().getNodes().length > 0);
        
        boolean canRemove = false;
        Node[] nodes = listExplorerPanel.getExplorerManager().getSelectedNodes();
        for (int i = 0; i < nodes.length; i++) {
            if (!containsOriginal((MethodElement)nodes[i].getCookie(MethodElement.class))) {
                canRemove = true;
                break;
            }
        }
        removeButton.setEnabled(canRemove);
    }
    
    private boolean containsOriginal(MethodElement m) {
        MethodElement[] me = wizardData.getOriginalClass().getMethods();
        for (int i = 0; i < me.length; i++) {
            if (elementEquals(m, me[i]))
                return true;
        }
        return false;
    }

    private boolean elementEquals(MemberElement e1, MemberElement e2) {
        MethodElement.Key k1 = new MethodElement.Key((MethodElement)e1);
        MethodElement.Key k2 = new MethodElement.Key((MethodElement)e2);
        return k1.equals(k2);
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel headLabel;
    private javax.swing.JLabel avaliableMethodsLabel;
    private javax.swing.JLabel overridenMethodsLabel;
    private javax.swing.JCheckBox inheritedCheckBox;
    private javax.swing.JCheckBox abstractCheckBox;
    private javax.swing.JButton addButton;
    private javax.swing.JButton removeButton;
    private javax.swing.JButton addAllButton;
    private javax.swing.JButton removeAllButton;
    private javax.swing.JCheckBox superCheckBox;
    // End of variables declaration//GEN-END:variables
    private org.openide.explorer.view.ListView listView;
    private org.openide.explorer.view.BeanTreeView beanTreeView;
    private ExPanel treeExplorerPanel;
    private ExPanel listExplorerPanel;

    public void addNotify() {
        super.addNotify();
        if (managerListener == null) 
            managerListener = new PropertyChangeListener() {
                public void propertyChange(PropertyChangeEvent evt) {
                    if (ExplorerManager.PROP_SELECTED_NODES.equals(evt.getPropertyName())) {
                        updateButtons();                    
                    }
                }
            };
        treeExplorerPanel.getExplorerManager().addPropertyChangeListener(managerListener);
        listExplorerPanel.getExplorerManager().addPropertyChangeListener(managerListener);
        
        ((InheritanceChildren.OverridenMethods)listExplorerPanel
            .getExplorerManager().getExploredContext().getChildren()).refreshKeys();
    }

    public void removeNotify() {
        super.removeNotify();
        treeExplorerPanel.getExplorerManager().removePropertyChangeListener(managerListener);
        listExplorerPanel.getExplorerManager().removePropertyChangeListener(managerListener);
    }
    
    /////////////////////////////////////////////////////////////////////////////
    
    public org.openide.util.HelpCtx getHelp() {
        return org.openide.util.HelpCtx.DEFAULT_HELP;
    }
    
    public void readSettings(Object settings) {
    }
    
    public void storeSettings(Object settings) {
    }

    public boolean isValid() {
        return true;
    }
    
    public void addChangeListener(javax.swing.event.ChangeListener l) {
    }

    public void removeChangeListener(javax.swing.event.ChangeListener l) {
    }
    
    public java.awt.Component getComponent() {
        return this;
    }
    
    /////////////////////////////////////////////////////////////////////////////

    protected ElementFilter.Method getAvailableMethodsFilter(final boolean includeInherited,
            final boolean filterAbstract) {
        if (includeInherited) {
            return new ElementFilter.Method() {
                public boolean accept(MethodElement m) {
                    boolean ok = wizardData.getInheritance().isOverridable(m);
                    if (filterAbstract) {
//                        ok &= !wizardData.getInheritance().isImplemented(m);
                        ok &= java.lang.reflect.Modifier.isAbstract(m.getModifiers());
                    }
                    return ok;
                }
            };
        } else {
            return new ElementFilter.Method() {
                public boolean accept(MethodElement m) {
                    boolean ok = wizardData.getInheritance().isOverridable(m);
//                                 && support.isDeclaredFirst(m, m.getDeclaringClass());
                    if (filterAbstract) {
//                        ok &= !wizardData.getInheritance().isImplemented(m);
                        ok &= java.lang.reflect.Modifier.isAbstract(m.getModifiers());
                    }
                    return ok;
                }
             };
        }
    }

    private class ClassComparator implements java.util.Comparator {
        public int compare(Object value1, Object value2) {
            ClassElement v1 = (ClassElement)value1;
            ClassElement v2 = (ClassElement)value2;
            
            if (v1.isClassOrInterface() != v2.isClassOrInterface()) {
                return v1.isClassOrInterface() ? -1 : 1;
            }
            if (v1.isClassOrInterface()) {
                String fqn = v2.getName().getFullName();
                while (v1.getSuperclass() != null) {
                    if (v1.getSuperclass().getFullName().equals(fqn))
                        return 1;
                    v1 = wizardData.getInheritance().getClass(v1.getSuperclass(), true);
                }
                return -1;
            }
            return v1.getName().getFullName().compareTo(v2.getName().getFullName());
        }
        
        public boolean equals(final java.lang.Object p1) {
            return p1 instanceof ClassComparator;
        }
    }
    
    private static class ExPanel extends javax.swing.JPanel implements ExplorerManager.Provider {
        private ExplorerManager manager;
        /** Get the explorer manager.
         * @return the manager
         */
        public ExplorerManager getExplorerManager() {
            if (manager == null)
                manager = new ExplorerManager();
            return manager;
        }
        
        public java.awt.Dimension getPreferredSize() {
            return new java.awt.Dimension(235, 0);
        }
    }

    /** Comparator that orders methods as follows:
     *  1. sorts them in ascending alphabetical order
     *  2. methods with the same name, but less arguments are preceding methods with more arguments
     *  3. methods with the same number of arguments are sorted by their argument declarations
     */
    private static class MethodComparator implements java.util.Comparator {
        public int compare(Object a,Object b) {
            MethodElement ma = (MethodElement)a;
            MethodElement mb = (MethodElement)b;
            int result;

            if (ma == mb)
            return 0;
            result = ma.getName().getSourceName().compareTo(mb.getName().getSourceName());
            if (result != 0)
            return result;
            result = ma.getParameters().length - mb.getParameters().length;
            if (result != 0)
            return result;
            // compare parameter names/types.
            MethodParameter[] paramA = ma.getParameters();
            MethodParameter[] paramB = mb.getParameters();

            for (int i = 0; i < paramA.length; i++) {
                result = paramA[i].getFullString().compareTo(paramB[i].getFullString());
                if (result != 0)
                return result;
            }
            return 0;
        }

        public boolean equals(Object ob) {
            return ob instanceof MethodComparator;
        }
    }

}
