/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.java.ui.wizard;

import java.util.ResourceBundle;
import java.util.StringTokenizer;

import org.openide.src.*;
import org.openide.util.Utilities;

class Util extends Object {
    /** Returns true, if the passed string can be used as a qualified identifier.
     * it does not check for semantic, only for syntax.
     * The function returns true for any sequence of identifiers separated by
     * dots.
     */
    public static boolean isValidPackageName(String str) {
        StringTokenizer tukac = new StringTokenizer(str, ".");
        while (tukac.hasMoreTokens()) {
            String token = tukac.nextToken();
            if ("".equals(token))
                return false;
            if (!Utilities.isJavaIdentifier(token))
                return false;
        }
        return true;
    }
    
    public static boolean isValidTypeIdentifier(String ident) {
        if (ident == null || "".equals(ident))
            return false;
        try {
            Type t = Type.parse(ident);
            return t.isClass();
        } catch (IllegalArgumentException e) {
            return false;
        }
    }
    
    public static boolean implementsInterface(ClassElement clazz, Identifier id) {
        Identifier[] implemented = clazz.getInterfaces();
        for (int i = 0; i < implemented.length; i++) {
            if (implemented[i].equals(id)) {
                return true;
            }
        }
        return false;
    }
}
