/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.wizard;

import java.io.IOException;
import java.util.*;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Component;
import java.awt.Dialog;

import org.openide.*;
import org.openide.nodes.Children;
import org.openide.nodes.IndexedNode;
import org.openide.nodes.Node;
import org.openide.src.*;
import org.openide.src.nodes.FieldElementNode;
import org.openide.src.nodes.FieldCustomizer;
import org.openide.util.datatransfer.NewType;

/** 
 * This is an abstract node that serves as a grouping node for the fields.
 * Aside from ordinary Category node, this node supports indexing of its
 * children.
 */
class FieldsGroupNode extends IndexedNode {
    ClassElement declaringClass;

    /** Creates new FieldsGroupNode */
    public FieldsGroupNode(ClassElement clazz) {
        this(clazz, new Children(clazz));
    }

    private FieldsGroupNode(ClassElement clazz, Children ch) {
        super(ch, ch.getIndex());
        this.declaringClass = clazz;
    }

    public String getName() {
        return Util.getString("NAME_DeclaredFieldsNode");
    }

    static class Children extends IndexedPropertyChildren {
        ClassElement declaringClass;

        Children(ClassElement clazz) {
            super(clazz, ElementProperties.PROP_FIELDS);
            this.declaringClass = clazz;
        }

        protected Node[] createNodes(Object key) {
            return new Node[] {
                createNode((FieldElement)key)
            };
        }

        private Node createNode(FieldElement f) {
            FieldElementNode fn = new FieldElementNode(f, true);
            return fn;
        }
    }
}
