/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.wizard;

import java.lang.reflect.Modifier;
import java.text.MessageFormat;

import java.awt.Component;
import java.awt.Dimension;

import org.openide.src.*;
import org.openide.src.nodes.FieldCustomizer;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.nodes.IndexedNode;
import org.openide.nodes.Index;

class FieldsPanel extends MemberElementPanel {
    /** Format for new field names. An unique number within the wizard session will
        be substituted for the 0th parameter.
    */
    private static final MessageFormat FIELD_NAME_FORMAT = 
        new java.text.MessageFormat(bundle.getString("FMT_NewFieldName"));
    
    /** Default type for new fields.
    */
    private static final Type DEFAULT_TYPE = Type.INT;
    
    /** Default modifiers for new fields.
    */
    private static final int DEFAULT_MODIFIERS = Modifier.PRIVATE;

    /** Creates new FieldsPanel */
    public FieldsPanel(JavaWizardData data) {
        super(data);
    }
    
    protected Dimension getCustomizerDimension() { 
        return new Dimension(475, 300);
    }
    
    protected Node createRootNode() {
        return new FieldsGroupNode(getTarget());
    }
    
    protected String getListLabelKey() {
        return "LBL_FieldsInClass";
    }
    
    protected Component getCustomizer(MemberElement e) {
        return new FieldCustomizer((FieldElement)e);
    }
    
    protected MemberElement createNewElement() throws SourceException {
        String name = FIELD_NAME_FORMAT.format(
                new Object[] {
                    new Integer(newElementCount)
                });
        FieldElement fe = new FieldElement();
        fe.setName(Identifier.create(name));
        fe.setModifiers(DEFAULT_MODIFIERS);
        fe.setType(DEFAULT_TYPE);

        // the following call might throw an exception if the field
        // is already present in the class.            
        getTarget().addField(fe);
        newElementCount++;
        return fe;
    }
}