/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */
package org.netbeans.modules.java.wizard;

import java.io.IOException;

import org.openide.nodes.AbstractNode;
import org.openide.nodes.FilterNode;
import org.openide.nodes.Node;

import org.openide.src.*;
import org.openide.src.nodes.ClassElementNode;

/** This is filter for ClassElement node. Its purpose is to redirect destroy()
 *  so it only removes the represented interface from the implementing class instead
 *  of deleting the (interface) class itself.
 *
 */
class ImplementedInterfaceNode extends FilterNode {
    /** Class element that is implementing this interface.
    */
    ClassElement parent;

    /** Controls if destroy() action is ever allowed. For some necessary interfaces,
        this action can be disabled for class customization.
    */
    boolean destroyAllowed;

    /** Creates new ImplementedInterfaceNode */
    public ImplementedInterfaceNode(ClassElement parent, ClassElementNode original,
        boolean destroyAllowed) {
        super(original, Children.LEAF);
        this.parent = parent;
        this.destroyAllowed = destroyAllowed;
        disableDelegation(DELEGATE_DESTROY);
    }

    /** Removes the represented interface from the parent class' implements
        clause, then destroys the node. Does not destroy the node's class element.
    */
    public void destroy() throws IOException {
        ClassElement clazz = (ClassElement)getOriginal().getCookie(ClassElement.class);
        if (clazz == null) {
            return;
        }
        try {
            parent.removeInterfaces(clazz.getName());
        } catch (SourceException e) {
            // should not happen ??
        }
        super.destroy();
    }
    
    public boolean equals(Object o) {
        if (!(o instanceof ImplementedInterfaceNode))
            return false;
        ClassElement cls1 = (ClassElement)getCookie(ClassElement.class);
        ClassElement cls2 = (ClassElement)((Node)o).getCookie(ClassElement.class);
        if (cls1.getName() == null && cls2.getName() == null)
            return true;
        if (cls1.getName() == null || cls2.getName() == null)
            return false;
        return cls1.getName().equals(cls2.getName());
    }

    /** Can't be renamed.
    */
    public boolean canRename() {
        return false;
    }

    public boolean canDestroy() {
        return destroyAllowed;
    }
}
