/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.wizard;

import java.lang.reflect.InvocationTargetException;
import java.util.Collection;
import java.util.Arrays;
import java.util.LinkedList;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import org.openide.TopManager;
import org.openide.NotifyDescriptor;

import org.openide.explorer.propertysheet.DefaultPropertyModel;
import org.openide.explorer.propertysheet.PropertyModel;
import org.openide.nodes.Index;

/**
 * @deprecated This class is obsolete and should not be used any more. Implementation
 * suitable to Java NFT Wizard has been moved to package ui.wizard.
 * The class will be removed before final release of NetBeans 3.2
 */
public abstract class IndexedPropertyChildren extends Index.KeysChildren {
    /** Property model for getting/settings the property.
    */
    PropertyModel model;

    /** Listener that updates the display from the underlying bean.
    */
    PropertyChangeListener beanListener;

    /** States that changes from the bean should be ignored.
    */
    boolean ignoreBeanChanges;

    public IndexedPropertyChildren(Object bean, String propertyName) {
        this(new DefaultPropertyModel(bean, propertyName));
    }

    /** Creates new PropertyChildren */
    public IndexedPropertyChildren(PropertyModel m) {
        super(new LinkedList());
        this.model = m;
        
        if (!m.getPropertyType().isArray()) {
            throw new IllegalArgumentException("Property of array type expected."); //NOI18N
        }
    }

    /** Called when the children are about to be displayed.
        Attaches a listener to the target bean via property model and monitors its changes.
    */
    protected void addNotify() {
        model.addPropertyChangeListener(getBeanListener());
        updateFromSource();
    }

    /** Called after the children are removed from the display.
        Currently only removes the property listener.
    */
    protected void removeNotify() {
        model.removePropertyChangeListener(getBeanListener());
    }

    protected final PropertyChangeListener getBeanListener() {
        if (beanListener != null) {
            return beanListener;
        }
        return beanListener = createBeanListener();
    }

    protected PropertyChangeListener createBeanListener() {
        return new SourceListener();
    }

    /** Updates the children list from the source.
        This implementation obtains the current value from the property model and
        calls update().
    */
    protected final void updateFromSource() {
        Collection c;
        try {
            c = Arrays.asList((Object[])model.getValue());
        } catch (InvocationTargetException e) {
            return;
        }

        this.list.clear();
        this.list.addAll(c);
        update();
    }

    protected Object[] createValueHolder() {
        Class compType = model.getPropertyType().getComponentType();
        return (Object[])java.lang.reflect.Array.newInstance(compType, list.size());
    }

    /** Performs a reorder on the indexed property.
        This implementation reorders children's node list using superclass'
        reorder() method, then tries to update the bean with the new property value.
        If this fails, reorder updates the subnode list from the bean's current
        property value.
    */
    protected void reorder(int []permutation) {
        super.reorder(permutation);
        try {
            try {
                ignoreBeanChanges = true;
                Object[] value = createValueHolder();
                list.toArray(value);
                model.setValue(value);
            } finally {
                ignoreBeanChanges = false;
            }
        } catch (InvocationTargetException e) {
            handleReorderException(e);
        }
    }

    protected void handleReorderException(InvocationTargetException e) {
        TopManager.getDefault().notify(new NotifyDescriptor.Exception(e));
        updateFromSource();
    }

    /*************************************************************************/
    private class SourceListener implements PropertyChangeListener {
        public void propertyChange(PropertyChangeEvent evt) {
            if (PropertyModel.PROP_VALUE.equals(evt.getPropertyName())) {
                IndexedPropertyChildren.this.updateFromSource();
            }
        }
    }
}
