/*
 * InheritancePanel.java
 *
 * Created on May 9, 2000, 1:22 PM
 */

package org.netbeans.modules.java.wizard;

import java.beans.PropertyEditor;
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.text.MessageFormat;

import org.openide.explorer.propertysheet.editors.ModifierEditor;
import org.openide.src.*;
import org.openide.explorer.propertysheet.*;
import org.openide.util.Utilities;
import org.openide.ErrorManager;
import org.openide.NotifyDescriptor;
import org.openide.TopManager;

import org.openide.loaders.TemplateWizard;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataFolder;
import org.openide.WizardDescriptor;

import org.netbeans.modules.java.tools.SuperclassList;
/**
 *
 * @author  sdedic
 * @version
 * @deprecated Wizard UI implementation has been moved to ui.wizard package.
 */
public class InheritancePanel extends javax.swing.JPanel implements WizardDescriptor.Panel, WizardDescriptor.FinishPanel {
    /** Class element whose properties are being customized. This member is updated
     * from JavaWizardData every time the panel is displayed.
     */
    private ClassElement clazz;
    
    /** JavaWizardData holds all relevant data for class customization.
     */
    private JavaWizardData data;
    
    /** This is set to true if the user has been warned about changing superclass
     * specified in the template.
     * The warning is not issued on subsequent changes.
     */
    private boolean superclassChangeWarning = false;
    
    /** Property editor that edits modifiers on the class element.
     */
    private PropertyPanel modifEditor;
    
    /** Holds a reference to inner panel displaying list of superclasses.
     */
    private SuperclassList superclassList;
    
    /** Was initialization finished ?
     */
    private boolean initialized;
    
    /** Predefined types in the type combo */
    private static final String[] COMMON_TYPES = {
        "java.lang.Object", // NOI18N
        "java.awt.Component", // NOI18N
        "javax.swing.JComponent", // NOI18N
        "javax.swing.JPanel" // NOI18N
    };
    
    /** Specialized editor for class' implemented interfaces.
     */
    private InterfaceArrayPanel interfaceEditor;
    
    /** Creates new form InheritancePanel */
    public InheritancePanel(JavaWizardData data) {
        this.data = data;
    }
    
    private void initialize() {
        initComponents ();
        initBorders();
        modifEditor = new PropertyPanel (
            new ClassElement(),
            ElementProperties.PROP_MODIFIERS,
            PropertyPanel.PREF_CUSTOM_EDITOR
        );
        interfaceEditor = new InterfaceArrayPanel();
        superclassList = new SuperclassList();
        
        modifiersPanel.add(modifEditor, BorderLayout.CENTER);
        interfacesPanel.add(interfaceEditor, BorderLayout.CENTER);
        inheritancePanel.add(superclassList, BorderLayout.CENTER);
        initialized = true;
    }
    
    private void initBorders() {
        // interfaces panel
        addEtchedBorder(classPanel, Util.getString("LBL_ClassInformation"));
        addEtchedBorder(modifiersPanel, Util.getString("LBL_ClassModifiers"));
        addEtchedBorder(inheritancePanel, Util.getString("LBL_InheritanceInformation"));
        addEtchedBorder(interfacesPanel, Util.getString("LBL_ImplementedInterfaces"));
    }
    
    private void addEtchedBorder(JPanel p, String title) {
        TitledBorder titled = new TitledBorder(title);
        titled.setBorder(new EtchedBorder(EtchedBorder.LOWERED));
        p.setBorder(new CompoundBorder(titled, new EmptyBorder(new Insets(8, 8, 8, 8))));
    }
    
    public void addNotify() {
        String qualifier = clazz.getName().isQualified() ?
        clazz.getName().getQualifier() : Util.getString("TXT_DefaultPackageName");
        
        nameField.setText(clazz.getName().getSourceName());
        if (clazz.isClass()) {
            String superName = clazz.getSuperclass() == null ? "" : clazz.getSuperclass().getFullName();
            superclassCombo.setSelectedItem(superName);
        }
        else
        {   classPanel.remove(superclassCombo);
            classPanel.remove(jLabel2);
        }
        packageField.setText(qualifier);
        modifEditor.setModel(new DefaultPropertyModel(
        clazz,
        ElementProperties.PROP_MODIFIERS
        ));
        PropertyEditor propEdit = modifEditor.getPropertyEditor();
        if (propEdit instanceof ModifierEditor) {
            ((ModifierEditor)propEdit).setMask(clazz.getModifiersMask());
        }
        superclassList.setData(data.getInheritance());
        super.addNotify();
        interfaceEditor.setObject(clazz);
    }
    
    public void removeNotify() {
        super.removeNotify();
    }
    
    /** Sets the new class element for all interacting property editors and
     * panels.
     */
    public void setObject(ClassElement clazz) {
        this.clazz = clazz;
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        classPanel = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        nameField = new javax.swing.JTextField();
        jLabel2 = new javax.swing.JLabel();
        superclassCombo = new javax.swing.JComboBox(COMMON_TYPES);
        jLabel3 = new javax.swing.JLabel();
        packageField = new javax.swing.JTextField();
        modifiersPanel = new javax.swing.JPanel();
        interfacesPanel = new javax.swing.JPanel();
        inheritancePanel = new javax.swing.JPanel();
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        setPreferredSize(new java.awt.Dimension(700, 550));
        
        classPanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints2;
        
        jLabel1.setText(Util.getString("LBL_ClassName"));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.insets = new java.awt.Insets(12, 0, 0, 8);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.EAST;
        classPanel.add(jLabel1, gridBagConstraints2);
        
        
        nameField.setEditable(false);
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 1;
        gridBagConstraints2.gridy = 0;
        gridBagConstraints2.gridwidth = 0;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.insets = new java.awt.Insets(12, 0, 0, 8);
        gridBagConstraints2.weightx = 1.0;
        classPanel.add(nameField, gridBagConstraints2);
        
        
        jLabel2.setText(Util.getString("LBL_Superclass"));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 0;
        gridBagConstraints2.gridy = 2;
        gridBagConstraints2.gridheight = 0;
        gridBagConstraints2.insets = new java.awt.Insets(8, 0, 0, 8);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.NORTHEAST;
        gridBagConstraints2.weighty = 1.0;
        classPanel.add(jLabel2, gridBagConstraints2);
        
        
        superclassCombo.setEditable(true);
        superclassCombo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                superclassComboActionPerformed(evt);
            }
        }
        );
        superclassCombo.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                superclassComboFocusLost(evt);
            }
        }
        );
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 1;
        gridBagConstraints2.gridy = 2;
        gridBagConstraints2.gridwidth = 0;
        gridBagConstraints2.gridheight = 0;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.insets = new java.awt.Insets(8, 0, 0, 8);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.NORTH;
        gridBagConstraints2.weightx = 1.0;
        classPanel.add(superclassCombo, gridBagConstraints2);
        
        
        jLabel3.setText(Util.getString("LBL_PackageName"));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 0;
        gridBagConstraints2.gridy = 1;
        gridBagConstraints2.insets = new java.awt.Insets(12, 0, 0, 8);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.EAST;
        classPanel.add(jLabel3, gridBagConstraints2);
        
        
        packageField.setEditable(false);
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 1;
        gridBagConstraints2.gridy = 1;
        gridBagConstraints2.gridwidth = 0;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.insets = new java.awt.Insets(12, 0, 0, 8);
        gridBagConstraints2.weightx = 1.0;
        classPanel.add(packageField, gridBagConstraints2);
        
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(8, 8, 8, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints1.weightx = 1.0;
        add(classPanel, gridBagConstraints1);
        
        
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(8, 8, 8, 8);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(modifiersPanel, gridBagConstraints1);
        
        
        interfacesPanel.setLayout(new java.awt.BorderLayout());
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(0, 8, 8, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(interfacesPanel, gridBagConstraints1);
        
        
        inheritancePanel.setLayout(new java.awt.BorderLayout());
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(0, 8, 8, 8);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints1.weightx = 1.0;
        add(inheritancePanel, gridBagConstraints1);
        
    }//GEN-END:initComponents
    
  private void superclassComboFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_superclassComboFocusLost
      updateSuperclass();
  }//GEN-LAST:event_superclassComboFocusLost
  
  private void superclassComboActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_superclassComboActionPerformed
      updateSuperclass();
  }//GEN-LAST:event_superclassComboActionPerformed
  
  private void updateSuperclass() {
      // in case that the template originally specified an ancestor (not Object) for the class,
      // warn the user once.
      Identifier superid = clazz.getSuperclass();
      String oldValueStr = superid == null ? "" : superid.getFullName();
      Object selItem = superclassCombo.getSelectedItem();
      
      if (selItem == null) {
          superclassCombo.setSelectedItem(oldValueStr);
          return;
      }
      
      String newValueStr = selItem.toString();
      
      if (superclassChangeWarning == false) {
          // test if there's a superclass other than the Object.
          if (superid != null && ClassElement.ROOT_OBJECT.equals(superid)) {
              if (warnSuperclassChange()) {
                  // revert the changes back...
                  superclassCombo.setSelectedItem(superid);
                  return;
              }
          }
          superclassChangeWarning = true;
      }
      
      boolean ok = false;
      try {
          if (!oldValueStr.equals(newValueStr)) {
              try {
                  Identifier id;
                  if (newValueStr.equals("")) { //NOI18N
                      id = null;
                  } else {
                      id = Identifier.create(newValueStr);
                  }
                  clazz.setSuperclass(id);
                  ok = true;
              }
              catch (SourceException e) {
                  TopManager.getDefault().notifyException(e);
              }
          }
      }
      catch (IllegalArgumentException e) {
          TopManager.getDefault().notify(
          new NotifyDescriptor.Message(
          Util.getString("ERR_InvalidClassIdentifier"),
          NotifyDescriptor.ERROR_MESSAGE
          )
          );
      }
      if (!ok) {
          superclassCombo.setSelectedItem(oldValueStr);
      }
  }
  
  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JPanel classPanel;
  private javax.swing.JLabel jLabel1;
  private javax.swing.JTextField nameField;
  private javax.swing.JLabel jLabel2;
  private javax.swing.JComboBox superclassCombo;
  private javax.swing.JLabel jLabel3;
  private javax.swing.JTextField packageField;
  private javax.swing.JPanel modifiersPanel;
  private javax.swing.JPanel interfacesPanel;
  private javax.swing.JPanel inheritancePanel;
  // End of variables declaration//GEN-END:variables
  
    /** This method warns that the superclass is being changed. It should prompt
     * the user with a question if he really wants to do it since it might break
     * compilability of the Template.
     * @return true, if the action was cancelled. False if user confirmed.
     */
  private boolean warnSuperclassChange() {
      return false;
  }
  
    /** Get the component displayed in this panel.
     * @return the component
     */
  public java.awt.Component getComponent() {
      if (!initialized)
          initialize();
      return this;
  }
  
    /** Help for this panel.
     * When the panel is active, this is used as the help for the wizard dialog.
     * @return the help or <code>null</code> if no help is supplied
     */
  public org.openide.util.HelpCtx getHelp() {
      return null;
  }
  
    /** Provides the wizard panel with the current data--either
     * the default data or already-modified settings, if the user used the previous and/or next buttons.
     * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
     * @param settings the object representing wizard panel state, as originally supplied to {@link WizardDescriptor#WizardDescriptor(WizardDescriptor.Iterator,Object)}
     */
  public void readSettings(Object settings) {
      if (settings instanceof TemplateWizard) {
          TemplateWizard wizardInstance = (TemplateWizard)settings;
          
          DataObject dobj = null;
          DataFolder target = null;
          
          try {
              target = wizardInstance.getTargetFolder();
              dobj = wizardInstance.getTemplate();
          } catch (java.io.IOException ex) {
          }
          
          String name = wizardInstance.getTargetName();
          if (dobj == null || target == null) {
              throw new IllegalStateException("Expected target folder & template, got null(s)");
          }
          // checking for java-compatible name - both the folder name and the target name
          // must be acceptable.
          String fullTarget = target.getPrimaryFile().getPackageNameExt('.', '.');
          if (!Util.isValidPackageName(fullTarget)) {
              String msg = MessageFormat.format(Util.getString("FMTERR_IllegalFolderName"),
              new Object[] {
                  target.getPrimaryFile().getPackageNameExt('/', '.'),
                  fullTarget
              });
              throw (IllegalArgumentException)TopManager.getDefault().getErrorManager().annotate(
                new IllegalArgumentException("Illegal folder name"),
                ErrorManager.USER, null, msg,
                null, null
              );
          }
          if (name == null) {
              // make up an name using the Template's name
              name = org.openide.filesystems.FileUtil.findFreeFileName(target.getPrimaryFile(),
              dobj.getName(), "java");
          } else if (!Utilities.isJavaIdentifier(name)) {
              String msg = MessageFormat.format(Util.getString("FMTERR_IllegalTargetName"),
              new Object[] {
                  name
              });
              throw (IllegalArgumentException)TopManager.getDefault().getErrorManager().annotate(
                new IllegalArgumentException("Illegal target name"),
                ErrorManager.USER, null, msg,
                null, null
              );
          }
          data.setTemplate(dobj);
          data.setTarget(target.getPrimaryFile(), name);
      }
      ClassElement el = data.getCustomizedClass();
      setObject(el);
  }
  
    /** Provides the wizard panel with the opportunity to update the
     * settings with its current customized state.
     * Rather than updating its settings with every change in the GUI, it should collect them,
     * and then only save them when requested to by this method.
     * Also, the original settings passed to {@link #readSettings} should not be modified (mutated);
     * rather, the (copy) passed in here should be mutated according to the collected changes.
     * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
     * @param settings the object representing a settings of the wizard
     */
  public void storeSettings(Object settings) {
  }
  
    /** Test whether the panel is finished and it is safe to proceed to the next one.
     * If the panel is valid, the "Next" (or "Finish") button will be enabled.
     * @return <code>true</code> if the user has entered satisfactory information
     */
  public boolean isValid() {
      return true;
  }
    /** Add a listener to changes of the panel's validity.
     * @param l the listener to add
     * @see #isValid
     */
  public void addChangeListener(ChangeListener l) {
  }
    /** Remove a listener to changes of the panel's validity.
     * @param l the listener to remove
     */
  public void removeChangeListener(ChangeListener l) {
  }
}
