/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.wizard;

import java.util.*;

import java.awt.event.*;
import javax.swing.DefaultListModel;

import org.openide.src.*;
import org.openide.src.nodes.ClassElementNode;

import org.openide.NotifyDescriptor;
import org.openide.TopManager;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.explorer.view.NodeRenderer;
import org.openide.explorer.ExplorerManager;
import org.openide.util.Utilities;
import org.openide.util.datatransfer.NewType;

public class InterfaceArrayPanel extends IndexableExplorerList implements ExplorerManager.Provider {
    private ClassElement clazz;
    /** Special node that hosts interface list
    */
    private ImplementsNode parentNode;

    /** Explorer manager for the interface panel.
    */
    private ExplorerManager manager;

    InterfaceArrayPanel() {
        parentNode = new ImplementsNode(new ClassElement());
        addButton().addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                addNewInterface();
            }
        });
    }

    /** Sets clazz whose interfaces should be displayed in the explorer window.
    */
    public void setObject(ClassElement el) {
        this.clazz = el;
        parentNode = new ImplementsNode(el);
        getExplorerManager().setRootContext(parentNode);
        getExplorerManager().setExploredContext(parentNode);
    }

    /** Maintains an explorer manager instance for the explorer window
    */
    public ExplorerManager getExplorerManager() {
        if (manager == null) {
            manager = new ExplorerManager();
        }
        return manager;
    }

    private void addNewInterface() {
        NewType nt = new ImplementsNode.NewInterfaceType(clazz);
        try {
            nt.create();
        } catch (java.io.IOException ex) {
            org.openide.TopManager.getDefault().notify(
              new org.openide.NotifyDescriptor.Exception(ex)
            );
        }
    }
}
