/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.java.wizard;

import java.util.ResourceBundle;
import java.text.MessageFormat;
import java.beans.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.lang.reflect.Modifier;
import javax.swing.event.ChangeListener;

import org.openide.awt.SplittedPanel;
import org.openide.explorer.ExplorerPanel;
import org.openide.explorer.ExplorerManager;
import org.openide.explorer.propertysheet.PropertySheetView;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Node;
import org.openide.nodes.Children;
import org.openide.WizardDescriptor;
import org.openide.src.*;
import org.openide.src.nodes.*;
import org.openide.util.NbBundle;

public abstract class MemberElementPanel extends ExplorerPanel 
    implements WizardDescriptor.Panel, WizardDescriptor.FinishPanel {
    protected static final ResourceBundle bundle = NbBundle.getBundle(MemberElementPanel.class);
    /** Target class for the field manipulation.
    */
    private ClassElement targetClass;
    
    /** (invisible) root of the field list
    */
    private Node root;
    
    /** Current FieldElementCustomizer that is displayed in the right pane.
    */
    private Component currentCustomizer;
    
    /** Listener that is attached to the explorer manager.
    */
    private PropertyChangeListener expListener;
    
    /** Listener that intercepts Add button action.
    */
    private ActionListener addListener;
    
    /** Panel used as a substitution for field customizer,
        in case there's no field selected.
    */
    private javax.swing.JPanel dummyPanel;
    
    /** Internal Wizard's data structures.
     */
    JavaWizardData data;
    
    /** Number of new Elements that have been created so far for the particular
     * customized class element. The counter is reset whenever ClassElement is
     * replaced by another one.
     */
    protected int newElementCount;
    
    private boolean initialized;

    /** Creates new form FieldPanel */
    public MemberElementPanel(JavaWizardData data) {
        this.data = data;
    }
    
    protected boolean nodesEquals(Node n1, Node n2) {
        MemberElement el1, el2;
        
        el1 = (MemberElement)n1.getCookie(Element.class);
        el2 = (MemberElement)n2.getCookie(Element.class);
        return el1.getName().equals(el2.getName());
    }
 
    private void initGUI() {
        initComponents ();
        IndexableExplorerList list = new IndexableExplorerList() {
            protected Node reselectNode(Node n) {
                Node[] nodes = this.getExplorerManager().getExploredContext().getChildren().getNodes();
                for (int i = 0; i < nodes.length; i++) {
                    if (nodesEquals(n, nodes[i])) {
                        return nodes[i];
                    }
                }
                return null;
            }
        };
        
        itemHolder.add(list, BorderLayout.CENTER);
        
        list.addButton().addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                insertMember();
            }
        });

       Dimension d = getCustomizerDimension();
       dummyPanel = new javax.swing.JPanel();
       dummyPanel.setPreferredSize(d);
       dummyPanel.setMinimumSize(d);
       dummyPanel.add(new javax.swing.JLabel(Util.getString("LBL_NoElementSelected")), BorderLayout.CENTER);
       initialized = true;
    }
    
    protected abstract Node createRootNode();

    protected abstract Dimension getCustomizerDimension();

    /** Creates a new member element, inserts it into the target class.
      */
    protected abstract MemberElement createNewElement() throws SourceException;
    
    /** Construct and returns a customizer suitable for the element.
    */
    protected abstract Component getCustomizer(MemberElement el);
    
    protected ClassElement getTarget() {
        return this.targetClass;
    }
    
    public void addNotify() {
        if (expListener == null) 
            expListener = new ExplorerL();

        ExplorerManager m = getExplorerManager();
        if (root == null) {
            root = createRootNode();
            m.setRootContext(root);
        }
        m.addPropertyChangeListener(expListener);
        
        Node[] nodes = m.getSelectedNodes();
        if (nodes == null || nodes.length != 1) {
            replaceCustomizer(null);
        } else {
            replaceCustomizer((MemberElement)nodes[0].getCookie(MemberElement.class));
        }
        super.addNotify();
    }

    public void removeNotify() {
        super.removeNotify();
        getExplorerManager().removePropertyChangeListener(expListener);
    }
    
    protected void setRoot(ClassElement target) {
        this.targetClass = target;
        reset();
    }
    
    protected void reset() {
        newElementCount = 1;
        root = null;
    }
    
    private void replaceCustomizer(MemberElement el) {
        if (currentCustomizer != null) {
            jPanel1.remove(currentCustomizer);
        }
        if (el != null) {
            currentCustomizer = getCustomizer(el);
        } else {
            currentCustomizer = dummyPanel;
        }
        
        
        jPanel1.add(currentCustomizer, BorderLayout.CENTER);
        
        revalidate();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jLabel1 = new javax.swing.JLabel();
        itemHolder = new javax.swing.JPanel();
        jPanel1 = new javax.swing.JPanel();
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        setPreferredSize(new java.awt.Dimension(700, 550));
        
        jLabel1.setText(Util.getString(getListLabelKey()));
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridwidth = -1;
        gridBagConstraints1.gridheight = -1;
        gridBagConstraints1.insets = new java.awt.Insets(8, 8, 0, 8);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.SOUTHWEST;
        add(jLabel1, gridBagConstraints1);
        
        
        itemHolder.setLayout(new java.awt.BorderLayout());
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.gridheight = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(8, 8, 8, 4);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints1.weighty = 1.0;
        add(itemHolder, gridBagConstraints1);
        
        
        jPanel1.setLayout(new java.awt.BorderLayout());
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridwidth = 0;
        gridBagConstraints1.gridheight = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(8, 4, 8, 8);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHEAST;
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(jPanel1, gridBagConstraints1);
        
    }//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel1;
    private javax.swing.JPanel itemHolder;
    private javax.swing.JPanel jPanel1;
    // End of variables declaration//GEN-END:variables

    protected abstract String getListLabelKey();
    
    /** Inserts a new field into the class with some defaults set.
    */
    private void insertMember() {
        final ExplorerManager m = getExplorerManager();
        final MemberElement member;

        try {
             member = createNewElement();
        } catch (SourceException e) {
            return;
        }
        String name = member.getName().getSourceName();
        
        // Select the new node in the Explorer. invoke it later to be sure it is
        // done after any readlock held currently on the children.
        final Node found = root.getChildren().findChild(name);
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                try {
                    m.setSelectedNodes(new Node[] {
                        found
                    });
                } catch (PropertyVetoException ex) {
                    // silently ignore.
                }
            }
        });
    }
   
    /** Get the component displayed in this panel.
     * @return the component
     */
    public java.awt.Component getComponent() {
        if (!initialized)
            initGUI();
        return this;
    }
    
    /** Help for this panel.
     * When the panel is active, this is used as the help for the wizard dialog.
     * @return the help or <code>null</code> if no help is supplied
     */
    public org.openide.util.HelpCtx getHelp() {
        return null;
    }
    
    /** Provides the wizard panel with the current data--either
     * the default data or already-modified settings, if the user used the previous and/or next buttons.
     * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
     * @param settings the object representing wizard panel state, as originally supplied to {@link WizardDescriptor#WizardDescriptor(WizardDescriptor.Iterator,Object)}
     */
    public void readSettings(Object settings) {
        // restore appropriate ClassElement from the JavaWizard's data:
        ClassElement cls = data.getCustomizedClass();
        if (cls != targetClass) {
            setRoot(cls);
        }
    }
    
    /** Provides the wizard panel with the opportunity to update the
     * settings with its current customized state.
     * Rather than updating its settings with every change in the GUI, it should collect them,
     * and then only save them when requested to by this method.
     * Also, the original settings passed to {@link #readSettings} should not be modified (mutated);
     * rather, the (copy) passed in here should be mutated according to the collected changes.
     * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
     * @param settings the object representing a settings of the wizard
     */
    public void storeSettings(Object settings) {
    }

    /** Test whether the panel is finished and it is safe to proceed to the next one.
     * If the panel is valid, the "Next" (or "Finish") button will be enabled.
     * @return <code>true</code> if the user has entered satisfactory information
     */
    public boolean isValid() {
        return true;
    }
    
    /** Add a listener to changes of the panel's validity.
     * @param l the listener to add
     * @see #isValid
     */
    public void addChangeListener(ChangeListener l) {
    }
    /** Remove a listener to changes of the panel's validity.
     * @param l the listener to remove
     */
    public void removeChangeListener(ChangeListener l) {
    }

    private class ExplorerL implements PropertyChangeListener {
        public void propertyChange(PropertyChangeEvent evt) {
            if (ExplorerManager.PROP_SELECTED_NODES.equals(evt.getPropertyName())) {
                Node[] nodes = (Node[])evt.getNewValue();
                if (nodes.length != 1) {
                    replaceCustomizer(null);
                } else {
                    replaceCustomizer((MemberElement)nodes[0].getCookie(MemberElement.class));
                }
            }
        }
    }
    
}
