/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */
package org.netbeans.modules.java.wizard;

import java.lang.reflect.Modifier;
import java.text.MessageFormat;

import java.awt.Component;
import java.awt.Dimension;

import org.openide.src.*;
import org.openide.src.nodes.MethodCustomizer;
import org.openide.nodes.Node;

/*
 * @deprecated Wizard UI implementation has been moved to ui.wizard package.
 */
public class MethodsPanel extends MemberElementPanel {
    /** Format for new field names. An unique number within the wizard session will
        be substituted for the 0th parameter.
    */
    private static final MessageFormat METHOD_NAME_FORMAT = 
        new java.text.MessageFormat(bundle.getString("FMT_NewMethodName"));
    
    /** Default type for new fields.
    */
    private static final Type DEFAULT_RETURN_TYPE = Type.VOID;
    
    /** Default modifiers for new fields.
    */
    private static final int DEFAULT_MODIFIERS = Modifier.PUBLIC;
    
    /** Creates new MethodsPanel */
    public MethodsPanel(JavaWizardData data) {
        super(data);
    }

    public void setObject(ClassElement clazz) {
        setRoot(clazz);
    }
    
    protected boolean nodesEquals(Node n1, Node n2) {
        MethodElement el1 = (MethodElement)n1.getCookie(MethodElement.class);
        MethodElement el2 = (MethodElement)n2.getCookie(MethodElement.class);
        MethodElement.Key k1 = new MethodElement.Key(el1);
        MethodElement.Key k2 = new MethodElement.Key(el2);
        return k1.equals(k2);
    }
    
    protected Node createRootNode() {
        return new IndexedRoot(new MethodsChildren(getTarget()));
    }
    
    protected String getListLabelKey() {
        return "LBL_MethodsInClass";
    }
    
    protected Dimension getCustomizerDimension() {
        return new Dimension(460, 450);
    }
    
    protected Component getCustomizer(MemberElement el) {
        return new MethodCustomizer((MethodElement)el);
    }
    
    protected MemberElement createNewElement() throws SourceException {
        String name = METHOD_NAME_FORMAT.format(
                new Object[] {
                    new Integer(newElementCount)
                });
                 
        MethodElement el = new MethodElement();
        el.setName(Identifier.create(name));
        el.setModifiers(DEFAULT_MODIFIERS);
        el.setReturn(DEFAULT_RETURN_TYPE);
        getTarget().addMethod(el);
        newElementCount++;
        return el;
    }
}