/*****************************************************************************
 * Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License Version
 * 1.0 (the "License"). You may not use this file except in compliance with
 * the License. A copy of the License is available at http://www.sun.com/

 * The Original Code is the CVS Client Library.
 * The Initial Developer of the Original Code is Robert Greig.
 * Portions created by Robert Greig are Copyright (C) 2000.
 * All Rights Reserved.

 * Contributor(s): Robert Greig.
 *****************************************************************************/
package org.netbeans.lib.cvsclient.command;

/**
 * This exception is thrown when an error occurs while executing a command.
 * It is nearly always a container for another exception.
 * @author  Robert Greig
 * @version $Revision: 1.6 $
 */
public class CommandException extends Exception
{
    private Exception underlyingException;

    public CommandException(Exception e)
    {
        underlyingException = e;
    }

    public CommandException(String message)
    {
        super(message);
    }

    public String getMessage()
    {
        if (underlyingException != null)
        {
            return "Underlying exception: " + underlyingException.getMessage();
        }
        return super.getMessage();
    }

    public Exception getUnderlyingException() {
        return underlyingException;
    }

    public void printStackTrace()
    {
        if (underlyingException != null)
        {
            underlyingException.printStackTrace();
        }
        else
        {
            super.printStackTrace();
        }
    }
}
