/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.lib.cvsclient.command.add;

    /**
     * Handles the building of add information object and the firing of
     * events when complete objects are built.
     *
     * @author  mkleint
     */
import org.netbeans.lib.cvsclient.event.*;
import java.io.*;
import org.netbeans.lib.cvsclient.file.*;
import org.netbeans.lib.cvsclient.command.Builder;
import org.netbeans.lib.cvsclient.command.BuildableCommand;
import org.netbeans.lib.cvsclient.command.BasicCommand;
import org.netbeans.lib.cvsclient.command.checkout.CheckoutCommand;





public class AddBuilder implements Builder
{
        /**
         * The status object that is currently being built
         */
    protected AddInformation addInformation;
    
    /**
     * The event manager to use
     */
    protected EventManager eventManager;
    
        /**
         * The directory in which the file being processed lives. This is
         * relative to the local directory
         */
    protected String fileDirectory;
    
    protected AddCommand addCommand;
    
    protected boolean readingTags = false;
    private static final String UNKNOWN = "cvs server: nothing known about";
    private static final String ADDED = " added to the repository";
    private static final String WARNING = "cvs server: warning: ";
    private static final String ALREADY_ENTERED = " has already been entered";
    private static final String SCHEDULING = "cvs server: scheduling file `";
    private static final String USE_COMMIT = "cvs server: use 'cvs commit' ";
    private static final String DIRECTORY = "Directory ";
    private static final String READDING = "cvs server: re-adding file ";
    private static final String RESURRECTED = ", resurrected";
    private static final String RESUR_VERSION = ", version ";
    
    public AddBuilder(EventManager eventMan, AddCommand comm) 
    {
        eventManager = eventMan;
        addCommand = comm;
    }
    
    public void outputDone() 
    {
        if (addInformation != null) 
        {
            eventManager.fireCVSEvent(new FileInfoEvent(this, addInformation));
            addInformation = null;
        }
    }
    
    public void parseLine(String line, boolean isErrorMessage) 
    {
        if (line.endsWith(ADDED)) 
        {
            addDirectory(line.substring(DIRECTORY.length(), line.indexOf(ADDED)));
        }
        if (line.startsWith(SCHEDULING)) 
        {
            addFile(line.substring(SCHEDULING.length(), line.indexOf('\'')));
        }
        if (line.startsWith(READDING)) 
        {
            String filename = line.substring(READDING.length(), line.indexOf('(')).trim();
            addFile(filename);
        }
        if (line.endsWith(RESURRECTED)) 
        {
            resurrectFile(line.substring(0, line.length() - RESURRECTED.length()));
        }
        // ignore the rest..
    }
    
    protected File createFile(String fileName) {
        StringBuffer path = new StringBuffer();
        File locFile = addCommand.getFileEndingWith(fileName);
        if (locFile == null) {
                System.out.println("JAVACVS ERROR!! wrong algorithm for assigning path to single files(1)!!");
        }
        return locFile;
    }
    
    protected void addDirectory(String name) 
    {
        addInformation = new AddInformation();
        addInformation.setDirectory(true);
        addInformation.setType('A');
        String dirName = name.replace('\\', '/');
        int index = dirName.lastIndexOf('/');
        if (index > 0) {
            dirName = dirName.substring(index + 1, dirName.length());
        }
        addInformation.setFile(createFile(dirName));
        outputDone();
    }
    
    protected void addFile(String name) 
    {
        addInformation = new AddInformation();
        addInformation.setDirectory(false);
        addInformation.setFile(createFile(name));
        addInformation.setType('A');
        outputDone();
    }
    
    protected void resurrectFile(String line) 
    {
        System.out.println("line1=" + line);
        int versionIndex = line.lastIndexOf(RESUR_VERSION);
        System.out.println("versionIndex=" + versionIndex);
        String version = line.substring(versionIndex + RESUR_VERSION.length()).trim();
        System.out.println("version=" +version);
        String cutLine = line.substring(0, versionIndex).trim();
        int fileIndex = cutLine.lastIndexOf(' ');
        System.out.println("fileindex=" + fileIndex);
        String name = cutLine.substring(fileIndex).trim();
        System.out.println("filename=" + name);
        addInformation = new AddInformation();
        addInformation.setType('U');
        addInformation.setDirectory(false);
        addInformation.setFile(createFile(name));
        outputDone();
    }
}
