/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.lib.cvsclient.command.checkout;

    /**
     * Handles the building of module list information object and the firing of
     * events when complete objects are built.
     *
     * @author  mkleint
     */
import org.netbeans.lib.cvsclient.event.*;
import java.io.*;
import java.util.*;

import org.netbeans.lib.cvsclient.file.*;
import org.netbeans.lib.cvsclient.command.Builder;
import org.netbeans.lib.cvsclient.command.*;



public class ModuleListBuilder implements Builder
{
        /**
         * The module  object that is currently being built
         */
    protected ModuleListInformation moduleInformation;
    
    /**
     * The event manager to use
     */
    protected EventManager eventManager;
    
        /**
         * The directory in which the file being processed lives. This is
         * relative to the local directory
         */
    protected String fileDirectory;
    
    protected CheckoutCommand checkoutCommand;
    
    protected boolean readingTags = false;
    
    public ModuleListBuilder(EventManager eventMan, CheckoutCommand comm) {
        eventManager = eventMan;
        checkoutCommand = comm;
    }
    
    public void outputDone() {
        if (moduleInformation != null) {
            eventManager.fireCVSEvent(new FileInfoEvent(this, moduleInformation));
            moduleInformation = null;
        }
    }
    
    public void parseLine(String line, boolean isErrorMessage) {
        line = line.replace('\t', ' ');
        if (!line.startsWith(" ")) {
            processModule(line, true);
        } else {
            processModule(line, false);
        }
    }
    
    protected void processModule(String line, boolean firstLine) {
        StringTokenizer tok = new StringTokenizer(line, " ", false);
        if (firstLine) {
            outputDone();
            moduleInformation = new ModuleListInformation();
            String modName = tok.nextToken();
            moduleInformation.setModuleName(modName);
            if (checkoutCommand.isShowModulesWithStatus()) {
                String stat = tok.nextToken();
                moduleInformation.setModuleStatus(stat);
            }
        }
        while (tok.hasMoreTokens()) {
            String nextTok = tok.nextToken();
            if (nextTok.startsWith("-")) {
                moduleInformation.setType(nextTok);
                continue;
            }
            moduleInformation.addPath(nextTok);
        }
    }
    
}
