/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */
package org.netbeans.lib.cvsclient.command.checkout;

import java.io.File;
import java.io.IOException;

import org.netbeans.lib.cvsclient.command.BasicCommand;
import org.netbeans.lib.cvsclient.command.BuildableCommand;
import org.netbeans.lib.cvsclient.command.Builder;
import org.netbeans.lib.cvsclient.event.EventManager;
import org.netbeans.lib.cvsclient.event.FileInfoEvent;

/**
 * Handles the building of "checkout with -p switch" information object and storing of
 * the checked out file to the temporary file and the firing of
 * events when complete objects are built.
 *
 * TS, 2001-01-15; TODO: change protected field to private!
 *
 * @author  mkleint
 */
public class PipedFilesBuilder implements Builder {

    /**
     * The module object that is currently being built.
     */
    protected PipedFileInformation fileInformation;

    /**
     * The event manager to use.
     */
    protected EventManager eventManager;

    /**
     * The directory in which the file being processed lives.
     * This is relative to the local directory.
     */
    protected String fileDirectory;

    protected BuildableCommand command;

    protected TemporaryFileCreator tempFileCreator;

    private static final String ERR_START = "=======";
    private static final String ERR_CHECK = "Checking out ";
    private static final String ERR_RCS   = "RCS:  ";
    private static final String ERR_VERS  = "VERS: ";
    private static final String EXAM_DIR = "cvs server: Updating";

    /**
     * Creates a new Builder for the PipeFileResponse.
     */
    public PipedFilesBuilder(EventManager         eventManager,
                             BuildableCommand     command,
                             TemporaryFileCreator tempFileCreator)
    {
        this.eventManager    = eventManager;
        this.command         = command;
        this.tempFileCreator = tempFileCreator;
    }

    public void outputDone()
    {
        if (fileInformation == null)
        {
            return;
        }

        try
        {
            fileInformation.closeTempFile();
        }
        catch (IOException exc)
        {
            //TODO
        }
        eventManager.fireCVSEvent(new FileInfoEvent(this, fileInformation));
        fileInformation = null;
    }

    public void parseLine(String line, boolean isErrorMessage)
    {
        if (isErrorMessage)
        {
            if (line.startsWith(EXAM_DIR))
            {
                fileDirectory = line.substring(21);
                return;
            }

            if (line.startsWith(ERR_CHECK))
            {
                processFile(line);
            }
            if (line.startsWith(ERR_RCS))
            {
                if (fileInformation != null)
                {
                    String repositoryName =
                                       line.substring(ERR_RCS.length()).trim();
                    fileInformation.setRepositoryFileName(repositoryName);
                }
            }
            if (line.startsWith(ERR_VERS))
            {
                if (fileInformation != null)
                {
                    String repositoryRevision =
                                        line.substring(ERR_RCS.length()).trim();
                    fileInformation.setRepositoryRevision(repositoryRevision);
                }
            }
            // header stuff..
        }
        else
        {
            if (fileInformation != null)
            {
                try
                {
                    fileInformation.addToTempFile(line);
                }
                catch (IOException exc)
                {
                    outputDone();
                }
            }
        }
    }

    protected void processFile(String line)
    {
        outputDone();
        try
        {
            File file = tempFileCreator.createTempFile();
            fileInformation = new PipedFileInformation(file);
        }
        catch (IOException exc)
        {
            fileInformation = null;
            return;
        }
        fileInformation.setFile(createFile(line.substring(ERR_CHECK.length())));
    }

    protected File createFile(String fileName)
    {
        StringBuffer path = new StringBuffer();
        BasicCommand bComm = null;

        // Hack: ask the command itself
        if (command instanceof CheckoutCommand)
        {
            //TODO bad path of file .. due to the absolute repository pointing in checkout
            // -unlike in other cvs commands which are relative
            CheckoutCommand cCom = (CheckoutCommand)command;
            path.append(cCom.getLocalPath());
        }
        if (command instanceof BasicCommand)
        {
            BasicCommand bCom = (BasicCommand)command;
            path.append(bCom.getLocalPath());
        }
        path.append(File.separator);
        if (fileDirectory == null && bComm != null)
        {
            // happens for single files only
            // (for directories, the dir name is always sent before the actual files)
            File locFile = bComm.getFileEndingWith(fileName);
            if (locFile == null)
            {
                System.out.println("JAVACVS ERROR!! wrong algorithm for assigning path to single files(1)!!");
                path.append(fileName);
            }
            else
            {
                path = new StringBuffer(locFile.getAbsolutePath());
            }
        }
        else
        {
            path.append(fileName);
        }
        return new File(path.toString());
    }
}
