/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.lib.cvsclient.command.commit;

    /**
     * Handles the building of update information object and the firing of
     * events when complete objects are built.
     *
     * @author  mkleint
     */
import org.netbeans.lib.cvsclient.event.*;
import java.io.*;
import org.netbeans.lib.cvsclient.file.*;
import org.netbeans.lib.cvsclient.command.Builder;
import org.netbeans.lib.cvsclient.command.BuildableCommand;
import org.netbeans.lib.cvsclient.command.BasicCommand;
import org.netbeans.lib.cvsclient.command.checkout.CheckoutCommand;





public class CommitBuilder implements Builder
{
        /**
         * The status object that is currently being built
         */
    protected CommitInformation commitInformation;

    /**
     * The event manager to use
     */
    protected EventManager eventManager;

        /**
         * The directory in which the file being processed lives. This is
         * relative to the local directory
         */
    protected String fileDirectory;

    protected CommitCommand commitCommand;

    protected boolean isAdding = false;
    private static final String UNKNOWN = "cvs commit: nothing known about `";
    private static final String EXAM_DIR = "cvs server: Examining";
    private static final String CHECKING_IN = "Checking in ";
    private static final String REMOVING = "Removing ";
    private static final String NEW_REVISION = "new revision:";
    private static final String INITIAL_REVISION = "initial revision:";
    private static final String DONE = "done";
    private static final String RCS_FILE = "RCS file: ";
    private static final String ADD = "cvs commit: use `cvs add' to create an entry for ";



    public CommitBuilder(EventManager eventMan, CommitCommand comm) {
        eventManager = eventMan;
        commitCommand = comm;
    }

    public void outputDone() {
        if (commitInformation != null) {
            eventManager.fireCVSEvent(new FileInfoEvent(this, commitInformation));
            commitInformation = null;
        }
    }

    public void parseLine(String line, boolean isErrorMessage)
    {
        if (line.startsWith(UNKNOWN))
        {
            processUnknownFile(line.substring(UNKNOWN.length()));
            return;
        }
        if (line.startsWith(ADD))
        {
            processToAddFile(line.substring(ADD.length()));
            return;
        }
        if (line.startsWith(CHECKING_IN))
        {
            processFile(line.substring(CHECKING_IN.length(), line.length() - 1));
            // - 1 means to cut the ';' character
            if (isAdding)
            {
                commitInformation.setType(commitInformation.ADDED);
                isAdding = false;
            } else
            {
                commitInformation.setType(commitInformation.CHANGED);
            }
            return;
        }
        if (line.startsWith(REMOVING))
        {
            processFile(line.substring(REMOVING.length(),line.length() - 1));
            // - 1 means to cut the ';' character
            commitInformation.setType(commitInformation.REMOVED);
            return;
        }
        if (line.startsWith(EXAM_DIR)) {
            fileDirectory = line.substring(21);
            return;
        }
        if (line.startsWith(RCS_FILE))
        {
            isAdding = true;
        }
        if (line.startsWith(DONE))
        {
            outputDone();
        }
        if (line.startsWith(INITIAL_REVISION))
        {
            processRevision(line.substring(INITIAL_REVISION.length()));
        }
        if (line.startsWith(NEW_REVISION))
        {
            processRevision(line.substring(NEW_REVISION.length()));
        }
    }

    protected File createFile(String fileName) {
        StringBuffer path = new StringBuffer();
        path.append(commitCommand.getLocalPath());
        path.append(File.separator);
/*        if (fileDirectory == null) {
           // happens for single files only
            // (for directories, the dir name is always sent before the actual files)
            File locFile = removeCommand.getFileEndingWith(fileName);
            if (locFile == null) {
                path.append(fileName);
            } else {
                path = new StringBuffer(locFile.getAbsolutePath());
            }

        } else {
 */
//            path.append(fileDirectory);
//            path.append(File.separator);
            path.append(fileName);
//        }
        String toReturn = path.toString();
//        toReturn = toReturn.replace('/', File.separatorChar);
        return new File(path.toString());
    }


    protected void processUnknownFile(String line)
    {
        commitInformation = new CommitInformation();
        commitInformation.setType(commitInformation.UNKNOWN);
        int index = line.indexOf('\'');
        String fileName = line.substring(0, index - 1).trim();
        commitInformation.setFile(createFile(fileName));
        outputDone();
    }

    protected void processToAddFile(String line)
    {
        commitInformation = new CommitInformation();
        commitInformation.setType(commitInformation.TO_ADD);
        String fileName = line.trim();
        if (fileName.endsWith(";")) {
            fileName = fileName.substring(0, fileName.length() - 2);
        }
        commitInformation.setFile(createFile(fileName));
        outputDone();
    }

    protected void processFile(String line)
    {
        if (commitInformation == null)
        {
          commitInformation = new CommitInformation();
        }

    //    String type = line.substring(0, 1).trim();
//        commitInformation.setType(type);
//        String fileName = line.substring(2).trim();
        String fileName = line;
        if (fileName.startsWith("no file"))
        {
            fileName = fileName.substring(8);
        }
        commitInformation.setFile(createFile(fileName));
    }

    protected void processRevision(String cutLine)
    {
        int index = cutLine.indexOf(';');
        if (index >= 0)
        {
            String newRev = cutLine.substring(0, index);
            commitInformation.setRevision(newRev.trim());
            return;
        }
        String newRev = cutLine;
        commitInformation.setRevision(newRev.trim());
    }
}
