/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */
package org.netbeans.lib.cvsclient.command.commit;

import java.io.File;
import org.netbeans.lib.cvsclient.command.FileInfoContainer;


/**
 * Describes commit information for a file. This is the result of doing a
 * cvs commit command. The fields in instances of this object are populated
 * by response handlers.
 * @author  Milos Kleint
 * @version $Revision: 1.1 $
 */
public class CommitInformation extends FileInfoContainer {
    /**
     * Holds value of property file.
     */
    private File file;
    
    /** 
     * Means wheater the file was added/removed/changed
     */
    private String type;
 
    public static final String ADDED = "Added";
    public static final String REMOVED = "Removed";
    public static final String CHANGED = "Changed";
    public static final String UNKNOWN = "Unknown";
    public static final String TO_ADD = "To-be-added";

    /**
     * new revision of file. for removed hold value of "delete"
     */
    private String revision;
    
    public CommitInformation() {
    }
    
    /**
     * Getter for property file.
     * @return Value of property file.
     */
    public File getFile() {
        return file;
    }
    
    /**
     * Setter for property file.
     * @param file New value of property file.
     */
    public void setFile(File file) {
        this.file = file;
    }
    
    /** Getter for property type.
     * @return Value of property type.
     */
    public String getType() {
        return type;
    }
    
    /** Setter for property type.
     * @param type New value of property type.
     */
    public void setType(String type) {
        this.type = type;
    }

    /** Getter for property revision.
     * @return Value of property revision.
     */
    public String getRevision() {
        return revision;
    }
    
    /** Setter for property revision.
     * @param revision New value of property revision.
     */
    public void setRevision(String revision) {
        this.revision = revision;
    }
    
    
    /**
     * Return a string representation of this object. Useful for debugging.
     */
    public String toString() {
        StringBuffer buf = new StringBuffer(30);
        buf.append(type);
        buf.append("  " + ((file != null)?file.getAbsolutePath():"null"));
        return buf.toString();
    }
    
    
}