/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.lib.cvsclient.command.log;

import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.netbeans.lib.cvsclient.command.FileInfoContainer;

/**
 * Describes log information for a file. This is the result of doing a
 * cvs log command. The fields in instances of this object are populated
 * by response handlers.
 * @author  Milos Kleint
 * @version $Revision: 1.6 $
 */
public class LogInformation extends FileInfoContainer
{
    /** Holds value of property file. */
    private File file;

    /** Holds value of property repositoryFilename. */
    private String repositoryFilename;

    /** Holds value of property headRevision. */
    private String headRevision;

    /** Holds value of property branch. */
    private String branch;

    /** Holds value of property accessList. */
    private String accessList;

    /** Holds value of property keywordSubstitution. */
    private String keywordSubstitution;

    /** Holds value of property totalRevisions. */
    private String totalRevisions;

    /** Holds value of property selectedRevisions. */
    private String selectedRevisions;

    /** Holds value of property description. */
    private String description;

    /** Holds value of property locks. */
    private String locks;

    private List revisions;

    private List symbolicNames;

    public LogInformation() {
        revisions = new ArrayList();
        symbolicNames = new ArrayList();
    }

    /** Getter for property file.
     * @return Value of property file.
     */
    public File getFile() {
        return file;
    }

    /** Setter for property file.
     * @param file New value of property file.
     */
    public void setFile(File file) {
        this.file = file;
    }

    /** Getter for property repositoryFilename.
     * @return Value of property repositoryFilename.
     */
    public String getRepositoryFilename() {
        return repositoryFilename;
    }

    /** Setter for property repositoryFilename.
     * @param repositoryFilename New value of property repositoryFilename.
     */
    public void setRepositoryFilename(String repositoryFilename) {
        this.repositoryFilename = repositoryFilename;
    }

    /** Getter for property headRevision.
     * @return Value of property headRevision.
     */
    public String getHeadRevision() {
        return headRevision;
    }

    /** Setter for property headRevision.
     * @param headRevision New value of property headRevision.
     */
    public void setHeadRevision(String headRevision) {
        this.headRevision = headRevision;
    }

    /** Getter for property branch.
     * @return Value of property branch.
     */
    public String getBranch() {
        return branch;
    }

    /** Setter for property branch.
     * @param branch New value of property branch.
     */
    public void setBranch(String branch) {
        this.branch = branch;
    }


    /** Getter for property accessList.
     * @return Value of property accessList.
     */
    public String getAccessList() {
        return accessList;
    }

    /** Setter for property accessList.
     * @param accessList New value of property accessList.
     */
    public void setAccessList(String accessList) {
        this.accessList = accessList;
    }

    /** Getter for property keywordSubstitution.
     * @return Value of property keywordSubstitution.
     */
    public String getKeywordSubstitution() {
        return keywordSubstitution;
    }

    /** Setter for property keywordSubstitution.
     * @param keywordSubstitution New value of property keywordSubstitution.
     */
    public void setKeywordSubstitution(String keywordSubstitution) {
        this.keywordSubstitution = keywordSubstitution;
    }

    /** Getter for property totalRevisions.
     * @return Value of property totalRevisions.
     */
    public String getTotalRevisions() {
        return totalRevisions;
    }

    /** Setter for property totalRevisions.
     * @param totalRevisions New value of property totalRevisions.
     */
    public void setTotalRevisions(String totalRevisions) {
        this.totalRevisions = totalRevisions;
    }

    /** Getter for property selectedRevisions.
     * @return Value of property selectedRevisions.
     */
    public String getSelectedRevisions() {
        return selectedRevisions;
    }

    /** Setter for property selectedRevisions.
     * @param selectedRevisions New value of property selectedRevisions.
     */
    public void setSelectedRevisions(String selectedRevisions) {
        this.selectedRevisions = selectedRevisions;
    }

    /** Getter for property description.
     * @return Value of property description.
     */
    public String getDescription() {
        return description;
    }

    /** Setter for property description.
     * @param description New value of property description.
     */
    public void setDescription(String description) {
        this.description = description;
    }



    /** Getter for property locks.
 * @return Value of property locks.
 */
    public String getLocks() {
  return locks;
    }

    /** Setter for property locks.
 * @param locks New value of property locks.
 */
    public void setLocks(String locks) {
  this.locks = locks;
    }

    /** adds a revision info  to the LogInformation instance
     */

    public void addRevision(LogInformation.Revision newRevision) {
       revisions.add(newRevision);
    }

    /** return the all revisions attached to this log
     * (if more sophisticated method are supplied, this might get obsolete)
     */
    public List getRevisionList() {
        return revisions;
    }

    /** Search the revisions by number of revision. If not found, return null.
     */
    public LogInformation.Revision getRevision(String number) {
        Iterator it = revisions.iterator();
        LogInformation.Revision item;
        while (it.hasNext()) {
            item = (LogInformation.Revision)it.next();
            if (item.getNumber().equals(number)) {
                return item;
            }
        }
        return null;
    }

    /** adds a symbolic name to the list of names and attaches it to a revision number.
     */
    public void addSymbolicName(String symName, String revisionNumber) {
        SymName newName = new SymName();
        newName.setName(symName);
        newName.setRevision(revisionNumber);
        symbolicNames.add(newName);
    }

    public List getAllSymbolicNames() {
        return symbolicNames;
    }

    /** Search the symbolic names by number of revision. If not found, return null.
     */
    public List getSymNamesForRevision(String revNumber) {
        Iterator it = symbolicNames.iterator();
        LogInformation.SymName item;
        LinkedList list = new LinkedList();
        while (it.hasNext()) {
            item = (LogInformation.SymName)it.next();
            if (item.getRevision().equals(revNumber)) {
                list.add(item);
            }
        }
        return list;
    }

    /** Search the symbolic names by name of tag (symbolic name). If not found, return null.
     */
    public LogInformation.SymName getSymName(String symName) {
        Iterator it = symbolicNames.iterator();
        LogInformation.SymName item;
        while (it.hasNext()) {
            item = (LogInformation.SymName)it.next();
            if (item.getName().equals(symName)) {
                return item;
            }
        }
        return null;
    }


    public Revision createNewRevision(String number) {
        Revision rev = new Revision();
        rev.setNumber(number);
        return rev;
    }

    /**
     * Return a string representation of this object. Useful for debugging.
     */
    public String toString()
    {
        StringBuffer buf = new StringBuffer(30);
        buf.append("\nFile: " + ((file != null)?file.getAbsolutePath():"null"));
        buf.append("\nRepositoryFile: " + repositoryFilename);
        buf.append("\nHead revision: " + headRevision);
        return buf.toString();
    }

    public class SymName {
        private String name;
        private String revision;

        public SymName() {
        }

        public String getName() {return name;}
        public void setName(String symName) {name = symName;}
        public void setRevision(String rev) {revision = rev;}
        public String getRevision() {return revision;}
    }

    public class Revision {

        /** Constructor */
        public Revision() {
        }

        /** Holds value of property number. */
        private String number;

        /** Holds value of property date. */
        private String date;

        /** Holds value of property author. */
        private String author;

        /** Holds value of property state. */
        private String state;

        /** Holds value of property lines. */
        private String lines;

        /** Holds value of property message. */
        private String message;

        /** Holds value of property branches. */
        private String branches;


        /** Getter for property number.
         * @return Value of property number.
         */
        public String getNumber() {
            return number;
        }

        /** Setter for property number.
         * @param number New value of property number.
         */
        public void setNumber(String newNumber) {
            number = newNumber;
        }

        /** Getter for property date.
         * @return Value of property date.
         */
        public String getDate() {
            return date;
        }

        /** Setter for property date.
         * @param date New value of property date.
         */
        public void setDate(String newDate) {
            date = newDate;
        }

        /** Getter for property author.
         * @return Value of property author.
         */
        public String getAuthor() {
            return author;
        }

        /** Setter for property author.
         * @param author New value of property author.
         */
        public void setAuthor(String newAuthor) {
            author = newAuthor;
        }

        /** Getter for property state.
         * @return Value of property state.
         */
        public String getState() {
            return state;
        }

        /** Setter for property state.
         * @param state New value of property state.
         */
        public void setState(String newState) {
            state = newState;
        }

        /** Getter for property lines.
         * @return Value of property lines.
         */
        public String getLines() {
            return lines;
        }

        /** Setter for property lines.
         * @param lines New value of property lines.
         */
        public void setLines(String newLines) {
            lines = newLines;
        }

        /** Getter for property message.
         * @return Value of property message.
         */
        public String getMessage() {
            return message;
        }

        /** Setter for property message.
         * @param message New value of property message.
         */
        public void setMessage(String newMessage) {
            message = newMessage;
        }

        /** Getter for property branches.
         * @return Value of property branches.
         */
        public String getBranches() {
            return branches;
        }

        /** Setter for property branches.
         * @param branches New value of property branches.
         */
        public void setBranches(String branches) {
            this.branches = branches;
        }

    }
}
