/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.lib.cvsclient.command.remove;

    /**
     * Handles the building of remove information object and the firing of
     * events when complete objects are built.
     *
     * @author  mkleint
     */
import org.netbeans.lib.cvsclient.event.*;
import java.io.*;
import org.netbeans.lib.cvsclient.file.*;
import org.netbeans.lib.cvsclient.command.Builder;
import org.netbeans.lib.cvsclient.command.BuildableCommand;
import org.netbeans.lib.cvsclient.command.BasicCommand;





public class RemoveBuilder implements Builder
{
        /**
         * The status object that is currently being built
         */
    protected RemoveInformation removeInformation;
    
    /**
     * The event manager to use
     */
    protected EventManager eventManager;
    
        /**
         * The directory in which the file being processed lives. This is
         * relative to the local directory
         */
    protected String fileDirectory;
    
    protected RemoveCommand removeCommand;
    
    private static final String UNKNOWN = "cvs server: nothing known about";
    private static final String WARNING = "cvs server: warning: ";
    private static final String SCHEDULING = "cvs server: scheduling `";
    private static final String USE_COMMIT = "cvs server: use 'cvs commit' ";
    private static final String DIRECTORY = "cvs server: Removing ";
    private static final String STILL_IN_WORKING = "cvs server: file `";
    private static final String REMOVE_FIRST = "first";
    private static final String UNKNOWN_FILE = "?";
    
    public RemoveBuilder(EventManager eventMan, RemoveCommand comm) {
        eventManager = eventMan;
        removeCommand = comm;
    }
    
    public void outputDone() {
        if (removeInformation != null) {
            eventManager.fireCVSEvent(new FileInfoEvent(this, removeInformation));
            removeInformation = null;
        }
    }
    
    public void parseLine(String line, boolean isErrorMessage) {
        if (line.startsWith(SCHEDULING)) {
            int endingIndex = line.indexOf('\'');
            String fn = line.substring(SCHEDULING.length(), endingIndex).trim();
            addFile(fn);
            removeInformation.setRemoved(true);
            outputDone();
        }
        if (line.startsWith(UNKNOWN_FILE)) {
            addFile(line.substring(UNKNOWN_FILE.length()));
            removeInformation.setRemoved(false);
            outputDone();
        }    
        if (line.startsWith(STILL_IN_WORKING)) {
            int endingIndex = line.indexOf('\'');
            String fn = line.substring(STILL_IN_WORKING.length(), endingIndex).trim();
            addFile(fn);
            removeInformation.setRemoved(false);
            outputDone();
        }    
        // ignore the rest..
    }
    
    protected File createFile(String fileName) {
        StringBuffer path = new StringBuffer();
        path.append(removeCommand.getLocalPath());
        path.append(File.separator);
        if (fileDirectory == null) {
           // happens for single files only
            // (for directories, the dir name is always sent before the actual files)
            File locFile = removeCommand.getFileEndingWith(fileName);
            if (locFile == null) {
                path.append(fileName);
            } else {
                path = new StringBuffer(locFile.getAbsolutePath());
            }
 
        } else {
//            path.append(fileDirectory);
//            path.append(File.separator);
            path.append(fileName);
        }
        String toReturn = path.toString();
        toReturn = toReturn.replace('/', File.separatorChar);
        return new File(path.toString());
    }
    
    
    protected void addFile(String name) {
        removeInformation = new RemoveInformation();
        removeInformation.setFile(createFile(name));
    }
}
