/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.lib.cvsclient.command.tag;

import java.io.File;

import org.netbeans.lib.cvsclient.command.Builder;
import org.netbeans.lib.cvsclient.command.DefaultFileInfoContainer;
import org.netbeans.lib.cvsclient.event.EventManager;
import org.netbeans.lib.cvsclient.event.FileInfoEvent;

/**
 * @author  Thomas Singer
 * @version $Revision: 1.2 $
 */
public class TagBuilder implements Builder
{
    private static final String STATES        = "T D ? ";
    private static final String CVS_SERVER    = "cvs server: ";

    /**
     * The status object that is currently being built.
     */
    private DefaultFileInfoContainer fileInfoContainer;

    /**
     * The event manager to use.
     */
    private EventManager eventManager;

    /**
     * The local path the command run in.
     */
    private String localPath;

    public TagBuilder(EventManager eventManager, String localPath)
    {
        this.eventManager = eventManager;
        this.localPath    = localPath;
    }

    public void outputDone()
    {
        if (fileInfoContainer != null)
        {
            eventManager.fireCVSEvent(new FileInfoEvent(this, fileInfoContainer));
            fileInfoContainer = null;
        }
    }

    public void parseLine(String line, boolean isErrorMessage)
    {
        if (!line.startsWith(CVS_SERVER))
        {
            String firstChar = line.substring(0, 2);
            if (STATES.indexOf(firstChar) >= 0)
            {
                processFile(line);
            }
            else
            {
                System.err.println("Don't know how to handle the response: " + line);
            }
        }
    }

    private void processFile(String line) {
        if (fileInfoContainer == null)
        {
            fileInfoContainer = new DefaultFileInfoContainer();
        }
        fileInfoContainer.setType(line.charAt(0));

        String fileName = line.substring(2).trim();
        if (fileName.startsWith("no file"))
        {
            fileName = fileName.substring(8);
        }
        fileInfoContainer.setFile(createFile(fileName));
        eventManager.fireCVSEvent(new FileInfoEvent(this, fileInfoContainer));
        fileInfoContainer = null;
    }

    private File createFile(String fileName)
    {
        return new File(localPath, fileName);
    }


}
