/*****************************************************************************
 * Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License Version 
 * 1.0 (the "License"). You may not use this file except in compliance with 
 * the License. A copy of the License is available at http://www.sun.com/

 * The Original Code is the CVS Client Library.
 * The Initial Developer of the Original Code is Robert Greig.
 * Portions created by Robert Greig are Copyright (C) 2000.
 * All Rights Reserved.

 * Contributor(s): Robert Greig.
 *****************************************************************************/
package org.netbeans.lib.cvsclient.commandLine;

import java.lang.reflect.Method;

import org.netbeans.lib.cvsclient.command.Command;

/**
 * A factory for commands. Given a command name, and any arguments passed to
 * that command on the command line, it will return a configured Command
 * object, ready for execution.
 * @author  Robert Greig
 * @version $Revision: 1.3.2.1 $
 * @see org.netbeans.lib.cvsclient.command.Command
 */
public class CommandFactory
{
    public static Command getCommand(String commandName, String[] args,
                                     int startingIndex)
        throws IllegalArgumentException
    {
        Class helper;
        try
        {
            helper = Class.forName("org.netbeans.lib.cvsclient.commandLine." +
                                   "command." + commandName);
        }
        catch (Exception e)
        {
//            System.err.println("Exception is: " + e);
            throw new IllegalArgumentException("Unknown command " + 
                                               commandName);
        }
        
        // the method invoked can throw an exception
        try
        {
            Method m = helper.getMethod("createCommand", new Class[]{
                                                         String[].class,
                                                         Integer.class});
            return (Command) m.invoke(null, new Object[]{args, new Integer(
                                                             startingIndex)});
        }
        catch (IllegalArgumentException e)
        {            
            throw e;
        }
        catch (Exception e)
        {
            throw new IllegalArgumentException(e.getMessage());
        }
    }    
}