/*
 * ModuleExpansionEvent.java
 *
 * Created on 24 February 2001, 01:27
 */

package org.netbeans.lib.cvsclient.event;

/**
 * This event is really intended only for the use in the Checkout command.
 * During a checkout command, the client must ask the server to expand modules
 * to determine whether there are aliases defined for a particular module.
 * The client must then use the expansion to determine if a local directory
 * exists and if so, send appropriate Modified requests etc.
 * @author Robert Greig
 * @version $Revision: 1.1 $
 */
public class ModuleExpansionEvent extends CVSEvent
{
    /**
     * The expanded module name
     */
    private String module;
    
    /**
     * Creates new ModuleExpansionEvent.
     * @param source the source of the event
     * @param theModule the module name that the original request has been
     * "expanded" to.
     */
    public ModuleExpansionEvent(Object source, String theModule)
    {
        super(source);
        module = theModule;
    }
    
    /**
     * Get the module name that the original module name has been expanded to.
     * @return the expanded name
     */
    public String getModule()
    {
        return module;
    }
    
    /**
     * Fire the event to the event listener. Subclasses should call the
     * appropriate method on the listener to dispatch this event.
     * @param listener the event listener
     */
    protected void fireEvent(CVSListener listener)
    {
        listener.moduleExpanded(this);
    }
}
