/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */
package org.netbeans.lib.cvsclient.request;

/**
 * The global options request. Sends global switch to the server.
 * @author  Milos Kleint
 * @version $Revision: 1.3 $
 */
public class GlobalOptionRequest extends Request
{
    /**
     * The option to pass in
     */
    private String option;
    
    /**
     * Create a new request
     * @param theOption the option to use
     */
    public GlobalOptionRequest(String theOption)
    {
        option = theOption;
    }
    
    /**
     * Set the option
     */
    public void setOption(String theOption)
    {
        option = theOption;
    }
    
    /**
     * Get the request String that will be passed to the server 
     * @return the request String
     */
    public String getRequestString() throws UnconfiguredRequestException
    {
        if (option == null)
        {
            throw new UnconfiguredRequestException(
                                "global option has not been set");
        }
        
        return "Global_option " + option + "\n";
    }
    
    /**
     * Is a response expected from the server?
     * @return true if a response is expected, false if no response if
     * expected
     */
    public boolean isResponseExpected()
    {
        return false;
    }
}