/*****************************************************************************
 * Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License Version 
 * 1.0 (the "License"). You may not use this file except in compliance with 
 * the License. A copy of the License is available at http://www.sun.com/

 * The Original Code is the CVS Client Library.
 * The Initial Developer of the Original Code is Robert Greig.
 * Portions created by Robert Greig are Copyright (C) 2000.
 * All Rights Reserved.

 * Contributor(s): Robert Greig.
 *****************************************************************************/
package org.netbeans.lib.cvsclient.request;

/**
 * Implements the Root request
 * @author  Robert Greig
 * @version $Revision: 1.3 $
 */
public final class RootRequest extends Request
{
    /**
     * The CVS root to send
     */
    private String CVSRoot;
    
    /**
     * Construct a new Root request
     * @param theCVSRoot the CVS root to use as an argument in the request
     */
    public RootRequest(String theCVSRoot)
    {
        CVSRoot = theCVSRoot;
    }
    
    /**
     * Get the request String that will be passed to the server 
     * @return the request String
     */
    public String getRequestString() throws UnconfiguredRequestException
    {
        if (CVSRoot == null)
        {
            throw new UnconfiguredRequestException(
                                "CVS root has not been set");
        }
        
        return "Root " + CVSRoot + "\n";
    }
    
    public void setCVSRoot(String root)
    {
        CVSRoot = root;
    }
    
    /**
     * Is a response expected from the server?
     * @return true if a response is expected, false if no response if
     * expected
     */
    public boolean isResponseExpected()
    {
        return false;
    }
}