/*****************************************************************************
 * Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License Version
 * 1.0 (the "License"). You may not use this file except in compliance with
 * the License. A copy of the License is available at http://www.sun.com/

 * The Original Code is the CVS Client Library.
 * The Initial Developer of the Original Code is Robert Greig.
 * Portions created by Robert Greig are Copyright (C) 2000.
 * All Rights Reserved.

 * Contributor(s): Robert Greig.
 *****************************************************************************/
package org.netbeans.lib.cvsclient.response;

import java.io.DataInputStream;
import java.io.IOException;

import org.netbeans.lib.cvsclient.event.MessageEvent;

import org.netbeans.lib.cvsclient.util.LoggedDataInputStream;

/**
 * Handles a message that the server requests is sent to stderr. Note that
 * this does not mean that an error occurred, only that the message is
 * to be sent to stderr.
 * @author  Robert Greig
 * @version $Revision: 1.9 $
 * @see org.netbeans.lib.cvsclient.response.ErrorResponse
 */
public class ErrorMessageResponse implements Response
{
    /**
     * Process the data for the response.
     * @param r the buffered reader allowing the client to read the server's
     * response. Note that the actual response name has already been read
     * and the reader is positioned just before the first argument, if any.
     */
    public void process(LoggedDataInputStream dis, ResponseServices services)
        throws ResponseException
    {
        try
        {
            final String line = dis.readLine();
            MessageEvent event = new MessageEvent(this, line, true);
            services.getEventManager().fireCVSEvent(event);
        }
        catch (IOException e)
        {
            throw new ResponseException(e);
        }
    }

    /** Is this a terminal response, i.e. should reading of responses stop
     * after this response. This is true for responses such as OK or
     * an error response
     */
    public boolean isTerminalResponse()
    {
        return false;
    }
}
