/*****************************************************************************
 * Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License Version 
 * 1.0 (the "License"). You may not use this file except in compliance with 
 * the License. A copy of the License is available at http://www.sun.com/

 * The Original Code is the CVS Client Library.
 * The Initial Developer of the Original Code is Robert Greig.
 * Portions created by Robert Greig are Copyright (C) 2000.
 * All Rights Reserved.

 * Contributor(s): Robert Greig.
 *****************************************************************************/
package org.netbeans.lib.cvsclient.response;

import java.io.DataInputStream;
import java.io.IOException;

import java.text.SimpleDateFormat;

import java.util.Locale;
import java.util.TimeZone;

import org.netbeans.lib.cvsclient.util.LoggedDataInputStream;

import org.netbeans.lib.cvsclient.admin.Entry;

/**
 * Sets the modification time of the next file sent to a specified time.
 * @author  Robert Greig
 * @version $Revision: 1.7 $
 */
public class ModTimeResponse implements Response
{

    /**
     * The formatter responsible for converting server dates to our own dates
     */
    protected static final SimpleDateFormat dateFormatter;
    
    /**
     * The way the server formats dates
     */
    protected static final String SERVER_DATE_FORMAT = "dd MMM yyyy HH:mm:ss";
    
    static
    {        
        dateFormatter = new SimpleDateFormat(SERVER_DATE_FORMAT, Locale.US);
        dateFormatter.setTimeZone(TimeZone.getTimeZone("GMT+0000"));
    }
           
    /** 
     * Process the data for the response.
     * @param dis the data inputstream allowing the client to read the server's
     * response. Note that the actual response name has already been read
     * and the input stream is positioned just before the first argument, if
     * any.
     */
    public void process(LoggedDataInputStream dis, ResponseServices services) 
        throws ResponseException
    {
        try
        {
            String date = dis.readLine();
                        
            // we assume the date is in GMT, this appears to be the case
            // We remove the ending because SimpleDateFormat does not parse
            // +xxxx, only GMT+xxxx and this avoid us having to do String
            // concat
            services.setNextFileDate(dateFormatter.parse(
                                date.substring(0, date.length()-6)));
        }
        catch (Exception e)
        {
            throw new ResponseException(e);
        }
    }
    
    /** 
     * Is this a terminal response, i.e. should reading of responses stop
     * after this response. This is true for responses such as OK or
     * an error response
     */
    public boolean isTerminalResponse()
    {
        return false;
    }
}