/*****************************************************************************
 * Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License Version 
 * 1.0 (the "License"). You may not use this file except in compliance with 
 * the License. A copy of the License is available at http://www.sun.com/

 * The Original Code is the CVS Client Library.
 * The Initial Developer of the Original Code is Robert Greig.
 * Portions created by Robert Greig are Copyright (C) 2000.
 * All Rights Reserved.

 * Contributor(s): Robert Greig.
 *****************************************************************************/
package org.netbeans.lib.cvsclient.response;

import java.io.DataInputStream;
import java.io.File;
import java.io.IOException;

import java.util.Date;
import java.util.TimeZone;                         

import org.netbeans.lib.cvsclient.admin.Entry;

import org.netbeans.lib.cvsclient.event.FileAddedEvent;
import org.netbeans.lib.cvsclient.event.FileUpdatedEvent;

import org.netbeans.lib.cvsclient.file.GzippedFileHandler;
import org.netbeans.lib.cvsclient.file.UncompressedFileHandler;

import org.netbeans.lib.cvsclient.util.LoggedDataInputStream;

/**
 * This Template response allows the server to send a template file that is
 * used when committing changes. The client tools can read the Template file
 * which is stored in CVS/Template and display it to the user to be used as a
 * prompt for commit comments.
 * @author Robert Greig
 * @version $Revision: 1.3 $
 */
public class TemplateResponse extends java.lang.Object implements Response
{
    /**
     * A reference to an uncompressed file handler
     */
    protected static UncompressedFileHandler uncompressedFileHandler;
    
    /**
     * The local path of the new file
     */
    protected String localPath;
    
    /**
     * The full repository path of the file
     */
    protected String repositoryPath;
    
    /**
     * Creates new TemplateResponse
     */
    public TemplateResponse()
    {
    }
    
    protected static UncompressedFileHandler getUncompressedFileHandler()
    {
        if (uncompressedFileHandler==null)
            uncompressedFileHandler = new UncompressedFileHandler();
        return uncompressedFileHandler;
    }
    
    /**
     * Process the data for the response.
     * @param dis the data inputstream allowing the client to read the server's
     * response. Note that the actual response name has already been read
     * and the input stream is positioned just before the first argument, if
     * any.
     */
    public void process(LoggedDataInputStream dis, ResponseServices services) 
        throws ResponseException
    {
        try
        {
            //System.err.println("Got update response.");
            localPath = dis.readLine();
            //System.err.println("LocalPath is: " + localPath);
            repositoryPath = dis.readLine();
            //System.err.println("Repository path is: " + repositoryPath);              
            int length = Integer.parseInt(dis.readLine());

            //System.err.println("File length is: " + length);
            // now read in the file
            final String filePath = services.convertPathname(localPath,
                                                             repositoryPath) +
                                                             "CVS/Template";                                    
            // just pass in null for the file's mode
            getUncompressedFileHandler().writeTextFile(filePath,
                                                       null, dis, length);
        }
        catch (IOException e)
        {
            throw new ResponseException(e);
        }
    }
    
    /**
     * Is this a terminal response, i.e. should reading of responses stop
     * after this response. This is true for responses such as OK or
     * an error response
     */
    public boolean isTerminalResponse()
    {
        return false;
    }    
}
