/*****************************************************************************
 * Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License Version 
 * 1.0 (the "License"). You may not use this file except in compliance with 
 * the License. A copy of the License is available at http://www.sun.com/

 * The Original Code is the CVS Client Library.
 * The Initial Developer of the Original Code is Robert Greig.
 * Portions created by Robert Greig are Copyright (C) 2000.
 * All Rights Reserved.

 * Contributor(s): Robert Greig.
 *****************************************************************************/
package org.netbeans.lib.cvsclient.util;

import java.io.DataInputStream;
import java.io.FilterInputStream;
import java.io.InputStream;
import java.io.IOException;

/**
 * This input stream worked exactly like the normal DataInputStream except that
 * it logs anything read to a file
 * @author  Robert Greig
 * @version $Revision: 1.4 $
 */
public class LoggedDataInputStream extends FilterInputStream
{
    /**
     * The underlying stream
     */
    private DataInputStream dis;
    
    /**
     * Construct a logged stream using the specified underlying stream
     * @param in the stream
     */
    public LoggedDataInputStream(InputStream in)
    {
        // we don't really care about the stream being created here
        super(in);
        dis = new DataInputStream(in);       
    }
    
    /**
     * Read a line (up to the newline character) from the stream, logging
     * it too
     */
    public String readLine() throws IOException
    {        
        final String value = dis.readLine();
        Logger.logInput(value + "\n");
        return value;
    }
    
    /**
     * Closes this input stream and releases any system resources associated 
     * with the stream. 
     */
    public void close() throws IOException
    {
        dis.close();
    }           

    /**
     * Reads up to byte.length bytes of data from this input stream into an 
     * array of bytes. 
     */
    public int read(byte[] b) throws IOException
    {
       return dis.read(b);       
    }

    /**
     * Reads up to len bytes of data from this input stream into an array of
     * bytes 
     */
    public int read(byte[] b, int off, int len) throws IOException
    {        
        return dis.read(b, off, len);        
    }
     
    /**
     * Read a single byte from this stream
     */
    public byte readByte() throws IOException
    {        
        byte val = dis.readByte();
        Logger.logInput((char)val);
        return val;
    }
    
    public InputStream getUnderlyingStream()
    {        
        return in;
    }
    
    public void setUnderlyingStream(InputStream is)
    {
        in = is;        
        dis = new DataInputStream(in);        
    }
}