/*****************************************************************************
 * Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License Version 
 * 1.0 (the "License"). You may not use this file except in compliance with 
 * the License. A copy of the License is available at http://www.sun.com/

 * The Original Code is the CVS Client Library.
 * The Initial Developer of the Original Code is Robert Greig.
 * Portions created by Robert Greig are Copyright (C) 2000.
 * All Rights Reserved.

 * Contributor(s): Robert Greig.
 *****************************************************************************/
package org.netbeans.lib.cvsclient.util;

import java.io.DataOutputStream;
import java.io.FilterOutputStream;
import java.io.OutputStream;
import java.io.IOException;

/**
 * A data output stream that also logs everything sent to a Writer (via the
 * logger).
 * @author  Robert Greig
 * @version $Revision: 1.4 $
 */
public class LoggedDataOutputStream extends FilterOutputStream
{
    /**
     * The underlying stream
     */
    private DataOutputStream dos;
    
    /**
     * Construct a logged stream using the specified underlying stream
     * @param in the stream
     */
    public LoggedDataOutputStream(OutputStream out)
    {
        // we don't really care about the stream being created here
        super(out);
        dos = new DataOutputStream(out);
    }
    
    /**
     * Write a line (adding a newline character) to the stream, logging
     * it too
     */
    public void writeChars(String line) throws IOException
    {
        dos.writeChars(line);
        Logger.logOutput(line);        
    }
    
    /**
     * Write a line (adding a newline character) to the stream, logging
     * it too
     */
    public void writeBytes(String line) throws IOException
    {
        dos.writeBytes(line);
        Logger.logOutput(line);        
    }
    
    /**
     * Closes this input stream and releases any system resources associated 
     * with the stream. 
     */
    public void close() throws IOException
    {
        dos.close();
    }     
    
    public OutputStream getUnderlyingStream()
    {
        return out;
    }
    
    public void setUnderlyingStream(OutputStream os)
    {
        out = os;
        dos = new DataOutputStream(out);
    }
}