/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient;

import java.text.*;
import java.awt.event.ActionEvent;
import java.util.*;
import javax.swing.JMenuItem;
import org.openide.awt.JMenuPlus;
import javax.swing.JMenu;
import javax.swing.event.*;

import org.openide.awt.Actions;
import org.openide.awt.JInlineMenu;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.*;
import org.openide.filesystems.FileObject;
import org.openide.nodes.*;
import org.openide.loaders.*;
import org.openide.filesystems.*;

import org.netbeans.modules.vcscore.util.Table;


import org.netbeans.modules.cvsclient.util.*;
import org.netbeans.modules.cvsclient.commands.*;

/** Action that displays a submenu when presented in a menu or popup.
 *
 * @author  mkleint
 */

public class CvsClientAction extends NodeAction 
                             implements Presenter.Menu, Presenter.Popup, 
                                        java.awt.event.ActionListener,
                                        javax.swing.event.MenuKeyListener {
    private Debug E=new Debug("CvsClientAction", true); // NOI18N
    private Debug D=E;    
    
    boolean CTRL_Down = false;
    private ArrayList switchableList;
    
    protected JavaCvsFileSystem fileSystem = null;
    
    static final long serialVersionUID = -627768697097225916L;

    protected Collection selectedFileObjects = null;
    
    public CvsClientAction() {
        this(null);
    }

    public CvsClientAction(Collection fos) {
        setSelectedFileObjects(fos);
    }
    
    
    protected void setSelectedFileObjects(Collection fos) {
        selectedFileObjects = fos;
    }
    
    
    public void setFileSystem(JavaCvsFileSystem fs) {
        this.fileSystem = fs;
    }
    
    protected void performAction(Node[] activatedNodes) {
    }
    
    public String getName () {
        return g("CvsClientAction.name");
    }
    
    protected String iconResource () {
        return null; //"SampleActionIcon.gif";
    }
    
    public HelpCtx getHelpCtx () {
        return HelpCtx.DEFAULT_HELP;
        // If you will provide context help then use:
        // return new HelpCtx (SampleAction.class);
    }
    
    
    /**
     * Get a menu item that can present this action in a <code>JMenu</code>.
     */
    public JMenuItem getMenuPresenter() {
        return getPresenter(true);
    }
    
    /**
     * Get a menu item that can present this action in a <code>JPopupMenu</code>.
     */
    public JMenuItem getPopupPresenter() {
        return getPresenter(false);
    }
    
    public JMenuItem getPresenter(boolean isMenu){
        JInlineMenu inlineMenu = new JInlineMenu();
        JMenu menu=new JMenuPlus(g("CvsClientAction.displayName")); // NOI18N
        switchableList = new ArrayList();
        menu.addMenuKeyListener(this);
        if (isMenu) {
            menu.setIcon(getIcon());
        }
        menu.addMenuListener(new MenuListener() {
            public void menuDeselected(MenuEvent e) {
//                deselectedMenu();
//                System.out.println("menu deselected");
            }    
            public void menuCanceled(MenuEvent e) {
//                deselectedMenu();
//                System.out.println("menu canceled");
            }    
            public void menuSelected(MenuEvent e) {
                deselectedMenu();
//                System.out.println("Selected menu");
            }    
        });    
        HelpCtx.setHelpIDString (menu, CvsClientAction.class.getName ());
        JMenuItem item=null;
        JMenu submenu = null;
        boolean isOnDir = isOnDirectory();
        boolean isOnRoot = isOnRoot();
        item=createItem("CvsClientAction.refresh", false); // NOI18N
        menu.add(item);
        if (isOnDir) {
            item=createItem("CvsClientAction.refreshRecursively", false); // NOI18N
            menu.add(item);
        }
        menu.addSeparator();
        
        item = createItem("CvsClientAction.Commit", false);
        menu.add(item);
        item = createItem("CvsClientAction.Update", true);
        menu.add(item);
        item = createItem("CvsClientAction.Diff", true);
        menu.add(item);
        item = createItem("CvsClientAction.Status", true);
        menu.add(item);
        item = createItem("CvsClientAction.Log", true);
        menu.add(item);
        menu.addSeparator();
        item=createItem("CvsClientAction.Checkout", true); // NOI18N
        menu.add(item);
        menu.addSeparator();

        if (!isOnRoot) {
            item=createItem("CvsClientAction.Add", true); // NOI18N
            menu.add(item);
            item=createItem("CvsClientAction.Remove", true); // NOI18N
            menu.add(item);
        }
        item=createItem("CvsClientAction.Tag", false); // NOI18N
        menu.add(item);
        menu.addSeparator();
        item = createItem("CvsClientAction.ListModules", false);
        menu.add(item);
        menu.addSeparator();
        if (!isOnRoot && !isOnDir) {
            item = createItem("CvsClientAction.RevisionExplorer", false);
            menu.add(item);
        }
        JMenuItem[] menus = new JMenuItem[1];
        menus[0] = menu;
        inlineMenu.setMenuItems(menus);
        return inlineMenu;
    }
    
    protected boolean enable(Node[] selNodes) {
        return true;
    }
    
    
    
    /**
     * Get the path of a node.
     * @param n the node to retrieve the path of
     * @return the path
     */
    protected String getNodePath(Node n){
        StringBuffer sb=new StringBuffer(80);
        Node parent=null;
        while( (parent=n.getParentNode())!= null ){
            if(parent.getParentNode()!=null){
                sb.insert(0,"/"); // NOI18N
                sb.insert(0,n.getName());
            }
            n=parent;
        }
        int len=sb.length();
        if( len<=1 ){
            return ""; // NOI18N
        }
        int from=(sb.charAt(0)=='/' ? 1:0);
        int to=(sb.charAt(len-1)=='/' ? len-1:len);
        String res=sb.substring(from,to);
        return res;
    }
    
    
    /**
     * Test if all of the selected nodes are directories.
     * @return <code>true</code> if all selected nodes are directories,
     *         <code>false</code> otherwise.
     */
    protected boolean isOnDirectory() {
        if (selectedFileObjects != null) {
            for (Iterator it = selectedFileObjects.iterator(); it.hasNext(); ) {
                FileObject fo = (FileObject) it.next();
                if (!fo.isFolder()) return false;
            }
            return selectedFileObjects.size() > 0;
        }
        Node[] nodes = getActivatedNodes();
        for (int i = 0; i < nodes.length; i++) {
            DataObject dd = (DataObject) (nodes[i].getCookie(DataObject.class));
            if (dd != null && !dd.getPrimaryFile().isFolder()) return false;
        }
        return nodes.length > 0;
    }

    /**
     * Test if all selected nodes are files.
     * @return <code>true</code> if all selected nodes are files,
     *         <code>false</code> otherwise.
     */
    protected boolean isOnFile() {
        if (selectedFileObjects != null) {
            for (Iterator it = selectedFileObjects.iterator(); it.hasNext(); ) {
                FileObject fo = (FileObject) it.next();
                if (fo.isFolder()) return false;
            }
            return selectedFileObjects.size() > 0;
        }
        Node[] nodes = getActivatedNodes();
        for (int i = 0; i < nodes.length; i++) {
            DataObject dd = (DataObject) (nodes[i].getCookie(DataObject.class));
            if (dd != null && dd.getPrimaryFile().isFolder()) return false;
        }
        return nodes.length > 0;
    }

    /**
     * Test if the selected node is the root node.
     * @return <code>true</code> if the selected node is the root node,
     *         <code>false</code> otherwise.
     */
    protected boolean isOnRoot() {
        if (selectedFileObjects != null) {
            for (Iterator it = selectedFileObjects.iterator(); it.hasNext(); ) {
                FileObject fo = (FileObject) it.next();
                if (fo.getPackageNameExt('/', '.').length() != 0) return false;
            }
            return selectedFileObjects.size() > 0;
        }
        Node[] nodes = getActivatedNodes();
        for (int i = 0; i < nodes.length; i++) {
            DataObject dd = (DataObject) (nodes[i].getCookie(DataObject.class));
            if (dd == null) return false;
            String path = dd.getPrimaryFile().getPackageNameExt('/','.');
            //String path = getNodePath(nodes[i]);
            if (path.length() != 0) return false;
        }
        return nodes.length > 0;
    }

    /**
     * Add files marked as important.
     * @param dd the data object from which the files are read.
     * @param res the <code>Table</code> of path and FileObject pairs which are important.
     * @param all whether to add unimportant files as well
     */
    protected void addImportantFiles(Collection fos, Table res, boolean all) {
        addImportantFiles(fos, res, all, fileSystem);
    }

    /**
     * Add files.
     * @param dd the data object from which the files are read.
     * @param res the <code>Table</code> of path and FileObject pairs.
     * @param all whether to add unimportant files as well
     * @param fileSystem the file system
     */
    public static void addImportantFiles(Collection fos, Table res, boolean all, JavaCvsFileSystem fileSystem) {
        for(Iterator it = fos.iterator(); it.hasNext(); ) {
            FileObject ff = (FileObject) it.next();
            try {
                if (ff.getFileSystem() != fileSystem)
                    continue;
            } catch (FileStateInvalidException exc) {
                continue;
            }
            String fileName = ff.getPackageNameExt('/','.');
            if (all || fileSystem.isImportant(fileName)) {
                res.put(fileName, ff);
            }
        }
    }
    
    //-------------------------------------------
    private JMenuItem createItem(String name, boolean switchable){
        JMenuItem item=null ;
        
        item=new JMenuItem(g(name));
        item.setActionCommand(name);
        item.addActionListener(this);
//        item.addMenuKeyListener(this);
        assignHelp (item, name);
        if (switchable && switchableList != null && fileSystem.getUiMode() == JavaCvsFileSystem.MODE_NOVICE) {
            switchableList.add(item);
        }    
        return item;
    }
    
    private void assignHelp (JMenuItem item, String commandName) {
        HelpCtx.setHelpIDString (item, CvsClientAction.class.getName () + "." + commandName); // NOI18N
    }
    

        //-------------------------------------------
    String g(String s) {
        return NbBundle.getBundle
        ("org.netbeans.modules.cvsclient.Bundle").getString (s);
    }
    String  g(String s, Object obj) {
        return MessageFormat.format (g(s), new Object[] { obj });
    }
    String g(String s, Object obj1, Object obj2) {
        return MessageFormat.format (g(s), new Object[] { obj1, obj2 });
    }
    String g(String s, Object obj1, Object obj2, Object obj3) {
        return MessageFormat.format (g(s), new Object[] { obj1, obj2, obj3 });
    }
    
    
    public void actionPerformed(java.awt.event.ActionEvent e){
        D.deb("actionPerformed() =" + e.getActionCommand());
        Table fls = new Table();
        boolean processAll = false; //TODO.. if the fs gets switch to process all, 
                                     //eg. *.class as well.. set it to it's value
        Set foSet = new HashSet();
        if (selectedFileObjects != null) {
            addImportantFiles(selectedFileObjects, fls, processAll);
            foSet.addAll(selectedFileObjects);
        } else {
            final Node[] nodes = getActivatedNodes();
            for(int i = 0; i < nodes.length; i++) {
                //D.deb("nodes["+i+"]="+nodes[i]); // NOI18N
                DataObject dd = (DataObject) (nodes[i].getCookie(DataObject.class));
                if (dd != null) {
                    addImportantFiles(dd.files(), fls, processAll);
                    foSet.addAll(dd.files());
                }
                else continue;
            }
        }
        Set files = fls.keySet();
        Object[] foArray = foSet.toArray();
        Vector vec = fileSystem.getImportantFiles(foArray);
        String cmd = e.getActionCommand();
        if (cmd.startsWith("CvsClientAction.refresh")) {
            fileSystem.doRefresh(vec.toArray(),cmd.equals("CvsClientAction.refreshRecursively"));
        }
        if (cmd.equals("CvsClientAction.Status")) {
            fileSystem.doStatus(vec.toArray(), CTRL_Down);
        }
        if (cmd.equals("CvsClientAction.Log")) {
            fileSystem.doLog(vec.toArray(), CTRL_Down);
        }        
        if (cmd.equals("CvsClientAction.Diff")) {
            fileSystem.doDiff(vec.toArray(), CTRL_Down);
        }        
        if (cmd.equals("CvsClientAction.Update")) {
            fileSystem.doUpdate(vec.toArray(), CTRL_Down);
        }        
        if (cmd.equals("CvsClientAction.Checkout")) {
            fileSystem.doCheckout(vec.toArray(), CTRL_Down);
        }        
        if (cmd.equals("CvsClientAction.ListModules")) {
            fileSystem.doListModules();
        }        
        if (cmd.equals("CvsClientAction.Add")) {
            fileSystem.doAdd(vec.toArray(), CTRL_Down);
        }
        if (cmd.equals("CvsClientAction.Remove")) {
            fileSystem.doRemove(vec.toArray(), CTRL_Down);
        }
        if (cmd.equals("CvsClientAction.Tag")) {
            fileSystem.doTag(vec.toArray(), CTRL_Down);
        }
        if (cmd.equals("CvsClientAction.Commit")) {
            fileSystem.doCommit(vec.toArray(), CTRL_Down);
        }        
        if (cmd.equals("CvsClientAction.RevisionExplorer")) {
            fileSystem.doRevisionExplorer(vec.toArray());
        }        
        CTRL_Down = false; // HACK :(
    }
    
    public void menuKeyTyped(final javax.swing.event.MenuKeyEvent p1) {
    }    
// HACK :(
    public void menuKeyPressed(final javax.swing.event.MenuKeyEvent p1) {
        boolean newCTRL_Down = (p1.CTRL_MASK == p1.getModifiers());
        changeCtrlSigns(newCTRL_Down);
        CTRL_Down = newCTRL_Down;
    }    
// HACK :(
    public void menuKeyReleased(final javax.swing.event.MenuKeyEvent p1) {
        boolean newCTRL_Down = (p1.CTRL_MASK == p1.getModifiers());
        changeCtrlSigns(newCTRL_Down);
        CTRL_Down = newCTRL_Down;
    }    

    private void deselectedMenu() {
        changeCtrlSigns(false);
        CTRL_Down = false;
    }   

    private void changeCtrlSigns(boolean newValue) {
        if (newValue == CTRL_Down) return;
//        System.out.println("new value = " + newValue);
        Iterator it = switchableList.iterator();
        while (it.hasNext()) {
            JMenuItem item = (JMenuItem)it.next();
            String text = item.getText();
            if (newValue) {
                // do turn ctrl sign on
                if (!text.endsWith("+")) {text = text + "+";}
            }   else { 
                // turn it off - ctrl released
                if (text.endsWith("+")) {text = text.substring(0,text.length() - 1);}
            }    
            item.setText(text);
        }    
    }    
}
