/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient;

import java.beans.*;
import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.openide.util.NbBundle;

/** Property editor for displayType property of the JavaCvsFileSystem
*
* @author Milos Kleint

*/
public class DisplayTypePropertyEditor extends PropertyEditorSupport {

    private static final java.util.ResourceBundle bundle = NbBundle.getBundle(DisplayTypePropertyEditor.class);

    /** localized string*/
    private final static String FORMATTED = bundle.getString("DisplayTypePropertyEditor.formatted");

    /** localized string*/
    private final static String SIMPLE = bundle.getString("DisplayTypePropertyEditor.simple");


    /** array of hosts */
    private static final String[] modes = {FORMATTED, SIMPLE};

    /** @return names of the supported LookAndFeels */
    public String[] getTags() {
        return modes;
    }

    /** @return text for the current value */
    public String getAsText () {
        Integer mode = (Integer) getValue();
        if (mode.intValue() == 0 ) {
            return SIMPLE;
        } 
       else {
            return FORMATTED;
        }
    }

    /** @param text A text for the current value. */
    public void setAsText (String text) {
        if (text.equals(SIMPLE)) {
            setValue(new Integer(0));
            return;
        }
        if (text.equals(FORMATTED)) {
            setValue(new Integer(1));
            return;
        }
        throw new IllegalArgumentException ();
    }

    public void setValue(Object value) {
        super.setValue(value);
    }
}

