/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient;

import org.netbeans.lib.cvsclient.*;
import org.netbeans.lib.cvsclient.connection.*;
import org.netbeans.lib.cvsclient.file.*;
import org.netbeans.lib.cvsclient.response.*;
import org.netbeans.lib.cvsclient.command.*;
import org.netbeans.lib.cvsclient.admin.StandardAdminHandler;
import org.netbeans.lib.cvsclient.event.*;
import org.netbeans.lib.cvsclient.util.IgnoreFileFilter;

import org.netbeans.modules.cvsclient.passwd.*;
import org.netbeans.modules.cvsclient.commands.*;
import org.netbeans.modules.vcscore.util.*;
import org.netbeans.modules.vcscore.cache.CacheHandler;
import org.netbeans.modules.cvsclient.caching.JavaCvsCache;
import org.netbeans.modules.cvsclient.caching.CvsFsCache;
import java.io.*;
import org.openide.util.NbBundle;
import org.openide.util.SharedClassObject;
import java.text.MessageFormat;


/**
 *
 * @author  mkleint
 * @version 
 */
public class IndependantClient implements ClientProvider, Serializable {
    private Debug E=new Debug("IndependantClient", true); // NOI18N
    private Debug D=E;

    private GlobalOptions globalOptions;
    private String cvsRoot;
    private String localPath;
    private boolean offLine;
    private int displayType;
    private int uiMode;
    private IgnoreFileFilter fileFilter;
    
    private static final String CVS_DIR = "CVS";
    private static final String ROOT_FILE = "Root";
    private static final String CVS_ROOTFILE = File.separator  + CVS_DIR + File.separator + ROOT_FILE;

    /** Creates new IndependantClient */
    public IndependantClient(String root) {
        this();
        globalOptions = new GlobalOptions();
        globalOptions.setCVSRoot(root);
        setCvsRootString(root);
    }
    
    public IndependantClient() {
        initCache();
        // making sure cache exists..
    }
    
    private void initCache() {
        CacheHandler handler = CacheHandler.getInstance();
        JavaCvsCache cache = (JavaCvsCache)handler.getCache(JavaCvsCache.JAVA_CACHE_NAME);
        if (cache == null) {
            cache = CvsFsCache.getCacheObject();
            handler.registerCacheType(JavaCvsCache.JAVA_CACHE_NAME, cache);
        }
    }


    protected String readRoot(File dirFile) {
        if (dirFile.exists()) {
            BufferedReader buff = null;
            String line = null;
            try {
                buff = new BufferedReader(new InputStreamReader(
                                             new FileInputStream(dirFile.getAbsolutePath())));
                line = buff.readLine();
            } catch (IOException exc) { //doesn't matter - nothing will be filled in
                line = null;
            }
            finally {
                if (buff != null) {
                    try {
                        buff.close();
                    } catch (IOException eIO) {}
                }
            }
            return line;
        }
        else {
            return null;
        }
    }
    
    private PServerConnection setupConnection(PasswdEntry entry) throws AuthenticationException {
       PServerConnection conn = new PServerConnection();
       conn.setUserName(entry.getUser());
       conn.setHostName(entry.getServer());
       conn.setRepository(entry.getRoot());
       conn.setEncodedPassword(entry.getPasswd());
       conn.open();
       return conn;
    }
    
    public void setLocalPath(String path) {
        localPath = path;
    }
    
    public String getLocalPath() {
        return localPath;
    }
    
    public Client createClient(String root) throws ClientCreationException {
            Client myClient = new Client(null, new StandardAdminHandler());    
            myClient.setLocalPath(localPath);
            myClient.setIgnoreFileFilter(getIgnoreFilter());
            return myClient;
    }
    
    public void openConnection(Client client, String root) throws AuthenticationException {
            CVSPasswd pass = new CVSPasswd();
            pass.loadPassFile();
            PasswdEntry entry = pass.find(root);
            if (entry == null) {
                ClientCreationException exc = new ClientCreationException("",
                   NbBundle.getMessage(IndependantClient.class, "IndependantClient.notInPassFile"));
                throw exc;
            }   //TODO - not found in .cvspass file - ignore or show error??
            D.deb("doRefreshFor() entry =" + entry.getEntry(true));
            client.setConnection(setupConnection(entry));
    }

    public JavaCvsCommandFactory getCommandFactory() {
        return JavaCvsCommandFactory.getInstance();
    }
    
    public boolean isOffLine() {
        return offLine;
    }
    
    public void setOffLine(boolean offLine) {
        this.offLine = offLine;
    }
    
    public boolean isFileSystemClient() {
        return false;
    }
    
    public GlobalOptions getGlobalOptions() {
        return globalOptions;
    }
    
    public void setGlobalOptions(GlobalOptions opt) {
        globalOptions = opt;
    }
    
    public int getUiMode() {
        return uiMode;
    }
    
    public void setUiMode(int mode) {
        uiMode = mode;
    }
    
    public String getCvsRootString() {
        return cvsRoot;
    }
    
    public void setCvsRootString(String root) {
        cvsRoot = root;
    }
    
    public int getDisplayType() {
        return displayType;
    }
    
    public void setDisplayType(int type) {
        displayType = type;
    }
    
    
        //------------------------------------------------------------------------------
    // ---------------- serialization stuff
    //-------------------------------------------
    private void readObject(ObjectInputStream in) throws
    ClassNotFoundException, IOException, NotActiveException{
        in.defaultReadObject();
        E=new Debug("IndependantClient", true); // NOI18N
        D=E; //transient??
    }
    
    
    //-------------------------------------------
    private void writeObject(ObjectOutputStream out) throws IOException {
        out.defaultWriteObject();
    }
    
    public IgnoreFileFilter getIgnoreFilter() {
        return fileFilter;
    }    
    public void  setIgnoreFilter(IgnoreFileFilter fileFilter) {
        this.fileFilter = fileFilter;
    }    
    
}
