/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient;

import java.awt.Image;
import java.beans.*;

import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.explorer.propertysheet.editors.DirectoryOnlyEditor;
import org.openide.filesystems.FileSystem;
import org.netbeans.modules.cvsclient.customizer.JavaCvsCustomizer;

/** Description of the file system.
 *
 * @author mkleint
 */
public class JavaCvsFileSystemBeanInfo extends SimpleBeanInfo {

    public BeanInfo[] getAdditionalBeanInfo () {
        try {
            return new BeanInfo[] { Introspector.getBeanInfo (FileSystem.class) };
        } catch (IntrospectionException ie) {
            if (Boolean.getBoolean ("netbeans.debug.exceptions"))
                ie.printStackTrace ();
            return null;
        }
    }

    
    // If you have a visual dialog to customize configuration of the file system:
    public BeanDescriptor getBeanDescriptor () {
        BeanDescriptor bd = new BeanDescriptor (JavaCvsFileSystem.class, JavaCvsCustomizer.class);
        bd.setValue(org.netbeans.modules.vcscore.VcsFileSystem.VCS_PROVIDER_ATTRIBUTE, new Boolean(true));
        return bd;
    }
    

    public PropertyDescriptor[] getPropertyDescriptors () {
        try {
            // Included only to make it a writable property (it is read-only in FileSystem):
            PropertyDescriptor readOnly = new PropertyDescriptor ("readOnly", JavaCvsFileSystem.class);
            readOnly.setDisplayName (NbBundle.getMessage (JavaCvsFileSystemBeanInfo.class, "PROP_readOnly"));
            readOnly.setShortDescription (NbBundle.getMessage (JavaCvsFileSystemBeanInfo.class, "HINT_readOnly"));
            // This could be whatever properties you use to configure the filesystem:
/*            PropertyDescriptor rootDirectory = new PropertyDescriptor ("rootDirectory", JavaCvsFileSystem.class);
            rootDirectory.setDisplayName (NbBundle.getMessage (JavaCvsFileSystemBeanInfo.class, "PROP_rootDirectory"));
            rootDirectory.setShortDescription (NbBundle.getMessage (JavaCvsFileSystemBeanInfo.class, "HINT_rootDirectory"));
            rootDirectory.setPropertyEditorClass (RootEd.class);
 */
  // working directory - together with rel mount makes up rootDirectory.
            PropertyDescriptor workingDir = new PropertyDescriptor ("workingDir", JavaCvsFileSystem.class);
            workingDir.setDisplayName (NbBundle.getMessage (JavaCvsFileSystemBeanInfo.class, "PROP_workingDir"));
            workingDir.setShortDescription (NbBundle.getMessage (JavaCvsFileSystemBeanInfo.class, "HINT_workingDir"));
          //  workingDir.setPropertyEditorClass (RootEd.class);
  // rel mount          
            PropertyDescriptor relMount = new PropertyDescriptor ("relMount", JavaCvsFileSystem.class);
            relMount.setDisplayName (NbBundle.getMessage (JavaCvsFileSystemBeanInfo.class, "PROP_relMount"));
            relMount.setShortDescription (NbBundle.getMessage (JavaCvsFileSystemBeanInfo.class, "HINT_relMount"));
  // work off-line
            PropertyDescriptor offLine = new PropertyDescriptor ("offLine", JavaCvsFileSystem.class);
            offLine.setDisplayName (NbBundle.getMessage (JavaCvsFileSystemBeanInfo.class, "PROP_offLine"));
            offLine.setShortDescription (NbBundle.getMessage (JavaCvsFileSystemBeanInfo.class, "HINT_offLine"));
  // automatic refresh
            PropertyDescriptor autoRefresh = new PropertyDescriptor ("autoRefresh", JavaCvsFileSystem.class);
            autoRefresh.setDisplayName (NbBundle.getMessage (JavaCvsFileSystemBeanInfo.class, "PROP_autoRefresh"));
            autoRefresh.setShortDescription (NbBundle.getMessage (JavaCvsFileSystemBeanInfo.class, "HINT_autoRefresh"));
            autoRefresh.setPropertyEditorClass(RefreshModePropertyEditor.class);
//            autoRefresh.setPropertyEditorClass();
  // debug - TEMPORARY
/*            PropertyDescriptor debug = new PropertyDescriptor ("debug", JavaCvsFileSystem.class);
            debug.setDisplayName ("Debug");
            debug.setShortDescription ("Debugging switch");
 */
  // send/receive zipped
            PropertyDescriptor zippedTransfer = new PropertyDescriptor ("zippedTransfer", JavaCvsFileSystem.class);
            zippedTransfer.setDisplayName (NbBundle.getMessage (JavaCvsFileSystemBeanInfo.class, "PROP_zippedTransfer"));
            zippedTransfer.setShortDescription (NbBundle.getMessage (JavaCvsFileSystemBeanInfo.class, "HINT_zippedTransfer"));
  // User Interface Mode
            PropertyDescriptor uiMode = new PropertyDescriptor ("uiMode", JavaCvsFileSystem.class);
            uiMode.setDisplayName (NbBundle.getMessage (JavaCvsFileSystemBeanInfo.class, "PROP_uiMode"));
            uiMode.setShortDescription (NbBundle.getMessage (JavaCvsFileSystemBeanInfo.class, "HINT_uiMode"));            
            uiMode.setPropertyEditorClass(UiModePropertyEditor.class);
  // Display Type
            PropertyDescriptor displayType = new PropertyDescriptor ("displayType", JavaCvsFileSystem.class);
            displayType.setDisplayName (NbBundle.getMessage (JavaCvsFileSystemBeanInfo.class, "PROP_displayType"));
            displayType.setShortDescription (NbBundle.getMessage (JavaCvsFileSystemBeanInfo.class, "HINT_displayType"));            
            displayType.setPropertyEditorClass(DisplayTypePropertyEditor.class);

            PropertyDescriptor cvsIgnore = new PropertyDescriptor ("cvsIgnoreList", JavaCvsFileSystem.class, "getCvsIgnoreList", null);
            cvsIgnore.setDisplayName (NbBundle.getMessage (JavaCvsFileSystemBeanInfo.class, "PROP_cvsIgnoreList"));
            cvsIgnore.setShortDescription (NbBundle.getMessage (JavaCvsFileSystemBeanInfo.class, "HINT_cvsIgnoreList"));
            
            return new PropertyDescriptor[] { readOnly, workingDir, relMount, offLine, autoRefresh, 
                          zippedTransfer, uiMode, displayType, cvsIgnore };
        } catch (IntrospectionException ie) {
            if (Boolean.getBoolean ("netbeans.debug.exceptions"))
                ie.printStackTrace ();
            return null;
        }
    }

    private static Image icon;
    private static Image icon32;
    public Image getIcon (int type) {
        if ((type == BeanInfo.ICON_COLOR_16x16) || (type == BeanInfo.ICON_MONO_16x16)) {
            if (icon == null)
                icon = loadImage ("/org/netbeans/modules/cvsclient/JavaCvsFileSystemIcon.gif");
            return icon;
        } else {
            if (icon32 == null)
                icon32 = loadImage ("/org/netbeans/modules/cvsclient/JavaCvsFileSystemIcon32.gif");
            return icon32;
        }
    }

    /** A directory chooser with special context help. */
    public static class RootEd extends DirectoryOnlyEditor {

        protected HelpCtx getHelpCtx () {
            return new HelpCtx (RootEd.class);
        }

    }
 

}