/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient;

import java.util.Hashtable;
import java.util.HashSet;
import java.util.HashMap;
import java.util.StringTokenizer;
import java.util.Properties;
import java.io.*;

import org.openide.util.NbBundle;
import org.openide.options.SystemOption;
import org.openide.util.HelpCtx;
import org.openide.TopManager;
import org.openide.util.Utilities;
import org.openide.util.SharedClassObject;
import org.netbeans.modules.vcscore.settings.VcsSettings;


/** Options for javacvs module
*
* @author Milos Kleint
*/
public class JavaCvsSettings extends SystemOption  {

    public static final String PROP_USE_GLOBAL         = "useGlobal"; // NOI18N
    public static final String PROP_OFFLINE            = "offLine"; // NOI18N
    public static final String PROP_UI_MODE            = "uiMode"; // NOI18N
    public static final String PROP_AUTO_REFRESH       = "autoRefresh";
    public static final String PROP_DISPLAY_TYPE       = "displayType"; // NOI18N
    public static final String PROP_CVS_HOME           = "cvsHome";
    
    private static boolean useGlobal = true;

    private static boolean offLine = false; // NOI18N

    private static int uiMode = 1; // NOI18N

    private static int autoRefresh = 0;
    
    private static int displayType = 1;
    
    private static java.io.File cvsHome;

    private static volatile org.openide.windows.InputOutput outputCVS = null; 
    
    // temporary help variable.. for use in different threads..
    private static String homeDir;

    /** javacvs settings */
/*    public static JavaCvsSettings OPTIONS = (JavaCvsSettings)SharedClassObject.findObject(
                                                JavaCvsSettings.class, true);
*/
    /** Utility field used by bound properties. */
    private java.beans.PropertyChangeSupport propertyChangeSupport =  new java.beans.PropertyChangeSupport (this);    

    static final long serialVersionUID = -1463396366224442828L;


    public JavaCvsSettings() {
        cvsHome = getHome();
    }

    /** This is a project option. */
    private boolean isGlobal() {
        return false;
    }

    /** human presentable name */
    public String displayName() {
        return NbBundle.getBundle(JavaCvsSettings.class).getString("CTL_JavaCvs_settings");
    }


    /** Reads from the serialized state */
/*    public void readExternal (ObjectInput in)
    throws IOException, ClassNotFoundException {
        super.readExternal(in);
    }
 */
   
    public void setUseGlobal(boolean global) {
        if (useGlobal != global) {
           useGlobal = global;
           firePropertyChange(PROP_USE_GLOBAL, new Boolean(!global), new Boolean(global));
        }
   }
    
    public boolean isUseGlobal() {
        return useGlobal;
    }


    public HelpCtx getHelpCtx () {
        return new HelpCtx (JavaCvsSettings.class);
    }

    
    /** Getter for property offLine.
     * @return Value of property offLine.
 */
    public boolean isOffLine() {
        return offLine;
    }
    
    /** Setter for property offLine.
     * @param offLine New value of property offLine.
 */
    public void setOffLine(boolean newOffLine) {
        if (offLine != newOffLine) {
            boolean oldOffLine = offLine;
            offLine = newOffLine;
            firePropertyChange (PROP_OFFLINE, new Boolean (oldOffLine), new Boolean (offLine));
        }
    }
    
    /** Getter for property uiMode.
     * @return Value of property uiMode.
 */
    public int getUiMode() {
        return uiMode;
    }
    
    /** Setter for property uiMode.
     * @param uiMode New value of property uiMode.
 */
    public void setUiMode(int newUiMode) {
        if (uiMode != newUiMode) {
            int oldUiMode = uiMode;
            uiMode = newUiMode;
            firePropertyChange (PROP_UI_MODE, new Integer (oldUiMode), new Integer (newUiMode));
        }
    }
    
    /** Getter for property autoRefresh.
     * @return Value of property autoRefresh.
 */
    public int getAutoRefresh() {
        return autoRefresh;
    }
    
    /** Setter for property autoRefresh.
     * @param autoRefresh New value of property autoRefresh.
 */
    public void setAutoRefresh(int newAutoRefresh) {
        if (autoRefresh != newAutoRefresh) {
            int oldAutoRefresh = autoRefresh;
            autoRefresh = newAutoRefresh;
            firePropertyChange (PROP_AUTO_REFRESH, new Integer (oldAutoRefresh), new Integer (newAutoRefresh));
        }
    }
    
        /** Getter for property displayType.
     * @return Value of property displayType.
 */
    public int getDisplayType() {
        return displayType;
    }
    
    /** Setter for property displayType.
     * @param displayType New value of property displayType.
 */
    public void setDisplayType(int newDisplayType) {
        if (displayType != newDisplayType) {
            int oldDisplayType = displayType;
            displayType = newDisplayType;
            firePropertyChange (PROP_DISPLAY_TYPE, new Integer (oldDisplayType), new Integer (newDisplayType));
        }
    }
    
    public org.openide.windows.InputOutput getCvsOutput() {
        if (outputCVS == null) {
            outputCVS = TopManager.getDefault().getIO("CVS");
        }
        return outputCVS;
    }

// Copied from CvsPasswd.java    
/** This method tries to find out the path to the .cvspass file. Generally it does just
 * look for the user.home property. On Windoze it's a bit more complicated. 
 * Due to the 2-face nature of the system (Win/DOS) we need to look for the DOS env variables.
 * these are not accessible from java alone. Thus we need to run an external java program in the command shell,
 * and it'll read the enviroment variables we need (HOME or HOMEDRIVE+HOMEPATH). 
 * if not found this way, we return the user.home at least.
 * @return the found path to the user's home directory.
 */
     public File getHome() {
      VcsSettings settings = (VcsSettings)SharedClassObject.findObject(
                                       VcsSettings.class, true); 
      String home = settings.getHome();
      if (home == null) {
          home = System.getProperty("user.home");
      }
      return new File(home);
         /*  
!!!!   Now reads from the VcsSettings object
          
        if (Utilities.isWindows()) {
//            System.out.println("Is Windows");
            int os = Utilities.getOperatingSystem();
            String cmd = "cmd /c";
            String java = System.getProperty("java.home") + File.separator + "bin" + File.separator + "java";
            String classpath = System.getProperty("netbeans.user") + File.separator + "modules" + File.separator + "javacvs.jar" + File.pathSeparator +
                               System.getProperty("netbeans.home") + File.separator + "modules" + File.separator + "javacvs.jar";
            String defs;
            defs = " -DHOME=%HOME% -DHOMEPATH=%HOMEPATH% -DHOMEDRIVE=%HOMEDRIVE%";
            Process proc=null;
            Thread stdoutThread=null;
            //Thread stderrThread=null;
            try {
                try {
                    String command = cmd+ " \"" + java + defs + " -cp " +classpath
                    + " org.netbeans.modules.cvsclient.passwd.ExternalCheckHome \"";
//                    System.out.println("Exec("+command+")");
                    proc=Runtime.getRuntime().exec(command);
                } catch (IOException e) {
//                    System.out.println("Runtime.exec failed."); // NOI18N
                    homeDir = System.getProperty("user.home");
                    return new File(homeDir);
                }
                final BufferedReader out = new BufferedReader(new InputStreamReader(proc.getInputStream()));
                stdoutThread = new Thread(
                new Runnable() {
                    public void run() {
                        try {  
                            homeDir = out.readLine();
//                            System.out.println("readLine("+homeDir+")");
                        } catch (IOException io) {
//                            System.out.println("Error reading output of ExternalCheckHome");    
                            homeDir = System.getProperty("user.home");
                        }      
                    }                               
                }
                );            
                stdoutThread.start();
                int exit = proc.waitFor();
                stdoutThread.join();
                if (homeDir == null) { 
                    homeDir =  System.getProperty("user.home");
                } else if (homeDir.equals("")) {
                    homeDir =  System.getProperty("user.home");
                }
            } catch (InterruptedException inter) {
//                System.out.println("interrupted.");
                stdoutThread.interrupt();
                long stopTimeout=1000;
                try {
                    stdoutThread.join(stopTimeout);
                } catch (InterruptedException e){
                }  
                proc.destroy();
                homeDir = System.getProperty("user.home");
            }
            return new File(homeDir);  
        }    
        else { //IT is not windows
//            System.out.println("Is NOT Windows");
            homeDir =  System.getProperty("user.home");
            return new File(homeDir);
        }  
          */
    }    

/*    public File getHome() {
        System.out.println("is at getHome!!");
        if (Utilities.isWindows()) {
           Process proc=null;
           Thread stdoutThread=null;
           Thread stderrThread=null;
            try {    
                try {
                    String loc = File.separator + "modules" + File.separator + "javacvs.jar";
                    String command = "cmd /c \"java -DHOME=%HOME% -DHOMEPATH=%HOMEPATH% -DHOMEDRIVE=%HOMEDRIVE% -cp " 
                    + System.getProperty("netbeans.user") +  loc 
                    + File.pathSeparator + System.getProperty("netbeans.home") + loc
                    + " org.netbeans.modules.cvsclient.passwd.ExternalCheckHome \"";
                    proc=Runtime.getRuntime().exec(command);
                } catch (IOException e){
                    homeDir = System.getProperty("user.home");
                }
                final BufferedReader out = new BufferedReader(new InputStreamReader(proc.getInputStream()));
                stdoutThread = new Thread(
                new Runnable() {
                    public void run() {
                        try {  
                            homeDir = out.readLine();
                        } catch (IOException io) {
                            System.out.println("Error reading output of ExternalCheckHome");    
                            homeDir = System.getProperty("user.home");
                        }      
                    }                               
                }
                );            
                stdoutThread.start();
                int exit=proc.waitFor();
                stdoutThread.join();
                if (homeDir == null) { 
                     homeDir =  System.getProperty("user.home");}
                else if (homeDir.equals("")) { 
                    homeDir =  System.getProperty("user.home");}    
            } catch (InterruptedException inter) {
                stdoutThread.stop();
                long stopTimeout=1000;
                try{
                    stdoutThread.join(stopTimeout);
                }catch (InterruptedException e){
                }  
                proc.destroy();
                homeDir =  System.getProperty("user.home");
            }     
            return new File(homeDir);  
        }    
        homeDir = System.getProperty("user.home");
        return new File(homeDir);
    }    
 */
    
    /** Getter for property cvsHome.
     * @return Value of property cvsHome.
 */
    public java.io.File getCvsHome() {
        return cvsHome;
    }
    
    /** Setter for property cvsHome.
     * @param cvsHome New value of property cvsHome.
 */
    public void setCvsHome(java.io.File newCvsHome) {
        if (newCvsHome == null) {
            newCvsHome = new File(System.getProperty("user.home"));
        }
        if (cvsHome == null || !cvsHome.equals(newCvsHome)) {
            if (!newCvsHome.exists() || !newCvsHome.isDirectory()) {
                newCvsHome = new File(System.getProperty("user.home"));
            }
            java.io.File oldCvsHome = cvsHome;
            cvsHome = newCvsHome;
            firePropertyChange (PROP_CVS_HOME, oldCvsHome, newCvsHome);
        }
    }
    
}