/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient;

import java.io.Serializable;
import java.util.*;
import org.netbeans.lib.cvsclient.file.FileStatus;
import org.openide.util.NbBundle;

/**
 *  This class takes care of everytging that's connected with status
 *
 * @author  mkleint
 * @version 
 */
public class JavaCvsStatusManager extends java.lang.Object implements Serializable {
    
    public static final String LOCAL            = "Local";
    public static final String LOCALLY_MODIFIED = "LMod";
    public static final String NEEDS_PATCH      = "NPatch";
    public static final String NEEDS_CHECKOUT   = "NCheckout";
    public static final String NEEDS_MERGE      = "NMerge";
    public static final String UPTODATE         = "Upd";
    public static final String UNKNOWN          = "Unknown";
    public static final String NOT_IN_SYNCH     = "NotInSynch";
    public static final String LOCALLY_ADDED    = "LAdded";
    public static final String LOCALLY_REMOVED  = "LRemoved";
    public static final String CONFLICT         = "Conflict";
    public static final String HAS_HIDDEN       = "HiddenFiles";
    
    private static transient JavaCvsStatusManager instance = null;
    
    private HashMap mapping;
    
    /** Creates new JavaCvsStatusManager */
    public JavaCvsStatusManager() {
        mapping = new HashMap();
        mapping.put(LOCAL, NbBundle.getMessage(JavaCvsStatusManager.class, "JavaCvsStatusManager.LOCAL"));
        mapping.put(LOCALLY_MODIFIED,  NbBundle.getMessage(JavaCvsStatusManager.class, "JavaCvsStatusManager.LOCALLY_MODIFIED"));
        mapping.put(LOCALLY_ADDED,  NbBundle.getMessage(JavaCvsStatusManager.class, "JavaCvsStatusManager.LOCALLY_ADDED"));
        mapping.put(LOCALLY_REMOVED,  NbBundle.getMessage(JavaCvsStatusManager.class, "JavaCvsStatusManager.LOCALLY_REMOVED"));
        mapping.put(UNKNOWN,  NbBundle.getMessage(JavaCvsStatusManager.class, "JavaCvsStatusManager.UNKNOWN"));
        mapping.put(UPTODATE,  NbBundle.getMessage(JavaCvsStatusManager.class, "JavaCvsStatusManager.UPTODATE"));
        mapping.put(NEEDS_CHECKOUT,  NbBundle.getMessage(JavaCvsStatusManager.class, "JavaCvsStatusManager.NEEDS_CHECKOUT"));
        mapping.put(NEEDS_MERGE,  NbBundle.getMessage(JavaCvsStatusManager.class, "JavaCvsStatusManager.NEEDS_MERGE"));
        mapping.put(NEEDS_PATCH,  NbBundle.getMessage(JavaCvsStatusManager.class, "JavaCvsStatusManager.NEEDS_PATCH"));
        mapping.put(NOT_IN_SYNCH,  NbBundle.getMessage(JavaCvsStatusManager.class, "JavaCvsStatusManager.NOT_IN_SYNCH"));
        mapping.put(CONFLICT,  NbBundle.getMessage(JavaCvsStatusManager.class, "JavaCvsStatusManager.CONFLICT"));
        mapping.put(HAS_HIDDEN, NbBundle.getMessage(JavaCvsStatusManager.class, "JavaCvsStatusManager.HAS_HIDDEN"));
    }
    
    public String getStatus(String id) {
        String toReturn = (String) mapping.get(id);
        if (toReturn == null) toReturn = " "; //TODO
        return toReturn;
    }
    
    public void setStatus(String id, String value) {
        mapping.put(id, value);
    }
    
    public String getId(String value) {
       if (!mapping.containsValue(value)) return null;
       Collection col = mapping.values();
       Set keys = mapping.keySet();
       Iterator it = col.iterator();
       Iterator keyIter = keys.iterator();
       while (it.hasNext()) {
           if (value.equals(it.next())) {
               if (keyIter.hasNext()) {
                   return (String)keyIter.next();
               }
           }
           if (keyIter.hasNext()) keyIter.next();
       }
       return null;
    }
    
    public static JavaCvsStatusManager getInstance() {
        if (instance == null) {
            instance = new JavaCvsStatusManager();
        }
        return instance;
    }
    
    public String[] getAllStates() {
       Collection col = mapping.values();
       String[] toReturn = new String[col.size()];
       toReturn = (String[])col.toArray(toReturn);
       return toReturn;
    }

    /** Converts the status from the library's format to the filesystem's 
     */
    public static String convertStatus(FileStatus stat) {
        if (stat == FileStatus.UP_TO_DATE) {
            return UPTODATE;
        }
        if (stat == FileStatus.HAS_CONFLICTS) {
            return CONFLICT;
        }
        if (stat == FileStatus.MODIFIED) {
            return LOCALLY_MODIFIED;
        }
        if (stat == FileStatus.NEEDS_CHECKOUT) {
            return NEEDS_CHECKOUT;
        }
        if (stat == FileStatus.ADDED) {
            return LOCALLY_ADDED;
        }
        if (stat == FileStatus.NEEDS_MERGE) {
            return NEEDS_MERGE;
        }
        if (stat == FileStatus.NEEDS_PATCH) {
            return NEEDS_PATCH;
        }
        if (stat == FileStatus.REMOVED) {
            return LOCALLY_REMOVED;
        }
        if (stat == FileStatus.UNKNOWN) {
            return UNKNOWN;
        }
        return "";
    }

}
