/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient;


import org.openide.util.NbBundle;
import javax.swing.DefaultComboBoxModel;
import org.netbeans.modules.vcscore.util.*;
import org.netbeans.modules.cvsclient.commands.*;
import org.netbeans.modules.cvsclient.commands.update.CvsUpdate;
import org.netbeans.modules.cvsclient.commands.add.CvsAdd;
import org.netbeans.modules.cvsclient.commands.remove.CvsRemove;
import org.netbeans.modules.cvsclient.commands.checkout.CvsCheckout;
import org.netbeans.modules.cvsclient.commands.checkout.CvsCompleteModList;
import org.netbeans.modules.cvsclient.commands.commit.CvsCommit;
import org.netbeans.modules.cvsclient.commands.diff.CvsDiff;
import org.netbeans.modules.cvsclient.commands.status.CvsStatus;
import org.netbeans.modules.cvsclient.commands.log.CvsLog;
import org.netbeans.lib.cvsclient.command.GlobalOptions;
import org.netbeans.lib.cvsclient.util.DefaultIgnoreFileFilter;
import org.netbeans.lib.cvsclient.util.IgnoreFileFilter;
import javax.swing.*;

import javax.swing.tree.*;
import java.awt.Component;
import javax.swing.event.*;
import java.io.File;
import java.io.*;
import java.util.Comparator;
import java.util.Arrays;
import java.util.LinkedList;
import org.netbeans.modules.cvsclient.customizer.JavaCvsCustomizer;
import org.openide.util.*;
import org.openide.*;

/**
 *  Starting panel for execution of filesystem independant cvs commands.
 * @author  mkleint
 */
public class ToolsCvsExecutionPanel extends javax.swing.JPanel implements TreeSelectionListener {

    private Object[] commandValues;

    /** Creates new form ToolsCvsExecutionPanel */
    public ToolsCvsExecutionPanel() {
        initComponents();
        helpTextArea.setFont (javax.swing.UIManager.getFont ("Label.font"));
        helpTextArea.setDisabledTextColor (javax.swing.UIManager.getColor ("Label.foreground"));
        helpTextArea.setText (java.util.ResourceBundle.getBundle ("org/netbeans/modules/cvsclient/Bundle").getString ("ToolsCvsExecutionPanel.helpTextArea.text"));
        lblCurrentDir.setDisplayedMnemonic (java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/Bundle").getString("ToolsCvsExecutionPanel.lblCurrentDir.mnemonic").charAt (0));
        lblCurrentDir.setLabelFor (cbCurrentDir);
        lblFiles.setDisplayedMnemonic (java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/Bundle").getString("ToolsCvsExecutionPanel.lblFiles.mnemonic").charAt (0));
        lblFiles.setLabelFor (trFiles);
        lblCommand.setDisplayedMnemonic (java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/Bundle").getString("ToolsCvsExecutionPanel.lblCommand.mnemonic").charAt (0));
        lblCommand.setLabelFor (cbCommand);
        lblCvsRoot.setDisplayedMnemonic (java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/Bundle").getString("ToolsCvsExecutionPanel.lblCvsRoot.mnemonic").charAt (0));
        lblCvsRoot.setLabelFor (txCvsRoot);
        btnCurrentDir.setMnemonic (java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/Bundle").getString("ToolsCvsExecutionPanel.btnCurrentDir.mnemonic").charAt (0));
        final String UPDATE = NbBundle.getBundle(ToolsCvsExecutionPanel.class).getString("CvsClientAction.Update");
        final String COMMIT = NbBundle.getBundle(ToolsCvsExecutionPanel.class).getString("CvsClientAction.Commit");
        final String DIFF = NbBundle.getBundle(ToolsCvsExecutionPanel.class).getString("CvsClientAction.Diff");
        final String STATUS = NbBundle.getBundle(ToolsCvsExecutionPanel.class).getString("CvsClientAction.Status");
        final String LOG = NbBundle.getBundle(ToolsCvsExecutionPanel.class).getString("CvsClientAction.Log");
        final String CHECKOUT = NbBundle.getBundle(ToolsCvsExecutionPanel.class).getString("CvsClientAction.Checkout");
        final String ADD = NbBundle.getBundle(ToolsCvsExecutionPanel.class).getString("CvsClientAction.Add");
        final String REMOVE = NbBundle.getBundle(ToolsCvsExecutionPanel.class).getString("CvsClientAction.Remove");
        final String LISTMODULES = NbBundle.getBundle(ToolsCvsExecutionPanel.class).getString("CvsClientAction.ListModules");
        commandValues = new Object[] {
            UPDATE, COMMIT, DIFF, STATUS, LOG, CHECKOUT, ADD, REMOVE, LISTMODULES };
            cbCommand.setModel(new DefaultComboBoxModel(commandValues));
        cbCurrentDir.setModel(new DefaultComboBoxModel());
        addTreeListeners();
        createTree("");
        DefaultTreeCellRenderer  def = new MyTreeCellRenderer();
        trFiles.setCellRenderer(def);
        trFiles.putClientProperty("JTree.lineStyle", "Angled");
    }
   
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        helpTextArea = new javax.swing.JTextArea();
        lblCurrentDir = new javax.swing.JLabel();
        cbCurrentDir = new javax.swing.JComboBox();
        btnCurrentDir = new javax.swing.JButton();
        lblCvsRoot = new javax.swing.JLabel();
        txCvsRoot = new javax.swing.JTextField();
        lblFiles = new javax.swing.JLabel();
        spFiles = new javax.swing.JScrollPane();
        trFiles = new javax.swing.JTree();
        commandPanel = new javax.swing.JPanel();
        lblCommand = new javax.swing.JLabel();
        cbCommand = new javax.swing.JComboBox();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        helpTextArea.setWrapStyleWord(true);
        helpTextArea.setLineWrap(true);
        helpTextArea.setEditable(false);
        helpTextArea.setEnabled(false);
        helpTextArea.setOpaque(false);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(12, 12, 5, 11);
        add(helpTextArea, gridBagConstraints1);
        
        lblCurrentDir.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/Bundle").getString("ToolsCvsExecutionPanel.lblCurrentDir.label"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(12, 12, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(lblCurrentDir, gridBagConstraints1);
        
        cbCurrentDir.setMaximumRowCount(10);
        cbCurrentDir.setEditable(true);
        cbCurrentDir.setPreferredSize(new java.awt.Dimension(300, 25));
        cbCurrentDir.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                cbCurrentDirFocusLost(evt);
            }
        });
        
        cbCurrentDir.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                cbCurrentDirItemStateChanged(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(12, 12, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints1.weightx = 1.0;
        add(cbCurrentDir, gridBagConstraints1);
        
        btnCurrentDir.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/Bundle").getString("ToolsCvsExecutionPanel.btnCurrentDir.text"));
        btnCurrentDir.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnCurrentDirActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 2;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.insets = new java.awt.Insets(12, 5, 0, 11);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(btnCurrentDir, gridBagConstraints1);
        
        lblCvsRoot.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/Bundle").getString("ToolsCvsExecutionPanel.lblCvsRoot.text"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(5, 12, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(lblCvsRoot, gridBagConstraints1);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(5, 12, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints1.weightx = 1.0;
        add(txCvsRoot, gridBagConstraints1);
        
        lblFiles.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/Bundle").getString("ToolsCvsExecutionPanel.lblFiles.text"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.insets = new java.awt.Insets(12, 12, 2, 11);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(lblFiles, gridBagConstraints1);
        
        spFiles.setPreferredSize(new java.awt.Dimension(300, 150));
        spFiles.setMinimumSize(new java.awt.Dimension(300, 150));
        spFiles.setViewportView(trFiles);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(0, 12, 0, 11);
        gridBagConstraints1.weighty = 1.0;
        add(spFiles, gridBagConstraints1);
        
        commandPanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints2;
        
        lblCommand.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/Bundle").getString("ToolsCvsExecutionPanel.lblCommand.text"));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 1;
        gridBagConstraints2.gridy = 4;
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.EAST;
        commandPanel.add(lblCommand, gridBagConstraints2);
        
        cbCommand.setMaximumRowCount(10);
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 2;
        gridBagConstraints2.gridy = 4;
        gridBagConstraints2.insets = new java.awt.Insets(0, 12, 0, 0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.EAST;
        commandPanel.add(cbCommand, gridBagConstraints2);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 5;
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.insets = new java.awt.Insets(12, 12, 11, 11);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.EAST;
        add(commandPanel, gridBagConstraints1);
        
    }//GEN-END:initComponents

   private void cbCurrentDirItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_cbCurrentDirItemStateChanged
       // Add your handling code here:
       String item = (String)cbCurrentDir.getSelectedItem();
       createTree(item);
       findCvsRoot(item);
 
    }//GEN-LAST:event_cbCurrentDirItemStateChanged

   private void cbCurrentDirFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_cbCurrentDirFocusLost
 /*      // Add your handling code here:
        DefaultComboBoxModel model = (DefaultComboBoxModel)cbCurrentDir.getModel();
        String item = (String)cbCurrentDir.getEditor().getItem();
       System.out.println("FocusLost");
       System.out.println("selItem=" + item);
        model.insertElementAt(item, 0);
        cbCurrentDir.setSelectedIndex(0);
  */
    }//GEN-LAST:event_cbCurrentDirFocusLost

    private void btnCurrentDirActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnCurrentDirActionPerformed
        // Add your handling code here:
        String selected = (String)cbCurrentDir.getSelectedItem();
        ChooseDirDialog chooseDir=new ChooseDirDialog(new JFrame(), new File(selected));
        JavaCvsCustomizer.centerWindow (chooseDir);
        HelpCtx.setHelpIDString (chooseDir.getRootPane (), ToolsCvsExecutionPanel.class.getName ());
        chooseDir.show();
        selected = chooseDir.getSelectedDir();
        if( selected==null ){
            //D.deb("no directory selected"); // NOI18N
            return ;
        }
        addToCombo(selected);
        createTree(selected);
        findCvsRoot(selected);
    }//GEN-LAST:event_btnCurrentDirActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextArea helpTextArea;
    private javax.swing.JLabel lblCurrentDir;
    private javax.swing.JComboBox cbCurrentDir;
    private javax.swing.JButton btnCurrentDir;
    private javax.swing.JLabel lblCvsRoot;
    private javax.swing.JTextField txCvsRoot;
    private javax.swing.JLabel lblFiles;
    private javax.swing.JScrollPane spFiles;
    private javax.swing.JTree trFiles;
    private javax.swing.JPanel commandPanel;
    private javax.swing.JLabel lblCommand;
    private javax.swing.JComboBox cbCommand;
    // End of variables declaration//GEN-END:variables

    private void addToCombo(String fileString) {
        DefaultComboBoxModel model = (DefaultComboBoxModel)cbCurrentDir.getModel();
        int size = model.getSize();
        for (int index = 0; index < size; index++) {
            String one = (String)model.getElementAt(index);
            if (fileString.equals(one)) { 
                return;
            }
        }
        model.insertElementAt(fileString, 0);
        cbCurrentDir.setSelectedIndex(0);
    }
    
    
    private void addTreeListeners() {
        trFiles.addTreeSelectionListener(this);
        trFiles.addTreeWillExpandListener(new TreeWillExpandListener() {
            public void treeWillExpand(TreeExpansionEvent evt) {
                TreePath path = evt.getPath();
                final MyTreeNode node = (MyTreeNode) path.getLastPathComponent();
                Runnable treeBuild = new Runnable() {
                    public void run() {
                        folderTreeNodes(node);
                    }
                };
                new Thread(treeBuild, "Mount Panel Tree Build").start();
            }
            public void treeWillCollapse(TreeExpansionEvent evt) {
                TreePath path = evt.getPath();
                final MyTreeNode node = (MyTreeNode) path.getLastPathComponent();
                node.setWasRead(false);
            }
        });
    }

    private void createTree(String rootString) {
        trFiles.setModel(new DefaultTreeModel(new MyTreeNode()));
        File rootDir = new File(rootString);
        if (rootDir.isDirectory()) {
            final MyTreeNode root = new MyTreeNode(rootDir);
//            Runnable treeBuild = new Runnable() {
//                public void run() {
                    //recursiveTreeNodes(root);
                    folderTreeNodes(root);
//                }
//            };
//            new Thread(treeBuild, "Mount Panel Tree Build").start();
            trFiles.setModel(new DefaultTreeModel(root));
        }
    }
    
    private void folderTreeNodes(final MyTreeNode parent) {
        boolean hasChild = false;
        if (parent.isWasRead()) return;
        synchronized (trFiles) {
            parent.removeAllChildren();
            parent.setWasRead(true);
            File parentFile = (File) parent.getUserObject();
            if (parentFile == null) return;
            MyTreeNode child;
            File childFile;
            File[] list = parentFile.listFiles();
            Arrays.sort(list, new MyComparator());
            for (int index = 0; index < list.length; index++) {
                if (list[index].exists()) {
                    childFile = list[index];
                    if (!childFile.getName().equals("CVS")) { //CVS dirs go out..
                        hasChild = true;
                        child = new MyTreeNode(new File(childFile.getAbsolutePath()));
                        child.setAllowsChildren(childFile.isDirectory());
                        parent.add(child);
                    }
                }
            }
            if (!hasChild) {
                parent.setAllowsChildren(false);
                trFiles.collapsePath(new TreePath(parent.getPath()));
            }
        }
        ((DefaultTreeModel) trFiles.getModel()).nodeStructureChanged(parent);
        if (hasChild) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    trFiles.scrollPathToVisible(new TreePath(((MyTreeNode) parent.getLastChild()).getPath()));
                    trFiles.scrollPathToVisible(new TreePath(((MyTreeNode) parent.getFirstChild()).getPath()));
                }
            });
        }
    }
    
    public void valueChanged(javax.swing.event.TreeSelectionEvent p1) {
/*        TreePath path = trFiles.getSelectionPath();
        if (path != null) {
            MyTreeNode node = (MyTreeNode) path.getLastPathComponent();
            File selFile = (File) node.getUserObject();
            MyTreeNode rootNode = (MyTreeNode) node.getRoot();
            File rootFile = (File) rootNode.getUserObject();
            if (rootFile.getAbsolutePath().equals(selFile.getAbsolutePath())) {
                return;
            }
            String toWrite = selFile.getAbsolutePath().substring(rootFile.getAbsolutePath().length());
            if (toWrite.startsWith(File.separator)) {
                toWrite = toWrite.substring(File.separator.length());
            }
        } else {
        }        
 */
    }
    
    public void executeCommand() {
        Object selected = cbCurrentDir.getEditor().getItem();
        addToCombo((String)selected);
        TreePath[] paths = trFiles.getSelectionPaths();
        LinkedList list = new LinkedList();
        if (paths != null) {
            for (int i = 0; i < paths.length; i++) {
                TreePath path = paths[i];
                Object fobj = path.getLastPathComponent();
                DefaultMutableTreeNode node = (DefaultMutableTreeNode)fobj;
                fobj = node.getUserObject();
                if (fobj instanceof File) {
                    list.add(fobj);
                }
            }
        }
        if (list.size() == 0) {
            String item = (String)cbCurrentDir.getSelectedItem();
            list.add(new File(item));
        }
        File[] files = new File[list.size()];
        files = (File[])list.toArray(files);
        IndependantClient client = new IndependantClient();
        GlobalOptions opt = new GlobalOptions();
        opt.setCVSRoot(txCvsRoot.getText());
        client.setGlobalOptions(opt);
        client.setCvsRootString(txCvsRoot.getText());
        JavaCvsSettings settings = (JavaCvsSettings)SharedClassObject.findObject(
                                                JavaCvsSettings.class, true);         
        client.setUiMode(settings.getUiMode());
        client.setDisplayType(settings.getDisplayType());
        String localPath = (String)cbCurrentDir.getSelectedItem();
        client.setLocalPath(localPath);
        // load ignore list..
        java.util.List ignoreList = JavaCvsFileSystem.constructIgnoreList(settings.getCvsHome(), 
                                    new java.io.File(localPath, "CVSROOT"));
        IgnoreFileFilter ignoreFilter = new DefaultIgnoreFileFilter(ignoreList);
        client.setIgnoreFilter(ignoreFilter);
        FileSystemCommand command = createCommand(client, files);
        if (command != null) {
            command.setFiles(files);
            command.startCommand();
        } else {
            //TODO error.
        }
    }
    
    private FileSystemCommand createCommand(IndependantClient client, File[] files) {
        int index = cbCommand.getSelectedIndex();
        switch (index) {
            case 0 : return new CvsUpdate(files, client);
            case 1 : return new CvsCommit(files, client);
            case 2 : return new CvsDiff(files, client);
            case 3 : return new CvsStatus(files, client);
            case 4 : return new CvsLog(files, client);
            case 5 : {
                CvsCheckout command = new CvsCheckout(client);
                String root = (String)cbCurrentDir.getSelectedItem();
                command.setLocalPath(root);
                for (int i = 0; i < files.length; i++) {
                    String module = files[i].getAbsolutePath().substring(root.length());
                    module = module.replace('\\','/');
                    if (module.length() == 0) {
                        module = ".";
                    } else if (module.charAt(0) == '/') {
                        module = module.substring(1);
                    }
                    command.addModule(module);
                }
                return command;
            }
            case 6 : return new CvsAdd(files, client);
            case 7 : return new CvsRemove(files, client);
            case 8 : return new CvsCompleteModList(client);
        }
        return null;
    }
    
    private void findCvsRoot(String dirName) {
        File dir = new File(dirName);
        if (dir.exists()) {
            final String rootName = "CVS" + File.separator + "Root";
            File rootFile = new File(dirName, rootName);
            if (rootFile.exists()) {
                BufferedReader buff = null;
                try {
                    buff = new BufferedReader(new InputStreamReader(new FileInputStream(rootFile)));
                    String line = buff.readLine();
                    txCvsRoot.setText(line);
                } catch (IOException exc) { //doesn't matter - nothing will be filled in
                    txCvsRoot.setText("");
                }
                finally {
                    if (buff != null) {
                        try {
                            buff.close();
                        } catch (IOException eIO) {}
                    }
                }
                return;
            }
        }
        txCvsRoot.setText("");
        return;
    }
    
    
    private class MyTreeNode extends DefaultMutableTreeNode {
        
        static final long serialVersionUID = 2478352100056378993L;
        private boolean wasRead = false;
        
        public MyTreeNode() {
            super();
        }
        
        public MyTreeNode(Object userObject) {
            super(userObject);
        }

        public boolean isLeaf() {
            return !getAllowsChildren();
        }
        
        public void setWasRead(boolean read) {
            wasRead = read;
        }
        public boolean isWasRead() {
            return wasRead;
        }
    }
    
    private class MyComparator implements Comparator {
        
        public MyComparator() {
        }
        
        public boolean equals(Object obj) {
            if (obj instanceof MyComparator) {
                MyComparator obj2 = (MyComparator)obj;
                return true;
            }
            return false;
        }
        
        public int compare(java.lang.Object obj1,java.lang.Object obj2) {
            if (!(obj1 instanceof File) ||
            !(obj2 instanceof File)) return 0;
            File rev1 = (File)obj1;
            File rev2 = (File)obj2;
            int result = 0;
            if ((rev1.isDirectory() && !rev2.isDirectory())) {
                return  - 1;
            }
            if (!rev1.isDirectory() && rev2.isDirectory()) {
                return 1;
            }
            result = rev1.getName().compareTo(rev2.getName());
            return result;
        }
    }

    private class MyTreeCellRenderer extends DefaultTreeCellRenderer {
        String DEFAULT_FOLDER = "/org/openide/resources/defaultFolder.gif";
        String DEFAULT_OPEN_FOLDER = "/org/openide/resources/defaultFolderOpen.gif";
        String DEFAULT_FILE = "/org/openide/resources/defaultNode.gif";
        static final long serialVersionUID = -1075722862531035018L;
        
        
        /** Finds the component that is capable of drawing the cell in a tree.
         * @param value value can be either Node or a value produced by models (like
         *   NodeTreeModel, etc.)
         * @return component to draw the value
         */
        public Component getTreeCellRendererComponent(
        JTree tree, Object value,
        boolean sel, boolean expanded,
        boolean leaf, int row, boolean hasFocus
        ) {
            Component comp = super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, hasFocus);
            if (comp instanceof JLabel) {
                JLabel label = (JLabel) comp;
                MyTreeNode node = (MyTreeNode)value;
                if (node != null) {
                    Object userObj = node.getUserObject();
                    if (userObj != null) {
                        File file = (File) userObj;
                        label.setText(file.getName());
                        if (!file.isDirectory()) {
                            java.net.URL url1 = this.getClass().getResource(DEFAULT_FILE);
                            label.setIcon(new ImageIcon(url1));
                            
                        } else {
                            File cvsSubDir = new File(file.getAbsolutePath(), "CVS");
                            if (!cvsSubDir.exists()) {
                                label.setText(file.getName() + " [Local]");
                            }
                            if (!expanded) {
                                java.net.URL url1 = this.getClass().getResource(DEFAULT_FOLDER);
                                label.setIcon(new ImageIcon(url1));
                            } else {
                                java.net.URL url2 = this.getClass().getResource(DEFAULT_OPEN_FOLDER);
                                label.setIcon(new ImageIcon(url2));
                                //                  label.setIcon(new ImageIcon(new java.net.URL("org.netbeans.openide.resources.defaultFolder.gif")));
                            }
                            
                        }
                    }
                }
            }
            return comp;
        }
        
    }
    
    
}
