/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient;

import java.beans.*;
import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.openide.util.NbBundle;

/** Property editor for UiMode property of the JavaCvsFileSystem
*
* @author Milos Kleint

*/
public class UiModePropertyEditor extends PropertyEditorSupport {

    private static final java.util.ResourceBundle bundle = NbBundle.getBundle(UiModePropertyEditor.class);

    /** localized string*/
    private final static String DEFAULTS = bundle.getString("UiModePropertyEditor.defaults");

    /** localized string*/
    private final static String DETAILED = bundle.getString("UiModePropertyEditor.detailed");

    /** localized string*/
    private final static String COMMAND_LINE = bundle.getString("UiModePropertyEditor.commandLine");

    /** array of hosts */
    private static final String[] modes = {DEFAULTS, DETAILED, COMMAND_LINE};

    /** @return names of the supported LookAndFeels */
    public String[] getTags() {
        return modes;
    }

    /** @return text for the current value */
    public String getAsText () {
        Integer mode = (Integer) getValue();
        if (mode.intValue() == 0 ) {
            return DEFAULTS;
        } 
        else if (mode.intValue() == 2) {
            return COMMAND_LINE;
        }
        else {
            return DETAILED;
        }
    }

    /** @param text A text for the current value. */
    public void setAsText (String text) {
        if (text.equals(DEFAULTS)) {
            setValue(new Integer(0));
            return;
        }
        if (text.equals(DETAILED)) {
            setValue(new Integer(1));
            return;
        }
        if (text.equals(COMMAND_LINE)) {
            setValue(new Integer(2));
            return;
        }
        throw new IllegalArgumentException ();
    }

    public void setValue(Object value) {
        super.setValue(value);
    }
}

