/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient.caching;

import org.netbeans.lib.cvsclient.*;
import org.netbeans.lib.cvsclient.connection.*;
import org.netbeans.lib.cvsclient.file.*;
import org.netbeans.lib.cvsclient.response.*;
import org.netbeans.lib.cvsclient.command.*;
import org.netbeans.lib.cvsclient.admin.StandardAdminHandler;
import org.netbeans.lib.cvsclient.event.*;
import org.netbeans.lib.cvsclient.util.IgnoreFileFilter;
import org.netbeans.modules.vcscore.cache.*;
import org.netbeans.modules.cvsclient.passwd.*;
import org.netbeans.modules.cvsclient.commands.*;
import org.netbeans.modules.vcscore.util.*;
import java.io.*;
import org.openide.util.NbBundle;
import java.text.MessageFormat;


/**
 *
 * @author  mkleint
 * @version 
 */
public class CvsCacheClient  implements ClientProvider {
    private Debug E=new Debug("CvsCacheClient", true); // NOI18N
    private Debug D=E;

    protected static CvsCacheClient cacheClient = null;
    protected Client client;
    protected GlobalOptions globalOpt;
    private String cvsRoot;
    private static final String CVS_DIR = "CVS";
    private static final String ROOT_FILE = "Root";
    private static final String CVS_ROOTFILE = File.separator  + CVS_DIR + File.separator + ROOT_FILE;
    /** Creates new CvsCacheClient */
    private CvsCacheClient() {
        globalOpt = new GlobalOptions();
    }

    public static CvsCacheClient getInstance() {
       //REDO.. not finished/final yet.. just for testing purposes
        // maybe always create one instance -> to avoid conflicts when user quickly opens many directories
        if (cacheClient == null) {
             cacheClient = new CvsCacheClient();
        }
        return cacheClient;
    }
    
    /**  
     */
    public void doRefreshFor(File file) {
        refresh(file,false);
    }
    
    
    
    public void doRefreshRecursively(File dir) {
        refresh(dir, true);
    }

    /**  
     */
   private void refresh(File file, boolean recursive) {
        D.deb("doRefreshFor() file =" + file.getAbsolutePath());
/*        String pathToRoot;
        if (file.isDirectory()) {
          pathToRoot = file.getAbsolutePath() + CVS_ROOTFILE;
        } else {
          pathToRoot = file.getParentFile().getAbsolutePath() + CVS_ROOTFILE;
        }
 
        String root = readRoot(new File(pathToRoot));
        D.deb("doRefreshFor() root =" + root);
        if (root != null) {
            RefreshCommand status;
            setCvsRootString(root);
 */
            RefreshCommand status;
            if (file.isDirectory()) {
              status = new RefreshCommand(this, file, recursive);
              D.deb("doRefreshFor() LocalDir(d)=" + file.getAbsolutePath());
            } else {
              status = new RefreshCommand(this, file);
              D.deb("doRefreshFor() LocalDir(f)=" + file.getParentFile().getAbsolutePath());
            }
  //          gl.setCVSRoot(root);
            status.startCommand();
            D.deb("done doRefreshFor()");
/*        } else {
            TopManager.getDefault().setStatusText(
                NbBundle.getBundle(CvsCacheClient.class).getString("CvsCacheClient.notCvsDir"));
        }
 */
    }
    
    public void setCvsRootString(String root) {
        cvsRoot = root;
    }

    
    
    public String readRoot(File dirFile) {
        if (dirFile.exists()) {
            BufferedReader buff = null;
            String line = null;
            try {
                buff = new BufferedReader(new InputStreamReader(
                                             new FileInputStream(dirFile.getAbsolutePath())));
                line = buff.readLine();
            } catch (IOException exc) { //doesn't matter - nothing will be filled in
                line = null;
            }
            finally {
                if (buff != null) {
                    try {
                        buff.close();
                    } catch (IOException eIO) {}
                }
            }
            return line;
        }
        else {
            return null;
        }
    }
    
    private PServerConnection setupConnection(PasswdEntry entry) throws AuthenticationException {
       PServerConnection conn = new PServerConnection();
       conn.setUserName(entry.getUser());
       conn.setHostName(entry.getServer());
       conn.setRepository(entry.getRoot());
       conn.setEncodedPassword(entry.getPasswd());
       conn.open();
       return conn;
    }
    
    public Client createClient(String root) throws ClientCreationException {
            Client myClient = new Client(null, new StandardAdminHandler());    
            return myClient;
    }
    
    public void openConnection(Client client, String root) throws AuthenticationException {
            CVSPasswd pass = new CVSPasswd();
            pass.loadPassFile();
            PasswdEntry entry = pass.find(root);
            if (entry == null) {
                ClientCreationException exc = new ClientCreationException("", 
                   NbBundle.getMessage(RefreshCommand.class, "RefreshCommand.notInPassFile"));
                throw exc;
            }   //TODO - not found in .cvspass file - ignore or show error??
            D.deb("doRefreshFor() entry =" + entry.getEntry(true));
            client.setConnection(setupConnection(entry));
    }
    
    public JavaCvsCommandFactory getCommandFactory() {
        return JavaCvsCommandFactory.getInstance(); //is nessesary?
    }
    
    public boolean isOffLine() {
        return false;
    }
    
    public boolean isFileSystemClient() {
        return false;
    }
    
    public GlobalOptions getGlobalOptions() {
        return globalOpt;
    }
    
    public int getUiMode() {
        // No param input! 
        return org.netbeans.modules.cvsclient.JavaCvsFileSystem.MODE_NOVICE;
    }
    
    public String getCvsRootString() {
        return cvsRoot;
    }
    
    public int getDisplayType() {
        return 1; // don't put the display to output window
    }
    
    public IgnoreFileFilter getIgnoreFilter() {
        return null;
    }
    
}
