/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient.caching;

/**
 *
 * @author  mkleint
 * @version 
 */
import java.io.*;
import java.util.*;
import org.netbeans.modules.vcscore.util.Debug;
import org.netbeans.modules.vcscore.cache.*;
import org.netbeans.modules.cvsclient.JavaCvsStatusManager;



public class CvsCacheDir extends CacheDir {
    private Debug E=new Debug("CvsCacheDir", true); // NOI18N
    private Debug D=E;
    
//    protected String status;
    
    private static final String CACHE_FILENAME = File.separator + "CVS" + File.separator + "netbeans.cache";
    
    private static FileFilter cvsFilter =  new FileFilter() {
              public boolean accept(File f) {
                 if (f.isDirectory() && f.getName().equalsIgnoreCase("CVS")) return true;
                 return false;
              }
    };

    
    /** Creates new CvsCacheDir */
    public CvsCacheDir(String cacheName, File dirFile) {
        super(cacheName, dirFile);
        D.deb("Costructor", "dirName=" + dirFile.getAbsolutePath());
//        status = CvsCacheFile.LOCAL;
        File cvsSubDir = new File(dirFile,"CVS");
        if (!cvsSubDir.exists()) {
          setLocal(true);
          setStatus(JavaCvsStatusManager.LOCAL);
        } else {
           setLocal(false);
           setStatus("");
        }
    }

    public void populateWithLocal() {
//        E.err("populateWithLocal()" + dirFile.getAbsolutePath());
        File[] subFiles = dirFile.listFiles();
        if (subFiles == null) return;
        for (int index = 0; index < subFiles.length; index++) {
           File oneFile = subFiles[index];
           if (oneFile == null) continue;
           if (oneFile.isDirectory()) {
             D.deb("    populate- adding Dir=" + oneFile.getName());
             if (getSubDir(oneFile.getName()) == null) {
                 addChildDir(new CvsCacheDir(getCacheObject().getId(), oneFile), false);
             }
           } else {
             D.deb("    populate- adding File=" + oneFile.getName());
             if (getFile(oneFile.getName()) == null) {
                 CvsCacheFile file = new CvsCacheFile(getCacheObject().getId(), oneFile.getName());
                 this.addFile(file, false);
             }
           }
        }
        setAppliedLevel(CacheHandler.STRAT_LOCAL);
        D.deb("populateWithLocal() - is Local=" + isLocal());
    }
    
    public void repopulate() {
//        E.err("populateWithLocal()" + dirFile.getAbsolutePath());
        if (!dirFile.exists()) {
            removeAll(true);
            if (getParent() != null) {
                getParent().removeChildDir(getName(), true);
            }
            return;
        }
        File[] cvsDir = dirFile.listFiles(cvsFilter);
        setLocal(cvsDir.length == 0); 
        File[] subFiles = dirFile.listFiles();
        CacheDir[] cacheDirs = getSubDirs();
        // remove wrong directories..
        for (int index = 0; index < cacheDirs.length; index++) {
           CacheDir dir = cacheDirs[index];
           if (dir == null) continue;
           File found = null;
           for (int ind2 = 0; ind2 < subFiles.length; ind2++) {
               if (subFiles[ind2] == null) continue;
               if (subFiles[ind2].getName().equals(dir.getName())) {
                   found = subFiles[ind2];
                   subFiles[ind2] = null;
                   break;
               }
           }
           if (found == null) {
               removeChildDir(dir.getName(), false);
           } else 
               if (!found.isDirectory()) {
                   removeChildDir(dir.getName(), false);
               }
        }
        // remove wrong files..
        CacheFile[] cacheFiles = getFiles();
        for (int index = 0; index < cacheFiles.length; index++) {
           CacheFile cfile = cacheFiles[index];
//           System.out.println("cFile=" + cfile.getName());
           if (cfile == null) continue;
           File found = null;
           for (int ind2 = 0; ind2 < subFiles.length; ind2++) {
               if (subFiles[ind2] == null) continue;
               if (subFiles[ind2].getName().equals(cfile.getName())) {
                   found = subFiles[ind2];
                   subFiles[ind2] = null;
                   break;
               }
           }
           if (found == null) {
//               System.out.println("not found");
               if (isLocal()) {
                   removeFile(cfile.getName(), false);
               } else {
                   if (cfile.getStatus().equals(JavaCvsStatusManager.LOCALLY_REMOVED)) {
                       // keep it.
                   } 
                   else if (cfile.getStatus().equals(JavaCvsStatusManager.LOCAL)
                          || cfile.getStatus().equals(JavaCvsStatusManager.LOCALLY_ADDED)) {
                       removeFile(cfile.getName(), false);
                   } 
                   else {
                       cfile.setStatus(JavaCvsStatusManager.NEEDS_CHECKOUT);
                   }
               }
           } else {
//               System.out.println("found");
               if (isLocal() &&  
                   (!cfile.getStatus().equals(JavaCvsStatusManager.LOCAL))) 
               {
                   removeFile(cfile.getName(), false);
               }
           }
        }
        //now add the ones that were not present.. 
        for (int ind3 = 0; ind3 < subFiles.length; ind3++) {
            if (subFiles[ind3] == null) continue;
            File file = subFiles[ind3];
            if (!file.exists()) continue;
            if (file.isDirectory()) {
                addChildDir(new CvsCacheDir(JavaCvsCache.JAVA_CACHE_NAME, file), false);
            } else {
                addFile(new CvsCacheFile(JavaCvsCache.JAVA_CACHE_NAME, file.getName()), false);
            }
        }
        
        
        D.deb("populateWithLocal() - is Local=" + isLocal());
    }
    /** returns the name of the directory
     */
    public String getName() {
        if (dirFile.getParentFile() == null) {
            return dirFile.getAbsolutePath();
        }
        return dirFile.getName(); // return short name, just name of file..
        //        return dirName;
    }    
    
    public void writeToDisk() {
        D.deb("writeToDisk() -START" + this.getName());
        if (!isComplete()) return;
        File[] cvsDir = dirFile.listFiles(cvsFilter);
        if (cvsDir == null || cvsDir.length == 0)  return; // it's not a CVS dir.. nothing is written..
        D.deb("writeToDisk()", "next step");
        File netbeansFile = new File(getCacheFileName());

        // even if this directory is not modified , the cache file is written down 
        // .. BUT only when if already doesn't exist.. (we don't want to overwrite not-read files, do we? :)
        // useful to create cache files and populate the structure with netbeans.cache files
        if ((!this.isModified()) && (netbeansFile.exists())) return; // nothing to save.. noting changed
        // Do not write down directories that hasn't been at least locally read (that would write empty dir.
        // Also do not write if the file netbeans.cache exists AND wasn't read.
        if ((!netbeansFile.exists() && (getAppliedLevel() < CacheHandler.STRAT_DISK))
            || (getAppliedLevel() < CacheHandler.STRAT_DISK)) return;
        D.deb("writeToDisk() -- let's do it! for = " + this.getName());
        BufferedWriter out = null;
        try{
            out = new BufferedWriter(
                      new OutputStreamWriter(
                          new BufferedOutputStream(
                              new FileOutputStream(getCacheFileName())))); // NOI18N
            // files get written first
            Collection files = cachedFiles.values();
            Iterator it = files.iterator();
            while (it.hasNext()) {
               CvsCacheFile file = (CvsCacheFile)it.next();
               out.write(file.writeLineToDisk());
               out.newLine();
            }
            // then directories
            D.deb(this.getAbsolutePath() + " files written");
            Collection dirs = childDirs.values();
            it = dirs.iterator();
            while (it.hasNext()) {
               CvsCacheDir dir = (CvsCacheDir)it.next();
               if (dir.getAppliedLevel() >= CacheHandler.STRAT_LOCAL) {
                 out.write(dir.writeLineToDisk());
                 out.newLine();
               }
            }
            D.deb(this.getAbsolutePath() + " dirs written");
            out.flush();
        }
        catch (IOException e){
            E.err(e,"writeDirToDiskCache() failed for "+getName()); // NOI18N
        } finally {
            try {
                if (out != null) out.close();
            } catch (IOException e) {
                E.err(e,"writeDirToDiskCache() failed for "+getName()); // NOI18N
            }
        }
    }
    
    public boolean readFromDisk() {
        boolean isOk = false;
        File cacheFile=new File(getCacheFileName()); // the actual netbeans.cache file
 //       E.err("readFromDisk()" + this.getAbsolutePath());
        if(cacheFile.exists() && cacheFile.canRead() ){
            BufferedReader in = null;
            try{
               in = new BufferedReader
                                   (new InputStreamReader
                                    (new BufferedInputStream
                                     (new FileInputStream(cacheFile))));
                String line=null;
                while ((line=in.readLine())!=null ){
                    CvsCacheFile file = CvsCacheFile.readFileCache(line, this);
                }
                in.close();
                isOk= true;
                setAppliedLevel(CacheHandler.STRAT_DISK);
                return isOk;
            }catch (IOException e){
                E.err(e,"readDirFromDiskCache() failed"); // NOI18N
            } finally {
                if (in != null) { 
                    try {in.close();
                    } catch (IOException exc) {}
                    in = null;
                }
            }
        } else {
            // if not exists, let's assume we applied the level
            setAppliedLevel(CacheHandler.STRAT_DISK);
        }
        return isOk;
    }
    
    public void checkServer() {
      D.deb("checkServer() for " + getName() + "    curr.level=" + getAppliedLevel());
      // thread has to start here..
      JavaCvsCache cache = (JavaCvsCache)getCacheObject();
      cache.prepareCache(new File(this.getAbsolutePath()), false, CacheHandler.STRAT_REFRESHING);
      cache.fireCacheHandlerEvent(FileSystemCache.EVENT_CHANGED, this);
      cache.lockFileObjects(this, false);
      CvsCacheClient client = cache.getCacheClient();
      client.doRefreshFor(new File(getAbsolutePath()));
 
//      setAppliedLevel(CacheHandler.STRAT_ALL);  
      // waits for the commands to finish
    }
    
    public void checkServerRecursive() {
      D.deb("checkServer() for " + getName() + "    curr.level=" + getAppliedLevel());
      // thread has to start here..
      JavaCvsCache cache = (JavaCvsCache)getCacheObject();
      cache.prepareCache(new File(this.getAbsolutePath()), true, CacheHandler.STRAT_REFRESHING);
      cache.fireCacheHandlerEvent(FileSystemCache.EVENT_CHANGED_RECURSIVELY, this);
      cache.lockFileObjects(this, true);
      CvsCacheClient client = cache.getCacheClient();
      client.doRefreshRecursively(new File(getAbsolutePath()));
 
//      setAppliedLevel(CacheHandler.STRAT_ALL);  
      // waits for the commands to finish
    }
    

//--   gets overriden because we need to exclude the CVS directories 
    
    public String[] getDirContentNames() {
       D.deb("getDirContentName() -- beginning of " + this.getAbsolutePath());
       Collection colDr = childDirs.values();
       Collection colFl = cachedFiles.values();
       int dirSize = colDr.size() + colFl.size();
       int size1 = colDr.size();
       if (dirSize == 0) {
           return null;
       }
       if (!this.isLocal()) { //for CVS directories make the list shorter by one - excluding (hiding) cvs directory
         dirSize = dirSize - 1;
         size1 = size1 - 1; // size fo directories array is shorter by one (CVS)
       } 
       String[] content = new String[dirSize]; 
       Iterator it = colDr.iterator();
       int index = 0;
       while (index < size1 && it.hasNext()) {
           CacheDir subDir = (CacheDir)it.next();
           if (!subDir.getName().equalsIgnoreCase("CVS")) { // do exclude CVS dir (hidin it from fs children() method)
             content[index] = subDir.getName();
             index = index + 1;
           }
       }
       it = colFl.iterator();
       for (int index2 = 0; index2 < colFl.size(); index2++) {
           content[index2 + size1] = ((CacheFile)it.next()).getName();
       }
       return content;
    }
    

    public void rename(File newLocation) {
        super.rename(newLocation);
        status = JavaCvsStatusManager.LOCAL;
    }

    

    
    
    /** Path string to the netbeans.cache file of this directory
     */
    protected String getCacheFileName() {
        return dirName + CACHE_FILENAME;
    }
    
    public void setStatus(String stat) {
        status = stat;
        setModified(true);
    }
    
    public String getStatus() {
//      if (super.getStatus().equals(" ")) {  
  //      if (isShadow()) return CvsCacheFile.NEEDS_CHECKOUT;   // shadow directory
        File cvsDir = new File(getAbsolutePath() + File.separator + "CVS");
        if (!cvsDir.exists()) { setStatus(JavaCvsStatusManager.LOCAL);}
        return super.getStatus();
/*      } else {
          // when the status was set during the live of the instance
          return super.getStatus();
      }
 */
    }
    public String writeLineToDisk() {
       StringBuffer buff = new StringBuffer("D/");
       buff.append(getName());
       buff.append("/");
       buff.append(getStatus());
       buff.append("///");
       return buff.toString();
    }
    
}
