/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient.caching;
import java.io.*;
import java.util.*;
import java.beans.*;
import java.text.*;

import org.netbeans.lib.cvsclient.file.FileStatus;
import org.netbeans.modules.vcscore.util.*;
import org.netbeans.modules.vcscore.cache.*;
import org.netbeans.modules.cvsclient.JavaCvsStatusManager;

/** File.
 *
 * @author Milos Kleint
 */
//-------------------------------------------
public class CvsCacheFile extends CacheFile {
    private Debug E=new Debug("CvsCacheFile", false); // NOI18N
    private Debug D=E;
    
    int size=0;
    String date=""; // NOI18N
    String time=""; // NOI18N
    String locker=""; // NOI18N
    String attr=""; // NOI18N
    // true if the file is not in CVS
    private boolean local = false;
    
    
    //-------------------------------------------
    public CvsCacheFile(String memberOfCache){
        super(memberOfCache);
        this.local=true;
        setStatusNonModifying(JavaCvsStatusManager.LOCAL);
    }
    
    //-------------------------------------------
    public CvsCacheFile(String memberOfCache, String name){
        super(memberOfCache, name);
        this.local=true;
        setStatusNonModifying(JavaCvsStatusManager.LOCAL);
    }
    
    
    //-------------------------------------------
    public void setLocal (boolean local) {
        this.local=local;
    }
    //-------------------------------------------
    public boolean isLocal () {
        return local;
    }
    
    public static final CvsCacheFile readFileCache(java.lang.String line, CvsCacheDir par) {
        if (line.startsWith("D")) {
            StringTokenizer token = new StringTokenizer(line,"/",false);
            String name = token.nextToken();
            name = token.nextToken();
            CvsCacheFile newFile = (CvsCacheFile)par.getFile(name);
            if (newFile == null) { // not found == it's not on the local disk -> it's a shadow file
//                par.setStatus(JavaCvsStatusManager.NOT_IN_SYNCH);                
                return newFile;
            }
            String stat = token.nextToken();
            newFile.setStatusNonModifying(stat);
            return newFile;
        }
        if (!line.startsWith("/")) {
//            System.out.println("ERROR : Bad cache disk format (CvsCacheFile)");
            return null;
        }
        
        StringTokenizer token = new StringTokenizer(line,"/",false);
        if (token.countTokens() < 2) return null;
        String name = token.nextToken();
        CvsCacheFile newFile = (CvsCacheFile)par.getFile(name);
        String status = token.nextToken();
        if (newFile == null) { // not found == it's not on the local disk -> it's a shadow file
            CvsCacheFile file = new CvsCacheFile(par.getCacheName(), name);
            par.addFile(file, false);
            newFile = file;
            // status AI - for all but removed and local change status to  NEEDS_CHECKOUT.
            if (status.equals(JavaCvsStatusManager.LOCALLY_REMOVED) 
                      || status.equals(JavaCvsStatusManager.LOCAL)) {
                newFile.setStatusNonModifying(status);
            } else {
                newFile.setStatusNonModifying(JavaCvsStatusManager.NEEDS_CHECKOUT);
            }
        } else {
            newFile.setStatusNonModifying(status);
        }
        return newFile;
    }
    
    public String writeLineToDisk() {
        StringBuffer buff = new StringBuffer("/");
        buff.append(getName());
        buff.append("/");
        buff.append(getStatus());
        buff.append("///"); //TODO - temporary
        //TODO
        return buff.toString();
    }
    
    
    //-------------------------------------------
    public String getStatus(){
        return this.status; //+" "+locker; //+" "+date+" "+time+" "+size+" "+attr; // NOI18N
    }
    
    /** set the status..
     * @param stat any of the status constants in this class
     */
    public void setStatus (String stat) {
        //TODO the status changing logic move someplace else.
        //        if (status.equals(LOCAL)) return; // local files don't change status
        if (status.equals(JavaCvsStatusManager.LOCALLY_ADDED) && 
            stat.equals(JavaCvsStatusManager.LOCALLY_MODIFIED)) return; // for loc added files, ignore modified updates
        if (status.equals(JavaCvsStatusManager.NEEDS_PATCH) && 
            stat.equals(JavaCvsStatusManager.LOCALLY_MODIFIED)) {
            status = JavaCvsStatusManager.NEEDS_MERGE;
            getParent().setModified(true);
            return;
        }
        // TODO? other status logic to be added
        status = stat;
        getParent().setModified(true);
    }
    
    /** does not set the setModified flag on the directory.. should be called
     * only when reading from disk cache
     * not modified -> read from disk, populated from local disk.
     */
    public void setStatusNonModifying(String stat) {
        status = stat;
    }
    
    
    //-------------------------------------------
    public String toString(){
        return "CvsCacheFile["+ // NOI18N
        "name='"+name+"'"+ // NOI18N
        ",status="+status+ // NOI18N
        ",locker="+locker+ // NOI18N
        ",attr="+attr+ // NOI18N
        ",size="+size+ // NOI18N
        ",date="+date+ // NOI18N
        ",time="+time+ // NOI18N
        ",local="+local+ // NOI18N
        "]"; // NOI18N
    }
    
    
}


