/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient.caching;

/**
 *
 * @author  mkleint
 * @version 
 */
import org.netbeans.modules.cvsclient.JavaCvsFileSystem;
import org.netbeans.modules.cvsclient.JavaCvsStatusManager;
import org.netbeans.modules.vcscore.util.*;
import org.openide.util.*;
import org.openide.filesystems.FileObject;
import java.util.*;
import java.io.*;
import java.lang.ref.*;
import java.text.MessageFormat;
import org.netbeans.modules.vcscore.cache.*;


public class CvsFsCache extends java.lang.Object  implements CachedFileSystem {
    private Debug E=new Debug("CvsFsCache", true); // NOI18N
    private Debug D=E; 
    
    private JavaCvsFileSystem cvsfs;    
//    private HashMap fileStatuses = new HashMap();
    private CacheHandler handler;
    
    private static JavaCvsCache cacheObject = null;
    
    /** Creates new CvsFsCache */
    public CvsFsCache(JavaCvsFileSystem fs) {
        cvsfs = fs;
   //     initFileStatuses();
        handler = CacheHandler.getInstance();
        getCacheObject().addCacheHandlerListener(cvsfs);
    }
    
    public static JavaCvsCache getCacheObject() {
        if (cacheObject == null) {
            cacheObject = new JavaCvsCache();
        }
        return cacheObject;
    }
    
    private JavaCvsStatusManager getStatusManager() {
        return JavaCvsStatusManager.getInstance();
    }

    /** for strategies higher than STRAT_DISK, does adjustation depending 
     * on the value of filesystem's offLine property
     */
    private int adjustStrategy (int reqStrategy) {
        if (reqStrategy > CacheHandler.STRAT_DISK) {
            if (cvsfs.isOffLine()) return CacheHandler.STRAT_DISK;
        }
        return reqStrategy;
    }
    /** for strategies higher than STRAT_DISK, permit intended action depending
     * on the value of filesystem's offLine property
     */
    
    private boolean permitAction(int reqStrategy) {
        if (reqStrategy > CacheHandler.STRAT_DISK) {
            if (cvsfs.isOffLine()) return false;
        }
        return true;
    }
    
    
    
    /** Performs status check on the specified "important" file(s).. 
     * see JavaCvsFileSystem.annotateName()
     */
    public String getStatus(Vector importantFiles) {
        D.deb("getStatus("+importantFiles+")"); // NOI18N
        int currentStrategy;
        if (cvsfs.getAutoRefresh() == JavaCvsFileSystem.AUTO_SIMPLE) {
            currentStrategy = CacheHandler.STRAT_DISK_OR_REFRESH;
        } else {
            currentStrategy = CacheHandler.STRAT_DISK;
        }
        
        String status=""; // NOI18N
        int len = importantFiles.size();
        CacheFile file=null;
        String name;
        String fullName;
        String stat;

        //D.deb("len="+len); // NOI18N
        if( len < 1 ){
           return getStatusManager().getStatus(JavaCvsStatusManager.UNKNOWN);
        }
        else if( len==1 ){
            name = (String) importantFiles.elementAt(0);
            fullName = cvsfs.getRootDirectory().getAbsolutePath() + File.separator + name;
            D.deb("getStatus(singleFile)- fullName:" + fullName);
            file = handler.getCacheFile(new File(fullName), adjustStrategy(currentStrategy), this);
            if (file != null) stat = getStatusManager().getStatus(file.getStatus());
            else stat = "";
            return stat;
        }
        // for multidata objects do the following
        //D.deb("status="+status); // NOI18N
        String result = getStatusManager().getStatus(JavaCvsStatusManager.NOT_IN_SYNCH);
        name = (String) importantFiles.elementAt(0);
        String rootDir = cvsfs.getRootDirectory().getAbsolutePath();
        fullName = rootDir + File.separator + name;
        file = handler.getCacheFile(new File(fullName), adjustStrategy(currentStrategy), this);
        if (file != null) stat = getStatusManager().getStatus(file.getStatus());
        else stat = "";
//        stat = file.getStatus();
        
        for(int i=1;i<len;i++){
            String name1 = (String) importantFiles.elementAt(i);
            String fullName1 = rootDir + File.separator + name1;
            file = handler.getCacheFile(new File(fullName1), adjustStrategy(currentStrategy), this);
            String stat1;
            if (file != null) stat1 = getStatusManager().getStatus(file.getStatus());
            else stat1 = "";
            //D.deb("file="+file); // NOI18N
            if( !stat1.equals(stat) ){
                return result;
            }
        }
        return stat;
    }
    
    /** Returns String[] array of subfiles and subdirs. used in javaCvsFileSystem.children()
     */
    public String[] getDirContent(File dir) {
        D.deb("getDirContent()","--------------------------------------------------------------------");
        D.deb("getDirContent:" + dir.getAbsolutePath());
        CvsCacheDir cacheDir = (CvsCacheDir)handler.getCacheFile(dir, CacheHandler.STRAT_LOCAL, this);
        if (cacheDir != null) return cacheDir.getDirContentNames();
        return null;
/*
        File[] subFiles = dir.listFiles();
        String[] names;
        File cvsDir = new File(dir.getAbsolutePath() + File.separator + "CVS");
        if (cvsDir.exists()) {
            names = new String[subFiles.length - 1];
        } else {
            names = new String[subFiles.length];
        }
        int nameIndex = 0;
        for (int index = 0; index < subFiles.length; index++) {
           File oneFile = subFiles[index];
           if (oneFile == null) continue;
           if (oneFile.isDirectory() && oneFile.getName().equalsIgnoreCase("CVS")) continue;
           names[nameIndex] = oneFile.getName();
           nameIndex = nameIndex + 1;
        }
        return names;
 */
    }
    
    /** Adds a newly creted file to the cache. Used by JavaCvsFileSystem.createFolder and createData.
     *  
     */
    public void addToCache(File newFile) {
        D.deb("addToCache()","--------------------------------------------------------------------");
        File parent = newFile.getParentFile();
        CvsCacheDir parentCache = (CvsCacheDir)handler.getCacheFile(parent, CacheHandler.STRAT_LOCAL, this);
        if (parentCache == null) return;
        parentCache.setComplete(false);
/*        if (newFile.isDirectory()) {
            CvsCacheDir newCacheDir = new CvsCacheDir(this.getCacheType(), newFile);
            parentCache.addChildDir(newCacheDir, true);
        }
        else {
            CvsCacheFile newCacheFile = new CvsCacheFile(this.getCacheType(), newFile.getName());
            parentCache.addFile(newCacheFile, true);
        }
        parentCache.writeToDisk(); // write to disk immediately .. could be removed..
        D.deb("addToCache()","===================================================================");
  */      
    }
    
    public void removeFromCache(File deletedFile, boolean wasDirectory) {
//        E.err("removeFromCache()===file =" + deletedFile.getName() + " ==========");
        File parent = deletedFile.getParentFile();
//        E.err("removeFromCache() parentpath="+parent.getAbsolutePath());
        CvsCacheDir parentCache = (CvsCacheDir)handler.getCacheFile(parent, CacheHandler.STRAT_LOCAL, this);
        if (parentCache == null) {
            // we want to remove a chache node that doesn't have a parent -> needs to be root!
            // do remove it manually..
//            E.err("manualRemoval of cache");
            CvsCacheDir deletedCache = (CvsCacheDir)handler.getCacheFile(deletedFile, CacheHandler.STRAT_LOCAL, this);
            handler.getCache(this).unregisterDir(deletedCache);
            return;
        }
        if (wasDirectory) {
//            E.err("removeFromCache() - isDirectory");
            parentCache.removeChildDir(deletedFile.getName(), true);
        }
        else {
//            E.err("removeFromCache() - isFile");
            parentCache.removeFile(deletedFile.getName(), true);
        }
        parentCache.setComplete(false);
//        parentCache.writeToDisk(); // write to disk immediately .. could be removed..
//        E.err("removeFromCache() --------------------------------------------------------------------");
    }

    public void removeFromCache(CacheFile file) {
        D.deb("removeFromCache()","== file = =========");
        CacheDir parentCache = file.getParent();
        if (file instanceof CacheDir) {
            D.deb("removeFromCache()","isDirectory");
            if (parentCache == null){
                // we want to remove a chache node that doesn't have a parent -> needs to be root!
                // do remove it manually..
                CacheDir dir = (CacheDir)file;
                handler.getCache(this).unregisterDir(dir);
                return;
            }
            parentCache.removeChildDir(file.getName(), true);
        }
        else {
            D.deb("removeFromCache()","isFile");
            if (parentCache != null) {
                parentCache.removeFile(file.getName(), true);
            }
        }
        parentCache.setComplete(false);
        D.deb("removeFromCache()","--------------------------------------------------------------------");
    }

    
    public void renameTo(File oldFile, File newFile) {
        //TODO rewrite
        
        D.deb("renameTo()","===================================================================");
        // first let's do remove old node in cache
        File parent = oldFile.getParentFile();
        CacheDir parentCache = (CacheDir)handler.getCacheFile(parent, CacheHandler.STRAT_LOCAL, this);
        if (parentCache == null) { //root node.. is dir..cannot be renamed..
            return;
        }
        if (newFile.isDirectory()) { // the renamed node was a directory - do recursive rename
            D.deb("renameTo()","isDirectory=" + oldFile.getName());
            CacheDir renamedCache = (CacheDir)parentCache.getSubDir(oldFile.getName());
            parentCache.removeChildDir(renamedCache.getName(),false);
            renamedCache.rename(new File(parentCache.getAbsolutePath() + File.separator + newFile.getName()));
            parentCache.addChildDir(renamedCache,false);
            D.deb("renameTo():renamed to path = " + renamedCache.getAbsolutePath());
            renamedCache.renameChildDirs(renamedCache,false);
        }
        else { // just a file - lets rename it in the paarent directory.. and it's done :)
            D.deb("renameTo()","isFile");
            parentCache.removeFile(oldFile.getName(), true);
            CacheFile newCache = new CvsCacheFile(getCacheId(), newFile.getName());
            parentCache.addFile(newCache, true);
            return;
        }
        parentCache.setModified(true);
        parentCache.writeToDisk(); //write to disk immediately..
        D.deb("removeFromCache()","--------------------------------------------------------------------");
    }
    
    public CacheFile[] convertToCacheFiles(File[] files) {
        CacheFile[] cvsFiles = new CacheFile[files.length];
        for (int index = 0; index < files.length; index++) {
            cvsFiles[index] = handler.getCacheFile(files[index], CacheHandler.STRAT_DISK, this);
        }
        return cvsFiles;
    }

    
    public String getLocker(Vector importantFiles) {
     //TODO   
      return "";
    }

    public void refreshDir(String path) {}    

    /* checks if the Cache instance has all objects written to the disk. 
     * Is checked only when serializing the filesystems.
     * We need this check to aboid duplicate saving - since there's just one cache for all filesystems.
     */
    public void saveToDisk() {
       if (!handler.getCache(this).isWrittenToDisk()) {
          handler.getCache(this).writeAllToDisk(); 
       }
    }
    
    public void saveToDiskFromNode(String path) {
         String fullName = cvsfs.constructRootDirectory() + File.separator + path;
         if (fullName.endsWith(File.separator)) {
             fullName = fullName.substring(0,fullName.length() - 1);
         }
         D.deb("saveToDiskFromNode()- fullName:" + fullName);
         CacheDir dir = (CacheDir)handler.getCacheFile(new File(fullName), CacheHandler.STRAT_DISK, this);
         if (dir != null) {
             dir.writeToDiskRecursively();
         }
    }
    
    
    public void fileModified(File modified) {
        CacheFile cFile = handler.getCacheFile(modified,CacheHandler.STRAT_NONE, this);
        if (cFile != null) { // should never be null.. just a I-want-to-be-sure check.
           String oldStatus = cFile.getStatus();
           if (oldStatus.equals(JavaCvsStatusManager.LOCAL) || oldStatus.equals(JavaCvsStatusManager.UNKNOWN)) return;
           if (oldStatus.equals(JavaCvsStatusManager.NEEDS_PATCH)) {
               cFile.setStatus(JavaCvsStatusManager.NEEDS_MERGE);
           }
           else {
               cFile.setStatus(JavaCvsStatusManager.LOCALLY_MODIFIED);
           }
           cacheObject.fireCacheHandlerEvent(JavaCvsCache.EVENT_CHANGED, cFile);
        }
    }
    
    public void doRefresh(File[] files, boolean recursive) {
        int  strategy = (recursive ? CacheHandler.STRAT_REFRESH_RECURS : CacheHandler.STRAT_REFRESH);
            // in this case, just apply a dir reading strategy that will handle that do us
             for (int index = 0; index < files.length; index++) {
               //REDO - once more commands on one connection -> use just one client 
               CacheFile cFile = handler.getCacheFile(files[index], strategy, this);        
             }
/*
        int  strategy = (recursive ? CacheHandler.STRAT_REFRESH_RECURS : CacheHandler.STRAT_REFRESH);
        if (recursive) {
            files = findRelevantDirs(files);
            // <DEBUG>
            for (int i = 0; i < files.length; i++) {
                System.out.println("REFRESHED DIR=" + files[i]);
            }
            // </DEBUG>
        }
        // in this case, just apply a dir reading strategy that will handle that do us
        for (int index = 0; index < files.length; index++) {
            //REDO - once more commands on one connection -> use just one client
            File refFile = files[index];
            
            CacheFile cFile = handler.getCacheFile(files[index], strategy, this);
            E.err("CvsFsCache: ------- refresh started------" + files[index].getAbsolutePath());
        }
         */
    }
    

    
    
    public String getCacheId() {
        return JavaCvsCache.JAVA_CACHE_NAME;
    }    
    
    public Reference createReference(FileObject fo) {
        String path = fo.getPackageNameExt(File.separatorChar,'.');
        String absolutePath = cvsfs.constructRootDirectory() + File.separator + path;
        D.deb("Now i'm searching for the file when creating reference");
        File file = new File(absolutePath);
        return CacheHandler.getInstance().createReference(fo, file, getCacheId());

    }
    
    public void removeNotify() {
        getCacheObject().unregisterFileSystem(cvsfs);
    }
    
}