/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient.caching;

import org.openide.*;
import org.openide.util.*;
import java.io.*;
import java.util.*;
import java.awt.Dialog;

import org.netbeans.modules.cvsclient.*;
import org.netbeans.modules.cvsclient.commands.*;

import org.netbeans.lib.cvsclient.event.*;
import org.netbeans.lib.cvsclient.command.status.*;
import org.netbeans.lib.cvsclient.command.*;
import org.netbeans.modules.vcscore.util.Debug;
import org.netbeans.modules.vcscore.cache.*;
import org.netbeans.lib.cvsclient.file.FileStatus;



/**
 *
 * @author  mkleint
 * @version
 */
public class RefreshCommand extends CacheUpdatingFsCommand {
    private Debug E=new Debug("RefreshCommand", true); // NOI18N
    private Debug D=E;
    private static final String CVS_DIR = "CVS";
    private static final String ROOT_FILE = "Root";
    private static final String CVS_ROOTFILE = File.separator  + CVS_DIR + File.separator + ROOT_FILE;
    
    StatusCommand command;
    ArrayList resultList;
    Object[] resultArray;
    File[] cvsSubdirs;
    boolean recursive;
    
    /** Creates new refresh command - before running the command, do setClient manually */
    public RefreshCommand(ClientProvider cliProvider, File dirToCheck, boolean recurs) {
        super(cliProvider);
        File[] fls = new File[1];
        fls[0] = dirToCheck;
        setFiles(fls);
        recursive = recurs;
        if (recursive) {
            cvsSubdirs = findRelevantDirs(fls);
        }
        D.deb("Constructor");
    }
    
    /** Creates new refresh command - before running the command, do setClient manually */
    public RefreshCommand(ClientProvider cliProvider, File file) {
        this(cliProvider, file, false);
    }
    
    public String getName() {
        File[] fls = getFiles();
        if (recursive)  return NbBundle.getBundle(RefreshCommand.class).getString("RefreshCommand.nameRecursive") + " " + fls[0].toString();
        return NbBundle.getBundle(RefreshCommand.class).getString("RefreshCommand.name") + " " + fls[0].toString();
    }
    
    protected void applyInputParams() {}
    
    protected void initCommand() {
        // here the the commands is initiated just right before running the command
        D.deb("initCommand()");
        int dirNum = 0;
        CvsRefreshCommand command;
        File[] files = getFiles();
        command = new CvsRefreshCommand();
        if (files[0].isDirectory()) {
            // libclient has to be set, because it cannot be set during creation -> no filesystem available
            libClient.setLocalPath(files[0].getAbsolutePath());
            command.setDirectoryCheck(true);
            command.setRecursive(recursive);
        } else {
            libClient.setLocalPath(files[0].getParentFile().getAbsolutePath());
            command.setDirectoryCheck(false);
        }
        if (cvsSubdirs != null) {
            command.setFiles(cvsSubdirs);
        } else {
            command.setFiles(files);
        }
        toDoCommands.addElement(command);
    }
    
            // for dirs to be recursively refreshed, perform check wheather they are CVS dirs. If not..
    // search for all it's subdirs that are..
    private File[] findRelevantDirs(File[] files) {
        LinkedList localOnes = new LinkedList();
        LinkedList okFiles = new LinkedList();
        for (int index = 0; index < files.length; index++) {
            File file = files[index];
            if (file.isDirectory()) {
                File[] cvsDir = file.listFiles(new FileFilter() {
                    public boolean accept(File f) {
                        if (f.isDirectory() && f.getName().equalsIgnoreCase("CVS")) return true;
                        return false;
                    }
                });
                if (cvsDir.length == 0)  {
                    localOnes.add(file);
//                    cacheObject.fireCacheHandlerEvent(FileSystemCache.EVENT_CHANGED, file);
                }
                else okFiles.add(file);
            } else {
                // okFiles.add(file);
                // files are not included.. recursive refresh makes no sense for them
            }
        }
        if (localOnes.size() == 0) {
            // everything is ok.. return the initial file[] array.
            return files;
        } 
        Iterator it = localOnes.iterator();
        while (it.hasNext()) {
            File main = (File)it.next();
            File[] toFind = main.listFiles();
            if (toFind.length > 0) {
                toFind = findRelevantDirs(toFind);
                if (toFind.length > 0) {
                    for (int ind = 0; ind < toFind.length; ind++) {
                        okFiles.add(toFind[ind]);
                    }
                }
            }
        }
        File[] toReturn = new File[okFiles.size()];
        toReturn = (File[])okFiles.toArray(toReturn);
        return toReturn;
    }
    
    protected void finishedCommand() {
        // called when the communication with server ends and the command thread is about to end..
        // used for calling visualizer stuff..
        D.deb("finishedCommand finished");
        fireUpdateCache();
    }
    
    protected void beforeEachExecute() {
        D.deb("BeforeEachExecute()");
        resultList = new ArrayList();
        CvsCacheClient client = (CvsCacheClient)getClientProvider();
        File[] fls = getFiles();
        File file = fls[0];
        if (cvsSubdirs != null && cvsSubdirs.length > 0) {
            file = cvsSubdirs[0];
        }
        
        String pathToRoot;
        if (file.isDirectory()) {
            pathToRoot = file.getAbsolutePath() + CVS_ROOTFILE;
//            libClient.setLocalPath(file.getAbsolutePath()); //TODO remove
        } else {
//            libClient.setLocalPath(file.getParentFile().getAbsolutePath()); //TODO remove
            pathToRoot = file.getParentFile().getAbsolutePath() + CVS_ROOTFILE;
        }
        String root = client.readRoot(new File(pathToRoot));
        if (root != null) {
            client.setCvsRootString(root);
            client.getGlobalOptions().setCVSRoot(root);
            setGlobalOptions(client.getGlobalOptions());    
        } else {
            hardCommandStop();
            TopManager.getDefault().setStatusText(
                NbBundle.getBundle(RefreshCommand.class).getString("CvsCacheClient.notCvsDir"));
        }
    }
    
    protected void afterEachExecute() {
        D.deb("AfterEachExecute()");
        /*
        CvsRefreshCommand statCom = (CvsRefreshCommand)getCurrentCommand();
        if (resultList.size() > 0) {
            File[] flz = statCom.getFiles();
            if (flz[0].isDirectory()) {
                // TODO directory display.
                updateCacheForDir(resultList);
            } else { // we display single files..
                Iterator it = resultList.iterator();
                while (it.hasNext()) {
                    StatusInformation info = (StatusInformation)it.next();
                    updateCacheForSingleFile(info);
                }
            }
        }
         */
    }
  
    
/*
    
    private void changeStatus(StatusInformation info, File file, boolean fireEvent) {
        CacheFile cvsFile = CacheHandler.getInstance().getCacheFile(file,
                                CacheHandler.STRAT_REFRESHING, JavaCvsCache.JAVA_CACHE_NAME);
        D.deb("(changeStatus() " + file.getName());
        if (cvsFile == null) {
            // TODO prolly "needs chechout" file.. virtual FILE
            D.deb("(changeStatus() - Not locally written file");
            File parentFile = file.getParentFile();
            CacheDir cvsDir = (CacheDir)CacheHandler.getInstance().getCacheFile(parentFile, 
                                        CacheHandler.STRAT_REFRESHING, JavaCvsCache.JAVA_CACHE_NAME);
            if (cvsDir != null) {            
                CvsCacheFile newFile = new CvsCacheFile(cvsDir.getCacheName(), file.getName());
                cvsDir.addFile(newFile, true);
                cvsFile = newFile;
            } else {
                E.err(" javacvs: We've encountered a shadow directory. - should never happen.");
            }    
        }
        cvsFile.setStatus(JavaCvsStatusManager.convertStatus(info.getStatus()));
        D.deb("file" + info.getFile().toString() + "   status:" + cvsFile.getStatus());
        // now fire the cache event, once we'returndone with the update
        if (fireEvent) {
            FileSystemCache cache = null;
            if (cvsFile instanceof CvsCacheDir) {
                CvsCacheDir dir = (CvsCacheDir)cvsFile;
                cache = dir.getCacheObject();
            } else {
                cache = cvsFile.getParent().getCacheObject();
            }
            cache.fireCacheHandlerEvent(FileSystemCache.EVENT_CHANGED,cvsFile);
        }
    }
*/
    protected void executeFailed(Exception exc) {
        // DO fire EVENT_CHANGED event on all files/dirs
        final String excStr =  exc.getLocalizedMessage();
        JavaCvsFileSystem.errorMessage(org.openide.util.NbBundle.getBundle(RefreshCommand.class).getString(
        "RefreshCommand.error") + " " + excStr);
        fireUpdateCache();
/*        CvsRefreshCommand statCom = (CvsRefreshCommand)getCurrentCommand();
        if (statCom.getRecursive() || files[0].isDirectory())  {
            CacheDir rootCacheDir = null;
            rootCacheDir = (CacheDir)CacheHandler.getInstance().getCacheFile(files[0],
            CacheHandler.STRAT_REFRESHING, JavaCvsCache.JAVA_CACHE_NAME);
            if (rootCacheDir != null) {
                int event = (statCom.getRecursive() ? FileSystemCache.EVENT_CHANGED_RECURSIVELY
                                                    : FileSystemCache.EVENT_CHANGED);
                JavaCvsCache cache = (JavaCvsCache)rootCacheDir.getCacheObject();
                cache.fireCacheHandlerEvent(event, rootCacheDir);
                cache.removeLockedFileObjects(rootCacheDir);
            }
        } else {
            // refresh is on files.
            for (int index = 0; index < files.length; index++) {
                CacheFile cacheFile = CacheHandler.getInstance().getCacheFile(files[index],
                CacheHandler.STRAT_REFRESHING, JavaCvsCache.JAVA_CACHE_NAME);
                if (cacheFile != null) {
                    CacheDir rootCacheDir = cacheFile.getParent();
                    if (rootCacheDir != null) {
                        rootCacheDir.getCacheObject().fireCacheHandlerEvent(
                                       FileSystemCache.EVENT_CHANGED, cacheFile);
                    }
                }
            }
        }
  */      
    }
    
    
    /**
     * Called when file status information has been received
     */
    public void fileInfoGenerated(FileInfoEvent e) {
        D.deb("fileStatus()");
        StatusInformation info = (StatusInformation)e.getInfoContainer();
        if (info != null) {
            resultList.add(info);
            updateCache(info);
 //           System.out.println("got info=" + info.getFile().getName());
        }
    }
    
    protected Class getDefaultDisplayerType() {
        return org.netbeans.modules.cvsclient.commands.NullDisplayer.class;
    }
    
    
    /** if no param input was set before staring the command, 
 * the default defined in this method is used
 */
    protected CommandParamInput createDefaultParamInput() {
        return null;
    }
    
    protected void fireUpdateCache() {
        fireUpdateCache(CacheHandler.STRAT_REFRESHING);
        fireCacheEvents(CacheHandler.STRAT_REFRESHING);
    }
    
    public void messageSent(MessageEvent e) {
        // do nothing..
    }
    
    class CvsRefreshCommand extends StatusCommand {
        private boolean directoryCheck;
        
        public CvsRefreshCommand() {
            super();
        }
        
        public void setDirectoryCheck(boolean isDir) {
            directoryCheck = isDir;
        }
        
        public boolean isDirectoryCheck() { return directoryCheck;}
        
        protected Builder createBuilder(EventManager eventMan) {
            return new MyBuilder(eventMan, CvsRefreshCommand.this);
        }
        
        class MyBuilder implements Builder {
            private EventManager eventManager;
            private StatusCommand statusCommand;
            private StatusInformation info;
            private String fileDirectory = "";
            private static final String UNKNOWN = "cvs server: nothing known about";
            private static final String EXAM_DIR = "cvs server: Examining";
            private static final String NOT_IN_REPOSITORY = "No revision control file";
            
            private static final String FILE              = "File: ";
            private static final String WORK_REV          = "   Working revision:";
            private static final String REP_REV           = "   Repository revision:";
            private static final String TAG               = "   Sticky Tag:";
            private static final String DATE              = "   Sticky Date:";
            private static final String OPTIONS           = "   Sticky Options:";
            
            public MyBuilder(EventManager eventMan, StatusCommand statCom) {
                eventManager = eventMan;
                statusCommand = statCom;
            }
            
            public void outputDone() {
             //TODO shall I care here?    
            }
            
            public void parseLine(String line, boolean isErrorMessage) {
                D.deb("line = " +line);
                if (line.startsWith(UNKNOWN)) {
                    eventManager.fireCVSEvent(new
                    FileInfoEvent(this, info));
                    info = null;
                }
                if (line.startsWith(EXAM_DIR)) {
                    fileDirectory = line.substring(22);
                    return;
                }
                if (line.startsWith(FILE)) {
                    info = new StatusInformation();
                    processFileAndStatusLine(line.substring(FILE.length()));
                    return;
                }
                if (line.startsWith(REP_REV)) {
                    line = line.substring(REP_REV.length()).trim();
                    if (line.startsWith(NOT_IN_REPOSITORY)) {
                        eventManager.fireCVSEvent(new
                        FileInfoEvent(this, info));
                        info = null;
                    }
                    return;
                }
                if (line.startsWith(OPTIONS)) {
                    eventManager.fireCVSEvent(new
                    FileInfoEvent(this, info));
                    info = null;
                }
            }
            
            protected void processFileAndStatusLine(String line) {
                int statusIndex = line.lastIndexOf("Status:");
                String fileName = line.substring(0, statusIndex).trim();
                if (fileName.startsWith("no file")) {
                    fileName = fileName.substring(8);
                }
                //System.err.println("Filename found is: " + fileName);
                info.setFile(new File(statusCommand.getLocalPath() + File.separator +
                ((!fileDirectory.equals("."))? (fileDirectory+ File.separator) :  "")  + fileName));
                String status = line.substring(statusIndex + 8).trim();
                if (status.equals("Up-to-date"))
                    info.setStatus(FileStatus.UP_TO_DATE);
                else if (status.equals("Locally Modified"))
                    info.setStatus(FileStatus.MODIFIED);
                else if (status.equals("Locally Added"))
                    info.setStatus(FileStatus.ADDED);
                else if (status.equals("Locally Removed"))
                    info.setStatus(FileStatus.REMOVED);
                else if (status.equals("Needs Checkout"))
                    info.setStatus(FileStatus.NEEDS_CHECKOUT);
                else if (status.equals("Needs Patch"))
                    info.setStatus(FileStatus.NEEDS_PATCH);
                else if (status.equals("Needs Merge"))
                    info.setStatus(FileStatus.NEEDS_MERGE);
                else if (status.equals("File had conflicts on merge"))
                    info.setStatus(FileStatus.HAS_CONFLICTS);
                else if (status.equals("Unknown"))
                    info.setStatus(FileStatus.UNKNOWN);
                else
                    D.deb("Unknown file status: " + status);
            }
            
        } // MyBulider class end.
        
        
    } //CvsRefreshCommand ends here
    
    
    
}
