/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */



package org.netbeans.modules.cvsclient.commands;


import org.openide.util.*;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileSystem;
import java.io.*;
import java.util.*;
import java.awt.Dialog;

import org.netbeans.modules.cvsclient.*;
import org.netbeans.lib.cvsclient.command.update.*;
import org.netbeans.lib.cvsclient.command.add.*;
import org.netbeans.lib.cvsclient.command.status.*;
import org.netbeans.lib.cvsclient.command.checkout.*;
import org.netbeans.lib.cvsclient.command.remove.*;
import org.netbeans.lib.cvsclient.command.commit.*;
import org.netbeans.lib.cvsclient.command.*;
import org.netbeans.modules.vcscore.cache.*;
import org.netbeans.modules.cvsclient.caching.*;
import org.netbeans.modules.cvsclient.JavaCvsStatusManager;

import org.netbeans.modules.cvsclient.util.Debug;

/**
 *
 * @author  mkleint
 * @version
 */
public abstract class CacheUpdatingFsCommand extends FileSystemCommand {
    private Debug E=new Debug("CacheUpdatingFsCommand", true); // NOI18N
    private Debug D=E;
    
    public static final String UPD_CONFLICT = "C";
    public static final String UPD_MODIFIED = "M";
    public static final String UPD_ADDED = "A";
    public static final String UPD_REMOVE = "R";
    public static final String UPD_UPDATE = "U";    
    public static final String UPD_PATCH = "P";    
    public static final String UPD_UNKNOWN = "?"; 
    public static final String UPD_TAG = "T";
    public static final String UPD_DEL_TAG = "D";
    
    private HashMap dirs;
    
    public CacheUpdatingFsCommand() {
        super();
        dirs = new HashMap();
    }
    
    public CacheUpdatingFsCommand(ClientProvider provider) {
        this();
        setClientProvider(provider);
    }
    
/*    public CacheUpdatingFsCommand(ClientProvider provider, JavaCvsFileSystem fs) {
        super(provider, fs);
    }
 */
    
    protected void updateCache(DefaultFileInfoContainer updInfo) {
        if (updInfo == null || updInfo.getFile() == null) return;       
        D.deb("CacheUpdated for " + updInfo.getFile());
        HashMap dir = getDirMap(updInfo.getFile().getParentFile());
        String type = updInfo.getType();
        String status = "";
        if (type.equals(UPD_TAG) || type.equals(UPD_DEL_TAG)) {
            return;
        }
        if (type.equals(UPD_UPDATE) || type.equals(UPD_PATCH)) {
            // we are up-to-date
            status = JavaCvsStatusManager.UPTODATE;
        }
        // TODO- HACK ?? - MAYBE ADDED SHOULD BECOME Up-to-date.. are just A because being added to repository??
        else if (type.equals(UPD_ADDED)) {
            status = JavaCvsStatusManager.LOCALLY_ADDED;
        }
        else if (type.equals(UPD_REMOVE)) {
            status = JavaCvsStatusManager.LOCALLY_REMOVED;
        }
        else if (type.equals(UPD_MODIFIED)) {
            status = JavaCvsStatusManager.LOCALLY_MODIFIED;
        }
        else if (type.equals(UPD_CONFLICT)) {
            status = JavaCvsStatusManager.CONFLICT;
        }
        else if (type.equals(UPD_UNKNOWN)) {
            // TODO unknown or Local???
            status = JavaCvsStatusManager.UNKNOWN;
        }
        dir.put(updInfo, status);
    }
    
    private HashMap getDirMap(File parentFile) {
        HashMap dir = (HashMap)dirs.get(parentFile);
        if (dir == null) {
            dir = new HashMap();
            dirs.put(parentFile, dir);
        }
        return dir;
    }
    
    protected void updateCache(AddInformation addInfo) {
        D.deb("updateCache for " + addInfo.getFile());
        HashMap dir = getDirMap(addInfo.getFile().getParentFile());
        String type = addInfo.getType();
        String status = "";
        if (addInfo.isDirectory()) {
            status = " ";
        } else {
            if (type.equals(UPD_UPDATE)) {
                // we are up-to-date
                status = JavaCvsStatusManager.UPTODATE;
            }
            else if (type.equals(UPD_ADDED)) {
                status = JavaCvsStatusManager.LOCALLY_ADDED;
            } 
        }
        dir.put(addInfo, status);
    }

    protected void updateCache(RemoveInformation removeInfo) {
        D.deb("updateCache for " + removeInfo.getFile());
        HashMap dir = getDirMap(removeInfo.getFile().getParentFile());
        String status;
        if (removeInfo.isRemoved()) {
            status = JavaCvsStatusManager.LOCALLY_REMOVED;
            dir.put(removeInfo, status);        
        }
    }

    protected void updateCache(CommitInformation commitInfo) {
        D.deb("updateCache for " + commitInfo.getFile());
        HashMap dir = getDirMap(commitInfo.getFile().getParentFile());
        String status;
        if (commitInfo.getType().equals(CommitInformation.REMOVED)) {
            dir.put(commitInfo, null);        
        } else {
            status = JavaCvsStatusManager.UPTODATE;
            dir.put(commitInfo, status);
        }
    }

    /** TEMPORARY solution until I write the correct stuff.
     *  It should be similar to the way refresh works in the cache.
     * However it should use less force than refresh.
     * checkServer(File rootDir, List returnedInfo)
     * 1. prepare and locks the rootDir recursively.
     * 2. perform the cache updating activities on known files.
     * 3. unlock the fileobjects.
     * this assumes I create a new Strategy for access into the cache.. COMMANDS = 6.
     *  (needs to be higher then Disk, but should IMHO load the disk level as well)
     */
/*    protected void checkServer(File[] filesToCheck, boolean recursive) {
            getFileSystem().doRefresh(filesToCheck, recursive);
    }
 */
    
    protected void updateCache(StatusInformation info) {
        D.deb("updateCache for " + info.getFile());
        HashMap dir = getDirMap(info.getFile().getParentFile());
        dir.put(info, JavaCvsStatusManager.convertStatus(info.getStatus()));
    }
    
    protected void fireCacheEvents(int strategy) {
        CacheHandler handler = CacheHandler.getInstance();
        JavaCvsCache cacheObject = (JavaCvsCache)handler.getCache(JavaCvsCache.JAVA_CACHE_NAME);
        HashSet set = new HashSet();
        File[] fls = getFiles();
        for (int index = 0; index < fls.length; index++) {
            File file = fls[index];
  //          System.out.println("file=" + file.getName());
            if (file != null) {
                if (file.isDirectory()) {
                    set.add(file);
                } else {
                    set.add(file.getParentFile());
                }
//                System.out.println("added to set=" + file.getName());
            }
        }
        Iterator it = set.iterator();
        while (it.hasNext()) {
            File dir = (File)it.next();
//            System.out.println("looking for =" + dir.getName());
            CacheFile dirCache = handler.getCacheFile(dir,
                   strategy, JavaCvsCache.JAVA_CACHE_NAME);
            if (dirCache != null) {
                    cacheObject.fireCacheHandlerEvent(cacheObject.EVENT_ADD, dirCache);
//                    System.out.println("fired!");
            }
        }
    }
    
    protected void fireUpdateCache() {
        fireUpdateCache(JavaCvsCache.STRAT_COMMANDS);
        fireCacheEvents(JavaCvsCache.STRAT_COMMANDS);
    }
    
    protected void fireUpdateCache(int strategy) {
        D.deb("fireCacheUpdate!");
/*        if (!isFileSystemSet()) {
            dirs = new HashMap();
            return;
        }
 */
        
        CacheHandler handler = CacheHandler.getInstance();
        JavaCvsCache cacheObject = (JavaCvsCache)handler.getCache(JavaCvsCache.JAVA_CACHE_NAME);
/*
        // first refresh the getFiles() array (directories only).. Just to make sure they are in cache.
        File[] fls = getFiles();
        if (fls != null) {
            for (int i = 0; i < fls.length; i++) {
                if (!fls[i].isDirectory()) continue;
                if (isFileSystemSet()) {
                    String fsRoot = getFileSystem().getRootDirectory().getAbsolutePath();
                    // make path relative to fs..
                    String path = fls[i].getAbsolutePath().substring(fsRoot.length());
                    path = path.replace('\\','/');
                    if (path.startsWith("/")) path = path.substring(1);
                    FileObject fo = getFileSystem().findResource(path);
                }
                CacheFile dirCache = handler.getCacheFile(fls[i],
                        strategy, JavaCvsCache.JAVA_CACHE_NAME);
                if (dirCache != null) {
                    cacheObject.fireCacheHandlerEvent(cacheObject.EVENT_ADD, dirCache);
                }
            }
        } else {
            // special case for Checkout
             if (isFileSystemSet()) {
                FileObject fo = getFileSystem().findResource("");
                CacheFile dirCache = handler.getCacheFile(getFileSystem().getRootDirectory(),
                    strategy, JavaCvsCache.JAVA_CACHE_NAME);
                if (dirCache != null) {
                    cacheObject.fireCacheHandlerEvent(cacheObject.EVENT_ADD, dirCache);
                }
             }
        }
*/        
        TreeSet drs = new TreeSet(dirs.keySet());
        Iterator it = drs.iterator();
        // iterate through the directories
        //        System.out.println("cacheObject=" + cacheObject);
        //        boolean dirToRefresh;
        while (it.hasNext()) {
            //            dirToRefresh = false;
            File dir = (File)it.next();
            if (dir == null) continue;
            // do lock and prepare cache
            // TODO should we make sure and lock at fs root? I think not.. the locking of fileobjects should do that for us..
//            System.out.println("doing locking for directory:" + dir.getAbsolutePath());
            cacheObject.lockFileObjects(dir.getAbsolutePath(), false);
            cacheObject.prepareCache(dir, false, strategy);
            
            HashMap fileList = (HashMap)dirs.get(dir);
            Iterator innerIt = fileList.keySet().iterator();
            while (innerIt.hasNext()) {
                // perform update
                FileInfoContainer info = (FileInfoContainer)innerIt.next();
                CacheFile file = handler.getCacheFile(info.getFile(), strategy, JavaCvsCache.JAVA_CACHE_NAME);
                if (file != null) {
                    String status = (String)fileList.get(info);
                    if (status == null) {
                        // status is null->> means to remove the file
                        CacheDir dirCache = (CacheDir)handler.getCacheFile(dir, strategy, JavaCvsCache.JAVA_CACHE_NAME);
                        if (dirCache != null) {
                            dirCache.removeFile(file.getName(),false);
                            D.deb("parentDir found - removed " + file.getName());
                            //                        dirToRefresh = true;
                        } else {
                            D.deb("dir not found! = " + dir);
                        }
                        
                    }
                    file.setStatus(status);
//                    cacheObject.fireCacheHandlerEvent(FileSystemCache.EVENT_CHANGED, file);
                } else {
                    CacheDir dirCache = (CacheDir)handler.getCacheFile(dir, strategy, JavaCvsCache.JAVA_CACHE_NAME);
                    if (dirCache != null) {
                        file = new CvsCacheFile(JavaCvsCache.JAVA_CACHE_NAME, info.getFile().getName());
                        dirCache.addFile(file, false);
                        D.deb("parentDir found - added " + file.getName());
                        String status = (String)fileList.get(info);
                        file.setStatus(status);
//                        cacheObject.fireCacheHandlerEvent(FileSystemCache.EVENT_CHANGED, file);
                        //                        dirToRefresh = true;
                    } else {
                        D.deb("dir not found = " + dir);
                    }
                }
            } // end of file iteration within the directory
            // release locks
            cacheObject.removeLockedFileObjects(dir.getAbsolutePath());
        }
        dirs = new HashMap();
    }
    
}
