/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient.commands;

import javax.swing.text.*;
import javax.swing.*;
import org.openide.DialogDescriptor;
import org.openide.util.NbBundle;
import org.netbeans.modules.vcscore.util.Debug;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 *
 * @author  mkleint
 * @version
 */

public class ErrorLogPanel extends javax.swing.JPanel implements ActionListener {
    private Debug E=new Debug("ErrorLogPanel", false); // NOI18N
    private Debug D=E;
    
    private boolean expanded;
    private StyledDocument logListing;
    private StringBuffer buff;
    private String commandLineForm;
    private DialogDescriptor dd;
    private boolean noLogAvailable;
    private java.awt.Dimension bigDim;
    private java.awt.Dimension smallDim;
    
    
    /** Creates new form ErrorLogPanel */
    public ErrorLogPanel(String message, FileSystemCommand comm, Exception exc) {
        initComponents ();
        lblMessage.setText(message);
        spLogList.setVisible(false);
        expanded = false;
        logListing = null;
        buff = comm.getCurrentCommandLog();
        buff.append("\n\n" + exc.toString());
        if (buff == null) noLogAvailable = true;
                    else  noLogAvailable = false;
        commandLineForm = comm.getCurrentCommand().getCVSCommand();
        lblCommand.setText("( " + commandLineForm + " )");
        dd= null;
        bigDim = new java.awt.Dimension(400, 250);
        smallDim = new java.awt.Dimension(400, 150);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        spLogList = new javax.swing.JScrollPane();
        tpLogList = new javax.swing.JTextPane();
        pnlMessage = new javax.swing.JPanel();
        lblMessage = new javax.swing.JLabel();
        lblCommand = new javax.swing.JLabel();
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints2;
        
        spLogList.setPreferredSize(new java.awt.Dimension(150, 100));
        spLogList.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        
        tpLogList.setPreferredSize(new java.awt.Dimension(440, 200));
        spLogList.setViewportView(tpLogList);
        
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 0;
        gridBagConstraints2.gridy = 1;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints2.insets = new java.awt.Insets(6, 12, 11, 11);
        gridBagConstraints2.weightx = 1.0;
        gridBagConstraints2.weighty = 1.0;
        add(spLogList, gridBagConstraints2);
        
        
        pnlMessage.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        lblMessage.setText("jLabel1");
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(0, 1, 5, 0);
        gridBagConstraints1.weightx = 1.0;
        pnlMessage.add(lblMessage, gridBagConstraints1);
        
        
        lblCommand.setText("jLabel1");
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(0, 1, 0, 0);
        pnlMessage.add(lblCommand, gridBagConstraints1);
        
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints2.insets = new java.awt.Insets(12, 12, 5, 11);
        add(pnlMessage, gridBagConstraints2);
        
    }//GEN-END:initComponents
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JScrollPane spLogList;
    private javax.swing.JTextPane tpLogList;
    private javax.swing.JPanel pnlMessage;
    private javax.swing.JLabel lblMessage;
    private javax.swing.JLabel lblCommand;
    // End of variables declaration//GEN-END:variables
    
    public boolean showLog() {
        if (expanded) {
            expanded = false;
            bigDim = getSize();
            spLogList.setVisible(false);
            setPreferredSize(smallDim);
            return false;
        } else {
            if (logListing == null) {
                loadDocument();
            }
            expanded = true;
            smallDim = this.getSize();
            spLogList.setVisible(true);
            setPreferredSize(bigDim);
            return true;
        }
    }
    
    private void loadDocument() {
        if (buff == null) return;
        logListing = new DefaultStyledDocument();
        AttributeSet defaultAttrSet = new SimpleAttributeSet();
        try {
            logListing.insertString(0, buff.toString(), defaultAttrSet);
            tpLogList.setDocument(logListing);
            spLogList.getVerticalScrollBar().setValue(spLogList.getVerticalScrollBar().getMaximum());
        } catch (BadLocationException exc) {
            //TODO catch what to do??
            E.err("error while loading logs");
        }
    }
    
    public void setDialogDescriptor(DialogDescriptor dialogDesc) {
        dd = dialogDesc;
    }
    
    public void actionPerformed(ActionEvent evt) {
        if (dd == null) return; //sanity check - dialog descriptor needs to be set after creating the dialog descriptor - in FileSystemCommand
        String button = evt.getActionCommand();
        if (button.equals(NbBundle.getBundle(ErrorLogPanel.class).getString("ErrorLogPanel.closeButton"))) {
            return;
        }
        boolean showing = showLog();
        if (showing) {
            int size;
            if (noLogAvailable) size = 1;
            else size = 2;
            String[] options = new String[size];
            options[0] = NbBundle.getBundle(ErrorLogPanel.class).getString("ErrorLogPanel.hideButton");
            options[size - 1] = NbBundle.getBundle(ErrorLogPanel.class).getString("ErrorLogPanel.closeButton");
            Object[] closing = new Object[1];
            closing[0] = options[size - 1];
            dd.setOptions(options);
            dd.setClosingOptions(closing); // all are closing
        } else {
            int size;
            if (noLogAvailable) size = 1;
            else size = 2;
            String[] options = new String[size];
            options[0] = NbBundle.getBundle(ErrorLogPanel.class).getString("ErrorLogPanel.logButton");
            options[size - 1] = NbBundle.getBundle(ErrorLogPanel.class).getString("ErrorLogPanel.closeButton");
            Object[] closing = new Object[1];
            closing[0] = options[size - 1];
            dd.setOptions(options);
            dd.setClosingOptions(closing); // all are closing
        }
    }
    
    public void initDialogButtons(DialogDescriptor dd) {
            int size;
            if (noLogAvailable) size = 1;
            else size = 2;
            String[] options = new String[size];
            options[size - 1] = NbBundle.getBundle(ErrorLogPanel.class).getString("ErrorLogPanel.closeButton");
            if (!noLogAvailable) options[0] = NbBundle.getBundle(ErrorLogPanel.class).getString("ErrorLogPanel.logButton");
            Object[] closing = new Object[1];
            closing[0] = options[size - 1];
            dd.setOptions(options);
            dd.setClosingOptions(closing); // all are closing
    }
}
