/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.cvsclient.commands;

import java.io.*;
import org.openide.util.*;
import org.openide.util.actions.SystemAction;
import org.openide.util.actions.CallableSystemAction;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import org.openide.awt.JInlineMenu;
import javax.swing.JMenuItem;
import org.openide.awt.JMenuPlus;
import javax.swing.JMenu;
import org.netbeans.modules.cvsclient.IndependantClient;


/**
 * Subclass of TopComponent that is used within the JavaCvs mode.
 * @author  mkleint
 */
public class JavaCvsTopComponent extends org.openide.windows.TopComponent {

    private volatile PersistentCommandDisplayer displayer;
    
    static final long serialVersionUID = 5519624846009571015L;
    
    /** Creates new JavaCvsTopComponent */
    public JavaCvsTopComponent() {
        super();
    }

    public void addDisplayer(PersistentCommandDisplayer displ) {
        displayer = displ;
        add(displ.getComponent());
    }
    
    public CommandDisplayer getEqualDisplayer(File file, Class type,  Object comparisonData) {
        if (displayer == null) return null;
        if (displayer.equalDisplayedData(file, type, comparisonData)) {
            return displayer;
        }
        return null;
    }
    
    public PersistentCommandDisplayer getDisplayer() {
        return displayer;
    }
    
    /** Get the system actions which will appear in
     * the popup menu of this component.
     * <p>Subclasses are encouraged to override this method to specify
     * their own sets of actions.
     * <p>Remember to call the super method when overriding and add your actions
     * to the superclass' ones (in some order),?
     * because the default implementation provides support for standard
     * component actions like save, close, and clone.
     * @return system actions for this component
 */
    public SystemAction[] getSystemActions() {
        SystemAction[] retValue;
        retValue = super.getSystemActions();
        SystemAction[] all = new SystemAction[retValue.length + 1];
        all[0] = new JTCAction();
        for (int i=0; i < retValue.length; i++) {
            all[i + 1] = retValue[i];
        }
        return all;
    }  

        /**
         * Disable serialization.
         * @return null
         */
        protected Object writeReplace () throws java.io.ObjectStreamException {
            return null;
        }
    
    
    class JTCAction extends CallableSystemAction implements ActionListener {
        
        static final long serialVersionUID = -9081158884374905985L;
        
        public JTCAction() {
        }

    /** Human presentable name of the action. This should be
    * presented as an item in a menu.
    * @return the name of the action
    */
    public String getName () {
        return NbBundle.getBundle (JTCAction.class).
               getString ("CTL_JTCAction");
    }

    /** Help context where to find more about the action.
    * @return the help context for this action
    */
    public HelpCtx getHelpCtx () {
        return new HelpCtx(JTCAction.class);
    }
    
    public boolean isEnabled() {
        return true;
    }
    
    /** The action's icon location.
    * @return the action's icon location
    */
    protected String iconResource () {
        return "/org/netbeans/modules/cvsclient/JavaCvsFileSystemIcon.gif"; // NOI18N
    }

    /** Opens packaging view. */
    public void performAction () {
    }

    public void actionPerformed(java.awt.event.ActionEvent e){    
        Class commandClass = getDisplayer().getFileSystemCommandClass();
        try {
            FileSystemCommand command = (FileSystemCommand)commandClass.newInstance();
            String switches =  getDisplayer().getCommandSwitches();
            ClientProvider provider = getDisplayer().getClientProvider();
            File[] files = new File[1];
            files[0] = getDisplayer().getFileDisplayed();
            command.setFiles(files);
            command.setClientProvider(provider);
            command.setParamInput(new ParserParamInput(switches));
            command.startCommand();
        } catch (InstantiationException exc) {
        } catch (IllegalAccessException exc2) {
        }
    }
    
    public JMenuItem getPopupPresenter(){
        JMenuItem menu=new JMenuItem(g("JTCAction.displayName")); // NOI18N
        menu.setActionCommand(getName());
        menu.addActionListener(this);

        return menu;
    }

        //-------------------------------------------
    String g(String s) {
        return NbBundle.getBundle
        ("org.netbeans.modules.cvsclient.commands.Bundle").getString (s);
    }        
        
        
    }
}
