/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient.commands;

import org.netbeans.lib.cvsclient.command.Command;
import org.netbeans.lib.cvsclient.commandLine.GetOpt;
import java.util.StringTokenizer;
import java.util.LinkedList;



/**
 * A non-gui paramInput that can be used to set up command in user programs.
 * Should work with every default FileSystemCommand. 
 * (With the Exception of CvsCompleteModList, which OTOH doesn't require a ParamInput at all. :)
 *
 * @author  mkleint
 */

public class ParserParamInput implements CommandParamInput {

    private String parameters;
    
    /** Creates new ParserParamInput */
    public ParserParamInput(String parameters) {
        setCvsParameters(parameters);
    }

    /** fills the param input with a command.. the switches of the param input
     * act accordingly and set themselves that way.
     * NOTE: we ignore default settings in this ParamInput class. 
     * it is used just for settings the rigidly defined params.
     * 
     */
    public void getData(Command command) {
        // We ignore default settings
    }
    
    /** For GUI instances, uses the returned string as The dialog Label. 
     */
    public String getDialogTitle() {
        return null;
    }
    
    /** Defines wheather the ParamInput needs to be displayed or not. 
     * If set to true, it puts the class into the DialogDescriptor.
     */
    public boolean requiresUI() {
        return false;
    }
    
    /** sets the comand with the params the user selected in the dialog.
     */
    public void setData(Command command) {
        StringTokenizer tokens = new StringTokenizer(getCvsParameters(), " \"", true);
        boolean withinString = false;
        LinkedList stringList = new LinkedList();
        while (tokens.hasMoreTokens()) {
            String tok = tokens.nextToken();
            if (tok.equals("\"")) {
                 String param = "";
                 while (tokens.hasMoreTokens()) {
                    String toke2 = tokens.nextToken();
                    if (toke2.equals("\"")) break;
                    param = param + toke2;
                }
                stringList.add(param);
            } else {
                if (!tok.equals(" ")) {
                    stringList.add(tok);
                }
            }
        }
        String[] args = new String[stringList.size()];
        args = (String[])stringList.toArray(args);
        // DEBUG start
/*        for (int id = 0; id < args.length; id++) {
            System.out.println("args[" + id + "]=" + args[id]);
        }
 */
        // DEBUG end
        // 2. set the parameters to the command.
        command.resetCVSCommand();
        final String getOptString = command.getOptString();
        GetOpt go = new GetOpt(args, getOptString);
        int ch = -1;
        go.optIndexSet(0);
        while ((ch = go.getopt()) != go.optEOF)
        {   
            boolean ok = command.setCVSCommand((char)ch, go.optArgGet());
        }
    }
    
    public void setCvsParameters(String params) {
        parameters = params;
    }
    
    public String getCvsParameters() {
        return parameters;
    }
    
  
}
