/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient.commands;

import org.openide.util.*;
import org.netbeans.lib.cvsclient.command.*;
import org.netbeans.modules.cvsclient.commands.*;
import org.netbeans.modules.cvsclient.util.*;
import org.netbeans.modules.vcscore.util.Debug;
import java.util.*;
import javax.swing.event.*;
import javax.swing.table.*;

/**
 *
 * @author  mkleint
 * @version 
 */
public class TableInfoModel extends AbstractTableModel implements CommandsTableModel {
    private Debug E=new Debug("TableInfoModel", true); // NOI18N
    private Debug D=E;
    private int currentColumn;
    private int direction;
    
    public static final int ASCENDING = 1;
    public static final int DESCENDING = -1;
    
    
      protected LinkedList list;
      
      public TableInfoModel() {
          list = new LinkedList();
          D.deb("getRowCount()=" + list.size());
          setDirection(true);
          setActiveColumn(0);
      }

      public int getRowCount() {
          return list.size();
      }
      
      
      public int getColumnCount() {
          return 1;
      }
      
      public Object getValueAt(int row, int column) {
//          if (row < 0 || row >= getRowCount()) return "";
          FileInfoContainer info = (FileInfoContainer)list.get(row);
          switch (column) {
              case 0 : return info.getFile().getName();
          }
          return "";
      }
      
      
      public boolean isCellEditable(int rowIndex, int columnIndex)  {
          return false;
      }
      
      public java.lang.String getColumnName(int param) {
          switch (param) {
              case 0 : return NbBundle.getBundle(TableInfoModel.class).getString("TableInfoModel.fileName");
          }
          return "";
          
      }

// ------------------ CommandsTableModel -------      
// ===============================================      
      public Object getElementAt(int row) {
          if (row < 0 || row >= getRowCount()) return null;
          return list.get(row);
      }      
      
      public void addElement(Object object) {
          list.add(object);
      } 
      
    public void setActiveColumn(int index) {
        currentColumn = index;
    }    
    public int getActiveColumn() {
        return currentColumn;
    }    
    
    public void setDirection(boolean ascending) {
        if (ascending) direction = ASCENDING;
        else direction = DESCENDING;
    }    
    
    public int getDirection() {
        return direction;
    }    
      
    public boolean equals(Object obj) {
          if (obj instanceof TableInfoModel) {
              TableInfoModel obj2 = (TableInfoModel)obj;
              return (obj2.getActiveColumn() == getActiveColumn());
          }
          return false;
    }   
    
    public int compare(java.lang.Object obj1,java.lang.Object obj2) {
          if (!(obj1 instanceof FileInfoContainer) ||
          !(obj2 instanceof FileInfoContainer)) return 0;
          FileInfoContainer rev1 = (FileInfoContainer)obj1;
          FileInfoContainer rev2 = (FileInfoContainer)obj2;
          int result = 0;
          switch (getActiveColumn()) {
              case 0: // filesname
                  result = getDirection() * (rev1.getFile().getName().compareTo(rev2.getFile().getName()));
                  break;
          }
          return result;
    }
    
    public List getList() {
        return list;
    }    
    
    protected int compareVersions(String ver1, String ver2) {
          StringTokenizer token1 = new StringTokenizer(ver1, ".", false);
          StringTokenizer token2 = new StringTokenizer(ver2, ".", false);
          while (token1.hasMoreTokens() && token2.hasMoreTokens()) {
              try {
                Integer int1 = new Integer(token1.nextToken());
                Integer int2 = new Integer(token2.nextToken());
                int result = int1.compareTo(int2);
                if (result != 0) return result;
              } catch (NumberFormatException exc) {
                  return 0;
              }
          }
          if (token1.hasMoreTokens()) {
              return 1;
          } else {
              return -1;
          }
      }      

}

