/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */



package org.netbeans.modules.cvsclient.commands.add;

//import org.openide.*;
import org.openide.filesystems.*;
import org.openide.loaders.*;

import org.openide.util.*;
import java.io.*;
import java.util.*;

import org.netbeans.modules.cvsclient.*;
import org.netbeans.modules.cvsclient.commands.*;
import org.netbeans.lib.cvsclient.event.*;
import org.netbeans.lib.cvsclient.util.IgnoreFileFilter;

import org.netbeans.lib.cvsclient.command.add.*;
import org.netbeans.lib.cvsclient.command.*;
import org.netbeans.modules.cvsclient.caching.*;

import org.netbeans.modules.vcscore.util.Debug;

/** filesystem's idea of the cvs add command
 *
 * @author  mkleint
 * @version
 */
public class CvsAdd extends CacheUpdatingFsCommand {
    private Debug E=new Debug("CvsAdd", true); // NOI18N
    private Debug D=E;
    
    AddCommand command;
    private boolean recursive;
    private File[] binaryFiles;
    private int removedFilesCommandIndex;
    private int currentCommandIndex;
    
    
    /** Creates new Status */
    public CvsAdd(File[] files, ClientProvider fs) {
        super(fs);
        if (fs.isFileSystemClient()) {
            JavaCvsFileSystem fileSys = (JavaCvsFileSystem)fs;
            setFileSystem(fileSys);
        }        
        binaryFiles = null;
        setFiles(files);
        D.deb("Constructor");
        setRecursive(false); // default is cvs-style behaviour.
    }
    
    /** if set to recursive, for each directory added, adds everything in that directory as well.
     */
    
    public void setRecursive(boolean recursive) {
        this.recursive = recursive;
    }
    
    public boolean isRecursive() {
        return recursive;
    }
    
    public void setBinaryFiles(File[] binaries) {
        binaryFiles = binaries;
    }
    
    /** this method adds sub-files/dirs (when isRecursive()), otherwise does nothing, just returns it's parameter
     * CAUTION: might take quite some time if huge dir structure, to be called from non-AWT thread
     */
    private List processFiles(File[] files) {
            ArrayList list = new ArrayList();
            addFileArrayToList(list, files);
            // now return the array again.
/*            File[] toReturn = new File[list.size()];
            for (int ind = 0; ind < list.size(); ind++) {
                toReturn[ind] = (File)list.get(ind);
            }
 */
            return list;
    }
    /** used in processFiles() method only
     *  will recursively browse through the directories and add all "important files" to the list
     */
    private void addFileArrayToList(ArrayList list, File[] files) {
        for (int index= 0; index < files.length; index++) {
            File current = files[index];
            if (current == null) continue;
            if (current.getName().equalsIgnoreCase("CVS")) continue;
            if (current.isDirectory()) {
                list.add(current);
                addFileArrayToList(list, current.listFiles());
            } else {
                if (!isFileSystemSet()) {
                    list.add(current);
                    continue;
                }
                FileObject fo = getFileSystem().getFileObjectForFile(current);
                if (fo == null) continue;
                try {
                    DataObject dObj = DataObject.find(fo);
/*                    String root = getFileSystem().constructRootDirectory();
                    String absPath = current.getAbsolutePath();
                    String path = absPath.substring(root.length() + 1,absPath.length());
                    if (path.charAt(0) == File.separatorChar) { //another sanity check.
                        path = path.substring(1);
                    }
                    path = path.replace('\\', '/');
  */                addFileObjectsToList(dObj.files(), list, files);
                } catch (DataObjectNotFoundException exc) {
                    continue;
                }
                
            }
        }
    }
    
    /** add files related to FileObjects in Set into the ArrayList, and removes them from File[]
     *  FOR USE in addFileArrayToList()
     */
    private void addFileObjectsToList(Set set, ArrayList list, File[] files) {
        Iterator it = set.iterator();
        String root = getFileSystem().constructRootDirectory();
        while (it.hasNext()) {
            FileObject fo = (FileObject)it.next();
            String path = root + File.separator + fo.getPackageNameExt(File.separatorChar, '.');
            File file = new File(path);
            list.add(file);
            for (int index = 0; index < files.length; index++) {
                if (file.equals(files[index])) {
                    files[index] = null;
                    break;
                }
            }
        }
    }
    
    public String getName() {
        return NbBundle.getBundle(CvsAdd.class).getString("CvsAdd.name");
    }

    private LinkedList createBinaryAndOtherCommand(File[] normal, File[] binaryFiles, AddCommand originalCom) {
        LinkedList list = new LinkedList();
        if (binaryFiles != null) {
            command = new AddCommand();
                String localPath = libClient.getLocalPath();
                for (int i = 0; i < binaryFiles.length; i++) {
                    if (binaryFiles[i].getAbsolutePath().equals(localPath)) {
                        libClient.setLocalPath(binaryFiles[i].getParentFile().getAbsolutePath());
                    }
                }
            command.setFiles(binaryFiles);
            command.setMessage(originalCom.getMessage());
            command.setKeywordSubst("b");
            list.add(command);
        }
        File[] fls = normal;
        if (fls != null) {
            command = new AddCommand();
                String localPath = libClient.getLocalPath();
                for (int i = 0; i < fls.length; i++) {
                    if (fls[i].getAbsolutePath().equals(localPath)) {
                        libClient.setLocalPath(fls[i].getParentFile().getAbsolutePath());
                    }
                }
            command.setFiles(fls);
            command.setMessage(originalCom.getMessage());
            command.setKeywordSubst(originalCom.getKeywordSubst());
            list.add(command);
        }
        return list;
    }
    
    
/** if no param input was set before staring the command, 
 * the default defined in this method is used
 */
    protected CommandParamInput createDefaultParamInput() {
        return new AddParamInput(this);
    }

    
    protected Class getDefaultDisplayerType() {
        return AddInfoPanel.class;
    }
    protected void applyInputParams() {
        if (getDisplayerType() == AddInfoPanel.class) {
            setOutputDisplayer(new AddInfoPanel(getClientProvider(), this));
        }
        super.applyInputParams();
        if (binaryFiles != null) {
            AddCommand comm = (AddCommand)toDoCommands.get(0);
            comm.setKeywordSubst("b");
        }
        if (isRecursive()) {
                // adds all the subdirs + subfiles to the add..
                List list = new LinkedList();
                if (getFiles() != null) {
                    list.addAll(processFiles(getFiles()));
                }
                if (binaryFiles != null) {
                    list.addAll(processFiles(binaryFiles));
                }
                transform(list);
        }
        checkRemovedFiles();
        Iterator it = toDoCommands.iterator();
        while (it.hasNext()) {
            AddCommand comm = (AddCommand)it.next();
//            System.out.println("adding " + comm.getCVSArguments());
        }
    }
    
    protected void initCommand() {
        // here the the commands is initiated just right before running the command
        D.deb("initCommand()");
        int dirNum = 0;
        File[] binFls = binaryFiles;
        String localPath = libClient.getLocalPath();
        currentCommandIndex = 0;
        if (binFls != null) {
            command = (AddCommand)loadCommand(AddCommand.class);
            for (int i = 0; i < binFls.length; i++) {
                if (binFls[i].getAbsolutePath().equals(localPath)) {
                    libClient.setLocalPath(binFls[i].getParentFile().getAbsolutePath());
                }
            }
            command.setKeywordSubst("b");
            command.setFiles(binFls);
            toDoCommands.addElement(command);
        }
        command = (AddCommand)loadCommand(AddCommand.class);
        File[] fls = getFiles();
        for (int i = 0; i < fls.length; i++) {
            if (fls[i].getAbsolutePath().equals(localPath)) {
                libClient.setLocalPath(fls[i].getParentFile().getAbsolutePath());
            }
        }
        command.setFiles(getFiles());
        toDoCommands.addElement(command);
//        Iterator it = toDoCommands.iterator();
//        while (it.hasNext()) {
//            AddCommand comm = (AddCommand)it.next();
//            System.out.println("adding " + comm.getCVSArguments());
//        }
        super.initCommand();
    }
    
        
    /** gets executed before the current command in queue gets executed.
     * To access that command use the getCurrentCommand() method
     */
    protected void beforeEachExecute() {
        if (removedFilesCommandIndex >= currentCommandIndex) {
            AddCommand add = (AddCommand)getCurrentCommand();
            File[] fls = add.getFiles();
            if (fls != null && fls.length > 0) {
                libClient.setLocalPath(fls[0].getParentFile().getAbsolutePath());
            }
//            System.out.println("setting path to=" + fls[0].getParentFile().getAbsolutePath());
        }
        currentCommandIndex = currentCommandIndex + 1;
        super.beforeEachExecute();
    }
    
    protected void finishedCommand() {
        // update cache..
        fireUpdateCache();        
        super.finishedCommand();
    }
    
    /**
     * Called when file status information has been received
     */
    public void fileInfoGenerated(FileInfoEvent e) {
        D.deb("fileUpdated()");
        FileInfoContainer fInfo = e.getInfoContainer();
        if (fInfo != null) {
            if (fInfo instanceof AddInformation) {
                AddInformation info = (AddInformation)fInfo;
                updateCache(info);
            }
        }
        super.fileInfoGenerated(e);
    }
    
    /** if no displayer was set by the setOutputDisplayer() method,
     * a default one will be generated by the subclasses implementation
     *
     */
/*    protected CommandDisplayer createDefaultDisplayer() {
        // just one panel is generated for the whole command.. you can then add info while it's being generated
        return new AddInfoPanel(getClientProvider(), this);
    }
 */
    
    /**
     * Thi smethod performs the tranformation from 1 add command for all files/dirs to 
     * separate adds for dirs definied in the list. 
     * The original add command is moved to the end of the list and sends only files.
     * All this is done in order to enable recursive add
     */
/*    public void transform(List list) {
        int size = toDoCommands.size();
        AddCommand original = (AddCommand)toDoCommands.get(size - 1);
        toDoCommands.clear();
        Iterator it = list.iterator();
        while (it.hasNext()) {
            File file = (File)it.next();
            if (file.isDirectory()) {
                try {
                    AddCommand add = (AddCommand)original.clone();
                    File[] commFiles = new File[1];
                    commFiles[0] = file;
                    add.setFiles(commFiles);
                    toDoCommands.add(add);
                } catch (CloneNotSupportedException exc) {
                    //TODO??
                }
                it.remove();
            }
        }
        if (list.size() > 0) {
            // process through the ignoreFileFilter to get rid of the .cvsignore stuff..
            IgnoreFileFilter ignoreFilter = this.getClientProvider().getIgnoreFilter();
            if (ignoreFilter != null) {
                Iterator it2 = list.iterator();
                while (it2.hasNext()) {
                    File ignoreFile = (File)it2.next();
                    if (ignoreFilter.shouldBeIgnored(ignoreFile.getParentFile(), ignoreFile.getName())) {
                        it2.remove();
                    }
                }
            }
            File[] addFiles = new File[list.size()];
            addFiles = (File[])list.toArray(addFiles);
            original.setFiles(addFiles);
            toDoCommands.add(original);
        }
    }
    */
    public void transform(List list) {
        int size = toDoCommands.size();
        AddCommand originalCom;
        originalCom = (AddCommand)toDoCommands.get(toDoCommands.size() - 1);
        toDoCommands.clear();
        Iterator it = list.iterator();
        while (it.hasNext()) {
            File file = (File)it.next();
            if (file.isDirectory()) {
                AddCommand add = new AddCommand();
                File[] commFiles = new File[1];
                commFiles[0] = file;
                add.setFiles(commFiles);
                add.setMessage(originalCom.getMessage());
                toDoCommands.add(add);
                it.remove();
            }
        }
        if (list.size() > 0) {
            // process through the ignoreFileFilter to get rid of the .cvsignore stuff..
            IgnoreFileFilter ignoreFilter = this.getClientProvider().getIgnoreFilter();
            if (ignoreFilter != null) {
                Iterator it2 = list.iterator();
                while (it2.hasNext()) {
                    File ignoreFile = (File)it2.next();
                    if (ignoreFilter.shouldBeIgnored(ignoreFile.getParentFile(), ignoreFile.getName())) {
                        it2.remove();
                    }
                }
            }
            LinkedList binaryFiles = new LinkedList();
            LinkedList normalFiles = new LinkedList();
            Iterator it2 = list.iterator();
            while (it2.hasNext()) {
                File file = (File)it2.next();
                boolean binary = isBinary(file);
                if (binary) {
                      binaryFiles.add(file);
                } else {
                      normalFiles.add(file);
                }
            }
            File[] normal = new File[normalFiles.size()];
            File[] binary = new File[binaryFiles.size()];
            binary = (File[])binaryFiles.toArray(binary);
            if (binary.length == 0) {
                binary = null;
            }
            normal = (File[])normalFiles.toArray(normal);            
            if (normal.length == 0) {
                normal = null;
            }
            toDoCommands.addAll(createBinaryAndOtherCommand(normal, binary, originalCom));
        }
    }    
    
    private void checkRemovedFiles() {
        Iterator it = toDoCommands.iterator();
        LinkedList toDoRemovedCommands = new LinkedList();
        // 1. identify removed files and get them off regular commands.
        while (it.hasNext()) {
            HashMap removedList = new HashMap();
            AddCommand addCom = (AddCommand)it.next();
            File[] fls = addCom.getFiles();
            List notRemovedList = new LinkedList();
            for (int ind = 0; ind < fls.length; ind++) {
                if (!fls[ind].isDirectory() && !fls[ind].exists()) {
                    // add the removed file into the list.. 
                    // sort them by directory they belong to.
                    Vector vecFiles = (Vector)removedList.get(fls[ind].getParentFile().getAbsolutePath());
                    if (vecFiles == null) {
                        vecFiles = new Vector();
                        removedList.put(fls[ind].getParentFile().getAbsolutePath(), vecFiles);
                    }
                    vecFiles.add(fls[ind]);
                } else {
                    notRemovedList.add(fls[ind]);
                }
            }
            if (notRemovedList.size() == 0) {
                it.remove();
                continue;
            }
            if (notRemovedList.size() != fls.length) {
                File[] newFls = new File[notRemovedList.size()];
                newFls = (File[])notRemovedList.toArray(newFls);
                addCom.setFiles(newFls);
                // now create add commands for the grouped files..
                Iterator it2 = removedList.values().iterator();
                while (it2.hasNext()) {
                    Vector vecFiles = (Vector)it2.next();
                    AddCommand add = new AddCommand();
                    File[] remFiles = new File[vecFiles.size()];
                    remFiles = (File[])vecFiles.toArray(remFiles);
                    add.setFiles(remFiles);
                    add.setMessage(addCom.getMessage());
                    add.setKeywordSubst(addCom.getKeywordSubst());
                    toDoRemovedCommands.add(add);
                }
            }
        }
        // 
        if (toDoRemovedCommands.size() > 0) {
            removedFilesCommandIndex = toDoCommands.size();
            toDoCommands.addAll(toDoRemovedCommands);
        } else {
            removedFilesCommandIndex = -1;
        }
    }    
    
    private boolean isBinary(File file) {
        String ext = FileUtil.getExtension(file.getName());
        String mime = FileUtil.getMIMEType(ext);
        if (mime != null) {
           boolean text = mime.startsWith("content/unknown") || mime.startsWith("text/");
           return !text;
        }
        return false;
    }
    
    public void setFiles(File[] files) {
        if (files != null) {
            LinkedList binaryFiles = new LinkedList();
            LinkedList normalFiles = new LinkedList();
            for (int i = 0; i < files.length; i++) {
                File file = files[i];
                boolean binary = isBinary(file);
                if (binary) {
                      binaryFiles.add(files[i]);
//                      System.out.println("binary=" + files[i].getName());
                } else {
                      normalFiles.add(files[i]);
                }
            }
            File[] normal = new File[normalFiles.size()];
            File[] binary = new File[binaryFiles.size()];
            binary = (File[])binaryFiles.toArray(binary);
            normal = (File[])normalFiles.toArray(normal);
            super.setFiles(normal);
            setBinaryFiles(binary);
        } else {
            super.setFiles(files);
        }
    }
    
}
