/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */



package org.netbeans.modules.cvsclient.commands.checkout;


import org.openide.util.*;
import java.io.*;
import java.util.*;
import java.awt.Dialog;
import org.openide.*;

import org.netbeans.modules.cvsclient.JavaCvsFileSystem;
import org.netbeans.lib.cvsclient.event.*;

import org.netbeans.modules.cvsclient.caching.*;
import org.netbeans.modules.cvsclient.commands.*;
import org.netbeans.modules.cvsclient.commands.update.*;


import org.netbeans.lib.cvsclient.command.DefaultFileInfoContainer;
import org.netbeans.lib.cvsclient.command.update.*;
import org.netbeans.lib.cvsclient.command.checkout.*;
import org.netbeans.lib.cvsclient.command.*;


import org.netbeans.modules.cvsclient.util.Debug;

/**
 *
 * @author  mkleint
 * @version
 */
public class CvsCheckout extends CacheUpdatingFsCommand  {
    private Debug E=new Debug("CvsCheckout", true); // NOI18N
    private Debug D=E;
    
    CheckoutCommand command;
    boolean finishedOk;
    
    boolean isListOnly = false;
    protected ArrayList modules;
    protected String localPath;
    
    /** Creates new Status */
    public CvsCheckout(ClientProvider fs) {
        super(fs);
        if (fs.isFileSystemClient()) {
            JavaCvsFileSystem fileSys = (JavaCvsFileSystem)fs;
            setFileSystem(fileSys);
        }        
        D.deb("Constructor");
        modules = new ArrayList();
        localPath = null;
    }
    
    public String getName() {
        return NbBundle.getBundle(CvsCheckout.class).getString("CvsCheckout.name");
    }

/** if no param input was set before staring the command, 
 * the default defined in this method is used
 */
    protected CommandParamInput createDefaultParamInput() {
        return new CheckoutParamInput(getClientProvider(), getModulesString());
    }
    
        
    protected Class getDefaultDisplayerType() {
        if (command != null && command.isPipeToOutput()) {
            return UpdateStdOutPanel.class;
        }
        if (command != null && (command.isShowModules() || command.isShowModulesWithStatus())) {
          return ModulesListPanel.class;
        }
        return UpdateInfoPanel.class;
    }
    
    /** this methods sets the localpath for the client needed for checkout (preferably the filesystem's working directory)
     * since we need to keep the directory structure in use and not checkout in a subtree of already checked out stuff.
     */
    public void setLocalPath(String path) {
        localPath = path;
        File[] files = new File[1];
        files[0] = new File(path);
        setFiles(files);
    }
    
    protected void initCommand() {
        // here the the commands is initiated just right before running the command
        command = (CheckoutCommand)loadCommand(CheckoutCommand.class);
        if (localPath != null) {
           libClient.setLocalPath(localPath); // needed because of the nature of checkout
        }
        //TODO setModules
        String[] mods = new String[modules.size()];
        mods = (String[])modules.toArray(mods);
        command.setModules(mods);
        toDoCommands.addElement(command);
        super.initCommand();
    }
    
    
    protected void applyInputParams() {
        super.applyInputParams();
        CheckoutCommand comm = (CheckoutCommand)toDoCommands.get(0);
        if (comm.isPipeToOutput() &&
            getClientProvider().getDisplayType() == JavaCvsFileSystem.DISP_TYPE_FORMATTED) {
            String message = NbBundle.getBundle(CvsCheckout.class).getString("CvsCheckout.pipeOutputQuestion");
            NotifyDescriptor nd = new NotifyDescriptor.Confirmation(message, NotifyDescriptor.YES_NO_OPTION);
            Object returnValue = TopManager.getDefault().notify(nd);
            //        System.out.println("return value=" + returnValue.getClass().getName() + " " + returnValue.toString());
            if (NotifyDescriptor.Confirmation.NO_OPTION.equals(returnValue)) {
                hardCommandStop();
            }
        }                
        if (getDisplayerType() == UpdateInfoPanel.class) {
           // just one panel is generated for the whole command.. you can then add info while it's being generated
           setOutputDisplayer(new UpdateInfoPanel(getClientProvider(), this));
           getDisplayer().displayOutputData(CommandDisplayer.MOMENT_AFTER_PARAM_INPUT, null);
        }
    }    
    
    protected void beforeEachExecute() {
        if (getDisplayerType() == ModulesListPanel.class) {
            setOutputDisplayer(new ModulesListPanel());
        }
        super.beforeEachExecute();
    }
    
    protected void finishedCommand() {
        //fire cache update TODO
        fireUpdateCache();
        super.finishedCommand();
    }
    
    
    protected void executeFailed(Exception exc) {
        // fire cache update TODO
        fireUpdateCache();
        super.executeFailed(exc);
    }
    
    
    /**
     * Called when file status information has been received
     */
    public void fileInfoGenerated(FileInfoEvent e) {
        D.deb("fileUpdated()");
        FileInfoContainer fInfo = e.getInfoContainer();
        if (fInfo != null) {
            if (fInfo instanceof DefaultFileInfoContainer) {
                DefaultFileInfoContainer info = (DefaultFileInfoContainer)fInfo;
                updateCache(info);
            }
            else if (fInfo instanceof PipedFileInformation) {
                if (getDisplayerType() == UpdateStdOutPanel.class) {
                    CheckoutCommand statCom = (CheckoutCommand)getCurrentCommand();
                    CommandDisplayer toReturn;
                    PipedFileInformation info = (PipedFileInformation)fInfo;
                    File fl = info.getFile();
               // find or create statusInfoPanel
                    UpdateStdOutPanel panel = (UpdateStdOutPanel)FileSystemCommand.findOpenDisplayer(
                                                    fl, UpdateStdOutPanel.class, info);
                    if (panel == null) {
                        panel = new UpdateStdOutPanel(statCom);
                    } else {
                        panel.setCommand(statCom);
                    }
                    panel.setFileSystemCommand(this);                    
                    setOutputDisplayer(panel);
                }
            }
        }
        super.fileInfoGenerated(e);
    }
    
    public void addModule(String module) {
        modules.add(module);
    }
    
    public String getModulesString() {
        Iterator it = modules.iterator();
        StringBuffer buf = new StringBuffer(" ");
        while (it.hasNext()) {
            String mod = (String)it.next();
            buf.append(mod);
            buf.append(" ");
        }
        return buf.toString();
    }
    
    /** if no displayer was set by the setOutputDisplayer() method, 
     * a default one will be generated by the subclasses implementation
     *
     */
/*    protected CommandDisplayer createDefaultDisplayer() {
       if (command.isShowModules() || command.isShowModulesWithStatus()) {
          return new ModulesListPanel();
       }
       if (command.isPipeToOutput()) {
          return new UpdateStdOutPanel(command);
       }
       // just one panel is generated for the whole command.. you can then add info while it's being generated
       return new UpdateInfoPanel(getClientProvider(), this);
    }
  */  
}
