/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.cvsclient.commands.checkout;

import java.util.*;
import org.netbeans.modules.cvsclient.commands.ClientProvider;
import org.netbeans.modules.cvsclient.commands.BlankParamInput;
import org.netbeans.modules.cvsclient.commands.CommandDisplayer;
import org.netbeans.modules.cvsclient.commands.FileSystemCommand;
import org.netbeans.modules.cvsclient.commands.CommandParamInput;
import org.netbeans.modules.cvsclient.JavaCvsFileSystem;

import org.netbeans.lib.cvsclient.command.checkout.*;
import org.netbeans.lib.cvsclient.command.*;
import org.netbeans.lib.cvsclient.event.FileInfoEvent;

import org.netbeans.modules.vcscore.util.Debug;
import org.openide.util.NbBundle;


/**
 *
 * @author  mkleint
 * @version
 */
public class CvsCompleteModList extends FileSystemCommand {
    private Debug E=new Debug("CvsCompleteModList", true); // NOI18N
    private Debug D=E;
    
    CheckoutCommand command;    
    private CheckoutCommand firstCommand;
    /** Creates new CvsCompleteModList */
    public CvsCompleteModList(ClientProvider fs) {
        super(fs);
        if (fs.isFileSystemClient()) {
            JavaCvsFileSystem fileSys = (JavaCvsFileSystem)fs;
            setFileSystem(fileSys);
        }        
        D.deb("Constructor");
        setLoadDefaultSwitches(false);
        setFixedDisplayType(true);
        setParamInput(new BlankParamInput());
    }

/** if no param input was set before staring the command, 
 * the default defined in this method is used
 */
    protected CommandParamInput createDefaultParamInput() {
        return new BlankParamInput();
    }
    
    protected void applyInputParams() {
        if (getDisplayerType() == ModulesListPanel.class) {
            setOutputDisplayer(new ModulesListPanel());
        }
        super.applyInputParams();
    }
    
    protected Class getDefaultDisplayerType() {
        return ModulesListPanel.class;
    }
    
    /**
     * Called when file status information has been received
     */
    public void fileInfoGenerated(FileInfoEvent e) {
        if (getCurrentCommand() == firstCommand) {
            super.fileInfoGenerated(e);
        } else {
            // now for checkout -s just search the received data and add the status to the right one.
           ModuleListInformation info = (ModuleListInformation)e.getInfoContainer();
           Iterator it = getResultList().iterator();
           String modName = info.getModuleName();
           while (it.hasNext()) {
               ModuleListInformation listInfo = (ModuleListInformation)it.next();
               if (modName.equals(listInfo.getModuleName())) {
                   listInfo.setModuleStatus(info.getModuleStatus());
                   getDisplayer().displayOutputData(CommandDisplayer.MOMENT_FILE_INFO_GENERATED, listInfo);
                   return;
               }
           }
           // just for sanity's sake.. should in 99.99% cases not happen.
           super.fileInfoGenerated(e);
        }
    }
    
    protected void executeFailed(Exception exc) {
        super.executeFailed(exc);
        stopCommand();
    }
    
    
    protected void initCommand() {
        // here the the commands is initiated just right before running the command
        D.deb("initCommand()");
        // first command - checkout -c
        // it doesn't care about defaults, since we got a fixed set of commands 
        // with fixed switches.
        command = (CheckoutCommand)loadCommand(CheckoutCommand.class);
        firstCommand = command; // to recognize which command the event came from
        command.setShowModules(true);
        toDoCommands.addElement(command);
        // next command -checkout -s
        command = (CheckoutCommand)loadCommand(CheckoutCommand.class);
        command.setShowModulesWithStatus(true);
        toDoCommands.addElement(command);
        super.initCommand();
    }
    
    public String getName() {
        return NbBundle.getBundle(CvsCompleteModList.class).getString("CvsCompleteModList.name");
    }

    /** if no displayer was set by the setOutputDisplayer() method, 
     * a default one will be generated by the subclasses implementation
     *
     */
/*    protected CommandDisplayer createDefaultDisplayer() {
          return new ModulesListPanel();
    }
 */

}
