/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient.commands.checkout;

/**
 *
 * @author  mkleint
 * @version 
 */

import org.netbeans.modules.cvsclient.commands.*;
import org.netbeans.lib.cvsclient.command.checkout.CheckoutCommand;
import java.util.*;
import org.openide.*;
import org.openide.util.NbBundle;
import java.awt.Dialog;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.*;
import javax.swing.table.*;


class ModuleChooser extends ModulesListPanel implements CommandChooser, CommandErrorListener {

      private ArrayList listeners;
      private FileSystemCommand comm;
      private String returnValue;
      private DialogDescriptor dd;
      private Dialog dial;
      private boolean finished;
      private JButton btnOk;
      
      public ModuleChooser(CvsCompleteModList command) {
          super();
          finished = false;
          comm = command;
      }

      public ModuleChooser(CvsCheckout command) {
          super();
          finished = false;
          comm = command;
      }
      
      
     /** in this method the displayer should use the data returned by the command to
     * produce it's own data structures/ fill in UI components
     * @param resultList - the data from the command. It is assumed the Displayer 
     *  knows what command the data comes from and most important in what format. 
     * (which FileInfoContainer class is used).
     */
    public void setDataToDisplay(Collection resultList) {
        super.setDataToDisplay(resultList);
    }
    
   /** this method should be used with Choosers to display the dialog.. 
 * then later the displayOutputData will just present the datat gotten from server
 * Reason: to have something displayer right away..
 * This method is to be called when creating the command and Chooser.
 */
    public void displayFrameWork() {
        if (dial != null) {
            dial.show();
            return;
        }
        String title = NbBundle.getBundle(ModuleChooser.class).getString("ModuleChooser.title");
        JPanel panel = new JPanel();
        String warning = NbBundle.getBundle(ModuleChooser.class).getString("ModuleChooser.loading");
        panel.add(new JLabel(warning));
        dd = new DialogDescriptor(panel, title);
        Object[] option = new Object[1];
        option[0] = DialogDescriptor.CANCEL_OPTION;
        dd.setOptions(option);
        dd.setClosingOptions(option);
        dd.setButtonListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                buttonWasPressed(evt);
            }
        });
        javax.swing.SwingUtilities.invokeLater(new Runnable () {
            public void run () {
               dial = TopManager.getDefault().createDialog(dd);
               dial.setModal(false);
               dial.show();
            }
        });        
/*        Thread thr = new Thread() {
            public void run() {
            }
        };
        thr.start();
 */
    }
    
    /** displays the info retrieved from server
     * 
     */
    public void displayOutputData() {
        Object[] option = new Object[2];
        
        btnOk = new JButton(NbBundle.getBundle(ModuleChooser.class)
                          .getString("ModuleChooser.okButton"));
        btnOk.setDefaultCapable(true);
        dd.setValue(btnOk);
        option[0] = btnOk;
        option[1] = DialogDescriptor.CANCEL_OPTION;
        dd.setOptions(option);
        dd.setClosingOptions(option);
        dd.setMessage(this);
        finished = true;
    }
    
    public void addChooserFinishListener(ChooserFinishListener chooserListener) {
        if (listeners == null) {
            listeners = new ArrayList ();
        }
        listeners.add (chooserListener);
    }    
    
    /** used for checking wheather the command finished successfully and can be redisplayed without running the command again
     */
    public boolean wasFinished() {
        return finished;
    }
    
    public boolean isShowingDial() {
        if (dial != null && dial.isVisible()) return true;
        return false;
    }

    public void removeChooserFinishListener(ChooserFinishListener chooserListener) {
        if (listeners != null) {
            listeners.remove (chooserListener);
        }            
    }
    
    private void fireChooserFinishListener() {
        ArrayList list;
        synchronized (this) {
            list = (ArrayList) this.listeners.clone();
        }
        ChooserFinishEvent event = new ChooserFinishEvent(this);
        
        for (int i=0; i<list.size(); i++) {
          ((ChooserFinishListener)list.get (i)).chooserFinished (event);
        }
    }
    
    /** will get rid of the dialog from outside.. to be used when errors occur
     */
    public void disposeChooser() {
        returnValue = null;
        fireChooserFinishListener();
        listeners.clear();
        if (dial.isVisible()) {
          dial.setVisible(false);
          dial.dispose();
        }
    }
    
    private void buttonWasPressed(ActionEvent evt) {
        if (evt.getSource() == btnOk) {
          int[] selected = tblModules.getSelectedRows();
          StringBuffer toReturn = new StringBuffer("");
          for (int index = 0; index < selected.length; index++) {
            toReturn.append(tblModules.getModel().getValueAt(selected[index], 0));
            toReturn.append(" ");
          }
          returnValue =   toReturn.toString();
        }
        if (evt.getSource() == NotifyDescriptor.CANCEL_OPTION) {
          if (comm.isRunning()) {
             comm.hardCommandStop();
             returnValue = null;
          }
        }
        fireChooserFinishListener();
    }
    
    /** if the user selects OK, this method returns a space-separated String (list) of modules chosen
     * 
     */
    public Object getValue() {
        return returnValue;
    }
    
    /** from CommandErrorListener */
    
    public void errorGenerated(CommandErrorEvent event) {
        if (event.endedWithError()) disposeChooser();
    }    
    
    public void displayOutputData(int moment,Object data) {
       if (moment == CommandDisplayer.MOMENT_FINISHED_COMMAND) 
       {
             List lst = (List)data;
             setDataToDisplay(lst);
             displayOutputData();
       }        
       if (moment == CommandDisplayer.MOMENT_AFTER_PARAM_INPUT) {
             displayFrameWork();
       }
    }
    
}
