/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient.commands.commit;

import org.netbeans.modules.cvsclient.commands.*;
import java.util.*;
import javax.swing.*;
import org.netbeans.lib.cvsclient.command.DefaultFileInfoContainer;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.Dialog;
import java.awt.BorderLayout;
import org.netbeans.modules.cvsclient.commands.update.CvsUpdate;

import org.openide.util.NbBundle;
import org.openide.*;
/**
 *
 * @author  mkleint
 * @version 
 */
public class CommitMessageChooser extends AbstractTreeInfoPanel 
                                  implements CommandChooser, CommandErrorListener {

    private ArrayList listeners;
    boolean finished;
    private JPanel messagePanel;
    private JTextArea messageText; // gets set up during the createMessageArea method
    private JTextArea generalText;
    private JPanel clearPanel;
    private JPanel loadingPanel;
    private JPanel mainPanel;
    private DefaultFileInfoContainer currentInfo;
    private HashMap messageMap;
    private DialogDescriptor dd;
    private Dialog dial;
    private CvsUpdate comm;
    private boolean forceCommit = false;
    private Collection dataCollection;
    
    /** Creates new CommitMessageChooser */
    public CommitMessageChooser(CvsUpdate usedCommand) {
        super();
        disableTree();
        messagePanel = createMessageArea();
        clearPanel = createClearPanel();
        loadingPanel = createLoadingPanel();
        messageMap = new HashMap();
        comm = usedCommand; 
        postInit();
    }
    
    public void setForceCommit(boolean force) {
        forceCommit = force;
    }

    public boolean isForceCommit() {
        return forceCommit;
    }
    
    public void setGeneralMessage(String message) {
        generalText.setText(message);
    }

    /** this method should be used with Choosers to display the dialog.. 
     * then later the displayOutputData will just present the datat gotten from server
     * Reason: to have something displayer right away..
     * This method is to be called when creating the command and Chooser.
     */
    public void displayFrameWork() {
        setDataToDisplay(new LinkedList());
        String title = NbBundle.getBundle(CommitMessageChooser.class).getString("CommitMessageChooser.title");
        dd = new DialogDescriptor(this, title);
        Object[] option = new Object[1];
        option[0] = DialogDescriptor.CANCEL_OPTION;
        dd.setOptions(option);
        dd.setClosingOptions(option);
        dd.setButtonListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                buttonWasPressed(evt);
            }
        });
        javax.swing.SwingUtilities.invokeLater(new Runnable () {
            public void run () {
               dial = TopManager.getDefault().createDialog(dd);
               dial.setModal(false);
               dial.show();
            }
        });        
    }

    
    public void displayOutputData() {
        LinkedList copy = new LinkedList(dataCollection);
        setDataToDisplay(copy);
        Object[] option = new Object[2];
        setClearPanel();
        option[0] = DialogDescriptor.OK_OPTION;
        option[1] = DialogDescriptor.CANCEL_OPTION;
        dd.setOptions(option);
        dd.setClosingOptions(option);
        dd.setMessage(this);
        finished = true;
    }
    
    public void redisplay() {
/*        LinkedList copy = new LinkedList(dataCollection);
        String title = NbBundle.getBundle(CommitMessageChooser.class).getString("CommitMessageChooser.title");
        dd = new DialogDescriptor(this, title);
        dd.setButtonListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                buttonWasPressed(evt);
            }
        });
        setDataToDisplay(copy);
        Object[] option = new Object[2];
        setClearPanel();
        option[0] = DialogDescriptor.OK_OPTION;
        option[1] = DialogDescriptor.CANCEL_OPTION;
        dd.setOptions(option);
        dd.setClosingOptions(option);
        dd.setMessage(this);
 */
        finished = true;
        javax.swing.SwingUtilities.invokeLater(new Runnable () {
            public void run () {
//               dial = TopManager.getDefault().createDialog(dd);
               dial.setModal(false);
               dial.show();
            }
        });        
        
    }
    
    public void addChooserFinishListener(ChooserFinishListener chooserListener) {    
        if (listeners == null) {
            listeners = new ArrayList ();
        }
        listeners.add (chooserListener);
    }    
    
    /** used for checking wheather the command finished successfully and can be redisplayed without running the command again
     */
    public boolean wasFinished() {
        return finished;
    }

    public void removeChooserFinishListener(ChooserFinishListener chooserListener) {
        if (listeners != null) {
            listeners.remove (chooserListener);
        }            
    }
    
    private void fireChooserFinishListener() {
        ArrayList list;
        synchronized (this) {
            list = (ArrayList) this.listeners.clone();
        }
        ChooserFinishEvent event = new ChooserFinishEvent(this);
        
        for (int i=0; i<list.size(); i++) {
          ((ChooserFinishListener)list.get (i)).chooserFinished (event);
        }
    }
    
    /** will get rid of the dialog from outside.. to be used when errors occur
     */
    public void disposeChooser() {
        messageMap.clear();
        fireChooserFinishListener();
        listeners.clear();
        if (dial.isVisible()) {
          dial.setVisible(false);
          dial.dispose();
        }
    }
    
    private void buttonWasPressed(ActionEvent evt) {
        if (evt.getSource() == DialogDescriptor.OK_OPTION) {
            if (currentInfo != null) {
                saveOldFileMessage();
            }
        }
        if (evt.getSource() == NotifyDescriptor.CANCEL_OPTION) {
          if (comm.isRunning()) {
             comm.hardCommandStop();
             messageMap.clear();
          }
        }
        fireChooserFinishListener();
    }
    
    /** if the user selects OK, this method returns a space-separated String (list) of modules chosen
     * 
     */
    public Object getValue() {
        return messageMap;
    }
    
    /** from CommandErrorListener */
    
    public void errorGenerated(CommandErrorEvent event) {
        if (event.endedWithError()) disposeChooser();
    }  
    
    private JPanel createMessageArea() {
        JPanel toReturn = new JPanel();
        toReturn.setPreferredSize(new java.awt.Dimension(350, 150));
        toReturn.setMinimumSize(new java.awt.Dimension(150, 50));
        
        // ------------ beg. gen. area
        JScrollPane spMessage = new javax.swing.JScrollPane();
        JTextArea taMessage = new javax.swing.JTextArea();
        JLabel lblMessage = new javax.swing.JLabel();
        JLabel lblGeneral = new javax.swing.JLabel();
        JScrollPane spGeneral = new javax.swing.JScrollPane();
        JTextArea taGeneral = new javax.swing.JTextArea();
        toReturn.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        spMessage.setPreferredSize(new java.awt.Dimension(200, 100));
        spMessage.setViewportView(taMessage);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.weighty = 0.8;
        gridBagConstraints1.weightx = 0.8;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(6, 6, 0, 6);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        toReturn.add(spMessage, gridBagConstraints1);
        
        lblMessage.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/commands/commit/Bundle").getString("CommitMessageChooser.lblMessage.text"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.weighty = 0.8;
        gridBagConstraints1.weightx = 0.8;
        gridBagConstraints1.insets = new java.awt.Insets(6, 6, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        toReturn.add(lblMessage, gridBagConstraints1);
        
        lblGeneral.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/commands/commit/Bundle").getString("CommitMessageChooser.lblGeneral.text"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.weighty = 0.8;
        gridBagConstraints1.weightx = 0.8;
        gridBagConstraints1.insets = new java.awt.Insets(6, 6, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        toReturn.add(lblGeneral, gridBagConstraints1);
        
        taGeneral.setEditable(false);
        taGeneral.setBackground(java.awt.Color.lightGray);
        spGeneral.setViewportView(taGeneral);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.weighty = 0.8;
        gridBagConstraints1.weightx = 0.8;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(6, 6, 0, 6);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        toReturn.add(spGeneral, gridBagConstraints1);
        //--- end gen. area
        taMessage.setLineWrap(false);
        spGeneral.setPreferredSize(new java.awt.Dimension(200, 100));
        messageText = taMessage; // hack to be able to read/write to text area
        generalText = taGeneral;
        return toReturn;
    }    
    
    private JPanel createClearPanel() {
        JPanel toReturn = new JPanel();
        toReturn.setPreferredSize(new java.awt.Dimension(350, 150));
        toReturn.setMinimumSize(new java.awt.Dimension(150, 50));
        // ------------ beg. gen. area
        JLabel lblMessage = new javax.swing.JLabel();
        toReturn.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;

        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        
        lblMessage.setText(java.util.ResourceBundle.getBundle(
           "org/netbeans/modules/cvsclient/commands/commit/Bundle").getString(
           "CommitMessageChooser.clearPanel.text"));
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        toReturn.add(lblMessage, gridBagConstraints1);        
        //--- end gen. area
        return toReturn;
    }    

    private JPanel createLoadingPanel() {
        JPanel toReturn = new JPanel();
        toReturn.setPreferredSize(new java.awt.Dimension(350, 150));
        toReturn.setMinimumSize(new java.awt.Dimension(150, 50));
        // ------------ beg. gen. area
        JLabel lblMessage = new javax.swing.JLabel();
        toReturn.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;

        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        lblMessage.setText(java.util.ResourceBundle.getBundle(
           "org/netbeans/modules/cvsclient/commands/commit/Bundle").getString(
           "CommitMessageChooser.loadingPanel.text"));
        toReturn.add(lblMessage, gridBagConstraints1);        

        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        JLabel lblMessage2 = new javax.swing.JLabel();
        lblMessage2.setText(java.util.ResourceBundle.getBundle(
           "org/netbeans/modules/cvsclient/commands/commit/Bundle").getString(
           "CommitMessageChooser.loadingPanel.text2"));
        toReturn.add(lblMessage, gridBagConstraints1);        
        //--- end gen. area
        return toReturn;
    }    
    
    protected void setPanel(Object infoData) {
        if (currentInfo == null) {
            // was clear before this one
            mainPanel.removeAll();
            mainPanel.add(messagePanel, BorderLayout.CENTER);
            repaint();
            revalidate();
        }    else {
            saveOldFileMessage();
        }
        currentInfo = (DefaultFileInfoContainer) infoData;
        loadFileMessage();
    }
    
    protected void setClearPanel() {
        if (currentInfo != null) {
            saveOldFileMessage();
        }
        mainPanel.removeAll();
        mainPanel.add(clearPanel, BorderLayout.CENTER);
        revalidate();
        repaint();
        currentInfo = null;
    }
    
    protected JComponent initPanel() {
        mainPanel = new JPanel();
        mainPanel.setLayout(new BorderLayout());
        mainPanel.add(loadingPanel, BorderLayout.CENTER);
        mainPanel.revalidate();
        mainPanel.repaint();
        return mainPanel;
    }
    
    private void saveOldFileMessage() {
        String curr = messageText.getText().trim();
        if (!curr.equals("")) {
            messageMap.put(currentInfo, curr);
        }
    } 
    
    private void loadFileMessage() {
        String mess = (String)messageMap.get(currentInfo);
        if (mess == null) mess = "";
        messageText.setText(mess);
    }
    
    /** in this method the displayer should use the data returned by the command to
     * produce it's own data structures/ fill in UI components
     * @param resultList - the data from the command. It is assumed the Displayer 
     * knows what command the data comes from and most important in what format. 
     * (which FileInfoContainer class is used).
 */
    public void setDataToDisplay(Collection resultList) {
        Iterator it = resultList.iterator();
        while (it.hasNext()) {
            DefaultFileInfoContainer info = (DefaultFileInfoContainer)it.next();
            //TODO.. if some are Patch or MErge.. throw error.
            if (info.getType().equals("U") && !isForceCommit()) {
                it.remove();
            }
        }
        super.setDataToDisplay(resultList);
    }
    
    private void setData(Collection col) {
        dataCollection = col;
    }
    
    /** Does the actual display - docking into the javacvs Mode, 
     *  displaying as single Dialog.. whatever.
 */
    public void displayOutputData(int moment,Object data) {
        if (moment == CommandDisplayer.MOMENT_AFTER_PARAM_INPUT) {
            displayFrameWork();
        }
        if (moment == CommandDisplayer.MOMENT_FINISHED_COMMAND) {
            Collection col = (Collection)data;
            setData(col);
            displayOutputData();
        }
    }
    
}
