/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */



package org.netbeans.modules.cvsclient.commands.commit;


import org.openide.util.*;
import java.io.*;
import java.util.*;
import java.awt.Dialog;

import org.netbeans.modules.cvsclient.*;
import org.netbeans.modules.cvsclient.commands.*;
import org.netbeans.lib.cvsclient.event.*;
import org.netbeans.lib.cvsclient.command.*;
import org.netbeans.modules.cvsclient.caching.*;
import org.netbeans.lib.cvsclient.command.commit.*;

import org.netbeans.modules.vcscore.util.Debug;

/**
 *
 * @author  mkleint
 * @version
 */
public class CvsCommit extends CacheUpdatingFsCommand {
    private Debug E=new Debug("CvsCommit", true); // NOI18N
    private Debug D=E;
    
    CommitCommand command;
    
    /** Creates new Status */
    public CvsCommit(File[] filesToCommit, ClientProvider fs) {
        super(fs);
        if (fs.isFileSystemClient()) {
            JavaCvsFileSystem fileSys = (JavaCvsFileSystem)fs;
            setFileSystem(fileSys);
        }        
        setFiles(filesToCommit);
        D.deb("Constructor");
    }
    
    public String getName() {
        return NbBundle.getBundle(CvsCommit.class).getString("CvsCommit.name");
    }

/** if no param input was set before staring the command, 
 * the default defined in this method is used
 */
    protected CommandParamInput createDefaultParamInput() {
        return new CommitParamInput(getClientProvider(), this);
    }
/*    
    protected void applyInputParams() {
        Iterator it = toDoCommands.iterator();
        // for all commands queued in the toDoCommand list,
        // apply the command line params to them (from the dialog)
        while (it.hasNext()) {
            CommitCommand sCom = (CommitCommand)it.next();
            getParamInput().setData(sCom);
//            D.deb("applyInputParams() pipeToOutput=" + sCom.isPipeToOutput());
        }
    }
  */  
    protected void initCommand() {
        // here the the commands is initiated just right before running the command
        D.deb("initCommand()");
        int dirNum = 0;
        command = (CommitCommand) loadCommand(CommitCommand.class);
        
        command.setFiles(getFiles());
        getParamInput().getData(command);
        toDoCommands.addElement(command);
        super.initCommand();
    }
    
    protected void applyInputParams() {
        if (getDisplayerType() == CommitInfoPanel.class) {
           // just one panel is generated for the whole command.. you can then add info while it's being generated
           setOutputDisplayer(new CommitInfoPanel(getClientProvider(), this));
        }
        super.applyInputParams();
    }
    
    protected void finishedCommand() {
        fireUpdateCache();
        super.finishedCommand();
    }
    
    protected void executeFailed(Exception exc) {
        fireUpdateCache();
        super.executeFailed(exc);
    }
    
    
    /**
     * Called when file status information has been received
     */
    public void fileInfoGenerated(FileInfoEvent e) {
        D.deb("fileUpdated()");
        FileInfoContainer fInfo = e.getInfoContainer();
        if (fInfo != null) {
            if (fInfo instanceof CommitInformation) {
                CommitInformation info = (CommitInformation)fInfo;
                updateCache(info);
            }
        }
        super.fileInfoGenerated(e);
    }
    
    /** if no displayer was set by the setOutputDisplayer() method, 
 * a default one will be generated by the subclasses implementation
 *
 */
    
    protected Class getDefaultDisplayerType() {
        return CommitInfoPanel.class;
    }
    
    
    /**
     * Thi smethod performs the tranformation from 1 commit command for all files/dirs to 
     * separate commits for files definied in the map. The keys value of  the map 
     * is UpdateInformation object, value member of the map being a String.
     * The original commit command is moved to the end of the list. 
     * All this is done in order to enable log messages for single files. 
     */
    public void transform(HashMap map) {
        int size = toDoCommands.size();
        CommitCommand original = (CommitCommand)toDoCommands.get(size - 1);
        toDoCommands.clear();
        Iterator it = map.keySet().iterator();
        while (it.hasNext()) {
            DefaultFileInfoContainer info = (DefaultFileInfoContainer)it.next();
            try {
                CommitCommand commit = (CommitCommand)original.clone();
                File[] commFiles = new File[1];
                commFiles[0] = info.getFile();
                commit.setFiles(commFiles);
                toDoCommands.add(commit);
            } catch (CloneNotSupportedException exc) {
                //TODO??
            }
        }
        toDoCommands.add(original);
    }
    

    
}
