/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient.commands.diff;


import org.openide.util.*;
import java.io.*;
import java.util.*;
import java.awt.Dialog;

import org.netbeans.modules.cvsclient.*;
import org.openide.filesystems.*;
import org.netbeans.lib.cvsclient.event.*;
import org.netbeans.lib.cvsclient.command.diff.DiffInformation;
import org.netbeans.lib.cvsclient.command.checkout.PipedFileInformation;
import org.netbeans.lib.cvsclient.command.update.UpdateCommand;
import org.netbeans.lib.cvsclient.command.diff.DiffCommand;
import org.netbeans.lib.cvsclient.command.Command;
import org.netbeans.lib.cvsclient.command.CommandException;
import org.netbeans.modules.cvsclient.commands.*;
import org.netbeans.modules.vcscore.diff.*;
import org.netbeans.modules.vcscore.util.Debug;

/**
 *
 * @author  mkleint
 * @version
 */
public class CvsDiff extends FileSystemCommand {
    private Debug E=new Debug("CvsDiff", true); // NOI18N
    private Debug D=E;
    
    private File startFile;
    private StringBuffer bufLeft;
    private StringBuffer bufRight;
    private int fileLineCount;
    private int leftLineCount;
    private int rightLineCount;
    private LinkedList updInfoList;
    
    /** Creates new Status */
    public CvsDiff(File[] filesToCheck, ClientProvider fs) {
        super(fs);
        if (fs.isFileSystemClient()) {
            JavaCvsFileSystem fileSys = (JavaCvsFileSystem)fs;
            setFileSystem(fileSys);
        }
        setFiles(filesToCheck);
        D.deb("Constructor");
        //        setParamInput(new DiffParamInput());
    }
    
    public String getName() {
        return NbBundle.getBundle(CvsDiff.class).getString("CvsDiff.name");
    }

/** if no param input was set before staring the command, 
 * the default defined in this method is used
 */
    protected CommandParamInput createDefaultParamInput() {
        return new DiffParamInput(); 
    }    
    
    protected void applyInputParams() {
        Iterator it = toDoCommands.iterator();
        // for all commands queued in the toDoCommand list,
        // apply the command line params to them (from the dialog)
        LinkedList checkoutCommands = new LinkedList();
        while (it.hasNext()) {
            DiffCommand sCom = (DiffCommand)it.next();
            getParamInput().setData(sCom);
            if (getClientProvider().getDisplayType() != JavaCvsFileSystem.DISP_TYPE_SIMPLE 
                 || isFixedDisplayType()) {
                checkNeedForCheckout(sCom, checkoutCommands);
            }
        }
        addCheckoutCommands(checkoutCommands);
    }
    
    protected void initCommand() {
        // here the the commands is initiated just right before running the command
        D.deb("initCommand()");
        int dirNum = 0;
        DiffCommand command;
        updInfoList = new LinkedList();
        File[] files = new File[getFiles().length];
        System.arraycopy(getFiles(),0,files,0, getFiles().length);
        
        for (int index = 0; index < files.length; index++) {
            if (files[index].isDirectory()) {
                //that way just one UI component is run after finish of the command
                command = (DiffCommand)loadCommand(DiffCommand.class);
                File[] fls = new File[1];
                fls[0] = files[index];
                command.setFiles(fls);
                toDoCommands.addElement(command);
                files[index] = null;
                dirNum = dirNum + 1;
            }
        }
        // now proceed all single files in one batch.
        if (dirNum == files.length) {
            return; // no files to check status on.
        }
        File[] singleFiles = new File[files.length - dirNum];
        int singleIndex = 0;
        for (int ind = 0; ind < files.length; ind++) {
            if (files[ind] != null) {
                singleFiles[singleIndex] = files[ind];
                singleIndex = singleIndex + 1;
            }
        }
        command = (DiffCommand)loadCommand(DiffCommand.class);
//        command.setDirectoryCheck(false);
        command.setFiles(singleFiles);
        toDoCommands.addElement(command);
        // just add oen command to the queue
        super.initCommand();
    }
    
    
    protected void beforeEachExecute() {
        clearResultList();
       if (getDisplayerType() == DiffTreeInfoPanel.class) {
            DiffCommand statCom = (DiffCommand)getCurrentCommand();
            File[] flz = statCom.getFiles();
            if (flz[0].isDirectory()) {
                File[] topDir = statCom.getFiles();
                DiffTreeInfoPanel treePanel = new DiffTreeInfoPanel(topDir[0], statCom);
                treePanel.setFileSystemCommand(this);                
                setOutputDisplayer(treePanel);
            }
       }
        super.beforeEachExecute();
    }
    
    protected void afterEachExecute() {
        D.deb("AfterEachExecute()");
        if (getCurrentCommand() instanceof UpdateCommand) {
            return;
        }
 
        Iterator it = updInfoList.iterator();
        while (it.hasNext()) {
            getResultList().add(it.next());
        }
        super.afterEachExecute();
        if (getDisplayerType() == DiffTreeInfoPanel.class ||
            getDisplayerType() == DiffDisplayer.class) {
            setOutputDisplayer(null);
        }
    }
    
    protected void executeFailed(Exception exc) {
        stopCommand();
        super.executeFailed(exc);
    }
    
    
    /**
     * Called when file status information has been received
     */
    public void fileInfoGenerated(FileInfoEvent e) {
        D.deb("fileinfo generated()");
        if (e.getInfoContainer() instanceof PipedFileInformation) {
            PipedFileInformation updInfo = (PipedFileInformation)e.getInfoContainer();
            updInfoList.add(updInfo);
            return;
        }
        if (getDisplayerType() == DiffDisplayer.class) {
           DiffCommand comm = (DiffCommand)getCurrentCommand();
           DiffDisplayer  panel = new DiffDisplayer(comm);
           panel.setFileSystemCommand(this);           
           setOutputDisplayer(panel);
        }
        // super is called only for the DiffCommands.
        super.fileInfoGenerated(e);
    }
    
    protected Class getDefaultDisplayerType() {
        if (getCurrentCommand() instanceof DiffCommand) {
           DiffCommand statCom = (DiffCommand)getCurrentCommand();
           File[] flz = statCom.getFiles();
           if (flz[0].isDirectory()) return DiffTreeInfoPanel.class;
           return DiffDisplayer.class;
        }
        return NullDisplayer.class;
    }
    /** if no displayer was set by the setOutputDisplayer() method, 
 * a default one will be generated by the subclasses implementation
 *
 */
/**    protected CommandDisplayer createDefaultDisplayer() {
        CommandDisplayer toReturn;
        if (getCurrentCommand() instanceof DiffCommand) {
            DiffCommand comm = (DiffCommand)getCurrentCommand();
            File[] flz = comm.getFiles();
            if (flz[0].isDirectory()) {
                File[] topDir = comm.getFiles();
                toReturn = new DiffTreeInfoPanel(topDir[0], comm, getClientProvider());
            } else {
                toReturn = new DiffDisplayer(getClientProvider(), comm);
            }
        } else {
            toReturn = new NullDisplayer();
        }
        return toReturn;
       
    }  
   */  

    private void checkNeedForCheckout(DiffCommand comm, LinkedList checkoutList) { 
  // Do always.. just to be sure we got the base file to start from..
  // therwise for Locally removed files we would have to make complicated lookup.. for example.
//        if (comm.getRevision2() != null || comm.getBeforeDate2() != null) {
            boolean old = isLoadDefaultSwitches();
            setLoadDefaultSwitches(false);
            UpdateCommand checkComm = (UpdateCommand)loadCommand(UpdateCommand.class);
            setLoadDefaultSwitches(old);
            checkComm.setFiles(comm.getFiles());
            checkComm.setPipeToOutput(true);
            checkComm.setRecursive(comm.getRecursive());
            if (comm.getRevision2() != null) {
                checkComm.setUpdateByRevision(comm.getRevision2());
            } else  if (comm.getBeforeDate2() != null) {
                checkComm.setUpdateByDate(comm.getBeforeDate2());
            }
            checkoutList.add(checkComm);
//        }
    }
    
    // adds the checkouts commands before the diff commands in order to have the 
    private void addCheckoutCommands(LinkedList checkoutList) {
        Iterator it = checkoutList.iterator();
        while (it.hasNext()) {
            toDoCommands.add(0, it.next());
        }
    }
    
}
