/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient.commands.diff;

import org.openide.*;
import org.openide.util.*;
import java.io.*;
import java.util.*;
import java.awt.Dialog;

import org.openide.windows.*;
import org.netbeans.modules.cvsclient.*;
import org.netbeans.modules.cvsclient.commands.*;
import org.openide.filesystems.*;
import org.netbeans.lib.cvsclient.event.*;
import org.netbeans.lib.cvsclient.command.diff.DiffInformation;
import org.netbeans.lib.cvsclient.command.checkout.PipedFileInformation;
import org.netbeans.lib.cvsclient.command.FileInfoContainer;
import org.netbeans.lib.cvsclient.command.diff.DiffCommand;
import org.netbeans.lib.cvsclient.command.Command;
import org.netbeans.lib.cvsclient.command.CommandException;
import org.netbeans.modules.vcscore.util.Debug;
import javax.swing.JComponent;
/**
 *
 * @author  mkleint
 * @version
 */
public class DiffDisplayer implements CommandDisplayer {
    private Debug E=new Debug("DiffDisplayer", true); // NOI18N
    private Debug D=E;
    
    DiffCommand command;
    Class fsCommand;
    DiffInformation diffInfo;
    AbstractDiffJavaCvs diff;
    
    ClientProvider provider;
    boolean finishedOk;
    ArrayList resultList;
    private File startFile;
    private StringBuffer bufLeft;
    private StringBuffer bufRight;
    private int fileLineCount;
    private int leftLineCount;
    private int rightLineCount;
    private File file1;
    private File file2;
    
    /** Creates new Status */
    public DiffDisplayer(DiffCommand diffComm) {
        D.deb("Constructor");
        command = diffComm;
    }
    
 /** Does the actual display - docking into the javacvs Mode, 
  *  displaying as single Dialog.. whatever.
  */
  public void displayOutputData(int moment,Object data) {
      if (moment == CommandDisplayer.MOMENT_AFTER_EACH_EXECUTE) {
           Collection col = (Collection)data;
          Iterator it =  col.iterator();
          // 1. sort out the diff and piped file Infos.
          LinkedList diffs = new LinkedList();
          LinkedList upds = new LinkedList();
          while (it.hasNext()) {
              FileInfoContainer info = (FileInfoContainer)it.next();
              if (info instanceof PipedFileInformation) {
                  upds.add(info);
              }
              if (info instanceof DiffInformation) {
                  diffs.add(info);
              }
          }
          // 2. iterate the diff infos and assign the right piped file info to it..
          it = diffs.iterator();
//          StatusCommand stComm = (StatusCommand)finishedCommand;
          while (it.hasNext()) {
              DiffInformation info = (DiffInformation)it.next();
              PipedFileInformation updInfo = findUpdInfo(upds, info.getRepositoryFileName());
//              E.err("diffing=" + info.getRepositoryFileName());
              if (updInfo != null) {
//                  E.err("found file revision=" + updInfo.getRepositoryRevision());
              } else {
//                  E.err("no file revision");
              }
              DiffDisplayer clonedOne = new DiffDisplayer(command);
              clonedOne.setData(info, command, updInfo);
              clonedOne.displayOutputData();
          }
      }
  }
    
    private PipedFileInformation findUpdInfo(LinkedList list, String name) {
        Iterator it = list.iterator();
        while (it.hasNext()) {
            PipedFileInformation info = (PipedFileInformation)it.next();
            if (info.getRepositoryFileName().equals(name)) {
                return info;
            }
        }
        return null;
    }
    
    /** Does the actual display - docking into the javacvs Mode, 
 *  displaying as single Dialog.. whatever.
 */
    public void displayOutputData() {
        String rightTit = (diffInfo.getRightRevision() != null ? diffInfo.getRightRevision() : "Working file");
        String leftTit = (diffInfo.getLeftRevision() != null ? diffInfo.getLeftRevision() : "Working file");
        diff.open(getName() + " " + diffInfo.getFile().getName(), getMIMEType(diffInfo), file1.getAbsolutePath(), file2.getAbsolutePath(), leftTit, rightTit);
    }
    
    protected String getName() {
        return NbBundle.getBundle(CvsDiff.class).getString("CvsDiff.name");
    }
    
    public AbstractDiffJavaCvs createAbstractDiff() {
        return new AbstractDiffJavaCvs();
    }
    
    public void setData(DiffInformation info, DiffCommand command, PipedFileInformation updInfo) {
        D.deb("setData() beginning");
        File startFile = info.getFile();
        if (updInfo != null) {
            if (!startFile.exists()) {
                startFile = updInfo.getTempFile();
            }
            if (command.getBeforeDate1() != null || command.getRevision2() != null) {
                startFile = updInfo.getTempFile();
            }
        }
        if (startFile == null) return;
        diffInfo = (DiffInformation)info;
        processDocuments(diffInfo, startFile);
        file1 = saveToFile(bufLeft);
        D.deb("file left=" + file1.getAbsolutePath());
        file2 = saveToFile(bufRight);
        D.deb("file right=" + file2.getAbsolutePath());
        diff = createAbstractDiff();
        DiffInformation.DiffChange change = diffInfo.getFirstChange();
        while (change != null) {
            if (change.getType() == DiffInformation.DiffChange.ADD) {
                diff.addAddAction(change.getLeftMin(),change.getRightMin(), change.getRightMax());
            } 
            if (change.getType() == DiffInformation.DiffChange.CHANGE) {
                diff.addChangeAction(change.getLeftMin(), change.getLeftMax(), 
                                     change.getRightMin(), change.getRightMax());
            }
            if (change.getType() == DiffInformation.DiffChange.DELETE) {
                diff.addDeleteAction(change.getLeftMin(),change.getLeftMax(), change.getRightMin());
            }
            change = diffInfo.getNextChange();
        }
        
    }
    
    private String getMIMEType(DiffInformation info) {
        File infoFile = info.getFile();
        if (infoFile == null) return "text/plain";
        String tempStr = infoFile.getName();
        String extension = null;
        D.deb("tempStr=" + tempStr);
        int ind = tempStr.indexOf('.');
        if (ind > 0) {
            extension = tempStr.substring(ind + 1);
        }
        D.deb("extension=" + extension);
        if (extension == null) {
            extension = "text/plain";
        }
        String type = FileUtil.getMIMEType(extension);
        if (type == null) {
            type = "text/plain";
        }
        return type;
    }

    
    private File saveToFile(StringBuffer buff) {
        // TEMPORARY, better solution is to write to the file instantly without the write to StringBuffer
        String TMP_ROOT=System.getProperty("netbeans.user")+File.separator+
                 "system"+File.separator+"javacvs"+File.separator+"tmp";
        File tmpDir = new File(TMP_ROOT);
        if (!tmpDir.exists()) {
            tmpDir.mkdirs();
        }
        long tmpId;
        do {
            tmpId = 10000 * (1 + Math.round (Math.random () * 8)) + Math.round (Math.random () * 1000);
        } while (new File(TMP_ROOT+File.separator+"tmp"+tmpId).exists()); // NOI18N
        String tempFileName = TMP_ROOT+File.separator+"tmp"+tmpId; // NOI18N
        File tempFile = new File(tempFileName);
        try {
        PrintWriter bw = new PrintWriter(new FileOutputStream(tempFile),true);
        bw.print(buff);
        bw.close();
        } catch (IOException exc) {
//            E.err("error writing temp file while doing diff");
        }
        return tempFile;
    }
   
    
    private void processDocuments(DiffInformation diffInfo, File startFile) {
        BufferedReader buf = null;
        leftLineCount = 0;
        rightLineCount = 0;
        try {
            buf = new BufferedReader(new FileReader(startFile));
        } catch (FileNotFoundException exc) {
//            E.err("file" + startFile.getAbsolutePath() + " was not found!");
            return;
        }
        bufLeft = new StringBuffer();
        bufRight = new StringBuffer();
        try {
            String line;
            fileLineCount = 0;
            int gap;
            DiffInformation.DiffChange change = diffInfo.getFirstChange();
            while (true) {
                if (change != null) {
                    boolean isDelete = change.getType() == DiffInformation.DiffChange.DELETE;
//                    int start = change.getMainBeginning();
                    int start = (isDelete ? change.getMainBeginning() : change.getMainBeginning() - 1);
                    if (change.getMainBeginning() >= fileLineCount) {
                        gap = fillFromFile(start, buf);
                        D.deb("filled = " + gap);
                    }
                    if (change.getType() == DiffInformation.DiffChange.ADD) {
                        D.deb("we got ADD change here!");
                        gap = parseAddChange(change, true);
                        skipGap(gap, buf);
                    }
                    if (change.getType() == DiffInformation.DiffChange.DELETE) {
                        D.deb("we got DELETE change here!");
                        gap = parseDeleteChange(change, true);
                    }
                    if (change.getType() == DiffInformation.DiffChange.CHANGE) {
                        D.deb("we got CHANGE change here!");
                        gap = parseChangedChange(change);
                        skipGap(gap, buf);
                    }
                    change = diffInfo.getNextChange();
                } else {
                    gap = fillFromFile(-1, buf);
                    // read till end of file
                    break;
                }
            }
            
        } catch (IOException ioExc) {
            Thread.dumpStack();
        }
        
    }
    
    private void skipGap(int gap, BufferedReader buf) throws IOException {
        for (int ind = 0; ind < gap; ind++) {
            String line = buf.readLine();
            fileLineCount = fileLineCount + 1;
            D.deb("skipped one line=" + line);
        }
        
    }
    
    private int fillFromFile(int start, BufferedReader buf)
    throws IOException {
        D.deb("fillingGap...");
        int readCount = 0;
        int max = start;
        // if max is -1, do read till end of file.
        while (fileLineCount < max || max == -1) {
                String line = buf.readLine();
                if (line == null) break;
                fileLineCount = fileLineCount + 1;
                readCount = readCount + 1;
                D.deb("reading linenum=" + readCount + "  line=" + line);
                leftLineCount = leftLineCount + 1;
                bufLeft.append(line + "\n");
                rightLineCount = rightLineCount + 1;
                bufRight.append(line + "\n");
        }
        return readCount;
    }
    
    private int parseAddChange(DiffInformation.DiffChange change, boolean add) {
        int ind;
        for (ind = change.getRightMin(); ind <= change.getRightMax(); ind++) {
            bufRight.append(change.getLine(ind, false) + "\n");
            rightLineCount = rightLineCount + 1;
            D.deb("adding to main=" + change.getLine(ind, false));
        }
        int gap = ind - change.getRightMin();
        D.deb("final ADDED gap =" + gap);
        return gap;
    }
    
    private int parseDeleteChange(DiffInformation.DiffChange change, boolean delete) {
        int ind;
        for (ind = change.getLeftMin(); ind <= change.getLeftMax(); ind++) {
            leftLineCount = leftLineCount + 1;
            bufLeft.append(change.getLine(ind, true) + "\n");
            D.deb("addding to diff=" + change.getLine(ind, true));
        }
        int gap = ind - change.getLeftMin();
        D.deb("final DELETE gap =" + gap);
        return gap;
    }
    
    private int parseChangedChange(DiffInformation.DiffChange change) {
        int gap = parseAddChange(change, false);
        int toReturn = gap;
        int gap2 = parseDeleteChange(change, false);
        return toReturn;
    }
 
    // -- persistent stuff follows.
    
    public void setCommand(DiffCommand comm) {
      command = comm;
    }
    
    public void setFileSystemCommand(FileSystemCommand fsCom) {
        fsCommand = fsCom.getClass();
        provider = fsCom.getClientProvider();
    }
    
  public File getFileDisplayed() {
      if (diffInfo == null) return null;
      return diffInfo.getFile();
  }
   
  public Object getComparisonData() {
      return diffInfo;
  }
   
  public JComponent getComponent() {
      return null;
  }

   // --------------------------------------------
  // persistantDisplayer stuff..
  // --------------------------------------------
  
  public boolean equalDisplayedData(File file, Class type, Object comparisonData) {
      if (!getClass().equals(type)) return false;
      if (diffInfo == null || diffInfo.getFile() == null
                || !diffInfo.getFile().equals(file)) {
          return false;
      }
      return true;
  }

  public Class getFileSystemCommandClass() {
      return fsCommand;
  }
  
  public String getCommandSwitches() {
      return command.getCVSArguments();
  }
  
  public ClientProvider getClientProvider() {
      return provider;
  }

    
}
