/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.cvsclient.commands.diff;

/**
 * Represents the Diff command switches in GUI.
 * @author  mkleint
 */

import java.util.*;
import javax.swing.*;
import org.openide.util.NbBundle;
import org.netbeans.modules.vcscore.util.Debug;

import org.netbeans.modules.cvsclient.commands.*;
import org.netbeans.lib.cvsclient.command.diff.*;
import org.netbeans.lib.cvsclient.command.*;


public class DiffParamInput extends javax.swing.JPanel implements CommandParamInput {
    
    private static final String K_NONE = NbBundle.getBundle(DiffParamInput.class).getString("DiffParamInput.NoSubstitution");
    private static final String K_BINARY = NbBundle.getBundle(DiffParamInput.class).getString("DiffParamInput.BinarySubst");
    private static final String K_KEYWORD_VALUE = NbBundle.getBundle(DiffParamInput.class).getString("DiffParamInput.KeywordValueSubst");
    private static final String K_KEYWORD_VALUE_LOCKER = NbBundle.getBundle(DiffParamInput.class).getString("DiffParamInput.KeywordValueLockerSubst");
    private static final String K_KEYWORD = NbBundle.getBundle(DiffParamInput.class).getString("DiffParamInput.Keyword");
    private static final String K_VALUE = NbBundle.getBundle(DiffParamInput.class).getString("DiffParamInput.Value");
    private static final String K_OLD = NbBundle.getBundle(DiffParamInput.class).getString("DiffParamInput.Old");
    
    private Object[] comboValues = new Object[] { K_NONE, K_BINARY, K_KEYWORD_VALUE, 
                K_KEYWORD_VALUE_LOCKER, K_KEYWORD, K_VALUE, K_OLD} ;
                
    private String[] cvsValues = new String[] { "", "b", "kv", "kvl", "k", "v", "o" };
    
    /** Creates new form DiffParamInput */
    public DiffParamInput() {
        initComponents ();
        comKeyword.setModel(new DefaultComboBoxModel(comboValues));
        comKeyword.setSelectedIndex(0);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        cbLocal = new javax.swing.JCheckBox();
        lblKeyword = new javax.swing.JLabel();
        comKeyword = new javax.swing.JComboBox();
        jSeparator1 = new javax.swing.JSeparator();
        lblRevision1 = new javax.swing.JLabel();
        txRevision1 = new javax.swing.JTextField();
        lblDate1 = new javax.swing.JLabel();
        txDate1 = new javax.swing.JTextField();
        lblRevision2 = new javax.swing.JLabel();
        txRevision2 = new javax.swing.JTextField();
        lblDate2 = new javax.swing.JLabel();
        txDate2 = new javax.swing.JTextField();
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        cbLocal.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/commands/diff/Bundle").getString("DiffParamInput.cbLocal.text"));
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(12, 12, 5, 6);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(cbLocal, gridBagConstraints1);
        
        
        lblKeyword.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/commands/diff/Bundle").getString("DiffParamInput.lblKeyword.text"));
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.insets = new java.awt.Insets(6, 12, 6, 6);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.EAST;
        add(lblKeyword, gridBagConstraints1);
        
        
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 1;
        add(comKeyword, gridBagConstraints1);
        
        
        jSeparator1.setPreferredSize(new java.awt.Dimension(40, 10));
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.gridwidth = 4;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.ipadx = 2;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(jSeparator1, gridBagConstraints1);
        
        
        lblRevision1.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/commands/diff/Bundle").getString("DiffParamInput.lblRevision1.text"));
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.EAST;
        add(lblRevision1, gridBagConstraints1);
        
        
        txRevision1.setPreferredSize(new java.awt.Dimension(40, 20));
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.ipadx = 30;
        gridBagConstraints1.insets = new java.awt.Insets(3, 3, 3, 3);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(txRevision1, gridBagConstraints1);
        
        
        lblDate1.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/commands/diff/Bundle").getString("DiffParamInput.lblDate1.text"));
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 2;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.insets = new java.awt.Insets(3, 3, 3, 3);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.EAST;
        add(lblDate1, gridBagConstraints1);
        
        
        txDate1.setToolTipText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/commands/diff/Bundle").getString("DiffParamInput.txDate.tooltip"));
        txDate1.setPreferredSize(new java.awt.Dimension(40, 20));
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 3;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.ipadx = 30;
        gridBagConstraints1.insets = new java.awt.Insets(0, 6, 0, 15);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(txDate1, gridBagConstraints1);
        
        
        lblRevision2.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/commands/diff/Bundle").getString("DiffParamInput.lblRevision2.text"));
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.EAST;
        add(lblRevision2, gridBagConstraints1);
        
        
        txRevision2.setPreferredSize(new java.awt.Dimension(40, 20));
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.ipadx = 30;
        gridBagConstraints1.insets = new java.awt.Insets(3, 3, 3, 3);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(txRevision2, gridBagConstraints1);
        
        
        lblDate2.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/commands/diff/Bundle").getString("DiffParamInput.lblDate2.text"));
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 2;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.insets = new java.awt.Insets(3, 3, 3, 3);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.EAST;
        add(lblDate2, gridBagConstraints1);
        
        
        txDate2.setToolTipText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/commands/diff/Bundle").getString("DiffParamInput.txDate.tooltip"));
        txDate2.setPreferredSize(new java.awt.Dimension(40, 20));
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 3;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.ipadx = 30;
        gridBagConstraints1.insets = new java.awt.Insets(0, 6, 0, 15);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(txDate2, gridBagConstraints1);
        
    }//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JCheckBox cbLocal;
    private javax.swing.JLabel lblKeyword;
    private javax.swing.JComboBox comKeyword;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JLabel lblRevision1;
    private javax.swing.JTextField txRevision1;
    private javax.swing.JLabel lblDate1;
    private javax.swing.JTextField txDate1;
    private javax.swing.JLabel lblRevision2;
    private javax.swing.JTextField txRevision2;
    private javax.swing.JLabel lblDate2;
    private javax.swing.JTextField txDate2;
    // End of variables declaration//GEN-END:variables

    
  public void setData(Command command) {
      DiffCommand diffCom = (DiffCommand)command;
      diffCom.resetCVSCommand();
      diffCom.setKeywordSubst(cvsValues[comKeyword.getSelectedIndex()]);
      diffCom.setRecursive(!cbLocal.isSelected());
      // revisions + dates .. TODO
      String rev1 = txRevision1.getText().trim();
      String rev2 = txRevision2.getText().trim();
      String dat1 = txDate1.getText().trim();
      String dat2 = txDate2.getText().trim();
      if (rev1.length() > 0) {
          diffCom.setRevision1(rev1);
      } else {
          diffCom.setRevision1(null);
      }
      if (rev2.length() > 0) {
          diffCom.setRevision2(rev2);
      } else {
          diffCom.setRevision2(null);
      }
      if (dat1.length() > 0) {
          diffCom.setBeforeDate1(dat1);
      } else {
          diffCom.setBeforeDate1(null);
      }
      if (dat2.length() > 0) {
          diffCom.setBeforeDate2(dat2);
      } else {
          diffCom.setBeforeDate2(null);
      }
  } 
  
  public void getData(Command command) {
      DiffCommand diffCom = (DiffCommand)command;
      cbLocal.setSelected(!diffCom.getRecursive());
      for (int index = 0; index < cvsValues.length; index++) {
          if (cvsValues[index].equals(diffCom.getKeywordSubst())) {
              comKeyword.setSelectedIndex(index);
              break;
          }    
      }    
      // set revisions + dates.
      txRevision1.setText(diffCom.getRevision1());
      txRevision2.setText(diffCom.getRevision2());
      txDate1.setText(diffCom.getBeforeDate1());
      txDate2.setText(diffCom.getBeforeDate2());
  }  

  public String getDialogTitle() {
     return NbBundle.getBundle(DiffParamInput.class).getString("DiffParamInput.dialogTitle");
  }
  
  public boolean requiresUI() {
      return true;
  }
  
}
