/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


/**
 *
 * @author  mkleint
 * @version
 */

package org.netbeans.modules.cvsclient.commands.diff;

import java.io.*;
import java.util.*;
import javax.swing.JComponent;

import org.netbeans.modules.vcscore.util.*;
import org.netbeans.modules.vcscore.diff.*;
import org.openide.util.NbBundle;
import org.openide.windows.Workspace;

import org.netbeans.lib.cvsclient.command.diff.DiffCommand;
import org.netbeans.lib.cvsclient.command.diff.DiffInformation;
import org.netbeans.lib.cvsclient.command.checkout.PipedFileInformation;
import org.netbeans.lib.cvsclient.command.FileInfoContainer;
import org.netbeans.modules.cvsclient.commands.*;

public class DiffTreeInfoPanel extends org.netbeans.modules.cvsclient.commands.AbstractTreeInfoPanel
                               implements PersistentCommandDisplayer {
    private Debug E=new Debug("DiffTreeInfoPanel", false); // NOI18N
    private Debug D=E;
    
    private DiffInformation clearDiffInfo;
    private DiffCommand command;
    private Class fsCommand;

    private TreeDiffDisplayer  diffPanel;
    private ClientProvider provider;
    private LinkedList updInfoList;
    
    /** Creates new DiffTreeInfoPanel */
    public DiffTreeInfoPanel(File topDir, DiffCommand comm) {
        super(topDir);
        command = comm;
//        setClearPanel();
//        initPanelComponents();
    }
    
    protected JComponent initPanel() {
        initClearInfo();
        diffPanel = new TreeDiffDisplayer(command);
        setClearPanel();
        
        //      diffPanel = new DiffInfoPanel();
        //      return diffPanel;
        return null;
    }
    
    private void initClearInfo() {
        clearDiffInfo = new DiffInformation();
        clearDiffInfo.setFile(null);
        clearDiffInfo.setLeftRevision("");
        clearDiffInfo.setRightRevision("");
        clearDiffInfo.setRepositoryFileName("");
        clearDiffInfo.setParameters("");
    }
    
    protected void setClearPanel() {
        //     diffPanel.setData(clearDiffInfo, command);
    }
    
    protected void setPanel(Object infoData) {
        DiffInformation diffData = (DiffInformation)infoData;
        PipedFileInformation updInfo = findUpdInfo(updInfoList, diffData.getRepositoryFileName());
        diffPanel.setData(diffData, command, updInfo);
        diffPanel.displayOutputData();
    }
    
    private PipedFileInformation findUpdInfo(LinkedList list, String name) {
        Iterator it = list.iterator();
        while (it.hasNext()) {
            PipedFileInformation info = (PipedFileInformation)it.next();
            if (info.getRepositoryFileName().equals(name)) {
                return info;
            }
        }
        return null;
    }
    
    public void setDataToDisplay(Collection resultList) {
        Iterator it = resultList.iterator();
        LinkedList diffs = new LinkedList();
        LinkedList upds = new LinkedList();
        while (it.hasNext()) {
            FileInfoContainer info = (FileInfoContainer)it.next();
            if (info instanceof PipedFileInformation) {
                upds.add(info);
            }
            if (info instanceof DiffInformation) {
                diffs.add(info);
            }
        }
        updInfoList = upds;
        super.setDataToDisplay(diffs);
    }
    
    class JavaCvsTreeDiffComponent extends JavaCvsDiffComponent {
        public JavaCvsTreeDiffComponent() {
        }
        public JavaCvsTreeDiffComponent(AbstractDiff diff) {
            super(diff);
        }
        public void open(Workspace workspace) {
            // ignores workspace.. just call this when about to be displayed
            // (diffDisplayer.displayOutputData)
            open();
        }
    }
  
/** Does the actual display - docking into the javacvs Mode, 
 *  displaying as single Dialog.. whatever.
 */
  public void displayOutputData() {
      /** Currently the directory diff is not supported. 
       * TODO - first we need to rewrite the DiffComponent in vcscore and make it a panel only, not topcomponent
       
       FileSystemCommand.displayOutputPanel(NbBundle.getBundle(DiffTreeInfoPanel.class).getString("CvsDiff.dialogTitle")
                + topDirectory.getName(), this);
       */
  }
  
  /** Does the actual display - docking into the javacvs Mode, 
   *  displaying as single Dialog.. whatever.
   */
  public void displayOutputData(int moment,Object data) {
      if (moment == CommandDisplayer.MOMENT_AFTER_EACH_EXECUTE) {
          Collection col = (Collection)data;
          setDataToDisplay(col);
          displayOutputData();
      }
  }  
    
    
  public void setCommand(DiffCommand comm) {
        command = comm;
  }
  
    public void setFileSystemCommand(FileSystemCommand fsCom) {
        fsCommand = fsCom.getClass();
        provider  = fsCom.getClientProvider();
    }
  
 
  public JComponent getComponent() {
      return null;
  }
  
  public Object getComparisonData() {
      return topDirectory;
  }
  
  /**
   * Persistent stuff...
   */
  public boolean equalDisplayedData(File file, Class type, Object comparisonData) {
      if (!getClass().equals(type)) return false;
      if (topDirectory == null || 
                !topDirectory.equals(file)) {
          return false;
      }
      return true;
  }
  
  public File getFileDisplayed() {
      return topDirectory;
  }
  
  public Class getFileSystemCommandClass() {
      return fsCommand;
  }
  
  public String getCommandSwitches() {
      return command.getCVSArguments();
  }
  
  public ClientProvider getClientProvider() {
      return provider;
  }

  
  
  
    class JavaCvsTreeAbstractDiff extends AbstractDiffJavaCvs {
        
    /** Creates new AbstractDiffJavaCvs */
        public JavaCvsTreeAbstractDiff() {
            super();
        }
        
        protected DiffComponent createDiffComponent(AbstractDiff diff) {
            return new JavaCvsTreeDiffComponent(diff);
        }
    }
    
    class TreeDiffDisplayer extends DiffDisplayer {
        public TreeDiffDisplayer(DiffCommand diffComm) {
            super(diffComm);
        }
        
        
        public AbstractDiffJavaCvs createAbstractDiff() {
            return new JavaCvsTreeAbstractDiff();
        }
    }
}
