/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient.commands.log;


import org.openide.util.*;
import java.io.*;
import java.util.*;
import java.awt.Dialog;

import org.netbeans.modules.cvsclient.*;
import org.netbeans.lib.cvsclient.event.*;
import org.netbeans.lib.cvsclient.command.log.*;
import org.netbeans.lib.cvsclient.command.*;
import org.netbeans.modules.vcscore.util.Debug;
import org.netbeans.modules.cvsclient.commands.*;

/**
 *
 * @author  mkleint
 * @version
 */
public class CvsLog extends FileSystemCommand {
    private Debug E=new Debug("CvsLog", true); // NOI18N
    private Debug D=E;
    
   
    /** Creates new Status */
    public CvsLog(File[] filesToCheck, ClientProvider fs) {
        super(fs);
/*        if (fs.isFileSystemClient()) {
            JavaCvsFileSystem fileSys = (JavaCvsFileSystem)fs;
            setFileSystem(fileSys);
        }        
 */
        setFiles(filesToCheck);
        D.deb("Constructor");
    }
    
    public CvsLog() {
        super();
    }
    
    public String getName() {
        return NbBundle.getBundle(CvsLog.class).getString("CvsLog.name");
    }
    
/** if no param input was set before staring the command, 
 * the default defined in this method is used
 */
    protected CommandParamInput createDefaultParamInput() {
        return new LogParamInput();
    }    
    
    protected void applyInputParams() {
        Iterator it = toDoCommands.iterator();
     // for all commands queued in the toDoCommand list, 
        // apply the command line params to them (from the dialog)   
        while (it.hasNext()) {
            LogCommand lCom = (LogCommand)it.next();
            getParamInput().setData(lCom);
        }
    }
    
    protected void initCommand() {
        // here the the commands is initiated just right before running the command
        D.deb("initCommand()");
        int dirNum = 0;
        LogCommand command;
        File[] files = new File[getFiles().length];
        System.arraycopy(getFiles(),0,files,0, getFiles().length);

        for (int index = 0; index < files.length; index++) {
            if (files[index].isDirectory()) {
              // each directory status command is sent separately.. 
              //that way just one UI component is run after finish of the command
              command = (LogCommand)loadCommand(LogCommand.class);
//              command.setDirectoryCheck(true);
              File[] fls = new File[1];
              fls[0] = files[index];
              command.setFiles(fls);
              toDoCommands.addElement(command);
              files[index] = null;
              dirNum = dirNum + 1;
            }
        }
        // now proceed all single files in one batch.
        if (dirNum == files.length) {
            return; // no files to check status on.
        }
        File[] singleFiles = new File[files.length - dirNum];
        int singleIndex = 0;
        for (int ind = 0; ind < files.length; ind++) {
            if (files[ind] != null) {
                singleFiles[singleIndex] = files[ind];
                singleIndex = singleIndex + 1;
            }
        }
        command = (LogCommand)loadCommand(LogCommand.class);
//        command.setDirectoryCheck(false);
        command.setFiles(singleFiles);
        toDoCommands.addElement(command);
        // just add oen command to the queue
        super.initCommand();
    }
    
    
    protected void beforeEachExecute() {
        clearResultList();
        if (getDisplayerType() == LogTreeInfoPanel.class) {
            LogCommand statCom = (LogCommand)getCurrentCommand();
            File[] flz = statCom.getFiles();
            if (flz[0].isDirectory()) {
                File[] topDir = statCom.getFiles();
                LogTreeInfoPanel treePanel = (LogTreeInfoPanel)FileSystemCommand.findOpenDisplayer(
                                flz[0], LogTreeInfoPanel.class, null);
                if (treePanel == null) {
                    treePanel = new LogTreeInfoPanel(topDir[0], statCom);
                    treePanel.setFileSystemCommand(this);
                } else {
                    treePanel.setCommand(statCom);
                }
                setOutputDisplayer(treePanel);
            }
        }
        super.beforeEachExecute();
    }
        
    protected void afterEachExecute() {
        super.afterEachExecute();
        if (getDisplayerType() == LogTreeInfoPanel.class ||
            getDisplayerType() == LogInfoPanel.class) {
            setOutputDisplayer(null);
        }
    }
    
    protected void executeFailed(Exception exc) {
        stopCommand();
        super.executeFailed(exc);
    }
    
    /**
     * Called when file status information has been received
     */
    public void fileInfoGenerated(FileInfoEvent e) {
        // kinda hack - don't know currently how to recreate a panel for each file otherwise.
        if (getDisplayerType() == LogInfoPanel.class) {
           LogCommand statCom = (LogCommand)getCurrentCommand();
           LogInformation info = (LogInformation)e.getInfoContainer();
           CommandDisplayer toReturn;
           File fl = info.getFile();
           // find or create statusInfoPanel
           LogInfoPanel panel = (LogInfoPanel)FileSystemCommand.findOpenDisplayer(
                                            fl, LogInfoPanel.class, null);
           if (panel == null) {
               panel = new LogInfoPanel(statCom);
               panel.setFileSystemCommand(this);
           } else {
               panel.setCommand(statCom);
           }
           setOutputDisplayer(panel);
        }
        super.fileInfoGenerated(e);
    }    

    /** if no displayer was set by the setOutputDisplayer() method,
     * a default one will be generated by the subclasses implementation
     *
     */
    
    protected Class getDefaultDisplayerType() {
       LogCommand statCom = (LogCommand)getCurrentCommand();
       File[] flz = statCom.getFiles();
       if (flz[0].isDirectory()) return LogTreeInfoPanel.class;
       return LogInfoPanel.class;
        
    }    
    
    /** if no displayer was set by the setOutputDisplayer() method, 
 * a default one will be generated by the subclasses implementation
 *
 */
    /*
    protected CommandDisplayer createDefaultDisplayer() {
       LogCommand statCom = (LogCommand)getCurrentCommand();
       CommandDisplayer toReturn;
       File[] flz = statCom.getFiles();
       if (flz[0].isDirectory()) {
           File[] topDir = statCom.getFiles();
           toReturn = new LogTreeInfoPanel(topDir[0], statCom);
       } else {
           toReturn = new LogInfoPanel(statCom);
       }
       return toReturn;        
    }
     */
    
}
