/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient.commands.log;

/**
 *
 * @author  mkleint
 * @version $Revision: 1.11.2.3 $
 */

import org.openide.util.*;
import org.openide.awt.SplittedPanel;
import org.netbeans.lib.cvsclient.command.log.LogInformation;
import org.netbeans.lib.cvsclient.command.log.LogCommand;
import org.netbeans.lib.cvsclient.command.Command;
import org.netbeans.modules.vcscore.util.Debug;
import org.netbeans.modules.cvsclient.commands.*;
import org.netbeans.modules.cvsclient.util.*;
import java.util.*;
import java.io.File;
import javax.swing.event.*;
import javax.swing.*;
import java.awt.GridBagConstraints;
import javax.swing.table.*;
import java.awt.event.*;
import java.awt.Dimension;
import java.lang.reflect.Method;

public class LogInfoPanel extends javax.swing.JPanel implements PersistentCommandDisplayer {
    private Debug E=new Debug("LogInfoPanel", true); // NOI18N
    private Debug D=E;
    private LogInformation logInfo;
    private boolean alreadyChanging = false;
    private LogCommand command;
    private Class fsCommand;
    private ClientProvider provider;
    
    private javax.swing.JSplitPane sppMain;
    private javax.swing.JScrollPane spSymNames;
    private javax.swing.JTable tblSymNames;
    private javax.swing.JScrollPane spMain;
    private javax.swing.JTable tblRevisions;
    
    /** Creates new form LogInfoPanel */
    
    
    public LogInfoPanel(Command comm) {
        initComponents ();
        command = (LogCommand)comm;
        SplittedPanel split = new SplittedPanel();
        split.setPreferredSize(new java.awt.Dimension(600, 350));
        split.setMinimumSize(new java.awt.Dimension(600, 250));
        split.add(initSymNames(),SplittedPanel.ADD_LEFT);
        split.add(initRevisionList(),SplittedPanel.ADD_RIGHT);
        split.setSplitAbsolute(false);
        split.setSplitDragable(true);
        split.setSplitPosition(30);
        GridBagConstraints gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.ipady = 13;
        gridBagConstraints1.insets = new java.awt.Insets(6, 12, 5, 11);
        gridBagConstraints1.weightx = 0.3;
        gridBagConstraints1.weighty = 0.3;
        add(split, gridBagConstraints1);

        setPreferredSize(new java.awt.Dimension(750, 400));
        setMinimumSize(new java.awt.Dimension(750, 400));          
        
    }
    
    
    public void setCommand(LogCommand comm) {
      command = comm;
    }
    
    public void setFileSystemCommand(FileSystemCommand fsCom) {
        fsCommand = fsCom.getClass();
        provider = fsCom.getClientProvider();
    }
    
    private JComponent initSymNames() {
        spSymNames = new javax.swing.JScrollPane();
        tblSymNames = new javax.swing.JTable();
        JTableHeader head = tblSymNames.getTableHeader();
        head.setUpdateTableInRealTime(true);
        ColumnSortListener listen = new ColumnSortListener(tblSymNames);
        head.addMouseListener(listen);
        tblSymNames.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        tblSymNames.getSelectionModel().addListSelectionListener(
        new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                symNamesValueChanged();
            }
        });
        
        spSymNames.setViewportView(tblSymNames);
        return spSymNames;
        
    }
    
    private JComponent initRevisionList() {
        spMain = new javax.swing.JScrollPane();
        tblRevisions = new javax.swing.JTable();
        spMain.setViewportView(tblRevisions);
        tblRevisions.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        tblRevisions.setRowSelectionAllowed(true);
        tblRevisions.getSelectionModel().addListSelectionListener( new ListSelectionListener() {
            //TODO
            public void valueChanged(ListSelectionEvent e) {
                int rowIndex = tblRevisions.getSelectedRow();
                if (rowIndex < 0) {
                    taRevLog.setText("");
                    return;
                }
                String revision = (String)tblRevisions.getModel().getValueAt(rowIndex,0);
                D.deb("Selected Revision=" + revision);
                // set the log message for the revision
                String text = logInfo.getRevision(revision).getMessage();
                taRevLog.setText(text);
            }
            
        });
        JTableHeader head = tblRevisions.getTableHeader();
        head.setUpdateTableInRealTime(true);
        ColumnSortListener listen = new ColumnSortListener(tblRevisions);
        head.addMouseListener(listen);
        return spMain;
        
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        pnlHead = new javax.swing.JPanel();
        lblWorkFile = new javax.swing.JLabel();
        txWorkFile = new javax.swing.JLabel();
        lblRepository = new javax.swing.JLabel();
        txRepository = new javax.swing.JLabel();
        lblHeadRev = new javax.swing.JLabel();
        txHeadRev = new javax.swing.JLabel();
        lblBranch = new javax.swing.JLabel();
        txBranch = new javax.swing.JLabel();
        lblLocks = new javax.swing.JLabel();
        txLocks = new javax.swing.JLabel();
        lblSelRev = new javax.swing.JLabel();
        txSelRev = new javax.swing.JLabel();
        lblTotalRev = new javax.swing.JLabel();
        txTotalRev = new javax.swing.JLabel();
        lblDescription = new javax.swing.JLabel();
        txDescription = new javax.swing.JLabel();
        spRevLog = new javax.swing.JScrollPane();
        taRevLog = new javax.swing.JEditorPane();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        pnlHead.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints2;
        
        lblWorkFile.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/commands/log/Bundle").getString("LogInfoPanel.lblWorkFile.text"));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.insets = new java.awt.Insets(12, 12, 0, 0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints2.weightx = 1.0;
        pnlHead.add(lblWorkFile, gridBagConstraints2);
        
        txWorkFile.setText("jLabel2");
        txWorkFile.setForeground(java.awt.Color.black);
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.ipadx = 50;
        gridBagConstraints2.insets = new java.awt.Insets(12, 12, 0, 0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints2.weightx = 2.0;
        pnlHead.add(txWorkFile, gridBagConstraints2);
        
        lblRepository.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/commands/log/Bundle").getString("LogInfoPanel.lblRepository.text"));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 0;
        gridBagConstraints2.gridy = 1;
        gridBagConstraints2.insets = new java.awt.Insets(12, 12, 0, 0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        pnlHead.add(lblRepository, gridBagConstraints2);
        
        txRepository.setText("jLabel4");
        txRepository.setForeground(java.awt.Color.black);
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 1;
        gridBagConstraints2.gridy = 1;
        gridBagConstraints2.gridwidth = 7;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.ipadx = 30;
        gridBagConstraints2.insets = new java.awt.Insets(12, 12, 0, 11);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints2.weightx = 1.0;
        pnlHead.add(txRepository, gridBagConstraints2);
        
        lblHeadRev.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/commands/log/Bundle").getString("LogInfoPanel.lblHeadRev.text"));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 0;
        gridBagConstraints2.gridy = 2;
        gridBagConstraints2.insets = new java.awt.Insets(12, 12, 0, 0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        pnlHead.add(lblHeadRev, gridBagConstraints2);
        
        txHeadRev.setText("jLabel6");
        txHeadRev.setForeground(java.awt.Color.black);
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 1;
        gridBagConstraints2.gridy = 2;
        gridBagConstraints2.ipadx = 30;
        gridBagConstraints2.insets = new java.awt.Insets(12, 12, 0, 0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        pnlHead.add(txHeadRev, gridBagConstraints2);
        
        lblBranch.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/commands/log/Bundle").getString("LogInfoPanel.lblBranch.text"));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 2;
        gridBagConstraints2.gridy = 2;
        gridBagConstraints2.insets = new java.awt.Insets(12, 17, 0, 0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints2.weightx = 1.0;
        pnlHead.add(lblBranch, gridBagConstraints2);
        
        txBranch.setText("jLabel8");
        txBranch.setForeground(java.awt.Color.black);
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 3;
        gridBagConstraints2.gridy = 2;
        gridBagConstraints2.ipadx = 41;
        gridBagConstraints2.insets = new java.awt.Insets(12, 12, 0, 0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints2.weightx = 2.0;
        pnlHead.add(txBranch, gridBagConstraints2);
        
        lblLocks.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/commands/log/Bundle").getString("LogInfoPanel.lblLocks.text"));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 4;
        gridBagConstraints2.gridy = 0;
        gridBagConstraints2.insets = new java.awt.Insets(12, 17, 0, 0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints2.weightx = 1.0;
        pnlHead.add(lblLocks, gridBagConstraints2);
        
        txLocks.setText("jLabel11");
        txLocks.setForeground(java.awt.Color.black);
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 5;
        gridBagConstraints2.gridy = 0;
        gridBagConstraints2.ipadx = 23;
        gridBagConstraints2.insets = new java.awt.Insets(12, 12, 0, 11);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints2.weightx = 2.0;
        pnlHead.add(txLocks, gridBagConstraints2);
        
        lblSelRev.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/commands/log/Bundle").getString("LogInfoPanel.lblSelRev.text"));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 4;
        gridBagConstraints2.gridy = 2;
        gridBagConstraints2.insets = new java.awt.Insets(12, 17, 0, 0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        pnlHead.add(lblSelRev, gridBagConstraints2);
        
        txSelRev.setText("jLabel16");
        txSelRev.setForeground(java.awt.Color.black);
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 5;
        gridBagConstraints2.gridy = 2;
        gridBagConstraints2.insets = new java.awt.Insets(12, 12, 0, 0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        pnlHead.add(txSelRev, gridBagConstraints2);
        
        lblTotalRev.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/commands/log/Bundle").getString("LogInfoPanel.lblTotalRev.text"));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 6;
        gridBagConstraints2.gridy = 2;
        gridBagConstraints2.insets = new java.awt.Insets(12, 17, 0, 0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints2.weightx = 1.0;
        pnlHead.add(lblTotalRev, gridBagConstraints2);
        
        txTotalRev.setText("jLabel18");
        txTotalRev.setForeground(java.awt.Color.black);
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 7;
        gridBagConstraints2.gridy = 2;
        gridBagConstraints2.insets = new java.awt.Insets(12, 12, 0, 11);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints2.weightx = 2.0;
        pnlHead.add(txTotalRev, gridBagConstraints2);
        
        lblDescription.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/commands/log/Bundle").getString("LogInfoPanel.lblDescription.text"));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 0;
        gridBagConstraints2.gridy = 3;
        gridBagConstraints2.insets = new java.awt.Insets(5, 12, 11, 0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        pnlHead.add(lblDescription, gridBagConstraints2);
        
        txDescription.setText("jLabel2");
        txDescription.setForeground(java.awt.Color.black);
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 1;
        gridBagConstraints2.gridy = 3;
        gridBagConstraints2.gridwidth = 7;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints2.insets = new java.awt.Insets(5, 12, 11, 11);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        pnlHead.add(txDescription, gridBagConstraints2);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        add(pnlHead, gridBagConstraints1);
        
        spRevLog.setPreferredSize(new java.awt.Dimension(600, 50));
        spRevLog.setMinimumSize(new java.awt.Dimension(600, 100));
        taRevLog.setEditable(false);
        taRevLog.setFont(new java.awt.Font("Default", 0, 11));
        taRevLog.setMinimumSize(new java.awt.Dimension(726, 50));
        spRevLog.setViewportView(taRevLog);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.ipady = 26;
        gridBagConstraints1.insets = new java.awt.Insets(0, 12, 11, 11);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTH;
        gridBagConstraints1.weightx = 0.5;
        gridBagConstraints1.weighty = 0.5;
        add(spRevLog, gridBagConstraints1);
        
    }//GEN-END:initComponents
    
  private void lstSymNamesValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_lstSymNamesValueChanged
      // Add your handling code here:
/*      if (alreadyChanging) {
          alreadyChanging = false;
          return;
      }
      String selName = (String)lstSymNames.getSelectedValue();
      D.deb("selName=" + selName);
      String revName = logInfo.getRevisionForSymName(selName);
      D.deb("revisionName=" + revName);
      LogInformation.Revision rev = logInfo.getRevision(revName);
      D.deb("Rev=" + rev);
      RevisionModel model = (RevisionModel)tblRevisions.getModel();
      int index = model.getRevisionIndex(rev.getNumber());
//      tblRevisions.getSelectionModel().setSelectionInterval(index, index);
      alreadyChanging = true;
      tblRevisions.changeSelection(index,0, false, false);
 */
  }//GEN-LAST:event_lstSymNamesValueChanged
  
  
  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JPanel pnlHead;
  private javax.swing.JLabel lblWorkFile;
  private javax.swing.JLabel txWorkFile;
  private javax.swing.JLabel lblRepository;
  private javax.swing.JLabel txRepository;
  private javax.swing.JLabel lblHeadRev;
  private javax.swing.JLabel txHeadRev;
  private javax.swing.JLabel lblBranch;
  private javax.swing.JLabel txBranch;
  private javax.swing.JLabel lblLocks;
  private javax.swing.JLabel txLocks;
  private javax.swing.JLabel lblSelRev;
  private javax.swing.JLabel txSelRev;
  private javax.swing.JLabel lblTotalRev;
  private javax.swing.JLabel txTotalRev;
  private javax.swing.JLabel lblDescription;
  private javax.swing.JLabel txDescription;
  private javax.swing.JScrollPane spRevLog;
  private javax.swing.JEditorPane taRevLog;
  // End of variables declaration//GEN-END:variables

  private static final java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/commands/log/Bundle");  
  
  private void symNamesValueChanged() {
      int row = tblSymNames.getSelectedRow();
      if (row < 0) return;
      if (!(tblSymNames.getModel() instanceof TableInfoModel)) return;
      if (!(tblRevisions.getModel() instanceof TableInfoModel)) return;
      LogSymNamesModel mod = (LogSymNamesModel)tblSymNames.getModel();
      LogInformation.SymName symName = mod.getSymName(row);
      LogRevisionModel revModel = (LogRevisionModel)tblRevisions.getModel();
      int revRow = revModel.getRevisionIndex(symName.getRevision());


      if (revRow == -1) {
          tblRevisions.clearSelection();
          return;
      }
//      tblRevisions.setRowSelectionAllowed(true);
      tblRevisions.changeSelection(revRow,0,false,false);
  }    
  
  public void setData(LogInformation info, LogCommand comm) {
      tblRevisions.clearSelection();
      tblSymNames.clearSelection();
      logInfo = info;
      command = comm;
      txWorkFile.setText(logInfo.getFile().getName());
      txRepository.setText(logInfo.getRepositoryFilename());
      txHeadRev.setText(logInfo.getHeadRevision());
      txBranch.setText(logInfo.getBranch());
      txLocks.setText(logInfo.getLocks());
      txSelRev.setText(logInfo.getSelectedRevisions());
      txTotalRev.setText(logInfo.getTotalRevisions());
      if (command.isHeaderOnly()) {
          pnlHead.remove(txDescription);
          pnlHead.remove(lblDescription);
      } else {
          txDescription.setText(logInfo.getDescription());
      }
      // populating the list of symbolic names
      D.deb("setData()");
      if (command.isNoTags()) {
//          sppMain.remove(spSymNames);
//          sppMain.setDividerSize(1);
          //            sppMain.remove(spMain);
          //            sppMain.setLeftComponent(spMain);
      } else {
          fillSymNames();
      }
      if (command.isHeaderAndDescOnly() || command.isHeaderOnly()) {
/*          if (command.isNoTags()) remove(sppMain);
          else sppMain.remove(spMain);
          sppMain.setDividerSize(1);
 */
          remove(spRevLog);
      } else {
          fillRevisions();
      }
  }
  
  private void fillSymNames() {
// setting the model....
        LogSymNamesModel model = new LogSymNamesModel(logInfo.getAllSymbolicNames());
      tblSymNames.setModel(model);
  }
  
  private void fillRevisions() {
      // populationg revisions
      tblRevisions.setModel(new LogRevisionModel(logInfo.getRevisionList()));
      createRevisionsColumnModel();
  }
  
  private void createRevisionsColumnModel() {
      TableColumnModel model = new DefaultTableColumnModel();
      // revision
      TableColumn col = new TableColumn();
      col.setIdentifier("Revision");
      col.setModelIndex(0);
      col.setHeaderValue(NbBundle.getBundle(LogInfoPanel.class).getString("LogInfoPanel.RevisionColumn"));
      col.setMaxWidth(100);
      col.setMinWidth(50);
      col.setPreferredWidth(50);
      model.addColumn(col);
      // author
      col = new TableColumn();
      col.setIdentifier("Author");
      col.setModelIndex(1);
      col.setHeaderValue(NbBundle.getBundle(LogInfoPanel.class).getString("LogInfoPanel.AuthorColumn"));
      col.setMaxWidth(150);
      col.setMinWidth(50);
      col.setPreferredWidth(75);
      model.addColumn(col);
      // date
      col = new TableColumn();
      col.setIdentifier("Date");
      col.setModelIndex(2);
      col.setHeaderValue(NbBundle.getBundle(LogInfoPanel.class).getString("LogInfoPanel.DateColumn"));
      col.setMaxWidth(120);
      col.setMinWidth(10);
      col.setPreferredWidth(110);
      model.addColumn(col);
      // state
      col = new TableColumn();
      col.setIdentifier("State");
      col.setModelIndex(3);
      col.setHeaderValue(NbBundle.getBundle(LogInfoPanel.class).getString("LogInfoPanel.StateColumn"));
      col.setMaxWidth(100);
      col.setMinWidth(30);
      col.setPreferredWidth(50);
      model.addColumn(col);
      // lines
      col = new TableColumn();
      col.setIdentifier("Lines");
      col.setModelIndex(4);
      col.setHeaderValue(NbBundle.getBundle(LogInfoPanel.class).getString("LogInfoPanel.LinesColumn"));
      col.setMaxWidth(100);
      col.setMinWidth(1);
      col.setPreferredWidth(70);
      model.addColumn(col);
      // message
      col = new TableColumn();
      col.setIdentifier("Message");
      col.setModelIndex(5);
      col.setHeaderValue(NbBundle.getBundle(LogInfoPanel.class).getString("LogInfoPanel.LogMessage"));
      col.setMaxWidth(500);
      col.setMinWidth(50);
      col.setPreferredWidth(100);
      model.addColumn(col);
      tblRevisions.setColumnModel(model);
  }
/*  
  private void addSortingToRevisions(List listWithData) {
      JTableHeader head = tblRevisions.getTableHeader();
      head.setUpdateTableInRealTime(true);
      LogRevisionModel model = (LogRevisionModel)tblRevisions.getModel();
      ColumnSortListener listen = new ColumnSortListener(tblRevisions);
      head.addMouseListener(listen);
 
  }
  */
  /** in this method the displayer should use the data returned by the command to
 * produce it's own data structures/ fill in UI components
 * @param resultList - the data from the command. It is assumed the Displayer 
 * knows what command the data comes from and most important in what format. 
 * (which FileInfoContainer class is used).
 */
  public void setDataToDisplay(LogInformation info) {
        setData(info, command);
  }  
  
  /** Does the actual display - docking into the javacvs Mode, 
 *  displaying as single Dialog.. whatever.
 */
  public void displayOutputData() {
      String title = NbBundle.getBundle(LogInfoPanel.class).getString("CvsLog.dialogTitle");
      title = title + logInfo.getFile().getName();
      FileSystemCommand.displayOutputPanel(title, this);
  }  
  
 /** Does the actual display - docking into the javacvs Mode, 
  *  displaying as single Dialog.. whatever.
  */
  public void displayOutputData(int moment,Object data) {
      if (moment == CommandDisplayer.MOMENT_FILE_INFO_GENERATED) {
          LogInformation info = (LogInformation)data;
//          LogInfoPanel clonedOne = new LogInfoPanel(command);
          setDataToDisplay(info);
          displayOutputData();
      }
  }
  
  
  public File getFileDisplayed() {
      if (logInfo == null) return null;
      return logInfo.getFile();
  }
   
  public Object getComparisonData() {
      return logInfo;
  }
   
  public JComponent getComponent() {
      return this;
  }

   // --------------------------------------------
  // persistantDisplayer stuff..
  // --------------------------------------------
  
  public boolean equalDisplayedData(File file, Class type, Object comparisonData) {
      if (!getClass().equals(type)) return false;
      if (logInfo == null || logInfo.getFile() == null
                || !logInfo.getFile().equals(file)) {
          return false;
      }
      return true;
  }

  public Class getFileSystemCommandClass() {
      return fsCommand;
  }
  
  public String getCommandSwitches() {
      return command.getCVSArguments();
  }
  
  public ClientProvider getClientProvider() {
      return provider;
  }
  
  
}
