/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient.commands.log;

import org.openide.util.*;
import org.netbeans.lib.cvsclient.command.log.LogInformation;
import org.netbeans.lib.cvsclient.command.log.LogCommand;
import org.netbeans.modules.cvsclient.commands.*;
import org.netbeans.modules.vcscore.util.Debug;
import java.util.*;
import javax.swing.event.*;
import javax.swing.*;
import javax.swing.table.*;
import java.awt.event.*;

/**
 *
 * @author  mkleint
 * @version 
 */
public class LogRevisionModel extends TableInfoModel {
    private Debug E=new Debug("LogRevisionModel", true); // NOI18N
    private Debug D=E;
      
      
      public LogRevisionModel(List logInfoList) {
          list = new LinkedList(logInfoList);
          D.deb("getRowCount()=" + list.size());
          setDirection(true);
          setActiveColumn(0);
          
      }

      
      
      public int getColumnCount() {
          return 6;
      }
      
      public Object getValueAt(int row, int column) {
//          if (row < 0 && row >= getRowCount()) return "";
          LogInformation.Revision rev = (LogInformation.Revision)list.get(row);
          switch (column) {
              case 0 : return rev.getNumber();
              case 1 : return rev.getAuthor();
              case 2 : return rev.getDate();
              case 3 : return rev.getState();
              case 4 : return rev.getLines();
              case 5 : return getFirstLineOfMessage(rev.getMessage());
          }
          return "";
      }
      
      private String getFirstLineOfMessage(String message) {
          int index = message.indexOf('\n');
          if (index > 0) {
              return message.substring(0,index);
          }
          return message;
      }
      
      
      
      public java.lang.String getColumnName(int param) {
          switch (param) {
              case 0 : return NbBundle.getBundle(LogRevisionModel.class).getString("LogInfoPanel.RevisionColumn");
              case 1 : return NbBundle.getBundle(LogRevisionModel.class).getString("LogInfoPanel.AuthorColumn");
              case 2 : return NbBundle.getBundle(LogRevisionModel.class).getString("LogInfoPanel.DateColumn");
              case 3 : return NbBundle.getBundle(LogRevisionModel.class).getString("LogInfoPanel.StateColumn");
              case 4 : return NbBundle.getBundle(LogRevisionModel.class).getString("LogInfoPanel.LinesColumn");
              case 5 : return NbBundle.getBundle(LogRevisionModel.class).getString("LogInfoPanel.LogMessage");
          }
          return "";
          
      }
      
      public int getRevisionIndex(String revision) {
          Iterator it = list.iterator();
          while (it.hasNext()) {
              LogInformation.Revision rev = (LogInformation.Revision)it.next();
              if (revision.equals(rev.getNumber())) {
                  return list.indexOf(rev);
              }
          }
          return -1;
      }
      
      
      public int compare(Object obj1, Object obj2) {
          if (!(obj1 instanceof LogInformation.Revision) ||
          !(obj2 instanceof LogInformation.Revision)) return 0;
          LogInformation.Revision rev1 = (LogInformation.Revision)obj1;
          LogInformation.Revision rev2 = (LogInformation.Revision)obj2;
          int result = 0;
          switch (getActiveColumn()) {
              case 0: // revision
                  result = getDirection() * compareVersions(rev1.getNumber(), rev2.getNumber());
                  break;
              case 1: // author
                  result = getDirection() * rev1.getAuthor().compareTo(rev2.getAuthor());
                  break;
              case 2: // date
                  result = getDirection() * rev1.getDate().compareTo(rev2.getDate());
                  break;
              case 3: // state
                  result = getDirection() * rev1.getState().compareTo(rev2.getState());
                  break;
              case 4: // lines changed
                  String changed1 = (rev1.getLines() == null ? "" : rev1.getLines());
                  String changed2 = (rev2.getLines() == null ? "" : rev2.getLines());
                  result = getDirection() * changed1.compareTo(changed2);
                  break;
              case 5: // log message
                  result = getDirection() * rev1.getMessage().compareTo(rev2.getMessage());
                  break;
          }
          return result;
      }

}

