/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient.commands.log;


import java.io.*;
import java.util.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.tree.DefaultMutableTreeNode;

import org.netbeans.modules.vcscore.util.Debug;
import org.netbeans.lib.cvsclient.command.log.LogCommand;
import org.netbeans.lib.cvsclient.command.log.LogInformation;
import org.openide.util.*;
import org.netbeans.modules.cvsclient.commands.*;
import org.netbeans.lib.cvsclient.command.*;
/**
 *
 * @author  mkleint
 * @version
 */
public class LogTreeInfoPanel extends AbstractTreeInfoPanel 
                              implements PersistentCommandDisplayer {
    private Debug E=new Debug("LogTreeInfoPanel", true); // NOI18N
    private Debug D=E;
    
    private LogInfoPanel logPanel;
    private LogInformation clearLogInfo;
    private LogCommand command;
    private Class fsCommand;
    private ClientProvider provider;
    private JTextField txFilter;
    private JLabel lblFilter;
    private JLabel lblTitle;
    private String filterString = null;
    private int totalCount = 0;
    private int selectedCount = 0;
    private javax.swing.JLabel    lblCount;   
    /** Creates new form StatusTreeInfoPanel */
    public LogTreeInfoPanel(File topDir, LogCommand comm) {
        super(topDir);
        command = comm;
        initPanelComponenents();
        postInit();
 //       recreateModel();
    }
    
    private void initPanelComponenents() {
        JPanel panel = getButtonPanel();
        lblTitle = new javax.swing.JLabel();
        lblFilter = new javax.swing.JLabel();
        txFilter = new javax.swing.JTextField();
        lblCount = new javax.swing.JLabel();
        panel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        lblTitle.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/commands/log/Bundle").getString("LogTreeInfoPanel.lblTitle.text"));
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        panel.add(lblTitle, gridBagConstraints1);
        
        
        lblFilter.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/commands/log/Bundle").getString("LogTreeInfoPanel.filterLog"));
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        panel.add(lblFilter, gridBagConstraints1);
        
        
        txFilter.setPreferredSize(new java.awt.Dimension(80, 20));
        txFilter.setText("");
        txFilter.setMinimumSize(new java.awt.Dimension(60, 20));
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        panel.add(txFilter, gridBagConstraints1);
        
        
        lblCount.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/commands/log/Bundle").getString("LogTreeInfoPanel.lblCount"));
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.gridwidth = 2;
        panel.add(lblCount, gridBagConstraints1);
 

//---- ENDO OF COPIED STUFF --- ===================================================
// =========================================================================
        txFilter.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                filterLogMessages();
            }    
        });      
    }    
    
    
  private void initClearInfo() {
      clearLogInfo = new LogInformation();
      clearLogInfo.setFile(new File(""));
      clearLogInfo.setAccessList("");
      clearLogInfo.setBranch("");
      clearLogInfo.setDescription("");
      clearLogInfo.setHeadRevision("");
      clearLogInfo.setKeywordSubstitution("");
      clearLogInfo.setLocks("");
      clearLogInfo.setRepositoryFilename("");
      clearLogInfo.setSelectedRevisions("");
      clearLogInfo.setTotalRevisions("");
  }
  
  
  protected void setPanel(Object infoData) {
      LogInformation logData = (LogInformation)infoData;
      logPanel.setData(logData, command);
  }
  
  protected JComponent initPanel() {
      initClearInfo();
      logPanel = new LogInfoPanel(command);
      setClearPanel();
      return logPanel;
  }
  
  protected void setClearPanel() {
      logPanel.setData(clearLogInfo, command);
  }

/** Does the actual display - docking into the javacvs Mode, 
 *  displaying as single Dialog.. whatever.
 */
  public void displayOutputData() {
     FileSystemCommand.displayOutputPanel(NbBundle.getBundle(LogTreeInfoPanel.class).getString("CvsLog.dialogTitle")
                + topDirectory.getName(), this);

  }
  
  private void filterLogMessages() {
      String filter = txFilter.getText(); 
      if (filter.trim().equals("")) filterString = null;
      else filterString = filter;
      totalCount = 0;
      selectedCount = 0;
      recreateModel();
      lblCount.setText(selectedCount + " " + 
                         NbBundle.getBundle(LogTreeInfoPanel.class).getString("LogTreeInfoPanel.lblCount") + 
                         " " + totalCount);

  }    
  
  /** defines which InfoContainer instances will be added to the tree
   * By default adds all, can be overriden by subclasses to define filtering
 */
  protected void addFileNode(FileInfoContainer info,DefaultMutableTreeNode parent) {
      LogInformation logInfo = (LogInformation)info;
      totalCount = totalCount + 1;
      if (shouldAddLogInfo(logInfo)) {
            DefaultMutableTreeNode child = new DefaultMutableTreeNode(info);
            parent.add(child);
            selectedCount = selectedCount + 1;
      }    
  }
  
  /** defines which InfoContainer instances will be added to the table
   * By default adds all, can be overriden by subclasses to define filtering
 */
  protected boolean addToList(FileInfoContainer info) {
      LogInformation logInfo = (LogInformation)info;
      return shouldAddLogInfo(logInfo);
  }
  
  private boolean shouldAddLogInfo(LogInformation info) {
      if (filterString == null) return true;
      List revList = info.getRevisionList();
      Iterator it = revList.iterator();
      while (it.hasNext()) {
          LogInformation.Revision revis = (LogInformation.Revision)it.next();
          if (revis.getMessage().indexOf(filterString) > 0) {
              return true;
          }    
      }    
      return false;
  }    
  
  /** in this method the displayer should use the data returned by the command to
   * produce it's own data structures/ fill in UI components
   * @param resultList - the data from the command. It is assumed the Displayer 
   * knows what command the data comes from and most important in what format. 
   * (which FileInfoContainer class is used).
 */
  public void setDataToDisplay(Collection resultList) {
      totalCount = 0;
      selectedCount = 0;
      super.setDataToDisplay(resultList);
      lblCount.setText(selectedCount + " " + 
                         NbBundle.getBundle(LogTreeInfoPanel.class).getString("LogTreeInfoPanel.lblCount") + 
                         " " + totalCount);
  }

  /** Does the actual display - docking into the javacvs Mode, 
   *  displaying as single Dialog.. whatever.
   */
  public void displayOutputData(int moment,Object data) {
      if (moment == CommandDisplayer.MOMENT_AFTER_EACH_EXECUTE) {
          Collection col = (Collection)data;
          setDataToDisplay(col);
          displayOutputData();
      }
  }  
  
  public void setCommand(LogCommand comm) {
        command = comm;
  }
  
    public void setFileSystemCommand(FileSystemCommand fsCom) {
        fsCommand = fsCom.getClass();
        provider  = fsCom.getClientProvider();
    }
  
 
  public JComponent getComponent() {
      return this;
  }
  
  public Object getComparisonData() {
      return topDirectory;
  }
  
  /**
   * Persistent stuff...
   */
  public boolean equalDisplayedData(File file, Class type, Object comparisonData) {
      if (!getClass().equals(type)) return false;
      if (topDirectory == null || 
                !topDirectory.equals(file)) {
          return false;
      }
      return true;
  }
  
  public File getFileDisplayed() {
      return topDirectory;
  }
  
  public Class getFileSystemCommandClass() {
      return fsCommand;
  }
  
  public String getCommandSwitches() {
      return command.getCVSArguments();
  }
  
  public ClientProvider getClientProvider() {
      return provider;
  }
  
}
