/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient.commands.log;

import java.awt.event.ActionListener;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import java.util.*;
import java.io.File;
import org.netbeans.modules.cvsclient.commands.*;
import org.netbeans.lib.cvsclient.command.Command;
import org.netbeans.modules.cvsclient.commands.update.CvsUpdate;
import org.netbeans.modules.cvsclient.commands.diff.CvsDiff;
import org.netbeans.modules.vcscore.revision.RevisionNode;
import org.openide.awt.JMenuPlus;
import org.openide.util.actions.*;
import org.openide.filesystems.*;
import org.openide.nodes.*;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;

import org.netbeans.modules.cvsclient.JavaCvsFileSystem;

/**
 * Node action for the revision explorer in javaCvs module
 * @author  mkleint
 * @version 
 */
public class RevisionExplorerAction extends NodeAction  {

    /** Creates new RevisionAction 
     * Gets revision actions from filesystem and acts on the givn file object.
     * @param fileSystem the file system to get the actions from
     * @param fo the file object to act on
     */
    public RevisionExplorerAction() {
    }

    public String getName(){
        return NbBundle.getBundle(RevisionExplorerAction.class).
                     getString("CTL_Revision_Action"); // NOI18N
    }

    //-------------------------------------------

    //-------------------------------------------
    public boolean enable(Node[] nodes){
        //D.deb("enable()"); // NOI18N
        return (nodes.length > 0 && nodes.length < 3);
    }

    //-------------------------------------------
    public HelpCtx getHelpCtx(){
        //D.deb("getHelpCtx()"); // NOI18N
        return null;
    }
    
        //-------------------------------------------
    public void performAction(Node[] nodes){
        //D.deb("performAction()"); // NOI18N
    }
    
    private JMenuItem createItem(String name, boolean switchable){
        JMenuItem item=null ;
        item=new JMenuItem(g(name));
        item.setActionCommand(name);
        item.addActionListener(this);
//        item.addMenuKeyListener(this);
        assignHelp (item, name);
        return item;
    }
    
    public static void openAction(String revision, JavaCvsFileSystem fs, FileObject fo) {
       final String fileName = fo.getPackageNameExt('/','.');
       File[] fls = new File[1];
       final String getRevision = revision;
       fls[0] = new File(fs.constructRootDirectory(), fileName);
       CvsUpdate command = new CvsUpdate(fls, fs.createClientProvider());
       command.setLoadDefaultSwitches(false);
       command.setFixedDisplayType(true);
       command.setForceParamInput(true);
       command.setParamInput(new ParserParamInput("-p -r " + getRevision));
       command.startCommand();
    }
    
    
    public JMenuItem getPopupPresenter() {
        Node[] nodes = getActivatedNodes();
        JMenu menu=new JMenuPlus(getName()); // NOI18N
        HelpCtx.setHelpIDString (menu, RevisionExplorerAction.class.getName ());
        JMenuItem item=null;
        JMenu submenu = null;
        boolean isOnOne = (nodes.length == 1);
        boolean isOnTwo = (nodes.length == 2);
        if (isOnOne) {
            item=createItem("RevisionExplorerAction.update", false); // NOI18N
            menu.add(item);
            item=createItem("RevisionExplorerAction.diff", false); // NOI18N
            menu.add(item);
            item=createItem("RevisionExplorerAction.merge", false); // NOI18N
            menu.add(item);
        }
        if (isOnTwo) {
            item=createItem("RevisionExplorerAction.merge2Rev", false); // NOI18N
            menu.add(item);
            item=createItem("RevisionExplorerAction.diff2Rev", false); // NOI18N
            menu.add(item);
        }
        return menu;
    }
    
    private File[] getFiles(Node[] nodes, JavaCvsFileSystem fileSystem) {
        File[] files = new File[nodes.length];
        for (int i = 0; i < nodes.length; i++) {
            RevisionExplorerInfo.InfoNode infoNode = (RevisionExplorerInfo.InfoNode)nodes[i];
            String fileName = infoNode.getFileObject().getPackageNameExt('/','.');
            files[i] = new File(fileSystem.constructRootDirectory(), fileName);
        }
        return files;
    }

    public void actionPerformed(final java.awt.event.ActionEvent e){
        String cmd = e.getActionCommand();
        Node[] nodes = getActivatedNodes();
        JavaCvsFileSystem fileSystem = null;
        FileObject fo = null;
        if (nodes.length == 0) {
            //E.err("internal error nodes.length<1 TODO");
            return ;
        } else {
            for (int i = 0; i < nodes.length; i++) {
                if (nodes[i] instanceof RevisionNode) {
                    JavaCvsFileSystem nodeFS = (JavaCvsFileSystem) ((RevisionNode) nodes[i]).getFileSystem();
                    FileObject nodeFO = ((RevisionNode) nodes[i]).getFileObject();
                    if (fileSystem == null) fileSystem = nodeFS;
                    else if (fileSystem != nodeFS) return ;
                    if (fo == null) fo = nodeFO;
                    else if (fo != nodeFO) return ;
                }
            }
            if (fileSystem == null) return ;
        }
        
        ClientProvider provider = fileSystem.createClientProvider();
        File[] files = getFiles(nodes, fileSystem);
        if (cmd.equals("RevisionExplorerAction.update")) {
            RevisionExplorerInfo.InfoNode infoNode = (RevisionExplorerInfo.InfoNode)nodes[0];
            CvsUpdate command = new CvsUpdate(files, provider);
            command.setLoadDefaultSwitches(false);
            command.setFixedDisplayType(true);
            command.setParamInput(new ParserParamInput("-r " + 
                                         infoNode.getItem().getRevision()));
            command.startCommand();
        }
        if (cmd.equals("RevisionExplorerAction.diff")) {
            RevisionExplorerInfo.InfoNode infoNode = (RevisionExplorerInfo.InfoNode)nodes[0];
            CvsDiff command = new CvsDiff(files, provider);
            command.setLoadDefaultSwitches(false);
            command.setFixedDisplayType(true);
            command.setParamInput(new ParserParamInput("-r " + 
                                         infoNode.getItem().getRevision()));
            command.startCommand();
        }
        if (cmd.equals("RevisionExplorerAction.merge")) {
            RevisionExplorerInfo.InfoNode infoNode = (RevisionExplorerInfo.InfoNode)nodes[0];
            CvsUpdate command = new CvsUpdate(files, provider);
            command.setLoadDefaultSwitches(false);
            command.setFixedDisplayType(true);
            command.setParamInput(new ParserParamInput("-j " + 
                                         infoNode.getItem().getRevision()));
            command.startCommand();
        }        
        if (cmd.equals("RevisionExplorerAction.merge2Rev")) {
            RevisionExplorerInfo.InfoNode infoNode = (RevisionExplorerInfo.InfoNode)nodes[0];
            RevisionExplorerInfo.InfoNode infoNode2 = (RevisionExplorerInfo.InfoNode)nodes[1];
            CvsUpdate command = new CvsUpdate(files, provider);
            command.setLoadDefaultSwitches(false);
            command.setFixedDisplayType(true);
            command.setParamInput(new ParserParamInput("-j " + 
                                         infoNode.getItem().getRevision() + 
                                         " -j " + infoNode2.getItem().getRevision()));
            command.startCommand();
        }        
        if (cmd.equals("RevisionExplorerAction.diff2Rev")) {
            RevisionExplorerInfo.InfoNode infoNode = (RevisionExplorerInfo.InfoNode)nodes[0];
            RevisionExplorerInfo.InfoNode infoNode2 = (RevisionExplorerInfo.InfoNode)nodes[1];
            CvsDiff command = new CvsDiff(files, provider);
            command.setLoadDefaultSwitches(false);
            command.setFixedDisplayType(true);
            command.setParamInput(new ParserParamInput("-r " + 
                                         infoNode.getItem().getRevision() +
                                        " -r " + infoNode2.getItem().getRevision()));
            command.startCommand();
        }
    }
    
    
    private String g(String s) {
        return NbBundle.getBundle
        ("org.netbeans.modules.cvsclient.commands.log.Bundle").getString (s);
    }
    
    private void assignHelp (JMenuItem item, String commandName) {
        HelpCtx.setHelpIDString (item, RevisionExplorerAction.class.getName () + "." + commandName); // NOI18N
    }
}
