/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient.commands.log;

import org.netbeans.modules.vcscore.revision.*;
import org.netbeans.modules.cvsclient.commands.CommandDisplayer;
import org.netbeans.modules.cvsclient.commands.ClientProvider;
import org.netbeans.modules.cvsclient.JavaCvsFileSystem;
import org.netbeans.lib.cvsclient.command.log.LogInformation;
import java.util.*;

import org.openide.util.actions.SystemAction;
import org.openide.cookies.OpenCookie;
import org.openide.nodes.*;
import org.openide.actions.*;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import java.io.File;



/**
 * Displayer for the log command that executes the RevisionExplorer.
 * @author  mkleint
 */
public class RevisionExplorerInfo extends Object implements CommandDisplayer {

    private LinkedList nodeList;
    private JavaCvsFileSystem fSystem;
    private ClientProvider provider;
    /** Creates new RevisionExplorerInfo */
    public RevisionExplorerInfo(JavaCvsFileSystem fs, ClientProvider provider) {
        nodeList = new LinkedList();
        fSystem = fs;
        this.provider = provider;
    }

    /** in this method the displayer should use the data returned by the command to
     * produce it's own data structures/ fill in UI components
     * @param resultList - the data from the command. It is assumed the Displayer 
     * knows what command the data comes from and most important in what format. 
     * (which FileInfoContainer class is used).
 */
    public void setDataToDisplay(LogInformation info) {
            RevisionList revList = new RevisionList();
            LinkedList list = new LinkedList();
            revList.setFileSystem(fSystem);
            FileObject fObject = findFO(info.getFile());
            revList.setFileObject(fObject);
            // setup node.
            InfoNode node = new InfoNode(revList);
            String name = revList.getFileObject().getNameExt();
            Set setFO = new HashSet();
            setFO.add(revList.getFileObject());
            name = fSystem.getStatus().annotateName(name, setFO);
            node.setRootDisplay(revList.getFileObject().getNameExt());
            node.setDisplayName(name);
            node.setName(info.getFile().getName());
            node.setFileNode(true);
            java.awt.Image rootIcon;
            try {
                rootIcon = DataObject.find(fObject).getNodeDelegate().getIcon(java.beans.BeanInfo.ICON_COLOR_16x16);
            } catch (DataObjectNotFoundException exc) {
                rootIcon = null;
            }
            // load revisions
            Iterator logListIt = info.getRevisionList().iterator();
            while (logListIt.hasNext()) {
                LogInformation.Revision infoRev = (LogInformation.Revision)logListIt.next();
                RevisionItem item = new RevisionItem(infoRev.getNumber());
                item.setAuthor(infoRev.getAuthor());
                item.setDate(infoRev.getDate());
                item.setMessage(infoRev.getMessage());
                item.setDisplayName(infoRev.getNumber());
                item.setTagNames(processTags(info.getAllSymbolicNames(), infoRev.getNumber()));
                list.add(item);
                createBranchItems(list, info, infoRev);
            }
            // sort out ..
            Collections.sort(list, Collections.reverseOrder());
            Iterator itList = list.iterator();
            while (itList.hasNext()) {
                RevisionItem itm = (RevisionItem)itList.next();
                revList.add(itm);
            }
            
/*            final FileStatusListener statusListener = new FileStatusListener() {
              public void annotationChanged(FileStatusEvent evt) {
                  if (evt.hasChanged(fo)) {
                      node.setDisplayName(fSystem.annotateName(node.getPath()));
                  }
              }
             };
            fSystem.addFileStatusListener(statusListener);
 */
//            nodeList.add(node);
            // open explorer
            RevisionExplorer.Panel panel = RevisionExplorer.getRevisionExplorer(node, node.getRootDisplay(), rootIcon);
            panel.open(org.openide.TopManager.getDefault().getWindowManager().getCurrentWorkspace());
            
    }
    
    private String[] processTags(List lst, String revision) {
        List tags = new LinkedList();
        Iterator it = lst.iterator();
        while (it.hasNext()) {
            LogInformation.SymName tag = (LogInformation.SymName)it.next();
            if (tag.getRevision().equals(revision)) {
                tags.add(tag.getName());
            }
        }
        String[] toReturn = new String[tags.size()];
        toReturn = (String[])tags.toArray(toReturn);
        return toReturn;
    }
    
    private FileObject findFO(File file) {
        String path = file.getAbsolutePath();
        String fsRoot = fSystem.constructRootDirectory();
        path = path.substring(fsRoot.length());
        path = path.replace('\\','/');
        if (path.startsWith("/")) path = path.substring(1);
        FileObject topFO = fSystem.findResource(path);
//        System.out.println("FO=" + topFO.getPackageNameExt('/', '.'));
        return topFO;
    }
    
    private void createBranchItems(List addToList, LogInformation info, LogInformation.Revision revInfo) {
        String pattern = revInfo.getNumber() + ".0.";
        Iterator it = info.getAllSymbolicNames().iterator();
        while (it.hasNext()) {
            LogInformation.SymName name = (LogInformation.SymName)it.next();
            if (name.getRevision().startsWith(pattern)) {
                String idName = name.getRevision();
                int lastInd = idName.lastIndexOf(".0.");
                idName = idName.substring(0, lastInd) + 
                               idName.substring(lastInd + 2, idName.length());
                RevisionItem rev = new RevisionItem(idName);
                rev.setDisplayName(rev.getRevision() + " (" + name.getName() + ")");
                String[] tagNames = new String[1];
                tagNames[0] = name.getName();
                rev.setTagNames(tagNames);
                addToList.add(rev);
            }
            else {
                String number = revInfo.getNumber();
                String revString = name.getRevision();
                if (revString.startsWith(number) && number.length() < revString.length()) {
                    String remains = revString.substring(number.length());
                    if (remains.lastIndexOf('.') == 0) {
                        RevisionItem rev = new RevisionItem(revString);
                        rev.setDisplayName(rev.getRevision() + " (" + name.getName() + ")");
                        String[] tagNames = new String[1];
                        tagNames[0] = name.getName();
                        rev.setTagNames(tagNames);
                        addToList.add(rev);
                    }
                }
            }
        }
    }
        
    
    /** Does the actual display - docking into the javacvs Mode, 
     *  displaying as single Dialog.. whatever.
 */
    public void displayOutputData(int moment, Object data) {
        if (moment == CommandDisplayer.MOMENT_FILE_INFO_GENERATED) {
            LogInformation info = (LogInformation)data;
            setDataToDisplay(info);
        }
    }
    
    public class InfoNode extends RevisionNode {
        private  String rootDisplay = "";
        private FileObject FObj;
        public InfoNode(RevisionList list) {
            super(list);
            FObj = list.getFileObject();
        }
        
        public InfoNode(RevisionChildren children) {
            super(children);
        }
        
        public InfoNode(RevisionList list, RevisionItem item) {
            super(list, item);
            FObj = list.getFileObject();
        }
        
        public String getPath() {
            return getFileObject().getPackageNameExt('/','.');
        }
        
        public void setRootDisplay(String rootDisp) {
            rootDisplay = rootDisp;
        }

        public String getRootDisplay() {
            return rootDisplay;
        }
        
        public void open() {
            JavaCvsFileSystem fs = (JavaCvsFileSystem)getFileSystem();
            RevisionExplorerAction.openAction(getItem().getRevision(), fs, getFileObject());
        }
        
        public SystemAction[] getActions() {
            ArrayList actions = new ArrayList();
            JavaCvsFileSystem fs = (JavaCvsFileSystem)getFileSystem();
            if (isFileNode()) {
                SystemAction[] foActions = fs.getActions(Collections.singleton(FObj));
                if (foActions != null) {
                    for(int i = 0; i < foActions.length; i++) actions.add(foActions[i]);
                }
                actions.add(null);
                actions.add(SystemAction.get(PropertiesAction.class));
            } else {
                actions.add(SystemAction.get(OpenAction.class));
                actions.add(null);
                actions.add(SystemAction.get(RevisionExplorerAction.class));
                actions.add(null);
                actions.add(SystemAction.get(PropertiesAction.class));
            }
            SystemAction[] array = new SystemAction [actions.size()];
            actions.toArray(array);
            return array;
        }
        
        public RevisionNode createInstance(RevisionList list,RevisionItem item) {
            return new InfoNode(list, item);
        }
        
        public RevisionNode createInstance(RevisionChildren children) {
            return new InfoNode(children);
        }
        
    }
    
}
