/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */



package org.netbeans.modules.cvsclient.commands.remove;


import org.openide.filesystems.*;

import org.openide.util.*;
import java.io.*;
import java.util.*;

import org.netbeans.modules.cvsclient.*;
import org.netbeans.modules.cvsclient.commands.*;
import org.netbeans.lib.cvsclient.event.*;
import org.netbeans.lib.cvsclient.command.remove.*;
import org.netbeans.lib.cvsclient.command.*;
import org.netbeans.modules.cvsclient.caching.*;

import org.netbeans.modules.vcscore.util.Debug;

/** filesystem's idea of the cvs remove command
 *
 * @author  mkleint
 * @version
 */
public class CvsRemove extends CacheUpdatingFsCommand {
    private Debug E=new Debug("CvsRemove", true); // NOI18N
    private Debug D=E;
    
    RemoveCommand command;
    
    /** Creates new Status */
    public CvsRemove(File[] files, ClientProvider fs) {
        super(fs);
        if (fs.isFileSystemClient()) {
            JavaCvsFileSystem fileSys = (JavaCvsFileSystem)fs;
            setFileSystem(fileSys);
        }
        setFiles(files);
        D.deb("Constructor");
    }
    
    
    
    public String getName() {
        return NbBundle.getBundle(CvsRemove.class).getString("CvsRemove.name");
    }
    
/** if no param input was set before staring the command, 
 * the default defined in this method is used
 */
    protected CommandParamInput createDefaultParamInput() {
        return new RemoveParamInput();
    }
    
    protected void initCommand() {
        // here the the commands is initiated just right before running the command
        D.deb("initCommand()");
        int dirNum = 0;
        command = (RemoveCommand)loadCommand(RemoveCommand.class);
        command.setFiles(getFiles());
        toDoCommands.addElement(command);
        super.initCommand();
    }
    
    protected void finishedCommand() {
        // update cache..
        fireUpdateCache();        
        super.finishedCommand();
    }
    
     
    protected Class getDefaultDisplayerType() {
        return RemoveInfoPanel.class;
    }
    protected void applyInputParams() {
        if (getDisplayerType() == RemoveInfoPanel.class) {
            setOutputDisplayer(new RemoveInfoPanel(this));
        }
        super.applyInputParams();
    }   
    
    /**
     * Called when file status information has been received
     */
    public void fileInfoGenerated(FileInfoEvent e) {
        D.deb("fileUpdated()");
        FileInfoContainer fInfo = e.getInfoContainer();
        if (fInfo != null) {
            if (fInfo instanceof RemoveInformation) {
                RemoveInformation info = (RemoveInformation)fInfo;
                updateCache(info);
            }
        }
        super.fileInfoGenerated(e);
    }
    
    /** if no displayer was set by the setOutputDisplayer() method,
     * a default one will be generated by the subclasses implementation
     *
     */
    protected CommandDisplayer createDefaultDisplayer() {
        // just one panel is generated for the whole command.. you can then add info while it's being generated
        return new RemoveInfoPanel(this);
    }
    
}
