/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient.commands.remove;

import org.openide.util.*;
import org.netbeans.lib.cvsclient.command.remove.RemoveInformation;
import org.netbeans.modules.cvsclient.commands.*;
import javax.swing.event.TableModelEvent;

import org.netbeans.modules.vcscore.util.Debug;
import java.util.*;

/**
 *
 * @author  mkleint
 */
public class RemoveTableInfoModel extends TableInfoModel {
    private Debug E=new Debug("RemoveTableInfoModel", true); // NOI18N
    private Debug D=E;
    
    private String IS_DIRECTORY;
    private String IS_FILE;
    private String TYPE_LABEL;
    private String FILE_LABEL;
    private String NOT_REMOVED;
    
      public RemoveTableInfoModel() {
          super();
          NOT_REMOVED = NbBundle.getBundle(RemoveTableInfoModel.class).getString("RemoveTableInfoModel.notRemoved");
          IS_FILE = NbBundle.getBundle(RemoveTableInfoModel.class).getString("RemoveTableInfoModel.file");
          FILE_LABEL = NbBundle.getBundle(TableInfoModel.class).getString("TableInfoModel.fileName");
          TYPE_LABEL = NbBundle.getBundle(RemoveTableInfoModel.class).getString("RemoveTableInfoModel.type");
      }
      
      public int getColumnCount() {
          return 2;
      }
      
      public Object getValueAt(int row, int column) {
//          if (row < 0 || row >= getRowCount()) return "";
          RemoveInformation info = (RemoveInformation)list.get(row);
          switch (column) {
              case 0 : return getType(info);
              case 1 : return info.getFile().getName();
          }
          return "";
      }
      
      private String getType(RemoveInformation info) {
          if (!info.isRemoved())       return NOT_REMOVED;
          if (info.isRemoved())        return IS_FILE;
          return "";
      }    
      
      public java.lang.String getColumnName(int param) {
          switch (param) {
              case 1 : return FILE_LABEL;
              case 0 : return TYPE_LABEL;
          }
          return "";
          
      }
      
      
      public int compare(Object obj1, Object obj2) {
          if (!(obj1 instanceof RemoveInformation) ||
          !(obj2 instanceof RemoveInformation)) return 0;
          RemoveInformation rev1 = (RemoveInformation)obj1;
          RemoveInformation rev2 = (RemoveInformation)obj2;
          int result = 0;
          switch (getActiveColumn()) {
              case 1: // filesname
                  result = getDirection() * (rev1.getFile().getName().compareTo(rev2.getFile().getName()));
                  break;
              case 0:     //status
                  result = getDirection() * (getType(rev1).compareTo(getType(rev2)));
                  break;
          }
          return result;
      }
      
      public void addElement(Object object)
      {
          super.addElement(object);
          fireTableRowsInserted(list.size(), list.size());
      }      
      
}

