/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient.commands.status;


import org.openide.util.*;
import java.io.*;
import java.util.*;
import java.awt.Dialog;
import java.awt.BorderLayout;

import org.netbeans.lib.cvsclient.event.*;
import org.netbeans.lib.cvsclient.command.*;
import org.netbeans.lib.cvsclient.command.status.*;
import org.netbeans.lib.cvsclient.file.FileStatus;
import org.netbeans.modules.cvsclient.JavaCvsStatusManager;

import org.netbeans.modules.cvsclient.*;
import org.netbeans.modules.cvsclient.commands.*;
import org.netbeans.modules.vcscore.util.Debug;
import org.netbeans.modules.vcscore.cache.*;
import org.netbeans.modules.cvsclient.caching.*;

/**
 *
 * @author  mkleint
 * @version
 */
public class CvsStatus extends CacheUpdatingFsCommand {
    private Debug E=new Debug("CvsStatus", true); // NOI18N
    private Debug D=E;
    
    StatusCommand command;
    
    /** Creates new Status */
    public CvsStatus(File[] filesToCheck, ClientProvider fs) {
        super(fs);
/*        if (fs.isFileSystemClient()) {
            JavaCvsFileSystem fileSys = (JavaCvsFileSystem)fs;
            setFileSystem(fileSys);
        }
 */
        setFiles(filesToCheck);
        D.deb("Constructor");
    }
    
    public CvsStatus() {
        super();
    }
    
    public String getName() {
        return NbBundle.getBundle(CvsStatus.class).getString("CvsStatus.name");
    }

/** if no param input was set before staring the command, 
 * the default defined in this method is used
 */
    protected CommandParamInput createDefaultParamInput() {
        return new StatusParamInput();
    }

    
    protected void initCommand() {
        // here the the commands is initiated just right before running the command
        D.deb("initCommand()");
        int dirNum = 0;
//        CvsStatusCommand command;        
        StatusCommand command;
        File[] files = new File[getFiles().length];
        System.arraycopy(getFiles(),0,files,0, getFiles().length);
        
        for (int index = 0; index < files.length; index++) {
            if (files[index].isDirectory()) {
                // each directory status command is sent separately..
                //that way just one UI component is run after finish of the command
                command = (StatusCommand) loadCommand(StatusCommand.class);
//                command.setDirectoryCheck(true);
                File[] fls = new File[1];
                fls[0] = files[index];
                command.setFiles(fls);
                toDoCommands.addElement(command);
                files[index] = null;
                dirNum = dirNum + 1;
            }
        }
        // now proceed all single files in one batch.
        if (dirNum == files.length) {
            return; // no files to check status on.
        }
        File[] singleFiles = new File[files.length - dirNum];
        int singleIndex = 0;
        for (int ind = 0; ind < files.length; ind++) {
            if (files[ind] != null) {
                singleFiles[singleIndex] = files[ind];
                singleIndex = singleIndex + 1;
            }
        }
        command = (StatusCommand) loadCommand(StatusCommand.class);
//        command.setDirectoryCheck(false);
        command.setFiles(singleFiles);
        toDoCommands.addElement(command);
        // just add oen command to the queue
        super.initCommand();
    }
    
    protected void finishedCommand() {
        // called when the communication with server ends and the command thread is about to end..
        // used for calling visualizer stuff..
        fireUpdateCache();
        super.finishedCommand();
    }
    
    protected void beforeEachExecute() {
        clearResultList();
        if (getDisplayerType() == StatusTreeInfoPanel.class) {
            StatusCommand statCom = (StatusCommand)getCurrentCommand();
            File[] flz = statCom.getFiles();
            if (flz[0].isDirectory()) {
                File[] topDir = statCom.getFiles();
                StatusTreeInfoPanel treePanel = (StatusTreeInfoPanel)FileSystemCommand.findOpenDisplayer(
                flz[0], StatusTreeInfoPanel.class, null);
                if (treePanel == null) {
                    treePanel = new StatusTreeInfoPanel(topDir[0], statCom, getClientProvider());
                    treePanel.setFileSystemCommand(this);
                } else {
                    treePanel.setCommand(statCom);
                }
                setOutputDisplayer(treePanel);
            }
        }
        super.beforeEachExecute();
    }
        
    protected void afterEachExecute() {
        super.afterEachExecute();
        if (getDisplayerType() == StatusTreeInfoPanel.class ||
            getDisplayerType() == StatusInfoPanel.class) {
            setOutputDisplayer(null);
        }
    }
    
    protected void executeFailed(Exception exc) {
        fireUpdateCache();
        stopCommand();
        super.executeFailed(exc);
    }
    
    
    /**
     * Called when file status information has been received
     */
    public void fileInfoGenerated(FileInfoEvent e) {
        StatusInformation info = (StatusInformation)e.getInfoContainer();
        if (info != null) {
            updateCache(info);
        }
        // kinda hack - don't know currently how to recreate a panel for each file otherwise.
        if (getDisplayerType() == StatusInfoPanel.class) {
           StatusCommand statCom = (StatusCommand)getCurrentCommand();
           CommandDisplayer toReturn;
           File fl = info.getFile();
           // find or create statusInfoPanel
           StatusInfoPanel panel = (StatusInfoPanel)FileSystemCommand.findOpenDisplayer(
                                            fl, StatusInfoPanel.class, null);
           if (panel == null) {
               panel = new StatusInfoPanel(getClientProvider(), statCom);
               panel.setFileSystemCommand(this);
           } else {
               panel.setCommand(statCom);
           }
           setOutputDisplayer(panel);
        }
        super.fileInfoGenerated(e);
    }

    /** if no displayer was set by the setOutputDisplayer() method,
     * a default one will be generated by the subclasses implementation
     *
     */
    
    protected Class getDefaultDisplayerType() {
       StatusCommand statCom = (StatusCommand)getCurrentCommand();
       File[] flz = statCom.getFiles();
       if (flz[0].isDirectory()) return StatusTreeInfoPanel.class;
       return StatusInfoPanel.class;
        
    }

    /*
    protected CommandDisplayer createDefaultDisplayer() {
       StatusCommand statCom = (StatusCommand)getCurrentCommand();
       CommandDisplayer toReturn;
       File[] flz = statCom.getFiles();
       if (flz[0].isDirectory()) {
           File[] topDir = statCom.getFiles();
           StatusTreeInfoPanel treePanel = (StatusTreeInfoPanel)FileSystemCommand.findOpenDisplayer(
                                            flz[0], StatusTreeInfoPanel.class, null);
           if (treePanel == null) {
               treePanel = new StatusTreeInfoPanel(topDir[0], statCom, getClientProvider());
           } else {
               treePanel.setProvider(getClientProvider());
               treePanel.setCommand(statCom);
           }
           toReturn = treePanel;
           
       } else {
           // find or create statusInfoPanel
           StatusInfoPanel panel = (StatusInfoPanel)FileSystemCommand.findOpenDisplayer(
                                            flz[0], StatusInfoPanel.class, null);
           if (panel == null) {
               panel = new StatusInfoPanel(getClientProvider(), statCom);
           } else {
               panel.setProvider(getClientProvider());
               panel.setCommand(statCom);
           }
           toReturn = panel;
       }
       return toReturn;
    }

     */
    
    /** overrides the FileSystembehaviour. Disabling the getDisplayer() call 
     * in order to decide about the appropriate Displayer at later point.
     * (getCurrentCommand() is needed in the createDefaultDisplayer() decition making.
     */
    protected void applyInputParams() {
        Iterator it = toDoCommands.iterator();
        // for all commands queued in the toDoCommand list,
        // apply the command line params to them (from the dialog)
        while (it.hasNext()) {
            Command sCom = (Command)it.next();
            getParamInput().setData(sCom);
        }
    }
    
}
