/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */



package org.netbeans.modules.cvsclient.commands.tag;


import org.openide.util.*;
import java.io.*;
import java.util.*;
import java.awt.Dialog;

import org.netbeans.modules.cvsclient.*;
import org.netbeans.modules.cvsclient.commands.*;
import org.netbeans.modules.cvsclient.commands.update.UpdateInfoPanel;

import org.netbeans.lib.cvsclient.event.*;
import org.netbeans.lib.cvsclient.command.tag.*;
import org.netbeans.lib.cvsclient.command.*;
import org.netbeans.modules.cvsclient.caching.*;

import org.netbeans.modules.vcscore.util.Debug;

/**
 *  The filesystem command class for the cvs tag command.
 * @author  mkleint
 */
public class CvsTag extends CacheUpdatingFsCommand {
    private Debug E=new Debug("CvsTag", true); // NOI18N
    private Debug D=E;
    
    TagCommand command;
    
    /** Creates new Status */
    public CvsTag(File[] filesToUpdate, ClientProvider fs) {
        super(fs);
        if (fs.isFileSystemClient()) {
            JavaCvsFileSystem fileSys = (JavaCvsFileSystem)fs;
            setFileSystem(fileSys);
        }
        setFiles(filesToUpdate);
    }
    
    public String getName() {
        return NbBundle.getBundle(CvsTag.class).getString("CvsTag.name");
    }

/** if no param input was set before staring the command, 
 * the default defined in this method is used
 */
    protected CommandParamInput createDefaultParamInput() {
        return new TagParamInput();
    }
    
    protected void initCommand() {
        // here the the commands is initiated just right before running the command
        D.deb("initCommand()");
        int dirNum = 0;
        command = (TagCommand) loadCommand(TagCommand.class);
        command.setFiles(getFiles());
        getParamInput().getData(command);
        toDoCommands.addElement(command);
        super.initCommand();
    }

    protected Class getDefaultDisplayerType() {
        return UpdateInfoPanel.class;
    }
    
    protected void applyInputParams() {
        super.applyInputParams();
        if (getDisplayerType() == UpdateInfoPanel.class) {
           // just one panel is generated for the whole command.. you can then add info while it's being generated
           setOutputDisplayer(new UpdateInfoPanel(getClientProvider(), this));
           getDisplayer().displayOutputData(CommandDisplayer.MOMENT_AFTER_PARAM_INPUT, null);
        }
    }
    
    protected void finishedCommand() {
        // called when the communication with server ends and the command thread is about to end..
        // used for calling visualizer stuff..
        fireUpdateCache();
        super.finishedCommand();
    }
    
    protected void executeFailed(Exception exc) {
        // do refresh.. refresh or recursive refresh, depending on getRecursive()
        // TODO needs some optimizing..
        fireUpdateCache();        
        super.executeFailed(exc);
    }
    
    
    /**
     * Called when file status information has been received
     */
    public void fileInfoGenerated(FileInfoEvent e) {
        D.deb("fileUpdated()");
        FileInfoContainer fInfo = e.getInfoContainer();
        if (fInfo != null) {
            if (fInfo instanceof DefaultFileInfoContainer) {
                DefaultFileInfoContainer info = (DefaultFileInfoContainer)fInfo;
                updateCache(info);
            }
        }
        super.fileInfoGenerated(e);
    }
    
    /** if no displayer was set by the setOutputDisplayer() method, 
 * a default one will be generated by the subclasses implementation
 *
 */
    /*
    protected CommandDisplayer createDefaultDisplayer() {
       // just one panel is generated for the whole command.. you can then add info while it's being generated
       return new UpdateInfoPanel(getClientProvider(), this);
    } 
     */   
}
