/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */



package org.netbeans.modules.cvsclient.commands.update;


import org.openide.util.*;
import java.io.*;
import java.util.*;
import java.awt.Dialog;
import org.openide.*;

import org.netbeans.modules.cvsclient.*;
import org.netbeans.modules.cvsclient.commands.*;
import org.netbeans.lib.cvsclient.event.*;
import org.netbeans.lib.cvsclient.command.update.*;
import org.netbeans.lib.cvsclient.command.DefaultFileInfoContainer;
import org.netbeans.lib.cvsclient.command.checkout.*;
import org.netbeans.lib.cvsclient.command.*;
import org.netbeans.modules.cvsclient.caching.*;

import org.netbeans.modules.vcscore.util.Debug;

/**
 *
 * @author  mkleint
 * @version
 */
public class CvsUpdate extends CacheUpdatingFsCommand {
    private Debug E=new Debug("CvsUpdate", true); // NOI18N
    private Debug D=E;
    
    UpdateCommand command;
    
    /** Creates new Status */
    public CvsUpdate(File[] filesToUpdate, ClientProvider fs) {
        super(fs);
        if (fs.isFileSystemClient()) {
            JavaCvsFileSystem fileSys = (JavaCvsFileSystem)fs;
            setFileSystem(fileSys);
        }
        setFiles(filesToUpdate);
        D.deb("Constructor");
    }
    
    public String getName() {
        return NbBundle.getBundle(CvsUpdate.class).getString("CvsUpdate.name");
    }

/** if no param input was set before staring the command, 
 * the default defined in this method is used
 */
    protected CommandParamInput createDefaultParamInput() {
        return new UpdateParamInput();
    }
    
    protected void initCommand() {
        // here the the commands is initiated just right before running the command
        D.deb("initCommand()");
        int dirNum = 0;
        command = (UpdateCommand) loadCommand(UpdateCommand.class);
        command.setFiles(getFiles());
        getParamInput().getData(command);
        toDoCommands.addElement(command);
        super.initCommand();
    }
    
    protected Class getDefaultDisplayerType() {
        if (command != null && command.isPipeToOutput()) {
            return UpdateStdOutPanel.class;
        }
        return UpdateInfoPanel.class;
    }
    
    protected void applyInputParams() {
        super.applyInputParams();
        File[] fls = getFiles();
        boolean hasDirectories = false;
        for (int i = 0; i < fls.length; i++) {
            if (fls[i].isDirectory()) {
                hasDirectories = true;
                break;
            }
        }
        UpdateCommand comm = (UpdateCommand)toDoCommands.get(0);        
        if (hasDirectories && comm.isPipeToOutput() &&
            getClientProvider().getDisplayType() == JavaCvsFileSystem.DISP_TYPE_FORMATTED) {
            String message = NbBundle.getBundle(CvsUpdate.class).getString("CvsUpdate.pipeOutputQuestion");
            NotifyDescriptor nd = new NotifyDescriptor.Confirmation(message, NotifyDescriptor.YES_NO_OPTION);
            Object returnValue = TopManager.getDefault().notify(nd);
            //        System.out.println("return value=" + returnValue.getClass().getName() + " " + returnValue.toString());
            if (NotifyDescriptor.Confirmation.NO_OPTION.equals(returnValue)) {
                hardCommandStop();
            }
        }
        if (getDisplayerType() == UpdateInfoPanel.class) {
           // just one panel is generated for the whole command.. you can then add info while it's being generated
           setOutputDisplayer(new UpdateInfoPanel(getClientProvider(), this));
           getDisplayer().displayOutputData(CommandDisplayer.MOMENT_AFTER_PARAM_INPUT, null);
        }
    }
    
    protected void finishedCommand() {
        // called when the communication with server ends and the command thread is about to end..
        // used for calling visualizer stuff..
        fireUpdateCache();
        super.finishedCommand();
    }
    
    protected void executeFailed(Exception exc) {
        // do refresh.. refresh or recursive refresh, depending on getRecursive()
        // TODO needs some optimizing..
        fireUpdateCache();        
        super.executeFailed(exc);
    }
    
    
    /**
     * Called when file status information has been received
     */
    public void fileInfoGenerated(FileInfoEvent e) {
        D.deb("fileUpdated()");
        FileInfoContainer fInfo = e.getInfoContainer();
        if (fInfo != null) {
            if (fInfo instanceof DefaultFileInfoContainer) {
                DefaultFileInfoContainer info = (DefaultFileInfoContainer)fInfo;
                updateCache(info);
            }
            else if (fInfo instanceof PipedFileInformation) {
                if (getDisplayerType() == UpdateStdOutPanel.class) {
                    UpdateCommand statCom = (UpdateCommand)getCurrentCommand();
                    CommandDisplayer toReturn;
                    PipedFileInformation info = (PipedFileInformation)fInfo;
                    File fl = info.getFile();
               // find or create statusInfoPanel
                    UpdateStdOutPanel panel = (UpdateStdOutPanel)FileSystemCommand.findOpenDisplayer(
                                                    fl, UpdateStdOutPanel.class, info);
                    if (panel == null) {
                        panel = new UpdateStdOutPanel(statCom);
                    } else {
                        panel.setCommand(statCom);
                    }
                    panel.setFileSystemCommand(this);                    
                    setOutputDisplayer(panel);
                }
            }
        }
        super.fileInfoGenerated(e);
    }
    
    /** if no displayer was set by the setOutputDisplayer() method, 
 * a default one will be generated by the subclasses implementation
 *
 */
/*    protected CommandDisplayer createDefaultDisplayer() {
       if (command.isPipeToOutput()) {
          UpdateCommand comm = (UpdateCommand)getCurrentCommand();
          return new UpdateStdOutPanel(comm);
       }
       // just one panel is generated for the whole command.. you can then add info while it's being generated
       return new UpdateInfoPanel(getClientProvider(), this);
    }    
 */
}
