/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient.customizer;


import org.openide.util.*;
import org.openide.*;

import javax.swing.event.*;
import javax.swing.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import org.netbeans.modules.cvsclient.passwd.*;
import org.netbeans.modules.cvsclient.JavaCvsFileSystem;
import org.netbeans.modules.cvsclient.util.*;
import org.netbeans.lib.cvsclient.connection.AuthenticationException;

/** Panel in the JavaCvsCustomizer which takes care of CVS server settings.
 * After the user fills in all info needed and the type of server is pserver, looks in the .cvspass file for settings.
 * If not found, it attempts to connect to the pserver and prompts for password.
 * If successful, the current settings are written to the .cvspass file.
 * @author mkleint
 * @version 1.0
 */
public class Cust2ServerPanel extends javax.swing.JPanel implements  WizardDescriptor.Panel{
    private Debug E=new Debug("Cust2ServerPanel", true); // NOI18N
    private Debug D=new Debug("Cust2ServerPanel", true); // NOI18N

    JavaCvsFileSystem fs;
    CVSPasswd passFile;
    
    private boolean allowNext = false;
    
    private ArrayList listeners = new ArrayList ();
    /** Creates new form Cust2Server */
    public Cust2ServerPanel() {
        initComponents ();
        lblServerType.setDisplayedMnemonic (java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/customizer/Bundle").getString("Cust2ServerPanel.lblServerType.mnemonic").charAt (0));
        lblServerType.setLabelFor (cbServerType);
        lbServer.setDisplayedMnemonic (java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/customizer/Bundle").getString("JavaCvsCustomizer.lbServer.mnemonic").charAt (0));
        lbServer.setLabelFor (txServer);
        lbRepository.setDisplayedMnemonic (java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/customizer/Bundle").getString("JavaCvsCustomizer.lbRepository.mnemonic").charAt (0));
        lbRepository.setLabelFor (txRepository);
        lbUser.setDisplayedMnemonic (java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/customizer/Bundle").getString("JavaCvsCustomizer.lbUser.mnemonic").charAt (0));
        lbUser.setLabelFor (txUser);
        lblCvsRoot.setDisplayedMnemonic (java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/customizer/Bundle").getString("Cust2ServerPanel.lblCvsRoot.mnemonic").charAt (0));
        lblCvsRoot.setLabelFor (txCvsRoot);
        btnLogin.setMnemonic (java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/customizer/Bundle").getString("Cust2ServerPanel.btnLogin.mnemonic").charAt (0));

        passFile = new CVSPasswd();  // should suit all OS
        passFile.loadPassFile();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        cbServerType = new javax.swing.JComboBox();
        txServer = new javax.swing.JTextField();
        txRepository = new javax.swing.JTextField();
        txUser = new javax.swing.JTextField();
        lblServerType = new javax.swing.JLabel();
        lbServer = new javax.swing.JLabel();
        lbRepository = new javax.swing.JLabel();
        lbUser = new javax.swing.JLabel();
        lblCvsRoot = new javax.swing.JLabel();
        txCvsRoot = new javax.swing.JTextField();
        jSeparator1 = new javax.swing.JSeparator();
        jPanel1 = new javax.swing.JPanel();
        btnLogin = new javax.swing.JButton();
        txPasswd = new javax.swing.JTextField();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        setPreferredSize(new java.awt.Dimension(450, 325));
        setMinimumSize(new java.awt.Dimension(450, 325));
        cbServerType.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cbServerTypeActionPerformed(evt);
            }
        });
        
        cbServerType.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                cbServerTypeFocusLost(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 12, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(cbServerType, gridBagConstraints1);
        
        txServer.setColumns(20);
        txServer.setMinimumSize(new java.awt.Dimension(100, 20));
        txServer.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                txServerFocusLost(evt);
            }
        });
        
        txServer.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                txServerKeyReleased(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(12, 12, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(txServer, gridBagConstraints1);
        
        txRepository.setColumns(20);
        txRepository.setMinimumSize(new java.awt.Dimension(100, 20));
        txRepository.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                txRepositoryFocusLost(evt);
            }
        });
        
        txRepository.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                txRepositoryKeyReleased(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(5, 12, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(txRepository, gridBagConstraints1);
        
        txUser.setPreferredSize(new java.awt.Dimension(160, 20));
        txUser.setMinimumSize(new java.awt.Dimension(100, 20));
        txUser.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                txUserFocusLost(evt);
            }
        });
        
        txUser.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                txUserKeyReleased(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(5, 12, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(txUser, gridBagConstraints1);
        
        lblServerType.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/customizer/Bundle").getString("Cust2ServerPanel.lblServerType.text"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.insets = new java.awt.Insets(4, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        add(lblServerType, gridBagConstraints1);
        
        lbServer.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/customizer/Bundle").getString("JavaCvsCustomizer.lbServer.text"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.insets = new java.awt.Insets(12, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(lbServer, gridBagConstraints1);
        
        lbRepository.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/customizer/Bundle").getString("JavaCvsCustomizer.lbRepository.text"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.insets = new java.awt.Insets(5, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(lbRepository, gridBagConstraints1);
        
        lbUser.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/customizer/Bundle").getString("JavaCvsCustomizer.lbUser.text"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.insets = new java.awt.Insets(5, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(lbUser, gridBagConstraints1);
        
        lblCvsRoot.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/customizer/Bundle").getString("Cust2ServerPanel.lblCvsRoot.text"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.insets = new java.awt.Insets(5, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(lblCvsRoot, gridBagConstraints1);
        
        txCvsRoot.setEditable(false);
        txCvsRoot.setDisabledTextColor(javax.swing.UIManager.getColor ("TextField.foreground"));
        txCvsRoot.setPreferredSize(new java.awt.Dimension(200, 20));
        txCvsRoot.setMinimumSize(new java.awt.Dimension(100, 20));
        txCvsRoot.setEnabled(false);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(5, 12, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(txCvsRoot, gridBagConstraints1);
        
        jSeparator1.setPreferredSize(new java.awt.Dimension(40, 5));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 5;
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(12, 0, 0, 0);
        add(jSeparator1, gridBagConstraints1);
        
        jPanel1.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints2;
        
        btnLogin.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/customizer/Bundle").getString("Cust2ServerPanel.btnLogin.text"));
        btnLogin.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnLoginActionPerformed(evt);
            }
        });
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 0;
        gridBagConstraints2.gridy = 0;
        gridBagConstraints2.insets = new java.awt.Insets(12, 0, 0, 0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        jPanel1.add(btnLogin, gridBagConstraints2);
        
        txPasswd.setEditable(false);
        txPasswd.setText("jTextField1");
        txPasswd.setDisabledTextColor(javax.swing.UIManager.getColor ("Label.foreground"));
        txPasswd.setPreferredSize(new java.awt.Dimension(150, 20));
        txPasswd.setEnabled(false);
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 1;
        gridBagConstraints2.gridy = 0;
        gridBagConstraints2.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.insets = new java.awt.Insets(12, 12, 0, 0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints2.weightx = 1.0;
        jPanel1.add(txPasswd, gridBagConstraints2);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(jPanel1, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void txRepositoryKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_txRepositoryKeyReleased
       eventCheckPassword();
       fireChangeState();
        // Add your handling code here:
    }//GEN-LAST:event_txRepositoryKeyReleased

    private void txUserKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_txUserKeyReleased
       eventCheckPassword();
       fireChangeState();
        // Add your handling code here:
    }//GEN-LAST:event_txUserKeyReleased

    private void txServerKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_txServerKeyReleased
       eventCheckPassword();
       fireChangeState();
        // Add your handling code here:
    }//GEN-LAST:event_txServerKeyReleased

  private void txUserFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_txUserFocusLost
// Add your handling code here:
       eventCheckPassword();
       fireChangeState();
  }//GEN-LAST:event_txUserFocusLost

  private void txRepositoryFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_txRepositoryFocusLost
// Add your handling code here:
       eventCheckPassword();
       fireChangeState();
  }//GEN-LAST:event_txRepositoryFocusLost

  private void txServerFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_txServerFocusLost
// Add your handling code here:
       eventCheckPassword();
       fireChangeState();
  }//GEN-LAST:event_txServerFocusLost

  private void cbServerTypeFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_cbServerTypeFocusLost
      // Add your handling code here:
       eventCheckPassword();
       fireChangeState();
  }//GEN-LAST:event_cbServerTypeFocusLost

  private void cbServerTypeActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cbServerTypeActionPerformed
// Add your handling code here:
      btnLogin.setEnabled(cbServerType.getSelectedIndex() + 1 == JavaCvsFileSystem.TYPE_PSERVER);
        // hack.. server was removed and to be backward compatible, ...
      eventCheckPassword();
      fireChangeState();
  }//GEN-LAST:event_cbServerTypeActionPerformed

  private void btnLoginActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnLoginActionPerformed
// Add your handling code here:
      SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                LoginDialog login = LoginDialog.createDialog(getCvsRoot(), passFile);
                login.addWindowListener(new WindowAdapter() {
                    public void windowClosed(WindowEvent e) {
                        eventCheckPassword();
                        fireChangeState();
                    }
                });
                JavaCvsCustomizer.centerWindow(login);
                login.show();
            }
      });
  }//GEN-LAST:event_btnLoginActionPerformed

  private void eventCheckPassword() {
      String root = getCvsRoot();
      txCvsRoot.setText(root);
      int selType = cbServerType.getSelectedIndex() + 1;
        // hack.. server was removed and to be backward compatible, ...
      checkPassword(root,selType);
  }
  
  private String getCvsRoot() {
      int selType = cbServerType.getSelectedIndex() + 1;
        // hack.. server was removed and to be backward compatible, ...
      String root = ":" + fs.getCvsServerTypeName(selType);
      root = root + ":" + txUser.getText().trim() + "@" + txServer.getText().trim();
      root = root + ":" + txRepository.getText().trim();
      return root;
  }

  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JComboBox cbServerType;
  private javax.swing.JTextField txServer;
  private javax.swing.JTextField txRepository;
  private javax.swing.JTextField txUser;
  private javax.swing.JLabel lblServerType;
  private javax.swing.JLabel lbServer;
  private javax.swing.JLabel lbRepository;
  private javax.swing.JLabel lbUser;
  private javax.swing.JLabel lblCvsRoot;
  private javax.swing.JTextField txCvsRoot;
  private javax.swing.JSeparator jSeparator1;
  private javax.swing.JPanel jPanel1;
  private javax.swing.JButton btnLogin;
  private javax.swing.JTextField txPasswd;
  // End of variables declaration//GEN-END:variables

    private static final java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("org/netbeans/modules/cvsclient/customizer/Bundle");    

/** name of panel
 * @return the name of the current panel
 */
    public String getName() {
      return org.openide.util.NbBundle.getBundle(Cust2ServerPanel.class).getString("Cust2ServerPanel.serverDialogLabel");        
    }
    
    
    public void setLast(boolean last) {
     //   isLastPanel = last;
    }
/** last panel check.
 * @return true if last else false.
 */    
    
    
    public boolean isLast() {
//        System.out.println("server is last =" + !iter.isNewSources());
//        return !iter.isNewSources();
        return false;
    }
    
/** If everything is ok, returns the number of the Cust3CheckoutPanel.
 * Otherwise returns it's onw number, so that the user has to stay here or cancel.
 * @return the number of next panel.
 */
    public int getNext() {
        
//        if (iter.isNewSources()) return 2;
       return 3; //.. if not new sources this one is final..
       // this makes the assumption that the server repository has something the user can checkout..
    }
    /** Get the component displayed in this panel.
 * @return the component
 */
    public java.awt.Component getComponent() {
        return this;
    }
    /** Help for this panel.
 * When the panel is active, this is used as the help for the wizard dialog.
 * @return the help or <code>null</code> if no help is supplied
 */
    public HelpCtx getHelp() {
        return null;
    }
    /** Provides the wizard panel with the current data--either
 * the default data or already-modified settings, if the user used the previous and/or next buttons.
 * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
 * @param settings the object representing wizard panel state, as originally supplied to {@link WizardDescriptor#WizardDescriptor(WizardDescriptor.Iterator,Object)}
 */
    public void readSettings(Object settings) {
        JavaCvsCustomizer customizer = (JavaCvsCustomizer)settings;
        if (customizer.isNewSources()) {
           putClientProperty("WizardPanel_contentSelectedIndex", new Integer(1));            
        } else {
           putClientProperty("WizardPanel_contentSelectedIndex", new Integer(2));            
        }
        fs = customizer.getFileSystem();
        cbServerType.setModel(new DefaultComboBoxModel(new String[] {"pserver"}));
        // hack.. server was removed and to be backward compatible, ...
        String st = customizer.getServerType();
        int sType = fs.getCvsServerType(st);
        if (sType >=0) cbServerType.setSelectedIndex(sType - 1 );
        // hack.. server was removed and to be backward compatible, ...
        txServer.setText(customizer.getServerName());
        txUser.setText(customizer.getUserName());
        txRepository.setText(customizer.getRepository());
        String cvsRoot= ":" + st + ":" + customizer.getUserName() + "@" + customizer.getServerName() +
            ":" + customizer.getRepository();
        eventCheckPassword();
        updateEnabled();
    }
    
    private void checkPassword(String root, int serverType) {
        if (serverType == JavaCvsFileSystem.TYPE_PSERVER) {
            PasswdEntry entry = passFile.find(root);
            if (entry != null) {
                txPasswd.setText(NbBundle.getBundle(Cust2ServerPanel.class).getString("Cust2ServerPanel.isLogged"));
                allowNext = true;
            } else {
                txPasswd.setText(NbBundle.getBundle(Cust2ServerPanel.class).getString("Cust2ServerPanel.isNotLogged"));
                allowNext = true;
            }
        } else {
            txPasswd.setText(NbBundle.getBundle(Cust2ServerPanel.class).getString("Cust2ServerPanel.isNotApplicable"));
            allowNext = true;
        }
        
    }
    
    /** Provides the wizard panel with the opportunity to update the
 * settings with its current customized state.
 * Rather than updating its settings with every change in the GUI, it should collect them,
 * and then only save them when requested to by this method.
 * Also, the original settings passed to {@link #readSettings} should not be modified (mutated);
 * rather, the (copy) passed in here should be mutated according to the collected changes.
 * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
 * @param settings the object representing a settings of the wizard
 */
    public void storeSettings(Object settings) {
        JavaCvsCustomizer customizer = (JavaCvsCustomizer)settings;
        fs = customizer.getFileSystem();
        int selType = cbServerType.getSelectedIndex() + 1;
        // hack.. server was removed and to be backward compatible, ...
        customizer.setServerType(fs.getCvsServerTypeName(selType));
        customizer.setServerName(txServer.getText());
        customizer.setUserName(txUser.getText());
        customizer.setRepository(txRepository.getText());
    }
/** Test whether the panel is finished and it is safe to proceed to the next one.
 * If the panel is valid, the "Next" (or "Finish") button will be enabled.
 * @return <code>true</code> if the user has entered satisfactory information
 */
    public boolean isValid() {
        return allowNext;
    }

    /** Add a listener to changes of the panel's validity.
 * @param l the listener to add
 * @see #isValid
 */
    public void addChangeListener(ChangeListener l) {
        if (listeners == null) {
          listeners = new ArrayList();
        }
        listeners.add (l);        
    }
    /** Remove a listener to changes of the panel's validity.
 * @param l the listener to remove
 */
    public void removeChangeListener(ChangeListener l) {
        if (listeners != null) {
            listeners.remove (l);
        }
    }
    
    private void fireChangeState() {
        ArrayList list;
        synchronized (this) {
            list = (ArrayList) this.listeners.clone();
        }
        ChangeEvent event = new ChangeEvent (this);
        for (int i=0; i<list.size(); i++)
            ((ChangeListener)list.get (i)).stateChanged (event);
    }        
    
        
    private void updateEnabled() {
      // in case some server type needs to disable something..  
        
    }    

}
