/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.cvsclient.customizer;

import java.io.*;
import java.util.*;
import java.beans.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import org.openide.*;
import org.openide.util.*;

import org.netbeans.modules.cvsclient.util.Debug;
import org.netbeans.modules.cvsclient.passwd.*;
import org.netbeans.modules.cvsclient.*;
import javax.swing.event.ChangeListener;


/** The CVS filesystem customizer. Descends from WizardDescriptor and implements the Customizer interface.
 * It stores all the data collected and finally applies the changes.
 * After completion, mounts the filesystem.
 * @author Milos Kleint
 * @version 1.0
 */
public class JavaCvsCustomizer extends WizardDescriptor
                               implements Customizer, PropertyChangeListener {
    private Debug E=new Debug("JavaCvsCustomizer", true); // NOI18N
    private Debug D=new Debug("JavaCvsCustomizer", true); // NOI18N
    private javax.swing.ButtonGroup bg;
    private static final String CVS_LOCAL = "local";
    private JavaCvsFileSystem fileSystem=null;
    private PropertyChangeSupport changeSupport=null;
    private EventListenerList listeners;
    
    
    private static CustomIterator iterator;
    
    public  String rootValue = "";
    private  String serverType;
    private  String userName;
    private  String serverName;
    private  String repository;
    private  File workingDir;
    private  String relMount;
    private  boolean newSources;
    
    private boolean isReadyForMount = false;
    
    
/** Creates now instance of customizer, creates the iterator and all panels.
 */
    
    public JavaCvsCustomizer() {
        super(iterator = new CustomIterator());
        this.setTitleFormat(new MessageFormat("{1} - {0}"));
        
        // INIT: turn on auto wizard panel creation (at least subtitle)
        putProperty("WizardPanel_autoWizardStyle", new Boolean(true));
        
        // INIT: turn on diplaying of steps
        // (content, behind which can be displayed image)
        putProperty("WizardPanel_contentDisplayed", new Boolean(true));
        
        // INIT: turn on displaying of help (html browser),
        // if steps are displayed then creates tabbed pane
//        wd.putProperty("WizardPanel_helpDisplayed", new Boolean(true));
        
        // INIT: turn on numbering of steps
        // (before every step is placed it's sequence number)
        putProperty("WizardPanel_contentNumbered", new Boolean(true));
        
        // INIT: set image displayed as background of content
//        wd.putProperty("WizardPanel_image", myImage);
        addPropertyChangeListener (this);
        
    }
    
    
/** returns the filesystem passed to the customizer via setObject() method
 * @return the currently edited CVS filesystem.
 */
    public JavaCvsFileSystem getFileSystem() {
        return fileSystem;
    }
    
    public static String[] setContentData(boolean newSources) {
        // set steps names
       String panel1 = NbBundle.getBundle(JavaCvsCustomizer.class).getString("JavaCvsCustomizer.Panel1.title");
       String panel2 = NbBundle.getBundle(JavaCvsCustomizer.class).getString("JavaCvsCustomizer.Panel2.title");       
       String panel3a = NbBundle.getBundle(JavaCvsCustomizer.class).getString("JavaCvsCustomizer.Panel3a.title");
       String panel3b = NbBundle.getBundle(JavaCvsCustomizer.class).getString("JavaCvsCustomizer.Panel3b.title");
       String[] content;
       if (newSources) {
           content = new String[] {panel1, panel2, panel3b};
       } else {
           content = new String[] {panel1, panel3a, panel2};
       }
       return content;
    }
    
/** Fills the filesystem (data) to the customizer.
 * @param bean the current cvsfilesystem
 */
    public void setObject(final java.lang.Object bean) {
        fileSystem=(JavaCvsFileSystem)bean;
        setServerType(fileSystem.getCvsServerTypeName());
        setUserName(fileSystem.getCvsUserName());
        setServerName(fileSystem.getCvsServerName());
        setRepository(fileSystem.getCvsRepository());
        setRelMount(fileSystem.getRelMount());
        setWorkingDir(fileSystem.getWorkingDir());


// set subtitle !!! format (semantic change)
        setTitleFormat(new MessageFormat("{0}"));

// set title
        String title = NbBundle.getBundle(JavaCvsCustomizer.class).getString("JavaCvsCustomizer.wizardName");
        setTitle(title);
        
        iterator.current().readSettings(this);
        this.updateState();
    }
    
/** Applies all changes done to the Cvs settings during the customizer.
 */
    public void applyChanges() {
        String lastRootDir = "";
        File f = fileSystem.getRootDirectory();
        if (f != null)
            lastRootDir = f.getAbsolutePath();
        if (!workingDir.equals (lastRootDir) || !relMount.equals (fileSystem.getRelMount())) {
            //We have to store files which can go ot of Repository
            org.openide.loaders.DataObject[] modifiedObjects = org.openide.loaders.DataObject.getRegistry().getModified();
            try {
                for (int i=0; i< modifiedObjects.length; i++) {
                    try {
                        if (modifiedObjects[i].getPrimaryFile().getFileSystem() == fileSystem) {
                            modifiedObjects[i].setValid (false);
                         }
                    } catch (org.openide.filesystems.FileStateInvalidException fsie) {/*Ignore it*/}
                }
            } catch (java.beans.PropertyVetoException pve) {
                // User pressed Cancel, cancel process
                return;
            }
        }
        String value;
        value = serverType;
        if (fileSystem.getCvsServerType() != fileSystem.getCvsServerType(value)) {
            fileSystem.setCvsServerType(fileSystem.getCvsServerType(value));
        }
        if (!fileSystem.getCvsUserName().equals(userName)) {
            fileSystem.setCvsUserName (userName);
        }
        if (!fileSystem.getCvsServerName().equals(serverName)) {
            fileSystem.setCvsServerName(serverName);
        }
        if (!fileSystem.getCvsRepository().equals(repository)) {
            fileSystem.setCvsRepository(repository);
        }
        fileSystem.setWorkingDir(workingDir);
        fileSystem.setRelMount(relMount);
    }
    
    
/*    private void fillTextFields() {
        BufferedReader buff = null;
        String dirName = workingDir;
        dirName = dirName + (relMount.equals("") ? "" : (File.separator + relMount));
        dirName = dirName + File.separator + "CVS" + File.separator + "Root";
        File dirFile = new File(dirName);
        if (dirFile.exists()) {
            try {
                buff = new BufferedReader(new InputStreamReader(new FileInputStream(dirFile.getAbsolutePath())));
                String line = buff.readLine();
                if (line.startsWith(":")) {
                    line = line.substring(1);
                    StringTokenizer token = new StringTokenizer(line, ":",false);
                    if (token.hasMoreTokens()) {
                        String type = token.nextToken();
                        if (type.equals(JavaCvsCustomizer.CVS_LOCAL))  {
                            return; // it's a local repository.. don't fill anything in :(
                        } else {   //some kind of server..
                            this.serverType = type;
                            if (token.hasMoreTokens()) {
                                String userServer =  token.nextToken();
                                this.userName = userServer.substring(0,userServer.indexOf('@'));
                                this.serverName = userServer.substring(userServer.indexOf('@') + 1);
                            }
                            D.deb("Parsing text fields...");
                            StringBuffer temp = new StringBuffer("");
                            while (token.hasMoreTokens()) {
                                temp.append(token.nextToken());
                                if (token.hasMoreTokens())
                                    temp.append(':');
                            }
                            repository = temp.toString();
                        }
                    }
                }
            } catch (IOException exc) { //doesn't matter - nothing will be filled in
            }
            finally {
                if (buff != null) {
                    try {
                        buff.close();
                    } catch (IOException eIO) {}
                }
            }
        }
    }
  */  
    
/** Checks if the finish panel was pressed. If so, does the final part of filesystem setup.
 * Fills in server name, repository root and user name regarding to the info in the CVS/Root file in the currently mounted directory.
 * It does so, even if new modules were checked out and the server panel was filled.
 * @param event the current property chneg event
 */
    public void propertyChange(final PropertyChangeEvent event) {
        if (event.getPropertyName().equals(DialogDescriptor.PROP_VALUE)){
            Object option = event.getNewValue();
            if (option == WizardDescriptor.FINISH_OPTION) {
                iterator.current().storeSettings(this);
                //      fillTextFields();
                applyChanges();
                org.openide.TopManager.getDefault ().getRepository ().addFileSystem (fileSystem);
                // if AutoRefresh is set to refresh only when mounting
                if (fileSystem.getAutoRefresh() == JavaCvsFileSystem.AUTO_RECURS_ONCE) {
                    Object[] root = new Object[1];
                    root[0] = "";
                    fileSystem.doRefresh(root, true);
                }
            }
        }
    }
    
    public static void centerWindow (Window w) {
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension dialogSize = w.getSize();
        w.setLocation((screenSize.width-dialogSize.width)/2,(screenSize.height-dialogSize.height)/2);
    }
    
    
    
/** Add a change listener to the iterator
 * @param l the listener to be added
 */
    public void addChangeListener(ChangeListener l) {
        if (listeners == null) {
            listeners = new EventListenerList ();
        }
        listeners.add (ChangeListener.class, l);
    }
    
/** removes the listener from the list.
 * @param l the listener to be removed.
 */
    public void removeChangeListener(ChangeListener l) {
        if (listeners != null) {
            listeners.remove (ChangeListener.class, l);
        }
    }
    
    
    private void fireChangeState() {
        Object[] list = listeners.getListenerList();
        ChangeEvent event = null;
        event = new ChangeEvent(this);
        for (int i=0; i< list.length - 1; i++){
            Object listen = list[i];
            if (listen == null) continue;
            if (listen.getClass() == ChangeListener.class) {
                ChangeListener listener = (ChangeListener) list[i];
                listener.stateChanged(event);
            }
            
        }
    }
    
    

    
    /** Getter for property serverType.
     * @return Value of property serverType.
     */
    public String getServerType() {
        return serverType;
    }
    
    /** Setter for property serverType.
     * @param serverType New value of property serverType.
     */
    public void setServerType(String serverType) {
        this.serverType = serverType;
    }
    
    /** Getter for property userName.
     * @return Value of property userName.
     */
    public String getUserName() {
        return userName;
    }
    
    /** Setter for property userName.
     * @param userName New value of property userName.
     */
    public void setUserName(String userName) {
        this.userName = userName;
    }
    
    /** Getter for property serverName.
     * @return Value of property serverName.
     */
    public String getServerName() {
        return serverName;
    }
    
    /** Setter for property serverName.
     * @param serverName New value of property serverName.
     */
    public void setServerName(String serverName) {
        this.serverName = serverName;
    }
    
    /** Getter for property repository.
     * @return Value of property repository.
     */
    public String getRepository() {
        return repository;
    }
    
    /** Setter for property repository.
     * @param repository New value of property repository.
     */
    public void setRepository(String repository) {
        this.repository = repository;
    }
    
    /** Getter for property workingDir.
     * @return Value of property workingDir.
     */
    public File getWorkingDir() {
        return workingDir;
    }
    
    /** Setter for property workingDir.
     * @param workingDir New value of property workingDir.
     */
    public void setWorkingDir(File workingDir) {
        this.workingDir = workingDir;
    }
    
    /** Getter for property relMount.
     * @return Value of property relMount.
     */
    public String getRelMount() {
        return relMount;
    }
    
    /** Setter for property relMount.
     * @param relMount New value of property relMount.
     */
    public void setRelMount(String relMount) {
        this.relMount = relMount;
    }
    
    /** Getter for property newSources.
     * @return Value of property newSources.
     */
    public boolean isNewSources() {
        return newSources;
    }
    
    /** Setter for property newSources.
     * @param newSources New value of property newSources.
     */
    public void setNewSources(boolean newSources) {
        this.newSources = newSources;
        iterator.chooseWorkflow(newSources);
    }
    

    
}
