/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/*
 * JavaDocFSSettings.java
 *
 * Created on 22. nor 2001, 15:02
 */

package org.netbeans.modules.javadoc.search;

import java.util.HashMap;

import org.openide.filesystems.FileSystem;
import org.openide.util.SharedClassObject;
import org.openide.TopManager;
import org.openide.ServiceType;

import org.netbeans.modules.javadoc.settings.*;

/**
 *
 * @author  Petr Suchomel
 * @version 1.0
 * Helper class
 */
public class JavaDocFSSettings extends Object {//SharedClassObject {

    /** Serialized Version UID */
    static final long serialVersionUID =-1524542458662425748L;
                                         
    private static String SUB_ROOT         = "subRoot";
    private static String SEARCH_ENGINE    = "searchEngine";

    private HashMap setting;
    
    private HashMap fsSetting = null;
    
    /** Creates new JavaDocFSSettings */
    public JavaDocFSSettings( FileSystem fs ) {
        //loads File system setting from System Option 
        if( fsSetting == null ){
            fsSetting = (HashMap)((DocumentationSettings)SharedClassObject.findObject(DocumentationSettings.class, true)).getFileSystemSettings();
            if( fsSetting.get(fs.getSystemName()) == null ){
                fsSetting.put(fs.getSystemName(), new HashMap());
                //save fs setting 
                ((DocumentationSettings)SharedClassObject.findObject(DocumentationSettings.class, true)).setFileSystemSettings(fsSetting);
            }
        }
        setting = (HashMap)fsSetting.get(fs.getSystemName());                    
    }
            
    /** Getter for property searchEngine.
     * @return Value of property searchEngine.
    */
    public ServiceType getSearchEngine() {
        JavadocSearchType.Handle searchType = (JavadocSearchType.Handle)setting.get(SEARCH_ENGINE);
        JavadocSearchType type = null;
        
        if (searchType != null) {
            type = (JavadocSearchType)searchType.getServiceType();
        }
        if (type == null) {
            return getDefaultJavaDocSearchType();
	}
        return type;        
    }

    /** Setter for property searchEngine.
     * @param searchEngine New value of property searchEngine.
     */
    public void setSearchEngine(ServiceType searchEngine) {
        if( searchEngine == null ){
            setting.remove(SEARCH_ENGINE);
            return;
        }
        setting.put(SEARCH_ENGINE , new JavadocSearchType.Handle(searchEngine));        
    }

    public JavadocSearchType getSearchTypeEngine(){
        return (JavadocSearchType)getSearchEngine();        
    }
    
    public String getSecondRoot(){
        return (String)setting.get(SUB_ROOT);
    }

    public void setSecondRoot(String secondRoot){
        if( secondRoot == null ){
            setting.remove(SUB_ROOT);
            return;
        }
        secondRoot = secondRoot.replace( java.io.File.separatorChar ,'/');
        setting.put(SUB_ROOT, secondRoot);
    }
    
    public static JavadocSearchType getDefaultJavaDocSearchType(){        
        return (JavadocSearchType)((DocumentationSettings)SharedClassObject.findObject(DocumentationSettings.class, true)).getSearchEngine();        
    }
}
