/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.javadoc.settings;

import java.io.File;
import java.io.IOException;
import java.util.Map;

import org.netbeans.modules.javadoc.ExternalJavadocType;
import org.openide.execution.*;
import org.openide.options.SystemOption;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.netbeans.modules.javadoc.settings.MemberConstants;
import org.openide.TopManager;

/**
 *
 * @author  Petr Suchomel
 * @version 0.1
 */
//for me, properties, which are shared must firePropertyChange !!

public class ExternalJavadocSettingsService extends ExternalJavadocType
{  
    /** generated Serialized Version UID */
    static final long serialVersionUID = 4753873543243545L;

    /** path to overview file */
    private File overview = null;

    /** members to show */
    private long members = MemberConstants.PROTECTED;

    /** override standard bootclasspath */
    public String bootclasspath;

    /** extension directories */
    public String extdirs = ""; // NOI18N

    /** generate JDK 1.1 style documentation*/
    public boolean style1_1;

    /** detail messages */
    public boolean verbose;

    /** encoding */
    public String encoding = ""; // NOI18N

    /** locale */
    public String locale = ""; // NOI18N

    /** max mamory */
    public int maxmem = 96;
    
    /** recursive */
    public boolean recursive = true;
    
    /** @return human presentable name */
    public String displayName() {
        return NbBundle.getBundle(ExternalJavadocSettingsService.class).getString("CTL_ExtJavadoc_settings");  // NOI18N
    }

    public HelpCtx getHelpCtx () {
        return new HelpCtx (ExternalJavadocSettingsService.class);
    }

    /** Getter for members
    */
    public long getMembers() {
        return members;
    }


    /** Setter for members
    */
    public void setMembers( long l ) {
        members = l;
    }

    /** Getter for path to overview file.
    */
    public File getOverview () {
        return overview;
    }

    /** Setter for path to overview file.
    */
    public synchronized void setOverview (File s) {
        overview = s;
    }

    /** Getter for bootclasspath
    */
    public String getBootclasspath () {
        return bootclasspath;
    }
    /** Setter for bootclasspath
    */
    public synchronized void setBootclasspath (String s) {
        bootclasspath = s;
    }

    /** Getter for extension directories
    */
    public String getExtdirs () {
        return extdirs;
    }
    /** Setter extension directories
    */
    public synchronized void setExtdirs (String s) {
        extdirs = s;
    }

    /** Getter for JDK 1.1 Style
    */
    public boolean isStyle1_1 () {
        return style1_1;
    }

    /** Setter for JDK 1.1 Style
    */
    public synchronized void setStyle1_1 (boolean b) {
        style1_1 = b;
    }

    /** Getter for verbose mode
    */
    public boolean isVerbose () {
        return verbose;
    }

    /** Setter for verbose mode
    */
    public synchronized void setVerbose (boolean b) {
        verbose = b;
    }

    /** Getter for encoding
    */
    public String getEncoding () {
        return encoding;
    }
    /** Setter for encoding
    */
    public synchronized void setEncoding (String s) {
        encoding = s;
    }

    /** Getter for locale
    */
    public String getLocale () {
        return locale;
    }
    /** Setter locale
    */
    public synchronized void setLocale (String s) {
        locale = s;
    }

    /** Getter for max memory
    */
    public int getMaxmemory () {
        return maxmem;
    }
    /** Setter max memory
    */
    public synchronized void setMaxmemory (int s) {
        maxmem = s;
    }

    /** Getter for recursive
    */
    public boolean getRecursive (){
        return recursive;
    }
    
    /** Setter max recursive
    */
    public void setRecursive (boolean rec){
        recursive = rec;
    }
}
