/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.nbbuild;

import java.io.File;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DirectoryScanner;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.taskdefs.MatchingTask;
import org.apache.tools.ant.taskdefs.Zip;

/** Task to handle .jar.MAKEME files.
 * Such files trigger the creation of a corresponding .jar file,
 * with contents taken from some standard tojar/ directory.
 *
 * @author Jesse Glick
 */
public class ToJar extends MatchingTask {
    
    private static final String SUFFIX = ".MAKEME";
    
    public ToJar () {
        setIncludes ("**/*.jar" + SUFFIX);
    }

    private String srcdir, unpackeddir;

    /** Source directory to look for <samp>*.jar.MAKEME</samp> files in. */
    public void setSrcdir (String dir) {
	srcdir = dir;
    }

    /** <samp>tojar/</samp> directory where JAR contents are unpacked.
     * Should match the source dir in structure, where instead of <samp>*.jar.MAKEME</samp>
     * files, there are <samp>*.jar/</samp> directories, whose contents are then
     * those of the JAR.
     */
    public void setUnpackeddir (String dir) {
	unpackeddir = dir;
    }

    public void execute () throws BuildException {
        log ("Warning: \"tojar\" task is deprecated, replace with simple \"jar\" tasks (and replace JARs with XML layers when possible)", Project.MSG_WARN);
        File base = getProject ().getBaseDir ();
        File start = new File (base, srcdir);
        DirectoryScanner scanner = getDirectoryScanner (start);
        scanner.scan ();
        String[] files = scanner.getIncludedFiles ();
        for (int i = 0; i < files.length; i++) {
            if (! files[i].endsWith (SUFFIX))
                throw new BuildException (files[i] + " did not end with " + SUFFIX,
                                          getLocation ());
            String stripMakeme = files[i].substring
                (0, files[i].length () - SUFFIX.length ());
            File dest = new File (start, stripMakeme);
            File from = new File (new File (base, unpackeddir), stripMakeme);
            if (dest.isDirectory ())
                throw new BuildException (dest.toString () + " is a directory",
                                          getLocation ());
            if (! from.isDirectory ())
                throw new BuildException (from.toString () + " is missing",
                                          getLocation ());
            Zip zip = (Zip) getProject ().createTask ("zip");
            zip.setOwningTarget (getOwningTarget ());
            try {
                Class[] params = {Boolean.TYPE};
                Object[] objs = {Boolean.FALSE};
                // Trying to invoke setCompess with parameter Boolean - ANT 1.3
                zip.getClass().getDeclaredMethod("setCompress",params).invoke(zip,objs);
                // Looks like ANT 1.3 - I hope that is ANT 1.3
                params[0] = File.class;
                objs[0] = from;
                zip.getClass().getDeclaredMethod("setBasedir",params).invoke(zip,objs);
            }
            catch (NoSuchMethodException ex)
            {
                // Looks like ANT 1.2 so use setCompress with String
                Class[] params = {String.class};
                Object[] objs = {"false"};
                try {
                    zip.getClass().getDeclaredMethod("setCompress",params).invoke(zip,objs);
                    objs[0] = from.toString();
                    zip.getClass().getDeclaredMethod("setBasedir",params).invoke(zip,objs);
                }
                catch (Exception ex1)
                {
                    throw new BuildException(ex1.fillInStackTrace());
                }
            }
            catch (Exception ex)
            {
                throw new BuildException(ex.fillInStackTrace());
            }
            zip.setZipfile (dest);
            zip.execute ();
        }
    }

}
