/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package test;

import java.io.*;
import java.util.*;

public class BundleTest {
    public static void main (String[] args) throws Exception {
        String name;
        if (args.length == 0) {
            name = ".";
        } else {
            name = args[0];
        }



        HashMap strings = findResources (new File (name));

        Properties p = new Properties ();
        p.load (new FileInputStream (new File (name, "Bundle.properties")));

        Set used = p.keySet ();

        TreeSet not = new TreeSet (strings.keySet ());
        not.removeAll (used);

        TreeSet more = new TreeSet (used);
        more.removeAll (strings.keySet ());

        if (more.size () == 0 && not.size () == 0) {
            System.out.println ("Ok");
            return;
        }

        if (more.size () != 0) {
            System.out.println ("Remove: ");
            Iterator it = more.iterator ();
            while (it.hasNext ()) {
                System.out.println ("  " + it.next ());
            }
        }

        if (not.size () != 0) {
            System.out.println ("Missing: ");
            Iterator it = not.iterator ();
            while (it.hasNext ()) {
                Object o = it.next ();
                System.out.println ("  " + o + " from " + strings.get (o));
            }
        }
    }


    /** Locates strings in all java files in the package */
    private static synchronized HashMap findResources (File folder) throws IOException {
        String[] list = folder.list ();

        set = new HashMap ();

        for (int i = 0; i < list.length; i++) {

            if (list[i].endsWith (".java")) {
                File f = new File (folder, list[i]);
                current = f;
                PushbackInputStream is = new PushbackInputStream (new BufferedInputStream (new FileInputStream (f)));

                readStream (is);
                is.close ();
            }
        }
        return set;
    }

    /** current file */
    private static File current;

    /** set of strings to locate */
    private static HashMap set;

    /** string that must be in bundle */
    private static void found (String s) {
        set.put (s, current);
    }


    /** Reads input stream scanning for getString text.
    */
    private static void readStream (PushbackInputStream is) throws IOException {
        int ch = is.read ();
        StringBuffer sb = new StringBuffer ();
        while (ch != -1) {
            if (Character.isLetter ((char)ch)) {
                sb.append ((char)ch);
            } else {
                String s = sb.toString ();
                if (s.startsWith ("get") && s.endsWith ("String")) {
                    is.unread (ch);
                    testString (is);
                }
                sb = new StringBuffer ();
            }
            ch = is.read ();
        }
    }

    private static void testString (PushbackInputStream is) throws IOException {
        if (readTill ('(', is) && readTill ('\"', is)) {
            StringBuffer sb = new StringBuffer ();
            int ch = is.read ();
            while (ch != (int)'\"') {
                sb.append ((char)ch);
                ch = is.read ();
            }

            found (sb.toString ());
        } else {
            System.out.println ("Spoiled getString in: " + current);
        }
    }


    private static boolean readTill (char goal, InputStream is) throws IOException {
        int ch = is.read ();
        while (Character.isSpace ((char)ch)) {
            ch = is.read ();
        }
        return goal == (char)ch;
    }
}


/*
 * Log
 *  3    src-jtulach1.2         10/22/99 Ian Formanek    NO SEMANTIC CHANGE - Sun
 *       Microsystems Copyright in File Comment
 *  2    src-jtulach1.1         8/9/99   Jaroslav Tulach 
 *  1    src-jtulach1.0         3/28/99  Jaroslav Tulach 
 * $
 */
