/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package test;

import org.openide.compiler.*;
import org.openide.compiler.Compiler;

/**
*
* @author Jaroslav Tulach
*/
public class CompilerJobTest extends Compiler {
    private String name;

    public CompilerJobTest(String s) {
        name = s;
    }


    public boolean isUpToDate () {
        System.out.println("Is up to date: " + this);
        return false;
    }

    public Class compilerGroupClass () {
        return Group.class;
    }

    public static void main (String[] args) throws Exception {
        CompilerJob job = new CompilerJob (Compiler.DEPTH_ONE);
        job.setDisplayName ("Job 1");

        CompilerJobTest c1 = new CompilerJobTest ("Hlup");
        job.add (c1);
        {
            CompilerJobTest c2 = new CompilerJobTest ("AfterHlup1");
            c2.dependsOn (c1);
            job.add (c2);
        }

        {
            CompilerJobTest c2 = new CompilerJobTest ("Hlup Parael");
            job.add (c2);
        }

        {
            CompilerJobTest c2 = new CompilerJobTest ("TwiceAdded");
            job.add (c2);
        }

        {
            CompilerJobTest c2 = new CompilerJobTest ("TwiceAdded");
            job.add (c2);
            c2.dependsOn (c1);
        }

        CompilerJob j2 = new CompilerJob (Compiler.DEPTH_ONE);
        j2.setDisplayName ("Job 2");

        CompilerJobTest d1 = new CompilerJobTest ("First");
        j2.add (d1);

        CompilerJobTest d2 = new CompilerJobTest ("Second");
        d2.dependsOn (d1);
        j2.add (d2);

        CompilerJobTest d3 = new CompilerJobTest ("Third");
        d3.dependsOn (job);
        j2.add (d3);

        j2.dependsOn (job);

        j2.start ();


    }

    public boolean equals (Object o) {
        return ((CompilerJobTest)o).name.equals (name);
    }

    public int hashCode () {
        return name.hashCode ();
    }

    public String toString () {
        return name;
    }


    public static class Group extends CompilerGroup {
        private StringBuffer sb = new StringBuffer ();
        {
            sb.append ("Group [\n");
        }

        public void add (Compiler c) {
            sb.append ("  ");
            sb.append (c.toString ());
            sb.append ("\n");
        }

        public boolean start () {
            System.out.print(sb);
            System.out.println("]");
            return true;
        }
    }
}

/*
* Log
*  3    Gandalf   1.2         12/23/99 Jaroslav Tulach Enhancing compiler API to
*       makefile capabilities
*  2    Gandalf   1.1         10/22/99 Ian Formanek    NO SEMANTIC CHANGE - Sun 
*       Microsystems Copyright in File Comment
*  1    Gandalf   1.0         9/10/99  Jaroslav Tulach 
* $
*/