/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package test;

import java.awt.BorderLayout;
import javax.swing.JComponent;
import javax.swing.JPanel;

import org.openide.TopManager;
import org.openide.awt.SplittedPanel;
import org.openide.explorer.ExplorerPanel;
import org.openide.explorer.ExplorerManager;
import org.openide.explorer.view.*;
import org.openide.nodes.*;

/** An example of using Explorer API.
 * Opens a new explorer with a context tree view and list view on the Repository.
 *
 * @author  Ian Formanek
 * @version 1.00
 */
public class ExplorerExample extends JComponent implements ExplorerManager.Provider {
    private ExplorerManager man = new ExplorerManager ();

    public ExplorerManager getExplorerManager () {
        return man;
    }


    /** Creates new ExplorerExample */
    public ExplorerExample () {
        setLayout (new java.awt.BorderLayout ());
        JPanel p = new JPanel (new java.awt.GridLayout (3, 3));

        p.add (new ContextTreeView ());
        p.add (new ListView () {
                   protected NodeListModel createModel () {
                       NodeListModel m = new NodeListModel ();
                       m.setDepth (3);
                       return m;
                   }
               });
        p.add (new BeanTreeView ());
        p.add (new IconView ());
        p.add (new MenuView ());
        //    p.add (new TabView ());

        add ("Center", p);

        add ("North", new ChoiceView ());
    }

    private static Node create (int len, int level) {

        if (len >= 0) {
            AbstractNode an = new AbstractNode (new Children.Array ());
            Node[] arr = new Node[len];
            while (--len >= 0) {
                arr[len] = create (len - 2, level + 1);
                arr[len].setName ("Level: " + level + " index: " + len);
            }
            an.getChildren ().add (arr);
            return an;
        } else {
            return new AbstractNode (Children.LEAF);
        }

    }

    public static void main (String args[]) {
        ExplorerExample panel = new ExplorerExample ();
        Node an;
        an = TopManager.getDefault ().getPlaces ().nodes ().repository ();

        //    an = create (5, 1);
        //    an.setName ("Root");

        panel.getExplorerManager ().setRootContext (an);

        final javax.swing.JFrame f = new javax.swing.JFrame ();
        f.getContentPane ().add (panel);

        javax.swing.SwingUtilities.invokeLater (new Runnable () {
                                                    public void run () {
                                                        f.setSize (1000, 600);
                                                        f.show ();
                                                    }
                                                });
    }

}

/*
 * Log
 *  3    Gandalf   1.2         10/22/99 Ian Formanek    NO SEMANTIC CHANGE - Sun
 *       Microsystems Copyright in File Comment
 *  2    Gandalf   1.1         9/6/99   Ian Formanek    Removed use of TabView
 *  1    Gandalf   1.0         9/2/99   Jaroslav Tulach 
 * $
 */
