/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package test;

import java.io.*;
import java.util.*;
import sun.tools.java.*;
import sun.tools.tree.*;
import sun.tools.javac.*;

public class SoutKiller {
    static int beforeLines = 1;
    static int afterLines = 1;
    static TreeSet lines;

    public static void testFile(File f) throws IOException {
        lines = new TreeSet();
        byte[] b = new byte[(int)f.length ()];
        InputStream is = new FileInputStream (f);
        is.read (b);
        is.close();
        String text = new String (b);

        Environment env = new BatchEnvironment (System.err, new ClassPath ());

        StringParser c = new StringParser (env, new ByteArrayInputStream (b));

        long oldmil = System.currentTimeMillis();
        c.parseFile();

        if (lines.size() > 0) {
            System.out.println ("*****************************************************************************");
            System.out.println ("   "+f.getAbsolutePath());
            LineNumberReader reader = new LineNumberReader(new InputStreamReader(new ByteArrayInputStream(b)));
            String line = reader.readLine();
            Iterator it = lines.iterator();
            int lastIndex = -1;
            while (it.hasNext()) {
                int index = ((Integer) it.next()).intValue();
                while ((reader.getLineNumber() < index) && (line != null)) {
                    line = reader.readLine();
                }
                if (line != null) {
                    if (lastIndex + 1< index)
                        System.out.println ();
                    System.out.println(index + ": "+line);
                }
                lastIndex = index;
            }
        }
    }

    private static void list(File dir) {
        try {
            String path = dir.getCanonicalPath();
            String[] files = dir.list();
            for (int i = 0; i < files.length; i++) {
                File f = new File(path + File.separator + files[i]);
                if (files[i].endsWith(".java"))
                    testFile(f);
                if (f.isDirectory())
                    list(f);
            }
        }
        catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void main(String[] args) throws Exception {
        File f = new File(".");
        list(f);
    }

    static int stav = 0;

    static class StringParser extends BatchParser {
        StringParser(Environment env, InputStream is) throws IOException {
            super(env, is);
        }

        public void scanIdentifier() throws IOException {
            super.scanIdentifier();
            String id = idValue.toString();
            if (id.equals("printStackTrace")) {
                addPos();
                stav = 0;
            }
            else {
                switch (stav) {
                case 1:
                    if (id.equals("out"))
                        addPos();
                    else if (id.equals("err"))
                        addPos();
                    break;
                case 2:
                    if (id.equals("dumpStack"))
                        addPos();
                    break;
                }
                if (id.equals("System")) {
                    stav = 1;
                }
                else {
                    if (id.equals("Thread"))
                        stav = 2;
                    else
                        stav = 0;
                }
            }
        }

        private void addPos() {
            int ln = (int) (pos >>> 32);
            for (int i = ln - beforeLines; i <= ln + afterLines; i++)
                lines.add(new Integer(i));
        }
    }
}

/*
 * Log
 *  3    Gandalf   1.2         10/22/99 Ian Formanek    NO SEMANTIC CHANGE - Sun
 *       Microsystems Copyright in File Comment
 *  2    Gandalf   1.1         10/10/99 Petr Hamernik   
 *  1    Gandalf   1.0         10/10/99 Petr Hamernik   
 * $
 */
