/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package test;
import java.util.*;

/** Time that counts time intervals that we spend in the each named section.
*
* @author Jaroslav Tulach
* @version 0.10, May 12, 1998
*/
public class TimeCounter extends Object {
    /** maps keys to counts (String => Data) */
    private static Hashtable counter = new Hashtable ();

    /** Begins counted area.
    * @param name of area to begin
    */
    public static synchronized void start (String name) {
        Data d = (Data)counter.get (name);
        if (d == null) {
            counter.put (name, d = new Data ());
        }
        if (d.started++ == 0) {
            d.time = System.currentTimeMillis ();
            d.count++;
        }
    }

    /** Ends counted area.
    * @param name of area to begin
    */
    public static synchronized void stop (String name) {
        Data d = (Data)counter.get (name);
        if (--d.started == 0) {
            d.time = (System.currentTimeMillis () - d.time);
            d.total += d.time;
        }
    }

    public static synchronized void print () {
        Enumeration en = counter.keys ();
        System.out.println ("---------------------");
        while (en.hasMoreElements ()) {
            String s = (String)en.nextElement ();
            Data d = (Data)counter.get (s);
            System.out.println ("Name: " + s + " Time: " + d.total + " Count: " + d.count);
            //      d.total = 0;
            //      d.count = 0;
        }
    }

    public static synchronized void reset () {
        counter = new Hashtable ();
    }

    public static void main (String[] args) {
        print ();
    }

    private static class Data {
        public int started;
        public int count;
        public long time;
        public long total;
    }

}

/*
 * Log
 *  3    Gandalf   1.0.1.1     1/16/00  Jaroslav Tulach Has main method that 
 *       prints current state.
 *  2    Gandalf   1.0.1.0     10/22/99 Ian Formanek    NO SEMANTIC CHANGE - Sun
 *       Microsystems Copyright in File Comment
 *  1    Tuborg    1.0         9/3/98   Jan Jancura     
 * $
 */
