/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.examples.modules.globalactions;

import java.io.IOException;

import org.openide.Places;
import org.openide.TopManager;
import org.openide.loaders.DataFolder;
import org.openide.modules.ModuleDescription;
import org.openide.modules.ModuleInstall;
import org.openide.util.Utilities;
import org.openide.util.actions.SystemAction;

import org.openidex.util.Utilities2;

/** Install {@link TestAction} into the main window's menu and toolbar.
*
* <p>Feel free to copy-and-paste pieces of this implementation for your own
* install code, if you are doing similar things.
*
* <p>Note that if we are using NetBeans 3.1 or a later API version, XML filesystem
* layers completely supercede this class, and so we return immediately from all
* its methods.
*/
public class Install extends ModuleInstall {

    /** Turned on if XML filesystem layers are possible in this version of the IDE. */
    private static boolean haveLayers = false;
    static {
        try {
            ModuleDescription.class.getField ("TAG_LAYER");
            haveLayers = true;
        } catch (NoSuchFieldException nsfe) {
            // leave it as false
        }
    }

    /** Install actions.
    * Adds the {@link TestAction} to View menu right after Web Browser (with a separator);
    * and to the end of the View toolbar.
    *
    * <p>Note that the {@link TestServiceAction} is completely handled by the manifest.
    * For {@link TestAction}, placing it in the manifest would add it as a service action,
    * which is permissible but unnecessary since it is installed specially here.
    *
    * <p>Also installs a key binding.
    */
    public void installed () {
        if (haveLayers) return;
        Places.Folders folders = TopManager.getDefault ().getPlaces ().folders ();
        try {
            Utilities2.createAction
                (TestAction.class,                             // class to create
                 DataFolder.create (folders.menus (), "View"), // where
                 "HTMLViewAction",                             // relative to
                 true,                                         // after?
                 false,                                        // skip over sep?
                 true,                                         // sep before it?
                 false);                                       // sep after it?
            Utilities2.createAction
                (TestAction.class,
                 DataFolder.create (folders.toolbars (), "View"),
                 "HTMLViewAction",
                 true,
                 false,
                 false,
                 false);
            // Simpler: no need to specify position here.
            Utilities2.createAction
                (TestAction.class,
                 DataFolder.create (folders.actions (), "View"));
        } catch (IOException ioe) {
            ioe.printStackTrace ();
        }
        bindKey ();
    }

    /** Uninstall the same actions we previously installed.
    */
    public void uninstalled () {
        if (haveLayers) return; 
        Places.Folders folders = TopManager.getDefault ().getPlaces ().folders ();
        try {
            Utilities2.removeAction
                (TestAction.class,
                 DataFolder.create (folders.menus (), "View"));
            Utilities2.removeAction
                (TestAction.class,
                 DataFolder.create (folders.toolbars (), "View"));
            Utilities2.removeAction
                (TestAction.class,
                 DataFolder.create (folders.actions (), "View"));
        } catch (IOException e) {
            e.printStackTrace ();
        }
    }

    /** Restore the module.
    */
    public void restored () {
        if (haveLayers) return; 
        bindKey ();
    }

    /** Called to make the keyboard shortcut.
     */
    private static void bindKey () {
	TopManager.getDefault ().getGlobalKeymap ().addActionForKeyStroke
	    (Utilities.stringToKey ("C-F12"),
	     SystemAction.get (TestAction.class));
    }

}
