/*
* TestFormExecutor.java
*/ 

package org.netbeans.examples.modules.testforms;

import java.awt.*;
import java.awt.event.*;
import java.io.IOException;
import java.text.MessageFormat;
import javax.swing.*;

import org.openide.TopManager;
import org.openide.execution.*;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.io.FoldingIOException;

/** Somehow handles a class present in the Repository by "running" it.
 *
 * @author jglick
 */
public class TestFormExecutor extends ThreadExecutor {

    // The default value for an unconfigured executor.
    //private String myProp = "someDefaultValue";

    public TestFormExecutor () {
    }

    // You may configure this executor as a JavaBean:
    //public String getMyProp () {
    //    return myProp;
    //}

    //public synchronized void setMyProp (String nue) {
    //    String old = myProp;
    //    myProp = nue;
    //    firePropertyChange ("myProp", old, nue);
    //}

    protected void checkClass (Class clazz) throws IOException {
        try {
            // Just check that there is nothing seriously wrong with the file,
            // e.g. totally inappropriate type of class, etc.
            Component thing = (Component) clazz.newInstance ();
        } catch (ThreadDeath td) {
            throw td;
        } catch (Throwable t) {
            throw new FoldingIOException (t);
        }
    }

    protected void executeClass (Class clazz, String[] args) {
        try {
            // Actually do something with this class, e.g.:
            System.err.println ("Starting form -->"); // shown on OutputWindow
            Component thing = (Component) clazz.newInstance ();
            // Any calls made in clazz to System.err, etc., will go to OutputWindow automatically:
            Window win;
            if (thing instanceof Window) {
                win = (Window) thing;
            } else {
                JFrame frame = new JFrame ("Testing " + clazz.getName ());
                frame.getContentPane ().setLayout (new BorderLayout ());
                frame.getContentPane ().add (thing, BorderLayout.CENTER);
                win = frame;
            }
            final Object holdOn = new Object ();
            win.addWindowListener (new WindowAdapter () {
                public void windowClosing (WindowEvent ev) {
                    synchronized (holdOn) {
                        holdOn.notifyAll ();
                    }
                }
            });
            win.pack ();
            win.show ();
            synchronized (holdOn) {
                holdOn.wait ();
            }
            System.err.println ("<-- and ending.");
        } catch (ThreadDeath td) {
            throw td;
        } catch (Throwable t) {
            // Will actually go to the OutputWindow automatically:
            t.printStackTrace ();
        }
    }

    /*
    public HelpCtx getHelpCtx () {
     *return new HelpCtx (TestFormExecutor.class);
}
    */

}
