/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.openide.actions;

import java.awt.event.KeyEvent;
import java.io.IOException;

import javax.swing.event.*;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;

import org.openide.TopManager;
import org.openide.util.HelpCtx;
import org.openide.util.Utilities;
import org.openide.loaders.*;
import org.openide.filesystems.FileObject;
import org.openide.WizardDescriptor;

/** Dialog that can be used in create from template.
*
* @author  Jaroslav Tulach, David Strupl
* @version 
*/
final class ClassNamePanel extends javax.swing.JPanel implements DocumentListener, WizardDescriptor.FinishPanel {

    /** listener to changes in panel */
    private ChangeListener listener;
    
    /** File extension of the template and of the created file -
     * it is used to test whether file already exists.
     */
    private String extension;
 
    /** */
    DataFolder targetFolder;
    
    /**  */
    public ClassNamePanel() {
        initComponents ();

        setName(getString("LAB_NewClassPanelName"));

        setBorder (new javax.swing.border.EmptyBorder(new java.awt.Insets(8, 8, 8, 8)));

        // registers itself to listen to changes in the content of document
        className.getDocument().addDocumentListener(this);
        className.unregisterKeyboardAction(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0));
        
        setClassName("");
    }

    /** Preffered size */
    public java.awt.Dimension getPreferredSize() {
        return new java.awt.Dimension(250, 100);
    }

    /** Request focus.
    */
    public void requestFocus () {
        className.requestFocus();
        className.selectAll ();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        namePanel = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        className = new javax.swing.JTextField();
        setLayout(new java.awt.BorderLayout(0, 8));
        
        namePanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        jLabel1.setText(java.util.ResourceBundle.getBundle("org/openide/actions/Bundle").getString("CTL_TemplateClassName"));
          gridBagConstraints1 = new java.awt.GridBagConstraints();
          gridBagConstraints1.gridx = 0;
          gridBagConstraints1.gridy = 0;
          gridBagConstraints1.insets = new java.awt.Insets(0, 0, 0, 8);
          gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
          namePanel.add(jLabel1, gridBagConstraints1);
          
          
        className.addFocusListener(new java.awt.event.FocusAdapter() {
              public void focusGained(java.awt.event.FocusEvent evt) {
                  classNameFocusGained(evt);
              }
          }
          );
          gridBagConstraints1 = new java.awt.GridBagConstraints();
          gridBagConstraints1.gridx = 1;
          gridBagConstraints1.gridy = 0;
          gridBagConstraints1.gridwidth = 0;
          gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
          gridBagConstraints1.weightx = 1.0;
          namePanel.add(className, gridBagConstraints1);
          
          
        add(namePanel, java.awt.BorderLayout.CENTER);
        
    }//GEN-END:initComponents

    private void packageNameFocusGained (java.awt.event.FocusEvent evt) {//GEN-FIRST:event_packageNameFocusGained

    }//GEN-LAST:event_packageNameFocusGained

    private void packageModelChanged (javax.swing.event.ListDataEvent evt) {//GEN-FIRST:event_packageModelChanged
    }//GEN-LAST:event_packageModelChanged

    private void classNameFocusGained (java.awt.event.FocusEvent evt) {//GEN-FIRST:event_classNameFocusGained
        if (
            Utilities.getOperatingSystem() == Utilities.OS_SOLARIS |
            Utilities.getOperatingSystem() == Utilities.OS_SUNOS
        ) {
            // does not work on CDE window manager, so better do nothin
            return;
        }

        className.selectAll ();
    }//GEN-LAST:event_classNameFocusGained

    private void templatesTreeValueChanged (javax.swing.event.TreeSelectionEvent evt) {//GEN-FIRST:event_templatesTreeValueChanged
    }//GEN-LAST:event_templatesTreeValueChanged


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel namePanel;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JTextField className;
    // End of variables declaration//GEN-END:variables

    /** */
    public void changedUpdate(final javax.swing.event.DocumentEvent p1) {
        if (p1.getDocument () == className.getDocument ()) {
            SwingUtilities.invokeLater (new Runnable () {
                public void run () {
                    if (className.getText().equals ("")) {
                        setClassName ("");
                    }
                    fireStateChanged();
                }
            });
        }
    }

    public void removeUpdate(final javax.swing.event.DocumentEvent p1) {
        changedUpdate (p1);
    }
    
    public void insertUpdate(final javax.swing.event.DocumentEvent p1) {
        changedUpdate (p1);
    }


    /** Help for this panel.
    * @return the help or <code>null</code> if no help is supplied
    */
    public org.openide.util.HelpCtx getHelp () {
        return new HelpCtx (ClassNamePanel.class);
    }

    /** Test whether the panel is finished and it is safe to proceed to the next one.
    * If the panel is valid, the "Next" (or "Finish") button will be enabled.
    * @return <code>true</code> if the user has entered satisfactory information
    */
    public boolean isValid () {
        // test whether the selected name already exists
        FileObject f = targetFolder.getPrimaryFile().getFileObject(className.getText(), extension);
        if (f != null) {
            return false;
        }

        return true;
    }//

        /** Add a listener to changes of the panel's validity.
    * @param l the listener to add
    * @see #isValid
    */
    public void addChangeListener (ChangeListener l) {
        if (listener != null) {
            throw new IllegalStateException ();
        }
        listener = l;
    }

    /** Remove a listener to changes of the panel's validity.
    * @param l the listener to remove
    */
    public void removeChangeListener (ChangeListener l) {
        listener = null;
    }

    /** Fires info to listener.
    */
    private void fireStateChanged () {
        if (listener != null) {
            listener.stateChanged (new ChangeEvent (this));
        }
    }
  
    /** Sets the class name to some reasonable value.
    * @param name the name to set the name to
    */
    private void setClassName (String name) {
        String n = name;
        if (name == null || name.length () == 0) {
            n = defaultClassName ();
        }

        className.getDocument().removeDocumentListener(this);
        className.setText (n);
        className.getDocument().addDocumentListener(this);

        if (name == null || name.length () == 0) {
            className.selectAll ();
        }
    }

    /** */
    public String getClassName() {
        return className.getText();
    }
    
    /** Getter for default name of a class.
    * @return the default name.
    */
    static String defaultClassName () {
        return getString("FMT_DefaultClassName");
    }
    
    private static String getString(String key) {
        return org.openide.util.NbBundle.getBundle(ClassNamePanel.class).getString(key);
    }
    /** Get the component displayed in this panel.
     * @return the component
     */
    public java.awt.Component getComponent() {
        return this;
    }
    /** Provides the wizard panel with the current data--either
     * the default data or already-modified settings, if the user used the previous and/or next buttons.
     * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
     * @param settings the object representing wizard panel state, as originally supplied to {@link WizardDescriptor#WizardDescriptor(WizardDescriptor.Iterator,Object)}
     */
    public void readSettings(Object settings) {
        TemplateWizard wizard = (TemplateWizard)settings;
        
        DataObject template = wizard.getTemplate ();
        if (template != null) {
            extension = template.getPrimaryFile().getExt();
        }
        
        try {
            targetFolder = wizard.getTargetFolder();
        } catch (IOException x) {
            TopManager.getDefault().getErrorManager().notify(x);
        }

    }
    /** Provides the wizard panel with the opportunity to update the
     * settings with its current customized state.
     * Rather than updating its settings with every change in the GUI, it should collect them,
     * and then only save them when requested to by this method.
     * Also, the original settings passed to {@link #readSettings} should not be modified (mutated);
     * rather, the (copy) passed in here should be mutated according to the collected changes.
     * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
     * @param settings the object representing a settings of the wizard
     */
    public void storeSettings(Object settings) {
        org.openide.loaders.TemplateWizard wizard = (org.openide.loaders.TemplateWizard)settings;
        wizard.setTargetName(className.getText());
    }
}
