/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.openide.actions;

import org.openide.TopManager;
import org.openide.cookies.ExecCookie;
import org.openide.nodes.Node;
import org.openide.util.actions.CallableSystemAction;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.SharedClassObject;
import org.openide.util.actions.ProjectSensitiveAction;

/** Executes the current project if it supports execution.
*
* @author   Ian Formanek
*/
public class ExecuteProjectAction extends ProjectSensitiveAction {
    static final long serialVersionUID =-7908144941318263878L;
    /** generated Serialized Version UID */
    //static final long serialVersionUID = -3365766607488094613L;

    /* Human presentable name of the action. This should be
    * presented as an item in a menu.
    * @return the name of the action
    */
    public String getName() {
        return NbBundle.getBundle(ExecuteProjectAction.class).getString("ExecuteProject");
    }

    /* Help context where to find more about the action.
    * @return the help context for this action
    */
    public HelpCtx getHelpCtx() {
        return new HelpCtx (ExecuteProjectAction.class);
    }

    /* Icon resource.
    * @return name of resource for icon
    */
    protected String iconResource () {
        return "/org/openide/resources/actions/executeProject.gif"; // NOI18N
    }

    private static final class ExecuteActionAccessible extends ExecuteAction {
        public void performAction (Node[] nodes) {
            super.performAction (nodes);
        }
    }
    public void performAction(final Node projectDesktop) {
        ((ExecuteActionAccessible) SharedClassObject.findObject (ExecuteActionAccessible.class, true)).
            performAction (new Node[] { projectDesktop });
    }

    /** Test whether the action is currently enabled.
    * @return <code>true</code> if so
    */
    public boolean enable (Node projectDesktop) {
        return projectDesktop.getCookie (ExecCookie.class) != null;
    }
}
