/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.openide.compiler;

import java.io.PrintStream;
import java.io.PrintWriter;

/** Exception created when a set of compilers should be compiled and
* there is a cyclic dependency between them.
* <P>
* The exception carries list of objects that form the cycle. 
*
* @author Jaroslav Tulach, Jesse Glick
*/

public final class DependencyException extends Exception {

    /** array of Compilable */
    private final Compilable[] array;

    /** Creates new DependencyException. */
    DependencyException (Compilable[] comp) {
        super (comp.length + " compilers formed a cycle"); // NOI18N
        array = comp;
    }

    /** Getter for list of compilables that form the cycle.
    * @return array of compilables
    */
    public Compilable[] getCompilables () {
        return array;
    }

    // Debugging info for e.g. #10585:

    public void printStackTrace () {
        printStackTrace (System.err);
    }

    public void printStackTrace (PrintStream ps) {
        super.printStackTrace (ps);
        ps.println ("Compiler cycle:"); // NOI18N
        for (int i = 0; i < array.length; i++) {
            ps.print ('\t');
            ps.println (array[i]);
        }
    }

    public void printStackTrace (PrintWriter pw) {
        super.printStackTrace (pw);
        pw.println ("Compiler cycle:"); // NOI18N
        for (int i = 0; i < array.length; i++) {
            pw.print ('\t');
            pw.println (array[i]);
        }
    }

}
