/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.openide.explorer.propertysheet;

import java.beans.FeatureDescriptor;

import org.openide.nodes.Node;

/**
 * Instance of this class contains information that
 * is being passed to the property editor (instance of
 * ExtendedPropertyEditor) from the IDE.
 * @author  dstrupl
 */
public final class PropertyEnv {
  
    /** */
    private FeatureDescriptor featureDescriptor;
    /** */
    private Object[] beans;
    
    /** Default constructor has package access -
     * we do not want the instances to be created outside
     * our package.
     */
    PropertyEnv() {
    }
    
    /**
     * Array of beans that the edited property belongs to.
     */
    public Object[] getBeans() {
        return beans;
    } 
    
    /**
     * Array of nodes that the edited property belongs to. 
     */
    void setBeans(Object[] beans) {
        this.beans = beans;
    } 
    
    /** 
     * Feature descritor that describes the property. It is feature
     * descriptor so one can plug in PropertyDescritor and also Node.Property
     * which both inherit from FeatureDescriptor
     */
    public FeatureDescriptor getFeatureDescriptor() {
        return featureDescriptor;
    }
    
    /** 
     * Feature descritor that describes the property. It is feature
     * descriptor so one can plug in PropertyDescritor and also Node.Property
     * which both inherit from FeatureDescriptor
     */
    void setFeatureDescriptor(FeatureDescriptor desc) {
        this.featureDescriptor = desc;
    }
    
    // [PENDING]
        /** The editor may be able to edit properties of different classes. It can decide to
         * be able to edit descendants of a base class.*/
    //Class propertyClass; // read-only property
  }
  