/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.openide.explorer.propertysheet.editors;

import java.util.ResourceBundle;
import java.awt.Dimension;

import org.openide.src.Type;
import org.openide.util.NbBundle;

/** Visual panel for editing one MethodParameter.
 *
 * @author Petr Hamernik
 */
class MethodParameterPanel extends javax.swing.JPanel {
    /** Resource bundle. */
    static final ResourceBundle bundle = NbBundle.getBundle(MethodParameterPanel.class);

    /** This array is used in type combo box. */
    static final String[] COMMON_TYPES = {
        "java.lang.String", // NOI18N
        "java.lang.Object", // NOI18N
        "boolean", // NOI18N
        "byte", // NOI18N
        "char", // NOI18N
      "short", // NOI18N
        "int", // NOI18N
      "long", // NOI18N
        "float" // NOI18N
    };

    static final long serialVersionUID =5505625069203018340L;
    /** Creates new form ParamPanel */
    public MethodParameterPanel() {
        initComponents ();

        typeLabel.setDisplayedMnemonic(bundle.getString("CTL_Type_Mnemonic").charAt(0));
        nameLabel.setDisplayedMnemonic(bundle.getString("CTL_Name_Mnemonic").charAt(0));
        finalCheckBox.setMnemonic(bundle.getString("CTL_Final_Mnemonic").charAt(0));
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        typeLabel = new javax.swing.JLabel();
        typeCombo = new javax.swing.JComboBox(COMMON_TYPES);
        nameLabel = new javax.swing.JLabel();
        nameTextField = new javax.swing.JTextField();
        finalCheckBox = new javax.swing.JCheckBox();
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(12, 12, 0, 11)));
        setMinimumSize(new java.awt.Dimension(500, 87));
        
        typeLabel.setText(bundle.getString("CTL_Type"));
        typeLabel.setLabelFor(typeCombo);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 6, 8);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.EAST;
        add(typeLabel, gridBagConstraints1);
        
        
        typeCombo.setEditable(true);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridwidth = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 6, 0);
        gridBagConstraints1.weightx = 1.0;
        add(typeCombo, gridBagConstraints1);
        
        
        nameLabel.setText(bundle.getString("CTL_Name"));
        nameLabel.setLabelFor(nameTextField);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 6, 8);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.EAST;
        add(nameLabel, gridBagConstraints1);
        
        
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridwidth = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 0, 6, 0);
        gridBagConstraints1.weightx = 1.0;
        add(nameTextField, gridBagConstraints1);
        
        
        finalCheckBox.setText(bundle.getString("CTL_Final"));
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.gridwidth = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.ipadx = 1;
        gridBagConstraints1.weightx = 1.0;
        add(finalCheckBox, gridBagConstraints1);
        
    }//GEN-END:initComponents

    public Dimension getPreferredSize() {
        Dimension orig = super.getPreferredSize();
        return new Dimension(orig.width + typeCombo.getPreferredSize().width, orig.height);
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    javax.swing.JLabel typeLabel;
    javax.swing.JComboBox typeCombo;
    javax.swing.JLabel nameLabel;
    javax.swing.JTextField nameTextField;
    javax.swing.JCheckBox finalCheckBox;
    // End of variables declaration//GEN-END:variables

}

/*
* Log
*  3    Gandalf   1.2         1/12/00  Ian Formanek    NOI18N
*  2    Gandalf   1.1         11/26/99 Patrik Knakal   
*  1    Gandalf   1.0         11/26/99 Petr Hamernik   
* $
*/
