/*
 * MIMEResolver.java
 *
 * Created on January 30, 2001, 6:02 PM
 */

package org.openide.filesystems;
import java.util.*;
/**
 * This class is intended as superclass for individual resolvers. Resolvers must implement
 * only one method: findMIMEType(FileObject fo). This method is called from MIMESupport.findMIMEType (FileObject), 
 * where registered subclasses of MIMEResolver are asked one by one to resolve MIME type of this FileObject.
 * Resolving is finished right after first resolver is able to resolve this FileObject or if all registered
 * resolvers returned null (not recognized).
 *
 *Resolvers are registered if they have their record in IDE_HOME\system\Services
 * in form *.instance e.g.: org-some-package-JavaResolver.instance
 *
  * @author  rmatous
 */

public abstract class MIMEResolver extends Object {    
/**
 * Resolves FileObject and returns recognized MIME type
 * @param fo is FileObject which should be resolved
 * @return  recognized MIME type or null if not recognized
 */
    public abstract String findMIMEType (FileObject fo);        
}
