/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.openide.filesystems;

import  java.util.*;
import java.io.OutputStream;
import java.lang.reflect.*;
import java.net.URL;
import  java.io.*;


import  org.xml.sax.AttributeList;
import org.xml.sax.*;

import   org.openide.util.io.NbMarshalledObject;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;
import org.openide.util.io.NbObjectInputStream;
/**
 *Holds in Map attributes: Map(String attrName,XMLMapAttr.Attr attribute). This map holds all atributes for one FileObject.
 *<BR><BR>
 *<H3>Detailed description</H3>
 * Each file object (file or folder element) can have 0..* attributes.<BR> <BR>
 * Each file object <I>atrribute</I> (attribute is here name of element) must have two attributes (here XML attribute).<BR>
 * <OL>
 * <LI>First attribute name is <I>id</I> , which is mandatory and value of this attribute serve as identifier to distinguish many
 * attributes for one file object.
 * <LI> Second attribute is also mandatory, but you can choose such attribute name and attribute value, which correspond to desirable data type (e.g. <I>stringValue</I>, <I>boolValue</I> etc.).
 * </OL>
 * Desirable data type can be one of primitive data types or object data types.
 * <BR>
 * <BR>
 * Moreover value of attribute can be passed:
 * <OL>
 * <LI><I>statically</I> - means that you would be able to use literal of primitive data types (e.g. <I>stringvalue</I>="This is a literal",<I>boolvalue</I>="true" etc.).
 * If you want statically create instance of object data type you can use <I>serialValue</I>, which value is serialized byte stream (e.g.: <I>serialvalue</I>="092A54....").
 * This should ensure back compatibility.
 * <LI><I>dynamically</I> -means that instead of constant value (literal), you pass name of class including name of method, which will be used for dynamic creation of desirable object
 * (e.g.: <I>methodvalue</I>="org.openide.mypackage.MyClass.myMethod"). For dynamic creation of primitive data types could be used methods that return wraper objects.
 * Implemetation of interface Attr will pass to method <I>myMethod</I> two parameters FileObject (file object which maintain this atrribute) and String (name of this attribute).
 * So here is sugestion of declaration of such method: <I>public static Object myMethod(FileObject myFo,String myName)</I>.
 *
 * <A NAME="primitive"><H4>Primitive data types</H4>
 * </A>
 * Here is sugested list of attribute names for primitive data types
 * (I expect that from the name is obvious which type of value is expected):
 * <OL>
 * <I>
 * <LI> bytevalue
 * <LI> shortvalue
 * <LI> intvalue
 * <LI> longvalue
 * <LI> floatvalue
 * <LI> doublevalue
 * <LI> boolvalue
 * <LI> charvalue
 * </I>
 * </OL>
 * <BR><BR>
 *<A NAME="object"><H4>Object data types</H4></A>
 * <OL>
 * <LI> <I>methodvalue</I> - dynamic creation (for primitive data could be returned wraper objects)
  <LI> <I>newvalue</I> - newInstance is called
 * <LI> <I>serialValue</I> - static creation
 *
 * </OL>
 *
 * @author rmatous
 * @version 1.0
 */
     final class XMLMapAttr  implements Map {
    Map map;
    /** Creates new XMLMapAttr and delegetaor is instanced */
    public XMLMapAttr() {
        this.map = Collections.synchronizedMap(new HashMap ());
    }


        /** According to name of attribute returns attribute as object
     * @param p1 is name of attribute
     * @return attribute, which is hold in XMLMapAttr.Attr or null if such attribute doesn`t exist or isn`t able to construct form String representation
     */    
    public java.lang.Object get (final java.lang.Object p1) {
        Object obj;
        Attr attr = (Attr)map.get(p1);
        try {
          obj = (attr == null)?attr : attr.get();
        } catch(Exception e) {
            obj = null;            
            ExternalUtil.exception (e);                          
        }
        return obj;
    }

    /** According to name of attribute returns attribute as object
     * @param params has sense only for methodvalue invocation; and only 2 parametres will be used
     * @return attribute, which is hold in XMLMapAttr.Attr or null if such attribute doesn`t exist or isn`t able to construct form String representation
     */        
    public java.lang.Object get (final java.lang.Object p1,Object[] params) {
        Object obj;
        Attr attr = (Attr)map.get(p1);
        try {
          obj = (attr == null)?attr : attr.get(params);
        } catch(Exception e) {
            obj = null;            
            ExternalUtil.exception (e);                          
        }
        return obj;
    }
    

    /**
 * @param p1 is name of attribute
 * @param p2 is attribute as object
 * @return previous value associated with specified key, or null if there was no mapping for key. 
 * A null return can also indicate that the HashMap previously associated null with the specified key.
 */        
    public java.lang.Object put(final java.lang.Object p1,final java.lang.Object p2) {
        if (p1 == null || !(p1 instanceof String)) return null;
        if (p2 == null || p2 instanceof Attr) return map.put(Attr.decode ((String)p1),p2);
        return map.put(Attr.decode ((String)p1),new Attr(p2));
    }
        
    /** 
     * Writes heading to XML file
     * @param pw where to write
     */           
    public static void writeHeading(PrintWriter pw) {
        pw.println("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>");
        pw.println("<attributes version=\"1.0\">");// NOI18N
    }
    
    /** 
     * Writes ending to XML file
     * @param pw where to write
     */           
    public static void writeEnding(PrintWriter pw) {
        pw.println("</attributes>");// NOI18N        
    }

    /**
     * Writes all attributes for one FileObject with fileName
     * @param pw where to write
     * @param fileName
     * @param blockPrefix is prefix which is used before each line
     */
    public void write(PrintWriter pw,final String fileName,String blockPrefix) {
        String attrElement;
        boolean isHeadingWr = false;
        if (isEmpty()) return;
        
        //pw.println(blockPrefix+"<fileobject name=\""+fileName+"\">");// NOI18N
        
        Iterator entryIter = entrySet().iterator();
        
        synchronized (map) {
            while (entryIter.hasNext()) {
                Map.Entry entry = (Map.Entry)entryIter.next();

                String attrName = (String)entry.getKey();
                Attr attr = (Attr)entry.getValue();

                if (attrName == null || attr == null || attrName.length() == 0 ||  attr.isValid() == -1  ) {
                    if (attrName != null && attrName.length() != 0 && (attr == null ||  attr.isValid() == -1))
                        entryIter.remove ();
                    continue;
                }
                
                if (attr != null) attr.transformMe ();

                    
                if (!isHeadingWr) {
                    isHeadingWr = true;
                    pw.println(blockPrefix+"<fileobject name=\""+fileName+"\">");// NOI18N                
                }
                pw.println(blockPrefix+blockPrefix+"<attr name=\""+Attr.encode (attrName)+"\" "+ attr.getKey ()+"=\""+attr.getValueForPrint()+"\"/>");// NOI18N
                attr.maybeAddSerValueComment(pw,blockPrefix+blockPrefix);

            }
        }
        if (isHeadingWr) 
            pw.println(blockPrefix+"</fileobject>");// NOI18N        
    }
    

    
    public void clear() {
        map.clear();
    }
    
    public java.lang.Object remove(java.lang.Object p1) {
        return map.remove (p1);
    }
    
    public boolean containsValue(java.lang.Object p1) {
        return map.containsValue(p1);
    }
    
    public int hashCode() {
        return map.hashCode();
    }
        
    public java.util.Set keySet() {
        return map.keySet();
    }
    
    public java.util.Collection values() {
        return map.values();
    }
    
    public java.util.Set entrySet() {
        return map.entrySet();
    }
    
    public void putAll(java.util.Map p1) {
        map.putAll(p1);
    }
    
    public boolean containsKey(java.lang.Object p1) {
        return map.containsKey(p1);
    }
    
    public boolean isEmpty() {
        return map.isEmpty();
    }
    
    public boolean equals(java.lang.Object p1) {
        return map.equals(p1);
    }
  
    public int size() {
        return map.size();
    }    
    
    /**
     * Holds textual representation of one attribute. And on request construct new instance of
     * attribute a returns it as Object. Each Attr contains pair key and value. Key is type. Value is real value (in textual form) of this type.
     * Detailed describtion is in <A HREF="XMLMapAttr.html">XMLMapAttr<A>
     */            
    final static class Attr extends java.lang.Object {
    private String  value;
    private int     keyIndex;
    private Object  obj;//back compatibility
   // static final long serialVersionUID = -62733358015297232L;

    private static final  String[]  ALLOWED_ATTR_KEYS =
    {"bytevalue","shortvalue","intvalue","longvalue","floatvalue","doublevalue","boolvalue","charvalue","stringvalue","methodvalue","serialvalue","urlvalue","newvalue"};    // NOI18N
    //{"BYTEVALUE","SHORTVALUE","INTVALUE","LONGVALUE","FLOATVALUE","DOUBLEVALUE","BOOLVALUE","CHARVALUE","STRINGVALUE","METHODVALUE","SERIALVALUE","URLVALUE"};

    /**
     * @param key One of the possible keys:
     * "bytevalue","shortvalue","intvalue","longvalue","floatvalue","doublevalue","boolvalue","charvalue","stringvalue","methodvalue","serialvalue","urlvalue"
     * @param value Corresponding value to key in textual form.
     */    
    Attr(String key, String value) {
        putEntry(key,value);
    }

    /**
     * This constructor is used for backward compatibility (serializated form of filesystem.attributes).
     * Mostly NbMarshalledObject is put in this constructor.
     * @param obj Arbitrary object
     */    
    Attr(Object obj) {
        this.obj = obj;            
    }

    Attr(int index, String value) {
        this.keyIndex = index; 
        if (isValid() != -1) 
            putEntry(ALLOWED_ATTR_KEYS[this.keyIndex],value);
        else  this.value = value;
    }
    
    /**
     * @return array of Strings. Each String is textual form of allowed type of attribute - textual form of key
     */        
    static String[] getAttrTypes () {
        return ALLOWED_ATTR_KEYS;
    }

    /**
     * Checks if key is valid and sets key and value
     * @param key Key of attribute. Defines type of attribute in textual form. 
     * @param value Value of attribute. Defines value of attribute as literal or  HEX expression of serialization. 
     */            
    private  final  void putEntry(String key, String value) {
        int index;
        index = isValid(key);        
        this.keyIndex = index; 
        if (index == isValid ("stringvalue")) {
            this.value = decode (value);
            return;
        }
        this.value = value;
    }
            
    /**
     *  added for future use - convert NbMarshalledObject to primitive data types and other supported types (if possible)
     */                
    static Object unMarshallObjectRecursively(Object mo) {
        Object o = mo;
        while(o instanceof  NbMarshalledObject) {
            try {
                o = ((NbMarshalledObject)o).get ();
            } catch (IOException e) {
                ExternalUtil.exception (e);                          
                return mo;
            }
            catch (ClassNotFoundException e) {
                ExternalUtil.exception (e);                          
                return mo;
            }                    
        }
        return (o == null)? mo : o;
    }

    /**Method for back compatibility; called in write*/
    private void  transformMe () {
        int objType;
        if (obj == null) return;
        Object unObj = unMarshallObjectRecursively (obj);
        
        if (unObj != null) {            
            if ((objType = XMLMapAttr.Attr.distinguishObject (unObj)) != XMLMapAttr.Attr.isValid("SERIALVALUE")) {            
                obj = null;            
                putEntry (ALLOWED_ATTR_KEYS[objType],unObj.toString());
            } else {
                String newValue;
                try { 
                     newValue = encodeValue(unObj);                
                } catch (IOException iox) {
                    return;
                }
                obj = null;                
                putEntry(ALLOWED_ATTR_KEYS[objType],newValue);
            }
        }
    }
    
    /**
     *  added for future use - convert NbMarshalledObject to primitive data types and other supported types (if possible)
     */                
    static int distinguishObject(Object o) {
        if (o instanceof Byte)      return isValid("BYTEVALUE");// NOI18N
        if (o instanceof Short)     return isValid("SHORTVALUE");// NOI18N        
        if (o instanceof Integer)   return isValid("INTVALUE");// NOI18N 
        if (o instanceof Long)      return isValid("LONGVALUE");// NOI18N        
        if (o instanceof Float)     return isValid("FLOATVALUE");// NOI18N
        if (o instanceof Double)    return isValid("DOUBLEVALUE");// NOI18N
        if (o instanceof Boolean)   return isValid("BOOLVALUE");// NOI18N
        if (o instanceof Character) return isValid("CHARVALUE");// NOI18N
        if (o instanceof String)    return isValid("STRINGVALUE");// NOI18N         
        if (o instanceof URL)       return isValid("URLVALUE");// NOI18N            
        
        return isValid("SERIALVALUE");// NOI18N
    }
    
    static String encode (String inStr) {
        StringBuffer   outStr = new StringBuffer (6*inStr.length());
        boolean isConv;
        char[] toReplace = {'&','<','>','\"','\''};
                
        for (int i = 0; i < inStr.length(); i++) {
            isConv = false;
            for (int j = 0; j < toReplace.length; j++) {                
                if (inStr.charAt(i) == toReplace[j] ) {                                        
                    outStr.append(encodeChar (inStr.charAt(i)));
                    isConv = true; break;
                }
            }
            if (!isConv) {
                if (Character.isISOControl(inStr.charAt(i))) {
                    outStr.append(encodeChar (inStr.charAt(i)));
                    continue;
                }
            }
            if (!isConv) outStr.append(inStr.charAt(i));
        }
        return outStr.toString ();
    }

    static String encodeChar (char ch) {
        String encChar= Integer.toString((int)ch,16);        
        return "\\u"+"0000".substring(0,"0000".length()-encChar.length()).concat(encChar);
    }
    
    static String decode (String inStr) {
        StringBuffer outStr = new StringBuffer (inStr.length());
        
        for (int i = 0; i < inStr.length(); i++) {
            char ch = inStr.charAt(i);
            if ( (i+5) <   inStr.length() && ch == '\\' && inStr.charAt(i+1) == 'u' && Character.isDigit(inStr.charAt(i+2))) {
                String decChar = inStr.substring(i+2,i+6);
                outStr.append((char) Integer.parseInt(decChar,16));
                i += 5;
            }else outStr.append(ch);
        }
        
        return outStr.toString();
    }
    

    /**
     * Constructs new attribute as Object. Used for static creation from literal or serialValue.
     * @return new attribute as Object
     */            
    private Object get() throws Exception {            
        return getObject(null);//getObject is ready to aobtain null
    }

    /**
     * Constructs new attribute as Object. Used for dynamic creation: methodvalue .
     * @param params has sense only for methodvalue invocation; and only 2 parametres will be used
     *@return new attribute as Object
     */
    private Object get(Object[] obj) throws Exception {            
        return getObject(obj);
    }    
    
    
    /**
     * @return key. Key expresses type of this attribute (in textual form) or "" if internal error. 
     */                    
    final String getKey() {
        String keyArray[] = getAttrTypes();
        if (obj != null) return "serialvalue";//back compatibility // NOI18N
        if (isValid() == -1)  return "";
        return keyArray[keyIndex];
    }
    
    /**
     * @return value in textual format or "" if internal error. 
     */                        
    final String getValue() {
        if (obj != null) {
            try {
                return encodeValue(obj);//back compatibility
            } catch (IOException ioe) {
                return ""; // NOI18N
            }
        }
        return (value != null)?value : ""; // NOI18N
    }

    final String getValueForPrint() {
        if (obj != null) 
            return getValue ();
        
        if (keyIndex == isValid ("stringvalue")) {
          return (value != null)? encode (value) : ""; // NOI18N
        }
        return (value != null)?value : ""; // NOI18N
    }
    
    final void maybeAddSerValueComment(PrintWriter pw, String indent) {
        if (obj != null) {
            // Important for debugging to know what this stuff really is.
            // Note this comment is only written to disk when the attr is
            // first saved; after that successive saves will just know the
            // ser value and will not print the comment. So look at .nbattrs
            // immediately after setting something serialized. --jglick
            pw.print(indent);
            pw.print("<!-- "); // NOI18N
            String s = obj.toString();
            if (s.indexOf("--") != -1) { // NOI18N
                // XML comment no-no.
                s = s.replace('-', '_'); // NOI18N
            }
            pw.print(s);
            pw.println(" -->"); // NOI18N
        }
    }
    
    
    /**
     * Creates serialized object, which was encoded in HEX format
     * @param value Encoded serialized object in HEX format
     * @return Created object from encoded HEX format
     * @throws IOException
     */            
    static Object decodeValue(String value) throws IOException {
        if ((value == null) ||(value.length() == 0)) return null;

        byte[] bytes = new byte[value.length()/2];
        int tempI;
        int count = 0;
        for (int i = 0; i < value.length(); i += 2) {
            try {
                tempI = Integer.parseInt(value.substring(i,i+2),16);
                if (tempI > 127) tempI -=256;
                bytes[count++] = (byte) tempI;
            } catch (NumberFormatException e) {
                throw (IOException)ExternalUtil.copyAnnotation (new IOException (),e);                              
            }
        }
        
        ByteArrayInputStream bis = new ByteArrayInputStream(bytes, 0, count);
        try {
            ObjectInputStream ois = new NbObjectInputStream(bis);
            Object ret = ois.readObject();
            return ret;
        } catch (Exception e) {
            throw (IOException)ExternalUtil.copyAnnotation (new IOException (),e);                              
        }
        /*unreachable code*/
        //throw new InternalError ();
    }

    /**
     * Encodes Object into String encoded in HEX format
     * @param value Object, which will be encoded
     * @return  serialized Object in String encoded in HEX format
     * @throws IOException
     */                
    static String encodeValue(Object value) throws IOException{
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        try {
            ObjectOutputStream oos = new ObjectOutputStream(bos);
            oos.writeObject(value);
            oos.close();
        } catch (Exception e) {
            throw (IOException)ExternalUtil.copyAnnotation (new IOException (),e);                              
        }
        byte bArray[] = bos.toByteArray();
        StringBuffer strBuff = new StringBuffer(bArray.length*2);
        for(int i = 0; i < bArray.length;i++) {
            if (bArray[i] < 16 && bArray[i] >= 0) strBuff.append("0");// NOI18N
            strBuff.append(Integer.toHexString(bArray[i] < 0?bArray[i]+256:bArray[i]));            
        }
        return strBuff.toString();
    }

    /**
     * Encodes Object into String encoded in HEX format
     * @param params Array (2 length) of objects ( Object[] o = {fo,name}).  Attribute is assigned to some fo-FileObject and has its name-String. 
     * params can be null.
     * @return   Object or null
     */                
    private  Object getObject(Object[] params) throws Exception {
        int index;
        if (obj != null) return obj;//back compatibility
        if ((index = isValid()) != -1) {
            switch(index) {
                case 0:
                    return new Byte(value);
                case 1:
                    return new Short(value);
                case 2:
                    return new Integer(value);//(objI);
                case 3:
                    return  new Long(value);
                case 4:
                    return  new Float(value);
                case 5:
                    return new Double(value);
                case 6:
                    return new Boolean(value);
                case 7:
                    if (value.trim().length() != 1) break;
                    return new Character(value.charAt(0));
                case 8:
                    return new String(value);
                case 9:
                    return methodValue (value,params);
                case 10:
                    return decodeValue(value);
                case 11:
                    return new URL(value);
                case 12:
                    Class cls =  ExternalUtil.findClass (Utilities.translate (value));
                    return cls.newInstance();
            }
            
        }
        throw new InstantiationException (value);
    }

    
    /** Constructs new attribute as Object. Used for dynamic creation: methodvalue .
     * @param only 2 parametres will be used
     * @return   Object or null
     */                    
    private final Object methodValue (String value,Object[] params) throws Exception {
        String className,methodName;
        int i = value.lastIndexOf('.');
        if (i != -1) {
            
            methodName = value.substring(i+1);
            
            className = value.substring(0,i);
            Class cls =  ExternalUtil.findClass (className);
                        
            Object objArray[][] = {null,null,null};
            Method methArray[] = {null,null,null};
            
            
            Class fParam = null, sParam = null;
            
            if (params != null) {
                if (params.length > 0) fParam = params[0].getClass();
                if (params.length > 1) sParam = params[1].getClass();
            }
            
            Method[] allMethods = cls.getDeclaredMethods();
            Class[] paramClss;
            
            for (int j=0; j < allMethods.length; j++) {
                
                if (!allMethods[j].getName().equals(methodName))  continue;
                
                
                paramClss = allMethods[j].getParameterTypes();
                
                if (params == null  || params.length == 0 || paramClss.length == 0) {
                    if (paramClss.length == 0 && methArray[0] == null && objArray[0] == null) {
                        methArray[paramClss.length] = allMethods[j];
                        objArray[paramClss.length] = new Object[] {};
                        continue;
                    }
                    
                    continue;
                }
                
                
                if (paramClss.length == 2 && params.length >= 2  && methArray[2] == null && objArray[2] == null)  {
                    Class pom = paramClss[0];
                    if (paramClss[0].isAssignableFrom(fParam) && paramClss[1].isAssignableFrom(sParam)) {
                        methArray[paramClss.length] = allMethods[j];
                        objArray[paramClss.length] = new Object[] {params[0],params[1]};
                        break;
                    }
                    
                    if (paramClss[0].isAssignableFrom(sParam) && paramClss[1].isAssignableFrom(fParam)) {
                        methArray[paramClss.length] = allMethods[j];
                        objArray[paramClss.length] = new Object[] {params[1],params[0]};
                        break;
                    }
                    
                    continue;
                }
                
                if (paramClss.length == 1 && params.length >= 1 && methArray[1] == null && objArray[1] == null)  {
                    if (paramClss[0].isAssignableFrom(fParam)) {
                        methArray[paramClss.length] = allMethods[j];
                        objArray[paramClss.length] = new Object[] {params[0]};
                        continue;
                    }
                    
                    if (paramClss[0].isAssignableFrom(sParam)) {
                        methArray[paramClss.length] = allMethods[j];
                        objArray[paramClss.length] = new Object[] {params[1]};
                        continue;
                    }
                    
                    continue;
                }
                
            }

            for (int k = methArray.length-1; k >= 0; k-- ) {//clsArray.length
                if (methArray[k] != null && objArray[k] != null)  {
                    //Method meth = cls.getDeclaredMethod(methodName,clsArray[k]);
                    methArray[k].setAccessible(true); //otherwise cannot invoke private
                    return methArray[k].invoke(null,objArray[k]);
                }
            }
        }
        // Some message to logFile
        throw new InstantiationException (value);
    }

    /**
     * Checks if key is valid 
     * @return Index to array of allowed keys or -1 which means error.
     */                
    final int isValid() {
        String keyArray[] = getAttrTypes();
        if (obj != null) return isValid("SERIALVALUE");//back compatibility // NOI18N
        if (keyIndex >= keyArray.length  || keyIndex < 0) return -1;
        return keyIndex;
    }


    /**
     * Checks if key is valid 
     * @return Index to array of allowed keys or -1 which means error.
     */                    
    final static int isValid(String key) {
        int index = -1,i;
        String strArray[] = getAttrTypes();
        String trimmedKey = key.trim();
        for (i = 0; i < strArray.length;i++) {
            if (trimmedKey.equalsIgnoreCase(strArray[i]) == true) {
                index = i;
                break;
            }
        }
        return index;
    }
    }
    
}
