 /*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.openide.loaders;

import java.awt.event.KeyEvent;
import java.io.IOException;
import java.lang.ref.*;
import java.util.StringTokenizer;
import java.beans.*;
import java.util.Enumeration;
import java.util.ArrayList;
import java.util.Arrays;
import java.text.MessageFormat;

import javax.swing.tree.TreeSelectionModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.DefaultTreeSelectionModel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import javax.swing.ListSelectionModel;
import javax.swing.JButton;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;

import org.openide.TopManager;
import org.openide.DialogDescriptor;
import org.openide.WizardDescriptor;
import org.openide.loaders.*;
import org.openide.nodes.*;
import org.openide.explorer.view.*;
import org.openide.explorer.ExplorerManager;
import org.openide.filesystems.*;
import org.openide.util.UserCancelException;
import org.openide.util.HelpCtx;
import org.openide.util.Mutex;
import org.openide.util.Utilities;
import org.openide.util.enum.*;

/** Dialog that can be used in create from template.
*
* @author  Jaroslav Tulach
* @version 
*/
final class TemplateWizard2 extends javax.swing.JPanel
            implements DocumentListener, WizardDescriptor.FinishPanel, DataFilter,
    PropertyChangeListener, VetoableChangeListener {
    /** format to for default package */
    private static MessageFormat defaultPackageName;

    /** listener to changes in the wizard */
    private ChangeListener listener;

    /** system reference (FileSystem) */
    private Reference  system = new WeakReference (null);

    /** root node */
    private Node rootNode;
    
    /** File extension of the template and of the created file -
     * it is used to test whether file already exists.
     */
    private String extension;

    /** Creates new form NewFromTemplatePanel */
    public TemplateWizard2 () {
        initComponents ();

        setName (DataObject.getString("LAB_TargetLocationPanelName"));

        rootNode = createPackagesNode ();

        packagesPanel.getExplorerManager ().setRootContext (rootNode);
        packagesPanel.getExplorerManager ().addPropertyChangeListener (this);
        packagesPanel.getExplorerManager ().addVetoableChangeListener (this);


        // registers itself to listen to changes in the content of document
        packageName.getDocument().addDocumentListener(this);
        packageName.unregisterKeyboardAction(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0));

        // registers itself to listen to changes in the content of document
        className.getDocument().addDocumentListener(this);
        className.unregisterKeyboardAction(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0));

        jLabel1.setDisplayedMnemonic(java.util.ResourceBundle.getBundle("org/openide/loaders/Bundle").getString("CTL_TemplateClassName_Mnemonic").charAt(0));
        jLabel2.setDisplayedMnemonic(java.util.ResourceBundle.getBundle("org/openide/loaders/Bundle").getString("CTL_TemplatePackageName_Mnemonic").charAt(0));
        descriptionLabel.setText (org.openide.util.NbBundle.getBundle(TemplateWizard2.class).getString("LAB_TargetLocationDescription"));
        descriptionLabel.setDisplayedMnemonic (org.openide.util.NbBundle.getBundle(TemplateWizard2.class).getString("LAB_TargetLocationDescription_Mnemonic").charAt(0));

        putClientProperty("WizardPanel_contentData", new String[] { getName() }); //NOI18N
        putClientProperty("WizardPanel_contentSelectedIndex", new Integer(0)); //NOI18N
    }

    /** Preffered size */
    public java.awt.Dimension getPreferredSize() {
        return TemplateWizard.PREF_DIM;
    }

    /** Request focus.
    */
    public void requestFocus () {
        className.requestFocus();
        className.selectAll ();
    }

    /** Creates node that displays all packages.
    */
    private Node createPackagesNode () {
        Node orig = TopManager.getDefault().getPlaces().nodes ().repository(this);
        return new FilterNode (orig, new ExCh (orig));
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        namePanel = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        className = new javax.swing.JTextField();
        packagesPanel = new org.openide.explorer.ExplorerPanel();
        packageNamePanel = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        packageName = new javax.swing.JTextField();
        jLabel3 = new javax.swing.JLabel();
        directoryName = new javax.swing.JTextField();
        beanTreeView = new org.openide.explorer.view.BeanTreeView();
        descriptionLabel = new javax.swing.JLabel();
        
        setLayout(new java.awt.BorderLayout(0, 11));
        
        setPreferredSize(new java.awt.Dimension(0, 0));
        namePanel.setLayout(new java.awt.BorderLayout(12, 0));
        
        jLabel1.setText(java.util.ResourceBundle.getBundle("org/openide/loaders/Bundle").getString("CTL_TemplateClassName"));
        jLabel1.setLabelFor(className);
        namePanel.add(jLabel1, java.awt.BorderLayout.WEST);
        
        className.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                classNameFocusGained(evt);
            }
        });
        
        namePanel.add(className, java.awt.BorderLayout.CENTER);
        
        add(namePanel, java.awt.BorderLayout.NORTH);
        
        packagesPanel.setLayout(new java.awt.BorderLayout(0, 5));
        
        packageNamePanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        jLabel2.setText(java.util.ResourceBundle.getBundle("org/openide/loaders/Bundle").getString("CTL_TemplatePackageName"));
        jLabel2.setLabelFor(packageName);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.insets = new java.awt.Insets(6, 0, 5, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        packageNamePanel.add(jLabel2, gridBagConstraints1);
        
        packageName.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                packageNameFocusGained(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(6, 8, 5, 0);
        gridBagConstraints1.weightx = 1.0;
        packageNamePanel.add(packageName, gridBagConstraints1);
        
        jLabel3.setText(java.util.ResourceBundle.getBundle("org/openide/loaders/Bundle").getString("CTL_TemplateDirectoryName"));
        jLabel3.setLabelFor(directoryName);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        packageNamePanel.add(jLabel3, gridBagConstraints1);
        
        directoryName.setText("jTextField1");
        directoryName.setEnabled(false);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 8, 0, 0);
        packageNamePanel.add(directoryName, gridBagConstraints1);
        
        packagesPanel.add(packageNamePanel, java.awt.BorderLayout.SOUTH);
        
        beanTreeView.setPopupAllowed(false);
        packagesPanel.add(beanTreeView, java.awt.BorderLayout.CENTER);
        
        descriptionLabel.setLabelFor(beanTreeView);
        packagesPanel.add(descriptionLabel, java.awt.BorderLayout.NORTH);
        
        add(packagesPanel, java.awt.BorderLayout.CENTER);
        
    }//GEN-END:initComponents

    private void packageNameFocusGained (java.awt.event.FocusEvent evt) {//GEN-FIRST:event_packageNameFocusGained
        packageName.selectAll ();
    }//GEN-LAST:event_packageNameFocusGained

    private void packageModelChanged (javax.swing.event.ListDataEvent evt) {//GEN-FIRST:event_packageModelChanged
    }//GEN-LAST:event_packageModelChanged

    private void classNameFocusGained (java.awt.event.FocusEvent evt) {//GEN-FIRST:event_classNameFocusGained
        if (
            Utilities.getOperatingSystem() == Utilities.OS_SOLARIS |
            Utilities.getOperatingSystem() == Utilities.OS_SUNOS
        ) {
            // does not work on CDE window manager, so better do nothin
            return;
        }

        className.selectAll ();
    }//GEN-LAST:event_classNameFocusGained

    private void templatesTreeValueChanged (javax.swing.event.TreeSelectionEvent evt) {//GEN-FIRST:event_templatesTreeValueChanged
    }//GEN-LAST:event_templatesTreeValueChanged


    // Variables declaration - do not modify//GEN-BEGIN:variables
    javax.swing.JPanel namePanel;
    javax.swing.JLabel jLabel1;
    javax.swing.JTextField className;
    org.openide.explorer.ExplorerPanel packagesPanel;
    javax.swing.JPanel packageNamePanel;
    javax.swing.JLabel jLabel2;
    javax.swing.JTextField packageName;
    javax.swing.JLabel jLabel3;
    javax.swing.JTextField directoryName;
    org.openide.explorer.view.BeanTreeView beanTreeView;
    javax.swing.JLabel descriptionLabel;
    // End of variables declaration//GEN-END:variables


    //
    // Filter to accept only folders
    //

    /** Should the data object be displayed or not?
    * @param obj the data object
    * @return <CODE>true</CODE> if the object should be displayed,
    *    <CODE>false</CODE> otherwise
    */
    public boolean acceptDataObject(DataObject obj) {
        Object o = obj.getCookie(DataFolder.class);
        if (o == null) {
            return false;
        }
        
        try {
            FileSystem fs = obj.getPrimaryFile ().getFileSystem ();
        
            return !fs.isHidden () || system.get () == fs;
        } catch (FileStateInvalidException ex) {
            return false;
        }
    }

    /** Allow only simple selection.
    */
    public void vetoableChange(PropertyChangeEvent ev)
    throws PropertyVetoException {
        if (ExplorerManager.PROP_SELECTED_NODES.equals (ev.getPropertyName ())) {
            Node[] arr = (Node[])ev.getNewValue();

            if (arr.length > 1) {
                throw new PropertyVetoException ("Only single selection allowed", ev); // NOI18N
            }
        }
    }

    /** Changes in selected node in packages.
    */
    public void propertyChange (PropertyChangeEvent ev) {
        if (ExplorerManager.PROP_SELECTED_NODES.equals (ev.getPropertyName ())) {
            Node[] arr = packagesPanel.getExplorerManager ().getSelectedNodes ();
            if (arr.length == 1) {
                DataFolder df = (DataFolder)arr[0].getCookie (DataFolder.class);
                if (df != null) {
                    try {
                        FileObject fo = df.getPrimaryFile();
                        FileSystem fs = fo.getFileSystem();
                        system = new WeakReference (fs);
                        String name = fo.getPackageName('.');
                        packageName.getDocument ().removeDocumentListener (this);
                        packageName.setText (name);
                        updateDirectory ();
                        packageName.getDocument ().addDocumentListener (this);
                        fireStateChanged ();
                        return;
                    } catch (FileStateInvalidException fsie) {
                        system = new WeakReference(null);
                    }
                }
            }
            setTargetFolder ((DataFolder)null);
        }
    }

    /** Fires info to listener.
    */
    private void fireStateChanged () {
        if (listener != null) {
            listener.stateChanged (new ChangeEvent (this));
        }
    }

    //
    // Modification of package name or class name
    //

    public void changedUpdate(final javax.swing.event.DocumentEvent p1) {
        if (p1.getDocument () == packageName.getDocument ()) {
            SwingUtilities.invokeLater (new Runnable () {
                                            public void run () {
                                                String text = packageName.getText ();
                                                if (text != null) {
                                                    setTargetFolder (text);
                                                    updateDirectory ();
                                                }
                                            }
                                        });
            return;
        }

        if (p1.getDocument () == className.getDocument ()) {
            SwingUtilities.invokeLater (new Runnable () {
                                            public void run () {
                                                if (className.getText().equals ("")) {
                                                    setClassName ("");
                                                }

                                                fireStateChanged ();
                                            }
                                        });
        }
    }

    public void removeUpdate(final javax.swing.event.DocumentEvent p1) {
        // when deleted => do no looking for folder
        // changedUpdate (p1);
        if (p1.getDocument () == packageName.getDocument ()) {
            SwingUtilities.invokeLater(new Runnable () {
                                           public void run () {
                                               if (packageName.getText ().length () == 0) {
                                                   FileSystem fs = (FileSystem)system.get ();
                                                   if (fs != null) {
                                                       DataFolder df = DataFolder.findFolder (fs.getRoot ());
                                                       setTargetFolder (df);
                                                       packageName.selectAll ();
                                                   }
                                               }
                                               updateDirectory ();
                                           }
                                       });
        }

        // so just check the name
        if (p1.getDocument () == className.getDocument ()) {
            SwingUtilities.invokeLater (new Runnable () {
                                            public void run () {
                                                if (className.getText().equals ("")) {
                                                    setClassName ("");
                                                }

                                                fireStateChanged ();
                                            }
                                        });
        }
    }
    public void insertUpdate(final javax.swing.event.DocumentEvent p1) {
        changedUpdate (p1);
    }


    //
    // Wizard
    //

    /** Get the component displayed in this panel.
    * @return the component
    */
    public java.awt.Component getComponent () {
        return this;
    }

    /** Help for this panel.
    * @return the help or <code>null</code> if no help is supplied
    */
    public org.openide.util.HelpCtx getHelp () {
        return new HelpCtx (TemplateWizard2.class);
    }

    /** Provides the wizard panel with the current data--either
    * the default data or already-modified settings, if the user used the previous and/or next buttons.
    * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
    * @param settings the object representing wizard panel state, as originally supplied to {@link WizardDescriptor#WizardDescriptor(WizardDescriptor.Iterator,Object)}
    */
    public void readSettings (Object settings) {
        TemplateWizard wizard = (TemplateWizard)settings;
        
        DataObject template = wizard.getTemplate ();
        if (template != null) {
            extension = template.getPrimaryFile().getExt();
        }
        
        setClassName (wizard.getClassName ());

        String name = wizard.getPackageName ();
        FileSystem fs = wizard.getSystem ();
        system = new WeakReference (fs);
        requestFocus ();

        // set the current target folder.
        if (fs != null && name != null) {
            FileObject fo = fs.find (name, null, null);
            if (fo != null) {
                // try it by folder
                setTargetFolder (DataFolder.findFolder (fo));
                return;
            }
        }

        // or if the folder has not been found by name
        setTargetFolder (name);

    }

    /** Provides the wizard panel with the opportunity to update the
    * settings with its current customized state.
    * Rather than updating its settings with every change in the GUI, it should collect them,
    * and then only save them when requested to by this method.
    * Also, the original settings passed to {@link #readSettings} should not be modified (mutated);
    * rather, the (copy) passed in here should be mutated according to the collected changes.
    * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
    * @param settings the object representing a settings of the wizard
    */
    public void storeSettings (Object settings) {
        TemplateWizard wizard = (TemplateWizard)settings;

        FileSystem fs = (FileSystem)system.get ();

        String name = packageName.getText ();
        if (name.equals (defaultPackageName (fs))) {
            name = "";
        }

        wizard.setNameSystem (name, fs);

        name = className.getText ();
        if (name.equals (defaultClassName ())) {
            name = null;
        }

        wizard.setTargetName (name);
    }

    /** Test whether the panel is finished and it is safe to proceed to the next one.
    * If the panel is valid, the "Next" (or "Finish") button will be enabled.
    * @return <code>true</code> if the user has entered satisfactory information
    */
    public boolean isValid () {
        // root node to be selected does not make sense - the user
        // has to specify at least a filesystem
        String text = packageName.getText ();
        if (text.length () == 0) {
            Node[] arr = packagesPanel.getExplorerManager ().getSelectedNodes ();
            if (arr.length == 1 && arr[0] == rootNode) {
                return false;
            }
        }
        // test whether the selected name already exists
        FileSystem fs = (FileSystem)system.get ();
        if (fs != null) {
            FileObject f = fs.find(text, className.getText(), extension);
            if (f != null) {
                return false;
            }
            if (text.equals (defaultPackageName(fs))) {
                f = fs.find("", className.getText(), extension); // NOI18N
                if (f != null) {
                    return false;
                }
            }
        }
        return true;
    }

    /** Add a listener to changes of the panel's validity.
    * @param l the listener to add
    * @see #isValid
    */
    public void addChangeListener (ChangeListener l) {
        if (listener != null) throw new IllegalStateException ();

        listener = l;
    }

    /** Remove a listener to changes of the panel's validity.
    * @param l the listener to remove
    */
    public void removeChangeListener (ChangeListener l) {
        listener = null;
    }

    /** Computes a suggestion for a given prefix and
    * a list of file objects.
    *
    * @param node the node to start with
    * @param pref prefix
    * @param first [0] is the first node that satisfies the suggestion
    * @return the longest continuation string for all folders that 
    *    starts with prefix
    */
    private static String computeSuggestion (
        Node node,
        String pref,
        Node[] first
    ) {
        Node[] arr = node.getChildren ().getNodes ();

        String match = null;

        for (int i = 0; i < arr.length; i++) {
            String name = arr[i].getName ();
            if (name.startsWith (pref)) {
                // ok, has the right prefix
                if (match == null) {
                    // first match
                    match = name;
                    if (first != null) {
                        first[0] = arr[i];
                    }
                } else {
                    // find common part of the names
                    int indx = pref.length ();
                    int end = Math.min (name.length (), match.length ());
                    while (indx < end && match.charAt (indx) == name.charAt (indx)) {
                        indx++;
                    }
                    match = match.substring (0, indx);
                }
            }
        }

        if (match == null || match.length () == pref.length ()) {
            return null;
        } else {
            return match.substring (pref.length ());
        }
    }


    /** Presets a target folder.
    * @param f the folder
    * @return true if succeeded
    */
    private boolean setTargetFolder (final DataFolder f) {
        boolean exact;
        Node n;
        String name;


        if (f != null) {
            FileObject fo = f.getPrimaryFile ();
            name = fo.getPackageName('.');

            StringTokenizer st = new StringTokenizer (name, "."); // NOI18N
            try {
                FileSystem fs = fo.getFileSystem ();

                if (fo.isRoot ()) {
                    name = defaultPackageName (fs);
                }

                system = new WeakReference (fs);

                Enumeration en = new SequenceEnumeration (
                                     new SingletonEnumeration (fs.getSystemName()),
                                     st
                                 );

                n = NodeOp.findPath (rootNode, en);
                exact = true;
            } catch (FileStateInvalidException ex) {
                // invalid state of file system => back to root
                n = rootNode;
                name = ""; // NOI18N
                exact = false;
            } catch (NodeNotFoundException ex) {
                n = ex.getClosestNode();
                DataFolder df = (DataFolder)n.getCookie (DataFolder.class);
                if (df != null) {
                    name = df.getPrimaryFile ().getPackageName ('.');
                } else {
                    name = "";
                }
                exact = false;
            }


        } else {
            // null folder => use root
            n = rootNode;
            name = null;
            exact = true;
        }

        // remove listener + do change + add listener
        ExplorerManager em = packagesPanel.getExplorerManager ();
        em.removePropertyChangeListener (this);
        packageName.getDocument ().removeDocumentListener (this);

        try {
            em.setSelectedNodes (new Node[] { n });
        } catch (PropertyVetoException ex) {
            throw new InternalError ();
        }

        packageName.setText (name);
        updateDirectory ();

        packageName.getDocument ().addDocumentListener (this);
        em.addPropertyChangeListener (this);

        fireStateChanged ();

        return exact;
    }

    /** Presets a target folder.
    * @param f the name of target folder
    * @return true if succeeded
    */
    private boolean setTargetFolder (final String f) {
        Node n = null;
        NodeNotFoundException closest = null;

        // first of all test the currently selected nod
        // for location of closest
        java.util.Collection selected = new java.util.HashSet (
                                            java.util.Arrays.asList (
                                                packagesPanel.getExplorerManager().getSelectedNodes()
                                            )
                                        );

        // scan
        Node[] arr = rootNode.getChildren ().getNodes ();

        for (int i = 0; i < arr.length; i++) {
            Node root = arr[i];

            StringTokenizer st = new StringTokenizer (f, "."); // NOI18N

            try {
                n = NodeOp.findPath (root, st);
                break;
            } catch (NodeNotFoundException ex) {
                if (!st.hasMoreElements ()) {
                    // a test for !hasMoreElements is here to be sure that
                    // all tokens has been read, so only the last item
                    // has not been found

                    // check whether we can continue from the nod
                    final String sugg = computeSuggestion (
                                            ex.getClosestNode (),
                                            ex.getMissingChildName(),
                                            null
                                        );

                    if
                    (
                        closest == null || selected.contains (root
                                                             ) && sugg != null
                    )
                        // if we can go on and there has been no suggestion o
                        // this is the current filesystem => go o
                        closest = ex;

                }
            }
        }

        if (n != null) {
            // closest node not used
            closest = null;
        } else {

            if (closest == null) {
                // the node has not been even found
                return false;
            }

            // we will select the closest node found
            n = closest.getClosestNode ();
        }

        // remove listener + do change + add listener
        ExplorerManager em = packagesPanel.getExplorerManager ();
        em.removePropertyChangeListener (this);

        // change the text if we want to add suggestion
        if (closest != null) {
            Node[] first = new Node[1];
            final String sugg = computeSuggestion (
                                    closest.getClosestNode (),
                                    closest.getMissingChildName(),
                                    first
                                );

            if (sugg != null) {
                packageName.getDocument ().removeDocumentListener (
                    TemplateWizard2.this
                );

                packageName.setText (f + sugg);
                updateDirectory ();

                javax.swing.text.Caret c = packageName.getCaret ();
                c.setDot (f.length () + sugg.length ());
                c.moveDot (f.length ());

                packageName.getDocument ().addDocumentListener (
                    TemplateWizard2.this
                );
            }

            if (first[0] != null) {
                // show the first node that fits
                n = first[0];
            }
        }


        // change the node
        try {
            em.setSelectedNodes (new Node[] { n });
            beanTreeView.expandNode (n);
        } catch (PropertyVetoException ex) {
            throw new InternalError ();
        }

        // change the selected filesystem
        DataFolder df = (DataFolder)n.getCookie (DataFolder.class);
        if (df != null) {
            try {
                FileSystem fs = df.getPrimaryFile ().getFileSystem ();
                system = new WeakReference (fs);
            } catch (FileStateInvalidException ex) {
            }
        }


        em.addPropertyChangeListener (this);

        fireStateChanged ();

        return closest == null;
    }

    /** Sets the class name to some reasonable value.
    * @param name the name to set the name to
    */
    private void setClassName (String name) {
        String n = name;
        if (name == null || name.length () == 0) {
            n = defaultClassName ();
        }

        className.getDocument().removeDocumentListener(this);
        className.setText (n);
        className.getDocument().addDocumentListener(this);

        if (name == null || name.length () == 0) {
            className.selectAll ();
        }
    }


    /** Creates default package name for given file system.
    * @param fs the file system
    * @return localized name of default package
    */
    private static String defaultPackageName (FileSystem fs) {
        if (defaultPackageName == null) {
            defaultPackageName = new MessageFormat (DataObject.getString (
                                                        "FMT_TemplateDefaultPackageName"
                                                    ));
        }

        String n = fs == null ? "" : fs.getDisplayName ();

        return defaultPackageName.format (new Object[] { n });
    }

    /** Getter for default name of a class.
    * @return the default name.
    */
    private static String defaultClassName () {
        return DataObject.getString ("FMT_TemplateDefaultClassName");
    }

    /** Updates directory name
    */
    private void updateDirectory () {
        StringBuffer sb = new StringBuffer ();
        FileSystem fs = (FileSystem)system.get ();
        if (fs != null) {
            sb.append (fs.getDisplayName ());
        }
        String name = packageName.getText ();
        if (name.equals (defaultPackageName (fs))) {
            name = "";
        }
        if (name.length () > 0) {
            sb.append (java.io.File.separatorChar);
            sb.append (name.replace ('.', java.io.File.separatorChar));
        }
        directoryName.setText (sb.toString ());
    }
    
    /** Special children that also display the system directory if necessary
     */
    private final class ExCh extends FilterNode.Children {
        public ExCh (Node or) {
            super (or);
        }
        
        /** Update the keys.
         */
        protected void addNotify () {
            updateKeys ();
        }
        
        /** Called when the filter node removes a child.
         * The default implementation makes a corresponding change.
         * @param ev info about the change
         */
        protected void filterChildrenRemoved(NodeMemberEvent ev) {
            updateKeys ();
        }
        
        /** Called when the filter node reorders its children.
         * The default implementation makes a corresponding change.
         * @param ev info about the change
         */
        protected void filterChildrenReordered(NodeReorderEvent ev) {
            updateKeys ();
        }
        
        /** Create nodes representing copies of the original node's children.
         * The default implementation returns exactly one representative for each original node,
         * as returned by {@link #copyNode}.
         * Subclasses may override this to avoid displaying a copy of an original child at all,
         * or even to display multiple nodes representing the original.
         * @param key the original child node
         * @return zero or more nodes representing the original child node
         */
        protected Node[] createNodes(Object key) {
            if (key == this) {
                // check the system filesystem
                FileSystem fs = (FileSystem)system.get ();
                if (fs != null && fs.isHidden ()) {
                    try {
                        // we have to add the filesystem, otherwise will not be visible
                        final DataFolder df = DataFolder.findFolder (fs.getRoot ());
                        final Children ch = df.createNodeChildren (TemplateWizard2.this);
                        class NoSynchBeanNode extends BeanNode {
                            public NoSynchBeanNode (Object bean, Children kids) throws IntrospectionException {
                                super (bean, kids);
                                setSynchronizeName (false);
                            }
                        }
                        Node bn = new NoSynchBeanNode (fs, ch);
                        bn.setName (fs.getSystemName ());
                        bn.setDisplayName (fs.getDisplayName ());
                        return new Node[] { bn };
                    } catch (IntrospectionException ex) {
                        TopManager.getDefault ().getErrorManager ().notify (ex);
                    }
                }
                return new Node[0];
            }
            
            return super.createNodes(key);
        }
        
        /** Called when the filter node adds a new child.
         * The default implementation makes a corresponding change.
         * @param ev info about the change
         */
        protected void filterChildrenAdded(NodeMemberEvent ev) {
            updateKeys ();
        }

        /** Updates the keys.
         */
        private void updateKeys () {
            Node[] arr = original.getChildren ().getNodes ();
            ArrayList al = new ArrayList (arr.length + 1);
            al.add (this);
            for (int i = 0; i < arr.length; i++) {
                al.add (arr[i].cloneNode ());
            }
            
            setKeys (al);
        }
    }
    
}
