/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.openide.text;

import java.awt.BorderLayout;
import java.awt.Toolkit;
import java.awt.event.*;
import java.awt.datatransfer.Transferable;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.io.*;
import java.util.*;
import java.text.MessageFormat;
import java.awt.print.PrinterJob;
import java.awt.print.Pageable;
import java.awt.print.Printable;
import java.awt.print.PageFormat;
import java.awt.print.PrinterException;
import java.awt.print.PrinterIOException;
import java.awt.print.PrinterAbortException;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Component;
import java.lang.ref.WeakReference;

import javax.swing.Action;
import javax.swing.JEditorPane;
import javax.swing.JScrollPane;
import javax.swing.JButton;
import javax.swing.SwingUtilities;
import javax.swing.text.*;
import javax.swing.event.HyperlinkListener;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.html.HTMLFrameHyperlinkEvent;
import javax.swing.text.html.HTMLDocument;
import javax.swing.undo.CannotUndoException;

import org.openide.awt.UndoRedo;
import org.openide.actions.*;
import org.openide.TopManager;
import org.openide.NotifyDescriptor;
import org.openide.cookies.EditorCookie;
import org.openide.cookies.OpenCookie;
import org.openide.cookies.CloseCookie;
import org.openide.cookies.SaveCookie;
import org.openide.cookies.PrintCookie;
import org.openide.filesystems.*;
import org.openide.loaders.*;
import org.openide.windows.*;
import org.openide.util.Task;
import org.openide.util.TaskListener;
import org.openide.util.WeakListener;
import org.openide.util.actions.SystemAction;
import org.openide.util.actions.CallbackSystemAction;
import org.openide.util.actions.CallableSystemAction;
import org.openide.util.actions.ActionPerformer;
import org.openide.util.datatransfer.PasteType;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;


/** Cloneable top component to hold the editor kit.
 */
public class CloneableEditor extends CloneableTopComponent {
    /** editor pane  */
    protected JEditorPane pane;

    /** Listener for copy action enabling  */
    private PropertyChangeListener copyL;

    /** Listener for cut action enabling  */
    private PropertyChangeListener cutL;

    /** Listener for data object's save cookie changes  */
    private PropertyChangeListener saveCookieL;

    /** Asociated editor support  */
    private CloneableEditorSupport support;

    /** keeps the instance of focus event for manipulating with the
     * caret visibility  */
    private static WeakReference mutableFocusEvent = new WeakReference (null);

    private static final String HELP_ID = "editing.editorwindow"; // !!! NOI18N
    
    static final long serialVersionUID = -185739563792410059L;
    
    /** For externalization of subclasses only  */
    public CloneableEditor() {
    }
    
    /** Creates new editor component associated with
    * support object.
    * @param support support that holds the document and operations above it
    */
    public CloneableEditor(CloneableEditorSupport support) {
       super();
       this.support = support;
       
       initialize ();
    }
    
    /** Gives access to support object under this CloneableEditor.
    * 
    * @return the support object that holds the document
    */
    protected CloneableEditorSupport cloneableEditorSupport () {
        return support;
    }

    /** Get context help for this editor pane.
     * If the registered editor kit provides a help ID in bean info
     * according to the protocol described for {@link InstanceSupport#findHelp},
     * then that it used, else general help on the editor is provided.
     * @return context help
     */
    public HelpCtx getHelpCtx() {
        HelpCtx fromKit = InstanceSupport.findHelp (new InstanceSupport.Instance (support.kit ()));
        if (fromKit != null)
            return fromKit;
        else
            return new HelpCtx (HELP_ID);
    }


    /** Make sure the EditorKit knows it is going bye-bye.  */
    public boolean canClose(Workspace workspace,boolean last) {
        boolean result = super.canClose(workspace, last);
        if (result) {
            SwingUtilities.invokeLater(
                new Runnable() {
                    public void run() {
                        Document doc = support.createStyledDocument (pane.getEditorKit ());
                        pane.setDocument (doc);
                        pane.setEditorKit (null);
                        removeAll();
                    }
                }
            );
        }
        return result;
    }



    /** Performs needed initialization  */
    private void initialize() {
        setCloseOperation (TopComponent.CLOSE_EACH);
        setLayout (new BorderLayout ());

        QuietEditorPane pane = new QuietEditorPane ();

        this.pane = pane;

        //      pane.setWorking(false);
        pane.setEditorKit (support.kit ());

        updateName();

/* JST:
        if (support.originalDoc instanceof HTMLDocument) {
            try {
                ((HTMLDocument)support.originalDoc).setBase (
                    support.getEntry ().getFile ().getURL ());
            } catch (FileStateInvalidException e) {
            }
            pane.addHyperlinkListener ( new HyperlinkListener () {
                                            public void hyperlinkUpdate (HyperlinkEvent e) {
                                                if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
                                                    if (e instanceof HTMLFrameHyperlinkEvent) {
                                                        HTMLFrameHyperlinkEvent  evt = (HTMLFrameHyperlinkEvent)e;
                                                        HTMLDocument doc = (HTMLDocument)pane.getDocument();
                                                        doc.processHTMLFrameHyperlinkEvent(evt);
                                                    } else {
                                                        try {
                                                            pane.setPage(e.getURL());
                                                        } catch (Exception ex) {
                                                            if (System.getProperty ("netbeans.debug.exceptions") != null) ex.printStackTrace(); // NOI18N
                                                        }
                                                    }
                                                }
                                            }
                                        });
            if (support.actions != null)
                add (SystemAction.createToolbarPresenter (support.actions),
                     BorderLayout.NORTH);
        }
        // listen to save cookie changes in asociated data object
        saveCookieL = new PropertyChangeListener() {
                          public void propertyChange(PropertyChangeEvent evt) {
                              if (DataObject.PROP_COOKIE.equals(evt.getPropertyName()) ||
                                      DataObject.PROP_NAME.equals(evt.getPropertyName())) {
                                  updateName();
                              }
                          }
                      };
        this.obj.addPropertyChangeListener(WeakListener.propertyChange(saveCookieL, this.obj));
        //      pane.setWorking(QuietEditorPane.FIRE);
*/      
        Document doc = support.getDocumentHack ();
        
        pane.setDocument (doc);
        
        if (doc instanceof NbDocument.CustomEditor) {
            NbDocument.CustomEditor ce = (NbDocument.CustomEditor)doc;
            add(
                ce.createEditor(pane),
                BorderLayout.CENTER
            );
        } else { // not custom editor
            add (new JScrollPane (pane), BorderLayout.CENTER);
        }

        pane.setWorking(QuietEditorPane.ALL);
    }

    protected CloneableTopComponent createClonedObject() {
        return support.createCloneableTopComponent ();
    }

    /** Overrides superclass version. Opens top component only if
     * it is in valid state.
     * (Editor top component may become invalid after deserialization).<br>
     * Also tries to open all other top components which are docked
     * in editor mode on given workspace, but not visible.<br>
     */
    public void open(Workspace workspace) {
        if (!discard ()) {
            Workspace realWorkspace = (workspace == null)
                                      ? TopManager.getDefault().getWindowManager().getCurrentWorkspace()
                                      : workspace;
            dockIfNeeded(realWorkspace);
            boolean modeVisible = false;
            TopComponent[] tcArray = editorMode(realWorkspace).getTopComponents();
            for (int i = 0; i < tcArray.length; i++) {
                if (tcArray[i].isOpened(realWorkspace)) {
                    modeVisible = true;
                    break;
                }
            }
            if (!modeVisible) {
                openOtherEditors(realWorkspace);
            }
            super.open(workspace);
            openOnOtherWorkspaces(realWorkspace);
        }
    }

    /** When closing last view, also close the document.
     * @return <code>true</code> if close succeeded
     */
    protected boolean closeLast() {
        if (!support.canClose ()) {
            // if we cannot close the last window
            return false;
        }
        // close everything and do not ask
        support.notifyClosed ();

        if (support.lastSelected == this) {
            support.lastSelected = null;
        }

        return true;
    }

    /** The undo/redo manager of the support.
     * @return the undo/redo manager shared by all editors for this support
     */
    public UndoRedo getUndoRedo() {
        return support.getUndoRedo ();
    }

    public SystemAction[] getSystemActions() {
        SystemAction[] sa = super.getSystemActions ();

        return SystemAction.linkActions (sa, 
            new SystemAction[] { null, SystemAction.get(FileSystemAction.class) }
        );
    }

    /** Transfer the focus to the editor pane.
     */
    public void requestFocus() {
        super.requestFocus ();
        pane.requestFocus ();
    }

    /** @return Preferred size of editor top component  */
    public Dimension getPreferredSize() {
        Rectangle bounds = TopManager.getDefault().getWindowManager().
                           getCurrentWorkspace().getBounds();
        return new Dimension(bounds.width / 2, bounds.height / 2);
    }


    /* JST: Ugly */
    Hashtable kitActions;
    private Action getAction (String s) {
        if (kitActions == null) {
            EditorKit kit = support.kit ();
            if (kit == null) { // kit is cleared in closeDocument()
                return null;
            }

            Action[] a = kit.getActions ();
            kitActions = new Hashtable (a.length);
            int i, k = a.length;
            for (i = 0; i < k; i++)
                kitActions.put (a [i].getValue (Action.NAME), a [i]);
        }
        return (Action) kitActions.get (s);
    }

    protected void componentActivated() {
        final Action copy = getAction (DefaultEditorKit.copyAction);
        if (copy != null) {
            final CallbackSystemAction sysCopy
            = ((CallbackSystemAction) SystemAction.get (CopyAction.class));
            sysCopy.setActionPerformer (
                new ActionPerformer () {
                    public void performAction (SystemAction action) {
                        copy.actionPerformed (new ActionEvent (CloneableEditor.this, 0, "")); // NOI18N
                    }
                }
            );
            copy.addPropertyChangeListener(
                copyL = new PropertyChangeListener() {
                            public void propertyChange(PropertyChangeEvent evt) {
                                if ("enabled".equals(evt.getPropertyName())) { // NOI18N
                                    sysCopy.setEnabled(((Boolean)evt.getNewValue()).booleanValue());
                                }
                            }
                        }
            );
            sysCopy.setEnabled(copy.isEnabled());
        }

        final Action cut = getAction (DefaultEditorKit.cutAction);
        if (cut != null) {
            final CallbackSystemAction sysCut
            = ((CallbackSystemAction) SystemAction.get (CutAction.class));
            sysCut.setActionPerformer (
                new ActionPerformer () {
                    public void performAction (SystemAction action) {
                        cut.actionPerformed (new ActionEvent (CloneableEditor.this, 0, "")); // NOI18N
                    }
                }
            );
            cut.addPropertyChangeListener(
                cutL = new PropertyChangeListener() {
                           public void propertyChange(PropertyChangeEvent evt) {
                               if ("enabled".equals(evt.getPropertyName())) { // NOI18N
                                   sysCut.setEnabled(((Boolean)evt.getNewValue()).booleanValue());
                               }
                           }
                       }
            );
            sysCut.setEnabled(cut.isEnabled());
        }

        final Action paste = getAction (DefaultEditorKit.pasteAction);
        if (paste != null) {
            PasteAction sysPaste
            = ((PasteAction) SystemAction.get (PasteAction.class));
            sysPaste.setPasteTypes (
                new PasteType[] {
                    new PasteType () {
                        public Transferable paste () {
                            paste.actionPerformed (new ActionEvent (CloneableEditor.this, 0, "")); // NOI18N
                            return null;
                        }

                    }
                }
            );
        }
        // commented as this caused some problems in focus managing
        //pane.requestFocus ();
        support.lastSelected = this;
        // HACKING focus problems
        // to prevent from bug #3432 (cursor is invisible)
        // beware, this is UGLY code, ideas welcomed
        SwingUtilities.invokeLater(new Runnable() {
                                       public void run () {
                                           Caret ourCaret = pane.getCaret();
                                           if (ourCaret instanceof FocusListener) {
                                               ((FocusListener)ourCaret).focusGained(mutableFocusEvent(CloneableEditor.this));
                                           }
                                       }
                                   });
    }

    protected void componentDeactivated() {
        final Action copy = getAction (DefaultEditorKit.copyAction);
        if (copy != null) {
            copy.removePropertyChangeListener(copyL);
        }

        final Action cut = getAction (DefaultEditorKit.cutAction);
        if (cut != null) {
            cut.removePropertyChangeListener(cutL);
        }
        // HACKING focus problems
        // to prevent from bug #3432 (cursor is invisible)
        // beware, this is UGLY code, ideas welcomed
        SwingUtilities.invokeLater(new Runnable() {
                                       public void run () {
                                           Caret ourCaret = pane.getCaret();
                                           if (ourCaret instanceof FocusListener) {
                                               ((FocusListener)ourCaret).focusLost(mutableFocusEvent(CloneableEditor.this));
                                           }
                                       }
                                   });
    }

    /** Updates the name of this top component according to
     * the existence of the save cookie in ascoiated data object.
     * Updates tooltip also.
     */
    protected void updateName() {
        setName (cloneableEditorSupport ().messageName ());
        setToolTipText (cloneableEditorSupport ().messageToolTip ());
/* old code
        // update name
        String name = CloneableEditor.this.obj.getNodeDelegate().getDisplayName();
        if (CloneableEditor.this.obj.getCookie(SaveCookie.class) != null)
            setName(name + support.modifiedAppendix);
        else
            setName(name);
        // update tooltip
        FileObject fo = support.getEntry().getFile();
        StringBuffer fullName = new StringBuffer(fo.getPackageName('.'));
        String extension = fo.getExt();
        if (extension.length() > 0) {
            fullName.append(" ["); // NOI18N
            fullName.append(extension);
            fullName.append(']');
        }
        setToolTipText(fullName.toString());
*/
    }

    public void writeExternal(ObjectOutput out) throws IOException {
        super.writeExternal(out);

        // save our data object
        out.writeObject(support.env ());
        // save cursor position
        out.writeObject(new Integer(pane.getCaret().getDot()));
        //      System.out.println(getName() + " written!"); // NOI18N
    }

    public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
        //      System.out.println("Reading editor TC..."); // NOI18N
        super.readExternal(in);

        int offset;

        Object firstObject = in.readObject ();
        if (firstObject instanceof DataObject) {
            // backward compatibility

            // read data object
            DataObject obj = (DataObject)firstObject;
            // load cursor position
            offset = ((Integer)in.readObject()).intValue();
            // try to load or associate editor support
            Object maybe = in.readObject();
            if (maybe instanceof CloneableEditorSupport)
                support = (CloneableEditorSupport)maybe;
            else {
                support = (CloneableEditorSupport)obj.getCookie(CloneableEditorSupport.class);
		if (support == null) {
		    EditorSupport supp = (EditorSupport)obj.getCookie(EditorSupport.class);
		    if (supp != null)
			support = supp.del;
		}
	    }

            /* JST: Cannot be here

            // ensure that file still can be accessed
            FileObject fo = support.getEntry().getFile();
            try {
                InputStream is = fo.getInputStream();
                is.close();
            } catch (FileNotFoundException exc) {
                // warn user
                String message = NbBundle.getBundle(EditorSupport.class).
                                 getString("FMT_FileNotFound"); // NOI18N
                TopManager.getDefault().getStdOut().println(
                    MessageFormat.format(message, new Object [] {
                                             fo.getPackageNameExt('/', '.')
                                         })
                );
                // signalize to readResolve method that we should
                // deserialize to null -> this will encourage window system
                // to discard this top component
                discard = true;
                return;
                //throw new org.openide.util.io.SafeException(exc);
            }
            support.openDocument();

            */
        } else {
            // new deserialization that uses Env environment
            CloneableOpenSupport.Env env = (CloneableOpenSupport.Env)firstObject;
            CloneableOpenSupport os = env.findCloneableOpenSupport ();
            if (os instanceof EditorSupport) {
                support = ((EditorSupport)os).del;
            } else {
                support = (CloneableEditorSupport)os;
            }
            // load cursor position
            offset = ((Integer)in.readObject()).intValue();

        }

        if (!discard ()) {
            // initialization of support is ok, now setup the document
            // and also reset the caret
            final Task t = support.prepareDocument ();

            initialize();

            // set the caret to right possition
            final int of = offset;
            t.addTaskListener (new TaskListener () {
                public void taskFinished (Task tt) {
                    pane.getCaret().setDot(of);
                    t.removeTaskListener (this);
                }
            });
        }
    }

    /** If this component should be discarded then return null.
    */
    protected Object readResolve() throws ObjectStreamException {
        if (discard ()) {
            return null;
        } else {
            support.initializeCloneableEditor (this);
            return this;
        }
    }


    /** This component should be discarded if the associated environment
    * is not valid.
    */
    private boolean discard () {
        return support == null || !support.env ().isValid ();
    }











    /** Dock this top component to editor mode if it is not docked
     * in some mode at this time  */
    private void dockIfNeeded(Workspace workspace) {
        // dock into editor mode if possible
        Mode ourMode = workspace.findMode(this);
        if (ourMode == null) {
            editorMode(workspace).dockInto(this);
        }
    }

    private Mode editorMode(Workspace workspace) {
        Mode ourMode = workspace.findMode(this);
        if (ourMode == null) {
            ourMode = workspace.createMode(
                          CloneableEditorSupport.EDITOR_MODE, getName(),
                          CloneableEditorSupport.class.getResource(
                              "/org/openide/resources/editorMode.gif" // NOI18N
                          )
                      );
        }
        return ourMode;
    }

    /** Utility method, opens this top component on all workspaces
     * where editor mode is visible and which differs from given
     * workspace.  */
    private void openOnOtherWorkspaces(Workspace workspace) {
        Workspace[] workspaces =
            TopManager.getDefault().getWindowManager().getWorkspaces();
        Mode curEditorMode = null;
        Mode tcMode = null;
        for (int i = 0; i < workspaces.length; i++) {
            // skip given workspace
            if (workspaces[i].equals(workspace)) {
                continue;
            }
            curEditorMode = workspaces[i].findMode(CloneableEditorSupport.EDITOR_MODE);
            tcMode = workspaces[i].findMode(this);
            if (
                !isOpened(workspaces[i]) &&
                curEditorMode != null &&
                (
                    tcMode == null ||
                    tcMode.equals(curEditorMode)
                )
            ) {
                // candidate for opening, but mode must be already visible
                // (= some opened top component in it)
                TopComponent[] tcArray = curEditorMode.getTopComponents();
                for (int j = 0; j < tcArray.length; j++) {
                    if (tcArray[j].isOpened(workspaces[i])) {
                        // yep, open this top component on found workspace too
                        pureOpen(this, workspaces[i]);
                        break;
                    }
                }
            }
        }
    }

    /** Utility method, opens top components which are opened
     * in editor mode on some other workspace.
     * This method should be called only if first top component is
     * being opened in editor mode on given workspace  */
    private void openOtherEditors(Workspace workspace) {
        // choose candidates for opening
        Set topComps = new HashSet(15);
        Workspace[] wsArray =
            TopManager.getDefault().getWindowManager().getWorkspaces();
        Mode curEditorMode = null;
        TopComponent[] tcArray = null;
        for (int i = 0; i < wsArray.length; i++) {
            curEditorMode = wsArray[i].findMode(CloneableEditorSupport.EDITOR_MODE);
            if (curEditorMode != null) {
                tcArray = curEditorMode.getTopComponents();
                for (int j = 0; j < tcArray.length; j++) {
                    if (tcArray[j].isOpened(wsArray[i])) {
                        topComps.add(tcArray[j]);
                    }
                }
            }
        }
        // open choosed candidates
        for (Iterator iter = topComps.iterator(); iter.hasNext(); ) {
            pureOpen((TopComponent)iter.next(), workspace);
        }
    }

    /** Utility method, calls super version of open if given
     * top component is of Editor type, or calls regular open otherwise.
     * The goal is to prevent from cycle open call between
     * Editor top components  */
    private void pureOpen(TopComponent tc,Workspace workspace) {
        if (tc instanceof CloneableEditor) {
            ((CloneableEditor)tc).dockIfNeeded(workspace);
            ((CloneableEditor)tc).superOpen(workspace);
        } else {
            tc.open(workspace);
        }
    }

    private void superOpen(Workspace workspace) {
        super.open(workspace);
    }


    /** Accessor to the mutable focus event. We keep the focus
     * event as mutable, because focus chnages may come frequently
     * and there is no need to create the venet again and again.  */
    private static FocusEvent mutableFocusEvent(Component source) {
        FocusEvent ev = (FocusEvent)mutableFocusEvent.get ();
        if (ev == null) {
            ev = new FocusEvent(
                     source,
                     FocusEvent.FOCUS_FIRST + FocusEvent.FOCUS_LAST
                 );
            mutableFocusEvent = new WeakReference (ev);
        }
        return ev;
    }


}
