/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.openide.xml;

import java.io.*;
import java.util.*;

import javax.swing.event.*;

import org.xml.sax.*;

import org.openide.loaders.*;
import org.openide.cookies.InstanceCookie;
import org.openide.util.*;
import org.openide.TopManager;

/**
 * Entity resolver resolving all entities registered by modules.
 * Use getDefault() to get the only instance in system. Any parser working with
 * unknown XML documents should use it to avoid unneccessary Internet
 * connections.
 *
 * @see     EntityResolverChain
 * @author  Petr Kuzel
 * @since   release 3.2
 * @version 1.0
 */
public abstract class EntityCatalog implements EntityResolver {
    /** 
     * DOCTYPE public id definig grammar used for entity registrartions.
     */
    public static final String PUBLIC_ID = "-//NetBeans//Entity Mapping Registration 1.0//EN";

    /**
     * The only instance of EntityCatalog.
     */
    private static EntityCatalog instance;

    /**
     * Return the only instance of EntityCatalog in JVM.
     */
    public static EntityCatalog getDefault() {
        if (instance != null) {
            return instance;
        } else {
            return initDefault ();
        }
    }

    /** Synchronized initialization.
     */
    private static synchronized EntityCatalog initDefault () {
        if (instance == null) {
            instance = new Forwarder ();
        }
        
        return instance;
    }

    /**
     * This catalog is forwarding implementation.
     */
    private static class Forwarder extends EntityCatalog {
        private Lookup.Result result;
        
        Forwarder() {
            Lookup.Template temp = new Lookup.Template (EntityCatalog.class);
            result = Lookup.getDefault ().lookup (temp);
        }
        
        public InputSource resolveEntity(String publicID, String systemID) throws IOException, SAXException {
            Iterator it = result.allInstances().iterator ();
            while (it.hasNext ()) {
                // using resolver's method because EntityCatalog extends EntityResolver
                EntityResolver res = (EntityResolver)it.next ();
                InputSource is = res.resolveEntity (publicID, systemID);
                if (is != null) {
                    return is;
                }
            }
            return null;
        }
    }
        
}
