/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.openidex.jarpackager;

import java.io.*;

/** 
 * An implementation of ArchiveEntry which allows the contents of a 
 * String to become an entry in a jar file.  This can be used for many 
 * purposes where the entry consists of a manageable amount of text.
* @deprecated Note that the interfaces and classes contained in 
* org.openidex.jarpackager are for use with version 1 of the jarpackager 
* module.  Starting with version 2, use the versions in 
* org.netbeans.modules.jarpackager.api .
 */
public class StringArchiveEntry implements ArchiveEntry {

    /**
     * The name of the file to be placed in archive.
     */
    private String name;

    /**
     * The contents of the file to be placed in archive.
     */
    private String contents;

    /**
     * The contents as a byte array
     */
    private byte byteContents[];

    /**
     * Constructs an entry with given name.
     * The contents of the file is provided in a String parameter.
     * 
     * @param name     The name of the file to be placed in archive.
     * @param contents The contents of the file to be placed in archive.
     */
    public StringArchiveEntry(String name, String contents) {
        this.name = name;
        this.contents = contents;
	this.byteContents = null;
    }

    /**
     * Return the name of the entry
     * 
     * @return the name of the entry
     */
    public String getName () {
        return name;
    }

    /**
     * Return an input stream for the File contents.
     * 
     * @return input stream object
     * @exception IOException
     */
    public InputStream createInputStream () throws IOException {
        return new ByteArrayInputStream(getByteContents());
    }

    /**
     * Return size of file represented by file object
     * 
     * @return Size of file represented by file object
     */
    public long getSize () {
        return getByteContents().length;
    }

    /**
     * Get byte version of String, converting if need be
     */
    private synchronized byte[] getByteContents() {
	if (byteContents == null) {	
	    byteContents = this.contents.getBytes();
	}
	return byteContents;
    }
}

