/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.openidex.windows;

import java.io.*;

import org.openide.windows.*;

// ---- SEE ALSO org.netbeans.core.NonGuiMain ----

/** Implementation of InputOutput that uses regular
 * InputStream and OutputStream to perform the IO operation.
 * @author David Strupl
 */
public class StandardStreamInputOutput implements InputOutput {
    /** Closing state. */
    private boolean isClosed;
    /** Reader created from the InputStream in.*/
    private Reader  reader;
    /** OutputWriter created from out*/
    private MyOutputWriter myOut;
    /** Parameter */
    private InputStream in;
    /** Parameter */
    private OutputStream out;
    /** Name fo the tab - used for printing the messages.*/
    private String name;
    
    /** The only public constructor - all parameters are required.*/
    public StandardStreamInputOutput(String name,InputStream in,OutputStream out) {
        this.isClosed = false;
        this.in = in;
        this.out = out;
        this.name = name;
    }
    
    /** Lazy initialization of myOut.*/
    public OutputWriter getOut() {
        if (myOut == null) {
            myOut = new MyOutputWriter(new PrintWriter(out));
        }
        return myOut;
    }
    
    /** Lazy initiliazation of reader.*/
    public Reader getIn() {
        if (reader == null) {
            reader = new InputStreamReader(in);
        }
        return reader;
    }
    
    /** Does the same as getOut --- sets myOut if it was null.*/
    public OutputWriter getErr() {
        if (myOut == null) {
            myOut = new MyOutputWriter(new PrintWriter(out));
        }
        return myOut;
    }
    
    /** Just sets isClosed to true.*/
    public void closeInputOutput() {
        isClosed = true;
    }
    
    /** Returns the close state.*/
    public boolean isClosed() {
        return isClosed;
    }
    
    /** Does nothing. */
    public void setOutputVisible(boolean value) {
    }
    
    /** Does nothing. */
    public void setErrVisible(boolean value) {
    }
    
    /** Does nothing. */
    public void setInputVisible(boolean value){
    }
    
    /** Does nothing. */
    public void select () {
    }
    
    /** Returns true. */
    public boolean isErrSeparated() {
        return true;
    }
    
    /** Does nothing. */
    public void setErrSeparated(boolean value) {
    }
    
    /** */
    public boolean isFocusTaken() {
        return true;
    }
    
    /** Does nothing. */
    public void setFocusTaken(boolean value) {
    }
    
    /** Does nothing. */
    public Reader flushReader() {
        return reader;
    }
    
    /** OutputWriter that overrides method println to print
     * name
     */
    class MyOutputWriter extends OutputWriter {
        
        /** Just calls super(out);*/
        MyOutputWriter(PrintWriter out) {
            super(out);
        }
        
        /** Prints the oubput and calls flush. */
        public void println(java.lang.String s, OutputListener l) {
            println(name + " OUTPUT : " + s);
            flush();
        }
        
        /** Does nothing.*/
        public void reset() {
        }
    }
}
