/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.projects;

import java.io.IOException;

import org.openide.actions.*;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.*;
import org.openide.util.NbBundle;
import org.openide.util.actions.SystemAction;

/** Recognizes project XML configuration files.
 *
 * @author mryzl
 */
public class DumbXMLDataLoader extends UniFileLoader {

    public static DumbXMLDataLoader INSTANCE;
    /** Extension used by primary file which must be present. */
    private static final String XML_EXT = "xml";

    public DumbXMLDataLoader () {
        this (DumbXMLDataObject.class);
    }

    public DumbXMLDataLoader (Class recognizedObject) {
        super (recognizedObject);
        INSTANCE = this;
    }

    protected String defaultDisplayName() {
        return NbBundle.getMessage(DumbXMLDataLoader.class, "LBL_ProjectXMLLoader"); // NOI18N
    }
    
    protected SystemAction[] defaultActions() {
        return new SystemAction[] {
                        SystemAction.get (FileSystemAction.class),
                        null,
                        SystemAction.get (PropertiesAction.class),
                    };
    }
    
    protected FileObject findPrimaryFile (FileObject fo) {
        // This can of course be much more sophisticated.
        if (fo.hasExt (XML_EXT)) {
            String lname = (String) fo.getAttribute (ProjectDataLoader.EA_ASSIGNED_LOADER);
            if (getClass().getName().equals(lname)) {
                return fo;
            }
        }
        return null;
    }

    protected MultiDataObject createMultiObject (FileObject primaryFile)
    throws DataObjectExistsException, IOException {
        return new DumbXMLDataObject (primaryFile, this);
    }

    protected MultiDataObject.Entry createPrimaryEntry (MultiDataObject obj, FileObject primaryFile) {
        return new FileEntry (obj, primaryFile);
    }
    
    public static DumbXMLDataLoader getInstance() {
        if (INSTANCE == null) {
            INSTANCE = new DumbXMLDataLoader();
        }
        return INSTANCE;
    }
}
