/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.projects;

import java.awt.Component;
import javax.swing.SwingUtilities;

import org.openide.TopManager;
import org.openide.loaders.DataObject;
import org.openide.nodes.Node;
import org.openide.util.HelpCtx;
import org.openide.util.actions.*;

import org.netbeans.modules.projects.importing.*;

/**
 * Import project from other developer envirnments.
 * @author  Petr Zajac
 */
public class ImportProjectAction extends CallableSystemAction {

    private static final String EMPTY_STRING = "";  // NOI18N

    /** import core */
    static Import im = new Import ();

    public Import getDefaultImport () {
        return im;
    }


    /* Icon resource.
     * @return name of resource for icon
     */
    protected String iconResource () {
        return "/org/netbeans/modules/projects/resources/ImportProject.gif"; // NOI18N
    }

    /**
     * Human presentable name of the action. This should be
     * presented as an item in a menu.
     * @return the name of the action
     */
    public String getName () {
        return org.openide.util.NbBundle.getBundle (ImportProjectAction.class).getString ("ACT_ImportProject");
    }

    /**
     * Help context where to find more about the action.
     * @return the help context for this action
     */
    public HelpCtx getHelpCtx () {
        return new HelpCtx (ImportProjectAction.class);
    }

    public void performAction() {
        actionPerformed(null);
    }
    
    public void actionPerformed(java.awt.event.ActionEvent ev) {
        Component window = getWindowAncestor(ev);
        
        ImportProject importPrj = im.chooseImportProject (window);
        
        String msg = null;
        msg = org.openide.util.NbBundle.getBundle (ImportProjectAction.class).getString ("MSG_ImportProject");

        TopManager.getDefault ().setStatusText (msg);

        if (importPrj != null) {
            importPrj.importProject(im);
            TopManager.getDefault ().setStatusText (EMPTY_STRING);
        } else {
            msg = org.openide.util.NbBundle.getBundle (ImportProjectAction.class).getString ("MSG_WrongImportFile");
            TopManager.getDefault ().setStatusText (msg);
        }
    }
    
    /** Find window ancestor of source component from event. */
    private Component getWindowAncestor(java.awt.event.ActionEvent ev) {
        if (ev != null) {
            Object source = ev.getSource();
            if (source != null && source instanceof Component) {
                return javax.swing.SwingUtilities.getWindowAncestor((Component) source);
            }
        }
        return null;
    }
    
}
