/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.projects;

import java.awt.*;
import java.io.*;
import javax.swing.*;

import org.openide.*;
import org.openide.cookies.ProjectCookie;
import org.openide.filesystems.FileSystem;
import org.openide.filesystems.*;
import org.openide.loaders.*;
import org.openide.nodes.*;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.UserCancelException;
import org.openide.util.actions.CallableSystemAction;

/**
 * Creates new project.
 * 
 * @author Jaroslav Tulach, David Strupl
 */
public class NewProjectAction extends CallableSystemAction {

    /**
     * generated Serialized Version UID
     */
    static final long serialVersionUID = -3323132668863606031L;

    {
        setEnabled (true);
    }

    /**
     * @return the action's icon
     */
    public String getName () {
        return ProjectDataObject.getLocalizedString ("ACT_NewProject"); // NOI18N
    }

    /**
     * @return the action's help context
     */
    public HelpCtx getHelpCtx () {
        return new HelpCtx (NewProjectAction.class);
    }

    /**
     * @return the action's icon
     */
    public String iconResource () {
        return "/org/netbeans/modules/projects/resources/newProject.gif"; // NOI18N
    }

    /**
     * Action performed, opens dialog that allows user to choose a project.
     * It is then opened.
     */
    public void performAction () {
        try {
            String name = ProjectManagerPanel.getProjectName();
            if (name == null) return;
            
            createProject(name);
        } catch (java.io.IOException x) {
            TopManager.getDefault().getErrorManager().notify(x);
        }
    }
    
    /** Creates new project.
     * @param name name of new project.
     */
    public void createProject(String name) throws IOException {
        if (name == null) throw new IllegalArgumentException("name"); // NOI18N
        
        // save current project first
        Node n = TopManager.getDefault().getPlaces().nodes().projectDesktop();
        ProjectCookie c1 = (ProjectCookie)n.getCookie(ProjectCookie.class);
        if (c1 != null) {
            c1.projectSave();
        }
        
        // looking for folder - get default
        DataFolder df = (DataFolder) TopManager.getDefault().getPlaces().folders().projects();
        // create new project
        ProjectDataObject pdo = ProjectDataObject.createProject(df, name); // NOI18N
        PSupport ps = pdo.support;
        // initialize and open the newly created project
        ps.initialize();
        ps.initialSave(false);
        ps.open();
    }
    
    private static String getString(String key) {
        return NbBundle.getMessage (NewProjectAction.class, key);
    }
}

