/*
 * ProjectLoader.java
 *
 * Created on 20. listopad 2000, 15:50
 */

package org.netbeans.modules.projects;

import java.io.*;

import org.openide.cookies.ProjectCookie;
import org.openide.filesystems.FileLock;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObject;

/**
 *
 * @author  mryzl
 * @version 
 */
public class ProjectLoader extends Object implements java.io.Serializable {

    private static final ObjectStreamField[] serialPersistentFields = {};
    
    static final String NAME = "project"; // NOI18N
    static final String EXT_NAME = "name"; // NOI18N
    static final String EXT_LAST = "last"; // NOI18N
    static final String DEFAULT = "Default";
    
    /** Creates new ProjectLoader.
     */
    ProjectLoader() {
    }

    /** Save name of the project into project.name file.
     * @param name - name of the project
     */
    static void saveProjectName(String name) throws IOException {
        FileLock lock = null;
        PrintStream ps = null;
        try {
            FileObject system = org.openide.TopManager.getDefault().getRepository().getDefaultFileSystem().getRoot();
            FileObject projectName = system.getFileObject(NAME, EXT_NAME);
            if (projectName == null) {
                projectName = system.createData(NAME, EXT_NAME);
            }
            lock = projectName.lock();
            ps = new PrintStream(projectName.getOutputStream(lock));
            ps.print(name);
        } finally {
            if (lock != null) lock.releaseLock();
            if (ps != null) ps.close();
        }
    }

    /** Load project name from the file.
     * @return name of the project or ProjectLoader.DEFAULT
     */
    static String getProjectName() {
        BufferedReader br = null;
        String name = DEFAULT;
        try {
            FileObject system = org.openide.TopManager.getDefault().getRepository().getDefaultFileSystem().getRoot();
            FileObject projectName = system.getFileObject(NAME, EXT_NAME);
            if (projectName != null) {
                br = new BufferedReader(new InputStreamReader(projectName.getInputStream()));
                name = br.readLine();
            }
        } catch (Exception ex) {
            // ignore
        } finally {
            try {
                if (br != null) br.close();
            } catch (Exception ex) {
                // really ignore
            }
        }
        return name;
    }
    
    
    /** Get project by name. If such a project doesn't exist, create it.
     * @param name - name of the project
     * @return an implementation of ProjectCookie
     */
    static ProjectCookie getProjectByName(String name) {
        DataFolder projects = org.openide.TopManager.getDefault().getPlaces().folders().projects();
        DataObject[] dobjs = projects.getChildren();
        for(int i = 0; i < dobjs.length; i++) {
            ProjectCookie cookie = (ProjectCookie) dobjs[i].getCookie(ProjectCookie.class);
            if (cookie != null) {
                if (dobjs[i].getName().equals(name)) {
                    return cookie;
                }
                
            }
        }
        try {
            ProjectDataObject pdo = ProjectDataObject.createProject(projects, name);
            return (ProjectCookie) pdo.getCookie(ProjectCookie.class);
        } catch (Exception ex) {
        }
        return null;
    }

    private Object readResolve()  {
        String projectName = getProjectName();
        return getProjectByName(projectName);
    }
    
    public static void saveProjectLast() throws IOException {
        FileLock lock = null;
        ObjectOutputStream oos = null;
        try {
            FileObject system = org.openide.TopManager.getDefault().getRepository().getDefaultFileSystem().getRoot();
            FileObject projectName = system.getFileObject(NAME, EXT_LAST);
            if (projectName == null) {
                projectName = system.createData(NAME, EXT_LAST);
            }
            lock = projectName.lock();
            oos = new ObjectOutputStream(projectName.getOutputStream(lock));
            oos.writeObject(new ProjectLoader());
        } finally {
            if (lock != null) lock.releaseLock();
            try {
                if (oos != null) oos.close();
            } catch (Exception ex) {
                // ignore
            }
        }
    }
    
    public static void main(String args[]) throws Exception {
        saveProjectLast();
        saveProjectName("Project");
    }
}
