/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.projects;

import java.awt.event.ActionEvent;
import java.io.IOException;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import javax.swing.SwingUtilities;

import org.openide.*;
import org.openide.cookies.ProjectCookie;
import org.openide.cookies.SaveCookie;
import org.openide.explorer.ExplorerManager;
import org.openide.nodes.Node;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObject;
import org.openide.util.UserCancelException;
import org.openide.util.NbBundle;
import org.openide.util.SharedClassObject;
import org.openide.util.RequestProcessor;

/**
 * Displays available projects and allows saving, switching etc.
 * @author  dstrupl
 */
public class ProjectManagerPanel extends javax.swing.JPanel implements ExplorerManager.Provider {

    /** Property indicates the need of a dialog close in which the manager panel
     * can be contained. */
    public static final String PROP_CLOSE_DIALOG = "PROP_CLOSE_DIALOG"; // NOI18N
    
    /** the instance of the explorer manager*/
    private ExplorerManager manager;
    
    /** Creates new form ProjectManagerPanel */
    public ProjectManagerPanel() {
        initComponents();
        getExplorerManager().addPropertyChangeListener(new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent ev) {
                updateButtonState();
            }
        });
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        projectListLabel = new javax.swing.JLabel();
        projectListLabel.setDisplayedMnemonic((java.util.ResourceBundle.getBundle("org/netbeans/modules/projects/Bundle").getString("CTL_ExistingProjects_Mnem")).charAt(0));
        controlPanel = new javax.swing.JPanel();
        newButton = new javax.swing.JButton();
        openButton = new javax.swing.JButton();
        importButton = new javax.swing.JButton();
        saveAsButton = new javax.swing.JButton();
        deleteButton = new javax.swing.JButton();
        renameButton = new javax.swing.JButton();
        projectList = new org.openide.explorer.view.ListView();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        setPreferredSize(new java.awt.Dimension(400, 300));
        setMinimumSize(new java.awt.Dimension(400, 300));
        setMaximumSize(new java.awt.Dimension(600, 600));
        projectListLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/projects/Bundle").getString("CTL_ExistingProjects"));
        projectListLabel.setForeground(java.awt.Color.black);
        projectListLabel.setLabelFor(projectList);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(12, 12, 5, 11);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(projectListLabel, gridBagConstraints1);
        
        controlPanel.setLayout(new java.awt.GridLayout(6, 1, 0, 5));
        
        newButton.setMnemonic(78);
        newButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/projects/Bundle").getString("CTL_New"));
        newButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                newButtonActionPerformed(evt);
            }
        });
        
        controlPanel.add(newButton);
        
        openButton.setMnemonic(79);
        openButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/projects/Bundle").getString("CTL_Open"));
        openButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                openButtonActionPerformed(evt);
            }
        });
        
        controlPanel.add(openButton);
        
        importButton.setMnemonic(73);
        importButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/projects/Bundle").getString("CTL_Import"));
        importButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                importButtonActionPerformed(evt);
            }
        });
        
        controlPanel.add(importButton);
        
        saveAsButton.setMnemonic(86);
        saveAsButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/projects/Bundle").getString("CTL_SaveAs"));
        saveAsButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveAsButtonActionPerformed(evt);
            }
        });
        
        controlPanel.add(saveAsButton);
        
        deleteButton.setMnemonic(68);
        deleteButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/projects/Bundle").getString("CTL_Delete"));
        deleteButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                deleteButtonActionPerformed(evt);
            }
        });
        
        controlPanel.add(deleteButton);
        
        renameButton.setMnemonic(82);
        renameButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/projects/Bundle").getString("CTL_Rename"));
        renameButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                renameButtonActionPerformed(evt);
            }
        });
        
        controlPanel.add(renameButton);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.insets = new java.awt.Insets(0, 12, 12, 11);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHEAST;
        add(controlPanel, gridBagConstraints1);
        
        projectList.setTraversalAllowed(false);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(0, 12, 12, 0);
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(projectList, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void importButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_importButtonActionPerformed
        RequestProcessor.postRequest(new Runnable() {
            public void run() {
                disableAllButtons();
                importProject();
                clearListSelection();
                updateButtonState();
            }
        });
    }//GEN-LAST:event_importButtonActionPerformed

    private void renameButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_renameButtonActionPerformed
        Node []na = getExplorerManager().getSelectedNodes();
        if ((na != null) && (na.length == 1)) {
            ProjectDataObject obj = (ProjectDataObject)na[0].getCookie (ProjectDataObject.class);
            if (obj != null) {
                NotifyDescriptor.InputLine line = new NotifyDescriptor.InputLine(
                    getString ("CTL_RenameLabel"),
                    getString ("CTL_RenameDialogTitle")
                );
                line.setInputText (obj.getName());
                Object ok = TopManager.getDefault ().notify (line);
                if (ok == NotifyDescriptor.OK_OPTION) {
                    String res = line.getInputText();
                    try {
                        obj.rename(res);
                    } catch (java.io.IOException x) {
                        TopManager.getDefault().getErrorManager().notify(x);
                    }
                }
            }
        }
        clearListSelection();
        updateButtonState();
    }//GEN-LAST:event_renameButtonActionPerformed

    private void deleteButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_deleteButtonActionPerformed
        Node []na = getExplorerManager().getSelectedNodes();
        if ((na != null) && (na.length > 0)) {
            final ProjectDataObject[] obj = new ProjectDataObject[na.length];
            for (int i = 0; i < obj.length; i++) {
                obj[i] =  (ProjectDataObject)na[i].getCookie (ProjectDataObject.class);
            }
            NotifyDescriptor.Confirmation confirm = new NotifyDescriptor.Confirmation(getString("MSG_ConfirmDelete"));
            Object ok = TopManager.getDefault ().notify (confirm);
            if (ok == NotifyDescriptor.YES_OPTION) {
                RequestProcessor.postRequest(new Runnable() {
                    public void run() {
                        disableAllButtons();
                        deleteProject(obj);
                        updateButtonState();
                    }
                });
            }
        }
    }//GEN-LAST:event_deleteButtonActionPerformed

    private void saveAsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveAsButtonActionPerformed
        Node []na = getExplorerManager().getSelectedNodes();
        if ((na != null) && (na.length == 1)) {
            final ProjectDataObject obj = (ProjectDataObject)na[0].getCookie (ProjectDataObject.class);
            if (obj != null) {
                NotifyDescriptor.InputLine line = new NotifyDescriptor.InputLine(
                    getString ("CTL_RenameLabel"),
                    getString ("CTL_SaveAsDialogTitle")
                );
                line.setInputText (obj.getName());
                Object ok = TopManager.getDefault ().notify (line);
                if (ok == NotifyDescriptor.OK_OPTION) {
                    final String res = line.getInputText();
                    if ((res == null) || ("".equals(res))) { // NOI18N
                        TopManager.getDefault().notify(
                            new NotifyDescriptor.Message(
                                getString("MSG_NameCannotBeEmpty"),
                                NotifyDescriptor.ERROR_MESSAGE
                            )
                        );
                    } else {
                        RequestProcessor.postRequest(new Runnable() {
                            public void run() {
                                disableAllButtons();
                                saveAsProject(obj, res);
                                updateButtonState();
                            }
                        });
                    }
                }
            }
        }
    }//GEN-LAST:event_saveAsButtonActionPerformed

    private void newButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_newButtonActionPerformed
        RequestProcessor.postRequest(new Runnable() {
            public void run() {
                newProject(getProjectName());
            }
        });
    }//GEN-LAST:event_newButtonActionPerformed

    private void openButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_openButtonActionPerformed
        Node []na = getExplorerManager().getSelectedNodes();
        if ((na != null) && (na.length == 1)) {
            final ProjectCookie c = (ProjectCookie)na[0].getCookie (ProjectCookie.class);
            RequestProcessor.postRequest(new Runnable() {
                public void run() {
                    openProject(c);
                }
            });
        }
    }//GEN-LAST:event_openButtonActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel projectListLabel;
    private javax.swing.JPanel controlPanel;
    private javax.swing.JButton newButton;
    private javax.swing.JButton openButton;
    private javax.swing.JButton importButton;
    private javax.swing.JButton saveAsButton;
    private javax.swing.JButton deleteButton;
    private javax.swing.JButton renameButton;
    private org.openide.explorer.view.ListView projectList;
    // End of variables declaration//GEN-END:variables

    /** Create new project. */
    private void newProject(String name) {
        if (name == null) return;
        firePropertyChange(PROP_CLOSE_DIALOG, null, null);
        
        NewProjectAction npa = (NewProjectAction)SharedClassObject.findObject(NewProjectAction.class);
        try {
            npa.createProject(name);
        } catch (java.io.IOException x) {
            TopManager.getDefault().getErrorManager().notify(x);
        }
    }
    
    /** Opens a dialog for input of project name.
     * @return project name or null if dialog was canceled.
     */
    protected static String getProjectName() {
        NotifyDescriptor.InputLine input = new NotifyDescriptor.InputLine (
                                               getString ("CTL_NewProjectName"), // NOI18N
                                               getString ("CTL_NewProjectTitle") // NOI18N
                                           );
        input.setInputText(getString("FMT_NewProjectName")); // NOI18N
        
        String projectName = null;
        
        if (TopManager.getDefault ().notify (input) == NotifyDescriptor.OK_OPTION) {
            projectName = input.getInputText ();
            if (projectName == null || projectName.length() == 0) {
                projectName = getString("FMT_NewProjectName"); // NOI18N
            }
        }
        
        return projectName;
    }
    
    /** Open a project. */
    private void openProject(ProjectCookie project) {
        if (project == null) throw new IllegalArgumentException("project"); // NOI18N
        firePropertyChange(PROP_CLOSE_DIALOG, null, null);
        try {
            TopManager.getDefault ().openProject (project);
        } catch (UserCancelException e) {
        } catch (IOException e) {
            TopManager.getDefault ().notifyException (e);
        }
    }
    
    /** Save project as new one.
     * @param old data object of project which will be saved
     * @param name name of new project
     */
    private void saveAsProject(ProjectDataObject old, String name) {
        if (old == null) throw new IllegalArgumentException("old"); // NOI18N
        if (name == null || name.length() == 0)
            throw new IllegalArgumentException("name"); // NOI18N
        
        try {
            // Save the project first
            SaveCookie sc = (SaveCookie) old.getCookie(SaveCookie.class);
            if (sc != null) {
                sc.save();
            }
            // get projects folder
            DataFolder df = (DataFolder)
                TopManager.getDefault().getPlaces().folders().projects();
            DataFolder newFolder = ProjectDataObject.createProject(df, name);
            ProjectDataObject.copyChildren(old, newFolder);
        } catch (java.io.IOException x) {
            TopManager.getDefault().getErrorManager().notify(x);
        }
    }
    
    /** Import a project. */
    private void importProject() {
        ImportProjectAction ipa = (ImportProjectAction)SharedClassObject.findObject(ImportProjectAction.class);
        ipa.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, null));
    }
    
    /** Delete projects. */
    private void deleteProject(ProjectDataObject[] objs) {
        if (objs == null) throw new IllegalArgumentException("objs"); // NOI18N
        try {
            for (int i = 0; i < objs.length; i++) {
                if (objs[i] != null) {
                    objs[i].delete();
                }
            }
        } catch (java.io.IOException x) {
            TopManager.getDefault().getErrorManager().notify(x);
        }
    }
    
    /**
     * Schedule updateButtonState1 to AWT dispatch thread if there is need.
     */
    private void updateButtonState() {
        if (SwingUtilities.isEventDispatchThread()) {
            updateButtonState1();
        } else {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    updateButtonState1();
                }
            });
        }
    }
    
    /**
     * Enables/disables the delete button according
     * to selected node.
     */
    private void updateButtonState1() {
        Node []selected = getExplorerManager().getSelectedNodes();
        
        if (newButton != null) newButton.setEnabled(true);
        if (importButton != null) importButton.setEnabled(true);
        if (deleteButton != null) {
            boolean enabled = true;
            if ((selected != null) && (selected.length > 0)){
                for (int i = 0; i < selected.length; i++) {
                    if (!selected[i].canDestroy()) {
                        enabled = false;
                    }
                }
            } else {
                enabled = false;
            }
            deleteButton.setEnabled(enabled);
        }
        if (openButton != null) {
            boolean enabled = true;
            if ((selected != null) && (selected.length == 1)){
                Node n = TopManager.getDefault().getPlaces().nodes().projectDesktop();
                ProjectCookie c1 = (ProjectCookie)n.getCookie(ProjectCookie.class);
                ProjectCookie c2 = (ProjectCookie)selected[0].getCookie(ProjectCookie.class);
                if (c1 == c2) {
                    enabled = false;
                }
            } else {
                enabled = false;
            }
            openButton.setEnabled(enabled);
        }
        
        boolean enabled = (selected != null && selected.length == 1);
        if (renameButton != null) renameButton.setEnabled(enabled);
        if (saveAsButton != null) saveAsButton.setEnabled(enabled);
    }
    
    /** Schedule disableAllButtonsImpl to AWT dispatch thread if there is need. */
    private void disableAllButtons() {
        if (SwingUtilities.isEventDispatchThread()) {
            disableAllButtonsImpl();
        } else {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    disableAllButtonsImpl();
                }
            });
        }
    }
    
    /** Disable all buttons impl. */
    private void disableAllButtonsImpl() {
        newButton.setEnabled(false);
        openButton.setEnabled(false);
        importButton.setEnabled(false);
        saveAsButton.setEnabled(false);
        deleteButton.setEnabled(false);
        renameButton.setEnabled(false);
    }
    
    /** Temporary workaround of #11326;
     * List view does not reflect changes properly.
     */
    private void clearListSelection() {
        Runnable task = new Runnable() {
            public void run() {
                try {
                    getExplorerManager().setSelectedNodes(new Node[0]);
                } catch (Exception ex) {
                    // does not matter
                }
            }
        };
        if (SwingUtilities.isEventDispatchThread()) {
            task.run();
        } else {
            SwingUtilities.invokeLater(task);
        }
    }
    
    /* Provides the explorer manager to all who are interested.
     * @return the manager
     */
    public ExplorerManager getExplorerManager () {
        if (manager == null) {
            synchronized (this) {
                if (manager == null) {
                    manager = new ExplorerManager ();
                }
            }
        }
        return manager;
    }
    
    private static String getString(String key) {
        return NbBundle.getMessage (ProjectManagerPanel.class, key);
    }
}
