/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.projects.content;

import java.io.IOException;

import org.openide.util.io.NbMarshalledObject;

/**
 *
 * @author  mryzl
 */

public interface ProjectContent extends java.io.Serializable {

    /** Get super project of the project.
    * @return super project or null if there is no one
    */
    public ProjectContent getSuperProject();

    /** Get control panel of the project.
    * @param force force creating new instead of throwing an exception
    * @return control panel
    */
    public SettingsSet getControlPanel();

    /** Store control panel. 
     */
    public void storeControlPanel();
    
    /** Get set of loaders specific for this project.
    * @param force force creating new instead of throwing an exception
    * @return loaders
    */
    public SettingsSet getLoaderPool();

    /** Store loaders.
     */
    public void storeLoaderPool();
    
    /** Get repository settings for the project.
    * @param force force creating new instead of throwing an exception
    * @return repository diff set
    */
    public DiffSet getRepository();

    /** Store repository.
     */
    public void storeRepository();
    
    /** Get services settings for the project.
    * @param force force creating new instead of throwing an exception
    * @return services diff set
    */
    public DiffSet getServices();
    
    /** Store services.
     */
    public void storeServices();

    /** Get windows for the project.
    * @return window manager
    */
    public NbMarshalledObject getWindowManager();

    /** Store windows of the project.
    * @return window manager
    */
    public void storeWindowManager();

    /** Store the project.
    */
    public void store();
}
