/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.projects.content;

import java.beans.*;

import org.openide.explorer.ExplorerManager;
import org.openide.nodes.Node;
import org.openide.*;

/**
 * Shows number of errors in one place.
 * @author  Dirk Ruiz, David Strupl
 */
public class ProjectErrorsPanel extends javax.swing.JPanel implements ExplorerManager.Provider {

    /** the instance of the explorer manager*/
    private ExplorerManager manager;
    
    /** Creates new form ProjectErrorsPanel2 */
      public ProjectErrorsPanel() {
        initComponents();
        getExplorerManager().addPropertyChangeListener(new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent ev) {
                updateDescription();
            }
        });

    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        detailsButton = new javax.swing.JButton();
        errorListTitle = new javax.swing.JLabel();
        errorList = new org.openide.explorer.view.ListView();
        detailsScrollPane = new javax.swing.JScrollPane();
        detailsTextPane = new javax.swing.JTextArea();
        detailsLabel = new javax.swing.JLabel();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        setPreferredSize(new java.awt.Dimension(400, 330));
        detailsButton.setMnemonic(84);
        detailsButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/projects/content/Bundle").getString("CTL_ShowDetails"));
        detailsButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                detailsButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.insets = new java.awt.Insets(17, 12, 11, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.SOUTHWEST;
        add(detailsButton, gridBagConstraints1);
        
        errorListTitle.setText(getExplorerManager().getRootContext().getDisplayName());
        errorListTitle.setForeground(java.awt.Color.black);
        errorListTitle.setLabelFor(errorList);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(12, 12, 5, 11);
        add(errorListTitle, gridBagConstraints1);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(0, 12, 0, 11);
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(errorList, gridBagConstraints1);
        
        detailsScrollPane.setAlignmentY(1.0F);
        detailsTextPane.setWrapStyleWord(true);
        detailsTextPane.setLineWrap(true);
        detailsTextPane.setEditable(false);
        detailsTextPane.setRows(3);
        detailsTextPane.setBackground(new java.awt.Color(204, 204, 204));
        detailsScrollPane.setViewportView(detailsTextPane);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 12, 0, 11);
        gridBagConstraints1.weightx = 1.0;
        add(detailsScrollPane, gridBagConstraints1);
        
        detailsLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/projects/content/Bundle").getString("LBL_Details"));
        detailsLabel.setForeground(java.awt.Color.black);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(12, 12, 5, 11);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints1.weightx = 1.0;
        add(detailsLabel, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void detailsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_detailsButtonActionPerformed
        Node []na = getExplorerManager().getSelectedNodes();
        if ((na != null) && (na.length == 1)) {
            showDetails(na[0].getValue(ProjectErrorManager.PROP_ERROR_DETAILS));
        }

    }//GEN-LAST:event_detailsButtonActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton detailsButton;
    private javax.swing.JLabel errorListTitle;
    private org.openide.explorer.view.ListView errorList;
    private javax.swing.JScrollPane detailsScrollPane;
    private javax.swing.JTextArea detailsTextPane;
    private javax.swing.JLabel detailsLabel;
    // End of variables declaration//GEN-END:variables
    /* Provides the explorer manager to all who are interested.
     * @return the manager
     */
    public ExplorerManager getExplorerManager () {
        if (manager == null) {
            synchronized (this) {
                if (manager == null) {
                    manager = new ExplorerManager ();
                }
            }
        }
        return manager;
    }
    
    void updateDescription() {
        boolean enabled = false;
        
        Node []na = getExplorerManager().getSelectedNodes();
        if ((na != null) && (na.length == 1)) {
            detailsTextPane.setText(na[0].getShortDescription());
            detailsTextPane.setCaretPosition(0);
            enabled = true;
        }
        detailsButton.setEnabled(enabled);
        errorListTitle.setText(getExplorerManager().getRootContext().getDisplayName());
    }
    
    /** Shows the details of the error.*/
    private void showDetails(Object entry) {
        if (entry instanceof ProjectErrorManager.ListEntry) {
            ProjectErrorManager.ListEntry e = (ProjectErrorManager.ListEntry) entry;
            TopManager.getDefault().getErrorManager().notify(ErrorManager.EXCEPTION, e.t);
        } else {
            TopManager.getDefault().getErrorManager().log(
            ErrorManager.EXCEPTION,
            "Internal problem in ProjectErrorManager.showDetails: " + entry.getClass() // NOI18N
            );
        }
    }
    

}
