/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.projects.importing;

import java.io.File;
import java.io.IOException;
import java.text.MessageFormat;

import org.openide.NotifyDescriptor;
import org.openide.TopManager;
import org.openide.filesystems.LocalFileSystem;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObject;
import org.openide.util.NbBundle;
import org.openide.util.RequestProcessor;

import org.netbeans.modules.projects.*;

/**
 * This class imports NetBeans project. 
 *
 * @author Martin Ryzl
 * @version 1.0
 */
public class ImportNetbeansProject implements ImportProject, Runnable {

    /** Name of system dir. */
    static final String systemDir = "system"; // NOI18N
    
    /** Name of project dir. */
    static final String projectsDir = "Projects"; // NOI18N
    
    File projectFile;
    
    public ImportNetbeansProject(File projectFile) {
        this.projectFile = projectFile;
    }
    
    /** Get filesystems that contain project
     * @return filesystems for mounting into repository
     */
    public java.io.File[] getFileSystems() {
        return new File[0];
    }
    
    /** Get project files
     * @return all files that are in project
     */
    public java.io.File[] getFiles() {
        return new File[0];
    }
    
    /** Get jar and zip files whitch will be mounted
     * @return jar and zip files whitch are in CLASSPATH of project
     */
    public java.io.File[] getJarsAndZips() {
        return new File[0];
    }
    
    /** Get Main clas of project.
     * @return MainClass of Project, when project hasn't MainClass, it is returned null
     */
    public String getMainClass() {
        return ""; // NOI18N
    }
    
    /** Get Name of project.
     * @return name of project which be imported
     */
    public String getName() {
        return ""; // NOI18N
    }
    
    /** get System jar and zip files. It can be founded in system configuratin
     * file of environment
     * @return systems classpath
     */
    public java.io.File[] getSystemJarsAndZips() {
        return new File[0];
    }
    
    
    /** Import netbeans project. 
     * @param im - the param is ignored.
     */
    public void importProject(Import im) {
        RequestProcessor.postRequest(this);
    }
    
    /** The real implementation of import.
     */
    public void run() {
        java.util.ResourceBundle bundle = NbBundle.getBundle(ImportNetbeansProject.class);
        boolean superProject;
        File root, p1;
        int num = 0;
        LocalFileSystem lfs = null;
        
        try {
            // check if is it either project super folder or project folder
            root = projectFile.getParentFile();
            if (root == null) return; // it should not be possible for projectFile file
            if (projectsDir.equals(root.getName())) { // probably main level
                superProject = true;
            } else {
                superProject = false;
                p1 = root.getParentFile();
                if (p1 != null) root = p1;
                else return; // don't accept, files must be in a folder
            }
            
            lfs = new LocalFileSystem();
            lfs.setRootDirectory(root);
            lfs.setHidden(true);
            // temporary add to filesystes
            TopManager.getDefault().getRepository().addFileSystem(lfs);
            
            DataFolder rootFolder = DataFolder.findFolder(lfs.getRoot());
            DataObject dobjs[] = rootFolder.getChildren();
            DataFolder projects = TopManager.getDefault().getPlaces().folders().projects();

            if (superProject) {
                
                Object response = TopManager.getDefault().notify(new NotifyDescriptor.Confirmation(
                    bundle.getString("MSG_GlobalProject"), // NOI18N
                    NotifyDescriptor.YES_NO_OPTION
                ));
                
                if (response.equals(NotifyDescriptor.YES_OPTION)) {
                
                    // check all subprojects, copy them into project repository
                    for(int i = 0; i < dobjs.length; i++) {
                        ProjectDataObject pdo = (ProjectDataObject) dobjs[i].getCookie(ProjectDataObject.class);
                        if (pdo != null) {
                            TopManager.getDefault().setStatusText(MessageFormat.format(
                                bundle.getString("FMT_StatusImportProject"), // NOI18N
                                new Object[] { pdo.getName() }
                            ));
                            pdo.copy(projects);
                            num++;
                        }
                    }
                    if (num > 0) TopManager.getDefault().setStatusText("Successfully imported."); // NOI18N
                }

            } else {
                String prjName = projectFile.getParentFile().getName(); 
                // take one project
                for(int i = 0; i < dobjs.length; i++) {
                    FileObject fo = dobjs[i].getPrimaryFile();
                    String dobjName = fo.getName();
                    if ((fo.getExt() != null) && (fo.getExt().length() > 0)) dobjName += '.' + fo.getExt();
                    
                    if (prjName.equals(dobjName)) {
                        ProjectDataObject pdo = (ProjectDataObject) dobjs[i].getCookie(ProjectDataObject.class);
                        if (pdo != null) {
                            TopManager.getDefault().setStatusText(MessageFormat.format(
                                bundle.getString("FMT_StatusImportProject"), // NOI18N
                                new Object[] { pdo.getName() }
                            ));
                            pdo.copy(projects);
                            num++;
                            TopManager.getDefault().setStatusText(
                                bundle.getString("MSG_StatusImportSuccessfull") // NOI18N
                            );

                        } else if (dobjs[i].getCookie(DataFolder.class) != null) {
                            Object response = TopManager.getDefault().notify(new NotifyDescriptor.Confirmation(
                                bundle.getString("MSG_ImportAnyway"), // NOI18N
                                NotifyDescriptor.YES_NO_OPTION
                            ));

                            if (response.equals(NotifyDescriptor.YES_OPTION)) {
                                final DataFolder src = (DataFolder) dobjs[i].getCookie(DataFolder.class);
                               
                                // create a new project with the same name
                                ProjectDataObject newProject = ProjectDataObject.createProject(projects, src.getName());
                                num++;
                                
                                // copy children
                                ProjectDataObject.copyChildren(src, newProject);
                                
                                TopManager.getDefault().setStatusText(
                                    bundle.getString("MSG_StatusImportSuccessfull") // NOI18N
                                );

                            }
                        }
                    }
                }
            }
            // copy it into project folder
        } catch (Exception ex) {
            TopManager.getDefault().setStatusText(
                bundle.getString("MSG_StatusImportUnSuccessfull") // NOI18N
            );
            TopManager.getDefault().notifyException(ex);
        } finally {
            if (lfs != null) TopManager.getDefault().getRepository().removeFileSystem(lfs);
        }
    }
    
    /** Return supposed project root. 
     */
    private File getProjectRoot() {
        File p1 = projectFile.getParentFile();
        if (p1 != null) {
            if (p1.equals(projectsDir)) return p1;
            return p1.getParentFile();
        }
        return null;
    }
}
