/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.properties.syntax;


import java.util.MissingResourceException;
import javax.swing.JEditorPane;

import org.netbeans.editor.LocaleSupport;
import org.netbeans.editor.Settings;
import org.netbeans.modules.editor.options.AllOptions;
import org.netbeans.modules.properties.PropertiesDataObject;
import org.netbeans.modules.properties.PropertiesModule;

import org.openide.modules.ModuleInstall;
import org.openide.options.SystemOption;
import org.openide.text.PrintSettings;
import org.openide.util.NbBundle;


/**
 * @author Petr Jiricka, Libor Kramolis, Jesse Glick
 */
public class RestoreColoring extends ModuleInstall {

    private static final long serialVersionUID = 4219541766134557769L;
    
    /** <code>Localizer</code> passed to editor. */
    private LocaleSupport.Localizer localizer;

    /** Overrides superclass method. */
    public void installed () {
        restored ();
    }

    /** Registers properties editor, installs options and copies settings. Overrides superclass method.  */
    public void restored () {
        addInitializer ();
        installOptions ();
        // Make sure it is registered in the main module.
        EditorSettingsCopy.getSettings ();
    }

    /** Uninstalls properties options. And cleans up editor settings copy. Overrides superclass method. */
    public void uninstalled () {
        uninstallOptions ();
        
        // Provide cleanup.
        EditorSettingsCopy.uninstallSettings();
    }

    /** Adds initializer and registers editor kit. */
    public void addInitializer() {
        Settings.addInitializer(new PropertiesSettingsInitializer());

        // Registration of the editor kits to JEditorPane
        JEditorPane.registerEditorKitForContentType(
            PropertiesDataObject.MIME_PROPERTIES,
            "org.netbeans.modules.properties.syntax.PropertiesKit", // NOI18N
            this.getClass().getClassLoader()
        );
    }

    /** Installs options. */
    public void installOptions() {
        AllOptions ao = (AllOptions)AllOptions.findObject (AllOptions.class, true);
        ao.addOption ((PropertiesOptions)SystemOption.findObject(PropertiesOptions.class, true));
        PrintSettings ps = (PrintSettings)PrintSettings.findObject (PrintSettings.class, true);
        ps.addOption ((PropertiesPrintOptions)SystemOption.findObject(PropertiesPrintOptions.class, true));
        
        
        // add localizer
        LocaleSupport.addLocalizer(localizer = new LocaleSupport.Localizer() {
            public String getString(String key) {
                try {
                    return NbBundle.getBundle(PropertiesModule.class).getString(key);
                } catch (MissingResourceException e) {
                    return null;
                }
            }
        });
    }

    /** Uninstall options. */
    public void uninstallOptions() {
        AllOptions ao = (AllOptions)AllOptions.findObject (AllOptions.class, true);
        PropertiesOptions po = (PropertiesOptions)SystemOption.findObject(PropertiesOptions.class, false);
        if (po != null) ao.removeOption (po);
        PrintSettings ps = (PrintSettings)PrintSettings.findObject (PrintSettings.class, true);
        PropertiesPrintOptions ppo = (PropertiesPrintOptions)SystemOption.findObject(PropertiesPrintOptions.class, false);
        if (ppo != null) ps.removeOption (ppo);
        
        // remove localizer
        LocaleSupport.removeLocalizer(localizer);
    }
} // End of class RestoreColoring.
