/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/* $Id: rmid_wrapper.cpp,v 1.2 2001/02/19 14:24:33 mryzl Exp $ */

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include <fcntl.h>
#include <process.h>
#include <commdlg.h>
#include <signal.h>

#define RMID_MAIN_CLASS "org.netbeans.rmi.RMIDWrapperImpl"
#define JDK_KEY "Software\\JavaSoft\\Java Development Kit"
//#define IDE_KEY "Software\\netbeans.org\\NetBeans IDE"
#define IDE_KEY "Software\\Sun Microsystems, Inc.\\Forte for Java"

// #define DEBUG

static char jdkhome[MAX_PATH];
static char idehome[MAX_PATH];
static char classpath[1024 * 16];

static void addToClassPath(const char *pathprefix, const char *path);
static int findJdkFromRegistry(const char* keyname, char jdkhome[]);
static void setIDEHome(char idehome[]);
static int processOptions(int argc, char **argv, char ***cmd);
static void UserHandler(int sig);


static void sysSignal(int sig, void (*func) ( int sig )) {
    signal(sig, (void (*)(int)) func);
}
 

/**
 * CTRL+Break handler
 */
static void UserHandler(int sig)
{
    // ignore it
    /* We need to reinstate the signal handler each time... */
    sysSignal(sig, UserHandler);
}

int main(int argc, char *argv[]) {

    int status;
    char buf[1024];
    char **cmd;    

    sysSignal(SIGBREAK, UserHandler);

    setIDEHome(idehome);
    findJdkFromRegistry(JDK_KEY, jdkhome);
    
    // printf("%s, %s\n", idehome, jdkhome);
    processOptions(argc, argv, &cmd);
    strcpy(buf, jdkhome);
    strcat(buf, "\\bin\\java.exe");

#ifdef DEBUG
    printf("process: %s\n", buf);
    printf("arguments: ");
    for(int i = 0; cmd[i] != NULL; i++) {
        if (i != 0) printf(", ");
        printf("%s", cmd[i]);
    }
    printf("\n");
#endif DEBUG

    status = spawnv(_P_WAIT, buf, cmd);
//    execv(buf, cmd);
    if (status == -1) {
        perror(buf);
    }
    return status;
}


/*
 * Returns string data for the specified registry value name, or
 * NULL if not found.
 */
static char * GetStringValue(HKEY key, const char *name)
{
    DWORD type, size;
    char *value = 0;

    if (RegQueryValueEx(key, name, 0, &type, 0, &size) == 0 && type == REG_SZ) {
        value = (char*) malloc(size);
        if (RegQueryValueEx(key, name, 0, 0, (unsigned char*)value, &size) != 0) {
            free(value);
            value = 0;
        }
    }
    return value;
}

static int findJdkFromRegistry(const char* keyname, char jdkhome[])
{
    HKEY hkey = NULL, subkey = NULL;
    char *ver = NULL;
    int rc = 1;
  
    if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, keyname, 0, KEY_READ, &hkey) == 0) {
        ver = GetStringValue(hkey, "CurrentVersion");
        if (ver == NULL)
            goto quit;

        if (RegOpenKeyEx(hkey, ver, 0, KEY_READ, &subkey) == 0) {
            char *home = GetStringValue(subkey, "JavaHome");
            if (home == NULL)
                goto quit;
            strcpy(jdkhome, home);
            free(home);
            rc = 0;
        }
    }

  quit:
    if (ver != NULL)
        free(ver);
    if (subkey != NULL)
        RegCloseKey(subkey);
    if (hkey != NULL)
        RegCloseKey(hkey);
    return rc;
}

void addToClassPath(const char *pathprefix, const char *path) {
    char buf[1024];
    
    strcpy(buf, pathprefix);
    if (path != NULL)
        strcat(strcat(buf, "\\"), path);

    if (classpath[0] != '\0')
        strcat(classpath, ";");
    strcat(classpath, buf);
}

/** Set IDE home. Get the whole process name and strip
 * the trailig \bin\<name>
 */
static void setIDEHome(char idehome[]) {
    char exepath[1024 * 4];
    char buf[1024 * 4], *pc;

    GetModuleFileName(0, buf, sizeof buf);
    strcpy(exepath, buf);

    pc = strrchr(buf, '\\');
    if (pc != NULL) {             // always holds
        *pc = '\0';	// remove .exe filename
    }

    // remove \bin
    pc = strrchr(buf, '\\');
    if (pc != NULL && 0 == stricmp("\\bin", pc)) {
        *pc = '\0';
    }
    strcpy(idehome, buf);
}


/** Process arguments.

-J options will be passed to java
the rest will remain and will processed by rmid

some exceptions

-Xms8m - 
-jdkhome  - obtain programatically and delete from list of options
-J-Djava.security.policy= - rmid.policy
// -log - ../tmp/
// -J-Dsun.rmi.server.activation.debugExec - default to true
-J-classpath - add library to the classpath


 */
static int processOptions(int argc, char **argv, char ***cmd) {
    int jac = 0, rac = 0, i, j;
    char *jav[64], *rav[64], *arg, **cmd2;
    int cpi = -1, spi = -1, nbi = -1, jdkhomeFlag = 0, cpFlag = 0;
    char buf[1024];
    
    jav[jac++] = "-ms8m";
    jav[jac++] = "-Dsun.rmi.server.activation.debugExec=true";
    
    for(i = 1; i < argc; i++) {
        arg = argv[i];
        if ((arg[0] == '-') && (arg[1] == 'J')) {
            arg += 2;
            jav[jac] = arg;
            
            if (cpFlag) {
                cpFlag = 0;
            }    
            
            if (strncmp(arg, "-Djava.security.policy=", 23) == 0) {
                spi = jac;
            }
            
            if (strncmp(arg, "-Dnetbeans.home=", 16) == 0) {
                nbi = jac;
            }

            if (strcmp(arg, "-classpath") == 0) {
                cpi = jac;
                cpFlag = 1;
            }
            
            jac++;
        } else {
            
            if (jdkhomeFlag) {
                strcpy(jdkhome, arg);
                jdkhomeFlag = 0;
                continue;
            }
            
            if (strcmp(arg, "-jdkhome") == 0) {
                jdkhomeFlag = 1;
            } else {
                rav[rac++] = arg;
            }
        }
    }

    if (jdkhomeFlag) {
        jdkhome[0] = 0;
    }

    // add policy file    
    if (spi == -1) {
        jav[jac++] = "-Djava.security.policy=rmid.policy";
    } 
    
    if (nbi == -1) {
        strcpy(buf, "-Dnetbeans.home=");
        strcat(buf, idehome);
        jav[jac++] = strdup(buf);
    } else {
        char *p = strchr(jav[nbi], '=');
        if (p != NULL) {
            strcpy(idehome, p + 1);            
        }
    }
    
    // update classpath
    if (cpi == -1) {
        jav[jac++] = "-classpath";
        strcpy(buf, idehome);
        strcat(buf, "\\lib\\ext\\rmi-ext.jar");        
        jav[jac++] = strdup(buf);
    } else if (cpFlag) {
        strcpy(buf, idehome);
        strcat(buf, "\\lib\\ext\\rmi-ext.jar");        
        jav[jac++] = strdup(buf);
    } else {
        strcpy(buf, jav[cpi + 1]);
        strcat(buf, ";");
        strcat(buf, idehome);
        strcat(buf, "\\lib\\ext\\rmi-ext.jar");        
        jav[cpi + 1] = strdup(buf);                
    }
    
    // add ide.home

    argc = jac + rac + 3; // process name, class, NULL
    cmd2 = *cmd = (char **) malloc(argc * sizeof(char *));
    
    cmd2[0] = argv[0];
    j = 1;
    for(i = 0; i < jac; i++) {
        cmd2[j++] = jav[i];
    }
    cmd2[j++] = RMID_MAIN_CLASS;
    for(i = 0; i < rac; i++) {
        cmd2[j++] = rav[i];
    }
    cmd2[j] = NULL;
         
    // test
    // for(i = 0; i < argc; i++) printf("%s\n", cmd2[i]);

    return 0;    
}

