/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.rmi;

import java.lang.reflect.*;

import org.openide.TopManager;
import org.openide.ErrorManager;
import org.openide.compiler.Compiler;
import org.openide.compiler.*;
import org.openide.loaders.*;
import org.openide.modules.ModuleInstall;
import org.openide.src.nodes.*;
import org.openide.util.*;

import org.openidex.util.*;

/** Module installer.
*
* @author Martin Ryzl, Jan Pokorsky
*/
public class RMIModule extends ModuleInstall {

    /** My own request processor. */
    static RequestProcessor rp = null;

    /** Serial version UID. */
    static final long serialVersionUID = -4791113880926743826L;

    public void installed() {
/*        try {
            DataFolder toolsFolder = DataFolder.create (TopManager.getDefault().getPlaces().folders().menus(), "Tools"); // NOI18N
            if (toolsFolder != null) {
                Utilities2.createAction(org.netbeans.modules.rmi.wizard.RMIWizardAction.class, toolsFolder, "ToolsAction", true, true, false, false); // NOI18N
            }
        } catch (Exception ex) {
            if (Boolean.getBoolean ("netbeans.debug.exceptions")) ex.printStackTrace (); // NOI18N
        }

        copyTemplates ();
        copyOther ();
 */
        restored();
    }

    public void restored() {
        java.beans.PropertyEditorManager.registerEditor(
            java.rmi.activation.ActivationGroupDesc.class,
            org.netbeans.modules.rmi.activation.ActivationGroupDescEditor.class
        );
        java.beans.PropertyEditorManager.registerEditor(
            java.rmi.activation.ActivationDesc.class,
            org.netbeans.modules.rmi.activation.ActivationDescEditor.class
        );
    }

    public void uninstalled() {
        
        // cancel parsing listener
        RMIDataLoader.cancelParsingListener();
        
        if (rp != null) rp.stop();
        rp = null;
        
        // [PENDING - remove installed files]
/*        try {
            DataFolder toolsFolder = DataFolder.create (TopManager.getDefault().getPlaces().folders().menus(), "Tools"); // NOI18N
            if (toolsFolder != null) {
                Utilities2.removeAction(org.netbeans.modules.rmi.wizard.RMIWizardAction.class, toolsFolder);
            }
        } catch (Exception ex) {
            if (Boolean.getBoolean ("netbeans.debug.exceptions")) ex.printStackTrace (); // NOI18N
        }
 */
    }

    public boolean closing() {
        if (rp != null) rp.stop();
        return true;
    }

    // -----------------------------------------------------------------------------
    // Private methods

    /** Get RMI RequestProcessor.
    */
    public static RequestProcessor getRP() {
        if (rp == null) rp = new RequestProcessor("RMI Request Processor"); // NOI18N
        return rp;
    }
    
    /** Get error manager for package of clazz. */
    public static ErrorManager getErrorManager(Class clazz) {
        return TopManager.getDefault().getErrorManager().getInstance(
            clazz.getClass().getPackage().getName()
        );
    }

    /** Install templates.
    */
    private void copyTemplates () {
        try {
            org.openide.filesystems.FileUtil.extractJar (
                org.openide.TopManager.getDefault ().getPlaces ().folders().templates ().getPrimaryFile (),
                getClass ().getClassLoader ().getResourceAsStream ("org/netbeans/modules/rmi/toinstall/templates.jar") // NOI18N
            );
        } catch (java.io.IOException e) {
            org.openide.TopManager.getDefault ().notifyException (e);
        }
    }

    /** Install other files.
    */
    private void copyOther () {
        try {
            org.openide.filesystems.FileUtil.extractJar (
                org.openide.TopManager.getDefault ().getRepository ().getDefaultFileSystem ().getRoot (),
                getClass ().getClassLoader ().getResourceAsStream ("org/netbeans/modules/rmi/toinstall/rmi.jar") // NOI18N
            );
        } catch (java.io.IOException e) {
            org.openide.TopManager.getDefault ().notifyException (e);
        }
    }

    // Remote patterns
    /** Dynamicaly registers ElementFactory.
     * @param className Name of class which registers the factories.
     * @param methodName Name of method for registering factories.
     * @param factory The factory to register.
     */
    private void invokeDynamic( String className, String methodName, FilterFactory factory ) {
        try {
            Class dataObject = TopManager.getDefault().systemClassLoader().loadClass( className );

            if ( dataObject == null )  return;

            Method method = dataObject.getDeclaredMethod( methodName, new Class[] { FilterFactory.class }  );
            if ( method == null ) return;

            method.invoke( null, new Object[] { factory } );
        }
        catch (Exception ex) {
            if (Boolean.getBoolean ("netbeans.debug.exceptions")) ex.printStackTrace (); // NOI18N
        }
    }
}
