/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.rmi.activation;

import org.openide.util.NbBundle;

/**
 * Find out data necessary for registration to a registry.
 * @author  vzboril, Jan Pokorsky
 */
public class RegistrationPanel extends javax.swing.JPanel {

    /** Creates new form RegistrationPanel */
    public RegistrationPanel(String registryHost) {
        initComponents();
        jTextArea1.setText(NbBundle.getMessage(RegistrationPanel.class, "LBL_RegistrationPanel.Info", registryHost)); // NOI18N
        jTextField1.setText(String.valueOf(java.rmi.registry.Registry.REGISTRY_PORT));
        jTextField1.selectAll();
    }
    
    /** Service name to bind to registry. */
    public String getServiceName() {
        return jTextField2.getText();
    }
    
    /** Port number to lookup registry.
     * @throws NumberFormatException if port is not 1-65535
     */
    public int getPort() throws NumberFormatException {
        String portStr = jTextField1.getText();
        int port = java.rmi.registry.Registry.REGISTRY_PORT;
        if (portStr != null && portStr.length() > 0) port = Integer.parseInt(portStr);
        if (port < 1 || port > 65535) throw new NumberFormatException();
        
        return port;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jLabel2 = new javax.swing.JLabel();
        jTextField1 = new javax.swing.JTextField();
        jLabel3 = new javax.swing.JLabel();
        jTextField2 = new javax.swing.JTextField();
        jTextArea1 = new javax.swing.JTextArea();
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        jLabel2.setText(getString("LBL_RegistrationPanel.Port"));
        jLabel2.setDisplayedMnemonic(getMnemonic("LBL_RegistrationPanel.Port"));
        jLabel2.setLabelFor(jTextField1);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.insets = new java.awt.Insets(12, 11, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(jLabel2, gridBagConstraints1);
        
        
        jTextField1.setColumns(15);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.gridwidth = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(12, 11, 0, 12);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints1.weightx = 1.0;
        add(jTextField1, gridBagConstraints1);
        
        
        jLabel3.setText(getString("LBL_RegistrationPanel.Name"));
        jLabel3.setDisplayedMnemonic(getMnemonic("LBL_RegistrationPanel.Name"));
        jLabel3.setLabelFor(jTextField2);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.insets = new java.awt.Insets(12, 11, 17, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(jLabel3, gridBagConstraints1);
        
        
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.gridwidth = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(11, 12, 17, 12);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints1.weightx = 1.0;
        add(jTextField2, gridBagConstraints1);
        
        
        jTextArea1.setLineWrap(true);
        jTextArea1.setEditable(false);
        jTextArea1.setForeground(new java.awt.Color (102, 102, 153));
        jTextArea1.setText(getString("LBL_RegistrationPanel.Info"));
        jTextArea1.setBackground(new java.awt.Color (204, 204, 204));
        jTextArea1.setWrapStyleWord(true);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.gridwidth = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(12, 12, 0, 12);
        add(jTextArea1, gridBagConstraints1);
        
    }//GEN-END:initComponents

    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel2;
    private javax.swing.JTextField jTextField1;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JTextField jTextField2;
    private javax.swing.JTextArea jTextArea1;
    // End of variables declaration//GEN-END:variables

  /** Localization. */
  private String getString(java.lang.String key) {
     return org.openide.util.NbBundle.getBundle (RegistrationPanel.class).getString(key);
  }    
  
  private static final String mnemonic_suffix = ".mnemonic"; // NOI18N
  
  private char getMnemonic(java.lang.String key) {
     return org.openide.util.NbBundle.getBundle (RegistrationPanel.class).getString(key + mnemonic_suffix).charAt(0);
  }
}
