/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.rmi.serialized;

import java.io.*;

import org.openide.*;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileLock;
import org.openide.util.io.NbObjectOutputStream;

import org.netbeans.modules.rmi.RMIExecutor;

/**
 *
 * @author  mryzl
 */

public class RMIExecutorGenerator extends Object {

    /** Creates new RMIExecutorGenerator. */
    public RMIExecutorGenerator() {
    }

    /** Get folder where serialized object should be stored.
    */
    protected static FileObject getFolder() {
        return TopManager.getDefault().getRepository().findResource("org/netbeans/modules/rmi/serialized"); // NOI18N
    }

    protected static void create(String fileName, int i) throws IOException {
        FileObject target, folder = getFolder();
        FileLock lock = null;
        ObjectOutputStream oos = null;

        if ((folder == null) || (!folder.isFolder())) throw new IOException("non existent folder"); // NOI18N

        target = folder.getFileObject(fileName, "ser"); // NOI18N
        if (target == null) {
            target = folder.createData(fileName, "ser"); // NOI18N
        }

        try {
            lock = target.lock();
            oos = new NbObjectOutputStream(target.getOutputStream(lock));
            oos.writeObject(new RMIReplacer(i));
        } finally {
            if (lock != null) lock.releaseLock();
            if (oos != null) oos.close();
        }
    }
    
    private static ExecutorData[] data;
    
    static ExecutorData[] getExecutorData() {
        if (data == null) {
            data = new ExecutorData[] {
                null,
                new ExecutorData(getString("PROP_RMIExecutorDbgName"), "{java.home}{/}bin{/}java", "{classic} -Xdebug -Djava.compiler=NONE -Xbootclasspath:{java.home}{/}..{/}lib{/}tools.jar{:}{bootclasspath} -cp {filesystems};{classpath} -Djava.security.policy={filesystemsIPURL}RMI/rmi.policy -Djava.rmi.server.codebase={filesystemsIPURL} -DJava.rmi.server.hostname={hostip} {classname} {arguments}"),
                new ExecutorData(getString("PROP_RMICLientExecutorName"), "{java.home}{/}bin{/}java", "-cp {filesystems}{:}{classpath} -Djava.security.policy={filesystemsIPURL}RMI/rmi.policy {classname} {arguments}"),
                new ExecutorData(getString("PROP_RMICLientExecutorDbgName"), "{java.home}{/}bin{/}java", "{classic} -Xdebug -Djava.compiler=NONE -Xbootclasspath:{java.home}{/}..{/}lib{/}tools.jar{:}{bootclasspath} -cp {filesystems}{:}{classpath} -Djava.security.policy={filesystemsIPURL}RMI/rmi.policy {classname} {arguments}"),
                new ExecutorData(getString("PROP_RMIUnicastExportName"), "{java.home}{/}bin{/}java", "-cp {filesystems}{:}{classpath}{:}{library} -Djava.security.policy={filesystemsIPURL}RMI/rmi.policy -Djava.rmi.server.codebase={filesystemsIPURL} -DJava.rmi.server.hostname={hostip} org.netbeans.modules.rmi.UnicastExportClass -port {port} -service {service}  {classname}"),
                new ExecutorData(getString("PROP_RMIUnicastExportDbgName"), "{java.home}{/}bin{/}java", "{classic} -Xdebug -Djava.compiler=NONE -Xbootclasspath:{java.home}{/}..{/}lib{/}tools.jar{:}{bootclasspath} -cp {filesystems}{:}{classpath}{:}{library} -Djava.security.policy={filesystemsIPURL}RMI/rmi.policy -Djava.rmi.server.codebase={filesystemsIPURL} -DJava.rmi.server.hostname={hostip} org.netbeans.modules.rmi.UnicastExportClass -port {port} -service {service} {classname}"),
            };
        }
        return data;
    }
    
    static String getString(String key) {
        String name = null;
        try {
            name = org.openide.util.NbBundle.getBundle(RMIReplacer.class).getString(key);
        } catch (java.util.MissingResourceException ex) {
            return key;
        }
        return name;
    }
    
    static int[] indices;
    static int lastIndex;
    
    static RMIExecutor getByIndex(int index) {
        if (indices == null) {
            indices = new int[getExecutorData().length];
            lastIndex = 0;
            for(int i = 0; i < indices.length; indices[i++] = -1);
        }
        if ((index >= indices.length) || (index < 0)) return getExecutor(null);
        if (indices[index] == -1) indices[index] = lastIndex++;
        return getExecutor(getExecutorData()[indices[index]]);
    }
    
    static RMIExecutor getExecutor(ExecutorData ed) {
        if (ed == null) return new RMIExecutor();
        return new RMIExecutor(ed.name, ed.processName, ed.arguments);
    }
    
    /**
    */
    public static void main(String[] args) throws IOException {
        /*
        create("RMIClientExecutor", "PROP_RMICLientExecutorName", "{java.home}{/}bin{/}java", "-cp {filesystems}{:}{classpath} -Djava.security.policy={filesystemsIPURL}RMI/rmi.policy {classname} {arguments}"); // NOI18N
        create("RMIClientExecutorDbg", "PROP_RMICLientExecutorDbgName", "{java.home}{/}bin{/}java", "-classic -Xdebug -Djava.compiler=NONE -Xbootclasspath:{java.home}{/}..{/}lib{/}tools.jar{:}{bootclasspath} -cp {filesystems}{:}{classpath} -Djava.security.policy={filesystemsIPURL}RMI/rmi.policy {classname} {arguments}"); // NOI18N
        create("RMIExecutorDbg", "PROP_RMIExecutorDbgName", "{java.home}{/}bin{/}java", "-classic -Xdebug -Djava.compiler=NONE -Xbootclasspath:{java.home}{/}..{/}lib{/}tools.jar{:}{bootclasspath} -cp {filesystems};{classpath} -Djava.security.policy={filesystemsIPURL}RMI/rmi.policy -Djava.rmi.server.codebase={filesystemsIPURL} -DJava.rmi.server.hostname={hostip} {classname} {arguments}"); // NOI18N
        create("RMIUnicastExport", "PROP_RMIUnicastExportName", "{java.home}{/}bin{/}java", "-cp {filesystems}{:}{classpath}{:}{library} -Djava.security.policy={filesystemsIPURL}RMI/rmi.policy -Djava.rmi.server.codebase={filesystemsIPURL} -DJava.rmi.server.hostname={hostip} org.netbeans.modules.rmi.UnicastExportClass -port {port} -service {service}  {classname}"); // NOI18N
        create("RMIUnicastExportDbg", "PROP_RMIUnicastExportDbgName", "{java.home}{/}bin{/}java", "-classic -Xdebug -Djava.compiler=NONE -Xbootclasspath:{java.home}{/}..{/}lib{/}tools.jar{:}{bootclasspath} -cp {filesystems}{:}{classpath}{:}{library} -Djava.security.policy={filesystemsIPURL}RMI/rmi.policy -Djava.rmi.server.codebase={filesystemsIPURL} -DJava.rmi.server.hostname={hostip} org.netbeans.modules.rmi.UnicastExportClass -port {port} -service {service} {classname}"); // NOI18N
        */
        ExecutorData[] ed = getExecutorData();
        
        for(int i = 0; i < ed.length; i++) {
            create("RMIExecutor_" + i, i);
        }
    }
    
    public static class ExecutorData {
        final String name, processName, arguments;
        
        ExecutorData(String name, String processName, String arguments) {
            this.name = name;
            this.processName = processName;
            this.arguments = arguments;
        }
    }
    
}
