/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.rmi.settings;

import java.io.*;
import java.util.*;

import org.openide.execution.*;
import org.openide.options.*;
import org.openide.util.*;

import org.netbeans.modules.rmi.activation.ActivationSystemItem;

/**
 *
 * @author  mryzl
 */

public class RMIRegistryItems extends SystemOption {

    /** Serial version UID. */
    static final long serialVersionUID = -3141182121111791783L;

    /** Name of the property regs. */
    public static final String PROP_REGS = "regs"; // NOI18N
    
    /** Name of the property. */
    public static final String PROP_AS = "as"; // NOI18N

    /** Lock. */
    private static Object lock = new Object();
    
    /** Holds value of property regs. */
    private static Set regs;

    /** */
    private static Set asitems = new HashSet(8);
    
    /** Creates new RMIRegistrySettings. 
    */
    public RMIRegistryItems() {
        // don't use setXXX here !!!
    }

    /** Get default instance.
     * @return default instance
     */
    public static RMIRegistryItems getInstance() {
        return (RMIRegistryItems) findObject(RMIRegistryItems.class, true);
    }

    /** Defines whether the option is project or global specific.
    * @return true if the option is global.
    */
    public boolean isGlobal() {
        return false;
    }

    /** Get a human presentable name of the action.
    * This may be
    * presented as an item in a menu.
    * @return the name of the action
    */
    public String displayName () {
        return NbBundle.getBundle(RMIRegistryItems.class).getString("PROP_RMIRegistryItems"); // NOI18N
    }

    /** Get a help context for the action.
    * @return the help context for this action
    */
    public HelpCtx getHelpCtx () {
        return new HelpCtx(RMIRegistryItems.class);
    }

    /** Getter for property regs.
     * @return Value of property regs.
     */
    public Set getRegs() {
        if (regs == null) regs = new HashSet(4);
        return regs;
    }

    /** Setter for property regs.
     *@param regs New value of property regs.
     */
    public void setRegs(Set regs) {
        Set old = this.regs;
        this.regs = regs;
        firePropertyChange(PROP_REGS, old, regs);
    }
    
    /** Getter for property as.
     * @return an array of items
     */
    public ActivationSystemItem[] getAS() {
        synchronized (lock) {
            return (ActivationSystemItem[]) asitems.toArray(new ActivationSystemItem[asitems.size()]);
        }
    }

    /** Return a new set containing all activation system items.
     * @return a new set of items
     */
    public Set getASSet() {
        synchronized (lock) {
            return Collections.unmodifiableSet(asitems);
        }
    }
    
    /** Setter for property as.
     * @param items a new item set
     */
    public void setAS(ActivationSystemItem[] items) {
        synchronized (lock) {
            asitems = new HashSet();
            for(int i = 0; i < items.length; i++) asitems.add(items[i]);
        }
        firePropertyChange(PROP_AS, null, null);
    }
    
    /** Add one activation system item.
     * @param item - a new item
     */
    public void addAS(ActivationSystemItem item) {
        boolean added;
        
        synchronized (lock) {
            added = asitems.add(item);
        }
        if (added) firePropertyChange(PROP_AS, null, null);
    }

    /** Remove one activation system item.
     * @param item - an item to be removed
     */
    public void removeAS(ActivationSystemItem item) {
        boolean removed = false;
        synchronized (lock) {
            removed = asitems.remove(item);
        }
        if (removed) firePropertyChange(PROP_AS, null, null);
        item.destroy();
    }
}
