/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.rmi.settings;

import java.beans.*;
import java.io.*;
import java.util.ResourceBundle;
import java.util.Properties;

import org.openide.options.ContextSystemOption;
import org.openide.options.SystemOption;
import org.openide.util.NbBundle;
import org.openide.util.io.ReaderInputStream;

import org.netbeans.modules.rmi.wizard.*;

/** Settings for RMI.
*
* @author Martin Ryzl
*/
public class RMISettings extends ContextSystemOption {

    /** Serial version UID. */
    static final long serialVersionUID = -8522143676848697297L;

    /** Name of the property detectRemote. */
    public static final String PROP_DETECT_REMOTE = "detectRemote"; // NOI18N

    /** Name of the property confirmConvert. */
    public static final String PROP_CONFIRM_CONVERT = "confirmConvert"; // NOI18N

    /** The resource bundle. */
    public static ResourceBundle bundle;

    /** Holds value of property detectRemote. */
    private static boolean detectRemote = true;

    /** Holds value of property confirmConvert. */
    private static boolean confirmConvert = true;

    public RMISettings() {
        addOption(RMIActivationSettings.getDefault());
        addOption(getRMIRegistrySettings());
    }

    /** Display name.
    */
    public String displayName () {
        return getString("CTL_RMI_SETTINGS"); // NOI18N
    }

    /** 
     * @return localized string 
     */
    static String getString(String s) {
        if (bundle == null) {
            bundle = NbBundle.getBundle(RMISettings.class);
        }
        return bundle.getString(s);
    }

    /** Returns RMIRegistrySettings.
     */
    public static RMIRegistrySettings getRMIRegistrySettings() {
        return (RMIRegistrySettings) RMIRegistrySettings.findObject(RMIRegistrySettings.class, true);
    }

    /** Getter for property detectRemote.
     * @return Value of property detectRemote.
     */
    public boolean isDetectRemote() {
        return detectRemote;
    }
    
    /** Setter for property detectRemote.
    * @param detectsRemote New value of property detectRemote.
    */
    public void setDetectRemote(boolean detectRemote) {
        boolean oldDetectRemote = this.detectRemote;
        this.detectRemote = detectRemote;
        firePropertyChange (PROP_DETECT_REMOTE, new Boolean(oldDetectRemote), new Boolean(detectRemote));
    }
    
    /** Getter for property confirmConvert.
     * @return Value of property confirmConvert.
     */
    public boolean isConfirmConvert() {
        return confirmConvert;
    }
    
    /** Setter for property confirmConverting.
     * @param confirmConvert New value of property confirmConvert.
     */
    public void setConfirmConvert(boolean confirmConvert) {
        boolean oldConfirmConvert = this.confirmConvert;
        this.confirmConvert = confirmConvert;
        firePropertyChange (PROP_CONFIRM_CONVERT, new Boolean(oldConfirmConvert), new Boolean(confirmConvert));
    }
}

/*
 * <<Log>>
 */
