/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.rmi.wizard;

import org.openide.NotifyDescriptor;
import org.openide.TopManager;
import org.openide.loaders.DataFolder;
import org.openide.nodes.Node;
import org.openide.util.NbBundle;
import org.openide.util.HelpCtx;
import org.openide.util.actions.*;

/** Action that is enabled on any folder and if selected opens
* a new Explorer with a simple version of the Object Browser.
*
* @author Martin Ryzl
*/
public class RMIWizardAction extends NodeAction {

    /** Name of the icon. */
    public static final String WIZARD_ICON = "/org/netbeans/modules/rmi/resources/rmiWizard.gif"; // NOI18N

    static final long serialVersionUID =2151471284881427046L;
    
    /** Create a new action. */
    public RMIWizardAction() {
    }

    /** Name of the action. */
    public String getName () {
        return NbBundle.getBundle (RMIWizardAction.class).getString ("CTL_RMIWizardAction"); // NOI18N
    }

    /** No help yet. */
    public org.openide.util.HelpCtx getHelpCtx () {
        return HelpCtx.DEFAULT_HELP;
    }

    /** Test whether the action should be enabled based on the currently activated nodes.
     * @param arr - can contain either an empty array or an array with one node that
     * has DataFolder cookie.
     */
    protected boolean enable(Node[] activatedNodes) {
        return true;
    }

    /** Open the view.
     * @param arr - can contain either an empty array or an array with one node that
     * has DataFolder cookie.
     */
    protected void performAction(Node[] activatedNodes) {
        RMIWizard wizard = new RMIWizard();
        try {
            int len = activatedNodes.length;
            if (len == 1) {
                DataFolder folder = (DataFolder) activatedNodes[0].getCookie(DataFolder.class);
                if (folder != null) wizard.instantiate(folder);
                else wizard.instantiate();
            } else { 
                wizard.instantiate();
            }
        } catch (java.io.IOException e) {
            String msg = e.getMessage();
            if ((msg == null) || msg.equals("")) // NOI18N
                msg = NbBundle.getBundle(RMIWizardAction.class).getString("EXC_WizardFailed"); // NOI18N
            TopManager.getDefault().notify(new NotifyDescriptor.Message(msg, NotifyDescriptor.ERROR_MESSAGE));
        }
    }

    protected String iconResource() {
        return WIZARD_ICON;
    }
}

/*
 * <<Log>>
 *  4    Gandalf   1.3         11/27/99 Patrik Knakal   
 *  3    Gandalf   1.2         10/23/99 Ian Formanek    NO SEMANTIC CHANGE - Sun
 *       Microsystems Copyright in File Comment
 *  2    Gandalf   1.1         10/13/99 Martin Ryzl     icon updated
 *  1    Gandalf   1.0         7/27/99  Martin Ryzl     
 * $
 */
