/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.scripting;

import org.openide.*;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.CallableSystemAction;
import org.openide.execution.ScriptType;

/** 
 * Opens a scripting console window.
 * @author Jaroslav Tulach
 */
public class ConsoleAction extends CallableSystemAction {
    /** The console.
     */
    private static EvaluateConsole console;
  
  
    public void performAction () {
        
        if (!atLeastOneScriptTypeExists()) {
            TopManager.getDefault().notify(new NotifyDescriptor.Message(
                NbBundle.getBundle (ConsoleAction.class).getString("MSG_NoScriptType")));
            return;
        }
        
        if (console == null) {
            console = new EvaluateConsole ();
        }
        console.open ();
        console.requestFocus ();
    }
  
    public String getName () {
        return NbBundle.getBundle (ConsoleAction.class).getString (
            "displayNameForConsoleAction"
        );
    }

    protected String iconResource () {
        return "ConsoleActionIcon.gif";
    }

    public HelpCtx getHelpCtx () {
        return HelpCtx.DEFAULT_HELP;
    // If you will provide context help then use:
    // return new HelpCtx (ConsoleAction.class);
    }

    /** This action is enabled only in case that there is
     * at least one ScriptType registered.
     */
    private boolean atLeastOneScriptTypeExists() {
        try {
            return ScriptType.getDefault() != null;
        } catch (Exception x) {
            // silently ignore any exceptions and
            // just returns false
        }
        return false;
    }

  /** Perform extra initialization of this action's singleton.
  * PLEASE do not use constructors for this purpose!
  protected void initialize () {
    super.initialize ();
    putProperty ("someProp", value);
  }
  */
}