/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.scripting;

import java.util.*;
import java.io.*;
import java.lang.reflect.*;

import org.openide.*;
//import koala.dynamicjava.interpreter.*;
//import koala.dynamicjava.parser.wrapper.*;

/**
 * This is an interface to the DynamicJava (http://www.inria.fr/koala/djava/)
 * @author David Strupl
 */

public class DynamicJavaScriptType extends AbstractScriptType {
    
    static final long serialVersionUID = -6919647689707066209L;
    
    /** The compile type is object - using reflection to call methods. 
     * Should hold reference to a bsh.Interpreter instance.
     */
    private transient Object interpreter;
    
    /** We call interpreter using reflection.*/
    private transient Class interpreterClass;
    
    /** 
     * When this is true we no longer log exceptions from the addVariable
     * method to the ErrorManager.
     */
    private transient boolean problemReported = false;
    
    /**
     * Registers djava file extension.
     */
    public DynamicJavaScriptType() {
        // Register extensions to be recognized for this script type
        getExtensions().addExtension("djava"); // NOI18N
    }
    
    /** 
      * create an interpreter - using reflection to call this:
      * interpreter = new koala.dynamicjava.interpreter.TreeInterpreter(
      *    new koala.dynamicjava.parser.wrapper.JavaCCParserFactory());        
      */
     private Object getInterpreter() throws Exception {
        
        if (interpreter == null) {
            // create an interpreter
            Class parserClass = Class.forName("koala.dynamicjava.parser.wrapper.JavaCCParserFactory"); //NOI18N
            Object parser = parserClass.newInstance();
            interpreterClass = Class.forName("koala.dynamicjava.interpreter.TreeInterpreter"); //NOI18N
            Class parserParamClass = Class.forName("koala.dynamicjava.parser.wrapper.ParserFactory"); //NOI18N
            Constructor c = interpreterClass.getConstructor(new Class[] { parserParamClass });
            interpreter = c.newInstance(new Object[] { parser } );
            // register to gain access to the IDE objects
            registerNavigationSupport();
        }
        return interpreter;

    }
    
    /** Under this name the type will be known.*/
    protected String displayName () {
        return "DynamicJava"; // NO-I18N
    }

    /**
     * Evaluate an expression. Uses reflection to call interpreter.interpret(...);
     */
    public Object eval(String s, Context context) throws InvocationTargetException {
        try {
            if (!s.endsWith(";")) { // NOI18N
                s += ";";           // NOI18N
            }
            Reader r = new StringReader(s);
            
            Object o = getInterpreter();
            Method m = interpreterClass.getMethod("interpret", new Class[] { java.io.Reader.class, String.class }); // NOI18N
            return m.invoke(o, new Object[] { r, displayName() });

           // return getInterpreter().interpret(r, displayName());
        } catch (Exception e) {
            if (problemReported) {
                TopManager.getDefault().notify(new NotifyDescriptor.Message("Maybe the file lib/ext/djava.jar is missing."));
            } 
            throw new InvocationTargetException(e, displayName());
        }
    }
    
    /**
     * Uses reflection to call interpreter.set(...).
     */
    public void addVariable(String name, Object value) {
        //getInterpreter().defineVariable(name, value);
        try {
            // getInterpreter().set(name, value);
            Object o = getInterpreter();
            Method m = interpreterClass.getMethod("defineVariable", new Class[] { String.class, Object.class }); // NOI18N
            m.invoke(o, new Object[] { name, value });
        } catch (Exception e) {
            if (!problemReported) {
                //TopManager.getDefault().getErrorManager().notify(ErrorManager.INFORMATIONAL, e);
                problemReported = true;
            }
        }
    }
}
