/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.scripting;

import java.awt.Component;

import javax.swing.text.*;
import javax.swing.JEditorPane;

import org.openide.cookies.EditorCookie;
import org.openide.nodes.Node;
import org.openide.util.actions.NodeAction;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;

/** Evaluates currently selected text and pastes the result
* in the cursor position.
*
* @author  Jaroslav Tulach
*/

public class EvaluateAction extends NodeAction {

    public HelpCtx getHelpCtx () {
        return HelpCtx.DEFAULT_HELP;
    }
  
    public String getName () {
        return NbBundle.getBundle (EvaluateAction.class).getString ("ACT_EvaluateAction");
    }
  
    protected boolean enable (Node[] n) {
        if (n.length != 1) {
            return false;
        }
    
        EditorCookie ec = (EditorCookie)n[0].getCookie (EditorCookie.class);
        if (ec == null) {
            return false;
        }
    
        JEditorPane[] panes = ec.getOpenedPanes ();
        
        if ((panes != null)&&(panes.length>0)) {
            ScriptObject so = (ScriptObject)n[0].getCookie (ScriptObject.class);
            if (so != null) {
                return true;
            }
            
            Component c = (Component)panes[0];
            String t = null;
      
            while (c != null) {
                c = c.getParent();
        
                if (c instanceof EvaluateConsole) {
                    return true;
                }
            }
        }
        return false;
    }

    /** Performs action
     */
    protected void performAction (Node[] n) {
        if (n.length != 1) return;
        
        EditorCookie ec = (EditorCookie)n[0].getCookie (EditorCookie.class);
        if (ec == null) return;
        
        JEditorPane[] panes = ec.getOpenedPanes ();
        if ((panes != null)&&(panes.length>0)) {
            ScriptObject so = (ScriptObject)n[0].getCookie (ScriptObject.class);
            if (so != null) {
                String lang = so.getScriptLanguage ();
                ScriptUtils.evaluate (panes[0], lang);
                return;
            }
            
            Component c = (Component)panes[0];
            String t = null;
            while (c != null) {
                c = c.getParent();
                if (c instanceof EvaluateConsole) {
                    t = ((EvaluateConsole)c).getScriptLanguage ();
                    ScriptUtils.evaluate (panes[0], t);
                    return;
                }
            }
        }
    }

    public void actionPerformed(java.awt.event.ActionEvent ev) {
        if (ev.getSource() instanceof JEditorPane) {
            Component c = (Component)ev.getSource ();
            String t = null;
            while (c != null) {
                c = c.getParent();
        
                if (c instanceof EvaluateConsole) {
                    t = ((EvaluateConsole)c).getScriptLanguage ();
                    break;
                }
            }
            if (t == null) {
                Node [] n = getActivatedNodes();
                if ((n != null) && (n.length > 0)) {
                    ScriptObject so = (ScriptObject)n[0].getCookie (ScriptObject.class);
                    if (so != null) {
                        t = so.getScriptLanguage ();
                    }
                }
            }
            ScriptUtils.evaluate ((JEditorPane)ev.getSource (), t);
        }
        else {
            Node [] n = getActivatedNodes();
            if ((n != null) && (n.length > 0)) {
                performAction(n);
                return;
            }
        }
    }
}
