/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.scripting;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import javax.swing.*;
import javax.swing.text.*;

import org.openide.windows.TopComponent;
import org.openide.explorer.propertysheet.*;
import org.openide.cookies.*;
import org.openide.util.*;
import org.openide.text.*;
import org.openide.nodes.*;
import org.openide.execution.*;

/** Window where one can execute script commands. 
*
* @author  Jaroslav Tulach, David Strupl
*/
public class EvaluateConsole extends TopComponent implements EditorCookie {
    /** engine to use */
    private String lang = null;
  
    /** Creates new form EvaluateConsole */
    public EvaluateConsole() {
        lang = ScriptType.getDefault().getName();
        initComponents ();
      
        setName(
            org.openide.util.NbBundle.getBundle(
                "org.netbeans.modules.scripting.Bundle"
            ).getString("EvaluateConsole.text")    
        );
        pane.setContentType("text/x-java");

        setActivatedNodes(new Node[] { new EvaluateNode(this) });
    }
  
    /** Setter for script type.
     */
    public void setScriptLanguage (String t) {
        lang = t;
    }
  
    /** Getter for script type.
     */
    public String getScriptLanguage () {
        return lang;
    }
    
    public Dimension getMinimumSize(){
        return new Dimension (400,400);
    }
  
        
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jLabel1 = new javax.swing.JLabel();
        jLabel1.setDisplayedMnemonic(org.openide.util.NbBundle.getBundle("org.netbeans.modules.scripting.Bundle").getString("LBL_ScriptLang").charAt(0));
        propertyPanel1 = new PropertyPanel(new LanguagePropertyModel(), PropertyPanel.PREF_INPUT_STATE);
        execute = new javax.swing.JButton();
        evaluate = new javax.swing.JButton();
        panePanel = new javax.swing.JScrollPane();
        pane = new javax.swing.JEditorPane();
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        setPreferredSize(new java.awt.Dimension(600, 700));
        setMinimumSize(new java.awt.Dimension(500, 300));
        
        jLabel1.setText(org.openide.util.NbBundle.getBundle("org.netbeans.modules.scripting.Bundle").getString("LBL_ScriptLang"));
        jLabel1.setLabelFor(propertyPanel1);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.insets = new java.awt.Insets(12, 12, 11, 11);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.EAST;
        add(jLabel1, gridBagConstraints1);
        
        
        propertyPanel1.setPreferredSize(new java.awt.Dimension(150, 24));
        propertyPanel1.setMinimumSize(new java.awt.Dimension(150, 24));
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(12, 0, 11, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints1.weightx = 0.4;
        add(propertyPanel1, gridBagConstraints1);
        
        
        execute.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/openide/resources/actions/execute.gif")));
        execute.setToolTipText(org.openide.util.NbBundle.getBundle("org.netbeans.modules.scripting.Bundle").getString("HINT_Execute"));
        execute.setPreferredSize(new java.awt.Dimension(24, 24));
        execute.setMaximumSize(new java.awt.Dimension(24, 24));
        execute.setMinimumSize(new java.awt.Dimension(24, 24));
        execute.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                executeActionPerformed(evt);
            }
        }
        );
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.insets = new java.awt.Insets(12, 0, 11, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints1.weightx = 1.0;
        add(execute, gridBagConstraints1);
        
        
        evaluate.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/netbeans/modules/scripting/resources/evaluate.gif")));
        evaluate.setToolTipText(org.openide.util.NbBundle.getBundle("org.netbeans.modules.scripting.Bundle").getString("HINT_Evaluate"));
        evaluate.setPreferredSize(new java.awt.Dimension(24, 24));
        evaluate.setMaximumSize(new java.awt.Dimension(24, 24));
        evaluate.setMinimumSize(new java.awt.Dimension(24, 24));
        evaluate.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                evaluateActionPerformed(evt);
            }
        }
        );
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridwidth = 0;
        gridBagConstraints1.insets = new java.awt.Insets(12, 2, 11, 11);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(evaluate, gridBagConstraints1);
        
        
        
        panePanel.setViewportView(pane);
        
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.gridwidth = 0;
        gridBagConstraints1.gridheight = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(panePanel, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void evaluateActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_evaluateActionPerformed
        ScriptUtils.evaluate (pane, lang);
    }//GEN-LAST:event_evaluateActionPerformed

    private void executeActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_executeActionPerformed
        ScriptUtils.execute (pane, lang);
    }//GEN-LAST:event_executeActionPerformed

    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel1;
    private org.openide.explorer.propertysheet.PropertyPanel propertyPanel1;
    private javax.swing.JButton execute;
    private javax.swing.JButton evaluate;
    private javax.swing.JScrollPane panePanel;
    private javax.swing.JEditorPane pane;
    // End of variables declaration//GEN-END:variables

  //  private static final java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("org/netbeans/modules/scripting/Bundle");    
   

    /** Implementing interface EditorCookie */
    public Task prepareDocument(){
        return null;
    }

    /** Implementing interface EditorCookie */
    public javax.swing.text.StyledDocument openDocument()throws java.io.IOException {
        return null;
    }

    /** Implementing interface EditorCookie */
    public StyledDocument getDocument(){
        Document d = pane.getDocument();
        if (d instanceof StyledDocument) {
            return (StyledDocument)d;
        } else {
            return null;
        }
    }

    /** Implementing interface EditorCookie */
    public void saveDocument() throws java.io.IOException {
    }

    public boolean isModified() {
        return false;
    }

    /** Implementing interface EditorCookie */
    public javax.swing.JEditorPane[] getOpenedPanes() {
        return new javax.swing.JEditorPane[] {pane};
    }

    /** Implementing LineCookie interface */
    public Line.Set getLineSet(){
        return null;
    }
    
    private class LanguagePropertyModel extends Object implements PropertyModel {
        /** support for the properties changes. */
        private PropertyChangeSupport support;

        public LanguagePropertyModel() {
            support = new PropertyChangeSupport(this);
        }
        
        /**
         * @return the class of the property.
         */
        public Class getPropertyType() {
            return String.class;
        }

        /** Getter for current value of a property.
         */
        public Object getValue()  {
            return getScriptLanguage();
        }

        /** Setter for a value of a property.
         * @param v the value
         * @exeception InvocationTargetException
         */
        public void setValue(Object v) {
            Object old = getScriptLanguage();
            setScriptLanguage((String)v);
            support.firePropertyChange("scriptLanguage", old, v);
        }

        /** The class of the property editor or <CODE>null</CODE>
         * if default property editor should be used.
         */
        public Class getPropertyEditorClass() {
            return EvaluateNode.LanguageSelectionPropertyEditor.class;
        }
        /** Adds listener to change of the value.
         */
        public void addPropertyChangeListener(PropertyChangeListener l) {
            support.addPropertyChangeListener(l);
        }

        /** Removes listener to change of the value.
         */
        public void removePropertyChangeListener(PropertyChangeListener l) {
            support.removePropertyChangeListener(l);
        }
    }
}
