/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.scripting;

import java.io.IOException;

import org.openide.actions.*;
import org.openide.filesystems.*;
import org.openide.loaders.*;
import org.openide.util.*;
import org.openide.util.actions.*;

/** Loader that is installed into the IDE to recognize all files with
* scripting extensions and create ScriptObjects for them.
*
* @author Jaroslav Tulach, David Strupl
*/
public class ScriptLoader extends MultiFileLoader {

    private static final long serialVersionUID = -6289558494331407755L;
    
    /** Creates new scripting loader. */
    public ScriptLoader() {
        super (ScriptObject.class);
    }
  
    /** Display name read from the bundle. */
    protected String defaultDisplayName() {
        return NbBundle.getBundle(ScriptLoader.class).getString("Loader_Name");
    }
    
    /** Actions initialization. */
    protected SystemAction[] defaultActions() {
        return new SystemAction [] {
            SystemAction.get (OpenAction.class),
            null,
            SystemAction.get(ExecuteAction.class),
            null,
            SystemAction.get(FileSystemAction.class),
            null,
            SystemAction.get(CutAction.class),
            SystemAction.get(CopyAction.class),
            SystemAction.get(PasteAction.class),
            null,
            SystemAction.get(DeleteAction.class),
            SystemAction.get(RenameAction.class),
            null,
            SystemAction.get(SaveAsTemplateAction.class),
            null,
            SystemAction.get(ToolsAction.class),
            SystemAction.get(PropertiesAction.class),
        };
    }
  
    /** Check if the file has one of valid extensions and if so,
     * it accepts it.
     * @param fo primary file
     * @return the file or null
     */
    protected FileObject findPrimaryFile (FileObject fo) {
        return findFor (fo) == null ? null : fo;
    }

    /** Creates the right data object for given primary file.
     * It is guaranteed that the provided file is realy primary file
     * returned from the method findPrimaryFile.
     *
     * @param primaryFile the primary file
     * @return the data object for this file
     * @exception DataObjectExistsException if the primary file already has data object
     */
    protected MultiDataObject createMultiObject (FileObject primaryFile)
        throws DataObjectExistsException, IOException {
        return new ScriptObject (primaryFile, this, findFor (primaryFile));
    }

    /** Create secondary entry shoud be never called.
     */
    protected MultiDataObject.Entry createPrimaryEntry (
        MultiDataObject obj,
        FileObject fo
    ) {
        return new FileEntry (obj, fo);
    }
  
    /** Create secondary entry shoud be never called.
     */
    protected MultiDataObject.Entry createSecondaryEntry (
        MultiDataObject obj,
        FileObject fo
    ) {
        throw new IllegalStateException ();
    }
  
    /** Finds script type for given primary file.
     */
    private static String findFor (FileObject fo) {
        return ScriptUtils.getLangFromFilename(fo);
    }
}

/* 
* $Log: 
*  3    Jaga      1.2         04/20/00 Jaroslav Tulach Does not recognize java and
*       javaclass languages.
*  2    Jaga      1.1         04/11/00 Jaroslav Tulach Rewritten to use Bean
*       Scripting Framework
*  1    Jaga      1.0         03/13/00 Jaroslav Tulach 
* $ 
*/ 
  