/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.scripting;

import java.lang.reflect.InvocationTargetException;
import java.awt.event.*;
import java.io.IOException;
import java.io.Serializable;
import java.util.ResourceBundle;

import org.openide.*;
import org.openide.execution.*;
import org.openide.windows.InputOutput;
import org.openide.explorer.propertysheet.*;
import org.openide.filesystems.*;
import org.openide.loaders.*;
import org.openide.cookies.ExecCookie;
import org.openide.nodes.Sheet;
import org.openide.nodes.PropertySupport;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;

/** Support for execution of a script fragment.
* @author David Strupl
*/
public class ScriptSupport extends Object implements ExecCookie {

    /** bundle */
    private static ResourceBundle bundle;

    /** entry to be associated with */
    protected MultiDataObject.Entry entry;

    /** Create new support for given entry. The file is taken from the
    * entry and is updated if the entry moves or renames itself.
    * @param entry entry to create instance from
    */
    public ScriptSupport(MultiDataObject.Entry entry) {
        this.entry = entry;
    }

    /* Starts the class.
    */
    public void start () {
        ScriptType script = getScriptType (entry);
        if (script == null) {
            script = ScriptType.getDefault();
        }
        try {
            final ScriptType s = script;
            final InputOutput io = TopManager.getDefault().getIO(s.getName(), false);
            final java.io.Reader r = new java.io.InputStreamReader(entry.getDataObject().getPrimaryFile().getInputStream());
            ExecutorTask task = TopManager.getDefault().getExecutionEngine().execute("scripting", new Runnable() {
                    public void run() {
                        try {
                            s.exec(r);
                        } catch (InvocationTargetException ite) {
                            Throwable t = ite.getTargetException();
                            if (t != null) {
                                t.printStackTrace(io.getOut());
                            } else {
                                ite.printStackTrace(io.getOut());
                            }
                        }
                    }
                }, io);
            
        } catch (final IOException ex) {
            TopManager.getDefault().notifyException(ex);
        } 
    }

    /** Get the script type for a given file object.
    * @param entry entry to obtain the scirpt type for
    * @return script type associated with the file, or null if the default should be used
    */
    private static ScriptType getScriptType (MultiDataObject.Entry entry) {
        java.util.Enumeration e = ScriptType.scriptTypes();
        while (e.hasMoreElements()) {
            ScriptType s = (ScriptType)e.nextElement();
            if (s.acceptFileObject(entry.getFile())) {
                return s;
            }
        }
        return null;
    }

    /** @return a localized String */
    static String getString(String s) {
        if (bundle == null) {
            bundle = NbBundle.getBundle(ScriptSupport.class);
        }
        return bundle.getString(s);
    }
}

