/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.scripting.remote;

import org.netbeans.modules.scripting.*;
import org.openide.execution.*;
import org.openide.TopManager;

import java.lang.reflect.*;
import java.net.*;
import java.io.*;
import java.util.*;

/** Object of this class handles one client connecting thru the
 * telnet interface. It is initialized with a scoket. <B> NOTE:</B> 
 * It depends on <code>org.netbeans.core.scriptSupport.*</code> package.
 */
class ClientHandler implements Runnable {
    /** */
    private ScriptType interpreter;
    /** */
    private Socket sock;
    /** */
    private boolean noScriptType = false;
    
    /** Remembers the sock var and sets the noScriptType var according
     * to whether it is able to find at least one script type by calling
     * ScriptType.getDefault()
     */
    public ClientHandler(Socket sock) {
        this.sock = sock;
        try {
            interpreter = ScriptType.getDefault();
        } catch (Exception x) {
            noScriptType = true;
        }
    }
    
    /** Prints language selection option and then runs the script
     * in an interactive mode (reads individual lines from the input
     * stream and executes them).
     */
    public void run() {
        try {
//            InputStream inS = sock.getInputStream();
//            PrintStream out = new PrintStream(sock.getOutputStream());
            
            InputStream inS = System.in;
            PrintStream out = System.out;
            
            if (noScriptType) {
                out.println("No scripting type found");
                out.flush();
                sock.close();
                return;
            }
            String line = null;
            // print available languages
            out.print("Available languages:\n\r");
            for (Enumeration enum = ScriptType.scriptTypes(); enum.hasMoreElements(); ) {
                ScriptType type = (ScriptType)enum.nextElement();
                out.print(type.getName()+ " ");
                out.print("\n\r");
                out.flush();
            }
            BufferedReader br = new BufferedReader(new InputStreamReader(inS));
            // language selection:
            out.print("Configuration mode, you can use key=value syntax.\n\r");
            out.print("Avalable commands: lang=<language>\n\r");
            out.print("mode=interactive | mode=batch\n\r"); // TODO: change this
            out.flush();
            String options = "";
            Map m = ScriptUtils.readParameters(inS, out, new String[] {"lang"}, new String[0]);
            ScriptType newType = ScriptType.find((String)m.get("lang"));
            if (newType != null) {
                    interpreter = newType;
            }
            String interpName = interpreter.getName();
            // read each line and execute it
            out.print(TelnetServer.PROMPT + "(" + interpName + ") > "); // NO-I18N
            out.flush();
            while ( ((line = ScriptUtils.readLine(br)) != null) && (!wantsExit(line))) {
                line = ScriptUtils.convertLine(line);
                try { // in this case we try to recover
                    interpreter.exec(line);
                    out.print("\n" + TelnetServer.PROMPT + "(" + interpName + ") > "); // NO-I18N
                    out.flush();
                } catch (InvocationTargetException ite) {
                    Throwable t = ite.getTargetException();
                    if (t != null) {
                        t.printStackTrace(out);
                    } else {
                        ite.printStackTrace(out);
                    }
                }
            }
            br.close();
            out.close();
            sock.close();
        } catch (java.io.IOException e) {
            TopManager.getDefault().notifyException(e);
        }
    }
    
    /** Scans the given command for words exit or quit. If it finds them
     * returns true.
     */
    private static boolean wantsExit(String command) {
        if (command.equalsIgnoreCase("exit")) {
            return true;
        }
        if (command.equalsIgnoreCase("quit")) {
            return true;
        }
        return false;
    }
}

