/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.scripting.remote;

import java.io.IOException;
import java.io.ObjectInput;
import java.text.MessageFormat;

import org.openide.options.SystemOption;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;
import org.openide.TopManager;
import org.openide.NotifyDescriptor;

/** Settings for the Telnet server.
* @author Jesse Glick, David Strupl
*/
public class ServerSettings extends SystemOption {

    public static final int DEFAULT_PORT = 2345;
    
    public static final String PROP_RUNNING = "running"; // NOI18N
    public static final String PROP_PORT = "port"; // NOI18N
    private static boolean running;
    private static int port = DEFAULT_PORT;

    private static boolean inited, pendingRunning;
    
    // default settings object, avoids calling the constructor repeatedly
    public static final ServerSettings DEFAULT = (ServerSettings) findObject (ServerSettings.class, true);

    private static final long serialVersionUID = -8091449028693180861L;
    
    private boolean isGlobal () {
        return true;
    }

    public String displayName () {
        return NbBundle.getBundle(org.netbeans.modules.scripting.ScriptModule.class).getString("PROP_telnetServer");
    }

    public HelpCtx getHelpCtx () {
        return new HelpCtx (ServerSettings.class);
    }

    protected void initialize () {
        //System.out.println("Settings.initialize");
        // Not what it looks like:
        inited = false;
        running = false;
        port = DEFAULT_PORT;
    }

    // The nasty part:
    public boolean isRunning () {
        synchronized (DEFAULT) {
            //System.out.println("Settings.isRunning: inited=" + inited + " running=" + running + " writeExternal=" + isWriteExternal ());
            if (! inited) {
                //System.out.println("Settings.isRunning: will automatically call setRunning(false)");
                inited = true;
                // Defaults to running == false
                setRunning (false);
            }
            return running;
        }
    }
    public void setRunning (boolean r) {
        synchronized (DEFAULT) {
            if (isReadExternal ()) {
                //System.out.println("Settings.setRunning (readExternal): r=" + r);
                pendingRunning = r;
            } else {
                //System.out.println("Settings.setRunning: " + running + " -> " + r);
                boolean old = running;
                running = r;
                firePropertyChange (PROP_RUNNING, new Boolean (old), new Boolean (r));
                if (old && ! r)
                    TelnetServer.shutdown ();
                else if (! old && r)
                    TelnetServer.startup ();
            }
        }
    }
    public void readExternal (ObjectInput in) throws IOException, ClassNotFoundException {
        //System.out.println("Settings.readExternal");
        super.readExternal (in);
        inited = true;
        //System.out.println("Settings.readExternal: pendingRunning=" + pendingRunning);
        setRunning (pendingRunning);
    }

    public int getPort () {
        return port;
    }
    public void setPort (int p) {
	if (p < 0 || p > 0xFFFF) {
            String msg = MessageFormat.format("Port out of range: {0}", // NOI18N
                                              new Object[] { new Integer(p) } );
            TopManager.getDefault().notify(new NotifyDescriptor.Message(msg,NotifyDescriptor.ERROR_MESSAGE));
            return;
        }

        synchronized (DEFAULT) {
            //System.out.println("Settings.port: " + port + " -> " + p);
            if (p == port) {
                return;
            }
            boolean restart = running;
            if (isReadExternal()) {
                running = false;
            } else {
                setRunning(false);
            }
            int old = port;
            port = p;
            firePropertyChange (PROP_PORT, new Integer (old), new Integer (p));
            if (restart) {
                if (isReadExternal()) {
                    running = true;
                } else {
                    setRunning(true);
                }
            }
        }
    }
}
