/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.scripting.remote;

import java.beans.*;
import java.net.*;
import java.io.*;
import java.util.*;
import java.lang.reflect.*;

import org.openide.*;
import org.openide.execution.*;
import org.openide.windows.*;
import org.openidex.windows.StandardStreamInputOutput;
import org.openide.util.NbBundle;

import org.netbeans.modules.scripting.*;

/** 
* @author David Strupl
*/
public class TelnetServer extends Object implements Runnable {

    private static ServerSocket s;
    public static String PROMPT = "[remote]";
    
    /** true if we should stop due to uninstallation */
    private static boolean stop;

    //private static int server_no = 0;
    
    static void startup () {
        //System.err.println("Server.startup");
        stop = false;
            //System.err.println("Server.startup: starting thread");
            new Thread (new TelnetServer(), "Scripting Server").start (); // NOI18N
    }

    public static void shutdown () {
        //System.err.println("Server.shutdown");
        stop = true;
            try {
                if (s != null) {
                    s.close();
                }
            } catch (IOException ioe) {
                ioe.printStackTrace ();
            }
    }

    /** set up the socket */
    private static void initSocket () {
        int port = ServerSettings.DEFAULT.getPort ();
        try {
            if (s != null) s.close ();
            s = new ServerSocket (port);
        } catch (IOException e) {
            TopManager.getDefault ().notify (new NotifyDescriptor.Message ("CannotBind : "));
            ServerSettings.DEFAULT.setRunning (false);
        }
    }
    /** Waits on the connection.
    */
    public void run () {
        //System.out.println(" server " + server_no++ + " started");
        try {
            if (ScriptType.getDefault() == null) {
                TopManager.getDefault().notify(new NotifyDescriptor.Message ("No Script type found. Scripting module probably not installed."));
                return;
            }
        } catch (Exception ex) {
            TopManager.getDefault().notifyException(ex);
            return;
        }
        initSocket ();
        if (s == null) {
            return;
        }
        try {
            // launch the server threads
            while (true) {
                Socket client = s.accept();
                ClientHandler c = new ClientHandler(client);
                ExecutorTask task = TopManager.getDefault().getExecutionEngine().execute(
                    "telnet", c, new StandardStreamInputOutput("telnet", client.getInputStream(), client.getOutputStream()));
            }
        } catch (IOException ioe) {
            // eg. when the socket s is closed
            //System.out.println("Catch part");
            //ioe.printStackTrace();
        } finally {
            try {
                if (s != null) s.close ();
            } catch (IOException ex) {
                //System.out.println("finally part");
                ex.printStackTrace();
            }
            s = null;
        }
    }
    
    /** Test run.*/
    public static void main (String[] args) {
      startup ();
    }

}
