/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.text;


import java.util.Enumeration;

import org.openide.loaders.DataLoader;
import org.openide.modules.ModuleInstall;
import org.openide.TopManager;
import org.openide.util.actions.SystemAction;
import org.openide.util.NbBundle;


/**
 * Module installation class for text module.
 *
 * @author Ian Formanek
 */
public class TXTModule extends ModuleInstall {

    /** Generated serial version UID. */
    static final long serialVersionUID =8769658476094354137L;

    
    /** Overrides superclass method. */
    public void installed () {
        restored ();
    }

    /** Overrides superclass method. */
    public void restored() {
        installConvertToTextAction();
    }

    /** Overrides superclass method. */
    public void uninstalled() {
        uninstallConvertToTextAction();
    }

    /** Sets <code>ConvertToTextAction</code> as default action for <code>DefaultDataObject</code>s. */
    private void installConvertToTextAction() {
        DataLoader DDOLoader = ConvertToTextAction.getDefaultDataLoader();
        
        SystemAction[] defaultActions = DDOLoader.getActions();

        // If the ConverttoText action is not already there add it to beginning.
        if(!(defaultActions[0] instanceof ConvertToTextAction)) {
            // merges the default actiosn with ConvertToTextAction, so the ConvertToText become first -> default
            SystemAction[] newActions = SystemAction.linkActions(
                new SystemAction[] {
                    SystemAction.get(ConvertToTextAction.class), null},
                defaultActions);

            DDOLoader.setActions(newActions);
        }
    }

    /** Removes <code>ConvertToTextAction</code> as default action for <code>DefaultDataObject</code>s. */
    private void uninstallConvertToTextAction() {
        DataLoader DDOLoader = ConvertToTextAction.getDefaultDataLoader();
        
        SystemAction[] defaultActions = DDOLoader.getActions();
        
        // If first action is ConvertToText give it away (together with separator, therefore 2 shift).
        if(defaultActions[0] instanceof ConvertToTextAction) {
            try {
                SystemAction[] newActions = new SystemAction[defaultActions.length-2];
                for(int i=2; i<defaultActions.length; i++) {
                    newActions[i-2] = defaultActions[i];
                }

                DDOLoader.setActions(newActions);
            } catch (ArrayIndexOutOfBoundsException aioe) {
                if(Boolean.getBoolean("netbeans.debug.exceptions")) { // NOI18N
                    aioe.printStackTrace();
                }
            }
        }
    }

}