/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.pdf;

import java.awt.Image;
import java.beans.*;

import org.openide.util.NbBundle;

/** BeanInfo for PDF viewer settings.
 * @author Jesse Glick
 */
public class PDFSettingsBeanInfo extends SimpleBeanInfo {

    public PropertyDescriptor[] getPropertyDescriptors () {
        try {
            PropertyDescriptor pdfviewer = new PropertyDescriptor ("PDFViewer", PDFSettings.class);
            pdfviewer.setDisplayName (NbBundle.getMessage (PDFSettingsBeanInfo.class, "PROP_pdfviewer"));
            pdfviewer.setShortDescription (NbBundle.getMessage (PDFSettingsBeanInfo.class, "HINT_pdfviewer"));
            return new PropertyDescriptor[] { pdfviewer };
        } catch (IntrospectionException ie) {
            if (Boolean.getBoolean ("netbeans.debug.exceptions"))
                ie.printStackTrace ();
            return null;
        }
    }
    
    /** Get the bean descriptor, hiding it.
     * Users probably do not want to change this setting
     * except once, which is why they are prompted to set it
     * if their current setting does not work.
     * @return a hidden descriptor
     */    
    public BeanDescriptor getBeanDescriptor () {
        BeanDescriptor desc = new BeanDescriptor (PDFSettings.class);
        desc.setHidden (true);
        return desc;
    }

    private static Image icon, icon32;
    public Image getIcon (int type) {
        if (type == BeanInfo.ICON_COLOR_16x16 || type == BeanInfo.ICON_MONO_16x16) {
            if (icon == null)
                icon = loadImage ("PDFDataIcon.gif");
            return icon;
        } else {
            if (icon32 == null)
                icon32 = loadImage ("PDFDataIcon32.gif");
            return icon32;
        }
    }

}
