/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.search;

import java.beans.*;
import java.awt.BorderLayout;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.util.*;

import org.openide.*;

import org.openidex.search.*;
import org.netbeans.modules.search.res.Res;

/**
 * Show customizer of underlayed criterion.
 *
 * @author  Petr Kuzel
 * @version
 */
public class CriterionView extends javax.swing.JPanel implements PropertyChangeListener
{
    private CriterionModel model;
    
    private String lastSavedName = null;
    
    private JCheckBox applyCheckBox;
    private JPanel presetPanel;
    private JPanel btnPanel;
    private JPanel dwnPanel;
    private JButton saveButton;
    private JButton restoreButton;
    
    /** Creates new form CriterionTab */
    public CriterionView(CriterionModel cModel) {
        model = cModel;
        
        applyCheckBox = new JCheckBox(Res.text("BUTTON_APPLY")); // NOI18N
        applyCheckBox.setMnemonic((Res.text("BUTTON_APPLY_MNEM")).charAt(0)); // NOI18N
        applyCheckBox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                model.setCustomized(applyCheckBox.isSelected());
            }
        });
        
        saveButton = new JButton(Res.text("BUTTON_SAVE_AS")); // NOI18N
        saveButton.setMnemonic((Res.text("BUTTON_SAVE_AS_MNEM")).charAt(0)); // NOI18N
        saveButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                saveCriterion();
            }
        });
        saveButton.setEnabled(false);
        
        restoreButton = new JButton(Res.text("BUTTON_RESTORE")); // NOI18N
        restoreButton.setMnemonic((Res.text("BUTTON_RESTORE_MNEM")).charAt(0)); // NOI18N
        restoreButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                restoreCriterion();
            }
        });
        
        dwnPanel = new JPanel();
        dwnPanel.setLayout(new GridLayout(2,1));
        
        btnPanel = new JPanel();
        btnPanel.setLayout(new BorderLayout(4,0));
        btnPanel.add(saveButton, BorderLayout.CENTER);
        btnPanel.add(restoreButton, BorderLayout.EAST);
        
        presetPanel = new JPanel();
        presetPanel.setLayout(new BorderLayout());
        presetPanel.add(btnPanel, BorderLayout.EAST);
        presetPanel.setBorder(BorderFactory.createEmptyBorder(6,2,3,2));
        
        setLayout(new BorderLayout(0,5));

        dwnPanel.setBorder(BorderFactory.createEmptyBorder(0,12,0,0));
        dwnPanel.add(applyCheckBox);
        dwnPanel.add(presetPanel);
        
       // add(applyCheckBox, BorderLayout.NORTH);
        add(model.getComponent(), BorderLayout.CENTER);
        add(dwnPanel, BorderLayout.SOUTH);
        
        setBorder(BorderFactory.createEmptyBorder(4,4,4,4));
        
        // obtain tab label string & icon TODO
        setName(model.getName());
        
        model.addPropertyChangeListener(this);
    }
    
    public void propertyChange(PropertyChangeEvent evt) {
        if (CriterionModel.PROP_CUSTOMIZED.equals(evt.getPropertyName())) {
            saveButton.setEnabled(model.isCustomized());
            applyCheckBox.setSelected(model.isCustomized());
        }
    }
    
    private void saveCriterion() {
        JPanel pane = new JPanel();
        pane.setLayout(new BorderLayout(12,0));
        JLabel nameLab = new JLabel(Res.text("LABEL_NAME")); // NOI18N
        nameLab.setDisplayedMnemonic((Res.text("LABEL_NAME_MNEM")).charAt(0)); // NOI18N
        pane.add(nameLab, BorderLayout.WEST); 
        
        JTextField textField;
        if (lastSavedName != null) {
            textField = new JTextField(lastSavedName, 20);
        }
        else textField = new JTextField(20);
        nameLab.setLabelFor(textField);
        pane.add(textField, BorderLayout.CENTER);
        pane.setBorder(BorderFactory.createEmptyBorder(12,12,0,11));
        
        DialogDescriptor desc = new DialogDescriptor(pane, Res.text("LABEL_SAVE_CRITERION")); // NOI18N
        Dialog dialog = TopManager.getDefault().createDialog(desc);
        
        while (true) {
            dialog.show();
            Object retVal = desc.getValue();
            
            if (retVal.toString().equals("0")) { // NOI18N
                String name = textField.getText();
                if (name.length() > 0) {
                    model.saveAs(name);
                    lastSavedName = name;
                    break;
                }
            }
            else return; // cancel
        }
    }
    
    private void restoreCriterion() {
        JPanel pane = new JPanel();
        pane.setLayout(new BorderLayout(12,0));
        JLabel resLabel = new JLabel(Res.text("LABEL_SELECT")); // NOI18N
        resLabel.setDisplayedMnemonic((Res.text("LABEL_SELECT_MNEM")).charAt(0)); // NOI18N        
        pane.add(resLabel, BorderLayout.WEST); 
        
        ArrayList criterList = new ArrayList(10);
        Enumeration en = TopManager.getDefault().getServices().services(model.getCriterionClass());
        while (en.hasMoreElements()) {
            SearchType type = (SearchType) en.nextElement();
            String name = type.getName();
            if (name != null) {
                if (name.equals(type.getTabText()))
                    criterList.add(Res.text("DEFAULT_CRITERION"));
                else criterList.add(name);
            }
        }
        
        JComboBox combo = new JComboBox(criterList.toArray());
        Dimension dim = combo.getPreferredSize();
        dim.width = 160;
        combo.setPreferredSize(dim);
        resLabel.setLabelFor(combo);
        pane.add(combo, BorderLayout.CENTER);
        pane.setBorder(BorderFactory.createEmptyBorder(12,12,0,11));
        
        DialogDescriptor desc = new DialogDescriptor(pane, Res.text("LABEL_RESTORE_CRITERION")); // NOI18N
        Dialog dialog = TopManager.getDefault().createDialog(desc);
        
        dialog.show();
        
        if (desc.getValue().toString().equals("0")) { // NOI18N
            int i = combo.getSelectedIndex();
            if (i >= 0)
                model.restoreCriterion((String)criterList.get(i));
        }
    }
}
