/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.search;


import java.awt.BorderLayout;
import java.awt.Image;
import java.awt.event.KeyEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyVetoException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.io.ObjectStreamException;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.netbeans.modules.search.res.Res;
import org.netbeans.modules.search.types.DetailHandler;

import org.openide.explorer.ExplorerManager;
import org.openide.explorer.ExplorerPanel;
import org.openide.explorer.view.BeanTreeView;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileStateInvalidException;
import org.openide.filesystems.FileSystem;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.loaders.DataObject;
import org.openide.TopManager;
import org.openide.windows.TopComponent;
import org.openide.windows.Workspace;
import org.openidex.search.DetailCookie;
import org.openidex.search.SearchEngine;
import org.openidex.search.SearchTask;


/**
 * Top component which displays search results in explorer like manner.
 * 
 * @author Petr Kuzel, Jiri Mzourek
 */
public class ResultViewTopComponent extends TopComponent implements ChangeListener, ExplorerManager.Provider {
    
    /** Result data holder. */
    private ResultModel model = null;
    
    /** ExplorerManager.Provider. */
    private ExplorerPanel exp;
    
    /** Search details of selected node. */
    private ArrayList details = new ArrayList(20);
    
    /** Dialog window for entering search kriteria. */
    private CriteriaView reuseView = null;
    
    /** Listener to found nodes. */
    private PropertyChangeListener managerListener;
    
    /** Status panel with info about location of selected nodes. */
    private LocationInfoPanel infoPanel;
    
    /** Creates new form ResultViewTopComponent */
    public ResultViewTopComponent(ResultModel model) {
        initComponents();
        splittedPanel1.setSwapPanesEnabled(false);
        splittedPanel1.setSplitTypeChangeEnabled(false);
        splittedPanel1.add(leftPanel, org.openide.awt.SplittedPanel.ADD_FIRST);
        splittedPanel1.add(panelDetails, org.openide.awt.SplittedPanel.ADD_SECOND);
        panelDetails.setVisible(true);
        ButtonGroup bg = new ButtonGroup();
        bg.add(jRadioButton1);
        bg.add(jRadioButton2);
        infoPanel = new LocationInfoPanel();
        forLocPanel.add(infoPanel, java.awt.BorderLayout.CENTER);
        exp = new ResultExplorerPanel();
        exp.setLayout(new BorderLayout());
        managerListener = new ManagerListener();
        exp.getExplorerManager().addPropertyChangeListener(managerListener);
        
        BeanTreeView tree =  new BeanTreeView();                
        tree.setBorder(new EtchedBorder());
        exp.add(tree, BorderLayout.CENTER);
        leftPanel.add(exp, java.awt.BorderLayout.CENTER);        
        
        setName(Res.text("TITLE_SEARCH_RESULTS")); // NOI18N
        
        // model should be set
        setModel(model);
        
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jLabel1 = new javax.swing.JLabel();
        jRadioButton1 = new javax.swing.JRadioButton();
        jRadioButton2 = new javax.swing.JRadioButton();
        jPanel3 = new javax.swing.JPanel();
        jPanel1 = new javax.swing.JPanel();
        jPanel2 = new javax.swing.JPanel();
        stopButton = new javax.swing.JButton();
        outputButton = new javax.swing.JButton();
        customizeButton = new javax.swing.JButton();
        closeButton = new javax.swing.JButton();
        helpButton = new javax.swing.JButton();
        splittedPanel1 = new org.openide.awt.SplittedPanel();
        leftPanel = new javax.swing.JPanel();
        panelDetails = new javax.swing.JPanel();
        panelDetailsInner = new javax.swing.JPanel();
        jLabel7 = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        detailList = new javax.swing.JList();
        forLocPanel = new javax.swing.JPanel();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        jLabel1.setText(Res.text("LABEL_SEARCH_RESULTS"));
        jLabel1.setLabelFor(exp);
        jLabel1.setHorizontalTextPosition(javax.swing.SwingConstants.LEFT);
        jLabel1.setDisplayedMnemonic(Res.text("LABEL_SEARCH_RESULTS").charAt(0));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(12, 12, 5, 11);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(jLabel1, gridBagConstraints1);
        
        jRadioButton1.setMnemonic(Res.text("TEXT_BUTTON_SORT_MNEM").charAt(0));
        jRadioButton1.setText(Res.text("BUTTON_SORT"));
        jRadioButton1.setEnabled(false);
        jRadioButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButton1ActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.insets = new java.awt.Insets(5, 12, 0, 12);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(jRadioButton1, gridBagConstraints1);
        
        jRadioButton2.setMnemonic(Res.text("TEXT_BUTTON_UNSORT_MNEM").charAt(0));
        jRadioButton2.setSelected(true);
        jRadioButton2.setText(Res.text("BUTTON_UNSORT"));
        jRadioButton2.setEnabled(false);
        jRadioButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButton1ActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.insets = new java.awt.Insets(5, 0, 0, 11);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(jRadioButton2, gridBagConstraints1);
        
        jPanel3.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints2;
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints2.weightx = 1.0;
        jPanel3.add(jPanel1, gridBagConstraints2);
        
        jPanel2.setLayout(new java.awt.GridLayout(1, 4, 5, 0));
        
        stopButton.setMnemonic(Res.text("BUTTON_STOP_MNEM").charAt(0));
        stopButton.setText(Res.text("BUTTON_STOP"));
        stopButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stopButtonActionPerformed(evt);
            }
        });
        
        jPanel2.add(stopButton);
        
        outputButton.setMnemonic(Res.text("BUTTON_FILL_MNEM").charAt(0));
        outputButton.setText(Res.text("BUTTON_FILL"));
        outputButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                outputButtonActionPerformed(evt);
            }
        });
        
        jPanel2.add(outputButton);
        
        customizeButton.setMnemonic(Res.text("BUTTON_CUSTOMIZE_MNEM").charAt(0));
        customizeButton.setText(Res.text("BUTTON_CUSTOMIZE"));
        customizeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                customizeButtonActionPerformed(evt);
            }
        });
        
        jPanel2.add(customizeButton);
        
        closeButton.setText(Res.text("BUTTON_CANCEL"));
        closeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                closeButtonActionPerformed(evt);
            }
        });
        
        jPanel2.add(closeButton);
        
        helpButton.setMnemonic(Res.text("BUTTON_HELP_MNEM").charAt(0));
        helpButton.setText(Res.text("BUTTON_HELP"));
        jPanel2.add(helpButton);
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.EAST;
        jPanel3.add(jPanel2, gridBagConstraints2);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(17, 12, 12, 12);
        gridBagConstraints1.weightx = 1.0;
        add(jPanel3, gridBagConstraints1);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(0, 12, 0, 11);
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(splittedPanel1, gridBagConstraints1);
        
        leftPanel.setLayout(new java.awt.BorderLayout());
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        add(leftPanel, gridBagConstraints1);
        
        panelDetails.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints3;
        
        panelDetailsInner.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints4;
        
        panelDetailsInner.setBorder(new javax.swing.border.TitledBorder(Res.text("LABEL_DETAILS")));
        jLabel7.setText(Res.text("LABEL_POSITION"));
        jLabel7.setLabelFor(detailList);
        jLabel7.setDisplayedMnemonic(Res.text("LABEL_POSITION_MNEM").charAt(0));
        gridBagConstraints4 = new java.awt.GridBagConstraints();
        gridBagConstraints4.gridx = 0;
        gridBagConstraints4.gridy = 1;
        gridBagConstraints4.insets = new java.awt.Insets(17, 12, 0, 0);
        gridBagConstraints4.anchor = java.awt.GridBagConstraints.WEST;
        panelDetailsInner.add(jLabel7, gridBagConstraints4);
        
        detailList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        detailList.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                detailListKeyPressed(evt);
            }
        });
        
        detailList.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                detailListValueChanged(evt);
            }
        });
        
        detailList.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                detailListMouseClicked(evt);
            }
        });
        
        jScrollPane1.setViewportView(detailList);
        
        gridBagConstraints4 = new java.awt.GridBagConstraints();
        gridBagConstraints4.gridx = 0;
        gridBagConstraints4.gridy = 2;
        gridBagConstraints4.gridwidth = 2;
        gridBagConstraints4.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints4.insets = new java.awt.Insets(5, 11, 12, 12);
        gridBagConstraints4.weightx = 1.0;
        gridBagConstraints4.weighty = 1.0;
        panelDetailsInner.add(jScrollPane1, gridBagConstraints4);
        
        forLocPanel.setLayout(new java.awt.BorderLayout());
        
        gridBagConstraints4 = new java.awt.GridBagConstraints();
        gridBagConstraints4.gridx = 0;
        gridBagConstraints4.gridy = 0;
        gridBagConstraints4.gridwidth = 2;
        gridBagConstraints4.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints4.weightx = 1.0;
        panelDetailsInner.add(forLocPanel, gridBagConstraints4);
        
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints3.insets = new java.awt.Insets(0, 11, 0, 0);
        gridBagConstraints3.weightx = 1.0;
        gridBagConstraints3.weighty = 1.0;
        panelDetails.add(panelDetailsInner, gridBagConstraints3);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        add(panelDetails, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void detailListMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_detailListMouseClicked
        if(evt.getClickCount() == 2)
            showDetail(DetailHandler.DH_GOTO);
    }//GEN-LAST:event_detailListMouseClicked

    private void detailListValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_detailListValueChanged
        if(!detailList.getValueIsAdjusting())
            showDetail(DetailHandler.DH_SHOW);
    }//GEN-LAST:event_detailListValueChanged

    private void detailListKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_detailListKeyPressed
        if(evt.getKeyCode() == KeyEvent.VK_ENTER)
            showDetail(DetailHandler.DH_GOTO);        
    }//GEN-LAST:event_detailListKeyPressed
    
    private void closeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_closeButtonActionPerformed
        close();
    }//GEN-LAST:event_closeButtonActionPerformed
    
    private void outputButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_outputButtonActionPerformed
        ResultViewTopComponent.this.model.fillOutput();
    }//GEN-LAST:event_outputButtonActionPerformed
    
    private void jRadioButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButton1ActionPerformed
        ExplorerManager em = null;
        Node [] selNodes = null;
        if ((em = exp.getExplorerManager()) != null){
            selNodes = em.getSelectedNodes();
        }
        boolean sort = !ResultViewTopComponent.this.model.isSorted();
        sortNodes(sort);
        try {
            if(selNodes!=null)em.setSelectedNodes(selNodes);
        }
        catch (PropertyVetoException ex) {
        }
    }//GEN-LAST:event_jRadioButton1ActionPerformed
    
    private void customizeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_customizeButtonActionPerformed
        customizeCriteria();
    }//GEN-LAST:event_customizeButtonActionPerformed
    
    private void stopButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stopButtonActionPerformed
        ResultViewTopComponent.this.model.stop();
    }//GEN-LAST:event_stopButtonActionPerformed
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel1;
    private javax.swing.JRadioButton jRadioButton1;
    private javax.swing.JRadioButton jRadioButton2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JButton stopButton;
    private javax.swing.JButton outputButton;
    private javax.swing.JButton customizeButton;
    private javax.swing.JButton closeButton;
    private javax.swing.JButton helpButton;
    private org.openide.awt.SplittedPanel splittedPanel1;
    private javax.swing.JPanel leftPanel;
    private javax.swing.JPanel panelDetails;
    private javax.swing.JPanel panelDetailsInner;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JList detailList;
    private javax.swing.JPanel forLocPanel;
    // End of variables declaration//GEN-END:variables
    

    /** Set new model. */
    public void setModel(ResultModel model) {
        if (this.model != null) {
            this.model.removeChangeListener(this);
        }
        this.model = model;
        Node root = model.getRoot();

        if(model.isSorted())
            jRadioButton1.setSelected(true);
        else
            jRadioButton2.setSelected(true);
        
        exp.getExplorerManager().setRootContext(root);
        model.addChangeListener(this);
        initButtons();
    }
    
    /** Delegates activation to explorer panel. */
    protected void componentActivated() {
        ((ResultExplorerPanel)exp).componentActivated();
    }
    
    /** Delegates deactivation to explorer panel. */
    protected void componentDeactivated() {
        ((ResultExplorerPanel)exp).componentDeactivated();
    }
    
    /** Set visibility of buttons & others... */
    private void initButtons() {
        stopButton.setEnabled(!model.isDone());
        jRadioButton1.setEnabled(model.isDone());
        jRadioButton2.setEnabled(model.isDone());
        
        infoPanel.showInfo(null);
    }
    
    /** Sorts nodes. */
    public void sortNodes(boolean sort) {
        Node root = model.sortNodes(sort);
        exp.getExplorerManager().setRootContext(root);
        initButtons();
    }
    
    /** Set the <code>CriteriaView</code> dialog for reuse. */
    public void setCriteriaView(CriteriaView view) {
        reuseView = view;
    }
    
    /** Gets icon. Overrides superclass method.
     * @return search icon. */
    public Image getIcon() {
        return Res.image("SEARCH"); // NOI18N
    }
    
    /** Gets explrer manager.
     * @return <code>ExplorerPanel.Provider</code> */
    public ExplorerManager getExplorerManager() {
        return exp.getExplorerManager();
    }
    
    /** Stops search before closing and hides details. Overrides superclass method. */
    public boolean canClose(Workspace workspace, boolean last) {
        if (model != null) {
            model.stop();
            model.removeChangeListener(this);
        }
        //        exp.getExplorerManager().removePropertyChangeListener(managerListener);
        showDetail(DetailHandler.DH_HIDE);
        
        return true;
    }
    
    /** (Re)open the dialog window for entering (new) search criteria. */
    public void customizeCriteria() {
        CriteriaModel criteria =  model.getCriteriaModel();
        
        CriteriaView dialog;
        if (reuseView != null) {
            dialog = reuseView;
        }
        else {
            dialog = new CriteriaView(criteria);
            reuseView = dialog;
        }
        
        dialog.show();
        
        if (dialog.getReturnStatus() == dialog.RET_OK) {
            //stop previous search
            model.stop();
            
            // start a new search
            SearchEngine engine = new SearchEngineImpl();
            ResultModel result = new ResultModel(criteria);
            SearchTask task = engine.search(criteria.getNodes(),
            criteria.getCustomizedCriteria(),
            result);
            result.setTask(task);
            setModel(result);
            initButtons();
        }
    }
    
    /** Show search details (of the selected node) in the detail list. */
    public void showDetails(DetailCookie detail) {
        details.clear();
        
        Enumeration detailItems = null;
        if (detail != null)
            detailItems = detail.detail();
        if (detailItems == null) {
            detailList.setListData(new Object[0]);
            detailList.setEnabled(false);

            return; // no details can be shown
        }
        
        ArrayList listData = new ArrayList(20);
        
        while (detailItems.hasMoreElements()) {
            Object obj = detailItems.nextElement();
            
            if (obj instanceof DetailHandler) {
                listData.add(((DetailHandler)obj).getShortDescription());
                details.add(obj);
            } else if (obj instanceof String) {
                listData.add(obj);
                details.add(null);
            }
        }
        
        if (listData.size() > 0) {
            detailList.setEnabled(true);
            detailList.setListData(listData.toArray());
        } else {
            detailList.setEnabled(false);
        }
            
    }
    
    /** Show one detail (selected in the detail list) on the target. */
    private void showDetail(int how) {
        int i = detailList.getSelectedIndex();
        if (i >= 0 && i < details.size()) {
            Object obj = details.get(i);
            if (obj != null) {
                DetailHandler det = (DetailHandler)obj;
                det.showDetail(how);
            }
        }
    }
    
    /** Respond to result model state change. */
    public void stateChanged(ChangeEvent event) {
        if (event.getSource() == model) {
            if (model.isDone())
                initButtons();
        }
    }

    /** Overrides superclass method to avoid serialization. */
    public void writeExternal(ObjectOutput out) throws IOException {}
    
    /** Overrides superclass method to avoid serialization. */
    public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {}
    
    /** Overrides superclass method to avoid serialization. */
    public Object writeReplace() throws ObjectStreamException {
        return null;
    }
    
    
    /** Listener on explorer manager properties.
     * It responds to changes in selection of found nodes.
     */
    private class ManagerListener implements PropertyChangeListener {

        /** Implements <code>PropertyCahngeListener</code> interface. */
        public void propertyChange(PropertyChangeEvent evt) {
            if (ExplorerManager.PROP_SELECTED_NODES.equals(evt.getPropertyName())) {
                
                Node[] nodes = exp.getExplorerManager().getSelectedNodes();
                setActivatedNodes(nodes);
                
                DetailCookie cake = null;
                
                if (nodes != null && nodes.length == 1) {
                    // show location info
                    infoPanel.showInfo(nodes[0]);
                    
                    // get the object holding search details
                    cake = (DetailCookie) nodes[0].getCookie(DetailCookie.class);
                }
                else infoPanel.showInfo(null);
                
                // show search details (if available)
                showDetails(cake);
            }
        }
    } // End of ManagerListener class.

    
    /** Status panel that presents info about location of selected nodes. */
    private class LocationInfoPanel extends JPanel {
        // labels
        JLabel jLabel3;
        JLabel jLabel5;
        JLabel fsLabel;
        JLabel packLabel;
        
        // explorer button
        JButton showButton;
        
        private FileSystem fileSys = null;
        private String path = null;
        private Node selectedNode = null;
        private DataObject selectedDO = null;

        /** Constructor. */
        public LocationInfoPanel() {
            this.setLayout(new BorderLayout(4,0));
            
            jLabel1 = new javax.swing.JLabel();
            fsLabel = new javax.swing.JLabel();
            fsLabel.setHorizontalTextPosition(javax.swing.SwingConstants.LEFT);
            jLabel3 = new javax.swing.JLabel();
            jLabel3.setHorizontalTextPosition(javax.swing.SwingConstants.LEFT);
            packLabel = new javax.swing.JLabel();
            packLabel.setHorizontalTextPosition(javax.swing.SwingConstants.LEFT);
            jLabel5 = new javax.swing.JLabel();
            jLabel5.setHorizontalTextPosition(javax.swing.SwingConstants.LEFT);
            showButton = new javax.swing.JButton();
            
            setLayout(new java.awt.GridBagLayout());
            java.awt.GridBagConstraints gridBagConstraints1;
            
            jLabel1.setText(Res.text("LABEL_LOCATION"));
            
            gridBagConstraints1 = new java.awt.GridBagConstraints();
            gridBagConstraints1.gridwidth = 2;
            gridBagConstraints1.insets = new java.awt.Insets(11, 11, 0, 0);
            gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
            add(jLabel1, gridBagConstraints1);
            
            
            
            gridBagConstraints1 = new java.awt.GridBagConstraints();
            gridBagConstraints1.gridx = 0;
            gridBagConstraints1.gridy = 1;
            gridBagConstraints1.insets = new java.awt.Insets(5, 23, 0, 0);
            gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
            add(jLabel3, gridBagConstraints1);
            
            
            jLabel3.setText(Res.text("LABEL_FILESYSTEM"));
            
            gridBagConstraints1 = new java.awt.GridBagConstraints();
            gridBagConstraints1.gridx = 1;
            gridBagConstraints1.gridy = 1;
            gridBagConstraints1.gridwidth = 2;
            gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
            gridBagConstraints1.insets = new java.awt.Insets(5, 5, 0, 5);
            gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
            gridBagConstraints1.weightx = 1.0;
            add(fsLabel, gridBagConstraints1);
            
            
            
            gridBagConstraints1 = new java.awt.GridBagConstraints();
            gridBagConstraints1.gridx = 0;
            gridBagConstraints1.gridy = 2;
            gridBagConstraints1.insets = new java.awt.Insets(5, 23, 0, 0);
            gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
            add(jLabel5, gridBagConstraints1);
            
            
            jLabel5.setText(Res.text("LABEL_PATH"));
            
            gridBagConstraints1 = new java.awt.GridBagConstraints();
            gridBagConstraints1.gridx = 1;
            gridBagConstraints1.gridy = 2;
            gridBagConstraints1.gridwidth = 2;
            gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
            gridBagConstraints1.insets = new java.awt.Insets(5, 5, 0, 5);
            gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
            gridBagConstraints1.weightx = 1.0;
            add(packLabel, gridBagConstraints1);
            
            
            showButton.setText(Res.text("BUTTON_SHOW"));
            showButton.setMnemonic(Res.text("BUTTON_SHOW_MNEM").charAt(0));
            
            gridBagConstraints1 = new java.awt.GridBagConstraints();
            gridBagConstraints1.gridx = 1;
            gridBagConstraints1.gridy = 3;
            gridBagConstraints1.gridwidth = 2;
            gridBagConstraints1.insets = new java.awt.Insets(5, 0, 0, 12);
            gridBagConstraints1.anchor = java.awt.GridBagConstraints.EAST;
            add(showButton, gridBagConstraints1);
            showButton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    showInExplorer();
                }
            });
            
        }
        
        
        /** Shows info for node. */
        public void showInfo(Node node) {
            String fsName = "", // NOI18N
            pkgName = ""; // NOI18N
            
            // find out filesystem and package of the node
            if (node != null) {
                selectedNode = node;
                fileSys = null;
                
                selectedDO = (DataObject) node.getCookie(DataObject.class);
                if (selectedDO != null) {
                    FileObject folder = selectedDO.getPrimaryFile().getParent();
                    if (folder != null) {
                        try {
                            fileSys = folder.getFileSystem();
                            fsName = fileSys.getDisplayName();
                            pkgName = folder.getPackageName('/') + "/"; // NOI18N
                            path = pkgName;
                        }
                        catch (FileStateInvalidException ex) {
                            // filesystem can't be discovered
                            fsName = Res.text("NO_FILESYSTEM"); // NOI18N
                        }
                    }
                    jLabel1.setVisible(true);
                    jLabel3.setVisible(true);
                    jLabel5.setVisible(true);
                    showButton.setVisible(true);
                    showButton.setEnabled(true);
                    //titleLabel.setText(Res.text("LABEL_DETAILS") + " " + selectedNode.getName()); // NOI18N
                    panelDetailsInner.setBorder(new javax.swing.border.TitledBorder(Res.text("LABEL_DETAILS") + " " + selectedNode.getName())); // NOI18N
                    
                }
            }
            else {
                selectedNode = null;
                fileSys = null;
                
               // jLabel1.setVisible(false);
               // jLabel3.setVisible(false);
               // jLabel5.setVisible(false);
               // showButton.setVisible(false);
                showButton.setEnabled(false);
            }
            
            fsLabel.setText(fsName);
            packLabel.setText(pkgName);
        }

        /** Shows in explorer. */
        private void showInExplorer() {
            if (fileSys == null || selectedNode == null || selectedDO == null)
                return;
            
            // get all TopComponents and find main explorer panel (it's not very nice, I know...)
            Object[] comps = TopComponent.getRegistry().getOpened().toArray();
            boolean foundAndSelected = false;
            
            for (int i=0; i  < comps.length; i++) {
                if (comps[i] instanceof ExplorerPanel) {
                    ExplorerPanel ep = (ExplorerPanel)comps[i];
                    ExplorerManager em;
                    Node rootContext;
                    if ((em = ep.getExplorerManager()) != null
                    && (rootContext = em.getRootContext()) != null) {
                        if ("Filesystems".equals(rootContext.getName())) { // NOI18N
                            // Now find the node in repository hierarchy...
                            // Node obtained from DataObject can't be used because
                            // it is not in the hieararchy (there is a FilterNode
                            // instead which points to this node).
                            Node node = findNodeInRepository();
                            if (node != null) {
                                try { // try to expand tree and select the node
                                    em.setSelectedNodes(new Node[] { node } );
                                    ep.open();
                                    ep.requestFocus();
                                    foundAndSelected = true;
                                }
                                catch (PropertyVetoException ex) {
                                }
                            }
                            break;
                        }
                    }
                }
            }
            
            if (!foundAndSelected) showButton.setEnabled(false);
        }

        /** Finds in repository. */
        private Node findNodeInRepository() {
            Node rootNode = TopManager.getDefault().getPlaces().nodes().repository(),
            node = null;
            Node[] nodes = rootNode.getChildren().getNodes();
            String filesysName = fileSys.getDisplayName(),
            folder;
            
            // look for the filesystem node with the same display name
            for (int i=0; i < nodes.length; i++) {
                if (nodes[i].getDisplayName().equals(filesysName)) {
                    node = nodes[i];
                    break;
                }
            }
            
            if (path != null && !path.equals("")) { // NOI18N
                StringTokenizer pathItems = new StringTokenizer(path,"/"); // NOI18N
                
                while (node != null && pathItems.hasMoreTokens()) {
                    node = node.getChildren().findChild(pathItems.nextToken());
                }
            }
            
            if (node == null) return null;
            
            String nodeName = selectedNode.getName();
            Children ch = node.getChildren();
            node = ch.findChild(nodeName);
            if (node == null) return null;
            
            Enumeration enum = null;
            do {
                DataObject dobj = (DataObject) node.getCookie(DataObject.class);
                if (dobj != null && dobj == selectedDO)
                    return node; // this is the right node in repository
                
                node = null;
                if (enum == null) enum = ch.nodes();
                
                while (enum.hasMoreElements()) {
                    Node nod = (Node)enum.nextElement();
                    if (nodeName.equals(nod.getName())) {
                        node = nod;
                        break;
                    }
                }
            } while (node != null);
            
            return null;
        }
    } // End of LocationInfoPanel class.

    
    /** Explorer panel in search result window. */
    private static class ResultExplorerPanel extends ExplorerPanel {
        
        /** Extends scope of superclass method. */
        public void componentActivated() {
            super.componentActivated();
        }
        
        /** Extends scope of superclass method. */
        public void componentDeactivated() {
            super.componentDeactivated();
        }
        
    } // End of class ResultExploererPanel.
    
    
    /** For debugging purposes only. */
    static private final boolean TRACE = false;
    /** For debugging purposes only. */
    static private void t(String str) {
        if (TRACE)
            System.out.println("ResultViewTopComponent> "+str); // NOI18N
    }
}

