/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.search.types;

import org.openide.loaders.*;
import org.openide.nodes.*;
import org.openide.filesystems.*;
import org.openide.cookies.InstanceCookie;
import org.openide.util.*;

import org.openidex.search.*;

import org.netbeans.modules.search.*;

/**
 * Type used by RepositoryScanner.
 *
 * @author  Petr Kuzel
 * @version 1.0
 */
public abstract class DataObjectType extends SearchType implements TypeConstants {

    public static final long serialVersionUID = 1L; // forever 1

    // detail object produced during search
    private transient SearchDetail detail;

    /** Creates new RepositoryType */
    public DataObjectType() {
        //a new criteron can be valid or invalid depending on type
        //be pesimistic
        setValid(false);
    }

    public boolean test(DataObject dobj, SearchDetail detail) {
        this.detail = detail;
        return test(dobj);
    }

    /** Add new detail to detail set. */
    protected void addDetail(Object detail) {
        this.detail.add(detail);
    }

    protected abstract boolean test(DataObject dobj);

    /** @return org.netbeans.modules.search.scanners.RepositoryScanner.class
    */
    public Class getScannerClass() {
        return org.netbeans.modules.search.scanners.RepositoryScanner.class;
    }

    /** Overload it if your criterion supports DetailHandler or so. */
    public Class[] getDetailClasses() {
        return new Class[] {String.class};
    }

    /** @return true if any node represent repository node
    */
    public boolean enabled(Node[] nodes) {

        if (nodes == null) return false;
        if (nodes.length == 0) return false;

        // NodeCookie test

        for (int i =0; i<nodes.length; i++ ) {

            Node obj = nodes[i];

            //     if (obj.isLeaf()) continue; // optimalization that can cause probs in object browser

            if ( obj.getCookie(org.openide.loaders.DataFolder.class) != null) {
                return true;
            }

            try {
                InstanceCookie ic = (InstanceCookie)obj.getCookie (InstanceCookie.class);
                if (ic != null && Repository.class.isAssignableFrom (ic.instanceClass ())) {
                    return true;
                }
            } catch (java.io.IOException ex) {
                // ok, does not provide repository
            } catch (ClassNotFoundException ex) {
                // ok, does not provide repository
            }
        }

        return false;
    }

    public HelpCtx getHelpCtx() {
        return null;
    }

}
