/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.search.types;

import java.io.*;
import java.text.MessageFormat;

import org.openide.util.*;
import org.openide.loaders.*;
import org.openide.filesystems.*;

import org.openidex.search.*;

import org.netbeans.modules.search.res.*;
import org.netbeans.modules.search.*;

/**
 * Test DataObject primaryFile for line full-text match.
 *
 * @author  Petr Kuzel
 * @version 1.0
 */
public class FullTextType extends TextType {

    public static final long serialVersionUID = 1L;

    /** Creates new FullTextType */
    public FullTextType() {
    }


    /**
    * @return true if object passes the test.
    */
    public boolean test (DataObject dobj) {
        try {
            String line = ""; //NOI18N
            boolean hit = false;

            // primary file of the DataObject
            FileObject fo = dobj.getPrimaryFile();
            if (fo == null) return false;

            // primary file content
            InputStream is = fo.getInputStream();
            LineNumberReader reader = new LineNumberReader(new InputStreamReader(is));

            while (true) {
                line = reader.readLine();
                if (line == null) break;

                if (matchString != null) {
                    int lineNum = reader.getLineNumber(); // current line + 1
                    int markLen = matchString.length();

                    int i = matchString(line, 0);
                    while (i >= 0) {
                        hit = true;
                        TextDetail det = new TextDetail(dobj);
                        det.setLine(lineNum);
                        det.setColumn(i+1); // current column + 1
                        setShortDesc(det);
                        setFullDesc(det, line);
                        det.setMarkLength(markLen);
                        addDetail(det);

                        i = matchString(line, i+1);
                    }
                }
                else if (matchRE(line)) {
                    hit = true;
                    TextDetail det = new TextDetail(dobj);
                    det.setLine(reader.getLineNumber());
                    setShortDesc(det);
                    setFullDesc(det, line);
                    addDetail(det);
                }
            }

            return hit;

        }
        catch (FileNotFoundException ex) {
            return false;
        }
        catch (IOException ex) {
            if (Boolean.getBoolean("netbeans.debug.exceptions")) // NOI18N
                ex.printStackTrace();
            return false;
        }
    }

    private static void setShortDesc(TextDetail det) {
        if (det.getColumn() > 0) {
            Object[] args = new Object[] { Integer.toString(det.getLine()),
                                           Integer.toString(det.getColumn()) };

            det.setShortDescription(MessageFormat.format(Res.text("DETAIL_FMT_SHORT1"), args)); // NOI18N
        }
        else {
            Object[] args = new Object[] { Integer.toString(det.getLine()) };

            det.setShortDescription(MessageFormat.format(Res.text("DETAIL_FMT_SHORT2"), args)); // NOI18N
        }
    }

    private static void setFullDesc(TextDetail det,String line) {
        FileObject fo = det.getDataObject().getPrimaryFile();
        String filename = fo.getName() + "." + fo.getExt(); // NOI18N

        if (det.getColumn() > 0) {
            Object[] args = new Object[] { line.trim(),
                                           filename,
                                           Integer.toString(det.getLine()),
                                           Integer.toString(det.getColumn()) };

            det.setDescription(MessageFormat.format(Res.text("DETAIL_FMT_FULL1"), args)); // NOI18N
        }
        else {
            Object[] args = new Object[] { line.trim(),
                                           filename,
                                           Integer.toString(det.getLine()) };

            det.setDescription(MessageFormat.format(Res.text("DETAIL_FMT_FULL2"), args)); // NOI18N
        }
    }
    
    /** @return string describing current state.
    */
    public String toString() {
        return "FullTextType: " + isValid() + " substring:" + matchString + " REstring:" + reString + " re:" + re; // NOI18N
    }

    public HelpCtx getHelpCtx() {
        return new HelpCtx(this.getClass());
    }

    public String getTabText() {
        return Res.text("FULLTEXT_CRITERION"); // NOI18N
    }

    public Class[] getDetailClasses() {
        return new Class[] { String.class, TextDetail.class };
    }
}
