/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.search.types;

import java.awt.*;
import java.beans.*;
import java.util.Date;

import javax.swing.*;

import org.openide.util.*;
import org.netbeans.modules.search.res.*;

/**
 * Customizer of criterion for testing date of primary file modification.
 *
 * @author  Petr Kuzel
 * @version 1.0
 */
public class ModificationDateCustomizer extends javax.swing.JPanel implements Customizer {

    private ModificationDateType peer;
    private String lastDays = "";
    private String lastBefore = "";
    private String lastAfter = "";

    /** Creates new form ModificationDateCustomizer */
    public ModificationDateCustomizer() {

        initComponents ();        
        javax.swing.border.TitledBorder tb = new javax.swing.border.TitledBorder(Res.text("LABEL_MODIFIED")); // NOI18N
        tb.setBorder(new javax.swing.border.CompoundBorder());
        setBorder (tb);
        
        ButtonGroup group = new ButtonGroup();
        group.add(daysRadioButton);
        group.add(betweenRadioButton);

        HelpCtx.setHelpIDString(this, ModificationDateType.class.toString());
    }

    /** Initialize customizer state.
    */
    public void setObject(final Object obj) {

        peer = (ModificationDateType) obj;

        //    System.err.println("New peer: " + peer );

        // set default coloring
        daysTextField.setForeground(Color.black);
        afterTextField.setForeground(Color.black);
        beforeTextField.setForeground(Color.black);

        daysTextField.setText(peer.WILDCARD);
        afterTextField.setText(peer.WILDCARD);
        beforeTextField.setText(peer.WILDCARD);


        // display object value in customizer
        if ( ! peer.getMatchAfter().equals(peer.WILDCARD) ) {
            afterTextField.setText(peer.getMatchAfter());
            if ( ! peer.getMatchAfter().equals(null) ) betweenRadioButton.setSelected(true);
        }

        if ( ! peer.getMatchBefore().equals(peer.WILDCARD) ) {
            beforeTextField.setText(peer.getMatchBefore());
            if ( ! peer.getMatchAfter().equals(null) ) betweenRadioButton.setSelected(true);
        }

        if ( ! peer.getDays().equals(peer.WILDCARD) ) {
            daysTextField.setText(peer.getDays());
            daysRadioButton.setSelected(true);
        }

    }

    public void addPropertyChangeListener(final java.beans.PropertyChangeListener p1) {
    }

    public void removePropertyChangeListener(final java.beans.PropertyChangeListener p1) {
    }


    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        daysRadioButton = new javax.swing.JRadioButton();
        daysLabel = new javax.swing.JLabel();
        daysTextField = new javax.swing.JTextField();
        betweenRadioButton = new javax.swing.JRadioButton();
        beginLabel = new javax.swing.JLabel();
        beginLabel.setDisplayedMnemonic((Res.text("LABEL_BEGIN_DATE_MNEM")).charAt(0));
        afterTextField = new javax.swing.JTextField();
        beforeTextField = new javax.swing.JTextField();
        jPanel1 = new javax.swing.JPanel();
        egLabel = new javax.swing.JLabel();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        daysRadioButton.setMnemonic((Res.text("BUTTON_PREC_MNEM")).charAt(0));
        daysRadioButton.setText(Res.text("BUTTON_PRECEDING"));
        daysRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                daysRadioButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.insets = new java.awt.Insets(4, 8, 0, 4);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(daysRadioButton, gridBagConstraints1);
        
        daysLabel.setText(Res.text("LABEL_DURING_LAST"));
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 2;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.insets = new java.awt.Insets(0, 4, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(daysLabel, gridBagConstraints1);
        
        daysTextField.setPreferredSize(new java.awt.Dimension(50, daysTextField.getPreferredSize().height));
        daysTextField.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                daysTextFieldKeyReleased(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(4, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints1.weightx = 0.5;
        add(daysTextField, gridBagConstraints1);
        
        betweenRadioButton.setMnemonic((Res.text("BUTTON_RANGE_MNEM")).charAt(0));
        betweenRadioButton.setText(Res.text("BUTTON_RANGE"));
        betweenRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                betweenRadioButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.insets = new java.awt.Insets(4, 8, 0, 4);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(betweenRadioButton, gridBagConstraints1);
        
        beginLabel.setText(Res.text("LABEL_BEGIN_DATE"));
        beginLabel.setLabelFor(beforeTextField);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 3;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.insets = new java.awt.Insets(0, 4, 0, 4);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(beginLabel, gridBagConstraints1);
        
        afterTextField.setPreferredSize(new java.awt.Dimension(100, afterTextField.getPreferredSize().height));
        afterTextField.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                afterTextFieldKeyReleased(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(4, 0, 0, 4);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints1.weightx = 0.5;
        add(afterTextField, gridBagConstraints1);
        
        beforeTextField.setPreferredSize(new java.awt.Dimension(100, beforeTextField.getPreferredSize().height));
        beforeTextField.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                beforeTextFieldKeyReleased(evt);
            }
        });
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 4;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(4, 0, 0, 12);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints1.weightx = 0.5;
        add(beforeTextField, gridBagConstraints1);
        
        jPanel1.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints2;
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.gridheight = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(jPanel1, gridBagConstraints1);
        
        egLabel.setText(Res.text("DATE_EXAMPLE") + new ModificationDateType.FormattedDate(new Date()).toString());
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(11, 8, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(egLabel, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void beforeTextFieldKeyReleased (java.awt.event.KeyEvent evt) {//GEN-FIRST:event_beforeTextFieldKeyReleased
        String text = beforeTextField.getText();
        if (!text.equals(lastBefore)) {
            setBetween();
            betweenRadioButton.setSelected(true);
            lastBefore = text;
        }
    }//GEN-LAST:event_beforeTextFieldKeyReleased

    private void afterTextFieldKeyReleased (java.awt.event.KeyEvent evt) {//GEN-FIRST:event_afterTextFieldKeyReleased
        String text = afterTextField.getText();
        if (!text.equals(lastAfter)) {
            setBetween();
            betweenRadioButton.setSelected(true);
            lastAfter = text;
        }
    }//GEN-LAST:event_afterTextFieldKeyReleased

    private void betweenRadioButtonActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_betweenRadioButtonActionPerformed
        setBetween();
    }//GEN-LAST:event_betweenRadioButtonActionPerformed

    private void daysRadioButtonActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_daysRadioButtonActionPerformed
        setDays();
    }//GEN-LAST:event_daysRadioButtonActionPerformed

    private void daysTextFieldKeyReleased (java.awt.event.KeyEvent evt) {//GEN-FIRST:event_daysTextFieldKeyReleased
        String text = daysTextField.getText();
        if (!text.equals(lastDays)) {
            setDays();
            daysRadioButton.setSelected(true);
            lastDays = text;
        }
    }//GEN-LAST:event_daysTextFieldKeyReleased

    private void setDays() {

        String text = daysTextField.getText();
        try {
            peer.setDays(text);
            daysTextField.setForeground(Color.black);
        } catch (IllegalArgumentException ex) {
            daysTextField.setForeground(Color.red);
        }
    }

    private void setBetween() {

        String after = afterTextField.getText();
        String before = beforeTextField.getText();

        try {
            peer.setMatchAfter(after);
            afterTextField.setForeground(Color.black);
        } catch (IllegalArgumentException ex) {
            afterTextField.setForeground(Color.red);
        }

        try {
            peer.setMatchBefore(before);
            beforeTextField.setForeground(Color.black);
        } catch (IllegalArgumentException ex) {
            beforeTextField.setForeground(Color.red);
        }

    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JRadioButton daysRadioButton;
    private javax.swing.JLabel daysLabel;
    private javax.swing.JTextField daysTextField;
    private javax.swing.JRadioButton betweenRadioButton;
    private javax.swing.JLabel beginLabel;
    private javax.swing.JTextField afterTextField;
    private javax.swing.JTextField beforeTextField;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JLabel egLabel;
    // End of variables declaration//GEN-END:variables

    public void requestFocus() {
        JTextField tf = betweenRadioButton.isSelected() ?
                          afterTextField : daysTextField;
        int n = tf.getText().length();
        if (n > 0) {
            tf.setCaretPosition(0);
            tf.moveCaretPosition(n);
        }
        tf.requestFocus();
    }
}
