/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


package org.netbeans.modules.search.types;


import java.beans.Customizer;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Vector;
import java.util.TreeSet;
import javax.swing.border.CompoundBorder;
import javax.swing.border.TitledBorder;
import javax.swing.JFrame;
import javax.swing.JPanel;

import org.netbeans.modules.search.res.Res;

import org.openide.loaders.DataLoader;
import org.openide.TopManager;


/**
 * Customizer of <code>ObjectTypeType</code> bean.
 *
 * @author  Petr Kuzel
 */
public class ObjectTypeCustomizer extends JPanel implements Customizer {

    /** Object to customize. */
    private ObjectTypeType peer;
    
    /** Flag indicating the are setting of object in run. */
    private boolean setting = false;

    
    /** Creates new form <code>ObjectTextCustomizer</code>. */
    public ObjectTypeCustomizer() {
        initComponents ();
        TitledBorder tb = new TitledBorder(Res.text("LABEL_OBJECT_TYPE"));         
        tb.setBorder(new CompoundBorder());
        setBorder (tb);
    }


    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jScrollPane1 = new javax.swing.JScrollPane();
        typeList = new javax.swing.JList();
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        typeList.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                typeListValueChanged(evt);
            }
        }
        );
        
        jScrollPane1.setViewportView(typeList);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(4, 4, 4, 4);
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(jScrollPane1, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void typeListValueChanged (javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_typeListValueChanged
        if(setting) 
            // ignore calls caused by setObject() implementation
            return; 

        DataLoader lds[] = TopManager.getDefault().getLoaderPool().toArray();

        Vector toret = new Vector();
        Object[] sel = typeList.getSelectedValues();

        for (int i=0; i<lds.length; i++) {
            String id = lds[i].getDisplayName();
            if (id == null) continue; //may be null :-(
            for (int j=0; j<sel.length; j++) {
                if (sel[j] == null) continue; //may be null :-(
                if (id.equals((String)sel[j])) {
                    toret.add(lds[i]);
                    break;
                }
            }
        }

        Class[] ret =  new Class[toret.size()];

        Iterator it = toret.iterator();
        int k = 0;
        while (it.hasNext()) {
            ret[k++] = it.next().getClass();
        }

        peer.setMask(ret);
    }//GEN-LAST:event_typeListValueChanged


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JList typeList;
    // End of variables declaration//GEN-END:variables

    /** Sets object to customize. Initialize customizer with proper values. 
     * Implements <code>Customizer</code> inetrface method.
     * @param obj object t customize */
    public void setObject(Object obj) {
        setting = true;

        try {
            peer = (ObjectTypeType)obj;

            DataLoader loaders[] = TopManager.getDefault().getLoaderPool().toArray();

            // Model data for list.
            TreeSet types = new TreeSet();

            ArrayList selected = new ArrayList();
            
            for(int i=0; i<loaders.length; i++) {
                String displayName = loaders[i].getDisplayName();
                if (displayName == null) 
                    continue;
                
                types.add(displayName);
                
                // Create indices of values to select.
                if (peer.mask == null) 
                    continue;

                Class repreClass = loaders[i].getRepresentationClass();
                if(repreClass == null)
                    continue;
                
                for(int j=0; j<peer.mask.length; j++) {
                    DataLoader loader = DataLoader.getLoader(peer.mask[j]);
                    if(loader != null && repreClass.equals(loader.getRepresentationClass())) {
                        selected.add(displayName);
                        break; 
                    }
                }
            }

            // select saved values
            typeList.setListData((String[])types.toArray(new String[types.size()]));

            int[] indexes = new int[selected.size()];
            
            ArrayList typesList = new ArrayList(types);

            for(int i = 0; i < indexes.length; i++) {
                indexes[i] = typesList.indexOf(selected.get(i));
            }

            typeList.setSelectedIndices(indexes);

        } catch(Exception e) {
            if(Boolean.getBoolean("netbeans.debug.exceptions")) // NOI18N
                e.printStackTrace();
        } finally {
            setting = false;
        }

    }

    /** Dummy implementation of <code>Customizer</code> interface method. */
    public void addPropertyChangeListener(PropertyChangeListener p1) {}

    /** Dummy implementation of <code>Customizer</code> interface method. */
    public void removePropertyChangeListener(PropertyChangeListener p1) {}

    /** Overrides superclass method. Requests ffocus si delegated to typeList component. */
    public void requestFocus() {
        typeList.requestFocus();
    }

    
    /** Main method. */
    public static void main(String args[]) {
        JFrame fr = new JFrame();
        ObjectTypeCustomizer me = new ObjectTypeCustomizer();
        ObjectTypeType ty = new ObjectTypeType();
        ty.mask = new Class[] {TopManager.getDefault().getLoaderPool().toArray()[3].getClass()};
        me.setObject(ty);
        fr.getContentPane().add(me);
        fr.setVisible(true);
    }
}
