/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.vcscore;

import java.beans.*;

import org.openide.TopManager;
import org.openide.ServiceType;
import org.openide.nodes.Node;

/** 
* Hack for installing VCS search type
* after project open.
*
* @author Miloslav Metelka, Martin Entlicher
*/
class ProjectChangeHack implements PropertyChangeListener, Runnable {

    /** TopManager listener. */
    private static PropertyChangeListener listener = null;

    /** Project node. */
    private Node node;
    
    //private static boolean isInstanceCreated = false;

    /** 
     * @param check - if true, check is performed immediately
     */
    private ProjectChangeHack(boolean check) {
        if (check) checkEngine();
        else node = TopManager.getDefault().getPlaces().nodes().projectDesktop();
    }
    
    /**
     * Called when an already-installed module is restored (at IDE startup time).
     * Should perform whatever initializations are required.
     */
    public static void restored() {
        if (listener == null) {
            listener = new ProjectChangeHack(true);
            TopManager.getDefault().addPropertyChangeListener(listener);
        }
    }

    /**
     * Called when the module is uninstalled (from a running IDE).
     * Should remove whatever functionality from the IDE that it had registered.
    */
    public static void uninstalled() {
        // [PENDING - remove installed files]
        if (listener != null) TopManager.getDefault().removePropertyChangeListener(listener);
    }

    public void propertyChange(PropertyChangeEvent ev) {
        if (ev.getPropertyName().equals(TopManager.PROP_PLACES)) checkEngine();
    }
    
    public void checkEngine() {
        org.openide.util.RequestProcessor.postRequest(this);
    }
    
    public void run() {
        Node newNode = TopManager.getDefault().getPlaces().nodes().projectDesktop();
        if (node != newNode) {
            node = newNode;

            // Project was changed
            initSearchService();
        }
    }

    public static void initSearchService() {
         ServiceType.Registry serviceRegistry = TopManager.getDefault().getServices();
         ServiceType stype = serviceRegistry.find(org.netbeans.modules.vcscore.search.VcsSearchType.class);
         if (stype == null) {
            java.util.List serviceList = serviceRegistry.getServiceTypes();
            stype = new org.netbeans.modules.vcscore.search.VcsSearchType();
            serviceList.add(stype);
            serviceRegistry.setServiceTypes(serviceList);
         }
    }

    
}
