/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.vcscore.actions;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.beans.BeanInfo;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.*;

import javax.swing.JMenuItem;

import org.openide.TopManager;
import org.openide.DialogDescriptor;
import org.openide.WizardDescriptor;
import org.openide.awt.JInlineMenu;
import org.openide.filesystems.FileSystem;
import org.openide.filesystems.Repository;
import org.openide.filesystems.RepositoryEvent;
import org.openide.filesystems.RepositoryReorderedEvent;
import org.openide.filesystems.RepositoryListener;
import org.openide.modules.ManifestSection;
import org.openide.nodes.BeanNode;
import org.openide.util.actions.CallableSystemAction;
import org.openide.util.Lookup;
import org.openide.util.LookupEvent;
import org.openide.util.LookupListener;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;

import org.netbeans.modules.vcscore.VcsFileSystem;

/**
 * The action for mounting VCS filesystems.
 * @author  Martin Entlicher
 */
public class VcsMountAction extends CallableSystemAction {

    /** empty array of menu items */
    //static JMenuItem[] NONE = new JMenuItem[] {};
    
    //private static final String FS_CHANGED = "FileSystems changed";

    static final long serialVersionUID = -1253387396434922262L;

    public void performAction() {
    }
    
    public HelpCtx getHelpCtx() {
        return null;
    }
    
    public boolean isEnabled() {
        return true;
    }
    
    public String getName() {
        return org.openide.util.NbBundle.getBundle(VcsMountAction.class).getString("CTL_MountActionName");
    }
    
    public javax.swing.JMenuItem getMenuPresenter() {
        return new FileSystems();
    }
    
    private static void createFS(ManifestSection.FileSystemSection section) throws java.io.IOException {
        try {
            FileSystem fs = section.createFileSystem ();
            BeanNode bn = new BeanNode(fs);
            java.awt.Component c = (java.awt.Component) bn.getCustomizer ();
            if (c == null) {
                org.openide.explorer.propertysheet.PropertySheet ps = new org.openide.explorer.propertysheet.PropertySheet ();
                ps.setNodes(new BeanNode[] {bn});
                c = ps;
            }
            if (c instanceof java.awt.Window) {
                c.setVisible (true);
            } else {
                DialogDescriptor dd = new DialogDescriptor (c, org.openide.util.NbBundle.getBundle(VcsMountAction.class).getString("LAB_FS_Configuration"));
                /*
                if (HelpCtx.findHelp (c).equals (HelpCtx.DEFAULT_HELP))
                    dd.setHelpCtx (getHelpCtx ());
                 */
                TopManager.getDefault ().createDialog (dd).show ();
                if (dd.getValue ().equals (DialogDescriptor.OK_OPTION) || dd.getValue ().equals (WizardDescriptor.FINISH_OPTION)) {
                    TopManager.getDefault().getRepository ().addFileSystem (fs);
                }
            }
        } catch (Exception ex) {
            java.io.IOException e = new java.io.IOException (ex.getMessage ());
            TopManager.getDefault ().getErrorManager ().copyAnnotation (e, ex);
            throw e;
        }
    }

    private static JMenuItem[] createMenu (Set fsSections) {
        //System.out.println("VcsMountAction.createMenu()");
        TreeSet fsSectionsSorted = new TreeSet(new Comparator () {
            public int compare(Object o1, Object o2) {
                if (o1 instanceof ManifestSection.FileSystemSection &&
                    o2 instanceof ManifestSection.FileSystemSection) {
                    String name1 = ((ManifestSection.FileSystemSection) o1).getName();
                    String name2 = ((ManifestSection.FileSystemSection) o2).getName();
                    //System.out.println("name1 = "+name1+", name2 = "+name2);
                    return name1.compareTo(name2);
                } else return 0;
            }
            public boolean equals(Object obj) {
                return false;
            }
        });
        fsSectionsSorted.addAll(fsSections);
        JMenuItem[] menuitem = new JMenuItem[fsSections.size()];
        int i = 0;
        for(Iterator it = fsSectionsSorted.iterator(); it.hasNext(); i++) {
            final ManifestSection.FileSystemSection section = (ManifestSection.FileSystemSection) it.next();
            String name = NbBundle.getMessage (VcsMountAction.class, "CTL_MountType", "&" + section.getName());
            menuitem[i] = new JMenuItem();
            org.openide.awt.Actions.setMenuText (menuitem[i], name, true);
            menuitem[i].setIcon(CallableSystemAction.get(VcsMountAction.class).getIcon());
            menuitem[i].addActionListener(new ActionListener () {
                public void actionPerformed (ActionEvent evt) {
                    try {
                        createFS(section);
                    }
                    catch (java.io.IOException exc) {
                        TopManager.getDefault().getErrorManager().notify(exc);
                    }
                }
            });
        }
        return menuitem;
    }

    private static Set getFS(final PropertyChangeListener propListener) {
        final Lookup.Result result = Lookup.getDefault().lookup(new Lookup.Template(ManifestSection.FileSystemSection.class));
        /*
        result.addLookupListener(new LookupListener() {
            public void resultChanged (LookupEvent ev) {
                System.out.println("Lookup changed: "+ev);
                propListener.propertyChange(new PropertyChangeEvent(ev.getSource(), FS_CHANGED, null, null));
                result.removeLookupListener(this);
            }
        });
         */
        Collection collectionSection = result.allInstances();
        HashSet fsSections = new HashSet();
        for (Iterator it = collectionSection.iterator(); it.hasNext(); ) {
            ManifestSection.FileSystemSection section = (ManifestSection.FileSystemSection) it.next();
            try {
                Class fs = section.getSectionClass();
                BeanInfo info = org.openide.util.Utilities.getBeanInfo(fs);
                info.getBeanDescriptor();
                //BeanNode bn = new BeanNode(fs);
                Object vcs = info.getBeanDescriptor().getValue(VcsFileSystem.VCS_PROVIDER_ATTRIBUTE);
                if (Boolean.TRUE.equals(vcs)) {
                    fsSections.add(section);
                }
            } catch (Exception exc) {
                // continue to the next one
                /*
            } catch (java.beans.IntrospectionException intrexc) {
                // continue to the next one
                 */
            }
        }
        return fsSections;
    }

    
    private static final class FileSystems extends JInlineMenu {
        
        static final long serialVersionUID = 3153147697434529731L;
        
        /** last registered items */
        private transient Set lastSections = Collections.EMPTY_SET;
        
        public FileSystems() {
        }
                
        public void addNotify() {
            Set fsSections = getFS(null);
            if (!lastSections.equals(fsSections)) {
                setMenuItems(createMenu(fsSections));
                lastSections = fsSections;
            }
            super.addNotify();
        }
        
    }

}
