/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.vcscore.actions;

import org.openide.TopManager;
import org.openide.awt.SplittedPanel;
import org.openide.explorer.ExplorerPanel;
import org.openide.explorer.propertysheet.PropertySheetView;
import org.openide.explorer.view.BeanTreeView;
import org.openide.nodes.BeanNode;
import org.openide.nodes.Node;
import org.openide.util.HelpCtx;
import org.openide.util.SharedClassObject;
import org.openide.util.actions.CallableSystemAction;

import org.netbeans.modules.vcscore.settings.VcsSettings;

/**
 * The action, that will present VCS global options.
 * @author  Martin Entlicher
 */
public class VcsSettingsAction extends CallableSystemAction {

    static final long serialVersionUID = 2267347903850705340L;
    
    /** Getter for help.
     */
    public HelpCtx getHelpCtx () {
        return null;
    }

    /** Getter for name
     */
    public String getName () {
        return org.openide.util.NbBundle.getBundle(VcsSettingsAction.class).getString("CTL_VcsSettingsActionName");
    }
    
    /**
     * Open VCS global options
     */
    public void performAction() {
        ExplorerPanel panel = new ExplorerPanel();
        Node root;
        try {
            root = new BeanNode(SharedClassObject.findObject(VcsSettings.class, true));
        } catch (java.beans.IntrospectionException intrexc) {
            TopManager.getDefault().notifyException(intrexc);
            return ;
        }
        panel.getExplorerManager().setRootContext(root);
        PropertySheetView propertySheetView = new PropertySheetView();
        org.openide.awt.SplittedPanel split = new org.openide.awt.SplittedPanel();
        split.setSplitType(org.openide.awt.SplittedPanel.HORIZONTAL);
        split.add(new BeanTreeView(), org.openide.awt.SplittedPanel.ADD_LEFT);
        split.add(propertySheetView, org.openide.awt.SplittedPanel.ADD_RIGHT);
        panel.add(split);
        panel.setName(org.openide.util.NbBundle.getBundle(VcsSettingsAction.class).getString("CTL_VcsSettingsName"));
        panel.open();
        panel.setActivatedNodes(new Node[] { root });
    }

}
