/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.vcscore.cache;


import java.io.*;
import java.lang.ref.*;
import org.openide.filesystems.FileObject;

/** Basic object that's stored in the cache. The filesystem shoud subclass it to create 
 *  it's own interpretaion of a file in filesystem.
 *
 *
 * @author  mkleint
 * @version 
 */

public abstract class CacheFile extends java.lang.Object {

    protected String name = ""; // NOI18N
    protected String status = ""; // NOI18N
    protected String locker = ""; // NOI18N
    protected String revision = ""; // NOI18N
    protected String sticky = ""; // NOI18N
    protected String attr = ""; // NOI18N
    protected String date = ""; // NOI18N
    protected String time = ""; // NOI18N
    protected int    size = 0;
    
    /** cacheID of the cache where this file belongs to */
    protected String parentCache;
    
    private CacheDir parent;
    
    /** the reference object associated with this file */
    protected Reference reference;

    /** Creates new CacheFile */
    public CacheFile(String memberOfCache) {
        setCacheName(memberOfCache);
        reference = null;
    }
    
    public CacheFile(String memberOfCache, String name) {
        this(memberOfCache);
        this.name = name;
    }
    
    /** set parent of this file.. in 99% cases it means it's parent directory
     *  @param par - directory where this file resides.
     */
    public void setParent(CacheDir par) {
      parent = par;
    }

    /* returns it's parent directory.. note that most file manipulation is triggered fromthe dir object
     *
     */
    public CacheDir getParent() {
      return parent;
    }
    
    /** return the file's name
     */
    public String getName () { return this.name; }
    /** sets the file's name
     */
    public void setName (String name) { this.name = name; }

    /** returns the file's status
     */
    public String getStatus(){ return this.status; }
    /** sets the file's status
     */
    public void setStatus(String stat) { status = stat; }
    
    /** Returns the current revision of the file.
     */
    public String getRevision() { return this.revision; }
    /** Sets the current revision of the file.
     */
    public void setRevision(String revision) { this.revision = revision; }
    /** Returns the current sticky information of the file.
     */
    public String getSticky() { return this.sticky; }
    /** Sets the current sticky information of the file.
     */
    public void setSticky(String sticky) { this.sticky = sticky; }
    
    public String getDate () { return this.date; }
    public void setDate (String date) {this.date = date; }

    public String getTime () { return this.time; }
    public void setTime (String time) {this.time = time; }

    public String getLocker () { return this.locker; }
    public void setLocker (String locker) {this.locker = locker; }

    public String getAttr () { return this.attr; }
    public void setAttr (String attr) {this.attr = attr; }

    public int getSize () { return this.size; }
    public void setSize (int size) {this.size = size; }

    public String getCacheName() {
        return parentCache;
    }
    
    public void setCacheName(String cacheName) {
        parentCache = cacheName;
    }
    
    /** file itself doesn't store the path, we need to access it's parent (directory)
     */
    public String getAbsolutePath() {
        if (parent != null) {
          return parent.getAbsolutePath() + File.separator + name;
        }
        else return null;
    }

    /** subclasses need to implement this to provide structure and data of the line 
     * that's written to disk cache.
     * *assumes that it's written into text files*
     */
    public abstract String writeLineToDisk();
    
    
}
