/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.vcscore.cache;


import java.lang.ref.*;
import org.openide.filesystems.FileObject;
import org.netbeans.modules.vcscore.util.Debug;
import java.io.*;

/** Thread subclass that makes sure the cache gets cleared when unused.
 *  Assumes all cached file references are in one ReferenceQueue
 *
 * @author  mkleint
 * @version
 */
public class CacheQueue extends java.lang.Thread {
    private Debug E=new Debug("CacheQueue", true); // NOI18N
    private Debug D=E;
    
    ReferenceQueue refQueue;
    /** Creates new CacheQueue */
    public CacheQueue() {
        refQueue = new ReferenceQueue();
        this.setName("FileSystem Cache Queue");
    }
    
    /** Once in a while checks if there are any references in the queue and proceeds them all. <p>
     *  For files - removes them from the cache immediately. Marks it's parent directory as uncomplete. <p>
     *  For dirs - removes them from cache only if the dir is already empty (no subdir or file in it).
     */
    public void run() {
        try {
            while (true) {
                Thread.sleep(5000);
                CacheReference ref = (CacheReference)refQueue.poll();
                while (ref != null) {
                    File file = ref.getCacheFile();
                    CacheFile cacheFile = CacheHandler.getInstance().getCacheFile(file, CacheHandler.STRAT_NONE, ref.getCacheName());
                      // I use strategy none, in order to build nothing, just check
                    if (cacheFile != null) {
                        if (!(cacheFile instanceof CacheDir)) {
                            // is a regular file
                            CacheDir parent = cacheFile.getParent();
                            D.deb("file in queue=" + parent.getAbsolutePath() + " /" + cacheFile.getName());
                            //                        cacheFile.clearReferenceObject();
                            if (parent != null) {
                                parent.setComplete(false);
                                parent.removeFile(cacheFile.getName(), false);
                            }
                        } else {
                            // is directory 
                            D.deb("dir in queue=" + cacheFile.getAbsolutePath());
                            CacheDir currDir = (CacheDir)cacheFile;
                            if (currDir.isEmpty()) {
                               CacheDir parent = currDir.getParent();
                               if (parent != null) {
                                  parent.setComplete(false);
                                  parent.removeChildDir(currDir.getName(), false);
                               }
                            }
                        }
                    }
                    ref = (CacheReference)refQueue.poll();
                }
            }
        } catch (InterruptedException exc) {
        }
    }
    
    /** Returns the instance of the ReferenceQueue currently in use.
     */
    ReferenceQueue getQueue() {
        return refQueue;
    }
}
