/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.vcscore.caching;

import java.io.File;

import org.netbeans.modules.vcscore.cache.CacheFile;
import org.netbeans.modules.vcscore.cache.CacheDir;
import org.netbeans.modules.vcscore.cache.CachedFileSystem;

/**
 * The provider of caching of VCS file attributes information.
 *
 * @author  Martin Entlicher
 */
public interface FileCacheProvider extends CachedFileSystem {
    
    public String getCacheId();
    
    public void setFSRoot(String fsRoot);
    
    public void setRelativeMountPoint(String relativeMountPoint);
    
    public boolean isDir(String path);

    public boolean isFile(String path);

    public CacheFile getFile(String path);

    public CacheDir getDir(String path);

    public String[] getFilesAndSubdirs(String path);
    
    public void readDir(String path);
    
    public void refreshCacheDir(String path);

    public void refreshCacheDirRecursive(String path);
    
    /**
     * Refresh the directory only from the cached data (usually located on the disk).
     */
    public void refreshDirFromDiskCache(File fpath);

    public void addFolder(String path);

    public void addFile(String path);
    
    public void rename(String oldPath, String newPath);
    
    public void remove(String path, boolean wasDirectory);
}

