/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.vcscore.caching;

import java.io.*;
import java.util.Collection;
import java.util.Iterator;

import org.openide.TopManager;

import org.netbeans.modules.vcscore.VcsFactory;
import org.netbeans.modules.vcscore.VcsDirReader;
import org.netbeans.modules.vcscore.cache.CacheDir;
import org.netbeans.modules.vcscore.cache.CacheFile;
import org.netbeans.modules.vcscore.cache.FileSystemCache;
import org.netbeans.modules.vcscore.cache.CacheHandler;
import org.netbeans.modules.vcscore.util.*;
import org.netbeans.modules.vcscore.commands.VcsCommandExecutor;

/**
 * Directory in the cache.
 * @author Martin Entlicher
 */
public class VcsCacheDir extends CacheDir {
    private Debug E=new Debug("VcsCacheDir", true); // NOI18N
    private Debug D=E;

    /** Vector<VcsDir> */
    //private Vector subdirs=new Vector(10);

    /** Vector<VcsFile> */
    //private Vector files=new Vector(10);

    /** Complete path from root including filename */
    //String path = ""; // NOI18N
    private volatile boolean loaded = false;
    private volatile boolean beingLoaded = false;
    /*
    private int rootNameLength = 0;
    private VcsFactory factory;
    */
    /**
     * Create a new VCS cache directory.
     * @args cacheName the cache name
     * @args dirFile the cache directory file
     * @args rootNameLength the length of the name of the cache root
     * @args factory the VCS factory
     */
    public VcsCacheDir(String cacheName, File dirFile/*, int rootNameLength, VcsFactory factory*/) {
        super(cacheName, dirFile);
        setLocal(true);
        setLoaded(false);
        //setStatus(
        //this.rootNameLength = rootNameLength;
        //this.factory = factory;
    }
    
    public String getPath () {
        //return ((VcsCache) getCacheObject()).getPath(dirFile);
        return dirFile.getAbsolutePath();
        //return dirName.substring(rootNameLength);
    }
    
    public File getFile() {
        return dirFile;
    }
    
    //public void setPath (String path) { this.path = path; }

    public boolean isLoaded () { return this.loaded; }
    public void setLoaded (boolean loaded) { this.loaded = loaded; }
    public boolean isBeingLoaded () { return this.beingLoaded; }
    public void setBeingLoaded (boolean beingLoaded) { this.beingLoaded = beingLoaded; }

    public void setLoadedRecursive (boolean loaded) {
        this.loaded = loaded;
        CacheDir[] subdirs = /*(VcsCacheDir[])*/ getSubDirs();
        for(int i = 0; i < subdirs.length; i++) {
            ((VcsCacheDir) subdirs[i]).setLoadedRecursive(loaded);
        }
    }
    
    public void add(CacheFile child) {
        if (child instanceof VcsCacheDir || child instanceof CacheDir) {
            addChildDir((CacheDir) child, true);
        } else {
            addFile(child, true);
        }
    }

    //-------------------------------------------
    public String[] getFilesAndSubdirs() {
        CacheDir[] dirs = getSubDirs();
        CacheFile[] files = getFiles();
        
        int subdirsLen = dirs.length;
        int filesLen = files.length;
        int len = subdirsLen + filesLen;
        String[] res = new String[len];
        int r = 0;

        for(int i = 0; i < subdirsLen; i++) {
            //VcsDir d=(VcsDir)subdirs.elementAt(i);
            //D.deb("d="+d.name); // NOI18N
            res[r++] = dirs[i].getName();
        }

        for(int i = 0; i < filesLen; i++) {
            //VcsFile f=(VcsFile)files.elementAt(i);
            //D.deb("f="+f.name); // NOI18N
            res[r++] = files[i].getName();
        }

        //D.deb("res="+VcsUtilities.arrayToString(res)); // NOI18N
        return res;
    }
    
    /** Path string to the netbeans.cache file of this directory
     */
    protected String getCacheFileName() {
        return ((VcsCache) getCacheObject()).getCacheFileName(dirFile);
    }

    public void populateWithLocal() {
        
        D.deb("populateWithLocal()", "dirName=" + dirName);
        FilenameFilter filter = ((VcsCache) getCacheObject()).getLocalFileFilter();
        File[] subFiles = (filter == null) ? dirFile.listFiles() : dirFile.listFiles(filter);
        if (subFiles != null) {
            for (int index = 0; index < subFiles.length; index++) {
                File oneFile = subFiles[index];
                if (oneFile == null) continue;
                if (oneFile.isDirectory()) {
                    D.deb("    populate- adding Dir=" + oneFile.getName());
                    VcsCacheDir dir = (VcsCacheDir) getCacheObject().getDir(oneFile.getAbsolutePath());
                    if (dir == null) {
                        dir = new VcsCacheDir(getCacheObject().getId(), oneFile);
                    }
                    dir.setStatus(((VcsCache) getCacheObject()).getLocalFileStatus());
                    dir.setLocker("");
                    dir.setRevision("");
                    dir.setSticky("");
                    dir.setAttr("");
                    dir.setDate("");
                    dir.setTime("");
                    dir.setSize(0);
                    dir.setLocal(true);
                    this.addChildDir(dir, false);
                } /*else {// I do not need local files in the cache !!
                 D.deb("    populate- adding File=" + oneFile.getName());
                 VcsCacheFile file = new VcsCacheFile(getCacheObject().getId(), oneFile.getName());
                 file.setStatus(((VcsCache) getCacheObject()).getLocalFileStatus());
                 this.addFile(file, false);
               }
                   */
            }
        }
        
        setAppliedLevel(CacheHandler.STRAT_LOCAL);
        /*
        CacheDir cvsDir = getSubDir("CVS");
        if (cvsDir == null) {
          setLocal(true);
          setStatus(CacheFile.LOCAL);
        } else {
           setLocal(false);
           setStatus(" ");
        }
         */
        //D.deb("populateWithLocal() - is Local=" + isLocal());
    }
    
    /*
    public void rename(String newPath) {
        String name = VcsUtilities.getFileNamePart(newPath);
        this.setName(name);
        setPath(newPath);
        for(int i = 0; i < subdirs.size(); i++) {
            VcsDir subdir = (VcsDir) subdirs.get(i);
            subdir.rename(newPath+"/"+subdir.getName());
        }
    }
     */
    
    public boolean readFromDisk() {
        boolean isOk = false;
        String cacheFilePath = getCacheFileName();
        if (cacheFilePath == null) return false;
        File cacheFile = new File(cacheFilePath); // the actual netbeans.cache file
        //D.deb("readFromDisk()","cacheFile = "+cacheFile+", does"+(cacheFile.exists() ? "" : " not")+" exist");
        //System.out.println("readFromDisk(): cacheFile = "+cacheFile+", does"+(cacheFile.exists() ? "" : " not")+" exist");
        if (cacheFile.exists() && cacheFile.canRead()) {
            BufferedReader in = null;
            try{
               in = new BufferedReader
                                   (new InputStreamReader
                                    (new BufferedInputStream
                                     (new FileInputStream(cacheFile))));
                String line = null;
                while ((line = in.readLine()) != null) {
                    CacheFile file = VcsCacheFile.readFileCache(line, getCacheName(), getFile());
                    add(file);
                    if (file instanceof CacheDir) getCacheObject().registerDir((CacheDir) file);
                }
                in.close();
                isOk= true;
                setAppliedLevel(CacheHandler.STRAT_DISK);
                //return isOk;
            }catch (IOException e){
                E.err(e,"readDirFromDiskCache() failed"); // NOI18N
            } finally {
                if (in != null) { 
                    try {
                        in.close();
                    } catch (IOException exc) {}
                    in = null;
                }
            }
        }
        return isOk;
    }
    
    public void writeToDisk() {
        //System.out.println("writeToDisk(): file = "+getCacheFileName());
        //System.out.println("writeToDisk(): dir = "+this);
        String cacheFilePath = getCacheFileName();
        if (cacheFilePath == null) return ;
        File cacheFile = new File(cacheFilePath); // the actual netbeans.cache file
        if (true) {//cacheFile.exists() && cacheFile.canWrite()) {
            BufferedWriter out = null;
            try {
                File cacheDir = cacheFile.getParentFile();
                if (!cacheDir.exists()) cacheDir.mkdirs();
                out = new BufferedWriter(
                            new OutputStreamWriter(
                                new BufferedOutputStream(
                                    new FileOutputStream(cacheFile))));
                // write files first
                Collection files = cachedFiles.values();
                Iterator it = files.iterator();
                while (it.hasNext()) {
                    VcsCacheFile file = (VcsCacheFile) it.next();
                    if (!file.isLocal()) {
                        out.write(file.writeLineToDisk());
                        out.newLine();
                    }
                }
                // then directories
                //D.deb(this.getAbsolutePath() + " files written");
                Collection dirs = childDirs.values();
                it = dirs.iterator();
                while (it.hasNext()) {
                    VcsCacheDir dir = (VcsCacheDir) it.next();
                    //if (dir.getAppliedLevel() >= CacheHandler.STRAT_LOCAL) {
                    if (!dir.isLocal()) {
                        out.write(dir.writeLineToDisk());
                        out.newLine();
                    }
                }
                //D.deb(this.getAbsolutePath() + " dirs written");
                out.flush();
            } catch (IOException exc) {
                TopManager.getDefault().notifyException(exc);
                //exc.printStackTrace();
            } finally {
                try {
                    if (out != null) out.close();
                } catch (IOException e) {
                    TopManager.getDefault().notifyException(e);
                    //e.printStackTrace();
                    //E.err(e,"writeDirToDiskCache() failed for "+getName()); // NOI18N
                }
            }
        }
    }
    
    public String writeLineToDisk() {
        String[] elements = RefreshCommandSupport.makeElements(this);
        elements[RefreshCommandSupport.ELEMENT_INDEX_FILE_NAME] += "/";
        return RefreshCommandSupport.getLineFromElements(elements);
    }

    public synchronized void checkServer() {
        if (isBeingLoaded()) return;
        //D.deb("checkServer() for " + getName() + "    curr.level=" + getAppliedLevel());
        // thread has to start here..
        VcsCache cache = (VcsCache) getCacheObject();
      
        if (!isLoaded() || !isLocal()) {
            setAppliedLevel(CacheHandler.STRAT_DISK);
            //cache.prepareCache(new File(this.getAbsolutePath()), false, CacheHandler.STRAT_REFRESHING);
            //System.out.println("VcsCacheDir.checkServer(): dir = "+this.toString());
            //cache.fireCacheHandlerEvent(FileSystemCache.EVENT_CHANGED, this);
            cache.lockFileObjects(this, false);
            cache.runVcsDirReader(dirFile);
        }
    }

    /** Associates with the STRAT_REFRESH_RECURS, if that strategy is requested this method is called.
     * do a recursive server check. Connect to server and request status update etc..
     * Note: within this method implementations need to make sure
     */
    public void checkServerRecursive() {
        if (isBeingLoaded()) return;
        //D.deb("checkServer() for " + getName() + "    curr.level=" + getAppliedLevel());
        // thread has to start here..
        VcsCache cache = (VcsCache) getCacheObject();
      
        if (!isLoaded() || !isLocal()) {
            setAppliedLevel(CacheHandler.STRAT_DISK);
            cache.lockFileObjects(this, true);
            //cache.prepareCache(new File(this.getAbsolutePath()), true, CacheHandler.STRAT_REFRESHING);
            //cache.fireCacheHandlerEvent(FileSystemCache.EVENT_CHANGED_RECURSIVELY, this);
            cache.runVcsDirReaderRecursive(dirFile);
        }
    }    


    public String toString() {
        CacheFile[] files = getFiles();
        String[] fileNames = new String[files.length];
        for (int i = 0; i < files.length; i++) fileNames[i] = files[i].getName();
        CacheDir[] dirs = getSubDirs();
        String[] dirNames = new String[dirs.length];
        for (int i = 0; i < dirs.length; i++) dirNames[i] = dirs[i].getName();
        String strOut =  "VcsDir[name='"+name+"',status="+getStatus()+ // NOI18N
                         "\n files="+VcsUtilities.arrayToString(fileNames)+",\n subdir="+VcsUtilities.arrayToString(dirNames)+"\n isLocal = "+isLocal()+" isLoaded = "+isLoaded()+" isBeingLoaded = "+isBeingLoaded()+"]"; // NOI18N
        
        if (getFiles() != null) {
            for (int i = 0; i < getFiles().length; i++) {
                strOut += "\nSubFile = " + getFiles()[i].toString();
            }
        }/*
        if (getSubDirs() != null) {
            for (int i = 0; i < getSubDirs().length; i++) {
                strOut += "\nSubDir = " + getSubDirs()[i].toString();
            }
        }
         */
        return strOut;
    }


    
}
