/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.vcscore.commands;

import java.io.*;
import java.util.ArrayList;

import org.openide.nodes.*;

/**
 * A node that define <code>VcsCommand</code> as the cookie.
 * @author  Martin Entlicher
 */
public class VcsCommandNode extends AbstractNode /*implements java.io.Externalizable */{

    //static final long serialVersionUID = -1033426784954732681L;

    //private Children children;
    private VcsCommand cmd;
    
    /* identification of serialization version.
     */
    //private int serialVersion = 1;
    
    public VcsCommandNode(boolean isLeaf, VcsCommand cmd) {
        this((isLeaf) ? Children.LEAF : new Index.ArrayChildren(), cmd);
    }
    
    /**
     * Creates new VcsCommandNode
     * @param children the children of this node
     * @param cmd the command associated to this node or null when this node should be a separator
     */
    public VcsCommandNode(Children children, VcsCommand cmd) {
        super(children);
        //this.children = children;
        this.cmd = cmd;
        if (cmd != null) {
            setCookie(cmd);
            setName(cmd.getName());
            setDisplayName(cmd.getDisplayName());
        }
    }

    private void setCookie(Node.Cookie cookie) {
        CookieSet set = new CookieSet();
        set.add(cookie);
        this.setCookieSet(set);
    }
    
    /*
    public Node.Handle getHandle() {
        if (getParentNode() != null) {
            return super.getHandle();
        } else {
            return new CommandRootHandle(cmd);
        }
    }
     */
    
    /* Serialize this vcs command node.
     * Subclasses wishing to store state must call the super method, then write to the stream.
     * @param out the stream to serialize to
     *
    public void writeExternal(final java.io.ObjectOutput p1) throws java.io.IOException {
        p1.writeInt(serialVersion);
        p1.writeObject(cmd);
    }
    
    /* Delegates instance of replacer class to be serialized instead
     * of command node itself. Replacer class calls writeExternal and
     * constructor, readExternal and readResolve methods properly, so
     * any vcs command node can behave like any other externalizable object.
     * Subclasses can override this method to perform their
     * serialization differentrly.
     *
    protected Object writeReplace () throws ObjectStreamException {
        ArrayList meAndChildren = new ArrayList();
        meAndChildren.add(new Replacer(this));
        Node[] children = getChildren().getNodes();
        for(int i = 0; i < children.length; i++) {
            meAndChildren.add(((VcsCommandNode) children[i]).writeReplace());
        }
        return meAndChildren;//new Replacer(this);
    }

    /* Deserialize this vcs command node.
     * Subclasses wishing to store state must call the super method, then read from the stream.
     * @param in the stream to deserialize from
     *
    public void readExternal(final java.io.ObjectInput p1) throws java.io.IOException, java.lang.ClassNotFoundException {
        this.serialVersion = p1.readInt();
        VcsCommand cmd = (VcsCommand) p1.readObject();
        this.cmd = cmd;
        setCookie(cmd);
        //this.children = getChildren();
    }
    
    private static final class Replacer implements Serializable {

        static final long serialVersionUID=-3572340936214788323L;

        /*
          * Asociation with VcsCommandNode which is to be serialized using
          * this replacer
          *
        transient VcsCommandNode vcn;
        
        public Replacer (VcsCommandNode vcn) {
            this.vcn = vcn;
        }

        private void writeObject (ObjectOutputStream oos) throws IOException, ClassNotFoundException {
            // write the command of the node first
            //oos.writeObject(tc.getClass().getName());
            // and now let top component to serialize itself
            //System.out.println("Replacers.writeObject(): handle = "+vcn.getHandle());
            oos.writeObject(vcn.getHandle());
            vcn.writeExternal(oos);
        }

        private void readObject (ObjectInputStream ois) throws IOException, ClassNotFoundException {
            Node.Handle handle = (Node.Handle) ois.readObject();
            if (handle == null) {
                vcn = new VcsCommandNode(Children.LEAF, null);
            } else {
                vcn = (VcsCommandNode) handle.getNode();
            }
            vcn.readExternal(ois);
        }

        /* Resolve to original vcs command node instance *
        private Object readResolve () throws ObjectStreamException {
            return vcn;
        }

    }
     */
}
