/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.vcscore.diff;

import java.util.*;
import java.io.*;
import java.net.*;
import javax.swing.*;
import javax.swing.text.*;
import java.awt.BorderLayout;

import org.openide.util.HelpCtx;
import org.openide.text.CloneableEditorSupport;
import org.openide.windows.Workspace;
import org.openide.windows.Mode;

import org.netbeans.modules.vcscore.util.Debug;
import org.netbeans.modules.vcscore.util.TopComponentCloseListener;

/**
 * This class displays two editor panes with two files and marks the differences
 * by a different color.
 * @author  Martin Entlicher
 */
public class DiffComponent extends org.openide.windows.TopComponent {

    private Debug E=new Debug("DiffComponent",true); // NOI18N
    private Debug D=E;

    private AbstractDiff diff = null;
    private DiffPanel diffPanel = null;
    
    private ArrayList closeListeners = new ArrayList();
    
    /**
     * Used for deserialization.
     */
    private boolean diffSetSuccess = true;

    static final long serialVersionUID =3683458237532937983L;
    
    /**
     * An empty constructor needed by deserialization process.
     */
    public DiffComponent() {
    }
    
    /** Creates new DiffComponent from AbstractDiff object*/
    public DiffComponent(AbstractDiff absDiff) {
        this.diff = absDiff;
        setLayout(new BorderLayout());
        diffPanel = new DiffPanel();
        diffPanel.addPrevLineButtonListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                diffPanel.goToPrevLine(diff.getPrevDiffLine());
            }
        });
        diffPanel.addNextLineButtonListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                diffPanel.goToNextLine(diff.getNextDiffLine());
            }
        });
        
        add(diffPanel, BorderLayout.CENTER);
//        initComponents ();
        //setTitle(org.openide.util.NbBundle.getBundle(DiffComponent.class).getString("DiffComponent.title"));
        setName(org.openide.util.NbBundle.getBundle(DiffComponent.class).getString("DiffComponent.title"));
        //HelpCtx.setHelpIDString (getRootPane (), DiffComponent.class.getName ());
    }

/*    private java.awt.Container getContentPane() {
        return this;
        //return getRootPane();
    }
  */  
    private void addWindowListener(java.awt.event.WindowListener listener) {
        java.awt.Component ancestor = getTopLevelAncestor();
        if (ancestor instanceof java.awt.Window) {
            ((java.awt.Window) ancestor).addWindowListener(listener);
        }
    }
    
    void addCloseListener(TopComponentCloseListener listener) {
        closeListeners.add(listener);
    }
    
    protected Mode getDockingMode(Workspace workspace) {
        Mode mode =workspace.findMode(CloneableEditorSupport.EDITOR_MODE);
        if (mode == null) {
            mode = workspace.createMode(
                CloneableEditorSupport.EDITOR_MODE, getName(),
                CloneableEditorSupport.class.getResource(
                "/org/openide/resources/editorMode.gif" // NOI18N
            ));
        }
        return mode;
    }
    
    public void open(Workspace workspace) {
        //System.out.println("workspace = "+workspace);
        if (workspace == null) {
            workspace = org.openide.TopManager.getDefault().getWindowManager().getCurrentWorkspace();
        }
        Mode editorMode = getDockingMode(workspace);
        editorMode.dockInto(this);
        super.open(workspace);
        diffPanel.open();
    }
    
    
    /**
     * Override for clean up reasons.
     * Will be moved to the appropriate method when will be made.
     */
    public boolean canClose(Workspace workspace, boolean last) {
        boolean can = super.canClose(workspace, last);
        if (last && can) {
            exitForm(null);
        }
        return can;
    }
    /*
    public void removeNotify() {
        System.out.println("removeNotify() called");
        exitForm(null);
        super.removeNotify();
    }
     */
    
    public void setFile1(URL url) throws IOException {
        diffPanel.setFile1(url);
    }
    
    public void setFile2(URL url) throws IOException {
        diffPanel.setFile2(url);
    }
    
    public void setFile1Title(String title) {
        diffPanel.setFile1Title(title);
    }
    
    public void setFile2Title(String title) {
        diffPanel.setFile2Title(title);
    }
    
    public void setMimeType1(String mime) {
        diffPanel.setMimeType1(mime);
    }
    
    public void setMimeType2(String mime) {
        diffPanel.setMimeType2(mime);
    }
    
    public void setDocument1(Document doc) {
        diffPanel.setDocument1(doc);
    }
    
    public void setDocument2(Document doc) {
        diffPanel.setDocument2(doc);
    }
    
    private void setHighlight(StyledDocument doc, int line1, int line2, java.awt.Color color) {
    }
    
    private void unhighlight(StyledDocument doc) {
    }
    
    public void unhighlightAll() {
        diffPanel.unhighlightAll();
    }
    
    public void highlightRegion1(int line1, int line2, java.awt.Color color) {
        D.deb("Highlight region 1"); // NOI18N
        diffPanel.highlightRegion1(line1, line2, color);
    }
    
    public void highlightRegion2(int line1, int line2, java.awt.Color color) {
        D.deb("Highlight region 2"); // NOI18N
        diffPanel.highlightRegion2(line1, line2, color);
    }
    
    public void addEmptyLines1(int line, int numLines) {
        diffPanel.addEmptyLines1(line, numLines);
    }
    
    public void addEmptyLines2(int line, int numLines) {
        diffPanel.addEmptyLines2(line, numLines);
    }
    
    public void writeExternal(ObjectOutput out) throws IOException {
        super.writeExternal(out);
        //try {
        out.writeObject(diff);
        /*
        } catch (IOException exc) {
            System.out.println("exc = "+exc);
            exc.printStackTrace();
            throw exc;
        }
         */
    }
    
    public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
        super.readExternal(in);
        Object obj = in.readObject();
        diff = (AbstractDiff) obj;
        diffPanel = new DiffPanel();
        this.diffSetSuccess = diff.setDiffComponent(this);
    }
    
    private Object readResolve() throws ObjectStreamException {
        if (this.diffSetSuccess) return this;
        else return null;
    }
    
    /**
     * Disable serialization.
     */
    protected Object writeReplace () throws java.io.ObjectStreamException {
        exitForm(null);
        return null;
    }
    
       /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {
        /*
        try {
            org.netbeans.editor.Settings.setValue(null, org.netbeans.editor.SettingsNames.LINE_NUMBER_VISIBLE, lineNumbersVisible);
        } catch (Throwable exc) {
            // editor module not found
        }
         */
        //System.out.println("exitForm() called.");
        //diff.closing();
        //close();
        //dispose ();
        for(Iterator it = closeListeners.iterator(); it.hasNext(); ) {
            ((TopComponentCloseListener) it.next()).closing();
        }
 
    }

    private Boolean lineNumbersVisible = Boolean.FALSE;
    

}
