/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.vcscore.resources;

import java.util.jar.Manifest;

import org.openide.modules.ManifestSection;
import org.openide.modules.ModuleDescription;
import org.openide.modules.IllegalModuleException;

/**
 *
 * @author  Martin Entlicher
 */
public final class FSSectionBuilder extends Object {
    
    private static final String FS_SECTION_MANIFEST="/org/netbeans/modules/vcscore/resources/cvsfs-manifest.mf";
    
    private static ManifestSection.FileSystemSection section = null;

    /** Creates new FSSectionBuilder */
    public FSSectionBuilder() {
    }
    
    public static ManifestSection.FileSystemSection createFSSection() {
        if (section != null) return section;
        Manifest mf;
        try {
            //System.out.println("getting manifest from '"+FS_SECTION_MANIFEST+"' ...");
            java.io.InputStream in = FSSectionBuilder.class.getResourceAsStream(FS_SECTION_MANIFEST);
            //System.out.println("in = "+in);
            mf = new Manifest(in);
        } catch (java.io.IOException exc) {
            exc.printStackTrace();
            return null;
        }
        //System.out.println("mf = "+mf);
        ModuleDescription description;
        try {
            description = new ModuleDescription("FSvcscore.jar", mf);
        } catch (IllegalModuleException exc) {
            exc.printStackTrace();
            return null;
        }
        //System.out.println("description = "+description);
        final ManifestSection.FileSystemSection[] mfSection = new ManifestSection.FileSystemSection[1];
        description.forEachSection(new ManifestSection.Iterator() {

            /** Process action section.
             * @param as the section
             * @exception InstantiationException if there is an error and the section should be ignored
             */
            public void processAction (ManifestSection.ActionSection as) throws InstantiationException {
            }
            
            /** Process option section.
             * @param os the section
             * @exception InstantiationException if there is an error and the section should be ignored
             */
            public void processOption (ManifestSection.OptionSection os) throws InstantiationException {
            }
            
            /** Process loader section.
             * @param is the section
             * @exception InstantiationException if there is an error and the section should be ignored
             */
            public void processLoader (ManifestSection.LoaderSection ls) throws InstantiationException {
            }
            
            /** Process debugger section.
             * @param ds the section
             * @exception InstantiationException if there is an error and the section should be ignored
             */
            public void processDebugger (ManifestSection.DebuggerSection ds) throws InstantiationException {
            }
            
            /** Process service section.
             * @param es the section
             * @exception Instantiationception if there is an error and the section should be ignored
             */
            public void processService (ManifestSection.ServiceSection es) throws InstantiationException {
            }
            
            /** Process file system section.
             * @param fs the section
             * @exception InstantiationException if there is an error and the section should be ignored
             */
            public void processFileSystem (ManifestSection.FileSystemSection fs) throws InstantiationException {
                //System.out.println("processFileSystem("+fs+")");
                mfSection[0] = fs;
            }
            
            /** Process node section.
             * @param es the section
             * @exception InstantiationException if there is an error and the section should be ignored
             */
            public void processNode (ManifestSection.NodeSection es) throws InstantiationException {
            }
            
            /** Process clipboard convertor section.
             * @param ccs the section
             * @exception InstantiationException if there is an error and the section should be ignored
             */
            public void processClipboardConvertor (ManifestSection.ClipboardConvertorSection ccs) throws InstantiationException {
            }
            
        });
        //System.out.println("createFSSection() = "+mfSection[0]);
        section = mfSection[0];
        return mfSection[0];
    }

}
