/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.vcscore.revision;

import java.awt.event.ActionListener;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import java.util.*;

import org.openide.awt.JMenuPlus;
import org.openide.awt.JInlineMenu;
import org.openide.util.actions.*;
import org.openide.filesystems.*;
import org.openide.nodes.*;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;

import org.netbeans.modules.vcscore.VcsFileSystem;
import org.netbeans.modules.vcscore.VcsAction;
import org.netbeans.modules.vcscore.Variables;
import org.netbeans.modules.vcscore.commands.*;
import org.netbeans.modules.vcscore.util.VcsUtilities;
import org.netbeans.modules.vcscore.util.Table;
import org.netbeans.modules.vcscore.util.FileEditorSupport;
import org.netbeans.modules.vcscore.util.TopComponentCloseListener;
//import org.netbeans.modules.vcscore.cmdline.*;
//import org.netbeans.modules.vcscore.cmdline.exec.ExternalCommand;

/**
 *
 * @author  Martin Entlicher
 */
public class RevisionAction extends NodeAction implements ActionListener {

    //private VcsFileSystem fileSystem = null;
    //private FileObject fo = null;
    
    //private Hashtable additionalVars = new Hashtable();
    
    //private VcsCommand openRevisionCommand = null;

    /** Creates new RevisionAction 
     * Gets revision actions from filesystem and acts on a file object.
     * Both the filesystem and file object are obtained from the revision node.
     */
    public RevisionAction() {
    }
    
    /* Creates new RevisionAction 
     * Gets revision actions from filesystem and acts on the givn file object.
     * @param fileSystem the file system to get the actions from
     * @param fo the file object to act on
     *
    public RevisionAction(VcsFileSystem fileSystem, FileObject fo) {
        this.fileSystem = fileSystem;
        this.fo = fo;
    }
     */

    public String getName(){
        return g("CTL_Revision_Action"); // NOI18N
    }

    //-------------------------------------------
    public void performAction(Node[] nodes){
        //D.deb("performAction()"); // NOI18N
    }

    //-------------------------------------------
    public boolean enable(Node[] nodes){
        //D.deb("enable()"); // NOI18N
        return nodes.length > 0;
    }

    //-------------------------------------------
    public HelpCtx getHelpCtx(){
        //D.deb("getHelpCtx()"); // NOI18N
        return null;
    }

    protected JMenuItem createItem(VcsCommand cmd) {
        JMenuItem item = null;
        /*
        if (cmd == null) {
            //E.err("Command "+name+" not configured."); // NOI18N
            item = new JMenuItem("'"+name+"' not configured.");
            item.setEnabled(false);
            return item;
        }
        */
        //Hashtable vars = fileSystem.getVariablesAsHashtable();
        String label = cmd.getDisplayName();
        /*
        if (label.indexOf('$') >= 0) {
            label = Variables.expandFast(vars, label, true);
        }
         */
        item = new JMenuItem(label);
        String[] props = cmd.getPropertyNames();
        if (props != null && props.length > 0) {
            item.setActionCommand(cmd.getName());
            item.addActionListener(this);
        }
        return item;
    }

    /**
     * Add a popup submenu.
     */
    private void addMenu(Node commands, JMenu parent, int numSelected) {
        Children children = commands.getChildren();
        for (Enumeration subnodes = children.nodes(); subnodes.hasMoreElements(); ) {
            Node child = (Node) subnodes.nextElement();
            VcsCommand cmd = (VcsCommand) child.getCookie(VcsCommand.class);
            if (cmd == null) {
                parent.addSeparator();
                continue;
            }
            int numRev = VcsCommandIO.getIntegerPropertyAssumeZero(cmd, VcsCommand.PROPERTY_NUM_REVISIONS);
            if (numRev != numSelected
                || VcsCommandIO.getBooleanPropertyAssumeDefault(cmd, VcsCommand.PROPERTY_HIDDEN)
                || cmd.getDisplayName() == null) continue;
            if (!child.isLeaf()) {
                JMenu submenu;
                String[] props = cmd.getPropertyNames();
                if (props == null || props.length == 0) {
                    submenu = new JMenuPlus(cmd.getDisplayName());
                } else {
                    submenu = new JMenuPlus();
                }
                addMenu(child, submenu, numSelected);
                parent.add(submenu);
            } else {
                JMenuItem item = createItem(cmd);
                parent.add(item);
            }
        }
    }

    /**
     * Get a menu item that can present this action in a <code>JPopupMenu</code>.
     */
    public JMenuItem getPopupPresenter() {
        JInlineMenu inlineMenu = new JInlineMenu();
        ArrayList menuItems = new ArrayList();
        Node[] nodes = getActivatedNodes();
        VcsFileSystem fileSystem = null;
        for (int i = 0; i < nodes.length; i++) {
            if (nodes[i] instanceof RevisionNode) {
                VcsFileSystem nodeFS = (VcsFileSystem) ((RevisionNode) nodes[i]).getFileSystem();
                if (fileSystem == null) fileSystem = nodeFS;
                else if (fileSystem != nodeFS) return null;
            }
        }
        if (fileSystem == null) return null;
        Node commands = fileSystem.getCommands();
        Children children = commands.getChildren();
        Node[] commandRoots = children.getNodes();
        for (int i = 0; i < commandRoots.length; i++) {
            VcsCommand cmd = (VcsCommand) commandRoots[i].getCookie(VcsCommand.class);
            int numRev = 0;
            if (cmd != null && (numRev = VcsCommandIO.getIntegerPropertyAssumeZero(cmd, VcsCommand.PROPERTY_NUM_REVISIONS)) > 0
                && !VcsCommandIO.getBooleanPropertyAssumeDefault(cmd, VcsCommand.PROPERTY_HIDDEN)
                && cmd.getDisplayName() != null) {
                    
                if (numRev == nodes.length) {
                    JMenuItem menuItem = getPopupPresenter(commandRoots[i], cmd, nodes.length);
                    if (menuItem != null) menuItems.add(menuItem);
                }
            }
        }
        inlineMenu.setMenuItems((JMenuItem[]) menuItems.toArray(new JMenuItem[menuItems.size()]));
        return inlineMenu;
    }

    private JMenuItem getPopupPresenter(Node commandRoot, VcsCommand cmd, int numSelected) {
        String name = commandRoot.getDisplayName();
        JMenuItem menu = new JMenuPlus(name);
        addMenu(commandRoot, (JMenu) menu, numSelected);
        if (menu.getSubElements().length == 0) {
            menu = createItem(cmd);
        }
        return menu;
    }
    
    public void actionPerformed(final java.awt.event.ActionEvent e){
        //D.deb("actionPerformed("+e+")"); // NOI18N
        String cmd = e.getActionCommand();
        //D.deb("cmd="+cmd); // NOI18N
        Node[] nodes = getActivatedNodes();
        VcsFileSystem fileSystem = null;
        FileObject fo = null;
        if (nodes.length == 0) {
            //E.err("internal error nodes.length<1 TODO");
            return ;
        } else {
            for (int i = 0; i < nodes.length; i++) {
                if (nodes[i] instanceof RevisionNode) {
                    VcsFileSystem nodeFS = (VcsFileSystem) ((RevisionNode) nodes[i]).getFileSystem();
                    FileObject nodeFO = ((RevisionNode) nodes[i]).getFileObject();
                    if (fileSystem == null) fileSystem = nodeFS;
                    else if (fileSystem != nodeFS) return ;
                    if (fo == null) fo = nodeFO;
                    else if (fo != nodeFO) return ;
                }
            }
            if (fileSystem == null) return ;
        }
        Table files = new Table();
        String mimeType = fo.getMIMEType();
        String fileName = fo.getPackageNameExt('/','.');
        files.put(fileName, fo);
        Hashtable additionalVars = new Hashtable();
        if (mimeType != null) additionalVars.put("MIMETYPE", mimeType); // NOI18N
        if (nodes.length > 0) {
            additionalVars.put("REVISION", ((RevisionNode) nodes[0]).getItem().getRevision());
        }
        for(int i = 0; i < nodes.length; i++) {
            //D.deb("nodes["+i+"]="+nodes[i]); // NOI18N
            additionalVars.put("REVISION"+(i+1), ((RevisionNode) nodes[i]).getItem().getRevision());
        }
        additionalVars.put("BRANCH", getBranch(nodes));
        //D.deb("files="+files); // NOI18N

        //doCommand (files, cmd, fileSystem);
        VcsAction.doCommand(files, fileSystem.getCommand(cmd), additionalVars, fileSystem);
    }
    
    private String getBranch(Node[] nodes) {
        String branchTag = "";
        for (int i = 0; i < nodes.length; i++) {
            RevisionItem item = ((RevisionNode) nodes[i]).getItem();
            if (item != null && item.isBranch()) {
                String[] tags = item.getTagNames();
                if (tags.length > 0) branchTag = tags[0];
            }
        }
        return branchTag;
    }
    
    private static VcsCommand getOpenRevisionCommand(Node commands) {
        Children ch = commands.getChildren();
        Node[] subCommands = ch.getNodes();
        for(int i = 0; i < subCommands.length; i++) {
            VcsCommand cmd = (VcsCommand) subCommands[i].getCookie(VcsCommand.class);
            if (cmd != null && VcsCommand.NAME_REVISION_OPEN.equals(cmd.getName())) {
                return cmd;
                //openCmd = cmd;
                //break;
            }
        }
        for(int i = 0; i < subCommands.length; i++) {
            VcsCommand openCmd = getOpenRevisionCommand(subCommands[i]);
            if (openCmd != null) return openCmd;
        }
        return null;//openCmd;
    }
    
    public static void openAction(String revision, final VcsFileSystem fileSystem, FileObject fo) {
        //final VcsCommand cmd = fileSystem.getOpenRevisionCmd();
        final VcsCommand cmd = getOpenRevisionCommand(fileSystem.getCommands());
        if (cmd == null) return;
        final Table files = new Table();
        //Vector files = new Vector(1);
        final String mimeType = fo.getMIMEType();
        final String fileName = fo.getPackageNameExt('/','.');
        //files.add(fileName);
        files.put(fileName, fo);
        //final Vector ffiles = files;
        Hashtable additionalVars = new Hashtable();
        if (mimeType != null) additionalVars.put("MIMETYPE", mimeType); // NOI18N
        additionalVars.put("REVISION", revision);
        final String revisionFileName = fo.getName()+"."+revision+"."+fo.getExt();
        final java.io.File tmpDir = VcsUtilities.createTMP();
        final String tmpFile = tmpDir.getAbsolutePath()+java.io.File.separator+revisionFileName;
        additionalVars.put("TEMPFILE", tmpFile);
        VcsCommandExecutor[] vces = VcsAction.doCommand(files, cmd, additionalVars, fileSystem);
        if (vces.length > 0) {
            final VcsCommandExecutor vce = vces[0];
            //System.out.println("openAction(): vce = "+vce);
            CommandListener finishListener = new CommandListener() {
                public void commandStarted(VcsCommandExecutor ec) {
                }
                
                public void commandDone(VcsCommandExecutor ec) {
                    //System.out.println("ec = "+ec+", vce = "+vce+", equals = "+(ec == vce)+", status = "+ec.getExitStatus());
                    if (ec == vce && ec.getExitStatus() == VcsCommandExecutor.SUCCEEDED) {
                        openFile(tmpFile, mimeType, tmpDir);
                        fileSystem.getCommandsPool().removeCommandListener(this);
                    }
                }
            };
            fileSystem.getCommandsPool().addCommandListener(finishListener);
            //System.out.println("vce.isRunning = "+fileSystem.getCommandsPool().isRunning(vce));
            if (!fileSystem.getCommandsPool().isRunning(vce)) {
                openFile(tmpFile, mimeType, tmpDir);
            }
        }
    }
    
    private static void openFile(String tmpFile, String mimeType, java.io.File tmpDir) {
        java.io.File file = new java.io.File(tmpFile);
        //System.out.println("file = "+file+", exists = "+file.exists());
        final FileEditorSupport.FileEnvironment env = new FileEditorSupport.FileEnvironment(file, mimeType);
        final FileEditorSupport editor = new FileEditorSupport(file, env);
        editor.addCloseListener(new CloseListener(tmpDir));// {
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                editor.open();
            }
        });
    }
    
    private String g(String s) {
        return NbBundle.getBundle
               ("org.netbeans.modules.vcscore.revision.Bundle").getString (s);
    }
    
    private static final class CloseListener implements TopComponentCloseListener, java.io.Serializable {
        
        private java.io.File tmpDir;
        
        static final long serialVersionUID =1299562894534560601L;
        public CloseListener(java.io.File tmpDir) {
            this.tmpDir = tmpDir;
        }
        
        public void closing() {
            VcsUtilities.deleteRecursive(tmpDir);
        }
    }
}
