/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.vcscore.revision;

import javax.swing.event.*;
import java.util.*;

import org.openide.nodes.*;
import org.openide.util.*;

/**
 *
 * @author  Martin Entlicher
 */
public class RevisionChildren extends Children.Keys implements ChangeListener, java.io.Serializable {

    private String acceptField = "";
    private int numAcceptDots = 1;
    private RevisionList list = null;
    private RevisionNode parentNode = null;
    
    /** Creates new RevisionChildren */
    public RevisionChildren(RevisionList list) {
        list.addChangeListener(WeakListener.change (this, this));
        this.list = list;
        stateChanged (null);
    }
    
    RevisionList getList() {
        return list;
    }
    
    public void setNode(RevisionNode node) {
        parentNode = node;
    }
    
    private void setAcceptField(String acceptField) {
        this.acceptField = acceptField;
    }
    
    private void setNumAcceptDots(int dots) {
        numAcceptDots = dots;
    }
    
    private boolean accept(RevisionItem item) {
        String rev = item.getRevision();
        if (rev.length() <= acceptField.length()) return false;
        //System.out.println("accept["+acceptField+", "+numAcceptDots+"]("+item.getRevision()+") = "+(rev.indexOf(acceptField) == 0 && RevisionItem.numDots(rev) == numAcceptDots));
        return (rev.regionMatches(0, acceptField, 0, acceptField.length())
                && (acceptField.length() == 0 || rev.charAt(acceptField.length()) == '.')
                && RevisionItem.numDots(rev) == numAcceptDots);
    }
    
    protected Node[] createNodes(Object key) {
        Node[] nodes = new Node[0]; //new Node[] { Node.EMPTY };
        RevisionItem item = (RevisionItem) key;
        //System.out.println("createNodes("+item.getRevision()+")");
        if (accept(item)) {
            //System.out.println("isRevision = "+(item.isRevision() && !item.isBranch()));
            //if (item.isRevision() && !item.isBranch()) {
            if (!list.containsSubRevisions(item.getRevision()) && !item.isBranch()) {
                RevisionNode newNode = parentNode.createInstance(list, item);
                nodes = new Node[] { newNode };
            } else {
                RevisionChildren children = new RevisionChildren(list);
                children.setNode(parentNode);
                children.setAcceptField(item.getRevision());
                children.setNumAcceptDots(numAcceptDots + 1);
                RevisionNode node = parentNode.createInstance(children);
                node.setName(item.getDisplayName());
                node.setItem(item);
                nodes = new Node[1]; // { node };
                nodes[0] = node;
                //list.fireChange();
            }
        }
        //if (nodes.length > 0) System.out.println("return node = "+nodes[0]);
        return nodes;
    }
    
    public void stateChanged(ChangeEvent evt) {
        //System.out.println("RevisionChildren.stateChanged("+evt+")");
        setKeys(list /*(Collection) evt.getSource()*/);
    }

}