/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.vcscore.revision;

import java.util.*;
import javax.swing.event.*;

import org.openide.filesystems.FileObject;

import org.openide.filesystems.FileSystem;

/**
 *
 * @author  Martin Entlicher
 * @version 
 * RevisionList is a sorted set of RevisionItem objects.
 */
public class RevisionList extends TreeSet {

    private FileSystem fileSystem = null;
    private FileObject fo = null; // The current File Object
    private Vector listeners;

    static final long serialVersionUID = -8578787400541124223L;
    
    /** Creates new RevisionList */
    public RevisionList() {
        listeners = new Vector();
    }
    
    public void setFileSystem(FileSystem fileSystem) {
        this.fileSystem = fileSystem;
    }
    
    public FileSystem getFileSystem() {
        return fileSystem;
    }
    
    public void setFileObject(FileObject fo) {
        this.fo = fo;
    }
    
    public FileObject getFileObject() {
        return fo;
    }
    
    public boolean add(Object obj) {
        boolean status = super.add(obj);
        //System.out.println("RevisionList.add("+((RevisionItem) obj).getRevision()+")");
        fireChanged();
        return status;
    }
    
    public boolean addAll(Collection c) {
        boolean status = super.addAll(c);
        //System.out.println("RevisionList.addAll("+c+"): c.size() = "+c.size());
        fireChanged();
        return status;
    }
    
    public boolean remove(Object obj) {
        boolean status = super.remove(obj);
        //System.out.println("RevisionList.remove("+((RevisionItem) obj).getRevision()+")");
        fireChanged();
        return status;
    }
    
    public boolean removeAll(Collection c) {
        boolean status = super.removeAll(c);
        //System.out.println("RevisionList.removeAll("+c+"): c.size() = "+c.size());
        fireChanged();
        return status;
    }
    
    public boolean removeRevision(String revision) {
        Iterator it = this.iterator();
        while(it.hasNext()) {
            RevisionItem rev = (RevisionItem) it.next();
            String revision2 = rev.getRevision();
            if (revision2.equals(revision)) {
                this.remove(rev);
                return true;
            }
        }
        return false;
    }
    
    public void fireChanged() {
        //System.out.println("RevisionList.fireChange()");
        for(Enumeration enum = listeners.elements(); enum.hasMoreElements(); ) {
            ChangeListener listener = (ChangeListener) enum.nextElement();
            listener.stateChanged(new ChangeEvent(this));
        }
    }
    
    public boolean containsSubRevisions(String revision) {
        Iterator it = this.iterator();
        while(it.hasNext()) {
            RevisionItem rev = (RevisionItem) it.next();
            String revision2 = rev.getRevision();
            if (revision2.length() > revision.length()
                && revision2.regionMatches(0, revision, 0, revision.length())
                && revision2.charAt(revision.length()) == '.') return true;
        }
        return false;
        /*
        return Collections.binarySearch(new ArrayList(this), new RevisionItem(revision), new SubRevisionComparator(revision));
        */
    }

    public void addChangeListener(ChangeListener listener) {
        listeners.add(listener);
    }
    
    public boolean removeChangeListener(ChangeListener listener) {
        return listeners.remove(listener);
    }
    
    /*
    private class SubRevisionComparator extends Object implements Comparator {
        
        private int length;
        
        public SubRevisionComparator(int length) {
            this.length = length;
        }
        
        public boolean equals(Object obj) {
            return 
        }
        
    }
    */
}