/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.vcscore.revision;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.ArrayList;
import java.util.Collections;

import org.openide.nodes.*;
import org.openide.actions.*;
import org.openide.filesystems.FileSystem;
import org.openide.filesystems.FileObject;
import org.openide.util.actions.SystemAction;
import org.openide.cookies.OpenCookie;

import org.netbeans.modules.vcscore.util.VcsUtilities;
import org.netbeans.modules.vcscore.VcsFileSystem;

/**
 *
 * @author  Martin Entlicher
 */
public class RevisionNode extends AbstractNode implements OpenCookie, java.io.Serializable {

    private static final String ICON_BRANCH = "/org/netbeans/modules/vcscore/revision/branchIcon";
    //private static final String ICON_OPEN_BRANCH = "/org/openide/resources/defaultFolderOpen.gif";
    private static final String ICON_REVISION = "/org/netbeans/modules/vcscore/revision/revisionIcon";
    private static final String ICON_REVISION_CURRENT = "/org/netbeans/modules/vcscore/revision/revisionCurrentIcon";

    private String fileName = "Class.java";
    private RevisionItem item = null;
    private RevisionList list = null;
    private boolean fileNode = false;
    
    /** Creates new RevisionNode */
    public RevisionNode(RevisionList list) {
        super(new RevisionChildren(list));
        ((RevisionChildren) this.getChildren()).setNode(this);
        init(list, null);
    }
    
    
    public RevisionNode(RevisionChildren children) {
        super(children);
        //children.setNode(this);
        init(children.getList(), null);
    }
    
    public RevisionNode(RevisionList list, RevisionItem item) {
        super(Children.LEAF);
        setName(item.getRevisionVCS());
        init(list, item);
    }
    
    private void init(RevisionList list, RevisionItem item) {
        this.list = list;
        this.item = item;
        addCookies();
        setIcon();
    }
    
    public RevisionNode createInstance(RevisionList list, RevisionItem item) {
        return new RevisionNode(list, item);
    }

    public RevisionNode createInstance(RevisionChildren children) {
        return new RevisionNode(children);
    }
    
    private void addCookies() {
        if (!isFileNode()) {
            if (item != null && !item.isBranch()) {
                getCookieSet().add(this);
            }
        }
    }
    
    private void setIcon() {
        if (!isFileNode()) {
            if (item != null && !item.isBranch()) {
                //System.out.println("setIconBase("+getName()+", "+getDisplayName()+") = "+ICON_REVISION);
                if (item.isCurrent()) {
                    setIconBase(ICON_REVISION_CURRENT);
                } else {
                    setIconBase(ICON_REVISION);
                }
            } else {
                //System.out.println("setIconBase("+getName()+", "+getDisplayName()+") = "+ICON_BRANCH);
                setIconBase(ICON_BRANCH);
            }
        } else {
            //System.out.println("setIconBase("+getName()+", "+getDisplayName()+") = "+ICON_BRANCH);
            setIconBase(ICON_BRANCH);
        }
        //return super.getIcon(type);
    }
    
    public void refreshIcons() {
        setIcon();
        fireIconChange();
        for (Enumeration enum = getChildren().nodes(); enum.hasMoreElements(); ) {
            ((RevisionNode) enum.nextElement()).refreshIcons();
        }
    }
    

    
    public void setItem(RevisionItem item) {
        this.item = item;
        addCookies();
        setIcon();
    }
    
    public RevisionItem getItem() {
        return item;
    }
    
    public void setFileNode(boolean fileNode) {
        this.fileNode = fileNode;
        setIcon();
    }
    
    protected boolean isFileNode() {
        //return (getParentNode() == null);
        return fileNode;//(item == null);
    }
    /*
    public String getFileName() {
        return this.fileName;
    }
    
    public void setFileName(String fileName) {
        this.fileName = fileName;
    }
     */

    public String getType() {
        if (item == null || item.isBranch()) return org.openide.util.NbBundle.getBundle(org.netbeans.modules.vcscore.revision.RevisionNode.class).getString("CTL_TypeBranch");
        else return org.openide.util.NbBundle.getBundle(org.netbeans.modules.vcscore.revision.RevisionNode.class).getString("CTL_TypeRevision");
    }
    
    public boolean canCopy() {
        return false;
    }
    
    public boolean canCut() {
        return false;
    }
    
    public boolean canDestroy() {
        return false;
    }
        
    public boolean canRename() {
        return false;
    }
    
    public Sheet createSheet() {
        Sheet sheet = Sheet.createDefault();
        Sheet.Set set = sheet.get(Sheet.PROPERTIES);
        if (item == null || isFileNode()) set.put(new PropertySupport.Name(this));
        if (item != null) createProperties(item, set);
        return sheet;
    }
    
    private void createProperties(final RevisionItem item, final Sheet.Set set) {
        if (item.getRevision() != null)
            set.put(new PropertySupport.ReadOnly(
                        "revision", String.class,
                        org.openide.util.NbBundle.getBundle(RevisionNode.class).getString("MSG_RevisionNumber"), ""
                        ) {
                            public Object getValue() {
                                return item.getRevisionVCS();
                            }
                    });
        if (item.getMessage() != null)
            set.put(new PropertySupport.ReadOnly(
                        "message", String.class,
                        org.openide.util.NbBundle.getBundle(RevisionNode.class).getString("MSG_Message"), ""
                        ) {
                            public Object getValue() {
                                return item.getMessage();
                            }
                    });
        if (item.getDate() != null)
            set.put(new PropertySupport.ReadOnly(
                        "date", String.class,
                        org.openide.util.NbBundle.getBundle(RevisionNode.class).getString("MSG_Date"), ""
                        ) {
                            public Object getValue() {
                                return item.getDate();
                            }
                    });
        if (item.getAuthor() != null)
            set.put(new PropertySupport.ReadOnly(
                        "author", String.class,
                        org.openide.util.NbBundle.getBundle(RevisionNode.class).getString("MSG_Author"), ""
                        ) {
                            public Object getValue() {
                                return item.getAuthor();
                            }
                    });
        Hashtable additional = item.getAdditionalProperties();
        for(Enumeration enum = additional.keys(); enum.hasMoreElements(); ) {
            final String name = (String) enum.nextElement();
            final String value = (String) additional.get(name);
            set.put(new PropertySupport.ReadOnly(
                        "additional_"+name, String.class, name, ""
                        ) {
                            public Object getValue() {
                                return value;
                            }
                    });
        }
        final String[] tags = item.getTagNames();
        if (tags != null && tags.length > 0) {
            set.put(new PropertySupport.ReadOnly(
                        "tag", String.class,
                        (tags.length > 1) ? org.openide.util.NbBundle.getBundle(RevisionNode.class).getString("MSG_TagNames")
                                          : org.openide.util.NbBundle.getBundle(RevisionNode.class).getString("MSG_TagName"), ""
                        ) {
                            public Object getValue() {
                                String tag = VcsUtilities.arrayToString(tags);
                                return tag.substring(1, tag.length() - 1);
                            }
                    });
        }
    }
    
    public FileSystem getFileSystem() {
        return list.getFileSystem();        
    }
    
    public FileObject getFileObject() {
        return list.getFileObject();        
    }
    
    public SystemAction [] getActions() {
        ArrayList actions = new ArrayList();
        VcsFileSystem fs = (VcsFileSystem)getFileSystem();
        if (isFileNode()) {
            SystemAction[] foActions = fs.getActions(Collections.singleton(list.getFileObject()));
            if (foActions != null) {
                for(int i = 0; i < foActions.length; i++) actions.add(foActions[i]);
            }
            actions.add(null);
            actions.add(SystemAction.get(PropertiesAction.class));
        } else {
            actions.add(SystemAction.get(OpenAction.class));
            actions.add(null);
            actions.add(SystemAction.get(org.netbeans.modules.vcscore.revision.RevisionAction.class)); //new RevisionAction(fs, list.getFileObject()));
            actions.add(null);
            actions.add(SystemAction.get(PropertiesAction.class));
        }
        SystemAction[] array = new SystemAction [actions.size()];
        actions.toArray(array);
        return array;
    }
    
    public void open() {
        VcsFileSystem fs = (VcsFileSystem)getFileSystem();
        //RevisionAction action = new RevisionAction(fs, getFileObject());
        RevisionAction.openAction(item.getRevisionVCS(), fs, getFileObject());
    }
    
    private String g(String resource) {
        return org.openide.util.NbBundle.getBundle(org.netbeans.modules.vcscore.revision.RevisionNode.class).getString(resource);
    }
    
}
