/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.vcscore.settings;

import org.openide.options.SystemOption;
import org.openide.util.NbBundle;

/**
 * The settings for all VCS filesystems.
 * @author  Milos Kleint, Martin Entlicher
 */
public class VcsSettings extends SystemOption {

    public static final String PROP_USE_GLOBAL         = "useGlobal"; // NOI18N
    public static final String PROP_OFFLINE            = "offLine"; // NOI18N
    public static final String PROP_AUTO_REFRESH       = "autoRefresh"; // NOI18N
    public static final String PROP_HOME               = "home"; // NOI18N
    
    public static final int AUTO_REFRESH_NO_REFRESH = 0;
    public static final int AUTO_REFRESH_ON_DIR_OPEN = 1;
    public static final int AUTO_REFRESH_ON_MOUNT = 2;
    public static final int AUTO_REFRESH_ON_RESTART = 3;
    public static final int AUTO_REFRESH_ON_MOUNT_AND_RESTART = 4;

    private static boolean useGlobal = true;

    private static boolean offLine = false;

    private static int autoRefresh = AUTO_REFRESH_ON_DIR_OPEN;

    static final long serialVersionUID = 2366734217889995634L;
    /** Creates new VcsSettings */
    public VcsSettings() {
    }

    /** Get human presentable name */
    public String displayName() {
        return NbBundle.getBundle(VcsSettings.class).getString("CTL_VcsSettings");
    }
    
    /** Whether these settings overide the filesystem settings.
     */
    public boolean isUseGlobal() {
        return useGlobal;
    }

    /** Set whether these settings overide the filesystem settings.
     */
    public void setUseGlobal(boolean global) {
        if (useGlobal != global) {
           useGlobal = global;
           firePropertyChange(PROP_USE_GLOBAL, new Boolean(!global), new Boolean(global));
        }
   }
    
    /** Getter for property offLine.
     * @return Value of property offLine.
     */
    public boolean isOffLine() {
        return offLine;
    }
    
    /** Setter for property offLine.
     * @param offLine New value of property offLine.
     */
    public void setOffLine(boolean newOffLine) {
        if (offLine != newOffLine) {
            boolean oldOffLine = offLine;
            offLine = newOffLine;
            firePropertyChange (PROP_OFFLINE, new Boolean (oldOffLine), new Boolean (offLine));
        }
    }
    
    /** Getter for property autoRefresh.
     * @return Value of property autoRefresh.
     */
    public int getAutoRefresh() {
        return autoRefresh;
    }
    
    /** Setter for property autoRefresh.
     * @param autoRefresh New value of property autoRefresh.
     */
    public void setAutoRefresh(int newAutoRefresh) {
        if (autoRefresh != newAutoRefresh) {
            int oldAutoRefresh = autoRefresh;
            autoRefresh = newAutoRefresh;
            firePropertyChange (PROP_AUTO_REFRESH, new Integer (oldAutoRefresh), new Integer (newAutoRefresh));
        }
    }
    
    public String getHome() {
        String home = System.getProperty("Env-HOME");
        if (home == null && org.openide.util.Utilities.isWindows()) {
            String homeDrive = System.getProperty("Env-HOMEDRIVE");
            String homeDir = System.getProperty("Env-HOMEPATH");
            if (homeDrive != null && homeDir != null) {
                home = homeDrive + homeDir;
            }
        }
        return home;
    }
    
    public void setHome(String home) {
        if (home == null) return ;
        System.setProperty("Env-HOME", home);
        System.setProperty("env-home", home.toLowerCase());
        if (org.openide.util.Utilities.isWindows()) {
            int index = home.indexOf(':');
            if (index > 0) {
                String homeDrive = home.substring(0, index + 1);
                String homeDir = home.substring(index + 1);
                System.setProperty("Env-HOMEDRIVE", homeDrive);
                System.setProperty("env-homedrive", homeDrive.toLowerCase());
                System.setProperty("Env-HOMEPATH", homeDir);
                System.setProperty("env-homepath", homeDir.toLowerCase());
            }
        }
        firePropertyChange(PROP_HOME, null, home);
    }
    
}
