/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.vcscore.wizard.mountcvs;

import org.openide.util.NbBundle;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;
import javax.swing.ButtonGroup;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import org.netbeans.modules.vcscore.wizard.mountcvs.util.CvsHelper;
import org.netbeans.modules.vcscore.wizard.mountcvs.util.Utilities;

/**
 *
 * @author  Martin Entlicher
 */
public class CMPanel extends AbstractWizardPanel implements DocumentListener, ActionListener {
    
    private CvsHelper helper_;

    /** Creates new form CMPanel */
    public CMPanel() {
        initComponents();
        this.jTextArea1.setFont (javax.swing.UIManager.getFont ("Label.font"));
        this.jTextArea1.setDisabledTextColor (javax.swing.UIManager.getColor ("Label.foreground"));
        setName(org.openide.util.NbBundle.getBundle(CMPanel.class).getString("CMPanel.title"));
        this.postInitComponents ();
        this.putClientProperty (CvsWizardData.SELECTED_INDEX, new Integer(2));
    }
    
    public void readCvsWizardSettings (CvsWizardData data) {
        try {
        // Already filled by user use it
        helper_ = data.getHelper ();
        if (CvsHelper.LOCAL.equals(helper_.getMethod())) {
            // The local selected
            this.localRadioButton.setSelected (true);
            postSelectedLocal ();
        }
        else {
            if (CvsHelper.SERVER.equals (helper_.getMethod())) {
                // Server selected
                this.serverRadioButton.setSelected (true);
                this.cvsRshLabel.setEnabled (false);
                this.cvsRshTextField.setEnabled (false);
            }
            else if (CvsHelper.EXTERNAL.equals(helper_.getMethod())) {
                // The external selected
                this.extRadioButton.setSelected (true);
                this.cvsRshLabel.setEnabled (true);
                this.cvsRshTextField.setEnabled (true);
                String rsh = System.getProperty ("env-cvs_rsh"); // No I18N
                if (rsh != null)
                    this.cvsRshTextField.setText (rsh);
            }
            else if (CvsHelper.PSERVER.equals(helper_.getMethod())) {
                this.cvsRshLabel.setEnabled (false);
                this.pserverRadioButton.setSelected (true);
                this.cvsRshTextField.setEnabled (false);
            }
            else {
                // If empty method or unsupported, convert to pserver
                if (!"".equals(helper_.getMethod()))
                    helper_.setCvsRoot (null);
                helper_.setMethod (CvsHelper.PSERVER);
                this.cvsRshLabel.setEnabled (false);
                this.pserverRadioButton.setSelected (true);
                this.cvsRshTextField.setEnabled (false);
            }
            this.serverTextField.setText (this.helper_.getServerName());
            this.userNameTextField.setText (this.helper_.getUserName());
            this.cvsRootTextField.setText (this.helper_.getCvsRoot());
            postSelectedServer();
        }
        this.repositoryTextField.setText (this.helper_.getRepository());
        }catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    public void storeCvsWizardSettings (CvsWizardData data) {
        data.setRsh (this.cvsRshTextField.getText());
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        buttonGroup = new javax.swing.ButtonGroup();
        connectionPanel = new javax.swing.JPanel();
        methodPanel = new javax.swing.JPanel();
        cmjLabel = new javax.swing.JLabel();
        localRadioButton = new javax.swing.JRadioButton();
        serverRadioButton = new javax.swing.JRadioButton();
        pserverRadioButton = new javax.swing.JRadioButton();
        extRadioButton = new javax.swing.JRadioButton();
        cvsRshLabel = new javax.swing.JLabel();
        cvsRshTextField = new javax.swing.JTextField();
        jSeparator1 = new javax.swing.JSeparator();
        repositoryPanel = new javax.swing.JPanel();
        serverLabel = new javax.swing.JLabel();
        serverTextField = new javax.swing.JTextField();
        userNameLabel = new javax.swing.JLabel();
        userNameTextField = new javax.swing.JTextField();
        repositoryLabel = new javax.swing.JLabel();
        repositoryTextField = new javax.swing.JTextField();
        repositoryButton = new javax.swing.JButton();
        cvsRootLabel = new javax.swing.JLabel();
        cvsRootTextField = new javax.swing.JTextField();
        jTextArea1 = new javax.swing.JTextArea();
        
        buttonGroup.add(localRadioButton);
        buttonGroup.add(serverRadioButton);
        buttonGroup.add(pserverRadioButton);
        buttonGroup.add(extRadioButton);
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        connectionPanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints2;
        
        methodPanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints3;
        
        cmjLabel.setText(org.openide.util.NbBundle.getBundle(CMPanel.class).getString("CMPanel.cmLabel"));
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.gridwidth = 4;
        gridBagConstraints3.anchor = java.awt.GridBagConstraints.WEST;
        methodPanel.add(cmjLabel, gridBagConstraints3);
        
        localRadioButton.setMnemonic(org.openide.util.NbBundle.getBundle(CMPanel.class).getString("CMPanel.localRadioButton_Mnemonic").charAt(0));
        localRadioButton.setText(org.openide.util.NbBundle.getBundle(CMPanel.class).getString("CMPanel.localRadioButton"));
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.gridy = 1;
        gridBagConstraints3.anchor = java.awt.GridBagConstraints.WEST;
        methodPanel.add(localRadioButton, gridBagConstraints3);
        
        serverRadioButton.setMnemonic(org.openide.util.NbBundle.getBundle(CMPanel.class).getString("CMPanel.serverRadioButton_Mnemonic").charAt(0));
        serverRadioButton.setText(org.openide.util.NbBundle.getBundle(CMPanel.class).getString("CMPanel.serverRadioButton"));
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.gridy = 2;
        gridBagConstraints3.anchor = java.awt.GridBagConstraints.WEST;
        methodPanel.add(serverRadioButton, gridBagConstraints3);
        
        pserverRadioButton.setMnemonic(org.openide.util.NbBundle.getBundle(CMPanel.class).getString("CMPanel.pserverRadioButton_Mnemonic").charAt(0));
        pserverRadioButton.setSelected(true);
        pserverRadioButton.setText(org.openide.util.NbBundle.getBundle(CMPanel.class).getString("CMPanel.pserverRadioButton"));
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.gridy = 3;
        gridBagConstraints3.anchor = java.awt.GridBagConstraints.WEST;
        methodPanel.add(pserverRadioButton, gridBagConstraints3);
        
        extRadioButton.setMnemonic(org.openide.util.NbBundle.getBundle(CMPanel.class).getString("CMPanel.extRadioButton_Mnemonic").charAt(0));
        extRadioButton.setText(org.openide.util.NbBundle.getBundle(CMPanel.class).getString("CMPanel.extRadioButton"));
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.gridy = 4;
        gridBagConstraints3.insets = new java.awt.Insets(0, 0, 8, 0);
        gridBagConstraints3.anchor = java.awt.GridBagConstraints.WEST;
        methodPanel.add(extRadioButton, gridBagConstraints3);
        
        cvsRshLabel.setText(org.openide.util.NbBundle.getBundle(CMPanel.class).getString("CMPanel.cvsRshLabel"));
        cvsRshLabel.setLabelFor(cvsRshTextField);
        cvsRshLabel.setDisplayedMnemonic(org.openide.util.NbBundle.getBundle(CMPanel.class).getString("CMPanel.cvsRshLabel_Mnemonic").charAt(0));
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.gridy = 4;
        gridBagConstraints3.gridwidth = 2;
        gridBagConstraints3.insets = new java.awt.Insets(0, 0, 8, 0);
        gridBagConstraints3.anchor = java.awt.GridBagConstraints.EAST;
        methodPanel.add(cvsRshLabel, gridBagConstraints3);
        
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.gridy = 4;
        gridBagConstraints3.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints3.insets = new java.awt.Insets(0, 12, 8, 0);
        gridBagConstraints3.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints3.weightx = 1.0;
        methodPanel.add(cvsRshTextField, gridBagConstraints3);
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.weightx = 1.0;
        connectionPanel.add(methodPanel, gridBagConstraints2);
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridy = 1;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.insets = new java.awt.Insets(4, 0, 4, 0);
        gridBagConstraints2.weightx = 1.0;
        connectionPanel.add(jSeparator1, gridBagConstraints2);
        
        repositoryPanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints4;
        
        serverLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcscore/wizard/mountcvs/Bundle").getString("CMPanel.serverLabel"));
        serverLabel.setLabelFor(serverTextField);
        serverLabel.setDisplayedMnemonic(org.openide.util.NbBundle.getBundle(CMPanel.class).getString("CMPanel.serverLabel_Mnemonic").charAt(0));
        gridBagConstraints4 = new java.awt.GridBagConstraints();
        gridBagConstraints4.gridy = 0;
        gridBagConstraints4.insets = new java.awt.Insets(8, 0, 0, 0);
        gridBagConstraints4.anchor = java.awt.GridBagConstraints.WEST;
        repositoryPanel.add(serverLabel, gridBagConstraints4);
        
        gridBagConstraints4 = new java.awt.GridBagConstraints();
        gridBagConstraints4.gridy = 0;
        gridBagConstraints4.gridwidth = 2;
        gridBagConstraints4.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints4.insets = new java.awt.Insets(8, 12, 0, 0);
        gridBagConstraints4.weightx = 1.0;
        repositoryPanel.add(serverTextField, gridBagConstraints4);
        
        userNameLabel.setText(org.openide.util.NbBundle.getBundle(CMPanel.class).getString("CMPanel.userNameLabel"));
        userNameLabel.setLabelFor(userNameTextField);
        userNameLabel.setDisplayedMnemonic(org.openide.util.NbBundle.getBundle(CMPanel.class).getString("CMPanel.userNameLabel_Mnemonic").charAt(0));
        gridBagConstraints4 = new java.awt.GridBagConstraints();
        gridBagConstraints4.gridy = 1;
        gridBagConstraints4.insets = new java.awt.Insets(8, 0, 0, 0);
        gridBagConstraints4.anchor = java.awt.GridBagConstraints.WEST;
        repositoryPanel.add(userNameLabel, gridBagConstraints4);
        
        gridBagConstraints4 = new java.awt.GridBagConstraints();
        gridBagConstraints4.gridy = 1;
        gridBagConstraints4.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints4.insets = new java.awt.Insets(8, 12, 0, 0);
        gridBagConstraints4.weightx = 1.0;
        repositoryPanel.add(userNameTextField, gridBagConstraints4);
        
        repositoryLabel.setText(org.openide.util.NbBundle.getBundle(CMPanel.class).getString("CMPanel.repositoryLabel"));
        repositoryLabel.setLabelFor(repositoryTextField);
        repositoryLabel.setDisplayedMnemonic(org.openide.util.NbBundle.getBundle(CMPanel.class).getString("CMPanel.repositoryLabel_Mnemonic").charAt(0));
        gridBagConstraints4 = new java.awt.GridBagConstraints();
        gridBagConstraints4.gridy = 2;
        gridBagConstraints4.insets = new java.awt.Insets(8, 0, 8, 0);
        gridBagConstraints4.anchor = java.awt.GridBagConstraints.WEST;
        repositoryPanel.add(repositoryLabel, gridBagConstraints4);
        
        gridBagConstraints4 = new java.awt.GridBagConstraints();
        gridBagConstraints4.gridy = 2;
        gridBagConstraints4.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints4.insets = new java.awt.Insets(8, 12, 8, 0);
        gridBagConstraints4.weightx = 1.0;
        repositoryPanel.add(repositoryTextField, gridBagConstraints4);
        
        repositoryButton.setMnemonic(org.openide.util.NbBundle.getBundle(CMPanel.class).getString("CMPanel.repositoryButton_Mnemonic").charAt(0));
        repositoryButton.setText(org.openide.util.NbBundle.getBundle(CMPanel.class).getString("CMPanel.repositoryButton"));
        repositoryButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                repositoryButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints4 = new java.awt.GridBagConstraints();
        gridBagConstraints4.gridy = 2;
        gridBagConstraints4.insets = new java.awt.Insets(3, 5, 3, 0);
        gridBagConstraints4.anchor = java.awt.GridBagConstraints.WEST;
        repositoryPanel.add(repositoryButton, gridBagConstraints4);
        
        cvsRootLabel.setText(org.openide.util.NbBundle.getBundle(CMPanel.class).getString("CMPanel.cvsRootLabel"));
        gridBagConstraints4 = new java.awt.GridBagConstraints();
        gridBagConstraints4.gridy = 3;
        gridBagConstraints4.anchor = java.awt.GridBagConstraints.WEST;
        repositoryPanel.add(cvsRootLabel, gridBagConstraints4);
        
        cvsRootTextField.setEditable(false);
        gridBagConstraints4 = new java.awt.GridBagConstraints();
        gridBagConstraints4.gridy = 3;
        gridBagConstraints4.gridwidth = 2;
        gridBagConstraints4.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints4.insets = new java.awt.Insets(0, 12, 0, 0);
        gridBagConstraints4.weightx = 1.0;
        repositoryPanel.add(cvsRootTextField, gridBagConstraints4);
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridy = 2;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.weightx = 1.0;
        connectionPanel.add(repositoryPanel, gridBagConstraints2);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTH;
        gridBagConstraints1.weightx = 1.0;
        add(connectionPanel, gridBagConstraints1);
        
        jTextArea1.setWrapStyleWord(true);
        jTextArea1.setLineWrap(true);
        jTextArea1.setEditable(false);
        jTextArea1.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcscore/wizard/mountcvs/Bundle").getString("TIP_ConnectionMethodPanel"));
        jTextArea1.setDisabledTextColor(java.awt.Color.black);
        jTextArea1.setEnabled(false);
        jTextArea1.setOpaque(false);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(17, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(jTextArea1, gridBagConstraints1);
        
    }//GEN-END:initComponents

  private void repositoryButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_repositoryButtonActionPerformed
// Add your handling code here:
    JFileChooser fd = new JFileChooser ();
    fd.setFileFilter ( new FileFilter () {
        public boolean accept (java.io.File file) {
            if (file.exists() && file.isDirectory())
                return true;
            else
                return false;
        }
        
        public String getDescription () {
            return NbBundle.getBundle(CMPanel.class).getString("TXT_Directory");
        }
    });
    fd.setName (NbBundle.getBundle(CMPanel.class).getString("TXT_RepositoryDir"));
    fd.setFileSelectionMode (JFileChooser.DIRECTORIES_ONLY);
    java.io.File dir = Utilities.findDirectory (this.repositoryTextField.getText());
        if (dir != null)
            fd.setCurrentDirectory (dir);
    if (fd.showOpenDialog (this) == JFileChooser.APPROVE_OPTION) {
        this.repositoryTextField.setText (fd.getSelectedFile().getAbsolutePath());
    }
  }//GEN-LAST:event_repositoryButtonActionPerformed

    
    public org.openide.util.HelpCtx getHelp() {
        return org.openide.util.HelpCtx.DEFAULT_HELP;
    }
    
    public boolean isValid() {
        if (this.localRadioButton.isSelected()) {
            return this.repositoryTextField.getText().length()>0;
        }
        else {
            return this.repositoryTextField.getText().length()>0 &&
                   this.userNameTextField.getText().length()>0 &&
                   this.serverTextField.getText().length()>0;
        }
    }
    
    public void removeUpdate(javax.swing.event.DocumentEvent event) {
        processChange (event);
    }
    
    public void changedUpdate(javax.swing.event.DocumentEvent event) {
        processChange (event);
    }
    
    public void insertUpdate(javax.swing.event.DocumentEvent event) {
        processChange (event);
    }
    
    public void actionPerformed (ActionEvent event) {
        try {
        if (event.getSource() == this.localRadioButton) {
            this.helper_.setMethod (CvsHelper.LOCAL);
            this.postSelectedLocal();
            this.cvsRootTextField.setText (this.helper_.getCvsRoot());
            this.cvsRshLabel.setEnabled (false);
            this.cvsRshTextField.setEnabled (false);
        }
        else if (event.getSource() == this.serverRadioButton) {
            this.helper_.setMethod (CvsHelper.SERVER);
            this.postSelectedServer();
            this.cvsRootTextField.setText (this.helper_.getCvsRoot());
            this.cvsRshLabel.setEnabled (false);
            this.cvsRshTextField.setEnabled (false);
        }
        else if (event.getSource() == this.pserverRadioButton) { 
            this.helper_.setMethod (CvsHelper.PSERVER);
            this.postSelectedServer();
            this.cvsRootTextField.setText (this.helper_.getCvsRoot());
            this.cvsRshLabel.setEnabled (false);
            this.cvsRshTextField.setEnabled (false);
        }
        else if (event.getSource() == this.extRadioButton) {
            this.helper_.setMethod (CvsHelper.EXTERNAL);
            this.postSelectedServer();
            this.cvsRootTextField.setText (this.helper_.getCvsRoot());
            this.cvsRshLabel.setEnabled (true);
            this.cvsRshTextField.setEnabled (true);
        }
        fireChange();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    private void postInitComponents () {
        this.localRadioButton.addActionListener (this);
        this.serverRadioButton.addActionListener (this);
        this.pserverRadioButton.addActionListener (this);
        this.extRadioButton.addActionListener (this);
        this.cvsRshTextField.getDocument().addDocumentListener(this);
        this.serverTextField.getDocument().addDocumentListener (this);
        this.userNameTextField.getDocument().addDocumentListener (this);
        this.repositoryTextField.getDocument().addDocumentListener (this);
    }
    
    private void postSelectedLocal () {
        this.serverLabel.setEnabled (false);
        this.serverTextField.setEnabled (false);
        this.userNameLabel.setEnabled (false);
        this.userNameTextField.setEnabled (false);
        this.cvsRshLabel.setEnabled (false);
        this.cvsRshTextField.setEnabled (false);
        this.repositoryButton.setEnabled (true);
    }
    
    
    private void postSelectedServer () {
        this.serverLabel.setEnabled (true);
        this.serverTextField.setEnabled (true);
        this.userNameLabel.setEnabled (true);
        this.userNameTextField.setEnabled (true);
        this.cvsRshLabel.setEnabled (true);
        if (!"localhost".equals(this.serverTextField.getText()))
            this.repositoryButton.setEnabled (false);
    }
    
    private void processChange (DocumentEvent event) {
        if (event.getDocument() == this.serverTextField.getDocument()) {
            if (!"localhost".equals(this.serverTextField.getText()))
                this.repositoryButton.setEnabled (false);
            else
                this.repositoryButton.setEnabled (true);
            this.helper_.setServerName (this.serverTextField.getText());
            this.cvsRootTextField.setText (this.helper_.getCvsRoot());
        }
        else if (event.getDocument() == this.userNameTextField.getDocument()) {
            this.helper_.setUserName (this.userNameTextField.getText());
            this.cvsRootTextField.setText (this.helper_.getCvsRoot());
        }
        else if (event.getDocument() == this.repositoryTextField.getDocument()) {
            this.helper_.setRepository (this.repositoryTextField.getText());
            this.cvsRootTextField.setText (this.helper_.getCvsRoot());
        }
        fireChange();
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup buttonGroup;
    private javax.swing.JPanel connectionPanel;
    private javax.swing.JPanel methodPanel;
    private javax.swing.JLabel cmjLabel;
    private javax.swing.JRadioButton localRadioButton;
    private javax.swing.JRadioButton serverRadioButton;
    private javax.swing.JRadioButton pserverRadioButton;
    private javax.swing.JRadioButton extRadioButton;
    private javax.swing.JLabel cvsRshLabel;
    private javax.swing.JTextField cvsRshTextField;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JPanel repositoryPanel;
    private javax.swing.JLabel serverLabel;
    private javax.swing.JTextField serverTextField;
    private javax.swing.JLabel userNameLabel;
    private javax.swing.JTextField userNameTextField;
    private javax.swing.JLabel repositoryLabel;
    private javax.swing.JTextField repositoryTextField;
    private javax.swing.JButton repositoryButton;
    private javax.swing.JLabel cvsRootLabel;
    private javax.swing.JTextField cvsRootTextField;
    private javax.swing.JTextArea jTextArea1;
    // End of variables declaration//GEN-END:variables

}
