/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.vcscore.wizard.mountcvs;

import org.openide.TopManager;
import org.openide.WizardDescriptor;
import org.openide.util.NbBundle;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;
import javax.swing.ButtonGroup;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import java.io.File;
import org.netbeans.modules.vcscore.wizard.mountcvs.util.CvsHelper;
import org.netbeans.modules.vcscore.wizard.mountcvs.util.Utilities;
/**
 *
 * @author  Martin Entlicher
 */
public class ClientPanel extends AbstractWizardPanel implements ActionListener, DocumentListener {

    private boolean windows_;
    
    /** Creates new form ClientPanel */
    public ClientPanel() {
        initComponents();
        setName(org.openide.util.NbBundle.getBundle(CMPanel.class).getString("ClientPanel.title"));
        postInitComponents ();
        this.putClientProperty (CvsWizardData.SELECTED_INDEX, new Integer(4));
        this.jTextArea1.setFont (javax.swing.UIManager.getFont ("Label.font"));
        this.jTextArea1.setDisabledTextColor (javax.swing.UIManager.getColor ("Label.foreground"));
    }
    
    
    public void readCvsWizardSettings (CvsWizardData data) {
        this.windows_ = data.isWindows();
        boolean javaCvsInstalled = this.isModuleInstalled ("org.netbeans.modules.cvsclient.JavaCvsSettings");
        boolean genCvsInstalled = this.isModuleInstalled ("org.netbeans.modules.vcs.cmdline.CvsFileSystem");
        
        if (!javaCvsInstalled && this.javaRadioButton.isEnabled()) 
            this.javaRadioButton.setEnabled (false);
        if (!genCvsInstalled && this.cmdlineRadioButton.isEnabled()) 
            this.cmdlineRadioButton.setEnabled (false);
        
        if (data.getMethodType() == CvsHelper.T_LOCAL || 
                data.getMethodType() == CvsHelper.T_EXTERNAL ||
                data.getMethodType() == CvsHelper.T_SERVER) {
            this.javaRadioButton.setEnabled (false);
        }
        else if (javaCvsInstalled) {
            if (!this.javaRadioButton.isEnabled())
                this.javaRadioButton.setEnabled (true);
        }
        
        if (data.isBuildInCvs() && this.javaRadioButton.isEnabled()) {
            this.javaRadioButton.setSelected(true);
            postBuildInSelected();
        }
        else if (genCvsInstalled) {
            this.cmdlineRadioButton.setSelected(true);
            postCmdSelected ();
        }
        else {
            if (this.javaRadioButton.isEnabled())
                this.javaRadioButton.setSelected (true);
            postBuildInSelected (); //Disable controls
        }
        
        String cvsExec = data.getCvsExecutable();
        this.cvsExeTextField.setText (cvsExec);
        String shellExec = data.getShellExecutalbe();
        if (shellExec != null)
            this.shellTextField.setText (shellExec);
    }
    
    public void storeCvsWizardSettings (CvsWizardData data) {
        data.buildInCvs (!this.cmdlineRadioButton.isSelected());
        data.setCvsExecutalbe (this.cvsExeTextField.getText());
        data.setShellExecutalbe (this.shellTextField.getText());
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        buttonGroup = new javax.swing.ButtonGroup();
        jPanel1 = new javax.swing.JPanel();
        cmdlineRadioButton = new javax.swing.JRadioButton();
        cvsExeLabel = new javax.swing.JLabel();
        cvsExeTextField = new javax.swing.JTextField();
        cvsExeButton = new javax.swing.JButton();
        shellLabel = new javax.swing.JLabel();
        shellTextField = new javax.swing.JTextField();
        shellButton = new javax.swing.JButton();
        javaRadioButton = new javax.swing.JRadioButton();
        jPanel2 = new javax.swing.JPanel();
        jTextArea1 = new javax.swing.JTextArea();
        
        buttonGroup.add(cmdlineRadioButton);
        buttonGroup.add(javaRadioButton);
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        jPanel1.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints2;
        
        cmdlineRadioButton.setMnemonic(org.openide.util.NbBundle.getBundle (ClientPanel.class).getString ("ClientPanel.cmdlineRadioButton_Mnemonic").charAt (0));
        cmdlineRadioButton.setSelected(true);
        cmdlineRadioButton.setText(org.openide.util.NbBundle.getBundle(ClientPanel.class).getString("ClientPanel.cmdlineRadioButton"));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridwidth = 3;
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints2.weightx = 1.0;
        jPanel1.add(cmdlineRadioButton, gridBagConstraints2);
        
        cvsExeLabel.setText(org.openide.util.NbBundle.getBundle(ClientPanel.class).getString("ClientPanel.cvsExecutable"));
        cvsExeLabel.setLabelFor(cvsExeTextField);
        cvsExeLabel.setDisplayedMnemonic (org.openide.util.NbBundle.getBundle (ClientPanel.class).getString ("ClientPanel.cvsExecutable_Mnemonic").charAt (0));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridy = 1;
        gridBagConstraints2.insets = new java.awt.Insets(0, 24, 0, 0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        jPanel1.add(cvsExeLabel, gridBagConstraints2);
        
        cvsExeTextField.setText("cvs");
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridy = 1;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.insets = new java.awt.Insets(0, 12, 0, 0);
        gridBagConstraints2.weightx = 1.0;
        jPanel1.add(cvsExeTextField, gridBagConstraints2);
        
        cvsExeButton.setMnemonic(org.openide.util.NbBundle.getBundle (ClientPanel.class).getString ("ClientPanel.cvsExeButton_Mnemonic").charAt (0));
        cvsExeButton.setText(org.openide.util.NbBundle.getBundle(ClientPanel.class).getString("ClientPanel.cvsExeButton"));
        cvsExeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cvsExeButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridy = 1;
        gridBagConstraints2.insets = new java.awt.Insets(0, 5, 0, 0);
        jPanel1.add(cvsExeButton, gridBagConstraints2);
        
        shellLabel.setText(org.openide.util.NbBundle.getBundle(ClientPanel.class).getString("ClientPanel.shellLabel"));
        shellLabel.setLabelFor(shellTextField);
        shellLabel.setDisplayedMnemonic (org.openide.util.NbBundle.getBundle (ClientPanel.class).getString ("ClientPanel.shellLabel_Mnemonic").charAt (0));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridy = 2;
        gridBagConstraints2.insets = new java.awt.Insets(2, 24, 0, 0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        jPanel1.add(shellLabel, gridBagConstraints2);
        
        shellTextField.setText("sh.exe");
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridy = 2;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.insets = new java.awt.Insets(2, 12, 0, 0);
        gridBagConstraints2.weightx = 1.0;
        jPanel1.add(shellTextField, gridBagConstraints2);
        
        shellButton.setMnemonic(org.openide.util.NbBundle.getBundle (ClientPanel.class).getString ("ClientPanel.shellButton_Mnemonic").charAt (0));
        shellButton.setText(org.openide.util.NbBundle.getBundle(ClientPanel.class).getString("ClientPanel.shellButton"));
        shellButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                shellButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridy = 2;
        gridBagConstraints2.insets = new java.awt.Insets(2, 5, 0, 0);
        jPanel1.add(shellButton, gridBagConstraints2);
        
        javaRadioButton.setMnemonic(org.openide.util.NbBundle.getBundle (ClientPanel.class).getString ("ClientPanel.javaRadioButton_Mnemonic").charAt (0));
        javaRadioButton.setText(org.openide.util.NbBundle.getBundle(ClientPanel.class).getString("ClientPanel.javaRadioButton"));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridy = 3;
        gridBagConstraints2.gridwidth = 3;
        gridBagConstraints2.insets = new java.awt.Insets(9, 0, 17, 0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints2.weightx = 1.0;
        jPanel1.add(javaRadioButton, gridBagConstraints2);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTH;
        gridBagConstraints1.weightx = 1.0;
        add(jPanel1, gridBagConstraints1);
        
        jPanel2.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints3;
        
        jTextArea1.setWrapStyleWord(true);
        jTextArea1.setLineWrap(true);
        jTextArea1.setEditable(false);
        jTextArea1.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcscore/wizard/mountcvs/Bundle").getString("TIP_CvsClientPanel"));
        jTextArea1.setBackground(java.awt.Color.lightGray);
        jTextArea1.setDisabledTextColor(java.awt.Color.black);
        jTextArea1.setEnabled(false);
        jTextArea1.setOpaque(false);
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints3.weightx = 1.0;
        gridBagConstraints3.weighty = 1.0;
        jPanel2.add(jTextArea1, gridBagConstraints3);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTH;
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(jPanel2, gridBagConstraints1);
        
    }//GEN-END:initComponents

  private void shellButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_shellButtonActionPerformed
// Add your handling code here:
      JFileChooser fd = new JFileChooser ();
      fd.setFileFilter ( new FileFilter () {
          public boolean accept (File file) {
              if (file.isDirectory())
                  return true;
              if (file.exists() && file.isFile())
                  return true;
              else
                  return false;
          }
          
          public String getDescription () {
              return NbBundle.getBundle(ClientPanel.class).getString("TXT_Executable");
          }
      });
      fd.setFileSelectionMode ( JFileChooser.FILES_ONLY);
      fd.setName ( NbBundle.getBundle (ClientPanel.class).getString("TXT_CvsExecutable"));
      java.io.File dir = Utilities.findDirectory (this.shellTextField.getText());
        if (dir != null)
            fd.setCurrentDirectory (dir);
      if (fd.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
          this.shellTextField.setText (fd.getSelectedFile().getAbsolutePath());
      }
  }//GEN-LAST:event_shellButtonActionPerformed

  private void cvsExeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cvsExeButtonActionPerformed
// Add your handling code here:
            JFileChooser fd = new JFileChooser ();
      fd.setFileFilter ( new FileFilter () {
          public boolean accept (File file) {
              if (file.isDirectory())
                  return true;
              if (file.exists() && file.isFile())
                  return true;
              else
                  return false;
          }   
          public String getDescription () {
              return NbBundle.getBundle(ClientPanel.class).getString("TXT_Executable");
          }
      });
      fd.setFileSelectionMode ( JFileChooser.FILES_ONLY);
      fd.setName ( NbBundle.getBundle (ClientPanel.class).getString("TXT_ShellExecutable"));
      java.io.File dir = Utilities.findDirectory (this.cvsExeTextField.getText());
        if (dir != null)
            fd.setCurrentDirectory (dir);
      if (fd.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
          this.cvsExeTextField.setText (fd.getSelectedFile().getAbsolutePath());
      }
  }//GEN-LAST:event_cvsExeButtonActionPerformed

    
    public org.openide.util.HelpCtx getHelp() {
        return org.openide.util.HelpCtx.DEFAULT_HELP;
    }
    
    private void postInitComponents () {
        this.javaRadioButton.addActionListener (this);
        this.cmdlineRadioButton.addActionListener (this);
    }
    
    
    public boolean isValid() {
        if (this.javaRadioButton.isSelected() && this.javaRadioButton.isEnabled())
            return true;
        return this.cvsExeTextField.getText().length()>0 &&
               this.shellTextField.getText().length()>0 &&
               this.cmdlineRadioButton.isEnabled();
    }
    
    public void actionPerformed (ActionEvent event) {
        if (this.cmdlineRadioButton.isSelected())
            postCmdSelected();
        else
            postBuildInSelected();
        fireChange();
    }
    
    public void insertUpdate (DocumentEvent event) {
        fireChange();
    }
    
    public void removeUpdate (DocumentEvent event) {
        fireChange();
    }
    
    public void changedUpdate (DocumentEvent event) {
        fireChange();
    }
    
    private void postBuildInSelected () {
        this.cvsExeLabel.setEnabled (false);
        this.cvsExeTextField.setEnabled (false);
        this.shellLabel.setEnabled (false);
        this.shellTextField.setEnabled (false);
        this.shellButton.setEnabled (false);
        this.cvsExeButton.setEnabled (false);
    }
    
    private void postCmdSelected () {
        if (this.windows_) {
            this.shellTextField.setEnabled (true);
            this.shellLabel.setEnabled (true);
            this.shellButton.setEnabled (true);
        }
        else
        {
            this.shellTextField.setEnabled (false);
            this.shellLabel.setEnabled (false);
            this.shellButton.setEnabled (false);
        }
        this.cvsExeLabel.setEnabled (true);
        this.cvsExeTextField.setEnabled (true);
        this.cvsExeButton.setEnabled (true);
    }
    
    private boolean isModuleInstalled (String moduleName) {
        try {
            TopManager tm = TopManager.getDefault ();
            ClassLoader cloader = tm.systemClassLoader();
            cloader.loadClass (moduleName);
            return true;
        }catch (ClassNotFoundException cnfe) {
            return false;
        }
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup buttonGroup;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JRadioButton cmdlineRadioButton;
    private javax.swing.JLabel cvsExeLabel;
    private javax.swing.JTextField cvsExeTextField;
    private javax.swing.JButton cvsExeButton;
    private javax.swing.JLabel shellLabel;
    private javax.swing.JTextField shellTextField;
    private javax.swing.JButton shellButton;
    private javax.swing.JRadioButton javaRadioButton;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JTextArea jTextArea1;
    // End of variables declaration//GEN-END:variables

}
