/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.vcscore.wizard.mountcvs;

import org.openide.WizardDescriptor;
import org.openide.util.NbBundle;
//import org.netbeans.modules.vcscore.wizard.mountcvs.passwd.CVSPasswd;
import org.netbeans.modules.vcscore.wizard.mountcvs.util.Utilities;

/**
 *
 * @author  Martin Entlicher
 */
public class CvsHomePanel extends AbstractWizardPanel implements javax.swing.event.DocumentListener {

    /** Creates new form CvsHomePanel */
    public CvsHomePanel() {
        initComponents();
        setName(org.openide.util.NbBundle.getBundle(CvsHomePanel.class).getString("CvsHomePanel.title"));
        this.cvsHomeTextField.getDocument().addDocumentListener (this);
        this.putClientProperty (CvsWizardData.SELECTED_INDEX, new Integer(3));
        this.jTextArea1.setFont (javax.swing.UIManager.getFont ("Label.font"));
        this.jTextArea1.setDisabledTextColor (javax.swing.UIManager.getColor ("Label.foreground"));
    }
    
    protected void readCvsWizardSettings (CvsWizardData data) {
        String homeDir__ = data.getHome ();
        if (homeDir__ == null)
            homeDir__ = Utilities.getHome();
        try {
            if (homeDir__ == null)
                homeDir__ = java.io.File.listRoots()[0].getAbsolutePath();
        }catch (ArrayIndexOutOfBoundsException be) {
            homeDir__ = "";
        }
        this.cvsHomeTextField.setText (homeDir__);
    }
    
    protected void storeCvsWizardSettings (CvsWizardData data) {
        data.setHome (this.cvsHomeTextField.getText());
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jPanel1 = new javax.swing.JPanel();
        cvsHomeLabel = new javax.swing.JLabel();
        cvsHomeTextField = new javax.swing.JTextField();
        cvsHomeButton = new javax.swing.JButton();
        jTextArea1 = new javax.swing.JTextArea();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        jPanel1.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints2;
        
        cvsHomeLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcscore/wizard/mountcvs/Bundle").getString("CvsHomePanel.cvsHomeLabel"));
        cvsHomeLabel.setLabelFor(cvsHomeTextField);
        cvsHomeLabel.setDisplayedMnemonic (org.openide.util.NbBundle.getBundle (CvsHomePanel.class).getString ("CvsHomePanel.cvsHomeLabel_Mnemonic").charAt (0));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        jPanel1.add(cvsHomeLabel, gridBagConstraints2);
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.insets = new java.awt.Insets(0, 12, 0, 0);
        gridBagConstraints2.weightx = 1.0;
        jPanel1.add(cvsHomeTextField, gridBagConstraints2);
        
        cvsHomeButton.setMnemonic(org.openide.util.NbBundle.getBundle (CvsHomePanel.class).getString ("CvsHomePanel.cvsHomeButton_Mnemonic").charAt (0));
        cvsHomeButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcscore/wizard/mountcvs/Bundle").getString("CvsHomePanel.cvsHomeButton"));
        cvsHomeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cvsHomeButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.insets = new java.awt.Insets(0, 5, 0, 0);
        jPanel1.add(cvsHomeButton, gridBagConstraints2);
        
        jTextArea1.setWrapStyleWord(true);
        jTextArea1.setLineWrap(true);
        jTextArea1.setEditable(false);
        jTextArea1.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcscore/wizard/mountcvs/Bundle").getString("TIP_HomePanel"));
        jTextArea1.setBackground(java.awt.Color.lightGray);
        jTextArea1.setDisabledTextColor(java.awt.Color.black);
        jTextArea1.setEnabled(false);
        jTextArea1.setOpaque(false);
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridy = 1;
        gridBagConstraints2.gridwidth = 3;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints2.insets = new java.awt.Insets(17, 0, 0, 0);
        gridBagConstraints2.weightx = 1.0;
        gridBagConstraints2.weighty = 1.0;
        jPanel1.add(jTextArea1, gridBagConstraints2);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(jPanel1, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void cvsHomeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cvsHomeButtonActionPerformed
        // Add your handling code here:
        javax.swing.JFileChooser fd = new javax.swing.JFileChooser ();
        fd.setFileFilter ( new javax.swing.filechooser.FileFilter () {
            
            public boolean accept (java.io.File file) {
                if (file.isDirectory())
                    return true;
                else
                    return false;
            }
            
            public String getDescription () {
                return NbBundle.getBundle(CvsHomePanel.class).getString("TXT_Directory");
            }
        });
        fd.setFileSelectionMode (javax.swing.JFileChooser.DIRECTORIES_ONLY);
        java.io.File dir = Utilities.findDirectory (this.cvsHomeTextField.getText());
        if (dir != null)
            fd.setCurrentDirectory (dir);
        if (fd.showOpenDialog (this) == javax.swing.JFileChooser.APPROVE_OPTION) {
            this.cvsHomeTextField.setText (fd.getSelectedFile().getAbsolutePath());
        }
    }//GEN-LAST:event_cvsHomeButtonActionPerformed

    
    public org.openide.util.HelpCtx getHelp() {
        return org.openide.util.HelpCtx.DEFAULT_HELP;
    }
    
    
    public boolean isValid() {
        String homeDir__ = this.cvsHomeTextField.getText();
        if (homeDir__ == null || homeDir__.length() == 0)
            return false;
        java.io.File dir = new java.io.File (homeDir__);
        if (!dir.exists() || !dir.isDirectory())
            return false;
        else
            return true;
    }
        
    
    public void changedUpdate (javax.swing.event.DocumentEvent e) {
        fireChange ();
    }
    
    public void removeUpdate (javax.swing.event.DocumentEvent e) {
        fireChange();
    }
    
    
    public void insertUpdate (javax.swing.event.DocumentEvent e) {
        fireChange();
    }
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel jPanel1;
    private javax.swing.JLabel cvsHomeLabel;
    private javax.swing.JTextField cvsHomeTextField;
    private javax.swing.JButton cvsHomeButton;
    private javax.swing.JTextArea jTextArea1;
    // End of variables declaration//GEN-END:variables

}
