/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.vcscore.wizard.mountcvs;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import org.openide.*;
import org.openide.util.*;
import java.beans.Customizer;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.util.ResourceBundle;
import org.openide.util.Utilities;
import org.netbeans.modules.vcscore.wizard.mountcvs.util.CvsHelper;
/**
 *
 * @author  root
 * @version 
 */
public class CvsWizardData extends WizardDescriptor implements Customizer, PropertyChangeListener {

    public static final String CLIENT_CLASS_BUILT_IN = "org.netbeans.modules.cvsclient.JavaCvsFileSystem";
    public static final String CLIENT_CLASS_CMD_LINE = "org.netbeans.modules.vcs.cmdline.CvsFileSystem";
    public static final String CLIENT_METHOD_LOGIN = "checkLogin";
    
    public static final String AUTO_WIZARD_STYLE = "WizardPanel_autoWizardStyle";
    public static final String CONTENT_DISPLAYED = "WizardPanel_contentDisplayed";
    public static final String CONTENT_NUMBERED = "WizardPanel_contentNumbered";
    public static final String CONTENT_DATA = "WizardPanel_contentData";
    public static final String SELECTED_INDEX = "WizardPanel_contentSelectedIndex";
    
    private static final String DEFAULT_CVS_EXEC = "cvs";
    private static final String DEFAULT_SHELL_EXEC = "sh";

    
    private String homeDir_;
    private String workingDir_;
    private File cvsRootFile_;
    private String rsh_;
    private static String cvsExec_;
    private static String shellExec_;
    private String mountPoint_;
    private String[] mountPoints_;
    private boolean useBuildIn_;
    private boolean performInitialCheckOut_;
    private boolean offline_;
    private boolean windows_;
    private CvsHelper helper_;    
    private CvsMountFS fs_;
    private ResourceBundle bundle_ = NbBundle.getBundle (CvsWizardData.class);
    
    static {
	cvsExec_ = DEFAULT_CVS_EXEC;
	shellExec_ = DEFAULT_SHELL_EXEC;
    }
    
    /** Creates new CvsWizardData */
    public CvsWizardData() {
        super (new WizardDescriptor.Panel[0]);
        MountCvsWizard iterator = new MountCvsWizard (this);
        this.setPanels (iterator);
        this.addPropertyChangeListener (this);
        this.windows_ = Utilities.getOperatingSystem() == Utilities.OS_WIN95 ||
                        Utilities.getOperatingSystem() == Utilities.OS_WIN98;  
        this.useBuildIn_ = true;
        setClosingOptions ( new Object[] {DialogDescriptor.CANCEL_OPTION, WizardDescriptor.FINISH_OPTION});
        setTitle (this.bundle_.getString("CVSWizard_Title"));
        setTitleFormat (new java.text.MessageFormat(this.bundle_.getString("CVSWizard_TitleFormat")));
        putProperty(AUTO_WIZARD_STYLE, new Boolean(true));  
        putProperty(CONTENT_DISPLAYED, new Boolean(true)); 
        putProperty(CONTENT_NUMBERED, new Boolean(true));  
        putProperty(CONTENT_DATA, iterator.getNames()); 
    }
    
    
    public void setObject (Object value) {
        this.fs_ = (CvsMountFS) value;
    }
    
    public void propertyChange (PropertyChangeEvent event) {
        if (event.getPropertyName().equals(DialogDescriptor.PROP_VALUE)){
            Object option = event.getNewValue();
            if (WizardDescriptor.FINISH_OPTION.equals(option)) {
                if (this.fs_ != null) {
                    this.fs_.setWizardData (this);
                }
            }
        }
    }
    
    
    public void setHome (String homeDir) {
        this.homeDir_ = homeDir;
    }
    
    public String getHome () {
        return this.homeDir_;
    }
    
    public void setRootFile (File file) {
        this.cvsRootFile_ = file;
        this.mountPoint_ = null;
        if (this.helper_ != null)
            this.helper_.setCvsRoot (this.getCvsString());
    }
    
    public File getRootFile () {
        return this.cvsRootFile_;
    }
    
    public boolean isExisting () {
        return this.cvsRootFile_ != null;
    }
        
    public void setWorkingDir (String dir) {
        this.workingDir_ = dir;
    }
    
    public String getWorkingDir () {
        return this.workingDir_;
    }
    
    public void setRsh (String rsh) {
        this.rsh_ = rsh;
    }
    
    public String getRsh () {
        return this.rsh_;
    }
    
    
    public String getCvsExecutable () {
        return cvsExec_;
    }
    
    public void setCvsExecutalbe (String cvsExec) {
        cvsExec_ = cvsExec;
    }
    
    public String getShellExecutalbe () {
        return this.shellExec_;
    }
    
    public void setShellExecutalbe (String shellExec) {
        this.shellExec_ = shellExec;
    }
    
    public boolean isBuildInCvs () {
        if (this.helper_ == null)
            initCvsHelper();
        if (this.helper_.getMethodType() == CvsHelper.T_LOCAL || 
            this.helper_.getMethodType() == CvsHelper.T_EXTERNAL)
                return false;
        return this.useBuildIn_;
    }
    
    public void buildInCvs (boolean flag) {
        this.useBuildIn_ = flag;
    }
    
    public boolean getPerformInitialCheckout () {
        return this.performInitialCheckOut_;
    }
    
    public void setPerformInitialCheckout (boolean checkOut) {
        this.performInitialCheckOut_ = checkOut;
    }
    
    public boolean isOffLine () {
        return this.offline_;
    }
    
    public void setOffline (boolean offline) {
        this.offline_ = offline;
    }
    
    public int getMethodType () {
        if (this.helper_ == null)
            initCvsHelper();
        return this.helper_.getMethodType();
    }
    
    public String getMethod () {
        if (this.helper_ == null)
            initCvsHelper();
        return this.helper_.getMethod();
    }
    
    public String getUserName () {
        if (this.helper_ == null)
            initCvsHelper();
        return this.helper_.getUserName();
    }
    
    public String getHostName () {
        if (this.helper_ == null)
            initCvsHelper();
        return this.helper_.getServerName();
    }
    
    public String getCvsRoot () {
        if (this.helper_ == null)
            initCvsHelper ();
        return this.helper_.getCvsRoot();
    }
    
    public String getCvsRepository () {
        if (this.helper_ == null)
            initCvsHelper ();
        return this.helper_.getRepository();
    }
    
    public CvsHelper getHelper () {
        if (this.helper_ == null)
            initCvsHelper();
        return this.helper_;
    }
   
    public String getMountPoint () {
        if (this.mountPoint_ != null)
            return this.mountPoint_;
        else if (this.cvsRootFile_ != null) {
            // First pass with CVS/Root
            String root = this.cvsRootFile_.getParentFile().getParent();
            if (!this.workingDir_.equals(root))
                return root;
        }
        return "";  // No I18N
    }
    
    public void setMountPoint (String mountPoint) {
        this.mountPoint_ = mountPoint;
        File tmpCvsRoot = new File (this.workingDir_,mountPoint_+"/CVS/Root");
        if (tmpCvsRoot.exists() && !tmpCvsRoot.equals (this.cvsRootFile_)) {
            this.cvsRootFile_ = tmpCvsRoot;
            initCvsHelper ();
        }
    }
    
    public void setMountPointOnly (String mountPoint) {
        this.mountPoint_ = mountPoint;
    }
    
    public String[] getMountPoints () {
        if (this.mountPoints_ != null)
            return this.mountPoints_;
        else {
            return new String[] { getMountPoint() };
        }
    }
    
    public void setMountPoints (String[] mountPoints) {
        this.mountPoints_ = mountPoints;
        for (int i = 0; i < mountPoints.length; i++) {
            File tmpCvsRoot = new File (this.workingDir_,mountPoints[i]+"/CVS/Root");
            if (tmpCvsRoot.exists() && !tmpCvsRoot.equals (this.cvsRootFile_)) {
                this.cvsRootFile_ = tmpCvsRoot;
                initCvsHelper ();
                break;
            }
        }
    }
    
    public boolean isWindows () {
        return this.windows_;
    }
    
    private void initCvsHelper () {
        this.helper_ = new CvsHelper (this.getCvsString());
    }
    
    /** Returns cvsString or null in case of fault
     *  or new repository
     */
    private String getCvsString () {
        String cvsString = null;
        if (this.cvsRootFile_ != null) {
            BufferedReader in = null;
            try {
                in = new BufferedReader ( new FileReader (this.cvsRootFile_));
                cvsString = in.readLine();
            }catch (java.io.IOException ioe) {}
            finally {
                try {
                    if (in!= null) 
                        in.close();
                }catch (java.io.IOException ex) {}
            }
        }
        else {
            cvsString = System.getProperty ("env-cvsroot");   // No I18N
        }
        return cvsString;
    }
}
