/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.vcscore.wizard.mountcvs;

import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.text.MessageFormat;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;

import org.openide.TopManager;
import org.openide.WizardDescriptor;
import org.openide.util.NbBundle;
import org.openide.util.RequestProcessor;
import org.netbeans.modules.vcscore.wizard.mountcvs.util.CvsHelper;
//import org.netbeans.modules.vcscore.wizard.mountcvs.passwd.CVSPasswd;
//import org.netbeans.modules.vcscore.wizard.mountcvs.passwd.PasswdEntry;
/**
 *
 * @author  Martin Entlicher
 */
public class LoginPanel extends AbstractWizardPanel implements ActionListener {
    
    private boolean logedIn_;
    private String cvsRoot_;
    //private CVSPasswd passwd_;
    private boolean buildInCvs_;
    private CvsWizardData data_ = null;
    

    /** Creates new form LoginPanel */
    public LoginPanel() {
        initComponents();
        setName(org.openide.util.NbBundle.getBundle(CMPanel.class).getString("LoginPanel.title"));
        this.logedIn_ = false;
        this.postInitComponents ();
        //this.passwd_ = new CVSPasswd();
        this.buildInCvs_ = true;
        this.jTextArea1.setText (NbBundle.getBundle(LoginPanel.class).getString("TXT_BeforeLoginNotLogged"));
        this.putClientProperty (CvsWizardData.SELECTED_INDEX, new Integer(5));
        this.jTextArea1.setFont (javax.swing.UIManager.getFont ("Label.font"));
        this.jTextArea2.setFont (javax.swing.UIManager.getFont ("Label.font"));
        this.jTextArea2.setDisabledTextColor (javax.swing.UIManager.getColor ("Label.foreground"));
    }
    
    public void readCvsWizardSettings (CvsWizardData data) {
        this.data_ = data;
        String hostname = data.getHostName();
        String username = data.getUserName();
        String cvsRoot = data.getHelper().getCvsRoot();
        boolean buildInCvs = data.isBuildInCvs();
        if (!cvsRoot.equals(this.cvsRoot_) || buildInCvs != this.buildInCvs_)
            this.logedIn_ = false;
        this.cvsRoot_ = cvsRoot;
        this.buildInCvs_ = buildInCvs;
        this.loginRadioButton.setText(MessageFormat.format (NbBundle.getBundle(LoginPanel.class).getString("LoginPanel.loginRadioButton.text"), new Object[] {hostname}));
        this.userNameValueLabel.setText (username);
        if (org.netbeans.modules.vcscore.wizard.mountcvs.util.Utilities.isLoggedInCvspass(data.getCvsRoot())) {
            this.jTextArea1.setText (NbBundle.getBundle(LoginPanel.class).getString("TXT_BeforeLoginIsLogged"));
            this.logedIn_ = true;
        } else if (!this.logedIn_) {
            this.jTextArea1.setText (NbBundle.getBundle(LoginPanel.class).getString("TXT_BeforeLoginNotLogged"));
        }
        if (data.isOffLine()) {
            this.offLineRadioButton.setSelected (true);
            this.passwordLabel.setEnabled (false);
            this.passwordField.setEnabled (false);
            this.loginButton.setEnabled (false);
            this.userNameLabel.setEnabled (false);
            this.userNameLabel.setEnabled (false);
        }
        else {
            this.loginRadioButton.setSelected (true);
            this.passwordLabel.setEnabled (true);
            this.passwordField.setEnabled (true);
            this.readPasswd();
            this.loginButton.setEnabled (true);
            this.userNameLabel.setEnabled (true);
            this.userNameLabel.setEnabled (true);
        }
    }
    
    public void storeCvsWizardSettings (CvsWizardData data) {
        data.setOffline (this.offLineRadioButton.isSelected());
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        buttonGroup = new javax.swing.ButtonGroup();
        jPanel1 = new javax.swing.JPanel();
        loginRadioButton = new javax.swing.JRadioButton();
        userNameLabel = new javax.swing.JLabel();
        userNameValueLabel = new javax.swing.JLabel();
        passwordLabel = new javax.swing.JLabel();
        passwordField = new javax.swing.JPasswordField();
        loginButton = new javax.swing.JButton();
        jTextArea1 = new javax.swing.JTextArea();
        offLineRadioButton = new javax.swing.JRadioButton();
        jTextArea2 = new javax.swing.JTextArea();
        
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        jPanel1.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints2;
        
        loginRadioButton.setMnemonic(org.openide.util.NbBundle.getBundle (LoginPanel.class).getString ("LoginClientPanel.loginRadioButton_Mnemonic").charAt (0));
        loginRadioButton.setSelected(true);
        loginRadioButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcscore/wizard/mountcvs/Bundle").getString("LoginClientPanel.loginRadioButton"));
        buttonGroup.add(loginRadioButton);
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridwidth = 3;
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints2.weightx = 1.0;
        jPanel1.add(loginRadioButton, gridBagConstraints2);
        
        userNameLabel.setText(org.openide.util.NbBundle.getBundle(LoginPanel.class).getString("LoginClientPanel.userNameLabel"));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridy = 1;
        gridBagConstraints2.insets = new java.awt.Insets(0, 24, 0, 0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        jPanel1.add(userNameLabel, gridBagConstraints2);
        
        userNameValueLabel.setText("anoncvs");
        userNameValueLabel.setForeground(java.awt.Color.black);
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridy = 1;
        gridBagConstraints2.insets = new java.awt.Insets(0, 12, 0, 0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        jPanel1.add(userNameValueLabel, gridBagConstraints2);
        
        passwordLabel.setText(org.openide.util.NbBundle.getBundle(LoginPanel.class).getString("LoginClientPanel.passwordLabel"));
        passwordLabel.setLabelFor(passwordField);
        passwordLabel.setDisplayedMnemonic (org.openide.util.NbBundle.getBundle (LoginPanel.class).getString ("LoginClientPanel.passwordLabel_Mnemonic").charAt (0));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridy = 2;
        gridBagConstraints2.insets = new java.awt.Insets(3, 24, 0, 0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        jPanel1.add(passwordLabel, gridBagConstraints2);
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridy = 2;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.insets = new java.awt.Insets(3, 12, 0, 0);
        gridBagConstraints2.weightx = 1.0;
        jPanel1.add(passwordField, gridBagConstraints2);
        
        loginButton.setMnemonic(org.openide.util.NbBundle.getBundle (LoginPanel.class).getString ("LoginClientPanel.loginButton_Mnemonic").charAt (0));
        loginButton.setText(org.openide.util.NbBundle.getBundle(LoginPanel.class).getString("LoginClientPanel.loginButton"));
        loginButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loginButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridy = 2;
        gridBagConstraints2.insets = new java.awt.Insets(3, 5, 0, 0);
        jPanel1.add(loginButton, gridBagConstraints2);
        
        jTextArea1.setWrapStyleWord(true);
        jTextArea1.setLineWrap(true);
        jTextArea1.setEditable(false);
        jTextArea1.setRows(3);
        jTextArea1.setText("Login successfull.");
        jTextArea1.setBackground(java.awt.Color.lightGray);
        jTextArea1.setDisabledTextColor(java.awt.Color.black);
        jTextArea1.setMargin(new java.awt.Insets(2, 2, 2, 2));
        jTextArea1.setEnabled(false);
        jTextArea1.setOpaque(false);
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridy = 3;
        gridBagConstraints2.gridwidth = 3;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.insets = new java.awt.Insets(11, 24, 0, 0);
        jPanel1.add(jTextArea1, gridBagConstraints2);
        
        offLineRadioButton.setMnemonic(org.openide.util.NbBundle.getBundle (LoginPanel.class).getString ("LoginClientPanel.offlineRadioButton_Mnemonic").charAt (0));
        offLineRadioButton.setText(org.openide.util.NbBundle.getBundle(LoginPanel.class).getString("LoginClientPanel.offlineRadioButton"));
        buttonGroup.add(offLineRadioButton);
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridy = 4;
        gridBagConstraints2.gridwidth = 3;
        gridBagConstraints2.insets = new java.awt.Insets(12, 0, 0, 0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        jPanel1.add(offLineRadioButton, gridBagConstraints2);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTH;
        gridBagConstraints1.weightx = 1.0;
        add(jPanel1, gridBagConstraints1);
        
        jTextArea2.setWrapStyleWord(true);
        jTextArea2.setLineWrap(true);
        jTextArea2.setEditable(false);
        jTextArea2.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcscore/wizard/mountcvs/Bundle").getString("TIP_LoginPanel"));
        jTextArea2.setDisabledTextColor(java.awt.Color.black);
        jTextArea2.setEnabled(false);
        jTextArea2.setOpaque(false);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.insets = new java.awt.Insets(17, 0, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints1.weighty = 1.0;
        add(jTextArea2, gridBagConstraints1);
        
    }//GEN-END:initComponents

  private void loginButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loginButtonActionPerformed
// Add your handling code here:
      final ResourceBundle bundle = NbBundle.getBundle(LoginPanel.class);
      Runnable r = new Runnable () {
          public void run () {
                LoginPanel.this.logedIn_ = false;
                /*
                Class loginClass;
                try {
                    if (buildInCvs_) {
                        loginClass = Class.forName(CvsWizardData.CLIENT_CLASS_BUILT_IN, true, TopManager.getDefault().currentClassLoader());
                    } else {
                        loginClass = Class.forName(CvsWizardData.CLIENT_CLASS_CMD_LINE, true, TopManager.getDefault().currentClassLoader());
                    }
                } catch (ClassNotFoundException ex) {
                    TopManager.getDefault().notifyException(ex);
                    return ;
                }
                 */
                String[] mountPoints = data_.getMountPoints();
                if (mountPoints != null && mountPoints.length > 0) {
                    data_.setMountPointOnly(mountPoints[0]);
                }
                Object loginFS = CvsMountFS.setupClientFS(data_);
                if (loginFS == null) return ;
                Class loginClass = loginFS.getClass();
                Method login;
                try {
                    login = loginClass.getMethod(CvsWizardData.CLIENT_METHOD_LOGIN, new Class[] { String.class, String.class });
                } catch (NoSuchMethodException nmex) {
                    TopManager.getDefault().notifyException(nmex);
                    return ;
                } catch (SecurityException sex) {
                    TopManager.getDefault().notifyException(sex);
                    return ;
                }
                Object loginReturn = null;
                try {
                    loginReturn = login.invoke(loginFS, new Object[] { LoginPanel.this.cvsRoot_, LoginPanel.this.passwordField.getText() });
                } catch (IllegalAccessException iaex) {
                    TopManager.getDefault().notifyException(iaex);
                    return ;
                } catch (IllegalArgumentException iargex) {
                    TopManager.getDefault().notifyException(iargex);
                    return ;
                } catch (InvocationTargetException itex) {
                    Throwable tex = itex.getTargetException();
                    LoginPanel.this.jTextArea1.setText (tex.getLocalizedMessage());
                }
                if (Boolean.TRUE.equals(loginReturn)) {
                    LoginPanel.this.jTextArea1.setText (bundle.getString("TXT_LoginSuccessfull"));
                    LoginPanel.this.logedIn_ = true;
                }
                LoginPanel.this.fireChange();
                org.openide.filesystems.FileSystem fs = (org.openide.filesystems.FileSystem) loginFS;
                fs.removeNotify(); // to clean up
          }
      };
      this.jTextArea1.setText (bundle.getString("TXT_LoginWait"));
      RequestProcessor.postRequest (r);
  }//GEN-LAST:event_loginButtonActionPerformed

  public void actionPerformed (ActionEvent event) {
      if (this.offLineRadioButton.isSelected()) {
          this.passwordLabel.setEnabled (false);
          this.passwordField.setEnabled (false);
          this.loginButton.setEnabled (false);
          this.userNameLabel.setEnabled (false);
          this.userNameLabel.setEnabled (false);
      }
      else {
          this.passwordLabel.setEnabled (true);
          this.passwordField.setEnabled (true);
          this.loginButton.setEnabled (true);
          this.userNameLabel.setEnabled (true);
          this.userNameLabel.setEnabled (true);
      }
      fireChange();
  }
  
  public org.openide.util.HelpCtx getHelp() {
      return org.openide.util.HelpCtx.DEFAULT_HELP;
  }
  
  public void postInitComponents () {
      javax.swing.ButtonGroup bg = new javax.swing.ButtonGroup ();
      bg.add (this.loginRadioButton);
      bg.add (this.offLineRadioButton);
      this.loginRadioButton.addActionListener (this);
      this.offLineRadioButton.addActionListener (this);
  }
    
  public boolean isValid() {
      return this.offLineRadioButton.isSelected() || this.logedIn_;
  }
  
  private void readPasswd () {
      /* TODO: Need to revert scrambled password to work correctly.
    PasswdEntry pwent = this.passwd_.find (this.cvsRoot_);
      if (pwent != null) 
          this.passwordField.setText (pwent.getPasswd());
       */
  }
        
  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.ButtonGroup buttonGroup;
  private javax.swing.JPanel jPanel1;
  private javax.swing.JRadioButton loginRadioButton;
  private javax.swing.JLabel userNameLabel;
  private javax.swing.JLabel userNameValueLabel;
  private javax.swing.JLabel passwordLabel;
  private javax.swing.JPasswordField passwordField;
  private javax.swing.JButton loginButton;
  private javax.swing.JTextArea jTextArea1;
  private javax.swing.JRadioButton offLineRadioButton;
  private javax.swing.JTextArea jTextArea2;
  // End of variables declaration//GEN-END:variables


}
