/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.vcscore.wizard.mountcvs;

import java.awt.Dialog;
import org.openide.TopManager;
import org.openide.DialogDescriptor;
import org.openide.WizardDescriptor;
import org.openide.util.NbBundle;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.util.ResourceBundle;
import org.netbeans.modules.vcscore.wizard.mountcvs.util.CvsHelper;

/**
 *
 * @author  mentlicher
 * @version 
 */
public class MountCvsWizard extends Object implements org.openide.WizardDescriptor.Iterator {

    private WizardDescriptor.Panel[] panels_;
    private ResourceBundle bundle_;
    private String[] names_;
    private java.util.ArrayList listeners_;
    private int currentIndex_;
    private int relativeIndex_;
    private CvsWizardData data_;
    
    
    public MountCvsWizard (CvsWizardData data) {
        this.data_ = data;
        this.listeners_ = new java.util.ArrayList ();
        this.bundle_ = NbBundle.getBundle (MountCvsWizard.class);
        this.panels_ = new WizardDescriptor.Panel[] {
            new WDPanel(),
            new RMPanel(),
            new CMPanel(),
            new CvsHomePanel(),
            new ClientPanel(),
            new LoginPanel(),
            new CheckoutPanel (),
        };
        this.names_ = new String[] {
            bundle_.getString ("TXT_WorkingDirectory"),
            bundle_.getString ("TXT_RelativeMountPoint"),
            bundle_.getString ("TXT_ConnectionMethod"),
            bundle_.getString ("TXT_CvsHome"),
            bundle_.getString ("TXT_CvsClient"),
            bundle_.getString ("TXT_CvsLogin"),
            bundle_.getString ("TXT_CheckOut")
        };
        this.currentIndex_ = this.relativeIndex_ = 0;
    }

    
    public synchronized void addChangeListener (ChangeListener listener) {
        this.listeners_.add (listener);
    }
    
    public synchronized void removeChangeListener (ChangeListener listener) {
        this.listeners_.remove (listener);
    }
    
    public void previousPanel () {
        switch (currentIndex_) {
            case 1:
                currentIndex_--;
                break;
            case 2:
                if (this.data_.isExisting())
                    currentIndex_--;
                else
                    currentIndex_-=2;
                break;
            case 3: // HOME set
                currentIndex_--;
                break;
            case 4: // Client selection
                if (this.data_.isWindows())
                    currentIndex_--;
                else
                    currentIndex_-=2;
                break;
            case 5: // Login
                currentIndex_--;
                break;
            case 6:
                if (this.data_.getMethodType() == CvsHelper.T_PSERVER)
                    currentIndex_--;
                else 
                    currentIndex_-=2;
        }
        this.relativeIndex_--;
        setContentData();
    }
    
    public void nextPanel () {
        switch (currentIndex_) {
            case 0: // Workind Directory Panel
                if (((WDPanel)panels_[0]).getCvsRootFile()!=null) // CvsWizardData can not be used nextPanel is called before storeSettings
                    this.currentIndex_++;
                else
                    this.currentIndex_+=2;
                break;
            case 1: // Relative Mount Point Panel
                this.currentIndex_++;
                break;
            case 2: // Connection Method
                if (this.data_.isWindows())
                    this.currentIndex_++;
                else 
                    this.currentIndex_+=2; // Client Selection
                break;
            case 3: // HOME set
                    this.currentIndex_++; // Client selection
                break;
            case 4: // Client selection
                if (this.data_.getMethodType() == CvsHelper.T_PSERVER)
                    this.currentIndex_++; // Login
                else 
                    this.currentIndex_+=2; // Checkout
                break;
            case 5: // Login
                this.currentIndex_++; // Checkout
        }
        this.relativeIndex_++;
        setContentData ();
    }
    
    /** Calculates transitive reflexive closure of epsilon
     * transitions to final state
     */
    public boolean hasNext () {
        if (this.currentIndex_ == 6 || 
            (this.currentIndex_ == 5 && this.data_.isExisting())||
            (this.currentIndex_ == 4 && this.data_.isExisting() && this.data_.getMethodType() != CvsHelper.T_PSERVER))
                return false;
        return true;
    }
    
    public boolean hasPrevious () {
        return (this.currentIndex_ > 0);
    }
    
    public String name () {
        return this.names_[this.currentIndex_];
    }
    
    public String[] getNames () {
        return this.getStrings();
    }
    
    public WizardDescriptor.Panel current() {
        return this.panels_[this.currentIndex_];
    }
    
    
    private void fireChange () {
        java.util.Iterator it__;
        javax.swing.event.ChangeEvent event__ = new javax.swing.event.ChangeEvent (this);
        synchronized (this) {
            it__ = ((java.util.ArrayList)this.listeners_.clone()).iterator();
        }
        while (it__.hasNext()) {
            ((ChangeListener)it__.next()).stateChanged (event__);
        }
    }
    
    /** Starts wizard
     */
    public static void run () {
        CvsWizardData wd = new CvsWizardData ();
        java.awt.Dialog dlg = TopManager.getDefault().createDialog (wd);
        dlg.setVisible (true);
    }


    public static void main(String[] args) {
        run ();
    }
    
    private void setContentData () {
        ((javax.swing.JPanel)this.panels_[this.currentIndex_]).putClientProperty (CvsWizardData.SELECTED_INDEX, new Integer(this.relativeIndex_));
        this.data_.putProperty (CvsWizardData.CONTENT_DATA, this.getStrings());
    }
    
    private String[] getStrings () {
        String[] msgs = new String[this.names_.length];
        int i,j;
        for (i=0, j=0; i< this.names_.length; i++) {
            if (i==0)
                msgs[j++] = this.names_[i];   // Always present
            else if (i==1 && (this.currentIndex_ == 0 || ((WDPanel)panels_[0]).getCvsRootFile()!=null))
                msgs[j++] = this.names_[i];
            else if (i==2)
                msgs[j++] = this.names_[i];
            else if (i==3 && this.data_.isWindows())
                msgs[j++] = this.names_[i];
            else if (i==4)
                msgs[j++] = this.names_[i];
            else if (i==5 && (this.data_.getMethodType() == CvsHelper.T_PSERVER || this.currentIndex_<= 2))
                msgs[j++] = this.names_[i];
            else if (i == 6 && ((((WDPanel)panels_[0]).getCvsRootFile()==null) || this.currentIndex_ == 0))
                msgs[j++] = this.names_[i];
        }
        String[] res = new String[j];
        System.arraycopy(msgs,0,res,0,j);
        return res;
    }
    
}
