/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.vcscore.wizard.mountcvs;

import org.openide.WizardDescriptor;
import java.io.File;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import org.openide.util.NbBundle;
import org.netbeans.modules.vcscore.wizard.mountcvs.util.Utilities;
import java.util.LinkedList;

/**
 *
 * @author  Martin Entlicher
 */
public class WDPanel extends AbstractWizardPanel implements java.awt.event.KeyListener {
  
  
    private static LinkedList previousItems = new LinkedList();
    private char lastChar = 0;


    /** Creates new form WDPanel */
    public WDPanel() {
        initComponents();
        setName(org.openide.util.NbBundle.getBundle(CMPanel.class).getString("WDPanel.title"));
        this.putClientProperty (CvsWizardData.SELECTED_INDEX, new Integer(0));
        this.putClientProperty(CvsWizardData.AUTO_WIZARD_STYLE, new Boolean(true));  
        this.putClientProperty(CvsWizardData.CONTENT_DISPLAYED, new Boolean(true)); 
        this.putClientProperty(CvsWizardData.CONTENT_NUMBERED, new Boolean(true));  
        this.descriptionTextArea.setFont (javax.swing.UIManager.getFont ("Label.font"));
        this.descriptionTextArea.setDisabledTextColor (javax.swing.UIManager.getColor ("Label.foreground"));
        setPreferredSize (new java.awt.Dimension (500, 410));
        this.jComboBox1.getEditor().getEditorComponent().addKeyListener (this);
    }
    
    protected void storeCvsWizardSettings (CvsWizardData data) {
        String item = (String)this.jComboBox1.getEditor().getItem();
        data.setWorkingDir (item);
        previousItems.addFirst (item);
        for (int i=1; i< Math.min(4, previousItems.size()); i++) 
            if (previousItems.get(i).equals(item))
                previousItems.remove (i);
        while (previousItems.size()>4)
            previousItems.removeLast();
        data.setRootFile (this.getCvsRootFile());
    }
    
    protected void readCvsWizardSettings (CvsWizardData data) {
        this.jComboBox1.removeAllItems();
        for (int i=0; i<previousItems.size();i++) {
            this.jComboBox1.addItem (previousItems.get(i));
        }
        String wd = data.getWorkingDir();
        if (wd != null)
            this.jComboBox1.setSelectedItem(wd);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jPanel1 = new javax.swing.JPanel();
        wdLabel = new javax.swing.JLabel();
        wdButton = new javax.swing.JButton();
        jComboBox1 = new javax.swing.JComboBox();
        jPanel2 = new javax.swing.JPanel();
        descriptionTextArea = new javax.swing.JTextArea();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        jPanel1.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints2;
        
        wdLabel.setText(org.openide.util.NbBundle.getBundle(WDPanel.class).getString("WDPanel.wdLabel"));
        wdLabel.setLabelFor(jComboBox1);
        wdLabel.setNextFocusableComponent(wdButton);
        wdLabel.setDisplayedMnemonic (org.openide.util.NbBundle.getBundle (WDPanel.class).getString ("WDPanel.wdLabel_Mnemonic").charAt (0));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        jPanel1.add(wdLabel, gridBagConstraints2);
        
        wdButton.setMnemonic(org.openide.util.NbBundle.getBundle (WDPanel.class).getString ("WDPanel.wdBrowse_Mnemonic").charAt (0));
        wdButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcscore/wizard/mountcvs/Bundle").getString("WDPanel.wdBrowse"));
        wdButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                browseDir(evt);
            }
        });
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 2;
        gridBagConstraints2.gridy = 0;
        gridBagConstraints2.insets = new java.awt.Insets(0, 5, 0, 0);
        jPanel1.add(wdButton, gridBagConstraints2);
        
        jComboBox1.setEditable(true);
        jComboBox1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jComboBox1ActionPerformed(evt);
            }
        });
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 1;
        gridBagConstraints2.gridy = 0;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.insets = new java.awt.Insets(0, 12, 0, 0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints2.weightx = 1.0;
        jPanel1.add(jComboBox1, gridBagConstraints2);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTH;
        gridBagConstraints1.weightx = 1.0;
        add(jPanel1, gridBagConstraints1);
        
        jPanel2.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints3;
        
        descriptionTextArea.setWrapStyleWord(true);
        descriptionTextArea.setLineWrap(true);
        descriptionTextArea.setEditable(false);
        descriptionTextArea.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcscore/wizard/mountcvs/Bundle").getString("TIP_WorkingDirectoryPanel"));
        descriptionTextArea.setBackground(java.awt.Color.lightGray);
        descriptionTextArea.setDisabledTextColor(java.awt.Color.black);
        descriptionTextArea.setEnabled(false);
        descriptionTextArea.setOpaque(false);
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints3.insets = new java.awt.Insets(17, 0, 0, 0);
        gridBagConstraints3.weightx = 1.0;
        gridBagConstraints3.weighty = 1.0;
        jPanel2.add(descriptionTextArea, gridBagConstraints3);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTH;
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;
        add(jPanel2, gridBagConstraints1);
        
    }//GEN-END:initComponents

private void jComboBox1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jComboBox1ActionPerformed
    // Add your handling code here:
    fireChange();
}//GEN-LAST:event_jComboBox1ActionPerformed



    private void browseDir(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_browseDir
        // Add your handling code here:
        JFileChooser fd = new JFileChooser ();
        fd.setFileFilter ( new FileFilter () {
            
            public boolean accept (File file) {
                if (file.exists() && file.isDirectory())
                    return true;
                else
                    return false;
            }
            
            public String getDescription () {
                return NbBundle.getBundle (WDPanel.class).getString("TXT_Directory");
            }
        });
        fd.setFileSelectionMode (JFileChooser.DIRECTORIES_ONLY);
        File dir = Utilities.findDirectory ((String)this.jComboBox1.getEditor().getItem());
        if (dir != null)
            fd.setCurrentDirectory (dir);
        if (fd.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
            String newWDName = fd.getSelectedFile().getAbsolutePath();
            this.jComboBox1.getEditor().setItem (newWDName);
            fireChange();
        }
    }//GEN-LAST:event_browseDir
    
    
    public org.openide.util.HelpCtx getHelp() {
        return new org.openide.util.HelpCtx(WDPanel.class);
    }
    
    public boolean isValid() {
        String wd__ = (String)jComboBox1.getEditor().getItem();
        if (this.lastChar != 0) {
	    switch (this.lastChar) {
	    case '\n':
	    case '\r':
		break;
	    case '\b':
                if (wd__.length()>0)
                    wd__ = wd__.substring(0,wd__.length()-1);
                break;
	    default:
		wd__ = wd__+this.lastChar;
	    }
            this.lastChar = 0;
        }
        if ( wd__ == null || wd__.length() == 0)
            return false;
        File f = new File (wd__);
        return f.exists() && f.isDirectory();
    }

    
    public  File getCvsRootFile () {
        File root = new File ((String)this.jComboBox1.getEditor().getItem());
        if (!root.isDirectory())
            return null;
        File res = getCvsRootFile(root);
        if (res != null)
            return res;
        else {
            File[] entries = root.listFiles (new java.io.FileFilter () {
                public boolean accept (File file) {
                    if (file.isDirectory())
                        return true;
                    else
                        return false;
                }
            });
            for (int i=0; i< entries.length; i++) {
                res = getCvsRootFile (entries[i]);
                if (res != null)
                    return res;
            }
        }
        return null;
    }
    
    private File getCvsRootFile (File root) {
        File cvsDir = new File (root, "CVS"); // No I18N
        if (cvsDir.exists() && cvsDir.isDirectory()) {
            File cvsRoot = new File (cvsDir,"Root"); // No I18N
            if (cvsRoot.exists() && cvsRoot.canRead()) 
                return cvsRoot;
        }
        return null;
    }
   
    public void keyPressed(java.awt.event.KeyEvent keyEvent) {
    }
    
    public void keyReleased(java.awt.event.KeyEvent keyEvent) {
        if (java.awt.event.KeyEvent.VK_ENTER == keyEvent.getKeyCode()) {
            javax.swing.SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    javax.swing.JButton defButton = WDPanel.this.getRootPane().getDefaultButton();
                    if (defButton != null) defButton.doClick();
                }
            });
        } else if (java.awt.event.KeyEvent.VK_ESCAPE == keyEvent.getKeyCode()) {
            javax.swing.SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    WDPanel.this.getRootPane().getParent().setVisible(false);
                }
            });
        }
    }

    public void keyTyped(java.awt.event.KeyEvent keyEvent) {
        this.lastChar = keyEvent.getKeyChar();
        fireChange();
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel jPanel1;
    private javax.swing.JLabel wdLabel;
    private javax.swing.JButton wdButton;
    private javax.swing.JComboBox jComboBox1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JTextArea descriptionTextArea;
    // End of variables declaration//GEN-END:variables

}
