/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.vcscore.wizard.mountcvs.util;

import java.io.*;
/**
 *
 * @author  tzezula
 * @version 
 */
public class Utilities {

    public static final String PASS_FILE = ".cvspass";

    /** Creates new Utilities */
    public Utilities() {
    }

    public static File findDirectory (String name) {
        while (name.length()>0) {
            File f = new File (name);
            if (f.isDirectory())
                return f;
            int index = name.lastIndexOf (File.separator);
            if (index == -1)
                break;
            name = name.substring(0,index);
        }
        return null;
    }
    
    /** Get the home directory for .cvspass and possibly other cvs configuration files.
     * Looks for HOME env variable, on Windows for HOMEDRIVE + HOMEPATH.
     * If it can not be retrieved from environmental variables, return the user.home at least.
     * @return the home directory
     */
    public static String getHome() {
        String home = System.getProperty("Env-HOME");
        if (home == null) home = System.getProperty("env-home");
        if (home == null && org.openide.util.Utilities.isWindows()) {
            String homePath = System.getProperty("env-homepath");
            String homeDrive = System.getProperty("env-homedrive");
            if (homePath != null && homeDrive != null) {
                home = homeDrive + homePath;
            }
        }
        if (home == null) home = System.getProperty("user.home");
        return home;
    }
    
    public static boolean isLoggedInCvspass(String cvsRoot) {
        String cvspass = getHome() + File.separator + PASS_FILE;
        File passFile = new File(cvspass);
        boolean isLogged = false;
        BufferedReader bf = null;
        try {
            bf = new BufferedReader(new FileReader(passFile.getAbsolutePath()));
            String line;
            do {
                line = bf.readLine();
                if (line != null) {
                    int space = line.indexOf(' ');
                    if (space > 0 && cvsRoot.equals(line.substring(0, space))) {
                        isLogged = true;
                        break;
                    }
                }
            } while (line != null);
            bf.close();
        } catch (IOException exc) {
            //D.deb(".cvspass reading error");
            return false;
        } finally {
            if (bf != null) {
                try {
                    bf.close();
                } catch (IOException ioexc) {}
            }
        }
        return isLogged;
    }
    
}
