/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.vcs.cmdline;

import javax.swing.event.*;
import java.awt.event.*;
import javax.swing.tree.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import org.openide.WizardDescriptor;
import org.openide.util.*;

import org.netbeans.modules.vcscore.*;
import org.netbeans.modules.vcscore.cmdline.RelativeMountPanel;

/**
 *
 * @author  mkleint
 * @version 
 */
public class Cust4MountPanel extends RelativeMountPanel implements CvsCustomPanel, WizardDescriptor.Panel, TreeSelectionListener {
    
    private NewCvsCustomizer customizer;
    private CvsFileSystem fs;
    private ArrayList listeners = new ArrayList ();
    private boolean isLastPanel = false;
    
    private volatile boolean isValid;

    static final long serialVersionUID = 5574652503187921114L;
    
    public Cust4MountPanel() {
        super();
        //initComponents ();
        isValid = false;
        //trRelMount.addTreeSelectionListener(this);
    }

    public String getName() {
        return org.openide.util.NbBundle.getBundle(NewCvsCustomizer.class).getString("NewCvsCustomizer.mountDialogLabel");
    }
    
    public void setLast(boolean last) {
        isLastPanel = last;
    }
/** last panel check.
 * @return true if last else false.
 */    
    
    public boolean isLast() {
        D("is last");
        return isLastPanel;
    }
    
    public int getNext() {
        return 2;
    }
/** Get the component displayed in this panel.
 * @return the component
 */
    public java.awt.Component getComponent() {
        return this;
    }
/** Help for this panel.
 * When the panel is active, this is used as the help for the wizard dialog.
 * @return the help or <code>null</code> if no help is supplied
 */
    public HelpCtx getHelp() {
        return null;
    }
/** Provides the wizard panel with the current data--either
 * the default data or already-modified settings, if the user used the previous and/or next buttons.
 * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
 * @param settings the object representing wizard panel state, as originally supplied to {@link WizardDescriptor#WizardDescriptor(WizardDescriptor.Iterator,Object)}
 */
    public void readSettings(Object settings) {
        customizer = (NewCvsCustomizer)settings;
        fs = customizer.getFileSystem();
        isValid = false;
        setTreeModel(new DefaultTreeModel(new DefaultMutableTreeNode()));
        setEnabledTree(false);
        waitForCheckout();
    }
/** Provides the wizard panel with the opportunity to update the
 * settings with its current customized state.
 * Rather than updating its settings with every change in the GUI, it should collect them,
 * and then only save them when requested to by this method.
 * Also, the original settings passed to {@link #readSettings} should not be modified (mutated);
 * rather, the (copy) passed in here should be mutated according to the collected changes.
 * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
 * @param settings the object representing a settings of the wizard
 */
    public void storeSettings(Object settings) {
        customizer = (NewCvsCustomizer)settings;
        fs = customizer.getFileSystem();
        customizer.relMount = getRelMount();
        if (!isLastPanel) fillTextFields();
    }
/** Test whether the panel is finished and it is safe to proceed to the next one.
 * If the panel is valid, the "Next" (or "Finish") button will be enabled.
 * @return <code>true</code> if the user has entered satisfactory information
 */
    public boolean isValid() {
        if (isValid) {
            if (customizer != null)
                customizer.setWait(false);
        }
        return isValid;
    }
/** Add a listener to changes of the panel's validity.
 * @param l the listener to add
 * @see #isValid
 */
    public void addChangeListener(ChangeListener l) {
        if (listeners == null) {
            listeners = new ArrayList ();
        }
        listeners.add (l);
    }
/** Remove a listener to changes of the panel's validity.
 * @param l the listener to remove
 */
    public void removeChangeListener(ChangeListener l) {
        if (listeners != null) {
            listeners.remove (l);
        }
    }
    
    private void fireChangeState() {
        ArrayList list;
        synchronized (this) {
            list = (ArrayList) this.listeners.clone();
        }
        ChangeEvent event = new ChangeEvent (this);
        for (int i=0; i<list.size(); i++)
            ((ChangeListener)list.get (i)).stateChanged (event);
    }
    
    public void waitForCheckout() {
        isValid= false;
        final NewCvsCustomizer fCustom = customizer;
        if (customizer.isWaiting()) {
            new Thread ("CVS-WaitingForCheckout") {
                public void run() {
                    try {
                        while (true) {
                            if (fCustom.isWaiting()) {
                                setInfoLabel(org.openide.util.NbBundle.getBundle(NewCvsCustomizer.class).getString("NewCvsCustomizer.waitingForCheckout"));
                            } else {
                                setInfoLabel(org.openide.util.NbBundle.getBundle(NewCvsCustomizer.class).getString("NewCvsCustomizer.checkoutFinished"));
                                initTree(fCustom.rootDir, fCustom.relMount);
                                isValid = true;
                                setEnabledTree(true);
                                fireChangeState();
                                return;
                            }
                            Thread.sleep(3000);
                        }
                    } catch (InterruptedException e) {
                        initTree(fCustom.rootDir, fCustom.relMount);
                        isValid = true;
                        setEnabledTree(true);
                        fireChangeState();
                        return;
                    }
                    
                }
            }.start();
        }
        else {
            isValid = true;
            initTree(customizer.rootDir, customizer.relMount);
            setEnabledTree(true);
        }
    }
    
   private void fillTextFields() {
     BufferedReader buff = null;    
     String dirName = customizer.rootDir;
     dirName = dirName + (customizer.relMount.equals("") ? "" : (File.separator + customizer.relMount));
     if (customizer.relMount.equals("")) {
         //no relative mount point.. just working dir..
         File current = new File(dirName);
         if (!current.exists()) return;
         File[] files = current.listFiles(new FileFilter()  {
              public boolean accept(File f) {
                 if (f.isDirectory() && f.getName().equalsIgnoreCase("CVS")) return true;
                 return false;
              }
          });
          if (files.length == 0) { //found cvs dir in directory
             files = current.listFiles(); 
             File sub = files[0]; // no matter which we pick.. alll should be the same..
             dirName = sub.getAbsolutePath();
          }
     }
     dirName = dirName + File.separator + "CVS" + File.separator + "Root";
     File dirFile = new File(dirName);
     if (dirFile.exists()) {
        try { 
          buff = new BufferedReader(new InputStreamReader(new FileInputStream(dirFile.getAbsolutePath())));
          String line = buff.readLine();
          if (line.startsWith(":")) {
            line = line.substring(1);
            StringTokenizer token = new StringTokenizer(line, ":",false);
            if (token.hasMoreTokens()) {
              customizer.serverType = token.nextToken();
              if (customizer.serverType.equals("local"))  {
                 customizer.repository =  line.substring(customizer.serverType.length() + 1);
              } else {   //some kind of server..
                if (token.hasMoreTokens()) {
                   String userServer =  token.nextToken();   
                   customizer.userName = userServer.substring(0,userServer.indexOf('@'));
                   customizer.serverName = userServer.substring(userServer.indexOf('@') + 1);
                }
                if (token.hasMoreTokens())  customizer.repository = token.nextToken();
              }  
            }      
          }    
        } catch (IOException exc) { //doesn't matter - nothing will be filled in
            customizer.userName = "";
            customizer.serverName = "";
            customizer.serverType = "local";
            customizer.repository = "";
        }              
        finally {
          if (buff != null) {
             try { 
               buff.close(); 
             } catch (IOException eIO) {}  
          }    
        }    
     } 
     else {
         customizer.userName = "";
         customizer.serverName = "";
         customizer.serverType = "local";
         customizer.repository = "";
     }
   }
    
    private static void D(String debug) {
//        System.out.println("Cust4MountPanel(): "+debug);
    }
 
    
}
