/*
 * CustomizerWizard.java
 *
 * Created on June 15, 2000, 9:53 AM
 */

package org.netbeans.modules.vcs.cmdline;

import org.openide.*;
import java.util.*;
import javax.swing.event.ChangeListener;
import java.awt.Panel;
import javax.swing.JPanel;
import javax.swing.event.*;
import java.awt.event.*;


/** The iterator (WizardDescriptor.Iterator) for the NewCvsCustomizer
 *
 * @author mkleint
 * @version 1.0
 */
public class CustomizerWizard implements WizardDescriptor.Iterator {
    private EventListenerList listeners;  
    private LinkedList list;
    private CvsCustomPanel current;
    
    private CvsCustomPanel working;
    private CvsCustomPanel server;
    private CvsCustomPanel mount;
    private CvsCustomPanel checkout;
    
    /** Creates new CustomizerWizard */
    public CustomizerWizard() {
      working = new Cust1WorkPanel();  
      server = new Cust2ServerPanel();
      mount = new Cust4MountPanel();
      checkout = new Cust3CheckoutPanel();
      list = new LinkedList();
      current = working;
      list.addLast(current);
    }
    


    /** Move to the previous panel.
 * I.e. decrement its index, need not actually change any GUI itself
 *
 * Removes the current panel from the list and gets the last one displayed.
 *
 */
    public void previousPanel() {
        list.removeLast();
        current = (CvsCustomPanel)list.getLast();
        ((Cust2ServerPanel)server).setLast(false);
    }
    /** Move to the next panel.
 * I.e. increment its index, need not actually change any GUI itself.
 * To find out which panel is the next, CvsCustomPanel.getNext() is called.
 * To the number the right panel is assigned then.
 * Then it's added to the list of panels which have been seen.
 */
    public void nextPanel() {
        CvsCustomPanel newCurr = null;
        int next = current.getNext();
        if (next == 4) {
            int ind = list.indexOf(server);
            if (ind < 0 ) ((Cust4MountPanel)mount).setLast(false); 
            else ((Cust4MountPanel)mount).setLast(true);
        }
        if (next == 2 && current == mount) { // we're before the second server panel
           ((Cust2ServerPanel)server).setLast(true); 
        } else {((Cust2ServerPanel)server).setLast(false); }
        if (next == 1) { newCurr = working;}
        if (next == 2) { newCurr = server;}
        if (next == 3) { newCurr = checkout;}
        if (next == 4) { newCurr = mount;}
        if (newCurr != current) {
          list.addLast(newCurr);
        }
        current = newCurr;
    }

/** Add a change listener to the iterator
 * @param l the listener to be added
 */
    public void addChangeListener(ChangeListener l) {
        if (listeners == null) {
          listeners = new EventListenerList ();
        }
        listeners.add (ChangeListener.class, l);        
    }

/** removes the listener from the list.
 * @param l the listener to be removed.
 */
    public void removeChangeListener(ChangeListener l) {
        if (listeners != null) {
            listeners.remove (ChangeListener.class, l);
        }
    }
    
   
    private void fireChangeState() {
      Object[] list = listeners.getListenerList();
      ChangeEvent event = null;
      event = new ChangeEvent(this);
      for (int i=0; i< list.length - 1; i++){
        Object listen = list[i];
        if (listen == null) continue;  
        if (listen.getClass() == ChangeListener.class) {
          ChangeListener listener = (ChangeListener) list[i];
          listener.stateChanged(event);
        }

      }
    }    
  
    
    
/** Indicates wheather a next panel exists or not.
 * @return false if it's the last one.
 */
    public boolean hasNext() {
       return (!current.isLast()); 
    }


/** Get the current panel.
 * @return the panel
 */
    public WizardDescriptor.Panel current() {
        return (WizardDescriptor.Panel) current;
    }
    /** Test whether there is a previous panel.
 * @return <code>true</code> if so
 */
    public boolean hasPrevious() {
        return !current.equals(list.getFirst());
    }
    /** Get the name of the current panel.
 * @return the name
 */
    public String name() {
        return org.openide.util.NbBundle.getBundle(NewCvsCustomizer.class).getString("NewCvsCustomizer.wizardName");
    }


}