/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.vcs.cmdline;

import java.beans.*;
import java.util.ResourceBundle;

import org.openide.util.NbBundle;
import org.openide.filesystems.*;

import org.netbeans.modules.vcscore.settings.VcsSettings;
import org.netbeans.modules.vcscore.settings.RefreshModePropertyEditor;
import org.netbeans.modules.vcscore.VcsFileSystem;

/** BeanInfo for CommandLineVcsFileSystem.
 * 
 * @author Pavel Buzek, Martin Entlicher
 */

//-------------------------------------------
public class CvsFileSystemBeanInfo extends SimpleBeanInfo {
    //private static Debug E=new Debug("CvsFileSystemBeanInfo", true); // NOI18N
    //private static Debug D=E;

    /** Array of property descriptors. */
    private static PropertyDescriptor[] desc;

    /** Icon for VCS filesystem. */
    private static java.awt.Image icon;
    private static java.awt.Image icon32;

    static {
        PropertyDescriptor rootDirectory=null;
        PropertyDescriptor debug=null;
        //PropertyDescriptor lock=null;
        //PropertyDescriptor lockPrompt=null;
        PropertyDescriptor edit = null;
        //PropertyDescriptor executePreCommit = null;
        PropertyDescriptor acceptUserParams = null;
        PropertyDescriptor runRefreshCommand = null;
        PropertyDescriptor processAllFiles = null;
        PropertyDescriptor annotationPattern = null;
        PropertyDescriptor autoRefresh = null;
        PropertyDescriptor notification = null;

        try {
            rootDirectory=new PropertyDescriptor
                          (AbstractFileSystem.PROP_ROOT, CvsFileSystem.class, "getRootDirectory", null); // NOI18N
            debug=new PropertyDescriptor
                  (VcsFileSystem.PROP_DEBUG,CvsFileSystem.class, "getDebug", "setDebug"); // NOI18N
            /*
            lock=new PropertyDescriptor
                 ("lock",CvsFileSystem.class,"isLockFilesOn","setLockFilesOn"); // NOI18N
            lockPrompt=new PropertyDescriptor
                       ("lockPrompt",CvsFileSystem.class,"isPromptForLockOn","setPromptForLockOn"); // NOI18N
             */
            edit = new PropertyDescriptor
                   (VcsFileSystem.PROP_CALL_EDIT, CvsFileSystem.class, "isCallEditFilesOn", "setCallEditFilesOn"); // NOI18N
            /*
            executePreCommit = new PropertyDescriptor
                               ("executePreCommit", CvsFileSystem.class, "isExecutePreCommit", "setExecutePreCommit"); // NOI18N
             */
            acceptUserParams = new PropertyDescriptor
                               (VcsFileSystem.PROP_EXPERT_MODE, CvsFileSystem.class, "isExpertMode", "setExpertMode"); // NOI18N
            runRefreshCommand = new PropertyDescriptor
                               (VcsSettings.PROP_OFFLINE, CvsFileSystem.class, "isOffLine", "setOffLine"); // NOI18N
            processAllFiles = new PropertyDescriptor
                               (VcsFileSystem.PROP_PROCESS_UNIMPORTANT_FILES, CvsFileSystem.class, "isProcessUnimportantFiles", "setProcessUnimportantFiles"); // NOI18N
            annotationPattern = new PropertyDescriptor
                               (VcsFileSystem.PROP_ANNOTATION_PATTERN, CvsFileSystem.class, "getAnnotationPattern", "setAnnotationPattern"); // NOI18N
            autoRefresh = new PropertyDescriptor
                               (VcsSettings.PROP_AUTO_REFRESH, CvsFileSystem.class, "getAutoRefresh", "setAutoRefresh"); // NOI18N
            autoRefresh.setPropertyEditorClass(RefreshModePropertyEditor.class);
            notification = new PropertyDescriptor
                               ("commandNotification", CvsFileSystem.class, "isCommandNotification", "setCommandNotification"); // NOI18N
            
            desc = new PropertyDescriptor[] {
                       rootDirectory, debug, /*lock, lockPrompt, */edit, acceptUserParams,
                       runRefreshCommand, processAllFiles, annotationPattern, autoRefresh,
                       notification
                   };

            ResourceBundle bundle = NbBundle.getBundle (CvsFileSystemBeanInfo.class);
            ResourceBundle bundleSettings = NbBundle.getBundle (VcsSettings.class);
            
            rootDirectory.setDisplayName      (bundle.getString("PROP_rootDirectory"));
            rootDirectory.setShortDescription (bundle.getString("HINT_rootDirectory"));
            debug.setDisplayName              (bundle.getString("PROP_debug"));
            debug.setShortDescription         (bundle.getString("HINT_debug"));
            /*
            lock.setDisplayName               (bundle.getString("PROP_lock"));
            lock.setShortDescription          (bundle.getString("HINT_lock"));
            lockPrompt.setDisplayName         (bundle.getString("PROP_lockPrompt"));
            lockPrompt.setShortDescription    (bundle.getString("HINT_lockPrompt"));
             */
            edit.setDisplayName               (bundle.getString("PROP_edit"));
            edit.setShortDescription          (bundle.getString("HINT_edit"));
            //executePreCommit.setDisplayName   (bundle.getString("PROP_executePreCommit"));
            //executePreCommit.setShortDescription(bundle.getString("HINT_executePreCommit"));
            acceptUserParams.setDisplayName   (bundle.getString("PROP_acceptUserParams"));
            acceptUserParams.setShortDescription(bundle.getString("HINT_acceptUserParams"));
            runRefreshCommand.setDisplayName  (bundleSettings.getString("PROP_offline"));
            runRefreshCommand.setShortDescription(bundleSettings.getString("HINT_offline"));
            processAllFiles.setDisplayName    (bundle.getString("PROP_processAllFiles"));
            processAllFiles.setShortDescription(bundle.getString("HINT_processAllFiles"));
            annotationPattern.setDisplayName  (bundle.getString("PROP_annotationPattern"));
            annotationPattern.setShortDescription(bundle.getString("HINT_annotationPattern"));
            autoRefresh.setDisplayName        (bundleSettings.getString("PROP_autoRefresh"));
            autoRefresh.setShortDescription   (bundleSettings.getString("HINT_autoRefresh"));
            notification.setDisplayName       (bundle.getString("PROP_commandNotification"));
            notification.setShortDescription  (bundle.getString("HINT_commandNotification"));

        } catch (IntrospectionException ex) {
            ex.printStackTrace ();
        }
    }

    /* Provides the VCSFileSystem's icon */
    public java.awt.Image getIcon(int type) {
        if (icon == null) {
            icon = loadImage("/org/netbeans/modules/vcs/cmdline/cvs.gif"); // NOI18N
            icon32 = icon;
        }
        if ((type == java.beans.BeanInfo.ICON_COLOR_16x16) || (type == java.beans.BeanInfo.ICON_MONO_16x16))
            return icon;
        else
            return icon32;
    }

    /* Descriptor of valid properties
    * @return array of properties
    */
    public PropertyDescriptor[] getPropertyDescriptors () {
        return desc;
    }


    public BeanDescriptor getBeanDescriptor(){
        //D.deb("getBeanDescriptor()"); // NOI18N
        BeanDescriptor bd = new BeanDescriptor(CvsFileSystem.class,
                                               org.netbeans.modules.vcs.cmdline.CvsCustomizer.Dialog.class);
        bd.setValue(VcsFileSystem.VCS_PROVIDER_ATTRIBUTE, new Boolean(true));
        return bd;
                                  //CvsCustomizerPanel.Dialog.class);
    }

}

/*
* Log
*  8    Jaga      1.6.1.0     2/24/00  Martin Entlicher Locking added.
*  7    Gandalf   1.6         1/6/00   Martin Entlicher 
*  6    Gandalf   1.5         10/25/99 Pavel Buzek     
*  5    Gandalf   1.4         10/23/99 Ian Formanek    NO SEMANTIC CHANGE - Sun 
*       Microsystems Copyright in File Comment
*  4    Gandalf   1.3         10/12/99 Pavel Buzek     
*  3    Gandalf   1.2         10/8/99  Pavel Buzek     icon changed
*  2    Gandalf   1.1         10/5/99  Pavel Buzek     VCS at least can be 
*       mounted
*  1    Gandalf   1.0         9/30/99  Pavel Buzek     
* $
*/
