/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.vcs.cmdline.commands;

import java.util.*;

import org.netbeans.modules.vcscore.VcsFileSystem;
import org.netbeans.modules.vcscore.commands.CommandOutputListener;
import org.netbeans.modules.vcscore.commands.CommandDataOutputListener;
import org.netbeans.modules.vcscore.cmdline.*;

import org.netbeans.modules.vcs.cmdline.CvsFileSystem;
import org.netbeans.modules.vcs.cmdline.passwd.LoginDialog;

/**
 * This class is used as a CVS login command.
 * @author  Martin Entlicher
 */
public class CvsLogin implements VcsAdditionalCommand {

    private VcsFileSystem fileSystem = null;

    public void setFileSystem(VcsFileSystem fileSystem) {
        this.fileSystem = fileSystem;
    }

    public boolean exec(Hashtable vars, String[] args,
                        CommandOutputListener stdoutNRListener, CommandOutputListener stderrNRListener,
                        CommandDataOutputListener stdoutListener, String dataRegex,
                        CommandDataOutputListener stderrListener, String errorRegex) {

        if (fileSystem instanceof CvsFileSystem) {
            CvsFileSystem cvsFileSystem = (CvsFileSystem) fileSystem;
            LoginDialog login = LoginDialog.createDialog(cvsFileSystem);
            login.setPserverName(cvsFileSystem.getCvsServer());
            String connectStr = ":pserver:" + cvsFileSystem.getCvsUserName()
                                + "@" + cvsFileSystem.getCvsServer()
                                + ":" + cvsFileSystem.getCvsRoot();
            login.setConnectString(connectStr);
            login.show();
            boolean loginCancelled = !login.isOffline();
            boolean isLoggedIn = login.isLoggedIn();
            if (!isLoggedIn && !loginCancelled) {
                fileSystem.setOffLine(true); // set offline mode
            } else if (isLoggedIn) {
                fileSystem.setOffLine(false); // unset offline mode
            }
            return isLoggedIn || loginCancelled;
        }
        return false;
    }
}

