/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.web.core;

import org.openide.compiler.Compiler;
import org.openide.cookies.EditorCookie;
import javax.swing.Action;
import java.io.IOException;
import java.util.Hashtable;
import java.util.HashMap;
import java.util.Vector;

import org.openide.ServiceType;
import org.openide.text.EditorSupport;
import org.openide.text.Line;
import org.openide.util.NbBundle;
import org.openide.loaders.MultiDataObject;

import org.netbeans.modules.web.core.jsploader.JspDataObject;
import org.netbeans.modules.web.core.jsploader.SourceMangler;
import org.netbeans.modules.web.core.execution.ExecParams;
import org.netbeans.modules.web.core.execution.DefaultExecParams;

public class FeatureFactory {

    private static FeatureFactory theFactory; // Singleton Factory instance
    
    /** HashMap of LanguageDescriptor instances for the content language. */
    private static HashMap contentLanguageDescriptors = new HashMap();
    
    /** HashMap of LanguageDescriptor instances for the scripting language. */
    private static HashMap scriptingLanguageDescriptors = new HashMap();
    
    public static FeatureFactory getFactory() {
	if ( null == theFactory) {  // Hardwired TODO change me
	    theFactory = new FeatureFactory();
	}
	return theFactory;
    }

    public static void setFeatureFactory(FeatureFactory f) {
	if ( null == f) {
	    throw new IllegalArgumentException("null == f");
	}
	theFactory = f;
    }
    
    
    /** Returns all content language descriptors registered in the content language registry. */
    public static LanguageDescriptor[] getJSPContentLanguages() {
        return (LanguageDescriptor[])contentLanguageDescriptors.values().toArray(
            new LanguageDescriptor[contentLanguageDescriptors.size()]);
    }
    
    /** Finds a content language descriptor for a given MIME type. */
    public static LanguageDescriptor findJSPContentLanguage(String mimeType) {
        return (LanguageDescriptor)contentLanguageDescriptors.get(mimeType);
    }
    
    /** Adds a content language descriptor to the content language registry. */
    public static void addJSPContentLanguage(LanguageDescriptor ld) {
        contentLanguageDescriptors.put(ld.getMIMEType(), ld);
    }
    
    /** Removes a content language descriptor from the content language registry. */
    public static void removeJSPContentLanguage(LanguageDescriptor ld) {
        contentLanguageDescriptors.remove(ld.getMIMEType());
    }
    

    
    /** Returns all scripting language descriptors registered in the scripting language registry. */
    public static LanguageDescriptor[] getJSPScriptingLanguages() {
        return (LanguageDescriptor[])scriptingLanguageDescriptors.values().toArray(
            new LanguageDescriptor[scriptingLanguageDescriptors.size()]);
    }
    
    /** Finds a scripting language descriptor for a given MIME type. */
    public static LanguageDescriptor findJSPScriptingLanguage(String mimeType) {
        return (LanguageDescriptor)scriptingLanguageDescriptors.get(mimeType);
    }
    
    /** Adds a scripting language descriptor to the scripting language registry. */
    public static void addJSPScriptingLanguage(LanguageDescriptor ld) {
        scriptingLanguageDescriptors.put(ld.getMIMEType(), ld);
    }
    
    /** Removes a scripting language descriptor from the scripting language registry. */
    public static void removeJSPScriptingLanguage(LanguageDescriptor ld) {
        scriptingLanguageDescriptors.remove(ld.getMIMEType());
    }
    
    
    
    /*public LanguageDescriptor[] getJSPContentLanguages() {
        ServletSettings settings = ServletSettings.OPTIONS;
        WebBrowser browser = settings.getWWWBrowser();
        return new LanguageDescriptor[] {
            // html
            new LanguageDescriptor("text/html", 
                NbBundle.getBundle(FeatureFactory.class).getString("CTL_lang_html"), browser),
            // plain text
            new LanguageDescriptor("text/plain", 
                NbBundle.getBundle(FeatureFactory.class).getString("CTL_lang_plain"), browser)
        };
    }

    public LanguageDescriptor[] getJSPScriptingLanguages() {
        return new LanguageDescriptor[] {
            // html
            new LanguageDescriptor("text/x-java", 
                NbBundle.getBundle(FeatureFactory.class).getString("CTL_scrlang_java"), null),
            // plain text
            new LanguageDescriptor("text/plain", 
                NbBundle.getBundle(FeatureFactory.class).getString("CTL_scrlang_plain"), null)
        };
    }*/
    
    public ExecParams createExecParams(ServiceType service) {
        return new DefaultExecParams();
    }

    public SourceMangler getSourceMangler(java.io.Reader servletReader) throws IOException {
	return null;
    }

}
