/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.web.core;

import java.beans.*;
import org.openide.awt.HtmlBrowser;

/** Class which describes a language
 *
 * @author  pjiricka
 * @version 
 */
public class LanguageDescriptor extends Object {
    
    protected String mimeType;
    
    protected String displayName;
    
    protected WebBrowser.Factory webBrowserFactory;
    
    private ServletSettings settings;
    
    private HtmlBrowser.BrowserComponent browser;

    /** Creates new LanguageDescriptor. The browser factory may be null. */
    public LanguageDescriptor(String mimeType, String displayName, WebBrowser.Factory webBrowserFactory) {
        this.mimeType = mimeType;
        this.displayName = displayName;
        this.webBrowserFactory = webBrowserFactory;
        settings = (ServletSettings)org.openide.options.SystemOption.findObject (ServletSettings.class);
        settings.addPropertyChangeListener (new PropertyChangeListener () {
            public void propertyChange (PropertyChangeEvent ev) {
                if (ServletSettings.PROP_WWWBROWSER.equals (ev.getPropertyName ())) {
                    browser = null;
                }
            }
        });
    }

    public boolean equals(Object other) {
        return (other instanceof LanguageDescriptor) ? mimeType.equals(((LanguageDescriptor)other).mimeType) : false;
    }
    
    public String getMIMEType() {
        return mimeType;
    }
    
    public String getDisplayName() {
        return displayName;
    }
    
    public HtmlBrowser.BrowserComponent getWebBrowser() {
        if (browser == null) {
            browser = new HtmlBrowser.BrowserComponent (settings.getWWWBrowser (), true, true);
        }
        return browser;
    }

}