/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.web.core;

import java.io.IOException;
import java.io.File;
import java.io.OutputStream;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.ResourceBundle;
import java.util.Properties;
import java.net.URL;
import java.net.MalformedURLException;
import java.net.InetAddress;
import java.net.UnknownHostException;

import org.openide.awt.HtmlBrowser;
import org.openide.cookies.InstanceCookie;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObject;
import org.openide.nodes.Node;
import org.openide.options.SystemOption;
import org.openide.options.ContextSystemOption;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.filesystems.FileObject;
import org.openide.TopManager;
import org.openide.execution.NbProcessDescriptor;
import org.openide.compiler.CompilerType;

import org.netbeans.modules.java.settings.JavaSettings;

/** Options for servlet support
*
* @author Petr Jiricka
* @version 0.01, May 18, 1999
*/
public class ServletSettings extends SystemOption implements WebBrowser.Factory {

    /** serialVersionUID */
    private static final long serialVersionUID = 4086443781681509635L;

    public static final String PROP_COMPILER = "compiler"; // NOI18N
    public static final String PROP_COMPILE_ERRORPAGE = "compileErrorPage"; // NOI18N
    public static final String PROP_COMPILE_INCL_FORW = "compileIncludedForwarded"; // NOI18N
    public static final String PROP_WWWBROWSER = "WebBrowser"; // NOI18N

    /** jswdk settings */
    public final static ServletSettings OPTIONS = new ServletSettings();


    public ServletSettings() {
        //addOption ((WebBrowserSettings)SystemOption.findObject(WebBrowserSettings.class, true));
    }

    /** human presentable name */
    public String displayName() {
        return NbBundle.getBundle(ServletSettings.class).getString("CTL_servlet_settings");
    }

    public HelpCtx getHelpCtx () {
        return new HelpCtx (ServletSettings.class);
    }


    /** @return CompilerType */
    public CompilerType getCompiler() {
        CompilerType.Handle compilerType = (CompilerType.Handle) getProperty(PROP_COMPILER);
        if (compilerType != null) {
            return (CompilerType) compilerType.getServiceType();
        }
        else {
            return ((JavaSettings) JavaSettings.findObject(JavaSettings.class, true)).getCompiler();
        }
    }

    /** Usese given CompilerType */
    public void setCompiler(CompilerType ct) {
        putProperty(PROP_COMPILER, new CompilerType.Handle(ct), true);
    }

    /** Returns true if the compilation should also compile jsp:forward and jsp:include-d pages. */
    public boolean isCompileIncludedForwarded() {
        Boolean isInclForw = (Boolean) getProperty(PROP_COMPILE_INCL_FORW);
        return (isInclForw == null) ? true : isInclForw.booleanValue();
    }

    /** Sets whether the compilation should also compile jsp:forward and jsp:include-d pages. */
    public void setCompileIncludedForwarded(boolean isInclForw) {
        putProperty(PROP_COMPILE_INCL_FORW, new Boolean(isInclForw), true);
    }

    /** Returns true if the compilation should also compile errorpage. */
    public boolean isCompileErrorPage() {
        Boolean isErrorP = (Boolean) getProperty(PROP_COMPILE_ERRORPAGE);
        return (isErrorP == null) ? true : isErrorP.booleanValue();
    }

    // Node.Handle is stored to refer to registered browser
    
    /** Use given WWWBrowser */
    public void setWWWBrowser(HtmlBrowser.Factory brow) {
        try {
            if (brow == null) {
                putProperty(PROP_WWWBROWSER, brow, true);    
                return;
            }

            FileObject fo = TopManager.getDefault ().getRepository ()
                .getDefaultFileSystem ().findResource ("Services/Browsers");   // NOI18N
            DataFolder folder = DataFolder.findFolder (fo);
            DataObject [] dobjs = folder.getChildren ();
            for (int i = 0; i<dobjs.length; i++) {
                Object o = ((InstanceCookie)dobjs[i].getCookie (InstanceCookie.class)).instanceCreate ();
                if ((o != null) && o.equals (brow)) {
                    putProperty(PROP_WWWBROWSER, dobjs[i].getNodeDelegate ().getHandle (), true);    
                    break;
                }
            }
        }
        catch (Exception ex) {
            TopManager.getDefault ().notifyException (ex);
        }
    }

    public HtmlBrowser.Factory getWWWBrowser() {
        try {
            Node.Handle hdl = (Node.Handle) getProperty(PROP_WWWBROWSER);
            if (hdl == null)
                return null;
            
            Node n = hdl.getNode ();
            Object o = ((InstanceCookie) n.getCookie (InstanceCookie.class)).instanceCreate ();
            return (HtmlBrowser.Factory)o;
        }
        catch (IOException ex) {
            if (Boolean.getBoolean ("netbeans.debug.exceptions"))   // NOI18N
                ex.printStackTrace ();
            return null;
        }
        catch (ClassNotFoundException ex) {
            if (Boolean.getBoolean ("netbeans.debug.exceptions"))   // NOI18N
                ex.printStackTrace ();
            return null;
        }
        catch (NullPointerException ex) {
            if (Boolean.getBoolean ("netbeans.debug.exceptions"))   // NOI18N
                ex.printStackTrace ();
            return null;
        }
    }

    /** Sets whether the compilation should also compile errorpage. */
    public void setCompileErrorPage(boolean isErrorP) {
        putProperty(PROP_COMPILE_ERRORPAGE, new Boolean(isErrorP), true);
    }

    /* Access the firePropertyChange from HTTPServer (which holds the enabled prop). */
    void firePropertyChange0 (String name, Object oldVal, Object newVal) {
        firePropertyChange (name, oldVal, newVal);
    }

    /* Implementation of WebBrower.Factory. */
    public WebBrowser getBrowser() {
        // return getWWWBrowser();
        return null;
    }
    
}

