/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.web.core;

import java.util.Enumeration;
import java.util.MissingResourceException;
import java.io.*;
//import java.lang.reflect.Method;
//import java.lang.reflect.InvocationTargetException;

import org.openide.modules.ModuleInstall;
import org.openide.execution.Executor;
import org.openide.execution.ExecutorTask;
import org.openide.execution.ProcessExecutor;
import org.openide.util.NbBundle;
import org.openide.loaders.DataObject;
import org.openide.TopManager;
import org.openide.NotifyDescriptor;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileLock;
import org.openide.filesystems.FileAlreadyLockedException;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.InstanceDataObject;
import org.openide.loaders.XMLDataObject;
import org.openide.loaders.DataFolder;

import org.netbeans.modules.web.core.execution.JakartaExecutor;
import org.netbeans.modules.web.core.execution.Execute2Action;
import org.netbeans.modules.web.core.jsploader.EditServletAction;

//import org.netbeans.modules.web.wizards.beanjsp.ide.JSPPageWizardAction;

//import org.netbeans.modules.web.wizards.beanjsp.ide.JSPPageWizardModule;
//import org.netbeans.modules.web.templates.nbcontrol.DWModuleInstaller;

/**
* Module installation class for servlet support
*
* @author Petr Jiricka
*/
public class ServletSupportModule extends ModuleInstall {

    /** serialVersionUID */
    private static final long serialVersionUID = -4736324822902413381L;

    //JSPPageWizardModule wizardModule = new JSPPageWizardModule();
    //DWModuleInstaller dwModule = new DWModuleInstaller();

    /** Module installed for the first time. */
    public void installed() {
        // 1. copy Servlet templates
        //copyTemplates ();

        // install actions
        //installActions();

        /*try {
            wizardModule.installed(this);
            dwModule.installed(this);
        }
        catch (MissingResourceException e) {
            TopManager.getDefault().notifyException(e);
        }*/

        restored ();
    }

    /** Module installed again.
    * Add applet executor
    */
    public void restored() {

        // install JSP syntax coloring
        //addEditorInitializer();
        //installEditorOptions();
        installLanguageDescriptors();

        // Adding Servlet Settings Initializer
        //org.netbeans.editor.Settings.addInitializer(new org.netbeans.modules.web.core.jsploader.ServletSettingsInitializer());
        
        // set correct MIME type for .tld files
        try {
            FileUtil.setMIMEType("tld", "text/xml");
        }
        catch (IllegalArgumentException e) {}
        registerDTDResources();

        //wizardModule.restored(this);
        //dwModule.restored(this);
    }

    /** Module was uninstalled. */
    public void uninstalled() {

        // uninstall options from the editor context option
        //uninstallEditorOptions();
        uninstallLanguageDescriptors();

        // uninstall actions
        //uninstallActions();

        //wizardModule.uninstalled(this);
        //dwModule.uninstalled(this);
    }

    /** Module is being closed. */
    public boolean closing () {

        /*if (!wizardModule.closing(this))
            return false;
        if (!dwModule.closing(this))
            return false;*/

        return true; // agree to close
    }

    /** Module is being closed. */
    public void close () {

        JakartaExecutor.killServerIfRunning();

        //wizardModule.close(this);
        //dwModule.close(this);
    }


    // -----------------------------------------------------------------------------
    // Private methods
    
    private void registerDTDResources() {
        // Web application DTD
        XMLDataObject.registerCatalogEntry(
            "-//Sun Microsystems, Inc.//DTD Web Application 2.2//EN",
            "org/netbeans/modules/web/core/resources/web.dtd",
            ServletSupportModule.class.getClassLoader ());

        // Tag library DTD
        XMLDataObject.registerCatalogEntry(
            "-//Sun Microsystems, Inc.//DTD JSP Tag Library 1.1//EN",
            "org/netbeans/modules/web/core/resources/web-jsptaglib_1_1.dtd",
            ServletSupportModule.class.getClassLoader ());
    }

    /*
    // editor
    private void addEditorInitializer() {
        try {
            Class settings = Class.forName
                             ("org.netbeans.editor.Settings", // NOI18N
                              false, this.getClass().getClassLoader()); // only test for editor module

            Class restore = Class.forName
                            ("org.netbeans.modules.web.core.syntax.RestoreColoring", // NOI18N
                             false, this.getClass().getClassLoader());
            Method restoreMethod = restore.getMethod ("addInitializer", null); // NOI18N
            restoreMethod.invoke (restore.newInstance(), null);
        } catch (ClassNotFoundException e) {
        } catch (NoSuchMethodException e) {
        } catch (InvocationTargetException e) {
        } catch (IllegalAccessException e) {
        } catch (InstantiationException e) {
        }
    }

    // editor
    private void installEditorOptions() {
        try {
            Class settings = Class.forName
                             ("org.netbeans.editor.Settings", // NOI18N
                              false, this.getClass().getClassLoader()); // only test for editor module

            Class restore = Class.forName
                            ("org.netbeans.modules.web.core.syntax.RestoreColoring", // NOI18N
                             false, this.getClass().getClassLoader());
            Method restoreMethod = restore.getMethod ("installOptions", null); // NOI18N
            restoreMethod.invoke (restore.newInstance(), null);
        } catch (ClassNotFoundException e) {
        } catch (NoSuchMethodException e) {
        } catch (InvocationTargetException e) {
        } catch (IllegalAccessException e) {
        } catch (InstantiationException e) {
        }
    }

    // editor
    private void uninstallEditorOptions() {
        try {
            Class settings = Class.forName
                             ("org.netbeans.editor.Settings", // NOI18N
                              false, this.getClass().getClassLoader()); // only test for editor module

            Class restore = Class.forName
                            ("org.netbeans.modules.web.core.syntax.RestoreColoring", // NOI18N
                             false, this.getClass().getClassLoader());
            Method restoreMethod = restore.getMethod ("uninstallOptions", null); // NOI18N
            restoreMethod.invoke (restore.newInstance(), null);
        } catch (ClassNotFoundException e) {
        } catch (NoSuchMethodException e) {
        } catch (InvocationTargetException e) {
        } catch (IllegalAccessException e) {
        } catch (InstantiationException e) {
        }
    }
    */

    private void installLanguageDescriptors() {
        WebBrowser.Factory factory = ServletSettings.OPTIONS;
        
        // html content language
        FeatureFactory.addJSPContentLanguage(
            new LanguageDescriptor("text/html", 
                NbBundle.getBundle(FeatureFactory.class).getString("CTL_lang_html"), factory)
        );
        
        // plain text content language
        FeatureFactory.addJSPContentLanguage(
            new LanguageDescriptor("text/plain", 
                NbBundle.getBundle(FeatureFactory.class).getString("CTL_lang_plain"), factory)
        );
        
        // java scripting language
        FeatureFactory.addJSPScriptingLanguage(
            new LanguageDescriptor("text/x-java", 
                NbBundle.getBundle(FeatureFactory.class).getString("CTL_scrlang_java"), null)
        );
        
        // plain text scripting language
        FeatureFactory.addJSPScriptingLanguage(
            new LanguageDescriptor("text/plain", 
                NbBundle.getBundle(FeatureFactory.class).getString("CTL_scrlang_plain"), null)
        );
    }

    private void uninstallLanguageDescriptors() {
        
        LanguageDescriptor ld;
        
        // html content language
        ld = FeatureFactory.findJSPContentLanguage("text/html");
        if (ld != null)
            FeatureFactory.removeJSPContentLanguage(ld);
        
        // plain text content language
        ld = FeatureFactory.findJSPContentLanguage("text/plain");
        if (ld != null)
            FeatureFactory.removeJSPContentLanguage(ld);
        
        // java scripting language
        ld = FeatureFactory.findJSPScriptingLanguage("text/x-java");
        if (ld != null)
            FeatureFactory.removeJSPScriptingLanguage(ld);
        
        // plain text scripting language
        ld = FeatureFactory.findJSPScriptingLanguage("text/plain");
        if (ld != null)
            FeatureFactory.removeJSPScriptingLanguage(ld);
    }

    /*private void installActions () {
        try {
            Utilities2.createAction (EditQueryStringAction.class,
                                     DataFolder.create (org.openide.TopManager.getDefault ().getPlaces ().folders ().actions (), "Build")); // NOI18N

            Utilities2.createAction (Execute2Action.class,
                                     DataFolder.create (org.openide.TopManager.getDefault ().getPlaces ().folders ().actions (), "Build")); // NOI18N

            Utilities2.createAction (EditServletAction.class,
                                     DataFolder.create (org.openide.TopManager.getDefault ().getPlaces ().folders ().actions (), "System")); // NOI18N

            //installWizardActions();
        } catch (Exception e) {
            if (System.getProperty ("netbeans.debug.exceptions") != null) {
                e.printStackTrace ();
            }
            // ignore failure to install
        }
    }*/


/*    private void installWizardActions () throws IOException {
        Utilities2.createAction (JSPPageWizardAction.class,
                                 DataFolder.create (org.openide.TopManager.getDefault ().getPlaces ().folders().menus (), "Tools"), // NOI18N
                                 "UnmountFSAction", true, true, false, false // NOI18N
                                );
        Utilities2.createAction (JSPPageWizardAction.class,
                                 DataFolder.create (org.openide.TopManager.getDefault ().getPlaces ().folders().actions (), "Tools") // NOI18N
                                );
    }*/


    /*private void uninstallActions () {
        try {
            Utilities2.removeAction (EditQueryStringAction.class,
                                     DataFolder.create (org.openide.TopManager.getDefault ().getPlaces ().folders ().actions (), "Build")); // NOI18N

            Utilities2.removeAction (Execute2Action.class,
                                     DataFolder.create (org.openide.TopManager.getDefault ().getPlaces ().folders ().actions (), "Build")); // NOI18N

            Utilities2.removeAction (EditServletAction.class,
                                     DataFolder.create (org.openide.TopManager.getDefault ().getPlaces ().folders ().actions (), "System")); // NOI18N

            //uninstallWizardActions();
        } catch (Exception e) {
            if (System.getProperty ("netbeans.debug.exceptions") != null) {
                e.printStackTrace ();
            }
            // ignore failure to install
        }
    }*/

/*    private void uninstallWizardActions () throws IOException {
        Utilities2.removeAction (JSPPageWizardAction.class,
                                 DataFolder.create (org.openide.TopManager.getDefault ().getPlaces ().folders().menus (), "Tools")); // NOI18N
        Utilities2.removeAction (JSPPageWizardAction.class,
                                 DataFolder.create (org.openide.TopManager.getDefault ().getPlaces ().folders().actions (), "Tools")); // NOI18N
    }*/

    /*private void copyTemplates () {
        copyTemplate("org.netbeans.modules.web.core.templates");
        // copyTemplate("org.netbeans.modules.web.templates.nbcontrol.templates");
    }*/

/*    private void copyTemplate(String resource) {
        try {
            FileUtil.extractJar(TopManager.getDefault().getPlaces().folders().templates().getPrimaryFile(),NbBundle.getLocalizedFile(resource,"jar").openStream());
        } 
        catch (IOException e) {
            TopManager.getDefault().notifyException(e);
        }
        catch (MissingResourceException e) {
            TopManager.getDefault().notifyException(e);
        }
    }*/

}

