/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.web.core;

import java.util.ArrayList;
import java.io.Serializable;
import java.net.URL;
import java.awt.Component;

/** 
 *
 * @author  pjiricka
 * @version 
 */
public class WebBrowser extends Object implements Serializable, Cloneable {

    public static final String BROWSER_WWW = "wwwBrowser";
    public static final String BROWSER_XML = "xmlBrowser";
    public static final String BROWSER_WAP = "wapBrowser";

    private ArrayList browsers;
    private int currentValue = 0;
  
    /** Creates new WebBrowser */
    public WebBrowser() {
        browsers = new ArrayList();
    }

    private WebBrowser(ArrayList browsers, int currentValue) {
        this.browsers = browsers;
        this.currentValue = currentValue;
    }

    public Object clone() {
        return new WebBrowser(browsers, currentValue);
    }

    public void showURL(URL url) {
        if (getImpl() == null)
            return;
        getImpl().showUrl(url);
    }

    public void addImpl(Impl browserImpl) {
        browsers.add(browserImpl);
    }
    
    public Impl[] getAllImpls() {
        return (Impl[])browsers.toArray(new Impl[browsers.size()]);
    }
    
    public int getImplIndex() {
        if (browsers.size() == 0)
            return -1;
        return currentValue;
    }
    
    public Impl getImpl() {
        if (browsers.size() == 0)
            return null;
        return (Impl)browsers.get(currentValue);
    }
    
    public void setImpl(Impl impl) {
        int newIndex = browsers.indexOf(impl);
        if (newIndex == -1)
            throw new IllegalArgumentException();
        currentValue = newIndex;
    }
    
    public static interface Impl extends Serializable {
      
        /** The implementation should display the given URL. */
        public void showUrl(URL url);
        
        /** Return the display name of this browser. */
        public String getDisplayName();

        /** Return the component used for configuring this web browser. */
        public Component getConfigComponent();
        
        /** The property editor notifies that editing the configuration stopped. */
        public void stopConfigEditing();
        
    }
    
    public static interface Factory {
        
        public WebBrowser getBrowser();
        
    }
  
}