/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.web.core;

import java.awt.*;
import javax.swing.*;

import org.openide.explorer.propertysheet.editors.EnhancedCustomPropertyEditor;
import org.openide.util.NbBundle;

/** 
 *
 * @author  pjiricka
 * @version 
 */
public class WebBrowserCustomEditor extends javax.swing.JPanel implements EnhancedCustomPropertyEditor {
  
    private WebBrowser browser;
    
    private ButtonGroup buttonGroup;
    private JRadioButton buttons[];

    /** Creates new form WebBrowserCustomEditor */
    public WebBrowserCustomEditor(WebBrowser browser) {
        this.browser = browser;
        buttonGroup = new ButtonGroup();
        buttons = new JRadioButton[browser.getAllImpls().length];
        initComponents ();
    }

    public Object getPropertyValue() {
        WebBrowser.Impl impls[] = browser.getAllImpls();
        for (int i = 0; i < impls.length; i++) {
            impls[i].stopConfigEditing();
            if (buttons[i].isSelected())
                browser.setImpl(impls[i]);
        }
        return browser.clone();
    }
  
  /** This method is called from within the constructor to
   * initialize the form.
   */
    private void initComponents() {
        setLayout(new java.awt.GridBagLayout());
        setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(1, 1, 8, 1)));
        WebBrowser.Impl[] impls = browser.getAllImpls();
        for (int i = 0; i < impls.length; i++) {
            JPanel implPanel = new JPanel();
            implPanel.setLayout(new java.awt.GridBagLayout());
            implPanel.setBorder(new javax.swing.border.EtchedBorder());
            
            // add button to the panel
            buttons[i] = new JRadioButton();
            buttons[i].setText(impls[i].getDisplayName());
            GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
            gridBagConstraints1.gridwidth = 0;
            gridBagConstraints1.gridheight = -1;
            gridBagConstraints1.insets = new java.awt.Insets(8, 8, 8, 8);
            gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
            gridBagConstraints1.weightx = 1.0;
            implPanel.add(buttons[i], gridBagConstraints1);
            buttonGroup.add(buttons[i]);
          
            // add config component to the panel
            Component c = impls[i].getConfigComponent();
            if (c != null) {
                gridBagConstraints1 = new GridBagConstraints();
                gridBagConstraints1.gridwidth = 0;
                gridBagConstraints1.gridheight = 0;
                gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
                gridBagConstraints1.weightx = 1.0;
                gridBagConstraints1.weighty = 1.0;
                implPanel.add(c, gridBagConstraints1);
            }
            
            // add the panel to the big panel
            gridBagConstraints1 = new GridBagConstraints();
            gridBagConstraints1.gridwidth = 0;
            gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
            gridBagConstraints1.insets = new java.awt.Insets(8, 8, 0, 8);
            gridBagConstraints1.weightx = 1.0;
            //gridBagConstraints1.weighty = 1.0;
            add(implPanel, gridBagConstraints1);
        }
        
        // set the current impl
        buttons[browser.getImplIndex()].setSelected(true);
        
        // panel for filling
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridwidth = 0;
        gbc.gridheight = 0;
        gbc.fill = java.awt.GridBagConstraints.BOTH;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        add(new JPanel(), gbc);
    }
}