/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.web.core;

import java.net.URL;
import java.io.IOException;
import java.awt.Component;
import java.awt.BorderLayout;
import javax.swing.border.TitledBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.JPanel;
import java.beans.*;

import org.openide.TopManager;
import org.openide.NotifyDescriptor;
import org.openide.util.NbBundle;
import org.openide.execution.NbProcessDescriptor;
import org.openide.execution.ExecInfo;
import org.openide.execution.ProcessExecutor;

import org.openide.explorer.propertysheet.editors.EnhancedCustomPropertyEditor;

/** 
 *
 * @author  pjiricka
 * @version 
 */
public class WebBrowserImpl extends Object {

    private static final java.util.ResourceBundle bundle = NbBundle.getBundle(WebBrowserImpl.class);

    private static NbProcessDescriptor DEFAULT_EXTERNAL_BROWSER = new NbProcessDescriptor(
        // empty string for process
        "", // NOI18N
        // {URL}
        " {" + BrowserFormat.TAG_URL + "}", // NOI18N
        NbBundle.getBundle(WebBrowserImpl.class).getString("MSG_BrowserExecutorHint")
    );

    public static WebBrowser getWWWBrowser() {
        WebBrowser browser = new WebBrowser();
        browser.addImpl(new InternalWWWBrowser());
        browser.addImpl(new ExternalBrowser());
        return browser;
    }

    public static WebBrowser getXMLBrowser() {
        WebBrowser browser = new WebBrowser();
        browser.addImpl(new ExternalBrowser());
        return browser;
    }

    public static WebBrowser getWAPBrowser() {
        WebBrowser browser = new WebBrowser();
        browser.addImpl(new ExternalBrowser());
        return browser;
    }

    public static class InternalWWWBrowser implements WebBrowser.Impl {

        public void showUrl(URL url) {
            TopManager.getDefault().showUrl(url);
        }
        
        public Component getConfigComponent() {
            return null;
        }
        
        public String getDisplayName() {
            return bundle.getString("CTL_InternalBrowser");
        }

        public void stopConfigEditing() {
        }
        
    }
  
    public static class ExternalBrowser implements WebBrowser.Impl {
      
        /** serialVersionUID */
        private static final long serialVersionUID = -8492050909772075794L;

        private NbProcessDescriptor process;
        private transient Component currentEditor;

        public ExternalBrowser() {
            process = DEFAULT_EXTERNAL_BROWSER;
        }
        
        public NbProcessDescriptor getProcess() {
            return process;
        }
        
        public void setProcess(NbProcessDescriptor process) {
            this.process = process;
        }

        public void showUrl(URL url) {
            try {
              process.exec(new BrowserFormat(new ExecInfo(""), url.toString())); // NOI18N
            } catch (IOException ex) {
              TopManager.getDefault().notify(
                new NotifyDescriptor.Exception(ex,
                  NbBundle.getBundle(WebBrowserImpl.class).getString("EXC_Invalid_Processor")
                )
              );
            }
        }
        
        public String getDisplayName() {
            return bundle.getString("CTL_ExternalBrowser");
        }

        public Component getConfigComponent() {
            /*JPanel editorPanel = new JPanel();
            editorPanel.setLayout(new BorderLayout());
            editorPanel.setBorder(new TitledBorder(
                new EtchedBorder(),
                NbBundle.getBundle(WebBrowserImpl.class).getString("CTL_EXTBROWSER_SETTINGS")));*/

            PropertyEditor edit = PropertyEditorManager.findEditor(NbProcessDescriptor.class);
            edit.setValue(process);
            currentEditor = edit.getCustomEditor();

            /*editorPanel.add(currentEditor, BorderLayout.CENTER);
            return editorPanel;*/
            return currentEditor;
        }
        
        public void stopConfigEditing() {
            if (currentEditor instanceof EnhancedCustomPropertyEditor) {
                EnhancedCustomPropertyEditor editor = (EnhancedCustomPropertyEditor)currentEditor;
                setProcess((NbProcessDescriptor)editor.getPropertyValue());
            }
        }
        
    }
  
    /** Default format that can format tags related to execution. Currently this is only the URL.
    */
    private static class BrowserFormat extends ProcessExecutor.Format {
        /** Tag replaced with the URL */
        public static final String TAG_URL = "URL"; // NOI18N

        static final long serialVersionUID =4315554797414856261L;
        /** @param info exec info about class to execute
        * @param classPath to substitute instead of CLASSPATH
        * @param bootClassPath boot class path
        * @param repository repository path
        * @param library library path
        */
        public BrowserFormat (ExecInfo info, String url) {
            super(info);
            java.util.Map map = getMap ();

            map.put (TAG_URL, url);
        }

    }
    
}