/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.web.core;

import java.io.IOException;
import java.io.NotActiveException;
import java.io.BufferedReader;
import java.io.FileWriter;
import java.io.OutputStream;
import java.io.FileOutputStream;
import java.io.BufferedWriter;
import java.io.InputStreamReader;
import java.io.File;
import java.net.MalformedURLException;
import java.net.UnknownHostException;
import java.net.URL;
import java.net.InetAddress;
import java.text.MessageFormat;

import org.openide.TopManager;
import org.openide.ErrorManager;
import org.openide.NotifyDescriptor;
import org.openide.awt.HtmlBrowser;
import org.openide.execution.NbProcessDescriptor;
import org.openide.execution.NbClassPath;
import org.openide.execution.ProcessExecutor;
import org.openide.execution.ExecInfo;
import org.openide.loaders.DataObject;
import org.openide.filesystems.FileObject;
import org.openide.util.NbBundle;
import org.openide.util.HttpServer;

import org.netbeans.modules.web.core.jsploader.JspCompileUtil;

/** Implementation of ExecCookie for internet objects
*
* @author Petr Jiricka
*/
public class WebExecUtil {

    /** constant for class extension */
    private static final String CLASS_EXT = "class"; // NOI18N
    /** constant for java extension */
    private static final String JAVA_EXT = "class"; // NOI18N

    public static final String DUMMY_RESOURCE = "/SOMENONSENSE_NONEXISTING_URL/xxxyyyzzz.nonexistingextension";

    public static boolean waitAndShowInBrowser(URL url, int timeout) {
        return waitAndShowInBrowser(url, "text/html", timeout); // PENDING
    }

    public static boolean waitAndShowInBrowser(URL url, String contentLanguage, int timeout) {
        URL connectURL;
        try {
            connectURL = new URL(url.getProtocol(), url.getHost(), url.getPort(), DUMMY_RESOURCE); // NOI18N
        }
        catch (MalformedURLException e) {
            connectURL = url;
        }
        if (Util.getUtil().waitForURLConnection(connectURL, timeout, 1000)) {
            showInBrowser(url, contentLanguage);
            return true;
        }
        else {
            //PENDING
            //TopManager.getDefault().notifyException(new IOException("Could not connect to the server")); // NOI18N
            return false;
        }
    }
    
    /** Shows URL url in appropriate browser (web browser, WAP browser etc.) 
     * @param contentLanguage MIME type of the content language of the page represented by URL url
     */
    public static void showInBrowser(URL url, String contentLanguage) {
        if (contentLanguage == null)
            contentLanguage = "text/html";
        LanguageDescriptor descriptor = LanguageEditor.findDescriptor(
            contentLanguage, FeatureFactory.getFactory().getJSPContentLanguages());
        if (descriptor == null) {
            // PENDING
            //TopManager.getDefault().notify(
            System.out.println("no descriptor found");
        }
        else {
            HtmlBrowser.BrowserComponent browser = descriptor.getWebBrowser();
            browser.setURL(url);
            browser.open ();
        }
    }

    /** Translates string name to a DataObject.
    * @param file
    * @return DataObject or null if not found 
    * @exception IOException if IO error occurs
    */
/*    public static DataObject classToDataObject(String className) throws IOException {
        String file = className.replace('.', '/');
        FileObject fo = TopManager.getDefault().getRepository().findResource(file + '.' + CLASS_EXT);
        if (fo == null) {
            fo = TopManager.getDefault().getRepository().findResource(file + '.' + JAVA_EXT);
        }
        if (fo == null)
            return null;

        return TopManager.getDefault().getLoaderPool().findDataObject(fo);
    }*/


    /** Returns string for localhost */
/*    public static String getLocalHost() {
        try {
            return InetAddress.getLocalHost().getHostName();
        }
        catch (UnknownHostException e) {
            return "localhost"; // NOI18N
        }
    }*/

    /** Gets the temporary directory file name, without the ending file separator.
    * Note that this directory does not need to exist.
    */
    public static String getTempDirectory() {
        String path = System.getProperty("netbeans.user");
        path = new File(path).getAbsolutePath();
        if (path == null || path.length() == 0) {
            FileObject sysRoot = TopManager.getDefault().getRepository().getDefaultFileSystem().getRoot();
            path = NbClassPath.toFile(sysRoot).getParentFile().getAbsolutePath();
            if (path == null)
                throw new InternalError();
        }
        if (!path.endsWith(File.separator))
            path = path + File.separator;
        path = path + "temp"; // NOI18N
        return path;
    }

    public static File getTomcatTemp() {
        String dirName = getTempDirectory() + File.separator + "jspwork";
        File ff = new File(dirName);
        myMkdirs(ff);
        return ff;
    }

    public static File getTomcatHome() {
        String dirName = getTempDirectory() + File.separator + "tomcat";
        File ff = new File(dirName);
        myMkdirs(ff);
        return ff;
    }

    /** Returns true if the dir exists when we finish */
    public static boolean myMkdirs(File f) {
        if (f.exists()) return true;
        if (!f.isAbsolute())
            f = f.getAbsoluteFile();
        String par = f.getParent();
        if (par == null) return false;
        if (!myMkdirs(new File(par))) return false;
        f.mkdir();
        return f.exists();
    }

/*    public static boolean isServlet(ExecInfo info) {
        return !(info instanceof WebExecInfo); // pending - should check whether it actually is a servlet
    }

    public static boolean isJsp(ExecInfo info) {
        return (info instanceof WebExecInfo);
    }*/

    // PENDING - context root path
/*    public static URL constructServletURL(DataObject servlet, int port) throws MalformedURLException, IOException {
        new Exception("not implemented").printStackTrace();
        // PENDING
        String mapping = "/servlet/" + servlet.getPrimaryFile().getPackageName('.'); // NOI18N
        URL url = new URL("http", getLocalHost(), port, mapping/\ + queryString\/); // NOI18N
        return url;
        /\    ServletExecParams sep = getServletExecParams((MultiDataObject)servlet);
            String queryString = sep.getQueryString();
            
            if ((queryString.length() > 0) && (!queryString.startsWith("?"))) // NOI18N
              queryString = "?" + queryString; // NOI18N
            
            String mapping = sep.getMapping();
            if (mapping.equals("") || mapping.startsWith(".")) { // NOI18N
              if (invoker)
                mapping = "/servlet/" + servlet.getPrimaryFile().getPackageName('.'); // NOI18N
              else
                throw new IOException(NbBundle.getBundle(EditServletParamsAction.class).getString("EXC_NoSuitableMapping"));
            }  
            
            URL url = new URL("http", WebExecSupport.getLocalHost(), port, mapping + queryString); // NOI18N
            return url;\/
    }*/

}



/*
 * Log
 *  10   Jaga      1.9         4/18/00  Petr Jiricka    Calling the monitor.
 *  9    Jaga      1.8         4/11/00  Petr Jiricka    Removed IOException from
 *       runMonitor()
 *  8    Jaga      1.7         4/11/00  Petr Jiricka    Public dummy URL.
 *  7    Jaga      1.6         4/11/00  Petr Jiricka    Monitor context DD.
 *  6    Jaga      1.5         4/11/00  Petr Jiricka    "Monitorroot" changed do
 *       "monitor".
 *  5    Jaga      1.4         4/10/00  Petr Jiricka    Fixes in execution and 
 *       debugging, integrating monitor.
 *  4    Jaga      1.3         4/5/00   Petr Jiricka    Deploy default DD 
 *       method.
 *  3    Jaga      1.2         4/5/00   Petr Jiricka    Fixing execution.
 *  2    Jaga      1.1         4/4/00   Petr Jiricka    Added parameters to 
 *       deploy method
 *  1    Jaga      1.0         4/4/00   Petr Jiricka    
 * $
 */
