/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.web.core.execution;

import java.net.URL;
import java.net.MalformedURLException;
import java.io.File;
import java.io.Serializable;
import java.io.IOException;
import java.beans.*;

import org.netbeans.modules.web.core.WebExecInfo;
/**
 *
 * @author  pjiricka
 * @version 
 */
public interface ExecParams extends Serializable {
    
    /** Returns a URL which should be displayed in the browser
     * when executing object represented by <code>executedObject</code>. 
     * May return null if the object is not appropriate for this executor.
     */
    public URL getURL(WebExecInfo executedObject) throws MalformedURLException, IOException;
    
    /** Returns the MIME type of the content language for object 
     * represented by <code>executedObject</code>. 
     * This is used to determine which browser (WWW, WAP, ...) to display the URL in.
     */
    public String getContentLanguage(WebExecInfo executedObject) throws IOException;
    
    /** Clones this parameters object. 
     */
    public Object clone() throws CloneNotSupportedException;
    
    /** Should determine whether the parameters have changed sufficiently since the 
     * last time so the server should be restarted. 
     * @param executedObject object which is now being executed
     * @param lastExecParams execution parameters used last time to start the server.
     *    may be null if the server was not executed yet
     * @return true if the server should be restarted
     */
    public boolean serverRestartRequired(WebExecInfo executedObject, ExecParams lastExecParams);
    
    /** Creates and deploys the Tomcat configuration file (server.xml) 
     * @return the configuration file
     */
    public File deployTomcatConfig(WebExecInfo executedObject) throws IOException;
    
    /** Directory containing the Tomcat home directory. 
     */
    public File getTomcatHome() throws IOException;
    
    /** Display name for the Tomcat process.
     */
    public String getProcessDisplayName();

    /** Allows the implementation to provide its own actions wehn an object is executed. 
     */
    public void runObject(WebExecInfo executedObject);
    
    public void addPropertyChangeListener (PropertyChangeListener listener);

    public void removePropertyChangeListener (PropertyChangeListener listener);
    
    public String executorDisplayName();
    
    public String debuggerTypeDisplayName();
    
}
