/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.web.core.execution;

import org.apache.tomcat.core.*;
import org.apache.tomcat.core.Constants;
import org.apache.tomcat.request.*;
import org.apache.tomcat.util.*;
import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.http.*;

import org.apache.tomcat.logging.*;

/**
 * Set the default properties of Tomcat context, override defaults set by Tomcat.
 * So this class actually does not run in Forte, but in the executed Tomcat process.
 *
 * @author Petr Jiricka
 */
public class ForteDefaults extends BaseInterceptor {

    private Properties props;

    public ForteDefaults() {
    }

    public void engineInit(ContextManager cm) throws TomcatException {
        try {
//System.out.println("tomcat home " + cm.getHome());
            File ff = new File(cm.getHome() + File.separator + "tempDirs.properties");
//System.out.println("loading " + ff);
            InputStream propStream = new FileInputStream(ff);
            props = new java.util.Properties();
            props.load(propStream);
            propStream.close();
        }
        catch (IOException e) {
            props = new java.util.Properties();
        }
    }

    /** Called when a new context is added to the server.
     *
     *  - Set defaults for WorkDir
     */
    public void addContext(ContextManager cm, Context ctx) throws TomcatException {
    }

    public void contextInit( Context ctx) throws TomcatException {
        // always set
        setWorkDir(ctx);

        if (! ctx.getWorkDir().exists()) {
            //log  System.out.println("Creating work dir " + ctx.getWorkDir() );
            ctx.getWorkDir().mkdirs();
        }
        ctx.setAttribute(Constants.ATTRIB_WORKDIR1, ctx.getWorkDir());
        ctx.setAttribute(Constants.ATTRIB_WORKDIR , ctx.getWorkDir());
    }

    private void setWorkDir(Context ctx ) {
        ContextManager cm=ctx.getContextManager();

        File wd = new File(cm.getWorkDir());
        String newWdName = (String)props.get(ctx.getDocBase());
        wd = (newWdName == null) ? 
            new File(wd, URLEncoder.encode(computeSystemName(ctx.getDocBase()))) :
            new File(newWdName);
        ctx.setWorkDir(wd);
//System.out.println("ForteDefaults setting workdir to " + wd);
    }

    // PENDING - taken from LocalFileSystem, may not work for other FS.
    private String computeSystemName (String rootFile) {
        return rootFile.replace(File.separatorChar, '/');
    }

}
