/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.web.core.jsploader;

import java.io.*;

import org.openide.filesystems.FileObject;
import org.openide.TopManager;

import org.apache.jasper.JspCompilationContext;
import org.apache.jasper.JasperException;
import org.apache.jasper.Options;
import org.apache.jasper.compiler.JspReader;
import org.apache.jasper.compiler.ServletWriter;
import org.apache.jasper.compiler.Compiler;
import org.apache.jasper.compiler.Mangler;

import org.netbeans.modules.web.core.jsploader.CompilationDescriptor;
/**
 * Holds data used on a per-page compilation context that would otherwise spill
 * over to other pages being compiled.  Things like the taglib classloaders
 * and directives.
 *
 *@author Danno Ferrin, Petr Jiricka
 */
public class AnalyzerCompilerContext implements JspCompilationContext {

    JspReader reader;
    //boolean errPage;
    String jspFile;
    String contentType;
    Options options;

    CompilationDescriptor cd;

    public AnalyzerCompilerContext(String newJspFile,
                                   CompilationDescriptor compDesc, Options newOptions) throws JasperException {
        jspFile = newJspFile;
        //errPage = newErrPage;
        options = newOptions;
        cd = compDesc;
    }

    /**
     * The classpath that is passed off to the Java compiler. 
     */
    public String getClassPath() {
        return "";
    }
    
    /** Add a jar to the classpath used by the loader
     */
    public void addJar( String jar ) throws IOException {
        if (Boolean.getBoolean("netbeans.debug.exceptions"))
            System.out.println("should add jar to classpath: " + jar);
        JspCompileUtil.mountJarIfNotMounted(jar);
    }

    /**
     * Get the input reader for the JSP text. 
     */
    public JspReader getReader() {
        return reader;
    }

    /**
     * Where is the servlet being generated?
     */
    public ServletWriter getWriter() {
        throw new UnsupportedOperationException();
    }

    /**
     * What class loader to use for loading classes while compiling
     * this JSP? I don't think this is used right now -- akv. 
     */
    public ClassLoader getClassLoader() {
        return TopManager.getDefault().currentClassLoader();
    }

    /**
     * Are we processing something that has been declared as an
     * errorpage? 
     */
    public boolean isErrorPage() {
        throw new UnsupportedOperationException();
        //return errPage;
    }

    /**
     * What is the scratch directory we are generating code into?
     * FIXME: In some places this is called scratchDir and in some
     * other places it is called outputDir.
     */
    public String getOutputDir() {
        return options.getScratchDir().toString();
    }

    /**
     * Path of the JSP URI. Note that this is not a file name. This is
     * the context rooted URI of the JSP file. 
     */
    public String getJspFile() {
        return jspFile;
    }

    /**
     * Just the class name (does not include package name) of the
     * generated class. 
     */
    public String getServletClassName() {
        throw new UnsupportedOperationException();
    }

    /**
     * The package name into which the servlet class is generated. 
     */
    public String getServletPackageName() {
        throw new UnsupportedOperationException();
    }

    /**
     * Utility method to get the full class name from the package and
     * class name. 
     */
    public String getFullClassName() {
        throw new UnsupportedOperationException();
    }

    /**
     * Full path name of the Java file into which the servlet is being
     * generated. 
     */
    public String getServletJavaFileName() {
        throw new UnsupportedOperationException();
    }

    /**
     * Are we keeping generated code around?
     */
    public boolean keepGenerated() {
        return options.getKeepGenerated();
    }

    /**
     * What's the content type of this JSP? Content type includes
     * content type and encoding. 
     */
    public String getContentType() {
        return contentType;
    }

    /**
     * Get hold of the Options object for this context. 
     */
    public Options getOptions() {
        return options;
    }

    public void setContentType(String contentType) {
        this.contentType = contentType;
    }

    public void setReader(JspReader reader) {
        this.reader = reader;
    }

    public void setWriter(ServletWriter writer) {
        throw new UnsupportedOperationException();
    }

    public void setServletClassName(String servletClassName) {
        throw new UnsupportedOperationException();
    }

    public void setServletPackageName(String servletPackageName) {
        throw new UnsupportedOperationException();
    }

    public void setServletJavaFileName(String servletJavaFileName) {
        throw new UnsupportedOperationException();
    }

    public void setErrorPage(boolean isErrPage) {
        throw new UnsupportedOperationException();
        //errPage = isErrPage;
    }

    /**
     * Create a "Compiler" object based on some init param data. This
     * is not done yet. Right now we're just hardcoding the actual
     * compilers that are created. 
     */
    public Compiler createCompiler() throws JasperException {
        throw new UnsupportedOperationException();
    }


    /**
     * Get the full value of a URI relative to this compilations context
     * uses current file as the base.
     */
    public String resolveRelativeUri(String uri) {
        return cd.resolveRelativeURI(uri);
    }


    /**
     * Gets a resource as a stream, relative to the meanings of this
     * context's implementation.
     *@returns a null if the resource cannot be found or represented 
     *         as an InputStream.
     */
    public java.io.InputStream getResourceAsStream(String res) {
        return cd.getResourceAsStream(res);
    }

    /**
     * Gets the actual path of a URI relative to the context of
     * the compilation.
     */
    public String getRealPath(String path) {
        return cd.getRealPath(path);
    }

    public FileObject getResourceAsObject(String path) {
        return cd.getResourceAsObject(path);
    }

}

