/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.web.core.jsploader;

import java.io.InputStream;
import java.io.IOException;
import java.io.File;
import java.io.FileNotFoundException;
import java.net.URL;
import java.net.MalformedURLException;

import org.openide.execution.NbClassPath;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileSystem;
import org.openide.filesystems.FileStateInvalidException;
import org.openide.TopManager;

/** Descriptor of a web application compilation context for a given URI.
 *  This is a per-context per-request structure, so every compiled JSP must have 
 *  its instance of compilation descriptor.
 *  Methods inherited from ContextDescriptor can handle relative URIs, in addition
 *  to context-relative URIs already handled by ContextDescriptor.
 */

public class CompilationDescriptor extends ContextDescriptor {

    private String baseURI;

    /** Constructs new ContextDescriptor.
    *  @param fs filesystem for which to create the ServletContext. 
    *  If this is null ServletContext for the whole repository is created.
    *  @param baseURI base URI against which relative URIs will be resolved. 
    *  This is typically a filesystem path of the compiled JSP 
    *  (<code>JspCompileUtil.getContextPath(jsp.getPrimaryFile())</code>), but in
    *  some cases, for example if a specific mapping is set up for a JSP, this
    *  may be different.
    */
    public CompilationDescriptor(FileSystem fs, String baseURI) {
        super(fs);
        this.baseURI = baseURI;
    }

    /** Resolves a relative URI to a context-relative URI relatively to this compilation context.*/
    public String resolveRelativeURI(String uri) {
        uri = uri.replace('\\', '/');
        return JspCompileUtil.resolveRelativeURL(baseURI, uri);
    }

    /** Returns a file: url
    * @param path a relative URI path
    */
    public URL getResource(String path) throws MalformedURLException {
        return super.getResource(resolveRelativeURI(path));
    }

    /** Returns inputstream of a FileObject.
    * @param path a relative URI path
    */
    public InputStream getResourceAsStream(String path) {
        return super.getResourceAsStream(resolveRelativeURI(path));
    }

    /** Returns a file object for a given path, or <code>null</code> if not found.
    * @param path a relative URI path
    */
    public FileObject getResourceAsObject(String path) {
        return super.getResourceAsObject(resolveRelativeURI(path));
    }

    /** Returns a real file path for a uri path.
    * @param path a relative URI path
    */
    public String getRealPath(String path) {
        return super.getRealPath(resolveRelativeURI(path));
    }

}

