/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */
 
package org.netbeans.modules.web.core.jsploader;           

import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;

import org.openide.compiler.*;
import org.openide.compiler.Compiler;
import org.openide.cookies.CompilerCookie;
import org.openide.actions.CompileAction;
import org.openide.filesystems.*;
import org.openide.loaders.*;
import org.openide.TopManager;
import org.openide.util.NbBundle;

import org.netbeans.modules.web.core.execution.JspServlet;

/** 
 *
 * @author  Petr Jiricka
 * @version 
 */
public class CompileServlet extends HttpServlet {
   
    /** Initializes the servlet.
    */  
    public void init(ServletConfig config) throws ServletException {
        super.init(config);

    }

    /** Destroys the servlet.
    */  
    public void destroy() {

    }

    /** Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
    * Expects request parameter
    * @param request servlet request
    * @param response servlet response
    */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, java.io.IOException {
        boolean compilationOK = false;
        try {
            String file = request.getParameter(JspServlet.COMPILE_SERVLET_PARAM_NAME);
            String uri = request.getParameter(JspServlet.COMPILE_SERVLET_JSP_URI);
            FileObject jsp = findFileObjectForFile(file, uri);
            if (jsp == null) {
                compilationOK = reportError(java.text.MessageFormat.format(
                    NbBundle.getBundle(CompileServlet.class).getString("EXC_FileToCompileNotFound"),
                    new Object[] {file}
                ) , file);
            }
            DataObject dobj = DataObject.find(jsp);
            CompilerCookie cookie = (CompilerCookie.Compile)dobj.getCookie(CompilerCookie.Compile.class);
            if (cookie != null) {
                compilationOK = CompileAction.compile(new CompileCookieEnumeration(cookie), 
                    jsp.getName());
            }
        }
        catch (DataObjectNotFoundException e) {
        }
        response.setContentType("text/plain");
        java.io.PrintWriter out = response.getWriter();
        out.println(compilationOK ? JspServlet.COMPILE_SERVLET_OK_RESPONSE : "ERROR");
        out.close();
//System.out.println("CompileServlet called");        
    } 
    
    /** Finds a fileobject for an absolute file name or null if not found */
    private static FileObject findFileObjectForFile(String fileName, String uriToEnsure) {
        Repository rep = TopManager.getDefault().getRepository();
        for (Enumeration en = rep.getFileSystems(); en.hasMoreElements(); ) {
            FileSystem fs = (FileSystem)en.nextElement();
            FileObject fo = JspCompileUtil.findFileObjectForFile(fs, fileName);
            uriToEnsure = JspCompileUtil.resolveRelativeURL("/", uriToEnsure);
            if ((fo != null) && uriToEnsure.equals(JspCompileUtil.getContextPath(fo)))
                return fo;
        }
        return null;
    }

    /** Reports an error in the compilation window, returns false (compilation failed). */
    private boolean reportError(final String errorMessage, String jobName) {
        CompilerCookie cookie = new CompilerCookie.Compile() {
            
            public void addToJob(CompilerJob job, Compiler.Depth depth) {
                Compiler error = new ErrorCompiler(TopManager.getDefault().getRepository().getDefaultFileSystem().getRoot(), 
                    new Exception(errorMessage), false);
                job.add(error);
            }
            
            public boolean isDepthSupported(Compiler.Depth depth) {
                return true;
            }
           
        };
        return CompileAction.compile(new CompileCookieEnumeration(cookie), jobName);
    }

    /** Handles the HTTP <code>GET</code> method.
    * @param request servlet request
    * @param response servlet response
    */
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, java.io.IOException {
        processRequest(request, response);
    } 

    /** Handles the HTTP <code>POST</code> method.
    * @param request servlet request
    * @param response servlet response
    */
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, java.io.IOException {
        processRequest(request, response);
    }

    /** Returns a short description of the servlet.
    */
    public String getServletInfo() {
        return "Servlet which launches compilation of a JSP";
    }
    
    private static class CompileCookieEnumeration implements Enumeration {
        
        private Object cookie;
        
        public CompileCookieEnumeration(Object cookie) {
            this.cookie = cookie;
        }
        
        public boolean hasMoreElements() {
            return cookie != null;
        }
        
        public Object nextElement() {
            if (cookie != null) {
                Object cookie1 = cookie;
                cookie = null;
                return cookie1;
            }
            throw new NoSuchElementException();
        }
        
    }

}
