/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.web.core.jsploader;

import java.util.Map;

/** High-level information about a JSP file.
* Implementation may be created with various level of error checking, 
* so reliability of this information may differ for each instance.
* This information is not resolved t any particular objects yet, only names of objects.
*/
public interface JspInfo {

    /** Files included by include directive,
    * resolved to absolute URL within the context. */
    public String[] getIncludedFiles();

    /** Data for beans used by this page. */
    public BeanData[] getBeans();

    /** File used as the error page,
    * resolved to absolute URL within the context. */
    public String[] getErrorPage();

    /** Files referenced by include and forward actions.
    * These are either absolute URLs within the context, or
    * JSP expressions. Distrinction may be made using JspUtil.isExpression(). */
    public String[] getReferencedPages();

    /** Returns information about tag libraries imported by this JSP. */
    public TagLibraryData[] getTagLibraryData();

    /** Returns whether this page is an error page. */
    public boolean isErrorPage();

    /** Returns encoding of this JSP. */
    public String getEncoding();

    /** Data describing an imported tag library.*/
    public interface TagLibraryData {

        /** Returns prefix with which the taglib was imported. */
        public String getPrefix();

        /** Returns literal value of the uri attribute. */
        public String getUnresolvedURI();

        /** Returns URI resolved to a context-relative URI. */
        public String getResolvedURI();

    } // interface TagLibraryData

    /** Interface which describes data for beans used by this page. */
    public interface BeanData {

        /** Identifier of the bean in the page (variable name). */
        public String getId();

        /** Scope for this bean. Returns constants defined in {@link javax.servlet.jsp.PageContext}. */
        public int getScope();

        /** Returns the class name for this bean. */
        public String getClassName();

    } // interface BeanData

}