/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.web.core.jsploader;

import java.io.*;
import java.net.URL;
import java.net.MalformedURLException;
import javax.swing.text.Document;

import org.openide.execution.NbClassPath;
import org.openide.cookies.EditorCookie;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileSystem;
import org.openide.filesystems.FileStateInvalidException;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.text.EditorSupport;
import org.openide.TopManager;

/** Descriptor of a web application compilation context for a given URI.
 *  This is a per-context per-request structure, so every compiled JSP must have 
 *  its instance of compilation descriptor.
 *  Methods inherited from ContextDescriptor can handle relative URIs, in addition
 *  to context-relative URIs already handled by ContextDescriptor.
 */

public class ParsingDescriptor extends CompilationDescriptor {

    /** Constructs new ParsingDescriptor.
    *  @param fs filesystem for which to create the ServletContext. 
    *  If this is null ServletContext for the whole repository is created.
    *  @param baseURI base URI against which relative URIs will be resolved. 
    *  This is typically a filesystem path of the compiled JSP 
    *  (<code>JspCompileUtil.getContextPath(jsp.getPrimaryFile())</code>), but in
    *  some cases, for example if a specific mapping is set up for a JSP, this
    *  may be different.
    */
    public ParsingDescriptor(FileSystem fs, String baseURI) {
        super(fs, baseURI);
    }

    /** Returns inputstream of a FileObject.
    * @param path a context-relative URI path
    */
    public InputStream getResourceAsStream(String path) {
        FileObject fo = getResourceAsObject(path);
        if (fo == null)
            return null;
        try {
            DataObject dobj = DataObject.find(fo);
            EditorSupport editor = (EditorSupport)dobj.getCookie(EditorSupport.class);
            if (editor == null)
                return super.getResourceAsStream(path);
            if (!editor.isModified() || !editor.isDocumentLoaded())
                return super.getResourceAsStream(path);
            final Document doc = editor.openDocument();
            final String[] str = new String[1];
            // safely take the text from the document
            Runnable run = new Runnable() {
                public void run() {
                    try {
                        str[0] = doc.getText(0, doc.getLength());
                    }
                    catch (javax.swing.text.BadLocationException e) {
                        // impossible
                    }
                }
            };
            doc.render(run);
            ByteArrayOutputStream out = new ByteArrayOutputStream();
            OutputStreamWriter wr = new OutputStreamWriter(out, getEncoding(dobj));
            wr.write(str[0]);
            wr.close();
            out.close();
            return new ByteArrayInputStream(out.toByteArray());
        }
        catch (DataObjectNotFoundException ex) {
            return null;
        }
        catch (IOException ex) {
            return null;
        }
    }

    public String getEncoding(DataObject obj) {
        return "8859_1";
    }

}

