/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.web.core.jsploader;

import java.util.*;

import org.openide.text.Line;
import org.openide.text.CloneableEditorSupport;

/** Line and lineset for servlets. 
 *
 * @author  Petr Jiricka
 * @version 
 */
public class ServletLine extends Line implements CESLine {
    
    private CloneableEditorSupport.Env editorEnv;
    private Line orig;

    /** Creates new ServletLine */
    public ServletLine(Line orig, ServletEditor editor) {
        super(editor.getServlet());
        this.orig   = orig;
        this.editorEnv = editor.jspEnv();
    }

    public CloneableEditorSupport getCloneableEditorSupport() {
        return (ServletEditor)editorEnv.findCloneableOpenSupport();
    }
    
    public String getDisplayName () {
        return orig.getDisplayName();
    }

    /**
     * Get the data object this line is a part of. This is the only method which we need to handle 
     * differently than the delegate.
    * @return data object
    */
/*    public final DataObject getDataObject () {
        return editor.getServlet();
    }*/

    public int hashCode () {
        return orig.hashCode ();
    }
    
    public boolean equals(Object o) {
        if ((o instanceof CESLine) && (o instanceof Line)) {
            Line l = (Line)o;
            CloneableEditorSupport otherEditor = ((CESLine)o).getCloneableEditorSupport();
            if (getCloneableEditorSupport() == otherEditor) {
                return getLineNumber () == l.getLineNumber ();
            }
        }
        return false;
    }

    public int getLineNumber () {
        return orig.getLineNumber();
    }

    public void show(int kind, int column) {
        orig.show(kind, column);
    }

    public void show(int kind) {
        orig.show(kind);
    }

    public void setBreakpoint(boolean b) {
        orig.setBreakpoint(b);
    }

    public boolean isBreakpoint() {
        return orig.isBreakpoint();
    }

    public void markError() {
        orig.markError();
    }

    public void unmarkError() {
        orig.unmarkError();
    }

    public void markCurrentLine() {
        orig.markCurrentLine();
    }

    public void unmarkCurrentLine() {
        orig.unmarkCurrentLine();
    }

    public boolean canBeMarkedCurrent (int action, Line previousLine) {
        return orig.canBeMarkedCurrent(action, previousLine);
    }

    
    public static class Set extends Line.Set {
        
        private ServletEditor editor;
        private Line.Set delegate;
        private ArrayList linesCache;

        public Set (Line.Set delegate, ServletEditor editor) {
            this.delegate = delegate;
            this.editor = editor;
        }
        
        public List getLines () {
            if (linesCache == null) {
                List lines = delegate.getLines();
                linesCache = new ArrayList();
                for (int i = 0; i < lines.size(); i++)
                    linesCache.add(new ServletLine((Line)lines.get(i), editor));
            }
            return linesCache;
        }


        public Line getOriginal (int line) throws IndexOutOfBoundsException {
            return new ServletLine(delegate.getOriginal(line), editor);
        }

        public Line getCurrent (int line) throws IndexOutOfBoundsException {
            return new ServletLine(delegate.getCurrent(line), editor);
        }

    }
    
}