/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.web.core.jsploader.jakarta;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import org.openide.execution.NbClassPath;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;

import org.netbeans.modules.web.core.jsploader.JspDataObject;
import org.netbeans.modules.web.core.jsploader.JspCompileUtil;
import org.netbeans.modules.web.core.WebExecUtil;

import org.apache.jasper.compiler.Mangler;
import org.apache.jasper.compiler.ClassName;
import org.apache.jasper.JspCompilationContext;
import org.apache.jasper.Constants;
import org.apache.jasper.JasperException;

/**
 * JspCompiler is an implementation of Compiler with a funky code
 * mangling and code generation scheme!
 *
 * The reason that it is both a sub-class of compiler and an implementation
 * of mangler is because the isOutDated method that is overridden and the
 * name mangulation both depend on the actual existance of other class and
 * java files.  I.e. the value of a mangled name is a function of both the
 * name to be mangled and also of the state of the scratchdir.
 *
 * @author Anil K. Vijendran
 */
public class ManglerImpl implements Mangler {

    String pkgName, className, javaFileName, classFileName;
    String onlyClassFileName;

    File jsp;
    FileObject baseOutputDir;
    FileObject javaOutputDir;
    JspDataObject jspdo;

    ClassFileData cfd;
    boolean outDated;


    public ManglerImpl(JspDataObject jspdo) throws IOException, JasperException {

        this.jsp = new File(JspCompileUtil.getContextPath(jspdo.getPrimaryFile()));
        this.jspdo = jspdo;
        //System.out.println("ManglerImpl: JspCompileUtil.getContextPath(jspdo.getPrimaryFile())=" + JspCompileUtil.getContextPath(jspdo.getPrimaryFile()));
        //System.out.println("ManglerImpl: jsp=" + jsp);
        //this.jsp = new File(JspCompileUtil.getFileObjectFileName(jspdo.getPrimaryFile()));
        //this.jsp = NbClassPath.toFile(jspdo.getPrimaryFile());
        this.baseOutputDir = JspCompileUtil.getContextOutputRoot(jspdo.getPrimaryFile());
        this.outDated = false;
        computePackageName();
        computeClassFileName();
        computeClassFileData();
        computeJavaOutputDir(jspdo);
        computeJavaFileName();
    }

    public final String getPackageName() {
        return pkgName;
    }

    public final String getClassName() {
        return className;
    }

    public final String getJavaFileName() {
        return javaFileName;
    }

    public final String getClassFileName() {
        return classFileName;
    }

    public final FileObject getJavaOutputDir() {
        return javaOutputDir;
    }

    public final FileObject getBaseOutputDir() {
        return baseOutputDir;
    }

    public final String getClassFileNameSansPackageSansExt() {
        return onlyClassFileName;
    }


    /**
     * Return true if the .class file is outdated w.r.t
     * the JSP file. 
     *
     * Can (meant to) be overridden by subclasses of JspCompiler. 
     */
    public boolean isOutDated() {
        return outDated;
    }

    public static String [] keywords = {
        "abstract", "boolean", "break", "byte",
        "case", "catch", "char", "class",
        "const", "continue", "default", "do",
        "double", "else", "extends", "final",
        "finally", "float", "for", "goto",
        "if", "implements", "import",
        "instanceof", "int", "interface",
        "long", "native", "new", "package",
        "private", "protected", "public",
        "return", "short", "static", "super",
        "switch", "synchronized", "this",
        "throw", "throws", "transient",
        "try", "void", "volatile", "while"
    };

    void computeJavaOutputDir(JspDataObject jspdo) throws IOException {
        FileObject fsRoot = JspCompileUtil.getContextOutputRoot(jspdo.getPrimaryFile());
        //System.out.println(">computeJavaOutputDir: pkgName=" + pkgName);
        if (pkgName == null)
            javaOutputDir = fsRoot;
        else {
            javaOutputDir = fsRoot.getFileSystem().find(pkgName, null, null);
            if (javaOutputDir == null) {
                javaOutputDir = FileUtil.createFolder(fsRoot, pkgName.replace('.', '/'));
            }
        }
        /*System.out.println("-mangler comupted javaOutputDir '" + NbClassPath.toFile(javaOutputDir) +
                           "'  exists:" + NbClassPath.toFile(javaOutputDir).exists());*/
        File dir = NbClassPath.toFile(javaOutputDir);
        if (!dir.exists())
            WebExecUtil.myMkdirs(dir);

    }

    void computePackageName() {
        String pathName = jsp.getPath();
        StringBuffer modifiedpkgName = new StringBuffer ();
        int indexOfSepChar = pathName.lastIndexOf(File.separatorChar);

        if (indexOfSepChar == -1 || indexOfSepChar == 0)
            pkgName = null;
        else {
            for (int i = 0; i < keywords.length; i++) {
                char fs = File.separatorChar;
                int index1 = pathName.indexOf(fs + keywords[i]);
                int index2 = pathName.indexOf(keywords[i]);
                if (index1 == -1 && index2 == -1) continue;
                int index = (index2 == -1) ? index1 : index2;
                while (index != -1) {
                    String tmpathName = pathName.substring (0,index+1) + '%';
                    pathName = tmpathName + pathName.substring (index+2);
                    index = pathName.indexOf(fs + keywords[i]);
                }
            }

            // XXX fix for paths containing '.'.
            // Need to be more elegant here.
            pathName = pathName.replace('.','_');

            pkgName = pathName.substring(0, pathName.lastIndexOf(
                                             File.separatorChar)).replace(File.separatorChar, '.');
            for (int i=0; i<pkgName.length(); i++)
                if (Character.isLetter(pkgName.charAt(i)) == true ||
                        pkgName.charAt(i) == '.') {
                    modifiedpkgName.append(pkgName.substring(i,i+1));
                }
                else
                    modifiedpkgName.append(mangleChar(pkgName.charAt(i)));

            if (modifiedpkgName.charAt(0) == '.') {
                String modifiedpkgNameString = modifiedpkgName.toString();
                pkgName = modifiedpkgNameString.substring(1,
                          modifiedpkgName.length ());
            }
            else
                pkgName = modifiedpkgName.toString();
        }

    }

    public final void computeJavaFileName() throws IOException {
        javaFileName = JspCompileUtil.getFileObjectFileName(javaOutputDir) + File.separatorChar + className + ".java";
        //System.out.println("ManglerImpl: computedJavaFileName=" + javaFileName);
    }

    void computeClassFileName() throws IOException {
        String prefix = getPrefix(jsp.getPath());
        onlyClassFileName = prefix + getBaseClassName();
        classFileName = JspCompileUtil.getFileObjectFileName(baseOutputDir) +
                        File.separatorChar + onlyClassFileName + ".class";
    }

    private final String getInitialClassName() {
        String prefix = getPrefix(jsp.getPath());

        return prefix + getBaseClassName() + Constants.JSP_TOKEN + "0";
    }

    private final String getBaseClassName() {
        String className;

        //System.out.println("ManglerImpl: jsp.getName()=" + jsp.getName());
        //System.out.println("ManglerImpl: jsp=" + jsp);
        if (jsp.getName().endsWith(".jsp"))
            className = jsp.getName().substring(0, jsp.getName().length() - 4);
        else
            className = jsp.getName();


        // Fix for invalid characters. If you think of more add to the list.
        StringBuffer modifiedClassName = new StringBuffer();
        for (int i = 0; i < className.length(); i++) {
            if (Character.isLetterOrDigit(className.charAt(i)) == true)
                modifiedClassName.append(className.substring(i,i+1));
            else
                modifiedClassName.append(mangleChar(className.charAt(i)));
        }

        return modifiedClassName.toString();
    }

    private final String getPrefix(String pathName) {
        if (pathName != null) {
            StringBuffer modifiedName = new StringBuffer();
            for (int i = 0; i < pathName.length(); i++) {
                if (Character.isLetter(pathName.charAt(i)) == true)
                    modifiedName.append(pathName.substring(i,i+1));
                else
                    modifiedName.append(mangleChar(pathName.charAt(i)));
            }
            return modifiedName.toString();
        }
        else
            return "";
    }

    private static final String mangleChar(char ch) {

        if(ch == File.separatorChar) {
            ch = '/';
        }
        String s = Integer.toHexString(ch);
        int nzeros = 5 - s.length();
        char[] result = new char[6];
        result[0] = '_';
        for (int i = 1; i <= nzeros; i++)
            result[i] = '0';
        for (int i = nzeros+1, j = 0; i < 6; i++, j++)
            result[i] = s.charAt(j);
        return new String(result);
    }


    private final void computeClassFileData()
    throws JasperException
    {
        File jspReal = NbClassPath.toFile(jspdo.getPrimaryFile());

        /* NOT NEEDED
        String initPath = jsp.getPath();
        String convPath = ctxt.getRealPath(initPath);
        if (convPath.equals(initPath))
            jspReal = jsp;
        else
            jspReal = new File(ctxt.getRealPath(jsp.getPath()));*/

        File classFile = new File(classFileName);

        if (!classFile.exists()) {
            className = getInitialClassName();
            cfd = new ClassFileData(true, classFileName, className);
            outDated = true;
        } else  {
//System.out.println("cf.lastModified " + classFile + " , " + classFile.lastModified());
//System.out.println("jspreal.lastModified " + jspReal + " , " + jspReal.lastModified());
            outDated = classFile.lastModified() < jspReal.lastModified();
            if (outDated) {
                cfd = new ClassFileData(outDated, classFileName, className);
                cfd.incrementNumber();

            } else {
                cfd = new ClassFileData(outDated, classFileName, null);
                //pj: increment anyhow, since we want the future
                cfd.incrementNumber();
            }
            String classNameFromFile = ClassName.getClassName(classFileName);
            String cn = cfd.getClassName();
            int lastDot = cn.lastIndexOf('.');
            if (lastDot != -1)
                className = cn.substring(lastDot+1, cn.length());
            else
                className = cn;
        }
    }
}

class ClassFileData {
    boolean outDated;
    String className;
    String classFile;
    String baseClassName;
    int number;

    /**
     * Lazy - find the class name ( by reading the .class file and extracting the
     * information ) only if some method need this info.
     *
     *  In "normal" usage, this method is not called - if the .class is not outdated,
     *  nobody needs the real class name or other info.
     *  If this will change - this method needs to be revisited.
     */
    private void findClassName() {
        try {
            className = ClassName.getClassName(classFile);
        } catch( JasperException ex) {
            // ops, getClassName should throw something
            ex.printStackTrace();
        }
        baseClassName = className.substring(0, className.lastIndexOf(Constants.JSP_TOKEN));
        this.number
        = Integer.valueOf(className.substring(className.lastIndexOf(Constants.JSP_TOKEN)+
                                              Constants.JSP_TOKEN.length(),
                                              className.length())).intValue();
    }

    public boolean isOutDated() {
        return outDated;
    }

    public String getClassName() {
        if(className==null)
            findClassName();
        return className;
    }

    public String getClassNameSansNumber() {
        if(className==null)
            findClassName();
        return baseClassName;
    }

    public String getClassFileName() {
        return classFile;
    }

    public void incrementNumber() {
        if(className == null)
            findClassName();
        number++;
        className = baseClassName + Constants.JSP_TOKEN + number;
    }

    public int getNumber() {
        if(className==null)
            findClassName();
        return number;
    }

    public ClassFileData(boolean outDated, String cf, String cn) {
        this.outDated = outDated;
        this.className = cn;
        this.classFile = cf;
        if (cn != null) {
            this.baseClassName = cn.substring(0, cn.lastIndexOf(Constants.JSP_TOKEN));
            this.number = Integer.valueOf(cn.substring(cn.lastIndexOf(Constants.JSP_TOKEN)+
                                          Constants.JSP_TOKEN.length(),
                                          cn.length())).intValue();
        } else {
            baseClassName = null;
            number = -1;
        }
    }
}
