/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.web.core.jsploader.jakarta;

import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.File;
import java.io.ObjectOutputStream;
import java.io.FileOutputStream;
import java.net.URL;
import java.net.MalformedURLException;

import javax.servlet.jsp.tagext.TagInfo;
import javax.servlet.jsp.tagext.TagLibraryInfo;

import org.apache.jasper.JasperException;
import org.apache.jasper.Constants;
import org.apache.jasper.JspCompilationContext;
import org.apache.jasper.compiler.Generator;
import org.apache.jasper.compiler.Mark;
import org.apache.jasper.compiler.ServletWriter;
import org.apache.jasper.compiler.CharDataGenerator;
import org.apache.jasper.compiler.CommentGenerator;

/**
 * Generates original "Jakarta"-style comments
 *
 * @author Yury Kamen
 */
public class VerboseCommentGenerator implements CommentGenerator {
    private static final boolean debug = false;
    static private final String suffix   = "Generator";                   // NOI18N
    static private final String compiler = "org.apache.jasper.compiler."; // NOI18N

    public VerboseCommentGenerator() {
        super();
        if ( debug ) {
	    System.err.println("VerboseCommentGenerator created");  // NOI18N
        }
    }

    private String getType(Generator generator) {
        String type = generator.getClass().getName();
        if ( type.startsWith(compiler)) {
            type = type.substring(compiler.length());
        }
        int index = type.lastIndexOf(suffix);
        if (index > 0) {
            type = type.substring(0, index);
        }
        return type;
    }

    /**
     * Generates "start-of the JSP-embedded code block" comment
     *
     * @param out The ServletWriter
     * @param start Start position of the block
     * @param stop End position of the block
     * @exception JasperException
     */
    public void generateStartComment(Generator generator, ServletWriter out, Mark start, Mark stop) throws JasperException {
        if ( debug ) {
	    System.err.println("VerboseCommentGenerator generateStartComment invoked"); // NOI18N
        }
        String html = ""; // NOI18N
        if (generator instanceof CharDataGenerator) {
	    html = "// HTML "; // NOI18N
        }
        if (start != null && stop != null) {
//             if (start.fileid == stop.fileid) {
            String fileName = out.quoteString(start.getFile ());
                out.println(html + "// begin [file=" + fileName+";from=" + start.toShortString() + ";to=" + stop.toShortString() + "] " + getType(generator)); // NOI18N
//             } else {
//                 out.println(html + "// begin [from="+start+";to="+stop+"] " + getType(generator)); // NOI18N
//             }
        } else {
            out.println(html + "// begin " + getType(generator)); // NOI18N
        }

        out.pushIndent();
    }

    /**
     * Generates "end-of the JSP-embedded code block" comment
     *
     * @param out The ServletWriter
     * @param start Start position of the block
     * @param stop End position of the block
     * @exception JasperException
     */
    public void generateEndComment(Generator generator, ServletWriter out, Mark start, Mark stop) throws JasperException {
        out.popIndent();
        out.println("// end"); // NOI18N
    }
}
