/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.web.core.syntax;

import java.util.*;
import java.beans.*;

import org.netbeans.modules.web.core.jsploader.JspInfo;

import org.openide.TopManager;
/**
 *
 * @author  pjiricka
 * @version 
 */
public class AttrSupports extends Object {

    public static class UseBeanScope extends AttributeValueSupport.Default {
        
        public UseBeanScope() {
            super(true, "jsp:useBean", "scope");
        }
        
        public List getPossibleValues(JspSyntaxSupport sup, SyntaxElement.TagDirective item) {
            ArrayList list = new ArrayList();
            list.add("application");
            list.add("page");
            list.add("request");
            list.add("session");
            return list;
        }
        
    }

    public static class GetSetPropertyName extends AttributeValueSupport.Default {
        
        public GetSetPropertyName(boolean tag, String longName, String attrName) {
            super(tag, longName, attrName);
        }
        
        public List getPossibleValues(JspSyntaxSupport sup, SyntaxElement.TagDirective item) {
            ArrayList list = new ArrayList();
            JspInfo.BeanData[] beanData = sup.getBeanData();
            for (int i = 0; i < beanData.length; i++) {
                list.add(beanData[i].getId());
            }
            return list;
        }
        
    }
    
    public static class GetPropertyName extends GetSetPropertyName {
        
        public GetPropertyName() {
            super(true, "jsp:getProperty", "name");
        }
        
    }
    
    public static class SetPropertyName extends GetSetPropertyName {
        
        public SetPropertyName() {
            super(true, "jsp:setProperty", "name");
        }
        
    }

    public static abstract class GetSetPropertyProperty extends AttributeValueSupport.Default {
        
        public GetSetPropertyProperty(boolean tag, String longName, String attrName) {
            super(tag, longName, attrName);
        }
        
        public List getPossibleValues(JspSyntaxSupport sup, SyntaxElement.TagDirective item, boolean setter) {
            ArrayList list = new ArrayList();
            String namePropertyValue = (String)item.getAttributes().get("name");
            if (namePropertyValue != null) {
                String className = null;
                JspInfo.BeanData[] beanData = sup.getBeanData();
                for (int i = 0; i < beanData.length; i++) {
                    if (beanData[i].getId().equals(namePropertyValue)) {
                        className = beanData[i].getClassName();
                        break;
                    }
                }
                if (className != null) {
                    try {
                        Class beanClass = Class.forName(className, false, 
                            TopManager.getDefault().currentClassLoader());
                        Introspector.flushFromCaches(beanClass);
                        BeanInfo benInfo = Introspector.getBeanInfo(beanClass);
                        PropertyDescriptor[] properties = benInfo.getPropertyDescriptors();
                        for (int j = 0; j < properties.length; j++) {
                            if (setter && (properties[j].getWriteMethod() != null))
                                list.add(properties[j].getName());
                            if (!setter && (properties[j].getReadMethod() != null))
                                list.add(properties[j].getName());
                        }
                    }
                    catch (ClassNotFoundException e) {
                        // do nothing
                    }
                    catch (IntrospectionException e) {
                        // do nothing
                    }
                }
            }    
            return list;
        }
        
    }
    
    public static class GetPropertyProperty extends GetSetPropertyProperty {
        
        public GetPropertyProperty() {
            super(true, "jsp:getProperty", "property");
        }
        
        public List getPossibleValues(JspSyntaxSupport sup, SyntaxElement.TagDirective item) {
            return getPossibleValues(sup, item, false);
        }
        
    }
    
    public static class SetPropertyProperty extends GetSetPropertyProperty {
        
        public SetPropertyProperty() {
            super(true, "jsp:setProperty", "property");
        }
        
        public List getPossibleValues(JspSyntaxSupport sup, SyntaxElement.TagDirective item) {
            List l = getPossibleValues(sup, item, true);
            l.add(0, "*");
            return l;
        }
        
    }
    
    public static class TaglibURI extends AttributeValueSupport.Default {
        
        public TaglibURI() {
            super(false, "taglib", "uri");
        }
        
        public List getPossibleValues(JspSyntaxSupport sup, SyntaxElement.TagDirective item) {
            List l = new ArrayList();
            Map m = sup.getTagLibraryMap();
            if (m != null) {
                Iterator it = m.keySet().iterator();
                for (; it.hasNext(); ) {
                    String s = (String)it.next();
                    l.add(s);
                }
            }
            return l;
        }
        
    }
    

}