/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.web.core.syntax;

import java.util.HashMap;
import java.util.Map;
import java.util.List;
import java.util.ArrayList;
import javax.swing.text.JTextComponent;

import org.netbeans.editor.ext.CompletionQuery;

/** Support for attribute value completion for JSP tags and directives.
 *
 * @author  Petr Jiricka
 * @version 
 */
public abstract class AttributeValueSupport extends Object {
    
    private static Map supports;
    
    public static void putSupport(AttributeValueSupport support) {
        if (supports == null)
            initialize();
        // trick so we can construct a 'dummy' key element and get the 'real' element
        supports.put(support, support);
    }
    
    public static AttributeValueSupport getSupport(boolean tag, String longName, String attrName) {
        if (supports == null)
            initialize();
        return (AttributeValueSupport)supports.get(
            new AttributeValueSupport(tag, longName, attrName) {
            
                public List getCompletionItems(int offset, JspSyntaxSupport sup, 
                    SyntaxElement.TagDirective item, String valuePart) {
                    return null;
                }
                
            }
        );
    }
    
    private static void initialize() {
        supports = new HashMap();
        putSupport(new AttrSupports.UseBeanScope());
        putSupport(new AttrSupports.GetPropertyName());
        putSupport(new AttrSupports.SetPropertyName());
        putSupport(new AttrSupports.GetPropertyProperty());
        putSupport(new AttrSupports.SetPropertyProperty());
        putSupport(new AttrSupports.TaglibURI());
        // PENDING - add supports for known attributes
    }
    
    protected boolean tag;
    protected String longName;
    protected String attrName;

    /** Creates new AttributeValueSupport 
     * @param isTag whether this support is for tag or directive
     * @param longName either directive name or tag name including prefix
     * @param attribute name
     */
    public AttributeValueSupport(boolean tag, String longName, String attrName) {
        this.tag = tag;
        this.longName = longName;
        this.attrName = attrName;
    }
    
    public boolean equals(Object obj) {
        AttributeValueSupport sup2 = (AttributeValueSupport)obj;
        return (tag == sup2.tag) &&
               (longName.equals(sup2.longName)) &&
               (attrName.equals(sup2.attrName));
    }
    
    public int hashCode() {
        return longName.hashCode() + attrName.hashCode();
    }

    /** Returns the complete result. */
    //public abstract CompletionQuery.Result getResult(JTextComponent component, int offset, JspSyntaxSupport sup, CompletionItem.TagDirective item, String valuePart);
    
    public abstract List getCompletionItems(int offset, JspSyntaxSupport sup, SyntaxElement.TagDirective item, String valuePart);

    /** Default implementation of AttributeValueSupport. */
    public static abstract class Default extends AttributeValueSupport {
        
        /** Creates new DefaultAttributeValueSupport 
         * @param isTag whether this support is for tag or directive
         * @param longName either directive name or tag name including prefix
         * @param attribute name
         */
        public Default(boolean tag, String longName, String attrName) {
            super(tag, longName, attrName);
        }
        
        /** Allows subclasses to override the default title. */
        /*protected String getCompletionTitle() {
            return "JSP Completion"; // PENDING
        }*/
    
        /** Returns the complete result. */
        public List getCompletionItems(int offset, JspSyntaxSupport sup, SyntaxElement.TagDirective item, String valuePart) {
            //int valuePartLength = valuePart.length();
            List values = sup.filterList(getPossibleValues(sup, item), valuePart);
            List items = new ArrayList();
            for (int i = 0; i < values.size(); i++) {
                items.add(new CompletionItem.AttributeValue((String)values.get(i)));
            }
            return items;
            /*return new CompletionQuery.DefaultResult(component, getCompletionTitle(), 
                items, offset - valuePartLength, valuePartLength);*/
        }
    
        /** Should return a list of Strings containing all possible values 
         * for this attribute. May return null if no options are available.
         */
        public abstract List getPossibleValues(JspSyntaxSupport sup, SyntaxElement.TagDirective item);
    }
    
}