/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.web.core.syntax;

import java.lang.reflect.Modifier;
import java.util.Map;
import java.util.List;
import java.util.Comparator;
import java.util.Iterator;
import java.util.HashMap;
import java.util.ArrayList;
import java.awt.Component;
import javax.swing.ListCellRenderer;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JList;

import org.netbeans.editor.ext.Completion;
import org.netbeans.editor.ext.CompletionQuery;
import org.netbeans.editor.ext.CompletionView;
import org.netbeans.editor.ext.ExtEditorUI;
import org.netbeans.editor.ext.ListCompletionView;

import org.netbeans.modules.editor.java.NbJCCellRenderer;


/**
* JSP Completion query specifications
*
* @author Petr Jiricka, Petr Nejedly
* @version 0.00
*/

public class JspCompletion extends Completion {
    
    protected Completion contentCompletion;
    protected Completion scriptingCompletion;

    public JspCompletion(ExtEditorUI extEditorUI) {
        super(extEditorUI);
    }

    public JspCompletion(ExtEditorUI extEditorUI, 
        Completion contentCompletion, Completion scriptingCompletion) {
        super(extEditorUI);
        this.contentCompletion = contentCompletion;
        this.scriptingCompletion = scriptingCompletion;
    }

    protected CompletionView createView() {
        return new ListCompletionView(new DelegatingCellRenderer());
    }

    protected CompletionQuery createQuery() {
        CompletionQuery contentQuery = (contentCompletion == null) ? null : contentCompletion.getQuery();
        CompletionQuery scriptingQuery = (scriptingCompletion == null) ? null : scriptingCompletion.getQuery();
        return new JspCompletionQuery(contentQuery, scriptingQuery);
    }

    /** Substitute the document's text with the text
    * that is appopriate for the selection
    * in the view. This function is usually triggered
    * upon pressing the Enter key.
    * @return true if the substitution was performed
    *  false if not.
    */
/*    public synchronized boolean substituteText( boolean flag ) {
        if( getLastResult() != null ) {
            int index = getView().getSelectedIndex();
            if (index >= 0) {
                ((JspCompletionQuery.JspResult)getLastResult()).substituteText( index, flag );
            }
            return true;
        } else {
            return false;
        }
    }*/


/* -------------------------------------------------------------------------- */

   public class DelegatingCellRenderer implements ListCellRenderer {
        ListCellRenderer defaultRenderer = new NbJCCellRenderer();//new DefaultListCellRenderer();
        
        public Component getListCellRendererComponent(JList list, Object value,
                int index, boolean isSelected, boolean cellHasFocus) {
            if( value instanceof CompletionQuery.ResultItem) {
                return ((CompletionQuery.ResultItem)value).getPaintComponent( list, isSelected, cellHasFocus );
            } else {
                return defaultRenderer.getListCellRendererComponent( list, value, index, isSelected, cellHasFocus);
            }
            
        }
    }

}


/*
 * $Log: 
 *  1    Jaga      1.0         3/16/00  Miloslav Metelka 
 * $
 */

