/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.web.core.syntax;

import java.util.ArrayList;
import java.util.List;

import org.netbeans.editor.ext.CompletionQuery;
import org.netbeans.editor.ext.ExtEditorUI;
import org.netbeans.editor.ext.java.*;
import org.netbeans.modules.editor.java.NbJavaCompletion;

public class JspJavaCompletion extends NbJavaCompletion {
    
    public JspJavaCompletion(ExtEditorUI extEditorUI) {
        super(extEditorUI);
    }

    public static class JCJspClass extends JavaCompletion.AbstractClass {
        
        /** name, pakcage name & class name of variables predefined in JSP */
        private String[][] standardVars = {
            {"request", "javax.servlet.http", "HttpServletRequest"},
            {"response", "javax.servlet.http", "HttpServletResponse"},
            {"out", "java.io","PrintWriter"},
            {"session", "javax.servlet.http","HttpSession"},
            {"application", "javax.servlet", "ServletContext"},
            {"config", "javax.servlet","ServletConfig"},
            {"pageContext", "javax.servlet.jsp","PageContext"},
            {"page", null, null}
        };
        
        public JCJspClass(String cls, String pkg) {
            super (cls, pkg, false, 0);   // name, pkg name, iface, modifiers
        }
        
        public boolean isInterface() {
            return false;
        }
        
        /** Init internal representation  */
        protected void init () {
            body = new JavaCompletion.AbstractClass.Body ();
            
            // superclass: HttpServlet unless there is <% page extends="pkg.class" %> 
            // PENDING
            List clsList = JavaCompletion.getFinder().findClasses(null, "javax.servlet.http.HttpServlet", true);
            if (clsList != null && clsList.size() > 0) {
                if (clsList.size() > 0) { // more matching classes
                    body.superClass = (JCClass)clsList.get(0); // get the first one
                }
            }
            
            // fields
            ArrayList lst = new ArrayList ();
            for (int i = 0; i<standardVars.length; i++) {
                try {
                    if (standardVars[i][1] != null)
                        lst.add ( new JavaCompletion.BaseField (this, standardVars[i][0], 
                            new JavaCompletion.BaseType (getClass (standardVars[i][1],standardVars[i][2]), 0), 0)
                        );
                    else
                        lst.add ( new JavaCompletion.BaseField (this, standardVars[i][0], 
                            new JavaCompletion.BaseType (this, 0), 0)
                        );
                }
                catch (IllegalArgumentException ex) {
                    if (Boolean.getBoolean ("netbeans.debug.exception")) {
                        ex.printStackTrace ();
                    }
                }
            }
            body.fields = new JCField[lst.size()];
            lst.toArray(body.fields);
            lst.clear();
            
            body.methods = new JCMethod[0];
            body.interfaces = new JCClass[0];
            body.constructors = new JCConstructor[0];
            body.tagOffset = 0;
        }
        
        /*
        public JCField[] getFields() {
        }
        
        public JCClass getSuperclass() {
        }
        
        public JCConstructor[] getConstructors() {
        }
         
        public String getFullName() {
        }
         */
        
        private JCClass getClass (String pkgName, String clsName) throws IllegalArgumentException {
            List pkgList = JavaCompletion.getFinder().findPackages (pkgName, true, false);
            for (int i=0; i<pkgList.size(); i++) {
                List clsList = JavaCompletion.getFinder().findClasses((JCPackage)(pkgList.get (i)), clsName, true);
                if (clsList != null && clsList.size() > 0) {
                    if (clsList.size() > 0) { // more matching classes
                        return (JCClass)clsList.get(0); // get the first one
                    }
                }
            }
            throw new IllegalArgumentException ("Cannot find JCClass for "+((pkgName==null)? "": (pkgName+"."))+clsName);
        }
        
    }
}
