/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.web.core.syntax;

import java.util.*;
import javax.swing.text.BadLocationException;

import org.netbeans.modules.web.core.jsploader.JspInfo;

import org.netbeans.editor.ext.java.*;
import org.netbeans.editor.BaseDocument;
import org.netbeans.modules.editor.java.NbJavaSyntaxSupport;

/**
 *
 * @author  pjiricka
 * @version 
 */
public class JspJavaSyntaxSupport extends NbJavaSyntaxSupport {
    
    JspSyntaxSupport jspSup;

    /** Creates new JspJavaSyntaxSupport */
    public JspJavaSyntaxSupport(BaseDocument doc, JspSyntaxSupport jspSup) {
        super(doc);
        this.jspSup = jspSup;
    }
    
    protected int getMethodStartPosition(int pos) {
        int retPos = pos;
        try {
            SyntaxElement el = jspSup.getElementChain(pos);
            if (( el != null) && 
                (el.getCompletionContext() == 
                 JspSyntaxSupport.SCRIPTINGL_COMPLETION_CONTEXT)) {
                retPos = el.getElementOffset();
            }
        }
        catch (BadLocationException e) {
            // ignore
        }
        return retPos;
    }
    
    /** Builds the global variable map by JSP rules, i.e. there are three sources of
     * variables
     * <ul>
     *   <li>the standard JPS variables (request, application, page, ...)</li>
     *   <li>beans imported by useBean action</li>
     *   <li>JSP valiables declared by declaration elements in this JSP - not implemented yet</li>
     *   <li>JSP variables declared by custom tags - not implemented yet</li>
     * </ul>
     */
    protected Map buildGlobalVariableMap(int pos) {
        Map globMap = new HashMap();
        // implicit JSP objects
        globMap.put("request",     getClazzRepresentation("javax.servlet.http.HttpServletRequest"));
        globMap.put("response",    getClazzRepresentation("javax.servlet.http.HttpServletResponse"));
        globMap.put("pageContext", getClazzRepresentation("javax.servlet.jsp.PageContext"));
        globMap.put("session",     getClazzRepresentation("javax.servlet.http.HttpSession"));
        globMap.put("application", getClazzRepresentation("javax.servlet.ServletContext"));
        globMap.put("out",         getClazzRepresentation("javax.servlet.jsp.JspWriter"));
        globMap.put("config",      getClazzRepresentation("javax.servlet.ServletConfig"));
        globMap.put("page",        getClazzRepresentation("javax.servlet.http.HttpServlet"));
        // used beans
        JspInfo.BeanData[] beanData = jspSup.getBeanData();
        for (int i = 0; i < beanData.length; i++) {
            globMap.put(beanData[i].getId(), getClazzRepresentation(beanData[i].getClassName()));
        }
        return globMap;
    }

    private Object getClazzRepresentation(String decType) {
        JCClass cls = getClassFromName(decType, true);
        if (cls != null) {
            return JavaCompletion.getType(cls, 0);
        }
        return null;
    }

    public boolean isStaticBlock(int pos) {
        return false;
    }
    
    public JCClass getClass(int pos) {
// System.out.println("JspJavaSyntaxSupport.getClass() called");
        org.openide.filesystems.FileObject fo = jspSup.getDataObject ().getPrimaryFile ();
        return new JspJavaCompletion.JCJspClass (fo.getName ()+"_jsp_", "");
    }
    
    public void refreshClassInfo() {
// System.out.println("JspJavaSyntaxSupport.refreshClassInfo() called, jcValid="+jcValid);
        if (!jcValid) {
            JCFinder finder = JavaCompletion.getFinder();
            JCClass classToAppend = getClass (0);
            if (classToAppend != null) {
                finder.append(new JavaCompletion.SingleProvider(classToAppend));
                //            refreshParents(classToAppend);  !!! takes too much time
            }
            jcValid = true;
        }
    }

}