/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.web.core.syntax;

import java.awt.Color;
import java.awt.Font;
import java.awt.event.KeyEvent;
import java.awt.event.InputEvent;
import javax.swing.KeyStroke;
import javax.swing.Action;
import java.util.*;
import org.openide.actions.SaveAction;
import org.openide.actions.CompileAction;
import org.openide.actions.ExecuteAction;
import org.openide.actions.CutAction;
import org.openide.actions.CopyAction;
import org.openide.actions.PasteAction;
import org.openide.actions.DeleteAction;
import org.openide.actions.ToolsAction;
import org.openide.actions.PropertiesAction;

import org.netbeans.editor.Settings;
import org.netbeans.editor.SettingsUtil;
import org.netbeans.editor.SettingsNames;
import org.netbeans.editor.SettingsDefaults;
import org.netbeans.editor.Coloring;
import org.netbeans.editor.Syntax;
import org.netbeans.editor.BaseKit;
import org.netbeans.editor.TokenID;
import org.netbeans.editor.TokenContext;
import org.netbeans.editor.TokenContextPath;
import org.netbeans.editor.TokenCategory;
import org.netbeans.editor.ext.ExtSettingsNames;
import org.netbeans.editor.ext.html.HTMLSyntax;
import org.netbeans.editor.ext.java.JavaSyntax;
import org.netbeans.editor.ext.java.JavaLayerTokenContext;
import org.netbeans.editor.ext.java.JavaTokenContext;
import org.netbeans.editor.MultiKeyBinding;
import org.netbeans.modules.editor.java.JavaKit;
import org.netbeans.modules.web.core.FeatureFactory;

public class JspMultiSettingsInitializer extends Settings.AbstractInitializer {

    /** Name assigned to initializer */
    public static final String NAME = "jsp-multi-settings-initializer";

    public JspMultiSettingsInitializer() {
        super(NAME);
    }

    public void updateSettingsMap (Class kitClass, Map settingsMap) {
        // Jsp Colorings
        if (kitClass == BaseKit.class) {
            new JspMultiTokenColoringInitializer().updateSettingsMap(kitClass, settingsMap);
            new JspJavaLayerTokenColoringInitializer().updateSettingsMap(kitClass, settingsMap);
        }

        // Jsp Settings
        if (kitClass == JSPKit.class) {
                /*settingsMap.put(ExtSettingsNames.POPUP_MENU_ACTION_NAME_LIST,
                                new ArrayList(Arrays.asList(
                                                            new String[] {
                                                                SaveAction.class.getName(),
                                                                null,
                                                                CompileAction.class.getName(),
                                                                null,
                                                                ExecuteAction.class.getName(),
                                                                null,
                                                                CutAction.class.getName(),
                                                                CopyAction.class.getName(),
                                                                PasteAction.class.getName(),
                                                                null,
                                                                DeleteAction.class.getName(),
                                                                null,
                                                                ToolsAction.class.getName(),
                                                                PropertiesAction.class.getName()
                                                            }
                                                            ))
                                    );*/


            settingsMap.put (org.netbeans.editor.SettingsNames.ABBREV_MAP, getJSPAbbrevMap());

            SettingsUtil.updateListSetting(settingsMap, SettingsNames.TOKEN_CONTEXT_LIST,
                new TokenContext[] { JspMultiTokenContext.context, JspJavaLayerTokenContext.context });
        }
    }

    Map getJSPAbbrevMap() {
        Map jspAbbrevMap = new TreeMap ();
        // <jsp:something tags
        jspAbbrevMap.put ("jspu", "<jsp:useBean id=\"");
        jspAbbrevMap.put ("jspg", "<jsp:getProperty name=\"");
        jspAbbrevMap.put ("jg", "<jsp:getProperty name=\"");
        jspAbbrevMap.put ("jsps", "<jsp:setProperty name=\"");
        jspAbbrevMap.put ("jspi", "<jsp:include page=\"");
        jspAbbrevMap.put ("jspf", "<jsp:forward page=\"");
        jspAbbrevMap.put ("jspp", "<jsp:plugin type=\"");
        // taglib
        jspAbbrevMap.put ("tglb", "<%@ taglib uri=\"");
        // <%@ page tags
        jspAbbrevMap.put ("pg", "<%@ page ");
        jspAbbrevMap.put ("pgl", "<%@ page language=\"");
        jspAbbrevMap.put ("pgex", "<%@ page extends=\"");
        jspAbbrevMap.put ("pgim", "<%@ page import=\"");
        jspAbbrevMap.put ("pgs", "<%@ page session=\"");
        jspAbbrevMap.put ("pgb", "<%@ page buffer=\"");
        jspAbbrevMap.put ("pga", "<%@ page autoFlush=\"");
        jspAbbrevMap.put ("pgin", "<%@ page info=\"");
        jspAbbrevMap.put ("pgit", "<%@ page isThreadSafe=\"");
        jspAbbrevMap.put ("pgerr", "<%@ page errorPage=\"");
        jspAbbrevMap.put ("pgc", "<%@ page contentType=\"");
        jspAbbrevMap.put ("pgie", "<%@ page isErrorPage=\"");
        // common java abbrevs
        jspAbbrevMap.put ("rg", "request.getParameter(\"");
        jspAbbrevMap.put ("sg", "session.getValue(\"");
        jspAbbrevMap.put ("sp", "session.putValue(\"");
        jspAbbrevMap.put ("sr", "session.removeValue(\"");
        jspAbbrevMap.put ("pcg", "pageContext.getAttribute(\"");
        jspAbbrevMap.put ("pcgn", "pageContext.getAttributeNamesInScope(");
        jspAbbrevMap.put ("pcgs", "pageContext.getAttributesScope(\"");
        jspAbbrevMap.put ("pcr", "pageContext.removeAttribute(\"");
        jspAbbrevMap.put ("pcs", "pageContext.setAttribute(\"");
        jspAbbrevMap.put ("ag", "application.getValue(\"");
        jspAbbrevMap.put ("ap", "application.putValue(\"");
        jspAbbrevMap.put ("ar", "application.removeValue(\"");
        jspAbbrevMap.put ("oup", "out.print(\"");
        jspAbbrevMap.put ("oupl", "out.println(\"");
        jspAbbrevMap.put ("cfgi", "config.getInitParameter(\"");

        return jspAbbrevMap;
    }

    static class JspMultiTokenColoringInitializer
    extends SettingsUtil.TokenColoringInitializer {

        private Coloring emptyColoring = new Coloring(null, null, null);

        private Coloring errorColoring = new Coloring(null, Color.red, null);

        private Coloring javaErrorColoring = new Coloring(null, new Color(255, 249, 223), Color.red);

        private Coloring javaAddColoring = new Coloring(null, null, new Color(255, 249, 223));

        private Font boldFont   = SettingsDefaults.defaultFont.deriveFont (Font.BOLD);
        private Font italicFont = SettingsDefaults.defaultFont.deriveFont (Font.ITALIC);
        private Font italicBoldFont = italicFont.deriveFont (Font.BOLD);
        private Settings.Evaluator lightGrayEvaluator
            = new SettingsUtil.ForeColorPrintColoringEvaluator(Color.lightGray);
        private Settings.Evaluator italicFontEvaluator
            = new SettingsUtil.FontStylePrintColoringEvaluator(Font.ITALIC);

        public JspMultiTokenColoringInitializer() {
            super(JspMultiTokenContext.context);
        }

        public Object getTokenColoring(TokenContextPath tokenContextPath,
        TokenCategory tokenIDOrCategory, boolean printingSet) {

            if (tokenContextPath == JspMultiTokenContext.contextPath) {
                 if (!printingSet) {
                      switch (tokenIDOrCategory.getNumericID()) {
                           case JspMultiTokenContext.ERROR_ID:
                               return errorColoring;
                      }

                } else { // printing set
                     return SettingsUtil.defaultPrintColoringEvaluator;
                }

            } else if (tokenContextPath == JspMultiTokenContext.jspTagContextPath) {
                // JspTag token colorings
                if (!printingSet) {
                    switch (tokenIDOrCategory.getNumericID()) {
                        case JspTagTokenContext.TEXT_ID:
                          return emptyColoring;

                        case JspTagTokenContext.ERROR_ID:
                          return errorColoring;

                        case JspTagTokenContext.TAG_ID:
                          return new Coloring(italicBoldFont, Color.blue, new Color(239, 240, 235));

                        case JspTagTokenContext.SYMBOL_ID:
                          return new Coloring(boldFont, Color.black, new Color(239, 240, 235));
  
                        case JspTagTokenContext.COMMENT_ID:
                          return new Coloring(boldFont, Color.gray, null);
  
                        case JspTagTokenContext.ATTRIBUTE_ID:
                          return new Coloring(null, Color.green.darker().darker(), new Color(239, 240, 235));
  
                        case JspTagTokenContext.ATTR_VALUE_ID:
                          return new Coloring(null, Color.magenta, new Color(239, 240, 235));
  
                        case JspTagTokenContext.SYMBOL2_ID:
                          return new Coloring(boldFont, Color.black, new Color(255, 249, 223)); // same as Java

                    }

                } else { // printing set
                    switch (tokenIDOrCategory.getNumericID()) {
                        case JspTagTokenContext.COMMENT_ID:
                            return lightGrayEvaluator;

                        default:
                             return SettingsUtil.defaultPrintColoringEvaluator;
                    }

                }

                return null;

            } else if (tokenContextPath == JspMultiTokenContext.javaContextPath) {
                // Java token colorings
                if (!printingSet) {
                    if (tokenIDOrCategory == JavaTokenContext.ERRORS) 
                        return javaErrorColoring;
                    
                    return new SettingsUtil.TokenColoringEvaluator(
                        tokenContextPath.getParent().getFullTokenName(tokenIDOrCategory),
                        javaAddColoring, printingSet);

                } else { // printing set
                     return SettingsUtil.defaultPrintColoringEvaluator;
                }

            } else if (tokenContextPath == JspMultiTokenContext.htmlContextPath) {
                // HTML token colorings
                if (!printingSet) {
                    return new SettingsUtil.TokenColoringEvaluator(
                        tokenContextPath.getParent().getFullTokenName(tokenIDOrCategory),
                        null, printingSet);


                } else { // printing set
                     return SettingsUtil.defaultPrintColoringEvaluator;
                }

            } else if (tokenContextPath == JspMultiTokenContext.plainContextPath) {
                // Plain text token colorings
                if (!printingSet) {
                    return new SettingsUtil.TokenColoringEvaluator(
                        tokenContextPath.getParent().getFullTokenName(tokenIDOrCategory),
                        null, printingSet);

                } else { // printing set
                     return SettingsUtil.defaultPrintColoringEvaluator;
                }

            }

            return null;

        }

    }

    static class JspJavaLayerTokenColoringInitializer
    extends SettingsUtil.TokenColoringInitializer {

        public JspJavaLayerTokenColoringInitializer() {
            super(JspJavaLayerTokenContext.context);
        }

        public Object getTokenColoring(TokenContextPath tokenContextPath,
        TokenCategory tokenIDOrCategory, boolean printingSet) {
            if (!printingSet) {
                return new SettingsUtil.TokenColoringEvaluator(
                    tokenContextPath.getParent().getFullTokenName(tokenIDOrCategory),
                    null, printingSet);

            } else { // printing set
                 return SettingsUtil.defaultPrintColoringEvaluator;
            }
        }

    }

}

