/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.web.core.syntax;

import javax.swing.JEditorPane;
import java.util.MissingResourceException;

import org.openide.TopManager;
import org.openide.modules.ModuleInstall;
import org.openide.options.SystemOption;
import org.openide.text.PrintSettings;
import org.openide.util.NbBundle;
import org.openide.filesystems.FileUtil;

import org.netbeans.modules.editor.options.AllOptions;
import org.netbeans.editor.Settings;
import org.netbeans.editor.LocaleSupport;

import org.netbeans.modules.web.core.jsploader.JspLoader;

/**
 * @author Petr Jiricka
 */
public class RestoreColoring extends ModuleInstall {

    private static final long serialVersionUID = -1716424022405419381L;

    public void installed () {
        restored ();
    }

    public void restored () {
        addInitializer ();
        installOptions ();
    }

    public void uninstalled () {
        uninstallOptions ();
    }

    public void addInitializer () {
        Settings.addInitializer (new JspMultiSettingsInitializer());

        // Registration of the editor kits to JEditorPane
        JEditorPane.registerEditorKitForContentType
            (JspLoader.JSP_MIME_TYPE,
             "org.netbeans.modules.web.core.syntax.JSPKit", // NOI18N
             this.getClass().getClassLoader());

        // add a localizer to editor
        LocaleSupport.addLocalizer(new LocaleSupport.Localizer() {
            public String getString(String key) {
                try {
                    return NbBundle.getBundle(RestoreColoring.class).getString(key);
                } catch (MissingResourceException e) {
                    return null;
                }
            }
        });

    }


    public void installOptions () {
        AllOptions ao = (AllOptions)AllOptions.findObject (AllOptions.class, true);
        ao.addOption ((JSPOptions)SystemOption.findObject(JSPOptions.class, true));
        PrintSettings ps = (PrintSettings)PrintSettings.findObject (PrintSettings.class, true);
        ps.addOption ((JSPPrintOptions)SystemOption.findObject(JSPPrintOptions.class, true));
    }

    public void uninstallOptions () {
        AllOptions ao = (AllOptions)AllOptions.findObject (AllOptions.class, true);
        JSPOptions jspo = (JSPOptions)SystemOption.findObject(JSPOptions.class, false);
        if (jspo != null) ao.removeOption (jspo);
        PrintSettings ps = (PrintSettings)PrintSettings.findObject (PrintSettings.class, true);
        JSPPrintOptions jsppo = (JSPPrintOptions)SystemOption.findObject(JSPPrintOptions.class, false);
        if (jsppo != null) ps.removeOption (jsppo);
    }

} // end of clas RestoreColoring
