/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.web.templates.nbcontrol;

import java.util.*;
import java.io.IOException;
import java.util.ResourceBundle;

import org.openide.actions.*;
import org.openide.loaders.UniFileLoader;
import org.openide.loaders.MultiDataObject;
import org.openide.loaders.DataObjectExistsException;
import org.openide.util.NbBundle;
import org.openide.util.actions.SystemAction;
import org.openide.text.EditorSupport;
import org.openide.filesystems.FileObject;

import org.netbeans.modules.web.templates.nbcontrol.DWTDataObject;
import org.netbeans.modules.web.templates.nbcontrol.NewJspAction;

/**
 *  Data loader which recognizes dwt files.
 *  This class is final only for performance reasons
 *  and may unfinaled if desired.
 *
 */

public final class DWTDataLoader extends UniFileLoader {

    /** serialVersionUID */
    private static final long serialVersionUID = -5204981537870025258L;
    
    private static final String i18nBundle =          // NOI18N
	"org.netbeans.modules.web.templates.resources.Bundle";
    private static ResourceBundle resBundle = NbBundle.getBundle(i18nBundle);

    /** Creates new DWTDataLoader */
    public DWTDataLoader() {
	super(DWTDataObject.class);
    }

    /** Does initialization. Initializes extension list.
     */
    protected void initialize () {

	getExtensions().addExtension(resBundle.getString("EI_DWTFileExtension"));

    }

    /** Creates new DWTDataObject for this FileObject.
     * @param primaryFile FileObject
     * @return new DWTDataObject
     */
    protected MultiDataObject createMultiObject (final FileObject primaryFile)
	throws DataObjectExistsException, IOException {

	MultiDataObject obj = new DWTDataObject(primaryFile, this);
	EditorSupport   es  = new EditorSupport (obj.getPrimaryEntry ());

	es.setMIMEType("text/x-jsp");
	obj.getCookieSet().add(es);
	return obj;
    }

    /** Get the default display name of this loader.
     * @return default display name
     */
    protected String defaultDisplayName () {
	return (resBundle.getString("EI_DWTFileDisplayName"));
    }
    
    /** Get default actions.
     * @return array of default system actions.
     */
    protected SystemAction[] defaultActions () {
	return new SystemAction[] {
	    SystemAction.get(DWTOpenAction.class),
	    SystemAction.get (FileSystemAction.class),
	    null,
	    SystemAction.get(CutAction.class),
	    SystemAction.get(CopyAction.class),
	    SystemAction.get(PasteAction.class),
	    null,
	    SystemAction.get(DeleteAction.class),
	    SystemAction.get(RenameAction.class),
	    null,
	    SystemAction.get(ToolsAction.class),
	    SystemAction.get(NewJspAction.class),
	    SystemAction.get(PropertiesAction.class),
	};
    }
    
}
