/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.web.templates.nbcontrol;

import org.openide.*;
import org.openide.actions.ViewAction;
import org.openide.filesystems.*;
import org.openide.loaders.*;
import org.openide.nodes.*;
import org.openide.util.*;
import org.openide.util.actions.*;
import org.openide.text.EditorSupport;

/**
 * Object that provides main functionality for dwt data loader.
 * This class is final only for performance reasons,
 * can be unfinaled if desired.
 *
 * @author Bezita Crosby 
 */

public final class DWTDataObject extends MultiDataObject {

    public DWTDataObject (final FileObject obj,
			  final MultiFileLoader loader)
	throws DataObjectExistsException {

        super (obj, loader);

        // use editor support
        EditorSupport es = new EditorSupport(getPrimaryEntry());
        es.setMIMEType ("text/x-jsp");
        getCookieSet().add(es);
    }

    /**
     * Provides node that should represent this data object. When
     * a node for representation in a parent is requested by a call
     * to getNode (parent) it is the exact copy of this node
     * with only parent changed. This implementation creates instance
     * <CODE>DataNode</CODE>.
     * <P>
     * This method is called only once.
     *
     * @return the node representation for this data object
     * @see DataNode
     */
    protected Node createNodeDelegate () {
	return new DWTNode(this);
    }

    /** Help context for this object.
     * @return help context
     */
    public org.openide.util.HelpCtx getHelpCtx () {
	return new org.openide.util.HelpCtx (DWTDataObject.class);
    }
}
