/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.web.templates.nbcontrol;

import java.util.ResourceBundle;
import java.io.IOException;
import java.lang.Runtime;

import org.openide.util.NbBundle;
import org.openide.util.HelpCtx;
import org.openide.nodes.Node;
import org.openide.actions.OpenAction;
import org.openide.loaders.DataObject;
import org.openide.execution.NbProcessDescriptor;
import org.openide.NotifyDescriptor;
import org.openide.TopManager;
import org.openide.filesystems.FileStateInvalidException;

//public class DWTOpenAction extends CallableSystemAction {
public class DWTOpenAction extends OpenAction {

  private final static String i18nBundle =          // NOI18N
      "org.netbeans.modules.web.templates.resources.Bundle";
  private static ResourceBundle resBundle = NbBundle.getBundle(i18nBundle);
  private static String QUOTE = resBundle.getString("EI_DWQuote");

  // --------------------
  /** Name of the action. */
  public String getName () {
      return resBundle.getString("EI_DWOpenTemplateAction");
  }

  // --------------------
  /** No help yet. */
  public HelpCtx getHelpCtx () {
    return new HelpCtx (DWTOpenAction.class);
  }

  // --------------------
  protected String iconResource () {
      return resBundle.getString("EI_DWNoIcon");
  }

  // --------------------
  public void performAction(Node[] activatedNodes) {
     // What's the global option?
     DWEditorSettings edSettings = DWEditorSettings.OPTIONS;

     if (edSettings.getEditorName().equals(
               resBundle.getString("EI_DWExternalEditor"))) {
         // Invoke Custom editor
         openInCustomEditor(activatedNodes);
     }
     else {
         // Invoke default editor
         super.performAction(activatedNodes);
     }
  }

  // --------------------
  private void openInCustomEditor(Node[] activatedNodes) {

    //System.out.println("--------------------------------------");
    //System.out.println("DWTOpenAction--invoking custom editor");
    DWEditorSettings edSettings = DWEditorSettings.OPTIONS;

    String procFullName = QUOTE +
                          edSettings.getEditorPath() +
                          System.getProperty("file.separator") +
                          edSettings.getEditorExecName() +
                          QUOTE;

    DataObject dataObj = (DataObject)(activatedNodes[0]).getCookie(DataObject.class);
    char fileSeparator = System.getProperty("file.separator").charAt(0);
    String fileName = dataObj.getPrimaryFile().getPackageNameExt(fileSeparator, '.');

    try {
         String packageName =
                dataObj.getPrimaryFile().getFileSystem().getDisplayName();
         fileName = packageName +
                    System.getProperty("file.separator") +
                    fileName;

         // I've identified a bug in NbProcessDescriptor
         // and am using Runtime as a workaround until the
         // bug is fixed.  Bezita 5/4/2000
         //NbProcessDescriptor proc = new NbProcessDescriptor(procFullName,
                                                            //fileName);
         String[] cmdArray = new String[2];
         cmdArray[0] = procFullName;
         cmdArray[1] = fileName;

         // Used for debugging and verification of path and args.
         //System.out.println("Executing: ");
         //for (int i = 0; i < cmdArray.length; i++) {
              //System.out.println("  " + i + ". = " + cmdArray[i]);
         //}

         //proc.exec();
         Runtime.getRuntime().exec(cmdArray);

     } catch (FileStateInvalidException e) {
         TopManager.getDefault().notify(
                    new NotifyDescriptor.Exception(
                               e,
                               resBundle.getString("EI_DWInvalidFileSystem")));
     } catch (IOException e) {
         TopManager.getDefault().notify(
               new NotifyDescriptor.Exception(
                          e,
                          resBundle.getString("EI_DWInvalidProcess")));
    }
  }
}

