/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package  org.netbeans.modules.web.templates.ui;

import java.awt.Dialog;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.JFileChooser;
import java.util.ResourceBundle;
import java.io.IOException;

import org.openide.util.NbBundle;
import org.openide.TopManager;
import org.openide.DialogDescriptor;
import org.openide.NotifyDescriptor;
import org.openide.NotifyDescriptor.Message;

import org.netbeans.modules.web.templates.model.DWBeanTemplate;

public class DWIntegrationToolDialog {

    public static final String i18nBundle =
	"org.netbeans.modules.web.templates.resources.Bundle";
    // NOI18N

    private static ResourceBundle resBundle = NbBundle.getBundle(i18nBundle);

    // ---------------------
    public static void showDialog() {

	String dialogMessage = resBundle.getString("EI_DWToolMessage"); 
	String dialogTitle   = resBundle.getString("EI_DWToolTitle"); 

	// Display a dialog describing Dreamweaver Integration Tool.
	DialogDescriptor dialogDescriptor = new DialogDescriptor(dialogMessage,
								 dialogTitle);
	Dialog dialog = TopManager.getDefault().createDialog(dialogDescriptor);
	dialog.show();

	// Process user input
	if (dialogDescriptor.getValue() == DialogDescriptor.OK_OPTION) {

	    // Ask user to choose and identify a template.
	    JFileChooser fileChooser = new JFileChooser();
	    fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
	    fileChooser.setDialogTitle(resBundle.getString("EI_DWFileChooserTitle"));

	    int userInput = fileChooser.showOpenDialog(
						       TopManager.getDefault().getWindowManager().getMainWindow());

	    // Process user input if a file was chosen.
	    // Display the result to user.
	    if(userInput == JFileChooser.APPROVE_OPTION) {

		try {
                    DWBeanTemplate dwTemplate =
			DWBeanTemplate.getCustomTemplate(
							 fileChooser.getSelectedFile());
                    showTemplateAccessSuccess();
		} catch (IOException e) {
                    showTemplateAccessError();
		}
	    }

	}  // if OK_OPTION

    }  // showDialog

    // ---------------------
    private static void showTemplateAccessError() {
	String dialogMessage = resBundle.getString("EI_DWErrorAccessTempDialog");
	String dialogTitle   = resBundle.getString("EI_DWTemplateDialogTitle");

	NotifyDescriptor message = new NotifyDescriptor.Message(
								dialogMessage,
								NotifyDescriptor.ERROR_MESSAGE);

	message.setTitle(dialogTitle);
	TopManager.getDefault().notify(message);
    }

    // ---------------------
    private static void showTemplateAccessSuccess() {

	String dialogMessage = resBundle.getString("EI_DWTemplateDialog");
	String dialogTitle   = resBundle.getString("EI_DWTemplateDialogTitle");

	NotifyDescriptor message = new NotifyDescriptor.Message(
								dialogMessage,
								NotifyDescriptor.INFORMATION_MESSAGE);

	message.setTitle(dialogTitle);
	TopManager.getDefault().notify(message);
    }
}
