/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package  org.netbeans.modules.web.templates.ui;

import java.awt.event.*;
import java.awt.Dialog;
import java.util.ResourceBundle;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.InputStream;

import org.openide.windows.TopComponent;
import org.openide.util.NbBundle;
import org.openide.TopManager;
import org.openide.DialogDescriptor;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataFolder;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileLock;
import org.openide.filesystems.FileSystem;
import org.openide.nodes.Node;

import org.netbeans.modules.web.templates.nbcontrol.DWTCookie;

public class NewJspDialog {

    public static final String i18nBundle =           // NOI18N
	"org.netbeans.modules.web.templates.resources.Bundle";

    private static ResourceBundle resBundle = NbBundle.getBundle(i18nBundle);
    private static NewJspFromTemplatePanel panel;
    private static FileObject dwtFileObj;

    // ---------------------
    public static void showDialog() {

	String panelTitle = resBundle.getString("EI_DWNewJspFromTemplate");

	// Get dwt file
	Node[] nodes = TopComponent.getRegistry().getCurrentNodes();
	Node node;
	DWTCookie dwtCookie;

	if ((nodes != null) && nodes.length > 0) {
            node = nodes[0];
            dwtCookie = (DWTCookie)node.getCookie(DWTCookie.class);
            dwtFileObj = dwtCookie.getDWTDataObject().getPrimaryFile();
	} else {
            return;
	}

	// Create and Display dialog
	// Process user's input
	//
	panel = new NewJspFromTemplatePanel();
	DialogDescriptor dialogDescriptor = new DialogDescriptor(
								 panel,
								 panelTitle,
								 true,
								 new ActionListener() {
									 public void actionPerformed (ActionEvent evt) {
									     doNewJspFromTemplate(evt);
									 }
								     });

	Dialog dialog = TopManager.getDefault().createDialog(dialogDescriptor);
	dialog.show();

    }  // showDialog

    // ---------------------
    private static void doNewJspFromTemplate(ActionEvent evt) {

	String userAction = evt.getActionCommand();
	if (userAction.equals("Cancel")) {
	    return;
	}

	// Get the folder inside which a jsp file will be created.
	DataFolder dataFolder = panel.getDataFolder();
	String jspFileName = panel.getFileName();

	// Nothing to do if the user didn't select a package
	// and a file name ...
	if ( (dataFolder == null) || (jspFileName.length() == 0) ) {
	    return;
	}

	FileObject parentFileObj = dataFolder.getPrimaryFile();
	FileObject jspFileObj;
	OutputStream outputStream;
	InputStream inputStream;

	try {
	    // Create the jsp file
	    // Get a lock on it
	    // Copy the contents of dwt file into the jsp file
	    //
	    jspFileObj = parentFileObj.createData(jspFileName, "jsp");
	    outputStream = jspFileObj.getOutputStream(jspFileObj.lock());

	    try {
		inputStream = dwtFileObj.getInputStream();

		PrintWriter jspWriter = new PrintWriter(outputStream);
		BufferedReader dwtReader = new BufferedReader(
							      new InputStreamReader(inputStream));

		String line;
		while ((line = dwtReader.readLine()) != null) {
		    jspWriter.println(line);
		}

		dwtReader.close();
		jspWriter.close();

	    } catch (FileNotFoundException e) {
		System.out.println(
				   "FileNotFoundException trying to get dwt FileObject inputStream.");
	    }

	} catch (IOException e) {
            System.out.println("IOException trying to create jsp file.");
	}
    }
}

