/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package  org.netbeans.modules.web.templates.ui;

import java.beans.*;
import java.lang.ref.*;
import java.io.File;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.BorderLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.event.KeyEvent;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import java.util.Enumeration;
import java.io.IOException;
import javax.swing.JPanel;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.border.EmptyBorder;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;

import org.openide.loaders.*;
import org.openide.nodes.*;
import org.openide.filesystems.*;
import org.openide.util.NbBundle;
import org.openide.util.HelpCtx;
import org.openide.util.Utilities;
import org.openide.util.enum.*;
import org.openide.TopManager;
import org.openide.DialogDescriptor;
import org.openide.explorer.ExplorerPanel;
import org.openide.explorer.ExplorerManager;
import org.openide.explorer.view.BeanTreeView;

// ---------------------------------

final class NewJspFromTemplatePanel extends JPanel
    implements DocumentListener,
    DataFilter,
    PropertyChangeListener,
    VetoableChangeListener {

    public static final String i18nBundle =           // NOI18N
	"org.netbeans.modules.web.templates.resources.Bundle";
    private static ResourceBundle resBundle = NbBundle.getBundle(i18nBundle);

    private JPanel        namePanel;
    private JPanel        packageNamePanel;
    private ExplorerPanel packagesPanel;

    private JTextField nameTextField;
    private JTextField packageTextField;

    private JLabel nameLabel;
    private JLabel packageLabel;
    private JLabel descriptionLabel;

    private BeanTreeView beanTreeView;
    private Node rootNode;

    // listener to changes in the panel 
    private ChangeListener listener;

    // preferred dimension
    static Dimension PREF_DIM = new Dimension(600, 300);

    // system reference (FileSystem)
    private Reference system = new WeakReference (null);

    // ---------------------
    NewJspFromTemplatePanel () {
	initComponents();
	setName(resBundle.getString("EI_DWNewJspFromTemplate"));
	setBorder(new EmptyBorder(new Insets(8, 8, 8, 8)));
	rootNode = createPackagesNode();

	packagesPanel.getExplorerManager ().setRootContext (rootNode);
	packagesPanel.getExplorerManager ().addPropertyChangeListener (this);
	packagesPanel.getExplorerManager ().addVetoableChangeListener (this);

	// registers itself to listen to changes in the content of document
	packageTextField.getDocument().addDocumentListener(this);
	packageTextField.unregisterKeyboardAction(
						  KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0));

	nameTextField.getDocument().addDocumentListener(this);
	nameTextField.unregisterKeyboardAction(
					       KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0));
    }

    // ---------------------
    private void initComponents () {

	namePanel        = new JPanel();
	packageNamePanel = new JPanel();
	packagesPanel    = new ExplorerPanel();

	nameTextField     = new JTextField();
	packageTextField  = new JTextField();

	nameLabel        = new JLabel();
	packageLabel     = new JLabel();
	descriptionLabel = new JLabel();
	beanTreeView     = new BeanTreeView();

	setLayout(new BorderLayout(0, 8));
	namePanel.setLayout(new GridBagLayout ());

	GridBagConstraints gridBagConstraints = new GridBagConstraints();
	gridBagConstraints.insets = new Insets (0, 0, 0, 8);
	gridBagConstraints.anchor = GridBagConstraints.WEST;

	nameLabel.setText(resBundle.getString("EI_DWNewJspDialogNameLabel"));
	namePanel.add(nameLabel, gridBagConstraints);
	descriptionLabel.setText(
				 resBundle.getString("EI_DWNewJspDialogDescriptionLabel"));

	nameTextField.addFocusListener (new FocusAdapter () {
		public void focusGained (FocusEvent evt) {
		    nameTextFieldFocusGained (evt);
		}
	    });

	gridBagConstraints = new GridBagConstraints();
	gridBagConstraints.gridwidth = 0;
	gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
	gridBagConstraints.weightx = 1.0;
	namePanel.add (nameTextField, gridBagConstraints);

	add(namePanel, BorderLayout.NORTH);
	packagesPanel.setLayout(new BorderLayout(0, 8));
	packageNamePanel.setLayout(new GridBagLayout());
	GridBagConstraints gridBagConstraints2;

	packageLabel.setText(resBundle.getString("EI_DWNewJspDialogPackageLabel"));
	gridBagConstraints2 = new GridBagConstraints();
	packageNamePanel.add(packageLabel, gridBagConstraints2);

	packageTextField.addFocusListener(new FocusAdapter () {
		public void focusGained (FocusEvent evt) {
		    packageTextFieldFocusGained (evt);
		}
	    });

	gridBagConstraints2 = new GridBagConstraints ();
	gridBagConstraints2.fill = GridBagConstraints.HORIZONTAL;
	gridBagConstraints2.insets = new Insets (0, 8, 0, 0);
	gridBagConstraints2.weightx = 1.0;
	packageNamePanel.add(packageTextField, gridBagConstraints2);

	packagesPanel.add (packageNamePanel, BorderLayout.SOUTH);
	beanTreeView.setPopupAllowed (false);
	packagesPanel.add (beanTreeView, BorderLayout.CENTER);
	packagesPanel.add (descriptionLabel, BorderLayout.NORTH);
	add (packagesPanel, java.awt.BorderLayout.CENTER);
    }

    // ---------------------
    public String getFileName() {
	return nameTextField.getText();
    }

    public String getPackageName() {
	return packageTextField.getText();
    }

    public DataFolder getDataFolder() {
        Node[] nodes = packagesPanel.getExplorerManager().getSelectedNodes();
        if (nodes.length == 1) {
            return (DataFolder)nodes[0].getCookie(DataFolder.class);
        } else {
            return null;
        }
    }

    // ---------------------
    // Creates node that displays all packages.
    private Node createPackagesNode () {
	return TopManager.getDefault().getPlaces().nodes().repository(this);
    }

    // ---------------------

    // Preferred size
    public Dimension getPreferredSize() {
        return PREF_DIM;
    }

    // Request focus
    public void requestFocus () {
	nameTextField.requestFocus();
	nameTextField.selectAll();
    }

    // ---------------------
    /**
     * Should the data object be displayed or not?
     * @param obj the data object
     * @return <CODE>true</CODE> if the object should be displayed,
     *    <CODE>false</CODE> otherwise
     */
    public boolean acceptDataObject(DataObject obj) {
	return obj instanceof DataFolder;
    }

    // ---------------------
    // Allow only simple selection.

    public void vetoableChange(PropertyChangeEvent ev)
	throws PropertyVetoException {

	if (ExplorerManager.PROP_SELECTED_NODES.equals (ev.getPropertyName ())) {
	    Node[] arr = packagesPanel.getExplorerManager().getSelectedNodes();

	    if (arr.length > 1) {
		throw new PropertyVetoException(
						"Only single selection allowed", ev); // NOI18N
	    }
	}
    }

    // ---------------------
    // Changes in selected node in packages.
    public void propertyChange (PropertyChangeEvent ev) {

	if (ExplorerManager.PROP_SELECTED_NODES.equals (ev.getPropertyName ())) {
	    Node[] arr = packagesPanel.getExplorerManager ().getSelectedNodes ();

	    if (arr.length == 1) {
		DataFolder df = (DataFolder)arr[0].getCookie (DataFolder.class);
		if (df != null) {
		    setTargetFolder(df);
		    return;
		}
	    }
	    setTargetFolder ((DataFolder)null);
	}
    }

    // ---------------------
    private void nameTextFieldFocusGained(FocusEvent evt) {
        nameTextField.selectAll();
    }

    private void packageTextFieldFocusGained(FocusEvent evt) {
        packageTextField.selectAll();
    }

    private void packageModelChanged(ListDataEvent evt) {
    }

    // Fires info to listener
    private void fireStateChanged () {
	if (listener != null) {
	    listener.stateChanged (new ChangeEvent (this));
	}
    }

    // ---------------------
    // Modification of package name or class name
    public void changedUpdate(final DocumentEvent p1) {

	if (p1.getDocument () == packageTextField.getDocument ()) {
	    SwingUtilities.invokeLater (new Runnable () {
		    public void run () {
			String text = packageTextField.getText ();
			if (text != null) {
			    setTargetFolder (text);
			}
		    }
		});
	    return;
	}

	if (p1.getDocument () == nameTextField.getDocument()) {
	    fireStateChanged ();
	}
    }

    // ---------------------
    public void removeUpdate(final DocumentEvent p1) {

	if (p1.getDocument () == packageTextField.getDocument ()) {
	    SwingUtilities.invokeLater(new Runnable () {
		    public void run () {
			if (packageTextField.getText ().length () == 0) {
			    FileSystem fs = (FileSystem)system.get ();
			    if (fs != null) {
				DataFolder df = DataFolder.findFolder (fs.getRoot ());
				setTargetFolder (df);
				packageTextField.selectAll ();
			    }
			}
		    }
		});
	}

	// so just check the name
	if (p1.getDocument () == nameTextField.getDocument ()) {
	    fireStateChanged ();
	}
    }

    // ---------------------
    public void insertUpdate(final DocumentEvent p1) {
        changedUpdate (p1);
    }

    // ---------------------

    /**
     * Get the component displayed in this panel.
     * @return the component
     */
    public Component getComponent () {
        return this;
    }

    // ---------------------

    /**
     * Help for this panel.
     * @return the help or <code>null</code> if no help is supplied
     */
    public HelpCtx getHelp () {
        return new HelpCtx (NewJspFromTemplatePanel.class);
    }

    // ---------------------
    public boolean isValid () {

        String text = packageTextField.getText ();
        if (text.length () == 0) {
            Node[] arr = packagesPanel.getExplorerManager ().getSelectedNodes ();
            if (arr.length == 1 && arr[0] == rootNode) {
                return false;
            }
        }

        StringTokenizer tok = new StringTokenizer (text, "."); // NOI18N
        while (tok.hasMoreElements ()) {
            String pkg = tok.nextToken ();
            if (!Utilities.isJavaIdentifier (pkg)) {
                return false;
            }
        }
        return true;
    }

    // ---------------------

    /** Computes a suggestion for a given prefix and
     * a list of file objects.
     *
     * @param node the node to start with
     * @param pref prefix
     * @param first [0] is the first node that satisfies the suggestion
     * @return the longest continuation string for all folders that
     *    starts with prefix
     */
    private static String computeSuggestion (Node node,
                                             String pref,
                                             Node[] first) {
        Node[] arr = node.getChildren ().getNodes ();
        String match = null;

        for (int i = 0; i < arr.length; i++) {
            String name = arr[i].getName ();
            if (name.startsWith (pref)) {
                // ok, has the right prefix
                if (match == null) {
                    // first match
                    match = name;
                    first[0] = arr[i];
                } else {
                    // find common part of the names
                    int indx = pref.length ();
                    int end = Math.min (name.length (), match.length ());
                    while (indx < end && match.charAt (indx) == name.charAt (indx)) {
                        indx++;
                    }
                    match = match.substring (0, indx);
                }
            }
        }

        if (match == null || match.length () == pref.length ()) {
            return null;
        } else {
            return match.substring (pref.length ());
        }
    }

    // ---------------------

    /**
     * Presets a target folder.
     * @param f the folder
     * @return true if succeeded
     */
    private boolean setTargetFolder (final DataObject f) {
        boolean exact;
        Node n;
        String name;
        String fullName = null;

        if (f != null) {
            FileObject fo = f.getPrimaryFile ();

            //name = fo.getPackageName('.');
            name = fo.getPackageNameExt(File.separatorChar, '.');

            // NOI18N
            StringTokenizer st = new StringTokenizer (name, "" + File.separatorChar);
            try {
                FileSystem fs = fo.getFileSystem ();

                if (fo.isRoot ()) {
                    fullName = defaultFSName (fs);
                } else {
                    fullName = defaultFSName(fs) + File.separatorChar + name;
                }

                system = new WeakReference (fs);

                Enumeration en = new SequenceEnumeration (
							  new SingletonEnumeration (fs.getSystemName()), st);

                n = NodeOp.findPath (rootNode, en);
                exact = true;
            } catch (FileStateInvalidException ex) {
                // invalid state of file system => back to root
                n = rootNode;
                name = ""; // NOI18N
                exact = false;
            } catch (NodeNotFoundException ex) {
                n = ex.getClosestNode();
                DataFolder df = (DataFolder)n.getCookie (DataFolder.class);
                if (df != null) {
                    name = df.getPrimaryFile ().getPackageName (File.separatorChar);
                } else {
                    name = "";
                }
                exact = false;
            }

        } else {
            // null folder => use root
            n = rootNode;
            name = null;
            exact = true;
        }

        // remove listener + do change + add listener
        ExplorerManager em = packagesPanel.getExplorerManager ();
        em.removePropertyChangeListener (this);
        packageTextField.getDocument ().removeDocumentListener (this);

        try {
            em.setSelectedNodes (new Node[] { n });
        } catch (PropertyVetoException ex) {
            throw new InternalError ();
        }

        packageTextField.setText (fullName != null ? fullName : name);

        packageTextField.getDocument ().addDocumentListener (this);
        em.addPropertyChangeListener (this);

        fireStateChanged ();
        return exact;
    }

    // ---------------------

    /**
     * Presets a target folder.
     * @param f the name of target folder
     * @return true if succeeded
     */
    private boolean setTargetFolder (final String f) {
        Node n = null;
        NodeNotFoundException closest = null;
        Node[] arr = rootNode.getChildren ().getNodes ();

        for (int i = 0; i < arr.length; i++) {
            Node root = arr[i];

            // NOI18N
            StringTokenizer st = new StringTokenizer (f, "" + File.separatorChar);

            try {
                n = NodeOp.findPath (root, st);
                break;
            } catch (NodeNotFoundException ex) {
                if (closest == null && !st.hasMoreElements ()) {
                    // a test for !hasMoreElements is here to be sure that
                    // all tokens has been read, so only the last item
                    // has not been found
                    closest = ex;
                }
            }
        }

        if (n != null) {
            // closest node not used
            closest = null;

        } else {

            if (closest == null) {
                // the node has not been even found
                return false;
            }

            // we will select the closest node found
            n = closest.getClosestNode ();
        }

        // remove listener + do change + add listener
        ExplorerManager em = packagesPanel.getExplorerManager ();
        em.removePropertyChangeListener (this);

        // change the text if we want to add suggestion
        if (closest != null) {
            Node[] first = new Node[1];
            final String sugg = computeSuggestion (closest.getClosestNode (),
                                                   closest.getMissingChildName(),
                                                   first);

            if (sugg != null) {
                packageTextField.getDocument ().removeDocumentListener (
									NewJspFromTemplatePanel.this);

                packageTextField.setText (f + sugg);
                javax.swing.text.Caret c = packageTextField.getCaret ();
                c.setDot (f.length () + sugg.length ());
                c.moveDot (f.length ());

                packageTextField.getDocument ().addDocumentListener(
								    NewJspFromTemplatePanel.this);
            }

            if (first[0] != null) {
                // show the first node that fits
                n = first[0];
            }
        }


        // change the node
        try {
            em.setSelectedNodes (new Node[] { n });
        } catch (PropertyVetoException ex) {
            throw new InternalError ();
        }

        // change the selected filesystem
        DataFolder df = (DataFolder)n.getCookie (DataFolder.class);

        if (df != null) {
            try {
                FileSystem fs = df.getPrimaryFile ().getFileSystem ();
                system = new WeakReference (fs);
            } catch (FileStateInvalidException ex) {
            }
        }

        em.addPropertyChangeListener (this);
        fireStateChanged ();
        return closest == null;
    }

    // ---------------------

    /**
     * Creates default package name for given file system.
     * @param fs the file system
     * @return localized name of default package
     */
    private static String defaultPackageName (FileSystem fs) {
        String n = fs == null ? "" : fs.getDisplayName ();
        return "<default package of " + n + ">";
    }

    // ---------------------

    /**
     * Returns default name for given file system.
     * @param fs the file system
     * @return localized name of default package
     */
    private static String defaultFSName (FileSystem fs) {
        String name = (fs == null ? "" : fs.getDisplayName ());
        return  name;
    }

}

