/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package  org.netbeans.modules.web.wizards.beanjsp.model;

import org.netbeans.modules.web.wizards.beanjsp.util.*;
import org.netbeans.modules.web.wizards.util.*;

import java.lang.*;
import java.io.*;
import java.util.*;
import java.lang.reflect.*;

import java.beans.*;


public class JSPBeanMethod extends Object implements JSPItem, Cloneable {

    MethodDescriptor methodDesc;
    JSPBean jspBean;
  
    // Method editing model data structure
  
    String beginScript;  String endScript;
    String returnVar;
    String[] paramVars;
  
    boolean edited = false;
    /** Constructor */  
    public JSPBeanMethod(JSPBean jspBean, MethodDescriptor methodDesc) { 
	this.jspBean = jspBean;
	this.methodDesc = methodDesc;
	
	beginScript="// before method call \n";  // // NOI18N	
	endScript="// after method call \n";	// NOI18N
	Method method = methodDesc.getMethod();
	Class[] params = method.getParameterTypes();
	Class ret = method.getReturnType();		
	returnVar = this.toClassTypeString(ret)+" retVar ";  // NOI18N	
	
	paramVars = new String[params.length];
	for(int i=0; i < params.length; ++i) {		
	    paramVars[i] = this.toClassTypeString(params[i]);					
	}
    }
    public Object clone() { 	  
	try {		
	    JSPBeanMethod cloneJSPBeanMethod = (JSPBeanMethod) super.clone(); 
	    cloneJSPBeanMethod.setReturnVar(this.getReturnVar());
	    cloneJSPBeanMethod.setParamVars(this.getParamVars());
	    return cloneJSPBeanMethod;	  
	}
	catch(Exception ex) { return null;}
    }  
    
    public String getName() { return this.methodDesc.getName();}

    public boolean hasKey(Object key) {
	if(!(key instanceof JSPBean))
	    return false;
	return isMethodOf((JSPBean)key);
    }
  
    public Object getKey() { return jspBean; }
  
    public boolean isMethodOf(JSPBean jspBean) {
	if(this.jspBean.getBeanName().equals(jspBean.getBeanName()))
	    return true;
	else
	    return false;
    }
    public JSPBean getJSPBean() { return jspBean; }    
    public boolean hasReturnType() {
	Method method = methodDesc.getMethod();
	Class ret = method.getReturnType();
	return !ret.isAssignableFrom(Void.TYPE);
    }
    public boolean isNonVoidMethod() {
	return ( (this.hasReturnType()) || (paramVars != null && paramVars.length > 0));  }  
    public void setEdited(boolean edited) { this.edited = edited;}
    public boolean isEdited(){return this.edited;}
  
    public Class[] getParamTypes() { 
	return methodDesc.getMethod().getParameterTypes(); 
    }  
    
    public Class getReturnType() { 
	return methodDesc.getMethod().getReturnType(); 
    }
  
    public void setReturnVar(String returnVar) { this.returnVar = returnVar; }
    public String getReturnVar() { return this.returnVar; }
    public String getParamVars(int idx) { return paramVars[idx]; }
    public String[] getParamVars() {
	String[] vars = new String[paramVars.length];
	System.arraycopy(paramVars,0,vars,0,paramVars.length);
	return vars;
    }
    public void setParamVars(int idx, String paramVar) { paramVars[idx] = paramVar;}
    public void setParamVars(String[] vars) {
	System.arraycopy(vars,0,paramVars,0,paramVars.length);
    }

    public void setBeginScript(String beginScript) { this.beginScript = beginScript;}
    public String getBeginScript() { 
	return this.beginScript;
    }   

    public void setEndScript(String endScript) { 
	this.endScript = endScript;
    }
    public String getEndScript() { return this.endScript;}

    public String getBeginJSPCode() {	
	String methodComments = ""; // NOI18N	
	String beginScriptTag = "<% \n"; // NOI18N
	if(this.isEdited()) {	   		
	    methodComments = 
"// This Script for method call is generated from user input. "+ // NOI18N
" This may or may not compile/execute correctly. Please Review \n"; // NOI18N	
	} 
	else {		
	    Class ret = methodDesc.getMethod().getReturnType();
	    if(!ret.isAssignableFrom(Void.TYPE) || 
	       (paramVars != null && paramVars.length > 0) )  { 
		methodComments = 
"// This method has non void return type or"+ // NOI18N
" one or more parameters required. Please update \n"; // NOI18N 
		
	    }
	}		
	return beginScriptTag+methodComments;
    }   

    public String getEndJSPCode() {
	String endScriptTag = "%> \n";		// NOI18N 
	return endScriptTag;
    }

    public String getMethodCallJSPCode() {
	String beanVar = jspBean.getBeanVariableName();	  
	  
	Method method = methodDesc.getMethod();
	Class ret = method.getReturnType();
	
	String methodCall = beanVar+"."+method.getName(); // NOI18N
	String methodComments = "";
   
	// not edited and have return value or parameters. so generate
	// code in comments		

	if(!this.isEdited() && (!ret.isAssignableFrom(Void.TYPE) ||
				(paramVars != null && paramVars.length
				 > 0)) )  { 
	    methodComments = "// ";		// NOI18N
	    // JSPPageWizard.doNonVoidMethodWarning = true;  
	    // show warning dialog		
	}
	StringBuffer methodCallFormat = new StringBuffer();
	methodCallFormat.append(methodComments);		
	if(!ret.isAssignableFrom(Void.TYPE)) {
	    methodCallFormat.append(returnVar+" = ");
	    // NOI18N		
	}
		
	
	methodCallFormat.append(methodCall+"(");// NOI18N
	if(paramVars != null && paramVars.length > 0) {
	    methodCallFormat.append(" "+paramVars[0]);							 // NOI18N
	    for(int i=1; i < paramVars.length; ++i) {
		methodCallFormat.append(", "+paramVars[i]);						 // NOI18N
	    }
	}
	methodCallFormat.append(" ); \n"); // NOI18N
	return methodCallFormat.toString();
    }    

    public String toJSPCode() {
	return  this.getBeginJSPCode()+
	    this.getBeginScript()+
	    this.getMethodCallJSPCode()+
	    this.getEndScript()+
	    this.getEndJSPCode();  
    }
    /* ***
       public String toJSPCode() {
	  
       String beanVar = jspBean.getBeanVariableName();	  
	  
       Method method = methodDesc.getMethod();
       Class ret = method.getReturnType();
       String methodCall = beanVar+"."+method.getName();								 // NOI18N
       
       if( ret.isAssignableFrom(Void.TYPE) && 
       (paramVars == null || paramVars.length == 0) )  {
       return " <% "+															// NOI18N
       methodCall+"( );"+														// NOI18N
       " %> \n";																// NOI18N
       
       } else {
       String methodComments = "<!-- This method has non void return type or"+									// NOI18N
       " one or more parameters required. Please update -->\n";		// NOI18N
       
       StringBuffer nonVoidMethodCall = new StringBuffer();
       
       if(!ret.isAssignableFrom(Void.TYPE))
       nonVoidMethodCall.append(returnVar+" = ");				 // NOI18N
       
       nonVoidMethodCall.append(methodCall+"(");								 // NOI18N
       if(paramVars != null && paramVars.length > 0) {
       nonVoidMethodCall.append(" "+paramVars[0]);							 // NOI18N
       for(int i=1; i < paramVars.length; ++i) {
       nonVoidMethodCall.append(", "+paramVars[i]);						 // NOI18N
       }
       }
       nonVoidMethodCall.append(" );");										 // NOI18N
       
       JSPPageWizard.doNonVoidMethodWarning = true;
			
       if(this.isEdited()){				methodComments = "<!-- This Script for method call is generated from user input. "+				// NOI18N
       " This may or may not compile/execute correctly. Please Review -->\n";		// NOI18N
       return methodComments+"<% \n"+this.getBeginScript()+" \n"+				 // NOI18N
       nonVoidMethodCall.toString()+
       "\n "+this.getEndScript()+"\n %> \n";			// NOI18N
       } else {					return methodComments+"<!-- <\\% "+									 // NOI18N
       nonVoidMethodCall.toString()+
       " %>  --> \n";			// NOI18N
       }
       }
       
       }    *** */
  
    public String getDisplayName() {
	
	Method method = methodDesc.getMethod();
	Class[] params = method.getParameterTypes();
	StringBuffer displayBuff = new StringBuffer();
	displayBuff.append(method.getName()+"("); // NOI18N

	if(params != null && params.length > 0) {

	    displayBuff.append(" " + 
			       this.toClassTypeString(params[0])); // NOI18N
	    for(int i=1; i < params.length; ++i) { 
		displayBuff.append(", " + 
				   this.toClassTypeString(params[i])); // NOI18N			
	    }
	}
	displayBuff.append(")");				 // NOI18N
	// return displayBuff.toString();	  
	return jspBean.getBeanVariableName()+"."+displayBuff.toString();
    }    

    
    public static String toClassTypeString(Class paramClass) {
	// Class arrayObjectClass = arrayClass.getComponentType();
	if(!paramClass.isArray())		  
	    return paramClass.getName();
	String arrayRep = paramClass.getName();
	int arrayDepth = arrayRep.lastIndexOf('[')+1;	  
	int nonPrimitiveIdx = arrayRep.lastIndexOf(';');	  
	String arrayObjectName = "";	// NOI18N	  
	if(nonPrimitiveIdx >= 0) {
	    arrayObjectName =
		arrayRep.substring(arrayDepth+1,nonPrimitiveIdx);
	} else { 
	    // primitive type
	    int typeChar = arrayRep.charAt(arrayDepth);
	    switch(typeChar) {		  case 'B' :
		arrayObjectName = "byte"; // NOI18N
	    break;		  
	    case 'C' :
		arrayObjectName = "char";     // NOI18N
		break;
	    case 'D' :
		arrayObjectName = "double";		// NOI18N
		break;
	    case 'F' :
		arrayObjectName = "float";		// NOI18N
		break;
	    case 'I' :
		arrayObjectName = "int";		// NOI18N
		break;
	    case 'J' :
		arrayObjectName = "long";		// NOI18N
		break;
	    case 'S' :
		arrayObjectName = "short";		// NOI18N
		break;
	    case 'Z' :
		arrayObjectName = "boolean";// NOI18N
		break;
	    }	  
	}	  
	StringBuffer arrayName = new StringBuffer();
	arrayName.append(arrayObjectName);
	arrayName.append(" ");	// NOI18N	  
	for(int i=0; i < arrayDepth; ++i) {
	    arrayName.append("[]");		// NOI18N	  
	}	  	  
	return arrayName.toString();
    }
  
    public String toString() { 
	return this.getDisplayName(); 
    }
}


    
