/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package  org.netbeans.modules.web.wizards.beanjsp.model;

import org.netbeans.modules.web.wizards.util.*;
import org.netbeans.modules.web.wizards.beanjsp.util.*;
import org.netbeans.modules.web.wizards.beanjsp.ide.*;


import org.netbeans.modules.web.templates.model.DWBeanTemplate;

import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObject;import org.openide.filesystems.*;
import org.openide.filesystems.FileObject;import org.openide.TopManager;


import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import javax.swing.event.*;
import javax.swing.*;
import javax.swing.table.*;
import java.util.*;

import java.io.*;
import java.lang.reflect.*;
import java.beans.*;

public class JSPPage extends Object {
    
	public static BeanManager beanManager = new BeanManager();	
	
	private String jspName;	
	
	boolean errorPage = false;
	private String errorURL;

	boolean submitAllowed = true;
	private String submitURL;
	private String submitButtonLabel;
	
	private int pageFormStyle;
	
	DWBeanTemplate jspTemplate;

	private JSPVector useBeans;
	private JSPVector initProperties;
	private JSPVector methodCalls;
	private JSPVector dispProperties;
	
	private JSPBeanTableModel useBeansModel;
	private JSPBeanPropInitTableModel initPropertiesModel;	
	private JSPItemListModel methodCallsModel;
	private JSPDisplayFieldTableModel dispPropertiesModel;


    public JSPPage() {
		initializeJSPPageModel();
    }                                     
  
    private void initializeJSPPageModel() {
	  jspName = getDefaultPageName();
	  
	  errorPage = false;
	  errorURL = "";					 // NOI18N
	  submitAllowed = true;
	  submitURL = "";					 // NOI18N
	  submitButtonLabel="Submit";		 // NOI18N
	  
	  pageFormStyle = HTMLForm.FS_COLUMN;	  
	  
	  try {
		jspTemplate = DWBeanTemplate.getDefaultTemplate();
	  }catch(Exception ex) { 
		jspTemplate = null; 
		Debug.print(ex); // ignore safely
	  }
	  
	  useBeans = new JSPVector();
	  initProperties = new JSPVector();
	  methodCalls = new JSPVector();	  
	  dispProperties = new JSPVector();
	  
	  initializeModels();
    }
    
    private String getDefaultPageName() {
        return "JSPPage";						 // NOI18N
    }
    
    /** Returns JSP name */
    public String getJSPName() {
      return jspName;    
    }

    /** Sets JSP name */
    public void setJSPName(String name) {
		this.jspName = name;
    }

	public boolean isErrorPage() { return errorPage; }
	public void setErrorPage(boolean errorPage) { this.errorPage = errorPage;}
	
	public String getErrorURL() { return errorURL;}
	public void setErrorURL(String errorURL) { this.errorURL = errorURL;}

	public boolean isSubmitAllowed() { return submitAllowed; }
	public void setSubmitAllowed(boolean submitAllowed) { this.submitAllowed = submitAllowed;}
	
	public String getSubmitURL() { return submitURL;}
	public void setSubmitURL(String submitURL) { this.submitURL = submitURL;}

	public String getSubmitButtonLabel() { return submitButtonLabel;}
	public void setSubmitButtonLabel(String submitButtonLabel) { this.submitButtonLabel = submitButtonLabel;}
	
	public int getPageFormStyle() { return pageFormStyle; }
	public void setPageFormStyle(int pageFormStyle) { this.pageFormStyle = pageFormStyle; }
	
	public DWBeanTemplate getJSPTemplate() { return jspTemplate; }
	public void setJSPTemplate(DWBeanTemplate jspTemplate) { this.jspTemplate = jspTemplate;}
	
	//// model management
	
	public void initializeModels() {
		
		useBeansModel = new JSPBeanTableModel(this.useBeans);
		methodCallsModel = new JSPItemListModel(this.methodCalls);
		initPropertiesModel = new JSPBeanPropInitTableModel(this.initProperties);		
		dispPropertiesModel = new JSPDisplayFieldTableModel(this.dispProperties);
		
		useBeansModel.addTableModelListener(new TableModelListener() {
			public void tableChanged(TableModelEvent evt) {
				if(evt instanceof JSPBeanTableModel.JSPBeanTableModelEvent) {
				if(evt.getType() == TableModelEvent.DELETE) {
					JSPBean jspBean = ((JSPBeanTableModel.JSPBeanTableModelEvent)evt).getJSPBean();
					removeFieldsAndMethods(jspBean);
				}else if(evt.getType() == TableModelEvent.INSERT) {
					JSPBean jspBean = ((JSPBeanTableModel.JSPBeanTableModelEvent)evt).getJSPBean();
					addFieldsAndMethods(jspBean);
				}
				}
			}
		});
		

	}

	private void addFieldsAndMethods(JSPBean jspBean) {
		// remove fields and methods from usebean fields & methods list
		// and add it to available beans fields & methods list
		// Debug.println("adding the Fields and Methods for Bean : "+jspBean.getBeanName());

		JSPVector setterFields = (JSPVector)JSPPage.beanManager.getValidJSPBeanSetterFields(jspBean);
		initPropertiesModel.addItems(setterFields);
		
	}
	
	
	private void removeFieldsAndMethods(JSPBean jspBean) {

		if(jspBean == null) {

			initPropertiesModel.removeAll();			
			methodCallsModel.removeAll();
			dispPropertiesModel.removeAll();
			
		} else {
		   // Debug.println("removing the Fields and Methods for Bean : "+jspBean.getBeanName());
			
			initPropertiesModel.removeAll(jspBean);		
			methodCallsModel.removeAll(jspBean);
			dispPropertiesModel.removeAll(jspBean);
		
		}
		
	}

	public JSPBeanTableModel getJSPBeansModel() {
		return this.useBeansModel;
	}

	public JSPBeanPropInitTableModel getInitPropertiesModel() {
		return this.initPropertiesModel;
	}
	
	
	public JSPItemListModel getMethodCallsModel() {
		return this.methodCallsModel;
	}
	
	public JSPDisplayFieldTableModel getDispPropertiesModel() {
		return this.dispPropertiesModel;
	}

	//// add init info to use bean
	
	public void updateUseBeanInitProperties() {
		Iterator iterator = this.initProperties.iterator();
		for(; iterator.hasNext(); ) {
			JSPBeanField initField = (JSPBeanField)iterator.next();
			//// make sure no duplicates for this
			initField.selfPopulateToUseBeanInitProperties();
		}
	}
	
	//// Code generation Methods
	
	public void writeJSPPageDirectiveCode(PrintWriter jspWriter) {
		if(this.isErrorPage()) {
			jspWriter.println("<%@ page isErrorPage=\"true\" %> \n");						 // NOI18N
		} else {
			String errorURL = this.getErrorURL();
			if(errorURL != null && errorURL.trim().length() > 0 ) {
				jspWriter.println("<%@ page errorPage=\""+this.getErrorURL()+"\" %> \n");		 // NOI18N	
			}
		}
	}
	
	public void writeJSPBeanUseBeanCode(PrintWriter jspWriter) {
		
		////todo: make sure that you are not calling this multiple times
		updateUseBeanInitProperties();
			
		jspWriter.println();
		Iterator beansIterator = useBeans.iterator();
		for(; beansIterator.hasNext(); ) {
			JSPBean jspBean = (JSPBean)beansIterator.next();
			jspWriter.println(jspBean.toJSPCode());
		}
		jspWriter.println();		
	}
	
	public void writeJSPBeanPropertyInitializationCode(PrintWriter jspWriter){
		
		jspWriter.println();
		Iterator iterator = this.initProperties.iterator();
		for(; iterator.hasNext(); ) {
			JSPBeanField jspBeanField = (JSPBeanField)iterator.next();
			if(jspBeanField.isAccessInit()) {
				jspWriter.println(jspBeanField.toSubmitProcessJSPCode());
			}
		}
		jspWriter.println();
	
	}
	
	public void writeJSPBeanMethodCallCode(PrintWriter jspWriter){
		
		jspWriter.println();
		Iterator iterator = this.methodCalls.iterator();
		for(; iterator.hasNext(); ) {
			JSPBeanMethod jspBeanMethod = (JSPBeanMethod)iterator.next();
			jspWriter.println(jspBeanMethod.toJSPCode());
		}
		jspWriter.println();
		
	}
	
	public void writeDynamicFormCode(PrintWriter jspWriter){	
		jspWriter.println();
		
		String submitLink = this.getSubmitURL();
		String submitValue = this.getSubmitButtonLabel();
		
		String formActionCode = " ";
		if(this.isSubmitAllowed()) {
			formActionCode = " method=post action=\""+submitLink+"\" ";
		}
		String formBeginCode = "<form name=\""+this.getJSPName()+"Form\""+	// NOI18N
							   formActionCode+" >"; // NOI18N
		
		jspWriter.println(formBeginCode);
		

		////todo: PLEASE implement this layout junk in HTMLForm 
		// Debug.println(" Form Layout : "+this.pageFormStyle);
		if(this.pageFormStyle == HTMLForm.FS_COLUMN) {
			jspWriter.println("<table border=0 cols=2 width=\"100%\" >");				 // NOI18N
		}else if(this.pageFormStyle == HTMLForm.FS_GRID_2) {				
			jspWriter.println("<table border=0 cols=4 width=\"100%\" >");				 // NOI18N
		} else {
			// NO LAYOUT
			// jspWriter.println("<TABLE BORDER=0 COLS=2 WIDTH=\"100%\" >");
		}
		
		Iterator iterator = this.dispProperties.iterator();
		
		if(this.pageFormStyle == HTMLForm.FS_COLUMN) {
			for(; iterator.hasNext(); ) {
				jspWriter.println("<tr>");												 // NOI18N
				JSPBeanField jspBeanField = (JSPBeanField)iterator.next();
				jspWriter.println(jspBeanField.toFormJSPCode(true));
				jspWriter.println("</tr>");												 // NOI18N
			}
		} else if(this.pageFormStyle == HTMLForm.FS_GRID_2) {
			int cols = 0;
			for(; iterator.hasNext(); ) {
				if(cols == 0) { 
					jspWriter.println("<tr>");											 // NOI18N
				}
				JSPBeanField jspBeanField = (JSPBeanField)iterator.next();
				jspWriter.println(jspBeanField.toFormJSPCode(true));
				++cols;
				if(cols == 2) {
					jspWriter.println("</tr>");											 // NOI18N
					cols = 0;
				}
			}
		} else {
			for(; iterator.hasNext(); ) {
				JSPBeanField jspBeanField = (JSPBeanField)iterator.next();
				jspWriter.println(jspBeanField.toFormJSPCode(false));
			}
		}
		
		
		if(this.isSubmitAllowed()) {
			String submitCode = " <input type=submit value=\""+submitValue+"\" > ";
			
			if(this.pageFormStyle == HTMLForm.FS_COLUMN ) {
				jspWriter.println("<tr><td>&nbsp;</td><td>"+submitCode+"</td></tr>");			 // NOI18N
			} else if(this.pageFormStyle == HTMLForm.FS_GRID_2) {
				jspWriter.println("<tr><td>&nbsp;</td><td>"+submitCode+"</td></tr>");			 // NOI18N
			}else {
				// NO Layout
				jspWriter.println("&nbsp;"+submitCode);									 // NOI18N
			}
		}
		
		if(this.pageFormStyle != HTMLForm.FS_NOLAYOUT) {
			jspWriter.println("</table>");												 // NOI18N
		}
		
		jspWriter.println("</form>");													 // NOI18N
		jspWriter.println();
	}
	
	public StringBuffer writeDynamicContent() {
		StringBuffer jspPageContents = new StringBuffer();
		try {
			
			if(jspTemplate == null) {				try {
					this.setJSPTemplate( this.isErrorPage() ? 
										 DWBeanTemplate.getErrorPageTemplate() :
									     DWBeanTemplate.getDefaultTemplate() );
				}catch(Exception ex) {
				
					  // you should not use UI in the model. Instead, through exception that will					  // be caught in ui to display error.					  // Debug.print(new Exception("Default Template is not Valid"));
					  IDEHelper.showErrorMessageI18N("JBW_DWTemplateNotValidMsg",new Object[] {"Default Template"});		 // NOI18N
					return jspPageContents;
				}			}			
			writeJSPPageDirectiveCode(jspTemplate.getPageDirectiveWriter());
			writeJSPBeanUseBeanCode(jspTemplate.getUseBeanWriter());
			writeJSPBeanPropertyInitializationCode(jspTemplate.getPropertyInitWriter());
			writeJSPBeanMethodCallCode(jspTemplate.getMethodCallWriter());
			writeDynamicFormCode(jspTemplate.getDynamicFormWriter());
		
			jspPageContents = jspTemplate.getJSPPageContents();
		
		}catch(Exception ex) { Debug.print(ex);} // ignored safely or throw code gen exception??
		
		return jspPageContents;
	}
	
	//// after writing the ( JSPPageTemplate + dynamic content) 
	
	public StringBuffer getJSPPageContent() {
		return writeDynamicContent();
	}
	
	public static void main(String[] args) {

		if(Debug.TEST) {
		try {
		
		String beanClassName = "jspbeans.SampleBean";		 // NOI18N
		if(args.length > 0 )
			beanClassName = args[0];
		
		
		JSPBean jspBean = (JSPBean)JSPPage.beanManager.createJSBBean(Class.forName(beanClassName));
		
		JSPVector jspMethods = (JSPVector)JSPPage.beanManager.getValidJSPBeanMethods(jspBean);
		
		JSPVector jspDisplayFields = (JSPVector)JSPPage.beanManager.getValidJSPBeanGetterFields(jspBean);
		
		JSPVector jspInpitFields = (JSPVector)JSPPage.beanManager.getValidJSPBeanSetterFields(jspBean);
		
		
		JSPPage errorPage = new JSPPage();
		errorPage.setErrorPage(true);
		
		JSPPage ioPage = new JSPPage();

		/* ***

		ioPage.addJSPBean(jspBean);
		
		for(int i=0; i < jspMethods.size(); ++i) {
			ioPage.addJSPBeanMethod((JSPBeanMethod)jspMethods.elementAt(i));
		}
		
		for(int i=0; i < jspDisplayFields.size(); ++i) {
			ioPage.addResultField((JSPBeanField)jspDisplayFields.elementAt(i));
		}
		
		for(int i=0; i < jspInpitFields.size(); ++i) {
			ioPage.addInputField((JSPBeanField)jspInpitFields.elementAt(i));
		}
		
		 *** */		
		ioPage.setSubmitAllowed(true);
		
	
		ioPage.setErrorPage(false);
		
		
		ioPage.setJSPName("TestJSPPage");							 // NOI18N
		
		ioPage.setSubmitURL(ioPage.getJSPName()+".jsp");			 // NOI18N
		
		ioPage.setErrorURL(errorPage.getJSPName()+".jsp");			 // NOI18N
		
		ioPage.setSubmitButtonLabel("Test Submit");					 // NOI18N
		
		
		
		// ioPage.setOverwrite(true);
		// ioPage.saveToFile();
		
		}catch(Exception ex) { Debug.print(ex);}
		
		}
		
	}

}


