/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package  org.netbeans.modules.web.wizards.beanjsp.model;


import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import javax.swing.event.*;
import javax.swing.*;
import javax.swing.table.*;
import java.util.*;

import java.text.MessageFormat;


import org.netbeans.modules.web.wizards.util.*;
import org.netbeans.modules.web.wizards.beanjsp.util.*;
import org.netbeans.modules.web.wizards.beanjsp.ide.*;
import org.netbeans.modules.web.templates.model.DWBeanTemplate;

import org.openide.util.*;
import org.openide.*;

import org.openide.loaders.DataFolder;
import org.openide.filesystems.FileObject;


public class JSPPageWizardModel {
	
    public JSPPage simpleJSPPage;
    public FileObject jspFolder;   
    public FileObject jspTemplateFileObject;
    public boolean overwriteJSPFile = false;	
    private JSPItemListModel beanPackagesModel;
    private JSPItemListModel beansModel;
    private JSPItemListModel methodsModel;
    private JSPItemListModel setterFieldsModel;
    private JSPItemListModel getterFieldsModel;
    
    public JSPPageWizardModel() {	  
	initWizardModel();
    }    
  
    public void initWizardModel() {
	simpleJSPPage = new JSPPage();
	simpleJSPPage.beanManager = new BeanManager();
	jspFolder = null;	jspTemplateFileObject = null;
	
	overwriteJSPFile = false;
	  
	beanPackagesModel = new JSPItemListModel();		
	beansModel = new JSPItemListModel();
	methodsModel = new JSPItemListModel();
	setterFieldsModel = new JSPItemListModel();
	getterFieldsModel = new JSPItemListModel();
	
	JSPBeanTableModel useBeansModel = simpleJSPPage.getJSPBeansModel();
	
	useBeansModel.addTableModelListener(new TableModelListener() {
		public void tableChanged(TableModelEvent evt) {
		    if(evt instanceof JSPBeanTableModel.JSPBeanTableModelEvent) {
			if(evt.getType() == TableModelEvent.DELETE) {
			    JSPBean jspBean = ((JSPBeanTableModel.JSPBeanTableModelEvent)evt).getJSPBean();
			    removeAvailableFieldsAndMethods(jspBean);
			}else if(evt.getType() == TableModelEvent.INSERT) {
			    JSPBean jspBean = ((JSPBeanTableModel.JSPBeanTableModelEvent)evt).getJSPBean();
			    addAvailableFieldsAndMethods(jspBean);
			}
		    }
		}
	    });
	
	  
    }    	public boolean isOverwriteJSPFile() { return this.overwriteJSPFile;}
    public void setOverwriteJSPFile(boolean overwrite) { this.overwriteJSPFile = overwrite;}

  
    public JSPItemListModel getBeanPackagesModel() {
	return this.beanPackagesModel;
    }
		
    public JSPItemListModel getAvailableBeansModel() {
	return this.beansModel;
    }
		
    public JSPItemListModel getAvailableBeanMethodsModel() {
	return this.methodsModel;
    }
		
    public JSPItemListModel getAvailableSetterFieldsModel() {
	return this.setterFieldsModel;
    }
		
    public JSPItemListModel getAvailableGetterFieldsModel() {
	return this.getterFieldsModel;
    }

    public boolean addToAvailableBeansFromBeanPackage(DataFolder beanPakFolder) {
	return addToAvailableBeansFromBeanPackage(beanPakFolder,false);
    }
	
    public boolean addToAvailableBeansFromBeanPackage(DataFolder
						      beanPakFolder,
						      boolean reload)
    {  
	    
	if(beanPakFolder != null) {
	    if(beanPackagesModel.getItems().contains(beanPakFolder))
		return true;  // already loaded 

	    Vector javaBeanInfos = IDEHelper.findBeansInFolder(beanPakFolder);
	    JSPVector jspBeans = (JSPVector)
		JSPPage.beanManager.createJSPBeansFromBeanInfos(javaBeanInfos);
	    
	    if(jspBeans.size() <= 0 )  {
		// no beans are there in the pacakge. So don't include
		// it in the list.			 
		return false;  
	    }
	    addToAvailableBeans(jspBeans);
	    beanPackagesModel.add(beanPakFolder);
	}	    
	return true;
    }
	
    public void addToAvailableBeans(JSPVector availableBeans) {
	Iterator beansIterator = availableBeans.iterator();
	for(;beansIterator.hasNext();) {
	    JSPBean jspBean = (JSPBean)((JSPBean)beansIterator.next()).clone();
	    beansModel.add(jspBean);
	}
    }
	
    private void addAvailableFieldsAndMethods(JSPBean jspBean) { 
	if(jspBean != null) {
	    JSPVector setterFields = (JSPVector)JSPPage.beanManager.getValidJSPBeanSetterFields(jspBean);
	    JSPVector getterFields = (JSPVector)JSPPage.beanManager.getValidJSPBeanGetterFields(jspBean);
	    JSPVector beanMethods = (JSPVector)JSPPage.beanManager.getValidJSPBeanMethods(jspBean);
	    
	    methodsModel.addItems(beanMethods);
	    setterFieldsModel.addItems(setterFields);
	    getterFieldsModel.addItems(getterFields);
	}
    }
	
    private void removeAvailableFieldsAndMethods(JSPBean jspBean) {
	if(jspBean == null) {
	    methodsModel.removeAll();
	    setterFieldsModel.removeAll();
	    getterFieldsModel.removeAll();
	} else {
	    methodsModel.removeAll(jspBean);
	    setterFieldsModel.removeAll(jspBean);
	    getterFieldsModel.removeAll(jspBean);
	}
    }
	
    public boolean areThereAnyNonVoidMethodCalls() {
	Collection methodCalls = simpleJSPPage.getMethodCallsModel().getItems();
	Iterator methodCallItr = methodCalls.iterator();		
	for(;methodCallItr.hasNext();) {			
	    JSPBeanMethod jspBeanMethod =
		(JSPBeanMethod)methodCallItr.next();
	    if(jspBeanMethod.isNonVoidMethod()) return true;
	}
	return false;	}	
    public static void main(String[]args) {

    }
}

