/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package  org.netbeans.modules.web.wizards.beanjsp.ui;

import org.netbeans.modules.web.wizards.beanjsp.model.*;
import org.netbeans.modules.web.wizards.wizardfw.*;
import org.netbeans.modules.web.wizards.beanjsp.ide.*;

import org.netbeans.modules.web.wizards.util.*;
import org.netbeans.modules.web.wizards.beanjsp.util.*;

import org.openide.util.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;

  
public class ButtonBarPanel extends JPanel {
  
    public static final int ADD = 0;  
    public static final int REMOVE = 1;
    public static final int ADD_ALL = 2;  
    public static final int REMOVE_ALL = 3;
    public static final int MOVE_UP = 4;  
    public static final int MOVE_DOWN = 5;    
    static final int BUTTONS_COUNT = 6;
    
    JButton[] buttons;  public ButtonBarPanel() {
	this(new int[] {ADD, REMOVE, ADD_ALL, REMOVE_ALL, MOVE_UP,
			MOVE_DOWN });  } 

    public ButtonBarPanel(int[] buttonIDs) {
	super();
	initComponents (buttonIDs);
    }  
    
    private void createButton(int buttonID, String tip, Icon icon) { 
	createButton(buttonID,tip,null,icon);
    }
  
    private void createButton(int buttonID, String tip, String text) {
	createButton(buttonID,tip,text,null);
    }

    private void createButton(int buttonID, String tip, String text,
			      Icon icon) {	  
	buttons[buttonID] = new javax.swing.JButton(text,icon);	  
	if(tip != null && tip.trim().length() > 0 )  { 
	    buttons[buttonID].setToolTipText(tip);	  
	}      
	buttons[buttonID].setMinimumSize (new java.awt.Dimension(73, 15));
	buttons[buttonID].setMaximumSize (new java.awt.Dimension(100, 27));
	  
    }


    private void createButton(int buttonID,
			      java.util.ResourceBundle resBundle, 
			      String lnfStyle) {
	String buttonText;	  
	ImageIcon buttonIcon;	  
	String buttonTip; 
	String imagePath = "/org/netbeans/modules/web/wizards/beanjsp/resources/images/"; // NOI18N	  
	switch(buttonID) {		
	case ADD:		  
	    buttonText = resBundle.getString("JBW_AddButtonText");
	    // NOI18N		  
	    buttonTip = resBundle.getString("JBW_AddButtonTip");							 // NOI18N
	    buttonIcon = new
		ImageIcon(this.getClass().getResource(imagePath+"add.gif")); //NOI18N		  
	    break;		
	case REMOVE:		  
	    buttonText = resBundle.getString("JBW_RemoveButtonText");
	    // NOI18N		  
	    buttonTip = resBundle.getString("JBW_RemoveButtonTip");							 // NOI18N
	    buttonIcon = new
		ImageIcon(this.getClass().getResource(imagePath+"remove.gif")); //NOI18N		  
		  
	    break;
	case ADD_ALL:		  
	    buttonText = resBundle.getString("JBW_AddAllButtonText");
	    // NOI18N		  
	    buttonTip = resBundle.getString("JBW_AddAllButtonTip");
	    // NOI18N	    
	    buttonIcon = new
		ImageIcon(this.getClass().getResource(imagePath+"addall.gif")); //NOI18N		  
	    break;
	case REMOVE_ALL:		 
	    buttonText =
		resBundle.getString("JBW_RemoveAllButtonText");
	    // NOI18N		  
	    buttonTip =resBundle.getString("JBW_RemoveAllButtonTip"); // NOI18N 
	    buttonIcon = new
		ImageIcon(this.getClass().getResource(imagePath+"removeall.gif")); //NOI18N		  
	    break;
	case MOVE_UP:		  
	    buttonText = resBundle.getString("JBW_MoveUpButtonText");
	    // NOI18N		  
	    buttonTip = resBundle.getString("JBW_MoveUpButtonTip");							 // NOI18N
		  
	    buttonIcon = new
		ImageIcon(this.getClass().getResource(imagePath+"up.gif")); //NOI18N		  
	    break;
	case MOVE_DOWN:		  
	    buttonText =
		resBundle.getString("JBW_MoveDownButtonText");
	    // NOI18N		  
	    buttonTip = resBundle.getString("JBW_MoveDownButtonTip");							 // NOI18N
	    buttonIcon = new
		ImageIcon(this.getClass().getResource(imagePath+"down.gif")); //NOI18N		  
	    break;
	default:
	    return;	  }	  
	if(lnfStyle.equalsIgnoreCase("ICON")) {		
	    createButton(buttonID,buttonTip,buttonIcon); 
	} 
	else if(lnfStyle.equalsIgnoreCase("ICON_TEXT")) {
	    createButton(buttonID,buttonTip,buttonText,buttonIcon); 
	    
	} else {  // TEXT
	    createButton(buttonID,buttonTip,buttonText); 	
	}
    }
  
    private void initComponents (int[] buttonIDs) {

	java.util.ResourceBundle resBundle =
	    NbBundle.getBundle(JSPPageWizardGlobals.jspPageWizardI18NBundle); 
	
	////  TEXT , ICON , ICON_TEXT
	String lnfProp = System.getProperty("JSPWizards.buttons.lnf","ICON");
	//// create components
	
	buttons = new JButton[BUTTONS_COUNT];
	for(int i=0; i < buttons.length; ++i) {		  
	    buttons[i] = null;
	}	  
	for(int i=0; i < buttonIDs.length; ++i) {
	    createButton(buttonIDs[i],resBundle,lnfProp);	  }	  
	//// layout and add components
	arrangeComponents();
    }
	
    //// layout components in 
    private void arrangeComponents() {
	arrangeCompsWithGridBag();
    }
	
    private void addGridBagComponent(Container parent, Component comp, 
				     int gridx, int gridy, 
				     int gridwidth, int gridheight, 
				     double weightx, double weighty,
				     int anchor, int fill,
				     Insets insets, int ipadx, int ipady) {
	GridBagConstraints cons = new GridBagConstraints();
	cons.gridx = gridx;
	cons.gridy = gridy;
	cons.gridwidth = gridwidth;
	cons.gridheight = gridheight;
	cons.weightx = weightx;
	cons.weighty = weighty;
	cons.anchor = anchor;
	cons.fill = fill;
	cons.insets = insets;
	cons.ipadx = ipadx;
	cons.ipady = ipady;
	parent.add(comp,cons);
    }
	
    private void arrangeCompsWithGridBag() {
	
	this.setLayout(new GridBagLayout());
	
	Component allGap = Box.createVerticalStrut(2);
	Component moveGap = Box.createVerticalStrut(6);
	
	Component remainderGlue = Box.createGlue();
	int gridx = 0;		
	int gridy = 0; 
	if(buttons[ADD] != null || buttons[REMOVE] != null) {
	    if(buttons[ADD] != null) {
		addGridBagComponent(this,buttons[ADD],
				    0,gridy,1,1,
				    100,0,
				    GridBagConstraints.CENTER, 
				    GridBagConstraints.HORIZONTAL,
				    new Insets(2,2,2,2),5,5	);
		++gridy;
		
	    }					
	    if(buttons[REMOVE] != null ) {
		addGridBagComponent(this,buttons[REMOVE],
				    0,gridy,1,1,
				    100,0,
				    GridBagConstraints.CENTER,
				    GridBagConstraints.HORIZONTAL, 
				    new Insets(2,2,2,2),5,5	);
		++gridy;			
	    }
		
	    addGridBagComponent(this,allGap,
				0,gridy,1,1,
				100,0,
				GridBagConstraints.CENTER,
				GridBagConstraints.HORIZONTAL, 
				new Insets(2,2,2,2),5,5	);
	    ++gridy;		
	}				
	
	if(buttons[ADD_ALL] != null || buttons[REMOVE_ALL] != null) {		
	    if(buttons[ADD_ALL] != null) {
		addGridBagComponent(this,buttons[ADD_ALL],
				    0,gridy,1,1,
				    100,0,
				    GridBagConstraints.CENTER,
				    GridBagConstraints.HORIZONTAL, 
				    new Insets(2,2,2,2),5,5	);
		++gridy;
	    }					
	    if(buttons[REMOVE_ALL] != null) {
		addGridBagComponent(this,buttons[REMOVE_ALL],
				    0,gridy,1,1,
				    100,0,
				    GridBagConstraints.CENTER,
				    GridBagConstraints.HORIZONTAL, 
				    new Insets(2,2,2,2),5,5	);
		++gridy;			
	    }		
	    addGridBagComponent(this,moveGap,
				0,gridy,1,1,
				100,0,
				GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL,
				new Insets(2,2,2,2),5,5	);
	    ++gridy;		
	}
	if(buttons[MOVE_UP] != null ) {
	    addGridBagComponent(this, buttons[MOVE_UP],
				0,gridy,1,1,
				100,0,
				GridBagConstraints.CENTER,
				GridBagConstraints.HORIZONTAL, 
				new Insets(2,2,2,2),5,5	);
	    ++gridy;		
	}		
	if( buttons[MOVE_DOWN] != null) {		
	    addGridBagComponent(this,buttons[MOVE_DOWN],
				0,gridy,1,1,
				100,0,
				GridBagConstraints.CENTER,
				GridBagConstraints.HORIZONTAL, 
				new Insets(2,2,2,2),5,5	);
	    ++gridy;		
	}
	
	addGridBagComponent(this,remainderGlue,
			    0,gridy,1,1,
			    100,100,
			    GridBagConstraints.CENTER,
			    GridBagConstraints.BOTH, 
			    new Insets(2,2,2,2),5,5	);
	
    }
	
        
    // Helper methods			
    public void addActionListener(int buttonID, ActionListener listener) {
	if(buttons[buttonID] != null) {			
	    buttons[buttonID].addActionListener(listener);
	}	
    }		

    public void removeActionListener(int buttonID, ActionListener listener) {
	if(buttons[buttonID] != null) {			
	    buttons[buttonID].removeActionListener(listener);
	}	
    }		

    public void addActionListener(ActionListener allListener) {
	for(int i=0; i < buttons.length; ++i) {
	    if(buttons[i] != null) 				
		buttons[i].addActionListener(allListener);
	}	
    }
    
    public void removeActionListener(ActionListener allListener) {
	for(int i=0; i < buttons.length; ++i) {
	    if(buttons[i] != null) 				
		buttons[i].removeActionListener(allListener);
	}	
    }		

    public void setEnabled(int buttonID, boolean enabled) { 
	
	if(buttons[buttonID] != null) 			
	    buttons[buttonID].setEnabled(enabled);
    }		

    public boolean isEnabled(int buttonID) { 
	if(buttons[buttonID] != null) 			
	    return buttons[buttonID].isEnabled();		
	else			
	    return false;   // this is not right. you should throw exp.
    }	

    public void enableAll() {
	for(int i=0; i < buttons.length; ++i) {
	    if(buttons[i] != null) 				
		buttons[i].setEnabled(true);
	}	
    }
	
    public void disableAll() {
	for(int i=0; i < buttons.length; ++i) {
	    if(buttons[i] != null) 				
		buttons[i].setEnabled(false);
	}	
    }	
	

    public void setButtonsState(int leftSize, int[]
				leftSelectionIndices,
				int rightSize, int[]
				rightSelectionIndices ) {
	
	setEnabled(ADD,(leftSize > 0 && leftSelectionIndices.length > 0 ));

	setEnabled(REMOVE,(rightSize > 0 &&
			   rightSelectionIndices.length > 0 ));		 
	setEnabled(ADD_ALL,(leftSize > 0));		
	setEnabled(REMOVE_ALL,(rightSize > 0));
	setEnabled(MOVE_UP, ((rightSize > 1) && 
			     (rightSelectionIndices.length > 0) && 
			     (rightSelectionIndices[0] > 0))
		   );
		
	setEnabled(MOVE_DOWN,
		   ((rightSize > 1) && 
		    (rightSelectionIndices.length > 0) && 
		    (rightSelectionIndices[rightSelectionIndices.length-1] < rightSize-1))
		   );		
    }	
		
    public static void main(String[] args) {	  

	if(Debug.TEST) {
	    JFrame testFrame = new JFrame("This is Test Frame"); // NOI18N
	    
	    testFrame.getContentPane().add(new ButtonBarPanel(),SwingConstants.CENTER);
	    testFrame.setSize(500,300);
	    testFrame.pack();
	    testFrame.show();	  }
    }
}
