/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package  org.netbeans.modules.web.wizards.beanjsp.ui;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import javax.swing.border.*;

import org.netbeans.modules.web.wizards.beanjsp.model.*;
import org.netbeans.modules.web.wizards.wizardfw.*;
import org.netbeans.modules.web.wizards.beanjsp.ide.*;

import org.netbeans.modules.web.wizards.util.*;
import org.netbeans.modules.web.wizards.beanjsp.util.*;

import org.openide.util.*;

  
public class JSPBeanMethodTable extends JScrollPane {
  
    public JSPBeanMethodTable() {
	super();
	initComponents ();
    }

    private void initComponents () {

	methodTable = new JTable();
	this.setViewportView(methodTable);

	setTableModel(new JSPBeanMethodTableModel(new JSPVector()));
	  
    }
        
    private javax.swing.JTable methodTable;
    private AbstractTableModel methodTableModel; 
  
    public void setTableModel(JSPBeanMethodTableModel tableModel) {
	methodTableModel = tableModel;
	methodTable.setModel(methodTableModel);
	methodTable.setPreferredScrollableViewportSize(new Dimension(200,100));
	
	//// make it look like list
	methodTable.setShowHorizontalLines(false);
	methodTable.setTableHeader(null);
	
	installMethodEditor();
	
    }
    //// delegation to JTable

    public JTable getTable() { return methodTable; }
  
    public void installMethodEditor() {
	TableColumn methodEditColumn = 
	    methodTable.getColumnModel().getColumn(JSPBeanMethodTableModel.EDIT_COL);
		
	methodEditColumn.setCellRenderer(new MethodEditorCellRenderer());
		
	// editor installation
		
	final JButton editorComponent = new JButton("...");   // NOI18N
	final MethodEditorCellEditor  cellEditor = new MethodEditorCellEditor(editorComponent);
	methodEditColumn.setCellEditor(cellEditor);
	editorComponent.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    JSPBeanMethodEditor.editJSPBeanMethod(cellEditor.jspBeanMethod);
		}
	    });
		
	editorComponent.setPreferredSize(new Dimension(30,27));
	int width = editorComponent.getPreferredSize().width;
	methodEditColumn.setPreferredWidth(width);
	methodEditColumn.setMinWidth(width);
	methodEditColumn.setMaxWidth(width);
	// methodEditColumn.setResizable(false);

    }
  
    class MethodEditorCellRenderer extends JButton
	implements TableCellRenderer {

        public MethodEditorCellRenderer() {
            super();			setText("..."); // NOI18N
            this.setOpaque(true); //MUST do this for background to show up.
        }

        public Component 
	    getTableCellRendererComponent(JTable table, Object color, 
					  boolean isSelected, 
					  boolean hasFocus, 
					  int row, int column) {
	    return this;		
	}
    }
  
    class MethodEditorCellEditor extends DefaultCellEditor { 
	
	JSPBeanMethod jspBeanMethod;
	public MethodEditorCellEditor(JButton editor) {
	    super(new JCheckBox()); 
	    //Unfortunately, the constructor
	    //expects a check box, combo box,
	    //or text field.
            editorComponent = editor;
            setClickCountToStart(1); //This is usually 1 or 2.

            //Must do this so that editing stops when appropriate.
	    editor.addActionListener(new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
			fireEditingStopped();
		    }
		});						
	    editor.addFocusListener(new FocusAdapter() {
		    public void focusGained(FocusEvent e) {						fireEditingStopped();
		    }});
        }

        protected void fireEditingStopped() {
            super.fireEditingStopped();
        }

        public Object getCellEditorValue() {
            return jspBeanMethod;
        }

        public Component getTableCellEditorComponent(JTable table, 
                                                     Object value,
                                                     boolean isSelected,
                                                     int row,
                                                     int column) {
	    jspBeanMethod = (JSPBeanMethod)value;
	    return editorComponent;
        }
    }
  
  
    public static JSPBeanMethodTableModel createModel(JSPVector items) {
	return new JSPBeanMethodTableModel(items);
    }
  
    public static class JSPBeanMethodTableModel extends
	javax.swing.table.AbstractTableModel {	

	public static final int METHOD_COL = 0;	
	public static final int EDIT_COL = 1;
	public static final int NUM_COLUMNS = 2;	
	JSPVector jspBeanMethods;	
	public JSPBeanMethodTableModel(JSPVector jspBeanMethods) {
	    this.jspBeanMethods = jspBeanMethods;	
	}		//// table model handling
	
	public int getColumnCount() { return NUM_COLUMNS; }
	
	public int getRowCount() { return jspBeanMethods.size(); }
	
	public String getColumnName(int column) {
	    switch (column) {
	    case METHOD_COL: 
		return "";			 // NOI18N        
	    case EDIT_COL: 
		return "";			 // NOI18N
	    default: 
		return "";			 // NOI18N
	    }
	}
	
	public Class getColumnClass(int column) {
	    switch (column) {
	    case METHOD_COL: 
		return String.class;        
	    case EDIT_COL: 
		return Object.class;
	    default: 
		return Object.class;
	    }
	}
	
	
	public boolean isCellEditable(int rowIndex, int columnIdx) {
	    switch (columnIdx) {
	    case METHOD_COL: 
		return false;        
	    case EDIT_COL: 
		return true;
	    default: 
		return false;
	    }

	}
	
	public Object getValueAt(int row, int column) {	  
	    JSPBeanMethod jspBeanMethod = (JSPBeanMethod)
		jspBeanMethods.get(row); 
	    switch (column) {
	    case METHOD_COL: 
		return jspBeanMethod.getDisplayName();
	    case EDIT_COL: 		  		  
		return jspBeanMethod;
	    default: 
		return "";			 // NOI18N
	    }
	}
	
	public void setValueAt(Object aValue, int row, int column) {
	    // JSPBeanMethod jspBeanMethod = (JSPBeanMethod)
	    // jspBeanMethods.get(row);	 
	    // do nothing 
	}	
	//// helper methods
	//// USE BEAN Method MODEL methos
	
	public void add(JSPBeanMethod jspBeanMethod) {
	    jspBeanMethods.add(jspBeanMethod);
	    int idx = jspBeanMethods.size()-1;
	    this.fireTableRowsInserted(idx,idx);
	    // this.fireTableDataChanged();
	}
	
	public JSPBeanMethod remove(int idx) {
	    JSPBeanMethod jspBeanMethod = (JSPBeanMethod)
		jspBeanMethods.remove(idx); 
	    this.fireTableRowsDeleted(idx,idx);
	    return jspBeanMethod;
	}
	
	public int moveUp(int idx) {
	    int newIdx = jspBeanMethods.moveUp(idx);
	    this.fireTableDataChanged();
	    return newIdx;
	}
	
	public int moveDown(int idx) {
	    int newIdx = jspBeanMethods.moveDown(idx);
	    this.fireTableDataChanged();
	    return newIdx;
	}
	
	public void removeAll() {
	    int idx = jspBeanMethods.size()-1;
	    jspBeanMethods.removeAllElements();
	    if(idx >= 0 )
		this.fireTableRowsDeleted(0,idx);
	}
	
	public boolean removeAll(Object key) {
	    boolean itemsRemoved = jspBeanMethods.removeAllByKey(key);
		
	    if(itemsRemoved) {
		this.fireTableDataChanged();
	    }
		
	    return itemsRemoved;
	}
	
	public void addItems(JSPVector newItems) {
	    if(newItems.size() <= 0)
		return;
	    int idx = jspBeanMethods.size()-1;
	    jspBeanMethods.addAll(newItems);
	    if(idx < 0 )
		idx = 0;
	    this.fireTableRowsInserted(idx, idx+newItems.size());		
	}
	
	// public int getSize() { return jspBeanMethods.size();}
	
	public JSPVector getJSPBeanMethods() { return jspBeanMethods; }
	
    }  
  
    public static void main(String[] args) {
    }
}
