/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package  org.netbeans.modules.web.wizards.beanjsp.ui;

import org.netbeans.modules.web.wizards.beanjsp.model.*;
import org.netbeans.modules.web.wizards.wizardfw.*;
import org.netbeans.modules.web.wizards.beanjsp.ide.*;

import org.netbeans.modules.web.wizards.util.*;
import org.netbeans.modules.web.wizards.beanjsp.util.*;

import org.openide.util.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;


  
public class JSPDisplayFieldsPanel extends StandardWizardPanel {
  

    // ---------------------------------------------------------------------------------------
    // WizardPanel initialization

    /** Creates new JspColumnsPanel */

    public JSPDisplayFieldsPanel() {
	super();
	initComponents ();
    }

    public HelpCtx getHelp () {
	return new HelpCtx (JSPDisplayFieldsPanel.class);
    }
  
    private void initComponents () {

	java.util.ResourceBundle resBundle =
	    NbBundle.getBundle(JSPPageWizardGlobals.jspPageWizardI18NBundle);		 
	//// create components 
		
	this.setTopMessage(resBundle.getString("JBW_DisplayFieldsMsg"));								 // NOI18N

	beanPropsLabel = new JLabel(resBundle.getString("JBW_AvailableDisplayFieldsListTitle"));		 // NOI18N	
	beanFieldsList = new JList();
	beanPropsSPane = new JScrollPane(beanFieldsList);
	  
	beanPropsSPane.setPreferredSize (new java.awt.Dimension(150, 150));
	beanPropsSPane.setMinimumSize (new java.awt.Dimension(150, 150));
	beanPropsSPane.setMaximumSize (new java.awt.Dimension(150, 150));
	  

	useBeanPropsLabel = new JLabel(resBundle.getString("JBW_DisplayFieldsTableTitle"));			 // NOI18N	
	jspDisplayFieldsTable = new JSPDisplayFieldTable();
	  

	buttonsPanel = new ButtonBarPanel();	  //// layout and add components
	  
	arrangeComponents();
	 
	//// create and add models
	createModels();	  
	//// configure listeners 
	addButtonBarActionListeners();
	addSelectionListeners();
	addDoubleClickListeners();	  	  setButtonPanelButtonStates();
    }
	
    //// layout components in 
    private void arrangeComponents() {
	arrangeCompsWithGridBag();
    }
	
    private void arrangeCompsWithGridBag() {
	
	// this.contentPane.setLayout(new GridBagLayout());
		
	
	addGridBagComponent(this.contentPane,beanPropsLabel,
			    0,0,1,1,
			    0,0,
			    GridBagConstraints.CENTER, GridBagConstraints.BOTH,
			    new Insets(2,2,2,2),5,5	);

	addGridBagComponent(this.contentPane,useBeanPropsLabel,
			    2,0,1,1,
			    100,0,
			    GridBagConstraints.CENTER, GridBagConstraints.BOTH,
			    new Insets(2,2,2,2),5,5	);
		
	addGridBagComponent(this.contentPane,beanPropsSPane,
			    0,1,1,1,
			    0,100,
			    GridBagConstraints.CENTER, GridBagConstraints.BOTH,
			    new Insets(2,2,2,2),5,5	);
		
	addGridBagComponent(this.contentPane,buttonsPanel,
			    1,1,1,1,
			    0,100,
			    GridBagConstraints.CENTER, GridBagConstraints.BOTH,
			    new Insets(2,2,2,2),5,5	);
		
	addGridBagComponent(this.contentPane,jspDisplayFieldsTable,
			    2,1,1,1,
			    100,100,
			    GridBagConstraints.CENTER, GridBagConstraints.BOTH,
			    new Insets(2,2,2,2),5,5	);
    }
	
    private void createModels() {	  

	JSPPageWizardModel jspPageWizardModel = JSPPageWizard.jspPageWizardModel;	
	JSPPage simpleJSPPage = jspPageWizardModel.simpleJSPPage;
	beanFieldsModel = jspPageWizardModel.getAvailableGetterFieldsModel();
	jspDisplayFieldsTableModel = simpleJSPPage.getDispPropertiesModel();
	
	this.beanFieldsList.setModel(beanFieldsModel);	  
	this.jspDisplayFieldsTable.setTableModel(jspDisplayFieldsTableModel);
    }	 


    private void addButtonBarActionListeners() {
	buttonsPanel.addActionListener(ButtonBarPanel.ADD, new
	    ActionListener() { 
		public void actionPerformed(ActionEvent evt) {
		    addToUseBeanFieldsTable();
		}
	    });

	buttonsPanel.addActionListener(ButtonBarPanel.REMOVE, new
	    ActionListener() { 
		public void actionPerformed(ActionEvent evt) {
		    removeFromUseBeanFieldsTable();
		}
	    });	  	  

	buttonsPanel.addActionListener(ButtonBarPanel.ADD_ALL, new
	    ActionListener() { 
		public void actionPerformed(ActionEvent evt) {
		    addAllToUseBeanFieldsTable();
		}
	    });

	buttonsPanel.addActionListener(ButtonBarPanel.REMOVE_ALL, new
	    ActionListener() { 
		public void actionPerformed(ActionEvent evt) {
		    removeAllFromUseBeanFieldsTable();
		}
	    });
	  	  
	buttonsPanel.addActionListener(ButtonBarPanel.MOVE_UP, new
	    ActionListener() { 
		public void actionPerformed(ActionEvent evt) {
		    moveUpUseBeanField();
		}
	    });

	buttonsPanel.addActionListener(ButtonBarPanel.MOVE_DOWN, new
	    ActionListener() { 
		public void actionPerformed(ActionEvent evt) {
		    moveDownUseBeanField();
		}
	    });
    }
  
    private void addSelectionListeners(){
	beanFieldsList.addListSelectionListener(new ListSelectionListener(){
		public void valueChanged(ListSelectionEvent e){
		    setButtonPanelButtonStates();
		}
	    });
	jspDisplayFieldsTable.getTable().getSelectionModel().addListSelectionListener(
										      new ListSelectionListener(){
											      public void valueChanged(ListSelectionEvent e){
												  setButtonPanelButtonStates();
											      }
											  });
    }

    private void addDoubleClickListeners(){
	beanFieldsList.addMouseListener (new java.awt.event.MouseAdapter () {
		public void mouseClicked (java.awt.event.MouseEvent evt) {
		    beanFieldsListMouseClicked(evt);
		}
	    });
	jspDisplayFieldsTable.getTable().addMouseListener (new java.awt.event.MouseAdapter () {
		public void mouseClicked (java.awt.event.MouseEvent evt) {
		    useBeanFieldsTableMouseClicked(evt);
		}
	    });
    }
          
    // Variables declaration - do not modify//GEN-BEGIN:variables
	
    private javax.swing.JLabel titleLabel;
  
    private javax.swing.JScrollPane beanPropsSPane;
    private javax.swing.JList beanFieldsList;
    private javax.swing.JLabel beanPropsLabel;
  
    private ButtonBarPanel buttonsPanel;    
    private JSPDisplayFieldTable jspDisplayFieldsTable;
    private javax.swing.JLabel useBeanPropsLabel;
  
    private JSPItemListModel beanFieldsModel;
    private JSPDisplayFieldTableModel jspDisplayFieldsTableModel;
  
    // methods
  
    public void setButtonPanelButtonStates() {	

	int leftSize = beanFieldsModel.getSize();
	int[] leftSelectionIndices = beanFieldsList.getSelectedIndices();
	  
	int rightSize = jspDisplayFieldsTable.getTable().getRowCount();
	int[] rightSelectionIndices =
	    jspDisplayFieldsTable.getTable().getSelectedRows();
	
	buttonsPanel.setButtonsState(leftSize,leftSelectionIndices,
				     rightSize,rightSelectionIndices);
    }   
    
    public void addAllToUseBeanFieldsTable() {
	JSPVector addFields = beanFieldsModel.getItems();
	jspDisplayFieldsTableModel.addItems(addFields);
	beanFieldsModel.removeAll();
	setButtonPanelButtonStates();
    }
  
    public void removeAllFromUseBeanFieldsTable() {
	JSPVector removeFields = jspDisplayFieldsTableModel.getItems();
	beanFieldsModel.addItems(removeFields);
	jspDisplayFieldsTableModel.removeAll();
	setButtonPanelButtonStates();
    }
  
  
    public void addToUseBeanFieldsTable() {
	int[] idx = beanFieldsList.getSelectedIndices();
	int  count = 0;
	int oldRows = jspDisplayFieldsTable.getTable().getRowCount();
	if( (idx.length > 0) && (idx[0] >= 0)){
	    for (int i=0; i<idx.length; i++) {
		Object obj = beanFieldsModel.remove(idx[i]-count);
		jspDisplayFieldsTableModel.add((JSPBeanField)obj);
		count++;
	    }
	    // set selection display fields
	    int rows = jspDisplayFieldsTable.getTable().getRowCount();
	    jspDisplayFieldsTable.getTable().getSelectionModel().setSelectionInterval(
										      oldRows, rows-1);
	    setButtonPanelButtonStates();
	}
    }
  
    public void removeFromUseBeanFieldsTable() {
	int[] idx = jspDisplayFieldsTable.getTable().getSelectedRows();
	int  count = 0;
	int oldRows = beanFieldsModel.getSize();
	if((idx.length > 0) && (idx[0] >= 0)) {
	    for (int i=0; i<idx.length; i++) {
		Object obj = jspDisplayFieldsTableModel.remove(idx[i]-count);
		beanFieldsModel.add(obj);
		count++;
	    }
	    // set selection display fields
	    int rows = beanFieldsModel.getSize();
	    beanFieldsList.getSelectionModel().setSelectionInterval(
								    oldRows, rows-1);
	    setButtonPanelButtonStates();
	}
    }
  
    public void moveUpUseBeanField() {
	int[] idx = jspDisplayFieldsTable.getTable().getSelectedRows();
	if((idx.length > 0) && (idx[0] > 0 )) {
	    for (int i=0; i<idx.length; i++){
		jspDisplayFieldsTableModel.moveUp(idx[i]);
	    }
	    jspDisplayFieldsTable.getTable().getSelectionModel().setSelectionInterval(
										      idx[0]-1, idx[idx.length-1]-1);	
	    setButtonPanelButtonStates();
	}
    }
  
    public void moveDownUseBeanField() {
	int[] idx = jspDisplayFieldsTable.getTable().getSelectedRows();
	int rowCount = jspDisplayFieldsTable.getTable().getRowCount();
	if((idx.length >0) && (idx[0] >= 0) && (idx[idx.length-1] < rowCount-1)) {
	    for (int i=idx.length-1; i>=0; i--){
		jspDisplayFieldsTableModel.moveDown(idx[i]);
	    }
	    jspDisplayFieldsTable.getTable().getSelectionModel().setSelectionInterval(
										      idx[0]+1,idx[idx.length-1]+1);
	    setButtonPanelButtonStates();
	}
    }

    private void beanFieldsListMouseClicked(java.awt.event.MouseEvent evt) {
	if (evt.getClickCount() == 2) {
	    addToUseBeanFieldsTable();
	}
    }

    private void useBeanFieldsTableMouseClicked(java.awt.event.MouseEvent evt) {
	if (evt.getClickCount() == 2) {
	    removeFromUseBeanFieldsTable();
	}
    }	  /* this method gets called on next && prev */  public void readSettings(Object setting) {
	setButtonPanelButtonStates();  }

    public static void main(String[] args) {	  if(Debug.TEST) {
	JFrame testFrame = new JFrame("This is Test Frame");		 // NOI18N
	testFrame.getContentPane().add(new JSPDisplayFieldsPanel(),SwingConstants.CENTER);
	testFrame.setSize(500,300);
	testFrame.pack();
	testFrame.show();	  }
	  
    }
  
}
