/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package  org.netbeans.modules.web.wizards.beanjsp.ui;


import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import javax.swing.event.*;
import javax.swing.*;
import javax.swing.table.*;
import java.util.*;

import java.text.MessageFormat;

import org.netbeans.modules.web.wizards.beanjsp.model.*;
import org.netbeans.modules.web.wizards.wizardfw.*;
import org.netbeans.modules.web.wizards.beanjsp.ide.*;

import org.netbeans.modules.web.wizards.util.*;
import org.netbeans.modules.web.wizards.beanjsp.util.*;
import org.netbeans.modules.web.templates.model.DWBeanTemplate;
import org.openide.util.*;
import org.openide.*;

import org.openide.loaders.DataFolder;
import org.openide.filesystems.FileObject;

public class JSPPageWizard  extends DefaultWizard {
  public static JSPPageWizardModel jspPageWizardModel;  
  public static final int JSP_PAGETYPE_PANEL  = 0;    public static final int JSP_BEAN_SELECTION_PANEL = 1;  public static final int JSP_INPUT_FILEDS_PANEL = 2;
  public static final int JSP_BIZ_METHODS_PANEL = 3;
  public static final int JSP_DISPLAY_FIELDS_PANEL = 4;  public static final int JSP_SUBMIT_AND_ERROR_URL_INFO_PANEL  = 5;	    
  public static final int JSP_PAGE_NAME_AND_STYLE_INFO_PANEL = 6;
  
  // public static final String i18nBundle = "org.netbeans.modules.web.wizards.beanjsp.resources.Bundle";			 // NOI18N
  

  public JSPPageWizard(DefaultWizardPanel[] panels) {
	  super(panels,new Object());
  }
  public static void initWizardModel() {	  jspPageWizardModel = new JSPPageWizardModel();  }
  
  public static void releaseWizardModel() {
	  jspPageWizardModel = null;  }
  
  //// wizard navigation implemetation
  
    /* Moves to the next panel.
    * @exception NoSuchElementException if the panel does not exist
    */
    public synchronized void nextPanel () {
      if (index + 1 == panels.length) throw new java.util.NoSuchElementException ();
	  if(!panels[index].validateInput())
		  return;
	  
	  index++;
	}
    
    /* Moves to previous panel.
    * @exception NoSuchElementException if the panel does not exist
    */
    public synchronized void previousPanel () {
      if (index == 0) throw new java.util.NoSuchElementException ();
	  index--;
	}
	
    /* Current name of the panel */
    public String name () {
	  java.util.ResourceBundle resBundle = NbBundle.getBundle(JSPPageWizardGlobals.jspPageWizardI18NBundle);
	  
      Object[] panelNumArgs = {
        new Integer (index + 1),
        new Integer (panels.length)
      };
	  
      MessageFormat panelNumMF = new MessageFormat (resBundle.getString ("JBW_WizardPanelNumber"));		 // NOI18N
      String panelNum = panelNumMF.format (panelNumArgs);
	  
	  String panelTitle = "";			 // NOI18N
	  switch(index) {		case JSP_PAGETYPE_PANEL:
		  panelTitle = resBundle.getString("JBW_PageTypeOptionPanelTitle");				 // NOI18N
		  break;
		case JSP_BEAN_SELECTION_PANEL :
		  panelTitle = resBundle.getString("JBW_JSPBeansSelectionPanelTitle");			 // NOI18N
		  break;
		case JSP_INPUT_FILEDS_PANEL:
		  panelTitle = resBundle.getString("JBW_PorpInitPanelTitle");				 // NOI18N
		  break;		case JSP_BIZ_METHODS_PANEL:
		  panelTitle = resBundle.getString("JBW_BizMethodsPanelTitle");					 // NOI18N
		  break;
		case JSP_DISPLAY_FIELDS_PANEL:
		  panelTitle = resBundle.getString("JBW_DisplayFieldsPanelTitle");				 // NOI18N
		  break;
		case JSP_SUBMIT_AND_ERROR_URL_INFO_PANEL:
		  panelTitle = resBundle.getString("JBW_JSPPageSubmitAndErrorURLInfoPanelTitle");	 // NOI18N		  break;
		case JSP_PAGE_NAME_AND_STYLE_INFO_PANEL:
		  panelTitle = resBundle.getString("JBW_JSPPageNameAndStyleInfoPanelTitle");	 // NOI18N	
		  break;
	  }
	  
	  MessageFormat titleMF = new MessageFormat (resBundle.getString("JBW_JSPPageWizardTitle"));	 // NOI18N
	
	  // Debug.println("Title MF "+titleMF.format(""));
	  // Debug.println("Panel Num MF "+panelNumMF.format(""));
		  
      Object[] titleMFArgs = {
        panelNum,
		panelTitle
      };
	  
	  return titleMF.format(titleMFArgs);
	  
    }
	

	//// wizard methods
  
    public boolean onFinish() {

	try {		
	    //  Debug.println(" On Finish Validating the Last Page");
	    //  validate data for NameAndStyle Panel if it is the
	    //  Last Panel ( Name And Style Panel) 		 
	    //  process last panel 
		
	    panels[panels.length-1].storeSettings(new Object());
	    if(!panels[panels.length-1].validateInput())
		return false;
	  
	    StringBuffer jspPageContents =
		jspPageWizardModel.simpleJSPPage.getJSPPageContent(); 
	  

	    if(jspPageWizardModel.areThereAnyNonVoidMethodCalls()) {
		IDEHelper.showWarningMessageI18N("JBW_NonVoidParamsInMethodsWarning");		 // NOI18N
		
		// return false;  // give user chance to correct 
	    }
	    IDEHelper.saveAndOpenJSPFileAs(jspPageWizardModel.jspFolder,
					   jspPageWizardModel.simpleJSPPage.getJSPName(), 
					   "jsp",jspPageContents,
					   jspPageWizardModel.isOverwriteJSPFile());
	    
	}
	catch(Exception ex) {
	    // Debug.print(ex);		  
	    IDEHelper.showErrorMessageI18N("JBW_UnableToCreateJSPErr"); // NOI18N
	    return false;
	}	  
	return true;  
    }
  
    public boolean onCancel() {
	return true;
    }
    
    public static JSPPageWizard createJSPPageWizard() {
	DefaultWizardPanel[] wizardPanels = {
	    new JSPPageTypeOptionPanel(), 
	    new JSPBeansSelectionPanel(), 
	    new JSPBeanPropInitPanel(),
	    new JSPBizMethodsPanel(),
	    new JSPDisplayFieldsPanel(),
	    new JSPSubmitAndErrorURLInfoPanel(),
	    new JSPPageNameAndStyleInfoPanel()
		};
	   
	JSPPageWizard jspPageWizard = new JSPPageWizard(wizardPanels);
	return jspPageWizard; 
    }  
    public static void showWizard() {
	   
	JSPPageWizard.initWizardModel();
	JSPPageWizard jspPageWizard =
	    JSPPageWizard.createJSPPageWizard();
	jspPageWizard.executeWizard();	  
	jspPageWizard.releaseWizardModel();
    }
  
    public static void main(String[]args) {
	JSPPageWizard.showWizard();
    }
}


