
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.ajde.compiler;

import java.util.*;
import java.io.*;
import org.aspectj.compiler.base.*;
import org.aspectj.compiler.base.parser.*;
import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.base.cst.*;
import org.aspectj.compiler.crosscuts.*;
import org.aspectj.asm.*;
import org.aspectj.ajde.TopManager;
import org.aspectj.asm.StructureManager;

/**
 * Adds the <CODE>StructureManager.UpdateStructurePass</CODE> pass to the
 * compile process and displays a <CODE>CompileProgressPanel</CODE> to inform
 * the user of the compile status.
 *
 * @author Mik Kersten
 */
public class AjdeCompiler extends AspectJCompiler {

    boolean structureDirty = true;
    private boolean doPostSymbolPasses = true;
    private String configFile = null;
    private StructureManager structureManager = null;
    private CompileProgressMonitor progressMonitor = null;
	private static final int MAX_PROGRESS_VAL = 100;

    public AjdeCompiler(StructureManager structureManager) {
        this.structureManager = structureManager;
    }

    public AjdeCompiler(StructureManager structureManager,
    					 ErrorHandler errorHandler,
    					 CompileProgressMonitor progressMonitor) {
        super(errorHandler);
        this.structureManager = structureManager;
        this.progressMonitor = progressMonitor;
        this.progressMonitor.setProgressBarMax(MAX_PROGRESS_VAL);
    }

    public void setDoPostSymbolPasses(boolean doPostSymbolPasses) {
        this.doPostSymbolPasses = doPostSymbolPasses;
    }

    public void setConfigFile(String configFile) {
        this.configFile = configFile;
    }

    public String getBuildConfigFile() {
        return configFile;
    }

    public void abort() {
        this.clearState();
    }

    public void addPostSymbolPasses() {
        addPass(new StructureManager.UpdateStructurePass(getCompiler(), structureManager));
        if (doPostSymbolPasses) {
            super.addPostSymbolPasses();
        }
    }

    protected void updateCompileState(String section, String subSection, double pct) {
        if (progressMonitor != null) {
            progressMonitor.setProgressBarVal((int)(pct*100.0));
            progressMonitor.setProgressText(section + ":" + subSection);
        }
    }

    protected void initializeWorld(List filenames) {
        super.initializeWorld(filenames);
        structureDirty = true;
    }

    protected void warnNoRuntime(boolean foundOld) {
        if (foundOld) {
            throw new BadRuntimeError();
        } else {
            throw new MissingRuntimeError();
        }
    }
}

class MissingRuntimeError extends Error { }

class BadRuntimeError extends Error { }

