
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.ajde.ui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import org.aspectj.ajde.TopManager;
import org.aspectj.ajde.compiler.CompileProgressMonitor;

/**
 * This dialog box is open while ajc is compiling the system and displays
 * a corresponding progress bar.
 *
 * @author  Mik Kersten
 */
public class CompileProgressPanel extends JPanel implements CompileProgressMonitor {

    private static final int MAX_VAL = 100;
    private JDialog dialog = null;

    BorderLayout borderLayout1 = new BorderLayout();
    JPanel cancel_panel = new JPanel();
    JButton cancel_button = new JButton();
    JPanel jPanel2 = new JPanel();
    JProgressBar compile_progressBar = new JProgressBar();
    JPanel jPanel3 = new JPanel();
    JLabel progress_label = new JLabel();
    BorderLayout borderLayout2 = new BorderLayout();

    /**
     * @param   maxVal          the value to which value to which the progress bar will
     *                          count up to (in seconds)
     */
    public CompileProgressPanel() {
        try {
            jbInit();
            compile_progressBar.setMaximum(MAX_VAL);
        }
        catch(Exception e) {
            throw new RuntimeException(e.toString());
        }
    }

    public void start() {
        dialog = new JDialog(TopManager.IDE_MANAGER.getRootFrame(), "ajc Build Progress", false);
//        progressDialog = new CompileProgressPanel();
        dialog.setContentPane(this);
        dialog.setSize(500, 110);
        dialog.setLocationRelativeTo(TopManager.IDE_MANAGER.getRootFrame());
        dialog.setVisible(true);
    }

    public void setProgressText(String text) {
        progress_label.setText("   " + text);
    }

    /**
     * Jumps the progress bar <CODE>newVal</CODE> seconds ahead.
     */
    public void setProgressBarVal(int newVal) {
        compile_progressBar.setValue(newVal);
    }

	public void setProgressBarMax(int maxVal) {
		compile_progressBar.setMaximum(maxVal);
	}

    public int getProgressBarMax() {
        return compile_progressBar.getMaximum();
    }

    /**
     * Makes the progress bar move one second ahead.
     */
    public void incrementProgressBarVal() {
        int newVal = compile_progressBar.getValue() + 1;
        compile_progressBar.setValue(newVal);
    }

    /**
     * Jumps the progress bar to the end.
     */
    public void finish() {
        compile_progressBar.setValue(compile_progressBar.getMaximum());
        dialog.dispose();
    }

    private void jbInit() throws Exception {
        this.setLayout(borderLayout1);
        cancel_button.setFont(new java.awt.Font("Dialog", 0, 11));
        cancel_button.setText("Cancel");
        cancel_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                cancel_button_actionPerformed(e);
            }
        });
        compile_progressBar.setPreferredSize(new Dimension(330, 14));
        progress_label.setFont(new java.awt.Font("Dialog", 0, 11));
        jPanel3.setLayout(borderLayout2);
        jPanel2.setPreferredSize(new Dimension(330, 24));
        this.add(cancel_panel, BorderLayout.SOUTH);
        cancel_panel.add(cancel_button, null);
        this.add(jPanel2, BorderLayout.NORTH);
        jPanel2.add(compile_progressBar, null);
        this.add(jPanel3, BorderLayout.CENTER);
        jPanel3.add(progress_label, BorderLayout.CENTER);
    }

    void cancel_button_actionPerformed(ActionEvent e) {
        TopManager.COMPILER_MANAGER.abortBuild();
    }

    static class CompileProgressThread extends Thread {

        private CompileProgressPanel progressDialog = null;
        private JDialog dialog = null;

        public CompileProgressThread(org.aspectj.ajde.compiler.CompilerManager.CompilerThread compilerThread, JFrame parent) {
//            dialog = new JDialog(parent, "ajc Build Progress", false);
//            progressDialog = new CompileProgressPanel();
//            dialog.setContentPane(progressDialog);
//            dialog.setSize(500, 110);
//            dialog.setLocationRelativeTo(parent);
        }

        public CompileProgressPanel getProgressPanel() {
            return progressDialog;
        }

        public void startCompileProgress() {
            dialog.setVisible(true);
        }

        public void stopCompileProgress() {
            progressDialog.finish();
            dialog.dispose();
        }

        public void setDialogTitle(String title) {
            dialog.setTitle(title);
        }
    }
}