/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.ajde.ui;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;
import java.util.*;
import org.aspectj.ajde.ui.ErrorHandler;
import org.aspectj.asm.*;
import org.aspectj.ajde.*;
import org.aspectj.ajde.views.BrowserManager;

/**
 * UI for setting user-configureable options.
 *
 * @author Mik Kersten
 */
public class OptionsFrame extends JFrame {
    public static final int STRUCTURE_PROPERTIES = 0;
    public static final int BUILD_PROPERTIES = 1;
    private static final String[] granularityLevels = { "package", "file", "type", "member" };

    private static final String ABOUT_TEXT =
        "<html><head></head>\n" +
        "<body>\n" +
        "<p><b>AspectJ Development Environment (AJDE) " + org.aspectj.compiler.Version.text + "</b></p>\n" +
        "<p>Submit bugs at <a href=\"http://aspectj.org/bugs\">http://aspectj.org/bugs</a><br>\n" +
        "For support questions email <a href=\"mailto:support@aspectj.org\">\n" +
        "support@aspectj.org</a> </p>\n" +
        "<p>The AspectJ compiler and core tools are distributed under the Mozilla Public \n" +
        "License version 1.1.&nbsp; This license has been approved by the Open Source \n" +
        "Initiative as conforming to the Open Source Definition.&nbsp; For more \n" +
        "information and the license please visit <a href=\"http://aspectj.org/license\">\n" +
        "http://aspectj.org/license</a></p>\n" +
        "</body>\n" +
        "</html>\n";

    private java.util.List activeRelations = new ArrayList();
    private AssociationsTreeCellRenderer associationsTreeCellRenderer = new AssociationsTreeCellRenderer();
    private boolean associationSelectionEditeable = true;

    private JTabbedPane main_tabbedPane = new JTabbedPane();
    private JPanel button_panel = new JPanel();
    private JButton apply_button = new JButton();
    private JButton cancel_button = new JButton();
    private JButton ok_button = new JButton();
    private TitledBorder titledBorder1;
    private TitledBorder titledBorder2;
    private TitledBorder titledBorder3;
    private Border border1;
    private TitledBorder titledBorder4;
    private TitledBorder titledBorder5;
    private Border border2;
    private TitledBorder titledBorder6;
    private JPanel compileOptions_panel = new JPanel();
    private Box temp_box = Box.createVerticalBox();
    private JTextField classToRun_field = new JTextField();
    private JPanel runOptions_panel = new JPanel();
    private JLabel jLabel4 = new JLabel();
    private JPanel build_panel = new JPanel();
    private JPanel structureView_panel = new JPanel();
    private BorderLayout borderLayout1 = new BorderLayout();
    private Border border3;
    private TitledBorder titledBorder7;
    private Border border4;
    private TitledBorder titledBorder8;
    private Border border5;
    private TitledBorder titledBorder9;
    private JPanel jPanel2 = new JPanel();
    private JPanel ordering_panel = new JPanel();
    private JPanel filtering_panel = new JPanel();
    private JCheckBox privileged_checkBox = new JCheckBox();
    private Box temp_box2 = Box.createVerticalBox();
    private Box temp_box1 = Box.createVerticalBox();
    private JPanel viewing_panel = new JPanel();
    private JCheckBox package_checkBox = new JCheckBox();
    private JCheckBox sort_checkBox = new JCheckBox();
    private JCheckBox groupByKind_checkBox = new JCheckBox();
    private JPanel viewing_panel2 = new JPanel();
    private GridLayout gridLayout1 = new GridLayout();
    private JPanel viewing_panel1 = new JPanel();
    private JCheckBox private_checkBox = new JCheckBox();
    private BorderLayout borderLayout6 = new BorderLayout();
    private BorderLayout borderLayout5 = new BorderLayout();
    private JCheckBox public_checkBox = new JCheckBox();
    private JPanel associations_panel = new JPanel();
    private BorderLayout borderLayout3 = new BorderLayout();
    private BorderLayout borderLayout8 = new BorderLayout();
    private Border border6;
    private TitledBorder titledBorder10;
    private JPanel hierarchy_panel = new JPanel();
    private JComboBox hierarchy_comboBox = null;
    private BorderLayout borderLayout2 = new BorderLayout();
    private JLabel jLabel1 = new JLabel();
    private JScrollPane jScrollPane1 = new JScrollPane();
    private JTree associations_tree = new JTree();
    private ButtonGroup views_buttonGroup = new ButtonGroup();
    private GridLayout gridLayout4 = new GridLayout();
    private JCheckBox groupByVisibility_checkBox = new JCheckBox();
    private GridLayout gridLayout3 = new GridLayout();
    private JPanel selectView_panel = new JPanel();
    private JLabel selectView_label = new JLabel();
    private JSlider granularity_slider = new JSlider();
    private JPanel granularity_panel = new JPanel();
    private BorderLayout borderLayout7 = new BorderLayout();
    private JCheckBox protected_checkBox = new JCheckBox();
    private JPanel viewing_panel3 = new JPanel();
    private JCheckBox synchronized_checkBox = new JCheckBox();
    private JCheckBox volatile_checkBox = new JCheckBox();
    private Box temp_box4 = Box.createVerticalBox();
    private JCheckBox transient_checkBox = new JCheckBox();
    private JCheckBox static_checkBox = new JCheckBox();
    private Box temp_box3 = Box.createVerticalBox();
    private JCheckBox abstract_checkBox = new JCheckBox();
    private JCheckBox final_checkBox = new JCheckBox();
    private Box temp_box6 = Box.createVerticalBox();
    private JCheckBox strictfp_checkBox = new JCheckBox();
    private JCheckBox native_checkBox = new JCheckBox();
    private GridLayout gridLayout6 = new GridLayout();
    private Border border7;
    private TitledBorder titledBorder11;
    private JPanel buildPaths_panel = new JPanel();
    private Border border8;
    private TitledBorder titledBorder12;
    private FlowLayout flowLayout1 = new FlowLayout();
    private BorderLayout borderLayout4 = new BorderLayout();
    private JTextField outputPath_field = new JTextField();
    private Box compileOptions_box2 = Box.createVerticalBox();
    private JLabel jLabel9 = new JLabel();
    private JLabel jLabel16 = new JLabel();
    private JLabel jLabel15 = new JLabel();
    private Box compileOptions_box3 = Box.createVerticalBox();
    private JTextField workingdir_field = new JTextField();
    private JTextField classpath_field = new JTextField();
    private JTextField compilerFlags_field = new JTextField();
    private JLabel jLabel5 = new JLabel();
    private Box temp_box5 = Box.createVerticalBox();
    private Box temp_box7 = Box.createVerticalBox();
    private JCheckBox field_checkBox = new JCheckBox();
    private JCheckBox type_checkBox = new JCheckBox();
    private JCheckBox pointcut_checkBox = new JCheckBox();
    private JCheckBox advice_checkBox = new JCheckBox();
    private JCheckBox method_checkBox = new JCheckBox();
    private Box temp_box8 = Box.createVerticalBox();
    private JCheckBox constructor_checkBox = new JCheckBox();
    private Box temp_box9 = Box.createVerticalBox();
    private JCheckBox declare_checkBox = new JCheckBox();
    private JCheckBox introduction_checkBox = new JCheckBox();
    private GridLayout gridLayout2 = new GridLayout();
    private JPanel about_panel = new JPanel();
    private BorderLayout borderLayout9 = new BorderLayout();
    private JPanel jPanel1 = new JPanel();
    private JEditorPane about_editorPane = new JEditorPane();
    private BorderLayout borderLayout10 = new BorderLayout();

    public OptionsFrame(Icons icons, boolean showStandaloneConfig) {
        try {
            initCompilerOptions();
            initPaths();
            initViews();
            jbInit();
            initGranularitySlider();

            hierarchy_comboBox.setFont(new java.awt.Font("Dialog", 0, 11));
            this.setTitle("Structure View Properties");
            this.setIconImage(((ImageIcon)icons.getBrowserOptionsIcon()).getImage());
            this.setSize(470, 640);
            this.setLocation(200, 10);

            about_editorPane.setContentType("text/html");
            about_editorPane.setText(ABOUT_TEXT);

            if (!showStandaloneConfig) {
                build_panel.remove(runOptions_panel);
                build_panel.remove(buildPaths_panel);
            }
            build_panel.remove(runOptions_panel);
       }
        catch(Exception e) {
            ErrorHandler.handleError("Could not open OptionsFrame", e, this);
        }
    }

    public void showOption(int index) {
        setVisible(true);
        main_tabbedPane.setSelectedIndex(index);
    }

    private void initViews() {
        for (Iterator it = TopManager.BROWSER_MANAGER.getViews().iterator(); it.hasNext(); ) {
            String buttonName = it.next().toString();
            JRadioButton button = new JRadioButton(buttonName);
            button.setFont(new java.awt.Font("Dialog", 0, 11));
            button.setToolTipText("");
            selectView_panel.add(button);
            views_buttonGroup.add(button);
            button.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    Enumeration buttons = views_buttonGroup.getElements();
                    while (buttons.hasMoreElements()) {
                        JRadioButton currButton = (JRadioButton)buttons.nextElement();
                        if (currButton.isSelected()) {
                            setSelectedView(currButton.getText());
                        }
                    }
                }
            });
            if (buttonName.equals("<custom>")) {
                setSelectedView(buttonName);
                button.setSelected(true);
            }
        }
    }

    private void initGranularitySlider() {
        granularity_slider.setMinorTickSpacing(1);
        granularity_slider.setMaximum(granularityLevels.length-1);
        granularity_slider.setMinimum(0);
        granularity_slider.setSnapToTicks(true);
        granularity_slider.setPaintTrack(true);
        granularity_slider.setPaintLabels(true);
        granularity_slider.setPaintTicks(false);
        Hashtable labelDictionary = new Hashtable();
        for (int i = 0; i < granularityLevels.length; i++ ) {
            labelDictionary.put(new Integer(i), new JLabel(granularityLevels[i]) {
                public void setFont(Font font) {
                    super.setFont(new java.awt.Font("Dialog", 0, 9));
                }
            });
        }
        granularity_slider.setLabelTable(labelDictionary);
//        this.granularity_slider.addChangeListener(
//            new ChangeListener() {
//                public void stateChanged(ChangeEvent e) {
//                    applyOptions();
//                }
//        });
        String value = TopManager.projectProperties.getActiveGranularity();
        if (value != null) {
            for (int i = 0; i < granularityLevels.length; i++) {
                if (value.equals(granularityLevels[i])) granularity_slider.setValue(i);
            }
        } else {
            granularity_slider.setValue(granularityLevels.length-1);
        }

    }

    private void setSelectedView(String viewName) {
        if (viewName.equals("<custom>")) {
            associations_tree.setEnabled(true);
            associationSelectionEditeable = true;
        } else {
            associations_tree.setEnabled(false);
            associationSelectionEditeable = false;
        }

        refreshAssociations(viewName);
        refreshHierarchies(viewName);
        refreshModifiers();
        refreshVisibility();
        refreshOrdering();
        refreshMemberKinds();

        // HACK
        Vector hierarchies = new Vector();
        String s1 = "contains relation (default)";
        String s2 = "uses pointcut relation";
        String s3 = "inherits relation";
        if (viewName.equals("crosscutting")) {
            hierarchies.add(s2);
        } else if (viewName.equals("inheritance")) {
            hierarchies.add(s3);
        } else {
            hierarchies.add(s1);
        }
        hierarchy_comboBox = new JComboBox(hierarchies);
        hierarchy_comboBox.setEnabled(false);
    }

    private void refreshVisibility() {
        java.util.List visibility = TopManager.projectProperties.getActiveVisibility();
        if (visibility != null) {
            if (visibility.contains("public")) public_checkBox.setSelected(true);
            if (visibility.contains("protected")) protected_checkBox.setSelected(true);
            if (visibility.contains("package")) package_checkBox.setSelected(true);
            if (visibility.contains("private")) private_checkBox.setSelected(true);
            if (visibility.contains("privileged")) privileged_checkBox.setSelected(true);
        }
    }

    private void refreshOrdering() {
        java.util.List ordering = TopManager.projectProperties.getStructureViewOrdering();
        if (ordering != null) {
            if (ordering.contains("sort")) sort_checkBox.setSelected(true);
            if (ordering.contains("groupByKind")) groupByKind_checkBox.setSelected(true);
            if (ordering.contains("groupByVisibility")) groupByVisibility_checkBox.setSelected(true);
        }
    }

    private void refreshModifiers() {
        java.util.List modifiers = TopManager.projectProperties.getActiveModifiers();
        if (modifiers != null) {
            if (modifiers.contains("static")) static_checkBox.setSelected(true);
            if (modifiers.contains("final")) final_checkBox.setSelected(true);
            if (modifiers.contains("abstract")) abstract_checkBox.setSelected(true);
            if (modifiers.contains("synchronized")) synchronized_checkBox.setSelected(true);
            if (modifiers.contains("transient")) transient_checkBox.setSelected(true);
            if (modifiers.contains("volatile")) volatile_checkBox.setSelected(true);
            if (modifiers.contains("native")) native_checkBox.setSelected(true);
            if (modifiers.contains("strictfp")) strictfp_checkBox.setSelected(true);
        }
    }

    private void refreshMemberKinds() {
        java.util.List kinds = TopManager.projectProperties.getActiveFilteredMemberKinds();
        if (kinds != null) {
            if (kinds.contains("field")) field_checkBox.setSelected(true);
            if (kinds.contains("constructor")) constructor_checkBox.setSelected(true);
            if (kinds.contains("method")) method_checkBox.setSelected(true);
            if (kinds.contains("type")) type_checkBox.setSelected(true);
            if (kinds.contains("pointcut")) pointcut_checkBox.setSelected(true);
            if (kinds.contains("advice")) advice_checkBox.setSelected(true);
            if (kinds.contains("declare")) declare_checkBox.setSelected(true);
        }
    }

    private void initCompilerOptions() {
        compilerFlags_field.setText(TopManager.projectProperties.getCompilerFlags());
    }

    private void initPaths() {
        String outputPath = TopManager.projectProperties.getOutputPath();
        if (outputPath != null) outputPath_field.setText(outputPath);

        String classpath = TopManager.projectProperties.getUserClasspath();
        if (classpath != null) classpath_field.setText(classpath);

        String workingDir = TopManager.projectProperties.getWorkingDir();
        if (workingDir != null) workingdir_field.setText(workingDir);

        String mainClass = TopManager.projectProperties.getClassToRun();
        if (mainClass != null) classToRun_field.setText(mainClass);
    }

    private void refreshHierarchies(String viewName) {
        String activeHierarchy = TopManager.projectProperties.getActiveHierarchy();
        final String SUFFIX = " association";
        Vector hierarchies = new Vector();
        hierarchies.add("composition association (default)");
        int index = 0;
        int selectionIndex = 0;
//        for (Iterator it = TopManager.STRUCTURE_MANAGER.getAssociations().iterator(); it.hasNext(); ) {
//            Association association = (Association)it.next();
//            if (association.isHierarchical()) {
//                hierarchies.add(association.getStereotypeName() + SUFFIX);
//                if (activeHierarchy != null && activeHierarchy.equals(association.getStereotypeName() + SUFFIX)) {
//                    selectionIndex = index;
//                }
//                index++;
//            }
//        }
        hierarchy_comboBox = new JComboBox(hierarchies);
        hierarchy_comboBox.setSelectedIndex(selectionIndex);
    }

    private void refreshAssociations(String viewName) {
        AssociationTreeNode root = new AssociationTreeNode("associations");
        java.util.List associations = TopManager.STRUCTURE_MANAGER.getAssociations();
        java.util.List savedRelations = TopManager.BROWSER_MANAGER.getViewProperties(viewName).getAssociations();
        for (Iterator it = associations.iterator(); it.hasNext(); ) {
            Association association = (Association)it.next();
            AssociationTreeNode associationNode = new AssociationTreeNode(association.getName());
            java.util.List relations = association.getRelations();
            for (Iterator it2 = relations.iterator(); it2.hasNext(); ) {
                Relation currRelation = (Relation)it2.next();
                AssociationTreeNode relationNode = new AssociationTreeNode(currRelation);
                associationNode.add(relationNode);
                if (savedRelations.contains(currRelation.toString())) {
                    //savedRelations.contains(currReltation.getForwardNavigationName())
                    //|| savedRelations.contains(currReltation.getBackNavigationName())) {
                    relationNode.checkBox.setSelected(true);
                    if (viewName.equals("<custom>")) {
                        activeRelations.add(currRelation.toString());
                    }
                }
            }
            root.add(associationNode);
        }

        associations_tree.setModel(new DefaultTreeModel(root));
        associations_tree.addMouseListener(new AssociationTreeMouseAdapter(associations_tree));
        associations_tree.setCellRenderer(associationsTreeCellRenderer);

        for (int j = 0; j < associations_tree.getRowCount(); j++) {
            associations_tree.expandPath(associations_tree.getPathForRow(j));
        }
    }

    private void applyOptions() {
        java.util.List modifiers = new ArrayList();
        if (static_checkBox.isSelected()) modifiers.add("static");
        if (final_checkBox.isSelected()) modifiers.add("final");
        if (abstract_checkBox.isSelected()) modifiers.add("abstract");
        if (synchronized_checkBox.isSelected()) modifiers.add("synchronized");
        if (transient_checkBox.isSelected()) modifiers.add("transient");
        if (volatile_checkBox.isSelected()) modifiers.add("volatile");
        if (native_checkBox.isSelected()) modifiers.add("native");
        if (strictfp_checkBox.isSelected()) modifiers.add("strictfp");
        TopManager.projectProperties.setActiveModifiers(modifiers);

        java.util.List accessibilityModifiers = new ArrayList();
        if (public_checkBox.isSelected()) accessibilityModifiers.add("public");
        if (protected_checkBox.isSelected()) accessibilityModifiers.add("protected");
        if (package_checkBox.isSelected()) accessibilityModifiers.add("package");
        if (private_checkBox.isSelected()) accessibilityModifiers.add("private");
        if (privileged_checkBox.isSelected()) accessibilityModifiers.add("privileged");
        TopManager.projectProperties.setActiveVisiblity(accessibilityModifiers);

        java.util.List memberKinds = new ArrayList();
        if (field_checkBox.isSelected()) memberKinds.add("field");
        if (constructor_checkBox.isSelected()) memberKinds.add("constructor");
        if (method_checkBox.isSelected()) memberKinds.add("method");
        if (type_checkBox.isSelected()) memberKinds.add("type");
        if (pointcut_checkBox.isSelected()) memberKinds.add("pointcut");
        if (advice_checkBox.isSelected()) memberKinds.add("advice");
        if (declare_checkBox.isSelected()) memberKinds.add("declare");
        if (introduction_checkBox.isSelected()) memberKinds.add("introduction");
        TopManager.projectProperties.setActiveFilteredMemberKinds(memberKinds);

        java.util.List ordering = new ArrayList();
        if (sort_checkBox.isSelected()) ordering.add("sort");
        if (groupByKind_checkBox.isSelected()) ordering.add("groupByKind");
        if (groupByVisibility_checkBox.isSelected()) ordering.add("groupByVisibility");
        TopManager.projectProperties.setStructureViewOrdering(ordering);

        TopManager.projectProperties.setActiveGranularity(granularityLevels[granularity_slider.getValue()]);

        TopManager.projectProperties.setCompilerFlags(compilerFlags_field.getText());
        TopManager.projectProperties.setActiveAssociations(activeRelations);
        TopManager.projectProperties.setOutputPath(outputPath_field.getText());
        TopManager.projectProperties.setUserClasspath(classpath_field.getText());
        TopManager.projectProperties.setWorkingDir(workingdir_field.getText());
        TopManager.projectProperties.setClassToRun(classToRun_field.getText());

        TopManager.BROWSER_MANAGER.updateView();
    }

    private String getSelectedView() {
        return "<custom>";
    }

    private void close() {
        this.setVisible(false);
    }

    private void apply_button_actionPerformed(ActionEvent e) {
        applyOptions();
    }

    private void ok_button_actionPerformed(ActionEvent e) {
        applyOptions();
        close();
    }

    private void cancel_button_actionPerformed(ActionEvent e) {
        close();
    }

    private class AssociationTreeNode extends DefaultMutableTreeNode {
        public JCheckBox checkBox = null;
        public Relation relation = null;

        public AssociationTreeNode(Relation relation) {
            super(relation.toString(), true);
            this.relation = relation;
            checkBox = new JCheckBox();
        }

        public AssociationTreeNode(String kind) {
            super(kind, true);
            checkBox = new JCheckBox();
        }

        public String getName() {
            return super.toString();
        }
    }

    private class AssociationTreeMouseAdapter extends MouseAdapter {
        private JTree tree = null;
        final JCheckBox checkBoxProto = new JCheckBox();
        final int width = checkBoxProto.getPreferredSize().width;

        public AssociationTreeMouseAdapter(JTree tree) {
            super();
            this.tree = tree;
        }

        public void mousePressed(MouseEvent e) {
            if (!associationSelectionEditeable) return;
            int x = e.getX();
            int y = e.getY();
            TreePath path = tree.getClosestPathForLocation(x,y);
            AssociationTreeNode node = (AssociationTreeNode)path.getLastPathComponent();

            node.checkBox.setSelected(!node.checkBox.isSelected());
            ((DefaultTreeModel)tree.getModel()).nodeChanged(node);
            if (node.relation != null) {
                if (node.checkBox.isSelected()) {
                    activeRelations.add(node.relation.toString());
                } else {
                    activeRelations.remove(node.relation.toString());
                }
            }
            super.mousePressed(e);
        }

        boolean isCheckBox(int x, int x_) {
            int d = x - x_;
            return (d < width) && (d > 0);
        }
    }

    private class AssociationsTreeCellRenderer extends DefaultTreeCellRenderer {

        public Component getTreeCellRendererComponent(JTree tree,
                                                      Object value,
                                                      boolean sel,
                                                      boolean expanded,
                                                      boolean leaf,
                                                      int row,
                                                      boolean hasFocus) {
            super.getTreeCellRendererComponent(tree, value, sel, expanded,
                                               leaf, row, hasFocus);
            JPanel p = new JPanel();
            p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
            p.setBackground(Color.white);
            if (leaf)  {
                setFont(new Font(this.getFont().getName(), Font.PLAIN, 11));
                final JCheckBox cbox = ((AssociationTreeNode)value).checkBox;
                cbox.setBackground(Color.white);
                cbox.setEnabled(associationSelectionEditeable);
                p.add(cbox);
                AssociationTreeNode val = (AssociationTreeNode)value;
                String name = val.getName();
            }

            p.add(this);
            return p;
        }
    }

    private void jbInit() throws Exception {
        titledBorder1 = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(156, 156, 158)),"Sorting");
        titledBorder2 = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(148, 145, 140)),"Associations (navigeable relations between sturcture nodes)");
        titledBorder3 = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(156, 156, 158)),"Member Filtering (nodes to exclude from view)");
        border1 = BorderFactory.createLineBorder(Color.black,2);
        titledBorder4 = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(148, 145, 140)),"Compile Options");
        titledBorder5 = new TitledBorder("");
        border2 = BorderFactory.createLineBorder(Color.black,2);
        titledBorder6 = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(148, 145, 140)),"Run Options");
        border3 = BorderFactory.createEtchedBorder(Color.white,new Color(156, 156, 158));
        titledBorder7 = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(156, 156, 158)),"Granularity (all nodes below selected level will be hidden)");
        border4 = BorderFactory.createEtchedBorder(Color.white,new Color(156, 156, 158));
        titledBorder8 = new TitledBorder(border4,"Member Visibility");
        border5 = BorderFactory.createEtchedBorder(Color.white,new Color(156, 156, 158));
        titledBorder9 = new TitledBorder(border5,"Member Modifiers");
        border6 = BorderFactory.createEmptyBorder();
        titledBorder10 = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(148, 145, 140)),"Structure View Properties");
        border7 = BorderFactory.createEtchedBorder(Color.white,new Color(156, 156, 158));
        titledBorder11 = new TitledBorder(border7,"Member Kinds");
        border8 = BorderFactory.createEtchedBorder(Color.white,new Color(148, 145, 140));
        titledBorder12 = new TitledBorder(border8,"Build Paths");
        apply_button.setFont(new java.awt.Font("Dialog", 0, 11));
        apply_button.setMaximumSize(new Dimension(70, 24));
        apply_button.setMinimumSize(new Dimension(63, 24));
        apply_button.setPreferredSize(new Dimension(70, 24));
        apply_button.setText("Apply");
        apply_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                apply_button_actionPerformed(e);
            }
        });
        cancel_button.setFont(new java.awt.Font("Dialog", 0, 11));
        cancel_button.setMaximumSize(new Dimension(70, 24));
        cancel_button.setMinimumSize(new Dimension(67, 24));
        cancel_button.setPreferredSize(new Dimension(70, 24));
        cancel_button.setText("Cancel");
        cancel_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                cancel_button_actionPerformed(e);
            }
        });
        ok_button.setFont(new java.awt.Font("Dialog", 0, 11));
        ok_button.setMaximumSize(new Dimension(70, 24));
        ok_button.setMinimumSize(new Dimension(49, 24));
        ok_button.setPreferredSize(new Dimension(70, 24));
        ok_button.setText("OK");
        ok_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                ok_button_actionPerformed(e);
            }
        });
        main_tabbedPane.setFont(new java.awt.Font("Dialog", 0, 11));
        titledBorder1.setTitle("Ordering (sort order of nodes)");
        titledBorder1.setTitleFont(new java.awt.Font("Dialog", 0, 11));
        titledBorder2.setTitle("Associations (navigeable relations between structure nodes)");
        titledBorder2.setTitleFont(new java.awt.Font("Dialog", 0, 11));
        titledBorder3.setTitleFont(new java.awt.Font("Dialog", 0, 11));
        titledBorder6.setTitleFont(new java.awt.Font("Dialog", 0, 11));
        titledBorder5.setTitleFont(new java.awt.Font("Dialog", 0, 11));
        titledBorder4.setTitle("Compiler Flags");
        titledBorder4.setTitleFont(new java.awt.Font("Dialog", 0, 11));
        compileOptions_panel.setBorder(titledBorder4);
        classToRun_field.setFont(new java.awt.Font("SansSerif", 0, 11));
        classToRun_field.setMinimumSize(new Dimension(200, 21));
        classToRun_field.setPreferredSize(new Dimension(250, 21));
        runOptions_panel.setBorder(titledBorder6);
        jLabel4.setFont(new java.awt.Font("Dialog", 0, 11));
        jLabel4.setText("Main class to run: ");
        build_panel.setLayout(borderLayout4);
        structureView_panel.setLayout(borderLayout1);
        titledBorder7.setTitleFont(new java.awt.Font("Dialog", 0, 11));
        titledBorder8.setTitle("Access Modifiers");
        titledBorder8.setTitleFont(new java.awt.Font("Dialog", 0, 11));
        titledBorder9.setTitleFont(new java.awt.Font("Dialog", 0, 11));
        ordering_panel.setLayout(gridLayout4);
        ordering_panel.setPreferredSize(new Dimension(250, 50));
        ordering_panel.setMinimumSize(new Dimension(250, 40));
        ordering_panel.setBorder(titledBorder1);
        ordering_panel.setFont(new java.awt.Font("Dialog", 0, 11));
        filtering_panel.setLayout(borderLayout5);
        privileged_checkBox.setFont(new java.awt.Font("Dialog", 0, 11));
        privileged_checkBox.setText("privileged");
        viewing_panel.setFont(new java.awt.Font("Dialog", 0, 11));
        viewing_panel.setBorder(titledBorder3);
        viewing_panel.setMinimumSize(new Dimension(250, 40));
        viewing_panel.setPreferredSize(new Dimension(255, 130));
        viewing_panel.setLayout(borderLayout6);
        package_checkBox.setFont(new java.awt.Font("Dialog", 0, 11));
        package_checkBox.setText("package");
        sort_checkBox.setFont(new java.awt.Font("Dialog", 0, 11));
        sort_checkBox.setToolTipText("");
        sort_checkBox.setText("Sort Alphabetically");
        groupByKind_checkBox.setEnabled(false);
        groupByKind_checkBox.setFont(new java.awt.Font("Dialog", 0, 11));
        groupByKind_checkBox.setText("Group by kind");
        viewing_panel2.setFont(new java.awt.Font("Dialog", 0, 11));
        viewing_panel2.setBorder(titledBorder9);
        viewing_panel2.setMinimumSize(new Dimension(250, 40));
        viewing_panel2.setPreferredSize(new Dimension(250, 140));
        viewing_panel2.setLayout(gridLayout6);
        viewing_panel1.setFont(new java.awt.Font("Dialog", 0, 11));
        viewing_panel1.setBorder(titledBorder8);
        viewing_panel1.setMinimumSize(new Dimension(250, 120));
        viewing_panel1.setPreferredSize(new Dimension(180, 140));
        viewing_panel1.setLayout(gridLayout1);
        private_checkBox.setFont(new java.awt.Font("Dialog", 0, 11));
        private_checkBox.setText("private");
        public_checkBox.setText("public");
        public_checkBox.setFont(new java.awt.Font("Dialog", 0, 11));
        associations_panel.setFont(new java.awt.Font("Dialog", 0, 11));
        associations_panel.setBorder(titledBorder2);
        associations_panel.setPreferredSize(new Dimension(495, 200));
        associations_panel.setLayout(borderLayout3);
        jPanel2.setLayout(borderLayout8);
        filtering_panel.setPreferredSize(new Dimension(400, 310));
        hierarchy_panel.setMaximumSize(new Dimension(32767, 27));
        hierarchy_panel.setMinimumSize(new Dimension(10, 24));
        hierarchy_panel.setPreferredSize(new Dimension(10, 24));
        hierarchy_panel.setLayout(borderLayout2);
        hierarchy_comboBox.setFont(new java.awt.Font("SansSerif", 0, 11));
        jLabel1.setFont(new java.awt.Font("Dialog", 0, 11));
        jLabel1.setToolTipText("");
        jLabel1.setText("Tree hierarchy grouped by ");
        associations_tree.setFont(new java.awt.Font("Dialog", 0, 11));
        associations_tree.setBorder(BorderFactory.createLoweredBevelBorder());
        titledBorder10.setTitleFont(new java.awt.Font("Dialog", 0, 11));
        groupByVisibility_checkBox.setText("Group by visibility");
        groupByVisibility_checkBox.setEnabled(false);
        groupByVisibility_checkBox.setFont(new java.awt.Font("Dialog", 0, 11));
        selectView_panel.setLayout(gridLayout3);
        selectView_label.setFont(new java.awt.Font("Dialog", 0, 11));
        selectView_label.setPreferredSize(new Dimension(40, 14));
        selectView_label.setText("  Preset ");
        granularity_slider.setPaintLabels(true);
        granularity_slider.setMaximumSize(new Dimension(32767, 60));
        granularity_slider.setMinimumSize(new Dimension(36, 50));
        granularity_slider.setPreferredSize(new Dimension(200, 57));
        granularity_panel.setFont(new java.awt.Font("Dialog", 0, 11));
        granularity_panel.setBorder(titledBorder7);
        granularity_panel.setMinimumSize(new Dimension(250, 40));
        granularity_panel.setPreferredSize(new Dimension(250, 70));
        granularity_panel.setLayout(borderLayout7);
        protected_checkBox.setText("protected");
        protected_checkBox.setFont(new java.awt.Font("Dialog", 0, 11));
        viewing_panel3.setLayout(gridLayout2);
        viewing_panel3.setPreferredSize(new Dimension(250, 70));
        viewing_panel3.setMinimumSize(new Dimension(250, 40));
        viewing_panel3.setBorder(titledBorder11);
        viewing_panel3.setFont(new java.awt.Font("Dialog", 0, 11));
        synchronized_checkBox.setFont(new java.awt.Font("Dialog", 0, 11));
        synchronized_checkBox.setPreferredSize(new Dimension(91, 20));
        synchronized_checkBox.setText("synchronized");
        volatile_checkBox.setFont(new java.awt.Font("Dialog", 0, 11));
        volatile_checkBox.setPreferredSize(new Dimension(58, 20));
        volatile_checkBox.setText("volatile");
        transient_checkBox.setText("transient");
        transient_checkBox.setPreferredSize(new Dimension(67, 20));
        transient_checkBox.setFont(new java.awt.Font("Dialog", 0, 11));
        static_checkBox.setFont(new java.awt.Font("Dialog", 0, 11));
        static_checkBox.setPreferredSize(new Dimension(51, 20));
        static_checkBox.setText("static");
        abstract_checkBox.setPreferredSize(new Dimension(65, 20));
        abstract_checkBox.setText("abstract");
        abstract_checkBox.setFont(new java.awt.Font("Dialog", 0, 11));
        final_checkBox.setFont(new java.awt.Font("Dialog", 0, 11));
        final_checkBox.setPreferredSize(new Dimension(45, 20));
        final_checkBox.setText("final");
        strictfp_checkBox.setText("strictfp");
        strictfp_checkBox.setPreferredSize(new Dimension(51, 20));
        strictfp_checkBox.setFont(new java.awt.Font("Dialog", 0, 11));
        native_checkBox.setFont(new java.awt.Font("Dialog", 0, 11));
        native_checkBox.setPreferredSize(new Dimension(54, 20));
        native_checkBox.setText("native");
        jScrollPane1.setPreferredSize(new Dimension(82, 300));
        titledBorder11.setTitleFont(new java.awt.Font("Dialog", 0, 11));
        buildPaths_panel.setBorder(titledBorder12);
        buildPaths_panel.setLayout(flowLayout1);
        titledBorder12.setTitleFont(new java.awt.Font("Dialog", 0, 11));
        outputPath_field.setPreferredSize(new Dimension(150, 21));
        outputPath_field.setMinimumSize(new Dimension(100, 21));
        outputPath_field.setFont(new java.awt.Font("SansSerif", 0, 11));
        jLabel9.setText("Workingdir (only relevant with -usejavac or -preprocess): ");
        jLabel9.setPreferredSize(new Dimension(285, 25));
        jLabel9.setFont(new java.awt.Font("Dialog", 0, 11));
        jLabel9.setMaximumSize(new Dimension(400, 25));
        jLabel16.setText("Classpath (defaults to current directory): ");
        jLabel16.setPreferredSize(new Dimension(285, 25));
        jLabel16.setMaximumSize(new Dimension(400, 25));
        jLabel16.setFont(new java.awt.Font("Dialog", 0, 11));
        jLabel15.setMaximumSize(new Dimension(400, 25));
        jLabel15.setFont(new java.awt.Font("Dialog", 0, 11));
        jLabel15.setPreferredSize(new Dimension(285, 25));
        jLabel15.setText("Output path (defaults to current directory): ");
        workingdir_field.setFont(new java.awt.Font("SansSerif", 0, 11));
        workingdir_field.setMinimumSize(new Dimension(100, 21));
        workingdir_field.setPreferredSize(new Dimension(150, 21));
        classpath_field.setFont(new java.awt.Font("SansSerif", 0, 11));
        classpath_field.setMinimumSize(new Dimension(100, 21));
        classpath_field.setPreferredSize(new Dimension(150, 21));
        compilerFlags_field.setFont(new java.awt.Font("SansSerif", 0, 11));
        compilerFlags_field.setMinimumSize(new Dimension(100, 21));
        compilerFlags_field.setPreferredSize(new Dimension(250, 21));
        jLabel5.setText("Compiler Flags: ");
        jLabel5.setFont(new java.awt.Font("Dialog", 0, 11));
        field_checkBox.setFont(new java.awt.Font("Dialog", 0, 11));
        field_checkBox.setMinimumSize(new Dimension(30, 24));
        field_checkBox.setPreferredSize(new Dimension(30, 24));
        field_checkBox.setText("field");
        type_checkBox.setText("type");
        type_checkBox.setFont(new java.awt.Font("Dialog", 0, 11));
        pointcut_checkBox.setFont(new java.awt.Font("Dialog", 0, 11));
        pointcut_checkBox.setText("pointcut");
        advice_checkBox.setFont(new java.awt.Font("Dialog", 0, 11));
        advice_checkBox.setText("advice");
        method_checkBox.setFont(new java.awt.Font("Dialog", 0, 11));
        method_checkBox.setText("method");
        constructor_checkBox.setFont(new java.awt.Font("Dialog", 0, 11));
        constructor_checkBox.setText("constructor");
        declare_checkBox.setText("declare");
        declare_checkBox.setFont(new java.awt.Font("Dialog", 0, 11));
        introduction_checkBox.setText("introduction");
        introduction_checkBox.setFont(new java.awt.Font("Dialog", 0, 11));
        about_panel.setLayout(borderLayout9);
        jPanel1.setLayout(borderLayout10);
        about_editorPane.setBackground(UIManager.getColor("OptionPane.background"));
        about_editorPane.setFont(new java.awt.Font("Dialog", 0, 11));
        about_editorPane.setEditable(false);
        compileOptions_panel.add(jLabel5, null);
        compileOptions_panel.add(compilerFlags_field, null);
        build_panel.add(runOptions_panel, BorderLayout.SOUTH);
        build_panel.add(buildPaths_panel, BorderLayout.CENTER);
        runOptions_panel.add(jLabel4, null);
        runOptions_panel.add(classToRun_field, null);
        about_panel.add(jPanel1,  BorderLayout.CENTER);
        jPanel1.add(about_editorPane,  BorderLayout.CENTER);
        main_tabbedPane.add(structureView_panel,      "Structure View");
        structureView_panel.add(jPanel2, BorderLayout.CENTER);
        temp_box7.add(field_checkBox, null);
        temp_box7.add(type_checkBox, null);
        viewing_panel3.add(temp_box8, null);
        temp_box5.add(pointcut_checkBox, null);
        temp_box5.add(advice_checkBox, null);
        viewing_panel3.add(temp_box9, null);
        viewing_panel3.add(temp_box7, null);
        temp_box8.add(method_checkBox, null);
        temp_box8.add(constructor_checkBox, null);
        viewing_panel3.add(temp_box5, null);
        temp_box9.add(declare_checkBox, null);
        temp_box9.add(introduction_checkBox, null);
        viewing_panel.add(viewing_panel1, BorderLayout.WEST);
        filtering_panel.add(granularity_panel, BorderLayout.SOUTH);
        granularity_panel.add(granularity_slider, BorderLayout.CENTER);
        filtering_panel.add(ordering_panel, BorderLayout.NORTH);
        viewing_panel.add(viewing_panel3, BorderLayout.SOUTH);
        viewing_panel2.add(temp_box4, null);
        temp_box4.add(synchronized_checkBox, null);
        temp_box4.add(volatile_checkBox, null);
        temp_box4.add(transient_checkBox, null);
        viewing_panel2.add(temp_box6, null);
        temp_box3.add(static_checkBox, null);
        temp_box3.add(final_checkBox, null);
        temp_box3.add(abstract_checkBox, null);
        main_tabbedPane.add(build_panel, "Build Options");
        main_tabbedPane.add(about_panel,   "About AJDE");
        temp_box6.add(strictfp_checkBox, null);
        temp_box6.add(native_checkBox, null);
        viewing_panel2.add(temp_box3, null);
        temp_box2.add(package_checkBox, null);
        temp_box2.add(private_checkBox, null);
        viewing_panel.add(viewing_panel2, BorderLayout.CENTER);
        viewing_panel1.add(temp_box1, null);
        temp_box1.add(public_checkBox, null);
        temp_box1.add(protected_checkBox, null);
        temp_box1.add(privileged_checkBox, null);
        viewing_panel1.add(temp_box2, null);
        ordering_panel.add(sort_checkBox, null);
        ordering_panel.add(groupByKind_checkBox, null);
        ordering_panel.add(groupByVisibility_checkBox, null);
        filtering_panel.add(viewing_panel, BorderLayout.CENTER);
        hierarchy_panel.add(jLabel1, BorderLayout.WEST);
        hierarchy_panel.add(hierarchy_comboBox, BorderLayout.CENTER);
        associations_panel.add(jScrollPane1, BorderLayout.CENTER);
        associations_panel.add(selectView_panel, BorderLayout.NORTH);
        selectView_panel.add(selectView_label, 0);
        jPanel2.add(filtering_panel, BorderLayout.SOUTH);
        jScrollPane1.getViewport().add(associations_tree, null);
        associations_panel.add(hierarchy_panel, BorderLayout.SOUTH);
        jPanel2.add(associations_panel,  BorderLayout.CENTER);
        this.getContentPane().add(button_panel, BorderLayout.SOUTH);
        button_panel.add(ok_button, null);
        button_panel.add(cancel_button, null);
        button_panel.add(apply_button, null);
        this.getContentPane().add(main_tabbedPane, BorderLayout.CENTER);
        compileOptions_box3.add(jLabel15, null);
        compileOptions_box3.add(jLabel16, null);
        compileOptions_box3.add(jLabel9, null);
        buildPaths_panel.add(compileOptions_box2, null);
        buildPaths_panel.add(compileOptions_box3, null);
        compileOptions_box2.add(outputPath_field, null);
        compileOptions_box2.add(classpath_field, null);
        compileOptions_box2.add(workingdir_field, null);
        build_panel.add(compileOptions_panel, BorderLayout.NORTH);
    }
}