
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.ajde.views;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;

import java.awt.*;
import java.io.*;
import javax.swing.border.*;
import java.awt.event.*;
import java.util.*;
import java.text.*;
import org.aspectj.ajde.*;
import org.aspectj.ajde.editor.*;
import java.beans.*;
import org.aspectj.ajde.ui.Icons;
import org.aspectj.ajde.ui.ErrorHandler;

/**
 * @author Mik Kersten
 */
class BrowserPanel extends JPanel {
    private StructureView masterView;
    private StructureView slaveView;
    private boolean slaveViewVisible = false;
    private String lastSelectedConfig = "";

    private Icons icons = null;
    private BorderLayout borderLayout1 = new BorderLayout();
    private Border default_border;
    private JPanel toolBar_panel = new JPanel();
    private BorderLayout borderLayout2 = new BorderLayout();
    JPanel mainToolBar_panel = new JPanel();
    JToolBar config_toolBar = new JToolBar();
    JComboBox configs_comboBox = null;
    BorderLayout borderLayout3 = new BorderLayout();
    JToolBar nav_toolBar = new JToolBar();
    JButton forward_button = new JButton();
    JButton back_button = new JButton();
    GridLayout gridLayout1 = new GridLayout();
    JSplitPane views_splitPane = new JSplitPane();
    JToolBar command_toolBar = new JToolBar();
    JToggleButton splitView_button = new JToggleButton();
    JToggleButton zoomToFile_button = new JToggleButton();
    JButton joinpointProbe_button = new JButton();

    public BrowserPanel(StructureView masterView, StructureView slaveView, Icons icons) {
        try {
            this.masterView = masterView;
            this.slaveView = slaveView;
            this.icons = icons;
            configs_comboBox = new JComboBox(TopManager.IDE_MANAGER.getConfigFiles().toArray());
            configs_comboBox.setRenderer(new ConfigsCellRenderer());
            jbInit();
            fixButtonBorders();
            views_splitPane.add(masterView, JSplitPane.TOP);
            views_splitPane.add(slaveView, JSplitPane.BOTTOM);
            setSlaveViewVisible(false);

            nav_toolBar.remove(joinpointProbe_button);
        } catch(Exception e) {
            e.printStackTrace();
        }
    }

    public void setSlaveViewVisible(boolean visible) {
        slaveViewVisible = visible;
        if (visible) {
            views_splitPane.add(slaveView, JSplitPane.BOTTOM);
            views_splitPane.setDividerLocation(this.getHeight()-250);
            masterView.scrollToHighlightedNode();
        } else {
            views_splitPane.remove(slaveView);
            views_splitPane.setDividerLocation(this.getHeight());
        }
    }

    public boolean isSlaveViewVisible() {
        return slaveViewVisible;
    }

    public void updateConfigs(java.util.List configsList) {
        configs_comboBox.removeAllItems();
        for (Iterator it = configsList.iterator(); it.hasNext(); ) {
            configs_comboBox.addItem(it.next());
        }
    }

    public void setSelectedConfig(String config) {
        for (int i = 0; i < configs_comboBox.getItemCount(); i++) {
            if (configs_comboBox.getItemAt(i).equals(config)) {
                configs_comboBox.setSelectedIndex(i);
            }
        }
    }

    public String getSelectedConfig() {
        return (String)configs_comboBox.getSelectedItem();
    }

    /**
     * @todo    get rid of this method and make the GUI-designer generated code work properly
     */
    private void fixButtonBorders() {
        back_button.setBorder(BorderFactory.createEmptyBorder());
        forward_button.setBorder(BorderFactory.createEmptyBorder());
        zoomToFile_button.setBorder(BorderFactory.createEmptyBorder());
        splitView_button.setBorder(BorderFactory.createMatteBorder(0, 0, 0, 0, Color.blue));
    }

    private void jbInit() throws Exception {
        default_border = BorderFactory.createEmptyBorder();
        this.setLayout(borderLayout1);
        toolBar_panel.setLayout(borderLayout2);
        toolBar_panel.setBorder(BorderFactory.createEtchedBorder());
        config_toolBar.setBorder(default_border);
        config_toolBar.setFloatable(false);
        configs_comboBox.setPreferredSize(new Dimension(200, 20));
        configs_comboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                configs_comboBox_actionPerformed(e);
            }
        });
        configs_comboBox.setMinimumSize(new Dimension(40, 20));
        configs_comboBox.setFont(new java.awt.Font("SansSerif", 0, 11));
        mainToolBar_panel.setLayout(borderLayout3);
        nav_toolBar.setFloatable(false);
        nav_toolBar.setBorder(default_border);
        forward_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                forward_button_actionPerformed(e);
            }
        });
        forward_button.setIcon(icons.getForwardIcon());
        forward_button.setToolTipText("Navigate forward");
        forward_button.setPreferredSize(new Dimension(20, 20));
        forward_button.setMinimumSize(new Dimension(20, 20));
        forward_button.setBorder(default_border);
        forward_button.setMaximumSize(new Dimension(24, 20));
        back_button.setMaximumSize(new Dimension(24, 20));
        back_button.setBorder(default_border);
        back_button.setMinimumSize(new Dimension(20, 20));
        back_button.setPreferredSize(new Dimension(20, 20));
        back_button.setToolTipText("Navigate back");
        back_button.setIcon(icons.getBackIcon());
        back_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                back_button_actionPerformed(e);
            }
        });
//        structureViews_box.add(comment_editorPane, null);
        views_splitPane.setOrientation(JSplitPane.VERTICAL_SPLIT);
        views_splitPane.setDividerSize(2);
        command_toolBar.setBorder(default_border);
        command_toolBar.setFloatable(false);
        splitView_button.setFont(new java.awt.Font("Dialog", 0, 11));
        splitView_button.setBorder(default_border);
        splitView_button.setMaximumSize(new Dimension(24, 24));
        splitView_button.setPreferredSize(new Dimension(20, 20));
        splitView_button.setToolTipText("Togge split-tree view mode");
        splitView_button.setIcon(icons.getSplitStructureViewIcon());
        splitView_button.setSelectedIcon(icons.getMergeStructureViewIcon());
        splitView_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                splitView_button_actionPerformed(e);
            }
        });
        zoomToFile_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                zoomToFile_button_actionPerformed(e);
            }
        });
        zoomToFile_button.setIcon(icons.getZoomStructureToFileModeIcon());
        zoomToFile_button.setSelectedIcon(icons.getZoomStructureToGlobalModeIcon());
        zoomToFile_button.setBorder(BorderFactory.createRaisedBevelBorder());
        zoomToFile_button.setMaximumSize(new Dimension(24, 24));
        zoomToFile_button.setPreferredSize(new Dimension(20, 20));
        zoomToFile_button.setToolTipText("Toggle file-view mode");
        zoomToFile_button.setFont(new java.awt.Font("Dialog", 0, 11));
        joinpointProbe_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                joinpointProbe_button_actionPerformed(e);
            }
        });
        joinpointProbe_button.setIcon(icons.getRelationAdviceForwardIcon());
        joinpointProbe_button.setToolTipText("Create joinpoint probe");
        joinpointProbe_button.setPreferredSize(new Dimension(20, 20));
        joinpointProbe_button.setMinimumSize(new Dimension(20, 20));
        joinpointProbe_button.setBorder(default_border);
        joinpointProbe_button.setMaximumSize(new Dimension(24, 20));
        this.add(toolBar_panel,  BorderLayout.NORTH);
        toolBar_panel.add(mainToolBar_panel, BorderLayout.NORTH);
        mainToolBar_panel.add(config_toolBar, BorderLayout.CENTER);
        config_toolBar.add(configs_comboBox, null);
        mainToolBar_panel.add(nav_toolBar,  BorderLayout.EAST);
        nav_toolBar.add(splitView_button, null);
        nav_toolBar.add(zoomToFile_button, null);
        nav_toolBar.add(joinpointProbe_button, null);
        nav_toolBar.add(back_button, null);
        nav_toolBar.add(forward_button, null);
        mainToolBar_panel.add(command_toolBar,  BorderLayout.WEST);
        this.add(views_splitPane,  BorderLayout.CENTER);
        views_splitPane.setDividerLocation(400);
    }

    void forward_button_actionPerformed(ActionEvent e) {
        TopManager.BROWSER_MANAGER.navigateForwardAction();
    }
    void back_button_actionPerformed(ActionEvent e) {
        TopManager.BROWSER_MANAGER.navigateBackAction();
    }

    void splitView_button_actionPerformed(ActionEvent e) {
        TopManager.BROWSER_MANAGER.setSplitViewMode(!slaveViewVisible);
    }

    static class ConfigsCellRenderer extends JLabel implements ListCellRenderer {
        public ConfigsCellRenderer() {
            setOpaque(true);
        }

        public Component getListCellRendererComponent(JList list,
                                                        Object value,
                                                        int index,
                                                        boolean isSelected,
                                                        boolean cellHasFocus) {
            if (value == null) return this;

            java.io.File file = new File(value.toString());
            setText(file.getName());
            setBackground(isSelected ? Color.gray : Color.lightGray);
//            setForeground(isSelected ? Color.lightGray : Color.gray);
            return this;
        }
    }

    void configDesigner_button_mouseClicked(MouseEvent e) {

    }
    void configDesigner_button_mousePressed(MouseEvent e) {

    }
    void configDesigner_button_mouseReleased(MouseEvent e) {

    }
    void configDesigner_button_mouseEntered(MouseEvent e) {

    }
    void configDesigner_button_mouseExited(MouseEvent e) {

    }
    void configDesigner_button_actionPerformed(ActionEvent e) {

    }
    void viewManager_button_mouseClicked(MouseEvent e) {

    }
    void viewManager_button_mousePressed(MouseEvent e) {

    }
    void viewManager_button_mouseReleased(MouseEvent e) {

    }
    void viewManager_button_mouseEntered(MouseEvent e) {

    }
    void viewManager_button_mouseExited(MouseEvent e) {

    }
    void viewManager_button_actionPerformed(ActionEvent e) {

    }

    void zoomToFile_button_actionPerformed(ActionEvent e) {
        TopManager.BROWSER_MANAGER.setGlobalMode(!TopManager.BROWSER_MANAGER.isGlobalMode());
        TopManager.BROWSER_MANAGER.updateView();
    }

    void configs_comboBox_actionPerformed(ActionEvent e) {
        if (configs_comboBox.getSelectedItem() != null) {
            if (!configs_comboBox.getSelectedItem().toString().equals(lastSelectedConfig)) {
                TopManager.BROWSER_MANAGER.readStructureView();
                TopManager.BROWSER_MANAGER.updateView();
                lastSelectedConfig = configs_comboBox.getSelectedItem().toString();
            }
        }
    }

    private void joinpointProbe_button_actionPerformed(ActionEvent e) {
        //joinpointProbeWizard();
    }
}