
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.ajde.views;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import org.aspectj.ajde.*;
import org.aspectj.asm.*;
import org.aspectj.ajde.ui.Icons;

/**
 * Represents the configuration of a structure view of the system, rendered
 * by the <CODE>StructureTreeManager</CODE>.
 *
 * @author Mik Kersten
 */
public class StructureView extends JPanel {

    private StructureTreeManager treeManager;
    private int depthSliderVal = 0;
    private JComboBox view_comboBox = null;
    private static final String VIEW_SUFFIX = " view";

    BorderLayout borderLayout1 = new BorderLayout();
    JScrollPane tree_ScrollPane = new JScrollPane();
    JPanel jPanel1 = new JPanel();
    JToolBar view_toolBar = new JToolBar();
    JSlider depth_slider = new JSlider();

    public StructureView(Icons icons, Map viewProperties, String visibleViewName) {
        try {
            Object[] viewsArray = viewProperties.values().toArray();
            String[] views = new String[viewsArray.length];
            int selectedIndex = 0;
            for (int i = 0; i < viewsArray.length; i++) {
                views[i] = viewsArray[i].toString() + VIEW_SUFFIX;
                if (viewsArray[i].toString().equals(visibleViewName)) {
                    selectedIndex = i;
                }
            }
            view_comboBox = new JComboBox(views);
            view_comboBox.setSelectedIndex(selectedIndex);
            ViewProperties visibleView = (ViewProperties)viewProperties.get(visibleViewName);
            treeManager = new StructureTreeManager(icons, visibleView);
            jbInit();
            initDepthSlider();
            tree_ScrollPane.getViewport().add(treeManager.getStructureTree(), null);
        }
        catch(Exception e) {
          e.printStackTrace();
        }
    }

    public void highlightNode(ProgramElementNode node) {
        treeManager.navigationAction(node, true, true);
    }

    public int getTreeDepth() {
        return depth_slider.getValue();
    }

    public void scrollToHighlightedNode() {

    }

    void updateTree() {
        treeManager.updateTree(depthSliderVal, view_comboBox.getSelectedItem().toString());
    }

    void displayProbe() {
        treeManager.displayProbe();
    }

    void updateTree(String filePath) {
        treeManager.updateTree(filePath, depthSliderVal, view_comboBox.getSelectedItem().toString());
    }

    ProgramElementNode getRootProgramElementNode() {
        return treeManager.getRootProgramElementNode();
    }

    private void initDepthSlider() {
        depth_slider.setMinimum(0);
        depth_slider.setMaximum(9);
        depth_slider.setMinorTickSpacing(1);
        depth_slider.setValue(9);
        depth_slider.setSnapToTicks(true);
        depth_slider.setPaintTrack(true);
        depth_slider.setPaintTicks(true);
        this.depth_slider.addChangeListener(
            new ChangeListener() {
                public void stateChanged(ChangeEvent e) {
                    depthSliderVal = depth_slider.getValue();
                    TopManager.BROWSER_MANAGER.updateView();
                }
            });
        depthSliderVal = depth_slider.getValue();
    }

    private void view_comboBox_actionPerformed(ActionEvent e) {
        String item =  view_comboBox.getSelectedItem().toString();
        item = item.substring(0, item.indexOf(VIEW_SUFFIX));
        treeManager.setActiveView(TopManager.BROWSER_MANAGER.getViewProperties(item));
        updateTree();
    }

    private void jbInit() throws Exception {
        this.setLayout(borderLayout1);
        view_comboBox.setPreferredSize(new Dimension(200, 20));
        view_comboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                view_comboBox_actionPerformed(e);
            }
        });
        view_comboBox.setMinimumSize(new Dimension(40, 20));
        view_comboBox.setFont(new java.awt.Font("SansSerif", 0, 11));
        depth_slider.setMaximumSize(new Dimension(32767, 25));
        depth_slider.setToolTipText("");
        depth_slider.setMinimumSize(new Dimension(30, 20));
        depth_slider.setBorder(null);
        depth_slider.setPreferredSize(new Dimension(30, 25));
        depth_slider.setMaximum(3);
        depth_slider.setPaintTicks(true);
        depth_slider.setValue(1);
        depth_slider.setPaintLabels(true);
        view_toolBar.setFloatable(false);
        this.add(tree_ScrollPane, BorderLayout.CENTER);
        this.add(view_toolBar, BorderLayout.NORTH);
        view_toolBar.add(view_comboBox, null);
        view_toolBar.add(depth_slider, null);
    }
}