
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */


package org.aspectj.asm;

import java.util.*;
import java.io.Serializable;
import org.aspectj.compiler.base.ast.ASTObject;

/**
 * @author Mik Kersten
 */
public class Relation implements Serializable {

    private String forwardNavigationName;
    private String backNavigationName;
    private String associationName;
    private boolean symmetrical;
    private boolean transitive;

    public Relation(String forwardNavigationName,
                    String backNavigationName,
                    String associationName,
                    boolean symmetrical,
                    boolean transitive) {
        this.forwardNavigationName = forwardNavigationName;
        this.backNavigationName = backNavigationName;
        this.associationName = associationName;
        this.symmetrical = symmetrical;
        this.transitive = transitive;
    }

    /**
     * Constructor for asymetrical relations.
     */
    public Relation(String forwardNavigationName,
                    String associationName,
                    boolean transitive) {
        this(forwardNavigationName, "<no back navigation name>", associationName, false, transitive);
    }

    public String getForwardNavigationName() {
        return forwardNavigationName;
    }

    public String getBackNavigationName() {
        return backNavigationName;
    }

    public String getAssociationName() {
        return associationName;
    }

    public boolean isSymmetrical() {
        return symmetrical;
    }

    public boolean isTransitive() {
        return transitive;
    }

    public String toString() {
        if (symmetrical) {
            return forwardNavigationName + " / " + backNavigationName;
        } else {
            return forwardNavigationName;
        }
    }
}