/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base;

import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.base.CompilerObject;

import java.util.*;

public abstract class ParallelCompilerPass extends CompilerObject implements CompilationUnitPass {
    public ParallelCompilerPass(JavaCompiler compiler) {
        super(compiler);
    }

    public abstract String getDisplayName();
    
    public double getWorkEstimate() { return 1.0; }
    
    public int getNumThreads() { return getOptions().procs; }

    private List cuList = new LinkedList();

    private int finished = 0;

    public synchronized CompilationUnit getWorkCu() {
        if (cuList.size() == 0) return null;
        return (CompilationUnit)cuList.remove(0);
    }

    public synchronized void setFinished() {
        finished += 1;
    }

    public void transformWorld() {
        if (getDisplayName() != null) getCompiler().beginSection(getDisplayName());
        for (Iterator i = getWorld().getCompilationUnits().iterator(); i.hasNext(); ) {
            CompilationUnit cu = (CompilationUnit)i.next();
            cuList.add(cu);
        }

        runThreads();
    }

    class PassThread extends Thread {
        public void run() {
            CompilationUnit cu = getWorkCu();
            while (cu != null) {
                transform(cu);
                getCompiler().completedFile(ParallelCompilerPass.this, cu);
                cu = getWorkCu();
            }
            setFinished();
        }
    }

    void runThreads() {
        for (int i = 0; i < getNumThreads(); i++) {
            new PassThread().start();
        }

        Thread myThread = Thread.currentThread();

        while (finished < getNumThreads()) {
            try {
                myThread.sleep(100);
            } catch (InterruptedException ie) {
                break;
            }
        }
    }

    
    public void transform(CompilationUnit cu) {
        for (Iterator j = cu.getDefinedTypes().iterator(); j.hasNext(); ) {
            TypeDec td = (TypeDec)j.next();
            transform(td);
        }
    }
    
    public void transform(TypeDec td) {
    }
}

