/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base;

import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.base.CompilerObject;

import java.util.*;

public abstract class WalkerPass extends Walker implements CompilationUnitPass {
    public WalkerPass(JavaCompiler compiler) {
        super(compiler);
    }

    public abstract String getDisplayName();
    
    public double getWorkEstimate() { return 1.0; }
    
    public void transformWorld() {
        if (getDisplayName() != null) getCompiler().beginSection(getDisplayName());
        for (Iterator i = getWorld().getCompilationUnits().iterator(); i.hasNext(); ) {
            CompilationUnit cu = (CompilationUnit)i.next();
            transform(cu);
            getCompiler().completedFile(this, cu);
        }
    }
    
    public void transform(CompilationUnit cu) {
        process(cu);
    }
}

