/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.compiler.base.FlowCheckerPass;

import org.aspectj.compiler.base.bcg.FieldBuilder;
import org.aspectj.compiler.base.bcg.CodeBuilder;
import org.aspectj.compiler.base.bcg.Label;

/**
  * @grammar booleanLiteral
  * @property boolean booleanValue
  */
public class BooleanLiteralExpr extends LiteralExpr {
    public boolean isConstantTrue() { return getBooleanValue() == true; }
    public boolean isConstantFalse() { return getBooleanValue() == false; }

    public String getStringValue() {
        return value;
    }

    public BooleanLiteralExpr(SourceLocation source, boolean _booleanValue) {
        this(source, source.getCompiler().getTypeManager().booleanType, "" + _booleanValue, _booleanValue);
    }


    // ------------------------------
    // INTRO from FlowCheckerPass

    public void walkFlow(FlowCheckerPass w) {
        if (getBooleanValue()) {
            w.setVars(w.getVars(), w.getAllVars());
        } else {
            w.setVars(w.getAllVars(), w.getVars());
        }
    }

    // ------------------------------
    // bcg
    protected void cgTest(CodeBuilder cb, Label tdest, Label fdest) {
        cb.emitJump(getBooleanValue() ? tdest : fdest);
    }
    protected void cgValue(CodeBuilder cb) {
        cb.emitIntConstant(booleanValue ? 1 : 0);
    }
    public void addConstant(FieldBuilder fb) {
        fb.setConstantValue(booleanValue);
    }

    //BEGIN: Generated from @child and @property
    protected boolean booleanValue;
    public boolean getBooleanValue() { return booleanValue; }
    public void setBooleanValue(boolean _booleanValue) { booleanValue = _booleanValue; }

    public BooleanLiteralExpr(SourceLocation location, Type _type, String _value, boolean _booleanValue) {
        super(location, _type, _value);
        setBooleanValue(_booleanValue);
    }
    protected BooleanLiteralExpr(SourceLocation source) {
        super(source);
    }

    public ASTObject copyWalk(CopyWalker walker) {
        BooleanLiteralExpr ret = new BooleanLiteralExpr(getSourceLocation());
        ret.preCopy(walker, this);
        ret.type = type;
        ret.value = value;
        ret.booleanValue = booleanValue;
        return ret;
    }


    public String getDefaultDisplayName() {
        return "BooleanLiteralExpr(type: "+type+", "+"value: "+value+", "+"booleanValue: "+booleanValue+")";
    }

    //END: Generated from @child and @property
}
