/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.*;
import java.io.IOException;

import org.aspectj.compiler.base.bcg.CodeBuilder;
import org.aspectj.compiler.base.bcg.Label;

public final class CharType extends IntishType {

    public String getName() { return "char"; }
    public int getTypeIndex() { return Type.CHAR; }
    public Type getRefType() { return getTypeManager().getType("java.lang", "Character"); }

    public CharType(JavaCompiler compiler) {
        super(compiler);
    }

    // vm internals
    public String getDescriptor() { return "C"; }

    // folding
    LiteralExpr foldCast(LiteralExpr rand) {
        return new IntLiteralExpr(rand.getSourceLocation(), this, (char)rand.getIntValue());
    }

    // code gen
    void emitCastFromInt(CodeBuilder cb)    { cb.emitI2C(); }
    void emitCastFromFloat(CodeBuilder cb)  { cb.emitF2I(); cb.emitI2C(); }
    void emitCastFromLong(CodeBuilder cb)   { cb.emitL2I(); cb.emitI2C(); }
    void emitCastFromDouble(CodeBuilder cb) { cb.emitD2I(); cb.emitI2C(); }

    void emitAload(CodeBuilder cb) { cb.emitCALOAD(); }
    void emitAstore(CodeBuilder cb) { cb.emitCASTORE(); }

}
