/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.*;
import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.compiler.base.CodeWriter;
import java.io.IOException;

/**
  * @grammar { stmts* }
  * @property boolean parsed
  */

public class CodeBody extends BlockStmt {

    public CodeDec getEnclosingCodeDec() {
        if (getParent() instanceof CodeDec) return (CodeDec)getParent();
        return super.getEnclosingCodeDec();
    }
    public Dec getEnclosingDec() {
        if (getParent() instanceof CodeDec) return (Dec)getParent();
        return super.getEnclosingDec();
    }

    public void cleanup() {
        //setParsed(false);
        super.cleanup();
    }

    //INTRO from ScopePass
    public void walkScope(ScopeWalker walker) {
        if (walker.walkBodies()) super.walkScope(walker);
    }


    // ------------------------------
    // Intro: ByteCodeCleanupPass

    // We're overriding this because CodeDecs must have codeBodies
    // inside, so we should not replace empty CodeBodies with emptyStmts.
    public ASTObject postCleanup(ByteCodeCleanupPass walker) {
        makeTempsExplicit();
        return this;
    }

    // ------------------------------
    // utility constructors

    public CodeBody(SourceLocation location, Stmts _stmts, boolean _parsed) {
        this(location, null, _stmts, _parsed);
    }

    //BEGIN: Generated from @child and @property
    protected boolean parsed;
    public boolean getParsed() { return parsed; }
    public void setParsed(boolean _parsed) { parsed = _parsed; }

    public CodeBody(SourceLocation location, Stmts _tmpStmts, Stmts _stmts, boolean _parsed) {
        super(location, _tmpStmts, _stmts);
        setParsed(_parsed);
    }
    protected CodeBody(SourceLocation source) {
        super(source);
    }

    public ASTObject copyWalk(CopyWalker walker) {
        CodeBody ret = new CodeBody(getSourceLocation());
        ret.preCopy(walker, this);
        if (tmpStmts != null) ret.setTmpStmts( (Stmts)walker.process(tmpStmts) );
        if (stmts != null) ret.setStmts( (Stmts)walker.process(stmts) );
        ret.parsed = parsed;
        return ret;
    }


    public String getDefaultDisplayName() {
        return "CodeBody(parsed: "+parsed+")";
    }

    //END: Generated from @child and @property
}
