/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.CodeWriter;
import java.io.IOException;

import java.util.*;

public class Comment {
    private List lines = new LinkedList();

    protected Comment nextComment = null;
    public void addComment(Comment newComment) {
        if (nextComment == null) {
            nextComment = newComment;
        } else {
            nextComment.addComment(newComment);
        }
    }

    public String getFormalComment() {
        if (nextComment == null) return "";
        return nextComment.getFormalComment();
    }

    public Comment getNextComment() {
        return nextComment;
    }

    public void addLine(String line) { lines.add(line); }
    public String getLine(int index) { return (String)lines.get(index); }
    public int size() { return lines.size(); }

    public void unparse(CodeWriter writer) throws IOException {
        writer.writeln("/*");
        for(int i=0; i<size(); i++) {
            writer.write(" * ");
            writer.write(getLine(i));
        }
        writer.writeln(" */");
    }
}
