/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;
import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.compiler.base.CompilerObject;
import org.aspectj.compiler.base.InternalCompilerError;

import java.util.*;

/**
  */
public class CopyWalker extends MovingWalker {

    public static ASTObject rawCopy(ASTObject object1, Type toType) {
        return new CopyWalker(object1.getCompiler(), null, toType) {
                public ASTObject process(ASTObject object) {
                    ASTObject copy = object.copyWalk(this);
                    return copy.postCopy(this, object);
                }
            }.process(object1);
    }


    public static ASTObject copy(ASTObject object) {
        return new CopyWalker(object.getCompiler()).process(object);
    }

    public static ASTObject copy(ASTObject object, Map map) {
        CopyWalker cw = new CopyWalker(object.getCompiler());
        cw.addMappings(map);
        return cw.process(object);
    }

    public CopyWalker(JavaCompiler compiler) {
        super(compiler);
    }
    public CopyWalker(JavaCompiler compiler, Type fromType, Type toType) {
        super(compiler, fromType, toType);
    }

    public ASTObject cp(ASTObject object) {
        if (object == null) return null;
        return process(object);
    }

    public ASTObject process(ASTObject object) {
        object.preMove(this);
        ASTObject copy = object.copyWalk(this);
        copy = copy.postCopy(this, object);
        return copy.postMove(this);
    }
}
