/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.JavaCompiler;

import org.aspectj.compiler.base.bcg.FieldBuilder;
import org.aspectj.compiler.base.bcg.CodeBuilder;
import org.aspectj.compiler.base.bcg.Label;

/**
  * @grammar floatLiteral
  * @property float floatValue
  */
public class FloatLiteralExpr extends NumericLiteralExpr {

    public int    getIntValue()    { return (int)floatValue; }
    public long   getLongValue()   { return (long)floatValue; }
    public double getDoubleValue() { return (double)floatValue; }
    public String getStringValue() { return "" + floatValue; }

    public FloatLiteralExpr(SourceLocation source, float _floatValue) {
        this(source, source.getCompiler().getTypeManager().floatType, createUsefulString(_floatValue), _floatValue);
    }

    private static String createUsefulString(float f) {
        if (Float.isNaN(f)) {
            return "(0.0f / 0.0f)";
        } else if (Float.isInfinite(f)) {
            return f > 0 ? "(1.0f / 0.0f)" : "(-1.0f / 0.0f)";
        } else {
            return f + "f";
        }
    }

    // ------------------------------
    // bcg
    protected void cgValue(CodeBuilder cb) {
        cb.emitFloatConstant(floatValue);
    }
    public void addConstant(FieldBuilder fb) {
        fb.setConstantValue(floatValue);
    }
    public boolean isConstantZero() { return floatValue == 0.0f; }

    //BEGIN: Generated from @child and @property
    protected float floatValue;
    public float getFloatValue() { return floatValue; }
    public void setFloatValue(float _floatValue) { floatValue = _floatValue; }

    public FloatLiteralExpr(SourceLocation location, Type _type, String _value, float _floatValue) {
        super(location, _type, _value);
        setFloatValue(_floatValue);
    }
    protected FloatLiteralExpr(SourceLocation source) {
        super(source);
    }

    public ASTObject copyWalk(CopyWalker walker) {
        FloatLiteralExpr ret = new FloatLiteralExpr(getSourceLocation());
        ret.preCopy(walker, this);
        ret.type = type;
        ret.value = value;
        ret.floatValue = floatValue;
        return ret;
    }


    public String getDefaultDisplayName() {
        return "FloatLiteralExpr(type: "+type+", "+"value: "+value+", "+"floatValue: "+floatValue+")";
    }

    //END: Generated from @child and @property
}
