/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.*;
import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.compiler.base.CodeWriter;

/**
  * @grammar modifiers typeD id (no initializer)
  */

public class FormalDec extends VarDec {

    public FormalDec(SourceLocation location, TypeD typeD, String id) {
        this(location,new Modifiers(location,0), typeD, id, null);
    }

    //XXX only used by pointcut and advice to find unbound formals
    public boolean isBound = false;

    public String toString() {
        return "FormalDec("+getTypeD().getString()+" "+getId()+")";
    }

    public boolean isField() { return false; }

    public void unparse(CodeWriter writer) {
        writeModifiers(writer);
        writer.write(typeD);
        writer.requiredSpace();
        writer.write(id);
    }

    // ------------------------------
    // INTRO from FlowCheckerPass

    public void walkFlow(FlowCheckerPass w) {
        w.setVars(w.getVars().addAssigned(this));
    }

    //BEGIN: Generated from @child and @property

    public FormalDec(SourceLocation location, Modifiers _modifiers, TypeD _typeD, String _id, Expr _initializer) {
        super(location, _modifiers, _typeD, _id, _initializer);

    }
    protected FormalDec(SourceLocation source) {
        super(source);
    }

    public ASTObject copyWalk(CopyWalker walker) {
        FormalDec ret = new FormalDec(getSourceLocation());
        ret.preCopy(walker, this);
        if (modifiers != null) ret.setModifiers( (Modifiers)walker.process(modifiers) );
        if (typeD != null) ret.setTypeD( (TypeD)walker.process(typeD) );
        ret.id = id;
        if (initializer != null) ret.setInitializer( (Expr)walker.process(initializer) );
        return ret;
    }


    public String getDefaultDisplayName() {
        return "FormalDec(id: "+id+")";
    }

    //END: Generated from @child and @property
}
