/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.*;
import org.aspectj.compiler.base.cst.*;

/**
  * @grammar import name(.*)?
  * @property Name typeName
  * @property boolean star
  */

public class Import extends ASTObject {

    public void checkSpec() {
        if (!getStar()) {
            getType();
        } else {
            //!!! validate that the package exists
        }
    }

    public boolean capturesId(String id) {
        return typeName.getId().equals(id);
    }

    public Scope makeTypeNameScope() {
        return new Scope(getCompiler(), null) {
                public Type findType(String name, ASTObject fromWhere) {
                    return this.getTypeManager().findType(null, name);
                }
                public Expr bindUnqualifiedName(String name, ASTObject fromWhere) {
                    return null;
                }
            };
    }

    private Type myType = null;
    public Type getType() {
        if (myType == null) {
            myType = typeName.resolveType(makeTypeNameScope());

            if (myType == null) {
                showError("type not found");
            } else if (getOptions().strict) {
                if (myType.getPackageName() == null) {
                    showError("import from default package not allowed");
                }
            }
        }

        return myType;
    }

    public String getName() {
        return typeName.toString();
    }

    public void unparse(CodeWriter writer) {
        writer.writeKeyword("import");
        writer.requiredSpace();
        writer.write(typeName.toString());
        if (star) writer.write(".*");
        writer.closeStmt();
        writer.newLine();
    }

    //BEGIN: Generated from @child and @property
    protected Name typeName;
    public Name getTypeName() { return typeName; }
    public void setTypeName(Name _typeName) { typeName = _typeName; }

    protected boolean star;
    public boolean getStar() { return star; }
    public void setStar(boolean _star) { star = _star; }

    public Import(SourceLocation location, Name _typeName, boolean _star) {
        super(location);
        setTypeName(_typeName);
        setStar(_star);
    }
    protected Import(SourceLocation source) {
        super(source);
    }

    public ASTObject copyWalk(CopyWalker walker) {
        Import ret = new Import(getSourceLocation());
        ret.preCopy(walker, this);
        ret.typeName = typeName;
        ret.star = star;
        return ret;
    }


    public String getDefaultDisplayName() {
        return "Import(typeName: "+typeName+", "+"star: "+star+")";
    }

    //END: Generated from @child and @property
}
