/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.compiler.base.CodeWriter;
import java.io.IOException;

import java.util.*;

import org.aspectj.compiler.base.bcg.CodeBuilder;
import org.aspectj.compiler.base.bcg.Label;

/**
 * @grammar - rand1
 *
 */
public class MinusOpExpr extends UnopExpr {


    protected LiteralExpr halfFold(Type type, LiteralExpr lit) {
        return type.foldMinusOp(lit);
    }

    protected Type discoverType() {
        Expr rand1 = getRand1();
        Type ty1 = rand1.getType();

        if (ty1.isNumeric()) {
            return getTypeManager().unaryNumericPromotion(ty1);
        } else {
            showOperatorTypeError(ty1);
            return getTypeManager().anyType;
        }
    }

    protected Type getLiftType() {
        return getType();
    }

    // ------------------------------
    // bcg
    public void cgValue(CodeBuilder cb) {
        Type liftType = getLiftType();
        getRand1().cgValue(cb, liftType);
        liftType.emitNeg(cb);
    }

    //BEGIN: Generated from @child and @property

    public MinusOpExpr(SourceLocation location, String _op, Expr _rand1) {
        super(location, _op, _rand1);

    }
    protected MinusOpExpr(SourceLocation source) {
        super(source);
    }

    public ASTObject copyWalk(CopyWalker walker) {
        MinusOpExpr ret = new MinusOpExpr(getSourceLocation());
        ret.preCopy(walker, this);
        ret.op = op;
        if (rand1 != null) ret.setRand1( (Expr)walker.process(rand1) );
        return ret;
    }


    public String getDefaultDisplayName() {
        return "MinusOpExpr(op: "+op+")";
    }

    //END: Generated from @child and @property
}
