/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.*;
import org.aspectj.compiler.base.cst.*;

import java.util.*;

/**
  */
public class MovingWalker extends Walker {
    protected Map remapNodes = new HashMap();
    protected Stack lexicalTypes = new Stack();
    public Type toType;
    public boolean isStatic;

    //XXX this should probably go away
    public MovingWalker(JavaCompiler compiler) {
        super(compiler);
    }

    public MovingWalker(JavaCompiler compiler, Type fromType, Type toType) {
        super(compiler);
        this.toType = toType;
        pushLexicalType(fromType);
    }

    public void addMappings(Map map) {
        remapNodes.putAll(map);
    }

    public boolean hasToType() { return toType != null; }

    public Type getToType() { return toType; }

    public void addMapping(Object oldObject, Object newObject) {
        remapNodes.put(oldObject, newObject);
    }

    public boolean inTopLexicalType() {
        return lexicalTypes.size() == 1;
    }

    public boolean hasLexicalType() {
        return lexicalTypes.size() > 0;
    }

    public Type getLexicalType() {
        return (Type)lexicalTypes.peek();
    }

    public Type getTopLexicalType() {
        return (Type)lexicalTypes.get(0);
    }

    public void pushLexicalType(Type t) {
        lexicalTypes.push(t);
    }

    public void popLexicalType() {
        lexicalTypes.pop();
    }

    public Expr moveThisExpr(ThisExpr thisExpr, Type thisType) {
        //System.out.println(this + ":  " + thisType + ", " + lexicalTypes);

        if (hasLexicalType() && thisType == getTopLexicalType()) {
            Expr ret = makeThisExpr();
            //System.out.println("    making: " + ret);
            if (ret != null) {
                return ret;
            }
        }
        return thisExpr;
    }
    
    /** 
     * returns what a varExpr should remap to.
     */
	public Expr moveVarExpr(VarExpr var) {
        VarDec ret = (VarDec)remapNodes.get(var.getVarDec());
        if (ret != null) {
            if (ret instanceof FieldDec) {
                return getAST().makeGet((FieldDec)ret);
            } else {
                var.setVarDec(ret);
                var.setType(null);
                var.getType();
                return var;
            }
        } else {
            return handleFreeVar(var);
        } 
    }
    
    /** returns what a free varExpr should remap to.
     */
    protected Expr handleFreeVar(VarExpr var) {
    	return var;
    }

    protected Expr makeThisExpr() { return null; }

    public Type moveType(Type type) {
        SemanticObject ret = (SemanticObject)remapNodes.get(type);
        if (ret != null && ret instanceof Type) return (Type)ret;
        return type;
    }

    public void moveLink(SOLink link) {
        SemanticObject ret = (SemanticObject)remapNodes.get(link.getTarget());
        if (ret != null) link.setTarget(ret);
    }

    public Expr moveLinkExpr(SOLink link) {
        SemanticObject ret = (SemanticObject)remapNodes.get(link.getTarget());
        if (ret != null) link.setTarget(ret);
        return (Expr)link;
    }

    public ASTObject process(ASTObject object) {
        object.preMove(this);
        object.walk(this);
        return object.postMove(this);
    }
}
