/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import java.util.*;

import org.aspectj.compiler.base.bcg.CodeBuilder;
import org.aspectj.compiler.base.*;
import java.io.IOException;

//ASPECT use introduction from crosscuts package
import org.aspectj.compiler.crosscuts.ast.*;
import org.aspectj.compiler.crosscuts.joinpoints.*;


/**
 * This Type should never appear in a correct program
 * It is used to represent types that are not found in order to provide
 * better error messages as processing continues
 */
public class NotFoundType extends NameType {
    public NotFoundType(TypeDec notFoundTypeDec) {
        super(notFoundTypeDec);
    }

    public boolean isCoercableTo(Type other) {
        return true;
    }

    public boolean dominates(Type other) {
        return false;
    }


    public Expr getClassExpr() {
        return getAST().makeNull();
    }

    public Expr makeObject(Expr expr) { return expr; }

    public Expr fromObject(Expr expr) {
        return getAST().makeCast(this, expr);
    }

    public Expr getNullExpr() {
        return getAST().makeNull();
    }

    public Type getRefType() { return this; }

    public Type getOutermostType() {
        return this;
    }

    synchronized public void ensureBuiltTypeGraph() { return; }
    synchronized public void ensureFinishedSignature() { return; }

    public PointcutSO getPointcut(String id, ASTObject fromWhere, boolean showError) {
        return null;
    }

    public Field getField(String id, ASTObject fromWhere, boolean showError) {
        return null;
    }

    public Type getInnerType(String id, ASTObject fromWhere, boolean showError) {
        return getTypeManager().TYPE_NOT_FOUND;
    }

    public Method getMethod(String id, ASTObject fromWhere, Exprs params, boolean showError) {
        return null;
    }

    // currently only used by ReceptionsPointcut and by TypeScope.findMethodLookupType
    public boolean hasMethodNamed(String id) {
        return false;
    }

    public Constructor getConstructor(ASTObject fromWhere, Exprs params, boolean showError) {
        return null;
    }

    public SemanticObject findMatchingSemanticObject(SemanticObject o) {
        return null;
    }

    public NameType getSuperClassType() {
        return getTypeManager().TYPE_NOT_FOUND;
    }

    public Type getDeclaringType() {
        return getTypeManager().TYPE_NOT_FOUND;
    }

    //private String prettyString = null;
    public String getPrettyString() {
        return "*";
    }

    private String makeLegalString() {
        return "*";
    }

    public String getSourceString() {
        return "*";
    }

    public String getExtendedId() {
        return "*";
    }

    public String getId() {
        return "*";
    }

    public String getExternalName() {
        return "*";
    }

    public String getClassName() {
        return "*";
    }

    public String getPackageName() {
        return null;
    }

    //!!! name should change to getPrettyClassName
    public String toShortString() {
        return "*";
    }

    public String toString() {
        return "NotFoundType()";
    }

    public boolean isEquivalent(Type other) {
        return true;
    }

    public boolean isMissing() { return true; }

    public boolean isInterface() { return false; }
    public boolean isClass() { return true; }
    public boolean isAspect() { return false; }

    public boolean isInnerType() { return false; }

    public boolean isAnonymous() { return false; }

    public boolean isAbstract() { return false; }
    public boolean isFinal() { return false; }

    public Type getOutermostLexicalType() { return getTypeManager().TYPE_NOT_FOUND; }

    // ------------------------------
    // Dealing with non-package-level types
    public boolean isInner() {
        return false;
    }
    public NameType getEnclosingInstanceType() {
        return getTypeManager().TYPE_NOT_FOUND;
    }

    public boolean isLocal() {
        return false;
    }
    public CodeDec getEnclosingCodeDec() {
        return null;
    }

    public boolean isPackageMember() {
        return true;
    }
    public NameType getEnclosingType() {
        return getTypeManager().TYPE_NOT_FOUND;
    }
}
