/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.compiler.base.CodeWriter;
import java.io.IOException;
import org.aspectj.compiler.base.bcg.CodeBuilder;
import org.aspectj.compiler.base.bcg.Label;

import java.util.*;

/**
 * @grammar rand1 <op> rand2
 */
public class NumericTestOpExpr extends BinopExpr {

    protected Type discoverType() {
        Expr rand1 = getRand1();
        Expr rand2 = getRand2();
        Type ty1 = rand1.getType();
        Type ty2 = rand2.getType();

        if (!(ty1 instanceof NumericType && ty2 instanceof NumericType)) {
            showOperatorTypeError(op, ty1, ty2);
        }
        return getTypeManager().booleanType;
    }

    protected Type getLiftType() {
        Expr rand1 = getRand1();
        Expr rand2 = getRand2();
        Type ty1 = rand1.getType();
        Type ty2 = rand2.getType();

        if (ty1 instanceof NumericType && ty2 instanceof NumericType) {
            return getTypeManager().binaryNumericPromotion(ty1, ty2);
        } else {
            showOperatorTypeError(op, ty1, ty2);
            return getTypeManager().anyType;
        }
    }

    protected LiteralExpr halfFold(Type type, LiteralExpr lit1, LiteralExpr lit2) {
        return type.foldNumericTestOp(getOp(), lit1, lit2);
    }

    // ------------------------------
    // bcg

    protected void cgTest(CodeBuilder cb, Label t, Label f) {
        Type liftType = getLiftType();
        if (liftType.hasFastNumericTestOp()) {
            liftType.emitFastNumericTestOp(cb, this, t, f);
        } else {
            getRand1().cgValue(cb, liftType);
            getRand2().cgValue(cb, liftType);
            liftType.emitNumericCompare(cb, getOp(), t, f);
        }
    }

    protected void cgOp(CodeBuilder cb, Type ty) {
        throw new RuntimeException("Attempt to generate code for effect op "
                                   + getOp());
    }

    //BEGIN: Generated from @child and @property

    public NumericTestOpExpr(SourceLocation location, Expr _rand1, String _op, Expr _rand2) {
        super(location, _rand1, _op, _rand2);

    }
    protected NumericTestOpExpr(SourceLocation source) {
        super(source);
    }

    public ASTObject copyWalk(CopyWalker walker) {
        NumericTestOpExpr ret = new NumericTestOpExpr(getSourceLocation());
        ret.preCopy(walker, this);
        if (rand1 != null) ret.setRand1( (Expr)walker.process(rand1) );
        ret.op = op;
        if (rand2 != null) ret.setRand2( (Expr)walker.process(rand2) );
        return ret;
    }


    public String getDefaultDisplayName() {
        return "NumericTestOpExpr(op: "+op+")";
    }

    //END: Generated from @child and @property
}
