/* -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.*;

import org.aspectj.compiler.base.bcg.CodeBuilder;

/**
 * @grammar typename.this
 * @child TypeD typeD
 */


//XXX Why does an AST need this subtype of ThisExpr?
public class QualifiedThisExpr extends ThisExpr {
    public Type discoverType() {
        return typeD.getType();
    }

    // INTRO from ASTFixerPass
    public ASTObject postFixAST(final ASTFixerPass fixer) {
        //??? This seems like an unavoidable issue with Java's rules for anonymous
        //??? inners and name binding
//        if (getType().getId().equals("ANONYMOUS")) {
//            return null;
//        }
        return this;
    }


    public void unparse(CodeWriter writer) {
        if (getBytecodeType() == getType()) {
            writer.write("this");
        } else {
            writer.write(getType().getId());
            writer.write(".this");
        }
    }

    // ------------------------------
    // INTRO from MemberClassMunger

    public ASTObject walkMemberMunger(MemberClassMunger w) {
        final AST ast = getAST();
        NameType currentType = w.currentType();
        NameType targetType = (NameType) getTypeD().getType();
        if (currentType == targetType) {
            return ast.makeThis(currentType);
        } else {
            CodeDec dec = getEnclosingCodeDec();
            if (dec instanceof ConstructorDec) {
                ConstructorDec cdec = (ConstructorDec) dec;
                Expr seed = ast.makeVar(cdec.getEnclosingInstanceFormal());
                return w.buildExprFromEnclosing(seed, targetType);
            } else {
                Expr seed = ast.makeThis(currentType);
                return w.buildExprFromThis(seed, targetType);
            }
        }
    }

    // ------------------------------
    // bcg
    protected void cgValue(CodeBuilder cb) {
        throw new RuntimeException("Invalid expression " + this);
    }

    //BEGIN: Generated from @child and @property
    protected TypeD typeD;
    public TypeD getTypeD() { return typeD; }
    public void setTypeD(TypeD _typeD) {
        if (_typeD != null) _typeD.setParent(this);
        typeD = _typeD;
    }

    public QualifiedThisExpr(SourceLocation location, TypeD _typeD) {
        super(location);
        setTypeD(_typeD);
    }
    protected QualifiedThisExpr(SourceLocation source) {
        super(source);
    }

    public ASTObject copyWalk(CopyWalker walker) {
        QualifiedThisExpr ret = new QualifiedThisExpr(getSourceLocation());
        ret.preCopy(walker, this);
        if (typeD != null) ret.setTypeD( (TypeD)walker.process(typeD) );
        return ret;
    }

    public ASTObject getChildAt(int childIndex) {
        switch(childIndex) {
        case 0: return typeD;
        default: return super.getChildAt(childIndex);
        }
    }
     public String getChildNameAt(int childIndex) {
        switch(childIndex) {
        case 0: return "typeD";
        default: return super.getChildNameAt(childIndex);
        }
    }
     public void setChildAt(int childIndex, ASTObject child) {
        switch(childIndex) {
        case 0: setTypeD((TypeD)child); return;
        default: super.setChildAt(childIndex, child); return;
        }
    }
     public int getChildCount() {
        return 1;
    }

    public String getDefaultDisplayName() {
        return "QualifiedThisExpr()";
    }

    //END: Generated from @child and @property
}
