/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.*;
import org.aspectj.compiler.base.cst.*;

/**
  * @grammar ...
  * @child Expr value
  * @child VarDec tmpDec
  */
public class ReferenceExpr extends Expr {
    public Type discoverType() { return value.getType(); }

    public ReferenceExpr(SourceLocation source, Expr value) {
        this(source, value, new VarDec(source, value.getType().makeTypeD(), "tmp", null));
    }

    public Expr makeReference() {
        return getAST().makeVar(tmpDec);
    }

    public VarDec makeVarDec() {
        tmpDec.getModifiers().setFinal(true);
        tmpDec.setInitializer(value);
        return tmpDec;
    }

    // ------------------------------
    // INTRO from FlowCheckerPass

    public void walkFlow(FlowCheckerPass w) {
        throw new RuntimeException("Unhandled reference expr");
    }

    // INTRO from ASTFixerPass
    public ASTObject postFixAST(final ASTFixerPass fixer) {
        BlockStmt body = getEnclosingCodeDec().getBody();
        body.addTemporary(tmpDec);  // this will screw up the walker...
        return getAST().makeParen(getAST().makeSet(tmpDec, value));
    }

    //BEGIN: Generated from @child and @property
    protected Expr value;
    public Expr getValue() { return value; }
    public void setValue(Expr _value) {
        if (_value != null) _value.setParent(this);
        value = _value;
    }

    protected VarDec tmpDec;
    public VarDec getTmpDec() { return tmpDec; }
    public void setTmpDec(VarDec _tmpDec) {
        if (_tmpDec != null) _tmpDec.setParent(this);
        tmpDec = _tmpDec;
    }

    public ReferenceExpr(SourceLocation location, Expr _value, VarDec _tmpDec) {
        super(location);
        setValue(_value);
        setTmpDec(_tmpDec);
    }
    protected ReferenceExpr(SourceLocation source) {
        super(source);
    }

    public ASTObject copyWalk(CopyWalker walker) {
        ReferenceExpr ret = new ReferenceExpr(getSourceLocation());
        ret.preCopy(walker, this);
        if (value != null) ret.setValue( (Expr)walker.process(value) );
        if (tmpDec != null) ret.setTmpDec( (VarDec)walker.process(tmpDec) );
        return ret;
    }

    public ASTObject getChildAt(int childIndex) {
        switch(childIndex) {
        case 0: return value;
        case 1: return tmpDec;
        default: return super.getChildAt(childIndex);
        }
    }
     public String getChildNameAt(int childIndex) {
        switch(childIndex) {
        case 0: return "value";
        case 1: return "tmpDec";
        default: return super.getChildNameAt(childIndex);
        }
    }
     public void setChildAt(int childIndex, ASTObject child) {
        switch(childIndex) {
        case 0: setValue((Expr)child); return;
        case 1: setTmpDec((VarDec)child); return;
        default: super.setChildAt(childIndex, child); return;
        }
    }
     public int getChildCount() {
        return 2;
    }

    public String getDefaultDisplayName() {
        return "ReferenceExpr()";
    }

    //END: Generated from @child and @property
}
