/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.*;
import org.aspectj.compiler.base.cst.*;

import java.util.*;

/**
  */
public class ScopeWalker extends Walker {
    public static void bindNames(ASTObject node, Scope topScope) {
        new ScopeWalker(node.getCompiler(), topScope).process(node);
    }

    public Scope scope;

    public ScopeWalker(JavaCompiler compiler) {
        super(compiler);
    }

    public ScopeWalker(JavaCompiler compiler, Scope initialScope) {
        this(compiler);
        scope = initialScope;
    }

    public void enterCU(CompilationUnit cu) {
    	scope = new CUScope(getCompiler(), null, cu);
    }
//        //System.err.println("hygiene entered: "  + cu);
//        scope = new CUScope(getCompiler(), null, cu) {
//            protected Type handleAmbiguousImport(String id, Type type1, Type type2, ASTObject showWhere) {
//                return this.getTypeManager().TYPE_NOT_FOUND;
//            }
//        };
//    }

    public Scope getScope() { return scope; }

    public void pushBlockScope() {
        pushScope(new BlockScope(getCompiler(), getScope()));
    }

    public void pushScope(Scope newScope) {
        scope = newScope;
    }

    public void popScope() {
        scope = scope.getParent();
    }

    /** enter a { */
    public void pushBlock() {
        if (scope instanceof BlockScope) ((BlockScope)scope).pushBlock();
    }

    /** exit a } */
    public void popBlock() {
        if (scope instanceof BlockScope) ((BlockScope)scope).popBlock();
    }

    public void addVarDec(VarDec varDec) {
        if (scope instanceof BlockScope) ((BlockScope)scope).addVarDec(varDec);
    }

    public void addTypeDec(TypeDec typeDec) {
        if (typeDec.getId().equals("ANONYMOUS")) return;

        if (scope instanceof BlockScope) ((BlockScope)scope).addTypeDec(typeDec);
    }

    public void addMethodDec(MethodDec methodDec) { }

    public ASTObject process(ASTObject object) {
        object.preScope(this);
        object.walkScope(this);
        return object.postScope(this);
    }

    private boolean walkBodies = true;
    public boolean walkBodies() { return walkBodies; }
    public void setWalkBodies(boolean b) { walkBodies = b; }

    public boolean walkSignatures() { return true; }

    public boolean useJavaScopes() { return false; }
}
