/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.*;

/**
  * Represents the platonic ideals of the declarations in the ast.
  *
  * One important purpose for these objects is to retain their identity across
  * incremental recompilation.
  *
  * This implementation treats these objects as proxies for Decs.  That isn't essential
  * to their nature but rather a convenient way to get started.
  */
public abstract class SemanticObject extends CompilerObject {
    protected Dec dec;

    public SemanticObject(JavaCompiler compiler) {
        super(compiler);
    }

    public SemanticObject(Dec dec) {
        super(dec.getCompiler());
        this.dec = dec;
    }
    public String getName() { return dec.getName(); }
    public String getId() { return dec.getId(); }
    public String getBytecodeId() { return dec.getBytecodeId(); }

    public Type getDeclaringType() { return dec.getDeclaringType(); }

    //public Type getResultType() { return dec.getResultType(); }

    public Dec getCorrespondingDec() { return dec; }

    public String getKind() { return dec.getKind(); }

    public String toShortString() { return dec.toShortString(); }

    //*********************************
    public String getDescriptor() { return dec.getDescriptor(); }
    public int getStackDelta() { return dec.getStackDelta(); }

    public Modifiers getModifiers() { return dec.getModifiers(); }

    public boolean isAbstract() { return dec.isAbstract(); }
    public boolean isStatic() { return dec.isStatic(); }

    public boolean isInherited(Type inType) { return dec.isInherited(inType); }
    public boolean conflictsWith(SemanticObject other) {
        return dec.conflictsWith(other.dec);
    }
    public boolean checkOverride(Type inType, SemanticObject other) {
        return dec.checkOverride(inType, other.dec);
    }
    public boolean dominates(SemanticObject other) {
        return dec.dominates(other.dec);
    }

    public boolean isMoreSpecificThan(SemanticObject other) {
        return dec.isMoreSpecificThan(other.dec);
    }
    public boolean isApplicable(Exprs params) { return dec.isApplicable(params); }
    public boolean isAlmostApplicable(Exprs params) { return dec.isAlmostApplicable(params); }

    public boolean isAccessible(ASTObject fromWhere) { return isAccessible(fromWhere, false); }
    public boolean isAccessible(ASTObject fromWhere, boolean inBytecode) {
        if (dec == null) {
            System.out.println("no dec: " + this);
            return false;
        }
        return dec.isAccessible(fromWhere, inBytecode);
    }

    public void showConflictError(SemanticObject other, String message) {
        getCorrespondingDec().showError(toShortString() + " conflicts with " +
                other.toShortString() + ": " + message);
    }

    public Expr updateTargetExpr(Expr expr) {
        if (isStatic() && InterfaceDec.isHelperClass(getDeclaringType().getTypeDec())) {
            return expr.getAST().makeTypeExpr(getDeclaringType());
        }
        return expr;
    }
}
