/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.compiler.base.TypeManager;

import org.aspectj.compiler.base.bcg.FieldBuilder;
import org.aspectj.compiler.base.bcg.CodeBuilder;
import org.aspectj.compiler.base.bcg.Label;


/**
  * @grammar StringLiteral
  * @property String stringValue
  */
public class StringLiteralExpr extends LiteralExpr {

    /** Stolen from CharacterLiteral inner class in {@link
        org.aspectj.compiler.base.parser.JavaTokenizer}.
    */
    private static void quoteCharacter(char ch, StringBuffer buf) {
        //if (ch >= 32 && ch <= 127) {
        //    buf.append(ch); return;
        //}

        switch (ch) {
        case '\b': buf.append("\\b"); return;
        case '\t': buf.append("\\t"); return;
        case '\n': buf.append("\\n"); return;
        case '\f': buf.append("\\f"); return;
        case '\r': buf.append("\\r"); return;
        case '\"': buf.append("\\\""); return;
        case '\'': buf.append("\\\'"); return;
        case '\\': buf.append("\\\\"); return;
        }

        buf.append(ch);

        //buf.append("\\u");
        //String hex = Integer.toHexString(ch);
        //for(int i=0; i<(4-hex.length()); i++) buf.append('0');
        //buf.append(hex);
    }

    private static String quoteString(String value) {
        StringBuffer buf = new StringBuffer("\"");
        for(int i=0; i<value.length(); i++) {
            quoteCharacter(value.charAt(i), buf);
        }
        buf.append("\"");
        return buf.toString();
    }

    public StringLiteralExpr(SourceLocation source, String _stringValue) {
        this(source, source.getCompiler().getTypeManager().getStringType(),
             quoteString(_stringValue),
             _stringValue);
    }

    // ------------------------------
    // bcg

    protected void cgValue(CodeBuilder cb) {
        cb.emitStringConstant(stringValue);
    }
    public void addConstant(FieldBuilder fb) {
        fb.setConstantValue(stringValue);
    }

    //BEGIN: Generated from @child and @property
    protected String stringValue;
    public String getStringValue() { return stringValue; }
    public void setStringValue(String _stringValue) { stringValue = _stringValue; }

    public StringLiteralExpr(SourceLocation location, Type _type, String _value, String _stringValue) {
        super(location, _type, _value);
        setStringValue(_stringValue);
    }
    protected StringLiteralExpr(SourceLocation source) {
        super(source);
    }

    public ASTObject copyWalk(CopyWalker walker) {
        StringLiteralExpr ret = new StringLiteralExpr(getSourceLocation());
        ret.preCopy(walker, this);
        ret.type = type;
        ret.value = value;
        ret.stringValue = stringValue;
        return ret;
    }


    public String getDefaultDisplayName() {
        return "StringLiteralExpr(type: "+type+", "+"value: "+value+", "+"stringValue: "+stringValue+")";
    }

    //END: Generated from @child and @property
}
