/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.*;
import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.compiler.base.CodeWriter;

import org.aspectj.compiler.base.bcg.CodeBuilder;
import org.aspectj.compiler.base.bcg.Label;

/**
 * @grammar throw expr;
 * @child Expr expr
 */

public class ThrowStmt extends Stmt {

    public void unparse(CodeWriter writer) {
        writer.writeKeyword("throw");
        writer.requiredSpace();
        writer.write(expr);
        writer.closeStmt();
    }

    public void checkSpec() {
        getExpr().assertType(getTypeManager().getThrowableType());
    }

    // ------------------------------
    // INTRO from FlowCheckerPass

    public void walkFlow(FlowCheckerPass w) {
        w.process(getExpr());
        w.doReturn();
        w.setLive(false);
        w.setExns(w.getExns().add((NameType) expr.getType()));
        w.setVars(w.getAllVars());
    }

    // ------------------------------
    // INTRO from ByteCodeCleanupPass

    public void walkCleanup(ByteCodeCleanupPass w) {
        w.setLive(false);
    }

    // ------------------------------
    // bcg

    protected void cgStmt(CodeBuilder cb) {
        expr.cgValue(cb);
        cb.emitATHROW();
    }

    //BEGIN: Generated from @child and @property
    protected Expr expr;
    public Expr getExpr() { return expr; }
    public void setExpr(Expr _expr) {
        if (_expr != null) _expr.setParent(this);
        expr = _expr;
    }

    public ThrowStmt(SourceLocation location, Expr _expr) {
        super(location);
        setExpr(_expr);
    }
    protected ThrowStmt(SourceLocation source) {
        super(source);
    }

    public ASTObject copyWalk(CopyWalker walker) {
        ThrowStmt ret = new ThrowStmt(getSourceLocation());
        ret.preCopy(walker, this);
        if (expr != null) ret.setExpr( (Expr)walker.process(expr) );
        return ret;
    }

    public ASTObject getChildAt(int childIndex) {
        switch(childIndex) {
        case 0: return expr;
        default: return super.getChildAt(childIndex);
        }
    }
     public String getChildNameAt(int childIndex) {
        switch(childIndex) {
        case 0: return "expr";
        default: return super.getChildNameAt(childIndex);
        }
    }
     public void setChildAt(int childIndex, ASTObject child) {
        switch(childIndex) {
        case 0: setExpr((Expr)child); return;
        default: super.setChildAt(childIndex, child); return;
        }
    }
     public int getChildCount() {
        return 1;
    }

    public String getDefaultDisplayName() {
        return "ThrowStmt()";
    }

    //END: Generated from @child and @property
}


