/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.FlowCheckerPass;
import org.aspectj.compiler.base.AssignmentCheckerPass;
import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.compiler.base.CodeWriter;
import org.aspectj.compiler.base.cst.*;
import org.aspectj.compiler.base.bcg.CodeBuilder;

import java.util.*;


/**
 * @grammar <op> rand1
 * @property String op
 * @child Expr rand1
 *
 */
public abstract class UnopExpr extends Expr {

    protected void bindNamesSelf() {
        //setType(discoverType());
    }

    public ASTObject postAssignmentCheck(AssignmentCheckerPass walker) {
        return tryToFold();
    }

    private ASTObject tryToFold() {
        if (getLiftType().isAnyType()) return this;
        if (rand1 instanceof LiteralExpr) {
            return halfFold(getLiftType(), (LiteralExpr)rand1).setSource(this);
        } else {
            return this;
        }
    }

    abstract protected Type getLiftType();

    /** returns a new unfinished ASTObject, that still needs to have
        {@link ASTObject#setSource} called on it. */
    abstract protected LiteralExpr halfFold(Type type, LiteralExpr lit);

    public static UnopExpr build(SourceLocation source, String op, Expr rand1) {
        if (op.equals("+")) {
            return new PlusOpExpr(source, op, rand1);
        } else if (op.equals("-")) {
            return new MinusOpExpr(source, op, rand1);
        } else if (op.equals("~")) {
            return new BitNotOpExpr(source, op, rand1);
        } else if (op.equals("!")) {
            return new LogNotOpExpr(source, op, rand1);
        } else {
            throw new RuntimeException("strange unary op: " + op);
        }
    }

    protected Type discoverType() {
        //??? is this true for all unops?
        return getRand1().getType();
    }

    public void unparse(CodeWriter writer) {
        writer.write(getOp());
        writer.write(getRand1());
    }

    public void showOperatorTypeError(Type a) {
        if (a.isAnyType()) return;
        showError("operator " + op + " cannot be applied to " + a.getString());
    }

    // ------------------------------
    // bcg

    protected void cgEffect(CodeBuilder cb) {
        rand1.cgEffect(cb);
    }

    //BEGIN: Generated from @child and @property
    protected String op;
    public String getOp() { return op; }
    public void setOp(String _op) { op = _op; }

    protected Expr rand1;
    public Expr getRand1() { return rand1; }
    public void setRand1(Expr _rand1) {
        if (_rand1 != null) _rand1.setParent(this);
        rand1 = _rand1;
    }

    public UnopExpr(SourceLocation location, String _op, Expr _rand1) {
        super(location);
        setOp(_op);
        setRand1(_rand1);
    }
    protected UnopExpr(SourceLocation source) {
        super(source);
    }

    public ASTObject getChildAt(int childIndex) {
        switch(childIndex) {
        case 0: return rand1;
        default: return super.getChildAt(childIndex);
        }
    }
     public String getChildNameAt(int childIndex) {
        switch(childIndex) {
        case 0: return "rand1";
        default: return super.getChildNameAt(childIndex);
        }
    }
     public void setChildAt(int childIndex, ASTObject child) {
        switch(childIndex) {
        case 0: setRand1((Expr)child); return;
        default: super.setChildAt(childIndex, child); return;
        }
    }
     public int getChildCount() {
        return 1;
    }

    public String getDefaultDisplayName() {
        return "UnopExpr(op: "+op+")";
    }

    //END: Generated from @child and @property
}
