/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.bcg;

/** A placeholder class for various bounds checking methods.  */
abstract public class Asserts {

    public static boolean isByte(int value) {
        return Byte.MIN_VALUE <= value && value <= Byte.MAX_VALUE;
    }
    public static boolean isShort(int value) {
        return Short.MIN_VALUE <= value && value <= Short.MAX_VALUE;
    }
    public static boolean isUByte(int value) {
        return 0 <= value && value <= 0xFF;
    }
    public static boolean isUShort(int value) {
        return 0 <= value && value <= 0xFFFF;
    }

    // Can a signed quantity be represented as a U2?
    public static void assertU2(int nextIndex, String msg) {
        if (Short.MIN_VALUE <= nextIndex && nextIndex <= Short.MAX_VALUE) return;
        throw new RuntimeException(msg);
    }

    // Can an unsigned quantity be represented as a U2?
    public static void assertUU2(int nextIndex, String msg) {
        if (isUShort(nextIndex)) return;
        throw new RuntimeException(msg);
    }


}
