/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.bcg;

import org.aspectj.compiler.base.bcg.pool.*;

import java.io.DataOutputStream;
import java.io.PrintStream;
import java.io.IOException;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import java.util.Stack;
import java.util.Iterator;
import java.util.ListIterator;

// XXX annoying big modularity violation.  
import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.compiler.base.ast.ControlContext;
import org.aspectj.compiler.base.ast.VarDec;
import org.aspectj.compiler.base.ast.LabeledStmt;
import org.aspectj.compiler.base.ast.WindingStmt;
import org.aspectj.compiler.base.ast.Stmt;
import org.aspectj.compiler.base.ast.Type;
import org.aspectj.compiler.base.ast.ArrayType;
import org.aspectj.compiler.base.ast.RefType;
import org.aspectj.compiler.base.ast.NameType;
import org.aspectj.compiler.base.ast.SourceLocation;

abstract class Code {
    int index; // long?
    int pc = -1;  // should be long
    List labels = new LinkedList();
    void addLabel(Label l) { labels.add(l); }
    void removeLabel(Label l) { labels.remove(l); }

    List getLabels() { return labels; }
    int getPc() { return pc; }
    void setPc(int pc) { this.pc = pc; }
    
    abstract int resolveBest(int pc);
    int resolveWorst(int pc) { return resolveBest(pc); }

    abstract void writeTo(DataOutputStream stream) throws IOException;

    boolean jumps() { return false; }

    void notifyNonLive() {}

    Code getRealNext() { return this; }
    void resolve(Code next) { }
    boolean isActive() { return true; }


}

