/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.cst;

import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.base.*;
import java.util.*;

/**
 * The scope of a type declaration
 */
public class TypeScope extends Scope {
    private Type type;
    
    public TypeScope(JavaCompiler compiler, Scope parent, Type type) {
        super(compiler, parent);
        this.type = type;
    }
    public String shortToString() { return "TYPE(" + type.getString() + ")"; }
    
    public Type findType(String name, ASTObject fromWhere) {
        Type ret = type.getInnerType(name, fromWhere, false);
        if (ret != getTypeManager().TYPE_NOT_FOUND && ret != null) return ret;
//        System.out.println(ret + " == " + getTypeManager().TYPE_NOT_FOUND + " ? " +
//                (ret == getTypeManager().TYPE_NOT_FOUND));        
        return super.findType(name, fromWhere);
    }
    
    public Expr bindUnqualifiedName(String name, ASTObject fromWhere) {
        Field ret = type.getField(name, fromWhere, false);
        if (ret != null) {
            Expr thisExpr;
            if (ret.isStatic()) thisExpr = getAST().makeTypeExpr(type);
            else thisExpr = getAST().makeQualifiedThis(type);
            return new FieldAccessExpr(fromWhere.getSourceLocation(), thisExpr, ret, false);
        }
        else return super.bindUnqualifiedName(name, fromWhere);
    }

    public Type findMethodLookupType(String name, ASTObject fromWhere) {
        if (type.hasMethodNamed(name)) return type;
        else return super.findMethodLookupType(name, fromWhere);
    }
}
