/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.cst;

import java.util.*;
import org.aspectj.compiler.base.ast.*;

import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.compiler.base.CodeWriter;


/**
  * @grammar expr.id
  * @property String id
  */

public class UnresolvedFieldAccessExpr extends FieldAccessExpr {
    public UnresolvedFieldAccessExpr(SourceLocation source, Expr expr, String id) {
        this(source,expr, null, false, id);
    }

    public Field resolveField() {
        Type type = expr.getType();
        if (type == null) {
            expr.showError("bad type");
            return null;
        }
        
        if (isSuper) type = ((NameType) type).getSuperClassType();

        return type.getField(id, this, true);
    }
    
    //INTRO from ScopeWalker
    public ASTObject postScope(ScopeWalker walker) {

        if (expr != null && expr instanceof SuperExpr) {
            setIsSuper(true);
            setExpr(((SuperExpr)expr).makeThis());
        }
        
        return new FieldAccessExpr(getSourceLocation(), getExpr(), resolveField(), getIsSuper());
  
    }    
    
    //BEGIN: Generated from @child and @property
    protected String id;
    public String getId() { return id; }
    public void setId(String _id) { id = _id; }
    
    public UnresolvedFieldAccessExpr(SourceLocation location, Expr _expr, Field _field, boolean _isSuper, String _id) {
        super(location, _expr, _field, _isSuper);
        setId(_id);
    }
    protected UnresolvedFieldAccessExpr(SourceLocation source) {
        super(source);
    }
    
    public ASTObject copyWalk(CopyWalker walker) {
        UnresolvedFieldAccessExpr ret = new UnresolvedFieldAccessExpr(getSourceLocation());
        ret.preCopy(walker, this);
        if (expr != null) ret.setExpr( (Expr)walker.process(expr) );
        ret.field = field;
        ret.isSuper = isSuper;
        ret.id = id;
        return ret;
    }
    
    
    public String getDefaultDisplayName() {
        return "UnresolvedFieldAccessExpr(field: "+field+", "+"isSuper: "+isSuper+", "+"id: "+id+")";
    }
    
    //END: Generated from @child and @property
}
