/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.ast;
import org.aspectj.compiler.base.ast.*;

import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.compiler.base.CodeWriter;
import java.io.IOException;

/**
 * @grammar name (, name)*
 * @children GenTypeName name
 */

public class GenTypeNames extends ASTObject {
    public void unparse(CodeWriter writer) {
        writer.writeChildrenWithCommas(this);
    }    
    
    //BEGIN: Generated from @child and @property
    protected int size;
    public GenTypeName[] children;
    
    public GenTypeNames(SourceLocation location, GenTypeName[] _children) {
    	super(location);
        for(int i=0; i<_children.length; i++) {
            if (_children[i] != null) _children[i].setParent(this);
        }
        children = _children;
        size = _children.length;
    }
    
    public GenTypeNames(SourceLocation location) {
        this(location, new GenTypeName[] {});
    }
    
    public GenTypeNames(SourceLocation location, GenTypeName child1) {
        this(location, new GenTypeName[] {child1});
    }
    
    public GenTypeNames(SourceLocation location, GenTypeName child1, GenTypeName child2) {
        this(location, new GenTypeName[] {child1, child2});
    }
    
    public GenTypeNames(SourceLocation location, GenTypeName child1, GenTypeName child2, GenTypeName child3) {
        this(location, new GenTypeName[] {child1, child2, child3});
    }
    
    public ASTObject copyWalk(CopyWalker walker) {
        final int N = size;
        GenTypeName[] copiedChildren = new GenTypeName[N];
        int newIndex = 0;
        for(int oldIndex=0; oldIndex<N; oldIndex++) {
            GenTypeName newChild = (GenTypeName)walker.process(children[oldIndex]);
            if (newChild != null) copiedChildren[newIndex++] = newChild;
        }
        GenTypeNames ret = new GenTypeNames(getSourceLocation(),copiedChildren);
        ret.size = newIndex;
        ret.setSource(this);
        return ret;
    }
    
    public ASTObject getChildAt(int childIndex) { return get(childIndex); }
    public void setChildAt(int childIndex, ASTObject child) { set(childIndex, (GenTypeName)child); }
    public String getChildNameAt(int childIndex) { return "name"+childIndex; }
    
    public int getChildCount() { return size; }
    public int size() { return size; }
    
    public GenTypeName get(int index) {
        if (index >= size) throw new ArrayIndexOutOfBoundsException();
        return children[index];
    }
    
    public void set(int index, GenTypeName child) {
        if (index >= size) throw new ArrayIndexOutOfBoundsException();
        children[index] = child;
        child.setParent(this);
    }
    
    public void resize(int newSize) {
        if (newSize > children.length) {
            GenTypeName[] newChildren = new GenTypeName[children.length*2 + 1];
            System.arraycopy(children, 0, newChildren, 0, children.length);
            children = newChildren;
        }
        size = newSize;
    }
    
    public void addAll(GenTypeNames collection) {
        addAll(size, collection);
    }
    
    public void addAll(int index, GenTypeNames collection) {
        for(int i=0; i<collection.size(); i++) {
            add(index+i, collection.get(i));
        }
    }
    
    public void add(GenTypeName child) {
        add(size, child);
    }
    
    public void add(int index, GenTypeName child) {
        if (child == null) return;
    
        if (index < 0 || index > size) throw new ArrayIndexOutOfBoundsException();
    
        resize(size+1);
        
        for(int moveIndex = size-1; moveIndex > index; moveIndex--) {
            children[moveIndex] = children[moveIndex-1];
        }
    
        children[index] = child;
        child.setParent(this);
    }
    
    public void remove(int index) {
        if (index < 0 || index > size) throw new ArrayIndexOutOfBoundsException();
    
        size -= 1;
        
        for(int moveIndex = index; moveIndex < size; moveIndex++) {
            children[moveIndex] = children[moveIndex+1];
        }
    }
    
    public String getDefaultDisplayName() {
        return "GenTypeNames()";
    }
    
    //END: Generated from @child and @property
}
