/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.ast;
import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.crosscuts.joinpoints.*;

import org.aspectj.compiler.base.JavaCompiler;

import java.util.*;
import org.aspectj.util.InvertableSet;

/**
  * @grammar handler(typeNamePattern)
  * @child GenTypeName typeName
  *
  * XXX This is actually a KindedPcd, but doesn't match the API
  */
public class HandlerPcd extends Pcd {
    public String toShortString() {
        return "handler(" + getTypeName().toShortString() + ")";
    }
    public void checkStatic() { }
    public int getJpKind() { return JoinPoint.HANDLER; }

    public JpPlanner makePlanner(PlanData planData) {
        return new JpPlanner() {
            public FastMatch fastMatch(JoinPoint jp) {
                if (!(jp.getTypeCode() == getJpKind())) return NO;
                //XXX vs. getArgTypes().get(0) 
                return getTypeName().matches(jp.getExceptionType()) ? YES : NO;
            }
        };
    }    
    
    //BEGIN: Generated from @child and @property
    protected GenTypeName typeName;
    public GenTypeName getTypeName() { return typeName; }
    public void setTypeName(GenTypeName _typeName) {
        if (_typeName != null) _typeName.setParent(this);
        typeName = _typeName;
    }
    
    public HandlerPcd(SourceLocation location, GenTypeName _typeName) {
        super(location);
        setTypeName(_typeName);
    }
    protected HandlerPcd(SourceLocation source) {
        super(source);
    }
    
    public ASTObject copyWalk(CopyWalker walker) {
        HandlerPcd ret = new HandlerPcd(getSourceLocation());
        ret.preCopy(walker, this);
        if (typeName != null) ret.setTypeName( (GenTypeName)walker.process(typeName) );
        return ret;
    }
    
    public ASTObject getChildAt(int childIndex) {
        switch(childIndex) {
        case 0: return typeName;
        default: return super.getChildAt(childIndex);
        }
    }
     public String getChildNameAt(int childIndex) {
        switch(childIndex) {
        case 0: return "typeName";
        default: return super.getChildNameAt(childIndex);
        }
    }
     public void setChildAt(int childIndex, ASTObject child) {
        switch(childIndex) {
        case 0: setTypeName((GenTypeName)child); return;
        default: super.setChildAt(childIndex, child); return;
        }
    }
     public int getChildCount() {
        return 1;
    }
    
    public String getDefaultDisplayName() {
        return "HandlerPcd()";
    }
    
    //END: Generated from @child and @property
}

