/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.ast;

import org.aspectj.compiler.base.ast.*;

import org.aspectj.compiler.base.JavaCompiler;

import java.util.*;

/**
 * @grammar names..*
 * @property String packageName
 * @property NamePattern className
 * @property boolean includeSubpackages
 */

public class NameTypeDPattern extends ASTObject {
    private Set _typeDecs = null;

    private String getMyPackageName() {
        if (packageName == null) {
            //!!! bad overriding of getPackageName method by NameTypeD
            return getCompilationUnit().getPackageName();
        } else {
            return packageName;
        }
    }

    public boolean matches(TypeD typeD) {
        return matches(typeD.getType());
    }

    private boolean matches(Type type) {
        if (type == null) return false;

        if (!className.matches(type.getId())) {
            return false;
        }
        String packageName = type.getPackageName();
        String myPackageName = getMyPackageName();
        if (myPackageName == null) return packageName == null;
        if (packageName == null) return false;

        if (includeSubpackages) {
            //!!! might want to check for . at end to be careful...
            return packageName.startsWith(myPackageName);
        } else {
            return myPackageName.equals(packageName);
        }
    }


//    public TypeDec[] getTypeDecs() {
//        return (TypeDec[])getTypeDecSet().toArray(new TypeDec[0]);
//    }

//    public Set getTypeDecSet() {
//        if (_typeDecs != null) return _typeDecs;
//
//        List allDecs = new ArrayList(); //getWorld().getTypes();
//        Set matchingDecs = new HashSet();
//
//        for(int i=0; i<allDecs.size(); i++) {
//            if (!(allDecs.get(i) instanceof TypeDec)) continue;
//
//            TypeDec typeDec = (TypeDec)allDecs.get(i);
//            if (this.matches(typeDec)) {
//                matchingDecs.add(typeDec);
//            }
//        }
//
//        _typeDecs = matchingDecs;
//        return matchingDecs;
//    }    
    
    //BEGIN: Generated from @child and @property
    protected String packageName;
    public String getPackageName() { return packageName; }
    public void setPackageName(String _packageName) { packageName = _packageName; }
    
    protected NamePattern className;
    public NamePattern getClassName() { return className; }
    public void setClassName(NamePattern _className) { className = _className; }
    
    protected boolean includeSubpackages;
    public boolean getIncludeSubpackages() { return includeSubpackages; }
    public void setIncludeSubpackages(boolean _includeSubpackages) { includeSubpackages = _includeSubpackages; }
    
    public NameTypeDPattern(SourceLocation location, String _packageName, NamePattern _className, boolean _includeSubpackages) {
        super(location);
        setPackageName(_packageName);
        setClassName(_className);
        setIncludeSubpackages(_includeSubpackages);
    }
    protected NameTypeDPattern(SourceLocation source) {
        super(source);
    }
    
    public ASTObject copyWalk(CopyWalker walker) {
        NameTypeDPattern ret = new NameTypeDPattern(getSourceLocation());
        ret.preCopy(walker, this);
        ret.packageName = packageName;
        ret.className = className;
        ret.includeSubpackages = includeSubpackages;
        return ret;
    }
    
    
    public String getDefaultDisplayName() {
        return "NameTypeDPattern(packageName: "+packageName+", "+"className: "+className+", "+"includeSubpackages: "+includeSubpackages+")";
    }
    
    //END: Generated from @child and @property
}
