/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.ast;
import org.aspectj.compiler.base.ast.*;

import org.aspectj.compiler.base.JavaCompiler;

/** @grammar ...
  * @child TypeDs trueNames
  * @child TypeDs falseNames
  */
public class NameTypeDsPattern extends ASTObject {
    public boolean matches(TypeDs otherNames) {
        if (otherNames == null) return trueNames.size() == 0;

        int i;
        final int N1 = trueNames.size();
        for(i=0; i<N1; i++) {
            //System.out.println("t: " + trueNames.get(i) + ", " + otherNames.get(0));
            if (!otherNames.contains(trueNames.get(i))) return false;
        }

        final int N2 = falseNames.size();
        for(i=0; i<N2; i++) {
            //System.out.println("f: " + falseNames.get(i));
            if (otherNames.contains(falseNames.get(i))) return false;
        }

        //System.out.println("matches");

        return true;
    }    
    
    //BEGIN: Generated from @child and @property
    protected TypeDs trueNames;
    public TypeDs getTrueNames() { return trueNames; }
    public void setTrueNames(TypeDs _trueNames) {
        if (_trueNames != null) _trueNames.setParent(this);
        trueNames = _trueNames;
    }
    
    protected TypeDs falseNames;
    public TypeDs getFalseNames() { return falseNames; }
    public void setFalseNames(TypeDs _falseNames) {
        if (_falseNames != null) _falseNames.setParent(this);
        falseNames = _falseNames;
    }
    
    public NameTypeDsPattern(SourceLocation location, TypeDs _trueNames, TypeDs _falseNames) {
        super(location);
        setTrueNames(_trueNames);
        setFalseNames(_falseNames);
    }
    protected NameTypeDsPattern(SourceLocation source) {
        super(source);
    }
    
    public ASTObject copyWalk(CopyWalker walker) {
        NameTypeDsPattern ret = new NameTypeDsPattern(getSourceLocation());
        ret.preCopy(walker, this);
        if (trueNames != null) ret.setTrueNames( (TypeDs)walker.process(trueNames) );
        if (falseNames != null) ret.setFalseNames( (TypeDs)walker.process(falseNames) );
        return ret;
    }
    
    public ASTObject getChildAt(int childIndex) {
        switch(childIndex) {
        case 0: return trueNames;
        case 1: return falseNames;
        default: return super.getChildAt(childIndex);
        }
    }
     public String getChildNameAt(int childIndex) {
        switch(childIndex) {
        case 0: return "trueNames";
        case 1: return "falseNames";
        default: return super.getChildNameAt(childIndex);
        }
    }
     public void setChildAt(int childIndex, ASTObject child) {
        switch(childIndex) {
        case 0: setTrueNames((TypeDs)child); return;
        case 1: setFalseNames((TypeDs)child); return;
        default: super.setChildAt(childIndex, child); return;
        }
    }
     public int getChildCount() {
        return 2;
    }
    
    public String getDefaultDisplayName() {
        return "NameTypeDsPattern()";
    }
    
    //END: Generated from @child and @property
}

