/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.ast;

import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.crosscuts.joinpoints.*;


import java.util.*;
import org.aspectj.util.InvertableSet;

import org.aspectj.compiler.base.JavaCompiler;

/**
 * @grammar pcd1 || pcd2
 */

public class OrPcd extends BinaryPcd {
    protected String getSymbol() { return "||"; }

    public JpPlanner makePlanner(PlanData planData) {
        return getPcd1().makePlanner(planData).or(getPcd2().makePlanner(planData));
    }    
    
    //BEGIN: Generated from @child and @property
    
    public OrPcd(SourceLocation location, Pcd _pcd1, Pcd _pcd2) {
        super(location, _pcd1, _pcd2);
    
    }
    protected OrPcd(SourceLocation source) {
        super(source);
    }
    
    public ASTObject copyWalk(CopyWalker walker) {
        OrPcd ret = new OrPcd(getSourceLocation());
        ret.preCopy(walker, this);
        if (pcd1 != null) ret.setPcd1( (Pcd)walker.process(pcd1) );
        if (pcd2 != null) ret.setPcd2( (Pcd)walker.process(pcd2) );
        return ret;
    }
    
    
    public String getDefaultDisplayName() {
        return "OrPcd()";
    }
    
    //END: Generated from @child and @property
}

