/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.ast;
import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.crosscuts.joinpoints.*;
import org.aspectj.util.*;

import org.aspectj.compiler.base.JavaCompiler;

/**
 * @grammar per*(...)
 */

public abstract class PerClause extends ASTObject {
    MethodDec aspectOfMethod;
    MethodDec hasAspectMethod;
    
    private AspectDec aspectDec = null;
    public AspectDec getAspectDec() {
        if (aspectDec == null) {
            aspectDec = (AspectDec)getParent();
        }
        return aspectDec;
    }

    TypeD getAspectTypeD() {
        return getAspectDec().getType().makeTypeD();
    }
    Type getAspectType() {
        return getAspectDec().getType();
    }
    
    /**
     * per-clauses have a unique notion of their lexically enclosing type
     * even though they're not in the body of their declaring aspect, they
     * need to be treated as if they were.
     */
    public Type getLexicalType() {
        return getAspectType();
    }

    public Type getDeclaringType() {
        return getAspectType();
    }
    
    public abstract JpPlanner makeInnerPlanner(PlanData planData);
    public abstract JpPlanner makeInitializerPlanner(PlanData planData);
    
    protected MethodDec makeAspectOfMethod(Formals formals, BlockStmt body) {
        final AST ast = getAST();
        Modifiers modifiers = ast.makeModifiers(Modifiers.PUBLIC | Modifiers.STATIC);
        String name = "aspectOf";
//        if (getAspectDec().isAbstract()) {
//            name =
//              JavaStrings.makeLegalIdentifier(name + '$' + getAspectType().getString());
//        }
        
        return ast.makeMethod(modifiers, getAspectType(), name, formals, body);
    }
    
    protected MethodDec makeHasAspectMethod(Formals formals, BlockStmt body) {
        final AST ast = getAST();
        Modifiers modifiers = ast.makeModifiers(Modifiers.PUBLIC | Modifiers.STATIC);
        String name = "hasAspect";
//        if (getAspectDec().isAbstract()) {
//            name =
//              JavaStrings.makeLegalIdentifier(name + '$' + getAspectType().getString());
//        }
        
        return ast.makeMethod(modifiers, getTypeManager().booleanType, name, formals, body);
    }
    
    protected abstract MethodDec makeAspectOfMethod();
    protected abstract MethodDec makeHasAspectMethod();
    
    
    public void setupAspect() {
        TypeDec typeDec = getAspectDec();
        if (typeDec.isAbstract()) return;
        
        aspectOfMethod = makeAspectOfMethod();
        typeDec.addToBodyAndType(aspectOfMethod);
        hasAspectMethod = makeHasAspectMethod();
        typeDec.addToBodyAndType(hasAspectMethod);
    }

    public Expr makeAspectOfExpr(Expr fromObject) { return null; }    
    
    //BEGIN: Generated from @child and @property
    
    public PerClause(SourceLocation location) {
        super(location);
    
    }
    
    
    public String getDefaultDisplayName() {
        return "PerClause()";
    }
    
    //END: Generated from @child and @property
}

