/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.ast;
import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.base.*;
import org.aspectj.compiler.crosscuts.joinpoints.*;

import java.util.*;

public class PlanData {
    public AspectDec inAspect;
    private JpPlannerMaker jpPlannerMaker;
    public Set bindingNames = new HashSet();
    
    public Type nameBindingType;

    /**
     * declare error: call(PlanData.new(..)) && 
     *      !(within(PlanData) || within(BuildAdvicePlannersPass)):
     *      "shouldn't be creating new PlanData here";
     */
    public PlanData(AspectDec inAspect, JpPlannerMaker jpPlannerMaker) {
        this.jpPlannerMaker = jpPlannerMaker;
        this.inAspect = inAspect;
    }
    
    /**
     * returns the ast object that is making this plan
     */
    public ASTObject getPlanMaker() {
        if (jpPlannerMaker instanceof ASTObject) {
        	return (ASTObject)jpPlannerMaker;
        } else {
            return getAspectDec();
        }
    }	

    public AspectDec getAspectDec() {
        return inAspect;
    }
    
    public Type getNameBindingType() {
        if (nameBindingType == null) {
            if (inAspect == null) return null;
            return inAspect.getType();
        } else {
            return nameBindingType;
        }
    }
    
    public PlanData makeInner(NamePcd bindingName) {
        PlanData pd = new PlanData(this.inAspect, this.jpPlannerMaker);
        pd.bindingNames.addAll(this.bindingNames);
        pd.bindingNames.add(bindingName);
        if (bindingName.getParentName() != null) {
            pd.nameBindingType = bindingName.getParentName().getType();
        } else {
            pd.nameBindingType = this.nameBindingType;
        }
        return pd;
    }
    
    public boolean isPlanning(NamePcd namePcd) {
        return bindingNames.contains(namePcd);
    }
}
