/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.ast;

import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.base.*;
import org.aspectj.compiler.crosscuts.*;
import org.aspectj.compiler.base.bcg.CodeBuilder;

import java.util.*;

/**
  * @grammar proceed(args)
  * @child Exprs args
  * @property AroundAdviceDec enclosingAround
  */
public class ProceedExpr extends Expr {
    protected Type discoverType() { return enclosingAround.getResultType(); } 
    
    public boolean isLegalStmt() {
        return true;
    }
    
    protected void cgValue(CodeBuilder cb) {
    	getCompiler().internalError(this, "should no longer exist");
    }

    
    /** returns all possible exceptions thrown by proceed
     */
    public Set getPossibleExceptions(boolean deFacto) {
        if (deFacto) {
            Set set = new HashSet();
            set.add(getTypeManager().getThrowableType());
            return set;
        } else {
            return Collections.EMPTY_SET;
        }
    }
        
    
    
    // ------------------------------
    // INTRO from FlowCheckerPass

    public void walkFlow(FlowCheckerPass w) {
        w.process(getArgs());
        w.setExns(FlowCheckerPass.ESet.getEmpty().add(getTypeManager().getThrowableType()));
    }    
    
    //BEGIN: Generated from @child and @property
    protected Exprs args;
    public Exprs getArgs() { return args; }
    public void setArgs(Exprs _args) {
        if (_args != null) _args.setParent(this);
        args = _args;
    }
    
    protected AroundAdviceDec enclosingAround;
    public AroundAdviceDec getEnclosingAround() { return enclosingAround; }
    public void setEnclosingAround(AroundAdviceDec _enclosingAround) { enclosingAround = _enclosingAround; }
    
    public ProceedExpr(SourceLocation location, Exprs _args, AroundAdviceDec _enclosingAround) {
        super(location);
        setArgs(_args);
        setEnclosingAround(_enclosingAround);
    }
    protected ProceedExpr(SourceLocation source) {
        super(source);
    }
    
    public ASTObject copyWalk(CopyWalker walker) {
        ProceedExpr ret = new ProceedExpr(getSourceLocation());
        ret.preCopy(walker, this);
        if (args != null) ret.setArgs( (Exprs)walker.process(args) );
        ret.enclosingAround = enclosingAround;
        return ret;
    }
    
    public ASTObject getChildAt(int childIndex) {
        switch(childIndex) {
        case 0: return args;
        default: return super.getChildAt(childIndex);
        }
    }
     public String getChildNameAt(int childIndex) {
        switch(childIndex) {
        case 0: return "args";
        default: return super.getChildNameAt(childIndex);
        }
    }
     public void setChildAt(int childIndex, ASTObject child) {
        switch(childIndex) {
        case 0: setArgs((Exprs)child); return;
        default: super.setChildAt(childIndex, child); return;
        }
    }
     public int getChildCount() {
        return 1;
    }
    
    public String getDefaultDisplayName() {
        return "ProceedExpr(enclosingAround: "+enclosingAround+")";
    }
    
    //END: Generated from @child and @property
}
