/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.ast;
import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.crosscuts.joinpoints.*;

import org.aspectj.compiler.base.JavaCompiler;

import java.util.*;
import org.aspectj.util.InvertableSet;


/**
 * @grammar value
 * @property String value
 */
public class SimpleName extends NamePattern {
    public String getLookupString() {
        return value;
    }

    public boolean matches(String s) {
        return value.equals(s);
    }

    public String toShortString() {
        return value;
    }

    public String getSimpleName() {
        return value;
    }    
    
    //BEGIN: Generated from @child and @property
    protected String value;
    public String getValue() { return value; }
    public void setValue(String _value) { value = _value; }
    
    public SimpleName(SourceLocation location, String _value) {
        super(location);
        setValue(_value);
    }
    protected SimpleName(SourceLocation source) {
        super(source);
    }
    
    public ASTObject copyWalk(CopyWalker walker) {
        SimpleName ret = new SimpleName(getSourceLocation());
        ret.preCopy(walker, this);
        ret.value = value;
        return ret;
    }
    
    
    public String getDefaultDisplayName() {
        return "SimpleName(value: "+value+")";
    }
    
    //END: Generated from @child and @property
}



