/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.ast;
import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.crosscuts.joinpoints.*;

import org.aspectj.compiler.base.JavaCompiler;

import org.aspectj.util.*;

import java.util.*;

/**
  * @grammar target(typeNamePattern)
  * @child GenTypeName typeName
  */
public class TargetPcd extends Pcd {
    public String toShortString() {
        return "target(" + getTypeName().toShortString() + ")";
    }

    public boolean allowsNameBinding() { return true; }
    public void checkStatic() { showNonStaticError(); }

    public JpPlanner makePlanner(PlanData planData) {
        return new JpPlanner() {
            public FastMatch fastMatch(JoinPoint jp) {
                //System.out.println(toShortString() + ": " + jp.getTargetType() + ", " + getTypeName());
                //XXX want to make this method more accurate
                if (jp.getTargetExprType() == null) return NO;
                FuzzyBoolean match = getTypeName().matchesInstance(jp.getTargetExprType());
                if (match.alwaysFalse()) return NO;
                else if (match.alwaysTrue()) return YES;
                return MAYBE;
            }
            public JpPlan makePlan(JoinPoint jp) {
                if (jp.makeTargetExpr() == null) return JpPlan.NO_PLAN;
                return getTypeName().makePlan(jp, jp.makeTargetExpr());
            }
        };
    }    
    
    //BEGIN: Generated from @child and @property
    protected GenTypeName typeName;
    public GenTypeName getTypeName() { return typeName; }
    public void setTypeName(GenTypeName _typeName) {
        if (_typeName != null) _typeName.setParent(this);
        typeName = _typeName;
    }
    
    public TargetPcd(SourceLocation location, GenTypeName _typeName) {
        super(location);
        setTypeName(_typeName);
    }
    protected TargetPcd(SourceLocation source) {
        super(source);
    }
    
    public ASTObject copyWalk(CopyWalker walker) {
        TargetPcd ret = new TargetPcd(getSourceLocation());
        ret.preCopy(walker, this);
        if (typeName != null) ret.setTypeName( (GenTypeName)walker.process(typeName) );
        return ret;
    }
    
    public ASTObject getChildAt(int childIndex) {
        switch(childIndex) {
        case 0: return typeName;
        default: return super.getChildAt(childIndex);
        }
    }
     public String getChildNameAt(int childIndex) {
        switch(childIndex) {
        case 0: return "typeName";
        default: return super.getChildNameAt(childIndex);
        }
    }
     public void setChildAt(int childIndex, ASTObject child) {
        switch(childIndex) {
        case 0: setTypeName((GenTypeName)child); return;
        default: super.setChildAt(childIndex, child); return;
        }
    }
     public int getChildCount() {
        return 1;
    }
    
    public String getDefaultDisplayName() {
        return "TargetPcd()";
    }
    
    //END: Generated from @child and @property
}

