/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.ast;
import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.base.*;
import org.aspectj.compiler.crosscuts.joinpoints.*;
import org.aspectj.compiler.base.cst.*;

import org.aspectj.util.FuzzyBoolean;

/**
 * @grammar id
 * @property FormalDec formalDec
 */

public class VarTypeName extends GenTypeName {
    public String toShortString() {
        return formalDec.getName();
    }

    public boolean matches(Type type) {
        //XXX probably an error condition...
        return false;
    }
    
    public FuzzyBoolean matchesInstance(Type exprType) {
        Type formalType = formalDec.getType();
        if (formalType.isObject()) return FuzzyBoolean.YES;
        
        if (formalType.isAssignableFrom(exprType)) return FuzzyBoolean.YES;
        
        // this is a conditional true based on the needed dynamic test
        if (exprType.isCoercableTo(formalType)) return FuzzyBoolean.MAYBE;
        
        return FuzzyBoolean.NO;
    }

    public JpPlan makePlan(JoinPoint jp, Expr expr) {        
        Type exprType = expr.getType();
        Type formalType = formalDec.getType();
        
        if (matchesInstance(exprType).alwaysFalse()) return JpPlan.NO_PLAN;
        
        JpPlan plan = new JpPlan(jp);
        
        //System.out.println("plan: " + point + ", " + exprType + ", " + formalType);
        
        if (formalType.isObject()) {
            if (exprType.isPrimitive()) {
                expr = exprType.makeObject(expr);
            }
            plan.bindExpr(formalDec, expr);
            return plan;
        }
        
        if (formalType.isAssignableFrom(exprType)) {
            plan.bindExpr(formalDec, expr);
            return plan;
        }
        
        if (exprType.isCoercableTo(formalType)) {
            final AST ast = getAST();
            
            Expr testExpr = ast.makeParen(ast.makeInstanceof(expr, formalType));
            plan.addExprTest(testExpr);
            plan.bindExpr(formalDec, expr);
            return plan;
        }
        
        return JpPlan.NO_PLAN;
    }    
    
    //BEGIN: Generated from @child and @property
    protected FormalDec formalDec;
    public FormalDec getFormalDec() { return formalDec; }
    public void setFormalDec(FormalDec _formalDec) { formalDec = _formalDec; }
    
    public VarTypeName(SourceLocation location, FormalDec _formalDec) {
        super(location);
        setFormalDec(_formalDec);
    }
    protected VarTypeName(SourceLocation source) {
        super(source);
    }
    
    public ASTObject copyWalk(CopyWalker walker) {
        VarTypeName ret = new VarTypeName(getSourceLocation());
        ret.preCopy(walker, this);
        ret.formalDec = formalDec;
        return ret;
    }
    
    
    public String getDefaultDisplayName() {
        return "VarTypeName(formalDec: "+formalDec+")";
    }
    
    //END: Generated from @child and @property
}

