/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.joinpoints;

import org.aspectj.compiler.crosscuts.ast.*;
import org.aspectj.compiler.base.ast.*;

import org.aspectj.compiler.base.TypeManager;
import org.aspectj.compiler.base.CompilerObject;
import org.aspectj.compiler.base.JavaCompiler;

import java.util.*;

/**
 */

public class DynamicPointTest extends CompilerObject {
    public Expr dynamicTest = null;
    public InstanceTest enclosingInstanceTest;
    public InstanceTest calledInstanceTest;

    public DynamicPointTest(JavaCompiler compiler) {
        super(compiler);
    }

//    public void addInstanceTest(InstanceTest instanceTest, boolean useCallThis) {
//        if (useCallThis) {
//            calledInstanceTest =
//                intersectInstanceTests(this.calledInstanceTest, instanceTest);
//        } else {
//            enclosingInstanceTest =
//                intersectInstanceTests(this.enclosingInstanceTest, instanceTest);
//        }
//    }
//
    public void addExprTest(Expr newExprTest) {
        dynamicTest = intersectTests(this.dynamicTest, newExprTest);
    }

    //XXX cut-and-paste from intersect
    public void addTest(DynamicPointTest other) {
        dynamicTest = intersectTests(dynamicTest, other.dynamicTest);
        calledInstanceTest = intersectInstanceTests(calledInstanceTest, other.calledInstanceTest);
        enclosingInstanceTest = intersectInstanceTests(enclosingInstanceTest, other.enclosingInstanceTest);
    }

    public DynamicPointTest intersect(DynamicPointTest other) {
        DynamicPointTest newTest = new DynamicPointTest(getCompiler());

        newTest.dynamicTest = intersectTests(dynamicTest, other.dynamicTest);
        newTest.calledInstanceTest = intersectInstanceTests(calledInstanceTest, other.calledInstanceTest);
        newTest.enclosingInstanceTest = intersectInstanceTests(enclosingInstanceTest, other.enclosingInstanceTest);

        return newTest;
    }

    public DynamicPointTest union(DynamicPointTest other) {
        DynamicPointTest newTest = new DynamicPointTest(getCompiler());

        newTest.dynamicTest = unionTests(dynamicTest, other.dynamicTest);
        newTest.calledInstanceTest = unionInstanceTests(calledInstanceTest, other.calledInstanceTest);
        newTest.enclosingInstanceTest = unionInstanceTests(enclosingInstanceTest, other.enclosingInstanceTest);

        return newTest;
    }

    public DynamicPointTest complement() {
        DynamicPointTest newTest = new DynamicPointTest(getCompiler());

        newTest.dynamicTest = complementTest(dynamicTest);
        newTest.calledInstanceTest = complementInstanceTest(calledInstanceTest);
        newTest.enclosingInstanceTest = complementInstanceTest(enclosingInstanceTest);

        return newTest;
    }



    private InstanceTest intersectInstanceTests(InstanceTest test1, InstanceTest test2) {
        if (test1 == null) return test2;
        if (test2 == null) return test1;
        return test1.intersect(test2);
    }

    private Expr intersectTests(Expr test1, Expr test2) {
        if (test1 == null) return test2;
        if (test2 == null) return test1;
        return getAST().makeBinop("&&", test1, test2);
    }

    private InstanceTest unionInstanceTests(InstanceTest test1, InstanceTest test2) {
        if (test1 == null) return test2;
        if (test2 == null) return test1;
        return test1.union(test2);
    }

    private Expr unionTests(Expr test1, Expr test2) {
        if (test1 == null) return test2;
        if (test2 == null) return test1;
        return getAST().makeBinop("||", test1, test2);
    }

    private InstanceTest complementInstanceTest(InstanceTest instanceTest) {
        if (instanceTest == null) return null;
        return instanceTest.complement();
    }

    private Expr complementTest(Expr test) {
        if (test == null) return null;
        return getAST().makeUnop("!", test);
    }



    public Expr getTestExpr() { //Expr calledThis, Expr enclosingThis) {
        Expr test = dynamicTest;
//        if (calledInstanceTest != null) {
//            test = intersectTests(test, calledInstanceTest.makeExpr(calledThis));
//        }
//        if (enclosingInstanceTest != null) {
//            test = intersectTests(test, enclosingInstanceTest.makeExpr(enclosingThis));
//        }
        return test;
    }


    public boolean hasTest() {
        return dynamicTest != null || calledInstanceTest != null ||
                    enclosingInstanceTest != null;
    }

//    public String toString() {
//        Expr testExpr = getTestExpr(new VarExpr(getCompiler(),"calledThis"), new VarExpr(getCompiler(),"this"));
//        String s;
//        if (testExpr == null) { 
//            s = "null";
//        } else {
//            s = testExpr.toShortString();
//        }
//        return "DynamicPointTest(" + s + ")";
//    }
}

