/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.joinpoints;

import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.crosscuts.ast.*;
import org.aspectj.compiler.crosscuts.*;

import org.aspectj.compiler.base.JavaCompiler;

import java.util.*;

public class ExceptionHandlerJp extends CodeBodyJp {
    CatchClause catchClause;

    public ExceptionHandlerJp(CatchClause catchClause, JoinPoint enclosingJoinPoint) {
        super(enclosingJoinPoint);
        this.catchClause = catchClause;
    }

    static final Kind KIND = new Kind("exception-handler", HANDLER);

    public Kind getKind() { return KIND; }


    protected Expr makeSignatureExpr() {
        return makeSignatureExpr(catchClause);
    }    
 
    public ASTObject getSourceLocation() { return catchClause; }

    public Expr makeThrowableExpr() {
        return getAST().makeVar(catchClause.getFormal());
    }
    
    public Type getTargetType() {
        return null;
    }

    //!!! better than null
    public Dec getTargetDec() { return catchClause.getEnclosingCodeDec(); }

    public Type getResultType() {
        return getTypeManager().voidType;
    }

    public Type getExceptionType() {
        return catchClause.getFormal().getType();
    }
    
    public Exprs makeArgsExprs() {
        return getAST().makeExprs(makeThrowableExpr());
    }

    public boolean canThrow(Type t) {
        //XXX this is wrong
        return t.isUncheckedThrowable();
    }
    
    public Collection getPossibleCheckedExceptions() {
    	//XXX this is wrong
    	return Collections.EMPTY_LIST;
    }



    public Stmts getStmts() {
        Stmt body = catchClause.getBody();
        if (body instanceof BlockStmt) {
            return ((BlockStmt)body).getStmts();
        } else {
            return getAST().makeStmts(body);
        }
    }

    public void setStmts(Stmts stmts) {
        catchClause.setBody(getAST().makeBlock(stmts));
    }

    public FormalDec getFormal() { return catchClause.getFormal(); }


    public String toString() {
        return "handler(" + catchClause.getFormal() + ")";
    }

    //XXX what is this all about
    private static int counter = 0;
    protected String getJoinPointNameModifier() { return "Handler" + counter++; }
}
