

/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */


package org.aspectj.tools.ajde.netbeans;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import javax.swing.*;

import org.openide.util.HelpCtx;
import org.openide.util.actions.CallableSystemAction;
import org.openide.windows.*;
import org.openide.Places;
import org.openide.TopManager;
import org.openide.loaders.DataFolder;
import org.openide.modules.ModuleInstall;
import org.openide.util.Utilities;
import org.openide.util.actions.SystemAction;
//import org.openidex.util.Utilities2;
import org.openide.explorer.*;
import org.openide.explorer.view.*;
import org.openide.nodes.*;
import org.openide.text.*;
import org.openide.*;
import org.openide.options.*;
import org.openide.filesystems.*;
import org.aspectj.ajde.*;

public class Actions {

    public static final CallableSystemAction START_BROWSER = new CallableSystemAction() {

//        private boolean structureViewVisible = false;
        private boolean installed = false;
        private boolean firstInstall = true;

        public StructureViewTab structureViewTab = null;

        public String getName () {
            return "AspectJ Development Environment (AJDE)";
        }

        public JMenuItem getMenuPresenter() {
            JMenuItem menuItem = super.getMenuPresenter();
            if (!installed) {
                BUILD.setEnabled(false);
                RUN.setEnabled(false);
                OPTIONS.setEnabled(false);
                menuItem.setIcon(ForteIcons.INSTANCE.getStopAjdeIcon());
                menuItem.setToolTipText("Start AspectJ Development Environment (AJDE)");
            } else {
                menuItem.setIcon(ForteIcons.INSTANCE.getStartAjdeIcon());
                menuItem.setToolTipText("Stop AspectJ Development Environment (AJDE)");
            }
            return menuItem;
        }

        public void performAction () {
            Workspace editing = TopManager.getDefault().getWindowManager().findWorkspace("Editing"); //NOI18N
            if (editing != null) {
                editing.activate();
            }

            if (!installed) {
                BUILD.setEnabled(true);
                RUN.setEnabled(true);
                OPTIONS.setEnabled(true);

                ForteManager.init();
                structureViewTab = new StructureViewTab(org.aspectj.ajde.TopManager.BROWSER_MANAGER.getBrowserPanel());

                structureViewTab.addComponentListener(new ComponentAdapter() {
                    public void componentHidden(ComponentEvent e)  {
                        if (installed) {
                            performAction();
                        }
                    }
                });
                TopManager.getDefault().getWindowManager().getMainWindow().addWindowListener(
                    new java.awt.event.WindowAdapter() {
                        public void windowDeiconified(WindowEvent e) {
                        }
                });
                Runnable update =
                  new Runnable() {
                    public void run() {
                        toggleStructureView(true);
                        structureViewTab.setVisible(true);
                   }
                };
                if (SwingUtilities.isEventDispatchThread()) {
                    update.run();
                } else {
                    try {
                        SwingUtilities.invokeAndWait(update);
                    } catch (Exception e) {}
                }
                installed = true;
                org.aspectj.ajde.TopManager.BROWSER_MANAGER.updateView();
            } else {
                BUILD.setEnabled(false);
                RUN.setEnabled(false);
                OPTIONS.setEnabled(false);
                toggleStructureView(false);
                installed = false;
            }
            if (firstInstall) {
                org.openide.TopManager.getDefault().getRepository().addRepositoryListener(new RepositoryListener() {
                    public void fileSystemAdded(RepositoryEvent ev) {
                        org.aspectj.ajde.TopManager.BROWSER_MANAGER.updateConfigsList();
                    }
                    public void fileSystemRemoved(RepositoryEvent ev) {
                        org.aspectj.ajde.TopManager.BROWSER_MANAGER.updateConfigsList();
                    }
                    public void fileSystemPoolReordered(RepositoryReorderedEvent ev) {
                        // ignored
                    }
                });
                firstInstall = false;
            }
        }

        public HelpCtx getHelpCtx () {
            return HelpCtx.DEFAULT_HELP;
        }

        protected String iconResource () {
            return "/org/aspectj/ajde/resources/actions/startAjde.gif";
        }

        private void toggleStructureView(boolean show) {
            try {
                JTabbedPane tabbedPane = getExplorerTabbedPane();
                if (show) {
                    tabbedPane.setSelectedIndex(0);
                    tabbedPane.add(structureViewTab, 0);
                    tabbedPane.setSelectedIndex(0);
                    structureViewTab.setVisible(true);
                    tabbedPane.setSelectedComponent(structureViewTab);
                    ExplorerManager.find(structureViewTab);
                } else {
                        if (tabbedPane.getComponentAt(0).getName().equals(StructureViewTab.NAME)) {
//                            System.err.println("> removing");
                            tabbedPane.remove(0);
                        }
                }
            } catch (Exception e) {
                System.err.println("> AJDE ERROR: could not toggle structure view: " + e.getMessage());
                e.printStackTrace();
            }
        }
    };


    /**
     * TODO: this way of finding the explorer tabbed main is completely stupid. Figure out the
     * right API to use.
     */
    private static JTabbedPane getExplorerTabbedPane() {
        Workspace[] workspaces = TopManager.getDefault().getWindowManager().getWorkspaces();
        for (int i = 0; i < workspaces.length; i++) {
            if (workspaces[i].getName().equals("Editing")) {
                Object[] modes = workspaces[i].getModes().toArray();
                for (int j = 0; j < modes.length; j++) {
                    Mode currMode = (Mode)modes[j];
                    if (currMode.getDisplayName().equals("Explorer")) {
                        TopComponent[] tops = currMode.getTopComponents();
                        if (tops != null && tops[0] != null && tops[0].getParent() != null) {
                            return (JTabbedPane)tops[0].getParent();
                        }
                    }
                }
            }
        }
        return null;
    }

    public static final CallableSystemAction BUILD = new CallableSystemAction() {

        public String getName () {
            return "Compile with ajc";
        }

        public HelpCtx getHelpCtx () {
            return HelpCtx.DEFAULT_HELP;
        }

        protected String iconResource () {
            return "/org/aspectj/ajde/resources/actions/build.gif";
        }

        public void performAction () {
            ForteManager.INSTANCE.build();
        }
    };

    public static final CallableSystemAction OPTIONS = new CallableSystemAction() {

        public String getName () {
            return "Options...";
        }

        public HelpCtx getHelpCtx () {
            return HelpCtx.DEFAULT_HELP;
        }

        protected String iconResource () {
            return "/org/aspectj/ajde/resources/actions/browseroptions.gif";
        }

        public void performAction () {
            org.aspectj.ajde.TopManager.showOptionsFrame();
        }
    };

    public static final CallableSystemAction RUN = new CallableSystemAction() {

        public String getName () {
            return "Run";
        }

        public HelpCtx getHelpCtx () {
            return HelpCtx.DEFAULT_HELP;
        }

        protected String iconResource () {
            return "/org/aspectj/ajde/resources/actions/execute.gif";
        }

        public void performAction () {
            ForteManager.INSTANCE.run();
        }
    };

    public static final CallableSystemAction DEBUG = new CallableSystemAction() {

        public String getName () {
            return "Debug";
        }

        public HelpCtx getHelpCtx () {
            return HelpCtx.DEFAULT_HELP;
        }

        protected String iconResource () {
            return "/org/aspectj/ajde/resources/actions/debug.gif";
        }

        public void performAction () {
            ForteManager.INSTANCE.debug();
        }
    };

    static class StructureViewComponent extends TopComponent {
        public StructureViewComponent(JPanel panel) {
            this.setName("AspectJ Browser");
            this.setLayout(new BorderLayout());
            this.add(panel, BorderLayout.CENTER);
        }

        public Image getIcon() {
            return ((ImageIcon)ForteIcons.INSTANCE.getStartAjdeIcon()).getImage();
        }

        public SystemAction[] getSystemActions() {
            return new SystemAction[0];
        }
    }

    static class StructureViewTab extends ExplorerPanel {
        public static final String NAME = "AspectJ Browser";
        public StructureViewTab(JPanel panel) {
            this.setName(NAME);
            this.setLayout(new BorderLayout());
            this.add(panel, BorderLayout.CENTER);
            this.add(panel);
        }

        public Image getIcon() {
            return ((ImageIcon)ForteIcons.INSTANCE.getStartAjdeIcon()).getImage();
        }
    }
}