

/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.tools.ajde.netbeans;

import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.text.StyledDocument;
import org.openide.TopManager;
import org.openide.filesystems.*;
import org.openide.loaders.*;
import org.openide.cookies.*;
import org.openide.text.Line;
import org.aspectj.ajde.editor.AjdeEditor;
import org.aspectj.ajde.ui.ErrorHandler;

public class EditorAdapter extends AjdeEditor {

    public String getCurrFile() {
        return null;
    }

    public String getCurrSourceFilePath() {
        return "";
    }

    public void showSourceLine(int lineNumber, boolean highlight) {
        showSourceLine(getCurrFile(), lineNumber, true);
    }

    public void showSourceLine(String filePath, int lineNumber, boolean highlight) {
        if (filePath == "") {
            return;
        }
        try {
            filePath = filePath.replace('\\', '/');
            String currFile = filePath.substring(filePath.lastIndexOf('/')+1, filePath.lastIndexOf('.'));
            String packageName = "";

            Vector filesystems = getFilesystemPaths();
            Iterator it = filesystems.iterator();
            while (it.hasNext()) {
                String filesystem = (String)it.next();
                filesystem = filesystem.replace('\\', '/');
                if (filePath.startsWith(filesystem)) {
                    try {
                        packageName = filePath.substring(filesystem.length() + 1, filePath.lastIndexOf('/')).replace('/', '.');
                    } catch (StringIndexOutOfBoundsException sioobe) { }
                }
            }

            FileObject fo = null;
            if (filePath.endsWith(".aj")) {
                fo = TopManager.getDefault().getRepository().find(packageName, currFile, "aj");
            } else {
                fo = TopManager.getDefault().getRepository().find(packageName, currFile, "java");
            }

            DataObject dataObject = TopManager.getDefault().getLoaderPool().findDataObject(fo);
            EditorCookie ec = (EditorCookie)(dataObject.getCookie(EditorCookie.class));
            ec.open();

            int lineToSeekTo;
            if (lineNumber < 0) {
                lineToSeekTo = 1;
            } else {
                lineToSeekTo = lineNumber - 1;
            }

            if (lineNumber != 1) {
              ec.getLineSet().getCurrent(lineToSeekTo).markCurrentLine();
            }
            ec.getLineSet().getCurrent(lineToSeekTo).show(Line.SHOW_GOTO);
        } catch (Exception e) {
            ErrorHandler.handleError("Could not seek to line in file: " + filePath + ".", e);
            System.out.println("> ajde could not open file: " + filePath + ", message: " + e.getMessage());
            e.printStackTrace();
        }
    }

    /**
     * Not implemented.
     */
    public void pasteToCaretPos(String text) { }

    /**
     * Not implemented.
     */
    public void showSourcelineAnnotation(String filePath, int lineNumber, JPopupMenu annotation) { }

    public void saveContents() throws IOException {
    }

    public JPanel getPanel() {
        return null;
    }

    private Vector getFilesystemPaths() {
        Vector filesystemPaths = new Vector();
        Repository repository = TopManager.getDefault().getRepository();
        Enumeration filesystems = repository.getFileSystems();
        File f = null;
        while (filesystems.hasMoreElements()) {
            FileSystem fs = (FileSystem)filesystems.nextElement();
            if (fs instanceof org.netbeans.core.ExLocalFileSystem) {
                f = new File(fs.getSystemName());
                filesystemPaths.add(f.getAbsolutePath());
            }
        }
        return filesystemPaths;
    }
}