
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.tools.ajde.netbeans;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import org.openide.windows.*;
import org.aspectj.ajde.*;
import org.aspectj.ajde.compiler.*;
import org.aspectj.ajde.ui.ErrorHandler;

public class ForteCompilerMessages implements CompilerMessages {

    private CompilerMessageWindow cmw = new CompilerMessageWindow();
    private TopComponent comp = new TopComponent();

    public ForteCompilerMessages() {
        comp.setLayout(new BorderLayout());
        comp.add(cmw, BorderLayout.CENTER);
        comp.setPreferredSize(new Dimension (600, 200));
        comp.setName("ajc compiler messages");
    }

    public void addMessage(String message, String file, int line, int col, int kind) {
        cmw.addMessage(message, null, file, line);
        comp.open();
        int x = org.openide.TopManager.getDefault().getWindowManager().getMainWindow().getX() +
                org.openide.TopManager.getDefault().getWindowManager().getMainWindow().getHeight();
        int y = org.openide.TopManager.getDefault().getWindowManager().getMainWindow().getY() +
                org.openide.TopManager.getDefault().getWindowManager().getMainWindow().getWidth();
        comp.setLocation(x, y);
        comp.requestFocus();
    }

    public void clearMessages() {
        comp.close();
        cmw.clearMessages();
    }

    class CompilerMessageWindow extends JPanel {
        JScrollPane jScrollPane1 = new JScrollPane();
        JScrollPane jScrollPane2 = new JScrollPane();
        JList list = new JList();
        DefaultListModel listModel = new DefaultListModel();
        BorderLayout borderLayout1 = new BorderLayout();

        public CompilerMessageWindow() {
            try {
                jbInit();
            }
            catch(Exception e) {
                e.printStackTrace();
            }
            list.setModel(listModel);

             MouseListener mouseListener = new MouseAdapter() {
                 public void mouseClicked(MouseEvent e) {
                     if (e.getClickCount() == 2) {
                         int index = list.locationToIndex(e.getPoint());
                         CompilerMessage cm = (CompilerMessage)listModel.getElementAt(index);
                         TopManager.EDITOR_MANAGER.showSourceLine(cm.file, cm.line, true);
                      }
                 }
             };
             list.addMouseListener(mouseListener);

        }

        public void addMessage(String message, Object icon, String file, int line) {

            listModel.addElement(new CompilerMessage(message, file, line));
        }

        public void clearMessages() {
            listModel.clear();
        }

        private void jbInit() throws Exception {
            this.setLayout(borderLayout1);
            this.add(jScrollPane1, BorderLayout.CENTER);
            jScrollPane1.getViewport().add(jScrollPane2, null);
            jScrollPane2.getViewport().add(list, null);
        }
    }

    class CompilerMessage {
        public String message = "";
        public String file = "";
        public int line = 1;

        public CompilerMessage(String message, String file, int line) {
            this.message = message;
            this.file = file;
            this.line = line;
        }

        public String toString() {
            return this.message;
        }
    }

}
