
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.tools.ajde.netbeans;

import java.util.Enumeration;
import java.io.*;
import java.util.*;
import org.aspectj.tools.ide.*;
import org.openide.util.HelpCtx;
import org.openide.util.actions.CallableSystemAction;
import org.openide.windows.*;
import org.openide.loaders.DataFolder;
import org.openide.modules.ModuleInstall;
import org.openide.util.actions.SystemAction;
//import org.openidex.util.Utilities2;
import org.openide.explorer.*;
import org.openide.execution.*;
import org.openide.explorer.view.*;
import org.openide.nodes.*;
import org.openide.text.*;
import org.openide.filesystems.*;
import org.openide.loaders.*;
import org.openide.cookies.*;
import org.netbeans.core.*;
import org.openide.src.*;
import org.openide.filesystems.*;
import javax.swing.*;
import javax.swing.text.StyledDocument;
import org.aspectj.ajde.*;
import org.aspectj.util.ConfigFileUtil;
import org.aspectj.ajde.ui.ErrorHandler;
import org.aspectj.ajde.ui.*;

public class ForteManager implements IdeManager {

    public static ForteManager INSTANCE = null;
    ProjectProperties projectProperties = new ForteProjectProperties();

    public static void init() {
        ForteManager.INSTANCE = new ForteManager();
    }

    private ForteManager() {
        org.aspectj.ajde.TopManager.initAjde(this, new EditorAdapter(), new ForteCompilerMessages(), projectProperties, true);
    }

    public void setStatusText(String text) {
        org.openide.TopManager.getDefault().setStatusText(text);
    }

    public void setEditorStatusText(String text) { }

    /**
     * Not implemented.
     */
    public void editConfigFile(String configFile) { }

    public String getDumpFilePath() {
        return null;
    }

    public Icons getIcons() {
        return ForteIcons.INSTANCE;
    }

    public List getConfigFiles() {
        List configs = new ArrayList();
//        configs.add(ProjectProperties.DEFAULT_CONFIG_NAME);
        Repository repository = org.openide.TopManager.getDefault().getRepository();
        Enumeration filesystems = repository.getFileSystems();
        while (filesystems.hasMoreElements()) {
            FileSystem fs = (FileSystem)filesystems.nextElement();
            if (fs instanceof ExLocalFileSystem) {
                ExLocalFileSystem exfs = (ExLocalFileSystem)fs;
                if (new File(exfs.getSystemName()).exists()) {
                    configs.addAll(ConfigFileUtil.getLstFilesInDir(exfs.getSystemName()));
                }
            }
        }
        if (configs.size() == 0) {
            ErrorHandler.handleWarning("Could not find any \".lst\" in the project filesystems.");
        }
        return configs;
    }

    /**
     * TODO: fix this ridiculous way of finding class and executor.
     */
    public void run() {
        try {
            String mainClass = projectProperties.getClassToRun();
//            JOptionPane.showInputDialog("Please type the fully qualified name of the class to be executed: ");
            if (mainClass == null || mainClass.length() == 0) {
                ErrorHandler.handleWarning("No main class specified, please select a class to run.", TopManager.IDE_MANAGER.getRootFrame());
                TopManager.showOptionsFrame(OptionsFrame.BUILD_PROPERTIES);
            } else {
                String classpath = projectProperties.getOutputPath() + File.pathSeparator + projectProperties.getClasspath();
                System.err.println("> cp: " + classpath);
                Enumeration e = ProcessExecutor.executors();
                boolean succeeded = false;
                while (e.hasMoreElements()) {
                    Object o = e.nextElement();
                    if (o instanceof ProcessExecutor &&  // sooooo dumb, ah well
                        o.getClass().toString().equals("class org.netbeans.modules.java.JavaProcessExecutor")) {
                        ((ProcessExecutor)o).setClassPath(new NbClassPath(classpath));
                        ((ProcessExecutor)o).execute(new ExecInfo(mainClass));
                        succeeded = true;
                    }
                }
            }
        } catch (Exception e) {
            ErrorHandler.handleError("Main clas not found.", e);
        }
    }

    public void build() {
        if (projectProperties.getOutputPath() != null) {
            TopManager.COMPILER_MANAGER.build();
        } else {
            ErrorHandler.handleWarning("No output path defined, please select an output path.");
            TopManager.showOptionsFrame(OptionsFrame.BUILD_PROPERTIES);
        }
    }

    /**
     * Disabled.
     */
    public void debug() {
//
//        setStatusText("ajdb starting...");
//
//        PrintStream out = System.out;
//        PrintStream err = System.err;
//        try {
//            DebuggerAdapter.install();
//            //TODO: This can stay empty
//            String classToRun = projectProperties.getClassToRun();
//            String sourcePath = ((ForteProjectProperties)projectProperties).getDebuggerSourcePath();
//            String classPath =  ((ForteProjectProperties)projectProperties).getDebuggerClasspath();
//            String workingdir = projectProperties.getWorkingDir();
//
//            System.err.println("> classToRun: :" + classToRun);
//            System.err.println("> sourcePath: :" + sourcePath);
//            System.err.println("> classPath: :" + classPath);
//            System.err.println("> workingdir: :" + workingdir);
//
//            String[] args = { "-workingdir", workingdir, "-classpath", classPath };
//            org.aspectj.debugger.ide.IDEInterface ideInterface = //new org.aspectj.debugger.ide.IDEInterfaceImpl();
//                new DebuggerAdapter(); //.singletonInstance;
//            org.aspectj.debugger.ide.SourceShower shower =
//                ((DebuggerAdapter)ideInterface).getSourceShower();
//
//          org.aspectj.debugger.ide.IDEDebugger.launch
//              (
//               classToRun, //jBuilderManager.getProjectProperties().getClassToRun(),
//               sourcePath, //jBuilderManager.getProjectProperties().getProjectSourcePath(),
//               classPath, //jBuilderManager.getProjectProperties().getClasspath(),
//               workingdir,
//               org.aspectj.debugger.base.Modes.FORTE, //org.aspectj.debugger.base.Modes.JBUILDER4,
//               args,
//               shower,
//               ideInterface); //org.aspectj.tools.ajde.jbuilder4.debugger.JBuilderIDEInterface.singletonInstance);
//
//        } catch (Exception e) {
//            ErrorHandler.handleError("error running ajdb");
//        }
//        setStatusText("");
    }

    public String getDefaultConfigFile() {
        List configs = getConfigFiles();
        if (configs != null && configs.size() > 0) {
            return (String)configs.get(0);
        } else {
            return null;
        }
    }

    public List getBuildOptions() {
        List options = new ArrayList();
        options.add("-g");
        String classpath = projectProperties.getClasspath();
        if (classpath != "") {
            options.add("-classpath");
            options.add(classpath);
        }
        String outpath = projectProperties.getOutputPath();
        if (outpath != "") {
            options.add("-d");
            options.add(outpath);
        }
        return options;
    }

    public JFrame getRootFrame() {
//        return org.openide.TopManager.getDefault().get
        return null;
    }

    public void saveAll() {
        org.openide.TopManager.getDefault().saveAll();
    }

    public void mountConfigFile(String configFile) {

    }

    public void removeConfigFile(String configFile) {

    }

    public void showMessages() {
        // ignore
    }

    public void hideMessages() {
        // ignore
    }

    private static String findClassNameToRun() {
        return "";
        /*
        String newClassToRun = "";
        Enumeration enum = org.openide.TopManager.getDefault().getRepository().getFileSystems();
        while (enum.hasMoreElements()) {
            FileSystem fileSystem = (FileSystem)enum.nextElement();
            if (!(fileSystem instanceof ExLocalFileSystem)) continue;
            ExLocalFileSystem exLocalFileSystem = (ExLocalFileSystem)fileSystem;
            File rootFile = exLocalFileSystem.getRootDirectory();
            List dirs = new Vector();
            List files = new Vector();
            dirs.add(rootFile);
            Vector classElements = new Vector();
            while (dirs.size() > 0) {
                File dir = (File)dirs.remove(0);
                File[] fileArray = dir.listFiles(new FileFilter() {
                        public boolean accept(File f) {
                            return f.isDirectory() || f.getName().endsWith(".java");
                        }
                    });
                for (int i = 0; i < fileArray.length; i++) {
                    File file = fileArray[i];
                    if (file.isDirectory()) {
                        dirs.add(file);
                    } else {
                        DataObject dataObject =
                            DebuggerAdapter.singletonInstance.dataObject(fileObject(rootFile.getAbsolutePath(), file.getAbsolutePath()));

                        if (dataObject == null) {
                            continue;
                        }
                        SourceElement source =
                            ((SourceCookie)dataObject.getCookie(SourceCookie.class)).getSource();
                        if (source == null) {
                            continue;
                        }
                        ClassElement[] classes = null;
                        try {
                            classes = source.getClasses();
                        } catch (Throwable t) {
                            classes = null;
                        }
                        if (classes == null) {
                            continue;
                        }
                        for (int j = 0; j < classes.length; j++) {
                            ClassElement ce = classes[j];
                            if (ce.hasMainMethod()) {
                                classElements.add(DebuggerAdapter.className(ce));
                                break;
                            }
                        }
                    }
                }
            }
            SortedSet sortedElements = new TreeSet(classElements);
            if (sortedElements.size() == 0) {

            } else if (sortedElements.size() == 1) {
                return sortedElements.first()+"";
            } else {
                Object[] classes = new Object[sortedElements.size()];
                int i = 0;
                Iterator sortedIter = sortedElements.iterator();
                while (sortedIter.hasNext()) {
                    classes[i++] = sortedIter.next();
                }
                Object o = JOptionPane.showInputDialog
                    (null,
                     "Please select a class to run",
                     "Select class",
                     JOptionPane.QUESTION_MESSAGE,
                     null,
                     classes,
                     classes[0]);
                if (o != null) {
                    return null;
                }
            }
        }
        return newClassToRun;
         */
    }

    private static FileObject fileObject(String baseDir, String fullSourceName) {
        baseDir = baseDir.replace('\\', '/');
        fullSourceName = fullSourceName.replace('\\', '/');
        String relativePath = fullSourceName.substring(baseDir.length()+1, fullSourceName.length());//.replace('/', '.');

        int ilastSlash = relativePath.lastIndexOf('/');
        int ilastDot = relativePath.lastIndexOf('.');

        if (ilastDot == -1 || ilastDot == relativePath.length()-1) {
            return null;
        }

        String packageName = ilastSlash != -1 ?
            relativePath.substring(0, ilastSlash).replace('/', '.') :
            "";
        String fileName = relativePath.substring(ilastSlash+1, ilastDot);
        String extension = relativePath.substring(ilastDot+1);

        if (!"java".equals(extension)) {
            return null;
        }
        return org.openide.TopManager.getDefault().getRepository().find(packageName, fileName, extension);
    }
}
