
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.tools.ajde.netbeans;

import java.util.*;
import java.io.*;
import javax.swing.*;
import org.aspectj.tools.ide.*;
import org.openide.util.HelpCtx;
import org.openide.util.actions.CallableSystemAction;
import org.openide.windows.*;
import org.openide.loaders.DataFolder;
import org.openide.modules.ModuleInstall;
import org.openide.util.actions.SystemAction;
//import org.openidex.util.Utilities2;
import org.openide.explorer.*;
import org.openide.execution.*;
import org.openide.explorer.view.*;
import org.openide.options.*;
import org.openide.nodes.*;
import org.openide.text.*;
import org.openide.filesystems.*;
import org.openide.loaders.*;
import org.openide.cookies.*;
import org.netbeans.core.*;
import org.openide.filesystems.*;
import org.aspectj.ajde.*;

public class ForteProjectProperties extends ProjectProperties {

    public boolean isGlobalMode() {
        return true;
    }

    public String getWorkingDir() {
        return getProjectDir();
    }

    public String getProjectDir() {
        Repository repository = org.openide.TopManager.getDefault().getRepository();
        Enumeration filesystems = repository.getFileSystems();
        File f = null;
        while (filesystems.hasMoreElements()) {
              FileSystem fs = (FileSystem)filesystems.nextElement();
              if (fs instanceof org.netbeans.core.ExLocalFileSystem) {
                    f = new File(fs.getSystemName());
                    if (f.getPath().endsWith("ajworkingdir")) {
                        return f.getAbsolutePath();
                    }
              }
        }
        return null;
    }

    public String getProjectSourcePath() {
        Vector filesystemPaths = new Vector();
        Enumeration filesystems = org.openide.TopManager.getDefault().getRepository().getFileSystems();
        File f = null;
        String paths = "";
        while (filesystems.hasMoreElements()) {
            FileSystem fs = (FileSystem)filesystems.nextElement();
            if (fs instanceof org.openide.filesystems.LocalFileSystem &&
                !fs.getSystemName().endsWith("ajworkingdir")) {
                f = new File(fs.getSystemName());
                paths += f.getAbsolutePath() + ";";
            }
        }
        return paths;
    }

    public List getProjectFiles() {
        List projectFiles = new ArrayList();
        projectFiles.add("yo.java");
        return projectFiles;
    }

    public String getClasspath() {
        Vector filesystemPaths = new Vector();
        Enumeration filesystems = org.openide.TopManager.getDefault().getRepository().getFileSystems();
        File f = null;
        String jars = "";
        while (filesystems.hasMoreElements()) {
            FileSystem fs = (FileSystem)filesystems.nextElement();
            if (fs instanceof org.openide.filesystems.JarFileSystem) {
                f = new File(fs.getSystemName());
                jars += f.getAbsolutePath() + ";";
            }
        }
        return jars + System.getProperty("netbeans.home") + "/lib/ext/aspectjrt.jar";
    }

    public String getDebuggerClasspath() {
        return getWorkingDir() + ";" + getClasspath();
    }

    public String getOutputPath() {
        String outputPath = getProperty("build.outputpath");
        if (outputPath == null) {
            return null;
        } else {
            return outputPath;
        }
    }

    public String getDebuggerSourcePath() {
        return getOutputPath();
    }

    public String getBootClasspath() {
        return null;
    }
}