package org.aspectj.tools.ajde.netbeans;

import java.awt.event.*;
import java.awt.Image;
import java.awt.BorderLayout;

import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.ImageIcon;
import javax.swing.SwingUtilities;

import org.openide.TopManager;
import org.openide.windows.*;
import org.openide.explorer.*;
import org.openide.explorer.view.*;
import org.openide.filesystems.*;
 
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.SystemAction;
import org.openide.util.actions.CallableSystemAction;

import org.aspectj.ajde.*;
import org.aspectj.ajde.views.*;

/** Action that can always be invoked and work procedurally.
 *
 * @author  Phil Sager
 */
public class StartBrowserAction extends CallableSystemAction {
    
    private boolean installed = false;
    private boolean firstInstall = true;
    
    public StructureViewTab structureViewTab = null;
    
    public void performAction() {
        Workspace editing = org.openide.TopManager.getDefault().getWindowManager().findWorkspace("Editing"); //NOI18N
        
        if (editing != null) {
            editing.activate();
        }
        
        if (!installed) {
            //            BUILD.setEnabled(true);
            //            RUN.setEnabled(true);
            //            OPTIONS.setEnabled(true);
            
            ForteManager.init();
            structureViewTab = new StructureViewTab(org.aspectj.ajde.TopManager.BROWSER_MANAGER.getBrowserPanel());
            
            structureViewTab.addComponentListener(new ComponentAdapter() {
                public void componentHidden(ComponentEvent e)  {
                    if (installed) {
                        performAction();
                    }
                }
            });
            org.openide.TopManager.getDefault().getWindowManager().getMainWindow().addWindowListener(
            new java.awt.event.WindowAdapter() {
                public void windowDeiconified(WindowEvent e) {
                }
            });
            Runnable update = new Runnable() {
                public void run() {
                    toggleStructureView(true);
                    structureViewTab.setVisible(true);
                }
            };
            if (SwingUtilities.isEventDispatchThread()) {
                update.run();
            } else {
                try {
                    SwingUtilities.invokeAndWait(update);
                } catch (Exception e) {}
            }
            installed = true;
            org.aspectj.ajde.TopManager.BROWSER_MANAGER.updateView();
        } else {
            //            BUILD.setEnabled(false);
            //            RUN.setEnabled(false);
            //            OPTIONS.setEnabled(false);
            toggleStructureView(false);
            installed = false;
        }
        if (firstInstall) {
            org.openide.TopManager.getDefault().getRepository().addRepositoryListener(new RepositoryListener() {
                public void fileSystemAdded(RepositoryEvent ev) {
                    org.aspectj.ajde.TopManager.BROWSER_MANAGER.updateConfigsList();
                }
                public void fileSystemRemoved(RepositoryEvent ev) {
                    org.aspectj.ajde.TopManager.BROWSER_MANAGER.updateConfigsList();
                }
                public void fileSystemPoolReordered(RepositoryReorderedEvent ev) {
                    // ignored
                }
            });
            firstInstall = false;
        }
    }
    
    public String getName() {
        return NbBundle.getMessage(StartBrowserAction.class, "LBL_StartBrowserAction");
    }
    
    protected String iconResource() {
        return "/org/aspectj/ajde/resources/actions/startAjde.gif";
    }
    
    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;
        // If you will provide context help then use:
        // return new HelpCtx (StartBrowserAction.class);
    }
    
    private void toggleStructureView(boolean show) {
        try {
            JTabbedPane tabbedPane = getExplorerTabbedPane();
            if (show) {
                tabbedPane.setSelectedIndex(0);
                tabbedPane.add(structureViewTab, 0);
                tabbedPane.setSelectedIndex(0);
                structureViewTab.setVisible(true);
                tabbedPane.setSelectedComponent(structureViewTab);
                ExplorerManager.find(structureViewTab);
            } else {
                if (tabbedPane.getComponentAt(0).getName().equals(StructureViewTab.NAME)) {
                    //                            System.err.println("> removing");
                    tabbedPane.remove(0);
                }
            }
        } catch (Exception e) {
            System.err.println("> AJDE ERROR: could not toggle structure view: " + e.getMessage());
            e.printStackTrace();
        }
    }

    /**
     * TODO: this way of finding the explorer tabbed main is completely stupid. Figure out the
     * right API to use.
     */
    private static JTabbedPane getExplorerTabbedPane() {
        Workspace[] workspaces = org.openide.TopManager.getDefault().getWindowManager().getWorkspaces();
        for (int i = 0; i < workspaces.length; i++) {
            if (workspaces[i].getName().equals("Editing")) {
                Object[] modes = workspaces[i].getModes().toArray();
                for (int j = 0; j < modes.length; j++) {
                    Mode currMode = (Mode)modes[j];
                    if (currMode.getDisplayName().equals("Explorer")) {
                        TopComponent[] tops = currMode.getTopComponents();
                        if (tops != null && tops[0] != null && tops[0].getParent() != null) {
                            return (JTabbedPane)tops[0].getParent();
                        }
                    }
                }
            }
        }
        return null;
    }    
    
/** Perform extra initialization of this action's singleton.
 * PLEASE do not use constructors for this purpose!
 * protected void initialize () {
 * super.initialize ();
 * putProperty (Action.SHORT_DESCRIPTION, NbBundle.getMessage (StartBrowserAction.class, "HINT_Action"));
 * }
 */


static class StructureViewComponent extends TopComponent {
    public StructureViewComponent(JPanel panel) {
        this.setName("AspectJ Browser");
        this.setLayout(new BorderLayout());
        this.add(panel, BorderLayout.CENTER);
    }
    
    public Image getIcon() {
        return ((ImageIcon)ForteIcons.INSTANCE.getStartAjdeIcon()).getImage();
    }
    
    public SystemAction[] getSystemActions() {
        return new SystemAction[0];
    }
}

static class StructureViewTab extends ExplorerPanel {
    public static final String NAME = "AspectJ Browser";
    public StructureViewTab(JPanel panel) {
        this.setName(NAME);
        this.setLayout(new BorderLayout());
        this.add(panel, BorderLayout.CENTER);
        this.add(panel);
    }
    
    public Image getIcon() {
        return ((ImageIcon)ForteIcons.INSTANCE.getStartAjdeIcon()).getImage();
    }
}
}
