/* -*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */
package org.aspectj.util;

import java.util.*;

public class ArgumentScanner {

    private final static char quote = '"';

    public List scanToList(String str) {
        str = (str == null ? "" : str.trim()) + " ";
        List args = new Vector();
        boolean inQuotes = false;
        boolean wasInQuotes = false;
        boolean add = false;
        int start = 0;
        int i = -1;
        try {
            for (i = start; i < str.length(); i++) {
                char c = str.charAt(i);
                if (Character.isWhitespace(c) && !inQuotes) {
                    args.add(str.substring(start,i-(wasInQuotes?1:0)));
                    start = i+1;
                } else if (c == quote && (inQuotes = !(wasInQuotes = inQuotes))) {
                    start = i+1;
                } else if (c == '\\') {
                    i++;
                }
            }
        } catch (IndexOutOfBoundsException ioobe) {
            ioobe.printStackTrace();
            throw new RuntimeException("at " + i);
        }
        if (inQuotes) {
            throw new RuntimeException("Unmatched quotes");
        }
        return args;
    }

    public String[] scanToArray(String str) {
        List list = scanToList(str);
        String[] args = new String[list.size()];
        for (int i = 0; i < args.length; i++) {
            args[i] = list.get(i) + "";
        }
        return args;
    }

    static String[] strings = {
        " a a \"asdf\"",
    };
    public static void main(String[] args) {
        for (int i = 0; i < strings.length; i++) {
            test(strings[i]);
        }
    }
    static void test(String argsStr) {
        List newList = new ArgumentScanner().scanToList(argsStr);
        System.out.println("argsStr=" + argsStr);
        System.out.println("newList=" + newList);
    }
}
