/*- Mode: JDE; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the debugger and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 */

package org.aspectj.util.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * A javax.swing.JDialog that is centered in the screen using
 * CenterHelper.center.
 *
 * @see CenterHelper
 * @author Jeff Palm
 */
public class CenteredJDialog extends JDialog {

    /**
     * Creates a new centerd dialog based on Frame f with String title.
     * 
     * @param f       Frame on which to center this dialog.
     * @param title   title of the dialog
     * @param modal   see description in javax.swing.JDialog
     */
    public CenteredJDialog(Frame f, String title, boolean modal) {
        super(f, title, modal);
    }

    /**
     * Packs and centers this component.
     *
     * @see CenterHelper#center
     */
    public void pack() {
        super.pack();
        CenterHelper.center(this);
    }

    /**
     * Create a javax.swing.JRootPane that exits when ESC is pressed.
     */
    protected JRootPane createRootPane() {
        JRootPane rootPane = new JRootPane();
        ActionListener listener = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
              setVisible(false);
              dispose();
        }};
        KeyStroke stroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);
        rootPane.registerKeyboardAction
            (listener, stroke,JComponent.WHEN_IN_FOCUSED_WINDOW);
        return rootPane;
    }

    /**
     * First packs and then shows this component.
     */
    public void show() {
        pack();
        super.show();
    }
}
