#!/bin/bash
#
#                 Sun Public License Notice
# 
# The contents of this file are subject to the Sun Public License
# Version 1.0 (the "License"). You may not use this file except in
# compliance with the License. A copy of the License is available at
# http://www.sun.com/
# 
# The Original Code is NetBeans. The Initial Developer of the Original
# Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
# Microsystems, Inc. All Rights Reserved.
#
# Contributors: John Cerney, Jan Lahoda



help() {
   cat 1>&2 <<HELP_END
Creates dll from object files. Supports only cygwin, to override, use "-f".
Usage:
create_dll [-e entry_point] -o output.dll object_files

Parameters:
-e --entry-point specify entry point of the dll (defaults to _dll_entry@12)
-o --output      specify output ".dll" file required
-l --lib-path    specify directory where 'libcygwin.a' is (defaults to /lib)
-f --force       do not check cygwin
-h --help        this help

HELP_END

exit;

}

translate_directories() {
   result=
   for file in $1; do
      slashes=`echo $file | tr '\\\\' '/'`;
      colon=`echo $slashes | tr -d ':'`; #This will remove ALL ':' in WHOLE filename.
      result="$result /cygdrive/$colon";
   done;
   echo $result;
}

LIBPATH=/lib

entry_point=_dll_entry@12

verbose=

for param in "$@"; do
   if test "$verbose" = "TRUE"; then
       echo "param=" $param;
   fi;
   case "$param" in
                 -e) follows=ENTRY;;
      --entry-point) follows=ENTRY;;
                 -o) follows=OUTPUT;;
           --output) follows=OUTPUT;;
                 -f) force=TRUE;;
            --force) force=TRUE;;
                 -l) follows=LIBPATH;;
         --lib-path) follows=LIBPATH;;
                 -v) verbose=TRUE;;
          --verbose) verbose=TRUE;;
                 -h) help;;
             --help) help;;

       *) if test "$verbose" = "TRUE"; then
              echo $follows;
          fi;
          case $follows in
              ENTRY) entry_point="$param";;
             OUTPUT) output="$param";;
            LIBPATH) LIBPATH="$param";;
                  *) objects="$objects $param";;
          esac;
          follows=;;

   esac;
done;

if test "$verbose" = "TRUE"; then
   echo "Parameters parsing done.";
fi;

case `uname` in
   *CYGWIN*) ;;
          *) if test "$force" != "TRUE"; then
                echo "This script could be used only with cygwin. Use '-f' to override.";
                exit 1;
             fi;;
esac;

if test "$output" == ""; then
   echo "$0: No output dll file specified.";
   help;
fi;

output="`translate_directories \"$output\"`";

if test "$objects" == ""; then
   echo "$0: No object files specified.";
   help;
fi;

objects="`translate_directories \"$objects\"`";

case $output in
     *.a) output_base=`dirname $output`/`basename $output .a`;
          a_file="$output";
          output="$output_base.dll";;
   *.dll) output_base=`dirname $output`/`basename $output .dll`;
          a_file="$output_base.a";;
       *) echo 1>&2 "The extension of file $output is not known";
          exit 1;;
esac;     

def_file="$output_base.def";

touch $def_file
gcc -mno-cygwin -Wl,--out-implib,$a_file -Wl,--output-def $def_file -Wl,--add-stdcall-alias -shared $objects -o $output

