/*
 * Copyright (c) 2002 PARC Inc.  All rights reserved.
 *
 * Use and copying of this software and preparation of derivative works based
 * upon this software are permitted.  Any distribution of this software or
 * derivative works must comply with all applicable United States export
 * control laws.
 *
 * This software is made available AS IS, and PARC Inc. makes no
 * warranty about the software, its performance or its conformity to any
 * specification.
 */

package com.xerox.printing;

import java.util.*;
import java.io.*;

import org.aspectj.lang.*;

public aspect BufferTest {
    /**
     * When PrinterBuffer.capacity(int) is called,
     * test it with repeatedly with a set of input
     * (validating the result) and then continue with
     * the original call.
     */
    int around(int original, PrinterBuffer buffer) :
        call(int PrinterBuffer.capacity(int)) && args(original)
        && target(buffer) {
        int[] input = new int[] { 0, 1, 10, 1000, -1, 4096 };
        for (int i = 0; i < input.length; i++) {
            int result = proceed(input[i], buffer); // invoke test
            validateResult(buffer, input[i], result);
        }
        return proceed(original, buffer); // do original processing
    }

    void validateResult(PrinterBuffer buffer, int input, int result) {
        Util.log("validating input=" + input + " result=" + result
                 + " buffer=" + buffer);
    }

    public static void main(String[] args) {
        PrinterBuffer p = new PrinterBuffer();
        int i = p.capacity(0);
        Util.log("main - result " + i);
    }
}

