/*
 * Copyright (c) 2002 PARC Inc.  All rights reserved.
 *
 * Use and copying of this software and preparation of derivative works based
 * upon this software are permitted.  Any distribution of this software or
 * derivative works must comply with all applicable United States export
 * control laws.
 *
 * This software is made available AS IS, and PARC Inc. makes no
 * warranty about the software, its performance or its conformity to any
 * specification.
 */
package com.xerox.printing; 

import java.util.*;
import java.io.*;
import org.aspectj.lang.*;

public aspect MainFailure { 

    public static void main (String[] args) { TargetClass.main(args); }

    // article page 42 - recording failures from main

    pointcut main(String[] args) :
        args(args) && execution(public static void main(String[]));

    /** log after failure, but do not affect exception */
    after(String[] args) throwing (Throwable t) : main(args) {
        logFailureCase(args, t, thisJoinPoint);
    }

    public static void logFailureCase(String[] args, Throwable t, 
                                      Object jp) {
        Util.log("failure case: args " + Arrays.asList(args));
    }
}

class TargetClass {
    static Thread thread;
    /** will throw error if exactly one argument */
    public static void main (String[] args) {
        // make sure to do at least one failure
        if (thread == null) {
            Runnable r = new Runnable() {
                    public void run() {
                        main(new String[] {"throwError" });
                    }
                };
            thread = new Thread(r);
            thread.start();
        }
        if (1 == args.length) {
            throw new Error("1 == args.length");
        }
        try { thread.join(); }
        catch (InterruptedException ie) { }
    } 
}


