
package figures;

aspect PointBoundsChecking {

    static final int MIN_X = 0;
    static final int MAX_X = 100;

    static final int MIN_Y = 0;
    static final int MAX_Y = 100;

    before(Point p, int x): call(void Point.setX(int)) && 
                            target(p) && 
	                    args(x) {
	checkX(x); 
    }

    before(Point p, int y): call(void Point.setY(int)) && 
	                    args(y) && 
                            target(p) { 
	checkY(y);
    }

    before(Point p, int dx, int dy): call(void Point.moveBy(int, int)) && 
	                             args(dx, dy) &&
	                             target(p) {
	checkX(p.getX() + dx);
	checkY(p.getY() + dy);
    }

    void checkX(int x) { 
	check("New x value illegal.", (MIN_X < x) && (x < MAX_X)); 
    }
    
    void checkY(int y) { 
	check("New y value illegal.", (MIN_Y < y) && (y < MAX_Y)); 
    }

    void check(String description, boolean test) {
	if ( !test )
	    throw new Error(description);
    }
    
}

