/*



Copyright (c) Xerox Corporation 2001.  All rights reserved.



Use and copying of this software and preparation of derivative works based

upon this software are permitted.  Any distribution of this software or

derivative works must comply with all applicable United States export control

laws.



This software is made available AS IS, and Xerox Corporation makes no warranty

about the software, its performance or its conformity to any specification.



|<---            this code is formatted to fit into 80 columns             --->|

|<---            this code is formatted to fit into 80 columns             --->|

|<---            this code is formatted to fit into 80 columns             --->|



*/



package figures.gui;



import figures.Figure;

import figures.FigureElement;

import figures.Line;

import figures.Mobility;

import figures.MoveCollecting;

import figures.MoveTracking;

import figures.Point;



import java.awt.*;

import java.awt.geom.*;

import java.awt.event.*;

import java.io.*;

import java.util.*;

import javax.swing.*;

import javax.swing.text.*;

import javax.swing.border.*;



public class FigurePanel extends JComponent {



    ButtonsPanel bp = new ButtonsPanel();

    FigureSurface fs = new FigureSurface(bp);

    ConsolePanel cp = new ConsolePanel();





    public FigurePanel() {

        setLayout(new BorderLayout());

        JPanel panel = new JPanel();

        panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

        panel.add(fs);

        panel.add(bp);

        JSplitPane sp = new JSplitPane(JSplitPane.VERTICAL_SPLIT, panel, cp);

        sp.setPreferredSize(new Dimension(500, 400));

        sp.setDividerLocation(250);

        add(BorderLayout.CENTER, sp);

    }



    class ButtonsPanel extends JPanel {

        private JLabel msgs = new JLabel("click to add a point or line");

        public ButtonsPanel() {

            setLayout(new FlowLayout(FlowLayout.LEFT));

            add(new JButton(new AbstractAction("Get Movers") {

                    public void actionPerformed(ActionEvent e) {

                        FigurePanel.this.fs.showMovers();

                    }

                }));

            final JButton testAndClearButton = new JButton("Test and Clear");

            testAndClearButton.addActionListener(new AbstractAction(testAndClearButton.getActionCommand()) {

                    Color defaultColor = testAndClearButton.getForeground();

                    Color trueColor = Color.green.darker();

                    Color falseColor = Color.red.darker();

                    public void actionPerformed(ActionEvent e) {

                        boolean b = MoveTracking.testAndClear();

                        testAndClearButton.setForeground(b ? trueColor : falseColor);

                        log(b ? "there were moves!" : "no moves :(");

                        SwingUtilities.invokeLater(new Runnable() {

                                public void run() {

                                    try {

                                        Thread.sleep(1000);

                                    } catch (Throwable t) {

                                    } finally {

                                        testAndClearButton.setForeground(defaultColor);

                                    }

                                }

                            });

                    }

                });

            add(testAndClearButton);

            add(new JButton(new AbstractAction("Main") {

                    public void actionPerformed(ActionEvent e) {

                        Main.main(new String[]{});

                        fs.repaint();

                    }

                }));

            add(new JCheckBox(new AbstractAction("Mobility") {

                    { Mobility.enableMoves(); }

                    public void actionPerformed(ActionEvent e) {

                        if (!((JCheckBox)e.getSource()).isSelected()) {

                            log("enabling moves");

                            Mobility.enableMoves();

                        } else {

                            log("disabling moves");

                            Mobility.disableMoves();

                        }

                    }

                }));

            add(msgs);

        }



        public void log(String msg) {

            msgs.setText(msg);

        }

    }



    static class ConsolePanel extends JPanel {



        JTextArea text = new JTextArea();



        public ConsolePanel() {

            super(new BorderLayout());

            text.setFont(StyleContext.getDefaultStyleContext().getFont("SansSerif", Font.PLAIN, 10));

            JScrollPane scroller = new JScrollPane(text);

            scroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

            add(BorderLayout.CENTER, scroller);

            System.setOut(new PrintStream(new BufferedOutputStream(new OutputStream() {

                    StringBuffer buf = new StringBuffer();

                    public void write(int b) throws IOException {

                        if (b == '\n') {

                            ConsolePanel.this.println(buf.toString());

                            buf = new StringBuffer();

                        } else {

                            buf.append((char)b);

                        }

                    }

                })));

        }



        public void println(String msg) {

            text.append(msg + '\n');

        }

    }



    final static Color BACKGROUND = Color.white;

    final static Color POINT_FILL = Color.red;

    final static Color POINT_OUTLINE = Color.black;

    final static Color LINE_FILL = Color.black;

    final static Color LINE_OUTLINE = Color.black;

    final static Color MOVED_FILL = Color.yellow;

    final static Color SELECTED_FILL = Color.blue.darker();



    static class FigureSurface extends JPanel implements MouseListener, MouseMotionListener {



        private Set movers = new HashSet();

        private boolean useMovers = false;

        private ButtonsPanel bp;



        public FigureSurface(ButtonsPanel bp) {

            this.bp = bp;

            addMouseMotionListener(this);

            addMouseListener(this);

            setPreferredSize(new Dimension(500,500));

        }



        public void showMovers() {

            useMovers = true;

            movers = MoveCollecting.getMovers();

            bp.log(movers.size() + " mover" + (movers.size() != 1 ? "s" : ""));

            paintImmediately(new Rectangle(0, 0, getSize().width, getSize().height));

            useMovers = false;

        }



        private Point addPoint(int x, int y) {

            Point p = figures.Figure.makePoint(x, y);

            repaint();

            return p;

        }



        private Line addLine(Point p1, Point p2) {

            if (p1.equals(p2) || Math.abs(p1.getX()-p2.getX()) < 3 || Math.abs(p1.getY()-p2.getY()) < 3) return null;

            Line line = null;

            if (p1 != null && p2 != null) line = figures.Figure.makeLine(p1, p2);

            repaint();

            return line;

        }



        public void paint(Graphics g) {

            Graphics2D g2 = (Graphics2D) g;

            g2.setPaint(BACKGROUND);

            g2.fill(new Rectangle2D.Float(0f, 0f, (float)g2.getClipBounds().width, (float)g2.getClipBounds().height));

            g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

            Iterator iter;

            iter = Figure.getFigureElementSet(figures.Line.class).iterator();

            while (iter.hasNext())

		((Line)iter.next()).paintDispatch(g2, this);

            iter = Figure.getFigureElementSet(figures.Point.class).iterator();

            while (iter.hasNext())

		((Point)iter.next()).paintDispatch(g2, this);

        }



	static aspect Painting {

	    abstract void FigureElement.paintDispatch(Graphics2D g2,

					     FigurePanel.FigureSurface s);

	    void Line.paintDispatch(Graphics2D g2,

                                    FigurePanel.FigureSurface s) {

		s.paint(this, g2);

            }

            void Point.paintDispatch(Graphics2D g2,

                                     FigurePanel.FigureSurface s) {

		s.paint(this, g2);

            }

	}



        public void paint(Point point, Graphics2D g2) {

            g2.setPaint(POINT_OUTLINE);

            g2.draw(point);

            g2.setPaint(useMovers && movers.contains(point) ? MOVED_FILL :

                        (point.equals(selected) ? SELECTED_FILL : POINT_FILL));

            g2.fill(point);

        }



        public void paint(Line line, Graphics2D g2) {

            g2.setPaint(useMovers && movers.contains(line) ? MOVED_FILL :

                        (line.equals(selected) ? SELECTED_FILL : LINE_FILL));

            g2.draw(line);

        }



        int lastX;

        int lastY;

        int pressX;

        int pressY;

        boolean dragging = false;

        Shape first = null;

        Point selected = null;

        Point point = null;



        public void mousePressed(MouseEvent e){

            int x = e.getX(), y = e.getY();

            first = shape(x, y);

            lastX = pressX = x;

            lastY = pressY = y;

            if (SwingUtilities.isLeftMouseButton(e)) {

                if (first == null && selected == null) {

                    point = addPoint(x, y);

                }

            } else if (SwingUtilities.isRightMouseButton(e)) {

                if (first == null && selected == null) {

                    first = addPoint(x, y);

                }

                if (selected == null && first != null && first instanceof Point) {

                    select((Point)first);

                }

            }

        }



        public void mouseDragged(MouseEvent e) {

            dragging = true;

            int x = e.getX(), y = e.getY(), dx = lastX-x, dy = lastY-y;

            lastX = x;

            lastY = y;

            if (first != null) ((FigureElement)first).incrXY(-dx, -dy);

            repaint();

        }



        public void mouseReleased(MouseEvent e){

            int x = e.getX(), y = e.getY();

            Shape landed = shape(x, y);

            if (dragging && landed == null && point == null) {

                // nothing

            } else if (first != null && selected == null) {

                first = null;

            } else if (SwingUtilities.isLeftMouseButton(e)) {

                if (dragging  && point != null && landed == null) {

                    addLine(point, addPoint(x, y));

                } else if (selected == null && point != null && landed == null) {

                    addLine(point, addPoint(x, y));

                } else if (selected == null && point != null && landed != null && landed instanceof Point) {

                    addLine(point, (Point)landed);

                } else if (selected != null && landed != null && landed instanceof Point) {

                    addLine(selected, (Point)landed);

                } else if (selected != null && first != null && first instanceof Point) {

                    addLine(selected, (Point)first);

                } else if (selected != null) {

                    addLine(selected, addPoint(x, y));

                }

            } else if (!dragging && SwingUtilities.isRightMouseButton(e)) {

                if (selected != null && landed != null) {

                    Line line = addLine(selected, (Point)landed);

                    if (line != null) select(line.getP2());

                } else if (selected != null && first != null) {

                    Line line = addLine(selected, (Point)first);

                    if (line != null) select(line.getP2());

                } else if (selected != null) {

                    Line line = addLine(selected, addPoint(x, y));

                    if (line != null) select(line.getP2());

                }

            } else if (dragging && SwingUtilities.isRightMouseButton(e) && first instanceof Point) {

                select((Point)first);

            }

            if (!dragging && !(SwingUtilities.isRightMouseButton(e))) {

                select(null);

            }

            dragging = false;

            first = null;

        }



        private void select(Point point) {

            selected = point;

            if (selected != null) bp.log("selected " + selected);

            repaint();

        }



        public void mouseMoved(MouseEvent e){}

        public void mouseClicked(MouseEvent e){}

        public void mouseExited(MouseEvent e){}

        public void mouseEntered(MouseEvent e){}



        private Shape shape(int x, int y) {

            Iterator iter;

            iter = Figure.getFigureElementSet(figures.Point.class).iterator();

            while (iter.hasNext()) {

                Shape s = (Shape)iter.next();

                if (s.contains(x,y)) return s;

            }

            iter = Figure.getFigureElementSet(figures.Line.class).iterator();

            while (iter.hasNext()) {

                Shape s = (Shape)iter.next();

                if (s.contains(x,y)) return s;

            }

            return null;

        }

    }

}

