/*



Copyright (c) Xerox Corporation 2001.  All rights reserved.



Use and copying of this software and preparation of derivative works based

upon this software are permitted.  Any distribution of this software or

derivative works must comply with all applicable United States export control

laws.

 

This software is made available AS IS, and Xerox Corporation makes no warranty

about the software, its performance or its conformity to any specification.



|<---            this code is formatted to fit into 80 columns             --->|

|<---            this code is formatted to fit into 80 columns             --->|

|<---            this code is formatted to fit into 80 columns             --->|



*/



package figures.gui;



import figures.FigureElement;

import figures.Point;

import figures.Line;



import java.awt.Rectangle;

import java.awt.Shape;

import java.awt.geom.AffineTransform;

import java.awt.geom.FlatteningPathIterator;

import java.awt.geom.PathIterator;

import java.awt.geom.Ellipse2D;

import java.awt.geom.Line2D;

import java.awt.geom.Point2D;

import java.awt.geom.Rectangle2D;



public aspect Shapes {



    /**

     * In order to call {@link Graphics2D#draw()}, {@link FigureElement} has to

     * implement {@link Shape}.

     */

    declare parents: FigureElement implements Shape;



    /**

     * Delegate to {@link FigureElement#getBounds2D()}.

     */

    public Rectangle2D FigureElement.getBounds2D() {

        return getBounds();

    }    



    /**

     * Delegate to {@link FigureElement#contains(Point2D)}.

     */

    public boolean FigureElement.contains(Point2D p) {

        return contains(p.getX(), p.getY());

    }



    /**

     * Delegate to {@link FigureElement#intersects(Rectangle2D)}.

     */

    public boolean FigureElement.intersects(Rectangle2D r) {

        return intersects(r.getX(), r.getY(), r.getWidth(), r.getHeight());

    }



    /**

     * Delegate to {@link FigureElement#contains(Rectangle2D)}.

     */

    public boolean FigureElement.contains(Rectangle2D r) {

        return contains(r.getX(), r.getY(), r.getWidth(), r.getHeight());

    }



    /**

     * Returns a {@link FlatteningPathIterator} based on

     * {@link FigureElement#getPathIterator(AffineTransform)}

     * and <code>flatness</code>.

     */

    public PathIterator FigureElement.getPathIterator(AffineTransform at,

                                                      double flatness) {

	return new FlatteningPathIterator(getPathIterator(at), flatness);

    }  







    // ----------------- Point ---------------------



    /** The height of displayed {@link Point}s. */

    private final static int Point.HEIGHT = 10;



    /** The width of displayed {@link Point}s. -- same as {@link HEIGHT}. */

    private final static int Point.WIDTH  = Point.HEIGHT;



    /**

     * Return a {@link Rectangle} with x and y coordinates offset by

     * {@link WIDTH}/2 and {@link HEIGHT}/2 respectively, and width

     * height and width {@link HEIGHT} and {@link WIDTH}.

     */

    public Rectangle Point.getBounds() {

        return new Rectangle(getX()-Point.WIDTH/2, 

			     getY()-Point.HEIGHT/2,

                             Point.HEIGHT, 

			     Point.WIDTH);

    }



    /**

     * Delegates to {@link Rectangle#contains(double,double)}

     * by calling {@link getBounds()}.

     */

    public boolean Point.contains(double x, double y) {

        return getBounds().contains(x, y);

    }



    /**

     * Delegates to {@link Rectangle#intersects(double,double,double,double)}

     * by calling {@link getBounds()}.

     */

    public boolean Point.intersects(double x, double y, double w, double h) {

        return getBounds().intersects(x, y, w, h);

    }

    

    /**

     * Delegates to {@link Rectangle#contains(double,double.double,double)}

     * by calling {@link getBounds()}.

     */

    public boolean Point.contains(double x, double y, double w, double h) {

        return getBounds().contains(x, y, w, h);

    }



    /**

     * Delegates to {@link Ellipse2D.Float#getPathIterator(AffineTransform)}

     * created from this {@link Point}s coordinates.

     */

    public PathIterator Point.getPathIterator(AffineTransform at) {

	return new Ellipse2D.Float((float)getX()-Point.WIDTH/2,

                                   (float)getY()-Point.HEIGHT/2,

                                   (float)Point.HEIGHT,

                                   (float)Point.WIDTH).getPathIterator(at);

    }

    



    

    // ----------------- Line  ---------------------



    /**

     * Used to determine if this line {@link contains(double,double)} a point.

     */

    final static int THRESHHOLD = 5;



    /** 

     * Returns a {@link Rectangle} created from the coordinates of points

     * returned from {@link Line#getX()} and {@link Line#getY()}.

     */

    public Rectangle Line.getBounds() {

        return new Rectangle(Math.min(getP1().getX() , getP2().getX()),

                             Math.min(getP1().getY() , getP2().getY()),

                             Math.abs(getP1().getX() - getP2().getX()),

                             Math.abs(getP1().getY() - getP2().getY()));

    }



    /**

     * Returns <code>true</code> if the point segment distance is less than

     * {@link THRESHHOLD}.

     */

    public boolean Line.contains(double x, double y) {

        return Line2D.ptSegDist((double)getP1().getX(), (double)getP1().getY(),

                                (double)getP2().getX(), (double)getP2().getY(),

                                x, y) < THRESHHOLD;

    }



    /**

     * Delegates to

     * {@link Rectangle2D.Double#intersectsLine(double,double,double,double)}.

     */

    public boolean Line.intersects(double x, double y, double w, double h) {

        return new Rectangle2D.Double(x, y, w, h).intersectsLine

            ((double)getP1().getX(), (double)getP1().getY(),

             (double)getP2().getX(), (double)getP2().getY());

    }



    /**

     * Returns <code>true</code> if <code>w == h == 0</code> and

     */

    public boolean Line.contains(double x, double y, double w, double h) {

        return w == 0 && h == 0 && contains(x, y);

    }



    /**

     * Delegates to {@link Line2D.Float#getPathIterator(AffineTransform)}

     * created from this {@link Point}s coordinates.

     */

    public PathIterator Line.getPathIterator(AffineTransform at) {

	return new Line2D.Float((float)getP1().getX(),

                                (float)getP1().getY(),

                                (float)getP2().getX(),

                                (float)getP2().getY()).getPathIterator(at);

    }

}

