/*
 * Copyright (c) 2002 PARC Inc.  All rights reserved.
 *
 * Use and copying of this software and preparation of derivative works based
 * upon this software are permitted.  Any distribution of this software or
 * derivative works must comply with all applicable United States export
 * control laws.
 *
 * This software is made available AS IS, and PARC Inc. makes no
 * warranty about the software, its performance or its conformity to any
 * specification.
 */

package com.xerox.printing;

import java.awt.Point;
import java.io.IOException;


aspect CompileTime {

    // article page 40 - warning
    declare warning : within(com.xerox..*) 
            && set(!public * *) && !withincode(* set*(..)) 
        : "writing field outside setter" ;

    /*
      The article pointcut picked out any delegate method in any subclass 
      of PrinterStream (in theory including methods not actually used
      as delegates of the write() method).  
      
            withincode(* PrinterStream+.delegate(..))

      But since the only method actually used by PrinterStream.write() 
      is delegate(), we can rewrite the pointcut without unneeded
      generality:

            withincode(void PrinterStream.delegate()) 
    */

    // article page 41 - error
    declare error : handler(IOException+) 
            && withincode(void PrinterStream.delegate()) 
        : "do not handle IOException in this method";

    /*
      The article pointcut picked out subclasses of the the return
      value and of the declaring type, and permitted any number of 
      arguments:

          !withincode(Point+ SubPoint+.create(..))  

      But since SubPoint.create() is a static factory that returns Point
      and takes no arguments, we can rewrite the pointcut without
      unnecessary generality.

          !withincode(Point SubPoint.create())
    */

    /** any call to Point constructors is an error unless in our factory */
    declare error  : !withincode(Point SubPoint.create())
            && within(com.xerox..*) 
            && call(Point+.new(..))
        : "use SubPoint.create() to create Point";
}


