/*
 * Copyright (c) 2002 PARC Inc.  All rights reserved.
 *
 * Use and copying of this software and preparation of derivative works based
 * upon this software are permitted.  Any distribution of this software or
 * derivative works must comply with all applicable United States export
 * control laws.
 *
 * This software is made available AS IS, and PARC Inc. makes no
 * warranty about the software, its performance or its conformity to any
 * specification.
 */

package com.xerox.printing;

public aspect RecordingInput {

    // article page 42 - recording input
    pointcut capacityCall (int i) :
        within(com.xerox..*) && args(i) 
        && call(public int PrinterBuffer.capacity(int)) ;

    /*
      Note the article pointcut uses a + which is redundant:

          call(public * PrinterBuffer+.capacity(int)) 

      Since PrinterBuffer defines the signature capacity(int),
      any implementation in PrinterBuffer subclasses has the
      same signature (i.e., overrides the superclass method).
      Thus, without the +, the pointcut still picks out methods
      in subclasses.

      For the same reason, there is no reason not to specify
      the return value as int, since you cannot have methods with
      the same name and parameter types but a different return value.
     */

    before (int i) : capacityCall(i) {
        log.print("<capacityCall tjp=\"" + thisJoinPoint 
                  + "\" input=\"" + i + "\"/>");
    }

    Log log = new Log();
    class Log {
        void print(String s) { 
            Util.log(s); 
        }
    }
    public static void main(String[] args) {
        PrinterBuffer p = new PrinterBuffer();
        p.capacity(1);
        p.capacity(2);
    } 
}
