/*
 * Copyright (c) 2002 PARC Inc.  All rights reserved.
 *
 * Use and copying of this software and preparation of derivative works based
 * upon this software are permitted.  Any distribution of this software or
 * derivative works must comply with all applicable United States export
 * control laws.
 *
 * This software is made available AS IS, and PARC Inc. makes no
 * warranty about the software, its performance or its conformity to any
 * specification.
 */
package com.xerox.printing;

import java.util.*;
import java.awt.Point;

import org.aspectj.lang.JoinPoint;

public class RunTime {
    public static void main(String[] a) {
        try { 
            SubPoint.create(); 
        } catch (Error e) {
            String s = "ok - got Error from FactoryValidation: "
                + e.getMessage();
            // fyi both this and target are static, hence null in message
            Util.log(s);
        } 
    }
}

aspect FactoryValidation {

    // article page 41 - runtime NPE
    /** @throws Error if the factory method returns null */
    after () returning (Point p) : call(Point SubPoint.create()) { 
        if (null == p) {
            String err = "Null Point constructed when this (" 
                + thisJoinPoint.getThis() 
                + ") called target (" + thisJoinPoint.getTarget() 
                + ") at join point (" + thisJoinPoint.getSignature() 
                + ") from source location (" 
                + thisJoinPoint.getSourceLocation().getFileName()
                + ":" + thisJoinPoint.getSourceLocation().getColumn()
                + ") with args ("
                + Arrays.asList(thisJoinPoint.getArgs())
                + ")";
            throw new Error(err);
        }
    }

    /*
      The pointcut above avoids some of the generality of the
      original article pointcut

          call(Point+ SubPoint+.create(..)) 

      for the same reasons mentioned in CompileTime

      And since we know there are no arguments to any factory,
      this is not necessary:

          + ") with args ("
          + Arrays.asList(thisJoinPoint.getArgs())

      However, it is nicer to print source file name and line
      (rather than relying on the class implementing SourceLocation
      to implement toString):

          + ") from source location (" 
          + thisJoinPoint.getSourceLocation().getFileName()
          + ":" + thisJoinPoint.getSourceLocation().getColumn()
    */
}
