
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.ajde;

import java.util.*;
import java.io.*;
import org.aspectj.ajde.views.*;
import org.aspectj.ajde.compiler.*;
import org.aspectj.ajde.editor.*;
import org.aspectj.ajde.configs.*;
import org.aspectj.asm.*;      
import org.aspectj.ajde.ui.*;

/**
 * Responsible for AJDE initialization.  Exposes the core AJDE functionality
 * in the form of public fields that contain the other manager classes
 *
 * @author Mik Kersten
 */
public class TopManager {
    //public static final TopManager INSTANCE = new TopManager();
    public static BrowserManager BROWSER_MANAGER = null;
    public static CompilerManager COMPILER_MANAGER = null;
    public static EditorManager EDITOR_MANAGER = null;
    public static IdeManager IDE_MANAGER = null;
    public static StructureManager STRUCTURE_MANAGER = new StructureManager();
    public static ProjectProperties projectProperties = null;
    private static OptionsFrame optionsFrame = null;

    /**
     * Used by standalone browser.
     */
    public static void initAjde(String[] configFiles, boolean visible) {
        try {
            AjdeManager ajdeManager = new AjdeManager(configFiles);
            IDE_MANAGER = ajdeManager;
            projectProperties = new ProjectProperties();
            Icons.INSTANCE = new Icons();
            CompilerMessagesPanel compilerMessagesPanel = new CompilerMessagesPanel();
            CompileProgressPanel compileProgressPanel = new CompileProgressPanel();

            BROWSER_MANAGER = new BrowserManager(true);
            COMPILER_MANAGER = new CompilerManager(compilerMessagesPanel, compileProgressPanel);
            EDITOR_MANAGER = new EditorManager();

            ajdeManager.initTopFrame(
                BROWSER_MANAGER.getBrowserPanel(),
                compilerMessagesPanel,
                EDITOR_MANAGER.getEditorPanel(),
                visible
            );

            optionsFrame = new OptionsFrame(IDE_MANAGER.getIcons(), true);

            if (configFiles.length == 0) {
                ErrorHandler.handleWarning("No build configuration selected. " +
                    "Select a \".lst\" build configuration file in order to compile and navigate structure.",
                    TopManager.IDE_MANAGER.getRootFrame());
            }
        } catch (Throwable t) {
            ErrorHandler.handleError("AJDE failed to initialize.", t, TopManager.IDE_MANAGER.getRootFrame());
        }
        BROWSER_MANAGER.readStructureView();
        BROWSER_MANAGER.updateView();
    }

    /**
     * For use by IDE extensions that use the default GUI features.
     */
    public static void initAjde(IdeManager ajdeManager,
                                AjdeEditor ajdeEditor,
                                CompilerMessages compilerMessages,
                                ProjectProperties projectProperties,
                                boolean useStandaloneProjectOptions) {
        TopManager.projectProperties = projectProperties;
        IDE_MANAGER = ajdeManager;
        BROWSER_MANAGER = new BrowserManager(IDE_MANAGER.getIcons());
        EDITOR_MANAGER = new EditorManager(ajdeEditor);
        COMPILER_MANAGER = new CompilerManager(new CompilerMessagesPanel(), new CompileProgressPanel());
        BROWSER_MANAGER.readStructureView();
        optionsFrame = new OptionsFrame(IDE_MANAGER.getIcons(), useStandaloneProjectOptions);
    }

    /**
     * For use by IDE that don't use the default GUI features.
     */
    public static void initAjdeNoGUI(IdeManager ajdeManager,
                                AjdeEditor ajdeEditor,
                                CompilerMessages compilerMessages,
                                CompileProgressMonitor compileProgressMonitor,
                                ProjectProperties projectProperties) {
        TopManager.projectProperties = projectProperties;
        IDE_MANAGER = ajdeManager;
        BROWSER_MANAGER = null;
        EDITOR_MANAGER = new EditorManager(ajdeEditor);
        COMPILER_MANAGER = new CompilerManager(compilerMessages, compileProgressMonitor);
    }

    public static void showOptionsFrame() {
        optionsFrame.showOption(OptionsFrame.STRUCTURE_PROPERTIES);
    }

    public static void showOptionsFrame(int index) {
        optionsFrame.showOption(index);
    }
}
