
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.ajde.editor;

import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.util.*;
import org.aspectj.ajde.ui.ErrorHandler;
import org.aspectj.ajde.ui.BasicEditor;
import org.aspectj.ajde.*;

/**
 * Responsible for controlling the editor.
 *
 * @todo    remove coupling to <CODE>BasicEditor</CODE>
 * @author  Mik Kersten
 */
public class EditorManager {

    private AjdeEditor editor = null;
    private ArrayList editorListeners = new ArrayList();
    private Vector editors = new Vector();
    private JPanel editor_panel = null;
    private Box editors_box = Box.createVerticalBox();

    public EditorManager() {
        editor = new BasicEditor();
        editors.add(editor);
        editors_box.add(editor.getPanel());
        editor_panel = new JPanel(new BorderLayout());
        editor_panel.add(editors_box, BorderLayout.CENTER);
    }

    public EditorManager(AjdeEditor ajdeEditor) {
        editors.add(ajdeEditor);
        this.editor = ajdeEditor;
    }

    public void addListener(EditorListener editorListener) {
        editorListeners.add(editorListener);
    }

    public void removeListener(EditorListener editorListener) {
        editorListeners.remove(editorListener);
    }

    public void notifyCurrentFileChanged(String filePath) {
        for (Iterator it = editorListeners.iterator(); it.hasNext(); ) {
            ((EditorListener)it.next()).currentFileChanged(filePath);
        }
    }

    /**
     * @todo    remove "instanceof AjdeManager" hack
     */
    public void addViewForSourceLine(final String filePath, final int lineNumber) {
        final BasicEditor newEditor = new BasicEditor();
        editors.add(newEditor);
        Runnable update = new Runnable() {
            public void run() {
                editors_box.remove(editor.getPanel());
                editors_box.add(newEditor.getPanel());
                newEditor.showSourceLine(filePath, lineNumber, true);

                if (TopManager.IDE_MANAGER instanceof org.aspectj.ajde.ui.AjdeManager) {
                    ((org.aspectj.ajde.ui.AjdeManager)TopManager.IDE_MANAGER).resetGUI();
                }
            }
        };

        if (SwingUtilities.isEventDispatchThread()) {
            update.run();
        } else {
            try {
                SwingUtilities.invokeAndWait(update);
            } catch (Exception ee) {
                ee.printStackTrace();
            }
        }
    }

    public String getCurrFile() {
        return editor.getCurrFile();
    }

    /**
     * @todo    remove "instanceof AjdeManager" hack
     */
    public void showSourceLine(String filePath, int lineNumber, boolean highlight) {
        if (editors.size() > 1) {
            editors_box.removeAll();
            editors_box.add(editor.getPanel());

            if (TopManager.IDE_MANAGER instanceof org.aspectj.ajde.ui.AjdeManager) {
                ((org.aspectj.ajde.ui.AjdeManager)TopManager.IDE_MANAGER).resetGUI();
            }

            editors.removeAllElements();
            editors.add(editor);
        }
        editor.showSourceLine(filePath, lineNumber, highlight);
    }

    public void pasteToCaretPos(String text) {
        editor.pasteToCaretPos(text);
    }

    public void showSourcelineAnnotation(String filePath, int lineNumber, JPopupMenu annotation) {
        editor.showSourcelineAnnotation(filePath, lineNumber, annotation);
    }

    public void saveContents() {
        try {
            for (Iterator it = editors.iterator(); it.hasNext(); ) {
                ((AjdeEditor)it.next()).saveContents();
            }
        } catch (IOException ioe) {
            ErrorHandler.handleError("Editor could not save the current file.", ioe, TopManager.IDE_MANAGER.getRootFrame());
        }
    }

    public JPanel getEditorPanel() {
        if (editor_panel != null) {
            return editor_panel;
        } else {
            return editor.getPanel();
        }
    }
}


