
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.ajde.ui;

import java.awt.*;
import javax.swing.*;
import javax.swing.text.*;
import java.io.*;
import java.net.URL;
import org.aspectj.ajde.TopManager;
import org.aspectj.ajde.editor.AjdeEditor;

/**
 * Bare-bones editor implementation used when the framework is being used
 * standalone.
 *
 * @author Mik Kersten
 */
public class BasicEditor extends AjdeEditor {

    private String filePath = "";
    private JPanel editor_panel = new JPanel();

    // @todo    get rid of these
    private int currHighlightStart = 0;
    private int currHighlightEnd = 0;

    private BorderLayout borderLayout1 = new BorderLayout();
    private JScrollPane jScrollPane1 = new JScrollPane();
    private JEditorPane editorPane = new JEditorPane();

    public BasicEditor() {
        try {
            editorPane.setEditable(true);
            editorPane.setContentType("text/plain");
            editorPane.setFont(new Font("Monospaced", 0, 11));
            editor_panel.add(editorPane);
            jbInit();
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }

    public String getCurrFile() {
        return filePath;
    }

    public void showSourceLine(int lineNumber, boolean highlight) {
        showSourceLine(filePath, lineNumber, highlight);
    }

    public void pasteToCaretPos(String text) {
        if (currHighlightEnd < 1) return;
        String contents = editorPane.getText();
        String pasted = contents.substring(0, currHighlightEnd) +
            text + contents.substring(currHighlightEnd, contents.length());
        editorPane.setText(pasted);
    }

    public void showSourceLine(String filePath, int lineNumber, boolean highlight) {
        String oldPath = this.filePath;
        this.filePath = filePath;
        if (oldPath != filePath && !TopManager.BROWSER_MANAGER.isGlobalMode()) {
            TopManager.BROWSER_MANAGER.updateView();
        }

        TopManager.IDE_MANAGER.setEditorStatusText(filePath);

        currHighlightStart = 0;
        currHighlightEnd = 0;
        editorPane.setText(readFile(filePath, lineNumber));
        try {
            editorPane.getHighlighter().addHighlight(currHighlightStart, currHighlightEnd, DefaultHighlighter.DefaultPainter);
            editorPane.setCaretPosition(currHighlightStart);
        } catch (BadLocationException ble) {
            System.err.println("ERROR: couldn't highlight location.");
        }
        TopManager.EDITOR_MANAGER.notifyCurrentFileChanged(filePath);
    }

    /**
     * Not implemented.
     */
    public void showSourcelineAnnotation(String filePath, int lineNumber, JPopupMenu annotation) { }

    public void saveContents() throws IOException {
        if (filePath != "" && editorPane.getText() != "") {
            BufferedWriter writer = new BufferedWriter(new FileWriter(filePath));
            writer.write(editorPane.getText());
            writer.flush();
        }
    }

    public JPanel getPanel() {
        return editor_panel;
    }

    public void showSourceForFile(String filePath) { }

    public void showSourceForLine(int lineNumber, boolean highlight) { }

    public void showSourceForSourceLine(String filePath, int lineNumber, boolean highlight) { }

    public String getCurrSourceFilePath() { return null; }

    public void setBreakpointRequest(String filePath, int lineNumber, boolean isDeferred) { }

    public void clearBreakpointRequest(String filePath, int lineNumber) { }

    private String readFile(String filePath, int lineNumber) {
        try {
            URL url = ClassLoader.getSystemResource(filePath);
            File file = new File(filePath);
            if (!file.exists()) {
                return "ERROR: file \"" + filePath + "\" does not exist.";
            }
            BufferedReader reader = new BufferedReader(new FileReader(file));
            StringBuffer contents = new StringBuffer();
            String line = reader.readLine();
            int numChars = 0;
            int numLines = 0;
            while (line != null) {
                numLines++;
                if (numLines < lineNumber) {
                    currHighlightStart += line.length()+1;
                }
                if (numLines == lineNumber) {
                    currHighlightEnd = currHighlightStart + line.length();
                }
                contents.append(line);
                contents.append('\n');
                line = reader.readLine();
            }
            return contents.toString();
        } catch (IOException ioe) {
            return "ERROR: could not read file \"" + filePath + "\", make sure that you have mounted /project/aop on X:\\";
        }
    }

    private void jbInit() throws Exception {
        editor_panel.setFont(new java.awt.Font("DialogInput", 1, 12));
        editor_panel.setLayout(borderLayout1);
        editor_panel.add(jScrollPane1, BorderLayout.CENTER);
        jScrollPane1.getViewport().add(editorPane, null);
    }
}
