/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */


package org.aspectj.ajde.ui;

import javax.swing.*;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import org.aspectj.asm.*;
import org.aspectj.ajde.*;
import javax.swing.event.*;
import java.io.*;
import org.aspectj.ajde.ui.ErrorHandler;
import org.aspectj.ajde.configs.BuildConfigFileUpdater;
import org.aspectj.util.ConfigParser;

/**
 * UI for editing build configuration (".lst") files via a graphical tree-based
 * representation.
 *
 * @author Mik Kersten
 */
public class ConfigFileEditorPanel extends JPanel {

    private static java.util.List selectedEntries = new ArrayList();
    private String configFile = null;
    private File sourcePath = null;
    private static FilenameFilter SOURCE_FILE_FILTER = new FilenameFilter() {
        public boolean accept(File dir, String name) {
            return name.endsWith(".java") || name.endsWith(".aj");
        }
    };

    BorderLayout borderLayout1 = new BorderLayout();
    JPanel jPanel1 = new JPanel();
    JLabel jLabel1 = new JLabel();
    JPanel jPanel2 = new JPanel();
    JButton cancel_button = new JButton();
    BorderLayout borderLayout2 = new BorderLayout();
    JButton save_button = new JButton();
    JScrollPane jScrollPane = new JScrollPane();
    JTree associations_tree = new JTree();

    public ConfigFileEditorPanel(String configFile) {
        try {
            if (configFile == null) {
                ErrorHandler.handleError("No structure is selected for editing.");
                return;
            }
            this.configFile = configFile;
            sourcePath = new File(new File(configFile).getParent());
            jbInit();
            jLabel1.setText(" Editing: " + configFile);

            ConfigTreeNode root = new ConfigTreeNode(sourcePath.getPath(), null, null);
            associations_tree.setModel(new DefaultTreeModel(root));
            associations_tree.addMouseListener(new ConfigFileMouseAdapter(associations_tree));
            associations_tree.setCellRenderer(new AssociationsTreeCellRenderer());

            ConfigParser configParser = new ConfigParser();
            configParser.parseConfigFile(new File(configFile));
            java.util.List configFiles = configParser.getFiles();
            makePackageTree(root, configFiles, sourcePath.getPath());

            for (int j = 0; j < associations_tree.getRowCount(); j++) {
                associations_tree.expandPath(associations_tree.getPathForRow(j));
            }
        } catch (ConfigParser.ParseException pe) {
            associations_tree.setModel(new DefaultTreeModel(new ConfigTreeNode("<no config file>", null, null)));
            ErrorHandler.handleWarning("Problem parsing configuration file: " + pe.getMessage(),
                TopManager.IDE_MANAGER.getRootFrame());
        } catch(Exception e) {
            e.printStackTrace();
        }
    }

    private void makePackageTree(ConfigTreeNode root, java.util.List files, String rootPath) {
        ArrayList dirs = new ArrayList();
        ArrayList dirNodes = new ArrayList();
        ArrayList rootDirFiles = new ArrayList();
        for (Iterator it = files.iterator(); it.hasNext(); ) {
            File file = (File)it.next();
            String parentPath = file.getParent();
            String relativeParentPath = "";
            if (parentPath.length() > rootPath.length()) {
                relativeParentPath = file.getPath().substring(rootPath.length()+1, parentPath.length());
            } else {
                rootDirFiles.add(file);
            }

            if (!dirs.contains(parentPath) && relativeParentPath != "") {
                ConfigTreeNode dirNode = new ConfigTreeNode(relativeParentPath, file.getParentFile(), null);
                root.add(dirNode);
                dirs.add(parentPath);
                dirNodes.add(dirNode);
            }
        }

        for (Iterator it = dirNodes.iterator(); it.hasNext(); ) {
            ConfigTreeNode dirNode = (ConfigTreeNode)it.next();
            File[] dirFiles = dirNode.sourceFile.listFiles(SOURCE_FILE_FILTER);
            for (int i = 0; i < dirFiles.length; i++) {
                String entry = dirFiles[i].getPath().substring(rootPath.length()+1);
                ConfigTreeNode fileNode = new ConfigTreeNode(dirFiles[i].getName(), dirFiles[i], entry);
                dirNode.add(fileNode);
                if (files.contains(dirFiles[i])) {
                    fileNode.checkBox.setSelected(true);
                    selectedEntries.add(entry);
                }
            }
        }

        File[] rootFiles = new File(rootPath).listFiles(SOURCE_FILE_FILTER);
        for (int j = 0; j < rootFiles.length; j++) {
            String entry = rootFiles[j].getName();
            ConfigTreeNode rootNode = new ConfigTreeNode(rootFiles[j].getName(), rootFiles[j], entry);
            root.add(rootNode);
            if (rootDirFiles.contains(rootFiles[j])) {
                rootNode.checkBox.setSelected(true);
                selectedEntries.add(entry);
            }
        }
    }

    private java.util.List removeWildcardEntries(java.util.List entries) {
        java.util.List newEntries = new ArrayList();
        for (Iterator it = entries.iterator(); it.hasNext(); ) {
            String entry = (String)it.next();
            if (entry.indexOf("*.") == -1) {
                newEntries.add(entry);
            }
        }
        return newEntries;
    }

    private void saveConfigFile() {
//        System.err.println(selectedEntries);
        BuildConfigFileUpdater.writeConfigFile(configFile, selectedEntries);
    }

    private void jbInit() throws Exception {
        this.setLayout(borderLayout1);
        jLabel1.setFont(new java.awt.Font("Dialog", 0, 11));
        jLabel1.setMaximumSize(new Dimension(80, 30));
        jLabel1.setMinimumSize(new Dimension(80, 20));
        jLabel1.setPreferredSize(new Dimension(80, 20));
        jLabel1.setText("Config File Editor");
        cancel_button.setFont(new java.awt.Font("Dialog", 0, 11));
        cancel_button.setMaximumSize(new Dimension(73, 20));
        cancel_button.setMinimumSize(new Dimension(73, 20));
        cancel_button.setPreferredSize(new Dimension(73, 20));
        cancel_button.setText("Cancel");
        cancel_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                cancel_button_actionPerformed(e);
            }
        });
        jPanel1.setLayout(borderLayout2);
        save_button.setText("Save");
        save_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                save_button_actionPerformed(e);
            }
        });
        save_button.setPreferredSize(new Dimension(73, 20));
        save_button.setMinimumSize(new Dimension(73, 20));
        save_button.setMaximumSize(new Dimension(73, 20));
        save_button.setFont(new java.awt.Font("Dialog", 0, 11));
        this.add(jPanel1,  BorderLayout.NORTH);
        jPanel1.add(jPanel2,  BorderLayout.EAST);
        jPanel2.add(save_button, null);
        jPanel2.add(cancel_button, null);
        jPanel1.add(jLabel1, BorderLayout.CENTER);
        this.add(jScrollPane,  BorderLayout.CENTER);
        jScrollPane.getViewport().add(associations_tree, null);
    }

    private class ConfigTreeNode extends DefaultMutableTreeNode {
        public JCheckBox checkBox = null;
        public File sourceFile = null;
        public String entry = null;

//        public ConfigTreeNode(Association association) {
//            super(association.getStereotypeName(), true);
//            this.association = association;
//            checkBox = new JCheckBox();
//        }

        public ConfigTreeNode(String name, File sourceFile, String entry) {
            super(name, true);
            this.sourceFile = sourceFile;
            this.entry = entry;
            checkBox = new JCheckBox();
        }

        public String getName() {
            return super.toString();
        }
    }

    private class ConfigFileMouseAdapter extends MouseAdapter {
        private JTree tree = null;
        final JCheckBox checkBoxProto = new JCheckBox();
        final int width = checkBoxProto.getPreferredSize().width;

        public ConfigFileMouseAdapter(JTree tree) {
            super();
            this.tree = tree;
        }

        public void mousePressed(MouseEvent e) {
            int x = e.getX();
            int y = e.getY();
            TreePath path = tree.getClosestPathForLocation(x,y);
            ConfigTreeNode node = (ConfigTreeNode)path.getLastPathComponent();

            // if (isCheckBox(x, tree.getPathBounds(path).x)) {
            node.checkBox.setSelected(!node.checkBox.isSelected());
            ((DefaultTreeModel)tree.getModel()).nodeChanged(node);
            if (node.entry != null) {
                if (node.checkBox.isSelected()) {
                    selectedEntries.add(node.entry);
                } else {
                    selectedEntries.remove(node.entry);
                }
            }
            super.mousePressed(e);

        }

        boolean isCheckBox(int x, int x_) {
            int d = x - x_;
            return (d < width) && (d > 0);
        }
    }

    static class AssociationsTreeCellRenderer extends DefaultTreeCellRenderer {
        public Component getTreeCellRendererComponent(JTree tree,
                                                      Object value,
                                                      boolean sel,
                                                      boolean expanded,
                                                      boolean leaf,
                                                      int row,
                                                      boolean hasFocus) {
            super.getTreeCellRendererComponent(tree, value, sel, expanded,
                                               leaf, row, hasFocus);
            JPanel p = new JPanel();
            p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
            p.setBackground(Color.white);
            if (leaf)  {
                setFont(new Font("Dialog", Font.PLAIN, 11));
                final JCheckBox cbox = ((ConfigTreeNode)value).checkBox;
                cbox.setBackground(Color.white);
                p.add(cbox);

                ConfigTreeNode ctn = (ConfigTreeNode)value;
                if (ConfigFileEditorPanel.selectedEntries.contains(ctn.sourceFile)) {
                    cbox.setSelected(true);
                }
            }

            p.add(this);
            return p;
        }
    }

    void cancel_button_actionPerformed(ActionEvent e) {
        resetEditorFrame();
    }

    void save_button_actionPerformed(ActionEvent e) {
        saveConfigFile();
        resetEditorFrame();
    }

    /**
     * @todo    remove "instanceof AjdeManager" hack
     */
    private void resetEditorFrame() {
        if (TopManager.IDE_MANAGER instanceof org.aspectj.ajde.ui.AjdeManager) {
            ((org.aspectj.ajde.ui.AjdeManager)TopManager.IDE_MANAGER).resetEditorFrame();
        }
    }
}

