
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.ajde.ui;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import javax.swing.filechooser.FileFilter;
import java.awt.*;
import java.io.*;
import javax.swing.border.*;
import java.awt.event.*;
import java.util.*;
import java.text.*;
import org.aspectj.ajde.*;

/**
 * UI for standalone operation.  Not used in conjunction with IDE integration.
 *
 * @author Mik Kersten
 */
public class TopFrame extends JFrame {

    JLabel statusText_label = new JLabel();

    private JPanel editor_panel = null;
    private JPanel sourceEditor_panel = null;

    private JMenuBar menuBar = new JMenuBar();
    private JMenu jMenu1 = new JMenu();
    private JMenu jMenu2 = new JMenu();
    private JMenuItem projectBuild_menuItem = new JMenuItem();
    private FlowLayout left_flowLayout = new FlowLayout();
    private Border border1;
    private Border border2;
    private JMenuItem jMenuItem1 = new JMenuItem();
    private JMenuItem exit_menuItem = new JMenuItem();
    private JSplitPane top_splitPane = new JSplitPane();
    private BorderLayout borderLayout2 = new BorderLayout();
    private BorderLayout borderLayout1 = new BorderLayout();
    private BorderLayout borderLayout3 = new BorderLayout();
    private JMenuItem projectRun_menuItem = new JMenuItem();
    private Border border3;
    private JPanel status_panel = new JPanel();
    private BorderLayout borderLayout4 = new BorderLayout();
    private Border emptyBorder = BorderFactory.createEmptyBorder();
    private JPanel toolbar_panel = new JPanel();
    private FlowLayout flowLayout1 = new FlowLayout();
    private JPanel browser_panel = null;
    private JSplitPane right_splitPane = new JSplitPane();
    private JPanel messages_panel = null;
    private JToolBar testing_toolBar = new JToolBar();
    private JButton save_button = new JButton();
    private JToolBar file_toolBar = new JToolBar();
    private JButton closeConfig_button = new JButton();
    private JButton openConfig_button = new JButton();
    private JButton run_button = new JButton();
    private JButton debug_button = new JButton();
    private JButton build_button = new JButton();
    private JToolBar project_toolBar = new JToolBar();
    private JButton readStructure_button = new JButton();
    private JButton dumpStructure_button = new JButton();
    private JButton emacsTest_button = new JButton();
    private JLabel jLabel1 = new JLabel();
    private JMenu tools_menu = new JMenu();
    private JMenuItem joinpointProbe_menuItem = new JMenuItem();
    private JMenuItem projectDebug_menuItem = new JMenuItem();
    private JButton editConfig_button = new JButton();
    private Border border4;
    private Border border5;
    private Border border6;
    private Border border7;
    private JButton options_button = new JButton();
    JMenuItem svProperties_menuItem = new JMenuItem();

    public TopFrame(JPanel browserPanel, JPanel compilerMessagesPanel, JPanel editorPanel) {
        try {
            this.browser_panel = browserPanel;
            this.messages_panel = compilerMessagesPanel;
            this.editor_panel = editorPanel;
            this.sourceEditor_panel = editorPanel;

            jbInit();
            svProperties_menuItem.setIcon(Icons.INSTANCE.getBrowserOptionsIcon());
//            joinpointProbe_menuItem.setIcon(Actions.JOINPOINT_PROBE.getIcon());
            projectBuild_menuItem.setIcon(Icons.INSTANCE.getBuildIcon());
            projectRun_menuItem.setIcon(Icons.INSTANCE.getExecuteIcon());
            projectDebug_menuItem.setIcon(Icons.INSTANCE.getDebugIcon());

            this.setJMenuBar(menuBar);
            this.setIconImage(((ImageIcon)Icons.INSTANCE.getAdviceIcon()).getImage());
            this.setLocation(75, 10);
            this.setSize(1000, 700);
            this.setTitle(IdeManager.TITLE);
            bindKeys();
            fixButtonBorders();

            messages_panel.setVisible(false);
            toolbar_panel.remove(testing_toolBar);
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }

    public void setEditorPanel(JPanel panel) {
        editor_panel = panel;
        right_splitPane.remove(editor_panel);
        right_splitPane.add(panel, JSplitPane.TOP);
        panel.setVisible(true);
    }

    /**
     * @todo    get rid of this method and make jbinit() work properly
     */
    private void fixButtonBorders() {
        build_button.setBorder(null);
        run_button.setBorder(null);
        debug_button.setBorder(null);
        options_button.setBorder(null);
        openConfig_button.setBorder(null);
        closeConfig_button.setBorder(null);
        save_button.setBorder(null);
        editConfig_button.setBorder(null);
    }

    private void bindKeys() {
        this.addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                if (e.getModifiers() == java.awt.Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()) {
                    if (e.getKeyCode() == e.VK_F11) {
                        TopManager.COMPILER_MANAGER.build();
                    } else if (e.getKeyCode() == e.VK_S) {
                        TopManager.EDITOR_MANAGER.saveContents();
                    } else if (e.getKeyCode() == e.VK_P) {
                        TopManager.showOptionsFrame(OptionsFrame.STRUCTURE_PROPERTIES);
                    }
                }
            }
        });
    }

    private void jbInit() throws Exception {
        border1 = BorderFactory.createBevelBorder(BevelBorder.LOWERED,Color.white,Color.white,new Color(148, 145, 140),new Color(103, 101, 98));
        border2 = BorderFactory.createEtchedBorder(Color.white,new Color(148, 145, 140));
        border3 = BorderFactory.createBevelBorder(BevelBorder.LOWERED,Color.white,Color.white,new Color(148, 145, 140),new Color(103, 101, 98));
        border4 = BorderFactory.createBevelBorder(BevelBorder.RAISED,Color.white,Color.white,new Color(148, 145, 140),new Color(103, 101, 98));
        border5 = BorderFactory.createEmptyBorder();
        border6 = BorderFactory.createEmptyBorder();
        border7 = BorderFactory.createBevelBorder(BevelBorder.RAISED,Color.white,Color.white,new Color(148, 145, 140),new Color(103, 101, 98));
        jMenu1.setFont(new java.awt.Font("Dialog", 0, 11));
        jMenu1.setText("File");
        jMenu2.setFont(new java.awt.Font("Dialog", 0, 11));
        jMenu2.setText("Project");
        projectBuild_menuItem.setFont(new java.awt.Font("Dialog", 0, 11));
        projectBuild_menuItem.setText("Build");
        projectBuild_menuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                projectBuild_menuItem_actionPerformed(e);
            }
        });
        left_flowLayout.setAlignment(FlowLayout.LEFT);
        jMenuItem1.setFont(new java.awt.Font("Dialog", 0, 11));
        jMenuItem1.setText("Save");
        jMenuItem1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                jMenuItem1_actionPerformed(e);
            }
        });
        exit_menuItem.setFont(new java.awt.Font("Dialog", 0, 11));
        exit_menuItem.setText("Exit");
        exit_menuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                exit_menuItem_actionPerformed(e);
            }
        });
        top_splitPane.setPreferredSize(new Dimension(706, 800));
        top_splitPane.setDividerSize(4);
        this.getContentPane().setLayout(borderLayout3);
        projectRun_menuItem.setEnabled(false);
        projectRun_menuItem.setFont(new java.awt.Font("Dialog", 0, 11));
        projectRun_menuItem.setText("Run");
        projectRun_menuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                projectRun_menuItem_actionPerformed(e);
            }
        });
        statusText_label.setFont(new java.awt.Font("Dialog", 0, 11));
        statusText_label.setBorder(BorderFactory.createLoweredBevelBorder());
        statusText_label.setMaximumSize(new Dimension(2000, 20));
        statusText_label.setPreferredSize(new Dimension(300, 20));
        status_panel.setLayout(borderLayout4);
        this.addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosed(WindowEvent e) {
                this_windowClosed(e);
            }
            public void windowClosing(WindowEvent e) {
                this_windowClosing(e);
            }
        });
        toolbar_panel.setLayout(flowLayout1);
        flowLayout1.setAlignment(FlowLayout.LEFT);
        right_splitPane.setOrientation(JSplitPane.VERTICAL_SPLIT);
        right_splitPane.setBorder(null);
        right_splitPane.setDividerSize(4);
        save_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                save_button_actionPerformed(e);
            }
        });
        save_button.setIcon(Icons.INSTANCE.getSaveIcon());
        save_button.setText("Save");
        save_button.setToolTipText("Save");
        save_button.setPreferredSize(new Dimension(55, 20));
        save_button.setMinimumSize(new Dimension(24, 20));
        save_button.setFont(new java.awt.Font("Dialog", 0, 11));
        save_button.setBorder(null);
        save_button.setMaximumSize(new Dimension(60, 20));
        closeConfig_button.setMaximumSize(new Dimension(100, 20));
        closeConfig_button.setFont(new java.awt.Font("Dialog", 0, 11));
        closeConfig_button.setBorder(null);
        closeConfig_button.setMinimumSize(new Dimension(24, 20));
        closeConfig_button.setPreferredSize(new Dimension(100, 20));
        closeConfig_button.setToolTipText("Close Config");
        closeConfig_button.setIcon(Icons.INSTANCE.getCloseConfigIcon());
        closeConfig_button.setText("Close Config");
        closeConfig_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                closeConfig_button_actionPerformed(e);
            }
        });
        openConfig_button.setMaximumSize(new Dimension(100, 20));
        openConfig_button.setFont(new java.awt.Font("Dialog", 0, 11));
        openConfig_button.setBorder(null);
        openConfig_button.setMinimumSize(new Dimension(24, 20));
        openConfig_button.setPreferredSize(new Dimension(100, 20));
        openConfig_button.setToolTipText("Open Config");
        openConfig_button.setIcon(Icons.INSTANCE.getOpenConfigIcon());
        openConfig_button.setText("Open Config");
        openConfig_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                openConfig_button_actionPerformed(e);
            }
        });
        run_button.setMaximumSize(new Dimension(60, 20));
        run_button.setEnabled(false);
        run_button.setFont(new java.awt.Font("Dialog", 0, 11));
        run_button.setBorder(null);
        run_button.setMinimumSize(new Dimension(24, 20));
        run_button.setPreferredSize(new Dimension(50, 20));
        run_button.setToolTipText("Run");
        run_button.setIcon(Icons.INSTANCE.getExecuteIcon());
        run_button.setText("Run");
        run_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                run_button_actionPerformed(e);
            }
        });
        debug_button.setMaximumSize(new Dimension(80, 20));
        debug_button.setEnabled(false);
        debug_button.setFont(new java.awt.Font("Dialog", 0, 11));
        debug_button.setBorder(null);
        debug_button.setMinimumSize(new Dimension(24, 20));
        debug_button.setPreferredSize(new Dimension(60, 20));
        debug_button.setToolTipText("Debug");
        debug_button.setIcon(Icons.INSTANCE.getDebugIcon());
        debug_button.setText("Debug");
        debug_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                debug_button_actionPerformed(e);
            }
        });
        build_button.setMaximumSize(new Dimension(80, 20));
        build_button.setFont(new java.awt.Font("Dialog", 0, 11));
        build_button.setBorder(null);
        build_button.setMinimumSize(new Dimension(24, 20));
        build_button.setPreferredSize(new Dimension(55, 20));
        build_button.setToolTipText(Actions.BUILD.getName());
        build_button.setIcon(Actions.BUILD.getIcon());
        build_button.setText(Actions.BUILD.getName());
        build_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
//                build_button_actionPerformed(e);
                Actions.BUILD.actionPerformed(e);
            }
        });
        readStructure_button.setText("Read Structure");
        readStructure_button.setToolTipText("Build");
        readStructure_button.setPreferredSize(new Dimension(85, 20));
        readStructure_button.setMinimumSize(new Dimension(24, 20));
        readStructure_button.setFont(new java.awt.Font("Dialog", 0, 11));
        readStructure_button.setBorder(BorderFactory.createEtchedBorder());
        readStructure_button.setMaximumSize(new Dimension(85, 20));
        dumpStructure_button.setMaximumSize(new Dimension(85, 20));
        dumpStructure_button.setBorder(BorderFactory.createEtchedBorder());
        dumpStructure_button.setFont(new java.awt.Font("Dialog", 0, 11));
        dumpStructure_button.setMinimumSize(new Dimension(24, 20));
        dumpStructure_button.setPreferredSize(new Dimension(85, 20));
        dumpStructure_button.setToolTipText("Build");
        dumpStructure_button.setText("Dump Structure");
        emacsTest_button.setText("Emacs Test");
        emacsTest_button.setToolTipText("Build");
        emacsTest_button.setPreferredSize(new Dimension(70, 20));
        emacsTest_button.setMinimumSize(new Dimension(24, 20));
        emacsTest_button.setFont(new java.awt.Font("Dialog", 0, 11));
        emacsTest_button.setBorder(BorderFactory.createEtchedBorder());
        emacsTest_button.setMaximumSize(new Dimension(70, 20));
        jLabel1.setFont(new java.awt.Font("Dialog", 0, 11));
        jLabel1.setText("Testing: ");
        tools_menu.setFont(new java.awt.Font("Dialog", 0, 11));
        tools_menu.setText("Tools");
        joinpointProbe_menuItem.setFont(new java.awt.Font("Dialog", 0, 11));
        joinpointProbe_menuItem.setText(Actions.JOINPOINT_PROBE.getName());
        joinpointProbe_menuItem.setIcon(Actions.JOINPOINT_PROBE.getIcon());
        joinpointProbe_menuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                joinpointProbe_menuItem_actionPerformed(e);
            }
        });
        projectDebug_menuItem.setEnabled(false);
        projectDebug_menuItem.setFont(new java.awt.Font("Dialog", 0, 11));
        projectDebug_menuItem.setText("Debug");
        editConfig_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                editConfig_button_actionPerformed(e);
            }
        });
        editConfig_button.setIcon(Icons.INSTANCE.getFileLstIcon());
        editConfig_button.setText("Edit Config");
        editConfig_button.setToolTipText("Edit Config...");
        editConfig_button.setPreferredSize(new Dimension(80, 20));
        editConfig_button.setMinimumSize(new Dimension(24, 20));
        editConfig_button.setFont(new java.awt.Font("Dialog", 0, 11));
        editConfig_button.setBorder(null);
        editConfig_button.setMaximumSize(new Dimension(80, 20));
        project_toolBar.setBorder(BorderFactory.createEmptyBorder());
        file_toolBar.setBorder(BorderFactory.createEmptyBorder());
        options_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                options_button_actionPerformed(e);
            }
        });
        options_button.setIcon(Icons.INSTANCE.getBrowserOptionsIcon());
        options_button.setText("Options");
        options_button.setToolTipText("Options...");
        options_button.setPreferredSize(new Dimension(60, 20));
        options_button.setMinimumSize(new Dimension(24, 20));
        options_button.setFont(new java.awt.Font("Dialog", 0, 11));
        options_button.setBorder(null);
        options_button.setMaximumSize(new Dimension(80, 20));
        svProperties_menuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                svProperties_menuItem_actionPerformed(e);
            }
        });
        svProperties_menuItem.setText("Options...");
        svProperties_menuItem.setActionCommand("AJDE Console...");
        svProperties_menuItem.setFont(new java.awt.Font("Dialog", 0, 11));
        this.getContentPane().add(top_splitPane, BorderLayout.CENTER);
        top_splitPane.add(browser_panel, JSplitPane.LEFT);
        top_splitPane.add(right_splitPane, JSplitPane.RIGHT);
        right_splitPane.add(messages_panel, JSplitPane.BOTTOM);
        right_splitPane.add(editor_panel, JSplitPane.TOP);
        this.getContentPane().add(status_panel, BorderLayout.SOUTH);
        status_panel.add(statusText_label, BorderLayout.CENTER);
        this.getContentPane().add(toolbar_panel, BorderLayout.NORTH);
        toolbar_panel.add(project_toolBar, null);
        project_toolBar.add(build_button, null);
        project_toolBar.add(run_button, null);
        project_toolBar.add(debug_button, null);
        toolbar_panel.add(file_toolBar, null);
        file_toolBar.add(openConfig_button, null);
        file_toolBar.add(closeConfig_button, null);
        file_toolBar.add(editConfig_button, null);
        file_toolBar.add(save_button, null);
        file_toolBar.add(options_button, null);
        toolbar_panel.add(testing_toolBar, null);
        testing_toolBar.add(jLabel1, null);
        testing_toolBar.add(emacsTest_button, null);
        testing_toolBar.add(dumpStructure_button, null);
        testing_toolBar.add(readStructure_button, null);
        menuBar.add(jMenu1);
        menuBar.add(jMenu2);
        menuBar.add(tools_menu);
        jMenu1.add(jMenuItem1);
        jMenu1.addSeparator();
        jMenu1.add(exit_menuItem);
        jMenu2.add(projectBuild_menuItem);
        jMenu2.add(projectRun_menuItem);
        jMenu2.add(projectDebug_menuItem);
        tools_menu.add(joinpointProbe_menuItem);
        tools_menu.add(svProperties_menuItem);
        top_splitPane.setDividerLocation(380);
        right_splitPane.setDividerLocation(500);
        testing_toolBar.addSeparator();
        testing_toolBar.addSeparator();
        project_toolBar.addSeparator();
        project_toolBar.addSeparator();
    }

    private void exit_menuItem_actionPerformed(ActionEvent e) {
        quit();
    }

    private void this_windowClosing(WindowEvent e) {
        quit();
    }

    private void quit() {
        this.dispose();
        System.exit(0);
    }


    void treeMode_comboBox_actionPerformed(ActionEvent e) { }

    void save_button_actionPerformed(ActionEvent e) {
        TopManager.EDITOR_MANAGER.saveContents();
    }


    void this_windowClosed(WindowEvent e) {
        quit();
    }

    public void showMessagesPanel() {
        right_splitPane.setDividerLocation(right_splitPane.getHeight()-100);
        messages_panel.setVisible(true);
    }

    public void hideMessagesPanel() {
        right_splitPane.setDividerLocation(right_splitPane.getHeight());
        messages_panel.setVisible(false);
    }

    void closeConfig_button_actionPerformed(ActionEvent e) {
        TopManager.IDE_MANAGER.removeConfigFile(TopManager.BROWSER_MANAGER.getCurrConfigFile());
        //(String)org.aspectj.ajde.views.BrowserPanel.configs_comboBox.getSelectedItem());
    }

    void emacsTest_button_actionPerformed(ActionEvent e) {
//        Tester.emacsCompile(TopManager.BROWSER_MANAGER.getCurrConfigFile());
    }

    void debug_button_actionPerformed(ActionEvent e) {

    }

    void openConfig_button_actionPerformed(ActionEvent e) {
        JFileChooser fileChooser = new JFileChooser();
        fileChooser.setFileFilter(new FileFilter() {
            public boolean accept(File f) {
                if (f.getPath().endsWith(".lst") || f.isDirectory()) {
                    return true;
                } else {
                    return false;
                }
            }
            public String getDescription() {
                return "AspectJ Build Configuration (*.lst)";
            }
        });
        int returnVal = fileChooser.showOpenDialog(this);
        if(returnVal == JFileChooser.APPROVE_OPTION) {
            TopManager.IDE_MANAGER.mountConfigFile(fileChooser.getSelectedFile().getAbsolutePath().replace('\\', '/'));
        }
    }

    void joinpointProbe_menuItem_actionPerformed(ActionEvent e) {
        Actions.JOINPOINT_PROBE.actionPerformed(e);
    }

    void jMenuItem1_actionPerformed(ActionEvent e) {
        TopManager.EDITOR_MANAGER.saveContents();
    }

    void projectBuild_menuItem_actionPerformed(ActionEvent e) {
        TopManager.IDE_MANAGER.saveAll();
        TopManager.IDE_MANAGER.build();
    }

    void run_button_actionPerformed(ActionEvent e) {
        TopManager.IDE_MANAGER.run();
    }

    void projectRun_menuItem_actionPerformed(ActionEvent e) {
        TopManager.IDE_MANAGER.run();
    }

    void options_button_actionPerformed(ActionEvent e) {
        TopManager.showOptionsFrame(OptionsFrame.STRUCTURE_PROPERTIES);
    }

    void editConfig_button_actionPerformed(ActionEvent e) {
        TopManager.IDE_MANAGER.editConfigFile(TopManager.BROWSER_MANAGER.getCurrConfigFile());
    }

    public void resetSourceEditorPanel() {
        right_splitPane.removeAll();
        right_splitPane.add(sourceEditor_panel, JSplitPane.TOP);
    }
    void svProperties_menuItem_actionPerformed(ActionEvent e) {
        TopManager.showOptionsFrame(OptionsFrame.STRUCTURE_PROPERTIES);
    }
}

//    private static class Tester {
//        public static void emacsCompile(String lstFile) {
//            try {
//                new org.aspectj.ajde.compiler.AjdeCompiler(null, true).compile(
//                    org.aspectj.util.ConfigFileUtil.expandLstFile(lstFile));
//            } catch (Exception e) {
//                e.printStackTrace();
//            }
//            System.err.println("> not implemented");
//        }
//
//        public static void dumpStructure() {
//            TopManager.BROWSER_MANAGER.dumpStructureView();
//        }
//
//        public static void readStructure() {
//            TopManager.BROWSER_MANAGER.readStructureView();
//            TopManager.BROWSER_MANAGER.updateView();
//        }
//    }