
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.asm;

import java.io.*;

/**
 * @author Mik Kersten
 */
public class LinkNode extends StructureNode {
    private ProgramElementNode programElementNode = null;

    /**
     * Used during de-serialization.
     */
    public LinkNode() { }

    public LinkNode(ProgramElementNode node) {
        super(node.getSignature().toString(), "internal", null);
        this.programElementNode = node;
    }

    public ProgramElementNode getProgramElementNode() {
        return programElementNode;
    }

    public String toString() {
        String name = "";
        if (programElementNode.getKind().equals(ProgramElementNode.KIND_ADVICE) ||
            programElementNode.getKind().equals(ProgramElementNode.KIND_INTRODUCTION) ||
            programElementNode.getKind().equals(ProgramElementNode.KIND_CODE)) {
            return programElementNode.getDeclaringType() + ": " + programElementNode.getName();
        } else if (programElementNode.isMemberKind()) {
            return programElementNode.getDeclaringType() + '.' + programElementNode.getName();
        } else {
            return programElementNode.toString();
        }
    }
}