
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.asm;

import java.util.*;
import java.io.*;
import org.aspectj.compiler.base.ast.*;
import org.aspectj.lang.Signature;

/**
 * @author Mik Kersten
 */
public class ProgramElementNode extends StructureNode {

    public static final String MODIFIER_STATIC = "static";
    public static final String MODIFIER_FINAL = "final";
    public static final String MODIFIER_ABSTRACT = "abstract";
    public static final String MODIFIER_SYNCHRONIZED = "synchronized";
    public static final String MODIFIER_VOLATILE = "volatile";
    public static final String MODIFIER_STRICTFP = "strictfp";
    public static final String MODIFIER_NATIVE = "native";

    public static final String ACCESSIBILITY_PUBLIC = "public";
    public static final String ACCESSIBILITY_PACKAGE = "package";
    public static final String ACCESSIBILITY_PROTECTED = "protected";
    public static final String ACCESSIBILITY_PRIVATE = "private";
    public static final String ACCESSIBILITY_PRIVILEGED = "privileged";

    public static final String KIND_PROJECT = "project";
    public static final String KIND_PACKAGE = "package";
    public static final String KIND_FILE = "file";
    public static final String KIND_CLASS = "class";
    public static final String KIND_INTERFACE = "interface";
    public static final String KIND_ASPECT = "aspect";
    public static final String KIND_INITIALIZER = "initializer";
    public static final String KIND_INTRODUCTION = "introduction";
    public static final String KIND_CONSTRUCTOR = "constructor";
    public static final String KIND_METHOD = "method";
    public static final String KIND_FIELD = "field";
    public static final String KIND_POINTCUT = "pointcut";
    public static final String KIND_ADVICE = "advice";
    public static final String KIND_DECLARE_PARENTS = "declare parents";
    public static final String KIND_DECLARE_WARNING = "declare warning";
    public static final String KIND_DECLARE_ERROR = "declare error";
    public static final String KIND_DECLARE_SOFT = "declare soft";
    public static final String KIND_CODE = "decBodyElement";

    private List modifiers;
    private List accessibility;
    private List relations;

    private String declaringType = "";
//    private String memberReferenceType = "";
    private String formalComment = "";
    private String packageName = null;
    private String sourceFilePath = null;
    private int beginLine = 0;
    private int beginColumn = 0;
    private boolean member = false;
    private transient ASTObject astObject = null;

    /**
     * Used during de-externalization.
     */
    public ProgramElementNode() { }

    public ProgramElementNode(String signature, String kind, List modifiers, List accessibility,
        String declaringType, String packageName, String formalComment, String sourceFilePath,
        int beginLine, List relations, List children, boolean member, ASTObject astObject) {

        super(signature, kind, children);
        this.modifiers = modifiers;
        this.accessibility = accessibility;
        this.declaringType = declaringType;
        this.packageName = packageName;
        this.formalComment = formalComment;
        this.sourceFilePath = sourceFilePath;
        this.beginLine = beginLine;
        this.relations = relations;
        this.member = member;
        this.astObject = astObject;

    }

    public String getSourceFilePath() {
        return sourceFilePath;
    }

    public int getBeginLine() {
        return beginLine;
    }

    public List getModifiers() {
        return modifiers;
    }

    public List getAccessibility() {
        return accessibility;
    }

    public String getDeclaringType() {
        return declaringType;
    }

    public String getPackageName() {
        return packageName;
    }

    public ASTObject getAstObject() {
        return astObject;
    }

    public int getBeginColumn() {
        return 0;
    }

    public String getKind() {
        return super.kind;
    }

    public String getSignature() {
        return super.name;
    }

    public boolean isCode() {
        return kind.equals("decBodyElement");
    }

    public boolean isMemberKind() {
        return member;
    }

    public List getRelations() {
        return relations;
    }

    public void setRelations(List relations) {
        if (relations.size() > 0) {
            this.relations = relations;
        }
    }

    public String getFormalComment() {
        return formalComment;
    }

    public String toString() {
        return super.name;
    }

    public boolean equals(ProgramElementNode node) {
        if (node == null) {
            return false;
        } else {
            if (name.equals(node.getName()) &&
                sourceFilePath.equals(node.getSourceFilePath()) &&
                beginLine == node.getBeginLine() &&
                beginColumn == node.getBeginColumn()) {
                return true;
            } else {
                return false;
            }
        }
    }
}
