
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.asm;

import java.util.*;
import java.io.*;

/**
 * @author Mik Kersten
 */
public class StructureNode implements Serializable, Comparable {

    protected String name = "";
    protected String kind = "";
    protected List children = new ArrayList();

    /**
     * Used during de-externalization.
     */
    public StructureNode() { }

    public StructureNode(String name, String kind, List children) {
        this.name = name;
        this.kind = kind;
        this.children = children;
    }

    public StructureNode(String name, String kind) {
        this.name = name;
        this.kind = kind;
    }

    public String toString() {
        return name;
    }

    public String getKind() {
        return kind;
    }

    public List getChildren() {
        return children;
    }

    public void addChild(StructureNode child) {
        if (children == null) {
            children = new ArrayList();
        }
        children.add(child);
    }

    /**
     * HACK: some structure nodes might have the same name/signature, ordering is
     * string-based and has nothing to do w/ structure.
     */
    public int compareTo(Object o) throws ClassCastException {
        if (this == o) {
            return 0;
        } else {
            StructureNode sn = (StructureNode)o;
            return this.getName().compareTo(sn.getName());
        }
    }

    public String getName() {
        return name;
    }
}


//    private void writeObject(ObjectOutputStream s) throws IOException {
//        s.defaultWriteObject();
//        // customized serialization code
//    }
//
//    private void readObject(ObjectInputStream s) throws IOException  {
//        s.defaultReadObject();
//        // customized deserialization code
//        ...
//        // followed by code to update the object, if necessary
//    }

//    public void writeExternal(ObjectOutput out) throws IOException {
//        System.err.println("> writing node: " + this.getClass());
//        if (this instanceof ProgramElementNode) {
//            out.writeInt(1);
//            writeString(name, out);
//            writeString(kind, out);
//            ((ProgramElementNode)this).writeExternal(out);
//        } if (this instanceof RelationNode) {
//            out.writeInt(1);
//            writeString(name, out);
//            writeString(kind, out);
//            ((RelationNode)this).writeExternal(out);
//        } if (this instanceof LinkNode) {
//            out.writeInt(3);
//            writeString(name, out);
//            writeString(kind, out);
//            ((LinkNode)this).writeExternal(out);
//        } else {
//            out.writeInt(0);
//            writeString(name, out);
//            writeString(kind, out);
//        }
//    }
//
//    public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
//        int kindint = in.readInt();
//        name = readString(in);
//        kind = readString(in);
//        System.err.println("> read: " + kind);
//
//        switch (kindint) {
//            case 1:
//                ((StructureNode)it.next()).readExternal(in);
//                break;
//            case 2:
//                ((RelationNode)it.next()).readExternal(in);
//                break;
//            case 3:
//                ((LinkNode)it.next()).readExternal(in);
//                break;
//        }
//    }
//
//    protected void writeString(String s, ObjectOutput out) throws IOException {
//        out.writeInt(s.length());
//        out.write(s.getBytes());
//    }
//
//    protected String readString(ObjectInput in) throws IOException {
//        int length = in.readInt();
//        byte[] nameArray = new byte[length];
//        in.read(nameArray, 0, length);
//        return new String(nameArray);
//    }