/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base;

import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.base.*;

import java.util.*;

/** This pass is responsible for removing dead and/or useless code
    from the AST.  As such, it includes some control-related stuff
    otherwise found in the flow-related passes.  It should signal no
    errors.  
*/

public class ByteCodeCleanupPass extends WalkerPass {
    public ByteCodeCleanupPass(JavaCompiler compiler) {
        super(compiler);
    }

    public String getDisplayName() {
        return "clean up ast";
    }

    public ASTObject process(ASTObject object) {
        object.walkCleanup(this);
        ASTObject ret = object.postCleanup(this);
        if (ret instanceof Stmt) {
            ((Stmt)ret).setCompletesNormally(isLive());
        }
        return ret;
    }

    private boolean liveFlag = true;
    public void setLive(boolean b) { liveFlag = b; }
    public boolean isLive() { return liveFlag; }    

    // ------------------------------
    private ControlContext context = new ControlContext();
    
    public void enterContext(Stmt s) { context.enter(s); }
    public void leaveContext() { context.exit(); }

    public void doBreak(String label) {
        breaks.add(context.getBreakTarget(label));
    }

    public void doContinue(String label) {
        continues.add(context.getContinueTarget(label));
    }

    private Set breaks = new HashSet();
    private Set continues = new HashSet();

    public boolean isBroken(Stmt s) { return breaks.contains(s); }
    public boolean isContinued(Stmt s) { return continues.contains(s); }
}
