/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base;

import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.base.*;

import java.util.*;

// This pass assigns frame locations to VarDecs, finally parts of
// TryStmts, and synchronizeBlocks.  The finally parts of TryStmts and
// synchronize blocks need two frame locations: one for the (possible)
// exception thrown, and one for the return pc for the JSR part. 
// It also sets the max frame size for MethodDecs. 

// This must be run after all local classes are lifted. 

// This is _totally_ a non-structure-effecting visitor pass.

public final class FrameLocPass extends WalkerPass {
    public FrameLocPass(JavaCompiler jc) { super(jc); }
    
    public String getDisplayName() { return "assigning frame locs"; }

    public boolean inTryFinally = false;
    private int maxfs = 0;
    private int fs = 0;

    // ------------------------------
    // stack and frame

    public int getmaxfs() {
        return maxfs;
    }
    public int getfs() {
        return fs;
    }
    public void setfs(int fs) {
        if (! inTryFinally)
            this.fs = fs;
        frameInvariant();
    }
    public void incfs(int delta) {
        this.fs += delta;
        frameInvariant();
    }
    public int allocate(int size) {
        int ret = this.fs;
        incfs(size);
        return ret;
    }
    private void frameInvariant() {
        if (fs > maxfs) maxfs = fs;
    }

    public ASTObject process(ASTObject object) {
        object.walkFrameLoc(this);
        return object;
    }
}
