/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.FlowCheckerPass;
import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.compiler.base.CodeWriter;
import java.io.IOException;

import java.util.*;

/**
 * @grammar {exprs}
 * @child Exprs exprs
 */
public class ArrayInitializer extends Expr {
    protected Type discoverType() {
        return getTypeManager().anyType;
    }

    public void unparse(CodeWriter writer) throws IOException {
        writer.openParen('{');
        writer.write(exprs);
        writer.closeParen('}');
    }

    public void cleanup() {
        exprs = null;
        super.cleanup();
    }

    //BEGIN: Generated from @child and @property
    protected Exprs exprs;
    public Exprs getExprs() { return exprs; }
    public void setExprs(Exprs _exprs) {
        if (_exprs != null) _exprs.setParent(this);
        exprs = _exprs;
    }

    public ArrayInitializer(SourceLocation location, Exprs _exprs) {
        super(location);
        setExprs(_exprs);
    }
    protected ArrayInitializer(SourceLocation source) {
        super(source);
    }

    public ASTObject copyWalk(CopyWalker walker) {
        ArrayInitializer ret = new ArrayInitializer(getSourceLocation());
        ret.preCopy(walker, this);
        if (exprs != null) ret.setExprs( (Exprs)walker.process(exprs) );
        return ret;
    }

    public ASTObject getChildAt(int childIndex) {
        switch(childIndex) {
        case 0: return exprs;
        default: return super.getChildAt(childIndex);
        }
    }
     public String getChildNameAt(int childIndex) {
        switch(childIndex) {
        case 0: return "exprs";
        default: return super.getChildNameAt(childIndex);
        }
    }
     public void setChildAt(int childIndex, ASTObject child) {
        switch(childIndex) {
        case 0: setExprs((Exprs)child); return;
        default: super.setChildAt(childIndex, child); return;
        }
    }
     public int getChildCount() {
        return 1;
    }

    public String getDefaultDisplayName() {
        return "ArrayInitializer()";
    }

    //END: Generated from @child and @property
}
