/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.*;
import org.aspectj.compiler.base.cst.*;

import java.util.*;


/**
 * @grammar componentTypeD[]
 * @child TypeD componentTypeD
 */
public class ArrayTypeD extends TypeD {
    public String getString() {
        return componentTypeD.getString()+"[]";
    }

    public String toShortString() {
        return componentTypeD.toShortString()+"[]";
    }

    public void unparse(CodeWriter writer) {
        writer.write(componentTypeD);
        writer.write("[]");
    }

    //??? The call to getArrayType is synchronized.  Should we cache
    //this?
    public Type getType() {
        return componentTypeD.getType().getArrayType();
    }

    //!!! replace with isEquivalent
    public boolean equals(Object other) {
        if (!(other instanceof ArrayTypeD)) return false;
        return componentTypeD.equals(((ArrayTypeD)other).getComponentTypeD());
    }

    //BEGIN: Generated from @child and @property
    protected TypeD componentTypeD;
    public TypeD getComponentTypeD() { return componentTypeD; }
    public void setComponentTypeD(TypeD _componentTypeD) {
        if (_componentTypeD != null) _componentTypeD.setParent(this);
        componentTypeD = _componentTypeD;
    }

    public ArrayTypeD(SourceLocation location, TypeD _componentTypeD) {
        super(location);
        setComponentTypeD(_componentTypeD);
    }
    protected ArrayTypeD(SourceLocation source) {
        super(source);
    }

    public ASTObject copyWalk(CopyWalker walker) {
        ArrayTypeD ret = new ArrayTypeD(getSourceLocation());
        ret.preCopy(walker, this);
        if (componentTypeD != null) ret.setComponentTypeD( (TypeD)walker.process(componentTypeD) );
        return ret;
    }

    public ASTObject getChildAt(int childIndex) {
        switch(childIndex) {
        case 0: return componentTypeD;
        default: return super.getChildAt(childIndex);
        }
    }
     public String getChildNameAt(int childIndex) {
        switch(childIndex) {
        case 0: return "componentTypeD";
        default: return super.getChildNameAt(childIndex);
        }
    }
     public void setChildAt(int childIndex, ASTObject child) {
        switch(childIndex) {
        case 0: setComponentTypeD((TypeD)child); return;
        default: super.setChildAt(childIndex, child); return;
        }
    }
     public int getChildCount() {
        return 1;
    }

    public String getDefaultDisplayName() {
        return "ArrayTypeD()";
    }

    //END: Generated from @child and @property
}
