/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.*;
import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.compiler.base.CodeWriter;
import java.io.IOException;

import org.aspectj.compiler.base.bcg.CodeBuilder;
import org.aspectj.compiler.base.bcg.Label;

/**
  * @grammar break label?
  * @property String label
  */
public class BreakStmt extends Stmt {

    public void unparse(CodeWriter writer) throws IOException {
        writer.writeKeyword("break");
        if (label != null) {
            writer.requiredSpace();
            writer.write(label);
        }
        writer.closeStmt();
    }

    // ------------------------------
    // INTRO from FlowCheckerPass

    public void walkFlow(FlowCheckerPass w) {
        try {
            w.doBreak(getLabel());
        } catch (java.util.NoSuchElementException e) {
            if (getLabel() == null)
                showError("break outside of switch or loop");
            else
                showError("undefined label: " + getLabel());
        }
        w.setLive(false);
        w.setVars(w.getAllVars());
    }

    // ------------------------------
    // INTRO from ByteCodeCleanupPass

    public void walkCleanup(ByteCodeCleanupPass w) {
        w.doBreak(getLabel());
        w.setLive(false);
    }

    // ------------------------------
    // bcg

    protected void cgStmt(CodeBuilder cb) {
        cb.doBreak(label);
    }
    // do not register a line number for me, since often I won't have a bytecode
    protected void registerLocation(CodeBuilder cb) { }

    //BEGIN: Generated from @child and @property
    protected String label;
    public String getLabel() { return label; }
    public void setLabel(String _label) { label = _label; }

    public BreakStmt(SourceLocation location, String _label) {
        super(location);
        setLabel(_label);
    }
    protected BreakStmt(SourceLocation source) {
        super(source);
    }

    public ASTObject copyWalk(CopyWalker walker) {
        BreakStmt ret = new BreakStmt(getSourceLocation());
        ret.preCopy(walker, this);
        ret.label = label;
        return ret;
    }


    public String getDefaultDisplayName() {
        return "BreakStmt(label: "+label+")";
    }

    //END: Generated from @child and @property
}


