/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.JavaCompiler;

import org.aspectj.compiler.base.bcg.FieldBuilder;
import org.aspectj.compiler.base.bcg.CodeBuilder;
import org.aspectj.compiler.base.bcg.Label;


/**
  * @grammar doubleLiteral
  * @property double doubleValue
  */
public class DoubleLiteralExpr extends NumericLiteralExpr {

    public int    getIntValue()    { return (int)doubleValue; }
    public long   getLongValue()   { return (long)doubleValue; }
    public float  getFloatValue()  { return (float)doubleValue; }
    public String getStringValue() { return "" + doubleValue; }

    public DoubleLiteralExpr(SourceLocation source, double _doubleValue) {
        this(source, source.getCompiler().getTypeManager().doubleType, createUsefulString(_doubleValue), _doubleValue);
    }

    private static String createUsefulString(double d) {
        if (Double.isNaN(d)) {
            return "(0.0 / 0.0)";
        } else if (Double.isInfinite(d)) {
            return d > 0 ? "(1.0 / 0.0)" : "(-1.0 / 0.0)";
        } else {
            return d + "d";
        }
    }

    // ------------------------------
    // bcg
    protected void cgValue(CodeBuilder cb) {
        cb.emitDoubleConstant(doubleValue);
    }
    public void addConstant(FieldBuilder fb) {
        fb.setConstantValue(doubleValue);
    }
    public boolean isConstantZero() { return doubleValue == 0.0d; }
    // end bcg

    //BEGIN: Generated from @child and @property
    protected double doubleValue;
    public double getDoubleValue() { return doubleValue; }
    public void setDoubleValue(double _doubleValue) { doubleValue = _doubleValue; }

    public DoubleLiteralExpr(SourceLocation location, Type _type, String _value, double _doubleValue) {
        super(location, _type, _value);
        setDoubleValue(_doubleValue);
    }
    protected DoubleLiteralExpr(SourceLocation source) {
        super(source);
    }

    public ASTObject copyWalk(CopyWalker walker) {
        DoubleLiteralExpr ret = new DoubleLiteralExpr(getSourceLocation());
        ret.preCopy(walker, this);
        ret.type = type;
        ret.value = value;
        ret.doubleValue = doubleValue;
        return ret;
    }


    public String getDefaultDisplayName() {
        return "DoubleLiteralExpr(type: "+type+", "+"value: "+value+", "+"doubleValue: "+doubleValue+")";
    }

    //END: Generated from @child and @property
}
