/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.*;
import org.aspectj.compiler.base.*;

/**
  * Used to keep track of the (currently FieldDec) that produces a Stmt
  * so that we can handle getLexicalType correctly both for accessibility and
  * for within matching for introduced fields with non-empty initializers
  *
  * @grammar stmt;
  * @child Stmt stmt
  * @property Dec fromDec
  */
public class FromDecStmt extends Stmt {
    public Type getLexicalType() {
        return fromDec.getLexicalType();
    }

    public void unparse(CodeWriter writer) {
        writer.write(stmt);
    }

    public ASTObject postFixAST(final ASTFixerPass fixer) {
        return stmt;
    }

    public ASTObject postCleanup(ByteCodeCleanupPass w) {
        return stmt;
    }

    //BEGIN: Generated from @child and @property
    protected Stmt stmt;
    public Stmt getStmt() { return stmt; }
    public void setStmt(Stmt _stmt) {
        if (_stmt != null) _stmt.setParent(this);
        stmt = _stmt;
    }

    protected Dec fromDec;
    public Dec getFromDec() { return fromDec; }
    public void setFromDec(Dec _fromDec) { fromDec = _fromDec; }

    public FromDecStmt(SourceLocation location, Stmt _stmt, Dec _fromDec) {
        super(location);
        setStmt(_stmt);
        setFromDec(_fromDec);
    }
    protected FromDecStmt(SourceLocation source) {
        super(source);
    }

    public ASTObject copyWalk(CopyWalker walker) {
        FromDecStmt ret = new FromDecStmt(getSourceLocation());
        ret.preCopy(walker, this);
        if (stmt != null) ret.setStmt( (Stmt)walker.process(stmt) );
        ret.fromDec = fromDec;
        return ret;
    }

    public ASTObject getChildAt(int childIndex) {
        switch(childIndex) {
        case 0: return stmt;
        default: return super.getChildAt(childIndex);
        }
    }
     public String getChildNameAt(int childIndex) {
        switch(childIndex) {
        case 0: return "stmt";
        default: return super.getChildNameAt(childIndex);
        }
    }
     public void setChildAt(int childIndex, ASTObject child) {
        switch(childIndex) {
        case 0: setStmt((Stmt)child); return;
        default: super.setChildAt(childIndex, child); return;
        }
    }
     public int getChildCount() {
        return 1;
    }

    public String getDefaultDisplayName() {
        return "FromDecStmt(fromDec: "+fromDec+")";
    }

    //END: Generated from @child and @property
}


