/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.*;
import java.io.IOException;


import org.aspectj.compiler.base.bcg.CodeBuilder;
import org.aspectj.compiler.base.bcg.Label;


public final class IntType extends IntishType {

    protected boolean isAssignableFromOtherType(Type other) {
        return (other instanceof IntishType);
    }

    public String getName() { return "int"; }
    public int getTypeIndex() { return Type.INT; }
    public Type getRefType() { return getTypeManager().getType("java.lang", "Integer"); }

    public IntType(JavaCompiler compiler) {
        super(compiler);
    }

    // vm internals
    public String getDescriptor() { return "I"; }

    // folding
    LiteralExpr foldCast(LiteralExpr rand) {
        return getAST().makeLiteral(rand.getIntValue());
    }

    // code gen
    void emitCastFromInt(CodeBuilder cb)    {               }
    void emitCastFromFloat(CodeBuilder cb)  { cb.emitF2I(); }
    void emitCastFromLong(CodeBuilder cb)   { cb.emitL2I(); }
    void emitCastFromDouble(CodeBuilder cb) { cb.emitD2I(); }

    void emitAload(CodeBuilder cb) { cb.emitIALOAD(); }
    void emitAstore(CodeBuilder cb) { cb.emitIASTORE(); }

    // ------------------------------
    // "fast" opcode handling

    boolean hasFastIncOp(Expr lhs, int rhs) {
        return (lhs instanceof VarExpr)
            && (Short.MIN_VALUE <= rhs)
            && (rhs <= Short.MAX_VALUE);
    }
    void emitFastIncOp(CodeBuilder cb, Expr lhs, int rhs) {
        cb.emitIINC(((VarExpr) lhs).getVarDec().getFrameLoc(), rhs);
    }
}


