/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.*;

import java.util.*;

import org.aspectj.compiler.base.JavaCompiler;

/**
 * @grammar ...
 * @property String packageName
 * @property String className
 */
public class LazyNameTypeD extends TypeD {
    public Type getType() {
        return getTypeManager().getType(packageName, className);
    }

    //BEGIN: Generated from @child and @property
    protected String packageName;
    public String getPackageName() { return packageName; }
    public void setPackageName(String _packageName) { packageName = _packageName; }

    protected String className;
    public String getClassName() { return className; }
    public void setClassName(String _className) { className = _className; }

    public LazyNameTypeD(SourceLocation location, String _packageName, String _className) {
        super(location);
        setPackageName(_packageName);
        setClassName(_className);
    }
    protected LazyNameTypeD(SourceLocation source) {
        super(source);
    }

    public ASTObject copyWalk(CopyWalker walker) {
        LazyNameTypeD ret = new LazyNameTypeD(getSourceLocation());
        ret.preCopy(walker, this);
        ret.packageName = packageName;
        ret.className = className;
        return ret;
    }


    public String getDefaultDisplayName() {
        return "LazyNameTypeD(packageName: "+packageName+", "+"className: "+className+")";
    }

    //END: Generated from @child and @property
}
