/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.FlowCheckerPass;
import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.compiler.base.CodeWriter;
import java.io.IOException;

import java.util.*;

import org.aspectj.compiler.base.bcg.CodeBuilder;
import org.aspectj.compiler.base.bcg.Label;

/**
 * @grammar ! rand1
 *
 */
public class LogNotOpExpr extends UnopExpr {

    protected LiteralExpr halfFold(Type type, LiteralExpr lit) {
        return type.foldLogNotOp(lit);
    }

    protected Type discoverType() {
        if (! getRand1().getType().isBoolean()) {
            showOperatorTypeError(getRand1().getType());
        }
        return getTypeManager().booleanType;
    }

    protected Type getLiftType() {
        return getType();
    }

    // ------------------------------
    // INTRO from FlowCheckerPass

    public void walkFlow(FlowCheckerPass w) {
        w.process(getRand1());
        FlowCheckerPass.Vars p1 = w.getVars();

        w.setVars(p1.getFalse(), p1.getTrue());
    }

    // ------------------------------
    // bcg
    protected void cgTest(CodeBuilder cb, Label tdest, Label fdest) {
        getRand1().cgTest(cb, fdest, tdest);
    }
    protected void cgValue(CodeBuilder cb) {
        getRand1().cgValue(cb);
        getTypeManager().booleanType.emitLogNot(cb);
    }

    //BEGIN: Generated from @child and @property

    public LogNotOpExpr(SourceLocation location, String _op, Expr _rand1) {
        super(location, _op, _rand1);

    }
    protected LogNotOpExpr(SourceLocation source) {
        super(source);
    }

    public ASTObject copyWalk(CopyWalker walker) {
        LogNotOpExpr ret = new LogNotOpExpr(getSourceLocation());
        ret.preCopy(walker, this);
        ret.op = op;
        if (rand1 != null) ret.setRand1( (Expr)walker.process(rand1) );
        return ret;
    }


    public String getDefaultDisplayName() {
        return "LogNotOpExpr(op: "+op+")";
    }

    //END: Generated from @child and @property
}
