/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.FlowCheckerPass;
import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.compiler.base.CodeWriter;

/**
 * @grammar expr.new innerName(args)
 * @property String innerName
 */

// this class should be in CST, really.
public class NewInnerInstanceExpr extends NewInstanceExpr {
    //INTRO from ScopeWalker
    public ASTObject postScope(ScopeWalker walker) {
        super.postScope(walker);
        return new NewInstanceExpr(getSourceLocation(), getEnclosingInstanceExpr(), getTypeD(), getArgs(), getTypeDec(), getConstructor());
    }

    public void preScope(ScopeWalker walker) {
        // XXX this is here because the type we construct may NOT be
        // inner.  If we were expr.new foo(args) { body }, then our
        // _supertype_ is inner, but if we live in static context then
        // we won't be inner.  We check that by checking if, by this
        // time, our enclosing instance expr is null.
        if (getEnclosingInstanceExpr() == null) {
            super.preScope(walker);
            return;
        }
        //System.out.println("pre-scope: " + this);
        setEnclosingInstanceExpr((Expr)walker.process(getEnclosingInstanceExpr()));
        Type myType =
            getEnclosingInstanceExpr().getType().getInnerType(getInnerName(),
                                                              this, true);
        setTypeD(myType.makeTypeD());
        if (typeDec != null) {
            ((ClassDec)typeDec).setSuperClass(myType.makeTypeD());
        }
    }

    //BEGIN: Generated from @child and @property
    protected String innerName;
    public String getInnerName() { return innerName; }
    public void setInnerName(String _innerName) { innerName = _innerName; }

    public NewInnerInstanceExpr(SourceLocation location, Expr _enclosingInstanceExpr, TypeD _typeD, Exprs _args, TypeDec _typeDec, Constructor _constructor, String _innerName) {
        super(location, _enclosingInstanceExpr, _typeD, _args, _typeDec, _constructor);
        setInnerName(_innerName);
    }
    protected NewInnerInstanceExpr(SourceLocation source) {
        super(source);
    }

    public ASTObject copyWalk(CopyWalker walker) {
        NewInnerInstanceExpr ret = new NewInnerInstanceExpr(getSourceLocation());
        ret.preCopy(walker, this);
        if (enclosingInstanceExpr != null) ret.setEnclosingInstanceExpr( (Expr)walker.process(enclosingInstanceExpr) );
        if (typeD != null) ret.setTypeD( (TypeD)walker.process(typeD) );
        if (args != null) ret.setArgs( (Exprs)walker.process(args) );
        if (typeDec != null) ret.setTypeDec( (TypeDec)walker.process(typeDec) );
        ret.constructor = constructor;
        ret.innerName = innerName;
        return ret;
    }


    public String getDefaultDisplayName() {
        return "NewInnerInstanceExpr(constructor: "+constructor+", "+"innerName: "+innerName+")";
    }

    //END: Generated from @child and @property
}
