/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.*;

import org.aspectj.compiler.base.bcg.CodeBuilder;

/**
 * @grammar typename.super
 * @child TypeD typeD
 */

public class QualifiedSuperExpr extends SuperExpr {
    public Type discoverType() {
        return typeD.getType();
    }

    public void unparse(CodeWriter writer) {
        writer.write(getType().getId());
        writer.write(".super");
    }
    
    public ThisExpr makeThis() {
    	return getAST().makeQualifiedThis(typeD.getType());
    }


    // ------------------------------
    // INTRO from MemberClassMunger
    public ASTObject walkMemberMunger(MemberClassMunger w) {
    	throw new RuntimeException("Invalid expression " + this);
    }


    // ------------------------------
    // bcg
    protected void cgValue(CodeBuilder cb) {
        throw new RuntimeException("Invalid expression " + this);
    }

    //BEGIN: Generated from @child and @property
    protected TypeD typeD;
    public TypeD getTypeD() { return typeD; }
    public void setTypeD(TypeD _typeD) {
        if (_typeD != null) _typeD.setParent(this);
        typeD = _typeD;
    }

    public QualifiedSuperExpr(SourceLocation location, TypeD _typeD) {
        super(location);
        setTypeD(_typeD);
    }
    protected QualifiedSuperExpr(SourceLocation source) {
        super(source);
    }

    public ASTObject copyWalk(CopyWalker walker) {
        QualifiedSuperExpr ret = new QualifiedSuperExpr(getSourceLocation());
        ret.preCopy(walker, this);
        if (typeD != null) ret.setTypeD( (TypeD)walker.process(typeD) );
        return ret;
    }

    public ASTObject getChildAt(int childIndex) {
        switch(childIndex) {
        case 0: return typeD;
        default: return super.getChildAt(childIndex);
        }
    }
     public String getChildNameAt(int childIndex) {
        switch(childIndex) {
        case 0: return "typeD";
        default: return super.getChildNameAt(childIndex);
        }
    }
     public void setChildAt(int childIndex, ASTObject child) {
        switch(childIndex) {
        case 0: setTypeD((TypeD)child); return;
        default: super.setChildAt(childIndex, child); return;
        }
    }
     public int getChildCount() {
        return 1;
    }

    public String getDefaultDisplayName() {
        return "QualifiedSuperExpr()";
    }

    //END: Generated from @child and @property
}
