/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.*;

import java.util.*;
import java.io.File;

public class SourceSourceLocation extends SyntheticSourceLocation {
    public SourceSourceLocation(ASTObject reference) {
        super(reference);
    }

    public boolean isSynthetic() {
        return true;
    }

    public int getStartPosition() {
        return getBaseSourceLocation().getStartPosition();
    }
    public int getEndPosition() {
        return getBaseSourceLocation().getEndPosition();
    }

    public int getBeginLine() {
//         System.out.println("getting line from: " + reference +
//                            " == " + getBaseSourceLocation().getBeginLine());
        return getBaseSourceLocation().getBeginLine();
    }

    public int getEndLine() {
        return getBaseSourceLocation().getEndLine();
    }

    public int getBeginColumn() {
        return getBaseSourceLocation().getBeginColumn();
    }

    public int getEndColumn() {
        return getBaseSourceLocation().getEndColumn();
    }


    public java.io.File getSourceFile() {
        return getBaseSourceLocation().getSourceFile();
    }

    public String getSourceFileName() {
        return getBaseSourceLocation().getSourceFileName();
    }

    public String getSourceDirectoryName() {
        return getBaseSourceLocation().getSourceDirectoryName();
    }

    public boolean hasSource() {
        return getBaseSourceLocation().hasSource();
    }


    public SourceLocation getBaseSourceLocation() {
        return getSourceObject().getSourceLocation();
    }

    public CompilationUnit getCompilationUnit() {
        return getSourceObject().getCompilationUnit();
    }

    public ASTObject getSourceObject() {
        return (ASTObject)reference;
    }
}
