/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.compiler.base.CompilerObject;

public class SyntheticGetSet extends CompilerObject {
    public BangExpr bangExpr;

    public SyntheticGetSet(JavaCompiler compiler, BangExpr bangExpr) {
        super(compiler);
        this.bangExpr = bangExpr;
    }

    public FieldAccessExpr makeGetExpr() {
        makeInnerExprs();
        return getExpr;
    }

    public AssignExpr makeSetExpr() {
        makeInnerExprs();
        return setExpr;
    }

    public BangExpr getBangExpr() { return bangExpr; }

    boolean isConcrete = false;
    public void makeConcrete() {
        if (isConcrete) return;
        //bangExpr.showWarning("  making concrete");
        isConcrete = true;
        makeInnerExprs();
        bangExpr.replaceWith(finishSetExpr(setExpr));
    }

    Expr finishSetExpr(Expr setExpr) {
        if (bangExpr instanceof PostfixExpr) {
            if (bangExpr.isInExprStmt()) return setExpr;

            PostfixExpr postfixExpr = (PostfixExpr)bangExpr;
            String op = "-";
            if (postfixExpr.getOp().equals("--")) op = "+";
            final AST ast = getAST();
            return ast.makeParen(ast.makeBinop(op, setExpr, ast.makeLiteral(1)));
        }
        return setExpr;
    }

    FieldAccessExpr getExpr;
    AssignExpr setExpr;

    AssignExpr makeAssignExpr(BangExpr bangExpr) {
        if (bangExpr instanceof AssignExpr) return (AssignExpr)bangExpr;
        if (bangExpr instanceof PrefixExpr) {
            PrefixExpr prefixExpr = (PrefixExpr)bangExpr;
            String op = prefixExpr.getOp().substring(1);
            final AST ast = getAST();
            return ast.makeSet(bangExpr.getLhs(), op, ast.makeLiteral(1));
        } else if (bangExpr instanceof PostfixExpr) {
            PostfixExpr postfixExpr = (PostfixExpr)bangExpr;
            String op = postfixExpr.getOp().substring(1);
            final AST ast = getAST();
            return ast.makeSet(bangExpr.getLhs(), op, ast.makeLiteral(1));
        }
        return null;
    }


    void makeInnerExprs() {
        if (getExpr != null) return;

        AST ast = getAST();

        AssignExpr assignExpr = makeAssignExpr(bangExpr);
        assignExpr.setParent(bangExpr.getParent());  //XXX this call indicates a design flaw

        FieldAccessExpr fieldAccessExpr = (FieldAccessExpr)bangExpr.getLhs();
        FieldDec fieldDec = fieldAccessExpr.getFieldDec();

        Expr baseExpr = fieldAccessExpr.getExpr();
        BlockStmt body = bangExpr.getEnclosingCodeDec().getBody();

        String op = assignExpr.getOp();

        Expr newExpr = null;
        if (baseExpr != null) newExpr = baseExpr.makeReference();
        getExpr = ast.makeGet(newExpr, fieldDec);

        Expr newValue =
          ast.makeCast(fieldDec.getType(),
            ast.makeParen(ast.makeBinop(op, getExpr, assignExpr.getRhs())));

        Expr firstBaseExpr = fieldAccessExpr.getExpr();
        FieldAccessExpr lhs = ast.makeGet(firstBaseExpr, fieldDec);

        setExpr = ast.makeSet(lhs, newValue);
    }
}
