/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.ast;

import org.aspectj.compiler.base.*;

import org.aspectj.compiler.base.bcg.CodeBuilder;
import org.aspectj.compiler.base.bcg.Label;

/**
 * @grammar this
 */

public class ThisExpr extends Expr {

    public Type discoverType() {
        //showWarning("discover type: " + getDeclaringType());
        return getDeclaringType();
    }

    public ThisExpr(SourceLocation source, Type type) {
        super(source);
        this.setType(type);
    }

    public Expr makeReference() {
        return (Expr)this.copy();
    }

    //INTRO from MovingWalker
    public ASTObject postMove(MovingWalker walker) {
        //System.out.println("moving: " + this + " by " + walker);
        return walker.moveThisExpr(this, getType());
    }

    // ------------------------------
    // INTRO from InnerInfoPass -- should have a more generic name

    public void postInnerInfo(InnerInfoPass walker) {
        if (!walker.isAccessibleExactly(getType())) {
            if (!fromSource() && getParent().fromSource()) {
                getParent().showError("no instance available");
            } else {
                showError("no instance available");
            }
        }
    }

    public void unparse(CodeWriter writer) {
        writer.writeKeyword("this");
    }

    // ------------------------------
    // bcg

    protected void cgValue(CodeBuilder cb) {
        cb.emitALOAD(0);
    }
    protected void cgEffect(CodeBuilder cb) { }

    //BEGIN: Generated from @child and @property

    public ThisExpr(SourceLocation location) {
        super(location);

    }

    public ASTObject copyWalk(CopyWalker walker) {
        ThisExpr ret = new ThisExpr(getSourceLocation());
        ret.preCopy(walker, this);

        return ret;
    }


    public String getDefaultDisplayName() {
        return "ThisExpr()";
    }

    //END: Generated from @child and @property
}
