/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.bcg;

import java.io.DataOutputStream;
import java.io.IOException;

public class Label {
    boolean anchored;
    Label(boolean anchored) { this.anchored = anchored; }

    boolean isAnchored() { return anchored; }
    
    Code target;

    void setTarget(Code target) { 
        this.target = target; 
        target.addLabel(this);
    }
    Code getTarget() { return target; }

    int getPc() { return target.getPc(); }

    void writeRelativeShort(int src, DataOutputStream stream) throws IOException {
        int relative = target.getPc() - src;
        if (! Asserts.isShort(relative)) {
            throw new CodeBuilder.TooWideJumpException();
        } else {
            stream.writeShort((short) relative);
        }
    }
    void writeRelativeInt(int src, DataOutputStream stream) throws IOException {
        long relative = target.getPc() - src;
        stream.writeInt((int) relative);
    }

    // only used for debugging, can be static
    static int labelCount = 0;
    int myLabelNum = -1;
    public String toString() {
        if (getTarget().getPc() != -1) return "" + getTarget().getPc();
        Label l = (CodeBuilder.DEBUG) ? this : (Label) getTarget().getLabels().get(0);
        if (l.myLabelNum == -1) l.myLabelNum = labelCount++;
        return "_" + l.myLabelNum;
    }
}
