/* -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.bcg.pool;

import org.aspectj.compiler.base.bcg.*;

import java.io.*;
import java.util.*;

import org.aspectj.compiler.base.ast.ArrayType;
import org.aspectj.compiler.base.ast.NameType;
import org.aspectj.compiler.base.ast.RefType;

public class FieldrefConstant extends Constant {
    ClassConstant _class;
    FieldrefConstant() {}
    NameAndTypeConstant signature;
    FieldrefConstant(ClassConstant _class,
                     NameAndTypeConstant signature) {
        this._class = _class;
        this.signature = signature;
    }
    void writeTo(DataOutputStream stream) throws IOException {
        stream.writeByte(FIELD_REF_TAG);
        _class.writeIndex(stream);
        signature.writeIndex(stream);
    }

    public String toString() {
        return rep(getIndex(),
                   "(fieldref " + _class + " " + signature + ")");
    }
    public boolean equals(Object other) {
        if (! (other instanceof FieldrefConstant)) return false;
        FieldrefConstant o = (FieldrefConstant) other;
        return _class == o._class && signature == o.signature;
    }
    public int hashCode() {
        return _class.hashCode() + signature.hashCode() + FIELD_REF_TAG;
    }

    void readFrom(DataInputStream stream, ConstantPool pool) throws IOException {
        _class = 
            (ClassConstant) pool.get(stream.readUnsignedShort(), new ClassConstant());
        signature = 
            (NameAndTypeConstant) pool.get(stream.readUnsignedShort(), new NameAndTypeConstant());
    }
}
