/* -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.bcg.pool;

import org.aspectj.compiler.base.bcg.*;

import java.io.*;
import java.util.*;

import org.aspectj.compiler.base.ast.ArrayType;
import org.aspectj.compiler.base.ast.NameType;
import org.aspectj.compiler.base.ast.RefType;

public class NameAndTypeConstant extends Constant {
    Utf8Constant name;
    NameAndTypeConstant() {}
    Utf8Constant descriptor;
    NameAndTypeConstant(Utf8Constant name, Utf8Constant descriptor) {
        this.name = name;
        this.descriptor = descriptor;
    }
    void writeTo(DataOutputStream stream) throws IOException {
        stream.writeByte(NAME_AND_TYPE_TAG);
        name.writeIndex(stream);
        descriptor.writeIndex(stream);
    }
    public String toString() {
        return rep(getIndex(),
                   "(nameAndType " + name + " " + descriptor + ")");
    }
    public boolean equals(Object other) {
        if (! (other instanceof NameAndTypeConstant)) return false;
        NameAndTypeConstant o = (NameAndTypeConstant) other;
        return name == o.name && descriptor == o.descriptor;
    }
    public int hashCode() {
        return name.hashCode() + descriptor.hashCode() + NAME_AND_TYPE_TAG;
    }
    void readFrom(DataInputStream stream, ConstantPool pool) throws IOException {
        name = (Utf8Constant) pool.get(stream.readUnsignedShort(), new Utf8Constant());
        descriptor = (Utf8Constant) pool.get(stream.readUnsignedShort(), new Utf8Constant());
    }
}
