/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.base.parser;

import java.io.File;
import java.util.*;
import org.aspectj.util.IntList;

public class SourceInfo {
    private File file;
    private int[] lineStarts;
    private char[] text; // there are some possible memory management issues here

    public SourceInfo(File _file, char[] _text) {
        file = _file;
        setText(_text);
    }
    
    public void setText(char[] text) {
        this.text = text; 
        if (text != null) {
            lineStarts = findNewLines(text);
        }
    }
    
    private int[] findNewLines(char[] text) {
        IntList lineStarts = new IntList();
        lineStarts.add(0);
        final int N = text.length;
        for(int i=0; i<N; i++) {
            char ch = text[i];
            if (ch == '\r') {
                if (i < N-1 && text[i+1] == '\n') {
                    text[i] = ' '; // this is needed to keep emacs-mode happy
                              // better handling of position information would
                              // work more effectively (and could handle the \\u cases)
                } else {
                    text[i] = '\n';
                    ch = '\n';
                }
            }
            if (ch == '\n') {
                lineStarts.add(i+1);
            }            
        }
        lineStarts.add(N+100);
        return lineStarts.toIntArray();
    }

    public File getFile() {
        return file;
    }

    // line numbers are given based on 1-indexing, i.e. the 1st line is 1 (not 0)
    public int getLine(int position) {
        if (lineStarts == null) return -1;
        int pos = Arrays.binarySearch(lineStarts, position);
        if (pos >= 0) return pos+1;
        else return (-pos)-1;
    }

    // column numbers are also 1-based
    public int getColumn(int position) {
        if (lineStarts == null) return -1;

        int line = 1;
        for(; line < lineStarts.length; line++) {
            if (lineStarts[line] > position) break;
        }

        return (position - lineStarts[line-1]) + 1;
    }
} 
