/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts;

import org.aspectj.compiler.crosscuts.ast.*;
import org.aspectj.compiler.crosscuts.joinpoints.*;

import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.base.*;
import org.aspectj.util.CollectionUtil;

import java.util.*;

public class AdvicePlannerPass extends WalkerPass {
    List planners/*JpPlanner*/;

    public AdvicePlannerPass(JavaCompiler compiler) {
        super(compiler);
    }

    public String getDisplayName() {
        return "planning advice";
    }

    public void transformWorld() {
        initializeJoinPointPlanners();
        super.transformWorld();
    }

    void initializeJoinPointPlanners() {
        planners = getWorld().getJpPlanners();
    }
    
    protected void postProcess(ASTObject object) {
        if (object instanceof TypeDec) plan((TypeDec)object);
    }

    public void plan(TypeDec typeDec) {        
        makePlans(typeDec.joinPoints0);
        makePlans(typeDec.joinPoints1);
        makePlans(typeDec.joinPoints2);
    }
    
    void planJoinPoint(JoinPoint jp) {
        //getCompiler().showMessage("    planning: " + jp);
        for (Iterator i = planners.iterator(); i.hasNext(); ) {
            TopJpPlanner jpPlanner = (TopJpPlanner)i.next();
            jpPlanner.plan(jp);
        }
        // if (jp.hasPlans()) getCompiler().showMessage("         PLANS!");
    }
    
    void makePlans(List jps) {
        for (Iterator i = jps.iterator(); i.hasNext(); ) {
            JoinPoint joinPoint = (JoinPoint)i.next();
            planJoinPoint(joinPoint);
        }
    }
}

