/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts;

import org.aspectj.compiler.base.*;
import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.crosscuts.ast.*;
import org.aspectj.util.CollectionUtil;

import org.aspectj.tools.ide.Declaration;
import org.aspectj.tools.ide.SymbolManager;

import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.compiler.base.CompilerObject;



import java.io.*;
//import java.io.Writer;
//import java.io.File;
//import java.io.FileWriter;
//import java.io.BufferedWriter;
//import java.io.IOException;


import java.util.*;

// Make sure that references to Vector use the standard version
// This lets us generate serialized files which interoperate with other code better
import java.util.Vector;


// ultimate goal is probably to do dependency tracking here to make enviro
// much nicer to use...

public class Correspondences extends CompilerObject {
    private Map corrs = new HashMap();

    public Correspondences(JavaCompiler compiler) {
        super(compiler);
    }

    public CompilationUnitCorrespondences getCompilationUnitCorrespondences(ASTObject node) {
        CompilationUnit cu = node.getCompilationUnit();

        CompilationUnitCorrespondences cuCorrs = (CompilationUnitCorrespondences)corrs.get(cu);
        if (cuCorrs == null) {
            cuCorrs = new CompilationUnitCorrespondences(cu);
            corrs.put(cu, cuCorrs);
        }
        return cuCorrs;
    }


    public File makeSymFile(File javaFile) {
        return SymbolManager.mapFilenameToSymbolFile(javaFile.getPath());
    }

    // dump these over the files that were read in
    public void dump(CompilationUnit cu, CompilationUnitCorrespondences cucorr) {
        //for(Iterator iter=corrs.entrySet().iterator(); iter.hasNext(); ) {
        //    Map.Entry entry = (Map.Entry)iter.next();
        //    CompilationUnit cu = (CompilationUnit)entry.getKey();
        //    if (cu == CompilationUnit.missingCompilationUnit) continue;
        //    CompilationUnitCorrespondences cucorr = (CompilationUnitCorrespondences)entry.getValue();
            dump(cu.getSourceFile(), cu, cucorr);
        //}
    }

    public void dump(File file, CompilationUnit cu, CompilationUnitCorrespondences cucorr) {
        if (file == null) return;
//         if (!getOptions().nosymbols) {
//             try {
//                 OutputStream out = new FileOutputStream(makeSymFile(file));
//                 try {
//                     cucorr.dumpSymbolFile(out);
//                 } finally {
//                     out.close();
//                 }
//             } catch (IOException ioe) {
//                 // handle ioexception intelligently here,,,
//             }
//         }
    }

    // a better name for me to keep track of what's really going on
    public void addPointsTo(ASTObject node1, ASTObject node2) {
        //System.err.println(node1.getCompilationUnit().getSource()+" points to "+node2.getCompilationUnit().getSource());

        if (node1 == null || node2 == null) return;

        //XXX hack to make the IDE's life easier for now
        getCompilationUnitCorrespondences(node1).addPointsTo(node1, node2);
        getCompilationUnitCorrespondences(node2).addPointedToBy(node2, node1);


    }

    /**
    public void addCorrespondence(ASTObject node1, ASTObject node2) {
        addPointsTo(node1, node2);
    }
    **/

    /**
     * Used by <CODE>StructureViewBuilder</CODE>
     */
     public Set getAffects(ASTObject astObject) {
        return getCompilationUnitCorrespondences(astObject).getPointsToSet(astObject);
     }

    /**
     * Used by <CODE>StructureViewBuilder</CODE>
     */
     public Set getAffectedBy(ASTObject astObject) {
        return getCompilationUnitCorrespondences(astObject).getPointedToBySet(astObject);
     }
}
