/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts;

import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.base.*;

import java.util.*;


public class MixinImplementationPass extends WalkerPass { 
    private TypeDec inTypeDec;
    
    public MixinImplementationPass(JavaCompiler compiler) {
        super(compiler);
    }
    
    public TypeDec getInTypeDec() { return inTypeDec; }
    public void setInTypeDec(TypeDec typeDec) { inTypeDec = typeDec; }
 
    public String getDisplayName() {
        return "desugaring mixin methods";
    }
    
    public Dec copyToClass(Dec dec, final TypeDec toClassDec) {
        final Type toType = toClassDec.getType();
        final Type fromType = dec.getBytecodeTypeDec().getType();
        
        //XXX we need a general and powerful simplification mechanism here...
        Walker w = new CopyWalker(getCompiler(), fromType, toType) {
            public Expr makeThisExpr() {
                return this.getAST().makeThis(this.toType);
            }
        };
        //System.out.println("copying: " + dec.unparse());
        Dec newDec = (Dec)w.process(dec);

        newDec.setBytecodeId(dec.getBytecodeId());
        newDec.setLexicalType(dec.getLexicalType());
        //System.out.println("  to: " + newDec.unparse());
        return newDec;
    }
        
    
    public ASTObject process(ASTObject object) {        
        object.implementMixin(this);
        return object.postImplementMixin(this);
    }
}
