/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts;

import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.base.*;

import java.util.*;


public class TreeCheckerPass extends WalkerPass {
    private boolean checkSourceInfo;
    private boolean checkParents;
    private boolean checkResolved;
    
    private int showRealParentOnExit = -1;
    
    public TreeCheckerPass(JavaCompiler compiler, boolean checkSourceInfo, boolean checkParents, boolean checkResolved) {
        super(compiler);
        this.checkSourceInfo = checkSourceInfo;
        this.checkParents = checkParents;
        this.checkResolved = checkResolved;
    }

    public String getDisplayName() {
        return "check source:" + checkSourceInfo + " parents: " + checkParents + " resolved: " + checkResolved;
    }

    
    public ASTObject process(ASTObject object) {
        //if (checkSourceInfo)
            //checkSourceInfo(object);
        if (checkParents) checkParent(object);
        if (checkResolved) checkResolved(object);
        
        object.walk(this);
        
        if (showRealParentOnExit > 0) showRealParentOnExit--;
        if (showRealParentOnExit == 0) {
            object.display(2);
            showRealParentOnExit = -1;
        }
        
        return object;
    }

    void checkSourceInfo(ASTObject object) {
        if (!object.hasSourceLocation()) {
            getCompiler().showError(object.getParent(), "no source location");
            
            object.display(2);
        }
    }

    void checkParent(ASTObject object) {
        if (object.getParent() == null && !(object instanceof CompilationUnit)) {
            object.showError("no parent");
            object.display(2);
            showRealParentOnExit = 3;
        }
    }

    void checkResolved(ASTObject object) {
        //XXX not implemented yet
    }


}
