/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.ast;

import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.crosscuts.*;
import org.aspectj.compiler.crosscuts.joinpoints.*;
import org.aspectj.compiler.base.JavaCompiler;

/**
 * @grammar before ...;
 */

public class BeforeAdviceDec extends AdviceDec {

    protected String getAdviceKind() { return "before"; }


    protected Stmts wrapStmts(Stmts stmts, AdvicePlan plan) {
        stmts.add(0, plan.makeCall(null));
        return stmts;
    }    
    
    //BEGIN: Generated from @child and @property
    
    public BeforeAdviceDec(SourceLocation location, Modifiers _modifiers, Formals _formals, TypeDs __throws, Pcd _pcd, CodeBody _body) {
        super(location, _modifiers, _formals, __throws, _pcd, _body);
    
    }
    protected BeforeAdviceDec(SourceLocation source) {
        super(source);
    }
    
    public ASTObject copyWalk(CopyWalker walker) {
        BeforeAdviceDec ret = new BeforeAdviceDec(getSourceLocation());
        ret.preCopy(walker, this);
        if (modifiers != null) ret.setModifiers( (Modifiers)walker.process(modifiers) );
        if (formals != null) ret.setFormals( (Formals)walker.process(formals) );
        if (_throws != null) ret.setThrows( (TypeDs)walker.process(_throws) );
        if (pcd != null) ret.setPcd( (Pcd)walker.process(pcd) );
        if (body != null) ret.setBody( (CodeBody)walker.process(body) );
        return ret;
    }
    
    
    public String getDefaultDisplayName() {
        return "BeforeAdviceDec()";
    }
    
    //END: Generated from @child and @property
}
