/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.ast;
import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.crosscuts.joinpoints.*;

import org.aspectj.compiler.base.JavaCompiler;

import java.util.*;
import org.aspectj.util.FuzzyBoolean;

/**
 * @grammar !name
 * @child GenTypeName name
 */

public class ComplementTypeName extends GenTypeName {
    public String toShortString() {
        return "!" + name.toShortString();
    }
    
    public boolean matches(Type type) {
        return !name.matches(type);
    }
    public FuzzyBoolean matchesInstance(Type type) {
        return name.matchesInstance(type).not();
    }
    
    public JpPlan makePlan(JoinPoint jp, Expr expr) {
        JpPlan plan = name.makePlan(jp, expr);
        return plan.not();
    }    
    
    //BEGIN: Generated from @child and @property
    protected GenTypeName name;
    public GenTypeName getName() { return name; }
    public void setName(GenTypeName _name) {
        if (_name != null) _name.setParent(this);
        name = _name;
    }
    
    public ComplementTypeName(SourceLocation location, GenTypeName _name) {
        super(location);
        setName(_name);
    }
    protected ComplementTypeName(SourceLocation source) {
        super(source);
    }
    
    public ASTObject copyWalk(CopyWalker walker) {
        ComplementTypeName ret = new ComplementTypeName(getSourceLocation());
        ret.preCopy(walker, this);
        if (name != null) ret.setName( (GenTypeName)walker.process(name) );
        return ret;
    }
    
    public ASTObject getChildAt(int childIndex) {
        switch(childIndex) {
        case 0: return name;
        default: return super.getChildAt(childIndex);
        }
    }
     public String getChildNameAt(int childIndex) {
        switch(childIndex) {
        case 0: return "name";
        default: return super.getChildNameAt(childIndex);
        }
    }
     public void setChildAt(int childIndex, ASTObject child) {
        switch(childIndex) {
        case 0: setName((GenTypeName)child); return;
        default: super.setChildAt(childIndex, child); return;
        }
    }
     public int getChildCount() {
        return 1;
    }
    
    public String getDefaultDisplayName() {
        return "ComplementTypeName()";
    }
    
    //END: Generated from @child and @property
}

