/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.ast;
import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.crosscuts.joinpoints.*;

import org.aspectj.compiler.base.JavaCompiler;

import java.util.*;
import org.aspectj.util.InvertableSet;

/**
  * @grammar modifiers fieldTypeName declaringType.id
  * @child GenTypeName fieldTypeName
  * @property NamePattern id
  */
public class FieldPattern extends DecPattern {
    public String toShortString() {
        String mods = modifiers.toShortString();
        if (mods.length() > 0) mods += " ";
        mods += fieldTypeName.toShortString()+" ";
        if (declaringTypeName != null) mods += declaringTypeName.toShortString() + ".";
        String ret = mods + id.toShortString();

        return ret;
    }

    public String getLookupId()  { return id.getLookupString(); }

    public JpPlan makePlan(JoinPoint joinPoint) {
        if (! (joinPoint.getTargetSO() instanceof Field) ) return JpPlan.NO_PLAN;
        
        Field m = (Field)joinPoint.getTargetSO();
        
        //System.out.println("matching: " + toShortString() + " == " + m.toShortString());
        
        if (!id.matches(m.getName())) return JpPlan.NO_PLAN;
        //System.out.println("matched id");
        if (!fieldTypeName.matches(m.getFieldType())) return JpPlan.NO_PLAN;
        
        //System.out.println("  matching in super");
        
        return super.makePlan(joinPoint);
    }
    
    protected boolean isStaticMatch(SemanticObject so) {
        return true;
    }    
    
    //BEGIN: Generated from @child and @property
    protected GenTypeName fieldTypeName;
    public GenTypeName getFieldTypeName() { return fieldTypeName; }
    public void setFieldTypeName(GenTypeName _fieldTypeName) {
        if (_fieldTypeName != null) _fieldTypeName.setParent(this);
        fieldTypeName = _fieldTypeName;
    }
    
    protected NamePattern id;
    public NamePattern getId() { return id; }
    public void setId(NamePattern _id) { id = _id; }
    
    public FieldPattern(SourceLocation location, Modifiers _modifiers, GenTypeName _declaringTypeName, GenTypeName _fieldTypeName, NamePattern _id) {
        super(location, _modifiers, _declaringTypeName);
        setFieldTypeName(_fieldTypeName);
        setId(_id);
    }
    protected FieldPattern(SourceLocation source) {
        super(source);
    }
    
    public ASTObject copyWalk(CopyWalker walker) {
        FieldPattern ret = new FieldPattern(getSourceLocation());
        ret.preCopy(walker, this);
        if (modifiers != null) ret.setModifiers( (Modifiers)walker.process(modifiers) );
        if (declaringTypeName != null) ret.setDeclaringTypeName( (GenTypeName)walker.process(declaringTypeName) );
        if (fieldTypeName != null) ret.setFieldTypeName( (GenTypeName)walker.process(fieldTypeName) );
        ret.id = id;
        return ret;
    }
    
    public ASTObject getChildAt(int childIndex) {
        switch(childIndex) {
        case 2: return fieldTypeName;
        default: return super.getChildAt(childIndex);
        }
    }
     public String getChildNameAt(int childIndex) {
        switch(childIndex) {
        case 2: return "fieldTypeName";
        default: return super.getChildNameAt(childIndex);
        }
    }
     public void setChildAt(int childIndex, ASTObject child) {
        switch(childIndex) {
        case 2: setFieldTypeName((GenTypeName)child); return;
        default: super.setChildAt(childIndex, child); return;
        }
    }
     public int getChildCount() {
        return 3;
    }
    
    public String getDefaultDisplayName() {
        return "FieldPattern(id: "+id+")";
    }
    
    //END: Generated from @child and @property
}

