/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.ast;

import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.crosscuts.joinpoints.*;

import org.aspectj.util.InvertableSet;

import java.util.*;

import org.aspectj.compiler.base.JavaCompiler;

/**
 * @grammar !pcd
 * @child Pcd pcd
 */

public class NotPcd extends Pcd {
    public String toShortString() {
        return "!" + getPcd().toShortString();
    }
    
    public void checkStatic() { getPcd().checkStatic(); }

    public JpPlanner makePlanner(PlanData planData) {
        return getPcd().makePlanner(planData).not();
    }    
    
    //BEGIN: Generated from @child and @property
    protected Pcd pcd;
    public Pcd getPcd() { return pcd; }
    public void setPcd(Pcd _pcd) {
        if (_pcd != null) _pcd.setParent(this);
        pcd = _pcd;
    }
    
    public NotPcd(SourceLocation location, Pcd _pcd) {
        super(location);
        setPcd(_pcd);
    }
    protected NotPcd(SourceLocation source) {
        super(source);
    }
    
    public ASTObject copyWalk(CopyWalker walker) {
        NotPcd ret = new NotPcd(getSourceLocation());
        ret.preCopy(walker, this);
        if (pcd != null) ret.setPcd( (Pcd)walker.process(pcd) );
        return ret;
    }
    
    public ASTObject getChildAt(int childIndex) {
        switch(childIndex) {
        case 0: return pcd;
        default: return super.getChildAt(childIndex);
        }
    }
     public String getChildNameAt(int childIndex) {
        switch(childIndex) {
        case 0: return "pcd";
        default: return super.getChildNameAt(childIndex);
        }
    }
     public void setChildAt(int childIndex, ASTObject child) {
        switch(childIndex) {
        case 0: setPcd((Pcd)child); return;
        default: super.setChildAt(childIndex, child); return;
        }
    }
     public int getChildCount() {
        return 1;
    }
    
    public String getDefaultDisplayName() {
        return "NotPcd()";
    }
    
    //END: Generated from @child and @property
}

