/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.ast;
import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.base.*;
import org.aspectj.compiler.crosscuts.joinpoints.*;
import org.aspectj.compiler.base.cst.*;

import org.aspectj.util.FuzzyBoolean;

/**
 * @grammar id
 * @property Type type
 * @property boolean includeSubTypes
 */

public class SimpleTypeName extends GenTypeName {
    public String toShortString() {
    	if (includeSubTypes) return type.getName() + "+";
    	else return type.getName();
    }
    
    
    public boolean matches(Type type) {
    	if (includeSubTypes) return type.isSubtypeOf(getType());
    	else return type == getType();
    }

    
    public FuzzyBoolean matchesInstance(Type exprType) {
        Type formalType = getType();
        //System.out.println("plan: " + exprType + ", " + formalType);
        if (formalType.isObject()) return FuzzyBoolean.YES;
        
        if (formalType.isAssignableFrom(exprType)) return FuzzyBoolean.YES;
        
        // this is a conditional true based on the needed dynamic test
        if (exprType.isCoercableTo(formalType)) return FuzzyBoolean.MAYBE;
        
        return FuzzyBoolean.NO;
    }

    public JpPlan makePlan(JoinPoint jp, Expr expr) {        
        Type exprType = expr.getType();
        Type formalType = getType();
        //System.out.println("plan: " + jp + ", " + exprType + ", " + formalType);
               
        if (matchesInstance(exprType).alwaysFalse()) return JpPlan.NO_PLAN;
        
        JpPlan plan = new JpPlan(jp);
        

        
        if (formalType.isObject()) {
            return plan;
        }
        
        if (formalType.isAssignableFrom(exprType)) {
            return plan;
        }
        
        if (exprType.isCoercableTo(formalType)) {
            final AST ast = getAST();
            
            Expr testExpr = ast.makeParen(ast.makeInstanceof(expr, formalType));
            plan.addExprTest(testExpr);
            return plan;
        }
        
        return JpPlan.NO_PLAN;
    }    
    
    //BEGIN: Generated from @child and @property
    protected Type type;
    public Type getType() { return type; }
    public void setType(Type _type) { type = _type; }
    
    protected boolean includeSubTypes;
    public boolean getIncludeSubTypes() { return includeSubTypes; }
    public void setIncludeSubTypes(boolean _includeSubTypes) { includeSubTypes = _includeSubTypes; }
    
    public SimpleTypeName(SourceLocation location, Type _type, boolean _includeSubTypes) {
        super(location);
        setType(_type);
        setIncludeSubTypes(_includeSubTypes);
    }
    protected SimpleTypeName(SourceLocation source) {
        super(source);
    }
    
    public ASTObject copyWalk(CopyWalker walker) {
        SimpleTypeName ret = new SimpleTypeName(getSourceLocation());
        ret.preCopy(walker, this);
        ret.type = type;
        ret.includeSubTypes = includeSubTypes;
        return ret;
    }
    
    
    public String getDefaultDisplayName() {
        return "SimpleTypeName(type: "+type+", "+"includeSubTypes: "+includeSubTypes+")";
    }
    
    //END: Generated from @child and @property
}

