/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.compiler.crosscuts.ast;
import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.crosscuts.joinpoints.*;

import org.aspectj.compiler.base.JavaCompiler;

import java.util.*;
import org.aspectj.util.InvertableSet;


/**
 * @grammar [prefix]*[suffix]
 * @property List pieces
 */
public class WildName extends NamePattern {
    public String getLookupString() {
        return "*";
    }

    public boolean isStar() { return false; }

    private boolean matches(String s, int sIndex, int pieceIndex) {
        //System.out.println(">>>" + this + " matches " + s + ": " + s.length() + ", " + sIndex + ", " + pieceIndex);
        
        if (pieceIndex == pieces.size()) {
            return sIndex == s.length();
        }
        
        String piece = (String)pieces.get(pieceIndex);
        if (piece == "*") {
            for (int i=sIndex; i<=s.length(); i++) {
                if (matches(s, i, pieceIndex+1)) return true;
            }
            return false;
        } else {
            if (!s.regionMatches(sIndex, piece, 0, piece.length())) return false;
            else return matches(s, sIndex+piece.length(), pieceIndex+1);
        }
    }
    
    
    public boolean matches(String s) {
        //System.out.println(this + " matches " + s + ": " + matches(s, 0, 0));
        return matches(s, 0, 0);
    }

    public String toShortString() {
        StringBuffer buf = new StringBuffer();
        for (Iterator i = pieces.iterator(); i.hasNext(); ) {
            buf.append(i.next());
        }
        return buf.toString();
    }    
    
    //BEGIN: Generated from @child and @property
    protected List pieces;
    public List getPieces() { return pieces; }
    public void setPieces(List _pieces) { pieces = _pieces; }
    
    public WildName(SourceLocation location, List _pieces) {
        super(location);
        setPieces(_pieces);
    }
    protected WildName(SourceLocation source) {
        super(source);
    }
    
    public ASTObject copyWalk(CopyWalker walker) {
        WildName ret = new WildName(getSourceLocation());
        ret.preCopy(walker, this);
        ret.pieces = pieces;
        return ret;
    }
    
    
    public String getDefaultDisplayName() {
        return "WildName(pieces: "+pieces+")";
    }
    
    //END: Generated from @child and @property
}


