/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.fix
 *
 * Contributor(s):
 */
package org.aspectj.compiler.crosscuts.joinpoints;

import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.crosscuts.ast.*;

import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.util.JavaStrings;

import java.util.*;


public class FieldGetJp extends ExprJp {
    public static interface FieldGetExprPromise {
        /** makes this concrete */
        public FieldAccessExpr getFieldAccessExpr();
        
        public FieldDec getFieldDec();
        public Expr getSourceExpr();
        public Type getTargetType();
        public ASTObject getSourceLocation();
    }
    
    FieldGetExprPromise fieldGetExprPromise;

    public FieldGetJp(FieldGetExprPromise fieldGetExprPromise, 
                                JoinPoint enclosingJoinPoint)
    {
        super(fieldGetExprPromise.getSourceExpr(), enclosingJoinPoint);
        this.fieldGetExprPromise = fieldGetExprPromise;
    }

    public static final Kind KIND = new Kind("field-get", FIELD_GET);

    public Kind getKind() { return KIND; }

    public ASTObject getSourceLocation() {
        return fieldGetExprPromise.getSourceLocation();
    }
    
    public List makeArgsTypes() { return Collections.EMPTY_LIST; }

    public Exprs makeArgsExprs() {
        return getAST().makeExprs();
    }
    
    public Type getTargetType() {
        return fieldGetExprPromise.getTargetType();
    }

    public Type getTargetExprType() {
        if (fieldGetExprPromise.getFieldDec().isStatic()) return null;
        return getTargetType();
    }

    public Dec getTargetDec() { return getCalledFieldDec(); }

    public FieldDec getCalledFieldDec() {
        return fieldGetExprPromise.getFieldDec();
    }

    public boolean canThrow(Type t) { return t.isUncheckedThrowable(); }
    public Collection getPossibleCheckedExceptions() {
    	return Collections.EMPTY_LIST;
    }
        
    public Type getResultType() {
        return getCalledFieldDec().getType();
    }

    protected ASTObject makeInnerCall(Expr targetExpr, Exprs argsExprs) {
        return getAST().makeGet(targetExpr, getCalledFieldDec());
    }
    
    protected AnonymousMethodExpr makeSyntheticCall() {
        final AST ast = getAST();
        FieldAccessExpr getExpr = fieldGetExprPromise.getFieldAccessExpr();
        return makeAnonMethodExpr(ast.makeFormals(), getExpr.getExpr(), 
                                  ast.makeExprs());
    }

    protected void replaceExprWith(Expr newExpr) {    
        fieldGetExprPromise.getFieldAccessExpr().replaceWith(newExpr);
    }
}
