/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */
package org.aspectj.compiler.crosscuts.joinpoints;

import org.aspectj.compiler.base.ast.*;
import org.aspectj.compiler.crosscuts.ast.*;

import org.aspectj.compiler.base.JavaCompiler;
import org.aspectj.util.JavaStrings;

import java.util.*;


public class FieldSetJp extends FieldGetJp {
    public static interface FieldSetExprPromise extends FieldGetExprPromise {
        /** makes this concrete */
        public AssignExpr getAssignExpr();
    }
    public FieldSetJp(FieldSetExprPromise fieldSetExprPromise, 
        JoinPoint enclosingJoinPoint)
    {
        super(fieldSetExprPromise, enclosingJoinPoint);
    }

    public static final Kind KIND = new Kind("field-set", FIELD_SET);

    public Kind getKind() { return KIND; }
    
    // newValue handling    
    FormalDec newValueFormal = null;
    public FormalDec getNewValueFormal() {
        if (newValueFormal == null) {
            newValueFormal = getAST().makeFormal(getResultType(), "newValue");
        }
        return newValueFormal;
    }
    
    //XXX clarify names vs. one below
    public Expr makeNewFieldValueExpr() {
       return getAST().makeVar(getNewValueFormal());
    }
    
    public Expr makeNewValueExpr() {
        return getResultType().makeObject(getAST().makeVar(getNewValueFormal()));
    }
    
    public List makeArgsTypes() {
        List ret = new ArrayList();
        ret.add(getResultType());
        return ret;
    }

    public Exprs makeArgsExprs() {
        return getAST().makeExprs(makeNewFieldValueExpr());
    }

    protected ASTObject makeInnerCall(Expr targetExpr, Exprs argsExprs) {
        return getAST().makeSet(targetExpr, getCalledFieldDec(), argsExprs.get(0));
    }
    
    protected AnonymousMethodExpr makeSyntheticCall() {
        final AST ast = getAST();
        AssignExpr setExpr =
            ((FieldSetExprPromise)fieldGetExprPromise).getAssignExpr();

        //XXX this cast expression wouldn't be needed if we handled int literals as
        //XXX shorts and bytes more uniformly elsewhere
        Expr newValue = ast.makeCast(getNewValueFormal().getType(), setExpr.getRhs());

        return makeAnonMethodExpr(ast.makeFormals(getNewValueFormal()),
                                  ((FieldAccessExpr)setExpr.getLhs()).getExpr(),
                                  ast.makeExprs(newValue));
    }
    protected void replaceExprWith(Expr newExpr) {    
        ((FieldSetExprPromise)fieldGetExprPromise).getAssignExpr().replaceWith(newExpr);
    }
}
