/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.lang;

import org.aspectj.lang.reflect.SourceLocation;

public interface JoinPoint {

    String toString();

    /**
     * returns an abbreviated string representation of the join point
     */
    String toShortString();

    /**
     * returns an extended string representation of the join point
     */
    String toLongString();
    
    /**
     * <p> returns the currently executing object.  </p>
     * 
     * <p> if this object is not available, returns null.</p>
     */
    Object getThis();                                      
  
    /**
     * <p> returns the target object.  </p>
     */
    Object getTarget();                                      
  
    /**
     * <p> returns the arguments at this join point.  </p>
     */
    Object[] getArgs();                                      
  
    /** returns the signature at the join point.  */
    Signature getSignature();

    /** <p>returns the source location corresponding to the join point.</p>
     *
     *  <p>If there is no source location available, returns null.</p> 
     *
     *  <p>returns the SourceLocation of the defining class for default constructors.</p>
     */
    SourceLocation getSourceLocation();
    
    /** <p> returns a String representing the kind of join point.  This String
     *       is guaranteed to be interned</p>
     */
    String getKind();
    
    public interface StaticPart {
        /** returns the signature at the join point.  */
        Signature getSignature();
        
        /** <p>returns the source location corresponding to the join point.</p>
        *
        *  <p>If there is no source location available, returns null.</p> 
        *
        *  <p>returns the SourceLocation of the defining class for default constructors.</p>
        */
        SourceLocation getSourceLocation();
        
        /** <p> returns a String representing the kind of join point.  This String
        *       is guaranteed to be interned</p>
        */
        String getKind();
        
        String toString();
        
        /**
        * returns an abbreviated string representation of the join point
        */
        String toShortString();
        
        /**
        * returns an extended string representation of the join point
        */
        String toLongString();
    }
    
    
    /** <p> returns an object that encapsulates the static parts of this join point </p>
     */
    StaticPart getStaticPart();
        
}
