/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.tools.ajde.netbeans;

import java.io.IOException;
import javax.swing.*;
import org.openide.Places;
import org.openide.TopManager;
import org.openide.loaders.DataFolder;
import org.openide.modules.ModuleInstall;
import org.openide.filesystems.FileObject;
import org.openide.util.Utilities;
import org.openide.util.actions.SystemAction;
import org.openide.explorer.*;
import org.openide.explorer.view.*;
import org.openide.windows.*;
import org.openide.awt.*;
//import org.openidex.util.Utilities2;
import org.aspectj.ajde.ui.ErrorHandler;

public class Install extends ModuleInstall {

    private static final String STARTUP_MESSAGE = "\nAspectJ Development Environment (AJDE) support " +
        org.aspectj.compiler.Version.text +
        " loaded.\n" +
        "Copyright (C) 2002 Xerox Corporation.  All rights reserved.\n";

    private DataFolder ajMenu = null;
    private DataFolder ajToolbar = null;
    private DataFolder ajActions = null;

    public void installed () {
        try {
            setFolders();

/*            Utilities2.createAction(Actions.START_BROWSER.getClass(), ajMenu, "", true, false, true, true);
            Utilities2.createAction(Actions.START_BROWSER.getClass(), ajToolbar, "", true, false, false, false);
            Utilities2.createAction(Actions.START_BROWSER.getClass(), ajActions);

            Utilities2.createAction(Actions.BUILD.getClass(), ajMenu, "", true, false, false, false);
            Utilities2.createAction(Actions.BUILD.getClass(), ajToolbar, "", true, false, false, false);
            Utilities2.createAction(Actions.BUILD.getClass(), ajActions);

            Utilities2.createAction(Actions.RUN.getClass(), ajMenu, "", true, false, false, false);
            Utilities2.createAction(Actions.RUN.getClass(), ajToolbar, "", true, false, false, false);
            Utilities2.createAction(Actions.RUN.getClass(), ajActions);

            Utilities2.createAction(Actions.OPTIONS.getClass(), ajMenu, "", true, false, false, false);
            Utilities2.createAction(Actions.OPTIONS.getClass(), ajToolbar, "", true, false, false, false);
            Utilities2.createAction(Actions.OPTIONS.getClass(), ajActions);

            Actions.BUILD.setEnabled(false);
            Actions.RUN.setEnabled(false);
            Actions.OPTIONS.setEnabled(false);
*/        } catch (java.lang.NoClassDefFoundError ncdfe) {
            ncdfe.printStackTrace();
            ErrorHandler.handleError("org.openidex.util.Utilities2 was not found. " +
                "Please verify that openidex.jar is in the Forte lib/ext directory.");
        } catch (Throwable t) {
            t.printStackTrace();
            ErrorHandler.handleError("AJDEForForte could not be installed because an unknown " +
                "error occured.  If possible " +
                "please mail the stack trace to support@aspectj.org", t);
        }
        bindKey();
        System.out.println(STARTUP_MESSAGE);
    }

    public void uninstalled () {
        try {
            setFolders();

/*            Utilities2.removeAction(Actions.START_BROWSER.getClass(), ajMenu);
            Utilities2.removeAction(Actions.START_BROWSER.getClass(), ajToolbar);
            Utilities2.removeAction(Actions.START_BROWSER.getClass(), ajActions);

            Utilities2.removeAction(Actions.BUILD.getClass(), ajMenu);
            Utilities2.removeAction(Actions.BUILD.getClass(), ajToolbar);
            Utilities2.removeAction(Actions.BUILD.getClass(), ajActions);

            Utilities2.removeAction(Actions.RUN.getClass(), ajMenu);
            Utilities2.removeAction(Actions.RUN.getClass(), ajToolbar);
            Utilities2.removeAction(Actions.RUN.getClass(), ajActions);

            Utilities2.removeAction(Actions.OPTIONS.getClass(), ajMenu);
            Utilities2.removeAction(Actions.OPTIONS.getClass(), ajToolbar);
            Utilities2.removeAction(Actions.OPTIONS.getClass(), ajActions);

            Utilities2.removeAction(Actions.DEBUG.getClass(), ajMenu);
            Utilities2.removeAction(Actions.DEBUG.getClass(), ajToolbar);
            Utilities2.removeAction(Actions.DEBUG.getClass(), ajActions);
  */      } catch (IOException e) {
            e.printStackTrace ();
        }
    }

    public void restored () {
        uninstalled();
        installed();
    }

    private void setFolders() throws IOException {
        Places.Folders folders = TopManager.getDefault().getPlaces().folders();
        if (ajMenu == null) {
            ajMenu = folders.menus().create(folders.menus().create(folders.menus(), "Tools"), "AspectJ");
        }
        if (ajToolbar == null) {
            ajToolbar = folders.toolbars().create(folders.toolbars(), "AspectJ");
        }
        if (ajActions == null) {
            ajActions = folders.toolbars().create(folders.actions(), "AspectJ");
        }
    }

    private static void bindKey() {
    	TopManager.getDefault ().getGlobalKeymap ().addActionForKeyStroke
    	    (Utilities.stringToKey ("CSA-F9"),
    	     SystemAction.get (Actions.BUILD.getClass()));
    }
}
