/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.util;


import java.util.*;

public class InvertableSet {
    private boolean inverse;
    private Set set;

    public InvertableSet(Set set, boolean inverse) {
        this.set = set;
        this.inverse = inverse;
    }

    public InvertableSet(Set set) {
        this.set = set;
        this.inverse = false;
    }

    public InvertableSet() {
        this.set = new HashSet();
        this.inverse = false;
    }

    public Set getSet() { return set; }
    public boolean isInverse() { return inverse; }


    public void addAll(InvertableSet other) {
        InvertableSet s = union(other);
        this.set = s.set;
        this.inverse = s.inverse;
    }

    public boolean contains(Object obj) {
        if (inverse) return !set.contains(obj);
        else return set.contains(obj);
    }

    public boolean containsAny(Collection c) {
        for (Iterator i = c.iterator(); i.hasNext(); ) {
            if (contains(i.next())) return true;
        }
        return false;
    }

    public InvertableSet intersect(InvertableSet s2) {
        InvertableSet s1 = this;

        //System.err.println("intersect: "+!s1.inverse+", "+!s2.inverse);
        //s1 & s2
        if (!s1.inverse && !s2.inverse) {
            s1.set.retainAll(s2.set);
            return s1;
        }
        //!s1 & !s2 -> !(s1 | s2)
        if (s1.inverse && s2.inverse) {
            s1.set.addAll(s2.set);
            return s1;
        }
        if (s1.inverse) {
            s1 = s2;
            s2 = this;
        }
        //s1 & !s2
        s1.set.removeAll(s2.set);
        return s1;
    }

    public InvertableSet invert() {
        inverse = !inverse;
        return this;
    }

    public InvertableSet union(InvertableSet s2) {
        // s1 | s2 -> !(!s1 & !s2)
        InvertableSet s0 = this.invert().intersect(s2.invert());
        return s0.invert();
    }
}
