/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.util;

import java.io.*;

public class UnicodeEscapeWriter extends Writer {
    protected OutputStream outputStream;
    protected byte[] buffer;
    protected static final int bufferSize = 2048;
    protected int index;
    protected boolean addSlash;

    public UnicodeEscapeWriter(OutputStream outputStream) {
        this.outputStream = outputStream;
        // the extra 16 lets us avoid thinking about overflow issues too often
        buffer = new byte[bufferSize+16];
        index = 0;
        addSlash = false;
    }

    public void write(int ch) throws IOException {
        if (addSlash && ch == 'u') {
            buffer[index++] = (byte)'\\';
            //buffer[index++] = 'u';
        }

        if (ch == '\\') {
            addSlash = !addSlash;
        } else {
            addSlash = false;
        }

        if (ch < 1 || ch > 126) {
            int b1 = ch>>12;
            int b2 = ch>>8 & 0xf;
            int b3 = ch>>4 & 0xf;
            int b4 = ch & 0xf;

            buffer[index++] = (byte)'\\';
            buffer[index++] = (byte)'u';
            buffer[index++] = (byte)Character.forDigit(b1, 16);
            buffer[index++] = (byte)Character.forDigit(b2, 16);
            buffer[index++] = (byte)Character.forDigit(b3, 16);
            buffer[index++] = (byte)Character.forDigit(b4, 16);
        } else {
            buffer[index++] = (byte)ch;
        }

        if (index >= bufferSize) flushBuffer();
    }

    public void flushBuffer() throws IOException {
        outputStream.write(buffer, 0, index);
        index = 0;
    }

    public void write(char[] cbuf, int off, int len) throws IOException {
        int index=0;
        while (index<len) {
            write(cbuf[off+index++]);
        }
    }

    public void close() throws IOException {
        flushBuffer();
        outputStream.close();
    }

    public void flush() throws IOException {
        flushBuffer();
        outputStream.flush();
    }

    // \\uxxxx


    public static void main(String[] args) throws IOException {
        UnicodeEscapeWriter writer = new UnicodeEscapeWriter(System.out);
        writer.write("x = '\u0000'; y = '\u0032\u0033\u1234\uabcd'\n");
        writer.write("\\"+"u"+"xxxx");
        writer.flush();
    }
}
