/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the compiler and core tools for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.util;

import java.util.*;


/** This class implements boolean that include a "maybe"
 */

public abstract class FuzzyBoolean {
    public abstract boolean alwaysTrue();
    public abstract boolean alwaysFalse();
    public abstract boolean maybeTrue();
    public abstract boolean maybeFalse();
    
    public abstract FuzzyBoolean and(FuzzyBoolean other);
    public abstract FuzzyBoolean or(FuzzyBoolean other);
    public abstract FuzzyBoolean not();
    
    private static class YesFuzzyBoolean extends FuzzyBoolean {
        public boolean alwaysFalse() {
            return false;
        }

        public boolean alwaysTrue() {
            return true;
        }


        public boolean maybeFalse() {
            return false;
        }

        public boolean maybeTrue() {
            return true;
        }
        
        public FuzzyBoolean and(FuzzyBoolean other) {
            return other;
        }

        public FuzzyBoolean not() {
            return FuzzyBoolean.NO;
        }

        public FuzzyBoolean or(FuzzyBoolean other) {
            return this;
        }

        public String toString() {
            return "YES";
        }
    }    
    private static class NoFuzzyBoolean extends FuzzyBoolean {
        public boolean alwaysFalse() {
            return true;
        }

        public boolean alwaysTrue() {
            return false;
        }


        public boolean maybeFalse() {
            return true;
        }

        public boolean maybeTrue() {
            return false;
        }
        
        public FuzzyBoolean and(FuzzyBoolean other) {
            return this;
        }

        public FuzzyBoolean not() {
            return FuzzyBoolean.YES;
        }

        public FuzzyBoolean or(FuzzyBoolean other) {
            return other;
        }

        public String toString() {
            return "NO";
        }
    }
    
    private static class MaybeFuzzyBoolean extends FuzzyBoolean {
        public boolean alwaysFalse() {
            return false;
        }

        public boolean alwaysTrue() {
            return false;
        }


        public boolean maybeFalse() {
            return true;
        }

        public boolean maybeTrue() {
            return true;
        }
        
        public FuzzyBoolean and(FuzzyBoolean other) {
            return other.alwaysFalse() ? other : this;
        }

        public FuzzyBoolean not() {
            return this;
        }

        public FuzzyBoolean or(FuzzyBoolean other) {
            return other.alwaysTrue() ? other : this;
        }

        public String toString() {
            return "MAYBE";
        }
    }
    
    public static final FuzzyBoolean YES   = new YesFuzzyBoolean();
    public static final FuzzyBoolean NO    = new NoFuzzyBoolean();
    public static final FuzzyBoolean MAYBE = new MaybeFuzzyBoolean();

}
